(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[   2036335,      67211]*)
(*NotebookOutlinePosition[   2054311,      67621]*)
(*  CellTagsIndexPosition[   2053363,      67591]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[StyleBox["SYMAP - Symbolic Analysis Program Package for Power \
Electronics",
  FontSize->35]], "Title",
  TextAlignment->Center,
  TextJustification->0],

Cell[TextData[StyleBox["Version 2.0\nCopyright \[Copyright] 2002 by Jian Sun\n\
All rights reserved.",
  FontSize->17]], "Section 1",
  TextAlignment->Center,
  TextJustification->0],

Cell[CellGroupData[{

Cell["Table of Contents", "Section"],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        ButtonBox["Introduction",
          ButtonData:>"Introduction",
          ButtonStyle->"Hyperlink"], TraditionalForm]]],
  "\n\n",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(Load\ The\ Package\),
          ButtonData:>"Load Package",
          ButtonStyle->"Hyperlink"], TraditionalForm]]],
  "\n\n",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(Circuit\ Model\),
          ButtonData:>"Circuit Model",
          ButtonStyle->"Hyperlink"], TraditionalForm]]],
  "\n\n",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(State - Space\ Modeling\),
          ButtonData:>"State-Space Modeling",
          ButtonStyle->"Hyperlink"], TraditionalForm]]],
  "\n\n",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(Averaged\ Modeling\),
          ButtonData:>"Averaged Modeling",
          ButtonStyle->"Hyperlink"], TraditionalForm]]],
  "\n\n",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(Ripple\ Estimation\),
          ButtonData:>"Ripple Estimation",
          ButtonStyle->"Hyperlink"], TraditionalForm]]],
  "\n\n",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(Large - Signal\ Simulation\),
          ButtonData:>"Large-Signal Simulation",
          ButtonStyle->"Hyperlink"], TraditionalForm]]],
  "\n\n",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(Linearization\ and\ Transfer\ Functions\),
          ButtonData:>"Small-Signal Analysis",
          ButtonStyle->"Hyperlink"], TraditionalForm]]]
}], "Text",
  CellTags->"TOC"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Introduction\t\t\t\t\t",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(back\ to\ TOC\),
          ButtonData:>"TOC",
          ButtonStyle->"Hyperlink"], TraditionalForm]],
    FontSize->12,
    FontWeight->"Plain"]
}], "Section",
  CellTags->"Introduction"],

Cell[TextData[{
  "SYMAP is developed for automated modeling and analysis of power electronic \
circuits. It is in fact an add-on for ",
  StyleBox["Mathematica. ",
    FontSlant->"Italic"],
  "The current version has been updated to work with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 3.0. If you are still running the old version of ",
  StyleBox["Mathematica ",
    FontSlant->"Italic"],
  "(2.2), you may contact the author and obtain a copy of the package that \
works with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 2.2. The basic functions of the two versions are similar, but some \
features of the current version are not available in the old one.\n\nThe \
package can be used for various modeling, analysis, and design tasks in power \
electronics, such as ",
  ButtonBox["state-space modeling",
    ButtonData:>"State-Space Modeling",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["averaged modeling",
    ButtonData:>"Averaged Modeling",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["ripple estimation",
    ButtonData:>"Ripple Estimation",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["large-signal transient simulation",
    ButtonData:>"Large-Signal Simulation",
    ButtonStyle->"Hyperlink"],
  ", and ",
  ButtonBox["small-signal analysis",
    ButtonData:>"Small-Signal Analysis",
    ButtonStyle->"Hyperlink"],
  " based on linearized averaged models. This tutorial will demonstrate each \
of these applications step-by-step by using a hard-switching PWM Boost \
converter as example."
}], "Text",
  CellTags->"Introduction"],

Cell["\<\
Notice that cells with cyanic background color, such as this one, \
are inputs that depend on the converter being analyzed. Other input cells can \
be used for different converter topologies without any modification.\
\>", 
  "Text",
  Background->RGBColor[0, 1, 1],
  CellTags->"Introduction"],

Cell[TextData[{
  "Notice also that input cells in this tutorial, except in sections ",
  ButtonBox["large-signal transient simulation",
    ButtonData:>"Large-Signal Simulation",
    ButtonStyle->"Hyperlink"],
  ", and ",
  ButtonBox["small-signal analysis",
    ButtonData:>"Small-Signal Analysis",
    ButtonStyle->"Hyperlink"],
  ", are sequential, which means that trying to evaluate a cell without \
having evaluated all preceding cells may not produce the desired results and \
will generate an error message."
}], "Text",
  CellTags->"Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Load the Package\t\t\t\t",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(back\ to\ TOC\),
          ButtonData:>"TOC",
          ButtonStyle->"Hyperlink"], TraditionalForm]],
    FontSize->12,
    FontWeight->"Plain"]
}], "Section",
  CellTags->"Load Package"],

Cell[TextData[{
  "The major functions of the package are implemented in three files:\n\n\"",
  StyleBox["Nodal.m",
    FontWeight->"Bold"],
  "\" for generating circuit equations from the netlist,\n\"",
  StyleBox["Average.m",
    FontWeight->"Bold"],
  "\" for converting the circuit equations into state-space model and \
averaging,\n\"",
  StyleBox["KBM.m",
    FontWeight->"Bold"],
  "\" for ripple estimation using KBM algorithm.\n\nThese files must be \
loaded before you can use any of the functions defined in them (but only once \
when you first start the kernel). Supposing that the files come in a floppy \
diskette A, they can be loaded as below. Depending on where you have put \
these files into your system, you need to modify the path accordingly. (You \
may use \"Get File Path\" under manual \"",
  StyleBox["I",
    FontVariations->{"Underline"->True}],
  "nput\" to get the path of these files.) The program ",
  StyleBox["Graphics.m",
    FontWeight->"Bold"],
  " has actually been developed by Wolfram Research and is included here for \
plotting Bode diagrams of the small-signal averaged model, see subsection ",
  ButtonBox["Bode Diagrams",
    ButtonData:>"Bode Diagram",
    ButtonStyle->"Hyperlink"],
  "."
}], "Text",
  CellTags->"Load Package"],

Cell[BoxData[{
    \(<< NodAl.m\), "\n", 
    \(<< Average.m\), "\n", 
    \(<< KBM.m\), "\n", 
    \(<< Graphic.m\)}], "Input",
  CellTags->"Load Package"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Circuit Model\t\t\t\t\t",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(back\ to\ TOC\),
          ButtonData:>"TOC",
          ButtonStyle->"Hyperlink"], TraditionalForm]],
    FontSize->12,
    FontWeight->"Plain"]
}], "Section",
  CellTags->"Circuit Model"],

Cell[TextData[{
  "The topology of a circuit to be analyzed is given by a ",
  StyleBox["NetList",
    FontSlant->"Italic"],
  ", each element of which corresponds to a component in the circuit. Each \
component of the converter circuit appears in NetList in the following \
format:\n",
  StyleBox["{ComponentType, {Node 1, Node 2}, Symbol}",
    FontWeight->"Bold"],
  "\n\n",
  StyleBox["ComponentType",
    FontWeight->"Bold"],
  " is the type of the component that can be VolatgeSource, Resistor, \
Inductor, Capacitor, or Switch, which represent the most commonly used \
components in power electronic circuits. By definition, a switch has two \
conduction states, ON and OFF, and can represent both controlled (BJT, IGBT, \
MOSFET, GTO, SCR, etc.) and uncontrolled (diode) switches. Notice that all \
these are two-terminal components; transformers and other components with \
more than two terminals will be implemented in the later version. \
(Transformers can be modeled in the current version by using their equivalent \
circuits.)\n\n",
  StyleBox["Node 1",
    FontWeight->"Bold"],
  " and ",
  StyleBox["Node 2",
    FontWeight->"Bold"],
  " are the two nodes to which the component is connected. If there are \
totally ",
  StyleBox["n",
    FontSlant->"Italic"],
  " nodes, they should be numbered as 0, 1, 2, ...., ",
  StyleBox["n",
    FontSlant->"Italic"],
  "-1; but the sequence at which they are numbered can be random. Also notice \
that for each component, the reference directions of current and voltage \
taken by the package are such that a positive current will enter the \
component from node 1 and leave it from node 2 (except for voltage source for \
which a positive current will leave node 1). The voltage across the component \
is taken as the potential of Node 1 minus that of Node 2.\n\n",
  StyleBox["Symbol",
    FontWeight->"Bold"],
  " is the symbol denoting the component that can be chosen arbitrarily. \
Notice that neither numerical parameter values nor the conduction states of \
switches need to be specified in  NetList."
}], "Text",
  CellTags->"Circuit Model"],

Cell["\<\
The NetList for the example Boost converter is as follows. Notice \
that an ESR (equivalent series resistance) has been considered for the output \
capacitor. The two switches, S and D, represent the active switch and the \
diode respectively. Notice that  inside the package the branches are also \
numbered according to the sequence in which they appeared in the \
NetList.\
\>", "Text",
  CellTags->"Circuit Model"],

Cell[CellGroupData[{

Cell["Circuit Description - Boost Converter", "Subsection",
  CellTags->"Circuit Model"],

Cell[BoxData[
    \(\(NetList\  = \ {{VoltageSource, \ {1, \ 0}, \ 
            Vg}, \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
{Inductor, \ {1, \ 2}, \ 
            L}, \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ {Switch, \
\ {2, \ 0}, \ 
            S}, \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ {Switch, \
\ {2, \ 3}, \ 
            D}, \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
{Resistor, \ {3, \ 4}, \ 
            r}, \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
{Capacitor, \ {4, \ 0}, \ 
            C}, \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
{Resistor, \ {3, \ 0}, \ R}};\)\)], "Input",
  Background->RGBColor[0, 1, 1],
  CellTags->{"NetList", "Circuit Model"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Converter Input", "Subsection",
  CellTags->"State-Space Modeling"],

Cell[TextData[{
  "Supposing that the converter undergoes ",
  StyleBox["m",
    FontSlant->"Italic"],
  " topological states in each switching cycle, there will be ",
  StyleBox["m",
    FontSlant->"Italic"],
  " different state-space models:\n\td",
  StyleBox["x",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  "/d",
  StyleBox["t",
    FontSlant->"Italic"],
  " = ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["A",
            FontWeight->"Bold"], "k"], TraditionalForm]]],
  StyleBox[" ",
    FontSlant->"Italic"],
  StyleBox["x",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  " + ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["B",
            FontWeight->"Bold"], "k"], TraditionalForm]]],
  StyleBox[" ",
    FontSlant->"Italic"],
  StyleBox["u",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "(",
  StyleBox["k ",
    FontSlant->"Italic"],
  "= 1, 2, .....,",
  StyleBox[" m",
    FontSlant->"Italic"],
  ")\nwhere ",
  StyleBox["x",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  " is the vector of state variables, and ",
  StyleBox["u",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  " is the vector of input variables. In most cases, a converter has only the \
input voltage as its input. This is specified by another list called ",
  StyleBox["InputVars",
    FontSlant->"Italic"],
  ", as given below for the example Boost converter:"
}], "Text",
  CellTags->"State-Space Modeling"],

Cell[BoxData[
    StyleBox[\(InputVars = {Vg};\),
      Background->RGBColor[0, 1, 1]]], "Input",
  Background->RGBColor[0, 1, 1],
  CellTags->"State-Space Modeling"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "State-Space Modeling\t\t\t",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(back\ to\ TOC\),
          ButtonData:>"TOC",
          ButtonStyle->"Hyperlink"], TraditionalForm]],
    FontSize->12,
    FontWeight->"Plain"]
}], "Section",
  CellTags->"State-Space Modeling"],

Cell[CellGroupData[{

Cell["Conduction States of Switches", "Subsection",
  CellTags->"State-Space Modeling"],

Cell[TextData[{
  "Since the state-space models have different system and input matrices in \
different topological states, they need to be derived individually. The \
topological states for which state-space models are to be derived are \
specified by a list called ",
  StyleBox["SwitchStates",
    FontSlant->"Italic"],
  ". For the example Boost converter, there are two different topological \
states when the converter operates in continuous conduction mode (CCM): \
S-ON/D-OFF and S-OFF/D-ON. Hence the list ",
  StyleBox["SwitchStates",
    FontSlant->"Italic"],
  " is"
}], "Text",
  CellTags->"State-Space Modeling"],

Cell[BoxData[
    \(\(SwitchStates\  = \ {{S\  -> \ 1, \ D\  -> \ 0}, \ {S\  -> \ 0, \ 
            D\  -> \ 1}};\)\)], "Input",
  Background->RGBColor[0, 1, 1],
  CellTags->"State-Space Modeling"]
}, Open  ]],

Cell[CellGroupData[{

Cell["State-Space Models", "Subsection",
  CellTags->"State-Space Modeling"],

Cell[TextData[{
  "Based on the converter circuit description, the package derives a \
state-space model for the converter in each topological state in four steps:\n\
\n1) Derive a set of circuit equations from the NetList by using the function \
",
  StyleBox["NodalAnalysis",
    FontWeight->"Bold"],
  " defined in \"",
  StyleBox["Nodal.m",
    FontWeight->"Bold"],
  "\", which is based on a method similar to the modified nodal analysis \
technique;\n2) Eliminating all variables except inductor currents and \
capacitor voltages (which are retained as state variables) by using function \
",
  StyleBox["StateSpace",
    FontWeight->"Bold"],
  ";\n3) Rearranging the remaining equation in the form of standard \
state-space model as\n\t",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["dx",
            FontSlant->"Italic"], "i"], TraditionalForm]]],
  "/",
  StyleBox["dt",
    FontSlant->"Italic"],
  " = ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["a", 
          StyleBox[
            RowBox[{
              StyleBox["i",
                FontSlant->"Italic"], "1"}]]], TraditionalForm]]],
  " ",
  Cell[BoxData[
      \(TraditionalForm\`x\_1\)]],
  " + ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["a", 
          StyleBox[
            RowBox[{
              StyleBox["i",
                FontSlant->"Italic"], "2"}]]], TraditionalForm]]],
  " ",
  Cell[BoxData[
      \(TraditionalForm\`x\_2\)]],
  " + .... + ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["a", 
          StyleBox["in",
            FontSlant->"Italic"]], TraditionalForm]]],
  " ",
  Cell[BoxData[
      \(TraditionalForm\`x\_n\)]],
  " + ",
  StyleBox[" ",
    FontSlant->"Italic"],
  Cell[BoxData[
      \(TraditionalForm\`b\_i\)]],
  StyleBox[" u",
    FontSlant->"Italic"],
  ";\n4) Determining both system matrix ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["A",
            FontWeight->"Bold"], "k"], TraditionalForm]]],
  " = [",
  StyleBox["a",
    FontSlant->"Italic"],
  Cell[BoxData[
      FormBox[
        SubscriptBox["", 
          StyleBox["ij",
            FontSlant->"Italic"]], TraditionalForm]]],
  "] and input matrix ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["B",
            FontWeight->"Bold"], "k"], TraditionalForm]]],
  " = [",
  StyleBox["b",
    FontSlant->"Italic"],
  Cell[BoxData[
      FormBox[
        SubscriptBox["", 
          StyleBox["i",
            FontSlant->"Italic"]], TraditionalForm]]],
  "].\n\nThe results are put in two lists, a list of system matrices ",
  StyleBox["A",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  " = ",
  Cell[BoxData[
      FormBox[
        RowBox[{"{", 
          SubscriptBox[
            StyleBox["A",
              FontWeight->"Bold"], "1"]}], TraditionalForm]]],
  ", ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["A",
            FontWeight->"Bold"], "2"], TraditionalForm]]],
  ", ...., ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["A",
            FontWeight->"Bold"], "m"], TraditionalForm]]],
  "} and a list of input matrices ",
  StyleBox["B",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  " = ",
  Cell[BoxData[
      FormBox[
        RowBox[{"{", 
          SubscriptBox[
            StyleBox["B",
              FontWeight->"Bold"], "1"]}], TraditionalForm]]],
  ", ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["B",
            FontWeight->"Bold"], "2"], TraditionalForm]]],
  ", ...., ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["B",
            FontWeight->"Bold"], "m"], TraditionalForm]]],
  "}. These are performed by using the function ",
  StyleBox["StateSpaceModel",
    FontWeight->"Bold"],
  ", as illustrated below. The function also returns a list (",
  StyleBox["StateVars",
    FontSlant->"Italic"],
  "), that shows the correspondance between the selected state space \
variables and the circuit variables (inductor currents and capacitor \
voltages)."
}], "Text",
  CellTags->"State-Space Modeling"],

Cell[CellGroupData[{

Cell[BoxData[
    \({A, \ B, \ StateVars}\  = \ 
      StateSpaceModel[NetList, \ InputVars, \ SwitchStates]\)], "Input",
  CellTags->"State-Space Modeling"],

Cell[BoxData[
    \({\(ib[2]\)[t] \[Rule] \(x[1]\)[t], \(vn[4]\)[t] \[Rule] \(x[2]\)[
          t]}\)], "Output"],

Cell[BoxData[
    \({{{{0, 
            0}, {0, \(-\(1\/\(C\ \((r + 
                        R)\)\)\)\)}}, {{\(-\(\(r\ R\)\/\(L\ \((r + 
                        R)\)\)\)\), \(-\(R\/\(L\ \((r + 
                        R)\)\)\)\)}, {R\/\(C\ \((r + R)\)\), \(-\(1\/\(C\ \
\((r + R)\)\)\)\)}}}, {{{1\/L}, {0}}, {{1\/L}, {0}}}, {\(x[1]\)[t], \(x[2]\)[
          t]}}\)], "Output"]
}, Open  ]],

Cell["For the example Boost converter, the results are:", "Text",
  CellTags->"State-Space Modeling"],

Cell[CellGroupData[{

Cell[BoxData[
    \(A\)], "Input",
  CellTags->"State-Space Modeling"],

Cell[BoxData[
    \({{{0, 
          0}, {0, \(-\(1\/\(C\ \((r + 
                      R)\)\)\)\)}}, {{\(-\(\(r\ R\)\/\(L\ \((r + 
                      R)\)\)\)\), \(-\(R\/\(L\ \((r + 
                      R)\)\)\)\)}, {R\/\(C\ \((r + R)\)\), \(-\(1\/\(C\ \((r \
+ R)\)\)\)\)}}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(B\)], "Input",
  CellTags->"State-Space Modeling"],

Cell[BoxData[
    \({{{1\/L}, {0}}, {{1\/L}, {0}}}\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Averaged Modeling\t\t\t\t",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(back\ to\ TOC\),
          ButtonData:>"TOC",
          ButtonStyle->"Hyperlink"], TraditionalForm]],
    FontSize->12,
    FontWeight->"Plain"]
}], "Section",
  CellTags->{"Averaging", "Averaged Modeling"}],

Cell[TextData[{
  "The averaging method implemented in the package is a general one which is \
based on partitioning the converter model into a ",
  StyleBox["slow subsystem",
    FontSlant->"Italic"],
  " and a ",
  StyleBox["fast subsystem",
    FontSlant->"Italic"],
  ". The slow subsystem contains the main energy-storage elements of the \
converter as well as its input and output sections, while switching devices \
and the associated resonant tank (and snubber circuits, when need to be \
considered) comprise the fast subsystem ",
  ButtonBox["Reference [1]",
    ButtonData:>"Reference [1]",
    ButtonStyle->"Hyperlink"],
  ". Based on the partitioning, an averaged model is then derived in three \
steps:\n\n1) ",
  ButtonBox["Relaxation",
    ButtonData:>"Relaxation",
    ButtonStyle->"Hyperlink"],
  ": The fast subsystem is first analyzed by assuming that the slow variables \
are constant over a switching cycle; the results are closed-form solutions \
for the fast variables over one switching cycle.\n\n2) ",
  ButtonBox["Decoupling",
    ButtonData:>"Decoupling",
    ButtonStyle->"Hyperlink"],
  ": The analytical solutions for fast variables calculated in 1) are \
substituted into the state-space model of the slow variables. This leads to a \
model for the slow subsystem which is decoupled from the fast subsystem and \
can be analyzed separately.\n\n3) ",
  ButtonBox["Averaging",
    ButtonData:>"Averaging",
    ButtonStyle->"Hyperlink"],
  ": The resulting slow subsystem model is time-varying due to the presence \
of time-varying solutions of the fast variables solved in 1). The \
time-varying terms can be removed by averaging the model over one switching \
cycle. This last step then yields the desired time-invariant, continuous, \
nonlinear averaged model describing the slow dynamic of the converter.\n\n\
Details on this general averaging method can be found in ",
  ButtonBox["Reference [1]",
    ButtonData:>"Reference [1]",
    ButtonStyle->"Hyperlink"],
  "."
}], "Text",
  CellTags->{"Averaging", "Averaged Modeling"}],

Cell[CellGroupData[{

Cell["Additional Inputs", "Subsection",
  CellTags->{"Averaging", "Averaged Modeling"}],

Cell[TextData[{
  "In addition to the ",
  ButtonBox["state-space model",
    ButtonData:>"State-Space Modeling",
    ButtonStyle->"Hyperlink"],
  ", the package needs some other inputs for deriving the averaged model. \
These additional information are provided by lists ",
  ButtonBox["SlowFast",
    ButtonData:>"SlowFast",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["BCs",
    ButtonData:>"BCs",
    ButtonStyle->"Hyperlink"],
  ", and ",
  ButtonBox["IV",
    ButtonData:>"IV",
    ButtonStyle->"Hyperlink"],
  ".\n\n",
  ButtonBox["SlowFast",
    ButtonData:>"SlowFast",
    ButtonStyle->"Hyperlink"],
  " = {",
  StyleBox["SlowVarIndex",
    FontSlant->"Italic"],
  ", ",
  StyleBox["FastVarIndex",
    FontSlant->"Italic"],
  "} tells the package which state variables are slow variables (",
  StyleBox["SlowVarIndex",
    FontSlant->"Italic"],
  ") and which are fast (",
  StyleBox["FastVarIndex",
    FontSlant->"Italic"],
  "). Both ",
  StyleBox["SlowVarIndex",
    FontSlant->"Italic"],
  " and ",
  StyleBox["FastVarIndex",
    FontSlant->"Italic"],
  " are again lists of the sequence numbers of the slow and fast variables as \
they appear in the state variable vector ",
  StyleBox["x",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  ". For the example Boost converter, both state variables ",
  Cell[BoxData[
      \(TraditionalForm\`i\_\(L\ \)\)]],
  "and ",
  Cell[BoxData[
      \(TraditionalForm\`v\_C\)]],
  " are slow variables, hence"
}], "Text",
  CellTags->{"Averaging", "Averaged Modeling"}],

Cell[BoxData[
    \(\(SlowFast\  = \ {{1, \ 2}, \ {}};\)\)], "Input",
  Background->RGBColor[0, 1, 1],
  CellTags->{"SlowFast", "Averaging", "Averaged Modeling"}],

Cell[TextData[{
  "The list ",
  ButtonBox["BCs",
    ButtonData:>"SlowFast",
    ButtonStyle->"Hyperlink"],
  " specifies the boundary conditions under which the circuit changes its \
topological state (switching). Three different types of boundary conditions \
can be implemented:\n\n",
  StyleBox["Absolute Time",
    FontSlant->"Italic"],
  ": The switching takes place at a fixed time measured from the beginning of \
a switching cycle. In this case, the corresponding element in ",
  StyleBox["BCs",
    FontSlant->"Italic"],
  " is a number, a symbol, or an expression representing the time. For \
example, the first and the last element of ",
  StyleBox["BCs",
    FontSlant->"Italic"],
  " are usually 0 and ",
  StyleBox["Ts",
    FontSlant->"Italic"],
  ", respectively, indicating the beginning and the end of a switching cycle, \
",
  StyleBox["Ts",
    FontSlant->"Italic"],
  " being the switching period of the converter.\n\n",
  StyleBox["Relative Time",
    FontSlant->"Italic"],
  ": The next switching is delayed by a certain time from the previous one. \
The corresponding element in ",
  StyleBox["BCs",
    FontSlant->"Italic"],
  " is a list of single element representing the delay time. For example, for \
hard-switching PWM converters in continuous conduction mode and with \
duty-cycle control, BCs can be written either as\n\t",
  StyleBox["BCs",
    FontSlant->"Italic"],
  " = {0, ",
  StyleBox["d",
    FontSlant->"Italic"],
  "*",
  StyleBox["Ts",
    FontSlant->"Italic"],
  ", {(1-",
  StyleBox["d",
    FontSlant->"Italic"],
  ")*",
  StyleBox["Ts",
    FontSlant->"Italic"],
  "}}\nwhere 0 and ",
  StyleBox["d",
    FontSlant->"Italic"],
  "*",
  StyleBox["Ts",
    FontSlant->"Italic"],
  " are absolute times and {(1-",
  StyleBox["d",
    FontSlant->"Italic"],
  ")*",
  StyleBox["Ts",
    FontSlant->"Italic"],
  "} represents a relative time, or\n\t",
  StyleBox["BCs",
    FontSlant->"Italic"],
  " = {0, ",
  StyleBox["d",
    FontSlant->"Italic"],
  "*",
  StyleBox["Ts",
    FontSlant->"Italic"],
  ", ",
  StyleBox["Ts",
    FontSlant->"Italic"],
  "}\nwhere all times are absolute, i.e., measured from the beginning of the \
switching cycle.\n\n",
  StyleBox["Implicit Conditions",
    FontSlant->"Italic"],
  ": This is the case when switching is dependent of state variables. For \
example, the conduction of a diode depends on the polarity of the voltage \
across it. In this case, the corresponding element in BCs is a list of two \
expressions: {exp1, exp2}, and the switching instant is to be solved from the \
equation exp1 = exp2. Both expressions may be functions of slow variables, \
and at least one is dependent of the fast variables. The solution is a \
relative time, i.e., it represents the delay from the previous switching.\n\n\
As discussed before, ",
  StyleBox["BCs",
    FontSlant->"Italic"],
  " for the example Boost converter is"
}], "Text",
  CellTags->{"Averaging", "Averaged Modeling"}],

Cell[BoxData[
    \(\(BCs\  = \ {0, \ \((d\ Ts)\), \ Ts};\)\)], "Input",
  Background->RGBColor[0, 1, 1],
  CellTags->{"SlowFast", "BCs", "Averaging", "Averaged Modeling"}],

Cell[TextData[{
  "Beside ",
  StyleBox["SlowFast",
    FontSlant->"Italic"],
  " and ",
  StyleBox["BCs",
    FontSlant->"Italic"],
  ", it is also necessary to specify the initial values of the fast state \
variables at the beginning of the switching cycle (by the list ",
  ButtonBox["IV",
    ButtonData:>"IV",
    ButtonStyle->"Hyperlink"],
  "). Each initial value can be given as either a number, a symbol, or a \
function dependent of slow variables and converter inputs. Since there is no \
fast variables in the example Boost converter, IV is given as an empty list:"
}], "Text",
  CellTags->{"Averaging", "Averaged Modeling"}],

Cell[BoxData[
    \(\(IV\  = \ {};\)\)], "Input",
  Background->RGBColor[0, 1, 1],
  CellTags->{"IV", "Averaging", "Averaged Modeling"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Averaged Modeling", "Subsection",
  CellTags->{"Averaging", "Averaged Modeling"}],

Cell[TextData[{
  "With the additional inputs given above and the state-space models in all \
topological states, an averaged model will be derived using three major \
functions: ",
  ButtonBox["Relaxation",
    ButtonData:>"Relaxation",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["Decoupling",
    ButtonData:>"Decoupling",
    ButtonStyle->"Hyperlink"],
  ", and ",
  ButtonBox["Averaging",
    ButtonData:>"Averaging",
    ButtonStyle->"Hyperlink"],
  ". The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " inputs for carrying out these are as follows, which can also be used for \
other converters without any modifications. Note that you MUST go through all \
these inputs, even the relaxation and decopuling steps may seem not to be \
necessary for converter topologies."
}], "Text",
  CellTags->{"Averaging", "Averaged Modeling"}],

Cell[BoxData[{
    \(\(ns\  = \ Length[SlowFast[\([1]\)]];\)\), "\n", 
    \(\(nf\  = \ Length[SlowFast[\([2]\)]];\)\), "\n", 
    \(\(SlowVars\  = \ 
        Table[StateVars[\([SlowFast[\([1, \ i]\)]]\)]\  /. \ {t\  -> \ 
                t0}, \ {i, \ 1, \ ns}];\)\), "\n", 
    \(\(FastVars\  = \ 
        If[nf == 0, \ {}, \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ Table[
            StateVars[\([SlowFast[\([2, \ i]\)]]\)], \ {i, \ 1, \ 
              ns}]];\)\)}], "Input",
  CellTags->{"SlowVars", "Averaging", "Averaged Modeling"}],

Cell[CellGroupData[{

Cell["Relaxation", "Subsubsection",
  CellTags->{"Relaxation", "Averaging", "Averaged Modeling"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(FastSolus\  = \ 
        Relaxation[A, \ B, \ InputVars, \ SlowFast, \ SlowVars, \ FastVars, \ 
          BCs, \ IV];\)\)], "Input",
  CellTags->{"Relaxation", "Averaging", "Averaged Modeling"}],

Cell[BoxData[
    \(First::"first" \(\(:\)\(\ \)\) 
      "\!\({}\) has a length of zero and no first element."\)], "Message"],

Cell[BoxData[
    \(First::"first" \(\(:\)\(\ \)\) 
      "\!\({}\) has a length of zero and no first element."\)], "Message"],

Cell[BoxData[
    \(First::"first" \(\(:\)\(\ \)\) 
      "\!\({}\) has a length of zero and no first element."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(First :: \"first\"\) will be suppressed during \
this calculation."\)], "Message"]
}, Open  ]],

Cell["\<\
This input may generate some error messages if the 2nd element of \
SlowFast is an empty list, in other words, when all state variables are slow \
variables, such as in the case of hard-switching PWM converters. But they \
will not affect the final results.\
\>", "Text",
  CellTags->{"Averaging", "Averaged Modeling"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Decoupling", "Subsubsection",
  CellTags->{"Decoupling", "Averaging", "Averaged Modeling"}],

Cell[BoxData[
    \(\(SlowEqus\  = \ 
        Decoupling[FastSolus, \ A, \ B, \ 
          SlowFast, \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
SlowVars, \ InputVars, \ FastSolus[\([3]\)]];\)\)], "Input",
  CellTags->{"Decoupling", "Averaging", "Averaged Modeling"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Averaging", "Subsubsection",
  CellTags->{"Averaging", "Averaged Modeling"}],

Cell[TextData[{
  "It is assumed here that the last element in ",
  ButtonBox["BCs",
    ButtonData:>"BCs",
    ButtonStyle->"Hyperlink"],
  " is Ts, the period of one switching cycle."
}], "Text"],

Cell[BoxData[
    \(\(AveragedModel\  = \ 
        Averaging[SlowEqus, BCs[\([\(-1\)]\)]];\)\)], "Input",
  CellTags->{"Averaging", "Averaged Modeling"}],

Cell[TextData[{
  "The final averaged model is contained in the list ",
  StyleBox["AveragedModel",
    FontSlant->"Italic"],
  " whose ",
  StyleBox["i",
    FontSlant->"Italic"],
  "th element is the righthand-side function, ",
  Cell[BoxData[
      \(TraditionalForm\`f\_i\)]],
  "(",
  StyleBox["x",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  ", ",
  StyleBox["u",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  "), of the averaged model ",
  Cell[BoxData[
      FormBox[
        RowBox[{"d", 
          SubscriptBox[
            StyleBox[
              OverscriptBox[
                StyleBox["x",
                  FontSlant->"Italic"], "_"],
              FontSlant->"Italic"], "i"]}], TraditionalForm]]],
  "/d",
  StyleBox["t",
    FontSlant->"Italic"],
  " = ",
  Cell[BoxData[
      \(TraditionalForm\`f\_i\)]],
  "(",
  StyleBox["x",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  ", ",
  StyleBox["u",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  ") for the ",
  StyleBox["i",
    FontSlant->"Italic"],
  "th slow variable ",
  Cell[BoxData[
      \(TraditionalForm\`x\_i\)]],
  ".\n\nFor the example Boost converter, both state variables x[1] (the \
inductor current, ",
  Cell[BoxData[
      \(TraditionalForm\`i\_L\)]],
  ") and x[2] (the capacitor voltage, ",
  Cell[BoxData[
      \(TraditionalForm\`v\_C\)]],
  ") are slow variable. By substituting x[1] with",
  StyleBox[" ",
    FontSlant->"Italic"],
  Cell[BoxData[
      \(TraditionalForm\`i\_L\)]],
  " and x[2] with ",
  Cell[BoxData[
      \(TraditionalForm\`v\_C\)]],
  ", the righthand sides of the averaged model become"
}], "Text",
  CellTags->{"Averaging", "Averaged Modeling"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      AveragedModel\  /. \ {\(x[1]\)[t0]\  -> \ iL, \ \(x[2]\)[t0]\  -> \ 
            vC}]\)], "Input",
  CellTags->{"Averaged Model", "Averaging", "Averaged Modeling"}],

Cell[BoxData[
    \({\(\((\(-1\) + d)\)\ iL\ r\ R + \((\(-1\) + d)\)\ R\ vC + \((r + R)\)\ \
Vg\)\/\(L\ \((r + R)\)\), \(iL\ R - d\ iL\ R - vC\)\/\(C\ r + C\ R\)}\)], \
"Output"]
}, Open  ]],

Cell[TextData[{
  "which means that the averaged model of the converter is\n\n\t",
  Cell[BoxData[
      FormBox[
        FractionBox[
          FormBox[
            SubscriptBox[
              StyleBox["di",
                FontSlant->"Italic"], "L"],
            "TraditionalForm"], 
          StyleBox["dt",
            FontSlant->"Italic"]], TraditionalForm]]],
  "= ",
  Cell[BoxData[
      FractionBox[
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{\(-1\), "+", 
                StyleBox["d",
                  FontSlant->"Italic"]}], ")"}], " ", 
            SubscriptBox[
              StyleBox["i",
                FontSlant->"Italic"], 
              StyleBox["L",
                FontSlant->"Italic"]], " ", 
            StyleBox["r",
              FontSlant->"Italic"], 
            StyleBox[" ",
              FontSlant->"Italic"], 
            StyleBox["R",
              FontSlant->"Italic"]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{\(-1\), "+", 
                StyleBox["d",
                  FontSlant->"Italic"]}], ")"}], " ", 
            StyleBox["R",
              FontSlant->"Italic"], " ", \(v\_C\)}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                StyleBox["r",
                  FontSlant->"Italic"], "+", 
                StyleBox["R",
                  FontSlant->"Italic"]}], ")"}], " ", "Vg"}]}], 
        RowBox[{
          StyleBox["L",
            FontSlant->"Italic"], " ", 
          RowBox[{"(", 
            RowBox[{
              StyleBox["r",
                FontSlant->"Italic"], 
              StyleBox["+",
                FontSlant->"Plain"], 
              StyleBox["R",
                FontSlant->"Italic"]}], ")"}]}]]]],
  ",   ",
  Cell[BoxData[
      FormBox[
        FractionBox[
          FormBox[
            SubscriptBox[
              StyleBox[
                RowBox[{
                  StyleBox["d",
                    FontSlant->"Italic"], "v"}]], "C"],
            "TraditionalForm"], 
          StyleBox["dt",
            FontSlant->"Italic"]], TraditionalForm]]],
  "= ",
  Cell[BoxData[
      FractionBox[
        RowBox[{
          RowBox[{
            SubscriptBox[
              StyleBox["i",
                FontSlant->"Italic"], 
              StyleBox["L",
                FontSlant->"Italic"]], " ", 
            RowBox[{"(", 
              StyleBox[\(R - d\ R\),
                FontSlant->"Italic"], ")"}]}], "-", \(v\_C\)}], 
        RowBox[{
          StyleBox["C",
            FontSlant->"Italic"], " ", 
          RowBox[{"(", 
            RowBox[{
              StyleBox["r",
                FontSlant->"Italic"], "+", 
              StyleBox["R",
                FontSlant->"Italic"]}], ")"}]}]]]],
  "\n\nNotice that this model is the same as the conventional state-space \
averaged model. In fact, it can be shown that the conventional state-space \
averaging is the special case of the general method applied to hard-switching \
PWM DC/DC converters."
}], "Text",
  CellTags->"Averaged Model"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Ripple Estimation\t\t\t\t",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(back\ to\ TOC\),
          ButtonData:>"TOC",
          ButtonStyle->"Hyperlink"], TraditionalForm]],
    FontSize->12,
    FontWeight->"Plain"]
}], "Section",
  CellTags->{"Ripple Functions", "Ripple Estimation"}],

Cell[TextData[{
  "Averaged models of switching power converters have proved to be very \
useful for fast transient response analysis and for control loop design. \
However, they do not provide any details for the current and voltage waveform \
within each switching cycle. To overcome this problem, the classical method \
of Krylov-Bogoliubov-Miltropolsky (KBM) ",
  ButtonBox["Reference [2]",
    ButtonData:>"Reference [2]",
    ButtonStyle->"Hyperlink"],
  " can be used to find ripple corrections for the slow state variables as \
well as other variables of interests.\n\nFor a variable ",
  StyleBox["x",
    FontSlant->"Italic"],
  "(",
  StyleBox["t",
    FontSlant->"Italic"],
  "), the KBM method can be applied to calculate a series of ripple functions \
",
  Cell[BoxData[
      FormBox[
        RowBox[{\(x\&^\^1\), 
          RowBox[{"(", 
            StyleBox["t",
              FontSlant->"Italic"], ")"}]}], TraditionalForm]]],
  ",  ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(x\&^\^2\), 
          RowBox[{"(", 
            StyleBox["t",
              FontSlant->"Italic"], ")"}]}], TraditionalForm]]],
  ", ......, ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(x\&^\^i\), 
          RowBox[{"(", 
            StyleBox["t",
              FontSlant->"Italic"], ")"}]}], TraditionalForm]]],
  ", ......, such that\n\t",
  StyleBox["x",
    FontSlant->"Italic"],
  "(",
  StyleBox["t",
    FontSlant->"Italic"],
  ") = ",
  Cell[BoxData[
      \(TraditionalForm\`x\&_\)]],
  "(",
  StyleBox["t",
    FontSlant->"Italic"],
  ") + ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(x\&^\^1\), 
          RowBox[{"(", 
            StyleBox["t",
              FontSlant->"Italic"], ")"}]}], TraditionalForm]]],
  " + ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(x\&^\^2\), 
          RowBox[{"(", 
            StyleBox["t",
              FontSlant->"Italic"], ")"}]}], TraditionalForm]]],
  " + ...... + ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(x\&^\^i\), 
          RowBox[{"(", 
            StyleBox["t",
              FontSlant->"Italic"], ")"}]}], TraditionalForm]]],
  " + ......\nwhere ",
  Cell[BoxData[
      \(TraditionalForm\`x\&_\)]],
  "(",
  StyleBox["t",
    FontSlant->"Italic"],
  ") is the local average of ",
  StyleBox["x",
    FontSlant->"Italic"],
  "(",
  StyleBox["t",
    FontSlant->"Italic"],
  ") over one switching cycle, as determined by the averaged model. Function \
",
  Cell[BoxData[
      FormBox[
        RowBox[{\(x\&^\^i\), 
          RowBox[{"(", 
            StyleBox["t",
              FontSlant->"Italic"], ")"}]}], TraditionalForm]]],
  " (",
  StyleBox["i",
    FontSlant->"Italic"],
  " = 1, 2, ......) is piecewise continuous and is an ",
  StyleBox["i",
    FontSlant->"Italic"],
  "th-order polynomials in each topological state, hence it is also called \
the ",
  StyleBox["i",
    FontSlant->"Italic"],
  "th-order ripple correction/estimation for ",
  StyleBox["x",
    FontSlant->"Italic"],
  "(",
  StyleBox["t",
    FontSlant->"Italic"],
  "). See ",
  ButtonBox["Reference [2]",
    ButtonData:>"Reference [2]",
    ButtonStyle->"Hyperlink"],
  " and ",
  ButtonBox["Reference[3]",
    ButtonData:>"Reference [3]",
    ButtonStyle->"Hyperlink"],
  " for details of the KBM method and the ripple estimation procedure."
}], "Text",
  CellTags->{"Ripple Functions", "Ripple Estimation"}],

Cell[TextData[{
  "The function ",
  StyleBox["KBM",
    FontWeight->"Bold"],
  " defined in file ",
  ButtonBox["KBM.m",
    ButtonData:>"Load Package",
    ButtonStyle->"Hyperlink"],
  " implements the KBM method and requires four arguments: the decoupled \
model of the slow subsystem (",
  ButtonBox["SlowEqus",
    ButtonData:>"Decoupling",
    ButtonStyle->"Hyperlink"],
  "), the averaged model (",
  ButtonBox["AveragedModel",
    ButtonData:>"Averaging",
    ButtonStyle->"Hyperlink"],
  "), the definition of slow variables (",
  ButtonBox["SlowVars",
    ButtonData:>"SlowVars",
    ButtonStyle->"Hyperlink"],
  "), and the required order of ripple corrections (",
  StyleBox["n",
    FontSlant->"Italic"],
  "). Since all ",
  ButtonBox["SlowEqus",
    ButtonData:>"Decoupling",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["AveragedModel",
    ButtonData:>"Averaging",
    ButtonStyle->"Hyperlink"],
  ", and ",
  ButtonBox["SlowVars",
    ButtonData:>"SlowVars",
    ButtonStyle->"Hyperlink"],
  " have been defined in the previous section for averaged modeling, no new \
inputs are needed for using the function KBM. \nThe following input cell \
determines the first and the second ripple corrections (",
  StyleBox["n",
    FontSlant->"Italic"],
  " = 2) and put the results in ",
  StyleBox["Ripple",
    FontSlant->"Italic"],
  ":"
}], "Text",
  CellTags->{"Ripple Functions", "Ripple Estimation"}],

Cell[BoxData[
    \(\(Ripple\  = \ 
        KBM[SlowEqus, \ AveragedModel, SlowVars, \ 2];\)\)], "Input",
  CellTags->{"Ripple Functions", "Ripple Estimation"}],

Cell[TextData[{
  "The format of the output of the function ",
  StyleBox["KBM",
    FontWeight->"Bold"],
  " (here assigned to ",
  StyleBox["Ripple",
    FontSlant->"Italic"],
  ") is as follows:\n{{{1st-order corrections of slow state variables in the \
1st topological state}, \n   {1st-order corrections of slow state variables \
in the 2nd topological state}, ......,\n   {1st-order corrections of slow \
state variables in the last topological state}},\n  {{2nd-order corrections \
of slow state variables in the 1st topological state}, \n   {2nd-order \
corrections of slow state variables in the 2nd topological state}, ......,\n  \
 {2nd-order corrections of slow state variables in the last topological \
state}},\n  ......,\n {{",
  StyleBox["n",
    FontSlant->"Italic"],
  "th-order corrections of slow state variables in the 1st topological \
state}, \n   {",
  StyleBox["n",
    FontSlant->"Italic"],
  "th-order corrections of slow state variables in the 2nd topological \
state}, ......,\n   {",
  StyleBox["n",
    FontSlant->"Italic"],
  "th-order corrections of slow state variables in the last topological \
state}}}\n\nFor the example Boost converter, the output with ",
  StyleBox["n",
    FontSlant->"Italic"],
  " = 2 is as follows where the ESR of the capacitor is set to 0 in order to \
limit the length of the expressions and the internally used state variables \
x[1] and x[2] are replaced by their corresponding current ",
  Cell[BoxData[
      \(TraditionalForm\`i\_L\)]],
  " and voltage ",
  Cell[BoxData[
      \(TraditionalForm\`v\_C\)]],
  " for easy understanding. Also notice that the time variable, ",
  StyleBox["t",
    FontSlant->"Italic"],
  ", is reset to zero at the beginning of each switching cycle."
}], "Text",
  CellTags->{"Ripple Functions", "Ripple Estimation"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      Ripple\  /. \ {r\  -> \ 0, \ \(x[1]\)[t0]\  -> \ 
            iL, \ \(x[2]\)[t0]\  -> \ vC}]\)], "Input",
  CellTags->{"Ripple Functions", "Ripple Estimation"}],

Cell[BoxData[
    \({{{\(\((\(-1\) + d)\)\ vC + Vg\)\/L, \(iL\ R - d\ iL\ R - vC\)\/\(C\ \
R\)}, {0, 
          0}}, {{{\(\((\(-1\) + d)\)\ \((\(-2\)\ t + d\ Ts)\)\ vC\)\/\(2\ \
L\), \(-\(\(\((\(-1\) + d)\)\ iL\ \((\(-2\)\ t + 
                        d\ Ts)\)\)\/\(2\ C\)\)\)}, {\(d\ \((\(-2\)\ t + Ts + \
d\ Ts)\)\ vC\)\/\(2\ L\), \(-\(\(d\ iL\ \((\(-2\)\ t + Ts + 
                        d\ Ts)\)\)\/\(2\ C\)\)\)}}, {{\(-\(\(\((\(-1\) + 
                        d)\)\ \((6\ t\^2 - 6\ d\ t\ Ts + 
                        d\ \((\(-1\) + 2\ d)\)\ Ts\^2)\)\ \((\((\(-1\) + 
                              d)\)\ iL\ R + 
                        vC)\)\)\/\(12\ C\ L\ R\)\)\), \(-\(\(\((\(-1\) + 
                        d)\)\ \((6\ t\^2 - 6\ d\ t\ Ts + 
                        d\ \((\(-1\) + 2\ d)\)\ Ts\^2)\)\ \((\((\(-1\) + 
                              d)\)\ vC + 
                        Vg)\)\)\/\(12\ C\ L\)\)\)}, {\(-\(\(d\ \((6\ t\^2 - 
                        6\ \((1 + d)\)\ t\ Ts + \((1 + 3\ d + 
                              2\ d\^2)\)\ Ts\^2)\)\ \((\((\(-1\) + 
                              d)\)\ iL\ R + 
                        vC)\)\)\/\(12\ C\ L\ R\)\)\), \(-\(\(d\ \((6\ t\^2 - 
                        6\ \((1 + d)\)\ t\ Ts + \((1 + 3\ d + 
                              2\ d\^2)\)\ Ts\^2)\)\ \((\((\(-1\) + d)\)\ vC + 
                        Vg)\)\)\/\(12\ C\ L\)\)\)}}}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Ripple estimations extend the use of averaged models to the design of \
converter power stage. Using the estimated ripple functions, proper values \
can be determined for the main energy-storage LC elements of the converter to \
satisfy the design specifications for current and voltage ripples. The \
estimated ripple functions also provide information about peak current and \
voltage of each power semiconductor device, allowing optimal selection of \
device ratings and more accurate estimation of their switching and conduction \
losses. Please refer to ",
  ButtonBox["Reference [4]",
    ButtonData:>"Reference [4]",
    ButtonStyle->"Hyperlink"],
  " for more details.\n\nAs demonstrated in ",
  ButtonBox["Reference [3]",
    ButtonData:>"Reference [3]",
    ButtonStyle->"Hyperlink"],
  ", the estimated ripple functions can also be used to derive \
frequency-dependent averaged models for PWM DC/DC converters that correctly \
predict the stability property of closed-loop system when linear state \
feedback control is applied. The essence of the approach is a more accurate \
model for the pulse-width modulator that takes into account the ripple \
components of the state feedback signals, not just their average values. \
Other applications of ripple estimations in converter modeling and analysis \
will be reported soon by the author (",
  ButtonBox["Reference [5]",
    ButtonData:>"Reference [5]",
    ButtonStyle->"Hyperlink"],
  ")."
}], "Text",
  CellTags->{"Ripple Functions", "Ripple Estimation"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Large-Signal Simulation\t\t\t",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(back\ to\ TOC\),
          ButtonData:>"TOC",
          ButtonStyle->"Hyperlink"], TraditionalForm]],
    FontSize->12,
    FontWeight->"Plain"]
}], "Section",
  CellTags->"Large-Signal Simulation"],

Cell[TextData[{
  "This section demonstrates the applications of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " for large-signal transient response simulation using both continuous \
averaged model and discontinuous switching model. The Boost PWM converter is \
used again as example. Since only built-in functions of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " are used, no detailed explanations will be provided."
}], "Text",
  CellTags->"Large-Signal Simulation"],

Cell[CellGroupData[{

Cell["Large-Signal Nonlinear Averaged Model", "Subsection",
  CellTags->"Small-Signal Analysis"],

Cell[TextData[{
  "The righthand sides of the ",
  ButtonBox["averaged model",
    ButtonData:>"Averaged Model",
    ButtonStyle->"Hyperlink"],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox[
                    StyleBox[
                      RowBox[{"d", 
                        StyleBox["i",
                          FontSlant->"Italic"]}]], "L"], "/", 
                  StyleBox[
                    RowBox[{
                      StyleBox["d",
                        FontSlant->"Plain"], 
                      StyleBox["t",
                        FontSlant->"Italic"]}]]}], " ", "=", " ", \(f\_i\)}], 
              ",", " ", 
              RowBox[{
                RowBox[{
                  SubscriptBox[
                    StyleBox[
                      RowBox[{"d", 
                        StyleBox["v",
                          FontSlant->"Italic"]}]], "C"], "/", 
                  StyleBox[
                    RowBox[{
                      StyleBox["d",
                        FontSlant->"Plain"], 
                      StyleBox["t",
                        FontSlant->"Italic"]}]]}], " ", "=", " ", 
                SubscriptBox[
                  StyleBox["f",
                    FontSlant->"Italic"], "v"]}]}], ")"}], " "}], 
        TraditionalForm]]],
  "are defined here as functions of state variables ",
  Cell[BoxData[
      \(TraditionalForm\`i\_L\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`v\_C\)]],
  ", the input voltage ",
  Cell[BoxData[
      \(TraditionalForm\`V\_g\)]],
  ", and the control variable ",
  StyleBox["d",
    FontSlant->"Italic"],
  ". or easy understanding, the genetic state variables are replaced by the \
circuit variables they represent ",
  Cell[BoxData[
      \(TraditionalForm\`\((i\_L\)\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\(\ v\_C\)\)]],
  " in the example Boost converter)"
}], "Text",
  CellTags->"Small-Signal Analysis"],

Cell[BoxData[{
    \(fi[iL_, \ vC_, \ Vg_, \ d_]\  := \ 
      AveragedModel[\([1]\)]\  /. \ {\(x[1]\)[t0]\  -> \ 
            iL, \ \(x[2]\)[t0]\  -> \ vC}\), "\n", 
    \(fv[iL_, \ vC_, \ Vg_, \ d_]\  := \ 
      AveragedModel[\([2]\)]\  /. \ {\(x[1]\)[t0]\  -> \ 
            iL, \ \(x[2]\)[t0]\  -> \ vC}\)}], "Input",
  CellTags->"Small-Signal Analysis"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection",
  CellTags->"Large-Signal Simulation"],

Cell["\<\
The parameters of the converter are required for numerical \
simulation and are given below.  Notice that the ESR of the capacitor is \
assumed to be zero in this section for simplicity.\
\>", "Text",
  CellTags->"Large-Signal Simulation"],

Cell[BoxData[
    \(\(Param1\  = \ {Vg\  -> \ 5, \ L\  -> \ 50\ 10^\(-6\), \ 
          C\  -> \ 4.4\ 10^\(-6\), \ R\  -> \ 10, \ r\  -> \ 0.0, \ 
          d\  -> \ 0.3, \ Ts\  -> \ 10^\(-5\)};\)\)], "Input",
  CellTags->"Large-Signal Simulation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Averaged Model Simulation", "Subsection",
  CellTags->"Large-Signal Simulation"],

Cell[TextData[{
  "The initial values (",
  Cell[BoxData[
      \(TraditionalForm\`i\&_\_L\)]],
  "(0) = 0.5 A, ",
  Cell[BoxData[
      \(TraditionalForm\`v\&_\_C\)]],
  "(0) = 5 V) correspond to the steady-state response of the converter when \
the duty ratio ",
  StyleBox["d",
    FontSlant->"Italic"],
  " is equal to 0. Hence the simulated responses correspond to the start-up \
transient of the converter (with ",
  StyleBox["d ",
    FontSlant->"Italic"],
  "increased from 0 to 0.3, as given above in ",
  StyleBox["Param",
    FontSlant->"Italic"],
  ")."
}], "Text",
  CellTags->"Large-Signal Simulation"],

Cell[BoxData[{
    \(\(T0\  = \ 0.4\ 10^\(-3\);\)\), "\n", 
    \(\(ResAve\  = \ 
        NDSolve[{\(iL'\)[t]\  == \ 
                fi[iL[t], \ vC[t], \ Vg, \ d], \ \(vC'\)[t]\  == \ 
                fv[iL[t], \ vC[t], \ Vg, \ d], iL[0]\  == \ 5/10, \ 
              vC[0]\  == \ 5}\  /. \ Param1, {iL, \ vC}, \ {t, \ 0, \ T0}, 
          MaxSteps\  -> \ 2\ 10^4];\)\)}], "Input",
  CellTags->"Large-Signal Simulation"],

Cell[TextData[{
  "Here are the simulated average response of the inductor current, ",
  Cell[BoxData[
      \(TraditionalForm\`i\_L\)]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(iLAve\  = \ 
        Plot[Evaluate[iL[10^\(-3\)\ t]\  /. \ ResAve], {t, \ 0, \ 
            10^3\ T0}, \ PlotRange\  -> \ All, \ 
          PlotStyle\  -> \ {{AbsoluteThickness[0.001], \ 
                RGBColor[0, \ 0, \ 1]}}];\)\)], "Input",
  CellTags->"Large-Signal Simulation"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 2.38095 -0.296368 0.622166 [
[.2619 .3133 -9 -9 ]
[.2619 .3133 9 0 ]
[.5 .3133 -9 -9 ]
[.5 .3133 9 0 ]
[.7381 .3133 -9 -9 ]
[.7381 .3133 9 0 ]
[.97619 .3133 -9 -9 ]
[.97619 .3133 9 0 ]
[.01131 .07693 -18 -4.5 ]
[.01131 .07693 0 4.5 ]
[.01131 .20136 -18 -4.5 ]
[.01131 .20136 0 4.5 ]
[.01131 .45023 -18 -4.5 ]
[.01131 .45023 0 4.5 ]
[.01131 .57466 -18 -4.5 ]
[.01131 .57466 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .3258 m
.2619 .33205 L
s
[(0.1)] .2619 .3133 0 1 Mshowa
.5 .3258 m
.5 .33205 L
s
[(0.2)] .5 .3133 0 1 Mshowa
.7381 .3258 m
.7381 .33205 L
s
[(0.3)] .7381 .3133 0 1 Mshowa
.97619 .3258 m
.97619 .33205 L
s
[(0.4)] .97619 .3133 0 1 Mshowa
.125 Mabswid
.07143 .3258 m
.07143 .32955 L
s
.11905 .3258 m
.11905 .32955 L
s
.16667 .3258 m
.16667 .32955 L
s
.21429 .3258 m
.21429 .32955 L
s
.30952 .3258 m
.30952 .32955 L
s
.35714 .3258 m
.35714 .32955 L
s
.40476 .3258 m
.40476 .32955 L
s
.45238 .3258 m
.45238 .32955 L
s
.54762 .3258 m
.54762 .32955 L
s
.59524 .3258 m
.59524 .32955 L
s
.64286 .3258 m
.64286 .32955 L
s
.69048 .3258 m
.69048 .32955 L
s
.78571 .3258 m
.78571 .32955 L
s
.83333 .3258 m
.83333 .32955 L
s
.88095 .3258 m
.88095 .32955 L
s
.92857 .3258 m
.92857 .32955 L
s
.25 Mabswid
0 .3258 m
1 .3258 L
s
.02381 .07693 m
.03006 .07693 L
s
[(0.6)] .01131 .07693 1 0 Mshowa
.02381 .20136 m
.03006 .20136 L
s
[(0.8)] .01131 .20136 1 0 Mshowa
.02381 .45023 m
.03006 .45023 L
s
[(1.2)] .01131 .45023 1 0 Mshowa
.02381 .57466 m
.03006 .57466 L
s
[(1.4)] .01131 .57466 1 0 Mshowa
.125 Mabswid
.02381 .10804 m
.02756 .10804 L
s
.02381 .13915 m
.02756 .13915 L
s
.02381 .17026 m
.02756 .17026 L
s
.02381 .23247 m
.02756 .23247 L
s
.02381 .26358 m
.02756 .26358 L
s
.02381 .29469 m
.02756 .29469 L
s
.02381 .35691 m
.02756 .35691 L
s
.02381 .38801 m
.02756 .38801 L
s
.02381 .41912 m
.02756 .41912 L
s
.02381 .48134 m
.02756 .48134 L
s
.02381 .51245 m
.02756 .51245 L
s
.02381 .54356 m
.02756 .54356 L
s
.02381 .04582 m
.02756 .04582 L
s
.02381 .01471 m
.02756 .01471 L
s
.02381 .60577 m
.02756 .60577 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 1 r
.001 Mabswid
.02381 .01472 m
.04262 .16754 L
.06244 .32436 L
.08255 .45771 L
.09396 .51602 L
.09905 .53741 L
.10458 .55724 L
.11004 .57331 L
.11508 .58504 L
.11989 .5935 L
.12232 .59676 L
.125 .59958 L
.12623 .60061 L
.12741 .60143 L
.12848 .60205 L
.12962 .60258 L
.13085 .60298 L
.13199 .60322 L
.13269 .60329 L
.13335 .60332 L
.13459 .60324 L
.13568 .60304 L
.13684 .6027 L
.13782 .6023 L
.1389 .60176 L
.14132 .60012 L
.14357 .5981 L
.14858 .59194 L
.15402 .58281 L
.16392 .56047 L
.18248 .50336 L
.22326 .35352 L
.24352 .29006 L
.25344 .26531 L
.26253 .24693 L
.27138 .23325 L
.27583 .22795 L
.2807 .22338 L
.2856 .22004 L
.28835 .21872 L
.28964 .21822 L
.29084 .21785 L
.29195 .21756 L
.29317 .21732 L
.2944 .21715 L
.2957 .21704 L
.29684 .21702 L
.29806 .21707 L
.29922 .21717 L
.30028 .21733 L
Mistroke
.30157 .21758 L
.30279 .2179 L
.30506 .21865 L
.30751 .21972 L
.31015 .22115 L
.31492 .22444 L
.3193 .22822 L
.32939 .23932 L
.34004 .25401 L
.37807 .31765 L
.39925 .35067 L
.41854 .3739 L
.42814 .38232 L
.4332 .38586 L
.43856 .38892 L
.44324 .39101 L
.44573 .39191 L
.44841 .3927 L
.45086 .39328 L
.4531 .39369 L
.45525 .39397 L
.45632 .39407 L
.4575 .39415 L
.45815 .39418 L
.45883 .3942 L
.46005 .39422 L
.46136 .3942 L
.46211 .39417 L
.4628 .39413 L
.46407 .39404 L
.46522 .39392 L
.4678 .39356 L
.4701 .39313 L
.4726 .39255 L
.47715 .39119 L
.48243 .38915 L
.48804 .38651 L
.49812 .38071 L
.53583 .35237 L
.55678 .33674 L
.56596 .33077 L
.576 .32509 L
.5851 .32083 L
.59483 .31727 L
.60021 .31576 L
.6051 .31467 L
.60746 .31425 L
.61001 .31385 L
.61244 .31354 L
.61466 .31332 L
Mistroke
.61701 .31314 L
.61831 .31306 L
.61954 .31301 L
.62067 .31297 L
.62173 .31295 L
.62288 .31294 L
.62411 .31294 L
.62481 .31295 L
.62546 .31296 L
.62617 .31298 L
.62692 .313 L
.62818 .31305 L
.62957 .31313 L
.63212 .31332 L
.63451 .31354 L
.63982 .31423 L
.64467 .31505 L
.65555 .31751 L
.67465 .32335 L
.69541 .33078 L
.71481 .33759 L
.72511 .34084 L
.73621 .34389 L
.74679 .34628 L
.75648 .34799 L
.76575 .34916 L
.77041 .34959 L
.7755 .34993 L
.77804 .35005 L
.77938 .3501 L
.78079 .35014 L
.78203 .35017 L
.78269 .35018 L
.78339 .35019 L
.78463 .35021 L
.78578 .35022 L
.78698 .35022 L
.78829 .35021 L
.78952 .3502 L
.79067 .35018 L
.7918 .35016 L
.79283 .35013 L
.79513 .35005 L
.79783 .34994 L
.80032 .3498 L
.80593 .34942 L
.81608 .34844 L
.83496 .34591 L
.85564 .34255 L
.87684 .3391 L
Mistroke
.89614 .33642 L
.90574 .33535 L
.91617 .33441 L
.92095 .33406 L
.92602 .33376 L
.93068 .33353 L
.93512 .33336 L
.93747 .33329 L
.94003 .33322 L
.94236 .33318 L
.94452 .33315 L
.94585 .33314 L
.94709 .33313 L
.94821 .33312 L
.94942 .33312 L
.95074 .33312 L
.95141 .33312 L
.95214 .33312 L
.95343 .33313 L
.95464 .33314 L
.95701 .33317 L
.95926 .33321 L
.96347 .3333 L
.96801 .33344 L
.97294 .33363 L
.97619 .33378 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  Background->GrayLevel[1],
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005X0000gR000`400?l00000o`00003oo`3ooomY
0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY
0?ooo`00o`3ooomY0?ooo`006P3oool00`000000oooo0?ooo`3o0?oood`0oooo000J0?ooo`030000
003oool0oooo0?l0ooooC03oool001X0oooo00<000000?ooo`3oool0o`3ooom<0?ooo`006P3oool0
0`000000oooo0?ooo`3o0?oood`0oooo000J0?ooo`030000003oool0oooo0?l0ooooC03oool001X0
oooo00<0003o0000003oool0o`3ooom<0?ooo`006P3oool00`000?l0oooo0?ooo`3o0?oood`0oooo
000J0?ooo`030000o`3oool0oooo0?l0ooooC03oool001X0oooo00<0003o0?ooo`3oool0o`3ooom<
0?ooo`006P3oool00`000000003o0?ooo`3o0?oood`0oooo000J0?ooo`03000000000?l0oooo0?l0
ooooC03oool001X0oooo00<000000000o`3oool0o`3ooom<0?ooo`006P3oool00`000000003o0?oo
o`3o0?oood`0oooo000J0?ooo`03000000000?l0oooo0?l0ooooC03oool001X0oooo00<000000000
o`3oool0o`3ooom<0?ooo`006P3oool00`000000003o0?ooo`3o0?oood`0oooo000J0?ooo`030000
003oool0003o0?l0ooooC03oool001X0oooo00<000000?ooo`000?l0o`3ooom<0?ooo`006P3oool0
0`000000oooo0000o`3o0?oood`0oooo000J0?ooo`030000003oool0003o0?l0ooooC03oool001X0
oooo00<000000?ooo`000?l0o`3ooom<0?ooo`006P3oool00`000000oooo0000o`3o0?oood`0oooo
000J0?ooo`030000003oool0003o0?l0ooooC03oool000@000001@3oool2000000H0oooo0`000006
0?ooo`030000003oool0003o0?l0ooooC03oool000@0oooo00<000000?ooo`3oool00P3oool20000
00D0oooo00D000000?ooo`3oool0oooo000000050?ooo`040000003oool0oooo0000ool0ooooB`3o
ool000@0oooo00<000000?ooo`3oool02@3oool01@000000oooo0?ooo`3oool0000000D0oooo00@0
00000?ooo`3oool0003oo`3ooom;0?ooo`00103oool00`000000oooo0?ooo`090?ooo`050000003o
ool0oooo0?ooo`0000001@3oool3000000030000o`3oool0oooo0?l0ooooB@3oool000@0oooo00<0
00000?ooo`3oool02@3oool4000000H0oooo00@000000?ooo`3oool0003oo`3ooom;0?ooo`00103o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00L0oooo00@000000?ooo`3oool0003o
o`3ooom;0?ooo`00103oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00H0oooo00@0
00000?ooo`3oool0003oo`3ooom;0?ooo`001000000>0?ooo`<000001@3oool010000000oooo0?oo
o`000?oo0?oood/0oooo000J0?ooo`050000003oool0oooo0?ooo`000?l0o`3ooom:0?ooo`006P3o
ool01@000000oooo0?ooo`3oool0003o0?l0ooooBP3oool001X0oooo00D000000?ooo`3oool0oooo
0000o`3o0?ooodX0oooo000J0?ooo`050000003oool0oooo0?ooo`000?l0o`3ooom:0?ooo`006P3o
ool01@000000oooo0?ooo`3oool0003o0?l0ooooBP3oool001X0oooo0P0000020?ooo`030000o`3o
ool0oooo0?l0ooooB03oool001X0oooo00D000000?ooo`3oool0oooo0000o`3o0?ooodX0oooo000J
0?ooo`030000003oool0oooo0080oooo00<0003o0?ooo`3oool0o`3ooom70?ooo`006P3oool00`00
0000oooo0?ooo`020?ooo`030000o`3oool0oooo0?l0ooooA`3oool001X0oooo00<000000?ooo`3o
ool00P3oool00`000?l0oooo0?ooo`3o0?ooodL0oooo000J0?ooo`030000003oool0oooo0080oooo
00<0003o0?ooo`3oool0o`3ooom70?ooo`006P3oool00`000000oooo0?ooo`020?ooo`030000o`3o
ool0oooo0?l0ooooA`3oool001X0oooo00<000000?ooo`3oool00P3oool00`000?l0oooo0?ooo`3o
0?ooodL0oooo000J0?ooo`030000003oool0oooo0080oooo00<0003o0?ooo`3oool0o`3ooom70?oo
o`006P3oool00`000000oooo0?ooo`020?ooo`030000o`3oool0oooo0?l0ooooA`3oool001X0oooo
00<000000?ooo`3oool00`3oool00`000?l0oooo0?ooo`3o0?ooodH0oooo000J0?ooo`800000103o
ool00`000?l0oooo0?ooo`3o0?ooodH0oooo000J0?ooo`030000003oool0oooo00<0oooo00<0003o
0?ooo`3oool0o`3ooom60?ooo`006P3oool00`000000oooo0?ooo`030?ooo`030000o`3oool0oooo
0?l0ooooAP3oool001X0oooo00<000000?ooo`3oool00`3oool00`000?l0oooo0?ooo`3o0?ooodH0
oooo000J0?ooo`030000003oool0oooo00<0oooo00<0003o0?ooo`3oool0o`3ooom60?ooo`006P3o
ool00`000000oooo0?ooo`030?ooo`030000o`3oool0oooo0?l0ooooAP3oool001X0oooo00<00000
0?ooo`3oool0103oool00`000?l0oooo0?ooo`3o0?ooodD0oooo000J0?ooo`030000003oool0oooo
00@0oooo00<0003o0?ooo`3oool0o`3ooom50?ooo`006P3oool00`000000oooo0?ooo`040?ooo`03
0000o`3oool0oooo0?l0ooooA@3oool001X0oooo00<000000?ooo`3oool0103oool00`000?l0oooo
0?ooo`3o0?ooodD0oooo000J0?ooo`8000001@3oool00`000?l0oooo0?ooo`3o0?ooodD0oooo000J
0?ooo`030000003oool0oooo00@0oooo00<0003o0?ooo`3oool0o`3ooom50?ooo`006P3oool00`00
0000oooo0?ooo`040?ooo`030000o`3oool0oooo0?l0ooooA@3oool001X0oooo00<000000?ooo`3o
ool0103oool00`000?l0oooo0?ooo`3o0?ooodD0oooo000J0?ooo`030000003oool0oooo00D0oooo
00<0003o0?ooo`3oool0o`3ooom40?ooo`006P3oool00`000000oooo0?ooo`050?ooo`030000o`3o
ool0oooo0?l0ooooA03oool001X0oooo00<000000?ooo`3oool01@3oool00`000?l0oooo0?ooo`3o
0?oood@0oooo000J0?ooo`030000003oool0oooo00D0oooo00<0003o0?ooo`3oool0o`3ooom40?oo
o`00100000050?ooo`8000001@3oool4000000H0oooo00<000000?ooo`3oool01@3oool00`000?l0
oooo0?ooo`3o0?oood@0oooo00040?ooo`030000003oool0oooo0080oooo0P0000040?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000
o`3oool0oooo0?l0ooooA03oool000@0oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01@3oool00`000?l0oooo0?oo
o`3o0?oood@0oooo00040?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`030?ooo`<000001@3oool00`000?l0oooo0?ooo`3o0?oood@0oooo00040?oo
o`030000003oool0oooo00T0oooo100000060?ooo`030000003oool0oooo00D0oooo00<0003o0?oo
o`3oool0o`3ooom40?ooo`00103oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`060?ooo`030000o`3oool0oooo0?l0
oooo@`3oool000@0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool01P3oool00`000?l0oooo0?ooo`3o0?oood<0oooo
0004000000`0oooo100000060?ooo`030000003oool0oooo00H0oooo00<0003o0?ooo`3oool0o`3o
oom30?ooo`006P3oool00`000000oooo0?ooo`060?ooo`030000o`3oool0oooo04d0oooo1`000?o^
0?ooo`006P3oool00`000000oooo0?ooo`060?ooo`030000o`3oool0oooo04`0oooo00<0003o0?oo
o`3oool01@3oool20000on`0oooo000J0?ooo`030000003oool0oooo00H0oooo00<0003o0?ooo`3o
ool0B`3oool00`000?l0oooo0?ooo`080?ooo`030000o`3oool0oooo0>T0oooo000J0?ooo`030000
003oool0oooo00H0oooo00<0003o0?ooo`3oool0B@3oool20000o``0oooo00<0003o0?ooo`3oool0
j03oool001X0oooo00<000000?ooo`3oool01P3oool00`000?l0oooo0?ooo`180?ooo`030000o`3o
ool0oooo00d0oooo00<0003o0?ooo`3oool0i`3oool001X0oooo0P0000070?ooo`030000o`3oool0
oooo04P0oooo00<0003o0?ooo`3oool03P3oool00`000?l0oooo0?ooo`3V0?ooo`006P3oool00`00
0000oooo0?ooo`070?ooo`030000o`3oool0oooo04H0oooo00<0003o0?ooo`3oool0403oool00`00
0?l0oooo0?ooo`3U0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030000o`3oool0oooo04H0
oooo00<0003o0?ooo`3oool04@3oool00`000?l0oooo0?ooo`3T0?ooo`006P3oool00`000000oooo
0?ooo`070?ooo`030000o`3oool0oooo04D0oooo00<0003o0?ooo`3oool04`3oool00`000?l0oooo
0?ooo`3S0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030000o`3oool0oooo04D0oooo00<0
003o0?ooo`3oool0503oool00`000?l0oooo0?ooo`3R0?ooo`006P3oool00`000000oooo0?ooo`07
0?ooo`030000o`3oool0oooo04@0oooo00<0003o0?ooo`3oool05@3oool00`000?l0oooo0?ooo`3R
0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030000o`3oool0oooo04<0oooo00<0003o0?oo
o`3oool05`3oool00`000?l0oooo0?ooo`3Q0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`03
0000o`3oool0oooo04<0oooo00<0003o0?ooo`3oool0603oool00`000?l0oooo0?ooo`3P0?ooo`00
6P3oool00`000000oooo0?ooo`070?ooo`030000o`3oool0oooo0480oooo00<0003o0?ooo`3oool0
6P3oool00`000?l0oooo0?ooo`3O0?ooo`006P3oool00`000000oooo0?ooo`080?ooo`030000o`3o
ool0oooo0440oooo00<0003o0?ooo`3oool06P3oool00`000?l0oooo0?ooo`3O0?ooo`006P3oool2
000000T0oooo00<0003o0?ooo`3oool0@03oool00`000?l0oooo0?ooo`0L0?ooo`030000o`3oool0
oooo0=h0oooo000J0?ooo`030000003oool0oooo00P0oooo00<0003o0?ooo`3oool0@03oool00`00
0?l0oooo0?ooo`0L0?ooo`030000o`3oool0oooo0=h0oooo000J0?ooo`030000003oool0oooo00P0
oooo00<0003o0?ooo`3oool0?`3oool00`000?l0oooo0?ooo`0N0?ooo`030000o`3oool0oooo0=d0
oooo000J0?ooo`030000003oool0oooo00P0oooo00<0003o0?ooo`3oool0?`3oool00`000?l0oooo
0?ooo`0O0?ooo`030000o`3oool0oooo0=`0oooo000J0?ooo`030000003oool0oooo00P0oooo00<0
003o0?ooo`3oool0?P3oool00`000?l0oooo0?ooo`0P0?ooo`030000o`3oool0oooo0=`0oooo000J
0?ooo`030000003oool0oooo00P0oooo00<0003o0?ooo`3oool0?P3oool00`000?l0oooo0?ooo`0Q
0?ooo`030000o`3oool0oooo0=/0oooo000J0?ooo`030000003oool0oooo00P0oooo00<0003o0?oo
o`3oool0>03oool4000000030?ooo`000?l0oooo0080oooo0P0000050?ooo`D000005@3oool00`00
0?l0oooo0?ooo`0T0?ooo`@000001@3oool2000000@0oooo1@00000l0?ooo`@000001@3oool20000
00D0oooo0`00000m0?ooo`@000001@3oool2000000L0oooo0`0000020?ooo`006P3oool00`000000
oooo0?ooo`080?ooo`030000o`3oool0oooo03L0oooo00<000000?ooo`3oool00P3oool00`000000
003o0?ooo`020?ooo`8000001`3oool00`000000oooo0?ooo`0F0?ooo`030000o`3oool0oooo0280
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`800000103oool01@000000
oooo0?ooo`3oool0000003/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?oo
o`800000103oool01@000000oooo0?ooo`3oool0000003/0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`020?ooo`800000203oool00`000000oooo0?ooo`010?ooo`006P3oool00`00
0000oooo0?ooo`090?ooo`030000o`3oool0oooo03H0oooo00<000000?ooo`3oool00P3oool00`00
0?l0oooo0?ooo`0;0?ooo`030000003oool0oooo01H0oooo00<0003o0?ooo`3oool08P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0?03oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0>@3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool00@3oool001X0
oooo00<000000?ooo`3oool02@3oool00`000?l0oooo0?ooo`0f0?ooo`030000003oool0oooo0080
oooo00<0003o0?ooo`3oool02`3oool00`000000oooo0?ooo`0G0?ooo`030000o`3oool0oooo0240
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo03/0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo03T0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`080?ooo`H000000P3oool001X0oooo
00<000000?ooo`3oool02@3oool00`000?l0oooo0?ooo`0f0?ooo`030000003oool0oooo0080oooo
00<0003o0?ooo`3oool02`3oool00`000000oooo0?ooo`0H0?ooo`030000o`3oool0oooo0200oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo03X0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`800000?03oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00T0oooo00@000000?ooo`3oool000000`3oool001X0oooo
0P00000:0?ooo`030000o`3oool0oooo03H0oooo00H000000?ooo`3oool0oooo0000o`00000=0?oo
o`030000003oool0oooo01P0oooo00<0003o0?ooo`3oool0803oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00T0oooo00@000000?ooo`3oool000000`3oool001X0oooo00<000000?oo
o`3oool02@3oool00`000?l0oooo0?ooo`0f0?ooo`060000003oool0oooo0?ooo`000?l000002`3o
ool3000001/0oooo00<0003o0?ooo`3oool07`3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo00P0oooo00D000000?ooo`3oool0oooo0000000k0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool0203oool01@000000oooo0?ooo`3oool0000003/0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0000000<0oooo000J0?ooo`030000
003oool0oooo00T0oooo00<0003o0?ooo`3oool0=`3oool3000000030000o`3oool0oooo00`0oooo
00<000000?ooo`3oool06@3oool00`000?l0oooo0?ooo`0P0?ooo`@00000303oool3000003d0oooo
1000000<0?ooo`<00000?@3oool4000000h0oooo0P0000030?ooo`006P3oool00`000000oooo0?oo
o`090?ooo`030000o`3oool0oooo03T0oooo00<0003o0?ooo`3oool0:P3oool00`000?l0oooo0?oo
o`3F0?ooo`006P3oool00`000000oooo0?ooo`090?ooo`030000o`3oool0oooo03T0oooo00<0003o
0?ooo`3oool0:`3oool00`000?l0oooo0?ooo`3E0?ooo`006P3oool00`000000oooo0?ooo`090?oo
o`030000o`3oool0oooo03T0oooo00<0003o0?ooo`3oool0:`3oool00`000?l0oooo0?ooo`3E0?oo
o`006P3oool00`000000oooo0?ooo`0:0?ooo`030000o`3oool0oooo03L0oooo00<0003o0?ooo`3o
ool0;@3oool00`000?l0oooo0?ooo`1<0?ooo`T0003oO`3oool001X0oooo00<000000?ooo`3oool0
2P3oool00`000?l0oooo0?ooo`0g0?ooo`030000o`3oool0oooo02d0oooo00<0003o0?ooo`3oool0
B@3oool30000o`T0oooo10000?mk0?ooo`006P3oool00`000000oooo0?ooo`0:0?ooo`030000o`3o
ool0oooo03L0oooo00<0003o0?ooo`3oool0;P3oool00`000?l0oooo0?ooo`150?ooo`<0003o403o
ool40000ogL0oooo000J0?ooo`030000003oool0oooo00X0oooo00<0003o0?ooo`3oool0=P3oool0
0`000?l0oooo0?ooo`0`0?ooo`030000o`3oool0oooo0400oooo10000?lG0?ooo`@0003oL`3oool0
0180oooo5@0000000`000?l000000000000f000000030000o`00000000000300000000<0003o0000
00000000?P0000020000oal000000`000?m/000000@0oooo000J0?ooo`030000003oool0oooo00X0
oooo00@0003o0?ooo`3oool000003`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool00`3oool00`000?l0oooo0?ooo`070?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`060?ooo`030000o`3oool0oooo
00@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool0103oool20000o`L0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`0:0?ooo`@0003o3`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool02P3oool001X0oooo00<000000?ooo`3oool02P3oool010000?l0
oooo0?ooo`00000?0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`030?ooo`030000o`3oool0oooo00L0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo00L0oooo00<0003o0?ooo`3oool00`3oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`030?ooo`030000o`3oool0oooo00L0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`0=0?ooo`04000000000?l0003o0000o``0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00X0oooo000J0?ooo`030000003oool0oooo
00X0oooo00<0003o0?ooo`3oool0=@3oool00`000?l0oooo0?ooo`080?ooo`030000003oool0oooo
02P0oooo00<0003o0?ooo`3oool08`3oool00`000000oooo0?ooo`0A0?ooo`80003o;P3oool30000
o`T0oooo00<000000?ooo`3oool0>03ooolF0000o``0oooo000J0?ooo`030000003oool0oooo00/0
oooo00<0003o0?ooo`3oool0=03oool00`000?l0oooo0?ooo`0d0?ooo`030000o`3oool0oooo03D0
oooo00<0003o0?ooo`3oool0<@3oool20000ocX0oooo20000?lR0?ooo`006P3oool00`000000oooo
0?ooo`0;0?ooo`030000o`3oool0oooo03@0oooo00<0003o0?ooo`3oool0=03oool00`000?l0oooo
0?ooo`0c0?ooo`80003o=P3oool20000oc40oooo1`000?lZ0?ooo`006P3oool00`000000oooo0?oo
o`0;0?ooo`030000o`3oool0oooo03<0oooo00<0003o0?ooo`3oool0=P3oool00`000?l0oooo0?oo
o`0a0?ooo`030000o`3oool0oooo03P0oooo0`000?lW0?ooo`L0003o<@3oool001X0oooo00<00000
0?ooo`3oool02`3oool00`000?l0oooo0?ooo`0c0?ooo`030000o`3oool0oooo03L0oooo00<0003o
0?ooo`3oool0;`3oool00`000?l0oooo0?ooo`0l0?ooo`L0003o6@3oool70000ocP0oooo000J0?oo
o`030000003oool0oooo00/0oooo00<0003o0?ooo`3oool0<`3oool00`000?l0oooo0?ooo`0g0?oo
o`030000o`3oool0oooo02d0oooo0P000?m60?ooo`L0003o2`3oool70000ocl0oooo000J0?ooo`03
0000003oool0oooo00/0oooo00<0003o0?ooo`3oool0<P3oool00`000?l0oooo0?ooo`0i0?ooo`03
0000o`3oool0oooo02/0oooo00<0003o0?ooo`3oool0C@3oool;0000odH0oooo000J0?ooo`030000
003oool0oooo00`0oooo00<0003o0?ooo`3oool0<@3oool00`000?l0oooo0?ooo`0j0?ooo`030000
o`3oool0oooo02T0oooo00<0003o0?ooo`3oool0W`3oool001X0oooo0P00000=0?ooo`030000o`3o
ool0oooo0340oooo00<0003o0?ooo`3oool0>P3oool00`000?l0oooo0?ooo`0W0?ooo`80003oXP3o
ool001X0oooo00<000000?ooo`3oool0303oool00`000?l0oooo0?ooo`0`0?ooo`030000o`3oool0
oooo03`0oooo00<0003o0?ooo`3oool09@3oool00`000?l0oooo0?ooo`2R0?ooo`006P3oool00`00
0000oooo0?ooo`0<0?ooo`030000o`3oool0oooo0300oooo00<0003o0?ooo`3oool0?@3oool00`00
0?l0oooo0?ooo`0S0?ooo`030000o`3oool0oooo0:<0oooo000J0?ooo`030000003oool0oooo00`0
oooo00<0003o0?ooo`3oool0<03oool00`000?l0oooo0?ooo`0n0?ooo`030000o`3oool0oooo0200
oooo0P000?nV0?ooo`006P3oool00`000000oooo0?ooo`0<0?ooo`030000o`3oool0oooo0300oooo
00<0003o0?ooo`3oool0?P3oool00`000?l0oooo0?ooo`0O0?ooo`030000o`3oool0oooo0:H0oooo
000J0?ooo`030000003oool0oooo00`0oooo00<0003o0?ooo`3oool0;`3oool00`000?l0oooo0?oo
o`100?ooo`030000o`3oool0oooo01d0oooo00<0003o0?ooo`3oool0Y`3oool001X0oooo00<00000
0?ooo`3oool03@3oool00`000?l0oooo0?ooo`0^0?ooo`030000o`3oool0oooo0440oooo00<0003o
0?ooo`3oool06P3oool20000ojX0oooo000J0?ooo`030000003oool0oooo00d0oooo00<0003o0?oo
o`3oool0;P3oool00`000?l0oooo0?ooo`120?ooo`030000o`3oool0oooo01P0oooo00<0003o0?oo
o`3oool0ZP3oool001X0oooo00<000000?ooo`3oool03@3oool00`000?l0oooo0?ooo`0^0?ooo`03
0000o`3oool0oooo04<0oooo0P000?lG0?ooo`030000o`3oool0oooo0:/0oooo000J0?ooo`030000
003oool0oooo00d0oooo00<0003o0?ooo`3oool0;@3oool00`000?l0oooo0?ooo`160?ooo`030000
o`3oool0oooo0180oooo0P000?n^0?ooo`006P3oool2000000h0oooo00<0003o0?ooo`3oool0;@3o
ool00`000?l0oooo0?ooo`170?ooo`030000o`3oool0oooo0100oooo00<0003o0?ooo`3oool0[P3o
ool001X0oooo00<000000?ooo`3oool03@3oool00`000?l0oooo0?ooo`0]0?ooo`030000o`3oool0
oooo04P0oooo10000?l<0?ooo`80003o/@3oool001X0oooo00<000000?ooo`3oool03P3oool00`00
0?l0oooo0?ooo`0/0?ooo`030000o`3oool0oooo04/0oooo1@000?l20?ooo`H0003o/`3oool001X0
oooo00<000000?ooo`3oool03P3oool00`000?l0oooo0?ooo`0[0?ooo`030000o`3oool0oooo0500
oooo0`000?ni0?ooo`006P3oool00`000000oooo0?ooo`0>0?ooo`030000o`3oool0oooo02/0oooo
00<0003o0?ooo`3oool0o`3oool=0?ooo`006P3oool00`000000oooo0?ooo`0>0?ooo`030000o`3o
ool0oooo02/0oooo00<0003o0?ooo`3oool0o`3oool=0?ooo`006P3oool00`000000oooo0?ooo`0>
0?ooo`030000o`3oool0oooo02/0oooo00<0003o0?ooo`3oool0o`3oool=0?ooo`006P3oool00`00
0000oooo0?ooo`0>0?ooo`030000o`3oool0oooo02X0oooo00<0003o0?ooo`3oool0o`3oool>0?oo
o`006P3oool00`000000oooo0?ooo`0?0?ooo`030000o`3oool0oooo02T0oooo00<0003o0?ooo`3o
ool0o`3oool>0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000o`3oool0oooo02T0oooo
00<0003o0?ooo`3oool0o`3oool>0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000o`3o
ool0oooo02T0oooo00<0003o0?ooo`3oool0o`3oool>0?ooo`006P3oool200000100oooo00<0003o
0?ooo`3oool0:03oool00`000?l0oooo0?ooo`3o0?ooo`l0oooo000J0?ooo`030000003oool0oooo
00l0oooo00<0003o0?ooo`3oool0:03oool00`000?l0oooo0?ooo`3o0?ooo`l0oooo000J0?ooo`03
0000003oool0oooo00l0oooo00<0003o0?ooo`3oool0:03oool00`000?l0oooo0?ooo`3o0?ooo`l0
oooo000J0?ooo`030000003oool0oooo00l0oooo00<0003o0?ooo`3oool09`3oool00`000?l0oooo
0?ooo`3o0?oooa00oooo000J0?ooo`030000003oool0oooo0100oooo00<0003o0?ooo`3oool09P3o
ool00`000?l0oooo0?ooo`3o0?oooa00oooo000J0?ooo`030000003oool0oooo0100oooo00<0003o
0?ooo`3oool09P3oool00`000?l0oooo0?ooo`3o0?oooa00oooo000J0?ooo`030000003oool0oooo
0100oooo00<0003o0?ooo`3oool09P3oool00`000?l0oooo0?ooo`3o0?oooa00oooo0005000000@0
oooo0P0000040?ooo`D000001P3oool00`000000oooo0?ooo`0@0?ooo`030000o`3oool0oooo02D0
oooo00<0003o0?ooo`3oool0o`3ooolA0?ooo`000P3oool00`000000oooo0?ooo`040?ooo`800000
103oool01@000000oooo0?ooo`3oool0000000H0oooo00<000000?ooo`3oool0403oool00`000?l0
oooo0?ooo`0U0?ooo`030000o`3oool0oooo0?l0oooo4@3oool00080oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0100oooo00<0003o0?ooo`3oool0
9@3oool00`000?l0oooo0?ooo`3o0?oooa40oooo00020?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool01P3oool300000140oooo00<0003o0?ooo`3oool0903oool00`000?l0oooo0?oo
o`3o0?oooa40oooo00020?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`0A0?ooo`030000o`3oool0oooo02<0oooo00<0003o0?ooo`3oool0o`3ooolB
0?ooo`000P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool04@3oool00`000?l0oooo0?ooo`0S0?ooo`030000o`3oool0oooo0?l0oooo4P3oool000<0
0000303oool01@000000oooo0?ooo`3oool0000000H0oooo00<000000?ooo`3oool04@3oool00`00
0?l0oooo0?ooo`0S0?ooo`030000o`3oool0oooo0?l0oooo4P3oool00080oooo00<000000?ooo`3o
ool02`3oool3000000L0oooo00<000000?ooo`3oool04@3oool00`000?l0oooo0?ooo`0S0?ooo`03
0000o`3oool0oooo0?l0oooo4P3oool001X0oooo00<000000?ooo`3oool04@3oool00`000?l0oooo
0?ooo`0R0?ooo`030000o`3oool0oooo0?l0oooo4`3oool001X0oooo00<000000?ooo`3oool04P3o
ool00`000?l0oooo0?ooo`0Q0?ooo`030000o`3oool0oooo0?l0oooo4`3oool001X0oooo00<00000
0?ooo`3oool04P3oool00`000?l0oooo0?ooo`0Q0?ooo`030000o`3oool0oooo0?l0oooo4`3oool0
01X0oooo00<000000?ooo`3oool04P3oool00`000?l0oooo0?ooo`0Q0?ooo`030000o`3oool0oooo
0?l0oooo4`3oool001X0oooo00<000000?ooo`3oool04P3oool00`000?l0oooo0?ooo`0P0?ooo`03
0000o`3oool0oooo0?l0oooo503oool001X0oooo00<000000?ooo`3oool04P3oool00`000?l0oooo
0?ooo`0P0?ooo`030000o`3oool0oooo0?l0oooo503oool001X0oooo0P00000D0?ooo`030000o`3o
ool0oooo01l0oooo00<0003o0?ooo`3oool0o`3ooolD0?ooo`006P3oool00`000000oooo0?ooo`0C
0?ooo`030000o`3oool0oooo01l0oooo00<0003o0?ooo`3oool0o`3ooolD0?ooo`006P3oool00`00
0000oooo0?ooo`0C0?ooo`030000o`3oool0oooo01h0oooo00<0003o0?ooo`3oool0o`3ooolE0?oo
o`006P3oool00`000000oooo0?ooo`0C0?ooo`030000o`3oool0oooo01h0oooo00<0003o0?ooo`3o
ool0o`3ooolE0?ooo`006P3oool00`000000oooo0?ooo`0C0?ooo`030000o`3oool0oooo01h0oooo
00<0003o0?ooo`3oool0o`3ooolE0?ooo`006P3oool00`000000oooo0?ooo`0D0?ooo`030000o`3o
ool0oooo01d0oooo00<0003o0?ooo`3oool0o`3ooolE0?ooo`006P3oool00`000000oooo0?ooo`0D
0?ooo`030000o`3oool0oooo01`0oooo00<0003o0?ooo`3oool0o`3ooolF0?ooo`006P3oool00`00
0000oooo0?ooo`0D0?ooo`030000o`3oool0oooo01`0oooo00<0003o0?ooo`3oool0o`3ooolF0?oo
o`006P3oool00`000000oooo0?ooo`0D0?ooo`030000o`3oool0oooo01`0oooo00<0003o0?ooo`3o
ool0o`3ooolF0?ooo`006P3oool00`000000oooo0?ooo`0D0?ooo`030000o`3oool0oooo01/0oooo
00<0003o0?ooo`3oool0o`3ooolG0?ooo`006P3oool2000001H0oooo00<0003o0?ooo`3oool06P3o
ool00`000?l0oooo0?ooo`3o0?oooaL0oooo000J0?ooo`030000003oool0oooo01D0oooo00<0003o
0?ooo`3oool06P3oool00`000?l0oooo0?ooo`3o0?oooaL0oooo000J0?ooo`030000003oool0oooo
01D0oooo00<0003o0?ooo`3oool06@3oool00`000?l0oooo0?ooo`3o0?oooaP0oooo000J0?ooo`03
0000003oool0oooo01D0oooo00<0003o0?ooo`3oool06@3oool00`000?l0oooo0?ooo`3o0?oooaP0
oooo000J0?ooo`030000003oool0oooo01H0oooo00<0003o0?ooo`3oool0603oool00`000?l0oooo
0?ooo`3o0?oooaP0oooo000J0?ooo`030000003oool0oooo01H0oooo00<0003o0?ooo`3oool05`3o
ool00`000?l0oooo0?ooo`3o0?oooaT0oooo000J0?ooo`030000003oool0oooo01H0oooo00<0003o
0?ooo`3oool05`3oool00`000?l0oooo0?ooo`3o0?oooaT0oooo000J0?ooo`030000003oool0oooo
01H0oooo00<0003o0?ooo`3oool05P3oool00`000?l0oooo0?ooo`3o0?oooaX0oooo000J0?ooo`03
0000003oool0oooo01L0oooo00<0003o0?ooo`3oool05@3oool00`000?l0oooo0?ooo`3o0?oooaX0
oooo000J0?ooo`030000003oool0oooo01L0oooo00<0003o0?ooo`3oool05@3oool00`000?l0oooo
0?ooo`3o0?oooaX0oooo000J0?ooo`030000003oool0oooo01L0oooo00<0003o0?ooo`3oool0503o
ool00`000?l0oooo0?ooo`3o0?oooa/0oooo000J0?ooo`800000603oool00`000?l0oooo0?ooo`0D
0?ooo`030000o`3oool0oooo0?l0oooo6`3oool001X0oooo00<000000?ooo`3oool05`3oool00`00
0?l0oooo0?ooo`0D0?ooo`030000o`3oool0oooo0?l0oooo6`3oool001X0oooo00<000000?ooo`3o
ool0603oool00`000?l0oooo0?ooo`0B0?ooo`030000o`3oool0oooo0?l0oooo703oool001X0oooo
00<000000?ooo`3oool0603oool00`000?l0oooo0?ooo`0B0?ooo`030000o`3oool0oooo0?l0oooo
703oool001X0oooo00<000000?ooo`3oool0603oool00`000?l0oooo0?ooo`0B0?ooo`030000o`3o
ool0oooo0?l0oooo703oool001X0oooo00<000000?ooo`3oool0603oool00`000?l0oooo0?ooo`0A
0?ooo`030000o`3oool0oooo0?l0oooo7@3oool001X0oooo00<000000?ooo`3oool06@3oool00`00
0?l0oooo0?ooo`0@0?ooo`030000o`3oool0oooo0?l0oooo7@3oool000D00000103oool2000000L0
oooo0`0000050?ooo`030000003oool0oooo01T0oooo00<0003o0?ooo`3oool0403oool00`000?l0
oooo0?ooo`3o0?oooad0oooo00020?ooo`030000003oool0oooo00@0oooo0P0000080?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool06@3oool00`000?l0oooo0?ooo`0?0?ooo`030000
o`3oool0oooo0?l0oooo7P3oool00080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo01X0oooo00<0003o0?ooo`3oool03P3oool00`000?l0oooo0?oo
o`3o0?oooah0oooo00020?ooo`030000003oool0oooo00X0oooo1P0000050?ooo`<000006P3oool0
0`000?l0oooo0?ooo`0=0?ooo`030000o`3oool0oooo0?l0oooo7`3oool00080oooo00<000000?oo
o`3oool02`3oool010000000oooo0?ooo`0000060?ooo`030000003oool0oooo01X0oooo00<0003o
0?ooo`3oool03@3oool00`000?l0oooo0?ooo`3o0?oooal0oooo00020?ooo`030000003oool0oooo
00/0oooo00@000000?ooo`3oool000001P3oool00`000000oooo0?ooo`0K0?ooo`030000o`3oool0
oooo00/0oooo00<0003o0?ooo`3oool0o`3ooolP0?ooo`000`00000>0?ooo`030000003oool00000
00H0oooo00<000000?ooo`3oool06`3oool00`000?l0oooo0?ooo`0;0?ooo`030000o`3oool0oooo
0?l0oooo803oool00080oooo00<000000?ooo`3oool03@3oool2000000H0oooo00<000000?ooo`3o
ool0703oool00`000?l0oooo0?ooo`090?ooo`030000o`3oool0oooo0?l0oooo8@3oool001X0oooo
00<000000?ooo`3oool07@3oool00`000?l0oooo0?ooo`080?ooo`030000o`3oool0oooo0?l0oooo
8@3oool001X0oooo00<000000?ooo`3oool07@3oool00`000?l0oooo0?ooo`070?ooo`030000o`3o
ool0oooo0?l0oooo8P3oool001X0oooo00<000000?ooo`3oool07P3oool00`000?l0oooo0?ooo`05
0?ooo`030000o`3oool0oooo0?l0oooo8`3oool001X0oooo00<000000?ooo`3oool07P3oool00`00
0?l0oooo0?ooo`050?ooo`030000o`3oool0oooo0?l0oooo8`3oool001X0oooo00<000000?ooo`3o
ool07`3oool20000o`<0oooo0P000?oo0?ooobH0oooo000J0?ooo`030000003oool0oooo0200oooo
1@000?oo0?ooobL0oooo000J0?ooo`800000o`3ooom=0?ooo`006P3oool00`000000oooo0?ooo`3o
0?oood`0oooo000J0?ooo`030000003oool0oooo0?l0ooooC03oool001X0oooo00<000000?ooo`3o
ool0o`3ooom<0?ooo`006P3oool00`000000oooo0?ooo`3o0?oood`0oooo003o0?ooofT0oooo003o
0?ooofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo0000
\
\>"],
  ImageRangeCache->{{{0, 359}, {221.375, 0}} -> {-0.0332863, 0.445245, \
0.00155747, 0.00596024}}]
}, Open  ]],

Cell[TextData[{
  "and the capacitor voltage, ",
  Cell[BoxData[
      \(TraditionalForm\`v\_C\)]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(vCAve\  = \ 
        Plot[Evaluate[vC[10^\(-3\)\ t]\  /. \ ResAve], {t, \ 0, \ 10^3\ T0}, 
          PlotRange\  -> \ All, \ 
          PlotStyle\  -> \ {{AbsoluteThickness[0.001], 
                RGBColor[0, \ 0, \ 1]}}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 2.38095 -0.862568 0.179312 [
[.2619 .02149 -9 -9 ]
[.2619 .02149 9 0 ]
[.5 .02149 -9 -9 ]
[.5 .02149 9 0 ]
[.7381 .02149 -9 -9 ]
[.7381 .02149 9 0 ]
[.97619 .02149 -9 -9 ]
[.97619 .02149 9 0 ]
[.01131 .12365 -18 -4.5 ]
[.01131 .12365 0 4.5 ]
[.01131 .2133 -6 -4.5 ]
[.01131 .2133 0 4.5 ]
[.01131 .30296 -18 -4.5 ]
[.01131 .30296 0 4.5 ]
[.01131 .39262 -6 -4.5 ]
[.01131 .39262 0 4.5 ]
[.01131 .48227 -18 -4.5 ]
[.01131 .48227 0 4.5 ]
[.01131 .57193 -6 -4.5 ]
[.01131 .57193 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .03399 m
.2619 .04024 L
s
[(0.1)] .2619 .02149 0 1 Mshowa
.5 .03399 m
.5 .04024 L
s
[(0.2)] .5 .02149 0 1 Mshowa
.7381 .03399 m
.7381 .04024 L
s
[(0.3)] .7381 .02149 0 1 Mshowa
.97619 .03399 m
.97619 .04024 L
s
[(0.4)] .97619 .02149 0 1 Mshowa
.125 Mabswid
.07143 .03399 m
.07143 .03774 L
s
.11905 .03399 m
.11905 .03774 L
s
.16667 .03399 m
.16667 .03774 L
s
.21429 .03399 m
.21429 .03774 L
s
.30952 .03399 m
.30952 .03774 L
s
.35714 .03399 m
.35714 .03774 L
s
.40476 .03399 m
.40476 .03774 L
s
.45238 .03399 m
.45238 .03774 L
s
.54762 .03399 m
.54762 .03774 L
s
.59524 .03399 m
.59524 .03774 L
s
.64286 .03399 m
.64286 .03774 L
s
.69048 .03399 m
.69048 .03774 L
s
.78571 .03399 m
.78571 .03774 L
s
.83333 .03399 m
.83333 .03774 L
s
.88095 .03399 m
.88095 .03774 L
s
.92857 .03399 m
.92857 .03774 L
s
.25 Mabswid
0 .03399 m
1 .03399 L
s
.02381 .12365 m
.03006 .12365 L
s
[(5.5)] .01131 .12365 1 0 Mshowa
.02381 .2133 m
.03006 .2133 L
s
[(6)] .01131 .2133 1 0 Mshowa
.02381 .30296 m
.03006 .30296 L
s
[(6.5)] .01131 .30296 1 0 Mshowa
.02381 .39262 m
.03006 .39262 L
s
[(7)] .01131 .39262 1 0 Mshowa
.02381 .48227 m
.03006 .48227 L
s
[(7.5)] .01131 .48227 1 0 Mshowa
.02381 .57193 m
.03006 .57193 L
s
[(8)] .01131 .57193 1 0 Mshowa
.125 Mabswid
.02381 .05192 m
.02756 .05192 L
s
.02381 .06985 m
.02756 .06985 L
s
.02381 .08779 m
.02756 .08779 L
s
.02381 .10572 m
.02756 .10572 L
s
.02381 .14158 m
.02756 .14158 L
s
.02381 .15951 m
.02756 .15951 L
s
.02381 .17744 m
.02756 .17744 L
s
.02381 .19537 m
.02756 .19537 L
s
.02381 .23123 m
.02756 .23123 L
s
.02381 .24917 m
.02756 .24917 L
s
.02381 .2671 m
.02756 .2671 L
s
.02381 .28503 m
.02756 .28503 L
s
.02381 .32089 m
.02756 .32089 L
s
.02381 .33882 m
.02756 .33882 L
s
.02381 .35675 m
.02756 .35675 L
s
.02381 .37468 m
.02756 .37468 L
s
.02381 .41055 m
.02756 .41055 L
s
.02381 .42848 m
.02756 .42848 L
s
.02381 .44641 m
.02756 .44641 L
s
.02381 .46434 m
.02756 .46434 L
s
.02381 .5002 m
.02756 .5002 L
s
.02381 .51813 m
.02756 .51813 L
s
.02381 .53607 m
.02756 .53607 L
s
.02381 .554 m
.02756 .554 L
s
.02381 .01606 m
.02756 .01606 L
s
.02381 .58986 m
.02756 .58986 L
s
.02381 .60779 m
.02756 .60779 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 1 r
.001 Mabswid
.02381 .03399 m
.02605 .02867 L
.02846 .02394 L
.03053 .02066 L
.03279 .01792 L
.03416 .01666 L
.03543 .01578 L
.03665 .01519 L
.03793 .01482 L
.03908 .01472 L
.04016 .01481 L
.04135 .01512 L
.04262 .01569 L
.04393 .01655 L
.04516 .01758 L
.04793 .02073 L
.05037 .02442 L
.05297 .02926 L
.06244 .05423 L
.06791 .07335 L
.07382 .09736 L
.08454 .14809 L
.10458 .25824 L
.12413 .3692 L
.14558 .47643 L
.15544 .5165 L
.166 .55147 L
.17113 .56525 L
.17598 .57633 L
.18067 .58521 L
.18507 .59191 L
.18966 .59722 L
.1921 .59936 L
.19335 .60028 L
.19472 .60114 L
.19598 .6018 L
.19733 .60238 L
.19808 .60264 L
.19875 .60284 L
.20007 .60313 L
.20129 .60328 L
.2026 .60332 L
.20384 .60324 L
.20498 .60308 L
.20604 .60284 L
.20718 .6025 L
.20924 .60165 L
.21169 .60027 L
.21393 .59867 L
.21879 .59411 L
Mistroke
.2234 .5885 L
.23301 .57323 L
.24325 .55258 L
.26165 .50792 L
.28166 .45544 L
.30328 .4033 L
.31345 .38263 L
.32419 .36439 L
.33335 .35204 L
.3434 .34209 L
.34866 .33839 L
.35124 .33695 L
.35363 .33584 L
.35575 .33502 L
.3581 .33431 L
.36045 .3338 L
.36174 .33359 L
.36295 .33346 L
.36417 .33337 L
.36487 .33334 L
.36551 .33333 L
.36682 .33335 L
.36755 .33339 L
.36821 .33344 L
.3694 .33356 L
.37069 .33374 L
.37302 .33421 L
.37577 .33497 L
.37867 .33602 L
.38387 .33849 L
.39334 .34475 L
.40349 .35359 L
.42181 .3734 L
.4416 .39716 L
.46314 .42158 L
.47316 .43138 L
.48381 .44027 L
.4938 .44698 L
.50296 .45166 L
.50751 .45345 L
.51231 .45495 L
.51713 .45606 L
.51986 .45652 L
.52108 .45669 L
.5224 .45684 L
.52364 .45695 L
.52478 .45704 L
.52587 .4571 L
.52703 .45715 L
.52806 .45717 L
Mistroke
.52903 .45718 L
.53009 .45717 L
.53123 .45714 L
.53244 .45709 L
.53376 .457 L
.53501 .4569 L
.53616 .45679 L
.54067 .45617 L
.54332 .45568 L
.54576 .45515 L
.55125 .45369 L
.56121 .45022 L
.57982 .44159 L
.61933 .42021 L
.63921 .41108 L
.64789 .40786 L
.65732 .40498 L
.66253 .40369 L
.66735 .40268 L
.67225 .40185 L
.67685 .40124 L
.68167 .40077 L
.68423 .4006 L
.68555 .40053 L
.68699 .40047 L
.68768 .40044 L
.68841 .40042 L
.68973 .40039 L
.69051 .40038 L
.69124 .40037 L
.6919 .40037 L
.69262 .40037 L
.69392 .40038 L
.69457 .40039 L
.6953 .4004 L
.69657 .40043 L
.69777 .40047 L
.70036 .40059 L
.70275 .40074 L
.70819 .4012 L
.71297 .40176 L
.71807 .40247 L
.73661 .40602 L
.77827 .41629 L
.79919 .42089 L
.80836 .42253 L
.8184 .42403 L
.82311 .4246 L
.82814 .42513 L
.83248 .42551 L
Mistroke
.8372 .42585 L
.83998 .42602 L
.84254 .42614 L
.84494 .42624 L
.84758 .42632 L
.84872 .42635 L
.84995 .42637 L
.85101 .42639 L
.85217 .42641 L
.85347 .42642 L
.8547 .42642 L
.85591 .42642 L
.85703 .42642 L
.85831 .42641 L
.85896 .42641 L
.85967 .4264 L
.8621 .42636 L
.86347 .42633 L
.86494 .42629 L
.86759 .4262 L
.87279 .42598 L
.87754 .42571 L
.88709 .42502 L
.89614 .42421 L
.93446 .41993 L
.97523 .416 L
.97619 .41593 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  Background->GrayLevel[1],
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005X0000gR000`400?l00000o`00003oo`3ooomY
0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00H03oool4
000000D0oooo0P0000050?ooo`D00000?03oool4000000D0oooo0P0000040?ooo`D00000?03oool4
000000D0oooo0P0000050?ooo`<00000?@3oool4000000D0oooo0P0000070?ooo`<000000P3oool0
05l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`8000001`3oool00`00
0000oooo0?ooo`0k0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool20000
00@0oooo00D000000?ooo`3oool0oooo0000000k0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool00P3oool2000000@0oooo00D000000?ooo`3oool0oooo0000000k0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool00P3oool2000000P0oooo00<000000?ooo`3oool00@3o
ool005l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo03/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo03`0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo03T0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo0040oooo001O0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0k0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0k0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`0i0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0203oool60000
0080oooo000J0?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0;0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0;0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0:0?ooo`800000?03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00T0
oooo00@000000?ooo`3oool000000`3oool001X0oooo00<000000?ooo`3oool0@P3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0>`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0>@3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0>@3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00T0oooo00@000000?ooo`3oool000000`3oool001X0
oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00T0
oooo0`00000m0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0203oool01@000000
oooo0?ooo`3oool0000003/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`080?oo
o`050000003oool0oooo0?ooo`000000>`3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`0000000`3oool001X0oooo00<000000?ooo`3oool0@`3oool40000
00h0oooo00<000000?ooo`3oool0?03oool4000000`0oooo0`00000m0?ooo`@00000303oool30000
03d0oooo1000000>0?ooo`8000000`3oool001X0oooo00<000000?ooo`3oool0o`3ooom<0?ooo`00
6P3oool200000080oooo10000?oo0?ooodL0oooo000J0?ooo`040000003oool0oooo0000o`<0oooo
00<0003o0?ooo`3oool0o`3ooom50?ooo`006P3oool00`000000oooo0000o`050?ooo`030000o`3o
ool0oooo0?l0ooooA03oool001X0oooo00<000000?ooo`000?l01P3oool00`000?l0oooo0?ooo`3o
0?oood<0oooo000J0?ooo`03000000000?l0oooo00H0oooo00<0003o0?ooo`3oool0o`3ooom30?oo
o`006P3oool00`000?l0oooo0?ooo`070?ooo`030000o`3oool0oooo0?l0oooo@P3oool00180oooo
200000000`000?l0000000000007000000030000o`00000000000?l00000?P0000040?ooo`006P3o
ool00`000000oooo0?ooo`080?ooo`030000o`3oool0oooo0080oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0:0?ooo`006P3oool00`000000oooo0?ooo`080?ooo`030000o`3oool0
oooo0480oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?ooo`1=0?ooo`030000003oool0
oooo04d0oooo00<000000?ooo`3oool02P3oool001X0oooo00<000000?ooo`3oool0203oool00`00
0?l0oooo0?ooo`3o0?oood40oooo000J0?ooo`030000003oool0oooo00T0oooo00<0003o0?ooo`3o
ool0o`3ooom00?ooo`006P3oool00`000000oooo0?ooo`090?ooo`030000o`3oool0oooo0?l0oooo
@03oool001X0oooo0P00000;0?ooo`030000o`3oool0oooo0?l0oooo?`3oool001X0oooo00<00000
0?ooo`3oool02P3oool00`000?l0oooo0?ooo`3o0?ooocl0oooo000J0?ooo`030000003oool0oooo
00X0oooo00<0003o0?ooo`3oool0o`3ooolo0?ooo`006P3oool00`000000oooo0?ooo`0;0?ooo`03
0000o`3oool0oooo0?l0oooo?P3oool001X0oooo00<000000?ooo`3oool02`3oool00`000?l0oooo
0?ooo`3o0?oooch0oooo000J0?ooo`030000003oool0oooo00/0oooo00<0003o0?ooo`3oool0o`3o
ooln0?ooo`006P3oool2000000d0oooo00<0003o0?ooo`3oool0o`3ooolm0?ooo`006P3oool00`00
0000oooo0?ooo`0<0?ooo`030000o`3oool0oooo0?l0oooo?@3oool001X0oooo00<000000?ooo`3o
ool0303oool00`000?l0oooo0?ooo`3o0?ooocd0oooo000J0?ooo`030000003oool0oooo00`0oooo
00<0003o0?ooo`3oool0o`3ooolm0?ooo`006P3oool00`000000oooo0?ooo`0=0?ooo`030000o`3o
ool0oooo0?l0oooo?03oool001X0oooo00<000000?ooo`3oool03@3oool00`000?l0oooo0?ooo`3o
0?oooc`0oooo000J0?ooo`8000003P3oool00`000?l0oooo0?ooo`3o0?oooc`0oooo000J0?ooo`03
0000003oool0oooo00d0oooo00<0003o0?ooo`3oool0o`3oooll0?ooo`006P3oool00`000000oooo
0?ooo`0>0?ooo`030000o`3oool0oooo0?l0oooo>`3oool001X0oooo00<000000?ooo`3oool03P3o
ool00`000?l0oooo0?ooo`3o0?oooc/0oooo000J0?ooo`030000003oool0oooo00h0oooo00<0003o
0?ooo`3oool0o`3ooolk0?ooo`006P3oool00`000000oooo0?ooo`0>0?ooo`030000o`3oool0oooo
0?l0oooo>`3oool001X0oooo0P00000@0?ooo`030000o`3oool0oooo0?l0oooo>P3oool001X0oooo
00<000000?ooo`3oool03`3oool00`000?l0oooo0?ooo`3o0?ooocX0oooo000J0?ooo`030000003o
ool0oooo00l0oooo00<0003o0?ooo`3oool0o`3ooolj0?ooo`00100000050?ooo`8000001@3oool4
000000H0oooo00<000000?ooo`3oool03`3oool00`000?l0oooo0?ooo`3o0?ooocX0oooo00040?oo
o`030000003oool0oooo0080oooo0P0000040?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000o`3oool0oooo0?l0oooo>@3oool000@0
oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0100
oooo00<0003o0?ooo`3oool0o`3oooli0?ooo`00103oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo00<0oooo0`00000@0?ooo`030000o`3oool0oooo0?l0oooo>@3oool000@00000303o
ool4000000H0oooo00<000000?ooo`3oool0403oool00`000?l0oooo0?ooo`3o0?ooocT0oooo0000
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0403oool0
0`000?l0oooo0?ooo`3o0?ooocT0oooo00000`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool04@3oool00`000?l0oooo0?ooo`3o0?ooocP0oooo0005000000/0
oooo1@0000050?ooo`030000003oool0oooo0140oooo00<0003o0?ooo`3oool0o`3ooolh0?ooo`00
6P3oool00`000000oooo0?ooo`0A0?ooo`030000o`3oool0oooo0?l0oooo>03oool001X0oooo0P00
000B0?ooo`030000o`3oool0oooo0?l0oooo>03oool001X0oooo00<000000?ooo`3oool04P3oool0
0`000?l0oooo0?ooo`3o0?ooocL0oooo000J0?ooo`030000003oool0oooo0180oooo00<0003o0?oo
o`3oool0o`3ooolg0?ooo`006P3oool00`000000oooo0?ooo`0B0?ooo`030000o`3oool0oooo0?l0
oooo=`3oool001X0oooo00<000000?ooo`3oool04P3oool00`000?l0oooo0?ooo`3o0?ooocL0oooo
000J0?ooo`030000003oool0oooo0180oooo00<0003o0?ooo`3oool0o`3ooolg0?ooo`006P3oool2
000001<0oooo00<0003o0?ooo`3oool0o`3ooolg0?ooo`006P3oool00`000000oooo0?ooo`0C0?oo
o`030000o`3oool0oooo0?l0oooo=P3oool001X0oooo00<000000?ooo`3oool04`3oool00`000?l0
oooo0?ooo`3o0?ooocH0oooo000J0?ooo`030000003oool0oooo01<0oooo00<0003o0?ooo`3oool0
o`3ooolf0?ooo`006P3oool00`000000oooo0?ooo`0C0?ooo`030000o`3oool0oooo0?l0oooo=P3o
ool001X0oooo00<000000?ooo`3oool04`3oool00`000?l0oooo0?ooo`3o0?ooocH0oooo000J0?oo
o`800000503oool00`000?l0oooo0?ooo`3o0?ooocH0oooo000J0?ooo`030000003oool0oooo01@0
oooo00<0003o0?ooo`3oool0o`3ooole0?ooo`006P3oool00`000000oooo0?ooo`0D0?ooo`030000
o`3oool0oooo0?l0oooo=@3oool001X0oooo00<000000?ooo`3oool0503oool00`000?l0oooo0?oo
o`3o0?ooocD0oooo000J0?ooo`030000003oool0oooo01@0oooo00<0003o0?ooo`3oool0o`3ooole
0?ooo`006P3oool00`000000oooo0?ooo`0D0?ooo`030000o`3oool0oooo0?l0oooo=@3oool001X0
oooo00<000000?ooo`3oool0503oool00`000?l0oooo0?ooo`3o0?ooocD0oooo000J0?ooo`800000
5P3oool00`000?l0oooo0?ooo`3o0?oooc@0oooo000J0?ooo`030000003oool0oooo01D0oooo00<0
003o0?ooo`3oool0o`3ooold0?ooo`006P3oool00`000000oooo0?ooo`0E0?ooo`030000o`3oool0
oooo0?l0oooo=03oool00140oooo0`0000060?ooo`030000003oool0oooo01D0oooo00<0003o0?oo
o`3oool0o`3ooold0?ooo`00403oool01@000000oooo0?ooo`3oool0000000D0oooo00<000000?oo
o`3oool05@3oool00`000?l0oooo0?ooo`3o0?oooc@0oooo000@0?ooo`050000003oool0oooo0?oo
o`0000001@3oool00`000000oooo0?ooo`0E0?ooo`030000o`3oool0oooo0?l0oooo=03oool00100
oooo00D000000?ooo`3oool0oooo000000050?ooo`<000005P3oool00`000?l0oooo0?ooo`3o0?oo
oc<0oooo000@0?ooo`@000001P3oool00`000000oooo0?ooo`0F0?ooo`030000o`3oool0oooo0?l0
oooo<`3oool00100oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0F0?ooo`030000
o`3oool0oooo0?l0oooo<`3oool00140oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0F0?ooo`030000o`3oool0oooo0?l0oooo<`3oool00180oooo0`0000050?ooo`030000003oool0
oooo01H0oooo00<0003o0?ooo`3oool0o`3ooolc0?ooo`006P3oool00`000000oooo0?ooo`0F0?oo
o`030000o`3oool0oooo0?l0oooo<`3oool001X0oooo0P00000H0?ooo`030000o`3oool0oooo0?l0
oooo<P3oool001X0oooo00<000000?ooo`3oool05`3oool00`000?l0oooo0?ooo`3o0?oooc80oooo
000J0?ooo`030000003oool0oooo01L0oooo00<0003o0?ooo`3oool0o`3ooolb0?ooo`006P3oool0
0`000000oooo0?ooo`0G0?ooo`030000o`3oool0oooo0?l0oooo<P3oool001X0oooo00<000000?oo
o`3oool05`3oool00`000?l0oooo0?ooo`3o0?oooc80oooo000J0?ooo`030000003oool0oooo01L0
oooo00<0003o0?ooo`3oool0o`3ooolb0?ooo`006P3oool2000001T0oooo00<0003o0?ooo`3oool0
o`3ooola0?ooo`006P3oool00`000000oooo0?ooo`0H0?ooo`030000o`3oool0oooo0?l0oooo<@3o
ool001X0oooo00<000000?ooo`3oool0603oool00`000?l0oooo0?ooo`3o0?oooc40oooo000J0?oo
o`030000003oool0oooo01P0oooo00<0003o0?ooo`3oool0o`3ooola0?ooo`006P3oool00`000000
oooo0?ooo`0H0?ooo`030000o`3oool0oooo0?l0oooo<@3oool001X0oooo00<000000?ooo`3oool0
603oool00`000?l0oooo0?ooo`3o0?oooc40oooo000J0?ooo`8000006P3oool00`000?l0oooo0?oo
o`3o0?oooc00oooo000J0?ooo`030000003oool0oooo01T0oooo00<0003o0?ooo`3oool0o`3oool`
0?ooo`006P3oool00`000000oooo0?ooo`0I0?ooo`030000o`3oool0oooo0?l0oooo<03oool001X0
oooo00<000000?ooo`3oool06@3oool00`000?l0oooo0?ooo`3o0?oooc00oooo000J0?ooo`030000
003oool0oooo01T0oooo00<0003o0?ooo`3oool0o`3oool`0?ooo`006P3oool00`000000oooo0?oo
o`0J0?ooo`030000o`3oool0oooo0?l0oooo;`3oool001X0oooo0P00000K0?ooo`030000o`3oool0
oooo0?l0oooo;`3oool001X0oooo00<000000?ooo`3oool06P3oool00`000?l0oooo0?ooo`3o0?oo
obl0oooo000J0?ooo`030000003oool0oooo01X0oooo00<0003o0?ooo`3oool0o`3oool_0?ooo`00
00@0oooo00000000000000001@3oool2000000D0oooo100000060?ooo`030000003oool0oooo01X0
oooo00<0003o0?ooo`3oool0o`3oool_0?ooo`0000D000000?ooo`3oool0oooo000000040?ooo`80
0000103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool06P3oool00`000?l0oooo0?ooo`3o0?ooobl0oooo00001@000000oooo0?ooo`3oool0000000l0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0K0?ooo`030000o`3oool0oooo0?l0
oooo;P3oool000050000003oool0oooo0?ooo`0000003`3oool00`000000oooo0?ooo`030?ooo`<0
00006`3oool00`000?l0oooo0?ooo`3o0?ooobh0oooo0004000000`0oooo100000060?ooo`030000
003oool0oooo01/0oooo00<0003o0?ooo`3oool0o`3oool^0?ooo`0000<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01/0oooo00<0003o0?ooo`3oool0o`3o
ool^0?ooo`0000<0oooo0000003oool03@3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo01/0oooo00<0003o0?ooo`3oool0o`3oool^0?ooo`000P3oool3000000/0oooo1@0000050?oo
o`030000003oool0oooo01`0oooo00<0003o0?ooo`3oool0o`3oool]0?ooo`006P3oool00`000000
oooo0?ooo`0L0?ooo`030000o`3oool0oooo0?l0oooo;@3oool001X0oooo0P00000M0?ooo`030000
o`3oool0oooo0?l0oooo;@3oool001X0oooo00<000000?ooo`3oool0703oool00`000?l0oooo0?oo
o`3o0?ooobd0oooo000J0?ooo`030000003oool0oooo01`0oooo00<0003o0?ooo`3oool0o`3oool]
0?ooo`006P3oool00`000000oooo0?ooo`0M0?ooo`030000o`3oool0oooo0?l0oooo;03oool001X0
oooo00<000000?ooo`3oool07@3oool00`000?l0oooo0?ooo`1>0?ooo`H0003oe`3oool001X0oooo
00<000000?ooo`3oool07@3oool00`000?l0oooo0?ooo`1<0?ooo`80003o1P3oool20000omD0oooo
000J0?ooo`8000007P3oool00`000?l0oooo0?ooo`1;0?ooo`030000o`3oool0oooo00P0oooo0P00
0?oC0?ooo`006P3oool00`000000oooo0?ooo`0M0?ooo`030000o`3oool0oooo04T0oooo0P000?l=
0?ooo`80003od@3oool001X0oooo00<000000?ooo`3oool07@3oool00`000?l0oooo0?ooo`180?oo
o`030000o`3oool0oooo00l0oooo00<0003o0?ooo`3oool0cP3oool001X0oooo00<000000?ooo`3o
ool07P3oool00`000?l0oooo0?ooo`160?ooo`030000o`3oool0oooo0140oooo00<0003o0?ooo`3o
ool0c@3oool001X0oooo00<000000?ooo`3oool07P3oool00`000?l0oooo0?ooo`150?ooo`030000
o`3oool0oooo01<0oooo00<0003o0?ooo`3oool0c03oool001X0oooo00<000000?ooo`3oool07P3o
ool00`000?l0oooo0?ooo`140?ooo`030000o`3oool0oooo01D0oooo00<0003o0?ooo`3oool0b`3o
ool001X0oooo0P00000O0?ooo`030000o`3oool0oooo04<0oooo00<0003o0?ooo`3oool05`3oool0
0`000?l0oooo0?ooo`3:0?ooo`006P3oool00`000000oooo0?ooo`0N0?ooo`030000o`3oool0oooo
0480oooo00<0003o0?ooo`3oool06@3oool00`000?l0oooo0?ooo`390?ooo`006P3oool00`000000
oooo0?ooo`0O0?ooo`030000o`3oool0oooo0400oooo00<0003o0?ooo`3oool06`3oool00`000?l0
oooo0?ooo`380?ooo`006P3oool00`000000oooo0?ooo`0O0?ooo`030000o`3oool0oooo0400oooo
00<0003o0?ooo`3oool06`3oool00`000?l0oooo0?ooo`380?ooo`006P3oool00`000000oooo0?oo
o`0O0?ooo`030000o`3oool0oooo03l0oooo00<0003o0?ooo`3oool07@3oool00`000?l0oooo0?oo
o`370?ooo`006P3oool00`000000oooo0?ooo`0O0?ooo`030000o`3oool0oooo03l0oooo00<0003o
0?ooo`3oool07P3oool00`000?l0oooo0?ooo`360?ooo`006P3oool200000200oooo00<0003o0?oo
o`3oool0?P3oool00`000?l0oooo0?ooo`0P0?ooo`030000o`3oool0oooo0<D0oooo000J0?ooo`03
0000003oool0oooo0200oooo00<0003o0?ooo`3oool0?@3oool00`000?l0oooo0?ooo`0Q0?ooo`03
0000o`3oool0oooo0<@0oooo000J0?ooo`030000003oool0oooo0200oooo00<0003o0?ooo`3oool0
?03oool00`000?l0oooo0?ooo`0S0?ooo`030000o`3oool0oooo0<<0oooo000B0?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0803oool00`000?l0oooo0?ooo`0l0?ooo`030000o`3o
ool0oooo02@0oooo00<0003o0?ooo`3oool0`P3oool00180oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`0P0?ooo`030000o`3oool0oooo03/0oooo00<0003o0?ooo`3oool09@3oool0
0`000?l0oooo0?ooo`320?ooo`004P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
0200oooo00<0003o0?ooo`3oool0>`3oool00`000?l0oooo0?ooo`0V0?ooo`030000o`3oool0oooo
0<40oooo000C0?ooo`030000003oool0oooo00@0oooo0`00000Q0?ooo`030000o`3oool0oooo03T0
oooo00<0003o0?ooo`3oool0:03oool00`000?l0oooo0?ooo`300?ooo`004`3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo0240oooo00<0003o0?ooo`3oool0>@3oool00`000?l0oooo
0?ooo`0Y0?ooo`030000o`3oool0oooo0;l0oooo000D0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool08@3oool00`000?l0oooo0?ooo`0h0?ooo`030000o`3oool0oooo02/0oooo00<0
003o0?ooo`3oool0_P3oool000l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0240oooo00<0003o0?ooo`3oool0>03oool00`000?l0oooo0?ooo`0/
0?ooo`030000o`3oool0oooo04P0oooo4`000?mR0?ooo`003`3oool6000000D0oooo00<000000?oo
o`3oool08@3oool00`000?l0oooo0?ooo`0g0?ooo`030000o`3oool0oooo02h0oooo00<0003o0?oo
o`3oool0@`3oool40000oa<0oooo0`000?mO0?ooo`006P3oool00`000000oooo0?ooo`0R0?ooo`03
0000o`3oool0oooo03H0oooo00<0003o0?ooo`3oool0;`3oool00`000?l0oooo0?ooo`0o0?ooo`<0
003o6P3oool40000oe/0oooo000J0?ooo`8000008`3oool00`000?l0oooo0?ooo`0e0?ooo`030000
o`3oool0oooo0300oooo00<0003o0?ooo`3oool0?03oool30000ob40oooo1@000?mF0?ooo`006P3o
ool00`000000oooo0?ooo`0R0?ooo`030000o`3oool0oooo03D0oooo00<0003o0?ooo`3oool0<@3o
ool00`000?l0oooo0?ooo`0i0?ooo`80003o:@3oool50000oe40oooo000J0?ooo`030000003oool0
oooo0280oooo00<0003o0?ooo`3oool0=03oool00`000?l0oooo0?ooo`0c0?ooo`030000o`3oool0
oooo03H0oooo0P000?l`0?ooo`@0003o>P3oool70000o``0oooo000J0?ooo`030000003oool0oooo
0280oooo00<0003o0?ooo`3oool0=03oool00`000?l0oooo0?ooo`0d0?ooo`030000o`3oool0oooo
03<0oooo0P000?lf0?ooo`@0003o:`3oool;0000oa<0oooo000J0?ooo`030000003oool0oooo0280
oooo00<0003o0?ooo`3oool0=03oool00`000?l0oooo0?ooo`0e0?ooo`030000o`3oool0oooo0300
oooo0P000?ll0?ooo`H0003o7`3oool60000oah0oooo000J0?ooo`030000003oool0oooo02<0oooo
00<0003o0?ooo`3oool0<P3oool00`000?l0oooo0?ooo`0g0?ooo`030000o`3oool0oooo02d0oooo
0P000?m40?oooal0003o903oool001X0oooo0P00000T0?ooo`030000o`3oool0oooo0380oooo00<0
003o0?ooo`3oool0>03oool20000ob`0oooo00<0003o0?ooo`3oool0Q`3oool001X0oooo00<00000
0?ooo`3oool08`3oool00`000?l0oooo0?ooo`0a0?ooo`030000o`3oool0oooo03/0oooo00<0003o
0?ooo`3oool09`3oool20000ohX0oooo000J0?ooo`030000003oool0oooo02<0oooo00<0003o0?oo
o`3oool0<@3oool00`000?l0oooo0?ooo`0l0?ooo`030000o`3oool0oooo02D0oooo00<0003o0?oo
o`3oool0RP3oool001X0oooo00<000000?ooo`3oool08`3oool00`000?l0oooo0?ooo`0`0?ooo`03
0000o`3oool0oooo03h0oooo00<0003o0?ooo`3oool08P3oool20000ohd0oooo000J0?ooo`030000
003oool0oooo02@0oooo00<0003o0?ooo`3oool0;`3oool00`000?l0oooo0?ooo`0o0?ooo`030000
o`3oool0oooo01l0oooo0P000?n?0?ooo`006P3oool00`000000oooo0?ooo`0T0?ooo`030000o`3o
ool0oooo02l0oooo00<0003o0?ooo`3oool0@03oool20000oad0oooo0P000?nA0?ooo`006P3oool2
000002D0oooo00<0003o0?ooo`3oool0;P3oool00`000?l0oooo0?ooo`130?ooo`80003o5`3oool4
0000oi<0oooo000J0?ooo`030000003oool0oooo02@0oooo00<0003o0?ooo`3oool0;P3oool00`00
0?l0oooo0?ooo`150?ooo`80003o4`3oool20000oiL0oooo000J0?ooo`030000003oool0oooo02@0
oooo00<0003o0?ooo`3oool0;@3oool00`000?l0oooo0?ooo`180?ooo`030000o`3oool0oooo00h0
oooo0P000?nI0?ooo`006P3oool00`000000oooo0?ooo`0U0?ooo`030000o`3oool0oooo02`0oooo
00<0003o0?ooo`3oool0B@3oool50000o`H0oooo1@000?nK0?ooo`006P3oool00`000000oooo0?oo
o`0U0?ooo`030000o`3oool0oooo02`0oooo00<0003o0?ooo`3oool0C@3oool70000oj00oooo000J
0?ooo`030000003oool0oooo02D0oooo00<0003o0?ooo`3oool0:`3oool00`000?l0oooo0?ooo`3e
0?ooo`006P3oool2000002H0oooo00<0003o0?ooo`3oool0:`3oool00`000?l0oooo0?ooo`3e0?oo
o`006P3oool00`000000oooo0?ooo`0U0?ooo`030000o`3oool0oooo02X0oooo00<0003o0?ooo`3o
ool0mP3oool001X0oooo00<000000?ooo`3oool09P3oool00`000?l0oooo0?ooo`0Y0?ooo`030000
o`3oool0oooo0?H0oooo00020?ooo`030000003oool0oooo00@0oooo0P0000050?ooo`@000001P3o
ool00`000000oooo0?ooo`0V0?ooo`030000o`3oool0oooo02T0oooo00<0003o0?ooo`3oool0mP3o
ool00080oooo00<000000?ooo`3oool0103oool2000000@0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo02H0oooo00<0003o0?ooo`3oool0:03oool0
0`000?l0oooo0?ooo`3g0?ooo`000P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool09P3oool00`000?l0oooo0?ooo`0X0?ooo`030000o`3oool0oooo
0?L0oooo00030?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool00`3oool3000002L0
oooo00<0003o0?ooo`3oool09P3oool00`000?l0oooo0?ooo`3h0?ooo`000`3oool00`000000oooo
0?ooo`0:0?ooo`@000001P3oool00`000000oooo0?ooo`0W0?ooo`030000o`3oool0oooo02H0oooo
00<0003o0?ooo`3oool0n03oool000@0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo02L0oooo00<0003o0?ooo`3oool09P3oool00`000?l0oooo0?oo
o`3h0?ooo`00103oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool09`3oool00`000?l0oooo0?ooo`0U0?ooo`030000o`3oool0oooo0?T0oooo00050000
00/0oooo1@0000050?ooo`030000003oool0oooo02P0oooo00<0003o0?ooo`3oool0903oool00`00
0?l0oooo0?ooo`3i0?ooo`006P3oool00`000000oooo0?ooo`0X0?ooo`030000o`3oool0oooo02@0
oooo00<0003o0?ooo`3oool0n@3oool001X0oooo0P00000Y0?ooo`030000o`3oool0oooo02<0oooo
00<0003o0?ooo`3oool0nP3oool001X0oooo00<000000?ooo`3oool0:@3oool00`000?l0oooo0?oo
o`0R0?ooo`030000o`3oool0oooo0?X0oooo000J0?ooo`030000003oool0oooo02T0oooo00<0003o
0?ooo`3oool08@3oool00`000?l0oooo0?ooo`3k0?ooo`006P3oool00`000000oooo0?ooo`0Y0?oo
o`030000o`3oool0oooo0240oooo00<0003o0?ooo`3oool0n`3oool001X0oooo00<000000?ooo`3o
ool0:@3oool00`000?l0oooo0?ooo`0Q0?ooo`030000o`3oool0oooo0?/0oooo000J0?ooo`030000
003oool0oooo02X0oooo00<0003o0?ooo`3oool07`3oool00`000?l0oooo0?ooo`3l0?ooo`006P3o
ool2000002/0oooo00<0003o0?ooo`3oool07`3oool00`000?l0oooo0?ooo`3l0?ooo`006P3oool0
0`000000oooo0?ooo`0Z0?ooo`030000o`3oool0oooo01h0oooo00<0003o0?ooo`3oool0o@3oool0
01X0oooo00<000000?ooo`3oool0:P3oool00`000?l0oooo0?ooo`0N0?ooo`030000o`3oool0oooo
0?d0oooo000J0?ooo`030000003oool0oooo02/0oooo00<0003o0?ooo`3oool07@3oool00`000?l0
oooo0?ooo`3m0?ooo`006P3oool00`000000oooo0?ooo`0[0?ooo`030000o`3oool0oooo01`0oooo
00<0003o0?ooo`3oool0oP3oool001X0oooo00<000000?ooo`3oool0:`3oool00`000?l0oooo0?oo
o`0L0?ooo`030000o`3oool0oooo0?h0oooo000J0?ooo`800000;03oool00`000?l0oooo0?ooo`0K
0?ooo`030000o`3oool0oooo0?l0oooo000J0?ooo`030000003oool0oooo02`0oooo00<0003o0?oo
o`3oool06P3oool00`000?l0oooo0?ooo`3o0?ooo`006P3oool00`000000oooo0?ooo`0/0?ooo`03
0000o`3oool0oooo01X0oooo00<0003o0?ooo`3oool0o`3oool001X0oooo00<000000?ooo`3oool0
;03oool00`000?l0oooo0?ooo`0I0?ooo`030000o`3oool0oooo0?l0oooo0@3oool001X0oooo00<0
00000?ooo`3oool0;03oool00`000?l0oooo0?ooo`0I0?ooo`030000o`3oool0oooo0?l0oooo0@3o
ool001X0oooo00<000000?ooo`3oool0;@3oool00`000?l0oooo0?ooo`0G0?ooo`030000o`3oool0
oooo0?l0oooo0P3oool001X0oooo0P00000^0?ooo`030000o`3oool0oooo01L0oooo00<0003o0?oo
o`3oool0o`3oool20?ooo`006P3oool00`000000oooo0?ooo`0]0?ooo`030000o`3oool0oooo01L0
oooo00<0003o0?ooo`3oool0o`3oool20?ooo`006P3oool00`000000oooo0?ooo`0^0?ooo`030000
o`3oool0oooo01D0oooo00<0003o0?ooo`3oool0o`3oool30?ooo`00403oool4000000H0oooo00<0
00000?ooo`3oool0;P3oool00`000?l0oooo0?ooo`0E0?ooo`030000o`3oool0oooo0?l0oooo0`3o
ool000l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo02l0oooo00<0003o0?ooo`3oool04`3oool00`000?l0oooo0?ooo`3o0?ooo`@0oooo000?0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0_0?oo
o`030000o`3oool0oooo01<0oooo00<0003o0?ooo`3oool0o`3oool40?ooo`003`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo0`00000`0?ooo`030000o`3oool0oooo0140
oooo00<0003o0?ooo`3oool0o`3oool50?ooo`00403oool4000000H0oooo00<000000?ooo`3oool0
<03oool00`000?l0oooo0?ooo`0A0?ooo`030000o`3oool0oooo0?l0oooo1@3oool000l0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0340oooo00<0
003o0?ooo`3oool03`3oool00`000?l0oooo0?ooo`3o0?ooo`H0oooo000?0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0a0?ooo`030000o`3oool0
oooo00h0oooo00<0003o0?ooo`3oool0o`3oool70?ooo`00403oool4000000H0oooo00<000000?oo
o`3oool0<P3oool00`000?l0oooo0?ooo`0=0?ooo`030000o`3oool0oooo0?l0oooo1`3oool001X0
oooo00<000000?ooo`3oool0<P3oool00`000?l0oooo0?ooo`0<0?ooo`030000o`3oool0oooo0?l0
oooo203oool001X0oooo0P00000d0?ooo`030000o`3oool0oooo00X0oooo00<0003o0?ooo`3oool0
o`3oool90?ooo`006P3oool00`000000oooo0?ooo`0d0?ooo`030000o`3oool0oooo00P0oooo00<0
003o0?ooo`3oool0o`3oool:0?ooo`006P3oool00`000000oooo0?ooo`0d0?ooo`030000o`3oool0
oooo00P0oooo00<0003o0?ooo`3oool0o`3oool:0?ooo`006P3oool00`000000oooo0?ooo`0e0?oo
o`030000o`3oool0oooo00D0oooo0P000?oo0?ooo`d0oooo000J0?ooo`030000003oool0oooo03H0
oooo0P000?l40?ooo`80003oo`3oool>0?ooo`006P3oool00`000000oooo0?ooo`0g0?ooo`D0003o
o`3oool@0?ooo`006P3oool00`000000oooo0?ooo`3o0?oood`0oooo000J0?ooo`800000o`3ooom=
0?ooo`006P3oool00`000000oooo0?ooo`3o0?oood`0oooo000J0?ooo`030000003oool0oooo0?l0
ooooC03oool001X0oooo00<000000?ooo`3oool0o`3ooom<0?ooo`00o`3ooomY0?ooo`00o`3ooomY
0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 359}, {221.375, 0}} -> {-0.0332863, 4.66937, \
0.00155747, 0.0206805}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1st-Order Ripple Correction", "Subsection",
  CellTags->"Large-Signal Simulation"],

Cell[TextData[{
  "Ripple corrections can be added to the above averaged responses. Using the \
",
  ButtonBox["ripple estimation results",
    ButtonData:>"Ripple Functions",
    ButtonStyle->"Hyperlink"],
  " obtained before, the 1st-order ripple component of ",
  Cell[BoxData[
      \(TraditionalForm\`i\_L\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`v\_C\)]],
  " are defined first as functions of time ",
  StyleBox["t",
    FontSlant->"Italic"],
  ": (Future versions of the package will be able generate these functions \
automatically from the ripple estimation results given before.)"
}], "Text",
  CellTags->"Large-Signal Simulation"],

Cell[BoxData[{
    \(iLRip1[t_]\  := \ 
      If[Mod[t, \ Ts] < 
              d\ Ts, \((d\  - \ 1)\)\ \((\(-2\)\ Mod[t, \ Ts]\  + \ d\ Ts)\), 
            d\ \((\(-2\)\ Mod[t, \ Ts]\  + \ Ts\  + \ d\ Ts)\)]*
          vC[t]/\((2\ L)\)\  /. \ Param1\), "\n", 
    \(vCRip1[t_]\  := \ 
      If[Mod[t, \ Ts] < 
              d\ Ts, \ \((\(-1\)\  + \ d)\)\ \((2\ Mod[t, \ Ts]\  - \ 
                  d\ Ts)\), \ d\ \((2\ Mod[t, \ Ts]\  - \ Ts\  - \ d\ Ts)\)]*
          iL[t]/\((2\ C)\)\  /. \ Param1\)}], "Input",
  CellTags->"Large-Signal Simulation"],

Cell["\<\
Transient responses of the averaged model with 1st-order ripple \
corrections can now be obtained by adding the ripple to the averaged \
responses calculated before:\
\>", "Text",
  CellTags->"Large-Signal Simulation"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(iLCor\  = \ 
        Plot[Evaluate[iLRip1[10^\(-3\)\ t]\  /. \ ResAve]\  + 
            Evaluate[iL[10^\(-3\)\ t]\  /. \ ResAve], {t, \ 0, \ 10^3\ T0}, \ 
          PlotPoints\  -> \ 200, 
          PlotStyle\  -> \ {{AbsoluteThickness[0.001], \ 
                RGBColor[1, \ 0, \ 1]}}];\)\)], "Input",
  CellTags->"Large-Signal Simulation"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 2.38095 -0.18043 0.494038 [
[.2619 .30111 -9 -9 ]
[.2619 .30111 9 0 ]
[.5 .30111 -9 -9 ]
[.5 .30111 9 0 ]
[.7381 .30111 -9 -9 ]
[.7381 .30111 9 0 ]
[.97619 .30111 -9 -9 ]
[.97619 .30111 9 0 ]
[.01131 .01719 -18 -4.5 ]
[.01131 .01719 0 4.5 ]
[.01131 .11599 -18 -4.5 ]
[.01131 .11599 0 4.5 ]
[.01131 .2148 -18 -4.5 ]
[.01131 .2148 0 4.5 ]
[.01131 .41242 -18 -4.5 ]
[.01131 .41242 0 4.5 ]
[.01131 .51122 -18 -4.5 ]
[.01131 .51122 0 4.5 ]
[.01131 .61003 -18 -4.5 ]
[.01131 .61003 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .31361 m
.2619 .31986 L
s
[(0.1)] .2619 .30111 0 1 Mshowa
.5 .31361 m
.5 .31986 L
s
[(0.2)] .5 .30111 0 1 Mshowa
.7381 .31361 m
.7381 .31986 L
s
[(0.3)] .7381 .30111 0 1 Mshowa
.97619 .31361 m
.97619 .31986 L
s
[(0.4)] .97619 .30111 0 1 Mshowa
.125 Mabswid
.07143 .31361 m
.07143 .31736 L
s
.11905 .31361 m
.11905 .31736 L
s
.16667 .31361 m
.16667 .31736 L
s
.21429 .31361 m
.21429 .31736 L
s
.30952 .31361 m
.30952 .31736 L
s
.35714 .31361 m
.35714 .31736 L
s
.40476 .31361 m
.40476 .31736 L
s
.45238 .31361 m
.45238 .31736 L
s
.54762 .31361 m
.54762 .31736 L
s
.59524 .31361 m
.59524 .31736 L
s
.64286 .31361 m
.64286 .31736 L
s
.69048 .31361 m
.69048 .31736 L
s
.78571 .31361 m
.78571 .31736 L
s
.83333 .31361 m
.83333 .31736 L
s
.88095 .31361 m
.88095 .31736 L
s
.92857 .31361 m
.92857 .31736 L
s
.25 Mabswid
0 .31361 m
1 .31361 L
s
.02381 .01719 m
.03006 .01719 L
s
[(0.4)] .01131 .01719 1 0 Mshowa
.02381 .11599 m
.03006 .11599 L
s
[(0.6)] .01131 .11599 1 0 Mshowa
.02381 .2148 m
.03006 .2148 L
s
[(0.8)] .01131 .2148 1 0 Mshowa
.02381 .41242 m
.03006 .41242 L
s
[(1.2)] .01131 .41242 1 0 Mshowa
.02381 .51122 m
.03006 .51122 L
s
[(1.4)] .01131 .51122 1 0 Mshowa
.02381 .61003 m
.03006 .61003 L
s
[(1.6)] .01131 .61003 1 0 Mshowa
.125 Mabswid
.02381 .04189 m
.02756 .04189 L
s
.02381 .06659 m
.02756 .06659 L
s
.02381 .09129 m
.02756 .09129 L
s
.02381 .14069 m
.02756 .14069 L
s
.02381 .1654 m
.02756 .1654 L
s
.02381 .1901 m
.02756 .1901 L
s
.02381 .2395 m
.02756 .2395 L
s
.02381 .2642 m
.02756 .2642 L
s
.02381 .28891 m
.02756 .28891 L
s
.02381 .33831 m
.02756 .33831 L
s
.02381 .36301 m
.02756 .36301 L
s
.02381 .38771 m
.02756 .38771 L
s
.02381 .43712 m
.02756 .43712 L
s
.02381 .46182 m
.02756 .46182 L
s
.02381 .48652 m
.02756 .48652 L
s
.02381 .53593 m
.02756 .53593 L
s
.02381 .56063 m
.02756 .56063 L
s
.02381 .58533 m
.02756 .58533 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 1 r
.001 Mabswid
.02381 .01472 m
.02608 .062 L
.02847 .11162 L
.02964 .13589 L
.03029 .14922 L
.0306 .15577 L
.03074 .15864 L
.03089 .16181 L
.03098 .16302 L
.03106 .16304 L
.03124 .16308 L
.03161 .16318 L
.03227 .16337 L
.03355 .1638 L
.03823 .16571 L
.0406 .16677 L
.0432 .16786 L
.0444 .16832 L
.04568 .16877 L
.0463 .16896 L
.04689 .16913 L
.04715 .1692 L
.04742 .16927 L
.04756 .1693 L
.04771 .17129 L
.04786 .17421 L
.04799 .17693 L
.05268 .27421 L
.05391 .29976 L
.05424 .30677 L
.05443 .31061 L
.0546 .31422 L
.05475 .31728 L
.05491 .31762 L
.05499 .31763 L
.05508 .31764 L
.05524 .31766 L
.05553 .31769 L
.05584 .31772 L
.05613 .31775 L
.0564 .31776 L
.05656 .31777 L
.05671 .31777 L
.05679 .31777 L
.05688 .31777 L
.05697 .31777 L
.05705 .31777 L
.05721 .31777 L
.05735 .31777 L
.05743 .31777 L
Mistroke
.05751 .31777 L
.05767 .31776 L
.05783 .31775 L
.05798 .31774 L
.05831 .31772 L
.05861 .31768 L
.05892 .31764 L
.05945 .31756 L
.06005 .31743 L
.06136 .31705 L
.06258 .31655 L
.06384 .31588 L
.06504 .3151 L
.06728 .31322 L
.06844 .31202 L
.06968 .31056 L
.07074 .30917 L
.07103 .30877 L
.07118 .30855 L
.07126 .30843 L
.07134 .30832 L
.07148 .30926 L
.07163 .31241 L
.0719 .31795 L
.07662 .41555 L
.07786 .44146 L
.0782 .44858 L
.07838 .45247 L
.07847 .45438 L
.07856 .45611 L
.0787 .45621 L
.07886 .45594 L
.0792 .45535 L
.08163 .45061 L
.08643 .43869 L
.09105 .42384 L
.09343 .41483 L
.09401 .41249 L
.09464 .40991 L
.09481 .40918 L
.09499 .40841 L
.09515 .40774 L
.09524 .40738 L
.09532 .40907 L
.09597 .42224 L
.09822 .46872 L
.10067 .51966 L
.10132 .53327 L
.10168 .54073 L
.10201 .54773 L
Mistroke
.10215 .55078 L
.10229 .55369 L
.10245 .55527 L
.1026 .5546 L
.10324 .55159 L
.10567 .53963 L
.11028 .51413 L
.11285 .4983 L
.11518 .48297 L
.11635 .47493 L
.11762 .46599 L
.1182 .46179 L
.11851 .45947 L
.11881 .45729 L
.11894 .45633 L
.11906 .4558 L
.1192 .4587 L
.11933 .46133 L
.11991 .47308 L
.1221 .51839 L
.12336 .54438 L
.12453 .56888 L
.12568 .59297 L
.12599 .59959 L
.12615 .60292 L
.12624 .60332 L
.12633 .60268 L
.12692 .59807 L
.12945 .57772 L
.13416 .53717 L
.13657 .51517 L
.13917 .49037 L
.14039 .47842 L
.14168 .46555 L
.14221 .46018 L
.14251 .45721 L
.14265 .45574 L
.14278 .45437 L
.14294 .45531 L
.14302 .457 L
.14311 .4588 L
.14341 .46494 L
.14399 .47697 L
.14641 .52688 L
.14768 .55327 L
.14902 .58122 L
.1496 .59341 L
.14977 .59692 L
.14993 .60024 L
.15007 .60109 L
Mistroke
.15022 .59947 L
.15091 .5923 L
.15154 .58556 L
.15386 .56059 L
.15879 .50547 L
.16124 .4771 L
.16353 .45 L
.16466 .43652 L
.16573 .4236 L
.16632 .41644 L
.16648 .41448 L
.16657 .4134 L
.16665 .41238 L
.16673 .41351 L
.16681 .41525 L
.16696 .41829 L
.16809 .44159 L
.17045 .49039 L
.1717 .51633 L
.17238 .53065 L
.17304 .54422 L
.17333 .55046 L
.17365 .55709 L
.17373 .55875 L
.17382 .56027 L
.17391 .5591 L
.174 .55801 L
.17432 .55404 L
.1755 .53949 L
.1778 .51074 L
.18276 .44766 L
.18523 .41575 L
.18753 .38585 L
.18868 .37095 L
.18976 .35684 L
.19004 .35305 L
.1902 .35098 L
.19035 .34903 L
.19043 .34802 L
.19051 .34814 L
.1906 .35004 L
.19069 .35178 L
.191 .35821 L
.19213 .38162 L
.19462 .43321 L
.19583 .45833 L
.19692 .4811 L
.19719 .48669 L
.19748 .49258 L
.19762 .49551 L
Mistroke
.19771 .4944 L
.19779 .49337 L
.19807 .48961 L
.19916 .47538 L
.20153 .44407 L
.20623 .38219 L
.20862 .35072 L
.21122 .31662 L
.21244 .30076 L
.2131 .29209 L
.21372 .28397 L
.21398 .28061 L
.21412 .27878 L
.21425 .27708 L
.21439 .27891 L
.21455 .28214 L
.21482 .2878 L
.21603 .31298 L
.2183 .35997 L
.21949 .38466 L
.22075 .41074 L
.22106 .41712 L
.22123 .42064 L
.22131 .42237 L
.22139 .42397 L
.22148 .42418 L
.22158 .4229 L
.22175 .42066 L
.22209 .41626 L
.22333 .40029 L
.22576 .36906 L
.23049 .30903 L
.23504 .25241 L
.23617 .23847 L
.23736 .22387 L
.23767 .22019 L
.23783 .21817 L
.23799 .21626 L
.23808 .21518 L
.23818 .21666 L
.23834 .22019 L
.23868 .22708 L
.23988 .25214 L
.24222 .30087 L
.24347 .32664 L
.24416 .34083 L
.24449 .34782 L
.24481 .3543 L
.24496 .35745 L
.2451 .36041 L
Mistroke
.24518 .36205 L
.24527 .3628 L
.24542 .36099 L
.24609 .35298 L
.24726 .339 L
.24955 .31188 L
.2545 .25473 L
.25696 .22694 L
.25926 .20142 L
.2604 .18897 L
.26096 .18288 L
.26147 .17727 L
.26176 .17416 L
.26191 .17269 L
.26199 .17442 L
.26207 .17604 L
.26271 .18953 L
.26384 .21308 L
.26607 .25938 L
.26734 .28552 L
.26795 .29823 L
.26851 .30977 L
.2688 .31572 L
.26888 .31735 L
.26896 .31907 L
.26911 .32014 L
.26925 .31865 L
.26938 .31726 L
.26968 .3141 L
.27093 .30089 L
.27347 .27437 L
.27816 .2269 L
.28054 .20342 L
.28314 .17833 L
.28434 .16685 L
.285 .16061 L
.28533 .15756 L
.28547 .15622 L
.28562 .15476 L
.28577 .15511 L
.28591 .15797 L
.28621 .16421 L
.28684 .17733 L
.28793 .20016 L
.29018 .24693 L
.29137 .27149 L
.29202 .28492 L
.29234 .29149 L
.29263 .29749 L
.29279 .30072 L
Mistroke
.29294 .3014 L
.29309 .29995 L
.29327 .29837 L
.29397 .29194 L
.2952 .2807 L
.29763 .25888 L
.30253 .21611 L
.30479 .19688 L
.30724 .17644 L
.3079 .17105 L
.30859 .16539 L
.30917 .16057 L
.30934 .15926 L
.30943 .15854 L
.30951 .15785 L
.30959 .15907 L
.30967 .16085 L
.30982 .16396 L
.31225 .21458 L
.31459 .26302 L
.31583 .2887 L
.31617 .29572 L
.31636 .29956 L
.31653 .30312 L
.31668 .30588 L
.31683 .30461 L
.31717 .30193 L
.31961 .28254 L
.3219 .26462 L
.3269 .22637 L
.32941 .20761 L
.33051 .19945 L
.33172 .19052 L
.33228 .18641 L
.33259 .18415 L
.33289 .182 L
.33316 .18001 L
.33331 .17889 L
.33345 .18126 L
.33397 .19207 L
.33636 .24164 L
.33863 .2887 L
.33925 .30158 L
.33991 .31518 L
.34018 .32078 L
.34033 .32386 L
.34047 .32678 L
.34055 .32645 L
.34063 .32582 L
.34079 .32473 L
Mistroke
.34108 .32258 L
.34367 .304 L
.3461 .28667 L
.35102 .25198 L
.35347 .23489 L
.35466 .22655 L
.35576 .21892 L
.35631 .21507 L
.35683 .21149 L
.35696 .21057 L
.35709 .20969 L
.35723 .21123 L
.35737 .21397 L
.35795 .22617 L
.36032 .27521 L
.36253 .32109 L
.36313 .33366 L
.36378 .34716 L
.36394 .35028 L
.3641 .35369 L
.36426 .35691 L
.3644 .35676 L
.36454 .35578 L
.36467 .35488 L
.36496 .35288 L
.36736 .33626 L
.3699 .31867 L
.37474 .28504 L
.37696 .26953 L
.37822 .26072 L
.3794 .25248 L
.38 .24828 L
.38056 .2443 L
.38072 .24323 L
.3808 .24264 L
.38088 .2421 L
.38096 .24173 L
.38105 .24365 L
.38122 .24703 L
.38181 .25934 L
.38435 .31207 L
.38659 .35848 L
.3872 .3712 L
.38754 .37818 L
.38785 .38477 L
.388 .38776 L
.38815 .3894 L
.38823 .38884 L
.38832 .38822 L
.38847 .38714 L
Mistroke
.38903 .38318 L
.39158 .36506 L
.394 .34771 L
.39887 .31236 L
.40112 .29586 L
.40231 .2871 L
.40357 .27774 L
.4042 .27303 L
.40437 .27173 L
.40456 .27034 L
.40464 .2697 L
.40473 .26902 L
.40481 .26986 L
.4049 .27161 L
.40613 .29709 L
.40855 .3474 L
.40977 .37266 L
.41092 .39653 L
.41155 .40974 L
.41173 .41336 L
.41182 .41534 L
.41191 .41698 L
.41207 .4158 L
.41224 .41449 L
.41345 .40539 L
.41813 .36953 L
.4231 .33057 L
.42559 .31077 L
.4268 .30106 L
.4279 .29222 L
.42817 .29004 L
.42832 .28885 L
.42845 .28774 L
.4286 .28738 L
.42868 .28901 L
.42876 .29075 L
.42905 .29667 L
.43013 .31909 L
.43251 .3684 L
.43374 .39394 L
.43439 .40742 L
.43508 .42173 L
.43537 .42774 L
.43552 .4309 L
.43568 .43422 L
.43576 .43459 L
.43586 .43383 L
.43602 .4325 L
.43634 .42987 L
.43751 .42029 L
Mistroke
.4424 .37952 L
.44712 .33946 L
.44952 .31871 L
.45079 .30771 L
.45148 .30168 L
.45182 .29872 L
.45197 .29743 L
.45213 .29602 L
.45228 .29467 L
.45242 .2947 L
.45256 .29752 L
.45271 .30052 L
.45333 .31335 L
.45464 .3406 L
.45696 .38862 L
.45808 .41205 L
.45868 .42437 L
.459 .4311 L
.4593 .43736 L
.45944 .4402 L
.45957 .44165 L
.45971 .44034 L
.45985 .43914 L
.46045 .43384 L
.4615 .42449 L
.46614 .38297 L
.47107 .3381 L
.47352 .31552 L
.47472 .30442 L
.47529 .29914 L
.47582 .29424 L
.47595 .29299 L
.47608 .29181 L
.47622 .29145 L
.47636 .29419 L
.47694 .30637 L
.47802 .32858 L
.48038 .37764 L
.48156 .40211 L
.4822 .41548 L
.4828 .42797 L
.48307 .43345 L
.4832 .43614 L
.48332 .43863 L
.48346 .43783 L
.4836 .43642 L
.48387 .43397 L
.48503 .42304 L
.48977 .37831 L
.49432 .33488 L
Mistroke
.49686 .31059 L
.49796 .29993 L
.49855 .29428 L
.49918 .28825 L
.49949 .28518 L
.49965 .28367 L
.49979 .28228 L
.49992 .28102 L
.50006 .28157 L
.5002 .28445 L
.50035 .28761 L
.50145 .31033 L
.50384 .35998 L
.50505 .38497 L
.50619 .40869 L
.50682 .42178 L
.50699 .42537 L
.50709 .42733 L
.50718 .42816 L
.50726 .42735 L
.50735 .42648 L
.50751 .42496 L
.50871 .4133 L
.51366 .36512 L
.51843 .31854 L
.52084 .29505 L
.52188 .28479 L
.52302 .27365 L
.52333 .27065 L
.52349 .26901 L
.52365 .26748 L
.52378 .26617 L
.52393 .2684 L
.52423 .27473 L
.52538 .29852 L
.5279 .3509 L
.52912 .37616 L
.52977 .38951 L
.53012 .39684 L
.53045 .40375 L
.53077 .41031 L
.53085 .41211 L
.53095 .41401 L
.53102 .41343 L
.53111 .41258 L
.53171 .40673 L
.53287 .39531 L
.53775 .3476 L
.54244 .30177 L
.54483 .27849 L
Mistroke
.54609 .26622 L
.54678 .2595 L
.54711 .25621 L
.54726 .25477 L
.54742 .25319 L
.54757 .25179 L
.5477 .25302 L
.54784 .25596 L
.54799 .2591 L
.54861 .27187 L
.54992 .29904 L
.55223 .34695 L
.55346 .37255 L
.55411 .38603 L
.55447 .39341 L
.55456 .39525 L
.55464 .39701 L
.5548 .39913 L
.55494 .39778 L
.55509 .39632 L
.5554 .39329 L
.55606 .38687 L
.55723 .37557 L
.56214 .3282 L
.56686 .28287 L
.56927 .25983 L
.56993 .25354 L
.57063 .2469 L
.57092 .24416 L
.57123 .24123 L
.57132 .24037 L
.5714 .23955 L
.57148 .24038 L
.57156 .24212 L
.57188 .24867 L
.57418 .29643 L
.57549 .32359 L
.57672 .34905 L
.57787 .37288 L
.57818 .37941 L
.57835 .38297 L
.57844 .3847 L
.57851 .38628 L
.57865 .38679 L
.57879 .3854 L
.5791 .38248 L
.58164 .35852 L
.58646 .31319 L
.59111 .26994 L
.59351 .24772 L
Mistroke
.59409 .24233 L
.59472 .23656 L
.5949 .23486 L
.59508 .23325 L
.59523 .2318 L
.59532 .23352 L
.5954 .23523 L
.59605 .24857 L
.59849 .29919 L
.59968 .32398 L
.60078 .34673 L
.60184 .36879 L
.6021 .37415 L
.60224 .3771 L
.60238 .37989 L
.60254 .37855 L
.60268 .37723 L
.60296 .37463 L
.60532 .35303 L
.60996 .3109 L
.6123 .2897 L
.61489 .26646 L
.61607 .25587 L
.61734 .24451 L
.61795 .23902 L
.61854 .23381 L
.61879 .23154 L
.61893 .2303 L
.61906 .22949 L
.61922 .23272 L
.61936 .23565 L
.61963 .24138 L
.62428 .33788 L
.62488 .35033 L
.62553 .36367 L
.62584 .37013 L
.62599 .37331 L
.62613 .37626 L
.62626 .37682 L
.6264 .37561 L
.62669 .37304 L
.6279 .3623 L
.62923 .35053 L
.63378 .3104 L
.63631 .28823 L
.63863 .26797 L
.6409 .24818 L
.64205 .23825 L
.64236 .23553 L
.64253 .23404 L
Mistroke
.64269 .23263 L
.64284 .23131 L
.64293 .23272 L
.64301 .23435 L
.6433 .24031 L
.64575 .29128 L
.64805 .33894 L
.64859 .35016 L
.64918 .36238 L
.64945 .36797 L
.64973 .37392 L
.64987 .37682 L
.65001 .37937 L
.65012 .37835 L
.65025 .37725 L
.65149 .36655 L
.65262 .3568 L
.65757 .31418 L
.66234 .27332 L
.66357 .26278 L
.66474 .25281 L
.66579 .24387 L
.66607 .24142 L
.66638 .23883 L
.6665 .23776 L
.66664 .23661 L
.66679 .23884 L
.66692 .24172 L
.66928 .29066 L
.67181 .34303 L
.67241 .35563 L
.67306 .36907 L
.67338 .3756 L
.67353 .37881 L
.67367 .38176 L
.6738 .38445 L
.67394 .38347 L
.67423 .38102 L
.67545 .37066 L
.67677 .35935 L
.68135 .32047 L
.6839 .29885 L
.68622 .27913 L
.68737 .26936 L
.68863 .25872 L
.6892 .25385 L
.68981 .24868 L
.69007 .24655 L
.6902 .24543 L
.69034 .24426 L
Mistroke
.69049 .24336 L
.69063 .2463 L
.69091 .25209 L
.6955 .34738 L
.69609 .35948 L
.69672 .37259 L
.69701 .37861 L
.69732 .38498 L
.69744 .3876 L
.69758 .39042 L
.69773 .39037 L
.69787 .38918 L
.69919 .37796 L
.70039 .36776 L
.70537 .32554 L
.70785 .30443 L
.71016 .28482 L
.71239 .26583 L
.71303 .26036 L
.71364 .2552 L
.7139 .25295 L
.71405 .25172 L
.71418 .25057 L
.71433 .25051 L
.7144 .25211 L
.71449 .25383 L
.71477 .25966 L
.71716 .30927 L
.71842 .33543 L
.71976 .36322 L
.72033 .37508 L
.72095 .38787 L
.72112 .39144 L
.7213 .39519 L
.72145 .39768 L
.72154 .39693 L
.72162 .39624 L
.72226 .39082 L
.72457 .37109 L
.72928 .33076 L
.73168 .31014 L
.73429 .28775 L
.73551 .27725 L
.7368 .26613 L
.73733 .26154 L
.73762 .25902 L
.7379 .25662 L
.73806 .25527 L
.73814 .25593 L
.73823 .25764 L
Mistroke
.73852 .26383 L
.73911 .27598 L
.74138 .32314 L
.74384 .37408 L
.7445 .38772 L
.74485 .39517 L
.74503 .3988 L
.74519 .4021 L
.74534 .4023 L
.74547 .4011 L
.74578 .39845 L
.74643 .39285 L
.74886 .37176 L
.75349 .33153 L
.75584 .31111 L
.75842 .28855 L
.7596 .27823 L
.76025 .27254 L
.76087 .26712 L
.76144 .2621 L
.7616 .26072 L
.76169 .25997 L
.76177 .25926 L
.76193 .25851 L
.76207 .26148 L
.76317 .28423 L
.76538 .33019 L
.76664 .35629 L
.76781 .38068 L
.76841 .39304 L
.7687 .39913 L
.76883 .4018 L
.76897 .40476 L
.76913 .40559 L
.76921 .40486 L
.76929 .40418 L
.76963 .4012 L
.77022 .39599 L
.77276 .37364 L
.77742 .33247 L
.77979 .3115 L
.78238 .28851 L
.78358 .27789 L
.78424 .27204 L
.78486 .26652 L
.78514 .26402 L
.78544 .26137 L
.78559 .25999 L
.78568 .2592 L
.78576 .25993 L
Mistroke
.78591 .26294 L
.78606 .26618 L
.78716 .28892 L
.78954 .33831 L
.7908 .3644 L
.79149 .37872 L
.79214 .39219 L
.79244 .39853 L
.79261 .40199 L
.79277 .40522 L
.79285 .40691 L
.79293 .40643 L
.79303 .40557 L
.79312 .40478 L
.79344 .40189 L
.79463 .39134 L
.79693 .37074 L
.80174 .32767 L
.80637 .28612 L
.8076 .27504 L
.80876 .26463 L
.80904 .26209 L
.80934 .25941 L
.8095 .258 L
.80958 .25901 L
.80967 .26071 L
.80997 .26691 L
.81129 .29448 L
.81356 .34144 L
.81475 .36616 L
.81601 .39231 L
.81632 .39871 L
.81649 .40224 L
.81657 .40397 L
.81665 .40558 L
.81674 .40531 L
.81684 .40442 L
.81701 .40285 L
.81735 .39979 L
.81859 .38864 L
.82102 .3667 L
.82575 .32393 L
.8303 .28276 L
.83143 .27248 L
.83263 .26165 L
.83293 .2589 L
.8331 .2574 L
.83325 .25597 L
.83334 .25541 L
.83344 .25741 L
Mistroke
.83361 .26092 L
.83394 .26777 L
.83514 .29273 L
.83749 .34142 L
.83873 .36725 L
.83942 .38149 L
.83976 .38851 L
.84007 .39503 L
.84022 .3982 L
.84037 .40118 L
.84045 .40283 L
.84053 .40294 L
.84068 .40157 L
.84135 .39551 L
.84252 .38489 L
.84482 .36405 L
.84976 .31914 L
.85223 .29677 L
.85453 .27588 L
.85566 .26557 L
.85622 .26049 L
.85674 .2558 L
.85689 .25445 L
.85703 .25318 L
.85717 .25274 L
.85725 .25434 L
.85733 .25607 L
.85798 .26947 L
.85911 .29292 L
.86148 .3421 L
.86273 .36811 L
.86342 .3824 L
.86376 .38942 L
.86391 .3925 L
.86407 .39589 L
.86423 .39913 L
.86437 .39954 L
.86453 .39814 L
.86469 .39663 L
.86537 .3905 L
.86655 .3798 L
.86885 .35889 L
.87383 .31371 L
.87632 .29113 L
.87863 .27022 L
.87978 .25978 L
.88034 .25467 L
.88059 .25243 L
.88072 .25124 L
.88086 .24997 L
Mistroke
.88101 .25042 L
.88115 .25335 L
.88146 .25974 L
.88211 .2732 L
.88324 .29665 L
.88564 .34634 L
.8869 .37253 L
.88723 .37943 L
.88759 .38687 L
.88776 .39046 L
.88793 .39388 L
.88808 .39695 L
.88816 .39679 L
.88824 .39608 L
.88943 .38531 L
.89074 .37344 L
.89305 .35254 L
.89777 .30994 L
.90017 .28822 L
.90144 .2768 L
.90278 .26473 L
.90398 .25394 L
.90431 .25098 L
.90449 .24935 L
.90458 .24855 L
.90466 .24781 L
.90482 .24801 L
.9049 .24975 L
.90499 .25161 L
.90529 .2579 L
.90761 .30594 L
.90998 .35518 L
.91102 .37674 L
.91131 .38265 L
.91161 .38897 L
.91176 .39203 L
.9119 .39493 L
.91202 .39402 L
.91216 .3928 L
.91329 .38258 L
.91449 .37183 L
.917 .34923 L
.92194 .30498 L
.92439 .283 L
.92559 .27226 L
.92669 .26244 L
.92723 .25761 L
.92782 .25234 L
.92808 .24994 L
.92823 .24861 L
Mistroke
.92837 .24737 L
.9285 .24621 L
.92864 .24704 L
.92877 .2497 L
.92889 .25213 L
.93125 .30123 L
.93372 .35239 L
.93434 .36522 L
.93492 .37736 L
.93545 .38826 L
.93559 .39125 L
.93574 .39352 L
.93588 .39236 L
.93602 .39107 L
.93823 .37131 L
.9406 .35016 L
.94545 .30692 L
.95012 .26539 L
.95073 .26 L
.95138 .25425 L
.95169 .25145 L
.95199 .24882 L
.95212 .24767 L
.95226 .24645 L
.95241 .24584 L
.95254 .24871 L
.95376 .27395 L
.95509 .30154 L
.95748 .35114 L
.95852 .37281 L
.95908 .38425 L
.95923 .38754 L
.95938 .39064 L
.95952 .39342 L
.95966 .39232 L
.96029 .38677 L
.96087 .38163 L
.96201 .37149 L
.96453 .34914 L
.96949 .30526 L
.97426 .26305 L
.97619 .24597 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  Background->GrayLevel[1],
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005X0000gR000`400?l00000o`00003oo`3ooomY
0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`006P3oool0
0`000000oooo0?ooo`3o0?oood`0oooo000J0?ooo`030000003oool0oooo0?l0ooooC03oool001X0
oooo00<000000?ooo`3oool0o`3ooom<0?ooo`00100000050?ooo`8000001`3oool3000000D0oooo
00<000000?ooo`3oool0o`3ooom<0?ooo`00103oool00`000000oooo0?ooo`020?ooo`800000203o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?l0ooooC03oool000@0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`040?ooo`030?l0o`3oool0oooo0?l0ooooC03oool0
00@0oooo00<000000?ooo`3oool0203oool6000000D0oooo00<0o`3o000000000000o`3ooom<0?oo
o`00103oool00`000000oooo0?ooo`090?ooo`040000003oool0oooo000000H0oooo00<0o`3o0?oo
o`3oool0o`3ooom<0?ooo`00103oool00`000000oooo0?ooo`090?ooo`040000003oool0oooo0000
00H0oooo00<0o`3o0?ooo`3oool0o`3ooom<0?ooo`00103oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0000000H0oooo00<0o`3o0?ooo`3oool0o`3ooom<0?ooo`001000000>0?ooo`800000
1P3oool00`3o0?l0oooo0?ooo`3o0?oood`0oooo000J0?ooo`030?l0o`3oool0oooo0?l0ooooC03o
ool001X0oooo00<0o`3o0?ooo`3oool0o`3ooom<0?ooo`006P3oool00`3o0?l0oooo0?ooo`3o0?oo
od`0oooo000J0?ooo`030000003o0?l0oooo0?l0ooooC03oool001X0oooo00<000000?l0o`3oool0
o`3ooom<0?ooo`006P3oool00`000000o`3o0?ooo`3o0?oood`0oooo000J0?ooo`030000003o0?l0
oooo0?l0ooooC03oool001X0oooo00<000000?l0o`3oool0o`3ooom<0?ooo`006P3oool00`000000
o`3o0?ooo`3o0?oood`0oooo000J0?ooo`030000003o0?l0oooo0?l0ooooC03oool001X0oooo00<0
00000?l0o`3oool0o`3ooom<0?ooo`006P3oool00`000000o`3o0?ooo`3o0?oood`0oooo000J0?oo
o`030000003o0?l0oooo0?l0ooooC03oool001X0oooo00<000000?l0o`3oool0o`3ooom<0?ooo`00
6P3oool00`000000o`3o0?ooo`3o0?oood`0oooo000J0?ooo`030000003o0?l0oooo0?l0ooooC03o
ool001X0oooo00<000000?l0o`3oool0o`3ooom<0?ooo`006P3oool00`000000o`3o0?ooo`3o0?oo
od`0oooo000J0?ooo`030000003o0?l0oooo0?l0ooooC03oool001X0oooo00<000000?ooo`3o0?l0
o`3ooom<0?ooo`006P3oool2000000030?l0o`3oool0oooo0?l0ooooBP3oool001X0oooo00<00000
0?ooo`3o0?l0o`3ooom<0?ooo`006P3oool00`000000oooo0?l0o`3o0?oood`0oooo000J0?ooo`03
0000003oool0o`3o0?l0ooooC03oool001X0oooo00<000000?ooo`3o0?l0o`3ooom<0?ooo`001000
00050?ooo`8000001P3oool3000000H0oooo00<000000?ooo`3o0?l0o`3ooom<0?ooo`00103oool0
0`000000oooo0?ooo`020?ooo`8000001@3oool01@000000oooo0?ooo`3oool0000000D0oooo00<0
00000?ooo`3o0?l0o`3ooom<0?ooo`00103oool00`000000oooo0?ooo`090?ooo`050000003oool0
oooo0?ooo`0000001@3oool00`000000oooo0?l0o`3o0?oood`0oooo00040?ooo`030000003oool0
oooo00T0oooo00D000000?ooo`3oool0oooo000000050?ooo`80000000<0o`3o0?ooo`3oool0o`3o
oom:0?ooo`00103oool00`000000oooo0?ooo`090?ooo`@000001P3oool00`000000oooo0?l0o`3o
0?oood`0oooo00040?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?l0o`3o0?oood`0oooo00040?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?l0o`3o0?oood`0oooo0004000000h0oooo0`0000050?ooo`030000
003oool0o`3o0?l0ooooC03oool001X0oooo00<000000?ooo`3o0?l0o`3ooom<0?ooo`006P3oool0
0`000000oooo0?l0o`3o0?oood`0oooo000J0?ooo`030000003oool0o`3o0?l0ooooC03oool001X0
oooo00<000000?ooo`3o0?l0o`3ooom<0?ooo`006P3oool2000000030?l0o`3oool0oooo0?l0oooo
BP3oool001X0oooo00<000000?ooo`3o0?l0o`3ooom<0?ooo`006P3oool00`000000oooo0?l0o`3o
0?oood`0oooo000J0?ooo`030000003oool0o`3o0?l0ooooC03oool001X0oooo00@000000?ooo`3o
ool0o`3oE03oool20?l0oo@0oooo000J0?ooo`040000003oool0oooo0?l0oe@0oooo0P3o0?l70?oo
o`030?l0o`3oool0oooo0>X0oooo000J0?ooo`040000003oool0oooo0?l0oe@0oooo0P3o0?l60?oo
o`80o`3ok03oool001X0oooo00<000000?ooo`3oool00P3o0?mC0?ooo`80o`3o1P3oool20?l0on`0
oooo000J0?ooo`8000000`3oool20?l0oe40oooo0P3o0?l60?ooo`80o`3ok03oool001X0oooo00<0
00000?ooo`3oool0103oool20?l0odl0oooo0P3o0?l60?ooo`80o`3ok03oool001X0oooo00<00000
0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`150?ooo`030?l0o`3oool0oooo00D0oooo0P3o0?l6
0?ooo`80o`3ok03oool001X0oooo00<000000?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`150?oo
o`80o`3o1P3oool20?l0o`H0oooo0P3o0?o/0?ooo`006P3oool00`000000oooo0?ooo`050?ooo`03
0?l0o`3oool0oooo04D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`L0oooo00<0o`3o0?ooo`3o
ool0hP3oool001X0oooo00<000000?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`150?ooo`80o`3o
1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3oi03oool001X0oooo00<000000?ooo`3oool01@3o
ool00`3o0?l0oooo0?ooo`150?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3oi03o
ool001X0oooo00<000000?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`150?ooo`80o`3o1P3oool2
0?l0o`H0oooo0P3o0?l60?ooo`80o`3oi03oool001X0oooo0P0000060?ooo`030?l0o`3oool0oooo
04D0oooo0P3o0?l60?ooo`80o`3o1@3oool00`3o0?l0oooo0?l0o`060?ooo`80o`3oi03oool001X0
oooo00<000000?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`150?ooo`80o`3o1@3oool00`3o0?l0
oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00H0oooo0P3o0?oT0?ooo`006P3oool00`000000oooo
0?ooo`050?ooo`030?l0o`3oool0oooo04D0oooo0P3o0?l50?ooo`030?l0o`3oool0o`3o00D0oooo
00<0o`3o0?ooo`3o0?l01P3oool20?l0on@0oooo000J0?ooo`030000003oool0oooo00D0oooo00<0
o`3o0?ooo`3oool0A@3oool20?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0
o`060?ooo`80o`3oi03oool001X0oooo00<000000?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`15
0?ooo`80o`3o1@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00H0oooo0P3o0?oT
0?ooo`006P3oool00`000000oooo0?ooo`060?ooo`030?l0o`3oool0oooo04@0oooo0P3o0?l50?oo
o`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`3T0?oo
o`00100000050?ooo`8000001@3oool4000000H0oooo00<000000?ooo`3oool01P3oool00`3o0?l0
oooo0?ooo`140?ooo`80o`3o1@3oool00`3o0?l0oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0
o`D0oooo00<0o`3o0?ooo`3o0?l0i03oool000@0oooo00<000000?ooo`3oool00P3oool2000000@0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00H0
oooo00<0o`3o0?ooo`3oool0A03oool20?l0o`D0oooo00<0o`3o0?ooo`3o0?l0103oool0103o0?l0
oooo0?ooo`3o0?l50?ooo`030?l0o`3oool0o`3o00H0oooo0P3o0?oL0?ooo`00103oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`060?ooo`030?l0o`3oool0oooo04@0oooo0P3o0?l50?ooo`030?l0o`3oool0o`3o00@0
oooo00@0o`3o0?ooo`3oool0o`3o1@3oool00`3o0?l0oooo0?l0o`060?ooo`80o`3og03oool000@0
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0
oooo0`0000060?ooo`030?l0o`3oool0oooo03`0oooo00<0o`3o0?ooo`3oool0103oool00`3o0?l0
oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0
103o0?l0oooo0?ooo`3o0?l60?ooo`80o`3og03oool000@0oooo00<000000?ooo`3oool02@3oool4
000000H0oooo00<000000?ooo`3oool01P3oool00`3o0?l0oooo0?ooo`0l0?ooo`80o`3o1@3oool0
0`3o0?l0oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o
103oool01@3o0?l0oooo0?ooo`3oool0o`3o00D0oooo0P3o0?oL0?ooo`00103oool00`000000oooo
0?ooo`080?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`060?ooo`030?l0o`3oool0oooo03`0oooo0P3o0?l50?ooo`030?l0o`3oool0o`3o00@0oooo
00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`050?l0o`3oool0oooo
0?ooo`3o0?l01@3oool20?l0om`0oooo00040?ooo`030000003oool0oooo00P0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00H0oooo00<0o`3o0?oo
o`3oool0?03oool20?l0o`D0oooo00<0o`3o0?ooo`3o0?l0103oool01@3o0?l0oooo0?ooo`3oool0
o`3o00<0oooo00@0o`3o0?ooo`3oool0o`3o103oool01@3o0?l0oooo0?ooo`3oool0o`3o00D0oooo
0P3o0?oL0?ooo`001000000<0?ooo`@000001P3oool00`000000oooo0?ooo`060?ooo`030?l0o`3o
ool0oooo03`0oooo0P3o0?l50?ooo`030?l0o`3oool0o`3o00@0oooo00D0o`3o0?ooo`3oool0oooo
0?l0o`030?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`050?oo
o`80o`3og03oool001X0oooo00<000000?ooo`3oool01P3oool00`3o0?l0oooo0?ooo`0l0?ooo`80
o`3o1@3oool00`3o0?l0oooo0?l0o`040?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool0103o
0?l0oooo0?ooo`3o0?l40?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0103oool00`3o0?l0oooo0?l0
o`1?0?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?oo
o`1j0?ooo`006P3oool00`000000oooo0?ooo`060?ooo`030?l0o`3oool0oooo03`0oooo0P3o0?l5
0?ooo`030?l0o`3oool0o`3o00@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`040?l0o`3o
ool0oooo0?l0o`@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`040?ooo`030?l0o`3oool0o`3o04l0
oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo07X0
oooo000J0?ooo`030000003oool0oooo00H0oooo00<0o`3o0?ooo`3oool0?03oool20?l0o`D0oooo
00<0o`3o0?ooo`3o0?l0103oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00D0o`3o0?ooo`3o
ool0oooo0?l0o`040?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0103oool00`3o0?l0oooo0?l0o`1?
0?ooo`030?l0o`3oool0oooo00@0oooo0P3o0?l70?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o
0?ooo`3oool0LP3oool001X0oooo0P0000070?ooo`030?l0o`3oool0oooo03`0oooo0P3o0?l50?oo
o`040?l0o`3oool0oooo0?l0o`80oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o
0?l30?ooo`030?l0o`3oool0oooo0080oooo00D0o`3o0?ooo`3oool0oooo0?l0o`040?ooo`030?l0
o`3oool0o`3o04L0oooo00<0o`3o0?ooo`3oool0103oool20?l0o`H0oooo0P3o0?l70?ooo`030?l0
o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool0LP3oool001X0oooo00<000000?ooo`3oool01P3o
ool00`3o0?l0oooo0?ooo`0l0?ooo`80o`3o103oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo
00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`050?l0o`3oool0oooo0?oo
o`3o0?l00`3oool00`3o0?l0oooo0?ooo`020?ooo`030?l0o`3oool0o`3o00L0oooo00<0o`3o0?oo
o`3oool0?@3oool00`3o0?l0oooo0?ooo`040?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l70?oo
o`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool0JP3oool001X0oooo00<000000?ooo`3o
ool01P3oool00`3o0?l0oooo0?ooo`0k0?ooo`030?l0o`3oool0o`3o00@0oooo00D0o`3o0?ooo`3o
ool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo0080oooo00@0o`3o0?ooo`3oool0o`3o103oool0
103o0?l0oooo0?ooo`3o0?l40?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3o0?l01P3o
ool20?l0ocl0oooo00<0o`3o0?ooo`3oool0103oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1`3o
ool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo04d0oooo00<0o`3o0?ooo`3oool01@3o
ool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00X0oooo000J0?ooo`030000003oool0
oooo00H0oooo00<0o`3o0?ooo`3oool0>`3oool00`3o0?l0oooo0?l0o`040?ooo`050?l0o`3oool0
oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`020?ooo`040?l0o`3oool0oooo0?l0o`@0oooo
00@0o`3o0?ooo`3oool0o`3o103oool00`3o0?l0oooo0?ooo`020?ooo`030?l0o`3oool0o`3o00H0
oooo0P3o0?lo0?ooo`030?l0o`3oool0oooo00@0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0
oooo0P3o0?l70?ooo`030?l0o`3oool0oooo03d0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0
oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0
oooo0?ooo`0:0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030?l0o`3oool0oooo03X0oooo
00<0o`3o0?ooo`3o0?l0103oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3o
ool00P3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00<0o`3o
0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o0?l50?ooo`80o`3o=`3oool00`3o0?l0oooo0?oo
o`040?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`L0oooo00<0
o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`0]0?ooo`030?l0o`3oool0oooo00D0oooo00<0
o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0oooo00<0
o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`0:0?ooo`006P3oool00`000000oooo0?ooo`07
0?ooo`030?l0o`3oool0oooo03X0oooo00<0o`3o0?ooo`3o0?l0103oool01@3o0?l0oooo0?ooo`3o
ool0o`3o0080oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0
o`3oool0oooo0?l0o`@0oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o0?l50?oo
o`80o`3o=`3oool00`3o0?l0oooo0?ooo`040?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?oo
o`80o`3o1P3oool20?l0o`L0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?oo
o`030?l0o`3oool0oooo01d0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?oo
o`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?oo
o`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool02P3oool001X0oooo00<000000?ooo`3o
ool01`3oool00`3o0?l0oooo0?ooo`0j0?ooo`030?l0o`3oool0o`3o00@0oooo00D0o`3o0?ooo`3o
ool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo0080oooo00@0o`3o0?ooo`3oool0o`3o103oool0
103o0?l0oooo0?ooo`3o0?l40?ooo`030?l0o`3oool0oooo0080oooo00@0o`3o0?ooo`3oool0o`3o
1@3oool20?l0ocL0oooo00<0o`3o0?ooo`3oool0103oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o
1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1`3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0
o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool03@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0
o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0
o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0
o`3oool0oooo00X0oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`3o0?ooo`3oool0>P3o
ool00`3o0?l0oooo0?l0o`040?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo
0?ooo`020?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool01@3o
0?l0oooo0?ooo`3oool0o`3o00<0oooo00<0o`3o0?ooo`3oool00`3oool20?l0ocL0oooo00<0o`3o
0?ooo`3oool0103oool20?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`05
0?ooo`030?l0o`3oool0o`3o00H0oooo0P3o0?l60?ooo`80o`3o1`3oool00`3o0?l0oooo0?ooo`05
0?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`05
0?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`05
0?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`05
0?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool02P3oool001X0oooo0P0000080?oo
o`030?l0o`3oool0oooo03X0oooo00<0o`3o0?ooo`3o0?l0103oool01`3o0?l0oooo0?ooo`3oool0
o`3o0?ooo`3o0?l01@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0
oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`030?l0o`3oool0oooo00<0oooo0P3o0?lf0?oo
o`80o`3o1P3oool20?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?oo
o`030?l0o`3oool0o`3o00H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`L0oooo00<0o`3o0?oo
o`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?oo
o`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?oo
o`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00@0oooo00<0o`3o0?oo
o`3o0?l01P3oool00`3o0?l0oooo0?ooo`0:0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`03
0?l0o`3oool0oooo03X0oooo00<0o`3o0?ooo`3o0?l0103oool01`3o0?l0oooo0?ooo`3oool0o`3o
0?ooo`3o0?l01@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo
00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3o
0?l0;`3oool00`3o0?l0oooo0?ooo`040?ooo`80o`3o1P3oool20?l0o`D0oooo00<0o`3o0?ooo`3o
0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`040?l0o`3oool0oooo0?l0o`D0oooo0P3o0?l60?oo
o`80o`3o1P3oool20?l0o`L0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?oo
o`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool0103oool20?l0o`L0oooo00<0o`3o0?oo
o`3oool0103oool20?l0o`L0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`040?oo
o`030?l0o`3oool0o`3o00H0oooo00<0o`3o0?ooo`3oool02P3oool001X0oooo00<000000?ooo`3o
ool01`3oool00`3o0?l0oooo0?ooo`0i0?ooo`040000003o0?l000000?l0o`<0oooo00T0o`3o0?oo
o`00000000000?ooo`3o0?l0oooo0?l0o`3oool0100000000`3o0?l0oooo0?l0o`050?ooo`040?l0
o`3oool0oooo0?l0o`@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`030?l0o`3oool0oooo
0080oooo00<0o`3o0?ooo`3o0?l01`3oool00`3o0?l0oooo0?ooo`0C0?ooo`@000001@3oool20000
00@0oooo0`0000000`3o0?l000000?ooo`040?ooo`80o`3o1P3oool20?l0o`D0oooo00<0o`3o0?oo
o`3o0?l01@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0
o`3o0?ooo`3oool0o`3o1@3oool20?l0o`D0oooo00@000000?l0o`3o0?l000001@3oool00`000000
o`3o0?ooo`040?ooo`030000003o0?l0o`3o00L0oooo00<0o`3o0?ooo`3oool0103oool20?l0o`H0
oooo0P3o0?l70?ooo`030?l0o`3oool0oooo00@0oooo00<0o`3o0?ooo`3o0?l01@3oool20?l0o`H0
oooo0P3o0?l50?ooo`040000003o0?l000000?l0o`D0oooo00<000000?l0o`3oool01P3oool30000
0080oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`3o0?ooo`3oool0>03oool01P000000
oooo0?l0o`3oool0o`3o00000080oooo00P0o`3o0?ooo`00000000000?ooo`3o0?l0oooo0?l0o`<0
oooo00D000000?ooo`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00D0o`3o0?oo
o`3oool0oooo0?l0o`030?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3o0?l01P3oool2
0?l0oa@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`800000103oool0
1@000000oooo0?l0o`3o0?l0000000D0oooo0P3o0?l50?ooo`030?l0o`3oool0o`3o00D0oooo00<0
o`3o0?ooo`3o0?l01@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0
oooo00@0o`3o0?ooo`3oool0o`3o1@3oool20?l0o`@0oooo00H000000?ooo`3o0?l0o`3o0?ooo`00
00040?ooo`80o`3o103oool01@000000oooo0?l0o`3o0?l0000000H0oooo00<0o`3o0?ooo`3oool0
103oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`050?ooo`80o`3o
1P3oool20?l0o`@0oooo00H000000?ooo`3o0?l0oooo0?l0o`0000040?ooo`030000003o0?l0oooo
00L0oooo00<000000?ooo`3oool00@3oool001X0oooo00<000000?ooo`3oool01`3oool00`3o0?l0
oooo0?ooo`0h0?ooo`060000003oool0o`3o0?ooo`3o0?l000000P3oool00`3o0?l0oooo0?ooo`02
0?ooo`030?l0o`3oool0o`3o00<0oooo00D000000?ooo`3o0?l0oooo0?l0o`050?ooo`030?l0o`3o
ool0o`3o00D0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`030?l0o`3oool0oooo0080oooo
00<0o`3o0?ooo`3o0?l01P3oool20?l0oa@0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`090?ooo`030000003o0?l0o`3o00H0oooo0P3o0?l50?ooo`040?l0o`3oool0oooo0?l0o`@0
oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0
oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o1@3oool20?l0o`@0oooo00H000000?ooo`3o0?l0
o`3o0?ooo`0000040?ooo`80o`3o1P3oool20?l0o`030000003oool0oooo00<0oooo0P3o0?l60?oo
o`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`030?l0o`3oool0o`3o00D0oooo0P3o0?l60?oo
o`80o`3o103oool01P000000oooo0?l0o`3oool0o`3o000000@0oooo00<0o`3o0?ooo`3oool01`3o
ool00`000000oooo0?ooo`010?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030?l0o`3oool0
oooo03<0oooo00<0o`3o0?ooo`3oool00P3oool00`000000oooo0?l0o`020?ooo`040?l0o`3oool0
oooo0?l0o`@0oooo00<0o`3o0?ooo`3o0?l00`3oool01@000000oooo0?l0o`3oool0o`3o00H0oooo
0P3o0?l50?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0
oooo0?l0o`060?ooo`80o`3o503oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00X0
oooo0P3o0?l60?ooo`80o`3o1@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo
0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0
o`3oool0oooo0?l0o`D0oooo00<0o`3o0?ooo`3o0?l00`3oool01P000000oooo0?l0o`3o0?l0oooo
000000@0oooo0P3o0?l60?ooo`80o`3o00<000000?ooo`3oool00`3oool20?l0o`H0oooo0P3o0?l6
0?ooo`80o`3o1P3oool20?l0o`H0oooo00<0o`3o0?ooo`3o0?l01@3oool20?l0o`H0oooo00<0o`3o
0?ooo`3o0?l00`3oool01P000000oooo0?l0o`3oool0o`3o000000@0oooo00<0o`3o0?ooo`3oool0
0`3oool600000080oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`3o0?ooo`3oool0<`3o
ool20?l0o`<0oooo00<000000?l0o`3oool00P3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`030?l0
o`3oool0o`3o00<0oooo00<000000?ooo`3oool00P3o0?l60?ooo`80o`3o1@3oool0103o0?l0oooo
0?ooo`3o0?l40?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3o0?l01P3oool20?l0oa@0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030?l0o`3oool0oooo00@0
oooo0P3o0?l60?ooo`80o`3o1@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo
0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0
o`3oool0oooo0?l0o`D0oooo00<0o`3o0?ooo`3o0?l00`3oool01P000000oooo0?l0o`3o0?l0oooo
000000@0oooo0P3o0?l60?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0
o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool20?l0o`H0oooo00<0o`3o0?ooo`3o0?l01@3oool2
0?l0o`H0oooo00<0o`3o0?ooo`3o0?l00`3oool01P000000oooo0?l0o`3oool0o`3o000000@0oooo
00<0o`3o0?ooo`3oool0103oool010000000oooo0?ooo`0000030?ooo`006P3oool00`000000oooo
0?ooo`070?ooo`030?l0o`3oool0oooo03<0oooo0P3o0?l30?ooo`030000003o0?l0oooo0080oooo
00@0o`3o0?ooo`3oool0o`3o103oool00`3o0?l0oooo0?l0o`030?ooo`030000003oool0oooo0080
o`3o1P3oool20?l0o`D0oooo00@0o`3o0?ooo`3oool0o`3o103oool00`3o0?l0oooo0?ooo`020?oo
o`030?l0o`3oool0o`3o00H0oooo0P3o0?lD0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool00`3oool00`3o0?l0oooo0?ooo`040?ooo`80o`3o00<000000?ooo`3oool00`3oool20?l0
o`D0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3o
ool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l50?oo
o`030?l0o`3oool0o`3o00<0oooo00H000000?ooo`3o0?l0o`3o0?ooo`0000040?ooo`80o`3o1P3o
ool20?l0o`030000003oool0oooo00<0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0
o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0
o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l00`3oool01P000000oooo0?l0
o`3oool0o`3o000000@0oooo00<0o`3o0?ooo`3oool0103oool010000000oooo0?ooo`0000030?oo
o`006P3oool00`000000oooo0?ooo`070?ooo`030?l0o`3oool0oooo03<0oooo0P3o0?l30?ooo`03
0000003o0?l0oooo0080oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?l0o`3oool3
00000080oooo00<0o`3o0?ooo`3oool01@3oool20?l0o`D0oooo00@0o`3o0?ooo`3oool0o`3o103o
ool00`3o0?l0oooo0?ooo`020?ooo`040?l0o`3oool0oooo0?l0o`D0oooo0P3o0?l70?ooo`030?l0
o`3oool0oooo00X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`80o`3o
103oool01@000000oooo0?l0o`3o0?l0000000D0oooo00<0o`3o0?ooo`3o0?l0103oool0103o0?l0
oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103o
ool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`D0oooo00<0o`3o0?ooo`3o
0?l00`3oool01P000000oooo0?l0o`3oool0o`3o000000@0oooo0P3o0?l40?ooo`050000003oool0
o`3o0?l0o`0000001@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0
o`3o0?ooo`3o0?l0103oool0103o0?l0oooo0?ooo`3o0?l50?ooo`030?l0o`3oool0o`3o00D0oooo
00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`030?ooo`060000003o0?l0oooo0?ooo`3o
0?l00000103oool00`3o0?l0oooo0?ooo`050?ooo`030000003oool0000000<0oooo000J0?ooo`80
0000203oool00`3o0?l0oooo0?ooo`0c0?ooo`80o`3o103oool01@3o0?l00000000000000000o`3o
0080oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0oooo0?l0o`030?ooo`040000003oool0oooo
0?l0o`L0oooo0P3o0?l50?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00<0o`3o0?ooo`3oool00P3o
ool0103o0?l0oooo0?ooo`3o0?l50?ooo`80o`3o1`3oool00`3o0?l0oooo0?ooo`0;0?ooo`800000
00<0o`3o0000003oool0103oool20?l0o`D0oooo00<000000?l0o`3o0?l01@3oool0103o0?l0oooo
0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0
103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0
o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040000003o0?l000000?l0o`D0oooo0P3o0?l5
0?ooo`030000003o0?l0o`3o00H0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`05
0?ooo`030?l0o`3oool0o`3o00@0oooo00@0o`3o0?ooo`3oool0o`3o1@3oool00`3o0?l0oooo0?l0
o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l0103oool0103o0?l000000000
003o0?l50?ooo`030?l0o`3oool0oooo00H0oooo0P0000030?ooo`006P3oool00`000000oooo0?oo
o`070?ooo`030?l0o`3oool0oooo03<0oooo0P3o0?l40?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0
0P3oool00`3o0?l0oooo0?ooo`030?ooo`80o`3o1P3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0
o`3oool0oooo00@0oooo00@0o`3o0?ooo`3oool0o`3o103oool01@3o0?l0oooo0?ooo`3oool0o`3o
00<0oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0oooo0?l0o`060?ooo`80o`3o3`3oool00`3o
0?l0oooo0?ooo`040?ooo`80o`3o1P3oool20?l0o`D0oooo00@0o`3o0?ooo`3oool0o`3o103oool0
103o0?l0oooo0?ooo`3o0?l30?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0103oool0103o0?l0oooo
0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`04
0?ooo`040?l0o`3oool0oooo0?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool20?l0o`H0oooo0P3o
0?l60?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0
o`040?ooo`040?l0o`3oool0oooo0?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo
0?l0o`050?ooo`030?l0o`3oool0o`3o00@0oooo00@0o`3o0?ooo`3oool0o`3o1@3oool00`3o0?l0
oooo0?ooo`0;0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030?l0o`3oool0oooo03<0oooo
0P3o0?l40?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`030?ooo`80
o`3o1P3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0oooo0P3o0?l50?ooo`05
0?l0o`3oool0oooo0?ooo`3o0?l00`3oool00`3o0?l0oooo0?ooo`020?ooo`030?l0o`3oool0o`3o
00H0oooo0P3o0?l70?ooo`030?l0o`3oool0oooo00@0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0
o`D0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l30?ooo`050?l0o`3o
ool0oooo0?ooo`3o0?l0103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0
o`<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0
o`3o0?ooo`3oool0o`3o1@3oool20?l0o`H0oooo0P3o0?l60?ooo`030?l0o`3oool0o`3o00D0oooo
00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0o`D0
oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0
o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool00`3o0?l0oooo0?ooo`0<0?ooo`006P3oool00`00
0000oooo0?ooo`070?ooo`030?l0o`3oool0oooo03<0oooo0P3o0?l40?ooo`070?l0o`3oool0oooo
0?ooo`3o0?l0oooo0?l0o`060?ooo`80o`3o1P3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3o
ool0oooo00D0oooo0P3o0?l50?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool00`3o0?l0oooo
0?ooo`020?ooo`030?l0o`3oool0o`3o00H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo
0P3o0?l60?ooo`80o`3o1@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0
o`<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0o`<0oooo00D0
o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0103oool0103o0?l0
oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00<0o`3o0?ooo`3o0?l01@3oool0
0`3o0?l0oooo0?l0o`050?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o
1@3oool00`3o0?l0oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0o`D0oooo00<0o`3o0?ooo`3o
0?l0103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o
0?ooo`3oool0o`3o103oool00`3o0?l0oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?ooo`07
0?ooo`030?l0o`3oool0oooo03<0oooo0P3o0?l40?ooo`070?l0o`3oool0oooo0?ooo`3o0?l0oooo
0?l0o`060?ooo`80o`3o1P3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0oooo
0P3o0?l50?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool00`3o0?l0oooo0?ooo`020?ooo`03
0?l0o`3oool0o`3o00H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80
o`3o1@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`<0oooo00D0o`3o
0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo
0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`040?ooo`040?l0o`3oool0oooo
0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`030?l0
o`3oool0o`3o00D0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?oo
o`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?oo
o`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o
0?l0oooo0?ooo`3o0?l40?ooo`030?l0o`3oool0oooo00`0oooo000J0?ooo`030000003oool0oooo
00L0oooo00<0o`3o0?ooo`3oool0<`3oool20?l0o`@0oooo00L0o`3o0?ooo`3oool0oooo0?l0o`3o
ool0o`3o00H0oooo0P3o0?l>0?ooo`030?l0o`3oool0oooo00D0oooo0P3o0?l50?ooo`050?l0o`3o
ool0oooo0?ooo`3o0?l00`3oool00`3o0?l0oooo0?ooo`020?ooo`030?l0o`3oool0o`3o00H0oooo
0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1@3oool0103o0?l0oooo
0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`03
0?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo
00D0o`3o0?ooo`3oool0oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?oo
o`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`030?l0o`3oool0o`3o00D0oooo00@0
o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0
o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3o
ool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?oo
o`030?l0o`3oool0oooo00`0oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`3o0?ooo`3o
ool00`3oool00`3o0?l0oooo0?ooo`0]0?ooo`80o`3o103oool01`3o0?l0oooo0?ooo`3oool0o`3o
0?ooo`3o0?l01P3oool00`3o0?l0oooo0?ooo`0E0?ooo`80o`3o1@3oool01@3o0?l0oooo0?ooo`3o
ool0o`3o00<0oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0oooo0?l0o`060?ooo`80o`3o1P3o
ool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1@3oool00`3o0?l0oooo0?l0o`050?ooo`040?l0o`3o
ool0oooo0?l0o`<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?oo
o`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0
o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`04
0?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o
0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0
oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103o
ool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3o
ool0o`3o103oool00`3o0?l0oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?ooo`080?ooo`04
0?l0o`3oool0oooo0?ooo`80o`3o;P3oool00`3o0?l0oooo0?l0o`040?ooo`070?l0o`3oool0oooo
0?ooo`3o0?l0oooo0?l0o`060?ooo`030?l0o`3oool0oooo01D0oooo0P3o0?l50?ooo`040?l0o`3o
ool0oooo0?l0o`@0oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0oooo0?l0o`060?ooo`80o`3o
1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1@3oool00`3o0?l0oooo0?l0o`050?ooo`040?l0
o`3oool0oooo0?l0o`<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo
0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo
0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o
00@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3o
ool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?oo
o`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?oo
o`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o
0?l0oooo0?ooo`3o0?l40?ooo`030?l0o`3oool0oooo00`0oooo000B0?oooa<0000000<0o`3o0000
000000000`3o0?l^000000030?l0o`000000o`3o00@0000000L0o`3o00000000000000000?l0o`00
0000o`3o00H0000000<0o`3o0000000000005@0000000`3o0?l0000000000004000000040?l0o`00
000000000?l0o`@0000000<0o`3o0000000000000P0000000`3o0?l000000?l0o`0600000080o`3o
1P0000020?l0o`H000000P3o0?l600000080o`3o1@0000000`3o0?l000000?l0o`04000000050?l0
o`00000000000000003o0?l00`0000001@3o0?l00000000000000000o`3o00<0000000D0o`3o0000
0000000000000?l0o`03000000050?l0o`00000000000000003o0?l00`0000001@3o0?l000000000
00000000o`3o00<0000000D0o`3o00000000000000000?l0o`03000000050?l0o`00000000000000
003o0?l010000000103o0?l000000000003o0?l4000000040?l0o`00000000000?l0o`@0000000@0
o`3o000000000000o`3o10000000103o0?l000000000003o0?l4000000040?l0o`00000000000?l0
o`@0000000@0o`3o000000000000o`3o10000000103o0?l000000000003o0?l4000000040?l0o`00
000000000?l0o`@0000000@0o`3o000000000000o`3o10000000103o0?l000000000003o0?l40000
00040?l0o`00000000000?l0o`@0000000<0o`3o000000000000200000040?ooo`006P3oool00`00
0000oooo0?ooo`080?ooo`@0o`3o00<0oooo0?l0o`3oool03P3oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo00`0oooo00<0o`3o0000003o0?l0103oool01`3o0?l0oooo0?ooo`3oool0
o`3o0?ooo`3o0?l00`3oool010000000oooo0?ooo`3o0?l=0?ooo`030000003oool0oooo00L0oooo
00<0o`3o0?ooo`3oool00`3oool01@000000o`3o0?ooo`3oool0o`3o00@0oooo00<0o`3o0?ooo`3o
ool00P3oool00`3o0?l0oooo0?l0o`060?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80
o`3o1@3oool00`3o0?l0oooo0?l0o`040?ooo`050?l0o`3oool0oooo0000003o0?l00`3oool01@3o
0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool000000?l0o`030?ooo`030?l0o`3o
ool0oooo0080oooo00@0o`3o0?ooo`3oool0o`3o0P3oool00`000000oooo0?l0o`020?ooo`050?l0
o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`000000o`3o00<0oooo00D0o`3o0?oo
o`3oool0oooo0?l0o`040?ooo`040?l0o`3oool000000?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o
103oool0103o0?l0oooo0000003o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?oo
o`000000o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool000000?l0o`@0oooo
00@0o`3o0?ooo`3oool0o`3o0`3oool01@3o0?l0oooo0?ooo`000000o`3o00@0oooo00@0o`3o0?oo
o`3oool0o`3o0`3oool0103o0?l0oooo0?ooo`00000<0?ooo`006P3oool00`000000oooo0?ooo`0=
0?ooo`030?l0o`3oool0oooo02`0oooo00<0o`3o0?ooo`3o0?l0103oool01`3o0?l0oooo0?ooo`3o
ool0o`3o0?ooo`3o0?l00`3oool010000000oooo0?ooo`3o0?lG0?ooo`030?l0o`3oool0oooo00@0
oooo00@0o`3o0?ooo`3oool0o`3o103oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o
0?ooo`3oool0103oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`D0oooo00<0o`3o
0?ooo`3o0?l0103oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo
0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool00`3o0?l0oooo0?ooo`020?ooo`04
0?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o
0?l30?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool00`3o0?l0oooo0?ooo`020?ooo`040?l0
o`3oool000000?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l3
0?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`050?l0
o`3oool0oooo0?ooo`3o0?l00`3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo
0?l0o`<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0
0`3oool0103o0?l0oooo0?ooo`00000<0?ooo`006P3oool00`000000oooo0?ooo`0=0?ooo`030?l0
o`3oool0oooo02`0oooo00<0o`3o0?ooo`3o0?l0103oool01`3o0?l0oooo0?ooo`3oool0o`3o0?oo
o`3o0?l07P3oool00`3o0?l0oooo0?ooo`040?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00D0o`3o
0?ooo`3oool0oooo0?l0o`030?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3o0?l01P3o
ool20?l0o`H0oooo0P3o0?l50?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l0103o
ool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`05
0?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`030?ooo`030?l0o`3oool0o`3o
00D0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l30?ooo`050?l0o`3o
ool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3o
ool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3o
ool0o`3o00<0oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`050?l0
o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00D0o`3o0?oo
o`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?oo
o`3oool0o`3o00<0oooo00<0o`3o0?ooo`3oool03@3oool001X0oooo00<000000?ooo`3oool03@3o
ool00`3o0?l0oooo0?ooo`0/0?ooo`030?l0o`3oool0o`3o00<0oooo00<0o`3o0?ooo`3oool00P3o
ool00`3o0?l0oooo0?l0o`0N0?ooo`030?l0o`3oool0oooo00@0oooo00@0o`3o0?ooo`3oool0o`3o
103oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0
oooo0?l0o`060?ooo`80o`3o1P3oool20?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool0103o0?l0
oooo0?ooo`3o0?l30?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3o
ool0o`3o0080oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o0?l50?ooo`030?l0
o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`04
0?l0o`3oool0oooo0?l0o`<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0
oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0
oooo0?l0o`030?ooo`030?l0o`3oool0oooo0080oooo00@0o`3o0?ooo`3oool0o`3o103oool01@3o
0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o
0?l30?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0
oooo00D0o`3o0?ooo`3oool0oooo0?l0o`0?0?ooo`006P3oool00`000000oooo0?ooo`0=0?ooo`03
0?l0o`3oool0oooo02`0oooo00<0o`3o0?ooo`3o0?l00`3oool00`3o0?l0oooo0?ooo`020?ooo`03
0?l0o`3oool0o`3o02H0oooo0P3o0?l50?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool00`3o
0?l0oooo0?ooo`020?ooo`030?l0o`3oool0o`3o00H0oooo0P3o0?l50?ooo`030?l0o`3oool0o`3o
00D0oooo00<0o`3o0?ooo`3o0?l01@3oool0103o0?l0oooo0?ooo`3o0?l30?ooo`050?l0o`3oool0
oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool0
0P3oool0103o0?l0oooo0?ooo`3o0?l50?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o
0?l01@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`<0oooo00D0o`3o
0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo
0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`030?l0o`3oool0oooo
0080oooo00@0o`3o0?ooo`3oool0o`3o103oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0
o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o0?l30?ooo`050?l0o`3oool0oooo0?ooo`3o
0?l0103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`l0oooo000J0?oo
o`030000003oool0oooo00d0oooo00<0o`3o0?ooo`3oool0;03oool00`3o0?l0oooo0?l0o`030?oo
o`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3o0?l09P3oool20?l0o`D0oooo00D0o`3o0?oo
o`3oool0oooo0?l0o`030?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3o0?l01@3oool0
0`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l0103oool0
1@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0
o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`020?ooo`040?l0o`3oool0oooo0?l0
o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o
00D0oooo00@0o`3o0?ooo`3oool0o`3o0`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0
o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0
oooo0?ooo`3oool0o`3o00<0oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o0?l3
0?ooo`030?l0o`3oool0oooo0080oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?oo
o`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o
0?l0oooo0?ooo`3o0?l?0?ooo`006P3oool00`000000oooo0?ooo`0=0?ooo`030?l0o`3oool0oooo
02`0oooo00<0o`3o0?ooo`3o0?l00`3oool00`3o0?l0oooo0?ooo`020?ooo`030?l0o`3oool0o`3o
02H0oooo0P3o0?l50?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool00`3o0?l0oooo0?ooo`02
0?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`05
0?ooo`030?l0o`3oool0o`3o00@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3o
ool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3o
ool00P3oool0103o0?l0oooo0?ooo`3o0?l50?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?oo
o`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`040?l0o`3oool0oooo0?l0o`<0oooo00D0o`3o
0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo
0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo
0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00@0oooo00@0o`3o0?ooo`3oool0o`3o
103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?oo
o`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l?0?ooo`006P3oool2000000h0oooo00<0o`3o
0?ooo`3oool0;03oool00`3o0?l0oooo0?l0o`030?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o
0?ooo`3o0?l09P3oool20?l0o`H0oooo00<0o`3o0?ooo`3o0?l0103oool00`3o0?l0oooo0?ooo`02
0?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`05
0?ooo`040?l0o`3oool0oooo0?l0o`<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0
o`3oool0oooo0080oooo00D0o`3o0?ooo`3oool0oooo0?l0o`040?ooo`030?l0o`3oool0o`3o00D0
oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0
oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`040?ooo`050?l0o`3oool0oooo0?oo
o`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0
o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0
oooo00D0o`3o0?ooo`3oool0oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o
0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0
oooo00@0o`3o0?ooo`3oool0o`3o3`3oool001X0oooo00<000000?ooo`3oool03@3oool00`3o0?l0
oooo0?ooo`0/0?ooo`040?l0o`3oool0oooo0?l0o`80oooo00<0o`3o0?ooo`3oool00P3oool00`3o
0?l0oooo0?l0o`0V0?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`040?ooo`030?l0o`3oool0oooo
0080oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o
00D0oooo00@0o`3o0?ooo`3oool0o`3o0`3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0
o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o0?l50?ooo`030?l0o`3oool0o`3o00D0oooo
00<0o`3o0?ooo`3o0?l01@3oool20?l0o`H0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo
0?l0o`050?ooo`030?l0o`3oool0o`3o00@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`05
0?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o
0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo
0?ooo`3oool0o`3o00@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l4
0?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`0@
0?ooo`006P3oool00`000000oooo0?ooo`0>0?ooo`030?l0o`3oool0oooo02/0oooo00@0o`3o0?oo
o`3oool0o`3o0P3oool00`3o0?l0oooo0?ooo`020?ooo`030?l0o`3oool0o`3o02H0oooo0P3o0?l6
0?ooo`030?l0o`3oool0o`3o00@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3o
ool0oooo00<0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`040?l0o`3o
ool0oooo0?l0o`<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo0080
oooo00@0o`3o0?ooo`3oool0o`3o1@3oool00`3o0?l0oooo0?l0o`050?ooo`80o`3o1P3oool20?l0
o`H0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o
00@0oooo00@0o`3o0?ooo`3oool0o`3o103oool01@3o0?l0oooo0?ooo`3oool0o`3o00@0oooo00@0
o`3o0?ooo`3oool0o`3o0`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3o
ool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0103oool0103o0?l0oooo0?ooo`3o
0?l40?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0
o`050?ooo`030?l0o`3oool0o`3o0100oooo000J0?ooo`030000003oool0oooo00h0oooo00<0o`3o
0?ooo`3oool0903oool00`3o0?l0oooo0?ooo`040?ooo`040?l0o`3oool0oooo0?l0o`80oooo00<0
o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?ooo`0U0?ooo`030?l0o`3oool0oooo00D0oooo00<0
o`3o0?ooo`3o0?l0103oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00<0o`3o0?ooo`3oool0
0P3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00@0oooo00D0o`3o0?ooo`3oool0
oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`020?oo
o`040?l0o`3oool0oooo0?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool20?l0o`H0oooo0P3o0?l6
0?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00@0oooo00@0o`3o
0?ooo`3oool0o`3o103oool01@3o0?l0oooo0?ooo`3oool0o`3o00@0oooo00@0o`3o0?ooo`3oool0
o`3o0`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`03
0?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`030?l0
o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0
o`3oool0o`3o0100oooo000J0?ooo`030000003oool0oooo00h0oooo00<0o`3o0?ooo`3oool0903o
ool00`3o0?l0oooo0?ooo`040?ooo`040?l0o`3oool0oooo0?l0o`80oooo00<0o`3o0?ooo`3oool0
0`3oool00`3o0?l0oooo0?ooo`0U0?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3o0?l0
103oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0
oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`03
0?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`020?ooo`040?l0o`3o
ool0oooo0?l0o`D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o
1P3oool00`3o0?l0oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0o`D0oooo00@0o`3o0?ooo`3o
ool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l30?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00D0
o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0103oool00`3o0?l0
oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0
oooo0?l0o`050?ooo`030?l0o`3oool0o`3o0100oooo000J0?ooo`030000003oool0oooo00h0oooo
00<0o`3o0?ooo`3oool0903oool00`3o0?l0oooo0?ooo`040?ooo`040?l0o`3oool0oooo0?l0o`80
oooo00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?ooo`0U0?ooo`030?l0o`3oool0oooo00D0
oooo00<0o`3o0?ooo`3o0?l0103oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?oo
o`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo0080oooo00@0o`3o0?ooo`3oool0o`3o103o
ool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`03
0?l0o`3oool0oooo0080oooo00@0o`3o0?ooo`3oool0o`3o1@3oool20?l0o`H0oooo0P3o0?l60?oo
o`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?oo
o`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?oo
o`3o0?l0103oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0
o`040?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0
o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l0403oool001X0oooo00<00000
0?ooo`3oool03P3oool00`3o0?l0oooo0?ooo`0T0?ooo`030?l0o`3oool0oooo00@0oooo00H0o`3o
0?ooo`3oool0o`3o0?ooo`3o0?l60?ooo`030?l0o`3oool0oooo02D0oooo00<0o`3o0?ooo`3oool0
1@3oool20?l0o`D0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?oo
o`3o0?l00P3oool00`3o0?l0oooo0?ooo`020?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool0
1@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?oo
o`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0
oooo0P3o0?l60?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0
oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0
oooo00@0o`3o0?ooo`3oool0o`3o103oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o
00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o
00D0oooo00<0o`3o0?ooo`3o0?l0403oool001X0oooo00<000000?ooo`3oool03P3oool00`3o0?l0
oooo0?ooo`0T0?ooo`80o`3o1@3oool01P3o0?l0oooo0?ooo`3o0?l0oooo0?l0o`H0oooo00<0o`3o
0?ooo`3oool0;@3oool20?l0o`D0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3o
ool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`020?ooo`050?l0o`3oool0oooo0?ooo`3o
0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00P3oool0103o
0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`D0oooo0P3o0?l60?ooo`80o`3o1P3o
ool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo00<0o`3o0?ooo`3o0?l01@3o
ool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3o
ool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3o
ool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3o
ool00`3o0?l0oooo0?l0o`0@0?ooo`006P3oool00`000000oooo0?ooo`0>0?ooo`030?l0o`3oool0
oooo02@0oooo0P3o0?l50?ooo`060?l0o`3oool0oooo0?l0o`3oool0o`3o1P3oool00`3o0?l0oooo
0?ooo`0]0?ooo`80o`3o1@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`050?l0o`3oool0oooo0?oo
o`3o0?l00P3oool00`3o0?l0oooo0?ooo`020?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool0
1@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?oo
o`3o0?l50?ooo`030?l0o`3oool0o`3o00D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo
0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`030?l0o`3oool0o`3o00D0oooo
00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo
00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo
0P3o0?l60?ooo`80o`3o1P3oool20?l0oa40oooo000J0?ooo`8000003`3oool00`3o0?l0oooo0?oo
o`0T0?ooo`80o`3o1@3oool01P3o0?l0oooo0?ooo`3o0?l0oooo0?l0ocH0oooo0P3o0?l50?ooo`04
0?l0o`3oool0oooo0?l0o`@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0
oooo0080oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0
0P3oool00`3o0?l0oooo0?ooo`030?ooo`030?l0o`3oool0o`3o00D0oooo0P3o0?l60?ooo`80o`3o
1P3oool00`3o0?l0oooo0?ooo`050?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o
1P3oool20?l0o`H0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0
o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0
o`3oool0o`3o00D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?lA0?ooo`006P3o
ool00`000000oooo0?ooo`0>0?ooo`030?l0o`3oool0oooo02@0oooo0P3o0?l40?ooo`070?l0o`3o
ool0oooo0?ooo`3o0?l0oooo0?l0o`0f0?ooo`80o`3o1@3oool0103o0?l0oooo0?ooo`3o0?l40?oo
o`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0
o`3o0?ooo`3oool0oooo0?l0o`030?ooo`040?l0o`3oool0oooo0?l0o`D0oooo00<0o`3o0?ooo`3o
0?l01@3oool20?l0o`H0oooo0P3o0?l60?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3o
ool01@3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo00<0o`3o0?ooo`3o
0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o
0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo0P3o0?l60?ooo`80
o`3o1P3oool20?l0o`H0oooo0P3o0?lA0?ooo`006P3oool00`000000oooo0?ooo`0>0?ooo`030?l0
o`3oool0oooo02@0oooo0P3o0?l40?ooo`070?l0o`3oool0oooo0?ooo`3o0?l0oooo0?l0o`0f0?oo
o`80o`3o1@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00D0
o`3o0?ooo`3oool0oooo0?l0o`030?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3o
ool0o`3o1@3oool20?l0o`H0oooo0P3o0?l60?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?oo
o`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?oo
o`80o`3o1P3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?oo
o`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo0P3o0?l60?oo
o`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o4@3oool001X0oooo00<000000?ooo`3o
ool03P3oool00`3o0?l0oooo0?ooo`0T0?ooo`80o`3o103oool01`3o0?l0oooo0?ooo`3oool0o`3o
0?ooo`3o0?l0=P3oool00`3o0?l0oooo0?ooo`040?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0
o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0
o`@0oooo00@0o`3o0?ooo`3oool0o`3o1@3oool20?l0o`H0oooo0P3o0?l60?ooo`030?l0o`3oool0
oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0
oooo00D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo00<0o`3o0?ooo`3o0?l01@3oool0
0`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool0
0`3o0?l0oooo0?l0o`050?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool2
0?l0oa40oooo000J0?ooo`030000003oool0oooo00h0oooo00<0o`3o0?ooo`3oool0903oool20?l0
o`@0oooo00L0o`3o0?ooo`3oool0oooo0?l0o`3oool0o`3o03H0oooo00<0o`3o0?ooo`3oool0103o
ool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3o
ool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`D0oooo0P3o
0?l60?ooo`80o`3o1P3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0oooo00<0
o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0oooo0P3o
0?l60?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0
o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o
0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?lA0?ooo`006P3oool00`000000oooo0?ooo`0>
0?ooo`030?l0o`3oool0oooo02<0oooo00<0o`3o0?ooo`3o0?l0103oool01`3o0?l0oooo0?ooo`3o
ool0o`3o0?ooo`3o0?l0=P3oool00`3o0?l0oooo0?ooo`040?ooo`040?l0o`3oool0oooo0?l0o`@0
oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0
oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o1@3oool20?l0o`H0oooo0P3o0?l60?ooo`030?l0
o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool03@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0
o`3oool0oooo00D0oooo0P3o0?l60?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0
o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o
1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0oa40oooo000J0?ooo`030000003o
ool0oooo00h0oooo00<0o`3o0?ooo`3oool08`3oool00`3o0?l0oooo0?l0o`040?ooo`070?l0o`3o
ool0oooo0?ooo`3o0?l0oooo0?l0o`0f0?ooo`030?l0o`3oool0oooo00@0oooo00@0o`3o0?ooo`3o
ool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0
o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l50?ooo`80o`3o1P3oool20?l0o`H0
oooo00<0o`3o0?ooo`3oool07@3oool00`3o0?l0oooo0?ooo`050?ooo`80o`3o1P3oool20?l0o`H0
oooo0P3o0?l60?ooo`030?l0o`3oool0o`3o00D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0
oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o4@3oool001X0oooo
00<000000?ooo`3oool03P3oool00`3o0?l0oooo0?ooo`0S0?ooo`030?l0o`3oool0o`3o00@0oooo
00@0o`3o0?ooo`3oool0oooo0P3o0?lg0?ooo`030?l0o`3oool0oooo00@0oooo00@0o`3o0?ooo`3o
ool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0
o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l50?ooo`80o`3o1P3oool20?l0o`H0
oooo00<0o`3o0?ooo`3oool07@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0
oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0
oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`L0oooo00<0o`3o0?ooo`3oool0103oool20?l0oa40
oooo000J0?ooo`8000003`3oool00`3o0?l0oooo0?ooo`0S0?ooo`030?l0o`3oool0o`3o00@0oooo
00@0o`3o0?ooo`3oool0oooo0P3o0?lg0?ooo`030?l0o`3oool0oooo00D0oooo0P3o0?l50?ooo`04
0?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o
0?l40?ooo`030?l0o`3oool0o`3o00H0oooo0P3o0?l60?ooo`80o`3o1P3oool00`3o0?l0oooo0?oo
o`0U0?ooo`030?l0o`3oool0oooo00D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o
0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`L0oooo00<0
o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`0?0?ooo`006P3oool00`000000oooo0?ooo`0?
0?ooo`030?l0o`3oool0oooo0280oooo00<0o`3o0?ooo`3o0?l0103oool0103o0?l0oooo0?ooo`3o
ool20?l0ocl0oooo0P3o0?l50?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0
o`3o1@3oool00`3o0?l0oooo0?l0o`040?ooo`030?l0o`3oool0o`3o00H0oooo0P3o0?l60?ooo`03
0?l0o`3oool0oooo02d0oooo00<0o`3o0?ooo`3oool01@3oool20?l0o`H0oooo0P3o0?l60?ooo`80
o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l70?ooo`03
0?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`0?0?ooo`00
6P3oool00`000000oooo0?ooo`0?0?ooo`030?l0o`3oool0oooo0280oooo00<0o`3o0?ooo`3o0?l0
103oool0103o0?l0oooo0?ooo`3oool20?l0ocl0oooo0P3o0?l50?ooo`040?l0o`3oool0oooo0?l0
o`@0oooo00@0o`3o0?ooo`3oool0o`3o1@3oool00`3o0?l0oooo0?l0o`040?ooo`030?l0o`3oool0
o`3o00H0oooo0P3o0?l60?ooo`030?l0o`3oool0oooo03D0oooo00<0o`3o0?ooo`3oool01@3oool2
0?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool2
0?l0o`L0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0
oooo00l0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0o`3o0?ooo`3oool08P3oool00`3o
0?l0oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?ooo`80o`3o?`3oool20?l0o`D0oooo00@0o`3o
0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l50?ooo`030?l0o`3oool0o`3o00@0oooo
00<0o`3o0?ooo`3o0?l01P3oool20?l0o`H0oooo00<0o`3o0?ooo`3oool0=@3oool00`3o0?l0oooo
0?ooo`050?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo
0P3o0?l70?ooo`030?l0o`3oool0oooo02L0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0
o`3o0?ooo`3oool08P3oool00`3o0?l0oooo0?l0o`030?ooo`030?l0o`3oool0oooo00<0oooo00<0
o`3o0?ooo`3oool0?@3oool20?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool0103o0?l0oooo0?oo
o`3o0?l50?ooo`030?l0o`3oool0o`3o00D0oooo0P3o0?l60?ooo`030?l0o`3oool0oooo00D0oooo
00<0o`3o0?ooo`3oool0?@3oool00`3o0?l0oooo0?ooo`050?ooo`80o`3o1P3oool20?l0o`H0oooo
0P3o0?l60?ooo`80o`3o1`3oool00`3o0?l0oooo0?ooo`0_0?ooo`001@0000040?ooo`800000103o
ool5000000H0oooo00<000000?ooo`3oool03`3oool00`3o0?l0oooo0?ooo`0R0?ooo`030?l0o`3o
ool0o`3o00<0oooo00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?ooo`0m0?ooo`80o`3o1P3o
ool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo00<0o`3o0?ooo`3oool0A@3o
ool00`3o0?l0oooo0?ooo`050?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o>@3o
ool00080oooo00<000000?ooo`3oool0103oool2000000@0oooo00D000000?ooo`3oool0oooo0000
00060?ooo`030000003oool0oooo00l0oooo00<0o`3o0?ooo`3oool08P3oool00`3o0?l0oooo0?l0
o`030?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3oool0?@3oool20?l0o`H0oooo0P3o
0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`030?l0o`3oool0oooo04d0oooo00<0
o`3o0?ooo`3oool01@3oool20?l0o`H0oooo0P3o0?m10?ooo`000P3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool03`3oool00`3o0?l0oooo0?ooo`03
0?ooo`030?l0o`3oool0oooo01`0oooo00@0o`3o0?ooo`3oool0o`3o0P3oool00`3o0?l0oooo0?oo
o`030?ooo`030?l0o`3oool0oooo03d0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o
0?l60?ooo`80o`3o1P3oool00`3o0?l0oooo0?ooo`2P0?ooo`000P3oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo00H0oooo0`00000?0?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o
0?ooo`3oool05@3oool00`3o0?l0oooo0?ooo`040?ooo`040?l0o`3oool0oooo0?l0o`80oooo00<0
o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?ooo`0m0?ooo`030?l0o`3oool0oooo00D0oooo0P3o
0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`030?l0o`3oool0oooo0:00oooo0002
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0?
0?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3oool05@3oool00`3o0?l0oooo0?ooo`04
0?ooo`040?l0o`3oool0oooo0?l0o`80oooo00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?oo
o`0m0?ooo`030?l0o`3oool0oooo00D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o
0?l60?ooo`030?l0o`3oool0oooo0:00oooo00020?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`0?0?ooo`030?l0o`3oool0oooo00<0oooo0P3o0?lF
0?ooo`030?l0o`3oool0oooo00@0oooo00@0o`3o0?ooo`3oool0o`3o0P3oool00`3o0?l0oooo0?oo
o`030?ooo`030?l0o`3oool0oooo03d0oooo00<0o`3o0?ooo`3oool01@3oool20?l0o`H0oooo0P3o
0?l60?ooo`80o`3o1P3oool00`3o0?l0oooo0?ooo`2X0?ooo`000`00000<0?ooo`050000003oool0
oooo0?ooo`0000001P3oool00`000000oooo0?ooo`0?0?ooo`030?l0o`3oool0oooo0080oooo00<0
o`3o0?ooo`3o0?l05P3oool00`3o0?l0oooo0?ooo`040?ooo`040?l0o`3oool0oooo0?l0o`80oooo
00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?ooo`150?ooo`80o`3o1P3oool20?l0o`H0oooo
0P3o0?l60?ooo`030?l0o`3oool0oooo0:P0oooo00020?ooo`030000003oool0oooo00/0oooo0`00
00070?ooo`030000003oool0oooo00l0oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0oooo0?l0
o`0F0?ooo`030?l0o`3oool0oooo00@0oooo00@0o`3o0?ooo`3oool0o`3o0P3oool00`3o0?l0oooo
0?ooo`030?ooo`030?l0o`3oool0oooo04D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo
00<0o`3o0?ooo`3oool0Z03oool001X0oooo00<000000?ooo`3oool03`3oool00`3o0?l0oooo0?oo
o`020?ooo`030?l0o`3oool0o`3o01H0oooo00<0o`3o0?ooo`3oool0103oool0103o0?l0oooo0?oo
o`3o0?l20?ooo`030?l0o`3oool0oooo04/0oooo00<0o`3o0?ooo`3oool01@3oool20?l0o`H0oooo
0P3o0?l60?ooo`030?l0o`3oool0oooo0:P0oooo000J0?ooo`030000003oool0oooo00l0oooo00D0
o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo01@0oooo00<0o`3o0?ooo`3oool0
103oool01P3o0?l0oooo0?ooo`3o0?l0oooo0?l0odh0oooo00<0o`3o0?ooo`3oool01@3oool00`3o
0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool0Z03oool001X0
oooo00<000000?ooo`3oool03`3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?oo
o`3oool0503oool20?l0o`@0oooo00L0o`3o0?ooo`3oool0oooo0?l0o`3oool0o`3o04h0oooo00<0
o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo0;00oooo000J
0?ooo`030000003oool0oooo00l0oooo00@0o`3o0?ooo`3oool0o`3o0`3oool00`3o0?l0oooo0?oo
o`0D0?ooo`80o`3o103oool01`3o0?l0oooo0?ooo`3oool0o`3o0?ooo`3o0?l0CP3oool00`3o0?l0
oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool0/03oool001X0oooo
0P00000@0?ooo`040?l0o`3oool0oooo0?l0o`<0oooo00<0o`3o0?ooo`3oool0503oool20?l0o`@0
oooo00L0o`3o0?ooo`3oool0oooo0?l0o`3oool0o`3o05H0oooo00<0o`3o0?ooo`3oool01@3oool0
0`3o0?l0oooo0?ooo`2`0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`040?l0o`3oool0oooo
0?l0o`<0oooo00<0o`3o0?ooo`3oool0503oool20?l0o`@0oooo00L0o`3o0?ooo`3oool0oooo0?l0
o`3oool0o`3o05H0oooo00<0o`3o0?ooo`3oool0^03oool001X0oooo00<000000?ooo`3oool03`3o
ool00`3o0?l0oooo0?l0o`040?ooo`030?l0o`3oool0oooo01<0oooo00<0o`3o0?ooo`3o0?l0103o
ool01`3o0?l0oooo0?ooo`3oool0o`3o0?ooo`3o0?l0o`3ooolB0?ooo`006P3oool00`000000oooo
0?ooo`0@0?ooo`80o`3o103oool00`3o0?l0oooo0?ooo`0C0?ooo`030?l0o`3oool0o`3o00@0oooo
00L0o`3o0?ooo`3oool0oooo0?l0o`3oool0o`3o0?l0oooo4P3oool001X0oooo00<000000?ooo`3o
ool0403oool20?l0o`@0oooo00<0o`3o0?ooo`3oool04`3oool00`3o0?l0oooo0?l0o`040?ooo`07
0?l0o`3oool0oooo0?ooo`3o0?l0oooo0?l0o`3o0?oooa80oooo000J0?ooo`030000003oool0oooo
0100oooo00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?ooo`0<0?ooo`030?l0o`3oool0oooo
00@0oooo00<0o`3o0?ooo`3o0?l0103oool01`3o0?l0oooo0?ooo`3oool0o`3o0?ooo`3o0?l0o`3o
oolB0?ooo`006P3oool00`000000oooo0?ooo`0@0?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o
0?ooo`3oool0103oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00@0oooo00<0o`3o
0?ooo`3o0?l0103oool01`3o0?l0oooo0?ooo`3oool0o`3o0?ooo`3o0?l0o`3ooolB0?ooo`006P3o
ool00`000000oooo0?ooo`0F0?ooo`030?l0o`3oool0oooo00@0oooo00<0o`3o0?ooo`3oool01@3o
ool00`3o0?l0oooo0?ooo`040?ooo`030?l0o`3oool0o`3o00@0oooo00@0o`3o0?ooo`3oool0oooo
0P3o0?oo0?oooa<0oooo000J0?ooo`8000005`3oool00`3o0?l0oooo0?ooo`040?ooo`030?l0o`3o
ool0oooo00D0oooo00<0o`3o0?ooo`3oool0103oool00`3o0?l0oooo0?l0o`040?ooo`040?l0o`3o
ool0oooo0?ooo`80o`3oo`3ooolC0?ooo`006P3oool00`000000oooo0?ooo`0F0?ooo`030?l0o`3o
ool0oooo00@0oooo00<0o`3o0?ooo`3oool01@3oool20?l0o`D0oooo00<0o`3o0?ooo`3o0?l00`3o
ool00`3o0?l0oooo0?ooo`020?ooo`80o`3oo`3ooolC0?ooo`006P3oool00`000000oooo0?ooo`0F
0?ooo`030?l0o`3oool0oooo00@0oooo00<0o`3o0?ooo`3oool01@3oool20?l0o`D0oooo00@0o`3o
0?ooo`3oool0o`3o0P3oool00`3o0?l0oooo0?ooo`020?ooo`80o`3oo`3ooolC0?ooo`006P3oool0
0`000000oooo0?ooo`0F0?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3o0?l01P3oool2
0?l0o`D0oooo00@0o`3o0?ooo`3oool0o`3o0P3oool00`3o0?l0oooo0?ooo`030?ooo`030?l0o`3o
ool0oooo0?l0oooo4@3oool001X0oooo00<000000?ooo`3oool05P3oool00`3o0?l0oooo0?ooo`03
0?ooo`030?l0o`3oool0o`3o00H0oooo0P3o0?l50?ooo`040?l0o`3oool0oooo0?l0o`80oooo00<0
o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?ooo`3o0?oooa40oooo000J0?ooo`030000003oool0
oooo01H0oooo00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?l0o`060?ooo`80o`3o1@3oool0
103o0?l0oooo0?ooo`3o0?l20?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3oool0o`3o
oolA0?ooo`006P3oool00`000000oooo0?ooo`0F0?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o
0?ooo`3o0?l01P3oool20?l0o`D0oooo00@0o`3o0?ooo`3oool0o`3o0P3oool00`3o0?l0oooo0?oo
o`030?ooo`030?l0o`3oool0oooo0?l0oooo4@3oool001X0oooo00<000000?ooo`3oool05P3oool0
0`3o0?l0oooo0?ooo`030?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool0
103o0?l0oooo0?ooo`3o0?l20?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3oool0o`3o
oolA0?ooo`006P3oool2000001L0oooo00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?l0o`05
0?ooo`030?l0o`3oool0o`3o00D0oooo00@0o`3o0?ooo`3oool0o`3o0P3oool00`3o0?l0oooo0?oo
o`030?ooo`030?l0o`3oool0oooo0?l0oooo4@3oool001X0oooo00<000000?ooo`3oool05P3oool0
0`3o0?l0oooo0?ooo`030?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool0
103o0?l0oooo0?ooo`3o0?l20?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3oool0o`3o
oolA0?ooo`006P3oool00`000000oooo0?ooo`0F0?ooo`030?l0o`3oool0oooo0080oooo00@0o`3o
0?ooo`3oool0o`3o1@3oool00`3o0?l0oooo0?l0o`040?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0
0P3oool00`3o0?l0oooo0?ooo`030?ooo`030?l0o`3oool0oooo0?l0oooo4@3oool001X0oooo00<0
00000?ooo`3oool05`3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool0
0`3oool00`3o0?l0oooo0?l0o`040?ooo`070?l0o`3oool0oooo0?ooo`3o0?l0oooo0?l0o`060?oo
o`030?l0o`3oool0oooo0?l0oooo4@3oool001X0oooo00<000000?ooo`3oool05`3oool01@3o0?l0
oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?l0o`040?oo
o`070?l0o`3oool0oooo0?ooo`3o0?l0oooo0?l0o`3o0?oooaX0oooo000J0?ooo`030000003oool0
oooo01L0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo00<0oooo00<0
o`3o0?ooo`3o0?l0103oool01`3o0?l0oooo0?ooo`3oool0o`3o0?ooo`3o0?l0o`3ooolJ0?ooo`00
1@0000040?ooo`8000001`3oool3000000D0oooo00<000000?ooo`3oool05`3oool01@3o0?l0oooo
0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00P3oool01@3o0?l0oooo0?ooo`3oool0o`3o
00<0oooo00L0o`3o0?ooo`3oool0oooo0?l0o`3oool0o`3o0?l0oooo6P3oool00080oooo00<00000
0?ooo`3oool0103oool2000000P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0G
0?ooo`040?l0o`3oool0oooo0?l0o`<0oooo00<0o`3o0?ooo`3oool00P3oool01@3o0?l0oooo0?oo
o`3oool0o`3o00<0oooo00L0o`3o0?ooo`3oool0oooo0?l0o`3oool0o`3o0?l0oooo6P3oool00080
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01L0
oooo00@0o`3o0?ooo`3oool0o`3o0`3oool00`3o0?l0oooo0?ooo`020?ooo`050?l0o`3oool0oooo
0?ooo`3o0?l00`3oool01`3o0?l0oooo0?ooo`3oool0o`3o0?ooo`3o0?l0o`3ooolJ0?ooo`000P3o
ool00`000000oooo0?ooo`0:0?ooo`H000001@3oool3000001L0oooo00@0o`3o0?ooo`3oool0o`3o
0`3oool00`3o0?l0oooo0?ooo`020?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01`3o0?l0
oooo0?ooo`3oool0o`3o0?ooo`3o0?l0o`3ooolJ0?ooo`000P3oool00`000000oooo0?ooo`0;0?oo
o`040000003oool0oooo000000H0oooo00<000000?ooo`3oool05`3oool0103o0?l0oooo0?ooo`3o
0?l30?ooo`030?l0o`3oool0oooo0080oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`070?l0
o`3oool0oooo0?ooo`3o0?l0oooo0?l0o`3o0?oooaX0oooo00020?ooo`030000003oool0oooo00/0
oooo00@000000?ooo`3oool000001P3oool00`000000oooo0?ooo`0G0?ooo`040?l0o`3oool0oooo
0?l0o`<0oooo00<0o`3o0?ooo`3oool00P3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00L0
o`3o0?ooo`3oool0oooo0?l0o`3oool0o`3o0?l0oooo6P3oool000<000003P3oool00`000000oooo
000000060?ooo`030000003oool0oooo01L0oooo00@0o`3o0?ooo`3oool0o`3o0`3oool00`3o0?l0
oooo0?ooo`020?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`020?oo
o`030?l0o`3oool0o`3o0?l0oooo6P3oool00080oooo00<000000?ooo`3oool03@3oool2000000H0
oooo00<000000?ooo`3oool05`3oool0103o0?l0oooo0?ooo`3o0?l30?ooo`030?l0o`3oool0oooo
0080oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo0080oooo0P3o0?oo
0?oooa/0oooo000J0?ooo`030000003oool0oooo01L0oooo00<0o`3o0?ooo`3o0?l0103oool01@3o
0?l0oooo0?ooo`3oool0o`3o00@0oooo00@0o`3o0?ooo`3oool0o`3o103oool20?l0ool0oooo6`3o
ool001X0oooo00<000000?ooo`3oool05`3oool00`3o0?l0oooo0?l0o`040?ooo`050?l0o`3oool0
oooo0?ooo`3o0?l0103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`80o`3oo`3ooolK0?ooo`006P3o
ool00`000000oooo0?ooo`0G0?ooo`030?l0o`3oool0o`3o00D0oooo00@0o`3o0?ooo`3oool0o`3o
103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`80o`3oo`3ooolK0?ooo`006P3oool2000001P0oooo
00<0o`3o0?ooo`3o0?l01@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0
o`@0oooo0P3o0?oo0?oooa/0oooo000J0?ooo`030000003oool0oooo01L0oooo00<0o`3o0?ooo`3o
0?l01@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`D0oooo00<0o`3o
0?ooo`3oool0o`3ooolI0?ooo`006P3oool00`000000oooo0?ooo`0G0?ooo`030?l0o`3oool0o`3o
00D0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l50?ooo`030?l0o`3o
ool0oooo0?l0oooo6@3oool001X0oooo00<000000?ooo`3oool0603oool00`3o0?l0oooo0?ooo`04
0?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o1@3oool00`3o0?l0oooo
0?ooo`3o0?oooaT0oooo000J0?ooo`030000003oool0oooo01P0oooo00<0o`3o0?ooo`3oool0103o
ool0103o0?l0oooo0?ooo`3o0?l40?ooo`030?l0o`3oool0o`3o00H0oooo00<0o`3o0?ooo`3oool0
o`3ooolI0?ooo`006P3oool00`000000oooo0?ooo`0H0?ooo`030?l0o`3oool0oooo00@0oooo00@0
o`3o0?ooo`3oool0o`3o103oool00`3o0?l0oooo0?l0o`060?ooo`030?l0o`3oool0oooo0?l0oooo
6@3oool001X0oooo00<000000?ooo`3oool0603oool00`3o0?l0oooo0?ooo`040?ooo`040?l0o`3o
ool0oooo0?l0o`@0oooo00<0o`3o0?ooo`3o0?l01P3oool00`3o0?l0oooo0?ooo`3o0?oooaT0oooo
000J0?ooo`030000003oool0oooo01l0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0
o`060?ooo`030?l0o`3oool0oooo0?l0oooo6@3oool001X0oooo0P00000P0?ooo`030?l0o`3oool0
o`3o00D0oooo00<0o`3o0?ooo`3o0?l01P3oool00`3o0?l0oooo0?ooo`3o0?oooaT0oooo000J0?oo
o`030000003oool0oooo01l0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`3o0?oo
ob80oooo000J0?ooo`030000003oool0oooo01l0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0
oooo0?l0o`3o0?ooob80oooo000J0?ooo`030000003oool0oooo01l0oooo00<0o`3o0?ooo`3o0?l0
1@3oool00`3o0?l0oooo0?l0o`3o0?ooob80oooo000J0?ooo`030000003oool0oooo01l0oooo00<0
o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`3o0?ooob80oooo000J0?ooo`030000003oool0
oooo01l0oooo00<0o`3o0?ooo`3o0?l01@3oool20?l0ool0oooo8`3oool001X0oooo00<000000?oo
o`3oool07`3oool00`3o0?l0oooo0?l0o`050?ooo`80o`3oo`3ooolS0?ooo`006P3oool00`000000
oooo0?ooo`0O0?ooo`030?l0o`3oool0o`3o00D0oooo0P3o0?oo0?ooob<0oooo000J0?ooo`800000
8@3oool20?l0o`D0oooo0P3o0?oo0?ooob<0oooo000J0?ooo`030000003oool0oooo0200oooo0P3o
0?l50?ooo`80o`3oo`3ooolS0?ooo`006P3oool00`000000oooo0?ooo`0P0?ooo`030?l0o`3oool0
oooo00D0oooo00<0o`3o0?ooo`3oool0o`3ooolQ0?ooo`006P3oool00`000000oooo0?ooo`0P0?oo
o`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool0o`3ooolQ0?ooo`006P3oool00`000000
oooo0?ooo`0P0?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool0o`3ooolQ0?ooo`00
6P3oool00`000000oooo0?ooo`0P0?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool0
o`3ooolQ0?ooo`001@0000040?ooo`8000001P3oool3000000H0oooo00<000000?ooo`3oool0803o
ool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo0?l0oooo8@3oool00080oooo00<00000
0?ooo`3oool0103oool2000000D0oooo00D000000?ooo`3oool0oooo000000050?ooo`030000003o
ool0oooo0200oooo00<0o`3o0?ooo`3oool0o`3ooolY0?ooo`000P3oool00`000000oooo0?ooo`0;
0?ooo`050000003oool0oooo0?ooo`0000001@3oool00`000000oooo0?ooo`3o0?oood`0oooo0002
0?ooo`030000003oool0oooo00/0oooo00D000000?ooo`3oool0oooo000000050?ooo`<00000o`3o
oom<0?ooo`000P3oool00`000000oooo0?ooo`0;0?ooo`@000001P3oool00`000000oooo0?ooo`3o
0?oood`0oooo00020?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`3o0?oood`0oooo0003000000h0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`3o0?oood`0oooo00020?ooo`030000003oool0oooo00d0oooo0`00003o0?oooe@0oooo
003o0?ooofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo
003o0?ooofT0oooo0000\
\>"],
  ImageRangeCache->{{{0, 359}, {221.375, 0}} -> {-0.0332863, 0.334837, \
0.00155747, 0.00750603}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(vCCor\  = \ 
        Plot[Evaluate[vCRip1[10^\(-3\)\ t]\  /. \ ResAve]\  + \ 
            Evaluate[vC[10^\(-3\)\ t]\  /. \ ResAve], {t, \ 0, \ 10^3\ T0}, \ 
          PlotPoints\  -> \ 200, \ 
          PlotStyle\  -> \ {{AbsoluteThickness[0.001], 
                RGBColor[1, \ 0, \ 1]}}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 2.38095 -1.22846 0.217015 [
[.2619 .06113 -9 -9 ]
[.2619 .06113 9 0 ]
[.5 .06113 -9 -9 ]
[.5 .06113 9 0 ]
[.7381 .06113 -9 -9 ]
[.7381 .06113 9 0 ]
[.97619 .06113 -9 -9 ]
[.97619 .06113 9 0 ]
[.01131 .18213 -18 -4.5 ]
[.01131 .18213 0 4.5 ]
[.01131 .29064 -6 -4.5 ]
[.01131 .29064 0 4.5 ]
[.01131 .39915 -18 -4.5 ]
[.01131 .39915 0 4.5 ]
[.01131 .50766 -6 -4.5 ]
[.01131 .50766 0 4.5 ]
[.01131 .61616 -18 -4.5 ]
[.01131 .61616 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .07363 m
.2619 .07988 L
s
[(0.1)] .2619 .06113 0 1 Mshowa
.5 .07363 m
.5 .07988 L
s
[(0.2)] .5 .06113 0 1 Mshowa
.7381 .07363 m
.7381 .07988 L
s
[(0.3)] .7381 .06113 0 1 Mshowa
.97619 .07363 m
.97619 .07988 L
s
[(0.4)] .97619 .06113 0 1 Mshowa
.125 Mabswid
.07143 .07363 m
.07143 .07738 L
s
.11905 .07363 m
.11905 .07738 L
s
.16667 .07363 m
.16667 .07738 L
s
.21429 .07363 m
.21429 .07738 L
s
.30952 .07363 m
.30952 .07738 L
s
.35714 .07363 m
.35714 .07738 L
s
.40476 .07363 m
.40476 .07738 L
s
.45238 .07363 m
.45238 .07738 L
s
.54762 .07363 m
.54762 .07738 L
s
.59524 .07363 m
.59524 .07738 L
s
.64286 .07363 m
.64286 .07738 L
s
.69048 .07363 m
.69048 .07738 L
s
.78571 .07363 m
.78571 .07738 L
s
.83333 .07363 m
.83333 .07738 L
s
.88095 .07363 m
.88095 .07738 L
s
.92857 .07363 m
.92857 .07738 L
s
.25 Mabswid
0 .07363 m
1 .07363 L
s
.02381 .18213 m
.03006 .18213 L
s
[(6.5)] .01131 .18213 1 0 Mshowa
.02381 .29064 m
.03006 .29064 L
s
[(7)] .01131 .29064 1 0 Mshowa
.02381 .39915 m
.03006 .39915 L
s
[(7.5)] .01131 .39915 1 0 Mshowa
.02381 .50766 m
.03006 .50766 L
s
[(8)] .01131 .50766 1 0 Mshowa
.02381 .61616 m
.03006 .61616 L
s
[(8.5)] .01131 .61616 1 0 Mshowa
.125 Mabswid
.02381 .09533 m
.02756 .09533 L
s
.02381 .11703 m
.02756 .11703 L
s
.02381 .13873 m
.02756 .13873 L
s
.02381 .16043 m
.02756 .16043 L
s
.02381 .20384 m
.02756 .20384 L
s
.02381 .22554 m
.02756 .22554 L
s
.02381 .24724 m
.02756 .24724 L
s
.02381 .26894 m
.02756 .26894 L
s
.02381 .31234 m
.02756 .31234 L
s
.02381 .33405 m
.02756 .33405 L
s
.02381 .35575 m
.02756 .35575 L
s
.02381 .37745 m
.02756 .37745 L
s
.02381 .42085 m
.02756 .42085 L
s
.02381 .44255 m
.02756 .44255 L
s
.02381 .46425 m
.02756 .46425 L
s
.02381 .48596 m
.02756 .48596 L
s
.02381 .52936 m
.02756 .52936 L
s
.02381 .55106 m
.02756 .55106 L
s
.02381 .57276 m
.02756 .57276 L
s
.02381 .59446 m
.02756 .59446 L
s
.02381 .05193 m
.02756 .05193 L
s
.02381 .03022 m
.02756 .03022 L
s
.02381 .00852 m
.02756 .00852 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 1 r
.001 Mabswid
.08622 0 m
.09078 .06527 L
.09323 .10163 L
.09381 .11042 L
.09443 .11971 L
.09495 .12767 L
.0951 .12986 L
.09525 .13189 L
.09539 .13016 L
.09553 .12858 L
.09772 .10177 L
.10008 .07203 L
.10121 .05756 L
.10183 .04956 L
.10213 .04564 L
.10226 .04392 L
.1024 .04266 L
.10256 .04499 L
.1027 .04719 L
.10303 .05207 L
.10371 .06244 L
.10491 .08079 L
.10983 .15683 L
.11456 .23117 L
.11693 .26865 L
.11797 .28508 L
.11856 .2944 L
.11871 .29673 L
.11885 .29894 L
.11897 .30092 L
.11911 .30129 L
.11925 .29938 L
.11939 .29736 L
.11966 .29372 L
.12024 .28558 L
.12144 .26889 L
.12395 .23334 L
.12509 .21709 L
.12539 .21279 L
.12572 .20809 L
.12586 .20594 L
.12602 .20365 L
.12617 .20148 L
.12631 .2031 L
.12746 .22088 L
.12852 .23712 L
.13335 .31098 L
.138 .3808 L
.14036 .41558 L
.14161 .43379 L
Mistroke
.14194 .43857 L
.1423 .44376 L
.14246 .44613 L
.14264 .44864 L
.14279 .45079 L
.14287 .45155 L
.14295 .45033 L
.14424 .43043 L
.14542 .41225 L
.14772 .37657 L
.1489 .35812 L
.14922 .35325 L
.14955 .34805 L
.14971 .34553 L
.14987 .34315 L
.15 .34111 L
.15015 .34319 L
.15122 .35798 L
.15238 .37401 L
.15732 .44006 L
.16208 .5006 L
.16447 .52979 L
.16551 .54228 L
.16606 .5488 L
.16636 .55236 L
.16651 .5541 L
.16665 .5557 L
.16681 .5536 L
.16698 .55077 L
.16727 .54585 L
.16785 .53626 L
.169 .51736 L
.17152 .47586 L
.17262 .45783 L
.17322 .44794 L
.17352 .44309 L
.17365 .44095 L
.17379 .4386 L
.17395 .4399 L
.17412 .44178 L
.17447 .44572 L
.17508 .45256 L
.17629 .46577 L
.18114 .51668 L
.18582 .56196 L
.18819 .58356 L
.18885 .5893 L
.18954 .59536 L
.18988 .59826 L
.19019 .60099 L
Mistroke
.19033 .60217 L
.19048 .60332 L
.19056 .602 L
.19064 .60056 L
.19079 .59802 L
.19558 .51677 L
.19615 .50708 L
.19678 .49656 L
.19707 .49176 L
.19738 .48666 L
.1975 .48457 L
.19764 .48283 L
.19779 .48404 L
.19793 .48518 L
.19925 .49576 L
.20045 .50515 L
.20541 .54138 L
.20788 .55797 L
.21019 .57254 L
.21241 .58581 L
.21305 .5895 L
.21365 .59292 L
.21392 .59439 L
.21406 .59519 L
.21419 .59594 L
.21434 .59552 L
.21442 .59419 L
.2145 .59277 L
.21478 .58793 L
.21716 .54741 L
.21842 .52623 L
.21976 .50384 L
.22043 .49267 L
.22076 .4872 L
.22106 .48219 L
.22121 .47982 L
.22134 .47762 L
.22148 .47643 L
.22163 .47725 L
.22225 .48049 L
.22455 .49221 L
.22925 .51389 L
.23164 .52387 L
.23424 .53395 L
.23546 .53839 L
.23674 .54293 L
.23727 .54475 L
.23755 .54569 L
.23784 .54667 L
.238 .54719 L
Mistroke
.23809 .54748 L
.23817 .5463 L
.23847 .54131 L
.23905 .53154 L
.24132 .49414 L
.24377 .45417 L
.24442 .44362 L
.24478 .43787 L
.24495 .43507 L
.24511 .43251 L
.24525 .43048 L
.24539 .43094 L
.2457 .43193 L
.24635 .43402 L
.24878 .44153 L
.25339 .45448 L
.25573 .46044 L
.25831 .46664 L
.25948 .46934 L
.26075 .47217 L
.26133 .47344 L
.26162 .47407 L
.2617 .47425 L
.26179 .47443 L
.26194 .47412 L
.26208 .47186 L
.26224 .46941 L
.26251 .46496 L
.26304 .45665 L
.26538 .41957 L
.26663 .40005 L
.26797 .37917 L
.26828 .37429 L
.26862 .36897 L
.26878 .36654 L
.26887 .36521 L
.26895 .36395 L
.2691 .36251 L
.26925 .36283 L
.27042 .36544 L
.27271 .37042 L
.27736 .38013 L
.27972 .38493 L
.28231 .39017 L
.2835 .39257 L
.28415 .3939 L
.28477 .39516 L
.28535 .39633 L
.28551 .39665 L
.28559 .39683 L
Mistroke
.28568 .39699 L
.28582 .3955 L
.28598 .39311 L
.28707 .37653 L
.2893 .34299 L
.29056 .32408 L
.29174 .30653 L
.29234 .29756 L
.29264 .29316 L
.29277 .29123 L
.29291 .28999 L
.29306 .29033 L
.29322 .29069 L
.29356 .29144 L
.29416 .29277 L
.2967 .29855 L
.30157 .31007 L
.30625 .32194 L
.30747 .3252 L
.30812 .32695 L
.30881 .32883 L
.30915 .32977 L
.30931 .33023 L
.30947 .33066 L
.30961 .32963 L
.30975 .32751 L
.31006 .32303 L
.31123 .30622 L
.31363 .27165 L
.31468 .25658 L
.31582 .2402 L
.31612 .23581 L
.31645 .23114 L
.31658 .22922 L
.31672 .22815 L
.31688 .22863 L
.31703 .22907 L
.31818 .23257 L
.3207 .24054 L
.32567 .25736 L
.32815 .26637 L
.33045 .27511 L
.3316 .27961 L
.33268 .28393 L
.33297 .28511 L
.33313 .28575 L
.33328 .28636 L
.33336 .28624 L
.33344 .28508 L
.33362 .28262 L
Mistroke
.33393 .2783 L
.33506 .26255 L
.33744 .22919 L
.3387 .21155 L
.33939 .20186 L
.33973 .19711 L
.34004 .19275 L
.34032 .18884 L
.34046 .18688 L
.34061 .18724 L
.34077 .1879 L
.34093 .18861 L
.34123 .18989 L
.34254 .19562 L
.34485 .20605 L
.34985 .23004 L
.35235 .24279 L
.35467 .25499 L
.35583 .26128 L
.3564 .2644 L
.35665 .26577 L
.35692 .26727 L
.35706 .26807 L
.3572 .26776 L
.35733 .266 L
.35746 .26411 L
.35805 .25605 L
.3593 .23887 L
.36171 .20556 L
.36298 .18796 L
.36367 .17832 L
.36401 .1736 L
.36416 .17154 L
.36424 .17038 L
.36432 .16999 L
.36447 .17082 L
.36462 .17169 L
.3649 .17331 L
.36552 .17689 L
.36684 .18459 L
.36916 .19842 L
.37371 .22678 L
.37624 .24318 L
.37856 .2586 L
.37974 .26658 L
.38032 .2705 L
.38057 .27222 L
.3807 .27312 L
.38084 .27408 L
.38099 .27437 L
Mistroke
.38112 .2725 L
.38139 .26881 L
.38198 .26069 L
.38323 .24344 L
.38562 .21028 L
.38667 .19571 L
.38722 .188 L
.38781 .17978 L
.38789 .17864 L
.38798 .1774 L
.38813 .17599 L
.38829 .17707 L
.38843 .17806 L
.38901 .18211 L
.39015 .19016 L
.39267 .20819 L
.39745 .24331 L
.40204 .27819 L
.40325 .28758 L
.40384 .29222 L
.4044 .29652 L
.40455 .29776 L
.4047 .29893 L
.40486 .29804 L
.40494 .29689 L
.40503 .29565 L
.40572 .28603 L
.40692 .26916 L
.4093 .23566 L
.41055 .21787 L
.41124 .20808 L
.41158 .20326 L
.41173 .20115 L
.41181 .20003 L
.41189 .19883 L
.41205 .19978 L
.4122 .20093 L
.41252 .20345 L
.41319 .20878 L
.41437 .21811 L
.41668 .23648 L
.42167 .2768 L
.42416 .29722 L
.42647 .31624 L
.42703 .32084 L
.42763 .3258 L
.4279 .32809 L
.42819 .33049 L
.42844 .33255 L
.42858 .33346 L
Mistroke
.42871 .33159 L
.42985 .31537 L
.43109 .29741 L
.43335 .26483 L
.43454 .24761 L
.43519 .23818 L
.43536 .23581 L
.43551 .23356 L
.43565 .23154 L
.4358 .23136 L
.43596 .23268 L
.43611 .2339 L
.43644 .23662 L
.43714 .24242 L
.43838 .25263 L
.44081 .27277 L
.44553 .31202 L
.44795 .33207 L
.44921 .34261 L
.45055 .35373 L
.45176 .3637 L
.45208 .36643 L
.45218 .36722 L
.45227 .36796 L
.45235 .36863 L
.45244 .36805 L
.45251 .36693 L
.4526 .3657 L
.45277 .36319 L
.45307 .3587 L
.45539 .32461 L
.45663 .30639 L
.45792 .28722 L
.45846 .27929 L
.45903 .27083 L
.4592 .26832 L
.45936 .26596 L
.45951 .26384 L
.45966 .26471 L
.45981 .26592 L
.45997 .2672 L
.46025 .26949 L
.46492 .30763 L
.46949 .34465 L
.47204 .36508 L
.47436 .3836 L
.47551 .39274 L
.47583 .39523 L
.476 .3966 L
.47609 .39727 L
Mistroke
.47617 .39788 L
.4763 .39638 L
.47645 .39415 L
.47677 .3894 L
.47905 .35515 L
.48025 .33721 L
.48152 .31803 L
.48214 .30872 L
.48273 .29993 L
.48298 .2961 L
.48312 .294 L
.48325 .29204 L
.4834 .29133 L
.48355 .29255 L
.48382 .29468 L
.48842 .33014 L
.49309 .36563 L
.49547 .38341 L
.49673 .39272 L
.49807 .40259 L
.49874 .40752 L
.49937 .41214 L
.49965 .41415 L
.4998 .41525 L
.49994 .41628 L
.50009 .41533 L
.50017 .4141 L
.50026 .41279 L
.50055 .40831 L
.50286 .37325 L
.50525 .33696 L
.5059 .32703 L
.5066 .31649 L
.50689 .31216 L
.50704 .30989 L
.50719 .30863 L
.50728 .30924 L
.50737 .3099 L
.50753 .31106 L
.50785 .31334 L
.51266 .34741 L
.51755 .38126 L
.51981 .39661 L
.52226 .41304 L
.52291 .41737 L
.52327 .41974 L
.52344 .42089 L
.5236 .42194 L
.52375 .4229 L
.52388 .42217 L
Mistroke
.52403 .41994 L
.52419 .41751 L
.52484 .40761 L
.52727 .37049 L
.52965 .33427 L
.53033 .32386 L
.53067 .31877 L
.53081 .31653 L
.53089 .31535 L
.53097 .31457 L
.53112 .31552 L
.53126 .31641 L
.53155 .31835 L
.53218 .32242 L
.53717 .35451 L
.54199 .38466 L
.54419 .39822 L
.54544 .40586 L
.54606 .40959 L
.54662 .41299 L
.5469 .41469 L
.54721 .41655 L
.54735 .4174 L
.5475 .4183 L
.54764 .41869 L
.54777 .41672 L
.54901 .39781 L
.55154 .3592 L
.55267 .34207 L
.55386 .324 L
.55416 .31944 L
.55449 .31456 L
.55458 .31312 L
.55467 .31176 L
.55475 .31054 L
.55484 .31088 L
.55492 .31138 L
.55501 .31191 L
.55517 .31286 L
.55637 .32003 L
.56093 .34684 L
.56577 .37484 L
.568 .38753 L
.56926 .39468 L
.56988 .39816 L
.57044 .40131 L
.57072 .40293 L
.57104 .40469 L
.57118 .4055 L
.57133 .40635 L
Mistroke
.57146 .40641 L
.5716 .40424 L
.57285 .38534 L
.5754 .3469 L
.5766 .32878 L
.57774 .31159 L
.57804 .307 L
.57837 .30212 L
.57847 .30067 L
.57856 .29931 L
.57864 .29944 L
.57872 .29993 L
.57906 .30178 L
.58026 .30852 L
.58521 .33601 L
.58997 .3622 L
.59237 .3753 L
.59342 .38101 L
.59456 .3872 L
.59486 .38885 L
.59503 .38976 L
.59519 .39061 L
.59532 .38966 L
.59546 .38751 L
.59577 .38294 L
.59691 .36578 L
.59944 .32804 L
.60069 .30931 L
.60186 .29192 L
.602 .28975 L
.60216 .28741 L
.60231 .28519 L
.60245 .28447 L
.60252 .28489 L
.60261 .28535 L
.60278 .28627 L
.60307 .2879 L
.6044 .29515 L
.60927 .32167 L
.61396 .34718 L
.61634 .36018 L
.6176 .36704 L
.61829 .37081 L
.61863 .37266 L
.61878 .37347 L
.61886 .3739 L
.61894 .37436 L
.61909 .37438 L
.61922 .3724 L
.61951 .36807 L
Mistroke
.62013 .35897 L
.62143 .33959 L
.62374 .30543 L
.62497 .28721 L
.62562 .27761 L
.62579 .27507 L
.62598 .27235 L
.62607 .27103 L
.62615 .26978 L
.62631 .26987 L
.62646 .27069 L
.6266 .27146 L
.62691 .27318 L
.62757 .27683 L
.62874 .28327 L
.63346 .30942 L
.63587 .32284 L
.63847 .33743 L
.6397 .34431 L
.64099 .35159 L
.6421 .35782 L
.6424 .35954 L
.64256 .36048 L
.64272 .36137 L
.64287 .36196 L
.64303 .35962 L
.64331 .35553 L
.64565 .3212 L
.64689 .30296 L
.64823 .28331 L
.64885 .27419 L
.6495 .26459 L
.64977 .26063 L
.64992 .25845 L
.65006 .25759 L
.65014 .25804 L
.65022 .25853 L
.65037 .25939 L
.65067 .26107 L
.65296 .27422 L
.6579 .30275 L
.66035 .31708 L
.66265 .33057 L
.66485 .34358 L
.66549 .34734 L
.66609 .35092 L
.66635 .35248 L
.6665 .35333 L
.66663 .35413 L
.66671 .35376 L
Mistroke
.66679 .35254 L
.66694 .35042 L
.66722 .34629 L
.66958 .31179 L
.67083 .29352 L
.67217 .27395 L
.67284 .26424 L
.67316 .25947 L
.67346 .25505 L
.6736 .25299 L
.67374 .25105 L
.67387 .25037 L
.67403 .25129 L
.67464 .25496 L
.67694 .26882 L
.68161 .29723 L
.68399 .3118 L
.68658 .32782 L
.68778 .33527 L
.68906 .34325 L
.68968 .34713 L
.68999 .34904 L
.69012 .34988 L
.69027 .3508 L
.6904 .35163 L
.69053 .35137 L
.69066 .34947 L
.6908 .34743 L
.69137 .33913 L
.69375 .30434 L
.69501 .28596 L
.69635 .26641 L
.69692 .25808 L
.69722 .2537 L
.69738 .25132 L
.69754 .24909 L
.69762 .2479 L
.6977 .24841 L
.69789 .24956 L
.69821 .25159 L
.69884 .25559 L
.70115 .27018 L
.70615 .30207 L
.70866 .31819 L
.71097 .33313 L
.71322 .34775 L
.71377 .35131 L
.71393 .35234 L
.71408 .35331 L
.71421 .35419 L
Mistroke
.71436 .35361 L
.71444 .35237 L
.71452 .35124 L
.71469 .34872 L
.715 .34419 L
.71561 .33538 L
.71813 .29859 L
.71923 .28244 L
.72044 .26471 L
.72101 .25644 L
.72117 .25406 L
.72132 .2518 L
.72146 .25051 L
.72161 .25149 L
.72271 .25864 L
.7251 .27431 L
.73002 .30679 L
.73246 .32301 L
.73475 .33828 L
.73695 .35292 L
.73755 .35694 L
.73785 .35896 L
.73793 .35951 L
.73802 .3601 L
.73818 .35935 L
.73833 .35721 L
.73849 .35489 L
.73877 .35068 L
.73931 .3428 L
.74166 .30833 L
.74291 .29003 L
.7436 .27994 L
.74425 .27037 L
.74486 .26134 L
.74503 .25887 L
.74512 .25751 L
.74521 .25623 L
.74529 .25618 L
.74538 .25677 L
.74553 .25779 L
.74671 .26565 L
.749 .28106 L
.75396 .31438 L
.75643 .33103 L
.75873 .34657 L
.75987 .35427 L
.76095 .36155 L
.76154 .36558 L
.7617 .36668 L
.76179 .36728 L
Mistroke
.76188 .36785 L
.76202 .36628 L
.76219 .36389 L
.76332 .34725 L
.76555 .31433 L
.76799 .27835 L
.76863 .26892 L
.76882 .2662 L
.76891 .26487 L
.76899 .26364 L
.76914 .26343 L
.76923 .26398 L
.76931 .26457 L
.77054 .27285 L
.77296 .2892 L
.77754 .32011 L
.78009 .33731 L
.78242 .353 L
.78357 .36077 L
.78418 .36487 L
.78482 .36923 L
.78539 .37303 L
.78554 .37406 L
.78562 .37461 L
.78571 .37519 L
.78587 .37299 L
.78601 .37086 L
.78711 .35463 L
.78951 .319 L
.7917 .28661 L
.79228 .27797 L
.79259 .27342 L
.79267 .27222 L
.79276 .27093 L
.79292 .2699 L
.79307 .27089 L
.7932 .27181 L
.79351 .27391 L
.79407 .2776 L
.79659 .29455 L
.80139 .32663 L
.806 .35737 L
.80722 .36552 L
.80837 .37318 L
.80901 .37741 L
.80919 .37857 L
.80937 .3798 L
.80953 .38073 L
.80962 .37941 L
.8097 .37817 L
Mistroke
.8109 .3603 L
.81316 .32685 L
.8156 .29052 L
.81625 .28091 L
.81641 .27858 L
.81658 .27606 L
.81667 .27475 L
.81676 .27533 L
.81694 .27649 L
.81817 .28463 L
.8206 .30065 L
.82538 .3321 L
.82998 .36219 L
.8312 .37014 L
.83235 .37763 L
.83299 .38176 L
.83316 .3829 L
.83326 .38352 L
.83334 .38386 L
.8335 .38151 L
.83367 .37894 L
.83488 .36102 L
.83712 .32762 L
.83831 .31 L
.83957 .29126 L
.83987 .28675 L
.8402 .28188 L
.84029 .28052 L
.84038 .27909 L
.84047 .27787 L
.84056 .27822 L
.84064 .27877 L
.84073 .27935 L
.84089 .28041 L
.84212 .28837 L
.84455 .30404 L
.84955 .33621 L
.85205 .35225 L
.85436 .36702 L
.85553 .37443 L
.85661 .38136 L
.8569 .38314 L
.85703 .38402 L
.85716 .38444 L
.85731 .38225 L
.85747 .37986 L
.85775 .37568 L
.859 .35706 L
.86127 .32328 L
.86246 .30553 L
Mistroke
.86372 .28678 L
.86403 .28217 L
.86412 .28084 L
.8642 .27959 L
.86436 .27883 L
.86445 .27941 L
.86454 .27994 L
.86473 .28114 L
.86507 .2833 L
.8663 .29115 L
.86874 .30654 L
.87347 .3364 L
.87803 .36498 L
.87916 .37209 L
.88036 .37958 L
.88066 .38147 L
.88075 .38201 L
.88083 .38253 L
.88099 .38274 L
.88108 .38142 L
.88116 .3802 L
.88134 .37745 L
.88167 .37253 L
.88288 .35462 L
.88533 .31808 L
.88653 .30022 L
.88763 .28392 L
.88789 .28002 L
.88804 .27785 L
.88817 .27748 L
.88833 .27846 L
.88848 .27939 L
.88876 .28115 L
.88983 .28785 L
.8922 .30258 L
.89686 .3315 L
.89922 .34615 L
.90181 .36217 L
.903 .36954 L
.90366 .37359 L
.90398 .37558 L
.90428 .37742 L
.90456 .37915 L
.9047 .38002 L
.90486 .379 L
.90501 .37668 L
.90518 .37419 L
.90548 .36968 L
.90658 .35338 L
.90896 .3181 L
Mistroke
.91021 .29947 L
.9109 .28924 L
.91124 .28422 L
.91155 .2796 L
.91171 .27727 L
.91185 .27509 L
.91201 .27495 L
.91209 .27545 L
.91217 .27598 L
.91285 .28016 L
.91403 .28744 L
.91634 .30166 L
.92132 .33236 L
.92382 .34771 L
.92612 .36191 L
.92728 .36901 L
.92784 .37249 L
.92809 .37401 L
.92823 .37488 L
.92836 .37568 L
.92849 .37647 L
.92863 .37611 L
.92877 .37394 L
.92891 .37197 L
.92949 .36327 L
.93074 .34475 L
.93325 .3076 L
.93435 .29129 L
.93494 .28264 L
.93526 .27787 L
.93542 .27552 L
.93556 .27334 L
.9357 .27126 L
.93585 .27197 L
.93617 .27392 L
.93673 .27733 L
.93781 .28403 L
.9402 .29871 L
.94522 .3296 L
.94774 .34512 L
.95005 .35939 L
.95123 .36661 L
.9518 .37013 L
.95205 .37167 L
.95218 .37248 L
.95232 .37334 L
.95246 .37249 L
.9526 .37049 L
.95287 .36653 L
.95346 .35781 L
Mistroke
.95471 .33932 L
.95709 .30415 L
.95813 .28876 L
.95868 .28062 L
.95926 .27194 L
.95942 .26958 L
.95957 .2684 L
.95971 .26923 L
.95985 .27013 L
.9604 .27352 L
.9616 .28092 L
.96412 .29647 L
.96905 .32704 L
.97381 .35653 L
.97619 .37131 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  Background->GrayLevel[1],
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005X0000gR000`400?l00000o`00003oo`3ooomY
0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`006P3oool00`000000oooo0?oo
o`0B0?ooo`030?l0o`3oool0oooo0?l0oooo=`3oool001X0oooo00<000000?ooo`3oool04P3oool0
0`3o0?l0oooo0?ooo`3o0?ooocL0oooo000J0?ooo`030000003oool0oooo0180oooo00<0o`3o0?oo
o`3oool0o`3ooolg0?ooo`006P3oool2000001<0oooo00<0o`3o0?ooo`3oool0o`3ooolg0?ooo`00
6P3oool00`000000oooo0?ooo`0B0?ooo`030?l0o`3oool0oooo0?l0oooo=`3oool001X0oooo00<0
00000?ooo`3oool04P3oool00`3o0?l0oooo0?ooo`3o0?ooocL0oooo000J0?ooo`030000003oool0
oooo01<0oooo00<0o`3o0?ooo`3oool0o`3ooolf0?ooo`006P3oool00`000000oooo0?ooo`0C0?oo
o`030?l0o`3oool0oooo0?l0oooo=P3oool001X0oooo00<000000?ooo`3oool04`3oool00`3o0?l0
oooo0?ooo`3o0?ooocH0oooo000J0?ooo`030000003oool0oooo01<0oooo00<0o`3o0?ooo`3oool0
o`3ooolf0?ooo`006P3oool2000001@0oooo00<0o`3o0?ooo`3oool0;@3oool4000000D0oooo0P00
00050?ooo`D00000?03oool4000000D0oooo0P0000040?ooo`D00000?03oool4000000D0oooo0P00
00050?ooo`<00000?@3oool4000000D0oooo0P0000070?ooo`<000000P3oool001X0oooo00<00000
0?ooo`3oool04`3oool00`3o0?l0oooo0?ooo`0/0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool00P3oool2000000L0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0080oooo0P0000040?ooo`050000003oool0oooo0?ooo`000000>`3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo0P0000040?ooo`050000003o
ool0oooo0?ooo`000000>`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo
0P0000080?ooo`030000003oool0oooo0040oooo000J0?ooo`030000003oool0oooo01<0oooo00<0
o`3o0?ooo`3oool0;03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool0>`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0?03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool0>@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool00@3oool001X0oooo00<000000?ooo`3oool04`3oool00`3o0?l0oooo0?ooo`0/
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0k
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0k
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0i
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0203oool600000080oooo000J0?oo
o`030000003oool0oooo01<0oooo00<0o`3o0?ooo`3oool0;03oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00X0oooo0P00000l0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool02@3oool010000000oooo0?ooo`0000030?ooo`006P3oool00`000000oooo0?ooo`0C0?oo
o`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3oool09`3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00T0oooo00@000000?ooo`3oool000000`3oool001X0oooo00<000000?oo
o`3oool04`3oool00`3o0?l0oooo0?ooo`020?ooo`030?l0o`3oool0oooo02L0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`090?ooo`<00000?@3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00P0oooo00D000000?ooo`3oool0oooo0000000k0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0203oool01@000000oooo0?ooo`3oool0000003/0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0000000<0oooo000J
0?ooo`030000003oool0oooo01@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`0Z0?ooo`@000003P3o
ool00`000000oooo0?ooo`0l0?ooo`@00000303oool3000003d0oooo1000000<0?ooo`<00000?@3o
ool4000000h0oooo0P0000030?ooo`006P3oool2000001D0oooo00D0o`3o0?ooo`3oool0oooo0?l0
o`3o0?oooc<0oooo000J0?ooo`030000003oool0oooo01@0oooo00<0o`3o0?ooo`3oool00P3o0?oo
0?oooc<0oooo000J0?ooo`030000003oool0oooo01@0oooo00<0o`3o0?ooo`3oool00P3o0?oo0?oo
oc<0oooo000J0?ooo`030000003oool0oooo01@0oooo00<0o`3o0?ooo`3oool00P3o0?oo0?oooc<0
oooo000J0?ooo`030000003oool0oooo01@0oooo00<0o`3o0?ooo`3oool00P3o0?oo0?oooc<0oooo
000J0?ooo`030000003oool0oooo01@0oooo00<0o`3o0?ooo`3oool00P3o0?oo0?oooc<0oooo000J
0?ooo`030000003oool0oooo01@0oooo00<0o`3o0?ooo`3oool00P3o0?oo0?oooc<0oooo000B0?oo
oal0000000H0o`3o000000000000o`3o0000003o0?oo000002h00000103oool001X0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`040?ooo`060?l0o`3oool0oooo0?l0o`3oool0o`3o
0`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0:0?ooo`006P3oool00`000000oooo0?ooo`0D0?ooo`060?l0o`3o
ool0oooo0?l0o`3oool0o`3o<`3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo04d0
oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`0:0?ooo`006P3oool00`000000oooo
0?ooo`0D0?ooo`060?l0o`3oool0oooo0?l0o`3oool0o`3oo`3ooolb0?ooo`006P3oool00`000000
oooo0?ooo`0E0?ooo`80o`3o0P3oool00`3o0?l0oooo0?ooo`3o0?oooc00oooo000J0?ooo`030000
003oool0oooo01D0oooo0P3o0?l20?ooo`030?l0o`3oool0oooo0?l0oooo<03oool001X0oooo00<0
00000?ooo`3oool05@3oool20?l0o`80oooo00<0o`3o0?ooo`3oool0o`3oool`0?ooo`006P3oool2
000001H0oooo0P3o0?l20?ooo`030?l0o`3oool0oooo0?l0oooo<03oool001X0oooo00<000000?oo
o`3oool05@3oool20?l0o`80oooo00<0o`3o0?ooo`3oool0o`3oool`0?ooo`006P3oool00`000000
oooo0?ooo`0E0?ooo`80o`3o0P3oool00`3o0?l0oooo0?ooo`3o0?oooc00oooo000J0?ooo`030000
003oool0oooo01D0oooo0P3o0?l20?ooo`030?l0o`3oool0oooo0?l0oooo<03oool001X0oooo00<0
00000?ooo`3oool05@3oool20?l0o`80oooo00<0o`3o0?ooo`3oool0o`3oool`0?ooo`006P3oool0
0`000000oooo0?ooo`0E0?ooo`80o`3o0P3oool00`3o0?l0oooo0?ooo`3o0?oooc00oooo000J0?oo
o`030000003oool0oooo01D0oooo0P3o0?l20?ooo`030?l0o`3oool0oooo0?l0oooo<03oool001X0
oooo00<000000?ooo`3oool05@3oool01@3o0?l0oooo0?ooo`3oool0o`3o0?l0oooo<P3oool001X0
oooo0P00000F0?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0o`3ooolb0?ooo`006P3oool00`000000
oooo0?ooo`0E0?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3oool0o`3oool_0?ooo`00
6P3oool00`000000oooo0?ooo`0E0?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3oool0
o`3oool_0?ooo`006P3oool00`000000oooo0?ooo`0E0?ooo`030?l0o`3oool0oooo0080oooo00<0
o`3o0?ooo`3oool0o`3oool_0?ooo`006P3oool00`000000oooo0?ooo`0E0?ooo`030?l0o`3oool0
oooo0080oooo00<0o`3o0?ooo`3oool0o`3oool_0?ooo`006P3oool00`000000oooo0?ooo`0E0?oo
o`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3oool0o`3oool_0?ooo`006P3oool00`000000
oooo0?ooo`0J0?ooo`030?l0o`3oool0oooo0?l0oooo;`3oool001X0oooo0P00000K0?ooo`030?l0
o`3oool0oooo0?l0oooo;`3oool001X0oooo00<000000?ooo`3oool06P3oool00`3o0?l0oooo0?oo
o`3o0?ooobl0oooo000J0?ooo`030000003oool0oooo01X0oooo00<0o`3o0?ooo`3oool0o`3oool_
0?ooo`006P3oool00`000000oooo0?ooo`0J0?ooo`030?l0o`3oool0oooo0?l0oooo;`3oool001X0
oooo00<000000?ooo`3oool06P3oool00`3o0?l0oooo0?ooo`3o0?ooobl0oooo000J0?ooo`030000
003oool0oooo01X0oooo00<0o`3o0?ooo`3oool0o`3oool_0?ooo`006P3oool00`000000oooo0?oo
o`0J0?ooo`030?l0o`3oool0oooo0?l0oooo;`3oool001X0oooo0P00000K0?ooo`030?l0o`3oool0
oooo0?l0oooo;`3oool001X0oooo00<000000?ooo`3oool06P3oool00`3o0?l0oooo0?ooo`3o0?oo
obl0oooo000J0?ooo`030000003oool0oooo01X0oooo00<0o`3o0?ooo`3oool0o`3oool_0?ooo`00
6P3oool00`000000oooo0?ooo`0J0?ooo`030?l0o`3oool0oooo05<0oooo00<0o`3o0?ooo`3oool0
f03oool001X0oooo00<000000?ooo`3oool06P3oool00`3o0?l0oooo0?ooo`1C0?ooo`030?l0o`3o
ool0oooo0=P0oooo0000103oool000000000000000050?ooo`8000001@3oool4000000H0oooo00<0
00000?ooo`3oool06P3oool00`3o0?l0oooo0?ooo`1C0?ooo`030?l0o`3oool0oooo00D0oooo00<0
o`3o0?ooo`3oool0d03oool000050000003oool0oooo0?ooo`000000103oool2000000@0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01/0oooo00<0
o`3o0?ooo`3oool0DP3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo0=00oooo0000
1@000000oooo0?ooo`3oool0000000l0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0K0?ooo`030?l0o`3oool0oooo0580oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?oo
o`3@0?ooo`0000D000000?ooo`3oool0oooo0000000?0?ooo`030000003oool0oooo00<0oooo0`00
000K0?ooo`030?l0o`3oool0oooo0580oooo0P3o0?l60?ooo`030?l0o`3oool0oooo0=00oooo0004
000000`0oooo100000060?ooo`030000003oool0oooo01/0oooo00<0o`3o0?ooo`3oool0BP3oool0
0`3o0?l0oooo0?ooo`050?ooo`80o`3o1P3oool00`3o0?l0oooo0?ooo`3@0?ooo`0000<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01/0oooo00<0o`3o0?oo
o`3oool0BP3oool00`3o0?l0oooo0?ooo`050?ooo`80o`3o1P3oool20?l0om40oooo00000`3oool0
00000?ooo`0=0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool06`3oool00`3o0?l0
oooo0?ooo`1:0?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3o0?l01@3oool20?l0om40
oooo00020?ooo`<000002`3oool5000000D0oooo00<000000?ooo`3oool06`3oool00`3o0?l0oooo
0?ooo`1:0?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`050?ooo`80o`3od@3oool001X0oooo00<0
00000?ooo`3oool06`3oool00`3o0?l0oooo0?ooo`1:0?ooo`80o`3o1@3oool0103o0?l0oooo0?oo
o`3o0?l50?ooo`80o`3o1P3oool00`3o0?l0oooo0?ooo`380?ooo`006P3oool00`000000oooo0?oo
o`0K0?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3oool0A@3oool20?l0o`D0oooo00@0
o`3o0?ooo`3oool0o`3o1@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0oooo0<P0oooo
000J0?ooo`800000703oool00`3o0?l0oooo0?ooo`020?ooo`030?l0o`3oool0oooo04D0oooo0P3o
0?l50?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o1@3oool00`3o0?l0
oooo0?ooo`380?ooo`006P3oool00`000000oooo0?ooo`0K0?ooo`030?l0o`3oool0oooo0080oooo
00<0o`3o0?ooo`3oool0A03oool00`3o0?l0oooo0?l0o`050?ooo`040?l0o`3oool0oooo0?l0o`@0
oooo00@0o`3o0?ooo`3oool0o`3o1@3oool00`3o0?l0oooo0?ooo`380?ooo`006P3oool00`000000
oooo0?ooo`0K0?ooo`040?l0o`3oool0oooo0?ooo`80o`3oAP3oool00`3o0?l0oooo0?l0o`050?oo
o`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o1@3oool00`3o0?l0oooo0?oo
o`380?ooo`006P3oool00`000000oooo0?ooo`0L0?ooo`030?l0o`3oool0oooo0080o`3oAP3oool0
0`3o0?l0oooo0?l0o`050?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o
1@3oool00`3o0?l0oooo0?ooo`380?ooo`006P3oool00`000000oooo0?ooo`0L0?ooo`030?l0o`3o
ool0oooo0080o`3oAP3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`050?l0o`3oool0oooo0?ooo`3o
0?l00`3oool0103o0?l0oooo0?ooo`3o0?l50?ooo`80o`3ob@3oool001X0oooo00<000000?ooo`3o
ool0703oool00`3o0?l0oooo0?ooo`020?l0odH0oooo00@0o`3o0?ooo`3oool0o`3o103oool01@3o
0?l0oooo0?ooo`3oool0o`3o00<0oooo00@0o`3o0?ooo`3oool0o`3o1@3oool20?l0olT0oooo000J
0?ooo`030000003oool0oooo01`0oooo00<0o`3o0?ooo`3oool00P3o0?m60?ooo`040?l0o`3oool0
oooo0?l0o`<0oooo00<0o`3o0?ooo`3oool00P3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo
00<0o`3o0?ooo`3oool00`3oool20?l0olT0oooo000J0?ooo`8000007@3oool00`3o0?l0oooo0?oo
o`020?l0odH0oooo00@0o`3o0?ooo`3oool0o`3o0`3oool00`3o0?l0oooo0?ooo`020?ooo`050?l0
o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`030?ooo`80o`3ob@3oool001X0oooo
00<000000?ooo`3oool0703oool00`3o0?l0oooo0?ooo`020?l0ocl0oooo00<0o`3o0?ooo`3oool0
103oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0
oooo0?ooo`3o0?l40?ooo`070?l0o`3oool0oooo0?ooo`3o0?l0oooo0?l0o`390?ooo`006P3oool0
0`000000oooo0?ooo`0L0?ooo`060?l0o`3oool0oooo0?l0o`3oool0o`3o?P3oool00`3o0?l0oooo
0?ooo`040?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`020?ooo`04
0?l0o`3oool0oooo0?l0o`@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0
oooo00<0oooo00<0o`3o0?ooo`3oool0`03oool001X0oooo00<000000?ooo`3oool0703oool01P3o
0?l0oooo0?ooo`3o0?l0oooo0?l0och0oooo00<0o`3o0?ooo`3oool0103oool01@3o0?l0oooo0?oo
o`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`05
0?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`030?ooo`030?l0o`3oool0oooo
0<00oooo000J0?ooo`030000003oool0oooo01`0oooo00H0o`3o0?ooo`3oool0o`3o0?ooo`3o0?ln
0?ooo`030?l0o`3oool0oooo00@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3o
ool0oooo00<0oooo00<0o`3o0?ooo`3o0?l0103oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo
00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?ooo`300?ooo`006P3oool00`000000oooo0?oo
o`0L0?ooo`060?l0o`3oool0oooo0?l0o`3oool0o`3o?P3oool20?l0o`D0oooo00<0o`3o0?ooo`3o
ool00P3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00@0oooo00D0o`3o0?ooo`3o
ool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3oool0`03oool001X0
oooo00<000000?ooo`3oool0703oool01P3o0?l0oooo0?ooo`3o0?l0oooo0?l0och0oooo0P3o0?l5
0?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`04
0?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`030?ooo`80o`3o`@3o
ool001X0oooo0P00000M0?ooo`060?l0o`3oool0oooo0?l0o`3oool0o`3o?P3oool00`3o0?l0oooo
0?l0o`030?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo
0?l0o`040?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`030?ooo`80
o`3o`@3oool001X0oooo00<000000?ooo`3oool0703oool01P3o0?l0oooo0?ooo`3o0?l0oooo0?l0
ocd0oooo00@0o`3o0?ooo`3oool0o`3o0`3oool00`3o0?l0oooo0?ooo`030?ooo`030?l0o`3oool0
o`3o00D0oooo00<0o`3o0?ooo`3o0?l0103oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0
o`3o0?ooo`3oool00`3oool20?l0oeH0oooo00<0o`3o0?ooo`3oool0J03oool001X0oooo00<00000
0?ooo`3oool07@3oool20?l0o`80oooo00<0o`3o0?ooo`3oool0>`3oool0103o0?l0oooo0?ooo`3o
0?l30?ooo`030?l0o`3oool0oooo00@0oooo0P3o0?l60?ooo`80o`3o103oool01@3o0?l0oooo0?oo
o`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00`3oool20?l0odh0oooo00<0o`3o0?ooo`3oool0
1@3oool20?l0o`H0oooo00<0o`3o0?ooo`3oool0H03oool001X0oooo00<000000?ooo`3oool07@3o
ool20?l0o`80oooo00<0o`3o0?ooo`3oool0>`3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo
00<0o`3o0?ooo`3oool0103oool20?l0o`H0oooo0P3o0?l50?ooo`030?l0o`3oool0o`3o00<0oooo
00<0o`3o0?ooo`3oool00`3oool20?l0odh0oooo0P3o0?l60?ooo`80o`3o1P3oool00`3o0?l0oooo
0?ooo`1P0?ooo`006P3oool00`000000oooo0?ooo`0M0?ooo`80o`3o0P3oool00`3o0?l0oooo0?oo
o`0k0?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`040?ooo`80o`3o
1P3oool20?l0o`D0oooo00<0o`3o0?ooo`3o0?l0103oool01`3o0?l0oooo0?ooo`3oool0o`3o0?oo
o`3o0?l0AP3oool00`3o0?l0oooo0?ooo`050?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?oo
o`030?l0o`3oool0oooo05P0oooo000J0?ooo`030000003oool0oooo01d0oooo0P3o0?l20?ooo`03
0?l0o`3oool0oooo03/0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo
00@0oooo0P3o0?l60?ooo`80o`3o1@3oool00`3o0?l0oooo0?l0o`040?ooo`070?l0o`3oool0oooo
0?ooo`3o0?l0oooo0?l0o`160?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3o
ool00`3o0?l0oooo0?ooo`1H0?ooo`006P3oool00`000000oooo0?ooo`0M0?ooo`80o`3o0P3oool0
0`3o0?l0oooo0?ooo`0k0?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3o0?l01P3oool2
0?l0o`H0oooo0P3o0?l50?ooo`030?l0o`3oool0o`3o00@0oooo00D0o`3o0?ooo`3oool0oooo0?l0
o`020?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3oool0?@3oool20?l0o`H0oooo0P3o
0?l60?ooo`030?l0o`3oool0o`3o00D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0oe40oooo000J
0?ooo`030000003oool0oooo01d0oooo0P3o0?l20?ooo`030?l0o`3oool0oooo03/0oooo00<0o`3o
0?ooo`3oool00P3oool00`3o0?l0oooo0?l0o`060?ooo`80o`3o1P3oool20?l0o`D0oooo00<0o`3o
0?ooo`3o0?l0103oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00`3o
ool00`3o0?l0oooo0?ooo`0m0?ooo`80o`3o1P3oool20?l0o`H0oooo00<0o`3o0?ooo`3o0?l01@3o
ool20?l0o`H0oooo0P3o0?l60?ooo`80o`3oD@3oool001X0oooo0P00000N0?ooo`80o`3o0P3oool0
0`3o0?l0oooo0?ooo`0k0?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3o0?l01`3oool0
0`3o0?l0oooo0?ooo`040?ooo`80o`3o1@3oool00`3o0?l0oooo0?l0o`040?ooo`050?l0o`3oool0
oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`030?ooo`030?l0o`3oool0oooo03D0oooo00<0
o`3o0?ooo`3oool01@3oool20?l0o`H0oooo0P3o0?l60?ooo`030?l0o`3oool0o`3o00D0oooo00<0
o`3o0?ooo`3o0?l01@3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o=`3oool00`3o0?l0oooo0?oo
o`0?0?ooo`006P3oool00`000000oooo0?ooo`0M0?ooo`80o`3o0P3oool00`3o0?l0oooo0?ooo`0k
0?ooo`030?l0o`3oool0oooo00<0oooo0P3o0?l>0?ooo`80o`3o1P3oool20?l0o`@0oooo00D0o`3o
0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3oool0=@3o
ool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3o
ool0o`3o00D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0obl0oooo00<0o`3o0?ooo`3oool01@3o
ool00`3o0?l0oooo0?ooo`0?0?ooo`006P3oool00`000000oooo0?ooo`0M0?ooo`80o`3o0`3oool0
0`3o0?l0oooo0?ooo`0j0?ooo`030?l0o`3oool0oooo00<0oooo0P3o0?l?0?ooo`030?l0o`3oool0
oooo00@0oooo0P3o0?l40?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?oo
o`030?ooo`030?l0o`3oool0oooo03D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo00<0
o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo0P3o
0?l60?ooo`80o`3o1P3oool20?l0oal0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?oo
o`050?ooo`030?l0o`3oool0oooo00l0oooo000J0?ooo`030000003oool0oooo01d0oooo0P3o0?l3
0?ooo`030?l0o`3oool0oooo03X0oooo00<0o`3o0?ooo`3oool00`3oool20?l0oaH0oooo0P3o0?l4
0?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`030?ooo`030?l0o`3o
ool0oooo03D0oooo0P3o0?l60?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`040?ooo`040?l0o`3o
ool0oooo0?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`80o`3o
1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1`3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0
o`3oool0oooo00@0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0oa40oooo000B0?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool07@3oool20?l0o`<0oooo00<0o`3o0?ooo`3oool0>P3o
ool00`3o0?l0oooo0?ooo`030?ooo`80o`3o5P3oool20?l0o`@0oooo00D0o`3o0?ooo`3oool0oooo
0?l0o`020?ooo`030?l0o`3oool0oooo00<0oooo0P3o0?lf0?ooo`80o`3o1P3oool20?l0o`H0oooo
00<0o`3o0?ooo`3o0?l0103oool0103o0?l0oooo0?ooo`3o0?l50?ooo`030?l0o`3oool0o`3o00D0
oooo00<0o`3o0?ooo`3o0?l01@3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0
oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o4@3oool00180oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0M0?ooo`80o`3o0`3oool00`3o0?l0oooo
0?ooo`0j0?ooo`030?l0o`3oool0oooo00<0oooo0P3o0?lF0?ooo`80o`3o103oool01@3o0?l0oooo
0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00`3oool20?l0obh0oooo00<0o`3o0?ooo`3o
ool01@3oool20?l0o`H0oooo00<0o`3o0?ooo`3o0?l0103oool0103o0?l0oooo0?ooo`3o0?l40?oo
o`040?l0o`3oool0oooo0?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`05
0?ooo`030?l0o`3oool0o`3o00D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l6
0?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o4@3oool00180oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0M0?ooo`80o`3o0`3oool00`3o0?l0oooo0?ooo`0j0?oo
o`030?l0o`3oool0oooo00@0oooo00<0o`3o0?ooo`3oool0503oool20?l0o`D0oooo00@0o`3o0?oo
o`3oool0o`3o0P3oool00`3o0?l0oooo0?ooo`030?ooo`80o`3o;P3oool00`3o0?l0oooo0?ooo`05
0?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00D0
o`3o0?ooo`3oool0oooo0?l0o`040?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l0
1@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo0P3o0?l60?ooo`80o`3o
1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?lA0?ooo`004`3o
ool00`000000oooo0?ooo`040?ooo`<000007@3oool20?l0o`<0oooo00<0o`3o0?ooo`3oool0<`3o
ool00`3o0?l0oooo0?ooo`030?ooo`030?l0o`3oool0oooo01`0oooo0P3o0?l50?ooo`040?l0o`3o
ool0oooo0?l0o`80oooo00<0o`3o0?ooo`3oool00`3oool20?l0o`H0oooo00<0o`3o0?ooo`3oool0
9@3oool00`3o0?l0oooo0?ooo`050?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`040?ooo`050?l0
o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00@0o`3o0?oo
o`3oool0o`3o1@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o
0?ooo`3o0?l01@3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l6
0?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`0@0?ooo`004`3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo01d0oooo0P3o0?l30?ooo`030?l0o`3oool0oooo03<0oooo00<0o`3o0?oo
o`3oool00`3oool00`3o0?l0oooo0?ooo`0L0?ooo`80o`3o1@3oool0103o0?l0oooo0?ooo`3o0?l3
0?ooo`030?l0o`3oool0oooo0080oooo0P3o0?l60?ooo`030?l0o`3oool0oooo02D0oooo0P3o0?l6
0?ooo`030?l0o`3oool0o`3o00@0oooo00@0o`3o0?ooo`3oool0o`3o103oool01@3o0?l0oooo0?oo
o`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?oo
o`3o0?l0103oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?oo
o`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?oo
o`80o`3o1P3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o0100oooo000D0?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool07@3oool00`3o0?l0oooo0?ooo`020?ooo`03
0?l0o`3oool0oooo03<0oooo00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?ooo`0M0?ooo`03
0?l0o`3oool0oooo00<0oooo00@0o`3o0?ooo`3oool0o`3o0`3oool01`3o0?l0oooo0?ooo`3oool0
o`3o0?ooo`3o0?l01P3oool00`3o0?l0oooo0?ooo`0U0?ooo`80o`3o1@3oool0103o0?l0oooo0?oo
o`3o0?l40?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o
00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0103o
ool00`3o0?l0oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0o`D0oooo00<0o`3o0?ooo`3o0?l0
1@3oool00`3o0?l0oooo0?l0o`050?ooo`80o`3o1P3oool20?l0o`H0oooo00<0o`3o0?ooo`3o0?l0
1@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l0
403oool000l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo01d0oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0oooo0?ooo`0c0?ooo`80o`3o103o
ool00`3o0?l0oooo0?ooo`0M0?ooo`030?l0o`3oool0oooo00<0oooo00@0o`3o0?ooo`3oool0o`3o
0`3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00`3oool00`3o0?l0
oooo0?ooo`0M0?ooo`030?l0o`3oool0oooo00D0oooo0P3o0?l50?ooo`040?l0o`3oool0oooo0?l0
o`@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3o
ool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`05
0?l0o`3oool0oooo0?ooo`3o0?l00`3oool0103o0?l0oooo0?ooo`3o0?l50?ooo`030?l0o`3oool0
o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0
o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0
o`3o00D0oooo00<0o`3o0?ooo`3o0?l0403oool000l0oooo1P0000050?ooo`030000003oool0oooo
01d0oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0oooo0?ooo`0c0?ooo`80o`3o103oool00`3o
0?l0oooo0?ooo`0S0?ooo`040?l0o`3oool0oooo0?l0o`<0oooo00D0o`3o0?ooo`3oool0oooo0?l0
o`020?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3oool07@3oool00`3o0?l0oooo0?oo
o`050?ooo`80o`3o1@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`050?l0o`3oool0oooo0?ooo`3o
0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`03
0?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo
00@0o`3o0?ooo`3oool0o`3o1@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0
oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0
oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o0100
oooo000J0?ooo`030000003oool0oooo0280oooo00<0o`3o0?ooo`3oool0<`3oool00`3o0?l0oooo
0?l0o`030?ooo`030?l0o`3oool0oooo02<0oooo00<0o`3o0?ooo`3o0?l0103oool01@3o0?l0oooo
0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00`3oool20?l0oah0oooo00<0o`3o0?ooo`3o
ool01@3oool00`3o0?l0oooo0?l0o`040?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o
0?l0oooo0?ooo`3oool0o`3o00<0oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o
0?l30?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0
oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo0080oooo00@0o`3o0?oo
o`3oool0o`3o1@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o
0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o
0?ooo`3o0?l01@3oool0103o0?l0oooo0?ooo`3o0?l?0?ooo`006P3oool00`000000oooo0?ooo`0R
0?ooo`030?l0o`3oool0oooo03<0oooo00<0o`3o0?ooo`3o0?l00`3oool00`3o0?l0oooo0?ooo`0T
0?ooo`80o`3o103oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00`3o
ool20?l0o`H0oooo00<0o`3o0?ooo`3oool05@3oool20?l0o`H0oooo00<0o`3o0?ooo`3o0?l0103o
ool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`03
0?l0o`3oool0oooo0080oooo00@0o`3o0?ooo`3oool0o`3o0`3oool01@3o0?l0oooo0?ooo`3oool0
o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0
0`3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0
oooo0?ooo`3o0?l50?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o
0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00D0oooo00@0o`3o0?ooo`3oool0o`3o103oool0
103o0?l0oooo0?ooo`3o0?l?0?ooo`006P3oool2000002<0oooo00<0o`3o0?ooo`3oool0<`3oool0
103o0?l0oooo0?ooo`3o0?l20?ooo`030?l0o`3oool0oooo02@0oooo0P3o0?l40?ooo`050?l0o`3o
ool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`030?ooo`80o`3o1P3oool00`3o0?l0oooo
0?ooo`0=0?ooo`030?l0o`3oool0oooo00D0oooo0P3o0?l60?ooo`030?l0o`3oool0o`3o00@0oooo
00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`030?l0o`3oool0oooo0080oooo00@0o`3o0?ooo`3o
ool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`<0oooo00D0
o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0
oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0
oooo00<0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`040?l0o`3oool0
oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`04
0?l0o`3oool0oooo0?l0o`l0oooo000J0?ooo`030000003oool0oooo0280oooo00<0o`3o0?ooo`3o
ool0<P3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool0903oool20?l0
o`@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo00<0oooo0P3o0?l6
0?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`05
0?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`040?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3o
ool00`3o0?l0oooo0?ooo`020?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0
o`3o103oool0103o0?l0oooo0?ooo`3o0?l30?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool0
1@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0
o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`030?ooo`040?l0o`3oool0oooo0?l0
o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3o
ool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l?0?oo
o`006P3oool00`000000oooo0?ooo`0R0?ooo`030?l0o`3oool0oooo0380oooo00D0o`3o0?ooo`3o
ool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo02@0oooo0P3o0?l40?ooo`050?l0o`3oool0oooo
0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`030?ooo`80o`3o1P3oool00`3o0?l0oooo0?ooo`05
0?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool20?l0o`H0oooo00@0o`3o
0?ooo`3oool0o`3o0`3oool00`3o0?l0oooo0?ooo`020?ooo`040?l0o`3oool0oooo0?l0o`@0oooo
00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`030?l0o`3oool0o`3o
00D0oooo00@0o`3o0?ooo`3oool0o`3o0`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0
o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool00`3o0?l0
oooo0?ooo`020?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0
103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0
o`3o103oool0103o0?l0oooo0?ooo`3o0?l?0?ooo`006P3oool00`000000oooo0?ooo`0R0?ooo`03
0?l0o`3oool0oooo0380oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0oooo0?l0o`0V0?ooo`80
o`3o103oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00`3oool20?l0
o`H0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`80o`3o1P3oool20?l0
o`D0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`030?l0o`3oool0oooo0080oooo00@0o`3o
0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0
oooo00<0o`3o0?ooo`3o0?l01@3oool0103o0?l0oooo0?ooo`3o0?l30?ooo`050?l0o`3oool0oooo
0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo
0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool00`3o0?l0oooo0?ooo`020?ooo`04
0?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o
0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`0>0?ooo`00
6P3oool00`000000oooo0?ooo`0R0?ooo`030?l0o`3oool0oooo0380oooo00<0o`3o0?ooo`3oool0
0P3oool00`3o0?l0oooo0?l0o`0V0?ooo`80o`3o1@3oool0103o0?l0oooo0?ooo`3o0?l20?ooo`03
0?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3o0?l01@3oool20?l0o`H0oooo0P3o0?l60?ooo`80
o`3o1P3oool00`3o0?l0oooo0?l0o`040?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool00`3o
0?l0oooo0?ooo`020?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o1@3o
ool20?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool0103o0?l0oooo0?ooo`3o0?l30?ooo`050?l0
o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?oo
o`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool00`3o0?l0oooo0?oo
o`020?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0
oooo0?ooo`3o0?l40?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`02
0?ooo`030?l0o`3oool0oooo00`0oooo000J0?ooo`030000003oool0oooo0280oooo00<0o`3o0?oo
o`3oool0<P3oool00`3o0?l0oooo0?ooo`030?ooo`80o`3o9P3oool20?l0o`D0oooo00@0o`3o0?oo
o`3oool0o`3o0P3oool00`3o0?l0oooo0?ooo`030?ooo`030?l0o`3oool0o`3o00D0oooo0P3o0?l6
0?ooo`80o`3o1P3oool20?l0o`H0oooo00<0o`3o0?ooo`3o0?l0103oool01@3o0?l0oooo0?ooo`3o
ool0o`3o00<0oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o0?l50?ooo`030?l0
o`3oool0o`3o00D0oooo0P3o0?l60?ooo`80o`3o1@3oool00`3o0?l0oooo0?l0o`050?ooo`040?l0
o`3oool0oooo0?l0o`<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo
0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00<0o`3o0?ooo`3oool00P3o
ool0103o0?l0oooo0?ooo`3o0?l40?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0
oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o0?l4
0?ooo`030?l0o`3oool0oooo00`0oooo000J0?ooo`030000003oool0oooo0280oooo00<0o`3o0?oo
o`3oool0<P3oool00`3o0?l0oooo0?ooo`030?ooo`80o`3o9P3oool20?l0o`D0oooo00@0o`3o0?oo
o`3oool0o`3o0`3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00`3o
ool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`040?ooo`050?l0o`3o
ool0oooo0?ooo`3o0?l00`3oool00`3o0?l0oooo0?ooo`020?ooo`040?l0o`3oool0oooo0?l0o`D0
oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool0103o0?l0
oooo0?ooo`3o0?l30?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3o
ool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`040?ooo`050?l0o`3oool0oooo0?ooo`3o
0?l00P3oool00`3o0?l0oooo0?ooo`020?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o
0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o
0?l40?ooo`030?l0o`3oool0oooo00`0oooo000J0?ooo`030000003oool0oooo02<0oooo00<0o`3o
0?ooo`3oool0<@3oool00`3o0?l0oooo0?ooo`040?ooo`030?l0o`3oool0oooo02D0oooo00<0o`3o
0?ooo`3oool00`3oool0103o0?l0oooo0?ooo`3o0?l30?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0
0P3oool00`3o0?l0oooo0?ooo`030?ooo`80o`3o1P3oool20?l0o`D0oooo00<0o`3o0?ooo`3o0?l0
1P3oool00`3o0?l0oooo0?l0o`040?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool00`3o0?l0
oooo0?ooo`020?ooo`040?l0o`3oool0oooo0?l0o`D0oooo0P3o0?l60?ooo`80o`3o1P3oool20?l0
o`H0oooo0P3o0?l50?ooo`030?l0o`3oool0o`3o00D0oooo00@0o`3o0?ooo`3oool0o`3o0`3oool0
1@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`040?ooo`050?l0
o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?oo
o`3oool0oooo0?l0o`030?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o
103oool00`3o0?l0oooo0?ooo`0<0?ooo`006P3oool2000002@0oooo00<0o`3o0?ooo`3oool0<@3o
ool00`3o0?l0oooo0?ooo`0/0?ooo`030?l0o`3oool0oooo00<0oooo00@0o`3o0?ooo`3oool0o`3o
0`3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00`3oool20?l0o`H0
oooo0P3o0?l50?ooo`040?l0o`3oool0oooo0?l0o`D0oooo00<0o`3o0?ooo`3o0?l0103oool01@3o
0?l0oooo0?ooo`3oool0o`3o00<0oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o
0?l50?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1@3oool00`3o0?l0oooo0?l0
o`050?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o0`3oool01@3o0?l0
oooo0?ooo`3oool0o`3o00@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0
oooo0?ooo`3o0?l00`3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0
oooo00@0o`3o0?ooo`3oool0o`3o103oool00`3o0?l0oooo0?ooo`0<0?ooo`006P3oool00`000000
oooo0?ooo`0S0?ooo`030?l0o`3oool0oooo0340oooo00<0o`3o0?ooo`3oool0<P3oool0103o0?l0
oooo0?ooo`3o0?l30?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`03
0?ooo`80o`3o1P3oool20?l0o`D0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?oo
o`3o0?l40?ooo`030?l0o`3oool0oooo0080oooo00@0o`3o0?ooo`3oool0o`3o103oool00`3o0?l0
oooo0?l0o`060?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l60?ooo`80o`3o1@3oool00`3o0?l0
oooo0?l0o`050?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o0`3oool0
1@3o0?l0oooo0?ooo`3oool0o`3o00@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`040?l0
o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l5
0?ooo`030?l0o`3oool0o`3o00@0oooo00<0o`3o0?ooo`3oool0303oool001X0oooo00<000000?oo
o`3oool08`3oool00`3o0?l0oooo0?ooo`020?ooo`030?l0o`3oool0oooo02`0oooo00<0o`3o0?oo
o`3oool0<`3oool00`3o0?l0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool0
0`3o0?l0oooo0?ooo`030?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l0103oool0
103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00<0o`3o0?ooo`3oool0
0P3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`030?l0o`3oool0o`3o00H0oooo0P3o0?l60?ooo`80
o`3o1`3oool00`3o0?l0oooo0?ooo`040?ooo`80o`3o1P3oool20?l0o`D0oooo00@0o`3o0?ooo`3o
ool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0
o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0
o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`040?ooo`030?l0o`3oool0oooo
00`0oooo000J0?ooo`030000003oool0oooo02<0oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0
oooo0?ooo`0/0?ooo`030?l0o`3oool0oooo03<0oooo0P3o0?l40?ooo`050?l0o`3oool0oooo0?oo
o`3o0?l00P3oool00`3o0?l0oooo0?ooo`030?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?oo
o`3o0?l0103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3o
ool00`3o0?l0oooo0?ooo`020?ooo`040?l0o`3oool0oooo0?l0o`D0oooo0P3o0?l60?ooo`80o`3o
1P3oool20?l0o`L0oooo00<0o`3o0?ooo`3oool0103oool20?l0o`H0oooo0P3o0?l50?ooo`040?l0
o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l4
0?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo
0?ooo`3o0?l50?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0
oooo0?ooo`0;0?ooo`006P3oool00`000000oooo0?ooo`0S0?ooo`030?l0o`3oool0oooo0080oooo
00<0o`3o0?ooo`3oool0;03oool00`3o0?l0oooo0?ooo`0c0?ooo`80o`3o103oool01@3o0?l0oooo
0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?l0o`050?ooo`03
0?l0o`3oool0o`3o00@0oooo00@0o`3o0?ooo`3oool0o`3o103oool01@3o0?l0oooo0?ooo`3oool0
o`3o0080oooo00<0o`3o0?ooo`3oool00`3oool0103o0?l0oooo0?ooo`3o0?l50?ooo`80o`3o1`3o
ool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3o
ool00`3o0?l0oooo0?ooo`040?ooo`80o`3o1P3oool00`3o0?l0oooo0?l0o`040?ooo`040?l0o`3o
ool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?oo
o`040?l0o`3oool0oooo0?l0o`@0oooo00@0o`3o0?ooo`3oool0o`3o1@3oool00`3o0?l0oooo0?l0
o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3oool02`3oool001X0oooo00<00000
0?ooo`3oool08`3oool0103o0?l0oooo0?ooo`3oool20?l0obh0oooo00<0o`3o0?ooo`3oool0<`3o
ool20?l0o`@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`030?l0o`3oool0oooo0080oooo
00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`040?ooo`040?l0o`3oool0oooo0?l0o`@0
oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?oo
o`3o0?l01P3oool20?l0o`L0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?oo
o`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool0103oool20?l0o`H0oooo00<0o`3o0?oo
o`3o0?l0103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00@0
o`3o0?ooo`3oool0o`3o103oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0
o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0oooo
00/0oooo000J0?ooo`030000003oool0oooo02<0oooo00@0o`3o0?ooo`3oool0oooo0P3o0?l^0?oo
o`030?l0o`3oool0oooo03<0oooo0P3o0?l40?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool0
0`3o0?l0oooo0?ooo`020?ooo`030?l0o`3oool0o`3o00@0oooo00@0o`3o0?ooo`3oool0o`3o103o
ool0103o0?l0oooo0?ooo`3o0?l40?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0
oooo0?ooo`030?ooo`030?l0o`3oool0o`3o00H0oooo0P3o0?l70?ooo`030?l0o`3oool0oooo00D0
oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00@0
oooo0P3o0?l60?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l0103oool0103o0?l0
oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`D0oooo00<0o`3o0?ooo`3o0?l0103oool0
103o0?l0oooo0?ooo`3o0?l50?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3o
ool00`3o0?l0oooo0?ooo`0;0?ooo`006P3oool2000002@0oooo00@0o`3o0?ooo`3oool0oooo0P3o
0?l^0?ooo`030?l0o`3oool0oooo03<0oooo0P3o0?l50?ooo`040?l0o`3oool0oooo0?l0o`<0oooo
00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo0080oooo00@0o`3o0?ooo`3o
ool0o`3o103oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00<0o`3o0?ooo`3oool00P3oool0
0`3o0?l0oooo0?l0o`060?ooo`80o`3o1P3oool20?l0o`L0oooo00<0o`3o0?ooo`3oool03@3oool0
0`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00@0oooo0P3o0?l60?ooo`030?l0o`3oool0
o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`040?ooo`040?l0o`3oool0
oooo0?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3o
ool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?ooo`0;0?ooo`006P3oool0
0`000000oooo0?ooo`0S0?ooo`040?l0o`3oool0oooo0?ooo`80o`3o;P3oool00`3o0?l0oooo0?oo
o`0c0?ooo`80o`3o1@3oool0103o0?l0oooo0?ooo`3o0?l30?ooo`050?l0o`3oool0oooo0?ooo`3o
0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool00P3oool01@3o
0?l0oooo0?ooo`3oool0o`3o00<0oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0oooo0?l0o`06
0?ooo`80o`3o1`3oool00`3o0?l0oooo0?ooo`0M0?ooo`030?l0o`3oool0oooo00@0oooo0P3o0?l6
0?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`04
0?ooo`040?l0o`3oool0oooo0?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0
o`050?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?oo
o`0;0?ooo`006P3oool00`000000oooo0?ooo`0S0?ooo`040?l0o`3oool0oooo0?ooo`80o`3o9`3o
ool00`3o0?l0oooo0?ooo`030?ooo`030?l0o`3oool0oooo03@0oooo0P3o0?l50?ooo`040?l0o`3o
ool0oooo0?l0o`<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?oo
o`3o0?l00P3oool00`3o0?l0oooo0?ooo`020?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool0
0`3o0?l0oooo0?ooo`020?ooo`030?l0o`3oool0o`3o00H0oooo0P3o0?l70?ooo`030?l0o`3oool0
oooo01d0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`040?ooo`030?l0o`3oool0
o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0
o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool20?l0o`H0oooo00<0o`3o0?ooo`3o0?l01@3oool0
0`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0oooo00/0oooo000J0?ooo`030000003oool0oooo
02<0oooo00@0o`3o0?ooo`3oool0oooo0P3o0?lW0?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o
0?ooo`3oool0=03oool20?l0o`D0oooo00@0o`3o0?ooo`3oool0o`3o0`3oool01@3o0?l0oooo0?oo
o`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?oo
o`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00@0oooo00<0o`3o0?ooo`3o0?l01P3oool2
0?l0obl0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`040?ooo`030?l0o`3oool0
o`3o00D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`050?ooo`80o`3o1P3oool2
0?l0o`H0oooo00<0o`3o0?ooo`3o0?l01P3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0
oooo00X0oooo000J0?ooo`030000003oool0oooo02@0oooo00H0o`3o0?ooo`3oool0o`3o0?ooo`3o
0?lV0?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3oool0=@3oool00`3o0?l0oooo0?oo
o`030?ooo`040?l0o`3oool0oooo0?l0o`<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?ooo`05
0?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00@0o`3o
0?ooo`3oool0o`3o1@3oool00`3o0?l0oooo0?l0o`060?ooo`80o`3o;`3oool00`3o0?l0oooo0?oo
o`050?ooo`030?l0o`3oool0oooo00@0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0
o`050?ooo`80o`3o1P3oool20?l0o`L0oooo00<0o`3o0?ooo`3oool01@3oool20?l0o`H0oooo00<0
o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`0:0?ooo`006P3oool00`000000oooo0?ooo`0T
0?ooo`060?l0o`3oool0oooo0?l0o`3oool0o`3o9P3oool20?l0o`@0oooo00<0o`3o0?ooo`3oool0
>`3oool0103o0?l0oooo0?ooo`3o0?l30?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00`3oool01@3o
0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`040?ooo`030?l0o`3o
ool0o`3o00H0oooo0P3o0?l70?ooo`030?l0o`3oool0oooo03D0oooo00<0o`3o0?ooo`3oool01@3o
ool00`3o0?l0oooo0?ooo`040?ooo`80o`3o1P3oool20?l0o`L0oooo00<0o`3o0?ooo`3oool01@3o
ool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3o
ool00`3o0?l0oooo0?ooo`0:0?ooo`006P3oool00`000000oooo0?ooo`0T0?ooo`060?l0o`3oool0
oooo0?l0o`3oool0o`3o9P3oool00`3o0?l0oooo0?l0o`030?ooo`030?l0o`3oool0oooo03/0oooo
00<0o`3o0?ooo`3o0?l0103oool01@3o0?l0oooo0?ooo`3oool0o`3o00<0oooo00D0o`3o0?ooo`3o
ool0oooo0?l0o`030?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0103oool00`3o0?l0oooo0?l0o`06
0?ooo`80o`3o1`3oool00`3o0?l0oooo0?ooo`0e0?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o
0?ooo`3oool0103oool20?l0o`L0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`05
0?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`0B
0?ooo`006P3oool2000002D0oooo00H0o`3o0?ooo`3oool0o`3o0?ooo`3o0?lV0?ooo`030?l0o`3o
ool0o`3o00<0oooo00<0o`3o0?ooo`3oool0?03oool20?l0o`D0oooo00@0o`3o0?ooo`3oool0o`3o
103oool0103o0?l0oooo0?ooo`3o0?l30?ooo`050?l0o`3oool0oooo0?ooo`3o0?l0103oool00`3o
0?l0oooo0?l0o`060?ooo`80o`3o?`3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo
00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo
00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`0J0?ooo`006P3oool00`000000
oooo0?ooo`0T0?ooo`060?l0o`3oool0oooo0?l0o`3oool0o`3o9P3oool0103o0?l0oooo0?ooo`3o
0?l20?ooo`030?l0o`3oool0oooo03`0oooo0P3o0?l50?ooo`040?l0o`3oool0oooo0?l0o`@0oooo
00@0o`3o0?ooo`3oool0o`3o103oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0o`3o00H0
oooo0P3o0?m70?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0
oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool08P3oool001X0oooo
00<000000?ooo`3oool0903oool01P3o0?l0oooo0?ooo`3o0?l0oooo0?l0obH0oooo00@0o`3o0?oo
o`3oool0o`3o0P3oool00`3o0?l0oooo0?ooo`0l0?ooo`80o`3o1@3oool0103o0?l0oooo0?ooo`3o
0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo
0?l0o`060?ooo`80o`3oC`3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0oooo
00<0o`3o0?ooo`3oool0:P3oool001X0oooo00<000000?ooo`3oool0903oool01P3o0?l0oooo0?oo
o`3o0?l0oooo0?l0obH0oooo00@0o`3o0?ooo`3oool0o`3o0P3oool00`3o0?l0oooo0?ooo`0l0?oo
o`80o`3o1@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00<0
o`3o0?ooo`3o0?l01P3oool20?l0o`H0oooo0P3o0?m?0?ooo`030?l0o`3oool0oooo00D0oooo00<0
o`3o0?ooo`3oool0<P3oool001X0oooo00<000000?ooo`3oool0903oool01P3o0?l0oooo0?ooo`3o
0?l0oooo0?l0obH0oooo00@0o`3o0?ooo`3oool0o`3o0P3oool00`3o0?l0oooo0?ooo`0l0?ooo`80
o`3o1@3oool0103o0?l0oooo0?ooo`3o0?l40?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00<0o`3o
0?ooo`3o0?l01P3oool20?l0o`L0oooo00<0o`3o0?ooo`3oool0RP3oool00080oooo00<000000?oo
o`3oool0103oool2000000D0oooo100000060?ooo`030000003oool0oooo02@0oooo00H0o`3o0?oo
o`3oool0o`3o0?ooo`3o0?lU0?ooo`030?l0o`3oool0oooo0080oooo0`3o0?ln0?ooo`80o`3o1P3o
ool20?l0o`D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0oooo0?l0o`060?ooo`80o`3o1`3o
ool00`3o0?l0oooo0?ooo`2:0?ooo`000P3oool00`000000oooo0?ooo`040?ooo`800000103oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0903oool0
0`3o0?l0oooo0?l0o`020?ooo`030?l0o`3oool0oooo02<0oooo00<0o`3o0?ooo`3oool00`3oool2
0?l0och0oooo0P3o0?l60?ooo`80o`3o1@3oool00`3o0?l0oooo0?l0o`060?ooo`80o`3o1P3oool2
0?l0oi@0oooo00020?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`0T0?ooo`030?l0o`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool08`3oool00`3o
0?l0oooo0?ooo`030?ooo`030?l0o`3oool0oooo03h0oooo00<0o`3o0?ooo`3oool0103oool20?l0
o`D0oooo00<0o`3o0?ooo`3o0?l01P3oool20?l0o`H0oooo0P3o0?nD0?ooo`000`3oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo00<0oooo0`00000U0?ooo`80o`3o0P3oool00`3o0?l0
oooo0?ooo`0S0?ooo`030?l0o`3oool0oooo04/0oooo0P3o0?l50?ooo`030?l0o`3oool0o`3o00H0
oooo0P3o0?l60?ooo`80o`3oU03oool000<0oooo00<000000?ooo`3oool02P3oool4000000H0oooo
00<000000?ooo`3oool09@3oool20?l0o`80oooo00<0o`3o0?ooo`3oool08`3oool00`3o0?l0oooo
0?ooo`1;0?ooo`80o`3o1@3oool00`3o0?l0oooo0?l0o`060?ooo`80o`3o1`3oool00`3o0?l0oooo
0?ooo`2B0?ooo`00103oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool09@3oool20?l0o`80oooo00<0o`3o0?ooo`3oool08`3oool00`3o0?l0oooo0?oo
o`1;0?ooo`80o`3o1P3oool20?l0o`H0oooo0P3o0?l70?ooo`030?l0o`3oool0oooo0980oooo0004
0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0U
0?ooo`80o`3o0`3oool00`3o0?l0oooo0?ooo`0R0?ooo`030?l0o`3oool0oooo04/0oooo0P3o0?l6
0?ooo`80o`3o1P3oool20?l0oi`0oooo0005000000/0oooo1@0000050?ooo`030000003oool0oooo
02D0oooo0P3o0?l30?ooo`030?l0o`3oool0oooo0280oooo00<0o`3o0?ooo`3oool0C03oool00`3o
0?l0oooo0?ooo`040?ooo`80o`3o1P3oool20?l0oi`0oooo000J0?ooo`030000003oool0oooo02D0
oooo0P3o0?l30?ooo`030?l0o`3oool0oooo0280oooo00<0o`3o0?ooo`3oool0C03oool00`3o0?l0
oooo0?ooo`040?ooo`80o`3o1`3oool00`3o0?l0oooo0?ooo`2J0?ooo`006P3oool00`000000oooo
0?ooo`0U0?ooo`80o`3o0`3oool00`3o0?l0oooo0?ooo`0R0?ooo`030?l0o`3oool0oooo05<0oooo
0P3o0?l70?ooo`030?l0o`3oool0oooo09X0oooo000J0?ooo`8000009P3oool20?l0o`<0oooo00<0
o`3o0?ooo`3oool08P3oool00`3o0?l0oooo0?ooo`1D0?ooo`030?l0o`3oool0oooo0:80oooo000J
0?ooo`030000003oool0oooo02D0oooo0P3o0?l30?ooo`030?l0o`3oool0oooo0280oooo00<0o`3o
0?ooo`3oool0E03oool00`3o0?l0oooo0?ooo`2R0?ooo`006P3oool00`000000oooo0?ooo`0U0?oo
o`80o`3o0`3oool00`3o0?l0oooo0?ooo`0R0?ooo`030?l0o`3oool0oooo0?T0oooo000J0?ooo`03
0000003oool0oooo02D0oooo0P3o0?l30?ooo`030?l0o`3oool0oooo01/0oooo00<0o`3o0?ooo`3o
ool0103oool00`3o0?l0oooo0?ooo`3i0?ooo`006P3oool00`000000oooo0?ooo`0U0?ooo`80o`3o
0`3oool00`3o0?l0oooo0?ooo`0K0?ooo`030?l0o`3oool0oooo00@0oooo00<0o`3o0?ooo`3oool0
n@3oool001X0oooo00<000000?ooo`3oool09@3oool20?l0o`<0oooo00<0o`3o0?ooo`3oool06`3o
ool00`3o0?l0oooo0?ooo`040?ooo`030?l0o`3oool0oooo0?T0oooo000J0?ooo`030000003oool0
oooo02D0oooo0P3o0?l30?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3oool05@3oool2
0?l0o`@0oooo00<0o`3o0?ooo`3oool0nP3oool001X0oooo0P00000V0?ooo`030?l0o`3oool0oooo
0080oooo00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?ooo`0E0?ooo`80o`3o103oool00`3o
0?l0oooo0?ooo`3j0?ooo`006P3oool00`000000oooo0?ooo`0U0?ooo`030?l0o`3oool0oooo0080
oooo00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?ooo`0E0?ooo`80o`3o103oool00`3o0?l0
oooo0?ooo`3j0?ooo`006P3oool00`000000oooo0?ooo`0U0?ooo`030?l0o`3oool0oooo0080oooo
00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?ooo`0E0?ooo`030?l0o`3oool0o`3o00<0oooo
00<0o`3o0?ooo`3oool0nP3oool001X0oooo00<000000?ooo`3oool09@3oool00`3o0?l0oooo0?oo
o`020?ooo`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3oool0503oool0103o0?l0oooo0?oo
o`3o0?l30?ooo`030?l0o`3oool0oooo0?X0oooo000J0?ooo`030000003oool0oooo02X0oooo00<0
o`3o0?ooo`3oool00P3oool20?l0oaH0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0
o`3oool0oooo0?X0oooo000J0?ooo`030000003oool0oooo02/0oooo00@0o`3o0?ooo`3oool0oooo
0P3o0?lF0?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`3j0?ooo`00
6P3oool00`000000oooo0?ooo`0[0?ooo`040?l0o`3oool0oooo0?ooo`80o`3o5P3oool01@3o0?l0
oooo0?ooo`3oool0o`3o0080oooo00<0o`3o0?ooo`3oool0nP3oool001X0oooo00<000000?ooo`3o
ool0:`3oool01`3o0?l0oooo0?ooo`3oool0o`3o0?ooo`3o0?l05@3oool01@3o0?l0oooo0?ooo`3o
ool0o`3o0080oooo00<0o`3o0?ooo`3oool0nP3oool001X0oooo0P00000/0?ooo`070?l0o`3oool0
oooo0?ooo`3o0?l0oooo0?l0o`0E0?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3o0?l0
o03oool001X0oooo00<000000?ooo`3oool0:`3oool01`3o0?l0oooo0?ooo`3oool0o`3o0?ooo`3o
0?l05@3oool00`3o0?l0oooo0?ooo`030?ooo`80o`3oo03oool001X0oooo00<000000?ooo`3oool0
:`3oool01`3o0?l0oooo0?ooo`3oool0o`3o0?ooo`3o0?l05@3oool00`3o0?l0oooo0?ooo`030?oo
o`80o`3oo03oool001X0oooo00<000000?ooo`3oool0:`3oool01`3o0?l0oooo0?ooo`3oool0o`3o
0?ooo`3o0?l05@3oool00`3o0?l0oooo0?ooo`030?ooo`030?l0o`3oool0oooo0?/0oooo000J0?oo
o`030000003oool0oooo02/0oooo00L0o`3o0?ooo`3oool0oooo0?l0o`3oool0o`3o00h0oooo00<0
o`3o0?ooo`3oool0103oool00`3o0?l0oooo0?ooo`3o0?ooo`80oooo000J0?ooo`030000003oool0
oooo02/0oooo00L0o`3o0?ooo`3oool0oooo0?l0o`3oool0o`3o00h0oooo00<0o`3o0?ooo`3oool0
103oool00`3o0?l0oooo0?ooo`3o0?ooo`80oooo000J0?ooo`030000003oool0oooo02/0oooo00L0
o`3o0?ooo`3oool0oooo0?l0o`3oool0o`3o00H0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0
oooo0?ooo`040?ooo`030?l0o`3oool0oooo0?l0oooo0P3oool001X0oooo0P00000]0?ooo`060?l0
o`3oool0oooo0?l0o`3oool0o`3o1P3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo
00@0oooo00<0o`3o0?ooo`3oool0o`3oool20?ooo`006P3oool00`000000oooo0?ooo`0/0?ooo`06
0?l0o`3oool0oooo0?l0o`3oool0o`3o1P3oool00`3o0?l0oooo0?ooo`040?ooo`030?l0o`3oool0
o`3o00D0oooo00<0o`3o0?ooo`3oool0o`3oool20?ooo`006P3oool00`000000oooo0?ooo`0/0?oo
o`040?l0o`3oool0oooo0?l0o`80oooo00<0o`3o0?ooo`3oool00`3oool00`3o0?l0oooo0?ooo`04
0?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3oool0o`3oool20?ooo`006P3oool00`00
0000oooo0?ooo`0/0?ooo`040?l0o`3oool0oooo0?l0o`80oooo00<0o`3o0?ooo`3oool00`3oool0
0`3o0?l0oooo0?ooo`040?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3oool0o`3oool2
0?ooo`00403oool4000000H0oooo00<000000?ooo`3oool0;03oool00`3o0?l0oooo0?l0o`030?oo
o`030?l0o`3oool0oooo00<0oooo00<0o`3o0?ooo`3oool0103oool00`3o0?l0oooo0?l0o`050?oo
o`030?l0o`3oool0oooo0?l0oooo0P3oool000l0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo02`0oooo00<0o`3o0?ooo`3o0?l00`3oool00`3o0?l0
oooo0?ooo`030?ooo`80o`3o1@3oool00`3o0?l0oooo0?l0o`050?ooo`030?l0o`3oool0oooo0?l0
oooo0P3oool000l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo02`0oooo00<0o`3o0?ooo`3o0?l00`3oool00`3o0?l0oooo0?ooo`020?ooo`030?l0
o`3oool0o`3o00D0oooo00@0o`3o0?ooo`3oool0o`3o103oool00`3o0?l0oooo0?ooo`3o0?ooo`80
oooo000?0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool3000002`0oooo
00<0o`3o0?ooo`3o0?l00`3oool00`3o0?l0oooo0?ooo`020?ooo`030?l0o`3oool0o`3o00D0oooo
00@0o`3o0?ooo`3oool0o`3o103oool00`3o0?l0oooo0?ooo`3o0?ooo`80oooo000@0?ooo`@00000
1P3oool00`000000oooo0?ooo`0/0?ooo`030?l0o`3oool0o`3o00<0oooo00<0o`3o0?ooo`3oool0
0P3oool00`3o0?l0oooo0?l0o`050?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00<0o`3o0?ooo`3o
ool0o`3oool20?ooo`003`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0;03oool00`3o0?l0oooo0?l0o`030?ooo`030?l0o`3oool0oooo0080oooo
00<0o`3o0?ooo`3o0?l01@3oool0103o0?l0oooo0?ooo`3o0?l30?ooo`030?l0o`3oool0oooo0?l0
oooo0`3oool000l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo02`0oooo00<0o`3o0?ooo`3o0?l00`3oool00`3o0?l0oooo0?ooo`020?ooo`030?l0
o`3oool0o`3o00D0oooo00@0o`3o0?ooo`3oool0o`3o0`3oool00`3o0?l0oooo0?ooo`3o0?ooo`<0
oooo000@0?ooo`@000001P3oool00`000000oooo0?ooo`0]0?ooo`80o`3o0`3oool00`3o0?l0oooo
0?ooo`020?ooo`030?l0o`3oool0o`3o00D0oooo00@0o`3o0?ooo`3oool0o`3o0`3oool00`3o0?l0
oooo0?ooo`3o0?ooo`<0oooo000J0?ooo`030000003oool0oooo02d0oooo0P3o0?l30?ooo`030?l0
o`3oool0oooo0080oooo00<0o`3o0?ooo`3o0?l01@3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080
oooo00<0o`3o0?ooo`3oool0o`3oool30?ooo`006P3oool00`000000oooo0?ooo`0]0?ooo`80o`3o
0`3oool00`3o0?l0oooo0?ooo`020?ooo`040?l0o`3oool0oooo0?l0o`@0oooo00D0o`3o0?ooo`3o
ool0oooo0?l0o`020?ooo`030?l0o`3oool0oooo0?l0oooo0`3oool001X0oooo00<000000?ooo`3o
ool0;@3oool20?l0o`<0oooo00<0o`3o0?ooo`3oool00P3oool0103o0?l0oooo0?ooo`3o0?l40?oo
o`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool00`3o0?l0oooo0?ooo`3o0?ooo`<0oooo000J0?oo
o`800000;P3oool20?l0o`@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`030?l0o`3oool0
oooo0080oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0oooo0?l0o`3o0?ooo`D0oooo000J0?oo
o`030000003oool0oooo02d0oooo0P3o0?l40?ooo`050?l0o`3oool0oooo0?ooo`3o0?l00P3oool0
0`3o0?l0oooo0?ooo`020?ooo`030?l0o`3oool0oooo0080oooo00<0o`3o0?ooo`3o0?l0o`3oool5
0?ooo`006P3oool00`000000oooo0?ooo`0]0?ooo`80o`3o103oool01@3o0?l0oooo0?ooo`3oool0
o`3o0080oooo00<0o`3o0?ooo`3oool00P3oool00`3o0?l0oooo0?ooo`020?ooo`030?l0o`3oool0
o`3o0?l0oooo1@3oool001X0oooo00<000000?ooo`3oool0;@3oool20?l0o`@0oooo00D0o`3o0?oo
o`3oool0oooo0?l0o`020?ooo`050?l0o`3oool0oooo0?ooo`3o0?l01P3oool20?l0ool0oooo1@3o
ool001X0oooo00<000000?ooo`3oool0;@3oool20?l0o`@0oooo00D0o`3o0?ooo`3oool0oooo0?l0
o`020?ooo`050?l0o`3oool0oooo0?ooo`3o0?l01P3oool20?l0ool0oooo1@3oool001X0oooo00<0
00000?ooo`3oool0;@3oool20?l0o`@0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`020?ooo`050?l0
o`3oool0oooo0?ooo`3o0?l01P3oool00`3o0?l0oooo0?ooo`3o0?ooo`@0oooo000J0?ooo`030000
003oool0oooo02d0oooo00<0o`3o0?ooo`3oool00`3oool01@3o0?l0oooo0?ooo`3oool0o`3o0080
oooo00D0o`3o0?ooo`3oool0oooo0?l0o`060?ooo`030?l0o`3oool0oooo0?l0oooo103oool001X0
oooo0P00000^0?ooo`030?l0o`3oool0oooo00<0oooo00D0o`3o0?ooo`3oool0oooo0?l0o`030?oo
o`040?l0o`3oool0oooo0?l0ool0oooo3@3oool001X0oooo00<000000?ooo`3oool0;@3oool00`3o
0?l0oooo0?ooo`040?ooo`040?l0o`3oool0oooo0?l0o`<0oooo00@0o`3o0?ooo`3oool0o`3oo`3o
ool=0?ooo`006P3oool00`000000oooo0?ooo`0d0?ooo`030?l0o`3oool0o`3o00@0oooo00@0o`3o
0?ooo`3oool0o`3oo`3oool=0?ooo`006P3oool00`000000oooo0?ooo`0d0?ooo`030?l0o`3oool0
o`3o00@0oooo00@0o`3o0?ooo`3oool0o`3oo`3oool=0?ooo`006P3oool00`000000oooo0?ooo`0d
0?ooo`030?l0o`3oool0o`3o00@0oooo00@0o`3o0?ooo`3oool0o`3oo`3oool=0?ooo`006P3oool0
0`000000oooo0?ooo`0d0?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?ooo`3o0?l0o`3oool=
0?ooo`006P3oool00`000000oooo0?ooo`0d0?ooo`030?l0o`3oool0o`3o00D0oooo00<0o`3o0?oo
o`3o0?l0o`3oool=0?ooo`006P3oool2000003D0oooo00<0o`3o0?ooo`3o0?l01@3oool00`3o0?l0
oooo0?l0o`3o0?ooo`d0oooo000J0?ooo`030000003oool0oooo03D0oooo0P3o0?l50?ooo`030?l0
o`3oool0o`3o0?l0oooo3@3oool001X0oooo00<000000?ooo`3oool0=@3oool20?l0o`D0oooo00<0
o`3o0?ooo`3o0?l0o`3oool=0?ooo`006P3oool00`000000oooo0?ooo`0e0?ooo`80o`3o1P3oool2
0?l0ool0oooo3@3oool001X0oooo00<000000?ooo`3oool0=@3oool20?l0o`H0oooo0P3o0?oo0?oo
o`d0oooo000J0?ooo`030000003oool0oooo03D0oooo0P3o0?l60?ooo`80o`3oo`3oool=0?ooo`00
6P3oool00`000000oooo0?ooo`0e0?ooo`80o`3o1P3oool00`3o0?l0oooo0?ooo`3o0?ooo``0oooo
000J0?ooo`030000003oool0oooo03D0oooo0P3o0?l60?ooo`030?l0o`3oool0oooo0?l0oooo303o
ool001X0oooo0P00000f0?ooo`80o`3o1P3oool00`3o0?l0oooo0?ooo`3o0?ooo``0oooo000J0?oo
o`030000003oool0oooo03D0oooo0P3o0?oo0?oooaD0oooo000J0?ooo`030000003oool0oooo03D0
oooo00<0o`3o0?ooo`3oool0o`3ooolD0?ooo`006P3oool00`000000oooo0?ooo`0e0?ooo`030?l0
o`3oool0oooo0?l0oooo503oool000@000001@3oool2000000D0oooo100000060?ooo`030000003o
ool0oooo0?l0ooooC03oool000@0oooo00<000000?ooo`3oool00P3oool2000000@0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0ooooC03oool0
00@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
0?l0ooooC03oool000@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`030?ooo`<0
0000o`3ooom<0?ooo`001000000<0?ooo`@00000o`3ooomE0?ooo`00103oool00`000000oooo0?oo
o`090?ooo`030000003oool0oooo0?l0ooooEP3oool000@0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`3o0?oooeH0oooo0004000000`0oooo1@00003o0?oooe@0oooo003o0?ooofT0
oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo0000\
\>"],
  ImageRangeCache->{{{0, 359}, {221.375, 0}} -> {-0.0332863, 5.60568, \
0.00155747, 0.0170876}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Switching Model Simulation", "Subsection",
  CellTags->"Large-Signal Simulation"],

Cell[TextData[{
  "The start-up transient can also be simulated based on the discontinuous \
switching model. For a better comparison with the responses given by the \
averaged model, the initial values for switching-model simulation are \
obtained by adding the corresponding 1st-order ripple corrections to the \
previous average initial values (",
  Cell[BoxData[
      \(TraditionalForm\`i\&_\_L\)]],
  "(0) = 0.5 A, ",
  Cell[BoxData[
      \(TraditionalForm\`v\&_\_C\)]],
  "(0) = 5 V)."
}], "Text",
  CellTags->"Large-Signal Simulation"],

Cell[BoxData[{
    \(\(iL0\  = \ \(Evaluate[
            iLRip1[0]\  + \ iL[0]\  /. \ ResAve]\)[\([1]\)];\)\), "\n", 
    \(\(vC0\  = \ \(Evaluate[
            vCRip1[0]\  + \ vC[0]\  /. \ ResAve]\)[\([1]\)];\)\)}], "Input",
  CellTags->"Large-Signal Simulation"],

Cell[BoxData[
    \(\(ResSwi\  = \ 
        NDSolve[{\(iL'\)[t]\  == \ 
                If[Mod[t, \ Ts] < d\ Ts, \ 
                  Vg/L, \((Vg\  - \ vC[t])\)/L], \(vC'\)[t]\  == \ 
                If[Mod[t, \ Ts] < d\ Ts, \(-vC[t]\)/\((R\ C)\), 
                  iL[t]/C\  - \ vC[t]/\((R\ C)\)], iL[0]\  == \ iL0, \ 
              vC[0]\  == \ vC0}\  /. \ Param1, {iL, \ vC}, \ {t, \ 0, \ 
            T0}, \ MaxSteps\  -> \ 2\ 10^4];\)\)], "Input",
  CellTags->"Large-Signal Simulation"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(iLSwi\  = \ 
        Plot[Evaluate[iL[10^\(-3\)\ t]\  /. \ ResSwi], {t, \ 0, \ 
            10^3\ T0}, \ PlotPoints\  -> \ 500, 
          PlotStyle\  -> \ {{AbsoluteThickness[0.001], 
                RGBColor[1, \ 0, \ 0]}}];\)\)], "Input",
  CellTags->"Large-Signal Simulation"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 2.38095 -0.18294 0.500392 [
[.2619 .30495 -9 -9 ]
[.2619 .30495 9 0 ]
[.5 .30495 -9 -9 ]
[.5 .30495 9 0 ]
[.7381 .30495 -9 -9 ]
[.7381 .30495 9 0 ]
[.97619 .30495 -9 -9 ]
[.97619 .30495 9 0 ]
[.01131 .01722 -18 -4.5 ]
[.01131 .01722 0 4.5 ]
[.01131 .1173 -18 -4.5 ]
[.01131 .1173 0 4.5 ]
[.01131 .21737 -18 -4.5 ]
[.01131 .21737 0 4.5 ]
[.01131 .41753 -18 -4.5 ]
[.01131 .41753 0 4.5 ]
[.01131 .51761 -18 -4.5 ]
[.01131 .51761 0 4.5 ]
[.01131 .61769 -18 -4.5 ]
[.01131 .61769 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .31745 m
.2619 .3237 L
s
[(0.1)] .2619 .30495 0 1 Mshowa
.5 .31745 m
.5 .3237 L
s
[(0.2)] .5 .30495 0 1 Mshowa
.7381 .31745 m
.7381 .3237 L
s
[(0.3)] .7381 .30495 0 1 Mshowa
.97619 .31745 m
.97619 .3237 L
s
[(0.4)] .97619 .30495 0 1 Mshowa
.125 Mabswid
.07143 .31745 m
.07143 .3212 L
s
.11905 .31745 m
.11905 .3212 L
s
.16667 .31745 m
.16667 .3212 L
s
.21429 .31745 m
.21429 .3212 L
s
.30952 .31745 m
.30952 .3212 L
s
.35714 .31745 m
.35714 .3212 L
s
.40476 .31745 m
.40476 .3212 L
s
.45238 .31745 m
.45238 .3212 L
s
.54762 .31745 m
.54762 .3212 L
s
.59524 .31745 m
.59524 .3212 L
s
.64286 .31745 m
.64286 .3212 L
s
.69048 .31745 m
.69048 .3212 L
s
.78571 .31745 m
.78571 .3212 L
s
.83333 .31745 m
.83333 .3212 L
s
.88095 .31745 m
.88095 .3212 L
s
.92857 .31745 m
.92857 .3212 L
s
.25 Mabswid
0 .31745 m
1 .31745 L
s
.02381 .01722 m
.03006 .01722 L
s
[(0.4)] .01131 .01722 1 0 Mshowa
.02381 .1173 m
.03006 .1173 L
s
[(0.6)] .01131 .1173 1 0 Mshowa
.02381 .21737 m
.03006 .21737 L
s
[(0.8)] .01131 .21737 1 0 Mshowa
.02381 .41753 m
.03006 .41753 L
s
[(1.2)] .01131 .41753 1 0 Mshowa
.02381 .51761 m
.03006 .51761 L
s
[(1.4)] .01131 .51761 1 0 Mshowa
.02381 .61769 m
.03006 .61769 L
s
[(1.6)] .01131 .61769 1 0 Mshowa
.125 Mabswid
.02381 .04224 m
.02756 .04224 L
s
.02381 .06726 m
.02756 .06726 L
s
.02381 .09228 m
.02756 .09228 L
s
.02381 .14232 m
.02756 .14232 L
s
.02381 .16733 m
.02756 .16733 L
s
.02381 .19235 m
.02756 .19235 L
s
.02381 .24239 m
.02756 .24239 L
s
.02381 .26741 m
.02756 .26741 L
s
.02381 .29243 m
.02756 .29243 L
s
.02381 .34247 m
.02756 .34247 L
s
.02381 .36749 m
.02756 .36749 L
s
.02381 .39251 m
.02756 .39251 L
s
.02381 .44255 m
.02756 .44255 L
s
.02381 .46757 m
.02756 .46757 L
s
.02381 .49259 m
.02756 .49259 L
s
.02381 .54263 m
.02756 .54263 L
s
.02381 .56765 m
.02756 .56765 L
s
.02381 .59267 m
.02756 .59267 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.001 Mabswid
.02381 .01472 m
.02567 .05377 L
.02769 .09636 L
.0296 .13636 L
.03009 .14671 L
.03056 .1565 L
.03076 .16077 L
.03087 .1631 L
.03092 .16424 L
.03097 .16485 L
.03103 .1649 L
.03109 .16495 L
.03121 .16507 L
.03143 .16526 L
.03237 .16604 L
.03338 .1668 L
.03529 .16799 L
.03712 .16884 L
.03807 .16917 L
.03908 .16943 L
.03957 .16953 L
.04011 .16961 L
.04035 .16964 L
.04061 .16967 L
.04086 .16969 L
.04108 .1697 L
.04119 .16971 L
.04131 .16971 L
.04137 .16971 L
.04143 .16971 L
.04148 .16972 L
.04153 .16972 L
.04159 .16972 L
.04165 .16972 L
.04171 .16972 L
.04177 .16972 L
.04183 .16972 L
.04189 .16972 L
.04192 .16972 L
.04195 .16972 L
.04202 .16971 L
.04214 .16971 L
.04224 .16971 L
.04248 .16969 L
.04268 .16968 L
.0429 .16966 L
.04336 .16961 L
.04385 .16953 L
.04473 .16935 L
.04574 .16906 L
Mistroke
.04621 .1689 L
.04671 .16871 L
.04697 .1686 L
.04721 .1685 L
.04743 .1684 L
.04749 .16837 L
.04755 .16834 L
.04761 .16832 L
.04767 .1693 L
.04855 .1879 L
.0505 .22882 L
.05237 .2682 L
.05333 .28826 L
.05383 .29884 L
.05436 .31006 L
.05459 .31486 L
.05471 .31738 L
.05477 .31844 L
.05481 .31848 L
.05484 .31851 L
.05498 .31863 L
.05511 .31875 L
.05536 .31896 L
.05628 .31963 L
.05679 .31992 L
.05727 .32015 L
.0577 .32031 L
.05794 .32039 L
.05816 .32045 L
.0584 .3205 L
.05862 .32053 L
.05874 .32055 L
.05886 .32056 L
.05893 .32057 L
.05899 .32057 L
.05906 .32058 L
.05912 .32058 L
.05918 .32058 L
.05924 .32058 L
.0593 .32058 L
.05935 .32058 L
.05938 .32058 L
.05942 .32058 L
.05948 .32058 L
.05951 .32058 L
.05955 .32058 L
.05961 .32058 L
.05974 .32057 L
.05986 .32056 L
.06013 .32052 L
Mistroke
.06036 .32048 L
.0606 .32042 L
.06103 .32029 L
.06155 .32009 L
.06203 .31986 L
.06294 .31928 L
.06394 .31847 L
.06589 .31632 L
.06778 .31357 L
.06874 .31192 L
.06978 .30994 L
.07027 .30895 L
.07078 .30786 L
.07099 .30739 L
.07122 .30689 L
.07129 .30674 L
.07135 .30659 L
.07141 .30646 L
.07147 .30734 L
.07153 .30857 L
.07159 .30988 L
.07171 .31224 L
.07352 .35042 L
.07546 .39108 L
.07732 .4302 L
.07782 .44071 L
.07828 .45047 L
.0784 .45301 L
.07852 .4554 L
.07858 .45653 L
.07862 .4565 L
.07865 .45648 L
.07877 .45639 L
.0793 .45592 L
.08028 .45483 L
.08119 .45356 L
.08302 .45029 L
.08489 .44594 L
.08687 .44026 L
.08879 .43378 L
.09063 .42665 L
.09258 .41814 L
.09349 .41389 L
.09447 .40909 L
.09473 .40776 L
.09501 .40635 L
.09512 .40576 L
.09518 .40544 L
.09524 .40527 L
.09528 .40598 L
Mistroke
.09531 .40674 L
.09538 .40808 L
.0955 .4107 L
.09647 .43111 L
.09844 .4724 L
.10033 .51217 L
.10128 .53213 L
.1017 .54086 L
.10215 .5504 L
.10227 .55292 L
.1023 .55359 L
.10233 .5543 L
.1024 .55522 L
.10246 .555 L
.10252 .5548 L
.10263 .55441 L
.10308 .55276 L
.10409 .54886 L
.10598 .54052 L
.10781 .53136 L
.10975 .52044 L
.11162 .50883 L
.11361 .49537 L
.11553 .48137 L
.11737 .46701 L
.11832 .45927 L
.11855 .45735 L
.1188 .45528 L
.11887 .4547 L
.11895 .45408 L
.11901 .45356 L
.11904 .45326 L
.11908 .45382 L
.11933 .45922 L
.12121 .49867 L
.12321 .5406 L
.12421 .56162 L
.12513 .58101 L
.12559 .59076 L
.12582 .59553 L
.12603 .59988 L
.12608 .60101 L
.12614 .60224 L
.12619 .60332 L
.12624 .60299 L
.12648 .60145 L
.12698 .59812 L
.12886 .58474 L
.13086 .56918 L
.13279 .55296 L
Mistroke
.13465 .53628 L
.13662 .51744 L
.13852 .49832 L
.14035 .47909 L
.14137 .46804 L
.1423 .45777 L
.14252 .45527 L
.14264 .4539 L
.14276 .45261 L
.14282 .45194 L
.14285 .45156 L
.14288 .45204 L
.14302 .45491 L
.14326 .45997 L
.14417 .47911 L
.14608 .51921 L
.14791 .55777 L
.14894 .57928 L
.14942 .58944 L
.14963 .5939 L
.14975 .59644 L
.14987 .59881 L
.14992 .59998 L
.14998 .60109 L
.15004 .60129 L
.15009 .60078 L
.15033 .59858 L
.15083 .59388 L
.15175 .58515 L
.15359 .56671 L
.15555 .54597 L
.15743 .52498 L
.15925 .50394 L
.16118 .48067 L
.16304 .45752 L
.16398 .44551 L
.16501 .43218 L
.16549 .42597 L
.166 .41929 L
.16625 .41603 L
.16648 .41295 L
.16658 .4116 L
.16663 .41092 L
.16669 .41094 L
.16675 .41223 L
.16681 .41343 L
.16692 .41572 L
.16878 .45492 L
.17077 .4966 L
.17176 .51742 L
Mistroke
.17268 .53675 L
.17313 .54637 L
.17356 .55544 L
.17368 .55788 L
.17374 .55921 L
.1738 .56046 L
.17383 .56031 L
.17387 .55994 L
.17394 .55917 L
.17406 .5578 L
.17451 .55278 L
.1765 .52995 L
.17842 .50699 L
.18026 .48408 L
.18222 .45891 L
.18411 .43398 L
.18593 .40945 L
.18786 .38281 L
.18875 .37041 L
.18926 .36335 L
.18972 .35675 L
.19019 .35017 L
.19033 .34827 L
.19039 .34733 L
.19045 .34647 L
.19051 .34678 L
.19057 .34801 L
.19069 .3506 L
.1917 .37193 L
.19353 .41023 L
.19547 .45102 L
.19636 .4698 L
.19683 .47971 L
.19733 .49027 L
.19748 .49327 L
.19755 .49475 L
.19758 .49539 L
.19761 .4961 L
.19767 .49563 L
.19773 .49489 L
.19786 .49334 L
.19835 .48748 L
.19932 .47576 L
.20127 .45157 L
.20314 .4276 L
.20513 .4015 L
.20705 .37582 L
.2089 .3507 L
.21086 .32362 L
.21184 .31005 L
Mistroke
.21276 .29727 L
.21363 .28501 L
.21388 .28147 L
.21401 .27972 L
.21412 .2781 L
.21423 .27664 L
.21429 .27582 L
.21434 .27693 L
.21457 .28187 L
.2165 .32235 L
.21835 .36129 L
.21929 .38103 L
.22032 .40271 L
.2208 .41266 L
.22106 .41814 L
.22119 .42084 L
.2213 .42334 L
.22136 .42454 L
.22142 .42567 L
.22147 .42542 L
.22153 .42469 L
.22178 .42175 L
.22222 .41657 L
.22408 .39422 L
.22606 .3699 L
.22796 .34601 L
.22979 .32268 L
.23174 .29755 L
.23362 .27314 L
.23562 .24704 L
.23662 .23393 L
.23705 .22817 L
.23754 .22182 L
.23776 .21889 L
.23788 .21727 L
.23795 .21648 L
.238 .21574 L
.23805 .21505 L
.23811 .21481 L
.23817 .21612 L
.23823 .2173 L
.23844 .22166 L
.23939 .24165 L
.24131 .28217 L
.24317 .32117 L
.24411 .34095 L
.24461 .35142 L
.24488 .35719 L
.24502 .36002 L
.24508 .36127 L
Mistroke
.24514 .36264 L
.24521 .36394 L
.24526 .36435 L
.24532 .36368 L
.24539 .36297 L
.24566 .36004 L
.24613 .3549 L
.24704 .34477 L
.24902 .32257 L
.25093 .30084 L
.25277 .27966 L
.25472 .2569 L
.2566 .23485 L
.25859 .21133 L
.2596 .19953 L
.26052 .18866 L
.26142 .17807 L
.26164 .1755 L
.26176 .17409 L
.26182 .17339 L
.26187 .17275 L
.2619 .17238 L
.26194 .17301 L
.26201 .17449 L
.26213 .17707 L
.26237 .18213 L
.26437 .22424 L
.2663 .26482 L
.26816 .30387 L
.26866 .31434 L
.26879 .31696 L
.2689 .31944 L
.26895 .32052 L
.26901 .32167 L
.26907 .32226 L
.26912 .32174 L
.26936 .31949 L
.26961 .31708 L
.27014 .31195 L
.27196 .29413 L
.2739 .27484 L
.27576 .25602 L
.27774 .23581 L
.27965 .2162 L
.28149 .19727 L
.28344 .17709 L
.28435 .16778 L
.28482 .16288 L
.28533 .15771 L
.28558 .15509 L
Mistroke
.28565 .15437 L
.28569 .15396 L
.28573 .15396 L
.28576 .15466 L
.2858 .15543 L
.28586 .15681 L
.28635 .16718 L
.28733 .18757 L
.28929 .22879 L
.29117 .26847 L
.29212 .28838 L
.29254 .29709 L
.29265 .29948 L
.29271 .30079 L
.29277 .30203 L
.29283 .30319 L
.29289 .3036 L
.29294 .30315 L
.29299 .30274 L
.29344 .299 L
.29392 .29504 L
.29492 .28665 L
.29682 .27052 L
.29864 .25475 L
.30058 .23771 L
.30245 .22112 L
.30443 .20334 L
.30634 .18613 L
.30818 .16954 L
.30867 .16513 L
.30913 .16099 L
.30939 .15869 L
.30946 .15806 L
.30949 .15772 L
.30953 .15757 L
.30959 .15889 L
.30966 .16034 L
.31014 .17042 L
.31201 .2098 L
.31401 .25166 L
.315 .27261 L
.31592 .29198 L
.31615 .29666 L
.31639 .30169 L
.3165 .30401 L
.31656 .30527 L
.31661 .30645 L
.31666 .30745 L
.31671 .30725 L
.31677 .30685 L
Mistroke
.31682 .30649 L
.31727 .30329 L
.31777 .29974 L
.3197 .28573 L
.32155 .27189 L
.32352 .25682 L
.32542 .24203 L
.32724 .22758 L
.32918 .212 L
.33105 .19687 L
.332 .18913 L
.33251 .18504 L
.33278 .18279 L
.33304 .18068 L
.33315 .17978 L
.33327 .17883 L
.33333 .17833 L
.33336 .17891 L
.3334 .17959 L
.33351 .18205 L
.33403 .19302 L
.33495 .21237 L
.33683 .2518 L
.33882 .2937 L
.33931 .3039 L
.33982 .3147 L
.34026 .32393 L
.34038 .32645 L
.34042 .32718 L
.34045 .32787 L
.34051 .32818 L
.34057 .32782 L
.34062 .32748 L
.34075 .32672 L
.34259 .31486 L
.34448 .3023 L
.34648 .28848 L
.34841 .27475 L
.35026 .26119 L
.35223 .24645 L
.35413 .23199 L
.35596 .21788 L
.35644 .21413 L
.3569 .21057 L
.35702 .20962 L
.35705 .20937 L
.35709 .20909 L
.35715 .20885 L
.35719 .20959 L
.35722 .21027 L
Mistroke
.35729 .21184 L
.35743 .21461 L
.35791 .22469 L
.35981 .26476 L
.36165 .3033 L
.36267 .32479 L
.3636 .34431 L
.36406 .35406 L
.36419 .35673 L
.36425 .35808 L
.36429 .35877 L
.36432 .35857 L
.36456 .35714 L
.36548 .35162 L
.36734 .3399 L
.36933 .32683 L
.37124 .3137 L
.37308 .30062 L
.37504 .28627 L
.37693 .27207 L
.37893 .25662 L
.37994 .24875 L
.38038 .24527 L
.38061 .24343 L
.38074 .24241 L
.3808 .24191 L
.38086 .24145 L
.38092 .24099 L
.38098 .24123 L
.38104 .24251 L
.3811 .24391 L
.38133 .2486 L
.38176 .25776 L
.38272 .2778 L
.38465 .31851 L
.38652 .35768 L
.38747 .37759 L
.38772 .38305 L
.38787 .38604 L
.38794 .38751 L
.388 .38886 L
.38806 .39015 L
.38809 .39079 L
.38813 .39064 L
.38826 .38982 L
.3885 .38838 L
.39041 .37637 L
.39235 .36344 L
.39422 .35038 L
.39621 .3359 L
Mistroke
.39813 .3214 L
.39997 .307 L
.40193 .29125 L
.4029 .28328 L
.40382 .27571 L
.40424 .27216 L
.40448 .27022 L
.40459 .26927 L
.40469 .26841 L
.40475 .26792 L
.4048 .26871 L
.40486 .26994 L
.40493 .27126 L
.40518 .27665 L
.40563 .28612 L
.4076 .32747 L
.40949 .36728 L
.41045 .38728 L
.41132 .40556 L
.41155 .41038 L
.41167 .41292 L
.4118 .41563 L
.41183 .41628 L
.41186 .41698 L
.41192 .41784 L
.41198 .41747 L
.41203 .41712 L
.41225 .41569 L
.41326 .40902 L
.41516 .39582 L
.41698 .38246 L
.41893 .36757 L
.4208 .35265 L
.42279 .33621 L
.42471 .31986 L
.42656 .30371 L
.42749 .2954 L
.42799 .29095 L
.42825 .28861 L
.42839 .28733 L
.42846 .2867 L
.42852 .28614 L
.42858 .28588 L
.42864 .28708 L
.42876 .28972 L
.42903 .2953 L
.4295 .3051 L
.43041 .32433 L
.43226 .36325 L
.43423 .40465 L
Mistroke
.43471 .41458 L
.43521 .42525 L
.43546 .43042 L
.43558 .43296 L
.43563 .43408 L
.43569 .43532 L
.43574 .43558 L
.43579 .43524 L
.4359 .43447 L
.43613 .43284 L
.43795 .41942 L
.43981 .40502 L
.44179 .38897 L
.44369 .37285 L
.44552 .3568 L
.44747 .33917 L
.44935 .32173 L
.4503 .31268 L
.45134 .30275 L
.45182 .29807 L
.45209 .29551 L
.45222 .29425 L
.45228 .2937 L
.45234 .29309 L
.45239 .29295 L
.45245 .29404 L
.45255 .29621 L
.45278 .30099 L
.45326 .31114 L
.45514 .3507 L
.45714 .39273 L
.45815 .41383 L
.45859 .42308 L
.45907 .43324 L
.45929 .43797 L
.45942 .44057 L
.45948 .44185 L
.45954 .4427 L
.45959 .4423 L
.45964 .44186 L
.45976 .44094 L
.45997 .43933 L
.46092 .43181 L
.46274 .41693 L
.46467 .40036 L
.46653 .38376 L
.46851 .36549 L
.47042 .34733 L
.47225 .32941 L
.4742 .30991 L
Mistroke
.4751 .30077 L
.47558 .29593 L
.47584 .29327 L
.47596 .29197 L
.47608 .29078 L
.47614 .29013 L
.4762 .28992 L
.47627 .29124 L
.47633 .29269 L
.47661 .29857 L
.47711 .30892 L
.47808 .32929 L
.48003 .37043 L
.48192 .41004 L
.48241 .42047 L
.48268 .42595 L
.48295 .43173 L
.48319 .4368 L
.48326 .43819 L
.48329 .43895 L
.48333 .43968 L
.48336 .43955 L
.4834 .43926 L
.48346 .43876 L
.48392 .43491 L
.48589 .41813 L
.48778 .40124 L
.48959 .3844 L
.49153 .36587 L
.49339 .34752 L
.49538 .32751 L
.49728 .30783 L
.49912 .28857 L
.49935 .2861 L
.49961 .28342 L
.49972 .28219 L
.49985 .28089 L
.4999 .28035 L
.49995 .27978 L
.50001 .27944 L
.50007 .28061 L
.50059 .29172 L
.50107 .3018 L
.50306 .34358 L
.50498 .38383 L
.50585 .40222 L
.50635 .4127 L
.50659 .41783 L
.50682 .42254 L
.50704 .42722 L
Mistroke
.50709 .42836 L
.50715 .42928 L
.50722 .42873 L
.50728 .42823 L
.50777 .42398 L
.50878 .41512 L
.51077 .39704 L
.51269 .37897 L
.51454 .36103 L
.51651 .34141 L
.5184 .32207 L
.52022 .30311 L
.52124 .29244 L
.52216 .28261 L
.52262 .27772 L
.52312 .27238 L
.52335 .26993 L
.52359 .26732 L
.52365 .26671 L
.52371 .26605 L
.52377 .26543 L
.52382 .26522 L
.52387 .26623 L
.52392 .26735 L
.52403 .26959 L
.52597 .31048 L
.52785 .34983 L
.5288 .36987 L
.52984 .39166 L
.53032 .40182 L
.53059 .40738 L
.53071 .4101 L
.53083 .4126 L
.53089 .41372 L
.53094 .41476 L
.53099 .41479 L
.53104 .4143 L
.53127 .41232 L
.53175 .40809 L
.53363 .3912 L
.53563 .37258 L
.53755 .35406 L
.53941 .33576 L
.54138 .31585 L
.54327 .2963 L
.54509 .27722 L
.54611 .2665 L
.54704 .25665 L
.54726 .25428 L
.54749 .25176 L
Mistroke
.54753 .2514 L
.54756 .25101 L
.5476 .25065 L
.54763 .25068 L
.54769 .25194 L
.54775 .25327 L
.54799 .25833 L
.5489 .27744 L
.55073 .3159 L
.55268 .35684 L
.55358 .37573 L
.55405 .38569 L
.55456 .39624 L
.55468 .39881 L
.55471 .39955 L
.55475 .40025 L
.55481 .40013 L
.55485 .39982 L
.55488 .39954 L
.55495 .39889 L
.55509 .39772 L
.55558 .39345 L
.55655 .38491 L
.55843 .36786 L
.56043 .34916 L
.56236 .33062 L
.56421 .31237 L
.56618 .29258 L
.56808 .27321 L
.56991 .25435 L
.57085 .24459 L
.57112 .24178 L
.57125 .24039 L
.57131 .23978 L
.57137 .2391 L
.57143 .23852 L
.57148 .23965 L
.5716 .24214 L
.57185 .24739 L
.57371 .28649 L
.57569 .32806 L
.57668 .34881 L
.5776 .36811 L
.57805 .37768 L
.57827 .38239 L
.57837 .38445 L
.57848 .38672 L
.57854 .38798 L
.5786 .38842 L
.57866 .38792 L
Mistroke
.57872 .38739 L
.57898 .38522 L
.57943 .38139 L
.58141 .36416 L
.58332 .34698 L
.58516 .32998 L
.58712 .31144 L
.589 .29322 L
.59101 .27351 L
.59294 .25426 L
.59382 .24535 L
.59432 .24027 L
.59479 .23554 L
.59503 .23312 L
.59509 .23254 L
.59515 .23193 L
.5952 .23141 L
.59526 .23137 L
.59529 .23202 L
.59532 .23272 L
.59538 .23399 L
.59552 .23683 L
.59575 .24181 L
.59677 .26311 L
.59875 .30474 L
.60065 .34484 L
.60162 .36505 L
.60204 .37386 L
.60215 .37627 L
.60227 .37883 L
.60232 .37988 L
.60238 .38101 L
.60243 .38069 L
.60249 .38025 L
.60298 .37629 L
.60343 .37251 L
.60444 .36409 L
.60643 .34704 L
.60834 .33008 L
.61019 .31334 L
.61214 .29512 L
.61403 .27723 L
.61584 .25978 L
.61685 .25 L
.61778 .24097 L
.61825 .23638 L
.61868 .2321 L
.6189 .22993 L
.61896 .22931 L
.61902 .22872 L
Mistroke
.61908 .22914 L
.61914 .23037 L
.61964 .24084 L
.6216 .28216 L
.62349 .32195 L
.62531 .3602 L
.62579 .37027 L
.62591 .37267 L
.62603 .37523 L
.62609 .37647 L
.62614 .37764 L
.62619 .37858 L
.62625 .37816 L
.6265 .37619 L
.62677 .37404 L
.62725 .37022 L
.62923 .35415 L
.63113 .33812 L
.63295 .32225 L
.6349 .30491 L
.63677 .28784 L
.63876 .26936 L
.64068 .25128 L
.64155 .24294 L
.64205 .23818 L
.6423 .23584 L
.64252 .23369 L
.64274 .23155 L
.6428 .23103 L
.64285 .23048 L
.64292 .23174 L
.64298 .23298 L
.64347 .24335 L
.64448 .26459 L
.64648 .30655 L
.6484 .34697 L
.64928 .36549 L
.64952 .37057 L
.64978 .37602 L
.64984 .37728 L
.64991 .37866 L
.64997 .37996 L
.65003 .38036 L
.65008 .37993 L
.65014 .37953 L
.65025 .37866 L
.65121 .37129 L
.65222 .36332 L
.65415 .34767 L
.65601 .33211 L
Mistroke
.65798 .31505 L
.65988 .2982 L
.66171 .28166 L
.66366 .26374 L
.66554 .24624 L
.66603 .24162 L
.66629 .23919 L
.66643 .23785 L
.66656 .2366 L
.66662 .23604 L
.66668 .23576 L
.66671 .23641 L
.66674 .23713 L
.6668 .23837 L
.66706 .24392 L
.66753 .25369 L
.66948 .29475 L
.67136 .33429 L
.67232 .35446 L
.67283 .36508 L
.67336 .3763 L
.67359 .38114 L
.67365 .38242 L
.67372 .38381 L
.67378 .38512 L
.67384 .38548 L
.67387 .38524 L
.67391 .38497 L
.67398 .38442 L
.67411 .38345 L
.67436 .38155 L
.67529 .37451 L
.67722 .35931 L
.67908 .34413 L
.68106 .32745 L
.68296 .31091 L
.68479 .29462 L
.68674 .27692 L
.68862 .2596 L
.68958 .25068 L
.68984 .24822 L
.69012 .24562 L
.69023 .24455 L
.69035 .24339 L
.69039 .24305 L
.69043 .24273 L
.69046 .24244 L
.69049 .24253 L
.69052 .2432 L
.69056 .24391 L
Mistroke
.69062 .24518 L
.69254 .28558 L
.69454 .32764 L
.69554 .34876 L
.69647 .36817 L
.69693 .37798 L
.69737 .38714 L
.69748 .38951 L
.69754 .39067 L
.69759 .39175 L
.69765 .39215 L
.69768 .39192 L
.69771 .39168 L
.69782 .39084 L
.69832 .38711 L
.70014 .37313 L
.70208 .35763 L
.70394 .34216 L
.70592 .32518 L
.70782 .30837 L
.70966 .29183 L
.71161 .27388 L
.71251 .26548 L
.71349 .25632 L
.71375 .25388 L
.71403 .25128 L
.71414 .25021 L
.7142 .24963 L
.71426 .24907 L
.71429 .24901 L
.71433 .24977 L
.71439 .25111 L
.71452 .25373 L
.71549 .2741 L
.71745 .31527 L
.71933 .35491 L
.7203 .37516 L
.7208 .3858 L
.72108 .39161 L
.72121 .39445 L
.72127 .39568 L
.72134 .39702 L
.7214 .39833 L
.72146 .39874 L
.72151 .39833 L
.72157 .3979 L
.72182 .39604 L
.72234 .39205 L
.72327 .38493 L
.72509 .37047 L
Mistroke
.72703 .35447 L
.72889 .33854 L
.73087 .32108 L
.73278 .30383 L
.73462 .28688 L
.73564 .27729 L
.73657 .26853 L
.73704 .26411 L
.73754 .25932 L
.73777 .25712 L
.7379 .25591 L
.73796 .25532 L
.73802 .25479 L
.73807 .25429 L
.73812 .25455 L
.73817 .25564 L
.73823 .25682 L
.73846 .26163 L
.74042 .30285 L
.74231 .34254 L
.74412 .3807 L
.74457 .39023 L
.74482 .39542 L
.74505 .40026 L
.74517 .40279 L
.74521 .40353 L
.74524 .40415 L
.7453 .40369 L
.74558 .40161 L
.74605 .39794 L
.7479 .38336 L
.74987 .36719 L
.75177 .35103 L
.75359 .335 L
.75553 .31746 L
.7574 .30018 L
.75938 .28144 L
.76037 .27195 L
.76086 .2673 L
.76129 .26308 L
.76152 .26094 L
.76163 .25982 L
.76175 .25863 L
.76181 .25813 L
.76186 .25759 L
.76192 .25753 L
.76198 .25872 L
.76219 .26307 L
.76313 .28301 L
.76513 .32486 L
Mistroke
.76704 .36519 L
.76792 .38364 L
.76842 .39414 L
.76867 .39928 L
.76877 .40153 L
.76889 .40399 L
.76895 .40517 L
.76901 .40642 L
.76906 .40717 L
.76911 .40677 L
.76935 .40496 L
.76984 .40111 L
.77085 .39309 L
.77278 .3773 L
.77463 .36152 L
.7766 .34416 L
.7785 .32693 L
.78033 .30996 L
.78227 .29151 L
.78414 .27344 L
.78463 .26868 L
.78516 .26347 L
.7854 .26108 L
.78553 .25977 L
.7856 .25913 L
.78566 .25854 L
.78571 .25803 L
.78577 .2591 L
.78588 .26151 L
.78613 .26664 L
.78796 .30512 L
.78991 .34609 L
.79178 .38552 L
.79227 .39586 L
.79253 .4013 L
.79267 .40428 L
.79274 .40574 L
.79281 .40707 L
.79287 .40805 L
.79292 .40761 L
.79305 .40663 L
.79331 .40455 L
.79378 .40086 L
.79561 .38587 L
.79757 .36926 L
.79945 .35269 L
.80145 .33453 L
.80338 .31656 L
.80524 .29887 L
.80721 .27971 L
Mistroke
.80819 .27005 L
.80867 .26531 L
.80911 .26097 L
.80933 .25882 L
.80939 .25823 L
.80945 .2576 L
.8095 .25706 L
.80956 .25766 L
.80962 .25883 L
.80967 .2599 L
.80978 .26231 L
.80999 .26668 L
.81094 .28654 L
.81284 .32661 L
.81468 .36515 L
.8157 .38665 L
.81618 .39681 L
.8164 .40127 L
.81652 .4038 L
.81657 .40504 L
.81663 .40617 L
.81668 .40683 L
.81674 .40641 L
.81685 .40549 L
.81709 .40359 L
.8176 .39955 L
.81851 .39209 L
.82035 .3766 L
.82231 .3595 L
.82419 .34249 L
.82619 .3239 L
.82813 .30554 L
.82998 .28753 L
.83093 .27828 L
.83196 .26806 L
.83243 .26333 L
.8327 .26072 L
.83294 .25825 L
.83317 .25595 L
.83324 .25532 L
.83327 .25497 L
.8333 .25465 L
.83336 .25492 L
.83342 .25624 L
.83386 .26543 L
.83572 .30461 L
.83771 .34626 L
.8387 .36707 L
.83962 .38639 L
.84007 .396 L
Mistroke
.8403 .40072 L
.8404 .40279 L
.84045 .40397 L
.8405 .40424 L
.84056 .40379 L
.84062 .40331 L
.84074 .40233 L
.841 .40026 L
.84145 .3966 L
.84344 .38004 L
.84536 .36344 L
.8472 .34695 L
.84916 .32889 L
.85105 .31106 L
.85286 .29357 L
.8548 .27462 L
.85568 .26582 L
.85619 .26081 L
.85666 .25613 L
.8569 .25373 L
.85701 .25255 L
.85706 .25204 L
.85712 .25147 L
.85718 .25209 L
.85722 .25279 L
.85725 .25344 L
.85738 .25629 L
.85762 .26127 L
.85863 .2826 L
.8605 .32183 L
.86249 .36354 L
.86348 .38438 L
.86373 .38961 L
.86396 .39452 L
.86417 .39894 L
.86423 .40015 L
.86429 .40135 L
.86434 .40094 L
.8644 .40048 L
.86529 .39329 L
.86624 .38546 L
.86818 .36892 L
.87005 .35242 L
.87205 .33434 L
.87396 .31645 L
.87581 .29885 L
.87777 .27978 L
.87875 .27019 L
.87966 .26114 L
.88009 .2569 L
Mistroke
.88054 .25243 L
.88077 .25008 L
.88084 .24944 L
.88087 .24909 L
.88091 .24875 L
.88094 .24844 L
.88097 .2487 L
.88103 .24993 L
.88148 .2594 L
.88345 .3008 L
.88535 .34067 L
.88717 .379 L
.88765 .38911 L
.88789 .39408 L
.88799 .39627 L
.88805 .39744 L
.88811 .39831 L
.88817 .39781 L
.88823 .39734 L
.88836 .3963 L
.88863 .3941 L
.88911 .39022 L
.89109 .37378 L
.89299 .35735 L
.89482 .34103 L
.89677 .32318 L
.89865 .30557 L
.90064 .28647 L
.90256 .26775 L
.90344 .2591 L
.90394 .25415 L
.90419 .25173 L
.90441 .24951 L
.90452 .24839 L
.90465 .24717 L
.9047 .24661 L
.90476 .24604 L
.90482 .24723 L
.90487 .24831 L
.90537 .2587 L
.90638 .27996 L
.9083 .32047 L
.91016 .35944 L
.9111 .3792 L
.91136 .38462 L
.91163 .39042 L
.91177 .39323 L
.91183 .39462 L
.91189 .3959 L
.91195 .3958 L
Mistroke
.91201 .39533 L
.91213 .39435 L
.91403 .37893 L
.91585 .36362 L
.91778 .34675 L
.91964 .33002 L
.92162 .31176 L
.92353 .29378 L
.92536 .27617 L
.92638 .26624 L
.92731 .25715 L
.92777 .2526 L
.92802 .2502 L
.92828 .24765 L
.92839 .24658 L
.9285 .24542 L
.92854 .24508 L
.92857 .24476 L
.92863 .24599 L
.92875 .24852 L
.92919 .25771 L
.93103 .29638 L
.93299 .33752 L
.93487 .37713 L
.93537 .38756 L
.9355 .39042 L
.93558 .39197 L
.93565 .39343 L
.93571 .39467 L
.93577 .39443 L
.9358 .39415 L
.93584 .39388 L
.9359 .39338 L
.93687 .38565 L
.93872 .37049 L
.94068 .35375 L
.94258 .33709 L
.94439 .32063 L
.94633 .30269 L
.94819 .28507 L
.95018 .26602 L
.95116 .2564 L
.95165 .2517 L
.95208 .24742 L
.95219 .24637 L
.9523 .24526 L
.95236 .24468 L
.9524 .24481 L
.95243 .24547 L
.95254 .24787 L
Mistroke
.95297 .25692 L
.95392 .27684 L
.95579 .31614 L
.95778 .35792 L
.95877 .37882 L
.95921 .38801 L
.95933 .39053 L
.95939 .39186 L
.95946 .39325 L
.95952 .39448 L
.95955 .39442 L
.95958 .39418 L
.95969 .3933 L
.96059 .38625 L
.96153 .3786 L
.96341 .36302 L
.96541 .34586 L
.96733 .3288 L
.96918 .31196 L
.97114 .29363 L
.97304 .27565 L
.97486 .25811 L
.97619 .24514 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  Background->GrayLevel[1],
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005X0000gR000`400?l00000o`00003oo`3ooomY
0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`006P3oool00`000000oooo0?oo
o`3o0?oood`0oooo000J0?ooo`030000003oool0oooo0?l0ooooC03oool001X0oooo00<000000?oo
o`3oool0o`3ooom<0?ooo`00100000050?ooo`8000001`3oool3000000D0oooo00<000000?ooo`3o
ool0o`3ooom<0?ooo`00103oool00`000000oooo0?ooo`020?ooo`800000203oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo0?l0ooooC03oool000@0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`040?ooo`030?l0003oool0oooo0?l0ooooC03oool000@0oooo00<00000
0?ooo`3oool0203oool6000000D0oooo00<0o`00000000000000o`3ooom<0?ooo`00103oool00`00
0000oooo0?ooo`090?ooo`040000003oool0oooo000000H0oooo00<0o`000?ooo`3oool0o`3ooom<
0?ooo`00103oool00`000000oooo0?ooo`090?ooo`040000003oool0oooo000000H0oooo00<0o`00
0?ooo`3oool0o`3ooom<0?ooo`00103oool00`000000oooo0?ooo`0:0?ooo`030000003oool00000
00H0oooo00<0o`000?ooo`3oool0o`3ooom<0?ooo`001000000>0?ooo`8000001P3oool00`3o0000
oooo0?ooo`3o0?oood`0oooo000J0?ooo`030?l0003oool0oooo0?l0ooooC03oool001X0oooo00<0
00000?l0003oool0o`3ooom<0?ooo`006P3oool00`000000o`000?ooo`3o0?oood`0oooo000J0?oo
o`030000003o0000oooo0?l0ooooC03oool001X0oooo00<000000?l0003oool0o`3ooom<0?ooo`00
6P3oool00`000000o`000?ooo`3o0?oood`0oooo000J0?ooo`030000003o0000oooo0?l0ooooC03o
ool001X0oooo00<000000?l0003oool0o`3ooom<0?ooo`006P3oool00`000000o`000?ooo`3o0?oo
od`0oooo000J0?ooo`030000003o0000oooo0?l0ooooC03oool001X0oooo00<000000?l0003oool0
o`3ooom<0?ooo`006P3oool00`000000o`000?ooo`3o0?oood`0oooo000J0?ooo`030000003o0000
oooo0?l0ooooC03oool001X0oooo00<000000?l0003oool0o`3ooom<0?ooo`006P3oool00`000000
o`000?ooo`3o0?oood`0oooo000J0?ooo`030000003oool0o`000?l0ooooC03oool001X0oooo00<0
00000?ooo`3o0000o`3ooom<0?ooo`006P3oool00`000000oooo0?l0003o0?oood`0oooo000J0?oo
o`030000003oool0o`000?l0ooooC03oool001X0oooo00<000000?ooo`3o0000o`3ooom<0?ooo`00
6P3oool2000000030?l0003oool0oooo0?l0ooooBP3oool001X0oooo00<000000?ooo`3o0000o`3o
oom<0?ooo`006P3oool00`000000oooo0?l0003o0?oood`0oooo000J0?ooo`030000003oool0o`00
0?l0ooooC03oool001X0oooo00<000000?ooo`3o0000o`3ooom<0?ooo`00100000050?ooo`800000
1P3oool3000000H0oooo00<000000?ooo`3o0000o`3ooom<0?ooo`00103oool00`000000oooo0?oo
o`020?ooo`8000001@3oool01@000000oooo0?ooo`3oool0000000D0oooo00<000000?ooo`3o0000
o`3ooom<0?ooo`00103oool00`000000oooo0?ooo`090?ooo`050000003oool0oooo0?ooo`000000
1@3oool00`000000oooo0?l0003o0?oood`0oooo00040?ooo`030000003oool0oooo00T0oooo00D0
00000?ooo`3oool0oooo000000050?ooo`80000000<0o`000?ooo`3oool0o`3ooom:0?ooo`00103o
ool00`000000oooo0?ooo`090?ooo`@000001P3oool00`000000oooo0?l0003o0?oood`0oooo0004
0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?l0003o
0?oood`0oooo00040?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?l0003o0?oood`0oooo0004000000h0oooo0`0000050?ooo`030000003oool0o`000?l0
ooooC03oool001X0oooo00<000000?ooo`3o0000o`3ooom<0?ooo`006P3oool00`000000oooo0?l0
003o0?oood`0oooo000J0?ooo`030000003oool0o`000?l0ooooC03oool001X0oooo00<000000?oo
o`3o0000o`3ooom<0?ooo`006P3oool2000000030?l0003oool0oooo0?l0ooooBP3oool001X0oooo
00<000000?ooo`3o0000o`3ooom<0?ooo`006P3oool00`000000oooo0?l0003o0?oood`0oooo000J
0?ooo`030000003oool0o`000?l0ooooC03oool001X0oooo00<000000?ooo`3o0000E@3oool20?l0
0?@0oooo000J0?ooo`030000003oool0o`0005D0oooo0P3o00060?ooo`80o`00k03oool001X0oooo
00@000000?ooo`3oool0o`00E03oool20?l000H0oooo0P3o003/0?ooo`006P3oool010000000oooo
0?ooo`3o001D0?ooo`80o`001P3oool20?l00>`0oooo000J0?ooo`80000000<0oooo0?l0003oool0
D`3oool20?l000H0oooo0P3o003/0?ooo`006P3oool010000000oooo0?ooo`3oool50?l004l0oooo
0P3o00060?ooo`80o`00k03oool001X0oooo00<000000?ooo`3oool01@3oool00`3o0000oooo0?oo
o`150?ooo`030?l0003oool0oooo00D0oooo0P3o00060?ooo`80o`00k03oool001X0oooo00<00000
0?ooo`3oool01@3oool00`3o0000oooo0?ooo`150?ooo`80o`001P3oool20?l000H0oooo0P3o003/
0?ooo`006P3oool00`000000oooo0?ooo`050?ooo`030?l0003oool0oooo04D0oooo0P3o00060?oo
o`80o`001@3oool00`3o0000oooo0?l000060?ooo`80o`00i03oool001X0oooo00<000000?ooo`3o
ool01P3oool00`3o0000oooo0?ooo`140?ooo`80o`001P3oool20?l000D0oooo00<0o`000?ooo`3o
00001P3oool20?l00>@0oooo000J0?ooo`030000003oool0oooo00H0oooo00<0o`000?ooo`3oool0
A03oool20?l000H0oooo0P3o00050?ooo`030?l0003oool0o`0000H0oooo0P3o003T0?ooo`006P3o
ool00`000000oooo0?ooo`060?ooo`030?l0003oool0oooo04@0oooo0P3o00050?ooo`030?l0003o
ool0o`0000D0oooo00<0o`000?ooo`3o00001P3oool20?l00>@0oooo000J0?ooo`030000003oool0
oooo00H0oooo00<0o`000?ooo`3oool0A03oool20?l000D0oooo00<0o`000?ooo`3o00001@3oool0
0`3o0000oooo0?l000060?ooo`80o`00i03oool001X0oooo0P0000070?ooo`030?l0003oool0oooo
04@0oooo0P3o00050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001P3oool20?l0
0>@0oooo000J0?ooo`030000003oool0oooo00H0oooo00<0o`000?ooo`3oool0A03oool20?l000D0
oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000060?ooo`80o`00i03oool001X0oooo
00<000000?ooo`3oool01P3oool00`3o0000oooo0?ooo`140?ooo`80o`001@3oool00`3o0000oooo
0?l000050?ooo`030?l0003oool0o`0000H0oooo0P3o003T0?ooo`006P3oool00`000000oooo0?oo
o`060?ooo`030?l0003oool0oooo04@0oooo0P3o00050?ooo`030?l0003oool0o`0000D0oooo00<0
o`000?ooo`3o00001P3oool20?l00>@0oooo000J0?ooo`030000003oool0oooo00H0oooo00<0o`00
0?ooo`3oool0@`3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`00
0?ooo`3o00001@3oool00`3o0000oooo0?l0003T0?ooo`00100000050?ooo`8000001@3oool40000
00H0oooo00<000000?ooo`3oool01P3oool00`3o0000oooo0?ooo`130?ooo`030?l0003oool0o`00
00@0oooo00@0o`000?ooo`3oool0o`001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0
o`0000L0oooo00<0o`000?ooo`3oool0fP3oool000@0oooo00<000000?ooo`3oool00P3oool20000
00@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
00H0oooo00<0o`000?ooo`3oool0@`3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0oooo
0?ooo`3o00000`3oool0103o0000oooo0?ooo`3o00050?ooo`030?l0003oool0o`0000H0oooo0P3o
003L0?ooo`00103oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`060?ooo`030?l0003oool0oooo03`0oooo00<0o`00
0?ooo`3oool0103oool00`3o0000oooo0?l000040?ooo`050?l0003oool0oooo0?ooo`3o00000`3o
ool0103o0000oooo0?ooo`3o00050?ooo`030?l0003oool0o`0000H0oooo0P3o003L0?ooo`00103o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3o
ool3000000H0oooo00<0o`000?ooo`3oool0?03oool00`3o0000oooo0?ooo`040?ooo`030?l0003o
ool0o`0000@0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`040?l0003oool0oooo0?l000D0
oooo00<0o`000?ooo`3o00001P3oool20?l00=`0oooo00040?ooo`030000003oool0oooo00T0oooo
100000060?ooo`030000003oool0oooo00H0oooo00<0o`000?ooo`3oool0?03oool00`3o0000oooo
0?ooo`040?ooo`030?l0003oool0o`0000@0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`04
0?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`001P3oool20?l00=`0oooo00040?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo00H0oooo00<0o`000?ooo`3oool0?03oool20?l000D0oooo00<0o`000?oo
o`3o0000103oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00@0o`000?ooo`3oool0o`00103o
ool0103o0000oooo0?ooo`3o00060?ooo`80o`00g03oool000@0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01P3o
ool00`3o0000oooo0?ooo`0l0?ooo`80o`001@3oool00`3o0000oooo0?l000040?ooo`050?l0003o
ool0oooo0?ooo`3o00000`3oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l0
00H0oooo0P3o003L0?ooo`001000000<0?ooo`@000001P3oool00`000000oooo0?ooo`060?ooo`03
0?l0003oool0oooo03`0oooo0P3o00050?ooo`030?l0003oool0o`0000@0oooo00D0o`000?ooo`3o
ool0oooo0?l000030?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`001P3o
ool20?l005L0oooo00<0o`000?ooo`3oool0PP3oool001X0oooo00<000000?ooo`3oool01P3oool0
0`3o0000oooo0?ooo`0l0?ooo`80o`001@3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0
oooo0?ooo`3o00000`3oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000H0
oooo0P3o001?0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000
oooo0?ooo`1j0?ooo`006P3oool00`000000oooo0?ooo`060?ooo`030?l0003oool0oooo03`0oooo
0P3o00050?ooo`030?l0003oool0o`0000@0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`05
0?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000D0oooo0P3o001?
0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`05
0?ooo`030?l0003oool0oooo0780oooo000J0?ooo`030000003oool0oooo00H0oooo00<0o`000?oo
o`3oool0?03oool20?l000D0oooo00<0o`000?ooo`3o0000103oool01@3o0000oooo0?ooo`3oool0
o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o0000
1@3oool20?l004L0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0
003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`1b0?ooo`006P3o
ool00`000000oooo0?ooo`060?ooo`030?l0003oool0oooo03`0oooo0P3o00050?ooo`030?l0003o
ool0o`0000@0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o
00000`3oool01@3o0000oooo0?ooo`3oool0o`0000@0oooo00<0o`000?ooo`3o00001`3oool00`3o
0000oooo0?ooo`0m0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0103oool20?l0
00H0oooo0P3o00070?ooo`030?l0003oool0oooo0780oooo000J0?ooo`8000001`3oool00`3o0000
oooo0?ooo`0l0?ooo`80o`001@3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0oooo0?oo
o`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l0
00040?ooo`030?l0003oool0o`0000H0oooo0P3o000o0?ooo`030?l0003oool0oooo00@0oooo0P3o
00060?ooo`80o`001P3oool20?l000L0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?oo
o`1=0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool04P3oool001X0oooo00<00000
0?ooo`3oool01P3oool00`3o0000oooo0?ooo`0l0?ooo`80o`00103oool01@3o0000oooo0?ooo`3o
ool0o`000080oooo00<0o`000?ooo`3oool00P3oool0103o0000oooo0?ooo`3o00040?ooo`050?l0
003oool0oooo0?ooo`3o00000`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0000H0
oooo0P3o000o0?ooo`030?l0003oool0oooo00@0oooo0P3o00060?ooo`80o`001P3oool20?l000L0
oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`150?ooo`030?l0003oool0oooo00D0
oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00X0
oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`000?ooo`3oool0>`3oool20?l000@0oooo
00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo0080oooo00@0o`000?ooo`3o
ool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`030?l0003oool0oooo0080oooo00<0o`00
0?ooo`3o00001P3oool20?l003L0oooo00<0o`000?ooo`3oool0103oool20?l000H0oooo0P3o0006
0?ooo`80o`001P3oool20?l000L0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`05
0?ooo`030?l0003oool0oooo03D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`05
0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`0:
0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030?l0003oool0oooo03/0oooo0P3o00040?oo
o`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`040?l0003oool0
oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool00`3o0000oooo0?ooo`020?ooo`030?l0
003oool0o`0000H0oooo0P3o000g0?ooo`030?l0003oool0oooo00@0oooo0P3o00060?ooo`80o`00
1P3oool20?l000H0oooo0P3o00070?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0
1@3oool00`3o0000oooo0?ooo`0]0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0
1@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0
1@3oool00`3o0000oooo0?ooo`0:0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030?l0003o
ool0oooo03/0oooo0P3o00040?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo
0?ooo`020?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool01@3o
0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool0103oool20?l003L0oooo00<0o`00
0?ooo`3oool0103oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o0006
0?ooo`80o`001`3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo01d0oooo00<0o`00
0?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`00
0?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`00
0?ooo`3oool02P3oool001X0oooo00<000000?ooo`3oool01`3oool00`3o0000oooo0?ooo`0k0?oo
o`80o`00103oool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool00P3oool0
103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00D0o`000?ooo`3oool0
oooo0?l000020?ooo`030?l0003oool0oooo00@0oooo0P3o000g0?ooo`030?l0003oool0oooo00@0
oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000L0
oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0
oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0
oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0
oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00X0
oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`000?ooo`3oool0>P3oool00`3o0000oooo
0?l000040?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`04
0?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?ooo`3o
ool0o`000080oooo00<0o`000?ooo`3oool0103oool20?l003H0oooo0P3o00060?ooo`80o`001P3o
ool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001`3o
ool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3o
ool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3o
ool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0103o
ool20?l000L0oooo00<0o`000?ooo`3oool02P3oool001X0oooo00<000000?ooo`3oool01`3oool0
0`3o0000oooo0?ooo`0j0?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo
0?l000020?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo
0?ooo`3o00040?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`030?oo
o`030?l0003oool0o`0002l0oooo00<0o`000?ooo`3oool0103oool20?l000H0oooo0P3o00050?oo
o`030?l0003oool0o`0000H0oooo00<0o`000?ooo`3o00001@3oool20?l000H0oooo0P3o00060?oo
o`80o`001P3oool20?l000L0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?oo
o`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?oo
o`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0103oool20?l000H0oooo0P3o00060?oo
o`80o`001`3oool00`3o0000oooo0?ooo`0:0?ooo`006P3oool2000000P0oooo00<0o`000?ooo`3o
ool0>P3oool0103o0000oooo0?ooo`3o00020?ooo`030?l0003oool0oooo0080oooo00@0o`000?oo
o`3oool0o`00103oool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000@0oooo00D0
o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo00<0oooo00<0o`000?ooo`3o0000
1`3oool00`3o0000oooo0?ooo`0U0?ooo`030?l0003oool0oooo00@0oooo0P3o00060?ooo`030?l0
003oool0o`0000@0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00050?ooo`80
o`001P3oool20?l000H0oooo00<0o`000?ooo`3o00001@3oool20?l000H0oooo0P3o00070?ooo`03
0?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`03
0?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0103oool20?l000H0oooo0P3o00060?ooo`80
o`001P3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0oooo00/0oooo000J0?ooo`030000
003oool0oooo00L0oooo00<0o`000?ooo`3oool0>P3oool0103o0000oooo0?ooo`3o00020?ooo`03
0?l0003oool0oooo0080oooo00@0o`000?ooo`3oool0o`00103oool00`3o0000oooo0?l000050?oo
o`040?l0003oool0oooo0?l000@0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003o
ool0oooo00<0oooo00<0o`000?ooo`3o00001`3oool00`3o0000oooo0?ooo`0U0?ooo`030?l0003o
ool0oooo00@0oooo0P3o00060?ooo`030?l0003oool0o`0000@0oooo00<0o`000?ooo`3o00001@3o
ool0103o0000oooo0?ooo`3o00050?ooo`80o`001P3oool20?l000H0oooo00<0o`000?ooo`3o0000
1@3oool20?l000H0oooo0P3o00070?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0
1@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00@0oooo0P3o00060?ooo`80o`00
1P3oool00`3o0000oooo0?l000050?ooo`80o`001P3oool00`3o0000oooo0?l000050?ooo`030?l0
003oool0oooo00/0oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`000?ooo`3oool0>@3o
ool01@000000o`00000000000000o`000080oooo00H0o`000?ooo`00000000000?ooo`3o00020?oo
o`030?l00000000000000080000000<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o0004
0?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`030?ooo`030?l0003o
ool0o`0000H0oooo0P3o000E0?ooo`@000001@3oool2000000@0oooo0`0000000`3o000000000?oo
o`040?ooo`80o`001P3oool00`3o0000oooo0?l000040?ooo`030?l0003oool0o`0000D0oooo00@0
o`000?ooo`3oool0o`001@3oool20?l000D0oooo00<0o`000?ooo`3o00001P3oool00`3o0000oooo
0?l000040?ooo`040000003o0000o`00000000D0oooo0P3o00050?ooo`80000000<0o`000?ooo`3o
ool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00@0oooo0P3o00060?ooo`03
0?l0003oool0o`0000D0oooo0P3o00060?ooo`030?l0003oool0o`0000D0oooo0P3o00050?ooo`04
0000003o000000000?l000D0oooo00<0o`000000003oool01P3oool300000080oooo000J0?ooo`03
0000003oool0oooo00L0oooo00<0o`000?ooo`3oool0<`3oool00`3o0000oooo0?ooo`020?ooo`03
0000003oool0o`000080oooo00D0o`000?ooo`3oool0o`000?ooo`02000000050?ooo`3o0000oooo
0?ooo`3o00000P3oool01@000000oooo0?l0003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`00
103oool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool00`3oool00`3o0000
oooo0?l000060?ooo`80o`00503oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080
oooo0P0000040?ooo`050000003oool0o`000?l0000000001@3oool20?l000H0oooo00<0o`000?oo
o`3o0000103oool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000D0oooo0P3o0005
0?ooo`040?l0003oool0oooo0?l000D0oooo00<0o`000?ooo`3o00000`3oool01P000000oooo0?l0
003o0000oooo000000@0oooo0P3o00040?ooo`050000003oool0oooo0?l0000000001P3oool00`3o
0000oooo0?ooo`040?ooo`80o`001P3oool20?l000H0oooo00<0o`000?ooo`3o00001@3oool20?l0
00H0oooo00<0o`000?ooo`3o00001@3oool20?l000@0oooo00H000000?ooo`3o0000oooo0?l00000
00040?ooo`030?l000000000oooo00L0oooo00<000000?ooo`3oool00@3oool001X0oooo00<00000
0?ooo`3oool01`3oool00`3o0000oooo0?ooo`0c0?ooo`030?l0003oool0oooo0080oooo00<00000
0?ooo`3o00000P3oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l00080oooo
00<000000?ooo`3oool00P3o00050?ooo`030?l0003oool0o`0000D0oooo00D0o`000?ooo`3oool0
oooo0?l000020?ooo`030?l0003oool0oooo00<0oooo00<0o`000?ooo`3o00001P3oool20?l001@0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030?l0003oool0oooo00<0
oooo00<000000?l0003o00001P3oool20?l000H0oooo00<0o`000?ooo`3o0000103oool00`3o0000
oooo0?l000050?ooo`040?l0003oool0oooo0?l000@0oooo00<0o`000?ooo`3o00001@3oool0103o
0000oooo0?ooo`3o00050?ooo`030?l0003oool0o`0000<0oooo00H000000?ooo`3o0000o`000?oo
o`0000040?ooo`80o`001P3oool20?l000030000003oool0oooo00@0oooo00<0o`000?ooo`3oool0
103oool00`3o0000oooo0?l000050?ooo`80o`001P3oool00`3o0000oooo0?l000050?ooo`80o`00
1P3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000<0oooo00H000000?ooo`3o0000
oooo0?l0000000040?ooo`030?l0003oool0oooo00L0oooo00<000000?ooo`3oool00@3oool001X0
oooo00<000000?ooo`3oool01`3oool00`3o0000oooo0?ooo`0c0?ooo`80o`000`3oool00`000000
oooo0?l000020?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`000P3oool0
0`000000oooo0?ooo`020?l000D0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o
00030?ooo`030?l0003oool0oooo00<0oooo00<0o`000?ooo`3o00001P3oool20?l001@0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030?l0003oool0oooo00@0oooo0P3o
00060?ooo`80o`001P3oool00`3o0000oooo0?l000040?ooo`030?l0003oool0o`0000D0oooo00@0
o`000?ooo`3oool0o`00103oool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000D0
oooo00<0o`000?ooo`3o00000`3oool01P000000o`000?ooo`3o0000oooo000000@0oooo0P3o0006
0?ooo`80o`0000<000000?ooo`3oool00`3oool20?l000H0oooo00<0o`000?ooo`3o00001@3oool2
0?l000H0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0
o`0000D0oooo00<0o`000?ooo`3o00000`3oool01P000000oooo0?l0003oool0o`00000000@0oooo
00<0o`000?ooo`3oool00`3oool600000080oooo000J0?ooo`030000003oool0oooo00L0oooo00<0
o`000?ooo`3oool0<`3oool20?l000<0oooo00<000000?l0003oool00P3oool0103o0000oooo0?oo
o`3o00050?ooo`80o`000`3oool00`000000oooo0?ooo`020?l000H0oooo0P3o00050?ooo`040?l0
003oool0oooo0?l000@0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l000060?ooo`80
o`001`3oool00`3o0000oooo0?ooo`0:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool00`3oool00`3o0000oooo0?ooo`040?ooo`80o`001P3oool20?l000D0oooo00@0o`000?ooo`3o
ool0o`00103oool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000@0oooo00<0o`00
0?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000<0oooo
00H000000?l0003oool0o`000?ooo`0000040?ooo`030?l0003oool0o`0000D0oooo0P3o00060?oo
o`80o`001P3oool00`3o0000oooo0?l000050?ooo`80o`001P3oool00`3o0000oooo0?l000050?oo
o`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o0003
0?ooo`060000003oool0o`000?ooo`3o00000000103oool00`3o0000oooo0?ooo`040?ooo`040000
003oool0oooo000000<0oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`000?ooo`3oool0
<`3oool20?l000<0oooo00<000000?l0003oool00P3oool0103o0000oooo0?ooo`3o00050?ooo`80
o`000`3oool00`000000oooo0?ooo`020?l000H0oooo0P3o00050?ooo`040?l0003oool0oooo0?l0
00@0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l000060?ooo`80o`001`3oool00`3o
0000oooo0?ooo`0:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool20?l0
00H0oooo0P3o00000`000000oooo0?ooo`020?ooo`030?l0003oool0o`0000D0oooo00@0o`000?oo
o`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo
00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo
0?l000<0oooo00H000000?l0003oool0o`000?ooo`0000040?ooo`030?l0003oool0o`0000D0oooo
0P3o00000`000000oooo0?ooo`030?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o0000
1@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o0000
1@3oool00`3o0000oooo0?l000040?ooo`040?l0003oool0oooo0?l000<0oooo00H000000?ooo`3o
0000oooo0?l0000000040?ooo`030?l0003oool0oooo00@0oooo00@000000?ooo`3oool000000`3o
ool001X0oooo0P0000080?ooo`030?l0003oool0oooo03<0oooo0P3o00030?ooo`030000003o0000
oooo0080oooo00@0o`000?ooo`3oool0o`001@3oool20?l000040?ooo`000000000000000080oooo
00<0o`000?ooo`3oool01@3oool20?l000D0oooo00@0o`000?ooo`3oool0o`00103oool00`3o0000
oooo0?ooo`020?ooo`030?l0003oool0o`0000H0oooo0P3o00060?ooo`80o`00303oool01P000000
oooo0?ooo`3o0000oooo000000@0oooo0P3o00040?ooo`050000003oool0o`000?l000000000103o
ool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0
o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`00
0?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00030?ooo`060000003o0000oooo0?l0003o
ool00000103oool00`3o0000oooo0?l000030?ooo`050000003o0000oooo0?l0000000001@3oool0
0`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool0
0`3o0000oooo0?l000040?ooo`040?l0003oool0oooo0?l000D0oooo00<0o`000?ooo`3o0000103o
ool0103o0000oooo0?ooo`3o00030?ooo`060000003oool0o`000?ooo`3o00000000103oool00`3o
0000oooo0?ooo`050?ooo`030000003oool0000000<0oooo000J0?ooo`030000003oool0oooo00L0
oooo00<0o`000?ooo`3oool0<`3oool20?l000@0oooo00D0o`0000000000000000000?l000020?oo
o`030?l0003oool0oooo00<0oooo0P3o00030?ooo`040000003oool0oooo0?l000L0oooo00<0o`00
0?ooo`3oool0103oool0103o0000oooo0?ooo`3o00040?ooo`050?l0003oool0oooo0?ooo`3o0000
0P3oool00`3o0000oooo0?ooo`040?ooo`80o`001P3oool20?l000L0oooo00<0o`000?ooo`3oool0
0`3oool2000000030?l000000000oooo00@0oooo0P3o00050?ooo`030000003o0000o`0000D0oooo
00<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l0
00@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003o
ool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o000000000?l0000000050?oo
o`030?l0003oool0o`0000@0oooo00<0o`000000003o00001P3oool00`3o0000oooo0?l000050?oo
o`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000040?oo
o`040?l0003oool0oooo0?l000D0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o
00040?ooo`040?l00000000000000?l000D0oooo00<0o`000?ooo`3oool01P3oool2000000<0oooo
000J0?ooo`030000003oool0oooo00L0oooo00<0o`000?ooo`3oool0<`3oool20?l000@0oooo00D0
o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo00<0oooo0P3o00060?ooo`030?l0
003oool0oooo00D0oooo00<0o`000?ooo`3oool0103oool00`3o0000oooo0?l000050?ooo`050?l0
003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`040?ooo`80o`001P3oool20?l000H0
oooo0P3o00070?ooo`030?l0003oool0oooo00@0oooo0P3o00060?ooo`80o`001@3oool0103o0000
oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103o
ool0103o0000oooo0?ooo`3o00030?ooo`050?l0003oool0oooo0?ooo`3o0000103oool0103o0000
oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103o
ool0103o0000oooo0?ooo`3o00040?ooo`030?l0003oool0o`0000H0oooo00<0o`000?ooo`3o0000
1@3oool00`3o0000oooo0?l000040?ooo`040?l0003oool0oooo0?l000D0oooo00<0o`000?ooo`3o
0000103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`00
0?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00050?ooo`030?l0003oool0oooo00/0oooo
000J0?ooo`030000003oool0oooo00L0oooo00<0o`000?ooo`3oool0<`3oool20?l000@0oooo00D0
o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo00<0oooo0P3o00060?ooo`030?l0
003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool20?l000D0oooo00D0o`000?ooo`3oool0
oooo0?l000030?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o00001P3oool20?l000H0
oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00050?ooo`040?l0003oool0oooo0?l0
00@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003o
ool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l000040?ooo`040?l0003oool0oooo0?l0
00@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003o
ool0oooo0?l000@0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00050?ooo`03
0?l0003oool0o`0000@0oooo00@0o`000?ooo`3oool0o`001@3oool00`3o0000oooo0?l000040?oo
o`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?oo
o`3o00040?ooo`040?l0003oool0oooo0?l000D0oooo00<0o`000?ooo`3oool02`3oool001X0oooo
00<000000?ooo`3oool01`3oool00`3o0000oooo0?ooo`0c0?ooo`80o`00103oool01@3o0000oooo
0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`050?ooo`03
0?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool20?l000D0oooo00D0o`000?ooo`3o
ool0oooo0?l000030?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o00001P3oool20?l0
00H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00050?ooo`040?l0003oool0oooo
0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o0000
103oool0103o0000oooo0?ooo`3o00030?ooo`050?l0003oool0oooo0?ooo`3o0000103oool0103o
0000oooo0?ooo`3o00030?ooo`050?l0003oool0oooo0?ooo`3o0000103oool0103o0000oooo0?oo
o`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00<0o`000?ooo`3o00001@3oool0103o0000
oooo0?ooo`3o00050?ooo`030?l0003oool0o`0000@0oooo00@0o`000?ooo`3oool0o`00103oool0
103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0
o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00<0o`00
0?ooo`3oool0303oool001X0oooo00<000000?ooo`3oool0203oool00`3o0000oooo0?ooo`020?oo
o`030?l0003oool0oooo02d0oooo0P3o00040?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool0
0`3o0000oooo0?ooo`030?ooo`030?l0003oool0oooo00d0oooo00<0o`000?ooo`3oool01@3oool2
0?l000D0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`030?l0003oool0oooo0080oooo00<0
o`000?ooo`3o00001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o
00050?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`05
0?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`00
0?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo
0?ooo`3oool0o`0000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o0004
0?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`001@3oool00`3o0000oooo
0?l000040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o
0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00
103oool0103o0000oooo0?ooo`3o00040?ooo`030?l0003oool0oooo00`0oooo000J0?ooo`030000
003oool0oooo00P0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?ooo`0]0?ooo`80o`00
103oool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool00`3oool00`3o0000
oooo0?ooo`0=0?ooo`030?l0003oool0oooo00D0oooo0P3o00050?ooo`040?l0003oool0oooo0?l0
00@0oooo00<0o`000?ooo`3oool00P3oool0103o0000oooo0?ooo`3o00050?ooo`80o`001P3oool2
0?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000D0oooo00@0o`000?ooo`3oool0o`000`3o
ool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`05
0?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`00
0?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o0000103oool0103o0000oooo
0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0
103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0
o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`00
0?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0
oooo00<0o`000?ooo`3oool0303oool001X0oooo00<000000?ooo`3oool0203oool0103o0000oooo
0?ooo`3oool20?l002l0oooo0P3o00040?ooo`070?l0003oool0oooo0?ooo`3o0000oooo0?l00006
0?ooo`030?l0003oool0oooo01D0oooo0P3o00050?ooo`040?l0003oool0oooo0?l000@0oooo00<0
o`000?ooo`3oool00P3oool0103o0000oooo0?ooo`3o00040?ooo`030?l0003oool0o`0000H0oooo
0P3o00060?ooo`80o`001P3oool20?l000D0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo
0?ooo`3o00030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0
o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o0000
0`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000040?oo
o`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?oo
o`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o
0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00
103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?oo
o`3oool0o`00103oool00`3o0000oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?ooo`080?oo
o`040?l0003oool0oooo0?ooo`80o`00;`3oool20?l000@0oooo00L0o`000?ooo`3oool0oooo0?l0
003oool0o`0000H0oooo00<0o`000?ooo`3oool05@3oool20?l000D0oooo00@0o`000?ooo`3oool0
o`00103oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00<0o`000?ooo`3oool00P3oool00`3o
0000oooo0?l000060?ooo`80o`001P3oool20?l000H0oooo0P3o00050?ooo`030?l0003oool0o`00
00D0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0
o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000
oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0
oooo0?ooo`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0
oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0
oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`04
0?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l000040?ooo`040?l0003oool0
oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool00`3o0000oooo0?ooo`0<0?ooo`004P3o
oolC000000060?l00000000000000?l000000000o`00;P0000000`3o000000000?l0000400000007
0?l00000000000000000003o000000000?l00006000000030?l000000000000001D000000P3o0005
000000040?l00000000000000?l000@0000000D0o`0000000000000000000?l00003000000030?l0
0000000000000080000000<0o`000000003o00001P0000020?l000H000000P3o0005000000030?l0
00000000o`0000D0000000<0o`000000003o00001@000000103o000000000000003o000300000005
0?l00000000000000000003o00000`0000001@3o00000000000000000000o`0000<0000000D0o`00
00000000000000000?l00003000000050?l00000000000000000003o00000`0000001@3o00000000
000000000000o`0000<0000000D0o`0000000000000000000?l00003000000050?l0000000000000
0000003o000010000000103o000000000000003o0004000000040?l00000000000000?l000@00000
00@0o`00000000000000o`0010000000103o000000000000003o0004000000040?l0000000000000
0?l000@0000000@0o`00000000000000o`0010000000103o000000000000003o0003000000050?l0
0000000000000000003o000010000000103o000000000000003o0004000000040?l0000000000000
0?l000@0000000<0o`00000000000000200000040?ooo`006P3oool00`000000oooo0?ooo`080?oo
o`<0o`000P3oool00`3o0000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool0303oool00`3o000000000?l000040?ooo`070?l0003oool0oooo0?ooo`3o0000oooo0?l0
00030?ooo`040000003oool0oooo0?l000d0oooo00<000000?ooo`3oool01`3oool00`3o0000oooo
0?ooo`030?ooo`050000003o0000oooo0?ooo`3o0000103oool01@3o0000oooo0?ooo`3oool0o`00
0080oooo00<000000?l0003oool00`3oool00`3o0000oooo0?l000060?ooo`80o`001@3oool00`3o
0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool0103o
0000oooo0000003o00030?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool01P3o0000oooo0?oo
o`3oool000000?l000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo
0000003o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool00000
0?l000030?ooo`050?l0003oool0oooo0?ooo`3o0000103oool0103o0000oooo0000003o00040?oo
o`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`000000o`00103oool0103o0000oooo0?oo
o`3o00040?ooo`040?l0003oool000000?l000<0oooo00D0o`000?ooo`3oool0oooo0?l000040?oo
o`040?l0003oool000000?l000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`060?l0003o
ool0oooo0000003oool0o`000`3oool0103o0000oooo0?ooo`3o00030?ooo`040?l0003oool0oooo
000000`0oooo000J0?ooo`030000003oool0oooo00d0oooo00<0o`000?ooo`3oool0;03oool00`3o
0000oooo0?l000040?ooo`070?l0003oool0oooo0?ooo`3o0000oooo0?l000030?ooo`040000003o
ool0oooo0?l001L0oooo00<0o`000?ooo`3oool0103oool0103o0000oooo0?ooo`3o00040?ooo`05
0?l0003oool0oooo0?ooo`3o00000`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`00
00H0oooo0P3o00050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o
0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l0
00020?ooo`030?l0003oool0oooo0080oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo
0?ooo`3oool0o`0000<0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?ooo`3oool0
o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`030?l0003oool0oooo0080oooo00@0
o`000?ooo`000000o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l0
00@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00030?ooo`050?l0003o
ool0oooo0?ooo`3o0000103oool01@3o0000oooo0?ooo`3oool0o`000080oooo00D0o`000?ooo`3o
ool0oooo0?l000030?ooo`030?l0003oool0oooo0080oooo00D0o`000?ooo`3oool0oooo0?l00002
0?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`000000oooo0?ooo`0:0?ooo`006P3oool0
0`000000oooo0?ooo`0=0?ooo`030?l0003oool0oooo02`0oooo00<0o`000?ooo`3o0000103oool0
1`3o0000oooo0?ooo`3oool0o`000?ooo`3o00007P3oool00`3o0000oooo0?ooo`040?ooo`040?l0
003oool0oooo0?l000@0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`030?l0003oool0oooo
0080oooo00<0o`000?ooo`3o00001P3oool20?l000D0oooo00<0o`000?ooo`3o00001@3oool0103o
0000oooo0?ooo`3o00040?ooo`030?l0003oool0o`0000@0oooo00D0o`000?ooo`3oool0oooo0?l0
00030?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`030?ooo`030?l0
003oool0o`0000@0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`040?l0003oool0oooo0?l0
00@0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3o
ool00`3o0000oooo0?ooo`020?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0
oooo0?l000040?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l00004
0?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`030?l0
003oool0oooo0080oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`00
00@0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`050?l0003oool0oooo0?ooo`3o00003`3o
ool001X0oooo00<000000?ooo`3oool03@3oool00`3o0000oooo0?ooo`0/0?ooo`030?l0003oool0
o`0000@0oooo00L0o`000?ooo`3oool0oooo0?l0003oool0o`0001h0oooo00<0o`000?ooo`3oool0
103oool0103o0000oooo0?ooo`3o00040?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool00`3o
0000oooo0?ooo`020?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o
0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000@0oooo00<0o`000?ooo`3o0000103oool0
1@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0
003oool0oooo00<0oooo00<0o`000?ooo`3o0000103oool01@3o0000oooo0?ooo`3oool0o`0000@0
oooo00<0o`000?ooo`3o0000103oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?oo
o`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?oo
o`3oool0o`0000<0oooo00<0o`000?ooo`3oool00P3oool0103o0000oooo0?ooo`3o00030?ooo`05
0?l0003oool0oooo0?ooo`3o0000103oool01@3o0000oooo0?ooo`3oool0o`000080oooo00D0o`00
0?ooo`3oool0oooo0?l000030?ooo`030?l0003oool0oooo0080oooo00@0o`000?ooo`3oool0o`00
0`3oool01@3o0000oooo0?ooo`3oool0o`0000@0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o
0000oooo0?ooo`3oool0o`0000l0oooo000J0?ooo`030000003oool0oooo00h0oooo00<0o`000?oo
o`3oool0:`3oool00`3o0000oooo0?l000040?ooo`070?l0003oool0oooo0?ooo`3o0000oooo0?l0
000U0?ooo`030?l0003oool0o`0000D0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`030?l0
003oool0oooo0080oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00040?ooo`03
0?l0003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0
o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o0000
0P3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0o`0000@0oooo00@0o`000?ooo`3oool0
o`001@3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o
0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003o
ool0oooo0?ooo`3o00000`3oool00`3o0000oooo0?ooo`020?ooo`040?l0003oool0oooo0?l000<0
oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`030?l0003oool0oooo0080oooo00@0o`000?oo
o`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`0000@0oooo00@0o`000?ooo`3oool0o`00
0`3oool01@3o0000oooo0?ooo`3oool0o`0000@0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o
0000oooo0?ooo`3oool0o`0000l0oooo000J0?ooo`030000003oool0oooo00h0oooo00<0o`000?oo
o`3oool0:`3oool00`3o0000oooo0?l000030?ooo`030?l0003oool0oooo0080oooo0P3o000W0?oo
o`80o`001@3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00<0o`000?ooo`3oool00P3oool0
0`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000@0oooo00<0o`000?ooo`3o00001@3o
ool0103o0000oooo0?ooo`3o00030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000
oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0
oooo00<0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o00050?ooo`030?l0003o
ool0o`0000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`05
0?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`00
0?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo
0?ooo`3oool0o`0000@0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0
o`0000@0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`0000@0oooo
00@0o`000?ooo`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`0000l0oooo000J0?ooo`03
0000003oool0oooo00h0oooo00<0o`000?ooo`3oool0:`3oool00`3o0000oooo0?l000030?ooo`03
0?l0003oool0oooo0080oooo0P3o000W0?ooo`80o`001@3oool01@3o0000oooo0?ooo`3oool0o`00
00<0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo
0?l000@0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00030?ooo`050?l0003o
ool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00<0o`000?ooo`3o
ool00P3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000@0oooo00@0o`000?ooo`3o
ool0o`001@3oool00`3o0000oooo0?l000040?ooo`040?l0003oool0oooo0?l000D0oooo00<0o`00
0?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00030?ooo`050?l0003oool0oooo0?ooo`3o0000
0`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?oo
o`050?l0003oool0oooo0?ooo`3o0000103oool0103o0000oooo0?ooo`3o00030?ooo`050?l0003o
ool0oooo0?ooo`3o0000103oool0103o0000oooo0?ooo`3o00030?ooo`050?l0003oool0oooo0?oo
o`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000l0oooo000J
0?ooo`8000003`3oool00`3o0000oooo0?ooo`0[0?ooo`030?l0003oool0o`0000<0oooo00<0o`00
0?ooo`3oool00P3oool20?l002L0oooo0P3o00050?ooo`050?l0003oool0oooo0?ooo`3o00000`3o
ool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`00
103oool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3o
ool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`03
0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l00005
0?ooo`030?l0003oool0o`0000@0oooo00@0o`000?ooo`3oool0o`001@3oool00`3o0000oooo0?l0
00050?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`05
0?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`00
0?ooo`3oool0oooo0?l000040?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0
oooo0?l000040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0
103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000l0oooo000J0?ooo`030000003o
ool0oooo00h0oooo00<0o`000?ooo`3oool0:`3oool0103o0000oooo0?ooo`3o00020?ooo`030?l0
003oool0oooo0080oooo0P3o000W0?ooo`030?l0003oool0oooo00@0oooo00@0o`000?ooo`3oool0
o`00103oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00<0o`000?ooo`3oool00P3oool0103o
0000oooo0?ooo`3o00040?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`000`3o
ool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool00P3oool0103o0000oooo
0?ooo`3o00050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000
oooo0?l000050?ooo`030?l0003oool0o`0000@0oooo00@0o`000?ooo`3oool0o`001@3oool00`3o
0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l0
00030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0
oooo00D0o`000?ooo`3oool0oooo0?l000040?ooo`040?l0003oool0oooo0?l000@0oooo00<0o`00
0?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo
00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o000?0?ooo`006P3oool00`000000
oooo0?ooo`0>0?ooo`030?l0003oool0oooo02@0oooo00<0o`000?ooo`3oool0103oool0103o0000
oooo0?ooo`3o00020?ooo`030?l0003oool0oooo00<0oooo00<0o`000?ooo`3oool09@3oool00`3o
0000oooo0?ooo`040?ooo`040?l0003oool0oooo0?l000@0oooo00D0o`000?ooo`3oool0oooo0?l0
00030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool00`3o0000oooo0?ooo`020?ooo`030?l0
003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`00
0080oooo00<0o`000?ooo`3oool00P3oool0103o0000oooo0?ooo`3o00050?ooo`030?l0003oool0
o`0000D0oooo00<0o`000?ooo`3o00001@3oool20?l000H0oooo00<0o`000?ooo`3o00001@3oool0
0`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o0000103oool0
1@3o0000oooo0?ooo`3oool0o`0000@0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000oooo
0?ooo`3oool0o`0000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o0004
0?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?oo
o`3o00040?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`003`3oool001X0oooo
00<000000?ooo`3oool03P3oool00`3o0000oooo0?ooo`0T0?ooo`030?l0003oool0oooo00@0oooo
00@0o`000?ooo`3oool0o`000P3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0oooo02D0
oooo00<0o`000?ooo`3oool0103oool0103o0000oooo0?ooo`3o00040?ooo`050?l0003oool0oooo
0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00<0o`000?ooo`3oool00P3o
ool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0
oooo0?l000020?ooo`030?l0003oool0oooo0080oooo00@0o`000?ooo`3oool0o`001@3oool00`3o
0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo0P3o00060?ooo`030?l0003oool0o`00
00D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`00
00@0oooo00D0o`000?ooo`3oool0oooo0?l000040?ooo`040?l0003oool0oooo0?l000<0oooo00D0
o`000?ooo`3oool0oooo0?l000040?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3oool0
o`00103oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3o
ool0o`00103oool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000l0oooo000J0?oo
o`030000003oool0oooo00h0oooo00<0o`000?ooo`3oool0903oool00`3o0000oooo0?ooo`040?oo
o`040?l0003oool0oooo0?l00080oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`0U
0?ooo`030?l0003oool0oooo00@0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?oo
o`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`030?l0003oool0oooo0080
oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00030?ooo`050?l0003oool0
oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`040?l0003oool0oooo0?l000D0oooo
00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`80o`001P3oool00`3o0000oooo
0?l000050?ooo`80o`001P3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000@0oooo
00D0o`000?ooo`3oool0oooo0?l000040?ooo`030?l0003oool0o`0000@0oooo00D0o`000?ooo`3o
ool0oooo0?l000040?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`00103oool0
0`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`00103o
ool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`000100oooo000J0?ooo`030000003oool0
oooo00h0oooo00<0o`000?ooo`3oool0903oool00`3o0000oooo0?ooo`040?ooo`040?l0003oool0
oooo0?l00080oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`0U0?ooo`030?l0003o
ool0oooo00@0oooo00<0o`000?ooo`3o00001@3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo
00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o
0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`040?l0003o
ool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`001@3oool20?l000H0oooo0P3o00060?ooo`80
o`001P3oool20?l000H0oooo0P3o00060?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o
0000103oool01@3o0000oooo0?ooo`3oool0o`0000@0oooo00<0o`000?ooo`3o0000103oool01@3o
0000oooo0?ooo`3oool0o`0000@0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l00005
0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l00005
0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o0000403oool001X0oooo00<000000?oo
o`3oool03P3oool00`3o0000oooo0?ooo`0T0?ooo`030?l0003oool0oooo00@0oooo00H0o`000?oo
o`3oool0o`000?ooo`3o00060?ooo`030?l0003oool0oooo02D0oooo00<0o`000?ooo`3oool0103o
ool00`3o0000oooo0?l000050?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo
0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo
0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00@0o`000?ooo`3oool0o`00
103oool00`3o0000oooo0?l000060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`00
1P3oool20?l000H0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000040?ooo`040?l0
003oool0oooo0?l000D0oooo00<0o`000?ooo`3o0000103oool01@3o0000oooo0?ooo`3oool0o`00
00@0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`00
00D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`00
00D0oooo00<0o`000?ooo`3o0000403oool001X0oooo00<000000?ooo`3oool03P3oool00`3o0000
oooo0?ooo`0T0?ooo`80o`001@3oool01P3o0000oooo0?ooo`3o0000oooo0?l000H0oooo00<0o`00
0?ooo`3oool0;03oool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000@0oooo00D0
o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000
oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`040?l0003oool0
oooo0?l000@0oooo00<0o`000?ooo`3o00001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3o
ool20?l000H0oooo0P3o00060?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3o
ool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000@0oooo00@0o`000?ooo`3oool0o`00
1@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o0000
1@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o0000
1@3oool00`3o0000oooo0?l0000@0?ooo`006P3oool00`000000oooo0?ooo`0>0?ooo`030?l0003o
ool0oooo02@0oooo0P3o00040?ooo`070?l0003oool0oooo0?ooo`3o0000oooo0?l000060?ooo`03
0?l0003oool0oooo02`0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00040?oo
o`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0
o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool0103o0000
oooo0?ooo`3o00040?ooo`030?l0003oool0o`0000H0oooo0P3o00060?ooo`80o`001P3oool20?l0
00H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo00<0o`000?ooo`3o00001@3oool00`3o
0000oooo0?l000050?ooo`030?l0003oool0o`0000@0oooo00@0o`000?ooo`3oool0o`001@3oool0
0`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool0
0`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool0
0`3o0000oooo0?l0000@0?ooo`006P3oool2000000l0oooo00<0o`000?ooo`3oool0903oool20?l0
00@0oooo00L0o`000?ooo`3oool0oooo0?l0003oool0o`0003H0oooo0P3o00050?ooo`040?l0003o
ool0oooo0?l000@0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?oo
o`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l0
00030?ooo`040?l0003oool0oooo0?l000D0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo
0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`030?l0003oool0o`0000D0oooo
00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo
00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo
00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo
00<0o`000?ooo`3o0000403oool001X0oooo00<000000?ooo`3oool03P3oool00`3o0000oooo0?oo
o`0T0?ooo`80o`00103oool01`3o0000oooo0?ooo`3oool0o`000?ooo`3o0000=P3oool20?l000D0
oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`00
0?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool0103o0000oooo
0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000D0oooo0P3o00060?ooo`80o`001P3oool20?l0
00H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`030?l0003oool0o`00
00D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`00
00D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`00
00D0oooo00<0o`000?ooo`3o00001@3oool20?l000H0oooo00<0o`000?ooo`3o00001@3oool20?l0
0140oooo000J0?ooo`030000003oool0oooo00l0oooo00<0o`000?ooo`3oool08`3oool20?l000@0
oooo00L0o`000?ooo`3oool0oooo0?l0003oool0o`0003H0oooo0P3o00050?ooo`040?l0003oool0
oooo0?l000@0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`040?l0003oool0oooo0?l000@0
oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0
oooo0?l000D0oooo0P3o00060?ooo`80o`001P3oool00`3o0000oooo0?ooo`050?ooo`030?l0003o
ool0oooo00D0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo00<0o`000?ooo`3o00001@3o
ool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3o
ool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3o
ool00`3o0000oooo0?l000050?ooo`80o`001P3oool00`3o0000oooo0?l000050?ooo`80o`004@3o
ool001X0oooo00<000000?ooo`3oool03`3oool00`3o0000oooo0?ooo`0S0?ooo`80o`00103oool0
1`3o0000oooo0?ooo`3oool0o`000?ooo`3o0000=P3oool20?l000D0oooo00@0o`000?ooo`3oool0
o`00103oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00@0o`000?ooo`3oool0o`00103oool0
103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0
o`001@3oool20?l000H0oooo0P3o00060?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3o
ool01@3oool00`3o0000oooo0?ooo`050?ooo`80o`001P3oool20?l000H0oooo00<0o`000?ooo`3o
00001@3oool20?l000H0oooo00<0o`000?ooo`3o00001@3oool20?l000H0oooo00<0o`000?ooo`3o
00001@3oool00`3o0000oooo0?l000050?ooo`80o`001P3oool00`3o0000oooo0?l000050?ooo`80
o`001P3oool00`3o0000oooo0?l000050?ooo`80o`004@3oool001X0oooo00<000000?ooo`3oool0
3`3oool00`3o0000oooo0?ooo`0S0?ooo`80o`00103oool01`3o0000oooo0?ooo`3oool0o`000?oo
o`3o0000=P3oool20?l000H0oooo00<0o`000?ooo`3o0000103oool01@3o0000oooo0?ooo`3oool0
o`0000<0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0
003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`001@3oool20?l000H0oooo0P3o00060?oo
o`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?oo
o`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool00`3o0000oooo0?l000050?oo
o`80o`001P3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo0P3o00060?oo
o`030?l0003oool0o`0000D0oooo0P3o00060?ooo`80o`001P3oool20?l00140oooo000J0?ooo`03
0000003oool0oooo00l0oooo00<0o`000?ooo`3oool08`3oool20?l000@0oooo00L0o`000?ooo`3o
ool0oooo0?l0003oool0o`0003H0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?l00004
0?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo
0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000D0oooo0P3o00060?ooo`80o`001P3oool00`3o
0000oooo0?ooo`0M0?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool00`3o
0000oooo0?l000050?ooo`80o`001P3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`00
00D0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`004@3oool001X0
oooo00<000000?ooo`3oool03`3oool00`3o0000oooo0?ooo`0S0?ooo`80o`00103oool01`3o0000
oooo0?ooo`3oool0o`000?ooo`3o0000=P3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0
o`0000D0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003o
ool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`001@3oool20?l000H0oooo0P3o00060?ooo`03
0?l0003oool0oooo01d0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo0P3o00060?ooo`80
o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool00`3o0000oooo0?l000050?ooo`80
o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001`3oool00`3o0000oooo0?ooo`0?0?ooo`00
6P3oool00`000000oooo0?ooo`0?0?ooo`030?l0003oool0oooo02<0oooo0P3o00030?ooo`030?l0
003oool0oooo0080oooo00<0o`000?ooo`3o0000=P3oool00`3o0000oooo0?ooo`050?ooo`80o`00
1P3oool00`3o0000oooo0?l000040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3o
ool0o`00103oool00`3o0000oooo0?l000060?ooo`80o`001P3oool20?l000H0oooo00<0o`000?oo
o`3oool09@3oool00`3o0000oooo0?ooo`050?ooo`80o`001P3oool20?l000H0oooo0P3o00060?oo
o`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?oo
o`80o`001`3oool00`3o0000oooo0?ooo`0?0?ooo`006P3oool200000100oooo00<0o`000?ooo`3o
ool08`3oool20?l000<0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l0000f0?ooo`03
0?l0003oool0oooo00D0oooo0P3o00060?ooo`030?l0003oool0o`0000@0oooo00@0o`000?ooo`3o
ool0o`001@3oool00`3o0000oooo0?l000040?ooo`030?l0003oool0o`0000H0oooo0P3o00060?oo
o`030?l0003oool0oooo02d0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo0P3o00060?oo
o`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00070?oo
o`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`0?0?oo
o`006P3oool00`000000oooo0?ooo`0?0?ooo`030?l0003oool0oooo02<0oooo0P3o00030?ooo`03
0?l0003oool0oooo0080oooo00<0o`000?ooo`3o0000?P3oool20?l000H0oooo00<0o`000?ooo`3o
0000103oool0103o0000oooo0?ooo`3o00050?ooo`030?l0003oool0o`0000D0oooo0P3o00060?oo
o`80o`001P3oool00`3o0000oooo0?ooo`0e0?ooo`80o`001P3oool20?l000H0oooo0P3o00060?oo
o`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000L0oooo00<0o`000?oo
o`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00l0oooo000J0?ooo`03
0000003oool0oooo00l0oooo00<0o`000?ooo`3oool08P3oool00`3o0000oooo0?l000030?ooo`03
0?l0003oool0oooo0080oooo00<0o`000?ooo`3o0000?P3oool20?l000H0oooo0P3o00050?ooo`04
0?l0003oool0oooo0?l000D0oooo00<0o`000?ooo`3o00001@3oool20?l000H0oooo0P3o00060?oo
o`030?l0003oool0oooo03D0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo0P3o00060?oo
o`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001`3oool00`3o0000oooo0?ooo`0W0?oo
o`006P3oool00`000000oooo0?ooo`0?0?ooo`030?l0003oool0oooo0280oooo00<0o`000?ooo`3o
00000`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0003h0oooo0P3o00060?ooo`80
o`001@3oool0103o0000oooo0?ooo`3o00050?ooo`030?l0003oool0o`0000D0oooo0P3o00060?oo
o`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0?@3oool20?l000H0oooo0P3o00060?oo
o`80o`001P3oool20?l000H0oooo0P3o00070?ooo`030?l0003oool0oooo02l0oooo000J0?ooo`03
0000003oool0oooo00l0oooo00<0o`000?ooo`3oool08P3oool00`3o0000oooo0?l000030?ooo`03
0?l0003oool0oooo0080oooo00<0o`000?ooo`3o0000?P3oool00`3o0000oooo0?ooo`050?ooo`80
o`001P3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo0P3o00060?ooo`03
0?l0003oool0oooo04D0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo0P3o00060?ooo`80
o`001P3oool20?l003T0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0o`000?ooo`3oool0
0`3oool00`3o0000oooo0?ooo`0L0?ooo`030?l0003oool0o`0000<0oooo00<0o`000?ooo`3oool0
0P3oool20?l003l0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo00<0o`000?ooo`3o0000
1@3oool00`3o0000oooo0?l000050?ooo`80o`001P3oool00`3o0000oooo0?ooo`1=0?ooo`030?l0
003oool0oooo00D0oooo0P3o00060?ooo`80o`00@@3oool000D00000103oool2000000@0oooo1@00
00060?ooo`030000003oool0oooo00l0oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?oo
o`0E0?ooo`030?l0003oool0oooo00@0oooo00<0o`000?ooo`3o00000`3oool00`3o0000oooo0?oo
o`030?ooo`030?l0003oool0oooo03d0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo0P3o
00060?ooo`80o`001P3oool20?l000H0oooo00<0o`000?ooo`3oool0X03oool00080oooo00<00000
0?ooo`3oool0103oool2000000@0oooo00D000000?ooo`3oool0oooo000000060?ooo`030000003o
ool0oooo00l0oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`0E0?ooo`030?l0003o
ool0oooo00@0oooo00<0o`000?ooo`3o00000`3oool00`3o0000oooo0?ooo`030?ooo`030?l0003o
ool0oooo03d0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo0P3o00060?ooo`80o`001P3o
ool20?l000H0oooo00<0o`000?ooo`3oool0X03oool00080oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo00l0oooo00<0o`000?ooo`3oool00P3oool2
0?l001L0oooo00<0o`000?ooo`3oool0103oool00`3o0000oooo0?l000030?ooo`030?l0003oool0
oooo00<0oooo00<0o`000?ooo`3oool0?@3oool00`3o0000oooo0?ooo`050?ooo`80o`001P3oool2
0?l000H0oooo0P3o00060?ooo`80o`001P3oool00`3o0000oooo0?ooo`2P0?ooo`000P3oool00`00
0000oooo0?ooo`0<0?ooo`030000003oool0oooo00H0oooo0`00000?0?ooo`030?l0003oool0oooo
0080oooo0P3o000G0?ooo`030?l0003oool0oooo00@0oooo00<0o`000?ooo`3o00000`3oool00`3o
0000oooo0?ooo`030?ooo`030?l0003oool0oooo03d0oooo00<0o`000?ooo`3oool01@3oool20?l0
00H0oooo0P3o00060?ooo`80o`001P3oool20?l00:T0oooo00020?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`030?l0003oool0oooo0080
oooo00<0o`000?ooo`3o00005P3oool00`3o0000oooo0?ooo`030?ooo`040?l0003oool0oooo0?l0
00<0oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`150?ooo`80o`001P3oool20?l0
00H0oooo0P3o00060?ooo`030?l0003oool0oooo0:P0oooo00020?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0?0?ooo`030?l0003oool0oooo0080
oooo00<0o`000?ooo`3o00005P3oool00`3o0000oooo0?ooo`030?ooo`040?l0003oool0oooo0?l0
00<0oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`150?ooo`80o`001P3oool20?l0
00H0oooo0P3o00060?ooo`030?l0003oool0oooo0:P0oooo0003000000`0oooo00D000000?ooo`3o
ool0oooo000000060?ooo`030000003oool0oooo00l0oooo00<0o`000?ooo`3oool00P3oool00`3o
0000oooo0?l0000F0?ooo`030?l0003oool0oooo00<0oooo00@0o`000?ooo`3oool0o`000`3oool0
0`3o0000oooo0?ooo`1;0?ooo`030?l0003oool0oooo00D0oooo0P3o00060?ooo`030?l0003oool0
oooo00D0oooo00<0o`000?ooo`3oool0Z03oool00080oooo00<000000?ooo`3oool02`3oool30000
00L0oooo00<000000?ooo`3oool03`3oool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`00
0?ooo`3oool0503oool00`3o0000oooo0?ooo`030?ooo`040?l0003oool0oooo0?l000<0oooo00<0
o`000?ooo`3oool0B`3oool00`3o0000oooo0?ooo`050?ooo`80o`001P3oool00`3o0000oooo0?oo
o`050?ooo`030?l0003oool0oooo0:P0oooo000J0?ooo`030000003oool0oooo00l0oooo00D0o`00
0?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo01@0oooo00<0o`000?ooo`3oool00`3o
ool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool0B`3oool00`3o0000oooo
0?ooo`050?ooo`80o`001P3oool00`3o0000oooo0?ooo`2`0?ooo`006P3oool00`000000oooo0?oo
o`0?0?ooo`040?l0003oool0oooo0?l000<0oooo00<0o`000?ooo`3oool0503oool20?l000@0oooo
00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo04/0oooo00<0o`000?ooo`3o
ool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo0;00oooo000J0?ooo`030000
003oool0oooo00l0oooo00@0o`000?ooo`3oool0o`000`3oool00`3o0000oooo0?ooo`0C0?ooo`03
0?l0003oool0o`0000@0oooo00L0o`000?ooo`3oool0oooo0?l0003oool0o`0005H0oooo00<0o`00
0?ooo`3oool01@3oool00`3o0000oooo0?ooo`2`0?ooo`006P3oool200000100oooo00@0o`000?oo
o`3oool0o`000`3oool00`3o0000oooo0?ooo`0C0?ooo`030?l0003oool0o`0000@0oooo00L0o`00
0?ooo`3oool0oooo0?l0003oool0o`0005H0oooo00<0o`000?ooo`3oool0^03oool001X0oooo00<0
00000?ooo`3oool03`3oool0103o0000oooo0?ooo`3o00030?ooo`030?l0003oool0oooo01<0oooo
00<0o`000?ooo`3o0000103oool01`3o0000oooo0?ooo`3oool0o`000?ooo`3o0000o`3ooolB0?oo
o`006P3oool00`000000oooo0?ooo`0@0?ooo`80o`00103oool00`3o0000oooo0?ooo`0C0?ooo`03
0?l0003oool0o`0000@0oooo00L0o`000?ooo`3oool0oooo0?l0003oool0o`000?l0oooo4P3oool0
01X0oooo00<000000?ooo`3oool0403oool20?l000@0oooo00<0o`000?ooo`3oool0303oool00`3o
0000oooo0?ooo`040?ooo`030?l0003oool0o`0000@0oooo00L0o`000?ooo`3oool0oooo0?l0003o
ool0o`000?l0oooo4P3oool001X0oooo00<000000?ooo`3oool0403oool20?l000@0oooo00<0o`00
0?ooo`3oool0103oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00@0oooo00<0o`00
0?ooo`3o0000103oool01`3o0000oooo0?ooo`3oool0o`000?ooo`3o0000o`3ooolB0?ooo`006P3o
ool00`000000oooo0?ooo`0@0?ooo`030?l0003oool0oooo00<0oooo00<0o`000?ooo`3oool0103o
ool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00@0oooo00<0o`000?ooo`3o0000103o
ool01`3o0000oooo0?ooo`3oool0o`000?ooo`3o0000o`3ooolB0?ooo`006P3oool00`000000oooo
0?ooo`0F0?ooo`030?l0003oool0oooo00@0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo
0?ooo`040?ooo`030?l0003oool0o`0000@0oooo00L0o`000?ooo`3oool0oooo0?l0003oool0o`00
0?l0oooo4P3oool001X0oooo00<000000?ooo`3oool05P3oool00`3o0000oooo0?ooo`040?ooo`03
0?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0103oool00`3o0000oooo0?l000040?ooo`04
0?l0003oool0oooo0?ooo`80o`00o`3ooolC0?ooo`006P3oool00`000000oooo0?ooo`0F0?ooo`03
0?l0003oool0oooo00@0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`040?ooo`03
0?l0003oool0o`0000@0oooo00@0o`000?ooo`3oool0oooo0P3o003o0?oooa<0oooo000J0?ooo`80
00005`3oool00`3o0000oooo0?ooo`040?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3o
ool0103oool00`3o0000oooo0?l000040?ooo`040?l0003oool0oooo0?ooo`80o`00o`3ooolC0?oo
o`006P3oool00`000000oooo0?ooo`0F0?ooo`030?l0003oool0oooo00@0oooo00<0o`000?ooo`3o
ool01@3oool20?l000D0oooo00<0o`000?ooo`3o00000`3oool00`3o0000oooo0?ooo`020?ooo`80
o`00o`3ooolC0?ooo`006P3oool00`000000oooo0?ooo`0F0?ooo`030?l0003oool0oooo00<0oooo
0P3o00070?ooo`80o`001@3oool00`3o0000oooo0?l000030?ooo`030?l0003oool0oooo0080oooo
0P3o003o0?oooa<0oooo000J0?ooo`030000003oool0oooo01H0oooo00<0o`000?ooo`3oool00`3o
ool20?l000L0oooo0P3o00050?ooo`040?l0003oool0oooo0?l00080oooo00<0o`000?ooo`3oool0
0`3oool00`3o0000oooo0?ooo`3o0?oooa40oooo000J0?ooo`030000003oool0oooo01H0oooo00<0
o`000?ooo`3oool00`3oool00`3o0000oooo0?l000060?ooo`80o`001@3oool0103o0000oooo0?oo
o`3o00020?ooo`030?l0003oool0oooo00<0oooo00<0o`000?ooo`3oool0o`3ooolA0?ooo`006P3o
ool00`000000oooo0?ooo`0F0?ooo`030?l0003oool0oooo00<0oooo00<0o`000?ooo`3o00001P3o
ool20?l000D0oooo00@0o`000?ooo`3oool0o`000P3oool00`3o0000oooo0?ooo`030?ooo`030?l0
003oool0oooo0?l0oooo4@3oool001X0oooo00<000000?ooo`3oool05P3oool00`3o0000oooo0?oo
o`030?ooo`030?l0003oool0o`0000H0oooo0P3o00050?ooo`040?l0003oool0oooo0?l00080oooo
00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`3o0?oooa40oooo000J0?ooo`030000003o
ool0oooo01H0oooo00<0o`000?ooo`3oool00P3oool0103o0000oooo0?ooo`3o00050?ooo`030?l0
003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`000P3oool00`3o0000oooo0?ooo`030?ooo`03
0?l0003oool0oooo0?l0oooo4@3oool001X0oooo0P00000G0?ooo`030?l0003oool0oooo0080oooo
00@0o`000?ooo`3oool0o`001@3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0oooo0?oo
o`3o00000P3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0oooo0?l0oooo4@3oool001X0
oooo00<000000?ooo`3oool05`3oool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?oo
o`3oool00`3oool00`3o0000oooo0?l000040?ooo`070?l0003oool0oooo0?ooo`3o0000oooo0?l0
00060?ooo`030?l0003oool0oooo0?l0oooo4@3oool001X0oooo00<000000?ooo`3oool05`3oool0
1@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?l0
00040?ooo`070?l0003oool0oooo0?ooo`3o0000oooo0?l0003o0?oooaX0oooo000J0?ooo`030000
003oool0oooo01L0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo00<0
oooo00<0o`000?ooo`3o0000103oool01`3o0000oooo0?ooo`3oool0o`000?ooo`3o0000o`3ooolJ
0?ooo`006P3oool00`000000oooo0?ooo`0G0?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool0
0`3o0000oooo0?ooo`030?ooo`030?l0003oool0o`0000@0oooo00L0o`000?ooo`3oool0oooo0?l0
003oool0o`000?l0oooo6P3oool000D00000103oool2000000L0oooo0`0000050?ooo`030000003o
ool0oooo01L0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo0080oooo
00@0o`000?ooo`3oool0o`00103oool01`3o0000oooo0?ooo`3oool0o`000?ooo`3o0000o`3ooolJ
0?ooo`000P3oool00`000000oooo0?ooo`040?ooo`800000203oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo01L0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0
oooo0080oooo00@0o`000?ooo`3oool0o`00103oool01`3o0000oooo0?ooo`3oool0o`000?ooo`3o
0000o`3ooolJ0?ooo`000P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool05`3oool0103o0000oooo0?ooo`3o00030?ooo`030?l0003oool0oooo0080
oooo00@0o`000?ooo`3oool0o`00103oool01`3o0000oooo0?ooo`3oool0o`000?ooo`3o0000o`3o
oolJ0?ooo`000P3oool00`000000oooo0?ooo`0:0?ooo`H000001@3oool3000001L0oooo00@0o`00
0?ooo`3oool0o`000`3oool00`3o0000oooo0?ooo`020?ooo`040?l0003oool0oooo0?l000<0oooo
00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l0003o0?oooaX0oooo00020?ooo`030000003o
ool0oooo00/0oooo00@000000?ooo`3oool000001P3oool00`000000oooo0?ooo`0G0?ooo`040?l0
003oool0oooo0?l000<0oooo00<0o`000?ooo`3oool00P3oool0103o0000oooo0?ooo`3o00030?oo
o`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o0000o`3ooolJ0?ooo`000P3oool00`000000
oooo0?ooo`0;0?ooo`040000003oool0oooo000000H0oooo00<000000?ooo`3oool05`3oool0103o
0000oooo0?ooo`3o00040?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool01@3o0000oooo0?oo
o`3oool0o`0000@0oooo00<0o`000?ooo`3o0000o`3ooolJ0?ooo`000`00000>0?ooo`030000003o
ool0000000H0oooo00<000000?ooo`3oool05`3oool0103o0000oooo0?ooo`3o00040?ooo`050?l0
003oool0oooo0?ooo`3o00000P3oool01@3o0000oooo0?ooo`3oool0o`0000@0oooo00<0o`000?oo
o`3o0000o`3ooolJ0?ooo`000P3oool00`000000oooo0?ooo`0=0?ooo`8000001P3oool00`000000
oooo0?ooo`0G0?ooo`040?l0003oool0oooo0?l000@0oooo00D0o`000?ooo`3oool0oooo0?l00002
0?ooo`050?l0003oool0oooo0?ooo`3o0000103oool00`3o0000oooo0?l0003o0?oooaX0oooo000J
0?ooo`030000003oool0oooo01L0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?oo
o`3oool0o`000080oooo00D0o`000?ooo`3oool0oooo0?l000040?ooo`030?l0003oool0o`000?l0
oooo6P3oool001X0oooo00<000000?ooo`3oool05`3oool0103o0000oooo0?ooo`3o00040?ooo`05
0?l0003oool0oooo0?ooo`3o00000P3oool01@3o0000oooo0?ooo`3oool0o`0000@0oooo00<0o`00
0?ooo`3o0000o`3ooolJ0?ooo`006P3oool00`000000oooo0?ooo`0G0?ooo`030?l0003oool0o`00
00D0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`050?l0003oool0oooo0?ooo`3o0000103o
ool00`3o0000oooo0?l0003o0?oooaX0oooo000J0?ooo`030000003oool0oooo01L0oooo00<0o`00
0?ooo`3o00001@3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00@0o`000?ooo`3oool0o`00
103oool00`3o0000oooo0?l0003o0?oooaX0oooo000J0?ooo`800000603oool00`3o0000oooo0?l0
00050?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool0103o0000oooo0?ooo`3o00050?ooo`03
0?l0003oool0oooo0?l0oooo6@3oool001X0oooo00<000000?ooo`3oool0603oool00`3o0000oooo
0?ooo`040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`001@3oool00`3o
0000oooo0?ooo`3o0?oooaT0oooo000J0?ooo`030000003oool0oooo01P0oooo00<0o`000?ooo`3o
ool0103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000D0oooo00<0o`00
0?ooo`3oool0o`3ooolI0?ooo`006P3oool00`000000oooo0?ooo`0H0?ooo`030?l0003oool0oooo
00@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00050?ooo`030?l0003o
ool0oooo0?l0oooo6@3oool001X0oooo00<000000?ooo`3oool0603oool00`3o0000oooo0?ooo`04
0?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`001@3oool00`3o0000oooo
0?ooo`3o0?oooaT0oooo000J0?ooo`030000003oool0oooo01l0oooo00@0o`000?ooo`3oool0o`00
103oool00`3o0000oooo0?l000060?ooo`030?l0003oool0oooo0?l0oooo6@3oool001X0oooo00<0
00000?ooo`3oool07`3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000H0oooo00<0
o`000?ooo`3oool0o`3ooolI0?ooo`006P3oool00`000000oooo0?ooo`0O0?ooo`030?l0003oool0
o`0000D0oooo00<0o`000?ooo`3o0000o`3ooolR0?ooo`006P3oool200000200oooo00<0o`000?oo
o`3o00001@3oool00`3o0000oooo0?l0003o0?ooob80oooo000J0?ooo`030000003oool0oooo01l0
oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l0003o0?ooob80oooo000J0?ooo`030000
003oool0oooo01l0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l0003o0?ooob80oooo
000J0?ooo`030000003oool0oooo01l0oooo00<0o`000?ooo`3o00001@3oool20?l00?l0oooo8`3o
ool001X0oooo00<000000?ooo`3oool07`3oool00`3o0000oooo0?l000050?ooo`80o`00o`3ooolS
0?ooo`006P3oool00`000000oooo0?ooo`0O0?ooo`030?l0003oool0o`0000D0oooo0P3o003o0?oo
ob<0oooo000J0?ooo`030000003oool0oooo01l0oooo00<0o`000?ooo`3o00001@3oool20?l00?l0
oooo8`3oool001X0oooo00<000000?ooo`3oool0803oool20?l000H0oooo00<0o`000?ooo`3oool0
o`3ooolQ0?ooo`006P3oool00`000000oooo0?ooo`0P0?ooo`80o`001P3oool00`3o0000oooo0?oo
o`3o0?ooob40oooo000J0?ooo`8000008@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0
oooo0?l0oooo8@3oool001X0oooo00<000000?ooo`3oool0803oool00`3o0000oooo0?ooo`050?oo
o`030?l0003oool0oooo0?l0oooo8@3oool001X0oooo00<000000?ooo`3oool0803oool00`3o0000
oooo0?ooo`050?ooo`030?l0003oool0oooo0?l0oooo8@3oool001X0oooo00<000000?ooo`3oool0
803oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo0?l0oooo8@3oool001X0oooo00<0
00000?ooo`3oool0o`3ooom<0?ooo`001@0000040?ooo`8000001P3oool3000000H0oooo00<00000
0?ooo`3oool0o`3ooom<0?ooo`000P3oool00`000000oooo0?ooo`040?ooo`8000001@3oool01@00
0000oooo0?ooo`3oool0000000D0oooo00<000000?ooo`3oool0o`3ooom<0?ooo`000P3oool00`00
0000oooo0?ooo`0;0?ooo`050000003oool0oooo0?ooo`0000001@3oool00`000000oooo0?ooo`3o
0?oood`0oooo00020?ooo`030000003oool0oooo00/0oooo00D000000?ooo`3oool0oooo00000005
0?ooo`<00000o`3ooom<0?ooo`000P3oool00`000000oooo0?ooo`0;0?ooo`@00000o`3ooomE0?oo
o`000P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0?l0ooooEP3oool000<00000
3P3oool00`000000oooo0?ooo`3o0?oooeD0oooo00020?ooo`030000003oool0oooo00d0oooo0`00
003o0?oooe@0oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo
003o0?ooofT0oooo0000\
\>"],
  ImageRangeCache->{{{0, 359}, {221.375, 0}} -> {-0.0332863, 0.343255, \
0.00155747, 0.00741071}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(vCSwi\  = \ 
        Plot[Evaluate[vC[10^\(-3\)\ t]\  /. \ ResSwi], {t, \ 0, \ 
            10^3\ T0}, \ PlotPoints\  -> \ 500, \ 
          PlotStyle\  -> \ {{AbsoluteThickness[0.001], 
                RGBColor[1, \ 0, \ 0]}}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 2.38095 -1.24801 0.22116 [
[.2619 .06645 -9 -9 ]
[.2619 .06645 9 0 ]
[.5 .06645 -9 -9 ]
[.5 .06645 9 0 ]
[.7381 .06645 -9 -9 ]
[.7381 .06645 9 0 ]
[.97619 .06645 -9 -9 ]
[.97619 .06645 9 0 ]
[.01131 .18953 -18 -4.5 ]
[.01131 .18953 0 4.5 ]
[.01131 .30011 -6 -4.5 ]
[.01131 .30011 0 4.5 ]
[.01131 .41069 -18 -4.5 ]
[.01131 .41069 0 4.5 ]
[.01131 .52127 -6 -4.5 ]
[.01131 .52127 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .07895 m
.2619 .0852 L
s
[(0.1)] .2619 .06645 0 1 Mshowa
.5 .07895 m
.5 .0852 L
s
[(0.2)] .5 .06645 0 1 Mshowa
.7381 .07895 m
.7381 .0852 L
s
[(0.3)] .7381 .06645 0 1 Mshowa
.97619 .07895 m
.97619 .0852 L
s
[(0.4)] .97619 .06645 0 1 Mshowa
.125 Mabswid
.07143 .07895 m
.07143 .0827 L
s
.11905 .07895 m
.11905 .0827 L
s
.16667 .07895 m
.16667 .0827 L
s
.21429 .07895 m
.21429 .0827 L
s
.30952 .07895 m
.30952 .0827 L
s
.35714 .07895 m
.35714 .0827 L
s
.40476 .07895 m
.40476 .0827 L
s
.45238 .07895 m
.45238 .0827 L
s
.54762 .07895 m
.54762 .0827 L
s
.59524 .07895 m
.59524 .0827 L
s
.64286 .07895 m
.64286 .0827 L
s
.69048 .07895 m
.69048 .0827 L
s
.78571 .07895 m
.78571 .0827 L
s
.83333 .07895 m
.83333 .0827 L
s
.88095 .07895 m
.88095 .0827 L
s
.92857 .07895 m
.92857 .0827 L
s
.25 Mabswid
0 .07895 m
1 .07895 L
s
.02381 .18953 m
.03006 .18953 L
s
[(6.5)] .01131 .18953 1 0 Mshowa
.02381 .30011 m
.03006 .30011 L
s
[(7)] .01131 .30011 1 0 Mshowa
.02381 .41069 m
.03006 .41069 L
s
[(7.5)] .01131 .41069 1 0 Mshowa
.02381 .52127 m
.03006 .52127 L
s
[(8)] .01131 .52127 1 0 Mshowa
.125 Mabswid
.02381 .10107 m
.02756 .10107 L
s
.02381 .12318 m
.02756 .12318 L
s
.02381 .1453 m
.02756 .1453 L
s
.02381 .16742 m
.02756 .16742 L
s
.02381 .21165 m
.02756 .21165 L
s
.02381 .23376 m
.02756 .23376 L
s
.02381 .25588 m
.02756 .25588 L
s
.02381 .278 m
.02756 .278 L
s
.02381 .32223 m
.02756 .32223 L
s
.02381 .34434 m
.02756 .34434 L
s
.02381 .36646 m
.02756 .36646 L
s
.02381 .38858 m
.02756 .38858 L
s
.02381 .43281 m
.02756 .43281 L
s
.02381 .45492 m
.02756 .45492 L
s
.02381 .47704 m
.02756 .47704 L
s
.02381 .49916 m
.02756 .49916 L
s
.02381 .05684 m
.02756 .05684 L
s
.02381 .03472 m
.02756 .03472 L
s
.02381 .0126 m
.02756 .0126 L
s
.02381 .54339 m
.02756 .54339 L
s
.02381 .5655 m
.02756 .5655 L
s
.02381 .58762 m
.02756 .58762 L
s
.02381 .60974 m
.02756 .60974 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.001 Mabswid
.0853 0 m
.08684 .02211 L
.08883 .04975 L
.09074 .07531 L
.09258 .09886 L
.09351 .11036 L
.09401 .11637 L
.09454 .12275 L
.09479 .12567 L
.09492 .12726 L
.09505 .12873 L
.09516 .12999 L
.09522 .13069 L
.09527 .1305 L
.09533 .12971 L
.0954 .12886 L
.09551 .1273 L
.09643 .11535 L
.0984 .09 L
.10029 .06603 L
.10124 .05416 L
.10166 .049 L
.10211 .04339 L
.10223 .04191 L
.1023 .0411 L
.10236 .04034 L
.10242 .04072 L
.10248 .0419 L
.10259 .04396 L
.10304 .05248 L
.10405 .07102 L
.10602 .10642 L
.10792 .13916 L
.10975 .16933 L
.11169 .19996 L
.11357 .22794 L
.11556 .25601 L
.11655 .2694 L
.11747 .28137 L
.11837 .29263 L
.11882 .29816 L
.11894 .29967 L
.11898 .30008 L
.11901 .30051 L
.11904 .30087 L
.11908 .30048 L
.11932 .29692 L
.12127 .26841 L
.12315 .24145 L
.12411 .22789 L
Mistroke
.12515 .21333 L
.12563 .20656 L
.1259 .20287 L
.12603 .20107 L
.12609 .20029 L
.12615 .19943 L
.1262 .19909 L
.12626 .2001 L
.12636 .20209 L
.12659 .20647 L
.12707 .21568 L
.12896 .25057 L
.13096 .28575 L
.13289 .31773 L
.13475 .34666 L
.13672 .3754 L
.13863 .40106 L
.14045 .42382 L
.14147 .43568 L
.1424 .44596 L
.14263 .44837 L
.14275 .44968 L
.14281 .45031 L
.14286 .45069 L
.14292 .44972 L
.14299 .44868 L
.14313 .44647 L
.14337 .44258 L
.14428 .42794 L
.14619 .39767 L
.14802 .36908 L
.14905 .35333 L
.14953 .34595 L
.14974 .34273 L
.14986 .3409 L
.14992 .34 L
.14998 .33919 L
.15003 .33944 L
.15009 .34039 L
.1502 .34248 L
.15044 .34673 L
.15095 .35557 L
.15186 .37124 L
.1537 .40141 L
.15566 .43131 L
.15755 .45793 L
.15937 .48147 L
.1613 .5043 L
.16316 .52406 L
.1641 .53326 L
Mistroke
.16514 .54269 L
.16561 .54681 L
.16612 .55105 L
.16637 .55305 L
.16649 .55402 L
.16655 .55444 L
.1666 .5549 L
.16666 .55532 L
.16671 .55471 L
.16681 .55285 L
.16704 .54894 L
.16891 .51721 L
.17089 .48408 L
.17189 .46776 L
.1728 .45276 L
.17326 .44533 L
.17349 .4417 L
.1737 .43837 L
.17376 .4374 L
.17381 .43657 L
.17387 .43746 L
.17393 .43844 L
.17419 .44235 L
.17464 .44908 L
.17652 .47567 L
.17851 .50154 L
.18043 .52408 L
.18227 .54354 L
.18424 .56183 L
.18613 .57709 L
.18794 .58957 L
.18895 .59558 L
.1894 .598 L
.18988 .60051 L
.19011 .60163 L
.19023 .60224 L
.19035 .6028 L
.1904 .60305 L
.19046 .60332 L
.19052 .60259 L
.19058 .60159 L
.19079 .59792 L
.19174 .58117 L
.19357 .54959 L
.19551 .51658 L
.1964 .50157 L
.19687 .4937 L
.19738 .48537 L
.1975 .48337 L
.19756 .48232 L
Mistroke
.1976 .48175 L
.19763 .48149 L
.19766 .48192 L
.1977 .4824 L
.19777 .48322 L
.19791 .48484 L
.1984 .49064 L
.19936 .50163 L
.20124 .52129 L
.20323 .53977 L
.20515 .55521 L
.207 .56787 L
.20897 .57898 L
.21086 .58739 L
.21268 .59336 L
.21362 .59564 L
.21388 .5962 L
.21402 .59645 L
.21414 .59668 L
.2142 .59679 L
.21425 .59688 L
.21431 .59646 L
.21437 .59547 L
.21462 .59107 L
.21647 .5588 L
.21845 .5251 L
.21943 .50855 L
.22035 .49324 L
.2208 .48573 L
.22123 .47865 L
.22134 .47676 L
.2214 .4758 L
.22143 .47539 L
.22146 .47568 L
.2216 .47691 L
.22172 .47803 L
.22217 .48206 L
.22403 .49743 L
.22601 .51154 L
.22792 .52296 L
.22975 .53194 L
.2317 .53935 L
.2326 .54203 L
.23358 .54441 L
.23412 .54548 L
.23461 .54632 L
.23511 .54704 L
.23558 .54758 L
.23601 .54797 L
.23624 .54813 L
Mistroke
.23648 .54827 L
.23662 .54834 L
.23675 .54839 L
.23688 .54843 L
.237 .54846 L
.23706 .54847 L
.23712 .54849 L
.23717 .54849 L
.23723 .5485 L
.23729 .5485 L
.23733 .5485 L
.23736 .5485 L
.23741 .5485 L
.23747 .5485 L
.23753 .5485 L
.2376 .54849 L
.23766 .54848 L
.23773 .54847 L
.23785 .54845 L
.23796 .54841 L
.23803 .54839 L
.23807 .54838 L
.23811 .54817 L
.23814 .54758 L
.23817 .54704 L
.23824 .54584 L
.2385 .54149 L
.23947 .52502 L
.24139 .49273 L
.24324 .46221 L
.24418 .44694 L
.24468 .43891 L
.24496 .4345 L
.24509 .43234 L
.24515 .43138 L
.24518 .43088 L
.24522 .43033 L
.24528 .43025 L
.24533 .43065 L
.24546 .43154 L
.24573 .43338 L
.2462 .43652 L
.24711 .44233 L
.24909 .45335 L
.251 .46203 L
.25283 .46858 L
.25385 .47148 L
.25478 .47364 L
.25574 .47542 L
.25665 .47667 L
Mistroke
.25712 .47715 L
.25738 .47736 L
.25762 .47753 L
.25784 .47766 L
.25804 .47776 L
.25826 .47784 L
.25839 .47788 L
.2585 .47791 L
.25856 .47792 L
.25863 .47793 L
.25868 .47794 L
.25874 .47794 L
.2588 .47795 L
.25886 .47795 L
.25891 .47795 L
.25896 .47795 L
.25899 .47795 L
.25903 .47795 L
.25909 .47794 L
.25915 .47794 L
.2592 .47793 L
.25933 .47791 L
.25946 .47788 L
.25967 .47781 L
.25988 .47772 L
.26034 .47744 L
.26081 .47706 L
.26131 .47652 L
.26152 .47627 L
.26174 .47598 L
.2618 .47589 L
.26186 .4758 L
.26192 .4755 L
.26198 .47452 L
.2622 .47084 L
.26417 .43883 L
.26607 .40857 L
.26789 .37998 L
.26837 .37253 L
.26883 .3655 L
.26895 .36362 L
.26899 .36309 L
.26902 .36258 L
.26908 .36232 L
.26912 .36253 L
.26915 .36273 L
.26922 .36316 L
.26935 .36393 L
.26983 .36667 L
.27174 .37647 L
Mistroke
.27357 .38432 L
.27551 .39099 L
.27739 .39578 L
.27841 .39772 L
.27888 .39846 L
.27938 .39913 L
.27984 .39965 L
.28034 .40011 L
.2808 .40043 L
.28123 .40065 L
.28147 .40074 L
.2816 .40077 L
.28173 .4008 L
.28185 .40082 L
.28191 .40083 L
.28196 .40084 L
.28202 .40084 L
.28205 .40084 L
.28209 .40084 L
.28214 .40085 L
.2822 .40085 L
.28226 .40085 L
.28232 .40084 L
.28237 .40084 L
.28243 .40083 L
.28255 .40082 L
.28267 .4008 L
.28287 .40074 L
.2831 .40067 L
.28355 .40044 L
.28403 .40011 L
.28491 .39924 L
.28537 .39865 L
.2855 .39848 L
.28557 .39838 L
.28563 .39828 L
.28569 .39819 L
.28573 .39796 L
.28576 .39746 L
.28587 .39564 L
.28689 .3798 L
.28878 .35062 L
.29061 .32306 L
.29162 .30793 L
.2921 .30081 L
.29255 .29423 L
.29266 .29255 L
.29277 .29097 L
.29283 .2901 L
.29286 .28968 L
Mistroke
.29289 .28987 L
.293 .29052 L
.2935 .29337 L
.29441 .29829 L
.29631 .30751 L
.29814 .315 L
.30009 .3215 L
.30196 .32632 L
.30299 .32836 L
.30395 .32992 L
.30491 .3311 L
.30534 .3315 L
.3058 .33187 L
.30626 .33215 L
.30647 .33225 L
.3067 .33234 L
.30695 .33241 L
.30709 .33244 L
.30722 .33247 L
.30728 .33247 L
.30734 .33248 L
.30739 .33249 L
.30745 .33249 L
.30751 .33249 L
.30756 .3325 L
.30762 .3325 L
.30766 .3325 L
.30769 .33249 L
.30775 .33249 L
.30778 .33249 L
.30782 .33249 L
.30793 .33248 L
.308 .33247 L
.30807 .33246 L
.30819 .33243 L
.30844 .33236 L
.30867 .33228 L
.3091 .33207 L
.30932 .33194 L
.30938 .3319 L
.30945 .33186 L
.30951 .33181 L
.30956 .33124 L
.30962 .33038 L
.30967 .32957 L
.30979 .32774 L
.31004 .32403 L
.31048 .31741 L
.31148 .30254 L
.31335 .27517 L
Mistroke
.3143 .26145 L
.31533 .24661 L
.31581 .23978 L
.31607 .23605 L
.31632 .23254 L
.31655 .22927 L
.31662 .22837 L
.31665 .22792 L
.31668 .2278 L
.31675 .22821 L
.3168 .22859 L
.31724 .23142 L
.31911 .24277 L
.3211 .25344 L
.32302 .26235 L
.32487 .26965 L
.32683 .27606 L
.32872 .28091 L
.33053 .28436 L
.33146 .28567 L
.33194 .28622 L
.33247 .28674 L
.33271 .28694 L
.33297 .28714 L
.33309 .28722 L
.33321 .2873 L
.33326 .28733 L
.33332 .28736 L
.33338 .28673 L
.33343 .28593 L
.33445 .27114 L
.33627 .24493 L
.33821 .21753 L
.3391 .20507 L
.33958 .19854 L
.34008 .19163 L
.34022 .18966 L
.34036 .18782 L
.34042 .18701 L
.34048 .18619 L
.34051 .18644 L
.34055 .18672 L
.34061 .18721 L
.3411 .19101 L
.34206 .19827 L
.34401 .21192 L
.34589 .2238 L
.34788 .23509 L
.3498 .24463 L
.35165 .25258 L
Mistroke
.35361 .25972 L
.35459 .26275 L
.3555 .26529 L
.35638 .26744 L
.35663 .26801 L
.35687 .26853 L
.35698 .26876 L
.35703 .26887 L
.35709 .26899 L
.35712 .26905 L
.35715 .269 L
.35721 .26816 L
.35732 .26652 L
.35929 .23829 L
.36119 .2116 L
.36302 .18638 L
.3635 .17981 L
.36395 .1736 L
.36408 .17195 L
.36421 .17019 L
.36424 .16975 L
.36427 .16927 L
.36431 .16936 L
.36435 .1697 L
.36448 .17092 L
.36496 .17532 L
.36694 .19265 L
.36884 .20804 L
.37067 .22163 L
.37262 .23477 L
.3745 .24612 L
.3765 .25676 L
.37842 .26563 L
.3793 .26924 L
.3798 .27117 L
.38027 .27289 L
.3805 .27372 L
.38073 .27449 L
.38085 .27491 L
.38089 .27503 L
.38092 .27515 L
.38096 .27519 L
.38099 .27474 L
.38122 .27131 L
.38224 .2567 L
.38416 .22926 L
.38602 .20332 L
.38696 .19033 L
.38746 .18351 L
.38773 .17976 L
Mistroke
.38787 .17792 L
.38799 .17623 L
.38806 .17539 L
.38811 .17504 L
.38817 .17568 L
.38824 .17639 L
.38851 .17921 L
.38898 .18408 L
.38989 .19339 L
.39176 .21133 L
.39374 .229 L
.39565 .24461 L
.39748 .2583 L
.39943 .27143 L
.40131 .28266 L
.40227 .28786 L
.40331 .29307 L
.4038 .29537 L
.40431 .2977 L
.40452 .29863 L
.40464 .29913 L
.4047 .29938 L
.40475 .2996 L
.40481 .29889 L
.40488 .29789 L
.405 .29613 L
.40523 .29269 L
.40712 .2652 L
.40893 .23923 L
.40994 .22501 L
.41086 .21207 L
.41133 .20556 L
.41156 .20239 L
.41177 .19952 L
.41182 .19877 L
.41188 .19795 L
.41194 .19803 L
.41199 .19862 L
.41222 .20132 L
.41272 .20697 L
.41462 .22754 L
.41644 .24597 L
.41837 .26413 L
.42024 .28015 L
.42222 .29558 L
.42413 .30889 L
.42597 .32023 L
.42699 .32593 L
.42792 .33071 L
.42815 .33182 L
Mistroke
.42839 .33297 L
.42845 .33325 L
.42851 .33356 L
.42855 .33373 L
.42859 .33362 L
.42862 .33311 L
.42865 .33262 L
.42889 .32898 L
.42981 .3153 L
.43165 .28803 L
.43361 .25954 L
.43458 .24558 L
.43506 .2388 L
.43527 .23583 L
.4355 .2326 L
.43561 .23109 L
.43566 .23027 L
.43572 .22964 L
.43577 .23026 L
.43583 .23093 L
.43593 .23209 L
.43637 .23739 L
.43732 .24825 L
.43928 .26973 L
.44118 .28884 L
.443 .30573 L
.44494 .32209 L
.44681 .33625 L
.44879 .34957 L
.44979 .35556 L
.45071 .36071 L
.4516 .36533 L
.45185 .3666 L
.4521 .36776 L
.4522 .36826 L
.45231 .36878 L
.45237 .36905 L
.4524 .36875 L
.45243 .36827 L
.45255 .36655 L
.45442 .33792 L
.45641 .30805 L
.45741 .29331 L
.45833 .27981 L
.45879 .27308 L
.45923 .2668 L
.45933 .26526 L
.45944 .26365 L
.4595 .26281 L
.45954 .26267 L
Mistroke
.45957 .26304 L
.45968 .26437 L
.46018 .27025 L
.46206 .29145 L
.46406 .31223 L
.46598 .33052 L
.46784 .34649 L
.46981 .3617 L
.4717 .37462 L
.47353 .38542 L
.47454 .39075 L
.47547 .39518 L
.47593 .39723 L
.47606 .39777 L
.47613 .39807 L
.47616 .39821 L
.47619 .39834 L
.47625 .39747 L
.4763 .39654 L
.47643 .39456 L
.47734 .38035 L
.47925 .35102 L
.48108 .32331 L
.4821 .30807 L
.48258 .30092 L
.48303 .29434 L
.48314 .29263 L
.48325 .29104 L
.48331 .29023 L
.48336 .29019 L
.4834 .29057 L
.48343 .29094 L
.48349 .29163 L
.48399 .29735 L
.4849 .30745 L
.48674 .32664 L
.4887 .34531 L
.49058 .36156 L
.49258 .37696 L
.49451 .38995 L
.49636 .40075 L
.49731 .40559 L
.49834 .4104 L
.49881 .41244 L
.49932 .4145 L
.49957 .41545 L
.4998 .41633 L
.4999 .4167 L
.49995 .41689 L
.50001 .4169 L
Mistroke
.50007 .41593 L
.50013 .41502 L
.50024 .41328 L
.5021 .38402 L
.50408 .35346 L
.50507 .33843 L
.50599 .32458 L
.50644 .31774 L
.50687 .31132 L
.50699 .3096 L
.50705 .30866 L
.50711 .30778 L
.50714 .30732 L
.50718 .30767 L
.50725 .30844 L
.50737 .30979 L
.50782 .31465 L
.50981 .33491 L
.51172 .35266 L
.51356 .36807 L
.51552 .38267 L
.5174 .39495 L
.51941 .4061 L
.52134 .415 L
.52223 .41847 L
.52273 .42027 L
.5232 .42184 L
.52344 .4226 L
.52366 .42329 L
.52372 .42347 L
.52376 .42357 L
.52379 .42366 L
.52382 .42357 L
.52386 .42298 L
.52392 .42194 L
.52416 .41817 L
.52517 .40212 L
.52704 .37302 L
.52902 .34263 L
.53001 .32767 L
.53049 .32044 L
.53072 .31703 L
.53082 .31554 L
.53088 .31469 L
.53093 .3139 L
.53098 .31383 L
.53104 .3144 L
.53115 .31556 L
.53139 .31798 L
.53182 .3223 L
Mistroke
.53276 .3315 L
.53475 .34941 L
.53667 .36489 L
.53851 .37812 L
.54047 .3904 L
.54236 .40048 L
.54418 .40854 L
.54519 .41234 L
.54611 .4154 L
.54702 .41799 L
.54724 .41857 L
.54747 .41915 L
.54754 .41932 L
.54758 .4194 L
.54761 .41948 L
.54767 .41876 L
.54773 .41774 L
.54797 .41392 L
.54987 .38413 L
.55169 .356 L
.5527 .34055 L
.55363 .32657 L
.55408 .3197 L
.55433 .31609 L
.55446 .31411 L
.55458 .31224 L
.5547 .31047 L
.55474 .30999 L
.55477 .30968 L
.55483 .31028 L
.55506 .31245 L
.55549 .31661 L
.55732 .33306 L
.55927 .34886 L
.56114 .36241 L
.56313 .375 L
.56505 .38536 L
.5669 .3937 L
.56886 .40081 L
.56984 .40368 L
.57031 .40493 L
.57075 .40597 L
.57118 .40691 L
.57128 .40712 L
.57134 .40723 L
.57139 .40734 L
.57142 .4074 L
.57146 .40698 L
.57151 .40607 L
.57163 .40429 L
Mistroke
.57257 .38949 L
.57454 .35899 L
.57644 .33015 L
.57739 .31584 L
.57781 .30964 L
.57826 .3029 L
.57838 .30111 L
.57845 .30013 L
.57851 .29921 L
.57856 .29843 L
.57862 .29876 L
.57868 .29935 L
.57874 .29989 L
.5792 .30408 L
.5802 .31298 L
.58211 .32859 L
.58394 .34206 L
.58589 .35472 L
.58776 .36527 L
.58976 .37473 L
.59168 .38213 L
.59353 .38767 L
.59448 .38993 L
.59496 .39091 L
.59509 .39116 L
.59517 .3913 L
.5952 .39136 L
.59523 .39142 L
.59527 .391 L
.5953 .39048 L
.59537 .38939 L
.59549 .38748 L
.59749 .35654 L
.59942 .32729 L
.60127 .29965 L
.60177 .29233 L
.60201 .28878 L
.60223 .28554 L
.60229 .2847 L
.60235 .28379 L
.60241 .28366 L
.60247 .28416 L
.60272 .28643 L
.60325 .29116 L
.60514 .30702 L
.60695 .32081 L
.60889 .33389 L
.61075 .34492 L
.61273 .35496 L
.61463 .363 L
Mistroke
.61647 .3692 L
.61749 .37203 L
.61842 .3742 L
.61864 .37467 L
.61888 .37514 L
.61894 .37526 L
.61901 .37538 L
.61904 .37545 L
.61908 .37502 L
.61914 .37399 L
.61938 .37026 L
.6203 .35624 L
.62221 .32721 L
.62405 .29979 L
.62508 .28468 L
.62556 .2776 L
.62577 .27451 L
.62589 .27276 L
.62601 .27113 L
.62612 .26954 L
.62618 .26865 L
.62623 .26886 L
.62629 .2694 L
.62636 .26999 L
.62647 .27105 L
.62698 .27559 L
.62789 .28353 L
.62986 .29941 L
.63175 .31315 L
.63357 .3249 L
.6355 .33584 L
.63737 .34484 L
.63935 .35276 L
.64034 .35608 L
.64126 .35879 L
.64215 .36106 L
.6424 .36165 L
.64264 .36218 L
.64275 .36241 L
.6428 .36253 L
.64286 .36262 L
.64292 .36172 L
.64298 .36074 L
.64309 .35902 L
.64496 .33058 L
.64695 .30091 L
.64795 .28627 L
.64887 .27285 L
.64933 .26617 L
.64976 .25994 L
Mistroke
.64987 .25841 L
.64992 .25759 L
.64998 .25682 L
.65004 .25682 L
.65007 .2571 L
.6501 .25741 L
.65021 .25846 L
.65071 .26306 L
.6527 .28043 L
.65463 .29559 L
.65647 .30871 L
.65844 .32106 L
.66034 .3314 L
.66216 .33989 L
.6641 .34737 L
.66499 .35028 L
.66596 .35306 L
.66622 .35372 L
.66636 .35407 L
.66649 .3544 L
.66655 .35454 L
.66661 .35469 L
.66665 .35477 L
.66668 .35456 L
.66672 .35405 L
.66675 .35358 L
.66698 .35003 L
.66795 .33535 L
.66982 .3073 L
.67181 .27802 L
.67281 .26357 L
.67325 .25726 L
.67348 .25393 L
.6736 .25218 L
.67367 .25122 L
.67373 .25034 L
.67378 .24954 L
.67384 .2495 L
.67388 .24983 L
.67391 .25014 L
.67397 .25072 L
.67419 .25284 L
.67462 .25693 L
.67557 .26565 L
.67742 .28156 L
.67939 .29691 L
.68129 .31014 L
.68311 .32142 L
.68506 .33187 L
.68692 .34041 L
Mistroke
.68788 .34417 L
.68891 .34784 L
.68939 .34939 L
.68991 .35093 L
.69015 .35164 L
.69028 .35198 L
.69033 .35213 L
.69039 .35229 L
.69044 .35243 L
.69049 .3523 L
.69054 .3515 L
.6906 .35065 L
.69083 .34719 L
.69282 .31713 L
.69474 .2887 L
.69659 .26184 L
.69708 .25475 L
.69732 .2513 L
.69743 .24979 L
.69754 .24815 L
.6976 .24727 L
.69766 .24744 L
.69771 .24798 L
.69777 .24856 L
.69802 .25103 L
.69855 .25618 L
.70055 .27459 L
.70248 .29075 L
.70433 .30481 L
.7063 .31816 L
.7082 .32943 L
.71003 .33881 L
.71197 .34721 L
.71287 .35053 L
.71334 .35215 L
.71385 .35376 L
.7141 .35453 L
.71417 .35473 L
.7142 .35484 L
.71424 .35495 L
.71428 .35505 L
.71431 .35468 L
.71435 .35417 L
.71438 .35369 L
.71487 .34621 L
.71583 .33155 L
.71771 .30349 L
.71971 .27421 L
.72019 .26716 L
.72071 .25974 L
Mistroke
.72115 .25343 L
.72127 .25171 L
.72133 .25076 L
.7214 .24986 L
.72146 .24971 L
.72149 .25004 L
.72152 .25036 L
.72163 .25149 L
.72348 .2696 L
.72537 .28657 L
.72737 .30294 L
.7293 .31708 L
.73115 .32916 L
.73313 .34036 L
.73503 .34953 L
.73686 .35687 L
.73734 .35857 L
.7378 .36007 L
.73792 .36046 L
.73799 .36067 L
.73805 .36087 L
.73808 .36097 L
.73812 .36063 L
.73816 .36003 L
.73819 .35948 L
.73833 .35746 L
.73881 .35012 L
.74071 .32128 L
.74255 .29403 L
.74357 .27903 L
.7445 .26554 L
.74497 .25884 L
.74509 .25701 L
.74516 .25601 L
.7452 .25553 L
.74523 .25508 L
.74528 .25544 L
.74534 .25606 L
.74547 .25736 L
.74638 .26661 L
.7483 .28487 L
.75014 .30094 L
.7521 .31643 L
.75399 .32975 L
.7558 .34107 L
.75773 .35152 L
.75959 .36001 L
.76054 .36374 L
.76103 .36555 L
.76131 .36649 L
Mistroke
.76157 .36736 L
.76169 .36775 L
.76182 .36816 L
.76185 .36826 L
.76188 .36837 L
.76192 .3682 L
.76196 .36765 L
.76208 .36567 L
.76255 .35845 L
.76347 .34443 L
.76546 .31456 L
.76737 .28632 L
.76786 .27907 L
.76833 .27227 L
.76875 .26619 L
.76887 .26453 L
.76893 .26362 L
.76899 .26277 L
.76904 .26205 L
.76909 .26236 L
.76915 .26298 L
.7692 .26355 L
.77015 .27322 L
.77116 .28311 L
.77308 .30069 L
.77492 .31607 L
.77688 .33078 L
.77877 .34332 L
.78059 .35387 L
.78252 .36348 L
.78439 .37113 L
.78489 .37292 L
.78535 .37449 L
.78559 .37524 L
.78565 .37545 L
.78569 .37555 L
.78572 .3756 L
.78577 .37472 L
.78584 .37376 L
.78637 .36557 L
.78826 .33663 L
.79009 .30929 L
.7911 .29429 L
.79203 .2807 L
.79248 .27403 L
.79261 .2722 L
.79274 .27026 L
.79278 .26981 L
.79281 .26932 L
.79287 .26877 L
Mistroke
.79293 .26939 L
.79298 .26996 L
.79389 .27925 L
.7958 .2975 L
.79763 .31356 L
.79957 .32901 L
.80145 .34228 L
.80344 .35464 L
.80536 .36484 L
.80721 .37309 L
.80814 .37667 L
.80864 .37841 L
.80891 .37932 L
.80917 .38016 L
.80929 .38053 L
.80942 .38092 L
.80945 .38103 L
.80949 .38114 L
.80952 .38123 L
.80956 .38075 L
.80968 .37876 L
.81015 .37153 L
.81107 .35744 L
.81304 .32751 L
.81494 .29922 L
.81589 .28518 L
.81631 .27909 L
.81642 .27743 L
.81654 .27566 L
.81659 .27493 L
.81665 .27415 L
.81671 .27426 L
.81676 .27483 L
.81724 .27975 L
.8177 .28432 L
.8187 .29406 L
.82068 .31193 L
.82259 .32749 L
.82442 .34092 L
.82637 .35353 L
.82825 .36404 L
.83024 .37345 L
.83124 .37749 L
.83216 .38081 L
.83263 .38233 L
.83306 .38367 L
.83317 .384 L
.83323 .38416 L
.83328 .38431 L
.83334 .38436 L
Mistroke
.83337 .38388 L
.8334 .38337 L
.83352 .38162 L
.83401 .37389 L
.83602 .34319 L
.83794 .31416 L
.83883 .30102 L
.83933 .29361 L
.8398 .28673 L
.84004 .28324 L
.84026 .27995 L
.84039 .27814 L
.84042 .27764 L
.84046 .27711 L
.84049 .27701 L
.84052 .27736 L
.84076 .27976 L
.84177 .28978 L
.84371 .30765 L
.84557 .32327 L
.84755 .33818 L
.84946 .35085 L
.8513 .36149 L
.85325 .37112 L
.85513 .37875 L
.85609 .38203 L
.85659 .38359 L
.85687 .38439 L
.857 .38477 L
.85706 .38494 L
.85709 .38502 L
.85713 .38511 L
.85719 .38448 L
.85724 .38359 L
.85736 .38185 L
.8576 .37801 L
.85813 .36989 L
.85905 .3557 L
.86086 .32817 L
.86187 .3131 L
.8628 .29938 L
.8637 .28608 L
.86392 .28286 L
.86416 .27942 L
.86422 .27841 L
.86426 .27791 L
.86429 .27757 L
.86435 .27816 L
.86441 .27881 L
.86465 .28121 L
Mistroke
.86655 .29922 L
.86837 .31505 L
.8703 .33029 L
.87217 .34335 L
.87415 .35552 L
.87606 .36556 L
.8779 .37365 L
.87893 .3775 L
.87985 .38059 L
.88032 .38199 L
.88057 .38268 L
.8807 .38305 L
.88082 .38339 L
.88088 .38354 L
.88093 .38368 L
.881 .38306 L
.88105 .38219 L
.8813 .37833 L
.88174 .37155 L
.88358 .34329 L
.88554 .31377 L
.88652 .29932 L
.88699 .29228 L
.88743 .28587 L
.88786 .27965 L
.88796 .27814 L
.88802 .27731 L
.88807 .27653 L
.88813 .27658 L
.88819 .27715 L
.88831 .27827 L
.88925 .28739 L
.8911 .30418 L
.89307 .32039 L
.89496 .33436 L
.89678 .34628 L
.89872 .35733 L
.90059 .36636 L
.90258 .37423 L
.90357 .37751 L
.90405 .37895 L
.90449 .38015 L
.9046 .38044 L
.90466 .38059 L
.90471 .38073 L
.90477 .38073 L
.9048 .38024 L
.90484 .37972 L
.90495 .37795 L
.90538 .37126 L
Mistroke
.90633 .35666 L
.90832 .32643 L
.91024 .29785 L
.91112 .28495 L
.91136 .28143 L
.91162 .27765 L
.91174 .27592 L
.9118 .27498 L
.91186 .2741 L
.91192 .27361 L
.91197 .27416 L
.91209 .27531 L
.91304 .28448 L
.91405 .29378 L
.91605 .31086 L
.91798 .32565 L
.91983 .33832 L
.9218 .35012 L
.9237 .35983 L
.92552 .36766 L
.92654 .37137 L
.92747 .37436 L
.92793 .3757 L
.92817 .37637 L
.9283 .37672 L
.92843 .37704 L
.92848 .37718 L
.92853 .37732 L
.92859 .37705 L
.92865 .37619 L
.9289 .37234 L
.92934 .36559 L
.93117 .33763 L
.93312 .30839 L
.93402 .29507 L
.93499 .28076 L
.93525 .27698 L
.93539 .27492 L
.93553 .273 L
.93564 .27136 L
.9357 .27044 L
.93576 .27075 L
.93582 .27138 L
.93589 .27205 L
.93602 .27328 L
.93699 .28253 L
.93894 .29997 L
.94083 .31519 L
.94283 .32965 L
.94476 .34191 L
Mistroke
.94661 .35214 L
.94859 .36134 L
.95049 .36856 L
.95144 .3716 L
.95186 .37279 L
.95208 .37339 L
.9522 .37371 L
.95231 .37401 L
.95237 .37415 L
.95243 .37337 L
.95256 .37133 L
.9528 .36775 L
.95325 .36073 L
.95426 .34535 L
.95624 .31552 L
.95723 .30084 L
.95815 .28731 L
.95903 .27437 L
.95929 .27066 L
.95941 .26884 L
.95947 .26805 L
.95953 .26732 L
.95958 .26786 L
.95963 .26836 L
.95975 .26945 L
.95998 .27174 L
.96185 .28913 L
.96384 .30598 L
.96575 .32058 L
.96759 .3331 L
.96955 .34478 L
.97143 .35442 L
.97344 .36294 L
.97537 .36947 L
.97619 .37176 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  Background->GrayLevel[1],
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005X0000gR000`400?l00000o`00003oo`3ooomY
0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY
0?ooo`00o`3ooomY0?ooo`006P3oool00`000000oooo0?ooo`0B0?ooo`030?l0003oool0oooo0?l0
oooo=`3oool001X0oooo00<000000?ooo`3oool04P3oool00`3o0000oooo0?ooo`3o0?ooocL0oooo
000J0?ooo`030000003oool0oooo0180oooo00<0o`000?ooo`3oool0o`3ooolg0?ooo`006P3oool0
0`000000oooo0?ooo`0B0?ooo`030?l0003oool0oooo0?l0oooo=`3oool001X0oooo0P00000C0?oo
o`030?l0003oool0oooo0?l0oooo=`3oool001X0oooo00<000000?ooo`3oool04P3oool00`3o0000
oooo0?ooo`3o0?ooocL0oooo000J0?ooo`030000003oool0oooo0180oooo00<0o`000?ooo`3oool0
o`3ooolg0?ooo`006P3oool00`000000oooo0?ooo`0B0?ooo`030?l0003oool0oooo0?l0oooo=`3o
ool001X0oooo00<000000?ooo`3oool04P3oool00`3o0000oooo0?ooo`3o0?ooocL0oooo000J0?oo
o`030000003oool0oooo0180oooo00<0o`000?ooo`3oool0o`3ooolg0?ooo`006P3oool00`000000
oooo0?ooo`0B0?ooo`030?l0003oool0oooo0?l0oooo=`3oool001X0oooo0P00000C0?ooo`030?l0
003oool0oooo02h0oooo100000050?ooo`8000001@3oool5000003`0oooo100000050?ooo`800000
103oool5000003`0oooo100000050?ooo`8000001@3oool3000003d0oooo100000050?ooo`800000
1`3oool300000080oooo000J0?ooo`030000003oool0oooo01<0oooo00<0o`000?ooo`3oool0;03o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo0P0000070?ooo`030000003o
ool0oooo03/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`800000103o
ool01@000000oooo0?ooo`3oool0000003/0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`020?ooo`800000103oool01@000000oooo0?ooo`3oool0000003/0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`020?ooo`800000203oool00`000000oooo0?ooo`010?ooo`00
6P3oool00`000000oooo0?ooo`0C0?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3oool0
9`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
>`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
?03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
>@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
0@3oool001X0oooo00<000000?ooo`3oool04`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003o
ool0oooo02L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo03/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo03/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo03T0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`080?ooo`H000000P3o
ool001X0oooo00<000000?ooo`3oool04`3oool0103o0000oooo0?ooo`3oool20?l002T0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo03/0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo03X0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`800000?03oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00T0oooo00@000000?ooo`3oool000000`3oool001X0oooo00<0
00000?ooo`3oool04`3oool0103o0000oooo0?ooo`3oool20?l002T0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`090?ooo`040000003oool0oooo000000<0oooo000J0?ooo`030000
003oool0oooo01<0oooo00@0o`000?ooo`3oool0oooo0P3o000Y0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool02@3oool3000003d0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`080?ooo`050000003oool0oooo0?ooo`000000>`3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00P0oooo00D000000?ooo`3oool0oooo0000000k0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool02P3oool00`000000oooo000000030?ooo`006P3oool00`00
0000oooo0?ooo`0C0?ooo`040?l0003oool0oooo0?ooo`80o`00:P3oool4000000h0oooo00<00000
0?ooo`3oool0?03oool4000000`0oooo0`00000m0?ooo`@00000303oool3000003d0oooo1000000>
0?ooo`8000000`3oool001X0oooo0P00000D0?ooo`040?l0003oool0oooo0?ooo`80o`00o`3ooolc
0?ooo`006P3oool00`000000oooo0?ooo`0C0?ooo`040?l0003oool0oooo0?ooo`80o`00o`3ooolc
0?ooo`006P3oool00`000000oooo0?ooo`0D0?ooo`030?l0003oool0oooo0080o`00o`3ooolc0?oo
o`006P3oool00`000000oooo0?ooo`0D0?ooo`030?l0003oool0oooo0080o`00o`3ooolc0?ooo`00
6P3oool00`000000oooo0?ooo`0D0?ooo`030?l0003oool0oooo0080o`00o`3ooolc0?ooo`006P3o
ool00`000000oooo0?ooo`0D0?ooo`030?l0003oool0oooo0080o`00o`3ooolc0?ooo`006P3oool0
0`000000oooo0?ooo`0D0?ooo`030?l0003oool0oooo0080o`00o`3ooolc0?ooo`004P3ooolO0000
00050?l000000000o`000000003o0000o`00000_000000@0oooo000J0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool0103oool01@3o0000oooo0?l0003oool0o`0000@0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool02P3oool001X0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`040?oo
o`050?l0003oool0o`000?ooo`3o0000103oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0:0?ooo`006P3oool00`00
0000oooo0?ooo`0D0?ooo`050?l0003oool0o`000?ooo`3o0000o`3ooolc0?ooo`006P3oool00`00
0000oooo0?ooo`0D0?ooo`050?l0003oool0o`000?ooo`3o0000o`3ooolc0?ooo`006P3oool00`00
0000oooo0?ooo`0D0?ooo`030?l0003oool0o`000080oooo00<0o`000?ooo`3oool0o`3oool`0?oo
o`006P3oool00`000000oooo0?ooo`0D0?ooo`030?l0003oool0o`000080oooo00<0o`000?ooo`3o
ool0o`3oool`0?ooo`006P3oool00`000000oooo0?ooo`0D0?ooo`030?l0003oool0o`000080oooo
00<0o`000?ooo`3oool0o`3oool`0?ooo`006P3oool2000001D0oooo00<0o`000?ooo`3o00000P3o
ool00`3o0000oooo0?ooo`3o0?oooc00oooo000J0?ooo`030000003oool0oooo01@0oooo00<0o`00
0?ooo`3o00000P3oool00`3o0000oooo0?ooo`3o0?oooc00oooo000J0?ooo`030000003oool0oooo
01D0oooo0P3o00020?ooo`030?l0003oool0oooo0?l0oooo<03oool001X0oooo00<000000?ooo`3o
ool05@3oool20?l00080oooo00<0o`000?ooo`3oool0o`3oool`0?ooo`006P3oool00`000000oooo
0?ooo`0E0?ooo`80o`000P3oool00`3o0000oooo0?ooo`3o0?oooc00oooo000J0?ooo`030000003o
ool0oooo01D0oooo0P3o00020?ooo`030?l0003oool0oooo0?l0oooo<03oool001X0oooo00<00000
0?ooo`3oool05@3oool20?l00080oooo00<0o`000?ooo`3oool0o`3oool`0?ooo`006P3oool20000
01H0oooo00D0o`000?ooo`3oool0oooo0?l0003o0?oooc80oooo000J0?ooo`030000003oool0oooo
01D0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?ooo`3o0?ooobl0oooo000J0?ooo`03
0000003oool0oooo01D0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?ooo`3o0?ooobl0
oooo000J0?ooo`030000003oool0oooo01D0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo
0?ooo`3o0?ooobl0oooo000J0?ooo`030000003oool0oooo01X0oooo00<0o`000?ooo`3oool0o`3o
ool_0?ooo`006P3oool00`000000oooo0?ooo`0J0?ooo`030?l0003oool0oooo0?l0oooo;`3oool0
01X0oooo00<000000?ooo`3oool06P3oool00`3o0000oooo0?ooo`3o0?ooobl0oooo000J0?ooo`03
0000003oool0oooo01X0oooo00<0o`000?ooo`3oool0o`3oool_0?ooo`006P3oool2000001/0oooo
00<0o`000?ooo`3oool0o`3oool_0?ooo`006P3oool00`000000oooo0?ooo`0J0?ooo`030?l0003o
ool0oooo0?l0oooo;`3oool001X0oooo00<000000?ooo`3oool06P3oool00`3o0000oooo0?ooo`3o
0?ooobl0oooo000J0?ooo`030000003oool0oooo01X0oooo00<0o`000?ooo`3oool0o`3oool_0?oo
o`006P3oool00`000000oooo0?ooo`0J0?ooo`030?l0003oool0oooo0?l0oooo;`3oool001X0oooo
00<000000?ooo`3oool06P3oool00`3o0000oooo0?ooo`3o0?ooobl0oooo000J0?ooo`030000003o
ool0oooo01X0oooo00<0o`000?ooo`3oool0o`3oool_0?ooo`006P3oool2000001/0oooo00<0o`00
0?ooo`3oool0o`3oool_0?ooo`006P3oool00`000000oooo0?ooo`0J0?ooo`030?l0003oool0oooo
05<0oooo00<0o`000?ooo`3oool0f03oool001X0oooo00<000000?ooo`3oool06P3oool00`3o0000
oooo0?ooo`1C0?ooo`030?l0003oool0oooo0=P0oooo000J0?ooo`030000003oool0oooo01X0oooo
00<0o`000?ooo`3oool0D`3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo0=00oooo
000J0?ooo`030000003oool0oooo01X0oooo00<0o`000?ooo`3oool0D`3oool00`3o0000oooo0?oo
o`050?ooo`030?l0003oool0oooo0=00oooo0000103oool000000000000000050?ooo`8000001@3o
ool4000000H0oooo00<000000?ooo`3oool06P3oool00`3o0000oooo0?ooo`1C0?ooo`030?l0003o
ool0oooo00D0oooo00<0o`000?ooo`3oool0d03oool000050000003oool0oooo0?ooo`000000103o
ool2000000@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo01X0oooo00<0o`000?ooo`3oool0B`3oool00`3o0000oooo0?ooo`050?ooo`030?l0003o
ool0oooo00D0oooo00<0o`000?ooo`3oool0d03oool000050000003oool0oooo0?ooo`0000003`3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01/0oooo00<0o`000?ooo`3oool0BP3o
ool00`3o0000oooo0?ooo`050?ooo`80o`001P3oool00`3o0000oooo0?ooo`3@0?ooo`0000D00000
0?ooo`3oool0oooo0000000?0?ooo`030000003oool0oooo00<0oooo0`00000K0?ooo`030?l0003o
ool0oooo04X0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo0P3o003A0?ooo`001000000<
0?ooo`@000001P3oool00`000000oooo0?ooo`0K0?ooo`030?l0003oool0oooo04X0oooo00<0o`00
0?ooo`3oool01@3oool20?l000H0oooo0P3o003A0?ooo`0000<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo01/0oooo00<0o`000?ooo`3oool0BP3oool00`3o
0000oooo0?ooo`050?ooo`80o`001@3oool00`3o0000oooo0?l000060?ooo`030?l0003oool0oooo
0<P0oooo00000`3oool000000?ooo`0=0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool06`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0oooo04D0oooo00<0o`000?ooo`3o
ool0103oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000H0oooo00<0o`000?ooo`3o
ool0b03oool00080oooo0`00000;0?ooo`D000001@3oool00`000000oooo0?ooo`0K0?ooo`030?l0
003oool0oooo0080oooo00<0o`000?ooo`3oool0A03oool20?l000H0oooo00@0o`000?ooo`3oool0
o`00103oool00`3o0000oooo0?l000060?ooo`030?l0003oool0oooo0<P0oooo000J0?ooo`030000
003oool0oooo01/0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?ooo`140?ooo`030?l0
003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`00103oool00`3o0000oooo0?l000060?ooo`03
0?l0003oool0oooo0<P0oooo000J0?ooo`030000003oool0oooo01/0oooo00<0o`000?ooo`3oool0
0P3oool00`3o0000oooo0?ooo`140?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3oool0
o`00103oool00`3o0000oooo0?l000060?ooo`030?l0003oool0oooo0<P0oooo000J0?ooo`800000
703oool0103o0000oooo0?ooo`3oool20?l004H0oooo00<0o`000?ooo`3o00001@3oool0103o0000
oooo0?ooo`3o00040?ooo`030?l0003oool0o`0000H0oooo00<0o`000?ooo`3oool0b03oool001X0
oooo00<000000?ooo`3oool06`3oool0103o0000oooo0?ooo`3oool20?l004H0oooo00<0o`000?oo
o`3o00001@3oool0103o0000oooo0?ooo`3o00040?ooo`030?l0003oool0o`0000H0oooo00<0o`00
0?ooo`3oool0b03oool001X0oooo00<000000?ooo`3oool06`3oool0103o0000oooo0?ooo`3oool2
0?l004H0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00040?ooo`030?l0003o
ool0o`0000D0oooo00<0o`000?ooo`3o0000b@3oool001X0oooo00<000000?ooo`3oool06`3oool0
103o0000oooo0?ooo`3oool20?l004H0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo
0?ooo`3o00040?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o0000b@3oool001X0oooo
00<000000?ooo`3oool06`3oool0103o0000oooo0?ooo`3oool20?l004H0oooo00@0o`000?ooo`3o
ool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00<0
o`000?ooo`3o0000b@3oool001X0oooo00<000000?ooo`3oool06`3oool0103o0000oooo0?ooo`3o
ool20?l003l0oooo00<0o`000?ooo`3oool0103oool0103o0000oooo0?ooo`3o00030?ooo`050?l0
003oool0oooo0?ooo`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`030?l0003oool0o`00
0<T0oooo000J0?ooo`030000003oool0oooo01/0oooo00@0o`000?ooo`3oool0oooo0P3o000o0?oo
o`030?l0003oool0oooo00@0oooo00@0o`000?ooo`3oool0o`000`3oool00`3o0000oooo0?ooo`02
0?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`030?l0003o
ool0o`0000H0oooo00<0o`000?ooo`3oool0`03oool001X0oooo0P00000L0?ooo`040?l0003oool0
oooo0?ooo`80o`00?`3oool00`3o0000oooo0?ooo`040?ooo`040?l0003oool0oooo0?l000<0oooo
00<0o`000?ooo`3oool00P3oool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3o
ool00P3oool00`3o0000oooo0?l000060?ooo`030?l0003oool0oooo0<00oooo000J0?ooo`030000
003oool0oooo01/0oooo00L0o`000?ooo`3oool0oooo0?l0003oool0o`0003h0oooo00<0o`000?oo
o`3oool0103oool0103o0000oooo0?ooo`3o00030?ooo`030?l0003oool0oooo0080oooo00D0o`00
0?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o00001P3o
ool00`3o0000oooo0?ooo`300?ooo`006P3oool00`000000oooo0?ooo`0K0?ooo`070?l0003oool0
oooo0?ooo`3o0000oooo0?l0000n0?ooo`80o`001@3oool0103o0000oooo0?ooo`3o00030?ooo`03
0?l0003oool0oooo0080oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo
0?ooo`3o00000P3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0oooo0<00oooo000J0?oo
o`030000003oool0oooo01/0oooo00L0o`000?ooo`3oool0oooo0?l0003oool0o`0003h0oooo0P3o
00050?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`050?l0
003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?oo
o`3oool00`3oool00`3o0000oooo0?ooo`300?ooo`006P3oool00`000000oooo0?ooo`0L0?ooo`06
0?l0003oool0oooo0?l0003oool0o`00?@3oool00`3o0000oooo0?l000050?ooo`050?l0003oool0
oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`040?l0003oool0oooo0?l000@0oooo
00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo00<0oooo0P3o00310?ooo`00
6P3oool00`000000oooo0?ooo`0L0?ooo`060?l0003oool0oooo0?l0003oool0o`00?@3oool00`3o
0000oooo0?l000050?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`02
0?ooo`040?l0003oool0oooo0?l000@0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0
003oool0oooo00<0oooo0P3o001F0?ooo`030?l0003oool0oooo06P0oooo000J0?ooo`030000003o
ool0oooo01`0oooo00H0o`000?ooo`3oool0o`000?ooo`3o000m0?ooo`030?l0003oool0o`0000D0
oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l000040?ooo`040?l0003oool0oooo0?l0
00@0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo00<0oooo0P3o001>
0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`1P
0?ooo`006P3oool00`000000oooo0?ooo`0L0?ooo`060?l0003oool0oooo0?l0003oool0o`00?@3o
ool00`3o0000oooo0?l000050?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o00001@3o
ool00`3o0000oooo0?l000040?ooo`040?l0003oool0oooo0?l000<0oooo00<0o`000?ooo`3oool0
0P3oool00`3o0000oooo0?l0001>0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0
1@3oool00`3o0000oooo0?ooo`1P0?ooo`006P3oool2000001d0oooo00<0o`000?ooo`3o00000P3o
ool00`3o0000oooo0?ooo`0k0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3oool00P3o
ool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000@0oooo00@0o`000?ooo`3oool0o`00
0`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0004H0oooo00<0o`000?ooo`3oool0
1@3oool20?l000H0oooo0P3o00060?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0
F03oool001X0oooo00<000000?ooo`3oool0703oool00`3o0000oooo0?l000020?ooo`030?l0003o
ool0oooo03/0oooo00@0o`000?ooo`3oool0o`000`3oool00`3o0000oooo0?ooo`030?ooo`030?l0
003oool0o`0000D0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o00030?ooo`03
0?l0003oool0oooo0080oooo00<0o`000?ooo`3o0000AP3oool00`3o0000oooo0?ooo`050?ooo`80
o`001P3oool20?l000H0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`1H0?ooo`00
6P3oool00`000000oooo0?ooo`0L0?ooo`030?l0003oool0o`000080oooo00<0o`000?ooo`3oool0
>`3oool0103o0000oooo0?ooo`3o00030?ooo`030?l0003oool0oooo00@0oooo0P3o00060?ooo`80
o`00103oool0103o0000oooo0?ooo`3o00030?ooo`030?l0003oool0oooo0080oooo00<0o`000?oo
o`3o00001P3oool00`3o0000oooo0?ooo`0m0?ooo`80o`001P3oool20?l000H0oooo0P3o00060?oo
o`80o`001P3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo0500oooo000J0?ooo`03
0000003oool0oooo01`0oooo00<0o`000?ooo`3o00000P3oool00`3o0000oooo0?ooo`0k0?ooo`04
0?l0003oool0oooo0?l000<0oooo00<0o`000?ooo`3oool0103oool20?l000H0oooo0P3o00040?oo
o`040?l0003oool0oooo0?l000<0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l00006
0?ooo`030?l0003oool0oooo03d0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o0006
0?ooo`80o`001P3oool00`3o0000oooo0?ooo`1@0?ooo`006P3oool00`000000oooo0?ooo`0L0?oo
o`030?l0003oool0o`000080oooo00<0o`000?ooo`3oool0>`3oool01@3o0000oooo0?ooo`3oool0
o`000080oooo00<0o`000?ooo`3oool0103oool20?l000H0oooo0P3o00050?ooo`030?l0003oool0
o`0000<0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l000060?ooo`030?l0003oool0
oooo03D0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool2
0?l000H0oooo0P3o00060?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0=P3oool0
0`3o0000oooo0?ooo`0?0?ooo`006P3oool00`000000oooo0?ooo`0L0?ooo`030?l0003oool0o`00
0080oooo00<0o`000?ooo`3oool0>`3oool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`00
0?ooo`3oool0303oool20?l000D0oooo00<0o`000?ooo`3o00000`3oool00`3o0000oooo0?ooo`02
0?ooo`030?l0003oool0o`0000H0oooo00<0o`000?ooo`3oool0=@3oool00`3o0000oooo0?ooo`05
0?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o0006
0?ooo`030?l0003oool0oooo02h0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`0?
0?ooo`006P3oool00`000000oooo0?ooo`0L0?ooo`030?l0003oool0o`000080oooo00<0o`000?oo
o`3oool0>`3oool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool0303oool2
0?l000D0oooo00<0o`000?ooo`3o0000103oool01`3o0000oooo0?ooo`3oool0o`000?ooo`3o0000
1P3oool00`3o0000oooo0?ooo`0e0?ooo`030?l0003oool0oooo00D0oooo0P3o00060?ooo`80o`00
1P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo00<0o`000?ooo`3oool0
1@3oool20?l001l0oooo00<0o`000?ooo`3oool0103oool20?l000H0oooo0P3o000A0?ooo`006P3o
ool2000001h0oooo0P3o00020?ooo`030?l0003oool0oooo03/0oooo00D0o`000?ooo`3oool0oooo
0?l000020?ooo`030?l0003oool0oooo01<0oooo00<0o`000?ooo`3o0000103oool01`3o0000oooo
0?ooo`3oool0o`000?ooo`3o00001P3oool00`3o0000oooo0?ooo`0e0?ooo`80o`001P3oool20?l0
00H0oooo00<0o`000?ooo`3o0000103oool00`3o0000oooo0?l000060?ooo`80o`001P3oool20?l0
00H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00070?ooo`030?l0003oool0oooo
00D0oooo00<0o`000?ooo`3oool0103oool20?l000H0oooo0P3o00060?ooo`80o`004@3oool001X0
oooo00<000000?ooo`3oool07@3oool20?l00080oooo00<0o`000?ooo`3oool0>`3oool00`3o0000
oooo0?ooo`020?ooo`030?l0003oool0o`0001D0oooo00<0o`000?ooo`3o0000103oool01`3o0000
oooo0?ooo`3oool0o`000?ooo`3o00001P3oool00`3o0000oooo0?ooo`0e0?ooo`80o`001P3oool2
0?l000H0oooo00<0o`000?ooo`3o0000103oool00`3o0000oooo0?l000060?ooo`030?l0003oool0
o`0000D0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool2
0?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o000A0?ooo`006P3oool00`00
0000oooo0?ooo`0M0?ooo`050?l0003oool0oooo0?ooo`3o0000?@3oool00`3o0000oooo0?ooo`03
0?ooo`80o`005@3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0oooo0?ooo`3o00000P3o
ool00`3o0000oooo0?ooo`030?ooo`80o`00;P3oool00`3o0000oooo0?ooo`050?ooo`80o`001P3o
ool20?l000H0oooo00<0o`000?ooo`3o0000103oool00`3o0000oooo0?l000060?ooo`030?l0003o
ool0o`0000D0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3o
ool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o000A0?ooo`006P3oool0
0`000000oooo0?ooo`0M0?ooo`050?l0003oool0oooo0?ooo`3o0000?@3oool00`3o0000oooo0?oo
o`030?ooo`80o`005@3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0oooo0?ooo`3o0000
0P3oool00`3o0000oooo0?ooo`030?ooo`80o`00;P3oool00`3o0000oooo0?ooo`050?ooo`80o`00
1P3oool20?l000H0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o00050?ooo`03
0?l0003oool0o`0000D0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80
o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o000A0?ooo`00
6P3oool00`000000oooo0?ooo`0M0?ooo`050?l0003oool0oooo0?ooo`3o0000=P3oool00`3o0000
oooo0?ooo`030?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0503oool20?l000D0
oooo00@0o`000?ooo`3oool0o`000P3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`00
00H0oooo00<0o`000?ooo`3oool09@3oool00`3o0000oooo0?ooo`050?ooo`80o`001P3oool00`3o
0000oooo0?l000050?ooo`030?l0003oool0o`0000@0oooo00@0o`000?ooo`3oool0o`00103oool0
103o0000oooo0?ooo`3o00040?ooo`030?l0003oool0o`0000H0oooo0P3o00060?ooo`80o`001P3o
ool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3o
ool20?l00140oooo000B0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool07@3oool0
1@3o0000oooo0?ooo`3oool0o`0003H0oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?oo
o`0L0?ooo`80o`001@3oool0103o0000oooo0?ooo`3o00020?ooo`030?l0003oool0oooo0080oooo
00<0o`000?ooo`3o00001P3oool00`3o0000oooo0?ooo`0U0?ooo`80o`001P3oool20?l000H0oooo
00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l0
00@0oooo00@0o`000?ooo`3oool0o`00103oool00`3o0000oooo0?l000060?ooo`80o`001P3oool2
0?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool2
0?l000H0oooo0P3o000A0?ooo`004P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
01d0oooo00D0o`000?ooo`3oool0oooo0?l0000f0?ooo`030?l0003oool0oooo00<0oooo00<0o`00
0?ooo`3oool0703oool20?l000D0oooo00@0o`000?ooo`3oool0o`000P3oool00`3o0000oooo0?oo
o`020?ooo`030?l0003oool0o`0000H0oooo00<0o`000?ooo`3oool09@3oool20?l000H0oooo00<0
o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0
oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0
oooo0?l000D0oooo00<0o`000?ooo`3o00001@3oool20?l000H0oooo0P3o00060?ooo`80o`001P3o
ool20?l000H0oooo0P3o00060?ooo`80o`001P3oool00`3o0000oooo0?l000050?ooo`80o`004@3o
ool00180oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0M0?ooo`050?l0003oool0
oooo0?ooo`3o0000=P3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0oooo01`0oooo0P3o
00050?ooo`040?l0003oool0oooo0?l00080oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo
0?l000060?ooo`030?l0003oool0oooo01d0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo
00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l0
00@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003o
ool0oooo0?l000D0oooo00<0o`000?ooo`3o00001@3oool20?l000H0oooo0P3o00060?ooo`80o`00
1P3oool20?l000H0oooo00<0o`000?ooo`3o00001@3oool20?l000H0oooo00<0o`000?ooo`3o0000
1@3oool20?l00140oooo000C0?ooo`030000003oool0oooo00@0oooo0`00000M0?ooo`050?l0003o
ool0oooo0?ooo`3o0000=P3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0oooo01d0oooo
00<0o`000?ooo`3oool00`3oool0103o0000oooo0?ooo`3o00030?ooo`070?l0003oool0oooo0?oo
o`3o0000oooo0?l000060?ooo`030?l0003oool0oooo01d0oooo00<0o`000?ooo`3oool01@3oool2
0?l000H0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003o
ool0oooo0?l000@0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`040?l0003oool0oooo0?l0
00@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00050?ooo`80o`001P3o
ool20?l000H0oooo00<0o`000?ooo`3o00001@3oool20?l000H0oooo00<0o`000?ooo`3o00001@3o
ool20?l000H0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l0000@0?ooo`004`3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo0280oooo00<0o`000?ooo`3oool0<P3oool0
0`3o0000oooo0?l000040?ooo`030?l0003oool0oooo02<0oooo00<0o`000?ooo`3o0000103oool0
1@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool00`3oool20?l001h0oooo00<0
o`000?ooo`3oool01@3oool20?l000H0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?oo
o`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00D0o`000?ooo`3oool0oooo0?l000030?oo
o`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?oo
o`3o00050?ooo`80o`001P3oool20?l000H0oooo00<0o`000?ooo`3o00001@3oool20?l000H0oooo
00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo
00<0o`000?ooo`3o0000403oool001@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0R0?ooo`030?l0003oool0oooo0380oooo00<0o`000?ooo`3o0000103oool00`3o0000oooo0?oo
o`0S0?ooo`030?l0003oool0o`0000@0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0
003oool0oooo00<0oooo0P3o00060?ooo`030?l0003oool0oooo01D0oooo00<0o`000?ooo`3oool0
1@3oool20?l000H0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`05
0?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00@0o`00
0?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000D0
oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0
oooo0P3o00060?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000
oooo0?l000050?ooo`030?l0003oool0o`000100oooo000?0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0R0?ooo`030?l0003oool0oooo0380oooo
00<0o`000?ooo`3o0000103oool00`3o0000oooo0?ooo`0S0?ooo`030?l0003oool0o`0000@0oooo
00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo00<0oooo0P3o00060?ooo`03
0?l0003oool0oooo00d0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo0P3o00060?ooo`03
0?l0003oool0o`0000@0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?ooo`3oool0
o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`040?l0003oool0oooo0?l000@0oooo
00D0o`000?ooo`3oool0oooo0?l000030?ooo`040?l0003oool0oooo0?l000D0oooo00<0o`000?oo
o`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?oo
o`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?oo
o`3o00001@3oool00`3o0000oooo0?l0000@0?ooo`003`3oool6000000D0oooo00<000000?ooo`3o
ool08P3oool00`3o0000oooo0?ooo`0b0?ooo`040?l0003oool0oooo0?l000<0oooo00<0o`000?oo
o`3oool08`3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool0
0`3o0000oooo0?ooo`030?ooo`80o`001P3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0
oooo00D0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo0P3o00060?ooo`030?l0003oool0
o`0000@0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo
00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o
0000oooo0?ooo`3oool0o`0000<0oooo00@0o`000?ooo`3oool0o`001@3oool00`3o0000oooo0?l0
00050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l0
00050?ooo`030?l0003oool0o`0000@0oooo00@0o`000?ooo`3oool0o`001@3oool00`3o0000oooo
0?l000050?ooo`030?l0003oool0o`000100oooo000J0?ooo`030000003oool0oooo0280oooo00<0
o`000?ooo`3oool0<P3oool0103o0000oooo0?ooo`3o00030?ooo`030?l0003oool0oooo02<0oooo
00<0o`000?ooo`3o0000103oool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3o
ool00P3oool00`3o0000oooo0?l000060?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3o
ool01@3oool00`3o0000oooo0?ooo`050?ooo`80o`001@3oool0103o0000oooo0?ooo`3o00040?oo
o`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?oo
o`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?oo
o`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00@0o`000?ooo`3oool0o`001@3o
ool00`3o0000oooo0?l000040?ooo`040?l0003oool0oooo0?l000D0oooo00<0o`000?ooo`3o0000
1@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000@0oooo00@0o`000?ooo`3oool0
o`001@3oool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000l0oooo000J0?ooo`03
0000003oool0oooo0280oooo00<0o`000?ooo`3oool0<P3oool0103o0000oooo0?ooo`3o00030?oo
o`030?l0003oool0oooo02@0oooo0P3o00040?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool0
0`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0000H0oooo00<0o`000?ooo`3oool01@3oool0
0`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo0P3o00050?ooo`040?l0003oool0
oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?ooo`3oool0o`0000<0
oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool0
0`3o0000oooo0?ooo`020?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?oo
o`3oool0o`000080oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?l000040?ooo`040?l0
003oool0oooo0?l000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`03
0?l0003oool0o`0000@0oooo00D0o`000?ooo`3oool0oooo0?l000040?ooo`030?l0003oool0o`00
00D0oooo00@0o`000?ooo`3oool0o`003`3oool001X0oooo0P00000S0?ooo`030?l0003oool0oooo
0380oooo00@0o`000?ooo`3oool0o`000`3oool00`3o0000oooo0?ooo`0T0?ooo`80o`00103oool0
1@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l0
00060?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?oo
o`050?ooo`80o`001@3oool0103o0000oooo0?ooo`3o00040?ooo`050?l0003oool0oooo0?ooo`3o
00000`3oool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool00P3oool01@3o
0000oooo0?ooo`3oool0o`0000<0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?oo
o`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo00<0
oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o00050?ooo`030?l0003oool0o`00
00@0oooo00@0o`000?ooo`3oool0o`001@3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0
oooo0?ooo`3o0000103oool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000l0oooo
000J0?ooo`030000003oool0oooo0280oooo00<0o`000?ooo`3oool0<P3oool0103o0000oooo0?oo
o`3o00030?ooo`030?l0003oool0oooo02@0oooo0P3o00040?ooo`040?l0003oool0oooo0?l000<0
oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l000060?ooo`030?l0003oool0oooo00D0
oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`80o`001@3oool0103o0000
oooo0?ooo`3o00040?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3o
ool0o`000080oooo00<0o`000?ooo`3oool00P3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo
00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?oo
o`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?oo
o`3oool0o`000080oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?l000040?ooo`050?l0
003oool0oooo0?ooo`3o0000103oool00`3o0000oooo0?l000040?ooo`050?l0003oool0oooo0?oo
o`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000@0oooo00@0o`000?ooo`3oool0o`003`3o
ool001X0oooo00<000000?ooo`3oool08P3oool00`3o0000oooo0?ooo`0b0?ooo`050?l0003oool0
oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`0T0?ooo`80o`00103oool0103o0000oooo0?oo
o`3o00030?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o00001P3oool20?l000H0oooo
0P3o00060?ooo`80o`001P3oool20?l000D0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000
oooo0?ooo`3oool0o`0000<0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l000040?oo
o`050?l0003oool0oooo0?ooo`3o0000103oool00`3o0000oooo0?l000050?ooo`040?l0003oool0
oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o
00000`3oool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool00`3oool00`3o
0000oooo0?l000040?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool0103o0000oooo0?ooo`3o
00040?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000@0
oooo00@0o`000?ooo`3oool0o`003`3oool001X0oooo00<000000?ooo`3oool08P3oool00`3o0000
oooo0?ooo`0b0?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o00009P3oool20?l000D0
oooo00<0o`000?ooo`3o00000`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0000H0
oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00050?ooo`040?l0003oool0oooo0?l0
00@0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`030?l0003oool0oooo0080oooo00<0o`00
0?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00040?ooo`030?l0003oool0o`0000D0oooo00@0
o`000?ooo`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3o
ool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`02
0?ooo`040?l0003oool0oooo0?l000@0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0
003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?oo
o`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00003`3oool001X0oooo00<00000
0?ooo`3oool08P3oool00`3o0000oooo0?ooo`0b0?ooo`030?l0003oool0oooo0080oooo0`3o000W
0?ooo`030?l0003oool0oooo00<0oooo00<0o`000?ooo`3o00000`3oool00`3o0000oooo0?ooo`02
0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001P3oool20?l000D0oooo00<0o`00
0?ooo`3o00001P3oool20?l000D0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?oo
o`3oool0o`0000<0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l000050?ooo`030?l0
003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00030?ooo`05
0?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`00
0?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo0080oooo00D0o`000?ooo`3oool0oooo
0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`00
00<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3o
ool01@3o0000oooo0?ooo`3oool0o`0000l0oooo000J0?ooo`030000003oool0oooo02<0oooo00<0
o`000?ooo`3oool0<@3oool00`3o0000oooo0?ooo`0b0?ooo`030?l0003oool0o`0000<0oooo00<0
o`000?ooo`3oool00P3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000H0oooo0P3o
00050?ooo`030?l0003oool0o`0000H0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?oo
o`3o00040?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool00`3o0000oooo0?ooo`020?ooo`03
0?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001P3oool20?l000D0oooo00<0o`000?ooo`3o
0000103oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l00003
0?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo
00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o
0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003o
ool0oooo0?ooo`3o00000`3oool00`3o0000oooo0?ooo`0=0?ooo`006P3oool00`000000oooo0?oo
o`0S0?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3oool0;03oool00`3o0000oooo0?oo
o`0b0?ooo`030?l0003oool0o`0000@0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0
003oool0oooo0080oooo00<0o`000?ooo`3o00001P3oool20?l000D0oooo00<0o`000?ooo`3o0000
1P3oool00`3o0000oooo0?l000040?ooo`040?l0003oool0oooo0?l000@0oooo00D0o`000?ooo`3o
ool0oooo0?l000030?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o00001P3oool20?l0
00H0oooo0P3o00050?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`00103oool0
103o0000oooo0?ooo`3o00030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo
0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo
0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo
0?l000040?ooo`040?l0003oool0oooo0?l000<0oooo00<0o`000?ooo`3oool03@3oool001X0oooo
00<000000?ooo`3oool08`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0oooo02`0oooo
00<0o`000?ooo`3oool0<P3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0oooo0?ooo`3o
00000P3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0000H0oooo0P3o00050?ooo`04
0?l0003oool0oooo0?l000D0oooo00<0o`000?ooo`3o0000103oool01@3o0000oooo0?ooo`3oool0
o`000080oooo00<0o`000?ooo`3oool00`3oool0103o0000oooo0?ooo`3o00040?ooo`030?l0003o
ool0o`0000H0oooo0P3o00060?ooo`80o`001@3oool00`3o0000oooo0?l000050?ooo`040?l0003o
ool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`00
00<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3o
ool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`05
0?l0003oool0oooo0?ooo`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`030?l0003oool0
oooo00`0oooo000J0?ooo`800000903oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0oooo
02`0oooo00<0o`000?ooo`3oool0<P3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0oooo
0?ooo`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0000H0oooo0P3o0005
0?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo
0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?l000060?ooo`80
o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001@3oool0103o0000oooo0?ooo`3o00040?oo
o`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003o
ool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3o
ool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3o
ool0o`0000@0oooo00@0o`000?ooo`3oool0o`00103oool00`3o0000oooo0?ooo`0<0?ooo`006P3o
ool00`000000oooo0?ooo`0S0?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3oool0;03o
ool00`3o0000oooo0?ooo`0b0?ooo`030?l0003oool0o`0000@0oooo00D0o`000?ooo`3oool0oooo
0?l000020?ooo`030?l0003oool0oooo0080oooo00@0o`000?ooo`3oool0o`001@3oool20?l000D0
oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`050?l0003oool0
oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0o`0000H0oooo0P3o
00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`030?l0003oool0o`0000@0oooo00@0
o`000?ooo`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3o
ool0oooo0?l000040?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l0
00030?ooo`050?l0003oool0oooo0?ooo`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`04
0?l0003oool0oooo0?l000@0oooo00<0o`000?ooo`3oool0303oool001X0oooo00<000000?ooo`3o
ool08`3oool0103o0000oooo0?ooo`3oool20?l002h0oooo00<0o`000?ooo`3oool0<P3oool00`3o
0000oooo0?l000040?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`02
0?ooo`040?l0003oool0oooo0?l000D0oooo0P3o00050?ooo`040?l0003oool0oooo0?l000@0oooo
00D0o`000?ooo`3oool0oooo0?l000030?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o
00001P3oool20?l000H0oooo0P3o00070?ooo`030?l0003oool0oooo00@0oooo0P3o00060?ooo`80
o`001P3oool20?l000D0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00040?oo
o`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?oo
o`3o00030?ooo`050?l0003oool0oooo0?ooo`3o0000103oool00`3o0000oooo0?l000050?ooo`04
0?l0003oool0oooo0?l000D0oooo00<0o`000?ooo`3oool02`3oool001X0oooo00<000000?ooo`3o
ool08`3oool0103o0000oooo0?ooo`3oool20?l002h0oooo00<0o`000?ooo`3oool0<`3oool20?l0
00@0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo0080oooo00@0o`00
0?ooo`3oool0o`001@3oool20?l000D0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo
0?ooo`3oool0o`0000<0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l000060?ooo`80
o`001`3oool00`3o0000oooo0?ooo`0=0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3o
ool0103oool20?l000D0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00040?oo
o`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?oo
o`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00<0o`000?ooo`3o00001@3oool0103o0000
oooo0?ooo`3o00050?ooo`030?l0003oool0oooo00/0oooo000J0?ooo`030000003oool0oooo02<0
oooo00@0o`000?ooo`3oool0oooo0P3o000^0?ooo`030?l0003oool0oooo03<0oooo0P3o00050?oo
o`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003o
ool0oooo0080oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00040?ooo`050?l0
003oool0oooo0?ooo`3o00000`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0000H0
oooo0P3o000O0?ooo`030?l0003oool0oooo00@0oooo0P3o00050?ooo`030?l0003oool0o`0000D0
oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0
oooo0?l000@0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00050?ooo`80o`00
1P3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0oooo00/0oooo000J0?ooo`030000003o
ool0oooo02<0oooo00@0o`000?ooo`3oool0oooo0P3o000^0?ooo`030?l0003oool0oooo03<0oooo
0P3o00050?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l000020?oo
o`030?l0003oool0oooo0080oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o
00040?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool00`3o0000oooo0?ooo`020?ooo`030?l0
003oool0o`0000H0oooo0P3o000O0?ooo`030?l0003oool0oooo00@0oooo0P3o00060?ooo`80o`00
1@3oool0103o0000oooo0?ooo`3o00040?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3o
ool0o`00103oool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000D0oooo0P3o0006
0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3oool02`3oool001X0oooo00<000000?oo
o`3oool08`3oool0103o0000oooo0?ooo`3oool20?l002L0oooo00<0o`000?ooo`3oool0103oool0
0`3o0000oooo0?ooo`0c0?ooo`80o`001@3oool0103o0000oooo0?ooo`3o00030?ooo`050?l0003o
ool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`040?l0003oool0oooo0?l000@0
oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00<0o`00
0?ooo`3oool00P3oool00`3o0000oooo0?l000070?ooo`030?l0003oool0oooo02D0oooo00<0o`00
0?ooo`3oool0103oool20?l000H0oooo00<0o`000?ooo`3o0000103oool00`3o0000oooo0?l00005
0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001P3oool00`3o0000oooo0?l00005
0?ooo`80o`001P3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0oooo00/0oooo000J0?oo
o`800000903oool0103o0000oooo0?ooo`3oool20?l002L0oooo00<0o`000?ooo`3oool00`3oool0
0`3o0000oooo0?ooo`0d0?ooo`80o`001@3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0
oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`040?l0003oool0oooo0?l000@0oooo
00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00<0o`000?oo
o`3oool00P3oool00`3o0000oooo0?l0000_0?ooo`030?l0003oool0oooo00@0oooo0P3o00060?oo
o`80o`001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?oo
o`3o00001P3oool20?l000H0oooo0P3o00070?ooo`030?l0003oool0oooo00@0oooo00<0o`000?oo
o`3oool02`3oool001X0oooo00<000000?ooo`3oool08`3oool0103o0000oooo0?ooo`3oool20?l0
02L0oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`0e0?ooo`030?l0003oool0oooo
00<0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o00030?ooo`030?l0003oool0
oooo0080oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`050?l0
003oool0oooo0?ooo`3o00000`3oool00`3o0000oooo0?ooo`030?ooo`80o`00;`3oool00`3o0000
oooo0?ooo`050?ooo`030?l0003oool0oooo00@0oooo0P3o00050?ooo`030?l0003oool0o`0000H0
oooo0P3o00050?ooo`030?l0003oool0o`0000H0oooo0P3o00060?ooo`80o`001`3oool00`3o0000
oooo0?ooo`050?ooo`030?l0003oool0oooo00X0oooo000J0?ooo`030000003oool0oooo02<0oooo
00@0o`000?ooo`3oool0oooo0P3o000W0?ooo`030?l0003oool0oooo00<0oooo00<0o`000?ooo`3o
ool0>`3oool00`3o0000oooo0?l000040?ooo`040?l0003oool0oooo0?l000<0oooo00<0o`000?oo
o`3oool00P3oool0103o0000oooo0?ooo`3o00040?ooo`050?l0003oool0oooo0?ooo`3o00000`3o
ool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0000H0oooo0P3o000g0?ooo`030?l0003o
ool0oooo00@0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001`3o
ool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool02P3o
ool001X0oooo00<000000?ooo`3oool08`3oool0103o0000oooo0?ooo`3oool20?l002L0oooo0P3o
00040?ooo`030?l0003oool0oooo03/0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?oo
o`3o00030?ooo`030?l0003oool0oooo0080oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`05
0?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0o`00
00L0oooo00<0o`000?ooo`3oool0=@3oool00`3o0000oooo0?ooo`040?ooo`80o`001P3oool20?l0
00H0oooo0P3o00060?ooo`80o`001P3oool20?l000L0oooo00<0o`000?ooo`3oool01@3oool00`3o
0000oooo0?ooo`0B0?ooo`006P3oool00`000000oooo0?ooo`0S0?ooo`040?l0003oool0oooo0?oo
o`80o`009P3oool00`3o0000oooo0?l000040?ooo`030?l0003oool0oooo03/0oooo00<0o`000?oo
o`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`050?l0003oool0oooo0?ooo`3o00000`3o
ool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00@0o`000?ooo`3oool0o`001@3oool00`3o0000
oooo0?l000170?ooo`030?l0003oool0oooo00@0oooo0P3o00060?ooo`80o`001P3oool20?l000L0
oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`0J0?ooo`006P3oool00`000000oooo
0?ooo`0S0?ooo`040?l0003oool0oooo0?ooo`80o`009P3oool00`3o0000oooo0?l000040?ooo`03
0?l0003oool0oooo03/0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o00040?oo
o`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00@0
o`000?ooo`3oool0o`001P3oool20?l004L0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo
0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo
0?ooo`0R0?ooo`006P3oool00`000000oooo0?ooo`0S0?ooo`060?l0003oool0oooo0?l0003oool0
o`009P3oool00`3o0000oooo0?l000040?ooo`030?l0003oool0oooo03`0oooo0P3o00040?ooo`04
0?l0003oool0oooo0?l000@0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0
oooo0?ooo`3o0000103oool00`3o0000oooo0?l000060?ooo`80o`00C`3oool00`3o0000oooo0?oo
o`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0:P3oool001X0oooo00<00000
0?ooo`3oool08`3oool01P3o0000oooo0?ooo`3o0000oooo0?l002H0oooo00<0o`000?ooo`3o0000
103oool00`3o0000oooo0?ooo`0l0?ooo`80o`001@3oool00`3o0000oooo0?l000040?ooo`040?l0
003oool0oooo0?l000@0oooo00D0o`000?ooo`3oool0oooo0?l000040?ooo`030?l0003oool0o`00
00H0oooo0P3o002<0?ooo`006P3oool2000002D0oooo00<0o`000?ooo`3o00000P3oool00`3o0000
oooo0?ooo`0S0?ooo`040?l0003oool0oooo0?l000<0oooo00<0o`000?ooo`3oool0?03oool20?l0
00D0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`050?l0003oool0
oooo0?ooo`3o0000103oool00`3o0000oooo0?l000070?ooo`030?l0003oool0oooo08X0oooo000J
0?ooo`030000003oool0oooo02@0oooo00<0o`000?ooo`3o00000P3oool00`3o0000oooo0?ooo`0S
0?ooo`040?l0003oool0oooo0?l000<0oooo00<0o`000?ooo`3oool0?03oool20?l000D0oooo00<0
o`000?ooo`3o00001@3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0oooo0?ooo`3o0000
103oool00`3o0000oooo0?l000070?ooo`030?l0003oool0oooo08X0oooo000J0?ooo`030000003o
ool0oooo02@0oooo00<0o`000?ooo`3o00000P3oool00`3o0000oooo0?ooo`0S0?ooo`050?l0003o
ool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`0l0?ooo`80o`001@3oool00`3o0000oooo
0?l000050?ooo`030?l0003oool0o`0000@0oooo00D0o`000?ooo`3oool0oooo0?l000040?ooo`03
0?l0003oool0o`0009@0oooo000J0?ooo`030000003oool0oooo02@0oooo00<0o`000?ooo`3o0000
0P3oool00`3o0000oooo0?ooo`0S0?ooo`040?l0003oool0oooo0?ooo`@0o`00?`3oool00`3o0000
oooo0?ooo`040?ooo`80o`001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000H0
oooo0P3o002D0?ooo`000P3oool00`000000oooo0?ooo`040?ooo`8000001@3oool4000000H0oooo
00<000000?ooo`3oool0903oool00`3o0000oooo0?l000020?ooo`030?l0003oool0oooo02<0oooo
00<0o`000?ooo`3oool00P3oool20?l004L0oooo0P3o00050?ooo`030?l0003oool0o`0000D0oooo
00<0o`000?ooo`3o00001P3oool20?l009@0oooo00020?ooo`030000003oool0oooo00@0oooo0P00
00040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0T0?ooo`030?l0003oool0o`000080oooo00<0o`000?ooo`3oool08`3oool00`3o0000oooo0?oo
o`1;0?ooo`80o`001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000H0oooo0P3o
002D0?ooo`000P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0903oool00`3o0000oooo0?l000020?ooo`030?l0003oool0oooo02<0oooo00<0o`00
0?ooo`3oool0B`3oool20?l000D0oooo00<0o`000?ooo`3o00001P3oool20?l000L0oooo00<0o`00
0?ooo`3oool0TP3oool000<0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`030?oo
o`<00000903oool00`3o0000oooo0?l000020?ooo`030?l0003oool0oooo02<0oooo00<0o`000?oo
o`3oool0B`3oool20?l000D0oooo00<0o`000?ooo`3o00001P3oool20?l009`0oooo00030?ooo`03
0000003oool0oooo00X0oooo100000060?ooo`030000003oool0oooo02@0oooo00<0o`000?ooo`3o
00000P3oool00`3o0000oooo0?ooo`0S0?ooo`030?l0003oool0oooo04/0oooo0P3o00060?ooo`80
o`001P3oool20?l009`0oooo00040?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`0U0?ooo`80o`000P3oool00`3o0000oooo0?ooo`0S0?ooo`030?l0
003oool0oooo04`0oooo00<0o`000?ooo`3oool0103oool20?l000H0oooo0P3o002L0?ooo`00103o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool09@3o
ool20?l000<0oooo00<0o`000?ooo`3oool08P3oool00`3o0000oooo0?ooo`1C0?ooo`80o`001`3o
ool00`3o0000oooo0?ooo`2J0?ooo`001@00000;0?ooo`D000001@3oool00`000000oooo0?ooo`0U
0?ooo`80o`000`3oool00`3o0000oooo0?ooo`0R0?ooo`030?l0003oool0oooo05@0oooo00<0o`00
0?ooo`3oool0XP3oool001X0oooo00<000000?ooo`3oool09@3oool20?l000<0oooo00<0o`000?oo
o`3oool08@3oool00`3o0000oooo0?ooo`3j0?ooo`006P3oool00`000000oooo0?ooo`0U0?ooo`80
o`000`3oool00`3o0000oooo0?ooo`0Q0?ooo`030?l0003oool0oooo0?X0oooo000J0?ooo`030000
003oool0oooo02D0oooo0P3o00030?ooo`030?l0003oool0oooo01/0oooo00<0o`000?ooo`3oool0
0`3oool00`3o0000oooo0?ooo`3j0?ooo`006P3oool2000002H0oooo0P3o00030?ooo`030?l0003o
ool0oooo01/0oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`3j0?ooo`006P3oool0
0`000000oooo0?ooo`0U0?ooo`80o`000`3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0
oooo01D0oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`3j0?ooo`006P3oool00`00
0000oooo0?ooo`0U0?ooo`80o`000`3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0oooo
01D0oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`3j0?ooo`006P3oool00`000000
oooo0?ooo`0U0?ooo`80o`000`3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0oooo01D0
oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`3j0?ooo`006P3oool00`000000oooo
0?ooo`0U0?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo
0?ooo`0E0?ooo`030?l0003oool0oooo00<0oooo00<0o`000?ooo`3oool0nP3oool001X0oooo00<0
00000?ooo`3oool09@3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0oooo0080oooo0P3o
000G0?ooo`80o`00103oool00`3o0000oooo0?ooo`3j0?ooo`006P3oool00`000000oooo0?ooo`0U
0?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3oool00P3oool20?l001L0oooo0P3o0004
0?ooo`030?l0003oool0oooo0?X0oooo000J0?ooo`800000:`3oool00`3o0000oooo0?ooo`020?oo
o`80o`005P3oool00`3o0000oooo0?l000040?ooo`030?l0003oool0oooo0?X0oooo000J0?ooo`03
0000003oool0oooo02X0oooo00<0o`000?ooo`3oool00P3oool20?l001H0oooo00@0o`000?ooo`3o
ool0o`000`3oool00`3o0000oooo0?ooo`3j0?ooo`006P3oool00`000000oooo0?ooo`0Z0?ooo`03
0?l0003oool0oooo0080oooo0P3o000F0?ooo`040?l0003oool0oooo0?l000<0oooo00<0o`000?oo
o`3oool0nP3oool001X0oooo00<000000?ooo`3oool0:P3oool00`3o0000oooo0?ooo`020?ooo`03
0?l0003oool0o`0001D0oooo00@0o`000?ooo`3oool0o`000`3oool00`3o0000oooo0?ooo`3j0?oo
o`006P3oool00`000000oooo0?ooo`0Z0?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o
00005@3oool0103o0000oooo0?ooo`3o00030?ooo`030?l0003oool0oooo0?X0oooo000J0?ooo`03
0000003oool0oooo02X0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l0000E0?ooo`04
0?l0003oool0oooo0?l000<0oooo00<0o`000?ooo`3oool0nP3oool001X0oooo00<000000?ooo`3o
ool0:`3oool01`3o0000oooo0?ooo`3oool0o`000?ooo`3o00005@3oool01@3o0000oooo0?ooo`3o
ool0o`000080oooo00<0o`000?ooo`3oool0nP3oool001X0oooo0P00000/0?ooo`070?l0003oool0
oooo0?ooo`3o0000oooo0?l0000>0?ooo`030?l0003oool0oooo00@0oooo00@0o`000?ooo`3oool0
oooo103o003l0?ooo`006P3oool00`000000oooo0?ooo`0[0?ooo`070?l0003oool0oooo0?ooo`3o
0000oooo0?l0000>0?ooo`030?l0003oool0oooo00@0oooo00<0o`000?ooo`3oool00P3oool20?l0
0?d0oooo000J0?ooo`030000003oool0oooo02/0oooo00L0o`000?ooo`3oool0oooo0?l0003oool0
o`0000H0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`040?ooo`030?l0003oool0
oooo0?l0oooo0P3oool001X0oooo00<000000?ooo`3oool0:`3oool01`3o0000oooo0?ooo`3oool0
o`000?ooo`3o00001P3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00@0oooo00<0
o`000?ooo`3oool0o`3oool20?ooo`006P3oool00`000000oooo0?ooo`0[0?ooo`070?l0003oool0
oooo0?ooo`3o0000oooo0?l000060?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0
103oool00`3o0000oooo0?ooo`3o0?ooo`80oooo000J0?ooo`030000003oool0oooo02/0oooo00L0
o`000?ooo`3oool0oooo0?l0003oool0o`0000H0oooo00<0o`000?ooo`3oool0103oool00`3o0000
oooo0?l000050?ooo`030?l0003oool0oooo0?l0oooo0P3oool001X0oooo00<000000?ooo`3oool0
:`3oool01`3o0000oooo0?ooo`3oool0o`000?ooo`3o00001P3oool00`3o0000oooo0?ooo`040?oo
o`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3oool0o`3oool20?ooo`006P3oool00`000000
oooo0?ooo`0/0?ooo`060?l0003oool0oooo0?l0003oool0o`001P3oool00`3o0000oooo0?ooo`04
0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3oool0o`3oool20?ooo`006P3oool20000
02d0oooo00H0o`000?ooo`3oool0o`000?ooo`3o00050?ooo`80o`001P3oool00`3o0000oooo0?l0
00050?ooo`030?l0003oool0oooo0?l0oooo0P3oool001X0oooo00<000000?ooo`3oool0;03oool0
0`3o0000oooo0?l000020?ooo`030?l0003oool0oooo00<0oooo0P3o00060?ooo`030?l0003oool0
o`0000D0oooo00<0o`000?ooo`3oool0o`3oool20?ooo`006P3oool00`000000oooo0?ooo`0/0?oo
o`030?l0003oool0o`000080oooo00<0o`000?ooo`3oool00`3oool20?l000H0oooo00<0o`000?oo
o`3o00001@3oool00`3o0000oooo0?ooo`3o0?ooo`80oooo000J0?ooo`030000003oool0oooo02`0
oooo00<0o`000?ooo`3o00000P3oool00`3o0000oooo0?ooo`030?ooo`80o`001P3oool00`3o0000
oooo0?l000050?ooo`030?l0003oool0oooo0?l0oooo0P3oool00100oooo100000060?ooo`030000
003oool0oooo02`0oooo00<0o`000?ooo`3o00000P3oool00`3o0000oooo0?ooo`030?ooo`80o`00
1P3oool00`3o0000oooo0?l000040?ooo`030?l0003oool0oooo0?l0oooo0`3oool000l0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02`0oooo00<0
o`000?ooo`3o00000`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0000D0oooo00<0
o`000?ooo`3o0000103oool00`3o0000oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0/0?ooo`030?l0003oool0
o`0000<0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0
o`0000@0oooo00<0o`000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00<0oooo0`00000/0?ooo`030?l0003oool0o`0000<0oooo00<0o`000?oo
o`3oool00P3oool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000<0oooo00<0o`00
0?ooo`3oool0o`3oool30?ooo`00403oool4000000H0oooo00<000000?ooo`3oool0;03oool00`3o
0000oooo0?l000030?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o00001@3oool0103o
0000oooo0?ooo`3o00030?ooo`030?l0003oool0oooo0?l0oooo0`3oool000l0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02`0oooo00<0o`000?oo
o`3o00000`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0000D0oooo00@0o`000?oo
o`3oool0o`000`3oool00`3o0000oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0]0?ooo`80o`000`3oool00`3o
0000oooo0?ooo`020?ooo`030?l0003oool0o`0000D0oooo00D0o`000?ooo`3oool0oooo0?l00002
0?ooo`030?l0003oool0oooo0?l0oooo0`3oool00100oooo100000060?ooo`030000003oool0oooo
02d0oooo0P3o00030?ooo`030?l0003oool0oooo0080oooo00@0o`000?ooo`3oool0o`00103oool0
1@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool0o`3oool30?ooo`006P3oool0
0`000000oooo0?ooo`0]0?ooo`030?l0003oool0oooo00<0oooo00@0o`000?ooo`3oool0o`000`3o
ool00`3o0000oooo0?ooo`020?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo
0?ooo`3o0?ooo`<0oooo000J0?ooo`030000003oool0oooo02d0oooo00<0o`000?ooo`3oool00`3o
ool0103o0000oooo0?ooo`3o00030?ooo`030?l0003oool0oooo0080oooo00D0o`000?ooo`3oool0
oooo0?l000020?ooo`030?l0003oool0oooo0?l0oooo0`3oool001X0oooo00<000000?ooo`3oool0
;@3oool00`3o0000oooo0?ooo`030?ooo`040?l0003oool0oooo0?l000<0oooo00<0o`000?ooo`3o
ool00P3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`000?l0oooo1@3oool001X0oooo
0P00000^0?ooo`030?l0003oool0oooo00<0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000
oooo0?ooo`3oool0o`0000D0oooo0P3o003o0?ooo`H0oooo000J0?ooo`030000003oool0oooo02d0
oooo00<0o`000?ooo`3oool00`3oool0103o0000oooo0?ooo`3o00030?ooo`050?l0003oool0oooo
0?ooo`3o00001@3oool20?l00?l0oooo1P3oool001X0oooo00<000000?ooo`3oool0;@3oool00`3o
0000oooo0?ooo`030?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l0
00060?ooo`030?l0003oool0oooo0?l0oooo103oool001X0oooo00<000000?ooo`3oool0;@3oool0
0`3o0000oooo0?ooo`030?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo
0?l0003o0?ooo`d0oooo000J0?ooo`030000003oool0oooo02d0oooo00<0o`000?ooo`3oool00`3o
ool0103o0000oooo0?ooo`3o00030?ooo`050?l0003oool0oooo0?ooo`3o0000o`3oool=0?ooo`00
6P3oool00`000000oooo0?ooo`0c0?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3o
ool0oooo0?l0003o0?ooo`d0oooo000J0?ooo`030000003oool0oooo03<0oooo00@0o`000?ooo`3o
ool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`000?l0oooo3@3oool001X0oooo0P00000d0?oo
o`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00o`3oool=0?ooo`006P3oool0
0`000000oooo0?ooo`0c0?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00
o`3oool=0?ooo`006P3oool00`000000oooo0?ooo`0d0?ooo`030?l0003oool0o`0000@0oooo00@0
o`000?ooo`3oool0o`00o`3oool=0?ooo`006P3oool00`000000oooo0?ooo`0d0?ooo`030?l0003o
ool0o`0000@0oooo00@0o`000?ooo`3oool0o`00o`3oool=0?ooo`006P3oool00`000000oooo0?oo
o`0d0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o0000o`3oool=0?ooo`006P3oool0
0`000000oooo0?ooo`0d0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o0000o`3oool=
0?ooo`006P3oool00`000000oooo0?ooo`0d0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?oo
o`3o0000o`3oool=0?ooo`006P3oool00`000000oooo0?ooo`0e0?ooo`80o`001@3oool00`3o0000
oooo0?l0003o0?ooo`d0oooo000J0?ooo`800000=P3oool20?l000D0oooo00<0o`000?ooo`3o0000
o`3oool=0?ooo`006P3oool00`000000oooo0?ooo`0e0?ooo`80o`001@3oool00`3o0000oooo0?l0
003o0?ooo`d0oooo000J0?ooo`030000003oool0oooo03D0oooo0P3o00060?ooo`030?l0003oool0
oooo0?l0oooo303oool001X0oooo00<000000?ooo`3oool0=@3oool20?l000H0oooo00<0o`000?oo
o`3oool0o`3oool<0?ooo`006P3oool00`000000oooo0?ooo`0e0?ooo`030?l0003oool0oooo0?l0
oooo503oool001X0oooo00<000000?ooo`3oool0=@3oool00`3o0000oooo0?ooo`3o0?oooa@0oooo
000J0?ooo`030000003oool0oooo0?l0ooooC03oool001X0oooo0P00003o0?ooodd0oooo000J0?oo
o`030000003oool0oooo0?l0ooooC03oool001X0oooo00<000000?ooo`3oool0o`3ooom<0?ooo`00
6P3oool00`000000oooo0?ooo`3o0?oood`0oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?oo
ofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo0000\
\>"],
  ImageRangeCache->{{{0, 359}, {221.375, 0}} -> {-0.0332863, 5.55552, \
0.00155747, 0.0167673}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Comparison of Responses", "Subsection",
  CellTags->"Large-Signal Simulation"],

Cell["\<\
The three responses (averaged, averaged with 1st-order ripple \
correction, and switching model) now can be compared with each other. As can \
be seen, the responses of the averaged model (blue lines) agree very well \
with the one-cycle moving average of the switching model responses (red \
lines), and the averaged responses with 1st-order ripple correction (green \
lines) are almost identical to switching model responses.\
\>", "Text",
  CellTags->"Large-Signal Simulation"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[iLAve, \ iLCor, \ iLSwi, \ PlotRange\  -> \ All];\)\)], "Input",
  CellTags->"Large-Signal Simulation"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 2.38095 -0.18043 0.494038 [
[.2619 .30111 -9 -9 ]
[.2619 .30111 9 0 ]
[.5 .30111 -9 -9 ]
[.5 .30111 9 0 ]
[.7381 .30111 -9 -9 ]
[.7381 .30111 9 0 ]
[.97619 .30111 -9 -9 ]
[.97619 .30111 9 0 ]
[.01131 .01719 -18 -4.5 ]
[.01131 .01719 0 4.5 ]
[.01131 .11599 -18 -4.5 ]
[.01131 .11599 0 4.5 ]
[.01131 .2148 -18 -4.5 ]
[.01131 .2148 0 4.5 ]
[.01131 .41242 -18 -4.5 ]
[.01131 .41242 0 4.5 ]
[.01131 .51122 -18 -4.5 ]
[.01131 .51122 0 4.5 ]
[.01131 .61003 -18 -4.5 ]
[.01131 .61003 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .31361 m
.2619 .31986 L
s
[(0.1)] .2619 .30111 0 1 Mshowa
.5 .31361 m
.5 .31986 L
s
[(0.2)] .5 .30111 0 1 Mshowa
.7381 .31361 m
.7381 .31986 L
s
[(0.3)] .7381 .30111 0 1 Mshowa
.97619 .31361 m
.97619 .31986 L
s
[(0.4)] .97619 .30111 0 1 Mshowa
.125 Mabswid
.07143 .31361 m
.07143 .31736 L
s
.11905 .31361 m
.11905 .31736 L
s
.16667 .31361 m
.16667 .31736 L
s
.21429 .31361 m
.21429 .31736 L
s
.30952 .31361 m
.30952 .31736 L
s
.35714 .31361 m
.35714 .31736 L
s
.40476 .31361 m
.40476 .31736 L
s
.45238 .31361 m
.45238 .31736 L
s
.54762 .31361 m
.54762 .31736 L
s
.59524 .31361 m
.59524 .31736 L
s
.64286 .31361 m
.64286 .31736 L
s
.69048 .31361 m
.69048 .31736 L
s
.78571 .31361 m
.78571 .31736 L
s
.83333 .31361 m
.83333 .31736 L
s
.88095 .31361 m
.88095 .31736 L
s
.92857 .31361 m
.92857 .31736 L
s
.25 Mabswid
0 .31361 m
1 .31361 L
s
.02381 .01719 m
.03006 .01719 L
s
[(0.4)] .01131 .01719 1 0 Mshowa
.02381 .11599 m
.03006 .11599 L
s
[(0.6)] .01131 .11599 1 0 Mshowa
.02381 .2148 m
.03006 .2148 L
s
[(0.8)] .01131 .2148 1 0 Mshowa
.02381 .41242 m
.03006 .41242 L
s
[(1.2)] .01131 .41242 1 0 Mshowa
.02381 .51122 m
.03006 .51122 L
s
[(1.4)] .01131 .51122 1 0 Mshowa
.02381 .61003 m
.03006 .61003 L
s
[(1.6)] .01131 .61003 1 0 Mshowa
.125 Mabswid
.02381 .04189 m
.02756 .04189 L
s
.02381 .06659 m
.02756 .06659 L
s
.02381 .09129 m
.02756 .09129 L
s
.02381 .14069 m
.02756 .14069 L
s
.02381 .1654 m
.02756 .1654 L
s
.02381 .1901 m
.02756 .1901 L
s
.02381 .2395 m
.02756 .2395 L
s
.02381 .2642 m
.02756 .2642 L
s
.02381 .28891 m
.02756 .28891 L
s
.02381 .33831 m
.02756 .33831 L
s
.02381 .36301 m
.02756 .36301 L
s
.02381 .38771 m
.02756 .38771 L
s
.02381 .43712 m
.02756 .43712 L
s
.02381 .46182 m
.02756 .46182 L
s
.02381 .48652 m
.02756 .48652 L
s
.02381 .53593 m
.02756 .53593 L
s
.02381 .56063 m
.02756 .56063 L
s
.02381 .58533 m
.02756 .58533 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 1 r
.001 Mabswid
.02381 .06659 m
.04262 .18795 L
.06244 .31247 L
.08255 .41836 L
.09396 .46465 L
.09905 .48164 L
.10458 .49739 L
.11004 .51015 L
.11508 .51946 L
.11989 .52618 L
.12232 .52877 L
.125 .53101 L
.12623 .53183 L
.12741 .53248 L
.12848 .53297 L
.12962 .53339 L
.13085 .53371 L
.13199 .5339 L
.13269 .53396 L
.13335 .53398 L
.13459 .53392 L
.13568 .53376 L
.13684 .53349 L
.13782 .53317 L
.1389 .53274 L
.14132 .53144 L
.14357 .52983 L
.14858 .52494 L
.15402 .51769 L
.16392 .49995 L
.18248 .45461 L
.22326 .33562 L
.24352 .28523 L
.25344 .26558 L
.26253 .25098 L
.27138 .24012 L
.27583 .23591 L
.2807 .23228 L
.2856 .22963 L
.28835 .22858 L
.28964 .22819 L
.29084 .22789 L
.29195 .22766 L
.29317 .22747 L
.2944 .22733 L
.2957 .22725 L
.29684 .22723 L
.29806 .22727 L
.29922 .22735 L
.30028 .22748 L
Mistroke
.30157 .22768 L
.30279 .22793 L
.30506 .22853 L
.30751 .22937 L
.31015 .23051 L
.31492 .23313 L
.3193 .23613 L
.32939 .24494 L
.34004 .2566 L
.37807 .30714 L
.39925 .33336 L
.41854 .3518 L
.42814 .35849 L
.4332 .3613 L
.43856 .36373 L
.44324 .36539 L
.44573 .3661 L
.44841 .36673 L
.45086 .36719 L
.4531 .36752 L
.45525 .36774 L
.45632 .36782 L
.4575 .36788 L
.45815 .36791 L
.45883 .36793 L
.46005 .36794 L
.46136 .36792 L
.46211 .3679 L
.4628 .36787 L
.46407 .36779 L
.46522 .3677 L
.4678 .36742 L
.4701 .36708 L
.4726 .36661 L
.47715 .36553 L
.48243 .36392 L
.48804 .36182 L
.49812 .35721 L
.53583 .33471 L
.55678 .3223 L
.56596 .31756 L
.576 .31305 L
.5851 .30966 L
.59483 .30684 L
.60021 .30564 L
.6051 .30478 L
.60746 .30444 L
.61001 .30412 L
.61244 .30388 L
.61466 .3037 L
Mistroke
.61701 .30355 L
.61831 .30349 L
.61954 .30345 L
.62067 .30342 L
.62173 .3034 L
.62288 .30339 L
.62411 .3034 L
.62481 .3034 L
.62546 .30341 L
.62617 .30343 L
.62692 .30345 L
.62818 .30349 L
.62957 .30355 L
.63212 .3037 L
.63451 .30388 L
.63982 .30442 L
.64467 .30508 L
.65555 .30703 L
.67465 .31166 L
.69541 .31756 L
.71481 .32298 L
.72511 .32555 L
.73621 .32797 L
.74679 .32987 L
.75648 .33123 L
.76575 .33216 L
.77041 .3325 L
.7755 .33277 L
.77804 .33286 L
.77938 .3329 L
.78079 .33294 L
.78203 .33296 L
.78269 .33297 L
.78339 .33298 L
.78463 .33299 L
.78578 .333 L
.78698 .333 L
.78829 .33299 L
.78952 .33298 L
.79067 .33297 L
.7918 .33295 L
.79283 .33293 L
.79513 .33287 L
.79783 .33278 L
.80032 .33267 L
.80593 .33236 L
.81608 .33159 L
.83496 .32957 L
.85564 .32691 L
.87684 .32417 L
Mistroke
.89614 .32205 L
.90574 .32119 L
.91617 .32045 L
.92095 .32017 L
.92602 .31993 L
.93068 .31975 L
.93512 .31961 L
.93747 .31956 L
.94003 .31951 L
.94236 .31947 L
.94452 .31945 L
.94585 .31944 L
.94709 .31943 L
.94821 .31942 L
.94942 .31942 L
.95074 .31942 L
.95141 .31942 L
.95214 .31943 L
.95343 .31943 L
.95464 .31944 L
.95701 .31946 L
.95926 .31949 L
.96347 .31957 L
.96801 .31968 L
.97294 .31983 L
.97619 .31994 L
Mfstroke
1 0 1 r
.02381 .01472 m
.02608 .062 L
.02847 .11163 L
.02964 .13589 L
.03029 .14922 L
.0306 .15577 L
.03074 .15864 L
.03089 .16181 L
.03098 .16302 L
.03106 .16304 L
.03124 .16308 L
.03161 .16318 L
.03227 .16337 L
.03355 .1638 L
.03823 .16571 L
.0406 .16677 L
.0432 .16786 L
.0444 .16832 L
.04568 .16877 L
.0463 .16896 L
.04689 .16913 L
.04715 .1692 L
.04742 .16927 L
.04756 .1693 L
.04771 .17129 L
.04786 .17421 L
.04799 .17693 L
.05268 .27421 L
.05391 .29976 L
.05424 .30677 L
.05443 .31061 L
.0546 .31422 L
.05475 .31728 L
.0549 .31762 L
.05499 .31763 L
.05508 .31764 L
.05524 .31766 L
.05553 .31769 L
.05584 .31772 L
.05613 .31775 L
.0564 .31776 L
.05656 .31777 L
.05671 .31777 L
.05679 .31777 L
.05688 .31777 L
.05697 .31777 L
.05705 .31777 L
.05721 .31777 L
.05735 .31777 L
.05743 .31777 L
Mistroke
.05751 .31777 L
.05767 .31776 L
.05783 .31775 L
.05798 .31774 L
.05831 .31772 L
.05861 .31768 L
.05892 .31764 L
.05945 .31756 L
.06005 .31743 L
.06136 .31705 L
.06258 .31655 L
.06384 .31588 L
.06504 .3151 L
.06728 .31322 L
.06844 .31202 L
.06968 .31056 L
.07074 .30917 L
.07103 .30877 L
.07118 .30855 L
.07126 .30843 L
.07134 .30832 L
.07148 .30926 L
.07163 .31241 L
.0719 .31795 L
.07662 .41555 L
.07786 .44146 L
.0782 .44858 L
.07838 .45247 L
.07847 .45438 L
.07856 .45611 L
.0787 .45621 L
.07886 .45594 L
.0792 .45535 L
.08163 .45061 L
.08643 .43869 L
.09105 .42384 L
.09343 .41483 L
.09401 .41249 L
.09464 .40991 L
.09481 .40918 L
.09499 .40841 L
.09515 .40774 L
.09524 .40738 L
.09532 .40907 L
.09597 .42224 L
.09822 .46872 L
.10067 .51966 L
.10132 .53327 L
.10168 .54073 L
.10201 .54773 L
Mistroke
.10215 .55078 L
.10229 .55369 L
.10245 .55527 L
.1026 .5546 L
.10324 .55159 L
.10567 .53963 L
.11028 .51413 L
.11285 .4983 L
.11518 .48297 L
.11635 .47493 L
.11762 .46599 L
.1182 .46179 L
.11851 .45947 L
.11881 .45729 L
.11894 .45633 L
.11906 .4558 L
.1192 .4587 L
.11933 .46133 L
.11991 .47308 L
.1221 .51839 L
.12336 .54438 L
.12453 .56888 L
.12568 .59297 L
.12599 .59959 L
.12615 .60292 L
.12624 .60332 L
.12633 .60268 L
.12692 .59807 L
.12945 .57772 L
.13416 .53717 L
.13657 .51517 L
.13917 .49037 L
.14039 .47842 L
.14168 .46555 L
.14221 .46018 L
.14251 .45721 L
.14265 .45574 L
.14278 .45437 L
.14294 .45531 L
.14302 .457 L
.14311 .4588 L
.14341 .46494 L
.14399 .47697 L
.14641 .52688 L
.14768 .55327 L
.14902 .58122 L
.1496 .59341 L
.14977 .59692 L
.14993 .60024 L
.15007 .60109 L
Mistroke
.15022 .59947 L
.15091 .5923 L
.15154 .58556 L
.15386 .56059 L
.15879 .50547 L
.16124 .4771 L
.16353 .45 L
.16466 .43652 L
.16573 .4236 L
.16632 .41644 L
.16648 .41448 L
.16657 .4134 L
.16665 .41238 L
.16673 .41351 L
.16681 .41525 L
.16696 .41829 L
.16809 .44159 L
.17045 .49039 L
.1717 .51633 L
.17238 .53065 L
.17304 .54422 L
.17333 .55046 L
.17365 .55709 L
.17373 .55875 L
.17382 .56027 L
.17391 .5591 L
.174 .55801 L
.17432 .55404 L
.1755 .53949 L
.1778 .51074 L
.18276 .44766 L
.18523 .41575 L
.18753 .38585 L
.18868 .37095 L
.18976 .35684 L
.19004 .35305 L
.1902 .35098 L
.19035 .34903 L
.19043 .34802 L
.19051 .34814 L
.1906 .35004 L
.19069 .35178 L
.191 .35821 L
.19213 .38162 L
.19462 .43321 L
.19583 .45833 L
.19692 .4811 L
.19719 .48669 L
.19748 .49258 L
.19762 .49551 L
Mistroke
.19771 .4944 L
.19779 .49337 L
.19807 .48961 L
.19916 .47538 L
.20153 .44407 L
.20623 .38219 L
.20862 .35072 L
.21122 .31662 L
.21244 .30076 L
.2131 .29209 L
.21372 .28397 L
.21398 .28061 L
.21412 .27878 L
.21425 .27708 L
.21439 .27891 L
.21455 .28214 L
.21482 .2878 L
.21603 .31298 L
.2183 .35997 L
.21949 .38466 L
.22075 .41074 L
.22106 .41712 L
.22123 .42064 L
.22131 .42237 L
.22139 .42397 L
.22148 .42418 L
.22158 .4229 L
.22175 .42066 L
.22209 .41626 L
.22333 .40029 L
.22576 .36906 L
.23049 .30903 L
.23504 .25241 L
.23617 .23847 L
.23736 .22387 L
.23767 .22019 L
.23783 .21817 L
.23799 .21626 L
.23808 .21518 L
.23818 .21666 L
.23834 .22019 L
.23868 .22708 L
.23988 .25214 L
.24222 .30087 L
.24347 .32664 L
.24416 .34083 L
.24449 .34782 L
.24481 .3543 L
.24496 .35745 L
.2451 .36041 L
Mistroke
.24518 .36205 L
.24527 .3628 L
.24542 .36099 L
.24609 .35298 L
.24726 .339 L
.24955 .31188 L
.2545 .25473 L
.25696 .22694 L
.25926 .20142 L
.2604 .18897 L
.26096 .18288 L
.26147 .17727 L
.26176 .17416 L
.26191 .17269 L
.26199 .17442 L
.26207 .17604 L
.26271 .18953 L
.26384 .21308 L
.26607 .25938 L
.26734 .28552 L
.26795 .29823 L
.26851 .30977 L
.2688 .31572 L
.26888 .31735 L
.26896 .31907 L
.26911 .32014 L
.26925 .31865 L
.26938 .31726 L
.26968 .3141 L
.27093 .30089 L
.27347 .27437 L
.27816 .2269 L
.28054 .20342 L
.28314 .17833 L
.28434 .16685 L
.285 .16061 L
.28533 .15756 L
.28547 .15622 L
.28562 .15476 L
.28577 .15511 L
.28591 .15797 L
.28621 .16421 L
.28684 .17733 L
.28793 .20016 L
.29018 .24693 L
.29137 .27149 L
.29202 .28492 L
.29234 .29149 L
.29263 .29749 L
.29279 .30072 L
Mistroke
.29294 .3014 L
.29309 .29995 L
.29327 .29837 L
.29397 .29194 L
.2952 .2807 L
.29763 .25888 L
.30253 .21612 L
.30479 .19688 L
.30724 .17644 L
.3079 .17105 L
.30859 .16539 L
.30917 .16057 L
.30934 .15926 L
.30943 .15854 L
.30951 .15785 L
.30959 .15907 L
.30967 .16085 L
.30982 .16396 L
.31225 .21458 L
.31459 .26302 L
.31583 .2887 L
.31617 .29572 L
.31636 .29956 L
.31653 .30312 L
.31668 .30588 L
.31683 .30461 L
.31717 .30193 L
.31961 .28255 L
.3219 .26462 L
.3269 .22637 L
.32941 .20761 L
.33051 .19945 L
.33172 .19052 L
.33228 .18641 L
.33259 .18415 L
.33289 .182 L
.33316 .18001 L
.33331 .17889 L
.33345 .18126 L
.33397 .19207 L
.33636 .24164 L
.33863 .2887 L
.33925 .30158 L
.33991 .31518 L
.34018 .32078 L
.34033 .32386 L
.34047 .32678 L
.34055 .32645 L
.34063 .32582 L
.34079 .32473 L
Mistroke
.34108 .32258 L
.34367 .304 L
.3461 .28667 L
.35102 .25198 L
.35347 .23489 L
.35466 .22655 L
.35576 .21892 L
.35631 .21507 L
.35683 .21149 L
.35696 .21057 L
.35709 .20969 L
.35723 .21123 L
.35737 .21397 L
.35795 .22617 L
.36032 .27521 L
.36253 .32109 L
.36313 .33366 L
.36378 .34716 L
.36394 .35028 L
.3641 .35369 L
.36426 .35691 L
.3644 .35676 L
.36454 .35578 L
.36467 .35488 L
.36496 .35288 L
.36736 .33626 L
.3699 .31867 L
.37474 .28504 L
.37696 .26953 L
.37822 .26072 L
.3794 .25248 L
.38 .24828 L
.38056 .2443 L
.38072 .24323 L
.3808 .24264 L
.38088 .2421 L
.38096 .24173 L
.38105 .24365 L
.38122 .24703 L
.38181 .25934 L
.38435 .31207 L
.38659 .35848 L
.3872 .3712 L
.38754 .37818 L
.38785 .38477 L
.388 .38776 L
.38815 .3894 L
.38823 .38884 L
.38832 .38822 L
.38847 .38714 L
Mistroke
.38903 .38318 L
.39158 .36506 L
.394 .34771 L
.39887 .31236 L
.40112 .29586 L
.40231 .2871 L
.40357 .27774 L
.4042 .27303 L
.40437 .27173 L
.40456 .27034 L
.40464 .2697 L
.40473 .26902 L
.40481 .26986 L
.4049 .27161 L
.40613 .29709 L
.40855 .3474 L
.40977 .37266 L
.41092 .39653 L
.41155 .40974 L
.41173 .41336 L
.41182 .41534 L
.41191 .41698 L
.41207 .4158 L
.41224 .41449 L
.41345 .40539 L
.41813 .36953 L
.4231 .33057 L
.42559 .31077 L
.4268 .30106 L
.4279 .29222 L
.42817 .29004 L
.42832 .28885 L
.42845 .28774 L
.4286 .28738 L
.42868 .28901 L
.42876 .29075 L
.42905 .29667 L
.43013 .31909 L
.43251 .3684 L
.43374 .39394 L
.43439 .40742 L
.43508 .42173 L
.43537 .42774 L
.43552 .4309 L
.43568 .43422 L
.43576 .43459 L
.43586 .43383 L
.43602 .4325 L
.43634 .42987 L
.43751 .42029 L
Mistroke
.4424 .37952 L
.44712 .33946 L
.44952 .31871 L
.45079 .30771 L
.45148 .30168 L
.45182 .29872 L
.45197 .29743 L
.45213 .29602 L
.45228 .29467 L
.45242 .2947 L
.45256 .29752 L
.45271 .30052 L
.45333 .31335 L
.45464 .3406 L
.45696 .38862 L
.45808 .41205 L
.45868 .42437 L
.459 .4311 L
.4593 .43736 L
.45944 .4402 L
.45957 .44165 L
.45971 .44034 L
.45985 .43914 L
.46045 .43384 L
.4615 .42449 L
.46614 .38297 L
.47107 .3381 L
.47352 .31552 L
.47472 .30442 L
.47529 .29914 L
.47582 .29424 L
.47595 .29299 L
.47608 .29181 L
.47622 .29145 L
.47636 .29419 L
.47694 .30637 L
.47802 .32858 L
.48038 .37764 L
.48156 .40211 L
.4822 .41548 L
.4828 .42797 L
.48307 .43345 L
.4832 .43614 L
.48332 .43863 L
.48346 .43783 L
.4836 .43642 L
.48387 .43397 L
.48503 .42304 L
.48977 .37831 L
.49432 .33488 L
Mistroke
.49686 .31059 L
.49796 .29993 L
.49855 .29428 L
.49918 .28825 L
.49949 .28518 L
.49965 .28367 L
.49979 .28228 L
.49992 .28102 L
.50006 .28157 L
.5002 .28445 L
.50035 .28761 L
.50145 .31033 L
.50384 .35998 L
.50505 .38497 L
.50619 .40869 L
.50682 .42178 L
.50699 .42537 L
.50709 .42733 L
.50718 .42816 L
.50726 .42735 L
.50735 .42648 L
.50751 .42496 L
.50871 .4133 L
.51366 .36512 L
.51843 .31854 L
.52084 .29505 L
.52188 .28479 L
.52302 .27365 L
.52333 .27065 L
.52349 .26901 L
.52365 .26748 L
.52378 .26617 L
.52393 .2684 L
.52423 .27473 L
.52538 .29852 L
.5279 .3509 L
.52912 .37616 L
.52977 .38951 L
.53012 .39684 L
.53045 .40375 L
.53077 .41031 L
.53085 .41211 L
.53095 .41401 L
.53102 .41343 L
.53111 .41259 L
.53171 .40673 L
.53287 .39531 L
.53775 .3476 L
.54244 .30177 L
.54483 .27849 L
Mistroke
.54609 .26622 L
.54678 .2595 L
.54711 .25621 L
.54726 .25477 L
.54742 .25319 L
.54757 .25179 L
.5477 .25302 L
.54784 .25596 L
.54799 .2591 L
.54861 .27187 L
.54992 .29904 L
.55223 .34695 L
.55346 .37255 L
.55411 .38603 L
.55447 .39341 L
.55456 .39525 L
.55464 .39701 L
.5548 .39913 L
.55494 .39778 L
.55509 .39632 L
.5554 .39329 L
.55606 .38687 L
.55723 .37557 L
.56214 .3282 L
.56686 .28287 L
.56927 .25983 L
.56993 .25354 L
.57063 .2469 L
.57092 .24416 L
.57123 .24123 L
.57132 .24037 L
.5714 .23955 L
.57148 .24038 L
.57156 .24212 L
.57188 .24867 L
.57418 .29643 L
.57549 .32359 L
.57672 .34905 L
.57787 .37288 L
.57818 .37941 L
.57835 .38297 L
.57844 .3847 L
.57851 .38628 L
.57865 .38679 L
.57879 .3854 L
.5791 .38248 L
.58164 .35852 L
.58646 .31319 L
.59111 .26994 L
.59351 .24772 L
Mistroke
.59409 .24233 L
.59472 .23656 L
.5949 .23486 L
.59508 .23325 L
.59523 .2318 L
.59532 .23352 L
.5954 .23523 L
.59605 .24857 L
.59849 .29919 L
.59968 .32398 L
.60078 .34673 L
.60184 .36879 L
.6021 .37415 L
.60224 .3771 L
.60238 .37989 L
.60254 .37855 L
.60268 .37723 L
.60296 .37463 L
.60532 .35303 L
.60996 .3109 L
.6123 .2897 L
.61489 .26646 L
.61607 .25587 L
.61734 .24451 L
.61795 .23902 L
.61854 .23381 L
.61879 .23154 L
.61893 .2303 L
.61906 .22949 L
.61922 .23272 L
.61936 .23565 L
.61963 .24138 L
.62428 .33788 L
.62488 .35033 L
.62553 .36367 L
.62584 .37013 L
.62599 .37331 L
.62613 .37626 L
.62626 .37682 L
.6264 .37561 L
.62669 .37304 L
.6279 .3623 L
.62923 .35053 L
.63378 .3104 L
.63631 .28823 L
.63863 .26797 L
.6409 .24818 L
.64205 .23825 L
.64236 .23553 L
.64253 .23404 L
Mistroke
.64269 .23263 L
.64284 .23131 L
.64293 .23272 L
.64301 .23435 L
.6433 .24031 L
.64575 .29128 L
.64805 .33894 L
.64859 .35016 L
.64918 .36238 L
.64945 .36797 L
.64973 .37392 L
.64987 .37682 L
.65001 .37937 L
.65012 .37835 L
.65025 .37725 L
.65149 .36655 L
.65262 .3568 L
.65757 .31418 L
.66234 .27332 L
.66357 .26278 L
.66474 .25281 L
.66579 .24387 L
.66607 .24142 L
.66638 .23883 L
.6665 .23776 L
.66664 .23661 L
.66679 .23884 L
.66692 .24172 L
.66928 .29066 L
.67181 .34303 L
.67241 .35563 L
.67306 .36907 L
.67338 .3756 L
.67353 .37881 L
.67367 .38176 L
.6738 .38445 L
.67394 .38347 L
.67423 .38102 L
.67545 .37066 L
.67677 .35935 L
.68135 .32047 L
.6839 .29885 L
.68622 .27913 L
.68737 .26936 L
.68863 .25872 L
.6892 .25385 L
.68981 .24868 L
.69007 .24655 L
.6902 .24543 L
.69034 .24426 L
Mistroke
.69049 .24336 L
.69063 .2463 L
.69091 .25209 L
.6955 .34738 L
.69609 .35948 L
.69672 .37259 L
.69701 .37861 L
.69732 .38498 L
.69744 .3876 L
.69758 .39042 L
.69773 .39037 L
.69787 .38918 L
.69919 .37796 L
.70039 .36776 L
.70537 .32554 L
.70785 .30443 L
.71016 .28482 L
.71239 .26583 L
.71303 .26036 L
.71364 .2552 L
.7139 .25295 L
.71405 .25172 L
.71418 .25057 L
.71433 .25051 L
.7144 .25211 L
.71449 .25383 L
.71477 .25966 L
.71716 .30927 L
.71842 .33543 L
.71976 .36322 L
.72033 .37508 L
.72095 .38787 L
.72112 .39144 L
.7213 .39519 L
.72145 .39768 L
.72154 .39693 L
.72162 .39624 L
.72226 .39082 L
.72457 .37109 L
.72928 .33076 L
.73168 .31014 L
.73429 .28775 L
.73551 .27725 L
.7368 .26613 L
.73733 .26154 L
.73762 .25902 L
.7379 .25662 L
.73806 .25527 L
.73814 .25593 L
.73823 .25764 L
Mistroke
.73852 .26383 L
.73911 .27598 L
.74138 .32314 L
.74384 .37408 L
.7445 .38772 L
.74485 .39517 L
.74503 .3988 L
.74519 .4021 L
.74534 .4023 L
.74547 .4011 L
.74578 .39845 L
.74643 .39285 L
.74886 .37176 L
.75349 .33153 L
.75584 .31111 L
.75842 .28855 L
.7596 .27823 L
.76025 .27254 L
.76087 .26712 L
.76144 .2621 L
.7616 .26072 L
.76169 .25997 L
.76177 .25926 L
.76193 .25851 L
.76207 .26148 L
.76317 .28423 L
.76538 .33019 L
.76664 .35629 L
.76781 .38068 L
.76841 .39304 L
.7687 .39913 L
.76883 .4018 L
.76897 .40476 L
.76913 .40559 L
.76921 .40486 L
.76929 .40418 L
.76963 .4012 L
.77022 .39599 L
.77276 .37364 L
.77742 .33247 L
.77979 .3115 L
.78238 .28851 L
.78358 .27789 L
.78424 .27204 L
.78486 .26652 L
.78514 .26402 L
.78544 .26137 L
.78559 .25999 L
.78568 .2592 L
.78576 .25993 L
Mistroke
.78591 .26294 L
.78606 .26618 L
.78716 .28892 L
.78954 .33831 L
.7908 .3644 L
.79149 .37872 L
.79214 .39219 L
.79244 .39853 L
.79261 .40199 L
.79277 .40522 L
.79285 .40691 L
.79293 .40643 L
.79303 .40557 L
.79312 .40478 L
.79344 .40189 L
.79463 .39134 L
.79693 .37074 L
.80174 .32767 L
.80637 .28612 L
.8076 .27504 L
.80876 .26463 L
.80904 .26209 L
.80934 .25941 L
.8095 .258 L
.80958 .25901 L
.80967 .26071 L
.80997 .26691 L
.81129 .29448 L
.81356 .34144 L
.81475 .36616 L
.81601 .39231 L
.81632 .39871 L
.81649 .40224 L
.81657 .40397 L
.81665 .40558 L
.81674 .40531 L
.81684 .40442 L
.81701 .40285 L
.81735 .39979 L
.81859 .38864 L
.82102 .3667 L
.82575 .32393 L
.8303 .28276 L
.83143 .27248 L
.83263 .26165 L
.83293 .2589 L
.8331 .2574 L
.83325 .25597 L
.83334 .25541 L
.83344 .25741 L
Mistroke
.83361 .26092 L
.83394 .26777 L
.83514 .29273 L
.83749 .34142 L
.83873 .36725 L
.83942 .38149 L
.83976 .38851 L
.84007 .39503 L
.84022 .3982 L
.84037 .40118 L
.84045 .40283 L
.84053 .40294 L
.84068 .40157 L
.84135 .39551 L
.84252 .38489 L
.84482 .36405 L
.84976 .31914 L
.85223 .29677 L
.85453 .27588 L
.85566 .26557 L
.85622 .26049 L
.85674 .2558 L
.85689 .25445 L
.85703 .25318 L
.85717 .25274 L
.85725 .25434 L
.85733 .25607 L
.85798 .26947 L
.85911 .29292 L
.86148 .3421 L
.86273 .36811 L
.86342 .3824 L
.86376 .38942 L
.86391 .3925 L
.86407 .39589 L
.86423 .39913 L
.86437 .39954 L
.86453 .39814 L
.86469 .39663 L
.86537 .3905 L
.86655 .3798 L
.86885 .35889 L
.87383 .31371 L
.87632 .29113 L
.87863 .27022 L
.87978 .25978 L
.88034 .25467 L
.88059 .25243 L
.88072 .25124 L
.88086 .24997 L
Mistroke
.88101 .25042 L
.88115 .25335 L
.88146 .25974 L
.88211 .2732 L
.88324 .29665 L
.88564 .34634 L
.8869 .37253 L
.88723 .37943 L
.88759 .38687 L
.88776 .39046 L
.88793 .39388 L
.88808 .39695 L
.88816 .39679 L
.88824 .39608 L
.88943 .38531 L
.89074 .37344 L
.89305 .35254 L
.89777 .30994 L
.90017 .28822 L
.90144 .2768 L
.90278 .26473 L
.90398 .25394 L
.90431 .25098 L
.90449 .24935 L
.90458 .24855 L
.90466 .24781 L
.90482 .24801 L
.9049 .24975 L
.90499 .25161 L
.90529 .2579 L
.90761 .30594 L
.90998 .35518 L
.91102 .37674 L
.91131 .38265 L
.91161 .38897 L
.91176 .39203 L
.9119 .39493 L
.91202 .39402 L
.91216 .3928 L
.91329 .38258 L
.91449 .37183 L
.917 .34923 L
.92194 .30498 L
.92439 .283 L
.92559 .27226 L
.92669 .26244 L
.92723 .25761 L
.92782 .25234 L
.92808 .24994 L
.92823 .24861 L
Mistroke
.92837 .24737 L
.9285 .24621 L
.92864 .24704 L
.92877 .2497 L
.92889 .25213 L
.93125 .30123 L
.93372 .35239 L
.93434 .36522 L
.93492 .37736 L
.93545 .38826 L
.93559 .39125 L
.93574 .39352 L
.93588 .39236 L
.93602 .39107 L
.93823 .37131 L
.9406 .35016 L
.94545 .30692 L
.95012 .26539 L
.95073 .26 L
.95138 .25425 L
.95169 .25145 L
.95199 .24882 L
.95212 .24767 L
.95226 .24645 L
.95241 .24584 L
.95254 .24871 L
.95376 .27395 L
.95509 .30154 L
.95748 .35114 L
.95852 .37281 L
.95908 .38425 L
.95923 .38754 L
.95938 .39064 L
.95952 .39342 L
.95966 .39232 L
.96029 .38677 L
.96087 .38163 L
.96201 .37149 L
.96453 .34914 L
.96949 .30526 L
.97426 .26305 L
.97619 .24597 L
Mfstroke
1 0 0 r
.02381 .01472 m
.02567 .05327 L
.02769 .09532 L
.0296 .13481 L
.03009 .14504 L
.03056 .1547 L
.03076 .15891 L
.03087 .16122 L
.03092 .16234 L
.03097 .16295 L
.03103 .16299 L
.03109 .16305 L
.03121 .16316 L
.03143 .16335 L
.03237 .16412 L
.03338 .16487 L
.03529 .16604 L
.03712 .16688 L
.03807 .16721 L
.03908 .16747 L
.03957 .16756 L
.04011 .16764 L
.04035 .16767 L
.04061 .1677 L
.04086 .16772 L
.04108 .16773 L
.04119 .16774 L
.04131 .16774 L
.04137 .16774 L
.04143 .16775 L
.04148 .16775 L
.04153 .16775 L
.04159 .16775 L
.04165 .16775 L
.04171 .16775 L
.04177 .16775 L
.04183 .16775 L
.04189 .16775 L
.04192 .16775 L
.04195 .16775 L
.04202 .16775 L
.04214 .16774 L
.04224 .16774 L
.04248 .16773 L
.04268 .16771 L
.0429 .16769 L
.04336 .16764 L
.04385 .16757 L
.04473 .16739 L
.04574 .1671 L
Mistroke
.04621 .16694 L
.04671 .16675 L
.04697 .16665 L
.04721 .16655 L
.04743 .16645 L
.04749 .16642 L
.04755 .16639 L
.04761 .16637 L
.04767 .16734 L
.04855 .1857 L
.0505 .2261 L
.05237 .26498 L
.05333 .28479 L
.05383 .29523 L
.05436 .30631 L
.05459 .31105 L
.05471 .31354 L
.05477 .31459 L
.05481 .31462 L
.05484 .31465 L
.05498 .31477 L
.05511 .31489 L
.05536 .3151 L
.05628 .31576 L
.05679 .31605 L
.05727 .31627 L
.0577 .31643 L
.05794 .3165 L
.05816 .31656 L
.0584 .31661 L
.05862 .31665 L
.05874 .31666 L
.05886 .31668 L
.05893 .31668 L
.05899 .31669 L
.05906 .31669 L
.05912 .3167 L
.05918 .3167 L
.05924 .3167 L
.0593 .3167 L
.05935 .3167 L
.05938 .3167 L
.05942 .3167 L
.05948 .3167 L
.05951 .3167 L
.05955 .3167 L
.05961 .31669 L
.05974 .31668 L
.05986 .31667 L
.06013 .31664 L
Mistroke
.06036 .3166 L
.0606 .31654 L
.06103 .31641 L
.06155 .31621 L
.06203 .31598 L
.06294 .31541 L
.06394 .31461 L
.06589 .31249 L
.06778 .30977 L
.06874 .30814 L
.06978 .3062 L
.07027 .30522 L
.07078 .30414 L
.07099 .30368 L
.07122 .30318 L
.07129 .30303 L
.07135 .30289 L
.07141 .30276 L
.07147 .30362 L
.07153 .30484 L
.07159 .30614 L
.07171 .30846 L
.07352 .34615 L
.07546 .3863 L
.07732 .42493 L
.07782 .4353 L
.07828 .44494 L
.0784 .44744 L
.07852 .44981 L
.07858 .45092 L
.07862 .45089 L
.07865 .45087 L
.07877 .45078 L
.0793 .45032 L
.08028 .44924 L
.08119 .44799 L
.08302 .44476 L
.08489 .44047 L
.08687 .43486 L
.08879 .42846 L
.09063 .42142 L
.09258 .41302 L
.09349 .40882 L
.09447 .40408 L
.09473 .40277 L
.09501 .40138 L
.09512 .4008 L
.09518 .40048 L
.09524 .40031 L
.09528 .40101 L
Mistroke
.09531 .40176 L
.09538 .40309 L
.0955 .40567 L
.09647 .42582 L
.09844 .46659 L
.10033 .50585 L
.10128 .52556 L
.1017 .53418 L
.10215 .5436 L
.10227 .54609 L
.1023 .54674 L
.10233 .54745 L
.1024 .54836 L
.10246 .54814 L
.10252 .54794 L
.10263 .54756 L
.10308 .54593 L
.10409 .54207 L
.10598 .53384 L
.10781 .5248 L
.10975 .51402 L
.11162 .50256 L
.11361 .48927 L
.11553 .47545 L
.11737 .46126 L
.11832 .45362 L
.11855 .45173 L
.1188 .44968 L
.11887 .44911 L
.11895 .4485 L
.11901 .44798 L
.11904 .44769 L
.11908 .44824 L
.11933 .45357 L
.12121 .49253 L
.12321 .53393 L
.12421 .55468 L
.12513 .57382 L
.12559 .58344 L
.12582 .58815 L
.12603 .59245 L
.12608 .59357 L
.12614 .59478 L
.12619 .59584 L
.12624 .59552 L
.12648 .594 L
.12698 .59072 L
.12886 .5775 L
.13086 .56214 L
.13279 .54613 L
Mistroke
.13465 .52965 L
.13662 .51106 L
.13852 .49218 L
.14035 .47319 L
.14137 .46228 L
.1423 .45215 L
.14252 .44968 L
.14264 .44832 L
.14276 .44705 L
.14282 .44638 L
.14285 .44601 L
.14288 .44649 L
.14302 .44932 L
.14326 .45432 L
.14417 .47322 L
.14608 .5128 L
.14791 .55087 L
.14894 .57211 L
.14942 .58215 L
.14963 .58655 L
.14975 .58905 L
.14987 .59139 L
.14992 .59255 L
.14998 .59365 L
.15004 .59384 L
.15009 .59334 L
.15033 .59116 L
.15083 .58653 L
.15175 .5779 L
.15359 .5597 L
.15555 .53922 L
.15743 .5185 L
.15925 .49772 L
.16118 .47475 L
.16304 .4519 L
.16398 .44004 L
.16501 .42688 L
.16549 .42075 L
.166 .41415 L
.16625 .41094 L
.16648 .40789 L
.16658 .40656 L
.16663 .40589 L
.16669 .40591 L
.16675 .40718 L
.16681 .40837 L
.16692 .41063 L
.16878 .44933 L
.17077 .49048 L
.17176 .51104 L
Mistroke
.17268 .53012 L
.17313 .53962 L
.17356 .54857 L
.17368 .55098 L
.17374 .5523 L
.1738 .55353 L
.17383 .55338 L
.17387 .55302 L
.17394 .55225 L
.17406 .5509 L
.17451 .54595 L
.1765 .52341 L
.17842 .50073 L
.18026 .47812 L
.18222 .45327 L
.18411 .42866 L
.18593 .40444 L
.18786 .37814 L
.18875 .3659 L
.18926 .35892 L
.18972 .35241 L
.19019 .34591 L
.19033 .34403 L
.19039 .34311 L
.19045 .34226 L
.19051 .34256 L
.19057 .34378 L
.19069 .34633 L
.1917 .36739 L
.19353 .40521 L
.19547 .44548 L
.19636 .46402 L
.19683 .47381 L
.19733 .48423 L
.19748 .4872 L
.19755 .48865 L
.19758 .48929 L
.19761 .48999 L
.19767 .48952 L
.19773 .4888 L
.19786 .48727 L
.19835 .48148 L
.19932 .4699 L
.20127 .44602 L
.20314 .42236 L
.20513 .39659 L
.20705 .37123 L
.2089 .34644 L
.21086 .3197 L
.21184 .3063 L
Mistroke
.21276 .29368 L
.21363 .28158 L
.21388 .27808 L
.21401 .27636 L
.21412 .27476 L
.21423 .27331 L
.21429 .2725 L
.21434 .2736 L
.21457 .27848 L
.2165 .31844 L
.21835 .35689 L
.21929 .37638 L
.22032 .39779 L
.2208 .40761 L
.22106 .41302 L
.22119 .41568 L
.2213 .41815 L
.22136 .41934 L
.22142 .42046 L
.22147 .42021 L
.22153 .41949 L
.22178 .41658 L
.22222 .41147 L
.22408 .3894 L
.22606 .36539 L
.22796 .3418 L
.22979 .31877 L
.23174 .29396 L
.23362 .26986 L
.23562 .24409 L
.23662 .23115 L
.23705 .22546 L
.23754 .21919 L
.23776 .2163 L
.23788 .2147 L
.23795 .21392 L
.238 .21319 L
.23805 .2125 L
.23811 .21227 L
.23817 .21356 L
.23823 .21473 L
.23844 .21903 L
.23939 .23876 L
.24131 .27878 L
.24317 .31728 L
.24411 .3368 L
.24461 .34715 L
.24488 .35284 L
.24502 .35564 L
.24508 .35687 L
Mistroke
.24514 .35822 L
.24521 .3595 L
.24526 .35991 L
.24532 .35925 L
.24539 .35854 L
.24566 .35566 L
.24613 .35058 L
.24704 .34058 L
.24902 .31867 L
.25093 .2972 L
.25277 .2763 L
.25472 .25383 L
.2566 .23206 L
.25859 .20884 L
.2596 .19718 L
.26052 .18645 L
.26142 .176 L
.26164 .17346 L
.26176 .17206 L
.26182 .17138 L
.26187 .17074 L
.2619 .17038 L
.26194 .171 L
.26201 .17246 L
.26213 .17501 L
.26237 .18001 L
.26437 .22158 L
.2663 .26165 L
.26816 .3002 L
.26866 .31053 L
.26879 .31312 L
.2689 .31557 L
.26895 .31664 L
.26901 .31777 L
.26907 .31836 L
.26912 .31784 L
.26936 .31562 L
.26961 .31324 L
.27014 .30818 L
.27196 .29058 L
.2739 .27154 L
.27576 .25296 L
.27774 .233 L
.27965 .21364 L
.28149 .19495 L
.28344 .17503 L
.28435 .16583 L
.28482 .161 L
.28533 .1559 L
.28558 .1533 L
Mistroke
.28565 .15259 L
.28569 .15219 L
.28573 .1522 L
.28576 .15289 L
.2858 .15364 L
.28586 .15501 L
.28635 .16524 L
.28733 .18537 L
.28929 .22607 L
.29117 .26525 L
.29212 .2849 L
.29254 .29351 L
.29265 .29586 L
.29271 .29716 L
.29277 .29838 L
.29283 .29952 L
.29289 .29993 L
.29294 .29949 L
.29299 .29908 L
.29344 .29539 L
.29392 .29148 L
.29492 .28319 L
.29682 .26728 L
.29864 .2517 L
.30058 .23488 L
.30245 .2185 L
.30443 .20095 L
.30634 .18395 L
.30818 .16757 L
.30867 .16322 L
.30913 .15913 L
.30939 .15686 L
.30946 .15624 L
.30949 .15591 L
.30953 .15576 L
.30959 .15706 L
.30966 .15849 L
.31014 .16844 L
.31201 .20732 L
.31401 .24865 L
.315 .26934 L
.31592 .28846 L
.31615 .29308 L
.31639 .29805 L
.3165 .30034 L
.31656 .30158 L
.31661 .30274 L
.31666 .30374 L
.31671 .30354 L
.31677 .30314 L
Mistroke
.31682 .30278 L
.31727 .29963 L
.31777 .29612 L
.3197 .28229 L
.32155 .26862 L
.32352 .25375 L
.32542 .23914 L
.32724 .22487 L
.32918 .2095 L
.33105 .19456 L
.332 .18692 L
.33251 .18288 L
.33278 .18066 L
.33304 .17858 L
.33315 .17768 L
.33327 .17675 L
.33333 .17625 L
.33336 .17683 L
.3334 .17749 L
.33351 .17993 L
.33403 .19075 L
.33495 .20986 L
.33683 .24878 L
.33882 .29016 L
.33931 .30023 L
.33982 .31089 L
.34026 .32 L
.34038 .32249 L
.34042 .32321 L
.34045 .3239 L
.34051 .3242 L
.34057 .32384 L
.34062 .32351 L
.34075 .32275 L
.34259 .31105 L
.34448 .29865 L
.34648 .28501 L
.34841 .27145 L
.35026 .25806 L
.35223 .24351 L
.35413 .22923 L
.35596 .21531 L
.35644 .2116 L
.3569 .20809 L
.35702 .20714 L
.35705 .20689 L
.35709 .20663 L
.35715 .20638 L
.35719 .20712 L
.35722 .20779 L
Mistroke
.35729 .20934 L
.35743 .21207 L
.35791 .22203 L
.35981 .26159 L
.36165 .29963 L
.36267 .32085 L
.3636 .34013 L
.36406 .34975 L
.36419 .35239 L
.36425 .35372 L
.36429 .3544 L
.36432 .3542 L
.36456 .35279 L
.36548 .34734 L
.36734 .33577 L
.36933 .32286 L
.37124 .3099 L
.37308 .29699 L
.37504 .28282 L
.37693 .2688 L
.37893 .25355 L
.37994 .24577 L
.38038 .24234 L
.38061 .24052 L
.38074 .23952 L
.3808 .23902 L
.38086 .23857 L
.38092 .23812 L
.38098 .23836 L
.38104 .23962 L
.3811 .241 L
.38133 .24563 L
.38176 .25468 L
.38272 .27446 L
.38465 .31465 L
.38652 .35333 L
.38747 .37298 L
.38772 .37837 L
.38787 .38133 L
.38794 .38278 L
.388 .38411 L
.38806 .38538 L
.38809 .38602 L
.38813 .38587 L
.38826 .38506 L
.3885 .38364 L
.39041 .37177 L
.39235 .35901 L
.39422 .34612 L
.39621 .33182 L
Mistroke
.39813 .3175 L
.39997 .30329 L
.40193 .28774 L
.4029 .27987 L
.40382 .27239 L
.40424 .26889 L
.40448 .26698 L
.40459 .26604 L
.40469 .26519 L
.40475 .26471 L
.4048 .26549 L
.40486 .26669 L
.40493 .268 L
.40518 .27333 L
.40563 .28268 L
.4076 .3235 L
.40949 .3628 L
.41045 .38255 L
.41132 .4006 L
.41155 .40535 L
.41167 .40787 L
.4118 .41054 L
.41183 .41118 L
.41186 .41187 L
.41192 .41273 L
.41198 .41235 L
.41203 .41201 L
.41225 .4106 L
.41326 .40401 L
.41516 .39098 L
.41698 .37779 L
.41893 .36309 L
.4208 .34836 L
.42279 .33212 L
.42471 .31598 L
.42656 .30004 L
.42749 .29184 L
.42799 .28745 L
.42825 .28513 L
.42839 .28387 L
.42846 .28325 L
.42852 .28269 L
.42858 .28243 L
.42864 .28362 L
.42876 .28623 L
.42903 .29174 L
.4295 .30142 L
.43041 .3204 L
.43226 .35882 L
.43423 .39969 L
Mistroke
.43471 .40951 L
.43521 .42004 L
.43546 .42514 L
.43558 .42765 L
.43563 .42876 L
.43569 .42998 L
.43574 .43024 L
.43579 .4299 L
.4359 .42914 L
.43613 .42753 L
.43795 .41428 L
.43981 .40007 L
.44179 .38422 L
.44369 .36831 L
.44552 .35245 L
.44747 .33505 L
.44935 .31783 L
.4503 .3089 L
.45134 .29909 L
.45182 .29448 L
.45209 .29194 L
.45222 .2907 L
.45228 .29015 L
.45234 .28956 L
.45239 .28941 L
.45245 .2905 L
.45255 .29263 L
.45278 .29735 L
.45326 .30738 L
.45514 .34643 L
.45714 .38793 L
.45815 .40876 L
.45859 .41789 L
.45907 .42792 L
.45929 .43259 L
.45942 .43516 L
.45948 .43643 L
.45954 .43727 L
.45959 .43687 L
.45964 .43644 L
.45976 .43553 L
.45997 .43394 L
.46092 .42651 L
.46274 .41182 L
.46467 .39546 L
.46653 .37908 L
.46851 .36103 L
.47042 .3431 L
.47225 .32541 L
.4742 .30616 L
Mistroke
.4751 .29714 L
.47558 .29236 L
.47584 .28973 L
.47596 .28845 L
.47608 .28728 L
.47614 .28664 L
.4762 .28642 L
.47627 .28773 L
.47633 .28916 L
.47661 .29497 L
.47711 .30518 L
.47808 .3253 L
.48003 .36591 L
.48192 .40502 L
.48241 .41531 L
.48268 .42072 L
.48295 .42643 L
.48319 .43144 L
.48326 .43281 L
.48329 .43357 L
.48333 .43428 L
.48336 .43415 L
.4834 .43387 L
.48346 .43338 L
.48392 .42957 L
.48589 .413 L
.48778 .39634 L
.48959 .37971 L
.49153 .36141 L
.49339 .34329 L
.49538 .32354 L
.49728 .3041 L
.49912 .2851 L
.49935 .28266 L
.49961 .28001 L
.49972 .2788 L
.49985 .27751 L
.4999 .27698 L
.49995 .27641 L
.50001 .27608 L
.50007 .27724 L
.50059 .2882 L
.50107 .29816 L
.50306 .33941 L
.50498 .37914 L
.50585 .3973 L
.50635 .40765 L
.50659 .41271 L
.50682 .41736 L
.50704 .42198 L
Mistroke
.50709 .4231 L
.50715 .42401 L
.50722 .42348 L
.50728 .42298 L
.50777 .41878 L
.50878 .41003 L
.51077 .39219 L
.51269 .37434 L
.51454 .35663 L
.51651 .33726 L
.5184 .31817 L
.52022 .29945 L
.52124 .28891 L
.52216 .2792 L
.52262 .27438 L
.52312 .26911 L
.52335 .26669 L
.52359 .26412 L
.52365 .26351 L
.52371 .26286 L
.52377 .26225 L
.52382 .26204 L
.52387 .26304 L
.52392 .26414 L
.52403 .26635 L
.52597 .30672 L
.52785 .34557 L
.5288 .36536 L
.52984 .38687 L
.53032 .3969 L
.53059 .40239 L
.53071 .40508 L
.53083 .40755 L
.53089 .40865 L
.53094 .40968 L
.53099 .40971 L
.53104 .40923 L
.53127 .40727 L
.53175 .40309 L
.53363 .38641 L
.53563 .36803 L
.53755 .34975 L
.53941 .33169 L
.54138 .31203 L
.54327 .29273 L
.54509 .27388 L
.54611 .2633 L
.54704 .25358 L
.54726 .25124 L
.54749 .24875 L
Mistroke
.54753 .2484 L
.54756 .24801 L
.5476 .24765 L
.54763 .24769 L
.54769 .24893 L
.54775 .25024 L
.54799 .25523 L
.5489 .27411 L
.55073 .31208 L
.55268 .35249 L
.55358 .37115 L
.55405 .38098 L
.55456 .3914 L
.55468 .39393 L
.55471 .39466 L
.55475 .39535 L
.55481 .39524 L
.55485 .39493 L
.55488 .39465 L
.55495 .39401 L
.55509 .39286 L
.55558 .38864 L
.55655 .38021 L
.55843 .36338 L
.56043 .34491 L
.56236 .32661 L
.56421 .30859 L
.56618 .28905 L
.56808 .26992 L
.56991 .25131 L
.57085 .24167 L
.57112 .23889 L
.57125 .23752 L
.57131 .23692 L
.57137 .23626 L
.57143 .23568 L
.57148 .2368 L
.5716 .23925 L
.57185 .24444 L
.57371 .28304 L
.57569 .32409 L
.57668 .34456 L
.5776 .36362 L
.57805 .37307 L
.57827 .37772 L
.57837 .37976 L
.57848 .382 L
.57854 .38324 L
.5786 .38367 L
.57866 .38318 L
Mistroke
.57872 .38266 L
.57898 .38051 L
.57943 .37674 L
.58141 .35972 L
.58332 .34276 L
.58516 .32598 L
.58712 .30768 L
.589 .28968 L
.59101 .27023 L
.59294 .25121 L
.59382 .24242 L
.59432 .23741 L
.59479 .23273 L
.59503 .23035 L
.59509 .22978 L
.59515 .22917 L
.5952 .22866 L
.59526 .22862 L
.59529 .22926 L
.59532 .22995 L
.59538 .2312 L
.59552 .23401 L
.59575 .23893 L
.59677 .25996 L
.59875 .30106 L
.60065 .34064 L
.60162 .3606 L
.60204 .3693 L
.60215 .37168 L
.60227 .3742 L
.60232 .37524 L
.60238 .37635 L
.60243 .37605 L
.60249 .37561 L
.60298 .37169 L
.60343 .36797 L
.60444 .35966 L
.60643 .34282 L
.60834 .32608 L
.61019 .30955 L
.61214 .29156 L
.61403 .2739 L
.61584 .25667 L
.61685 .24701 L
.61778 .2381 L
.61825 .23357 L
.61868 .22934 L
.6189 .2272 L
.61896 .22659 L
.61902 .22601 L
Mistroke
.61908 .22642 L
.61914 .22763 L
.61964 .23797 L
.6216 .27877 L
.62349 .31805 L
.62531 .35582 L
.62579 .36575 L
.62591 .36812 L
.62603 .37065 L
.62609 .37188 L
.62614 .37303 L
.62619 .37396 L
.62625 .37354 L
.6265 .3716 L
.62677 .36947 L
.62725 .36571 L
.62923 .34984 L
.63113 .33402 L
.63295 .31834 L
.6349 .30122 L
.63677 .28437 L
.63876 .26613 L
.64068 .24828 L
.64155 .24004 L
.64205 .23534 L
.6423 .23303 L
.64252 .23091 L
.64274 .2288 L
.6428 .22829 L
.64285 .22774 L
.64292 .22898 L
.64298 .23021 L
.64347 .24045 L
.64448 .26142 L
.64648 .30284 L
.6484 .34275 L
.64928 .36104 L
.64952 .36605 L
.64978 .37143 L
.64984 .37268 L
.64991 .37404 L
.64997 .37532 L
.65003 .37571 L
.65008 .37529 L
.65014 .3749 L
.65025 .37404 L
.65121 .36676 L
.65222 .35889 L
.65415 .34345 L
.65601 .32807 L
Mistroke
.65798 .31124 L
.65988 .2946 L
.66171 .27827 L
.66366 .26058 L
.66554 .2433 L
.66603 .23874 L
.66629 .23634 L
.66643 .23502 L
.66656 .23378 L
.66662 .23323 L
.66668 .23296 L
.66671 .2336 L
.66674 .2343 L
.6668 .23553 L
.66706 .24101 L
.66753 .25066 L
.66948 .2912 L
.67136 .33023 L
.67232 .35015 L
.67283 .36063 L
.67336 .3717 L
.67359 .37649 L
.67365 .37775 L
.67372 .37912 L
.67378 .38042 L
.67384 .38077 L
.67387 .38053 L
.67391 .38027 L
.67398 .37973 L
.67411 .37877 L
.67436 .37689 L
.67529 .36994 L
.67722 .35494 L
.67908 .33995 L
.68106 .32347 L
.68296 .30714 L
.68479 .29106 L
.68674 .27359 L
.68862 .25649 L
.68958 .24768 L
.68984 .24526 L
.69012 .24269 L
.69023 .24163 L
.69035 .24049 L
.69039 .24015 L
.69043 .23984 L
.69046 .23955 L
.69049 .23964 L
.69052 .2403 L
.69056 .241 L
Mistroke
.69062 .24226 L
.69254 .28214 L
.69454 .32367 L
.69554 .34451 L
.69647 .36368 L
.69693 .37337 L
.69737 .38241 L
.69748 .38475 L
.69754 .3859 L
.69759 .38696 L
.69765 .38736 L
.69768 .38713 L
.69771 .38689 L
.69782 .38607 L
.69832 .38238 L
.70014 .36858 L
.70208 .35328 L
.70394 .338 L
.70592 .32124 L
.70782 .30464 L
.70966 .28831 L
.71161 .27059 L
.71251 .2623 L
.71349 .25326 L
.71375 .25084 L
.71403 .24828 L
.71414 .24722 L
.7142 .24664 L
.71426 .24609 L
.71429 .24603 L
.71433 .24678 L
.71439 .24811 L
.71452 .25069 L
.71549 .27081 L
.71745 .31146 L
.71933 .35059 L
.7203 .37059 L
.7208 .38109 L
.72108 .38683 L
.72121 .38962 L
.72127 .39085 L
.72134 .39217 L
.7214 .39346 L
.72146 .39386 L
.72151 .39346 L
.72157 .39303 L
.72182 .3912 L
.72234 .38726 L
.72327 .38023 L
.72509 .36595 L
Mistroke
.72703 .35016 L
.72889 .33442 L
.73087 .31719 L
.73278 .30016 L
.73462 .28343 L
.73564 .27396 L
.73657 .2653 L
.73704 .26095 L
.73754 .25622 L
.73777 .25404 L
.7379 .25285 L
.73796 .25227 L
.73802 .25174 L
.73807 .25125 L
.73812 .25151 L
.73817 .25258 L
.73823 .25375 L
.73846 .2585 L
.74042 .2992 L
.74231 .33838 L
.74412 .37605 L
.74457 .38546 L
.74482 .39059 L
.74505 .39536 L
.74517 .39787 L
.74521 .39859 L
.74524 .39921 L
.7453 .39875 L
.74558 .3967 L
.74605 .39307 L
.7479 .37868 L
.74987 .36271 L
.75177 .34676 L
.75359 .33094 L
.75553 .31362 L
.7574 .29656 L
.75938 .27805 L
.76037 .26868 L
.76086 .26409 L
.76129 .25992 L
.76152 .25782 L
.76163 .25671 L
.76175 .25554 L
.76181 .25504 L
.76186 .25451 L
.76192 .25445 L
.76198 .25563 L
.76219 .25992 L
.76313 .2796 L
.76513 .32093 L
Mistroke
.76704 .36074 L
.76792 .37896 L
.76842 .38932 L
.76867 .3944 L
.76877 .39662 L
.76889 .39904 L
.76895 .40021 L
.76901 .40145 L
.76906 .40218 L
.76911 .40179 L
.76935 .4 L
.76984 .3962 L
.77085 .38828 L
.77278 .3727 L
.77463 .35712 L
.7766 .33997 L
.7785 .32297 L
.78033 .30622 L
.78227 .288 L
.78414 .27016 L
.78463 .26545 L
.78516 .26031 L
.7854 .25795 L
.78553 .25666 L
.7856 .25602 L
.78566 .25544 L
.78571 .25494 L
.78577 .25599 L
.78588 .25837 L
.78613 .26344 L
.78796 .30144 L
.78991 .34188 L
.79178 .38081 L
.79227 .39102 L
.79253 .39639 L
.79267 .39933 L
.79274 .40077 L
.79281 .40209 L
.79287 .40305 L
.79292 .40262 L
.79305 .40166 L
.79331 .3996 L
.79378 .39596 L
.79561 .38115 L
.79757 .36476 L
.79945 .3484 L
.80145 .33047 L
.80338 .31272 L
.80524 .29526 L
.80721 .27634 L
Mistroke
.80819 .26681 L
.80867 .26212 L
.80911 .25785 L
.80933 .25572 L
.80939 .25513 L
.80945 .25451 L
.8095 .25398 L
.80956 .25458 L
.80962 .25573 L
.80967 .25679 L
.80978 .25917 L
.80999 .26348 L
.81094 .28309 L
.81284 .32265 L
.81468 .3607 L
.8157 .38192 L
.81618 .39196 L
.8164 .39636 L
.81652 .39886 L
.81657 .40008 L
.81663 .4012 L
.81668 .40185 L
.81674 .40144 L
.81685 .40052 L
.81709 .39865 L
.8176 .39466 L
.81851 .3873 L
.82035 .372 L
.82231 .35512 L
.82419 .33833 L
.82619 .31997 L
.82813 .30185 L
.82998 .28407 L
.83093 .27493 L
.83196 .26484 L
.83243 .26017 L
.8327 .2576 L
.83294 .25516 L
.83317 .25289 L
.83324 .25226 L
.83327 .25192 L
.8333 .2516 L
.83336 .25187 L
.83342 .25317 L
.83386 .26225 L
.83572 .30093 L
.83771 .34205 L
.8387 .36259 L
.83962 .38167 L
.84007 .39116 L
Mistroke
.8403 .39582 L
.8404 .39786 L
.84045 .39902 L
.8405 .39929 L
.84056 .39885 L
.84062 .39838 L
.84074 .39741 L
.841 .39536 L
.84145 .39175 L
.84344 .3754 L
.84536 .35901 L
.8472 .34273 L
.84916 .3249 L
.85105 .3073 L
.85286 .29003 L
.8548 .27132 L
.85568 .26263 L
.85619 .25768 L
.85666 .25306 L
.8569 .2507 L
.85701 .24953 L
.85706 .24902 L
.85712 .24846 L
.85718 .24908 L
.85722 .24977 L
.85725 .25041 L
.85738 .25323 L
.85762 .25814 L
.85863 .2792 L
.8605 .31793 L
.86249 .35911 L
.86348 .37969 L
.86373 .38484 L
.86396 .3897 L
.86417 .39406 L
.86423 .39526 L
.86429 .39644 L
.86434 .39603 L
.8644 .39558 L
.86529 .38848 L
.86624 .38075 L
.86818 .36442 L
.87005 .34814 L
.87205 .33028 L
.87396 .31261 L
.87581 .29524 L
.87777 .27641 L
.87875 .26695 L
.87966 .25801 L
.88009 .25382 L
Mistroke
.88054 .24941 L
.88077 .24709 L
.88084 .24646 L
.88087 .24611 L
.88091 .24578 L
.88094 .24547 L
.88097 .24572 L
.88103 .24695 L
.88148 .2563 L
.88345 .29717 L
.88535 .33653 L
.88717 .37438 L
.88765 .38435 L
.88789 .38927 L
.88799 .39143 L
.88805 .39258 L
.88811 .39344 L
.88817 .39294 L
.88823 .39248 L
.88836 .39146 L
.88863 .38928 L
.88911 .38545 L
.89109 .36923 L
.89299 .353 L
.89482 .33689 L
.89677 .31926 L
.89865 .30188 L
.90064 .28302 L
.90256 .26454 L
.90344 .25599 L
.90394 .25111 L
.90419 .24872 L
.90441 .24653 L
.90452 .24542 L
.90465 .24422 L
.9047 .24367 L
.90476 .24311 L
.90482 .24428 L
.90487 .24534 L
.90537 .2556 L
.90638 .27659 L
.9083 .31658 L
.91016 .35506 L
.9111 .37457 L
.91136 .37992 L
.91163 .38565 L
.91177 .38843 L
.91183 .38979 L
.91189 .39106 L
.91195 .39096 L
Mistroke
.91201 .3905 L
.91213 .38953 L
.91403 .37431 L
.91585 .35919 L
.91778 .34253 L
.91964 .32601 L
.92162 .30799 L
.92353 .29023 L
.92536 .27285 L
.92638 .26304 L
.92731 .25408 L
.92777 .24958 L
.92802 .24721 L
.92828 .24469 L
.92839 .24363 L
.9285 .24249 L
.92854 .24216 L
.92857 .24184 L
.92863 .24306 L
.92875 .24555 L
.92919 .25463 L
.93103 .2928 L
.93299 .33342 L
.93487 .37253 L
.93537 .38282 L
.9355 .38564 L
.93558 .38718 L
.93565 .38862 L
.93571 .38985 L
.93577 .38961 L
.9358 .38933 L
.93584 .38907 L
.9359 .38857 L
.93687 .38094 L
.93872 .36597 L
.94068 .34944 L
.94258 .333 L
.94439 .31675 L
.94633 .29903 L
.94819 .28164 L
.95018 .26283 L
.95116 .25333 L
.95165 .24869 L
.95208 .24446 L
.95219 .24343 L
.9523 .24234 L
.95236 .24176 L
.9524 .24189 L
.95243 .24254 L
.95254 .24491 L
Mistroke
.95297 .25385 L
.95392 .27351 L
.95579 .31231 L
.95778 .35356 L
.95877 .3742 L
.95921 .38327 L
.95933 .38576 L
.95939 .38707 L
.95946 .38844 L
.95952 .38965 L
.95955 .38959 L
.95958 .38936 L
.95969 .3885 L
.96059 .38153 L
.96153 .37398 L
.96341 .3586 L
.96541 .34166 L
.96733 .32481 L
.96918 .30818 L
.97114 .29009 L
.97304 .27234 L
.97486 .25502 L
.97619 .24222 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  Background->GrayLevel[1],
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005X0000gR000`400?l00000o`00003oo`3ooomY
0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`006P3oool0
0`000000oooo0?ooo`3o0?oood`0oooo000J0?ooo`030000003oool0oooo0?l0ooooC03oool001X0
oooo00<000000?ooo`3oool0o`3ooom<0?ooo`00100000050?ooo`8000001`3oool3000000D0oooo
00<000000?ooo`3oool0o`3ooom<0?ooo`00103oool00`000000oooo0?ooo`020?ooo`800000203o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?l0ooooC03oool000@0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`040?ooo`030?l0003oool0oooo0?l0ooooC03oool0
00@0oooo00<000000?ooo`3oool0203oool6000000D0oooo00<0o`00000000000000o`3ooom<0?oo
o`00103oool00`000000oooo0?ooo`090?ooo`040000003oool0oooo000000H0oooo00<0o`000?oo
o`3oool0o`3ooom<0?ooo`00103oool00`000000oooo0?ooo`090?ooo`040000003oool0oooo0000
00H0oooo00<0o`000?ooo`3oool0o`3ooom<0?ooo`00103oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0000000H0oooo00<0o`000?ooo`3oool0o`3ooom<0?ooo`001000000>0?ooo`800000
1P3oool00`3o0000oooo0?ooo`3o0?oood`0oooo000J0?ooo`030?l0003oool0oooo0?l0ooooC03o
ool001X0oooo00<0o`3o0?l0003oool0o`3ooom<0?ooo`006P3oool00`3o0?l0o`000?ooo`3o0?oo
od`0oooo000J0?ooo`030000003o0000oooo0?l0ooooC03oool001X0oooo00<000000?l0003oool0
o`3ooom<0?ooo`006P3oool00`000000o`000?ooo`3o0?oood`0oooo000J0?ooo`030000003o0000
oooo0?l0ooooC03oool001X0oooo00<000000?l0003oool0o`3ooom<0?ooo`006P3oool00`000000
o`000?ooo`3o0?oood`0oooo000J0?ooo`030000003o0000oooo0?l0ooooC03oool001X0oooo00<0
00000?l0003oool0o`3ooom<0?ooo`006P3oool00`000000o`000?ooo`3o0?oood`0oooo000J0?oo
o`030000o`3o0000oooo0?l0ooooC03oool001X0oooo00<0003o0?l0003oool0o`3ooom<0?ooo`00
6P3oool00`000?l0o`000?ooo`3o0?oood`0oooo000J0?ooo`030000003o0?l0o`000?l0ooooC03o
ool001X0oooo00<000000?l0o`3o0000o`3ooom<0?ooo`006P3oool00`000000o`3o0?l0003o0?oo
od`0oooo000J0?ooo`030000003o0?l0o`000?l0ooooC03oool001X0oooo00<000000000o`3o0000
o`3ooom<0?ooo`006P3oool00`000000003o0?l0003o0?oood`0oooo000J0?ooo`030000003oool0
o`000?l0ooooC03oool001X0oooo00<000000?ooo`3o0000o`3ooom<0?ooo`006P3oool00`000000
oooo0?l0003o0?oood`0oooo000J0?ooo`030000003oool0o`000?l0ooooC03oool000@000001@3o
ool2000000H0oooo0`0000060?ooo`030000003oool0o`000?l0ooooC03oool000@0oooo00<00000
0?ooo`3oool00P3oool2000000D0oooo00D000000?ooo`3oool0oooo000000050?ooo`030000003o
ool0o`000?l0ooooC03oool000@0oooo00<000000?ooo`3oool02@3oool01@000000oooo0?ooo`3o
ool0000000D0oooo00@000000?ooo`3o0000003oo`3ooom;0?ooo`00103oool00`000000oooo0?oo
o`090?ooo`050000003oool0oooo0?ooo`0000001@3oool2000000030?l000000?l0oooo0?l0oooo
BP3oool000@0oooo00<000000?ooo`3oool02@3oool4000000H0oooo00@000000?ooo`3o0000003o
o`3ooom;0?ooo`00103oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00L0oooo00@0
00000?ooo`3o0000003oo`3ooom;0?ooo`00103oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00H0oooo00@000000?ooo`3o0000003oo`3ooom;0?ooo`001000000>0?ooo`<000001@3o
ool010000000oooo0?l000000?oo0?oood/0oooo000J0?ooo`050000003oool0o`000?ooo`000?l0
o`3ooom:0?ooo`006P3oool01@000000oooo0?l0003oool0003o0?l0ooooBP3oool001X0oooo00D0
00000?ooo`3o0000oooo0000o`3o0?ooodX0oooo000J0?ooo`050000003oool0o`000?ooo`000?l0
o`3ooom:0?ooo`006P3oool2000000030?l0003oool0003o0?l0ooooBP3oool001X0oooo00D00000
0?ooo`3o0000oooo0000o`3o0?ooodX0oooo000J0?ooo`030000003oool0o`000080oooo00<0003o
0?ooo`3oool0o`3ooom70?ooo`006P3oool00`000000oooo0?l000020?ooo`030000o`3oool0oooo
0500oooo0P3o003d0?ooo`006P3oool01P000000oooo0?l0003o0?l0oooo0000oe80oooo0P3o003d
0?ooo`006P3oool01P000000oooo0?l0003o0?l0oooo0000oe80oooo0P3o00060?ooo`80o`00k03o
ool001X0oooo00H000000?ooo`3oool0o`000?ooo`000?mB0?ooo`80o`001P3oool20?l00>`0oooo
000J0?ooo`060000003oool0oooo0?l0003o0?l0003oDP3oool20?l000H0oooo0P3o003/0?ooo`00
6P3oool2000000070?ooo`3o0000o`000?l0003o0?l0oooo0?l0001?0?ooo`80o`001P3oool20?l0
0>`0oooo000J0?ooo`030000003oool0oooo0080oooo103o001?0?ooo`80o`001P3oool20?l00>`0
oooo000J0?ooo`030000003oool0oooo00<0oooo00<0003o0?ooo`3o0000A`3oool20?l000H0oooo
0P3o00060?ooo`80o`00k03oool001X0oooo00<000000?ooo`3oool00`3oool00`000?l0oooo0?l0
00170?ooo`80o`001P3oool20?l000H0oooo0P3o003/0?ooo`006P3oool00`000000oooo0?ooo`03
0?ooo`030000o`3oool0o`0004L0oooo0P3o00060?ooo`80o`001@3oool00`3o0000o`3o0?l00006
0?ooo`80o`00i03oool001X0oooo00<000000?ooo`3oool00`3oool010000?l0oooo0?l0o`3o0016
0?ooo`80o`001P3oool20?l000D0oooo00<0o`000?l0o`3o00001P3oool20?l00>@0oooo000J0?oo
o`030000003oool0oooo00@0oooo00<0003o0?l0o`3o0000AP3oool20?l000H0oooo0P3o00050?oo
o`030?l0003o0?l0o`0000H0oooo0P3o003T0?ooo`006P3oool00`000000oooo0?ooo`040?ooo`03
0000o`3o0?l0o`0004H0oooo0P3o00050?ooo`030?l0003o0?l0o`0000D0oooo00<0o`000?l0o`3o
00001P3oool20?l00>@0oooo000J0?ooo`8000001@3oool00`000?l0o`3o0?l000160?ooo`80o`00
1@3oool00`3o0000o`3o0?l000050?ooo`030?l0003oool0o`0000H0oooo0P3o003T0?ooo`006P3o
ool00`000000oooo0?ooo`040?ooo`030000o`3o0?l0o`0004H0oooo0P3o00050?ooo`030?l0003o
ool0o`0000D0oooo00<0o`000?ooo`3o00001P3oool20?l00>@0oooo000J0?ooo`030000003oool0
oooo00@0oooo00<0003o0?l0o`3o0000AP3oool20?l000D0oooo00<0o`000?ooo`3o00001@3oool0
0`3o0000oooo0?l000060?ooo`80o`00i03oool001X0oooo00<000000?ooo`3oool0103oool00`00
0?l0o`3o0?l000160?ooo`80o`001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`00
00H0oooo0P3o003T0?ooo`006P3oool00`000000oooo0?ooo`050?ooo`030?l0o`3o0000oooo04D0
oooo0P3o00050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001P3oool20?l00>@0
oooo000J0?ooo`030000003oool0oooo00D0oooo00<0003o0?l0003oool0A03oool00`3o0000o`3o
0?l000040?ooo`040?l0003o0?l0oooo0?l000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000
oooo0?l0003T0?ooo`00100000050?ooo`8000001@3oool4000000H0oooo00<000000?ooo`3oool0
1@3oool00`000?l0o`000?ooo`140?ooo`030?l0003o0?l0o`0000@0oooo00D0o`000?l0o`3oool0
o`3o0?l000030?ooo`040?l0o`3o0000oooo0?l000D0oooo00<0o`000?ooo`3o00001P3oool20?l0
0=`0oooo00040?ooo`030000003oool0oooo0080oooo0P0000040?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000o`3o0000oooo04@0
oooo00<0o`000?l0o`3o0000103oool01@3o0000o`3o0?ooo`3o0?l0o`0000<0oooo00@0o`000?oo
o`3oool0o`001@3oool00`3o0000oooo0?l000060?ooo`80o`00g03oool000@0oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool01@3oool00`000?l0o`000?ooo`0m0?ooo`030?l0003oool0oooo00@0oooo00<0o`000?l0
o`3o0000103oool01@3o0000o`3o0?ooo`3o0?l0o`0000<0oooo00@0o`000?ooo`3oool0o`001@3o
ool00`3o0000oooo0?l000060?ooo`80o`00g03oool000@0oooo00<000000?ooo`3oool0203oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo0`0000050?ooo`030000o`3o0000
oooo03d0oooo00<0o`000?ooo`3oool0103oool00`3o0000oooo0?l000040?ooo`050?l0003oool0
oooo0?l0o`3o00000`3oool0103o0000oooo0?ooo`3o00040?ooo`040?l0o`3o0000oooo0?l000H0
oooo0P3o003L0?ooo`00103oool00`000000oooo0?ooo`090?ooo`@000001P3oool00`000000oooo
0?ooo`050?ooo`030000o`3o0000oooo03d0oooo0P3o00050?ooo`030?l0003oool0o`0000@0oooo
00D0o`000?ooo`3oool0o`3o0?l000030?ooo`040?l0003oool0oooo0?l000@0oooo00D0o`000?oo
o`3oool0o`000?l0o`050?ooo`80o`00g03oool000@0oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01P3oool00`3o
0000oooo0?ooo`0l0?ooo`80o`001@3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0oooo
0?l0o`3o00000`3oool0103o0000oooo0?ooo`3o00040?ooo`050?l0003oool0oooo0?l0003o0?l0
1@3oool20?l00=`0oooo00040?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00H0oooo00<0o`000?ooo`3oool0?03o
ool20?l000D0oooo00<0o`000?ooo`3o0000103oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo
00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?ooo`3o0000o`3o00D0oooo0P3o001G0?oo
o`030?l0003oool0oooo0880oooo0004000000`0oooo100000060?ooo`030000003oool0oooo00H0
oooo00<0o`000?ooo`3oool0?03oool20?l000D0oooo00<0o`000?ooo`3o0000103oool01@3o0000
oooo0?ooo`000?l0o`0000<0003o00@0o`000000o`000?l0o`00103oool01@3o0000oooo0?ooo`3o
0000o`3o00D0oooo0P3o001?0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3o
ool00`3o0000oooo0?ooo`1j0?ooo`006P3oool00`000000oooo0?ooo`060?ooo`030?l0003oool0
oooo03`0oooo0P3o00050?ooo`030?l0003oool0o`0000@0oooo00D0o`000000o`000?l0oooo0?l0
00030?ooo`050?l0003oool0oooo0?l0o`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000@0
oooo00<0o`3o0?l0003o0000C`3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0
oooo00<0o`000?ooo`3oool0NP3oool001X0oooo00<000000?ooo`3oool01P3oool00`3o0000oooo
0?ooo`0l0?ooo`80o`001@3oool00`3o0000oooo0?l000040?ooo`050?l0003oool0oooo0?ooo`3o
00000`3oool01@3o0000oooo0?ooo`3o0?l0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l00004
0?ooo`030?l0o`3o0000o`0004l0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`05
0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0LP3oool001X0oooo00<000000?oo
o`3oool01P3oool00`3o0000oooo0?ooo`0l0?ooo`80o`001@3oool00`3o0000oooo0?l000030?oo
o`030000o`3o0000oooo0080oooo00D0o`000?ooo`3oool0o`3o0?l000020?ooo`060?l0o`3o0000
003o0000o`3oool0o`000`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0000L0oooo
00<0o`000?ooo`3oool0?@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00@0oooo
0P3o00070?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0LP3oool001X0oooo0P00
00070?ooo`030?l0003oool0oooo03`0oooo0P3o00050?ooo`080?l0003oool0o`000?l0o`3oool0
003o0?l0o`3o00030?ooo`050?l0003oool0oooo0?l0o`3o00000P3oool01P3o0?l0o`000?ooo`3o
ool0003o0?l000<0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l000060?ooo`80o`00
?`3oool00`3o0000oooo0?ooo`040?ooo`80o`001P3oool20?l000H0oooo0P3o00070?ooo`030?l0
003oool0oooo00D0oooo00<0o`000?ooo`3oool0JP3oool001X0oooo00<000000?ooo`3oool01P3o
ool00`3o0000003o0?ooo`0l0?ooo`80o`00103oool0203o0000oooo0?ooo`3oool0o`000000o`3o
ool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`050?l0003oool0oooo0?ooo`3o00000`3o
ool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0000H0oooo0P3o000o0?ooo`030?l0003o
ool0oooo00@0oooo0P3o00060?ooo`80o`001P3oool20?l000L0oooo00<0o`000?ooo`3oool01@3o
ool00`3o0000oooo0?ooo`150?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3o
ool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00X0oooo000J0?ooo`030000003oool0
oooo00H0oooo00<0o`000000o`3oool0>`3oool00`3o0?l0o`000?l000040?ooo`050?l0003oool0
oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`040?l0003oool0oooo0?l000@0oooo
00H0o`000?ooo`3oool0o`3o0?l000000?l20?ooo`030?l0003oool0oooo0080oooo00<0o`000?oo
o`3o00001P3oool20?l003l0oooo00<0o`000?ooo`3oool0103oool20?l000H0oooo0P3o00060?oo
o`80o`001`3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?oo
o`3oool0=@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?oo
o`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00X0oooo000J0?ooo`03
0000003oool0oooo00H0oooo00<0o`3o0?l0003oool0>`3oool00`3o0?l0o`000?l000040?ooo`05
0?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`040?l0003oool0oooo
0?l000@0oooo00@0o`000?ooo`3oool0o`000P3oool00`000?l0oooo0?l000040?ooo`030?l0003o
ool0o`0000H0oooo0P3o000g0?ooo`030?l0003oool0oooo00@0oooo0P3o00060?ooo`80o`001P3o
ool20?l000H0oooo0P3o00060?ooo`030?l0o`3o0000oooo00H0oooo00<0o`000?ooo`3oool01@3o
ool00`3o0000oooo0?ooo`0]0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3o
ool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3o
ool00`3o0000oooo0?ooo`0:0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030?l0003oool0
oooo03X0oooo00<0o`3o0?l0003o0000103oool01@3o0000oooo0?ooo`000?l0o`000080oooo00<0
o`000?ooo`3oool00P3oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000<0
oooo00<0003o0?l0003oool00P3oool01@3o0000o`3o0?ooo`3o0000o`3o00D0oooo0P3o000g0?oo
o`030?l0003oool0oooo00@0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?oo
o`80o`001`3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?oo
o`3oool07@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?oo
o`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?oo
o`3oool01@3oool00`3o0000oooo0?ooo`0:0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`03
0?l0003oool0oooo03X0oooo00<0o`3o0?l0003o0000103oool01@3o0000oooo0000o`3oool0o`00
0080oooo00<0o`000?ooo`3oool00P3oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0
oooo0?l000@0oooo00T0o`000?ooo`3oool0oooo0?l0003o0?l0oooo0?l0003o0?l01@3oool20?l0
03L0oooo00<0o`000?ooo`3oool0103oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l0
00H0oooo0P3o00060?ooo`80o`001`3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo
00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo
00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo
00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo
00D0oooo00<0o`000?ooo`3oool02P3oool001X0oooo00<000000?ooo`3oool01`3oool00`3o0000
oooo0?ooo`0j0?ooo`030?l0003oool0o`0000@0oooo00D0o`000?ooo`000?l0oooo0?l000020?oo
o`030?l0003oool0oooo0080oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o
00040?ooo`090?l000000?l0oooo0?ooo`3o0000o`3o0?ooo`3o0000o`3o00D0oooo0P3o000g0?oo
o`030?l0003oool0oooo00@0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?oo
o`80o`001P3oool20?l000L0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?oo
o`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?oo
o`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?oo
o`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?oo
o`030?l0003oool0oooo00X0oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`000?ooo`3o
ool0>P3oool0103o0000oooo0?l0o`3o00030?ooo`050?l000000?l0oooo0?ooo`3o00000P3oool0
0`3o0000oooo0?ooo`020?ooo`040?l0003oool0o`000?l0o`@0oooo00@0o`000?ooo`3oool0o`00
103oool01@3o0000oooo0000o`3oool0o`000080oooo00<0o`000?l0o`3oool0103oool20?l003H0
oooo0P3o00060?ooo`80o`001@3oool00`3o0000oooo0?l000050?ooo`040?l0o`3o0000o`3o0?l0
00@0oooo00<0o`3o0?l0003o00001P3oool20?l000H0oooo0P3o00060?ooo`80o`001`3oool00`3o
0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o
0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o
0000oooo0?ooo`040?ooo`80o`001P3oool20?l000H0oooo0P3o00070?ooo`030?l0003oool0oooo
00X0oooo000J0?ooo`800000203oool00`3o0000003o0?ooo`0j0?ooo`040?l0003oool0o`3o0?l0
00<0oooo00P0o`000000o`3oool0oooo0?l0003oool0o`3o0?l000@0oooo00@0o`000?ooo`3o0000
o`3o103oool0103o0000oooo0?ooo`3o00040?ooo`050?l0003oool0oooo0000o`3o00000P3oool0
0`3o0000o`3o0?ooo`030?ooo`030?l0003o0?l0o`0000L0oooo00<0o`000?ooo`3oool09@3oool0
0`3o0000oooo0?ooo`040?ooo`80o`001P3oool20?l000D0oooo00<0o`000?ooo`3o00001@3oool0
103o0?l0o`000?l0o`3o00040?ooo`030?l0o`3o0000o`0000H0oooo0P3o00060?ooo`030?l0003o
0?l0o`0000D0oooo0P3o00060?ooo`80o`001`3oool00`3o0000oooo0?ooo`050?ooo`030?l0003o
ool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003o
ool0oooo00@0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00000`3o0?l0oooo0?oo
o`030?ooo`030?l0003o0?l0oooo00/0oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`00
0000o`3oool0>P3oool0103o0000oooo0?l0o`3o00020?ooo`030?l0003o0?l0oooo0080oooo00@0
o`000?ooo`3o0?l0o`00103oool0103o0000oooo0?l0003o0?l40?ooo`040?l0003oool0oooo0?l0
00@0oooo00D0o`000?ooo`3oool0003o0?l000020?ooo`030?l0003o0?l0oooo00<0oooo00<0o`00
0?ooo`3o00001`3oool00`3o0000oooo0?ooo`0U0?ooo`030?l0003oool0oooo00@0oooo0P3o0006
0?ooo`030?l0003o0?l0o`0000@0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?l0o`3o
00040?ooo`040?l0o`3o0000o`000?l0o`D0oooo0P3o00060?ooo`030?l0003o0?l0o`0000D0oooo
0P3o00060?ooo`80o`001`3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo
00<0o`000?ooo`3oool0103oool00`3o0?l0o`000?ooo`060?ooo`030?l0003oool0oooo00@0oooo
0P3o00060?ooo`030?l0003o0?l0o`0000D0oooo0P3o00060?ooo`030?l0003oool0o`0000D0oooo
00<0o`000?l0o`3oool02`3oool001X0oooo00<000000?ooo`3oool01`3oool00`3o0000003o0?oo
o`0i0?ooo`050000003o000000000?l0o`3o00000P3oool02@3o0000oooo000000000000oooo0?l0
003oool0o`3o0?l00004000000030?l0003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`00103o
ool01@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?l0o`3oool00`3oool00`3o0000oooo
0?l000060?ooo`80o`005@3oool4000000D0oooo0P0000040?ooo`<0000000<0o`000000003oool0
103oool20?l000H0oooo00<0o`000?l0o`3o0000103oool00`3o0000oooo0?l000050?ooo`040?l0
003oool0oooo0?l000@0oooo00@0o`3o0?l0003o0000o`3o103oool0103o0000oooo0?l0003o0?l5
0?ooo`030?l0003o0?l0o`0000@0oooo00@000000?l0003o000000001@3oool20?l000D0oooo00<0
00000?l0o`3o00001`3oool00`3o0000oooo0?ooo`040?ooo`030?l0o`3o0000oooo00D0oooo0P3o
00060?ooo`030?l0003o0?l0o`0000D0oooo0P3o00000`3o0?l0oooo0?ooo`030?ooo`030?l0003o
0?l0o`0000D0oooo0P3o00050?ooo`040000003o000000000?l000D0oooo00<0o`000?l0o`3oool0
1P3oool300000080oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`000000o`3oool0<`3o
ool00`3o0000oooo0?ooo`020?ooo`060000003oool0o`000?ooo`3o0?l0o`000P3oool02@3o0000
oooo000000000000oooo0?l0003oool0o`3o0?l000020?ooo`050000003oool0o`000?ooo`3o0000
1@3oool0103o0000oooo0?l0o`3o00040?ooo`090?l0003oool0oooo0?ooo`3o0000003o0?ooo`3o
0000o`3o00@0oooo00<0o`000?ooo`3o00001P3oool20?l001@0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`020?ooo`800000103oool01@000000oooo0?l0003o0000000000D0oooo
0P3o00050?ooo`040?l0o`3o0000o`3o0?l000@0oooo00<0o`000?ooo`3o00001@3oool0103o0000
oooo0?ooo`3o00040?ooo`040?l0o`3o0000o`000?l0o`@0oooo00@0o`000?ooo`3oool0o`001@3o
ool00`3o0000o`3o0?l000030?ooo`060000003oool0o`000?l0003oool00000103oool20?l000@0
oooo00D000000?ooo`3o0?l0o`00000000060?ooo`030?l0003oool0oooo00@0oooo0P3o00060?oo
o`80o`001P3oool00`3o0000o`3o0?l000050?ooo`80o`0000<0o`3o0?ooo`3oool00`3oool00`3o
0000o`3o0?l000050?ooo`80o`00103oool01P000000oooo0?l0003oool0o`00000000@0oooo00<0
o`000?l0o`3oool01`3oool00`000000oooo0?ooo`010?ooo`006P3oool00`000000oooo0?ooo`07
0?ooo`030?l000000?l0oooo03<0oooo00<0o`000?ooo`3oool00P3oool02@000000oooo0?l0003o
ool0o`3o0?l0003oool0003o0?l000040?ooo`040?l0003oool0o`3o0?l00080oooo00D000000?oo
o`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00T0o`000?ooo`3oool0oooo0?l0
003oool0003o0?l0003o0?l0103oool00`3o0000oooo0?l000060?ooo`80o`00503oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<0o`000?ooo`3oool00`3oool00`000000
o`000?l000060?ooo`80o`001@3oool0103o0?l0o`000?ooo`3o00040?ooo`040?l0003oool0o`00
0?l0o`@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?l0003o0?l40?ooo`040?l0
003oool0oooo0?l000D0oooo00<0o`000?l0o`3o00000`3oool01P000000oooo0?l0003o0000oooo
000000@0oooo0P3o00060?ooo`80o`0000<000000?ooo`3oool00`3oool00`3o0?l0o`000?ooo`05
0?ooo`030?l0003o0?l0o`0000D0oooo0P3o00060?ooo`030?l0003o0?l0o`0000D0oooo0P3o0000
0`3o0?l0oooo0?ooo`030?ooo`030?l0003o0?l0o`0000D0oooo00<0o`000?l0o`3o00000`3oool0
1P000000oooo0?l0003oool0o`00000000@0oooo00<0o`000?ooo`3oool01`3oool00`000000oooo
0?ooo`010?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030?l000000?l0oooo03<0oooo0P3o
00030?ooo`030000003oool0o`000080oooo00@0o`000000o`3oool0o`00103oool0103o0000oooo
0?l0o`3o00020?ooo`030000003oool0o`3o0080o`001@3oool00`3o0000o`3o0?l000050?ooo`04
0?l0003oool0oooo0?l00080oooo00<0003o0?l0003o0?l0103oool00`3o0000oooo0?l000060?oo
o`80o`00503oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<0o`000?oo
o`3oool0103oool20?l000H0oooo0P3o00050?ooo`040?l0o`3o0000oooo0?l000@0oooo00@0o`00
0?ooo`3o0000o`3o103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0o`000?l0o`@0
oooo00@0o`000?ooo`3oool0o`001@3oool00`3o0000oooo0?l000030?ooo`060000003o0000o`3o
0?l0003oool00000103oool20?l000H0oooo0P3o00000`000000oooo0?ooo`030?ooo`80o`001P3o
ool00`3o0000o`3o0?l000050?ooo`80o`001P3oool00`3o0000o`3o0?l000050?ooo`030?l0003o
ool0o`0000D0oooo00<0o`000?l0o`3o00001@3oool00`3o0000oooo0?l000030?ooo`060000003o
ool0o`000?ooo`3o00000000103oool00`3o0000oooo0?ooo`030?ooo`H000000P3oool001X0oooo
00<000000?ooo`3oool01`3oool00`3o0000003o0?ooo`0c0?ooo`80o`000`3oool00`000000o`00
0?ooo`020?ooo`040?l000000?l0oooo0?l000@0oooo00<0o`3o0?l0003o00000`3oool00`000000
oooo0?ooo`020?l000H0oooo0P3o00050?ooo`040?l0003oool0oooo0?l000<0oooo00<0003o0?l0
003oool00`3oool00`3o0000oooo0?l000060?ooo`80o`001`3oool00`3o0000oooo0?ooo`0:0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`3o0000oooo0?ooo`040?oo
o`80o`001P3oool20?l000D0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?l0003o
0?l40?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3o0000o`3o103oool0103o0000
oooo0?ooo`3o00040?ooo`040?l0003o0?l0oooo0?l000<0oooo00H000000?l0003o0?l0o`000?oo
o`0000040?ooo`030?l0003o0?l0o`0000D0oooo0P3o00060?ooo`80o`0000<0o`3o0?ooo`3oool0
0`3oool00`3o0000oooo0?l000050?ooo`80o`0000<0o`3o0?ooo`3oool00`3oool00`3o0000o`3o
0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?l0o`3o0000103oool0103o0000o`3o
0?ooo`3o00030?ooo`060000003oool0o`000?ooo`3o00000000103oool00`3o0000oooo0?ooo`04
0?ooo`040000003oool0oooo000000<0oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`00
0?ooo`000?l0<`3oool20?l000<0oooo00<000000?l0003oool00P3oool0103o0000oooo0?ooo`3o
00040?ooo`030?l0o`3o0000o`0000<0oooo00<000000?ooo`3oool00P3o00060?ooo`80o`001@3o
ool0103o0000oooo0?ooo`3o00040?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o0000
1P3oool20?l000L0oooo00<0o`000?ooo`3oool02P3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0080oooo0P3o00060?ooo`80o`0000<000000?ooo`3oool00P3oool00`3o0000o`3o
0?l000050?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o
0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00
103oool0103o0000o`3o0?ooo`3o00030?ooo`060000003o0000o`3o0?l0003oool00000103oool0
0`3o0000o`3o0?l000050?ooo`80o`0000<000000?ooo`3oool00`3oool00`3o0000oooo0?l00005
0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l00005
0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o0000103oool0103o0000o`3o0?ooo`3o
00030?ooo`060000003oool0o`000?ooo`3o00000000103oool00`3o0000oooo0?ooo`040?ooo`04
0000003oool0oooo000000<0oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`000?ooo`00
0?l0<`3oool20?l000<0oooo00<000000?l0003oool00P3oool0103o0000oooo0?ooo`3o00040?oo
o`040?l0o`3o0000o`000?ooo`<000000P3oool00`3o0000oooo0?ooo`050?ooo`80o`001@3oool0
103o0000oooo0?ooo`3o00040?ooo`030?l000000?l0oooo0080oooo00@0o`000?ooo`3o0000o`3o
1@3oool20?l000H0oooo0P3o000<0?ooo`060000003oool0oooo0?l0003oool00000103oool20?l0
00@0oooo00D000000?ooo`3o0000o`00000000040?ooo`040?l0003o0?l0o`000?l0o`@0oooo00@0
o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l0
00@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003o
0?l0oooo0?l000<0oooo00H000000?l0003o0?l0o`000?l0o`0000040?ooo`030?l0003o0?l0o`00
00<0oooo00D000000?l0003o0?l0o`00000000050?ooo`030?l0003oool0o`0000D0oooo00<0o`00
0?ooo`3o00001@3oool00`3o0000oooo0?l000040?ooo`040?l0o`3o0000oooo0?l000@0oooo00@0
o`000?l0o`3oool0o`001@3oool00`3o0000oooo0?l000040?ooo`040?l0003o0?l0oooo0?l000<0
oooo00H000000?l0o`3o0000oooo0?l0000000040?ooo`030?l0003oool0oooo00D0oooo00<00000
0?ooo`0000000`3oool001X0oooo0P0000080?ooo`030?l0003oool0003o03<0oooo0P3o00040?oo
o`050?l00000000000000000o`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`030?l0o`3o0000
o`0000<0oooo00@000000?ooo`3oool0o`001`3oool00`3o0000o`3o0?ooo`040?ooo`040?l0003o
ool0oooo0?l000@0oooo00T0o`000000o`3oool0oooo0?l0003o0?l0oooo0?l0003o0?l01@3oool2
0?l000H0oooo0P3o00070?ooo`030?l0003oool0oooo00<0oooo0P0000000`3o000000000?ooo`04
0?ooo`80o`001@3oool00`000000o`000?l000050?ooo`040?l0003oool0o`000?l0o`@0oooo00@0
o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l0
00@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003o
ool0oooo0?l000@0oooo00@0o`000?l0o`3o0000o`3o1@3oool00`3o0000o`3o0?l000040?ooo`03
0?l0003o0?l0o`0000H0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`03
0?l0003oool0o`0000@0oooo00@0o`3o0?l0003oool0o`00103oool0103o0000o`3o0?ooo`3o0005
0?ooo`030?l0003oool0o`0000@0oooo00@0o`000?l0o`3oool0o`00103oool0103o000000000000
003o00050?ooo`030?l0003oool0oooo00H0oooo0P0000030?ooo`006P3oool00`000000oooo0?oo
o`070?ooo`030?l0003oool0003o03<0oooo0P3o00040?ooo`050?l0003oool0oooo0000o`3o0000
0P3oool00`3o0000oooo0?ooo`030?ooo`80o`001P3oool00`3o0000oooo0?ooo`050?ooo`030?l0
003oool0oooo00@0oooo00@0o`000?ooo`3o0000o`3o103oool01@3o0000oooo0000o`3oool0o`00
0080oooo00<0o`000?l0o`3oool00`3oool00`3o0?l0o`000?l000060?ooo`80o`001P3oool20?l0
00L0oooo00<0o`000?ooo`3oool0103oool20?l000H0oooo0P3o00050?ooo`040?l0003oool0o`00
0?l0o`@0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0?l0o`000?ooo`3oool0o`0000@0oooo
00@0o`000?ooo`3oool0o`000`3oool01@3o0000o`3o0?ooo`3oool0o`0000<0oooo00D0o`3o0?l0
003oool0oooo0?l000040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?l0o`3oool0o`00
103oool0103o0000o`3o0?l0o`3o00040?ooo`030?l0003o0?l0o`0000H0oooo00<0o`000?ooo`3o
00001@3oool00`3o0000oooo0?l000040?ooo`040?l0003o0?l0oooo0?l000@0oooo00@0o`3o0?l0
003oool0o`00103oool0103o0000o`3o0?ooo`3o00040?ooo`040?l0003o0?l0oooo0?l000@0oooo
00@0o`000?l0o`3oool0o`00103oool0103o0000oooo0?ooo`3o00050?ooo`030?l0003oool0oooo
00/0oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`000?ooo`000?l0<`3oool20?l000@0
oooo00D0o`000?ooo`000?l0oooo0?l000020?ooo`030?l0003oool0oooo00<0oooo0P3o00060?oo
o`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool20?l000D0oooo00D0o`000?oo
o`3oool0003o0?l000030?ooo`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o00001P3oool2
0?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00050?ooo`040?l0003oool0
oooo0?l000@0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0?l0o`000?ooo`3oool0o`0000@0
oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000o`3o0?ooo`3oool0o`0000<0oooo00D0o`3o
0?l0003oool0oooo0?l000040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0
o`00103oool0103o0000o`3o0?l0o`3o00040?ooo`030?l0003o0?l0o`0000D0oooo00@0o`000?l0
o`3oool0o`001@3oool00`3o0000oooo0?l000040?ooo`040?l0003o0?l0oooo0?l000@0oooo00@0
o`3o0?l0003oool0o`00103oool0103o0000o`3o0?ooo`3o00040?ooo`040?l0003o0?l0oooo0?l0
00@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`030?l0o`3o
0000oooo00`0oooo000J0?ooo`030000003oool0oooo00L0oooo00<0o`000?ooo`000?l0<`3oool2
0?l000@0oooo00P0o`000?ooo`000?l0oooo0?l0003oool0o`3o0?l000D0oooo00<0o`000?l0o`3o
ool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo0P3o00050?ooo`05
0?l0003oool0oooo0?ooo`3o00000`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`00
00H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001@3oool0103o
0000oooo0?ooo`3o00030?ooo`050?l0003o0?l0oooo0?ooo`3o00000`3oool01@3o0000oooo0?oo
o`3oool0o`0000@0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`00
00<0oooo00D0o`3o0?l0003oool0oooo0?l000030?ooo`050?l0003o0?l0oooo0?ooo`3o0000103o
ool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0o`3o0?l000@0oooo00<0o`000?ooo`3o
00001@3oool0103o0000oooo0?ooo`3o00040?ooo`040?l0o`3o0000oooo0?l000@0oooo00@0o`00
0?l0o`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003o0?l0oooo0?l000@0
oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0
oooo0?l000@0oooo00<0o`000?ooo`3oool0303oool001X0oooo00<000000?ooo`3oool01`3oool0
0`3o0?l0o`000000o`030?ooo`030?l0003oool0oooo02d0oooo0P3o00040?ooo`080?l0003oool0
003o0?ooo`3o0000oooo0?l0o`3o00050?ooo`030?l0003o0?l0oooo00D0oooo00<0o`3o0?ooo`3o
ool01@3oool00`3o0000oooo0?ooo`050?ooo`80o`001@3oool01@3o0000oooo0?ooo`3oool0o`00
00<0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?l000060?ooo`80o`001P3oool20?l0
00H0oooo0P3o00060?ooo`80o`001P3oool20?l000D0oooo00@0o`000?ooo`3oool0o`000`3oool0
1@3o0000o`3o0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0
003oool0oooo0000o`3o00000P000?l00`3oool0o`000?ooo`020?ooo`050?l0003oool0oooo0?oo
o`3o00000`3oool01P3o0000oooo0?ooo`3oool0o`000?l0o`80oooo00<0o`000?ooo`3oool00P3o
ool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3o
0?l0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0o`3o0000oooo0?l000@0oooo00@0
o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l0
00@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003o
ool0oooo0?l000@0oooo00<0o`000?ooo`3oool0303oool001X0oooo00<000000?ooo`3oool01`3o
ool0103o0?l0o`000?ooo`000?l20?ooo`030?l0003oool0oooo02d0oooo0P3o00040?ooo`080?l0
00000?l0oooo0?ooo`3o0000oooo0?l0o`3o00050?ooo`030?l0003o0?l0oooo00d0oooo00<0o`00
0?ooo`3oool01@3oool20?l000D0oooo00H0o`000?ooo`3oool0o`000?l0o`000?l20?ooo`030?l0
003oool0oooo0080oooo00@0o`000?ooo`3o0?l0o`001@3oool20?l000H0oooo0P3o00060?ooo`80
o`001P3oool20?l000H0oooo0P3o00050?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?l0
o`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`000?l01P3o0000003o0000
o`3oool0o`000?ooo`80003o00D0o`000000o`000?l0003o0?l000030?ooo`050?l0003oool0oooo
0?ooo`3o00000`3oool01@3o0000o`3o0?ooo`3oool0o`0000@0oooo00@0o`000?ooo`3oool0o`00
103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0o`3o0?l000@0oooo00@0o`000?oo
o`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo
00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo
0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`030?l0
003oool0oooo00`0oooo000J0?ooo`030000003oool0oooo00L0oooo00D0o`3o0?l0003oool0003o
0?ooo`020?l002l0oooo0P3o00040?ooo`070?l000000?l0oooo0?ooo`3o0000oooo0?l000060?oo
o`030?l0003oool0oooo01D0oooo0P3o00050?ooo`090?l0003oool0oooo0?l0003o0?l0oooo0000
o`3oool0o`0000@0oooo00@0o`000?ooo`3o0?l0o`00103oool00`3o0000o`3o0?l000060?ooo`80
o`001P3oool20?l000H0oooo0P3o00050?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3o
ool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`000?l0003o0?l0
00030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0
003o00D0o`000000o`000?l0oooo0?l000030?ooo`050?l0003o0?l0oooo0?ooo`3o0000103oool0
103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0
o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`00
0?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0
oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0
oooo0?l000@0oooo00<0o`000?ooo`3oool0303oool001X0oooo00<000000?ooo`3oool0203oool0
103o0000oooo0000o`3oool20?l002h0oooo00<0o`3o0?l0003o0000103oool01`3o0000oooo0?oo
o`3oool0o`000?ooo`3o00001P3oool00`3o0000oooo0?ooo`0E0?ooo`80o`001@3oool0103o0000
oooo0?ooo`3o00030?ooo`030000o`3o0000oooo0080oooo00D0o`000?l0o`3oool0o`3o0?l00004
0?ooo`030?l0003o0?l0o`0000H0oooo0P3o00060?ooo`80o`001P3oool20?l000D0oooo00<0o`00
0?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00030?ooo`060?l0003oool0oooo0?ooo`3o0000
oooo0P000?l01@3o0000003o0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l00003
0?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`000?l0o`0000<0003o
00D0o`000?ooo`3oool0oooo0?l000040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?oo
o`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo
00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo
0?l000@0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000o`3o0?ooo`3oool0o`0000@0oooo
00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`030?l0003oool0oooo
00`0oooo000B0?oooa<0000000H0o`00000000000?l0o`000?l0o`3o000^000000030?l000000000
o`0000@0000000L0o`0000000000000000000?l000000000o`0000H0000000<0o`00000000000000
5@0000020?l000D0000000@0o`00000000000000o`00100000002@3o00000000000000000000o`00
0?l0o`000000o`3o0?l00004000000030?l0003o0?l0o`0000H000000P3o000600000080o`001@00
00000`3o0000o`3o0?l00005000000030?l000000000o`0000@0000000D0o`3o0?l0000000000000
0?l00003000000060?l00000000000000000o`3o0000003o0P0000001@3o00000000000000000000
o`0000<0000000D0o`0000000000000000000?l00003000000050?l00000000000000000003o0000
0`0000001@3o00000000000000000000o`0000<0000000D0o`000000o`000?l0003o0?l000030000
00050?l0003o0?l000000000003o000010000000103o000000000000003o0004000000040?l00000
000000000?l000@0000000@0o`00000000000000o`0010000000103o000000000000003o00040000
00040?l00000000000000?l000@0000000@0o`00000000000000o`0010000000103o000000000000
003o0003000000050?l0003o0?l000000000003o000010000000103o000000000000003o00040000
00040?l00000000000000?l000@0000000<0o`00000000000000200000040?ooo`006P3oool00`00
0000oooo0?ooo`080?ooo`<0o`0000<0o`3o0?ooo`3o00003`3oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo00`0oooo00<0o`000000003o00000`3oool00`000?l0o`000?ooo`020?oo
o`030?l0003oool0o`0000<0oooo00@000000?ooo`3oool0o`003@3oool00`000000oooo0?ooo`07
0?ooo`030?l0003oool0oooo00<0oooo00D000000?l0003oool0oooo0?l000040?ooo`090?l0003o
ool0oooo0?ooo`3o0000o`3o0?ooo`3o0?l0o`0000@0oooo00<0o`000?l0o`3o00001P3oool20?l0
00D0oooo00<0o`000?l0o`3o00001@3oool00`3o0000o`3o0?l000050?ooo`030?l0003oool0o`00
00@0oooo00D0o`3o0?l0003oool000000?l000030?ooo`050?l000000?l0003o0?ooo`3o00000P3o
ool01P3o0000o`3o0?ooo`3oool000000?l000<0oooo00H0o`000?ooo`3oool0oooo0?l0003o0?l2
0?ooo`060?l0003oool0oooo0000003o0000o`3o0P3oool01@3o0000oooo0?ooo`3oool0o`0000<0
oooo00D0o`000?ooo`3oool000000?l000020000o`030?ooo`3o0000oooo0080oooo00<0o`000?oo
o`3oool00P3oool0103o0000oooo0000003o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0
o`000?ooo`000000o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool000000?l0
00<0oooo00D0o`000?l0o`3oool0oooo0?l000040?ooo`040?l0003oool000000?l000<0oooo00D0
o`000?l0o`3oool0oooo0?l000030?ooo`060?l0003oool0oooo0000003o0?l0o`000`3oool0103o
0000oooo0?ooo`3o00030?ooo`040?l0003oool0oooo000000`0oooo000J0?ooo`030000003oool0
oooo00X0oooo00@0003o0?ooo`3oool0o`00;P3oool00`3o0000oooo0?l000030?ooo`030000o`3o
0000oooo0080oooo00<0o`000?ooo`3o00000`3oool010000000oooo0?ooo`3o0?lG0?ooo`030?l0
003oool0oooo00@0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000003o0?ooo`3oool0o`00
0080oooo00<0o`3o0?l0003oool00`3oool00`3o0000o`3o0?l000060?ooo`80o`001@3oool00`3o
0000o`3o0?l000050?ooo`030?l0003o0?l0o`0000D0oooo00<0o`000?ooo`3o0000103oool01@3o
0?l0o`000?ooo`3oool0o`000080oooo00<0003o0?l0003oool00P3oool01@3o0000oooo0?ooo`3o
0000o`3o00<0oooo00D0o`000?ooo`3oool0o`000?l0o`030?ooo`050?l0003o0?l0oooo0?ooo`3o
00000`3oool01@3o0000o`3o0?ooo`3o0000o`3o00<0oooo00D0o`000?l0o`3oool0oooo0?l00003
0?ooo`060?l0003oool0oooo0000o`3o0000003o0P3oool00`3o0000oooo0?ooo`020?ooo`040?l0
003oool000000?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o0003
0?ooo`050?l0o`3o0000oooo0?ooo`3o0000103oool0103o0000oooo0?ooo`3o00030?ooo`060?l0
003o0?l0oooo0?ooo`3o0000o`3o0`3oool01@3o0000oooo0?ooo`3o0?l0o`000080oooo00D0o`00
0?l0o`3oool0003o0?l000030000o`060?l000000?l0003o0000o`3o0?l0o`000P000?l01@3o0?l0
o`000000o`000?l0o`0000<0003o00@0o`000000o`000?l0003o303oool001X0oooo00<000000?oo
o`3oool02`3oool00`000?l0oooo0?l0000^0?ooo`030?l0003oool0o`0000<0oooo00<0003o0?l0
003oool00P3oool00`3o0000oooo0?l0000N0?ooo`030?l0003oool0oooo00@0oooo00@0o`000?oo
o`3oool0o`00103oool01@3o0000oooo0000o`3oool0o`0000<0oooo00<0o`000?ooo`3oool00P3o
ool00`3o0000oooo0?l000060?ooo`80o`001@3oool00`3o0000o`3o0?l000050?ooo`040?l0003o
ool0o`3o0?l000@0oooo00<0o`000?ooo`3o0000103oool02@3o0000oooo0?ooo`3oool0o`000?oo
o`000?l0oooo0?l000030?ooo`050?l0003oool0oooo0?l0003o0?l00`3oool0103o0?l0o`000?oo
o`3o00040?ooo`050?l0003o0?l0oooo0?l0o`3o00000`3oool01@3o0000o`3o0?ooo`3o0000o`3o
00<0oooo00D0o`000?l0o`3oool0oooo0?l000030?ooo`060?l0003oool0oooo0?ooo`3o0000oooo
0P000?l01P3o0000oooo0?ooo`3oool0o`3o0?l00080oooo00D0o`000?ooo`3oool0oooo0?l00003
0?ooo`060?l0003oool0oooo0?ooo`3o0?l0o`000P3oool01@3o0000oooo0?ooo`3oool0o`0000<0
oooo00<0o`000?ooo`3oool00P3oool01@3o0000oooo0?ooo`3o0?l0o`000080oooo00X0o`000?l0
o`3oool0oooo0?l0003o0?l0oooo0000o`3o0000o`3o0`000?l0103o0000003o0000o`3o00020000
o`030?ooo`3o0000oooo0080oooo00H0o`000?ooo`3oool0oooo0?l0o`3o00020?ooo`050?l0o`3o
0000oooo0?ooo`3o00000`3oool00`3o0000oooo0?ooo`0=0?ooo`006P3oool00`000000oooo0?oo
o`0;0?ooo`030000o`3oool0o`0002h0oooo00<0o`000?ooo`3o00000P3oool00`000?l0o`3o0?l0
00030?ooo`030?l0003oool0o`0001h0oooo00<0o`3o0?ooo`3oool0103oool0103o0000oooo0?oo
o`3o00040?ooo`050?l0003oool0oooo0000o`3o00000`3oool00`3o0000oooo0?ooo`020?ooo`03
0?l0003oool0o`0000D0oooo00<0o`000?l0o`3o00001@3oool00`3o0000o`3o0?l000050?ooo`04
0?l0003oool0o`3o0?l000@0oooo00@0o`000?ooo`3o0000o`3o0`3oool01P3o0000oooo0?ooo`3o
ool0o`000000o`80oooo00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo0080
oooo00@0o`3o0?l0003oool0o`00103oool0103o0000o`3o0?ooo`3o00050?ooo`030?l0003oool0
o`0000@0oooo00D0o`000?l0o`3oool0oooo0?l000030?ooo`050?l0003o0?l0oooo0?ooo`3o0000
0`3oool01@3o0000003o0000o`000?l0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?oo
o`060?l0003oool0oooo0?ooo`3o0?l0o`000P3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo
00<0o`000?ooo`3oool00P3oool01@3o0000oooo0?ooo`3o0?l0o`000080oooo00T0o`000?l0o`00
0?l0003o0?l0003o0?l0003o0?ooo`3o0000103oool0103o0000oooo0?ooo`3o00030?ooo`050?l0
003oool0oooo0?ooo`3o00000`3oool01@3o0?l0o`000?ooo`3oool0o`0000<0oooo00D0o`000?oo
o`3oool0oooo0?l0000?0?ooo`006P3oool00`000000oooo0?ooo`0;0?ooo`040000o`3oool0o`3o
0?l002d0oooo00<0o`000?ooo`3o00000P3oool00`000?l0o`3o0?l000030?ooo`030?l0003oool0
o`0002D0oooo00<0o`000?l0o`3o00001@3oool01@3o0000oooo0?ooo`000?l0o`0000<0oooo00<0
o`000?ooo`3oool00P3oool00`3o0000oooo0?l000050?ooo`040?l0003o0?l0o`3o0?l000@0oooo
00<0o`000?ooo`3o00001@3oool0103o0000oooo0?l0o`3o00030?ooo`050?l0003o0?l0oooo0?oo
o`3o00000`3oool01@3o0000oooo0000o`000?l0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l0
00020?ooo`030?l0003oool0oooo0080oooo00@0o`3o0?l0003oool0o`00103oool0103o0000o`3o
0?ooo`3o00050?ooo`030?l0003oool0o`0000@0oooo00D0o`000?l0o`3oool0oooo0?l000030?oo
o`050?l0003o0?l0oooo0?l0003o0?l00`3oool01@3o0000oooo0?ooo`3oool0o`0000<0003o00D0
o`000000o`000?l0003o0?l000030?ooo`060?l0003oool0oooo0?ooo`3o0?l0o`000P3oool01@3o
0000oooo0?ooo`3oool0o`0000<0oooo00H0o`000?ooo`3oool0oooo0?l0003o0?l20000o`060?l0
o`3o0000003o0000o`3o0000o`3o0P3oool01@3o0000o`3o0?ooo`3oool0o`0000@0oooo00@0o`00
0?ooo`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`0000@0oooo00@0o`000?ooo`3oool0
o`000`3oool01@3o0000o`3o0?ooo`3oool0o`0000l0oooo000J0?ooo`030000003oool0oooo00/0
oooo00@0003o0?ooo`3o0?l0o`00;@3oool01`3o0000oooo0?l0003oool0003o0?ooo`3o0000103o
ool20?l000030?l0o`3oool0oooo02@0oooo0P3o00050?ooo`050?l0003oool0oooo0?ooo`3o0000
0`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3o0?l0
o`00103oool00`3o0000oooo0?l000050?ooo`040?l0003oool0o`3o0?l000<0oooo00D0o`000?oo
o`3oool0oooo0?l000030?ooo`050?l000000?l0oooo0?ooo`3o00000`3oool01@3o0000oooo0?oo
o`3oool0o`000080oooo00<0o`000?ooo`3oool00P3oool0103o0?l0o`000?ooo`3o00040?ooo`04
0?l0003o0?l0oooo0?l000D0oooo00<0o`000?ooo`3o0000103oool0103o0000o`3o0?ooo`3o0004
0?ooo`050?l0003o0?l0oooo0?l0003o0?l00`3oool01@3o0?l0o`000?ooo`3oool0o`0000<0oooo
00D0o`000?ooo`3oool0oooo0?l000030000o`050?l000000?l0003o0000o`3o00000`000?l01@3o
0000003o0000o`000?l0o`0000<0003o00H0o`000000o`000?l0003o0?l0003o0?l20?ooo`050?l0
o`3o0000oooo0?ooo`3o00000`3oool01@3o0000o`3o0?ooo`3oool0o`0000@0oooo00@0o`000?oo
o`3oool0o`000`3oool01@3o0000o`3o0?ooo`3oool0o`0000@0oooo00@0o`000?ooo`3oool0o`00
0`3oool01@3o0000o`3o0?ooo`3oool0o`0000l0oooo000J0?ooo`030000003oool0oooo00/0oooo
00@0003o0?ooo`3o0?l0o`00;@3oool01`3o0000oooo0?l0003oool0003o0?ooo`3o0000103oool2
0?l000030?l0o`3oool0oooo02@0oooo0P3o00050?ooo`060?l0003oool0oooo0?ooo`3o0000003o
0P3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3o0?l0
o`00103oool00`3o0000oooo0?l000050?ooo`040?l0003oool0o`3o0?l000<0oooo00D0o`000?oo
o`3oool0oooo0?l000020?ooo`030000o`3o0000oooo0080oooo00D0o`000?ooo`3oool0o`000?l0
o`030?ooo`040?l0o`3o0000oooo0?l000@0oooo00@0o`3o0?l0003oool0o`00103oool0103o0000
o`3o0?ooo`3o00050?ooo`030?l0003oool0o`0000@0oooo00@0o`000?l0o`3oool0o`001@3oool0
103o0000oooo0?l0003o0?l30?ooo`050?l0o`3o0000oooo0?ooo`3o00000`3oool01@3o0000oooo
0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo
0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`3o0?l0003oool0oooo
0?l000030?ooo`050?l0003o0?l0oooo0?ooo`3o0000103oool0103o0000oooo0?ooo`3o00030?oo
o`050?l0003o0?l0oooo0?ooo`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003o
ool0oooo0?l000l0oooo000J0?ooo`8000003@3oool00`000?l0o`3o0?l0000]0?ooo`070?l0003o
ool0o`000?ooo`000?l0oooo0?l000040?ooo`80o`0000<0o`3o0?ooo`3oool0903oool20?l000D0
oooo00T0o`000?l0o`3oool0o`3o0?l0003oool0003o0?ooo`3o0000103oool00`3o0000oooo0?l0
00050?ooo`040?l0003oool0o`3o0?l000@0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo
0?ooo`3o00030?ooo`050?l0003oool0oooo0?ooo`3o00000P000?l00`3o0?l0o`000?ooo`020?oo
o`050?l0003oool0oooo0?l0003o0?l0103oool00`3o0000oooo0?l000050?ooo`030?l0003oool0
o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000040?ooo`040?l0003o0?l0
oooo0?l000D0oooo00<0o`000?ooo`3o0000103oool01@3o0?l0o`000?ooo`3oool0o`0000<0oooo
00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o
0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0o`3o
0000oooo0?ooo`3o00000`3oool01@3o0000o`3o0?ooo`3oool0o`0000@0oooo00@0o`000?ooo`3o
ool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0
o`000?ooo`3oool0o`003`3oool001X0oooo00<000000?ooo`3oool0303oool00`000?l0o`3o0?l0
000]0?ooo`040?l0003oool0oooo0?l00080oooo00<0o`000?ooo`3oool00P3oool20?l000030?l0
o`3oool0oooo02@0oooo00<0o`000?l0o`3oool0103oool0103o0000o`3o0?ooo`3o00030?ooo`03
0000o`3o0000oooo0080oooo00D0o`000?l0o`3oool0o`3o0?l000040?ooo`040?l0003oool0o`3o
0?l000@0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?ooo`3o00030?ooo`050?l0003o
ool0oooo0000o`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`040?l0003oool0oooo0?l000D0
oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003o0?l0o`0000D0
oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000@0
oooo00D0o`3o0?l0003oool0oooo0?l000030?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool0
1@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0
003oool0oooo0?ooo`3o00000`3oool01@3o0?l0o`000?ooo`3oool0o`0000@0oooo00@0o`000?oo
o`3o0000o`3o103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0o`3o0?l000@0oooo
00<0o`000?ooo`3o00001@3oool0103o0000oooo0?l0o`3o000?0?ooo`006P3oool00`000000oooo
0?ooo`0<0?ooo`030000o`3oool0o`0002H0oooo00<0o`000?ooo`3oool0103oool0103o0000oooo
0?ooo`3o00020?ooo`030?l0003oool0oooo0080oooo00<0o`3o0?l0003o0?l09P3oool00`3o0000
o`3o0?ooo`040?ooo`040?l0003o0?l0oooo0?l000@0oooo00D0o`000?ooo`3oool0oooo0?l00002
0?ooo`030?l0o`3o0000oooo0080oooo00D0o`000?l0o`3oool0o`3o0?l000040?ooo`030?l0003o
ool0o`0000D0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000oooo0000o`3oool0o`000080
oooo00<0o`000?ooo`3oool00P3oool0103o0000oooo0?ooo`3o00050?ooo`030?l0003oool0o`00
00D0oooo00<0o`000?l0o`3o00001@3oool20?l000H0oooo00<0o`000?ooo`3o00001@3oool00`3o
0000oooo0?l000050?ooo`030?l0003oool0o`0000@0oooo00@0o`3o0?l0003oool0o`00103oool0
1@3o0000oooo0?ooo`3oool0o`0000@0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000oooo
0?ooo`3oool0o`0000<0oooo00D0o`3o0?l0003oool0oooo0?l000030?ooo`050?l0o`3o0000oooo
0?ooo`3o0000103oool0103o0000oooo0?l0003o0?l40?ooo`040?l0003oool0o`3o0?l000@0oooo
00@0o`000?ooo`3o0?l0o`00103oool00`3o0000oooo0?l000050?ooo`040?l0003oool0o`3o0?l0
00l0oooo000J0?ooo`030000003oool0oooo00`0oooo00<0003o0?ooo`3o00009P3oool00`3o0000
oooo0?ooo`040?ooo`040?l0003oool0oooo0?l00080oooo00<0o`000?ooo`3oool00`3oool00`3o
0000oooo0?ooo`0U0?ooo`030?l0003oool0oooo00@0oooo00@0o`000?l0o`3oool0o`00103oool0
1@3o0000003o0?ooo`3oool0o`0000<0oooo00T0o`000?ooo`3oool0oooo0?l0003o0?l0oooo0?l0
o`3o0000103oool00`3o0000oooo0?l000040?ooo`050?l0o`3o0000oooo0?ooo`3o00000`3oool0
1@3o0000003o0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool00P3oool0103o0000oooo0?oo
o`3o00050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?l0o`3o00001@3oool20?l000H0oooo
00<0o`000?l0o`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000@0oooo
00@0o`3o0?l0003oool0o`00103oool01@3o0000oooo0?ooo`3oool0o`0000@0oooo00@0o`000?oo
o`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00D0o`3o0?l0003oool0o`00
0?l0o`030?ooo`050?l0o`3o0000oooo0?ooo`3o0000103oool0103o0000oooo0?l0003o0?l40?oo
o`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3o0?l0o`00103oool00`3o0000oooo0?l00005
0?ooo`040?l0003oool0o`3o0?l000l0oooo000J0?ooo`030000003oool0oooo00`0oooo00<0003o
0?ooo`3o00009P3oool00`3o0000oooo0?ooo`040?ooo`040?l0003oool0003o0?l00080oooo00<0
o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`0U0?ooo`030?l0003oool0oooo00@0oooo00@0
o`000?l0o`3oool0o`00103oool01@3o0000oooo0000o`3oool0o`0000<0oooo00T0o`000?ooo`3o
ool0oooo0?l0003o0?l0oooo0?l0o`3o00000`3oool01@3o0?l0o`000?ooo`3o0?l0o`0000<0oooo
00D0o`3o0?l0003oool0oooo0?l000020?ooo`030000o`3o0000oooo0080oooo00@0o`000?ooo`3o
ool0o`00103oool0103o0000oooo0?ooo`3o00050?ooo`030?l0003o0?l0o`0000D0oooo00<0o`00
0?l0o`3o00001@3oool20?l000H0oooo00<0o`000?l0o`3o00001@3oool20?l000H0oooo00<0o`00
0?ooo`3o0000103oool0103o0?l0o`000?ooo`3o00040?ooo`050?l0003o0?l0oooo0?ooo`3o0000
103oool0103o0000oooo0?l0003o0?l30?ooo`050?l0003oool0oooo0?l0o`3o00000`3oool01@3o
0?l0o`000?ooo`3o0000o`3o00<0oooo00D0o`3o0?l0003oool0oooo0?l000040?ooo`030?l0003o
ool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool0103o0000oooo0?l0o`3o00040?ooo`030?l0
003oool0o`0000D0oooo00<0o`000?ooo`3o0000403oool001X0oooo00<000000?ooo`3oool03@3o
ool00`000?l0o`000?ooo`0U0?ooo`030?l0003oool0oooo00@0oooo00L0o`000?ooo`000?l0o`00
0?ooo`3o0000o`3o00D0oooo00<0o`000?ooo`3oool09@3oool00`3o0000oooo0?ooo`040?ooo`04
0?l0003o0?l0o`000?l0o`@0oooo00D0o`000?ooo`3oool0003o0?l000030?ooo`050?l0003oool0
oooo0?ooo`3o00000P3oool00`3o0?l0o`000?ooo`020?ooo`050?l0003oool0oooo0?l0o`3o0000
0`3oool01@3o0000oooo0?ooo`3oool0o`000080003o00<0oooo0?l0003oool00P3oool0103o0000
oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000D0oooo0P3o00060?ooo`80o`001P3oool2
0?l000H0oooo0P3o00060?ooo`80o`001P3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0
o`0000@0oooo00D0o`000?l0o`3oool0o`3o0?l000040?ooo`030?l0003oool0o`0000@0oooo00D0
o`000?l0o`3oool0o`3o0?l000030?ooo`050?l0o`3o0000oooo0?l0003o0?l00`3oool01@3o0?l0
o`000?ooo`3o0000o`3o00@0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?oo
o`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l0000@0?oo
o`006P3oool00`000000oooo0?ooo`0=0?ooo`030000o`3o0000oooo02D0oooo00<0o`000?ooo`3o
ool0103oool01P3o0000oooo0000o`3o0000oooo0?l000H0oooo00<0o`000?ooo`3oool09@3oool0
0`3o0000oooo0?ooo`040?ooo`030?l0003o0?l0o`0000D0oooo00H0o`000?ooo`3oool0oooo0?l0
00000?l20?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0?l0o`000?ooo`020?ooo`05
0?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`000?l0o`000080oooo00<0o`3o
0?l0003oool00P3oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0o`000?l0o`D0oooo
0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool00`3o0000oooo
0?l000050?ooo`030?l0003oool0o`0000@0oooo00@0o`000?l0o`3oool0o`001@3oool00`3o0000
oooo0?l000040?ooo`050?l0003o0?l0oooo0?l0o`3o0000103oool0103o0000oooo0?l0003o0?l4
0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l00005
0?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l0000@
0?ooo`006P3oool00`000000oooo0?ooo`0=0?ooo`030000o`3o0000oooo02D0oooo0P3o00040?oo
o`070?l0003o0?l0003o0?ooo`3o0000oooo0?l000060?ooo`030?l0003oool0oooo02`0oooo00<0
o`000?l0o`3o00001@3oool01P3o0000oooo0?ooo`3o0000o`3o0?ooo`80003o00D0o`000?ooo`3o
ool0oooo0?l000020?ooo`030?l0o`3o0000oooo0080oooo00D0o`000?ooo`3oool0oooo0?l00003
0?ooo`050?l000000?l0003o0?ooo`3o00000P3oool00`3o0?l0o`000?ooo`020?ooo`040?l0003o
ool0oooo0?l000@0oooo00@0o`000?ooo`3o0000o`3o1@3oool20?l000H0oooo0P3o00060?ooo`80
o`001P3oool20?l000H0oooo0P3o00060?ooo`030?l0003o0?l0o`0000D0oooo00<0o`000?ooo`3o
00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000@0oooo00@0o`000?l0o`3o
ool0o`001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?oo
o`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?oo
o`3o00001@3oool00`3o0000oooo0?l0000@0?ooo`006P3oool00`000000oooo0?ooo`0=0?ooo`03
0000o`3o0000oooo02D0oooo0P3o00040?ooo`070?l0003o0?l0003o0?ooo`3o0000oooo0?l00006
0?ooo`030?l0003oool0oooo02`0oooo00<0o`000?l0o`3o00001@3oool0103o0000oooo0?ooo`3o
00040?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0?l0o`000?ooo`020?ooo`050?l0
003oool0oooo0?ooo`3o00000P3oool00`000?l0o`000?ooo`020?ooo`050?l0003oool0oooo0?l0
o`3o00000`3oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003o0?l0o`000?l0o`D0oooo0P3o
00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo00<0
o`000?l0o`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000@0oooo00@0
o`000?l0o`3oool0o`001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo
00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003o0?l0o`0000D0oooo
00<0o`000?l0o`3o00001@3oool00`3o0000o`3o0?l0000@0?ooo`006P3oool2000000h0oooo00<0
003o0?l0003oool09@3oool20?l000@0oooo00L0o`000?l0o`000?l0oooo0?l0003oool0o`0003H0
oooo0P3o00050?ooo`040?l0003oool0oooo0?l000@0oooo00D0o`000000o`000?l0003o0?l00002
0?ooo`030?l0o`3o0000oooo0080oooo00D0o`000?ooo`3oool0003o0?l000020000o`030?ooo`3o
0000oooo0080oooo00@0o`000?ooo`3oool0o`00103oool0103o0000o`3o0?ooo`3o00050?ooo`80
o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80
o`001P3oool00`3o0000o`3o0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o
00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o
00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003o0?l0o`0000D0oooo00<0o`000?l0o`3o
00001@3oool00`3o0000o`3o0?l000050?ooo`030?l0003o0?l0o`000100oooo000J0?ooo`030000
003oool0oooo00h0oooo00<0o`3o0?l0003oool0903oool20?l000@0oooo00L0o`000000o`3oool0
oooo0?l0003oool0o`0003H0oooo0P3o00050?ooo`040?l0003oool0oooo0?l000@0oooo00D0o`00
0?ooo`3oool0oooo0?l000030000o`050?l000000?l0003o0000o`3o00000P000?l00`3oool0o`00
0?ooo`020?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`04
0?l0003o0?l0oooo0?l000D0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo00<0o`000?oo
o`3oool01@3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool00`3o0000o`3o0?l000050?oo
o`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?oo
o`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?oo
o`030?l0003o0?l0o`0000D0oooo0P3o00060?ooo`030?l0003o0?l0o`0000D0oooo0P3o000A0?oo
o`006P3oool00`000000oooo0?ooo`0>0?ooo`030?l0o`3o0000oooo02@0oooo0P3o00040?ooo`07
0?l000000?l0oooo0?ooo`3o0000oooo0?l0000f0?ooo`80o`001@3oool0103o0000oooo0?ooo`3o
00040?ooo`050?l0003oool0oooo0?l0o`3o00000`3oool01@3o0000oooo0?ooo`3o0000o`3o00<0
oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003o0?l0
o`3o0?l000D0oooo0P3o00060?ooo`80o`001P3oool00`3o0000oooo0?ooo`050?ooo`030?l0003o
ool0oooo00D0oooo00<0o`000?l0o`3oool01@3oool20?l000H0oooo0P3o00060?ooo`030?l0003o
0?l0o`0000D0oooo0P3o00000`3o0?l0oooo0?ooo`030?ooo`030?l0003oool0o`0000D0oooo00<0
o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0oooo00<0
o`000?l0o`3o00001@3oool00`3o0000o`3o0?l000050?ooo`80o`001P3oool00`3o0000o`3o0?l0
00050?ooo`80o`004@3oool001X0oooo00<000000?ooo`3oool03P3oool00`3o0?l0o`000?ooo`0T
0?ooo`80o`00103oool01`3o0000003o0?ooo`3oool0o`000?ooo`3o0000=P3oool20?l000D0oooo
00@0o`3o0?l0003oool0o`00103oool01@3o0000oooo0?ooo`3o0?l0o`0000<0oooo00@0o`000?oo
o`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo
00@0o`000?l0o`3o0?l0o`001@3oool20?l000H0oooo0P3o00060?ooo`030?l0003oool0oooo00D0
oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`80o`001P3oool20?l000H0
oooo0P3o00060?ooo`80o`0000<0o`3o0?ooo`3oool00`3oool00`3o0000oooo0?l000050?ooo`80
o`0000<0o`3o0?ooo`3oool00`3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000D0
oooo0P3o00060?ooo`030?l0003o0?l0o`0000D0oooo0P3o00060?ooo`030?l0003o0?l0o`0000D0
oooo0P3o000A0?ooo`006P3oool00`000000oooo0?ooo`0>0?ooo`030?l0o`3o0000oooo02@0oooo
0P3o00040?ooo`070?l0003oool0oooo0?ooo`3o0000oooo0?l0000f0?ooo`80o`001@3oool0103o
0?l0o`000?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00
103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00<0o`000?l0
o`3o00001P3oool20?l000H0oooo0P3o00060?ooo`030?l0003oool0oooo00D0oooo00<0o`3o0?oo
o`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`80o`001P3oool20?l000H0oooo0P3o00060?oo
o`80o`0000<0o`3o0?ooo`3oool00`3oool00`3o0000oooo0?l000050?ooo`80o`0000<0o`3o0?oo
o`3oool00`3oool00`3o0000oooo0?l000050?ooo`030?l0003o0?l0o`0000D0oooo0P3o00060?oo
o`030?l0003o0?l0o`0000D0oooo0P3o00060?ooo`80o`001P3oool20?l00140oooo000J0?ooo`03
0000003oool0oooo00h0oooo00<0o`3o0?l0003oool08`3oool00`3o0?l0o`000?l000040?ooo`07
0?l0003oool0oooo0?ooo`3o0000oooo0?l0000f0?ooo`030?l0003oool0oooo00@0oooo00@0o`3o
0?l0003oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0
oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00050?ooo`80o`001P3oool2
0?l000H0oooo00<0o`000?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`0=0?ooo`030?l0o`3oool0
oooo00D0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo0P3o00060?ooo`80o`0000<0o`3o
0?ooo`3oool00`3oool00`3o0000oooo0?l000050?ooo`80o`0000<0o`3o0?ooo`3oool00`3oool0
0`3o0000o`3o0?l000050?ooo`030?l0003o0?l0o`0000D0oooo0P3o00060?ooo`80o`001P3oool2
0?l000H0oooo0P3o00060?ooo`030?l0o`3o0000oooo0100oooo000J0?ooo`030000003oool0oooo
00h0oooo00<0o`3o0?l0003oool08`3oool00`3o0?l0o`000?l000040?ooo`070?l0003oool0oooo
0?ooo`3o0000oooo0?l0000f0?ooo`030?l0003oool0oooo00@0oooo00@0o`3o0?l0003oool0o`00
103oool0103o0?l0o`000?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?oo
o`3oool0o`00103oool0103o0000oooo0?l0003o0?l50?ooo`80o`001P3oool20?l000H0oooo00<0
o`000?ooo`3oool07@3oool00`3o0000oooo0?ooo`050?ooo`80o`001P3oool20?l000H0oooo0P3o
00060?ooo`80o`0000<0o`3o0?ooo`3oool00`3oool20?l000H0oooo0P3o00060?ooo`030?l0003o
0?l0o`0000D0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`030?l0o`3o
0000oooo0100oooo000J0?ooo`030000003oool0oooo00h0oooo00<0o`3o0?l0003oool08`3oool0
0`3o0?l0o`000?l000030?ooo`030?l0003o0?l0oooo0080oooo00<0o`000?l0o`3o0000=P3oool0
0`3o0000oooo0?ooo`040?ooo`040?l0o`3o0000o`000?l0o`@0oooo00@0o`3o0?l0003oool0o`00
103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?oo
o`3o0000o`3o1@3oool20?l000H0oooo00<0o`000?l0o`3oool01@3oool00`3o0000oooo0?ooo`0M
0?ooo`030?l0o`3oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo0P3o0006
0?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o0006
0?ooo`80o`001`3oool00`3o0000oooo0?ooo`040?ooo`030?l0o`3o0000oooo0100oooo000J0?oo
o`8000003`3oool00`3o0?l0o`000?ooo`0S0?ooo`030?l0o`3o0000o`0000<0oooo00<0o`000?l0
o`3oool00P3oool00`3o0000o`3o0?l0000f0?ooo`030?l0o`3oool0oooo00D0oooo0P3o00050?oo
o`040?l0o`3o0000oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0?l0o`000?oo
o`3o00040?ooo`030?l0003oool0o`0000H0oooo0P3o00060?ooo`030?l0003o0?l0oooo00D0oooo
00<0o`3o0?ooo`3oool09@3oool00`3o0000oooo0?ooo`050?ooo`80o`001P3oool20?l000H0oooo
0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo
00<0o`3o0?l0003oool01P3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00l0oooo
000J0?ooo`030000003oool0oooo00l0oooo00<0o`000?ooo`3oool08P3oool00`3o0?l0o`000?l0
00030?ooo`030?l0003o0?l0oooo0080oooo00<0o`000?l0o`3o0000?P3oool20?l000D0oooo00@0
o`3o0?l0003oool0o`00103oool0103o0000oooo0?ooo`3o00050?ooo`030?l0003oool0o`0000@0
oooo00<0o`3o0?l0003o00001P3oool20?l000H0oooo00<0o`000?ooo`3oool0;@3oool00`3o0?l0
oooo0?ooo`050?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0
oooo0P3o00060?ooo`80o`001P3oool00`3o0?l0o`000?ooo`060?ooo`030?l0003oool0oooo00D0
oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`0?0?ooo`006P3oool00`000000oooo
0?ooo`0?0?ooo`030?l0003oool0oooo0280oooo00<0o`000?ooo`3o00000P3oool00`000?l0o`00
0?l0o`030?ooo`030?l0003o0?l0o`0003h0oooo0P3o00050?ooo`040?l0o`3o0000o`000?l0o`@0
oooo00@0o`000?ooo`3oool0o`001@3oool00`3o0000oooo0?l000040?ooo`030?l0o`3o0000o`00
00H0oooo00<0o`000?l0o`3oool01@3oool00`3o0000oooo0?ooo`0e0?ooo`030?l0003oool0oooo
00D0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l0
00H0oooo00<0o`3o0?l0003oool01P3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo
00D0oooo00<0o`3o0?ooo`3oool03`3oool001X0oooo00<000000?ooo`3oool03`3oool00`3o0000
003o0?ooo`0R0?ooo`030?l0003oool0o`000080oooo00<0003o0?l0003o0?l00`3oool00`3o0000
o`3o0?l0000n0?ooo`030?l0003o0?l0oooo00@0oooo00@0o`3o0?l0003o0000o`3o103oool0103o
0000oooo0?ooo`3o00050?ooo`030?l0003oool0o`0000@0oooo00<0o`3o0?l0003o00001P3oool0
0`3o0000o`3o0?ooo`050?ooo`030?l0o`3oool0oooo03D0oooo00<0o`3o0?ooo`3oool01@3oool2
0?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`030?l0o`3o0000
oooo00H0oooo00<0o`3o0?ooo`3oool09`3oool001X0oooo00<000000?ooo`3oool03`3oool00`3o
0000003o0?ooo`030?ooo`030?l0003oool0oooo01`0oooo00<0o`000?ooo`3o00000P3oool00`00
0?l0o`000?ooo`030?ooo`80o`00?`3oool00`3o0000o`3o0?ooo`040?ooo`030?l0o`3o0000o`00
00D0oooo00@0o`3o0?l0003oool0o`001@3oool00`3o0000oooo0?l000050?ooo`80o`001P3oool0
0`3o0000oooo0?ooo`050?ooo`030?l0o`3oool0oooo03d0oooo00<0o`000?ooo`3oool01@3oool2
0?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000L0oooo00<0o`3o0?ooo`3oool0;`3oool0
00D00000103oool2000000@0oooo1@0000060?ooo`030000003oool0oooo00l0oooo00<0o`000000
o`3oool00`3oool00`3o0000oooo0?ooo`0L0?ooo`030?l0003oool0o`000080oooo00<0003o0?l0
003oool00`3oool20?l003l0oooo00<0o`000?l0o`3oool01@3oool20?l000H0oooo0P3o00060?oo
o`030?l0003o0?l0o`0000D0oooo0P3o00060?ooo`030?l0003oool0oooo04D0oooo00<0o`3o0?oo
o`3oool01@3oool00`3o0000o`3o0?ooo`050?ooo`80o`001P3oool20?l000H0oooo0P3o0?li0?oo
o`000P3oool00`000000oooo0?ooo`040?ooo`800000103oool01@000000oooo0?ooo`3oool00000
00H0oooo00<000000?ooo`3oool03`3oool00`3o0000003o0?ooo`030?ooo`030?l0003oool0oooo
01D0oooo00<0o`000?ooo`3oool0103oool01`3o0000oooo0?l0003oool0003o0?ooo`3o00001@3o
ool00`3o0000oooo0?ooo`0m0?ooo`030?l0003o0?l0oooo00D0oooo0P3o00060?ooo`80o`001P3o
ool20?l000H0oooo0P3o00060?ooo`030?l0003oool0oooo04d0oooo00<0o`3o0?ooo`3oool01@3o
ool20?l0o`H0oooo0P3o0?m10?ooo`000P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool03`3oool00`3o0000003o0?ooo`030?ooo`030?l0003oool0
oooo01D0oooo00<0o`000?ooo`3oool0103oool01`3o0000oooo0?l0003o0?l0003o0?ooo`3o0000
1@3oool00`3o0000oooo0?ooo`0m0?ooo`030?l0003o0?l0oooo00D0oooo0P3o00060?ooo`80o`00
1P3oool20?l000H0oooo0P3o00060?ooo`030?l0003oool0oooo0:00oooo00020?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool01P3oool3000000l0oooo00<0o`000?ooo`000?l00P3o
ool20?l001L0oooo00<0o`000?ooo`3oool0103oool01`3o0000oooo0?l0003o0?l0003o0?ooo`3o
00001@3oool00`3o0000oooo0?ooo`0m0?ooo`030?l0003oool0oooo00D0oooo0P3o00060?ooo`80
o`001P3oool20?l000H0oooo0P3o00060?ooo`030?l0o`3oool0oooo0:00oooo00020?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`030?l0
003oool0003o0080oooo0P3o000G0?ooo`030?l0003oool0oooo00@0oooo00@0o`000?ooo`3o0000
o`3o0P3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0oooo03d0oooo00<0o`3o0?ooo`3o
ool01@3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool00`3o0000o`3o0?ooo`050?ooo`03
0?l0o`3oool0oooo0:00oooo00020?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0?0?ooo`030?l0003oool0003o0080oooo00<0o`000?l0o`3o0000
5P3oool00`3o0000oooo0?ooo`030?ooo`050?l0003o0?l0oooo0?l0003o0?l00P3oool00`3o0000
oooo0?ooo`030?ooo`030?l0003oool0oooo03d0oooo00<0o`3o0?ooo`3oool01@3oool20?l000H0
oooo0P3o00060?ooo`80o`001P3oool00`3o0000oooo0?ooo`2X0?ooo`000`00000<0?ooo`050000
003oool0oooo0?ooo`0000001P3oool00`000000oooo0?ooo`0?0?ooo`030?l0003oool0003o0080
oooo00<0o`000?ooo`3o00005P3oool00`3o0000oooo0?ooo`030?ooo`050?l0003o0?l0oooo0?l0
003o0?l00P3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0oooo04D0oooo00<0o`000?l0
o`3oool01@3oool20?l000H0oooo00<0o`000?l0o`3oool01@3oool00`3o0000oooo0?ooo`2X0?oo
o`000P3oool00`000000oooo0?ooo`0;0?ooo`<000001`3oool00`000000oooo0?ooo`0?0?ooo`03
0?l0003oool0003o0080oooo00<0o`000?ooo`3o00005P3oool00`3o0000oooo0?ooo`030?ooo`05
0?l0003o0?l0oooo0?l0003o0?l00P3oool00`3o0000oooo0?ooo`030?ooo`030?l0o`3oool0oooo
04D0oooo00<0o`000?l0o`3oool01@3oool20?l000H0oooo00<0o`000?l0o`3oool01@3oool00`3o
0000oooo0?ooo`2X0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`080?l0003oool0oooo0000
o`3o0000o`3o0?ooo`3o000F0?ooo`030?l0003oool0oooo00<0oooo00D0o`000?l0o`3oool0003o
0?l000020?ooo`030?l0003oool0oooo04/0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo
00<0o`000?l0o`3oool01@3oool00`3o0?l0oooo0?ooo`2X0?ooo`006P3oool00`000000oooo0?oo
o`0?0?ooo`050?l0003oool0oooo0000o`3o00000P3oool00`3o0000oooo0?ooo`0D0?ooo`030?l0
003oool0oooo00<0oooo00P0o`000?l0o`3oool0003o0?l0003oool0o`3o0?l004d0oooo00<0o`00
0?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`3o
0?ooo`3oool0Z03oool001X0oooo00<000000?ooo`3oool03`3oool01@3o0000oooo0?ooo`3o0000
o`3o0080oooo00<0o`000?ooo`3oool0503oool20?l000@0oooo00P0o`000?ooo`000?l0oooo0?l0
003oool0o`3o0?l004d0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`03
0?l0003oool0oooo0;00oooo000J0?ooo`030000003oool0oooo00l0oooo00@0o`000?ooo`3oool0
o`000`3oool00`3o0000oooo0?ooo`0C0?ooo`030?l0003o0?l0o`0000@0oooo00L0o`000?ooo`00
0?l0oooo0?l0003oool0o`0004h0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0000oooo0?ooo`05
0?ooo`030?l0003oool0oooo0;00oooo000J0?ooo`800000403oool01@3o0000oooo0?ooo`3o0000
003o0080oooo00<0o`000?ooo`3oool04`3oool00`3o0000o`3o0?l000040?ooo`070?l0003oool0
003o0?ooo`3o0000oooo0?l0001F0?ooo`030?l0003oool0oooo00D0oooo00<0o`3o0?ooo`3oool0
/03oool001X0oooo00<000000?ooo`3oool03`3oool01@3o0?l0o`000?ooo`3o0000003o0080oooo
00<0o`000?ooo`3oool04`3oool00`3o0000o`3o0?l000040?ooo`070?l000000?l0oooo0?ooo`3o
0000oooo0?l0001F0?ooo`030?l0o`3oool0oooo0;P0oooo000J0?ooo`030000003oool0oooo00l0
oooo00D0o`3o0?l0003o0000oooo0000o`020?ooo`030?l0003oool0oooo01<0oooo00<0o`000?oo
o`3o0000103oool01`3o0000003o0?ooo`3oool0o`000?ooo`3o0000o`3ooolB0?ooo`006P3oool0
0`000000oooo0?ooo`0@0?ooo`80o`0000D0oooo0000o`3oool0oooo0?l000060?ooo`030?l0003o
ool0oooo00D0oooo00<0o`000?ooo`3oool0103oool00`3o0000oooo0?l000040?ooo`070?l00000
0?l0oooo0?ooo`3o0000oooo0?l0003o0?oooa80oooo000J0?ooo`030000003oool0oooo0100oooo
00L0o`000?l0o`3oool0oooo0000o`3oool0o`0000H0oooo00<0o`000?ooo`3oool01@3oool00`3o
0000oooo0?ooo`040?ooo`030?l0003oool0o`0000@0oooo00L0o`000?ooo`3oool0oooo0?l0003o
ool0o`000?l0oooo4P3oool001X0oooo00<000000?ooo`3oool0403oool01`3o0?l0oooo0?ooo`3o
ool0003o0?ooo`3o00001P3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00@0oooo
00<0o`000?ooo`3o0000103oool01`3o0000oooo0?ooo`3oool0o`000?ooo`3o0000o`3ooolB0?oo
o`006P3oool00`000000oooo0?ooo`0@0?ooo`070?l0o`3oool0oooo0?ooo`000?l0oooo0?l00006
0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0103oool00`3o0000oooo0?l00004
0?ooo`070?l0003oool0oooo0?ooo`3o0000oooo0?l0003o0?oooa80oooo000J0?ooo`030000003o
ool0oooo01@0oooo00<0003o0?ooo`3o00001P3oool00`3o0000oooo0?ooo`050?ooo`030?l0003o
ool0oooo00@0oooo00<0o`000?ooo`3o00000`3oool00`000?l0o`000?ooo`020?ooo`80o`00o`3o
oolC0?ooo`006P3oool2000001H0oooo00<0003o0?l0003oool01@3oool00`3o0000oooo0?ooo`05
0?ooo`030?l0003oool0oooo00@0oooo00<0o`000?ooo`3o00000`3oool00`000?l0o`000?ooo`02
0?ooo`80o`00o`3ooolC0?ooo`006P3oool00`000000oooo0?ooo`0E0?ooo`030000o`3o0000oooo
00D0oooo00<0o`000?ooo`3oool01@3oool20?l000D0oooo00<0o`000?ooo`3o00000P3oool00`00
0?l0o`000?ooo`030?ooo`80o`00o`3ooolC0?ooo`006P3oool00`000000oooo0?ooo`0E0?ooo`03
0000o`3o0000oooo00@0oooo0P3o00070?ooo`80o`001@3oool01`3o0000oooo0?l0003o0?l0oooo
0000o`3o0000103oool20?l00?l0oooo4`3oool001X0oooo00<000000?ooo`3oool05P3oool00`3o
0000oooo0?ooo`030?ooo`80o`0000<0o`3o0?ooo`3oool0103oool20?l000D0oooo00L0o`000?oo
o`3o0000o`3o0000o`3oool0o`0000D0oooo00<0o`000?ooo`3oool0o`3ooolA0?ooo`006P3oool0
0`000000oooo0?ooo`0F0?ooo`030?l0003oool0oooo00<0oooo00<0o`000?ooo`3o00001P3oool2
0?l000D0oooo00L0o`000?ooo`3oool0o`000000o`3oool0o`0000D0oooo00<0o`000?ooo`3oool0
o`3ooolA0?ooo`006P3oool00`000000oooo0?ooo`0F0?ooo`030?l0003oool0oooo00<0oooo00<0
o`000?ooo`3o00001P3oool20?l000D0oooo00@0o`000?ooo`3oool0o`000P3oool00`3o0000oooo
0?ooo`030?ooo`030?l0003oool0oooo0?l0oooo4@3oool001X0oooo00<000000?ooo`3oool05P3o
ool00`3o0000003o0?ooo`030?ooo`030?l0003oool0o`0000H0oooo0P3o00050?ooo`040?l0003o
ool0oooo0?l00080oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`3o0?oooa40oooo
000J0?ooo`030000003oool0oooo01H0oooo00<0o`000000o`3oool00P3oool0103o0000o`3o0?oo
o`3o00050?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`000P3oool00`3o0000
oooo0?ooo`030?ooo`030?l0003oool0oooo0?l0oooo4@3oool001X0oooo0P00000G0?ooo`030?l0
00000?l0oooo0080oooo00@0o`000?l0o`3oool0o`001@3oool00`3o0000oooo0?l000040?ooo`05
0?l0003o0?l0oooo0000o`3o00000P3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0oooo
0?l0oooo4@3oool001X0oooo00<000000?ooo`3oool05P3oool00`3o0?l0o`000?ooo`020?ooo`04
0?l0003o0?l0oooo0?l000D0oooo00<0o`000?ooo`3o0000103oool0203o0000o`3o0?ooo`000?l0
o`000?ooo`3o0000o`3o1@3oool00`3o0000oooo0?ooo`3o0?oooa40oooo000J0?ooo`030000003o
ool0oooo01H0oooo00<0o`3o0?l000000?l00P3oool0103o0000oooo0?ooo`3o00050?ooo`030?l0
003oool0o`0000@0oooo00P0o`000?ooo`000?l0oooo0?l0003oool0o`000?l0o`D0oooo00<0o`3o
0?ooo`3oool0o`3ooolA0?ooo`006P3oool00`000000oooo0?ooo`0G0?ooo`050?l000000?l0oooo
0?ooo`3o00000P3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0o`0000@0oooo00L0o`00
0?ooo`000?l0oooo0?l0003oool0o`0000H0oooo00<0o`3o0?ooo`3oool0o`3ooolA0?ooo`006P3o
ool00`000000oooo0?ooo`0G0?ooo`050?l000000?l0oooo0?ooo`3o00000P3oool00`3o0000oooo
0?ooo`030?ooo`030?l0003oool0o`0000@0oooo00L0o`000000o`3oool0oooo0?l0003oool0o`00
0?l0oooo6P3oool001X0oooo00<000000?ooo`3oool05`3oool01@3o0000003o0?ooo`3oool0o`00
0080oooo00<0o`000?ooo`3oool00P3oool0103o0000o`3o0?ooo`3o00040?ooo`070?l000000?l0
oooo0?ooo`3o0000oooo0?l0003o0?oooaX0oooo0005000000@0oooo0P0000070?ooo`<000001@3o
ool00`000000oooo0?ooo`0G0?ooo`050?l0003oool0003o0?ooo`3o00000P3oool00`3o0000oooo
0?ooo`020?ooo`050?l0003oool0oooo0?l0003o0?l00`3oool01`3o0000oooo0?ooo`3oool0o`00
0?ooo`3o0000o`3ooolJ0?ooo`000P3oool00`000000oooo0?ooo`040?ooo`800000203oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo01L0oooo00D0o`000?ooo`000?l0o`3o0?l00002
0?ooo`030?l0003oool0oooo0080oooo00D0o`000?ooo`3oool0o`000?l0o`030?ooo`070?l0003o
ool0oooo0?ooo`3o0000oooo0?l0003o0?oooaX0oooo00020?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0G0?ooo`040?l0003oool0oooo0?l000<0
oooo00<0o`000?ooo`3oool00P3oool01@3o0000oooo0?ooo`3o0000o`3o0080oooo00<0o`000?l0
o`3oool00P3oool00`3o0000oooo0?l0003o0?oooaX0oooo00020?ooo`030000003oool0oooo00X0
oooo1P0000050?ooo`<000005`3oool01@3o0000oooo0?ooo`3o0000003o0080oooo00<0o`000?oo
o`3oool00P3oool01@3o0000oooo0?ooo`3o0000o`3o0080oooo00<0o`000?l0o`3oool00P3oool0
0`3o0000oooo0?l0003o0?oooaX0oooo00020?ooo`030000003oool0oooo00/0oooo00@000000?oo
o`3oool000001P3oool00`000000oooo0?ooo`0G0?ooo`050?l0003oool0oooo0?l000000?l00P3o
ool00`3o0?l0o`000?ooo`020?ooo`090?l0003oool0oooo0?l0003o0?l0oooo0000o`3o0000o`3o
00<0oooo00<0o`000?ooo`3o0000o`3ooolJ0?ooo`000P3oool00`000000oooo0?ooo`0;0?ooo`04
0000003oool0oooo000000H0oooo00<000000?ooo`3oool05`3oool02@3o0000oooo0?ooo`3o0000
oooo0000o`3oool0o`3o0?l000030?ooo`090?l0003oool0oooo0?l0003o0?l0oooo0000o`3o0000
o`3o00<0oooo00<0o`000?ooo`3o0000o`3ooolJ0?ooo`000`00000>0?ooo`030000003oool00000
00H0oooo00<000000?ooo`3oool05`3oool0103o0000oooo0?ooo`3o00020?ooo`030000o`3o0?l0
o`0000<0oooo00P0o`000?ooo`3oool0o`000?l0o`000?l0oooo0?l000@0oooo00<0o`000?ooo`3o
0000o`3ooolJ0?ooo`000P3oool00`000000oooo0?ooo`0=0?ooo`8000001P3oool00`000000oooo
0?ooo`0G0?ooo`040?l0003oool0oooo0?l00080oooo00<0003o0?l0o`3o00000`3oool01@3o0000
oooo0?ooo`3o0000o`3o0080oooo00<0o`000?ooo`3oool00P3oool00`3o0000o`3o0?l0003o0?oo
oaX0oooo000J0?ooo`030000003oool0oooo01L0oooo00@0o`000?ooo`3o0?l0o`000`3oool01P3o
0?l0o`000?ooo`3oool0o`3o0?l00080oooo00D0o`000?l0o`3oool0oooo0?l000040?ooo`030?l0
003o0?l0o`000?l0oooo6P3oool001X0oooo00<000000?ooo`3oool05`3oool00`3o0000oooo0?l0
00040?ooo`0:0?l0o`3o0000003o0?ooo`3o0?l0o`000?ooo`000?l0o`000?l0o`80oooo00<0o`00
0?ooo`3oool00P3oool00`3o0000o`3o0?l0003o0?oooaX0oooo000J0?ooo`030000003oool0oooo
01L0oooo00<0o`000?ooo`3o00001@3oool01P3o0000003o0000o`3o0?l0o`000000o`80oooo00@0
o`000?ooo`3oool0o`00103oool00`3o0000o`3o0?l0003o0?oooaX0oooo000J0?ooo`800000603o
ool00`3o0000oooo0?l000050?ooo`050?l0003oool0oooo0?l0o`3o00000`3oool0103o0000oooo
0?ooo`3o00040?ooo`030?l0o`3o0000oooo0?l0oooo6P3oool001X0oooo00<000000?ooo`3oool0
5`3oool00`3o0?l0o`000?l0o`050?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3o
ool0o`001@3oool00`3o0000oooo0?ooo`3o0?oooaT0oooo000J0?ooo`030000003oool0oooo01L0
oooo00<0o`3o0?l0003o0?l01@3oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo
0?l000D0oooo00<0o`000?ooo`3oool0o`3ooolI0?ooo`006P3oool00`000000oooo0?ooo`0H0?oo
o`030?l0003oool0oooo00@0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o
00050?ooo`030?l0003oool0oooo0?l0oooo6@3oool001X0oooo00<000000?ooo`3oool0603oool0
0`3o0000oooo0?ooo`040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3o0?l0o`00
1@3oool00`3o0000oooo0?ooo`3o0?oooaT0oooo000J0?ooo`030000003oool0oooo01P0oooo00<0
o`3o0?ooo`3oool0103oool0103o0000oooo0?ooo`3o00040?ooo`030?l0003oool0o`0000H0oooo
00<0o`000?ooo`3oool0o`3ooolI0?ooo`006P3oool00`000000oooo0?ooo`0H0?ooo`030?l0o`3o
ool0oooo00@0oooo00@0o`000?ooo`3o0000o`3o103oool00`3o0000oooo0?l000060?ooo`030?l0
003oool0oooo0?l0oooo6@3oool001X0oooo00<000000?ooo`3oool07`3oool00`3o0000oooo0?l0
00050?ooo`030?l0003oool0o`0000H0oooo00<0o`3o0?ooo`3oool0o`3ooolI0?ooo`006P3oool2
00000200oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000060?ooo`030?l0o`3oool0
oooo0?l0oooo6@3oool001X0oooo00<000000?ooo`3oool07`3oool00`3o0000oooo0?l000050?oo
o`030?l0003oool0o`000?l0oooo8P3oool001X0oooo00<000000?ooo`3oool07`3oool00`3o0000
oooo0?l000050?ooo`030?l0003oool0o`000?l0oooo8P3oool001X0oooo00<000000?ooo`3oool0
7`3oool00`3o0000oooo0?l000050?ooo`80o`0000<0o`3o0?ooo`3oool0o`3ooolP0?ooo`006P3o
ool00`000000oooo0?ooo`0O0?ooo`030?l0003oool0o`0000D0oooo0P3o00000`3o0?l0oooo0?oo
o`3o0?ooob00oooo000J0?ooo`030000003oool0oooo01l0oooo00<0o`000?ooo`3o00001@3oool2
0?l00?l0oooo8`3oool001X0oooo00<000000?ooo`3oool07`3oool00`3o0000oooo0?l000050?oo
o`030?l0o`3o0000oooo0?l0oooo8P3oool001X0oooo00<000000?ooo`3oool07`3oool00`3o0?l0
o`000?l000050?ooo`030?l0o`3o0000oooo0?l0oooo8P3oool001X0oooo0P00000Q0?ooo`030?l0
003o0?l0oooo00@0oooo00<0o`3o0?l0003oool0o`3ooolR0?ooo`006P3oool00`000000oooo0?oo
o`0P0?ooo`030?l0003o0?l0oooo00@0oooo00<0o`3o0?l0003oool0o`3ooolR0?ooo`006P3oool0
0`000000oooo0?ooo`0P0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0o`3ooolQ
0?ooo`006P3oool00`000000oooo0?ooo`0P0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?oo
o`3oool0o`3ooolQ0?ooo`006P3oool00`000000oooo0?ooo`0P0?ooo`030?l0003oool0oooo00D0
oooo00<0o`3o0?ooo`3oool0o`3ooolQ0?ooo`006P3oool00`000000oooo0?ooo`0P0?ooo`030?l0
o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool0o`3ooolQ0?ooo`001@0000040?ooo`8000001P3o
ool3000000H0oooo00<000000?ooo`3oool0803oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3o
ool0oooo0?l0oooo8@3oool00080oooo00<000000?ooo`3oool0103oool2000000D0oooo00D00000
0?ooo`3oool0oooo000000050?ooo`030000003oool0oooo0200oooo00<0o`3o0?ooo`3oool0o`3o
oolY0?ooo`000P3oool00`000000oooo0?ooo`0;0?ooo`050000003oool0oooo0?ooo`0000001@3o
ool00`000000oooo0?ooo`3o0?oood`0oooo00020?ooo`030000003oool0oooo00/0oooo00D00000
0?ooo`3oool0oooo000000050?ooo`<00000o`3ooom<0?ooo`000P3oool00`000000oooo0?ooo`0;
0?ooo`@000001P3oool00`000000oooo0?ooo`3o0?oood`0oooo00020?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3o0?oood`0oooo0003000000h0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3o0?oood`0oooo00020?ooo`030000
003oool0oooo00d0oooo0`00003o0?oooe@0oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?oo
ofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo0000\
\>"],
  ImageRangeCache->{{{0, 359}, {221.375, 0}} -> {-0.0332863, 0.334837, \
0.00155747, 0.00750603}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[vCAve, \ vCCor, \ vCSwi, \ PlotRange\  -> \ All];\)\)], "Input",
  CellTags->"Large-Signal Simulation"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 2.38095 -0.752038 0.160572 [
[.2619 .03832 -9 -9 ]
[.2619 .03832 9 0 ]
[.5 .03832 -9 -9 ]
[.5 .03832 9 0 ]
[.7381 .03832 -9 -9 ]
[.7381 .03832 9 0 ]
[.97619 .03832 -9 -9 ]
[.97619 .03832 9 0 ]
[.01131 .13111 -18 -4.5 ]
[.01131 .13111 0 4.5 ]
[.01131 .21139 -6 -4.5 ]
[.01131 .21139 0 4.5 ]
[.01131 .29168 -18 -4.5 ]
[.01131 .29168 0 4.5 ]
[.01131 .37197 -6 -4.5 ]
[.01131 .37197 0 4.5 ]
[.01131 .45225 -18 -4.5 ]
[.01131 .45225 0 4.5 ]
[.01131 .53254 -6 -4.5 ]
[.01131 .53254 0 4.5 ]
[.01131 .61282 -18 -4.5 ]
[.01131 .61282 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .05082 m
.2619 .05707 L
s
[(0.1)] .2619 .03832 0 1 Mshowa
.5 .05082 m
.5 .05707 L
s
[(0.2)] .5 .03832 0 1 Mshowa
.7381 .05082 m
.7381 .05707 L
s
[(0.3)] .7381 .03832 0 1 Mshowa
.97619 .05082 m
.97619 .05707 L
s
[(0.4)] .97619 .03832 0 1 Mshowa
.125 Mabswid
.07143 .05082 m
.07143 .05457 L
s
.11905 .05082 m
.11905 .05457 L
s
.16667 .05082 m
.16667 .05457 L
s
.21429 .05082 m
.21429 .05457 L
s
.30952 .05082 m
.30952 .05457 L
s
.35714 .05082 m
.35714 .05457 L
s
.40476 .05082 m
.40476 .05457 L
s
.45238 .05082 m
.45238 .05457 L
s
.54762 .05082 m
.54762 .05457 L
s
.59524 .05082 m
.59524 .05457 L
s
.64286 .05082 m
.64286 .05457 L
s
.69048 .05082 m
.69048 .05457 L
s
.78571 .05082 m
.78571 .05457 L
s
.83333 .05082 m
.83333 .05457 L
s
.88095 .05082 m
.88095 .05457 L
s
.92857 .05082 m
.92857 .05457 L
s
.25 Mabswid
0 .05082 m
1 .05082 L
s
.02381 .13111 m
.03006 .13111 L
s
[(5.5)] .01131 .13111 1 0 Mshowa
.02381 .21139 m
.03006 .21139 L
s
[(6)] .01131 .21139 1 0 Mshowa
.02381 .29168 m
.03006 .29168 L
s
[(6.5)] .01131 .29168 1 0 Mshowa
.02381 .37197 m
.03006 .37197 L
s
[(7)] .01131 .37197 1 0 Mshowa
.02381 .45225 m
.03006 .45225 L
s
[(7.5)] .01131 .45225 1 0 Mshowa
.02381 .53254 m
.03006 .53254 L
s
[(8)] .01131 .53254 1 0 Mshowa
.02381 .61282 m
.03006 .61282 L
s
[(8.5)] .01131 .61282 1 0 Mshowa
.125 Mabswid
.02381 .06688 m
.02756 .06688 L
s
.02381 .08294 m
.02756 .08294 L
s
.02381 .09899 m
.02756 .09899 L
s
.02381 .11505 m
.02756 .11505 L
s
.02381 .14716 m
.02756 .14716 L
s
.02381 .16322 m
.02756 .16322 L
s
.02381 .17928 m
.02756 .17928 L
s
.02381 .19534 m
.02756 .19534 L
s
.02381 .22745 m
.02756 .22745 L
s
.02381 .24351 m
.02756 .24351 L
s
.02381 .25957 m
.02756 .25957 L
s
.02381 .27562 m
.02756 .27562 L
s
.02381 .30774 m
.02756 .30774 L
s
.02381 .32379 m
.02756 .32379 L
s
.02381 .33985 m
.02756 .33985 L
s
.02381 .35591 m
.02756 .35591 L
s
.02381 .38802 m
.02756 .38802 L
s
.02381 .40408 m
.02756 .40408 L
s
.02381 .42014 m
.02756 .42014 L
s
.02381 .43619 m
.02756 .43619 L
s
.02381 .46831 m
.02756 .46831 L
s
.02381 .48437 m
.02756 .48437 L
s
.02381 .50042 m
.02756 .50042 L
s
.02381 .51648 m
.02756 .51648 L
s
.02381 .54859 m
.02756 .54859 L
s
.02381 .56465 m
.02756 .56465 L
s
.02381 .58071 m
.02756 .58071 L
s
.02381 .59677 m
.02756 .59677 L
s
.02381 .03476 m
.02756 .03476 L
s
.02381 .01871 m
.02756 .01871 L
s
.02381 .00265 m
.02756 .00265 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 1 r
.001 Mabswid
.02381 .05082 m
.02605 .04606 L
.02846 .04182 L
.03053 .03888 L
.03279 .03643 L
.03416 .03531 L
.03543 .03452 L
.03665 .03398 L
.03793 .03365 L
.03908 .03356 L
.04016 .03364 L
.04135 .03392 L
.04262 .03444 L
.04393 .0352 L
.04516 .03612 L
.04793 .03894 L
.05037 .04225 L
.05297 .04658 L
.06244 .06894 L
.06791 .08607 L
.07382 .10757 L
.08454 .15299 L
.10458 .25163 L
.12413 .351 L
.14558 .44702 L
.15544 .4829 L
.166 .51422 L
.17113 .52656 L
.17598 .53648 L
.18067 .54443 L
.18507 .55043 L
.18966 .55519 L
.1921 .5571 L
.19335 .55792 L
.19472 .5587 L
.19598 .55929 L
.19733 .55981 L
.19808 .56004 L
.19875 .56022 L
.20007 .56047 L
.20129 .56061 L
.2026 .56065 L
.20384 .56058 L
.20498 .56043 L
.20604 .56022 L
.20718 .55991 L
.20924 .55915 L
.21169 .55792 L
.21393 .55648 L
.21879 .5524 L
Mistroke
.2234 .54737 L
.23301 .5337 L
.24325 .51521 L
.26165 .47522 L
.28166 .42823 L
.30328 .38153 L
.31345 .36302 L
.32419 .34669 L
.33335 .33563 L
.3434 .32672 L
.34866 .32341 L
.35124 .32212 L
.35363 .32112 L
.35575 .32039 L
.3581 .31976 L
.36045 .31929 L
.36174 .31911 L
.36295 .31899 L
.36417 .31891 L
.36487 .31889 L
.36551 .31888 L
.36682 .3189 L
.36755 .31893 L
.36821 .31897 L
.3694 .31908 L
.37069 .31924 L
.37302 .31966 L
.37577 .32034 L
.37867 .32128 L
.38387 .3235 L
.39334 .3291 L
.40349 .33702 L
.42181 .35476 L
.4416 .37603 L
.46314 .3979 L
.47316 .40668 L
.48381 .41464 L
.4938 .42065 L
.50296 .42484 L
.50751 .42644 L
.51231 .42779 L
.51713 .42878 L
.51986 .42919 L
.52108 .42934 L
.5224 .42948 L
.52364 .42958 L
.52478 .42966 L
.52587 .42971 L
.52703 .42975 L
.52806 .42977 L
Mistroke
.52903 .42978 L
.53009 .42977 L
.53123 .42975 L
.53244 .4297 L
.53376 .42963 L
.53501 .42954 L
.53616 .42943 L
.54067 .42888 L
.54332 .42844 L
.54576 .42797 L
.55125 .42666 L
.56121 .42355 L
.57982 .41582 L
.61933 .39667 L
.63921 .3885 L
.64789 .38561 L
.65732 .38304 L
.66253 .38188 L
.66735 .38098 L
.67225 .38023 L
.67685 .37969 L
.68167 .37927 L
.68423 .37912 L
.68555 .37905 L
.68699 .379 L
.68768 .37897 L
.68841 .37895 L
.68973 .37893 L
.69051 .37892 L
.69124 .37891 L
.6919 .37891 L
.69262 .37891 L
.69392 .37892 L
.69457 .37892 L
.6953 .37894 L
.69657 .37896 L
.69777 .379 L
.70036 .37911 L
.70275 .37924 L
.70819 .37966 L
.71297 .38015 L
.71807 .38079 L
.73661 .38397 L
.77827 .39316 L
.79919 .39728 L
.80836 .39876 L
.8184 .40009 L
.82311 .40061 L
.82814 .40108 L
.83248 .40143 L
Mistroke
.8372 .40173 L
.83998 .40187 L
.84254 .40199 L
.84494 .40207 L
.84758 .40215 L
.84872 .40217 L
.84995 .40219 L
.85101 .40221 L
.85217 .40222 L
.85347 .40223 L
.8547 .40224 L
.85591 .40224 L
.85703 .40224 L
.85831 .40223 L
.85896 .40223 L
.85967 .40222 L
.8621 .40218 L
.86347 .40215 L
.86494 .40212 L
.86759 .40204 L
.87279 .40184 L
.87754 .4016 L
.88709 .40099 L
.89614 .40026 L
.93446 .39642 L
.97523 .3929 L
.97619 .39284 L
Mfstroke
1 0 1 r
.02381 .06998 m
.02608 .0534 L
.02847 .03527 L
.02964 .02613 L
.03029 .02103 L
.0306 .0185 L
.03074 .01739 L
.03089 .01616 L
.03098 .01572 L
.03106 .0158 L
.03124 .016 L
.03161 .01643 L
.03227 .01728 L
.03355 .01927 L
.03599 .02428 L
.03823 .03026 L
.04302 .04752 L
.04543 .05848 L
.04648 .06371 L
.04704 .06658 L
.04734 .06817 L
.04749 .06896 L
.04762 .06965 L
.04777 .06862 L
.04793 .06751 L
.04826 .06514 L
.04884 .06093 L
.04999 .0525 L
.05252 .03327 L
.05364 .02447 L
.05425 .01956 L
.0544 .01839 L
.05455 .01714 L
.05468 .01608 L
.05483 .01579 L
.05498 .01668 L
.05512 .0175 L
.05544 .01937 L
.05613 .02343 L
.05733 .03084 L
.06204 .06296 L
.06704 .10225 L
.06826 .11256 L
.06955 .12379 L
.07065 .13362 L
.07096 .13635 L
.07112 .13787 L
.07128 .13931 L
.07143 .14065 L
.07151 .14006 L
Mistroke
.07159 .13946 L
.07186 .1373 L
.07426 .11828 L
.0765 .0996 L
.0777 .0894 L
.07802 .08657 L
.0782 .08502 L
.07837 .08357 L
.07851 .08233 L
.07866 .08262 L
.07875 .08342 L
.07883 .08418 L
.07898 .08561 L
.08123 .10688 L
.08586 .15306 L
.09078 .20521 L
.09323 .23211 L
.09381 .23862 L
.09443 .24549 L
.09495 .25138 L
.0951 .253 L
.09525 .2545 L
.09539 .25322 L
.09553 .25206 L
.09772 .23222 L
.10008 .21021 L
.10121 .1995 L
.10183 .19359 L
.10213 .19069 L
.10226 .18941 L
.1024 .18848 L
.10256 .1902 L
.1027 .19184 L
.10303 .19544 L
.10371 .20312 L
.10491 .2167 L
.10983 .27295 L
.11456 .32796 L
.11693 .35569 L
.11797 .36785 L
.11856 .37474 L
.11871 .37647 L
.11885 .37811 L
.11897 .37957 L
.11911 .37984 L
.11925 .37843 L
.11939 .37693 L
.11966 .37424 L
.12024 .36822 L
.12144 .35587 L
Mistroke
.12395 .32957 L
.12509 .31755 L
.12539 .31436 L
.12572 .31089 L
.12586 .3093 L
.12602 .3076 L
.12617 .306 L
.12631 .30719 L
.12746 .32035 L
.12852 .33236 L
.13335 .38702 L
.138 .43867 L
.14036 .46441 L
.14161 .47789 L
.14194 .48142 L
.1423 .48526 L
.14246 .48701 L
.14264 .48887 L
.14279 .49046 L
.14287 .49103 L
.14295 .49012 L
.14424 .47539 L
.14542 .46194 L
.14772 .43554 L
.1489 .42189 L
.14922 .41829 L
.14955 .41445 L
.14971 .41258 L
.14987 .41082 L
.15 .40931 L
.15015 .41084 L
.15122 .42179 L
.15238 .43365 L
.15732 .48252 L
.16208 .52732 L
.16447 .54892 L
.16551 .55816 L
.16606 .56298 L
.16636 .56562 L
.16651 .5669 L
.16665 .56808 L
.16681 .56653 L
.16698 .56444 L
.16727 .5608 L
.16785 .5537 L
.169 .53972 L
.17152 .50901 L
.17262 .49567 L
.17322 .48835 L
.17352 .48476 L
Mistroke
.17365 .48318 L
.17379 .48144 L
.17395 .48241 L
.17412 .48379 L
.17447 .48671 L
.17508 .49177 L
.17629 .50154 L
.18114 .53921 L
.18582 .57272 L
.18819 .5887 L
.18885 .59295 L
.18954 .59743 L
.18988 .59958 L
.19019 .6016 L
.19033 .60247 L
.19048 .60332 L
.19056 .60234 L
.19064 .60128 L
.19079 .5994 L
.19558 .53928 L
.19615 .53211 L
.19678 .52433 L
.19707 .52078 L
.19738 .517 L
.1975 .51546 L
.19764 .51417 L
.19779 .51507 L
.19793 .51591 L
.19925 .52373 L
.20045 .53068 L
.20541 .55749 L
.20788 .56976 L
.21019 .58054 L
.21241 .59037 L
.21305 .59309 L
.21365 .59562 L
.21392 .59671 L
.21406 .59731 L
.21419 .59786 L
.21434 .59755 L
.21442 .59657 L
.2145 .59551 L
.21478 .59193 L
.21716 .56195 L
.21842 .54628 L
.21976 .52972 L
.22043 .52145 L
.22076 .5174 L
.22106 .51369 L
.22121 .51194 L
Mistroke
.22134 .51031 L
.22148 .50943 L
.22163 .51004 L
.22225 .51244 L
.22455 .52111 L
.22925 .53715 L
.23164 .54453 L
.23424 .55199 L
.23546 .55528 L
.23674 .55864 L
.23727 .55998 L
.23755 .56068 L
.23784 .5614 L
.238 .56179 L
.23809 .562 L
.23817 .56113 L
.23847 .55744 L
.23905 .55021 L
.24132 .52254 L
.24377 .49296 L
.24442 .48516 L
.24478 .4809 L
.24495 .47883 L
.24511 .47693 L
.24525 .47544 L
.24539 .47577 L
.2457 .47651 L
.24635 .47805 L
.24878 .48361 L
.25339 .49319 L
.25573 .4976 L
.25831 .50219 L
.25948 .50418 L
.26075 .50628 L
.26133 .50722 L
.26162 .50769 L
.2617 .50782 L
.26179 .50795 L
.26194 .50772 L
.26208 .50605 L
.26224 .50424 L
.26251 .50095 L
.26304 .49479 L
.26538 .46736 L
.26663 .45291 L
.26797 .43746 L
.26828 .43386 L
.26862 .42992 L
.26878 .42812 L
.26887 .42714 L
Mistroke
.26895 .42621 L
.2691 .42514 L
.26925 .42538 L
.27042 .42731 L
.27271 .43099 L
.27736 .43818 L
.27972 .44173 L
.28231 .44561 L
.2835 .44739 L
.28415 .44837 L
.28477 .4493 L
.28535 .45017 L
.28551 .4504 L
.28559 .45053 L
.28568 .45066 L
.28582 .44955 L
.28598 .44778 L
.28707 .43551 L
.2893 .4107 L
.29056 .3967 L
.29174 .38372 L
.29234 .37708 L
.29264 .37383 L
.29277 .3724 L
.29291 .37148 L
.29306 .37173 L
.29322 .372 L
.29356 .37256 L
.29416 .37354 L
.2967 .37781 L
.30157 .38634 L
.30625 .39513 L
.30747 .39754 L
.30812 .39883 L
.30881 .40022 L
.30915 .40092 L
.30931 .40126 L
.30947 .40158 L
.30961 .40082 L
.30975 .39924 L
.31006 .39593 L
.31123 .38349 L
.31363 .35791 L
.31468 .34676 L
.31582 .33465 L
.31612 .33139 L
.31645 .32794 L
.31658 .32652 L
.31672 .32573 L
.31688 .32608 L
Mistroke
.31703 .32641 L
.31818 .329 L
.3207 .33489 L
.32567 .34734 L
.32815 .354 L
.33045 .36048 L
.3316 .3638 L
.33268 .367 L
.33297 .36787 L
.33313 .36835 L
.33328 .3688 L
.33336 .36871 L
.33344 .36785 L
.33362 .36603 L
.33393 .36284 L
.33506 .35118 L
.33744 .3265 L
.3387 .31344 L
.33939 .30627 L
.33973 .30276 L
.34004 .29953 L
.34032 .29664 L
.34046 .29519 L
.34061 .29546 L
.34077 .29595 L
.34093 .29647 L
.34123 .29742 L
.34254 .30166 L
.34485 .30937 L
.34985 .32713 L
.35235 .33656 L
.35467 .34559 L
.35583 .35024 L
.3564 .35255 L
.35665 .35356 L
.35692 .35467 L
.35706 .35527 L
.3572 .35504 L
.35733 .35373 L
.35746 .35234 L
.35805 .34637 L
.3593 .33366 L
.36171 .30902 L
.36298 .29599 L
.36367 .28885 L
.36401 .28537 L
.36416 .28384 L
.36424 .28298 L
.36432 .28269 L
.36447 .2833 L
Mistroke
.36462 .28395 L
.3649 .28515 L
.36552 .2878 L
.36684 .29349 L
.36916 .30373 L
.37371 .32472 L
.37624 .33685 L
.37856 .34826 L
.37974 .35416 L
.38032 .35706 L
.38057 .35833 L
.3807 .359 L
.38084 .35971 L
.38099 .35993 L
.38112 .35854 L
.38139 .35581 L
.38198 .3498 L
.38323 .33704 L
.38562 .3125 L
.38667 .30172 L
.38722 .29602 L
.38781 .28994 L
.38789 .28909 L
.38798 .28818 L
.38813 .28713 L
.38829 .28793 L
.38843 .28867 L
.38901 .29166 L
.39015 .29762 L
.39267 .31096 L
.39745 .33694 L
.40204 .36275 L
.40325 .3697 L
.40384 .37313 L
.4044 .37632 L
.40455 .37723 L
.4047 .3781 L
.40486 .37744 L
.40494 .37659 L
.40503 .37567 L
.40572 .36855 L
.40692 .35607 L
.4093 .33128 L
.41055 .31812 L
.41124 .31087 L
.41158 .30731 L
.41173 .30575 L
.41181 .30492 L
.41189 .30403 L
.41205 .30473 L
Mistroke
.4122 .30558 L
.41252 .30745 L
.41319 .31139 L
.41437 .3183 L
.41668 .33189 L
.42167 .36172 L
.42416 .37683 L
.42647 .39091 L
.42703 .39431 L
.42763 .39798 L
.4279 .39967 L
.42819 .40145 L
.42844 .40297 L
.42858 .40364 L
.42871 .40226 L
.42985 .39026 L
.43109 .37697 L
.43335 .35286 L
.43454 .34013 L
.43519 .33315 L
.43536 .33139 L
.43551 .32973 L
.43565 .32824 L
.4358 .3281 L
.43596 .32908 L
.43611 .32998 L
.43644 .332 L
.43714 .33628 L
.43838 .34384 L
.44081 .35874 L
.44553 .38778 L
.44795 .40262 L
.44921 .41042 L
.45055 .41864 L
.45176 .42602 L
.45208 .42804 L
.45218 .42863 L
.45227 .42918 L
.45235 .42967 L
.45244 .42924 L
.45251 .42841 L
.4526 .4275 L
.45277 .42565 L
.45307 .42233 L
.45539 .3971 L
.45663 .38362 L
.45792 .36943 L
.45846 .36357 L
.45903 .35731 L
.4592 .35545 L
Mistroke
.45936 .3537 L
.45951 .35213 L
.45966 .35278 L
.45981 .35367 L
.45997 .35462 L
.46025 .35631 L
.46492 .38453 L
.46949 .41193 L
.47204 .42704 L
.47436 .44075 L
.47551 .44751 L
.47583 .44935 L
.476 .45036 L
.47609 .45086 L
.47617 .45132 L
.4763 .4502 L
.47645 .44855 L
.47677 .44504 L
.47905 .41969 L
.48025 .40642 L
.48152 .39223 L
.48214 .38534 L
.48273 .37884 L
.48298 .376 L
.48312 .37445 L
.48325 .373 L
.4834 .37248 L
.48355 .37337 L
.48382 .37495 L
.48842 .40119 L
.49309 .42745 L
.49547 .4406 L
.49673 .4475 L
.49807 .4548 L
.49874 .45844 L
.49937 .46186 L
.49965 .46335 L
.4998 .46417 L
.49994 .46493 L
.50009 .46423 L
.50017 .46332 L
.50026 .46234 L
.50055 .45903 L
.50286 .43309 L
.50525 .40624 L
.5059 .39889 L
.5066 .39109 L
.50689 .38789 L
.50704 .3862 L
.50719 .38527 L
Mistroke
.50728 .38572 L
.50737 .38621 L
.50753 .38707 L
.50785 .38876 L
.51266 .41397 L
.51755 .43902 L
.51981 .45038 L
.52226 .46253 L
.52291 .46574 L
.52327 .46748 L
.52344 .46834 L
.5236 .46911 L
.52375 .46983 L
.52388 .46928 L
.52403 .46763 L
.52419 .46583 L
.52484 .45851 L
.52727 .43105 L
.52965 .40425 L
.53033 .39655 L
.53067 .39278 L
.53081 .39112 L
.53089 .39025 L
.53097 .38967 L
.53112 .39037 L
.53126 .39103 L
.53155 .39246 L
.53218 .39548 L
.53717 .41922 L
.54199 .44153 L
.54419 .45156 L
.54544 .45722 L
.54606 .45997 L
.54662 .46249 L
.5469 .46375 L
.54721 .46513 L
.54735 .46575 L
.5475 .46642 L
.54764 .46671 L
.54777 .46526 L
.54901 .45126 L
.55154 .42269 L
.55267 .41002 L
.55386 .39665 L
.55416 .39328 L
.55449 .38966 L
.55458 .3886 L
.55467 .38759 L
.55475 .38669 L
.55484 .38694 L
Mistroke
.55492 .38731 L
.55501 .3877 L
.55517 .3884 L
.55637 .39371 L
.56093 .41355 L
.56577 .43426 L
.568 .44366 L
.56926 .44895 L
.56988 .45152 L
.57044 .45385 L
.57072 .45505 L
.57104 .45635 L
.57118 .45695 L
.57133 .45758 L
.57146 .45762 L
.5716 .45602 L
.57285 .44204 L
.5754 .41359 L
.5766 .40018 L
.57774 .38746 L
.57804 .38407 L
.57837 .38046 L
.57847 .37938 L
.57856 .37838 L
.57864 .37847 L
.57872 .37884 L
.57906 .38021 L
.58026 .38519 L
.58521 .40553 L
.58997 .42492 L
.59237 .43461 L
.59342 .43883 L
.59456 .44341 L
.59486 .44463 L
.59503 .44531 L
.59519 .44594 L
.59532 .44523 L
.59546 .44364 L
.59577 .44026 L
.59691 .42756 L
.59944 .39964 L
.60069 .38578 L
.60186 .37291 L
.602 .37131 L
.60216 .36957 L
.60231 .36793 L
.60245 .3674 L
.60252 .36771 L
.60261 .36805 L
.60278 .36873 L
Mistroke
.60307 .36994 L
.6044 .3753 L
.60927 .39492 L
.61396 .4138 L
.61634 .42341 L
.6176 .4285 L
.61829 .43128 L
.61863 .43265 L
.61878 .43325 L
.61886 .43357 L
.61894 .43391 L
.61909 .43392 L
.61922 .43246 L
.61951 .42925 L
.62013 .42252 L
.62143 .40818 L
.62374 .38291 L
.62497 .36942 L
.62562 .36232 L
.62579 .36045 L
.62598 .35843 L
.62607 .35746 L
.62615 .35653 L
.62631 .3566 L
.62646 .35721 L
.6266 .35777 L
.62691 .35905 L
.62757 .36175 L
.62874 .36651 L
.63346 .38586 L
.63587 .39579 L
.63847 .40658 L
.6397 .41167 L
.64099 .41706 L
.6421 .42167 L
.6424 .42294 L
.64256 .42364 L
.64272 .4243 L
.64287 .42474 L
.64303 .423 L
.64331 .41997 L
.64565 .39457 L
.64689 .38108 L
.64823 .36654 L
.64885 .35979 L
.6495 .35269 L
.64977 .34976 L
.64992 .34815 L
.65006 .34751 L
.65014 .34784 L
Mistroke
.65022 .34821 L
.65037 .34884 L
.65067 .35009 L
.65296 .35981 L
.6579 .38092 L
.66035 .39152 L
.66265 .40151 L
.66485 .41113 L
.66549 .41392 L
.66609 .41656 L
.66635 .41772 L
.6665 .41835 L
.66663 .41894 L
.66671 .41867 L
.66679 .41777 L
.66694 .4162 L
.66722 .41314 L
.66958 .38762 L
.67083 .3741 L
.67217 .35961 L
.67284 .35243 L
.67316 .3489 L
.67346 .34563 L
.6736 .34411 L
.67374 .34267 L
.67387 .34217 L
.67403 .34285 L
.67464 .34556 L
.67694 .35582 L
.68161 .37684 L
.68399 .38762 L
.68658 .39948 L
.68778 .40499 L
.68906 .41089 L
.68968 .41376 L
.68999 .41518 L
.69012 .4158 L
.69027 .41648 L
.6904 .41709 L
.69053 .4169 L
.69066 .41549 L
.6908 .41398 L
.69137 .40784 L
.69375 .3821 L
.69501 .3685 L
.69635 .35403 L
.69692 .34787 L
.69722 .34463 L
.69738 .34287 L
.69754 .34122 L
Mistroke
.69762 .34034 L
.6977 .34071 L
.69789 .34157 L
.69821 .34307 L
.69884 .34603 L
.70115 .35683 L
.70615 .38042 L
.70866 .39235 L
.71097 .4034 L
.71322 .41422 L
.71377 .41685 L
.71393 .41761 L
.71408 .41834 L
.71421 .41898 L
.71436 .41855 L
.71444 .41764 L
.71452 .4168 L
.71469 .41494 L
.715 .41159 L
.71561 .40507 L
.71813 .37785 L
.71923 .3659 L
.72044 .35278 L
.72101 .34666 L
.72117 .3449 L
.72132 .34323 L
.72146 .34227 L
.72161 .343 L
.72271 .34828 L
.7251 .35988 L
.73002 .38391 L
.73246 .39592 L
.73475 .40721 L
.73695 .41805 L
.73755 .42102 L
.73785 .42251 L
.73793 .42292 L
.73802 .42336 L
.73818 .4228 L
.73833 .42122 L
.73849 .4195 L
.73877 .41639 L
.73931 .41056 L
.74166 .38505 L
.74291 .37151 L
.7436 .36405 L
.74425 .35697 L
.74486 .35028 L
.74503 .34845 L
.74512 .34745 L
Mistroke
.74521 .34651 L
.74529 .34647 L
.74538 .3469 L
.74553 .34766 L
.74671 .35347 L
.749 .36487 L
.75396 .38953 L
.75643 .40185 L
.75873 .41335 L
.75987 .41904 L
.76095 .42443 L
.76154 .42741 L
.7617 .42823 L
.76179 .42867 L
.76188 .42909 L
.76202 .42793 L
.76219 .42616 L
.76332 .41385 L
.76555 .38949 L
.76799 .36287 L
.76863 .35589 L
.76882 .35388 L
.76891 .3529 L
.76899 .35199 L
.76914 .35183 L
.76923 .35224 L
.76931 .35267 L
.77054 .3588 L
.77296 .3709 L
.77754 .39377 L
.78009 .4065 L
.78242 .4181 L
.78357 .42386 L
.78418 .42689 L
.78482 .43011 L
.78539 .43292 L
.78554 .43369 L
.78562 .4341 L
.78571 .43452 L
.78587 .4329 L
.78601 .43132 L
.78711 .41931 L
.78951 .39295 L
.7917 .36898 L
.79228 .36259 L
.79259 .35922 L
.79267 .35834 L
.79276 .35738 L
.79292 .35661 L
.79307 .35735 L
Mistroke
.7932 .35803 L
.79351 .35958 L
.79407 .36232 L
.79659 .37485 L
.80139 .39859 L
.806 .42134 L
.80722 .42737 L
.80837 .43303 L
.80901 .43617 L
.80919 .43703 L
.80937 .43793 L
.80953 .43862 L
.80962 .43764 L
.8097 .43673 L
.8109 .42351 L
.81316 .39876 L
.8156 .37187 L
.81625 .36476 L
.81641 .36304 L
.81658 .36118 L
.81667 .3602 L
.81676 .36064 L
.81694 .36149 L
.81817 .36752 L
.8206 .37937 L
.82538 .40264 L
.82998 .42491 L
.8312 .43079 L
.83235 .43633 L
.83299 .43939 L
.83316 .44023 L
.83326 .44068 L
.83334 .44094 L
.8335 .4392 L
.83367 .4373 L
.83488 .42404 L
.83712 .39933 L
.83831 .38629 L
.83957 .37242 L
.83987 .36908 L
.8402 .36548 L
.84029 .36448 L
.84038 .36342 L
.84047 .36251 L
.84056 .36277 L
.84064 .36318 L
.84073 .36361 L
.84089 .36439 L
.84212 .37028 L
.84455 .38188 L
Mistroke
.84955 .40568 L
.85205 .41755 L
.85436 .42848 L
.85553 .43396 L
.85661 .43909 L
.8569 .44041 L
.85703 .44106 L
.85716 .44137 L
.85731 .43975 L
.85747 .43798 L
.85775 .43488 L
.859 .42111 L
.86127 .39612 L
.86246 .38298 L
.86372 .36911 L
.86403 .3657 L
.86412 .36472 L
.8642 .36379 L
.86436 .36323 L
.86445 .36365 L
.86454 .36405 L
.86473 .36494 L
.86507 .36653 L
.8663 .37234 L
.86874 .38373 L
.87347 .40582 L
.87803 .42697 L
.87916 .43223 L
.88036 .43777 L
.88066 .43917 L
.88075 .43957 L
.88083 .43995 L
.88099 .44011 L
.88108 .43913 L
.88116 .43823 L
.88134 .4362 L
.88167 .43256 L
.88288 .4193 L
.88533 .39226 L
.88653 .37905 L
.88763 .36699 L
.88789 .36411 L
.88804 .3625 L
.88817 .36222 L
.88833 .36295 L
.88848 .36364 L
.88876 .36495 L
.88983 .3699 L
.8922 .3808 L
.89686 .4022 L
Mistroke
.89922 .41303 L
.90181 .42489 L
.903 .43034 L
.90366 .43334 L
.90398 .43481 L
.90428 .43617 L
.90456 .43745 L
.9047 .4381 L
.90486 .43734 L
.90501 .43563 L
.90518 .43378 L
.90548 .43044 L
.90658 .41839 L
.90896 .39229 L
.91021 .3785 L
.9109 .37093 L
.91124 .36721 L
.91155 .36379 L
.91171 .36207 L
.91185 .36046 L
.91201 .36035 L
.91209 .36072 L
.91217 .36112 L
.91285 .36421 L
.91403 .3696 L
.91634 .38011 L
.92132 .40284 L
.92382 .41419 L
.92612 .4247 L
.92728 .42995 L
.92784 .43253 L
.92809 .43365 L
.92823 .43429 L
.92836 .43489 L
.92849 .43547 L
.92863 .4352 L
.92877 .4336 L
.92891 .43214 L
.92949 .4257 L
.93074 .412 L
.93325 .38451 L
.93435 .37244 L
.93494 .36604 L
.93526 .36252 L
.93542 .36078 L
.93556 .35916 L
.9357 .35762 L
.93585 .35815 L
.93617 .35959 L
.93673 .36212 L
Mistroke
.93781 .36708 L
.9402 .37794 L
.94522 .40079 L
.94774 .41228 L
.95005 .42283 L
.95123 .42817 L
.9518 .43078 L
.95205 .43192 L
.95218 .43252 L
.95232 .43316 L
.95246 .43253 L
.9526 .43104 L
.95287 .42811 L
.95346 .42166 L
.95471 .40798 L
.95709 .38196 L
.95813 .37057 L
.95868 .36455 L
.95926 .35813 L
.95942 .35638 L
.95957 .35551 L
.95971 .35612 L
.95985 .35679 L
.9604 .3593 L
.9616 .36477 L
.96412 .37628 L
.96905 .39889 L
.97381 .42072 L
.97619 .43165 L
Mfstroke
1 0 0 r
.02381 .06998 m
.02567 .05553 L
.02769 .04006 L
.02868 .03264 L
.0296 .0258 L
.03048 .01926 L
.03074 .01739 L
.0308 .01694 L
.03086 .01647 L
.03092 .01606 L
.03098 .01588 L
.03103 .01606 L
.03109 .01626 L
.03119 .0166 L
.03143 .01738 L
.03341 .02395 L
.03532 .03025 L
.03716 .03625 L
.03911 .04256 L
.04099 .04856 L
.04299 .05482 L
.04492 .06072 L
.04581 .06339 L
.04631 .06489 L
.04678 .06628 L
.04724 .06764 L
.04737 .06804 L
.0475 .06841 L
.04755 .06857 L
.04761 .06874 L
.04764 .06858 L
.04768 .06832 L
.04773 .06786 L
.04875 .05997 L
.05068 .04512 L
.05254 .03108 L
.05349 .02405 L
.05399 .02036 L
.05426 .01833 L
.05452 .01643 L
.05464 .01555 L
.05471 .01506 L
.05477 .01472 L
.0548 .01497 L
.05484 .01525 L
.05491 .01584 L
.05504 .01688 L
.05551 .02064 L
.05643 .02797 L
.05841 .04363 L
Mistroke
.06032 .05847 L
.06216 .07246 L
.06412 .08702 L
.06601 .10068 L
.06801 .11477 L
.06901 .12167 L
.06994 .1279 L
.07084 .13389 L
.07106 .13533 L
.0713 .13687 L
.07136 .13732 L
.0714 .13754 L
.07143 .13772 L
.07149 .13722 L
.07155 .13668 L
.07179 .13464 L
.07368 .11878 L
.0755 .10379 L
.07651 .09558 L
.07744 .08812 L
.07791 .08435 L
.07835 .08087 L
.07846 .07997 L
.07852 .07952 L
.07857 .07912 L
.07863 .07975 L
.07869 .08049 L
.0788 .0818 L
.0793 .08757 L
.0812 .10924 L
.08302 .12955 L
.08497 .15055 L
.08684 .17013 L
.08883 .19019 L
.09074 .20875 L
.09258 .22585 L
.09351 .2342 L
.09401 .23856 L
.09454 .24319 L
.09479 .24532 L
.09492 .24647 L
.09505 .24754 L
.09516 .24845 L
.09522 .24896 L
.09527 .24882 L
.09533 .24825 L
.0954 .24763 L
.09551 .2465 L
.09643 .23782 L
.0984 .21942 L
Mistroke
.10029 .20201 L
.10124 .19339 L
.10166 .18965 L
.10211 .18558 L
.10223 .1845 L
.1023 .18391 L
.10236 .18336 L
.10242 .18364 L
.10248 .18449 L
.10259 .18599 L
.10304 .19217 L
.10405 .20563 L
.10602 .23134 L
.10792 .25511 L
.10975 .27701 L
.11169 .29925 L
.11357 .31957 L
.11556 .33995 L
.11655 .34967 L
.11747 .35836 L
.11837 .36653 L
.11882 .37055 L
.11894 .37164 L
.11898 .37194 L
.11901 .37225 L
.11904 .37252 L
.11908 .37224 L
.11932 .36965 L
.12127 .34895 L
.12315 .32938 L
.12411 .31953 L
.12515 .30896 L
.12563 .30404 L
.1259 .30136 L
.12603 .30006 L
.12609 .29949 L
.12615 .29887 L
.1262 .29862 L
.12626 .29935 L
.12636 .3008 L
.12659 .30398 L
.12707 .31066 L
.12896 .336 L
.13096 .36154 L
.13289 .38476 L
.13475 .40576 L
.13672 .42663 L
.13863 .44526 L
.14045 .46179 L
.14147 .47039 L
Mistroke
.1424 .47786 L
.14263 .47961 L
.14275 .48056 L
.14281 .48102 L
.14286 .48129 L
.14292 .48059 L
.14299 .47983 L
.14313 .47823 L
.14337 .4754 L
.14428 .46477 L
.14619 .4428 L
.14802 .42204 L
.14905 .41061 L
.14953 .40525 L
.14974 .40291 L
.14986 .40158 L
.14992 .40093 L
.14998 .40034 L
.15003 .40052 L
.15009 .40121 L
.1502 .40273 L
.15044 .40581 L
.15095 .41223 L
.15186 .42361 L
.1537 .44551 L
.15566 .46722 L
.15755 .48655 L
.15937 .50364 L
.1613 .52022 L
.16316 .53456 L
.1641 .54124 L
.16514 .54809 L
.16561 .55108 L
.16612 .55416 L
.16637 .55561 L
.16649 .55632 L
.16655 .55662 L
.1666 .55696 L
.16666 .55726 L
.16671 .55681 L
.16681 .55547 L
.16704 .55263 L
.16891 .52959 L
.17089 .50554 L
.17189 .49369 L
.1728 .4828 L
.17326 .4774 L
.17349 .47476 L
.1737 .47234 L
.17376 .47164 L
Mistroke
.17381 .47104 L
.17387 .47169 L
.17393 .4724 L
.17419 .47524 L
.17464 .48012 L
.17652 .49943 L
.17851 .51821 L
.18043 .53457 L
.18227 .5487 L
.18424 .56198 L
.18613 .57306 L
.18794 .58213 L
.18895 .58649 L
.1894 .58825 L
.18988 .59007 L
.19011 .59088 L
.19023 .59132 L
.19035 .59173 L
.1904 .59192 L
.19046 .59211 L
.19052 .59158 L
.19058 .59085 L
.19079 .58818 L
.19174 .57602 L
.19357 .5531 L
.19551 .52913 L
.1964 .51823 L
.19687 .51252 L
.19738 .50647 L
.1975 .50502 L
.19756 .50426 L
.1976 .50384 L
.19763 .50366 L
.19766 .50397 L
.1977 .50431 L
.19777 .50491 L
.19791 .50609 L
.1984 .5103 L
.19936 .51828 L
.20124 .53255 L
.20323 .54597 L
.20515 .55718 L
.207 .56637 L
.20897 .57443 L
.21086 .58054 L
.21268 .58488 L
.21362 .58653 L
.21388 .58694 L
.21402 .58712 L
.21414 .58729 L
Mistroke
.2142 .58736 L
.21425 .58743 L
.21431 .58713 L
.21437 .58641 L
.21462 .58322 L
.21647 .55979 L
.21845 .53532 L
.21943 .5233 L
.22035 .51219 L
.2208 .50673 L
.22123 .50159 L
.22134 .50022 L
.2214 .49953 L
.22143 .49923 L
.22146 .49943 L
.2216 .50033 L
.22172 .50114 L
.22217 .50407 L
.22403 .51522 L
.22601 .52547 L
.22792 .53377 L
.22975 .54029 L
.2317 .54567 L
.2326 .54761 L
.23358 .54934 L
.23412 .55011 L
.23461 .55072 L
.23511 .55124 L
.23558 .55164 L
.23601 .55192 L
.23624 .55204 L
.23648 .55214 L
.23662 .55219 L
.23675 .55222 L
.23688 .55226 L
.237 .55228 L
.23706 .55229 L
.23712 .5523 L
.23717 .5523 L
.23723 .55231 L
.23729 .55231 L
.23733 .55231 L
.23736 .55231 L
.23741 .55231 L
.23747 .55231 L
.23753 .5523 L
.2376 .5523 L
.23766 .55229 L
.23773 .55229 L
.23785 .55227 L
Mistroke
.23796 .55224 L
.23803 .55223 L
.23807 .55222 L
.23811 .55207 L
.23814 .55164 L
.23817 .55125 L
.23824 .55038 L
.2385 .54721 L
.23947 .53526 L
.24139 .51181 L
.24324 .48965 L
.24418 .47857 L
.24468 .47274 L
.24496 .46954 L
.24509 .46797 L
.24515 .46727 L
.24518 .46691 L
.24522 .46651 L
.24528 .46645 L
.24533 .46674 L
.24546 .46739 L
.24573 .46873 L
.2462 .471 L
.24711 .47522 L
.24909 .48323 L
.251 .48952 L
.25283 .49428 L
.25385 .49639 L
.25478 .49796 L
.25574 .49925 L
.25665 .50016 L
.25712 .5005 L
.25738 .50065 L
.25762 .50078 L
.25784 .50087 L
.25804 .50094 L
.25826 .50101 L
.25839 .50103 L
.2585 .50105 L
.25856 .50106 L
.25863 .50107 L
.25868 .50107 L
.25874 .50108 L
.2588 .50108 L
.25886 .50108 L
.25891 .50108 L
.25896 .50108 L
.25899 .50108 L
.25903 .50108 L
.25909 .50108 L
Mistroke
.25915 .50107 L
.2592 .50107 L
.25933 .50105 L
.25946 .50103 L
.25967 .50098 L
.25988 .50092 L
.26034 .50072 L
.26081 .50043 L
.26131 .50005 L
.26152 .49986 L
.26174 .49965 L
.2618 .49958 L
.26186 .49952 L
.26192 .49931 L
.26198 .49859 L
.2622 .49592 L
.26417 .47268 L
.26607 .45071 L
.26789 .42996 L
.26837 .42455 L
.26883 .41944 L
.26895 .41808 L
.26899 .41769 L
.26902 .41732 L
.26908 .41713 L
.26912 .41729 L
.26915 .41743 L
.26922 .41774 L
.26935 .4183 L
.26983 .42029 L
.27174 .4274 L
.27357 .43311 L
.27551 .43795 L
.27739 .44143 L
.27841 .44284 L
.27888 .44337 L
.27938 .44386 L
.27984 .44423 L
.28034 .44457 L
.2808 .4448 L
.28123 .44496 L
.28147 .44502 L
.2816 .44505 L
.28173 .44507 L
.28185 .44509 L
.28191 .44509 L
.28196 .4451 L
.28202 .4451 L
.28205 .4451 L
.28209 .4451 L
Mistroke
.28214 .4451 L
.2822 .4451 L
.28226 .4451 L
.28232 .4451 L
.28237 .4451 L
.28243 .44509 L
.28255 .44508 L
.28267 .44507 L
.28287 .44503 L
.2831 .44497 L
.28355 .44481 L
.28403 .44457 L
.28491 .44394 L
.28537 .44351 L
.2855 .44338 L
.28557 .44331 L
.28563 .44324 L
.28569 .44317 L
.28573 .443 L
.28576 .44264 L
.28587 .44133 L
.28689 .42982 L
.28878 .40864 L
.29061 .38863 L
.29162 .37764 L
.2921 .37247 L
.29255 .3677 L
.29266 .36648 L
.29277 .36533 L
.29283 .36469 L
.29286 .36439 L
.29289 .36453 L
.293 .365 L
.2935 .36707 L
.29441 .37064 L
.29631 .37734 L
.29814 .38277 L
.30009 .38749 L
.30196 .39099 L
.30299 .39248 L
.30395 .39361 L
.30491 .39446 L
.30534 .39476 L
.3058 .39502 L
.30626 .39522 L
.30647 .3953 L
.3067 .39536 L
.30695 .39542 L
.30709 .39544 L
.30722 .39546 L
Mistroke
.30728 .39546 L
.30734 .39547 L
.30739 .39547 L
.30745 .39547 L
.30751 .39548 L
.30756 .39548 L
.30762 .39548 L
.30766 .39548 L
.30769 .39548 L
.30775 .39547 L
.30778 .39547 L
.30782 .39547 L
.30793 .39546 L
.308 .39546 L
.30807 .39545 L
.30819 .39543 L
.30844 .39538 L
.30867 .39532 L
.3091 .39517 L
.30932 .39507 L
.30938 .39504 L
.30945 .39501 L
.30951 .39498 L
.30956 .39456 L
.30962 .39394 L
.30967 .39335 L
.30979 .39202 L
.31004 .38933 L
.31048 .38452 L
.31148 .37373 L
.31335 .35386 L
.3143 .34389 L
.31533 .33312 L
.31581 .32816 L
.31607 .32545 L
.31632 .3229 L
.31655 .32053 L
.31662 .31988 L
.31665 .31955 L
.31668 .31946 L
.31675 .31976 L
.3168 .32004 L
.31724 .32209 L
.31911 .33033 L
.3211 .33808 L
.32302 .34455 L
.32487 .34985 L
.32683 .3545 L
.32872 .35803 L
.33053 .36053 L
Mistroke
.33146 .36148 L
.33194 .36188 L
.33247 .36225 L
.33271 .3624 L
.33297 .36254 L
.33309 .36261 L
.33321 .36266 L
.33326 .36269 L
.33332 .36271 L
.33338 .36225 L
.33343 .36167 L
.33445 .35093 L
.33627 .3319 L
.33821 .312 L
.3391 .30296 L
.33958 .29822 L
.34008 .2932 L
.34022 .29177 L
.34036 .29043 L
.34042 .28985 L
.34048 .28925 L
.34051 .28944 L
.34055 .28964 L
.34061 .29 L
.3411 .29275 L
.34206 .29803 L
.34401 .30793 L
.34589 .31656 L
.34788 .32476 L
.3498 .33169 L
.35165 .33746 L
.35361 .34264 L
.35459 .34484 L
.3555 .34668 L
.35638 .34825 L
.35663 .34866 L
.35687 .34904 L
.35698 .3492 L
.35703 .34929 L
.35709 .34937 L
.35712 .34941 L
.35715 .34938 L
.35721 .34877 L
.35732 .34758 L
.35929 .32708 L
.36119 .3077 L
.36302 .28939 L
.3635 .28462 L
.36395 .28011 L
.36408 .27891 L
Mistroke
.36421 .27764 L
.36424 .27732 L
.36427 .27697 L
.36431 .27703 L
.36435 .27728 L
.36448 .27817 L
.36496 .28136 L
.36694 .29394 L
.36884 .30512 L
.37067 .31498 L
.37262 .32453 L
.3745 .33276 L
.3765 .34049 L
.37842 .34693 L
.3793 .34955 L
.3798 .35095 L
.38027 .3522 L
.3805 .3528 L
.38073 .35337 L
.38085 .35367 L
.38089 .35375 L
.38092 .35384 L
.38096 .35387 L
.38099 .35354 L
.38122 .35105 L
.38224 .34045 L
.38416 .32052 L
.38602 .30169 L
.38696 .29226 L
.38746 .2873 L
.38773 .28459 L
.38787 .28325 L
.38799 .28202 L
.38806 .28141 L
.38811 .28116 L
.38817 .28162 L
.38824 .28214 L
.38851 .28418 L
.38898 .28772 L
.38989 .29448 L
.39176 .30751 L
.39374 .32033 L
.39565 .33167 L
.39748 .34161 L
.39943 .35114 L
.40131 .3593 L
.40227 .36307 L
.40331 .36686 L
.4038 .36852 L
.40431 .37022 L
Mistroke
.40452 .37089 L
.40464 .37125 L
.4047 .37143 L
.40475 .3716 L
.40481 .37108 L
.40488 .37035 L
.405 .36907 L
.40523 .36658 L
.40712 .34662 L
.40893 .32777 L
.40994 .31744 L
.41086 .30804 L
.41133 .30331 L
.41156 .30101 L
.41177 .29893 L
.41182 .29838 L
.41188 .29779 L
.41194 .29785 L
.41199 .29828 L
.41222 .30024 L
.41272 .30434 L
.41462 .31927 L
.41644 .33266 L
.41837 .34584 L
.42024 .35747 L
.42222 .36868 L
.42413 .37834 L
.42597 .38657 L
.42699 .39071 L
.42792 .39418 L
.42815 .39499 L
.42839 .39582 L
.42845 .39603 L
.42851 .39625 L
.42855 .39637 L
.42859 .39629 L
.42862 .39592 L
.42865 .39557 L
.42889 .39293 L
.42981 .38299 L
.43165 .36319 L
.43361 .34251 L
.43458 .33237 L
.43506 .32745 L
.43527 .3253 L
.4355 .32295 L
.43561 .32186 L
.43566 .32126 L
.43572 .3208 L
.43577 .32125 L
Mistroke
.43583 .32173 L
.43593 .32258 L
.43637 .32642 L
.43732 .33431 L
.43928 .3499 L
.44118 .36378 L
.443 .37604 L
.44494 .38793 L
.44681 .3982 L
.44879 .40787 L
.44979 .41222 L
.45071 .41596 L
.4516 .41932 L
.45185 .42024 L
.4521 .42108 L
.4522 .42144 L
.45231 .42182 L
.45237 .42202 L
.4524 .4218 L
.45243 .42145 L
.45255 .4202 L
.45442 .39942 L
.45641 .37773 L
.45741 .36702 L
.45833 .35722 L
.45879 .35234 L
.45923 .34778 L
.45933 .34666 L
.45944 .3455 L
.4595 .34488 L
.45954 .34478 L
.45957 .34505 L
.45968 .34602 L
.46018 .35029 L
.46206 .36568 L
.46406 .38077 L
.46598 .39404 L
.46784 .40564 L
.46981 .41668 L
.4717 .42606 L
.47353 .4339 L
.47454 .43777 L
.47547 .44099 L
.47593 .44248 L
.47606 .44287 L
.47613 .44308 L
.47616 .44319 L
.47619 .44328 L
.47625 .44265 L
.4763 .44198 L
Mistroke
.47643 .44054 L
.47734 .43023 L
.47925 .40893 L
.48108 .38881 L
.4821 .37774 L
.48258 .37255 L
.48303 .36778 L
.48314 .36654 L
.48325 .36538 L
.48331 .36479 L
.48336 .36476 L
.4834 .36504 L
.48343 .36531 L
.48349 .3658 L
.48399 .36996 L
.4849 .3773 L
.48674 .39123 L
.4887 .40478 L
.49058 .41658 L
.49258 .42776 L
.49451 .43719 L
.49636 .44503 L
.49731 .44854 L
.49834 .45204 L
.49881 .45352 L
.49932 .45501 L
.49957 .45571 L
.4998 .45634 L
.4999 .45661 L
.49995 .45675 L
.50001 .45676 L
.50007 .45605 L
.50013 .45539 L
.50024 .45413 L
.5021 .43288 L
.50408 .4107 L
.50507 .39979 L
.50599 .38973 L
.50644 .38477 L
.50687 .3801 L
.50699 .37885 L
.50705 .37817 L
.50711 .37753 L
.50714 .3772 L
.50718 .37745 L
.50725 .37801 L
.50737 .37899 L
.50782 .38252 L
.50981 .39723 L
.51172 .41012 L
Mistroke
.51356 .42131 L
.51552 .4319 L
.5174 .44082 L
.51941 .44892 L
.52134 .45538 L
.52223 .4579 L
.52273 .45921 L
.5232 .46035 L
.52344 .4609 L
.52366 .4614 L
.52372 .46153 L
.52376 .4616 L
.52379 .46167 L
.52382 .4616 L
.52386 .46117 L
.52392 .46042 L
.52416 .45768 L
.52517 .44603 L
.52704 .4249 L
.52902 .40284 L
.53001 .39197 L
.53049 .38672 L
.53072 .38425 L
.53082 .38317 L
.53088 .38255 L
.53093 .38198 L
.53098 .38193 L
.53104 .38234 L
.53115 .38318 L
.53139 .38494 L
.53182 .38808 L
.53276 .39475 L
.53475 .40776 L
.53667 .419 L
.53851 .4286 L
.54047 .43752 L
.54236 .44484 L
.54418 .45069 L
.54519 .45345 L
.54611 .45567 L
.54702 .45755 L
.54724 .45797 L
.54747 .45839 L
.54754 .45852 L
.54758 .45858 L
.54761 .45863 L
.54767 .45811 L
.54773 .45737 L
.54797 .4546 L
.54987 .43297 L
Mistroke
.55169 .41254 L
.5527 .40133 L
.55363 .39118 L
.55408 .38619 L
.55433 .38356 L
.55446 .38213 L
.55458 .38077 L
.5547 .37949 L
.55474 .37914 L
.55477 .37892 L
.55483 .37935 L
.55506 .38092 L
.55549 .38395 L
.55732 .39589 L
.55927 .40736 L
.56114 .41719 L
.56313 .42633 L
.56505 .43386 L
.5669 .43991 L
.56886 .44508 L
.56984 .44716 L
.57031 .44807 L
.57075 .44882 L
.57118 .4495 L
.57128 .44966 L
.57134 .44974 L
.57139 .44982 L
.57142 .44986 L
.57146 .44956 L
.57151 .4489 L
.57163 .4476 L
.57257 .43686 L
.57454 .41471 L
.57644 .39377 L
.57739 .38339 L
.57781 .37888 L
.57826 .37399 L
.57838 .37269 L
.57845 .37198 L
.57851 .37131 L
.57856 .37075 L
.57862 .37099 L
.57868 .37141 L
.57874 .37181 L
.5792 .37484 L
.5802 .38131 L
.58211 .39264 L
.58394 .40242 L
.58589 .41161 L
.58776 .41927 L
Mistroke
.58976 .42614 L
.59168 .43151 L
.59353 .43553 L
.59448 .43717 L
.59496 .43789 L
.59509 .43807 L
.59517 .43817 L
.5952 .43822 L
.59523 .43826 L
.59527 .43795 L
.5953 .43758 L
.59537 .43679 L
.59549 .4354 L
.59749 .41294 L
.59942 .3917 L
.60127 .37163 L
.60177 .36632 L
.60201 .36373 L
.60223 .36139 L
.60229 .36078 L
.60235 .36011 L
.60241 .36002 L
.60247 .36038 L
.60272 .36203 L
.60325 .36547 L
.60514 .37698 L
.60695 .38699 L
.60889 .39649 L
.61075 .4045 L
.61273 .41179 L
.61463 .41762 L
.61647 .42213 L
.61749 .42418 L
.61842 .42576 L
.61864 .4261 L
.61888 .42644 L
.61894 .42652 L
.61901 .42661 L
.61904 .42666 L
.61908 .42635 L
.61914 .42561 L
.61938 .4229 L
.6203 .41271 L
.62221 .39164 L
.62405 .37173 L
.62508 .36076 L
.62556 .35562 L
.62577 .35338 L
.62589 .35211 L
.62601 .35092 L
Mistroke
.62612 .34977 L
.62618 .34912 L
.62623 .34927 L
.62629 .34967 L
.62636 .35009 L
.62647 .35087 L
.62698 .35416 L
.62789 .35993 L
.62986 .37146 L
.63175 .38143 L
.63357 .38996 L
.6355 .39791 L
.63737 .40444 L
.63935 .41019 L
.64034 .4126 L
.64126 .41457 L
.64215 .41622 L
.6424 .41665 L
.64264 .41703 L
.64275 .4172 L
.6428 .41728 L
.64286 .41735 L
.64292 .4167 L
.64298 .41599 L
.64309 .41474 L
.64496 .39409 L
.64695 .37254 L
.64795 .36192 L
.64887 .35217 L
.64933 .34733 L
.64976 .3428 L
.64987 .34169 L
.64992 .34109 L
.64998 .34053 L
.65004 .34053 L
.65007 .34074 L
.6501 .34096 L
.65021 .34172 L
.65071 .34506 L
.6527 .35767 L
.65463 .36868 L
.65647 .37821 L
.65844 .38718 L
.66034 .39468 L
.66216 .40085 L
.6641 .40628 L
.66499 .40839 L
.66596 .41041 L
.66622 .41089 L
.66636 .41114 L
Mistroke
.66649 .41138 L
.66655 .41148 L
.66661 .41159 L
.66665 .41165 L
.66668 .4115 L
.66672 .41113 L
.66675 .41079 L
.66698 .40821 L
.66795 .39755 L
.66982 .37718 L
.67181 .35593 L
.67281 .34544 L
.67325 .34086 L
.67348 .33843 L
.6736 .33716 L
.67367 .33647 L
.67373 .33583 L
.67378 .33525 L
.67384 .33522 L
.67388 .33546 L
.67391 .33569 L
.67397 .3361 L
.67419 .33764 L
.67462 .34062 L
.67557 .34694 L
.67742 .35849 L
.67939 .36964 L
.68129 .37925 L
.68311 .38744 L
.68506 .39502 L
.68692 .40122 L
.68788 .40396 L
.68891 .40662 L
.68939 .40774 L
.68991 .40886 L
.69015 .40938 L
.69028 .40962 L
.69033 .40973 L
.69039 .40985 L
.69044 .40995 L
.69049 .40986 L
.69054 .40928 L
.6906 .40866 L
.69083 .40615 L
.69282 .38432 L
.69474 .36368 L
.69659 .34418 L
.69708 .33903 L
.69732 .33653 L
.69743 .33543 L
Mistroke
.69754 .33424 L
.6976 .3336 L
.69766 .33372 L
.69771 .33412 L
.69777 .33454 L
.69802 .33633 L
.69855 .34007 L
.70055 .35344 L
.70248 .36517 L
.70433 .37538 L
.7063 .38507 L
.7082 .39325 L
.71003 .40006 L
.71197 .40616 L
.71287 .40857 L
.71334 .40974 L
.71385 .41091 L
.7141 .41147 L
.71417 .41162 L
.7142 .4117 L
.71424 .41178 L
.71428 .41185 L
.71431 .41158 L
.71435 .41121 L
.71438 .41087 L
.71487 .40543 L
.71583 .39479 L
.71771 .37442 L
.71971 .35316 L
.72019 .34804 L
.72071 .34265 L
.72115 .33807 L
.72127 .33682 L
.72133 .33613 L
.7214 .33548 L
.72146 .33537 L
.72149 .33561 L
.72152 .33584 L
.72163 .33667 L
.72348 .34981 L
.72537 .36213 L
.72737 .37402 L
.7293 .38428 L
.73115 .39306 L
.73313 .40119 L
.73503 .40785 L
.73686 .41317 L
.73734 .41441 L
.7378 .4155 L
.73792 .41578 L
Mistroke
.73799 .41594 L
.73805 .41608 L
.73808 .41615 L
.73812 .4159 L
.73816 .41547 L
.73819 .41507 L
.73833 .4136 L
.73881 .40828 L
.74071 .38733 L
.74255 .36755 L
.74357 .35666 L
.7445 .34687 L
.74497 .342 L
.74509 .34067 L
.74516 .33995 L
.7452 .33959 L
.74523 .33927 L
.74528 .33953 L
.74534 .33998 L
.74547 .34093 L
.74638 .34764 L
.7483 .3609 L
.75014 .37257 L
.7521 .38381 L
.75399 .39348 L
.7558 .4017 L
.75773 .40929 L
.75959 .41546 L
.76054 .41816 L
.76103 .41947 L
.76131 .42016 L
.76157 .42079 L
.76169 .42108 L
.76182 .42137 L
.76185 .42145 L
.76188 .42153 L
.76192 .4214 L
.76196 .421 L
.76208 .41956 L
.76255 .41432 L
.76347 .40414 L
.76546 .38246 L
.76737 .36195 L
.76786 .35669 L
.76833 .35175 L
.76875 .34733 L
.76887 .34613 L
.76893 .34547 L
.76899 .34486 L
.76904 .34433 L
Mistroke
.76909 .34456 L
.76915 .34501 L
.7692 .34542 L
.77015 .35244 L
.77116 .35962 L
.77308 .37238 L
.77492 .38355 L
.77688 .39423 L
.77877 .40334 L
.78059 .411 L
.78252 .41797 L
.78439 .42353 L
.78489 .42483 L
.78535 .42596 L
.78559 .42651 L
.78565 .42667 L
.78569 .42674 L
.78572 .42678 L
.78577 .42613 L
.78584 .42544 L
.78637 .41949 L
.78826 .39848 L
.79009 .37863 L
.7911 .36773 L
.79203 .35787 L
.79248 .35303 L
.79261 .3517 L
.79274 .35029 L
.79278 .34996 L
.79281 .34961 L
.79287 .34921 L
.79293 .34966 L
.79298 .35007 L
.79389 .35682 L
.7958 .37007 L
.79763 .38173 L
.79957 .39295 L
.80145 .40258 L
.80344 .41155 L
.80536 .41896 L
.80721 .42495 L
.80814 .42755 L
.80864 .42881 L
.80891 .42948 L
.80917 .43008 L
.80929 .43035 L
.80942 .43063 L
.80945 .43071 L
.80949 .43079 L
.80952 .43086 L
Mistroke
.80956 .43051 L
.80968 .42907 L
.81015 .42382 L
.81107 .41359 L
.81304 .39186 L
.81494 .37132 L
.81589 .36113 L
.81631 .3567 L
.81642 .35549 L
.81654 .35421 L
.81659 .35369 L
.81665 .35312 L
.81671 .35319 L
.81676 .35361 L
.81724 .35718 L
.8177 .3605 L
.8187 .36757 L
.82068 .38055 L
.82259 .39184 L
.82442 .40159 L
.82637 .41075 L
.82825 .41838 L
.83024 .42521 L
.83124 .42814 L
.83216 .43056 L
.83263 .43166 L
.83306 .43263 L
.83317 .43287 L
.83323 .43299 L
.83328 .4331 L
.83334 .43313 L
.83337 .43278 L
.8334 .43241 L
.83352 .43115 L
.83401 .42553 L
.83602 .40324 L
.83794 .38217 L
.83883 .37262 L
.83933 .36724 L
.8398 .36225 L
.84004 .35972 L
.84026 .35733 L
.84039 .35601 L
.84042 .35565 L
.84046 .35527 L
.84049 .35519 L
.84052 .35544 L
.84076 .35719 L
.84177 .36446 L
.84371 .37744 L
Mistroke
.84557 .38878 L
.84755 .3996 L
.84946 .40881 L
.8513 .41653 L
.85325 .42352 L
.85513 .42906 L
.85609 .43144 L
.85659 .43257 L
.85687 .43316 L
.857 .43343 L
.85706 .43355 L
.85709 .43362 L
.85713 .43368 L
.85719 .43322 L
.85724 .43258 L
.85736 .43131 L
.8576 .42852 L
.85813 .42262 L
.85905 .41232 L
.86086 .39234 L
.86187 .38139 L
.8628 .37144 L
.8637 .36177 L
.86392 .35944 L
.86416 .35695 L
.86422 .35621 L
.86426 .35585 L
.86429 .3556 L
.86435 .35603 L
.86441 .3565 L
.86465 .35824 L
.86655 .37132 L
.86837 .38281 L
.8703 .39387 L
.87217 .40336 L
.87415 .41219 L
.87606 .41948 L
.8779 .42536 L
.87893 .42815 L
.87985 .43039 L
.88032 .43141 L
.88057 .43192 L
.8807 .43218 L
.88082 .43243 L
.88088 .43254 L
.88093 .43264 L
.881 .43219 L
.88105 .43156 L
.8813 .42875 L
.88174 .42383 L
Mistroke
.88358 .40332 L
.88554 .38188 L
.88652 .37139 L
.88699 .36628 L
.88743 .36162 L
.88786 .35711 L
.88796 .35601 L
.88802 .35541 L
.88807 .35484 L
.88813 .35488 L
.88819 .3553 L
.88831 .35611 L
.88925 .36273 L
.8911 .37492 L
.89307 .38669 L
.89496 .39683 L
.89678 .40549 L
.89872 .41351 L
.90059 .42006 L
.90258 .42578 L
.90357 .42816 L
.90405 .4292 L
.90449 .43008 L
.9046 .43028 L
.90466 .4304 L
.90471 .4305 L
.90477 .4305 L
.9048 .43014 L
.90484 .42976 L
.90495 .42848 L
.90538 .42362 L
.90633 .41302 L
.90832 .39108 L
.91024 .37033 L
.91112 .36096 L
.91136 .3584 L
.91162 .35566 L
.91174 .3544 L
.9118 .35372 L
.91186 .35308 L
.91192 .35273 L
.91197 .35312 L
.91209 .35396 L
.91304 .36062 L
.91405 .36737 L
.91605 .37977 L
.91798 .3905 L
.91983 .39971 L
.9218 .40827 L
.9237 .41533 L
Mistroke
.92552 .42101 L
.92654 .4237 L
.92747 .42587 L
.92793 .42684 L
.92817 .42733 L
.9283 .42759 L
.92843 .42782 L
.92848 .42792 L
.92853 .42802 L
.92859 .42783 L
.92865 .4272 L
.9289 .42441 L
.92934 .41951 L
.93117 .3992 L
.93312 .37798 L
.93402 .36831 L
.93499 .35791 L
.93525 .35517 L
.93539 .35368 L
.93553 .35228 L
.93564 .35109 L
.9357 .35042 L
.93576 .35065 L
.93582 .3511 L
.93589 .35159 L
.93602 .35249 L
.93699 .3592 L
.93894 .37187 L
.94083 .38291 L
.94283 .39341 L
.94476 .40231 L
.94661 .40974 L
.94859 .41642 L
.95049 .42166 L
.95144 .42387 L
.95186 .42474 L
.95208 .42517 L
.9522 .4254 L
.95231 .42562 L
.95237 .42572 L
.95243 .42515 L
.95256 .42368 L
.9528 .42108 L
.95325 .41597 L
.95426 .40481 L
.95624 .38315 L
.95723 .37249 L
.95815 .36267 L
.95903 .35327 L
.95929 .35058 L
Mistroke
.95941 .34926 L
.95947 .34869 L
.95953 .34816 L
.95958 .34855 L
.95963 .34891 L
.95975 .3497 L
.95998 .35136 L
.96185 .364 L
.96384 .37623 L
.96575 .38682 L
.96759 .39592 L
.96955 .4044 L
.97143 .4114 L
.97344 .41758 L
.97537 .42232 L
.97619 .42398 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  Background->GrayLevel[1],
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005X0000gR000`400?l00000o`00003oo`3ooomY
0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`006P3oool0
0`000000oooo0?ooo`3o0?oood`0oooo000J0?ooo`800000o`3ooom=0?ooo`006P3oool00`000000
oooo0?ooo`130?ooo`@000001@3oool2000000D0oooo1@00000l0?ooo`@000001@3oool2000000@0
oooo1@00000l0?ooo`@000001@3oool2000000D0oooo0`00000m0?ooo`@000001@3oool2000000L0
oooo0`0000020?ooo`006P3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool00P3oool2000000L0oooo00<000000?ooo`3oool0>`3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo0080oooo0P0000040?ooo`050000003oool0oooo0?ooo`00
0000>`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo0P0000040?ooo`05
0000003oool0oooo0?ooo`000000>`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0080oooo0P0000080?ooo`030000003oool0oooo0040oooo000J0?ooo`030000003oool0oooo0480
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo03/0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo03`0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo03T0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0040
oooo000J0?ooo`040000003oool0oooo0?l000L0oooo00<0o`000?ooo`3oool0=`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0>`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0>`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0>@3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00P0oooo1P0000020?ooo`006P3oool200000080o`00
00<0o`3o0?ooo`3oool00`3oool20?l003T0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0:0?ooo`800000?03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00T0oooo
00@000000?ooo`3oool000000`3oool001X0oooo00D000000?ooo`3o0000o`000?l0o`050?ooo`80
o`00>@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3o
ool0>`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool0>@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool0>@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00T0oooo00@000000?ooo`3o
ool000000`3oool001X0oooo00D000000?ooo`3o0000o`000?l0o`050?ooo`80o`00>@3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo00T0oooo0`00000m0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0203oool01@000000oooo0?ooo`3oool0000003/0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`080?ooo`050000003oool0oooo0?ooo`000000>`3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00X0oooo00<000000?ooo`0000000`3o
ool001X0oooo00D000000?ooo`3o0000o`000?l0o`050?ooo`80o`00>P3oool4000000h0oooo00<0
00000?ooo`3oool0?03oool4000000`0oooo0`00000m0?ooo`@00000303oool3000003d0oooo1000
000>0?ooo`8000000`3oool001X0oooo00H000000?ooo`3o0000oooo0?l0003o0?l40?ooo`80o`00
00<0o`3o0?ooo`3oool0o`3ooom00?ooo`006P3oool2000000040?l0003oool0o`000?l0o`80003o
0P3oool20?l000030?l0o`3oool0oooo0?l0oooo@03oool001X0oooo00<000000?ooo`3o00000P00
0?l00`3o0000o`3o0000o`020?ooo`030?l0003oool0o`000?l0oooo@P3oool001X0oooo00<00000
0?ooo`3o00000P3oool0203o0000o`3o0?ooo`000?l0o`000?l0o`3oool0o`00o`3ooom20?ooo`00
6P3oool00`000000oooo0?l000020?ooo`050?l0003o0?l0oooo0?ooo`3o00000P3oool00`3o0000
oooo0?ooo`3o0?oood00oooo000J0?ooo`030000003o0?l0o`000080oooo00P0o`000?ooo`3o0?l0
oooo0?l000000?l0oooo0?l00?l0oooo@P3oool001X0oooo00<0003o0?l0003oool00`3oool0203o
0000o`3o0?ooo`3o0000003o0?ooo`3o0000o`3oo`3ooom10?ooo`004P3oool8000000030000o`3o
0000000000<0000000P0o`000?l0o`000000o`00000000000?l000000?l00?l00000?@0000040?oo
o`006P3oool00`000000o`000?ooo`040?ooo`070?l0003o0?l0o`000?ooo`000?l0oooo0?l00002
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02P3oool001X0oooo00<000000?l0
003oool0103oool01`3o0000o`3o0?l0003oool0003o0?ooo`3o0000@P3oool00`000000oooo0?oo
o`1>0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?oo
o`0:0?ooo`006P3oool00`000000o`000?ooo`040?ooo`030?l0003o0?l0o`000080oooo00<0003o
0?l0003oool0o`3ooom00?ooo`006P3oool00`3o0000oooo0?ooo`050?ooo`80o`000P3oool00`00
0?l0o`000?ooo`3o0?oood00oooo000J0?ooo`030?l000000000oooo00D0oooo00<0o`000?ooo`3o
ool00P3oool00`3o0000oooo0?ooo`3o0?ooocl0oooo000J0?ooo`030?l0003oool0oooo00D0oooo
00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?ooo`3o0?ooocl0oooo000J0?ooo`030000003o
ool0oooo00X0oooo00<0o`000?ooo`3oool0o`3ooolo0?ooo`006P3oool00`000000oooo0?ooo`0:
0?ooo`030?l0003o0?l0oooo0?l0oooo?`3oool001X0oooo00<000000?ooo`3oool02P3oool00`3o
0000o`3o0?ooo`030?ooo`030?l0003oool0oooo0?l0oooo>@3oool001X0oooo00<000000?ooo`3o
ool02`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0oooo0?l0oooo>@3oool001X0oooo
0P00000<0?ooo`040?l000000?l0oooo0?ooo`80o`00o`3ooolk0?ooo`006P3oool00`000000oooo
0?ooo`0;0?ooo`040?l000000?l0oooo0?ooo`80o`00o`3ooolk0?ooo`006P3oool00`000000oooo
0?ooo`0;0?ooo`040?l000000?l0oooo0?ooo`80o`00o`3ooolk0?ooo`006P3oool00`000000oooo
0?ooo`0;0?ooo`040?l0003o0?l0003o0?ooo`80o`00o`3ooolk0?ooo`006P3oool00`000000oooo
0?ooo`0;0?ooo`040?l0003o0?l0003o0?ooo`80o`00o`3ooolk0?ooo`006P3oool2000000`0oooo
00@0o`000?l0o`000?l0oooo0P3o00000`3o0?l0oooo0?ooo`3o0?ooocP0oooo000J0?ooo`030000
003oool0oooo00/0oooo00L0o`000?l0o`000?l0oooo0?l0003oool0o`000?l0oooo>P3oool001X0
oooo00<000000?ooo`3oool02`3oool01`3o0000o`3o0?ooo`000?l0o`000?ooo`3o0000o`3ooolj
0?ooo`006P3oool00`000000oooo0?ooo`0;0?ooo`070?l0003o0?l0oooo0000o`3o0000oooo0?l0
003o0?ooocX0oooo000J0?ooo`030000003oool0oooo00`0oooo00H0o`000?ooo`3o0?l0o`000?oo
o`3o003o0?ooocX0oooo000J0?ooo`8000003@3oool01P3o0000oooo0?l000000?l0oooo0?l00?l0
oooo>P3oool001X0oooo00<000000?ooo`3oool0303oool01P3o0000oooo0?l000000?l0oooo0?l0
0?l0oooo>P3oool001X0oooo00<000000?ooo`3oool0303oool01P3o0000o`3o0?l000000?l0oooo
0?l00?l0oooo>P3oool000@000001@3oool2000000D0oooo100000060?ooo`030000003oool0oooo
00`0oooo00H0o`000?l0o`3o0000003o0?ooo`3o003o0?ooocX0oooo00040?ooo`030000003oool0
oooo0080oooo0P0000040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0<0?ooo`060?l0003o0?l0o`000?ooo`000?l0o`00o`3ooolj0?ooo`00103o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03@3o
ool01@3o0000oooo0?ooo`000?l0o`000?l0oooo>P3oool000@0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`030?ooo`<000003@3oool01P3o0000oooo0?ooo`000?l0o`000?l0ool0
oooo>@3oool000@00000303oool4000000H0oooo00<000000?ooo`3oool03@3oool01P3o0000oooo
0?ooo`000?l0o`000?l0ool0oooo>@3oool000030000003oool0oooo00d0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`0=0?ooo`060?l0003oool0oooo0?ooo`3o0000o`3oo`3oooli
0?ooo`0000<000000?ooo`3oool03@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
00d0oooo00H0o`3o0?ooo`3oool0oooo0000o`3o003o0?ooocT0oooo0005000000/0oooo1@000005
0?ooo`030000003oool0oooo0140oooo00<0003o0?l0003oool0o`3ooolh0?ooo`006P3oool20000
0180oooo00<0003o0?l0003oool0o`3ooolh0?ooo`006P3oool00`000000oooo0?ooo`0B0?ooo`03
0?l0003oool0oooo0?l0oooo=`3oool001X0oooo00<000000?ooo`3oool04P3oool00`3o0000oooo
0?ooo`3o0?ooocL0oooo000J0?ooo`030000003oool0oooo0180oooo00<0o`000?ooo`3oool0o`3o
oolg0?ooo`006P3oool00`000000oooo0?ooo`0B0?ooo`030?l0003oool0oooo0?l0oooo=`3oool0
01X0oooo00<000000?ooo`3oool04P3oool00`3o0000003o0?ooo`3o0?ooocL0oooo000J0?ooo`80
00004`3oool00`3o0000003o0?ooo`3o0?ooocL0oooo000J0?ooo`030000003oool0oooo0180oooo
00<0o`000?l0o`3oool0o`3ooolg0?ooo`006P3oool00`000000oooo0?ooo`0B0?ooo`030?l0003o
0?l0oooo0?l0oooo=`3oool001X0oooo00<000000?ooo`3oool04P3oool00`3o0000o`3o0?ooo`3o
0?ooocL0oooo000J0?ooo`030000003oool0oooo0180oooo00<0o`000?l0o`3oool0o`3ooolg0?oo
o`006P3oool2000001<0oooo00<0o`000?l0o`000?l0o`3ooolg0?ooo`006P3oool00`000000oooo
0?ooo`0C0?ooo`030?l000000?l0oooo0?l0oooo=P3oool001X0oooo00<000000?ooo`3oool04`3o
ool00`3o0000003o0?ooo`020?ooo`030?l0003oool0oooo0?l0oooo<@3oool001X0oooo00<00000
0?ooo`3oool04`3oool0103o0000003o0?ooo`3oool20?l00?l0oooo<`3oool001X0oooo00<00000
0?ooo`3oool04`3oool0103o0000003o0?ooo`3oool20?l00?l0oooo<`3oool001X0oooo00<00000
0?ooo`3oool04`3oool0103o0000o`3o0?ooo`3oool20?l00?l0oooo<`3oool001X0oooo0P00000D
0?ooo`040?l0003o0?l0003o0?ooo`80o`00o`3ooolc0?ooo`006P3oool00`000000oooo0?ooo`0C
0?ooo`040?l0003o0?l0003o0?ooo`80o`00o`3ooolc0?ooo`004@3oool3000000H0oooo00<00000
0?ooo`3oool0503oool00`3o0000003o0?ooo`020?l00?l0oooo<`3oool00100oooo00D000000?oo
o`3oool0oooo000000050?ooo`030000003oool0oooo01@0oooo00<0o`000000o`3oool00P3o003o
0?oooc<0oooo000@0?ooo`050000003oool0oooo0?ooo`0000001@3oool00`000000oooo0?ooo`0D
0?ooo`030?l000000?l0oooo0080o`00o`3ooolc0?ooo`00403oool01@000000oooo0?ooo`3oool0
000000D0oooo0`00000D0?ooo`060?l0003oool0o`000?l0o`3o0000o`3oo`3ooolb0?ooo`00403o
ool4000000H0oooo00<000000?ooo`3oool0503oool01P3o0000oooo0?l0003o0?l0o`000?l0ool0
oooo<P3oool00100oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`060?l0
003oool0o`000?l0o`3o0000o`3oo`3ooolb0?ooo`004@3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo01@0oooo00H0o`000?l0o`3o0000o`3o0?ooo`3o003o0?oooc80oooo000B0?oo
o`<000001@3oool00`000000oooo0?ooo`0D0?ooo`030?l0003o0?l0o`000080oooo00<0o`000?oo
o`3oool0o`3oool`0?ooo`006P3oool2000001D0oooo00<0o`000?l0o`3o00000P3oool00`3o0000
oooo0?ooo`3o0?oooc00oooo000J0?ooo`030000003oool0oooo01@0oooo00H0o`000?l0o`3o0000
003o0?ooo`3o003o0?oooc80oooo000J0?ooo`030000003oool0oooo01D0oooo0P3o00000`000?l0
oooo0?l0003o0?oooc80oooo000J0?ooo`030000003oool0oooo01D0oooo0P3o00000`000?l0oooo
0?l0003o0?oooc80oooo000J0?ooo`030000003oool0oooo01D0oooo0P3o00000`000?l0oooo0?l0
003o0?oooc80oooo000J0?ooo`030000003oool0oooo01D0oooo0P3o00000`000?l0oooo0?l0003o
0?oooc80oooo000J0?ooo`8000005P3oool01@3o0000oooo0000o`3oool0o`000?l0oooo<P3oool0
01X0oooo00<000000?ooo`3oool05@3oool01P3o0000oooo0?ooo`000?l0oooo0?l00?l0oooo<@3o
ool001X0oooo00<000000?ooo`3oool05@3oool01P3o0000oooo0?ooo`000?l0oooo0?l00?l0oooo
<@3oool001X0oooo00<000000?ooo`3oool05@3oool01P3o0?l0oooo0?ooo`000?l0oooo0?l00?l0
oooo<@3oool001X0oooo00<000000?ooo`3oool0603oool00`000?l0oooo0?l0003o0?oooc40oooo
000J0?ooo`8000006@3oool00`000?l0oooo0?l0003o0?oooc40oooo000J0?ooo`030000003oool0
oooo01T0oooo00<0003o0?l0003oool0o`3oool`0?ooo`006P3oool00`000000oooo0?ooo`0I0?oo
o`030000o`3o0000oooo0?l0oooo<03oool001X0oooo00<000000?ooo`3oool06@3oool00`000?l0
o`000?ooo`3o0?oooc00oooo000J0?ooo`030000003oool0oooo01T0oooo00<0003o0?l0003oool0
o`3oool`0?ooo`006P3oool00`000000oooo0?ooo`0I0?ooo`030000o`3o0000oooo0?l0oooo<03o
ool001X0oooo0P00000K0?ooo`030?l0003oool0oooo05<0oooo00<0o`000?ooo`3oool0f03oool0
01X0oooo00<000000?ooo`3oool06P3oool00`3o0000oooo0?ooo`1C0?ooo`030?l0003oool0oooo
00D0oooo00<0o`000?ooo`3oool0d03oool000040?ooo`0000000000000000D0oooo0P0000050?oo
o`@000001P3oool00`000000oooo0?ooo`0J0?ooo`030?l0003oool0oooo05<0oooo00<0o`000?oo
o`3oool01@3oool00`3o0000oooo0?ooo`3@0?ooo`0000D000000?ooo`3oool0oooo000000040?oo
o`800000103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool06P3oool00`3o0000oooo0?ooo`1C0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?oo
o`3oool0d03oool000050000003oool0oooo0?ooo`0000003`3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo01X0oooo00<0o`000?l0o`3oool0B`3oool00`3o0000oooo0?ooo`050?oo
o`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0d03oool000050000003oool0oooo0?oo
o`0000003`3oool00`000000oooo0?ooo`030?ooo`<000006`3oool00`3o0000oooo0?ooo`1:0?oo
o`030?l0003oool0oooo00D0oooo0P3o00060?ooo`030?l0003oool0oooo0=00oooo0004000000`0
oooo100000060?ooo`030000003oool0oooo01/0oooo00<0o`000?ooo`3oool0BP3oool00`3o0000
oooo0?ooo`050?ooo`80o`001P3oool20?l00=40oooo00000`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool06`3oool00`3o0000oooo0?ooo`020?ooo`030?l0
003oool0oooo04D0oooo00<0o`000?ooo`3oool0103oool00`3o0000o`3o0?l000050?ooo`030?l0
003o0?l0o`0000H0oooo00<0o`000?ooo`3oool0b03oool000030?ooo`000000oooo00d0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`0K0?ooo`030?l0003oool0oooo0080oooo00<0
o`000?ooo`3oool0A03oool20?l000030?l0o`3oool0oooo00<0oooo00@0o`000?l0o`3o0000o`3o
103oool00`3o0000o`3o0?l000060?ooo`030?l0003oool0oooo0<P0oooo00020?ooo`<000002`3o
ool5000000D0oooo00<000000?ooo`3oool06`3oool00`3o0000003o0?ooo`020?ooo`030?l0003o
ool0oooo04@0oooo0P3o00000`3o0?l0oooo0?ooo`030?ooo`040?l0003oool0oooo0?l000@0oooo
00<0o`000?l0o`3o00001P3oool00`3o0000oooo0?ooo`380?ooo`006P3oool2000001`0oooo00@0
o`000000o`3oool0oooo0P3o00160?ooo`030?l0003o0?l0o`0000D0oooo00@0o`000?ooo`3oool0
o`00103oool0103o0000oooo0?l0003o0?l50?ooo`030?l0003oool0oooo0<P0oooo000J0?ooo`03
0000003oool0oooo01/0oooo00@0o`000000o`3oool0oooo0P3o00160?ooo`030?l0003oool0o`00
00D0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?l0003o0?l50?ooo`030?l0003o
ool0oooo0<P0oooo000J0?ooo`030000003oool0oooo01/0oooo00@0o`000000o`3oool0oooo0P3o
00160?ooo`030?l0003oool0o`0000D0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo
0?l0003o0?l50?ooo`80o`00b@3oool001X0oooo00<000000?ooo`3oool06`3oool0103o0000o`3o
0?ooo`3oool20?l004H0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?ooo`3o0000
o`3o00<0oooo00@0o`000?ooo`3oool0o`00103oool00`3o0000o`3o0?l000390?ooo`006P3oool0
0`000000oooo0?ooo`0K0?ooo`040?l0003o0?l0003o0?ooo`80o`00?`3oool00`3o0000oooo0?oo
o`040?ooo`040?l0003oool0oooo0?l000<0oooo00H0o`000?l0o`000?l0003o0?l0003o0?l30?oo
o`040?l0003oool0oooo0?l000@0oooo00<0o`000?l0o`3o0000b@3oool001X0oooo00<000000?oo
o`3oool06`3oool0103o0000o`3o0000o`3oool20?l003l0oooo00<0o`000?ooo`3oool0103oool0
1@3o0000oooo0?ooo`3o0000oooo0080003o00<0o`000?ooo`3oool00P3oool01@3o0000003o0000
o`3oool0o`000080oooo00<0o`000?ooo`3oool00P3oool00`3o0000o`3o0?l000060?ooo`030?l0
003oool0oooo0<00oooo000J0?ooo`800000703oool01`3o0000o`3o0000o`3oool0o`000?l0o`3o
0000?P3oool00`3o0000oooo0?ooo`040?ooo`050?l0003oool0oooo0?l0003o0?l00P3oool00`3o
0000oooo0?ooo`020?ooo`050?l0003oool0oooo0?l0o`3o00000P3oool00`3o0000oooo0?ooo`02
0?ooo`030?l0003oool0o`0000H0oooo00<0o`000?ooo`3oool0`03oool001X0oooo00<000000?oo
o`3oool06`3oool01`3o0000o`3o0000o`3oool0o`000?l0o`3o0000?P3oool20?l000D0oooo00D0
o`000?ooo`000?l0o`000?l0o`020?ooo`030?l0003oool0oooo0080oooo00D0o`000?ooo`3oool0
o`3o0?l000020000o`030?l0003o0?l0oooo0080oooo00@0o`000?ooo`3oool0o`001@3oool00`3o
0000oooo0?ooo`300?ooo`006P3oool00`000000oooo0?ooo`0L0?ooo`060?l0003oool0003o0?l0
003oool0o`00?@3oool00`3o0000o`3o0?l000050?ooo`050?l000000?l0oooo0?ooo`3o00000P3o
ool00`3o0000oooo0?ooo`020?ooo`040?l0003oool0oooo0?l000<0oooo00<0003o0?l0003oool0
0P3oool0103o0000oooo0?ooo`3o00050?ooo`030?l0003oool0oooo0<00oooo000J0?ooo`030000
003oool0oooo01`0oooo00H0o`000?ooo`000?l0o`000?ooo`3o000m0?ooo`030?l0003o0?l0o`00
00D0oooo00P0o`000?ooo`3oool0oooo0?l0003o0?l0oooo0?l000@0oooo00@0o`000?l0o`3oool0
o`00103oool01@3o0000003o0?ooo`3oool0o`000080oooo00<0o`000?ooo`3oool00`3oool20?l0
05H0oooo00<0o`000?ooo`3oool0J03oool001X0oooo00<000000?ooo`3oool0703oool01P3o0000
oooo0000o`3o0000oooo0?l003d0oooo00<0o`000?l0o`3o0000103oool00`000?l0o`000?ooo`03
0?ooo`030?l0003oool0o`0000@0oooo00@0o`000?l0o`3oool0o`00103oool01@3o0000oooo0000
o`3oool0o`000080oooo00<0o`000?ooo`3oool00`3oool20?l004h0oooo00<0o`000?ooo`3oool0
1@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo0600oooo000J0?ooo`8000007@3o
ool01P3o0000oooo0?l0003o0?l0oooo0?l003d0oooo00<0o`000?l0o`3o00000`3oool00`000?l0
o`3o0?l000040?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o0000103oool01@3o0000
oooo0?ooo`3o0000o`3o0080oooo00<0o`000?ooo`3oool00P3oool00`3o0000o`3o0?l000160?oo
o`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo00<0o`000?oo
o`3oool01@3oool00`3o0000oooo0?ooo`1H0?ooo`006P3oool00`000000oooo0?ooo`0L0?ooo`06
0?l0003oool0o`000?l0o`3oool0o`00?@3oool0203o0000oooo0?l0003o0?l0oooo0000o`3oool0
o`001@3oool00`3o0000o`3o0?l000050?ooo`030?l0003oool0o`0000@0oooo00D0o`000?ooo`3o
ool0o`000?l0o`020?ooo`030?l0003oool0oooo0080oooo00<0o`000?l0o`3o0000AP3oool00`3o
0000oooo0?ooo`050?ooo`80o`001P3oool20?l000H0oooo00<0o`000?ooo`3oool01@3oool00`3o
0000oooo0?ooo`1H0?ooo`006P3oool00`000000oooo0?ooo`0L0?ooo`060?l0003o0?l0o`000000
o`3oool0o`00?@3oool01@3o0000oooo0?ooo`3o0000003o0080oooo00<0o`000?ooo`3oool0103o
ool20?l000H0oooo0P3o00040?ooo`080?l0003o0?l0oooo0?l0003oool0003o0?ooo`3o00040?oo
o`030?l0003o0?l0o`0000H0oooo00<0o`000?ooo`3oool0?@3oool20?l000H0oooo0P3o00060?oo
o`80o`001P3oool20?l000H0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`1@0?oo
o`006P3oool00`000000oooo0?ooo`0L0?ooo`060?l0003o0?l0o`000000o`3oool0o`00?@3oool0
1@3o0000oooo0?ooo`3o0000o`3o0080oooo00<0o`000?ooo`3oool0103oool20?l000H0oooo0P3o
00050?ooo`030?l0003oool0o`000080oooo00<0003o0?l0003oool00`3oool00`3o0000oooo0?l0
00060?ooo`030?l0003oool0oooo03D0oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo0P3o
00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`030?l0003oool0oooo00D0oooo00<0
o`000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0?0?ooo`006P3oool00`000000oooo0?ooo`0L
0?ooo`060?l0003o0?l0o`000000o`3oool0o`00?@3oool01@3o0000oooo0000o`3oool0o`000080
oooo00<0o`000?ooo`3oool0103oool20?l0o`H0oooo0P3o00050?ooo`030?l0003oool0o`0000<0
oooo00<0o`000?l0o`3oool00P3oool00`3o0000oooo0?l000060?ooo`030?l0003oool0oooo03D0
oooo00<0o`000?ooo`3oool01@3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0
oooo0P3o00060?ooo`80o`001P3oool00`3o0000oooo0?ooo`0^0?ooo`030?l0003oool0oooo00D0
oooo00<0o`000?ooo`3oool03`3oool001X0oooo00<000000?ooo`3oool0703oool01P3o0000o`3o
0?l000000?l0oooo0?l003d0oooo00P0o`000?ooo`000?l0oooo0?l0003o0?l0oooo0?l000H0oooo
0P3o0?l60?ooo`80o`001@3oool00`3o0000oooo0?l000040?ooo`080?l0003oool0oooo0?ooo`3o
0000oooo0?l0003o0?l50?ooo`030?l0003oool0oooo03D0oooo0P3o00060?ooo`80o`001P3oool2
0?l000D0oooo00@0o`000?l0o`3o0000o`3o1@3oool20?l000H0oooo0P3o00060?ooo`80o`001P3o
ool00`3o0000oooo0?ooo`050?ooo`80o`001P3oool20?l000l0oooo00<0o`000?ooo`3oool01@3o
ool00`3o0000oooo0?ooo`040?ooo`80o`001P3oool20?l00140oooo000J0?ooo`8000007P3oool2
0?l000030000o`3oool0o`0003d0oooo00<0o`000000o`3oool00P3oool00`3o0000oooo0?l0000>
0?ooo`80o`3o1@3oool00`3o0000o`3o0?l000040?ooo`080?l000000?l0oooo0?ooo`3o0000oooo
0?l0003o0?l50?ooo`030?l0003oool0oooo03D0oooo0P3o00060?ooo`80o`001P3oool00`3o0000
o`3o0?l000040?ooo`040?l0003o0?l0o`000?l0o`D0oooo0P3o00000`3o0?l0oooo0?ooo`030?oo
o`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?oo
o`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`004@3oool001X0oooo00<000000?ooo`3o
ool07@3oool01@3o0000o`3o0?ooo`000?l0o`0003d0oooo00<0o`000000o`3oool00`3oool20?l0
00l0oooo00<0o`3o0?ooo`3oool00`3oool00`3o0000o`3o0?l000040?ooo`050?l0003oool0003o
0?ooo`3o00000P3oool00`3o0000oooo0?ooo`030?ooo`80o`00;P3oool00`3o0000oooo0?ooo`05
0?ooo`80o`001P3oool20?l000H0oooo00<0o`000?l0o`3o0000103oool0103o0000o`3o0?l0003o
0?l50?ooo`030?l0003oool0o`0000D0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o
00060?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o
000A0?ooo`004P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01d0oooo00H0o`00
0?l0o`3oool0003o0?l0003o0?ll0?ooo`030?l0003oool0oooo00<0oooo0P3o000F0?ooo`80o`00
103oool01@3o0000oooo0?ooo`000?l0o`000080oooo00<0o`000?ooo`3oool00P3oool00`3o0000
o`3o0?l0000^0?ooo`030?l0003oool0oooo00D0oooo0P3o00060?ooo`80o`001P3oool00`3o0000
oooo0?l000040?ooo`040?l0003oool0oooo0?l000D0oooo00<0o`000?ooo`3o0000103oool0103o
0000o`3o0?l0003o0?l50?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool2
0?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o000A0?ooo`004P3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo01d0oooo00H0o`000?l0o`3oool0003o0?l0003o
0?le0?ooo`030?l0003oool0oooo00<0oooo00<0o`000?l0o`3oool0103oool20?l0oaH0oooo0P3o
00040?ooo`050?l0o`3o0000oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`030?l0
003o0?l0o`0000H0oooo00<0o`000?ooo`3oool09@3oool20?l000H0oooo0P3o00060?ooo`030?l0
003oool0o`0000D0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o00040?ooo`04
0?l0003o0?l0oooo0?l000@0oooo00@0o`000?l0o`3o0000o`3o1@3oool20?l000H0oooo0P3o0006
0?ooo`80o`001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo0P3o0006
0?ooo`80o`004@3oool00180oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0M0?oo
o`060?l0003o0?l0oooo0000o`3o0000o`3o=@3oool00`3o0000oooo0?ooo`030?ooo`030?l0003o
0?l0oooo00@0oooo0P3o0?lF0?ooo`80o`00103oool0203o0?l0o`000?ooo`3oool0o`000000o`3o
ool0o`00103oool00`3o0000o`3o0?l000060?ooo`030?l0003oool0oooo02D0oooo0P3o00060?oo
o`030?l0003o0?l0o`0000@0oooo00@0o`000?l0o`3oool0o`00103oool0103o0000oooo0?ooo`3o
00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?l0o`3oool0o`00103oool0103o0000
o`3o0?ooo`3o00050?ooo`030?l0003oool0o`0000D0oooo0P3o00060?ooo`80o`001P3oool20?l0
00H0oooo0P3o00060?ooo`80o`001P3oool20?l000H0oooo00<0o`000?l0o`3o00001@3oool20?l0
0140oooo000C0?ooo`030000003oool0oooo00@0oooo0`00000M0?ooo`060?l0003o0?l0oooo0?oo
o`3o0000o`3o=@3oool00`3o0000oooo0?ooo`020?ooo`030000o`3o0000oooo01d0oooo0P3o0005
0?ooo`080?l0003oool0oooo0?l0003oool0003o0?l0o`3o00030?ooo`030?l0003o0?l0o`0000H0
oooo00<0o`000?ooo`3oool07@3oool00`3o0000oooo0?ooo`050?ooo`80o`001P3oool00`3o0000
oooo0?l000040?ooo`040?l0003o0?l0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0
1@3o0000oooo0?ooo`3oool0o`0000<0oooo00@0o`000?ooo`3oool0o`00103oool0103o0000o`3o
0?ooo`3o00050?ooo`030?l0003oool0o`0000D0oooo0P3o00000`3o0?l0oooo0?ooo`030?ooo`80
o`001P3oool20?l000H0oooo0P3o00060?ooo`030?l0003o0?l0o`0000D0oooo0P3o00060?ooo`03
0?l0003o0?l0o`0000D0oooo00<0o`000?l0o`3o0000403oool001<0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0M0?ooo`80o`3o0P3oool00`000?l0o`000?ooo`0c0?ooo`80o`00
0`3oool00`000?l0oooo0?l0000N0?ooo`80o`3o1@3oool0103o0000oooo0?l0003o0?l20?ooo`03
0000o`3o0000oooo0080oooo00@0o`000?l0o`3o0?l0o`001@3oool20?l001h0oooo00<0o`000?oo
o`3oool01@3oool20?l000H0oooo00<0o`000?ooo`3o0000103oool0103o0000oooo0?ooo`3o0004
0?ooo`050?l0003oool0oooo0?l0003o0?l00`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo
00D0o`000?ooo`3oool0o`000?l0o`030?ooo`040?l0003o0?l0oooo0?l000@0oooo00@0o`000?l0
o`3oool0o`001@3oool20?l000030?l0o`3oool0oooo00<0oooo0P3o00060?ooo`030?l0003o0?l0
o`0000D0oooo0P3o00060?ooo`030?l0003o0?l0o`0000D0oooo0P3o00060?ooo`030?l0003oool0
o`0000D0oooo00<0o`000?ooo`3o0000403oool001@0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`0M0?ooo`060?l0o`3oool0oooo0?ooo`000?l0o`00=03oool00`3o0000o`3o0?l0
00020?ooo`030000o`3oool0o`0001l0oooo00<0o`3o0?ooo`3oool00`3oool0103o0000oooo0?l0
003o0?l30?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`3o0000oooo0?ooo`030?ooo`80
o`001P3oool00`3o0000oooo0?ooo`0=0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3o
ool01@3oool20?l000D0oooo00@0o`3o0?l0003oool0o`00103oool01@3o0000oooo0?ooo`3o0000
o`3o00<0oooo00D0o`000?ooo`3oool0oooo0?l000030000o`050?l000000?l0oooo0?ooo`3o0000
0`3oool01@3o0000oooo0?ooo`3o0000o`3o00<0oooo00@0o`000?l0o`3oool0o`00103oool0103o
0000oooo0?ooo`3o00050?ooo`030?l0003oool0o`0000D0oooo00<0o`000?ooo`3o00001@3oool0
0`3o0000oooo0?l000050?ooo`80o`001P3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0
o`0000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l0000@0?ooo`003`3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool07@3oool01P3o
0?l0oooo0?ooo`3oool0003o0?l003@0oooo00D0o`000?l0o`3o0000o`3o0000o`020?ooo`030?l0
003oool0oooo02<0oooo00@0o`000?ooo`3o0000o`3o0`3oool01@3o0000003o0?ooo`3oool0o`00
0080oooo00<0o`000?ooo`3oool00`3oool20?l000H0oooo00<0o`000?ooo`3oool01@3oool00`3o
0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo0P3o00060?ooo`80o`0000<0o`3o0?oo
o`3oool00P3oool0103o0?l0o`000?ooo`3o00040?ooo`060?l0003oool0oooo0?l0003o0?l0oooo
0P000?l01P3o0000003o0000o`000?l0o`000000o`80oooo00D0o`000?ooo`000?l0003o0?l00003
0000o`050?l000000?l0003o0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo
00@0o`000?ooo`3oool0o`00103oool0103o0?l0o`000?ooo`3o00050?ooo`030?l0003oool0o`00
00D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`00
00D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`00
0100oooo000?0?ooo`H000001@3oool00`000000oooo0?ooo`0Q0?ooo`030000o`3o0000oooo03<0
oooo00D0o`000?l0o`3oool0o`000000o`020?ooo`030?l0003oool0oooo02<0oooo00<0o`3o0?l0
003o0000103oool01@3o0000oooo0000o`3oool0o`000080oooo00<0o`000?ooo`3oool00P3oool0
0`3o0000o`3o0?l000060?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool0
0`3o0000oooo0?ooo`050?ooo`80o`001@3oool0103o0000o`3o0?ooo`3o00040?ooo`050?l0003o
ool0oooo0?l0003o0?l00`3oool01P3o0000oooo0?ooo`000?l0o`000000o`80oooo00H0o`000?oo
o`3oool0oooo0?l0003o0?l20?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo
0?ooo`000?l0o`0000<0003o00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo
0?l0003o0?l00`3oool0103o0?l0o`000?ooo`3o00040?ooo`040?l0003o0?l0oooo0?l000D0oooo
00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000050?ooo`030?l0003oool0o`0000@0oooo
00@0o`000?l0o`3oool0o`001@3oool00`3o0000oooo0?l000050?ooo`040?l0003oool0oooo0?l0
00l0oooo000J0?ooo`030000003oool0oooo0280oooo00<0o`000?ooo`3oool0<P3oool01@3o0000
o`3o0?ooo`3o0000o`3o0080oooo00<0o`000?ooo`3oool0903oool20?l000@0oooo00D0o`000?oo
o`000?l0oooo0?l000020?ooo`030?l0003oool0oooo0080oooo00<0o`000?l0o`3o00001P3oool0
0`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool2
0?l000D0oooo00@0o`000?l0o`3oool0o`00103oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo
00D0o`000000o`000?l0oooo0?l000030?ooo`060?l0003oool0oooo0?ooo`3o0000o`3o0P3oool0
1@3o0000oooo0?ooo`3oool0o`000080oooo00<0o`000?l0o`3oool00P3oool01@3o0000oooo0?oo
o`3oool0o`0000<0003o00D0o`000000o`3oool0oooo0?l000020?ooo`030?l0003o0?l0oooo0080
oooo00@0o`3o0?l0003oool0o`00103oool0103o0000oooo0?ooo`3o00050?ooo`030?l0003oool0
o`0000@0oooo00@0o`000?l0o`3oool0o`001@3oool0103o0000oooo0?l0003o0?l30?ooo`050?l0
003o0?l0oooo0?l0o`3o0000103oool0103o0000oooo0?l0003o0?l40?ooo`040?l0003oool0oooo
0?l000l0oooo000J0?ooo`8000008`3oool00`3o0000oooo0?ooo`0b0?ooo`050?l0003oool0oooo
0?l0003o0?l00P3oool00`3o0000oooo0?ooo`0T0?ooo`80o`00103oool01@3o0000oooo0?ooo`3o
0000o`3o0080oooo00<0o`000?ooo`3oool00P3oool00`3o0000o`3o0?l000060?ooo`030?l0003o
ool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`80o`001@3o
ool0103o0000o`3o0?ooo`3o00040?ooo`050?l0003oool0oooo0?ooo`3o00000`000?l02@3o0000
oooo0?ooo`3oool0o`000?l0o`3oool0o`000?l0o`030?ooo`050?l0003o0?l0oooo0?ooo`3o0000
0`3oool01@3o0000o`3o0?ooo`3o0000o`3o00<0oooo00D0o`3o0?l0003oool0oooo0?l000030?oo
o`050?l0003oool0003o0000o`3o00000P000?l00`3o0000o`3o0?ooo`020?ooo`050?l0003oool0
oooo0?ooo`3o00000`3oool0103o0000oooo0?ooo`3o00050?ooo`040?l0003oool0o`000?l0o`<0
oooo00D0o`000?l0o`3oool0o`3o0?l000040?ooo`040?l0003oool0o`000?l0o`<0oooo00D0o`00
0?l0o`3oool0oooo0?l000030?ooo`050?l0003o0?l0oooo0?ooo`3o0000103oool0103o0000oooo
0?ooo`3o000?0?ooo`006P3oool00`000000oooo0?ooo`0R0?ooo`030?l0003oool0oooo0380oooo
00@0o`000?ooo`000?l0oooo0P3o00000`3oool0o`000?ooo`0U0?ooo`80o`00103oool01@3o0000
oooo0?ooo`3o0000o`3o0080oooo00<0o`000?ooo`3oool00P3oool00`3o0000o`3o0?l000060?oo
o`80o`001P3oool00`3o0000oooo0?ooo`050?ooo`80o`001P3oool20?l000D0oooo00D0o`000?oo
o`3oool0o`000?l0o`030?ooo`060?l000000?l0003o0000o`3o0000o`3o0P3oool00`3o0000oooo
0?ooo`020?ooo`040?l0003oool0o`000?l0o`@0oooo00@0o`000?ooo`3oool0o`00103oool00`3o
0000oooo0?l000050?ooo`040?l0003oool0oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l0
00020?ooo`060000o`3o0000003o0000o`3oool0o`000`3oool01@3o0000oooo0?ooo`3oool0o`00
0080oooo00<0o`000?l0o`3oool00P3oool01@3o0?l0o`000?ooo`3o0000o`3o00<0oooo00D0o`00
0?l0o`3oool0oooo0?l000030?ooo`050?l0003o0?l0oooo0?ooo`3o00000`3oool01@3o0000o`3o
0?ooo`3oool0o`0000<0oooo00D0o`000?l0o`3oool0oooo0?l000040?ooo`040?l000000?l0003o
0?l000<0003o303oool001X0oooo00<000000?ooo`3oool08P3oool00`3o0?l0o`000?ooo`0b0?oo
o`030?l0003oool0003o0080oooo0`3o000V0?ooo`80o`001@3oool01`3o0000oooo0?l0003o0?l0
003o0?ooo`3o0000103oool0103o0000o`3o0?l0003o0?l40?ooo`030?l0003o0?l0o`0000H0oooo
0P3o00060?ooo`80o`001P3oool20?l000D0oooo00D0o`000?ooo`3oool0o`000?l0o`020?ooo`03
0000o`3o0000oooo0080oooo00D0o`000?l0o`3oool0oooo0?l000040?ooo`040?l0003oool0o`00
0?l0o`@0oooo00<0o`000?l0o`3o00001@3oool00`3o0000oooo0?l000050?ooo`040?l0003oool0
oooo0?l000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`050?l0003oool0oooo0000o`3o
00000P000?l00`3oool0o`000?ooo`020?ooo`050?l0003oool0oooo0?l0003o0?l00`3oool01@3o
0000oooo0?ooo`3oool0o`0000<0oooo00D0o`000?l0o`3oool0oooo0?l000030?ooo`050?l0003o
0?l0oooo0?ooo`3o00000`3oool01P3o0000o`3o0?ooo`3oool0o`000?ooo`80003o00D0o`000?l0
o`000?l0003o0?l000030000o`060?l0003oool0oooo0?ooo`3o0000o`3o3P3oool001X0oooo00<0
00000?ooo`3oool08P3oool00`3o0?l0o`000?ooo`0b0?ooo`030?l000000?l0oooo00<0oooo0P3o
0?lV0?ooo`80o`3o1@3oool01`3o0000oooo0?l0003o0?l0oooo0000o`3o0000103oool0103o0000
o`3o0?l0003o0?l40?ooo`030?l0003o0?l0o`0000H0oooo0P3o00050?ooo`030?l0003o0?l0o`00
00H0oooo00<0o`000?ooo`3o0000103oool01@3o0000oooo0?ooo`3o0000o`3o0080003o00<0oooo
0?l0003oool00P3oool01@3o0000o`3o0?ooo`3oool0o`0000@0oooo00@0o`000?l0o`3o0000o`3o
103oool00`3o0000o`3o0?l000050?ooo`030?l0o`3o0000o`0000D0oooo00<0o`000?ooo`3o0000
1@3oool0103o0000oooo0?ooo`3o00030?ooo`050?l0003oool0oooo0?ooo`3o00000P3oool00`00
0?l0o`000000o`020000o`050?l0003oool0oooo0?ooo`3o00000`3oool01@3o0000oooo0?ooo`3o
ool0o`0000<0oooo00D0o`000?l0o`3oool0oooo0?l000030?ooo`060?l0003o0?l0oooo0?ooo`3o
0000o`3o0P3oool01P3o0000o`3o0000o`000?l0o`000000o`80oooo00<0o`000?ooo`3oool00P3o
ool0103o0000oooo0?ooo`3o00030?ooo`030?l0003o0?l0oooo00d0oooo000J0?ooo`030000003o
ool0oooo02<0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?ooo`0/0?ooo`030?l00000
0?l0oooo00<0oooo0P3o0?lW0?ooo`030?l0o`3oool0oooo00<0oooo00@0o`000?ooo`3o0000o`3o
0P3oool00`000?l0o`000?ooo`020?ooo`040?l0003oool0oooo0?l000@0oooo00<0o`000?l0o`3o
00001P3oool20?l000D0oooo00<0o`000?l0o`3o00001P3oool00`3o0000oooo0?l000040?ooo`05
0?l0003oool0oooo0000o`3o00000`3oool01P3o0000oooo0?ooo`3oool0o`000?l0o`80oooo00<0
o`000?ooo`3oool00P3oool00`3o0000o`3o0?l000060?ooo`80o`001P3oool20?l000D0oooo00<0
o`000?ooo`3o00001@3oool0103o0000oooo0?l0o`3o00040?ooo`040?l0003oool0oooo0?l000<0
oooo00D0o`000?ooo`3oool0oooo0?l000030000o`050?l000000?l0003o0000o`3o00000`000?l0
1P3o0000o`3o0000o`000?l0o`000?l0o`80003o00H0o`000000o`000?l0003o0?l0003o0?l20000
o`060?l0003o0?l0003o0000o`3o0000o`3o0P3oool01@3o0000o`3o0?ooo`3oool0o`0000@0oooo
00@0o`000?ooo`3oool0o`00103oool00`3o0000oooo0?ooo`0<0?ooo`006P3oool2000002@0oooo
00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?ooo`0/0?ooo`030?l0003oool0oooo02`0oooo
00<0o`3o0?ooo`3oool00`3oool0103o0000oooo0?l0003o0?l30?ooo`050?l000000?l0oooo0?oo
o`3o00000P3oool00`3o0000oooo0?ooo`020?ooo`030?l0003o0?l0o`0000H0oooo0P3o00050?oo
o`040?l0003oool0oooo0?l000@0oooo00@0o`000?l0o`3oool0o`00103oool01@3o0000003o0000
o`3oool0o`000080oooo00<0o`000?l0o`3oool00`3oool0103o0000oooo0?ooo`3o00040?ooo`03
0?l0003o0?l0o`0000H0oooo0P3o00060?ooo`80o`001@3oool00`3o0000o`3o0?l000050?ooo`04
0?l0003oool0o`3o0?l000@0oooo00@0o`000?ooo`3oool0o`000`3oool01@3o0000oooo0?ooo`3o
ool0o`0000<0oooo00D0o`000?ooo`3oool0oooo0?l000030?ooo`060?l0003o0?l0oooo0?ooo`3o
0000o`3o0P3oool01P3o0000o`3o0?ooo`3oool0o`000?l0o`80oooo00H0o`000?l0o`3oool0oooo
0?l0003o0?l30?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool0
0`3o0000oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?ooo`0S0?ooo`040?l0003oool0oooo
0?ooo`80o`00;P3oool00`3o0000oooo0?ooo`0b0?ooo`040?l0003o0?l0o`000?l0o`<0oooo00D0
o`000?ooo`000?l0oooo0?l000020?ooo`030?l0003oool0oooo0080oooo00@0o`000?l0o`3o0?l0
o`001@3oool20?l000D0oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?ooo`3o0?l0
o`000080oooo00<0003o0?l0003oool00P3oool01@3o0000o`3o0?ooo`3o0000o`3o00@0oooo00<0
o`000?ooo`3o00001P3oool20?l000H0oooo0P3o00060?ooo`80o`001P3oool20?l000D0oooo00@0
o`3o0?l0003o0?l0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l0
00<0oooo00D0o`3o0?l0003oool0oooo0?l000040?ooo`050?l0003oool0oooo0?l0003o0?l00P3o
ool01@3o0000o`3o0?ooo`3oool0o`0000<0oooo00D0o`000?l0o`3oool0oooo0?l000040?ooo`04
0?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3oool0o`00103oool00`3o0000oooo0?ooo`0<
0?ooo`006P3oool00`000000oooo0?ooo`0S0?ooo`040?l0003oool0oooo0?ooo`80o`00;P3oool0
0`3o0000oooo0?ooo`0b0?ooo`030?l0003o0?l0o`0000@0oooo00D0o`000?ooo`3oool0003o0?l0
00020?ooo`030?l0003oool0oooo0080oooo00@0o`000?l0o`3oool0o`001@3oool20?l000030?l0
o`3oool0oooo0080oooo00@0o`000?ooo`3oool0o`00103oool01@3o0000oooo0?ooo`3oool0o`00
0080003o00<0oooo0?l0003oool00`3oool0103o0000oooo0?l0003o0?l50?ooo`80o`001P3oool0
0`3o0?l0o`000?ooo`050?ooo`030?l0o`3o0000oooo00H0oooo00<0o`000?ooo`3oool0103oool2
0?l000H0oooo0P3o00050?ooo`040?l0003oool0o`000?l0o`@0oooo00@0o`000?ooo`3oool0o`00
103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?oo
o`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003o0?l0o`000?l0o`@0oooo
00@0o`000?l0o`3oool0o`00103oool00`3o0?l0o`000?ooo`0<0?ooo`006P3oool00`000000oooo
0?ooo`0S0?ooo`040?l000000?l0oooo0?ooo`80o`00;@3oool00`000?l0o`000?ooo`0d0?ooo`80
o`00103oool02@3o0?l0o`000?ooo`3oool0o`000000o`3oool0o`3o0?l000030?ooo`040?l0003o
0?l0oooo0?l000D0oooo0P3o00000`3o0?l0oooo0?ooo`020?ooo`040?l0003oool0oooo0?l000@0
oooo00D0o`000?ooo`3oool0003o0?l000020?ooo`030?l0o`3o0000oooo00<0oooo00@0o`000?oo
o`3o0000o`3o1@3oool20?l000L0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`05
0?ooo`030?l0003oool0oooo00@0oooo00<0o`3o0?l0003oool01@3oool20?l000D0oooo00@0o`00
0?ooo`3o0000o`3o103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0oooo0?l000@0
oooo00@0o`000?ooo`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`040?l0003oool0
oooo0?l000D0oooo0P3o00000`3o0?l0oooo0?ooo`020?ooo`040?l0003o0?l0oooo0?l000D0oooo
00<0o`000?ooo`3oool02`3oool001X0oooo00<000000?ooo`3oool08`3oool0103o0000003o0?oo
o`3oool20?l002L0oooo00<0o`000?ooo`3oool00`3oool00`000?l0o`000?ooo`0d0?ooo`80o`00
103oool02@3o0?l0o`000?ooo`3oool0o`000?ooo`000?l0oooo0?l000030?ooo`040?l0003o0?l0
oooo0?l000@0oooo00@0o`000?l0o`3oool0o`00103oool0103o0000oooo0?ooo`3o00040?ooo`05
0?l000000?l0003o0?ooo`3o00000P3oool00`3o0?l0o`000?ooo`030?ooo`030?l0003oool0o`00
00H0oooo00<0o`3o0?l0003oool01P3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo
00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0000oooo0?ooo`040?ooo`80o`001P3oool20?l0
00030?l0o`3oool0oooo0080oooo00@0o`3o0?l0003oool0o`00103oool0103o0000oooo0?l0003o
0?l40?ooo`040?l0003oool0oooo0?l000@0oooo00@0o`000?ooo`3o0000o`3o103oool0103o0000
oooo0?ooo`3o00050?ooo`80o`0000<0o`3o0?ooo`3oool00`3oool00`3o0000oooo0?l000050?oo
o`030?l0003oool0oooo00/0oooo000J0?ooo`800000903oool0103o0000003o0?ooo`3oool20?l0
02L0oooo00<0o`000?ooo`3oool00P3oool00`000?l0o`000?l0o`0e0?ooo`80o`00103oool01@3o
0?l0o`000?ooo`3oool0o`000080oooo00<0003o0?l0003oool00P3oool0103o0000o`3o0?ooo`3o
00040?ooo`040?l0003oool0oooo0?l000@0oooo00D0o`000?ooo`3oool0o`000?l0o`030000o`05
0?l0003oool0oooo0?ooo`3o00000P3oool00`3o0?l0o`000?ooo`030?ooo`030?l0003o0?l0o`00
00H0oooo0P3o0?l70?ooo`030?l0o`3oool0oooo00d0oooo00<0o`3o0?ooo`3oool01@3oool00`3o
0?l0oooo0?ooo`040?ooo`030?l0o`3o0000oooo00D0oooo0P3o00000`3o0?l0oooo0?ooo`030?oo
o`80o`0000<0o`3o0?ooo`3oool00P3oool0103o0000oooo0?l0003o0?l40?ooo`040?l0003oool0
o`000?l0o`@0oooo00@0o`000?l0o`3o0000o`3o103oool0103o0?l0o`000?l0003o0?l50?ooo`80
o`0000<0o`3o0?ooo`3oool00`3oool20?l000030?l0o`3oool0oooo00<0oooo00<0o`000?ooo`3o
ool02`3oool001X0oooo00<000000?ooo`3oool08`3oool0103o0000003o0?ooo`3oool20?l002L0
oooo00<0o`000?ooo`3oool00P3oool00`000?l0o`000?l0o`0e0?ooo`030?l0o`3o0000oooo00@0
oooo00@0o`000?ooo`3o0000o`3o0`3oool01@3o0000003o0000o`3o0000o`3o0080oooo00<0o`00
0?l0o`3oool00P3oool0103o0000oooo0?ooo`3o00040?ooo`050?l0003oool0oooo0000o`3o0000
0`3oool02@3o0000oooo0?ooo`3oool0o`000?l0o`3oool0o`3o0?l000050?ooo`80o`001`3oool0
0`3o0?l0oooo0?ooo`0M0?ooo`030?l0o`3oool0oooo00@0oooo00<0o`3o0?l0003oool01@3oool0
0`3o0?l0o`000?l0o`050?ooo`80o`0000<0o`3o0?ooo`3oool00P3oool0103o0000o`3o0?l0003o
0?l40?ooo`040?l0o`3o0000o`000?l0o`@0oooo00@0o`000?l0o`3o0000o`3o1@3oool20?l00003
0?l0o`3oool0oooo00<0oooo0P3o00000`3o0?l0oooo0?ooo`030?ooo`030?l0o`3o0000o`3o00D0
oooo00<0o`3o0?l0003oool02`3oool001X0oooo00<000000?ooo`3oool08`3oool0103o0000003o
0?ooo`3oool20?l002L0oooo0P3o00020?ooo`030000o`3oool0o`0003H0oooo0P3o0?l50?ooo`04
0?l0003oool0o`000?l0o`<0oooo00T0o`000?ooo`3oool0o`000?l0o`000?l0oooo0?l0003o0?l0
0`3oool01@3o0000oooo0?ooo`3o0?l0o`0000<0oooo00D0o`000000o`000?l0oooo0?l000030?oo
o`030?l0003oool0oooo0080oooo00<0o`000?ooo`3o00001P3oool20?l000L0oooo00<0o`3o0?oo
o`3oool09@3oool00`3o0?l0oooo0?ooo`040?ooo`030?l0o`3o0000oooo00D0oooo0P3o00000`3o
0?l0oooo0?ooo`030?ooo`80o`0000<0o`3o0?ooo`3oool00`3oool20?l000030?l0o`3oool0oooo
00<0oooo0P3o00000`3o0?l0oooo0?ooo`030?ooo`80o`001P3oool00`3o0?l0o`000?l0o`050?oo
o`030?l0o`3o0000oooo00D0oooo00<0o`3o0?l0003oool02`3oool001X0oooo00<000000?ooo`3o
ool08`3oool0103o0000o`3o0000o`3oool20?l002H0oooo00<0o`000?l0o`3o00000P3oool00`00
0?l0oooo0?l0000g0?ooo`030?l0o`3oool0oooo00<0oooo00@0o`000?ooo`3o0000o`3o0`3oool0
1P3o0000oooo0?ooo`3o0000o`3o0?ooo`80003o00D0o`000000o`000?l0oooo0?l000020?ooo`03
0000o`3o0000003o0080003o00D0o`000?ooo`3oool0oooo0?l000020?ooo`030?l0003oool0oooo
00<0oooo00<0o`000?ooo`3o00001P3oool00`3o0?l0o`000?ooo`0^0?ooo`030?l0o`3oool0oooo
00D0oooo00<0o`000?ooo`3oool0103oool00`3o0?l0o`000?l0o`050?ooo`80o`0000<0o`3o0?oo
o`3oool00`3oool20?l000H0oooo0P3o00060?ooo`030?l0o`3o0000oooo00D0oooo00<0o`3o0?l0
003oool01P3oool00`3o0?l0oooo0?ooo`050?ooo`030?l0o`3oool0oooo00X0oooo000J0?ooo`03
0000003oool0oooo02<0oooo00@0o`000?l0o`000?l0oooo0P3o00000`3o0?l0oooo0?ooo`0S0?oo
o`030?l0003o0?l0o`000080oooo00<0003o0?ooo`3o0000=`3oool00`3o0?l0oooo0?ooo`030?oo
o`040?l0o`3o0000o`000?l0o`<0oooo00D0o`000?ooo`3oool0o`000?l0o`030?ooo`050?l0003o
ool0oooo0000o`3o00000`000?l01@3o0000oooo0?ooo`3oool0o`0000@0oooo00<0o`000?ooo`3o
00001P3oool20?l000L0oooo00<0o`3o0?ooo`3oool0=@3oool00`3o0?l0oooo0?ooo`040?ooo`03
0?l0o`3o0000oooo00D0oooo00<0o`3o0?l0003oool01@3oool00`3o0?l0o`000?ooo`060?ooo`03
0?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`03
0?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool02P3oool001X0oooo00<000000?ooo`3oool0
8`3oool01`3o0000o`3o0000o`3o0000o`3o0?l0003o0?l09@3oool01@3o0000o`3o0?l0003o0?l0
003o0080oooo00<0o`000?ooo`3oool0>`3oool00`3o0?l0o`000?l000040?ooo`050?l0o`3o0000
oooo0?l0003o0?l00`3oool01@3o0000o`3o0?ooo`3oool0o`0000<0oooo00D0o`000?ooo`3oool0
oooo0?l000040?ooo`030?l0003oool0o`0000H0oooo0P3o00070?ooo`030?l0o`3oool0oooo03D0
oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`040?ooo`030?l0o`3o0000oooo00H0
oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`050?ooo`030?l0o`3oool0oooo00D0
oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`0B0?ooo`006P3oool2000002@0oooo
00L0o`000?l0o`000?l0o`000?l0o`3oool0o`0002D0oooo00D0o`000?l0o`3oool0o`000?l0o`02
0?ooo`030?l0003oool0oooo03`0oooo0P3o00050?ooo`040?l0003oool0o`000?l0o`@0oooo00@0
o`000?ooo`3o0000o`3o0`3oool01@3o0000oooo0?ooo`3o0?l0o`0000@0oooo00<0o`000?ooo`3o
00001P3oool20?l004L0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`03
0?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`03
0?l0o`3oool0oooo01X0oooo000J0?ooo`030000003oool0oooo02@0oooo00H0o`000?ooo`3o0000
o`3o0?ooo`3o000U0?ooo`050?l0003o0?l0oooo0?l0003o0?l00P3oool00`3o0000oooo0?ooo`0l
0?ooo`80o`001@3oool0103o0000oooo0?l0003o0?l40?ooo`040?l0003oool0o`000?l0o`<0oooo
00D0o`000?l0o`3oool0o`3o0?l000040?ooo`030?l0003o0?l0o`0000H0oooo0P3o0?m70?ooo`03
0?l0o`3oool0oooo00D0oooo00<0o`3o0?ooo`3oool01@3oool00`3o0?l0oooo0?ooo`050?ooo`03
0?l0o`3oool0oooo02X0oooo00020?ooo`030000003oool0oooo00@0oooo0P0000050?ooo`@00000
1P3oool00`000000oooo0?ooo`0T0?ooo`060?l0003oool0o`000?l0o`3oool0o`009@3oool0103o
0000o`3o0?ooo`000?l40?l003h0oooo00<0o`3o0?l0003oool0103oool0103o0?l0o`000?l0003o
0?l40?ooo`040?l0003oool0o`000?l0o`@0oooo00@0o`000?ooo`3o0?l0o`00103oool00`3o0000
o`3o0?l000070?ooo`030?l0o`3oool0oooo08X0oooo00020?ooo`030000003oool0oooo00@0oooo
0P0000040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`0T0?ooo`060?l0003oool0o`000?l0o`3oool0o`009@3oool00`3o0000oooo0000o`020?oo
o`80o`0000<0o`3o0?ooo`3oool0?03oool20?l0o`H0oooo0P3o00050?ooo`030?l0003oool0o`00
00D0oooo00<0o`000?ooo`3o00001P3oool20?l000L0oooo00<0o`3o0?ooo`3oool0RP3oool00080
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02@0
oooo00<0o`000?ooo`3o00000P3oool00`3o0000oooo0?ooo`0S0?ooo`030?l0003oool0003o00<0
oooo0P3o0?ln0?ooo`80o`3o1P3oool20?l000D0oooo00<0o`000?ooo`3o00001@3oool00`3o0000
o`3o0?l000060?ooo`80o`00U03oool000<0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`030?ooo`<00000903oool01P3o0000oooo0?l000000?l0oooo0?l002D0oooo00<0o`000000
o`3oool00`3oool00`3o0?l0oooo0?ooo`0n0?ooo`030?l0o`3oool0oooo00@0oooo0P3o00050?oo
o`030?l0003oool0o`0000H0oooo0P3o00060?ooo`80o`3oU03oool000<0oooo00<000000?ooo`3o
ool02P3oool4000000H0oooo00<000000?ooo`3oool09@3oool20?l000030000o`3oool0o`0002D0
oooo00<0o`000000o`3oool0B`3oool20?l000D0oooo00<0o`3o0?l0003o00001P3oool20?l000H0
oooo0P3o0?nD0?ooo`00103oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool09@3oool20?l000040000o`3oool0o`3o0?l002@0oooo00<0o`000000o`3o
ool0B`3oool00`3o0?l0o`000?ooo`040?ooo`030?l0o`3o0000o`0000H0oooo00<0o`3o0?l0003o
ool01P3oool00`3o0?l0oooo0?ooo`2B0?ooo`00103oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool09@3oool20?l000040?ooo`000?l0oooo0?l002@0
oooo00<0o`000?ooo`3oool0C03oool00`3o0?l0oooo0?ooo`040?ooo`80o`001P3oool20?l0oi`0
oooo0005000000/0oooo1@0000050?ooo`030000003oool0oooo02D0oooo0P3o0000103oool0003o
0?ooo`3o000S0?ooo`030?l0003o0?l0oooo04d0oooo00<0o`3o0?ooo`3oool0103oool20?l0o`H0
oooo0P3o0?nL0?ooo`006P3oool00`000000oooo0?ooo`0U0?ooo`80o`0000@0oooo0000o`3oool0
o`007@3oool00`3o0000oooo0?ooo`030?ooo`030?l0003o0?l0oooo05@0oooo0P3o0?l70?ooo`03
0?l0o`3oool0oooo09X0oooo000J0?ooo`8000009P3oool20?l00080oooo00<0003o0?l0003oool0
103oool00`3o0000oooo0?ooo`0E0?ooo`030?l0003oool0oooo00<0oooo00<0o`000?l0o`3oool0
E@3oool00`3o0?l0oooo0?ooo`2R0?ooo`006P3oool00`000000oooo0?ooo`0U0?ooo`80o`000P3o
ool00`000?l0o`000?ooo`040?ooo`030?l0003oool0oooo01D0oooo00<0o`000?ooo`3oool00P3o
ool00`000?l0o`000?l0o`3k0?ooo`006P3oool00`000000oooo0?ooo`0U0?ooo`80o`000P3oool0
0`000?l0o`000?ooo`040?ooo`030?l0003oool0oooo01D0oooo00<0o`000?ooo`3oool00P3oool0
0`000?l0o`000?l0o`3k0?ooo`006P3oool00`000000oooo0?ooo`0U0?ooo`030?l0003o0?l0oooo
0080oooo00<0o`000?ooo`3oool00`3oool00`3o0000oooo0?ooo`0E0?ooo`030?l0003oool0oooo
0080oooo00<0003o0?l0003o0?l0n`3oool001X0oooo00<000000?ooo`3oool09@3oool00`3o0000
o`3o0?ooo`020?ooo`030?l0003oool0oooo0080oooo0P3o000G0?ooo`80o`000P3oool00`000?l0
oooo0?l0003l0?ooo`006P3oool2000002H0oooo00<0o`3o0?ooo`3oool00P3oool00`3o0000oooo
0?ooo`020?ooo`80o`005P3oool00`3o0000o`3o0?l000020?ooo`030000o`3oool0o`000?`0oooo
000J0?ooo`030000003oool0oooo02D0oooo00<0o`3o0?ooo`3oool00P3oool00`3o0000003o0?oo
o`020?ooo`80o`005P3oool01@3o0000o`3o0?l0003o0?l0003o0080oooo00<0o`000?ooo`3oool0
nP3oool001X0oooo00<000000?ooo`3oool09@3oool00`3o0?l0oooo0?ooo`020?ooo`030?l00000
0?l0oooo0080oooo0P3o000F0?ooo`050?l0003oool0oooo0?l000000?l00P3oool00`3o0000oooo
0?ooo`3j0?ooo`006P3oool00`000000oooo0?ooo`0Z0?ooo`030?l0003o0?l0oooo0080oooo00<0
o`000?l0o`3o00005@3oool01@3o0000oooo0?ooo`3o0000o`3o0080oooo00<0o`000?ooo`3oool0
nP3oool001X0oooo00<000000?ooo`3oool0:`3oool01`3o0000oooo0?ooo`3oool0o`000?l0o`3o
00005@3oool01@3o0000oooo0?ooo`000?l0o`000080oooo00<0o`000?ooo`3oool0nP3oool001X0
oooo0P00000/0?ooo`070?l000000?l0oooo0?ooo`3o0000oooo0?l0000>0?ooo`030?l0003oool0
oooo00@0oooo00@0o`000?ooo`000?l0oooo103o003l0?ooo`006P3oool00`000000oooo0?ooo`0[
0?ooo`070?l000000?l0oooo0?ooo`3o0000oooo0?l000060?ooo`030?l0003oool0oooo00D0oooo
00<0o`000?ooo`3oool0103oool00`3o0000oooo0000o`020?ooo`80o`0000<0o`3o0?ooo`3oool0
nP3oool001X0oooo00<000000?ooo`3oool0:`3oool01`3o0000003o0?ooo`3oool0o`000?ooo`3o
00001P3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00@0oooo00<0o`000000o`3o
ool00`3oool20?l0oo`0oooo000J0?ooo`030000003oool0oooo02/0oooo00L0o`000000o`3oool0
oooo0?l0003oool0o`0000H0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`040?oo
o`030?l000000?l0oooo00<0oooo00<0o`3o0?ooo`3oool0n`3oool001X0oooo00<000000?ooo`3o
ool0:`3oool01`3o0000oooo0000o`3oool0o`000?ooo`3o00001P3oool00`3o0000oooo0?ooo`04
0?ooo`030?l0003o0?l0o`0000D0oooo00<0o`000?ooo`3oool0o`3oool20?ooo`006P3oool00`00
0000oooo0?ooo`0[0?ooo`070?l0o`3o0000003o0?ooo`3o0000oooo0?l000050?ooo`80o`001P3o
ool00`3o0000o`3o0?l000050?ooo`030?l0003oool0oooo0?l0oooo0P3oool001X0oooo0P00000]
0?ooo`060?l000000?l0o`000?l0o`3oool0o`001@3oool20?l000H0oooo00<0o`000?l0o`3o0000
1@3oool00`3o0000oooo0?ooo`3o0?ooo`80oooo000J0?ooo`030000003oool0oooo02`0oooo00H0
o`000?ooo`3o0000o`3o0?ooo`3o00050?ooo`80o`001P3oool00`3o0000oooo0?l000040?ooo`03
0000o`3o0000oooo0?l0oooo0`3oool00100oooo100000060?ooo`030000003oool0oooo02`0oooo
00L0o`000?ooo`3o0000o`3o0?ooo`3o0000o`3o00@0oooo0P3o00060?ooo`030?l0003oool0o`00
00@0oooo00<0o`000?l0o`3oool0o`3oool30?ooo`003`3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0;03oool01`3o0000oooo0?l000000?l0oooo
0?l0003o0?l0103oool00`3o0000o`3o0?l000050?ooo`030?l0003oool0o`0000<0oooo00<0003o
0?l0003o0?l0o`3oool40?ooo`003`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0;03oool0103o0000oooo0?l000000?l20?ooo`030?l0003oool0
oooo0080oooo00<0o`000?ooo`3o00001@3oool00`3o0000oooo0?l000030?ooo`030000o`3o0000
o`3o0?l0oooo103oool000l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?oo
o`<00000;03oool01`3o0000oooo0?l0003oool0003o0?ooo`3o0000103oool00`3o0000oooo0?l0
00050?ooo`090?l0003oool0oooo0?l0003oool0003o0?ooo`3o0000o`3o0?l0oooo103oool00100
oooo100000060?ooo`030000003oool0oooo02`0oooo00L0o`000?ooo`3o0000oooo0000o`3oool0
o`0000@0oooo00<0o`000?ooo`3o00001@3oool02@3o0000oooo0?ooo`3o0000oooo0000o`3oool0
o`000?l0o`3o0?ooo`@0oooo000?0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0/0?ooo`030?l0003oool0o`000080oooo00<0003o0?l0003oool0
0`3oool00`3o0000oooo0?l000050?ooo`050?l0003oool0oooo0?l000000?l00P3oool00`3o0000
oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0]0?ooo`060?l0003o0?l0oooo0?ooo`000?l0o`000`3oool01@3o
0000o`3o0?ooo`3o0?l0o`0000@0oooo00D0o`000?ooo`3oool0o`3o0?l000020?ooo`030?l0003o
ool0oooo0?l0oooo0`3oool00100oooo100000060?ooo`030000003oool0oooo02d0oooo00<0o`00
0?l0o`3oool00P3oool02@3o0?l0o`000?ooo`3oool0o`000?l0o`3oool0o`3o0?l000040?ooo`05
0?l0003oool0003o0?ooo`3o00000P3oool00`3o0000oooo0?ooo`3o0?ooo`<0oooo000J0?ooo`03
0000003oool0oooo02d0oooo00<0o`000?l0o`3oool00P3oool01P3o0?l0o`000?ooo`3oool0o`00
0?l0o`80oooo00L0o`000?ooo`3oool0oooo0?l0003o0?l0003o0080oooo0P3o00000`3oool0o`00
0?ooo`3o0?ooo`@0oooo000J0?ooo`800000;P3oool00`3o0000o`3o0?ooo`030?ooo`050?l00000
0?l0oooo0?l0003o0?l00P3oool01P3o0000oooo0?ooo`3oool0o`000?l0o`@0oooo0P3o00000`3o
0?l0oooo0?ooo`3o0?ooo`<0oooo000J0?ooo`030000003oool0oooo02d0oooo00<0o`000?l0o`3o
ool00`3oool01@3o0000003o0?ooo`3o0000o`3o0080oooo00H0o`000?ooo`3oool0oooo0?l0003o
0?l40?ooo`80o`0000<0o`3o0?ooo`3oool0o`3oool30?ooo`006P3oool00`000000oooo0?ooo`0]
0?ooo`030?l0003o0?l0oooo00<0oooo00D0o`000?ooo`000?l0o`000?l0o`020?ooo`050?l0003o
ool0oooo0000o`3o00001P3oool20?l0ool0oooo1@3oool001X0oooo00<000000?ooo`3oool0;@3o
ool00`3o0000o`3o0?ooo`030?ooo`080?l0003oool0oooo0?l0003o0?l0003o0?ooo`3o00030000
o`030?l0003oool0oooo00@0oooo0P3o0?oo0?ooo`D0oooo000J0?ooo`030000003oool0oooo02d0
oooo0P3o0?l40?ooo`050?l0003oool0oooo0?l0003o0?l00P000?l01@3o0000003o0?ooo`3oool0
o`0000H0oooo00<0o`3o0?ooo`3oool0o`3oool40?ooo`006P3oool2000002h0oooo00<0o`3o0?oo
o`3oool00`3oool01@3o0000oooo0?ooo`3o0000o`3o0080oooo00D0o`3o0?l0003oool0oooo0?l0
003o0?ooo`d0oooo000J0?ooo`030000003oool0oooo02d0oooo00<0o`3o0?ooo`3oool00`3oool0
1@3o0000o`3o0?ooo`3o0000o`3o00<0oooo00@0o`000?ooo`3oool0o`00o`3oool=0?ooo`006P3o
ool00`000000oooo0?ooo`0d0?ooo`030?l0003oool0o`0000@0oooo00@0o`000?ooo`3oool0o`00
o`3oool=0?ooo`006P3oool00`000000oooo0?ooo`0d0?ooo`030?l0003oool0o`0000@0oooo00@0
o`3o0?l0003oool0o`00o`3oool=0?ooo`006P3oool00`000000oooo0?ooo`0d0?ooo`030?l0003o
ool0o`0000D0oooo00<0o`000?ooo`3o0000o`3oool=0?ooo`006P3oool2000003D0oooo00<0o`3o
0?l0003o00001@3oool00`3o0000oooo0?l0003o0?ooo`d0oooo000J0?ooo`030000003oool0oooo
03D0oooo0P3o00050?ooo`030?l0003oool0o`000?l0oooo3@3oool001X0oooo00<000000?ooo`3o
ool0=@3oool20?l000D0oooo00<0o`3o0?l0003o0?l0o`3oool=0?ooo`006P3oool00`000000oooo
0?ooo`0e0?ooo`80o`001P3oool00`3o0000o`3o0?ooo`3o0?ooo``0oooo000J0?ooo`030000003o
ool0oooo03D0oooo00<0o`000?l0o`3oool01@3oool20?l0ool0oooo3@3oool001X0oooo00<00000
0?ooo`3oool0=@3oool20?l0o`H0oooo00<0o`3o0?ooo`3oool0o`3oool<0?ooo`006P3oool20000
03H0oooo0P3o0?l60?ooo`030?l0o`3oool0oooo0?l0oooo303oool001X0oooo00<000000?ooo`3o
ool0=@3oool20?l0ool0oooo5@3oool000@000001@3oool2000000D0oooo100000060?ooo`030000
003oool0oooo03D0oooo00<0o`3o0?ooo`3oool0o`3ooolD0?ooo`00103oool00`000000oooo0?oo
o`020?ooo`800000103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0o`3ooom<0?ooo`00103oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool0o`3ooom<0?ooo`00103oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo00<0oooo0`00003o0?oood`0oooo0004000000`0oooo100000060?ooo`03
0000003oool0oooo0?l0ooooC03oool000@0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo0?l0ooooC03oool000@0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`3o0?oooeH0oooo0004000000`0oooo1@00003o0?oooe@0oooo003o0?oo
ofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo003o0?oo
ofT0oooo0000\
\>"],
  ImageRangeCache->{{{0, 359}, {221.375, 0}} -> {-0.0332863, 4.59872, \
0.00155747, 0.0230941}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Linearization and Transfer Functions\t",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(back\ to\ TOC\),
          ButtonData:>"TOC",
          ButtonStyle->"Hyperlink"], TraditionalForm]],
    FontSize->12,
    FontWeight->"Plain"]
}], "Section",
  CellTags->"Small-Signal Analysis"],

Cell["\<\
The large-signal, nonlinear averaged model can be linearized to \
generate a small-signal linear model around an operation point and to derive \
various transfer functions.\
\>", "Text",
  CellTags->"Small-Signal Analysis"],

Cell[CellGroupData[{

Cell["Parameters", "Subsection",
  CellTags->"Large-Signal Simulation"],

Cell["\<\
The ESR of the capacitor has been set zero in the previous \
large-signal analysis. To include the ESR for small-signal analysis, we \
redefine the parameters as follows.\
\>", "Text",
  CellTags->"Large-Signal Simulation"],

Cell[BoxData[
    \(\(Param2\  = \ {Vg\  -> \ 5, \ L\  -> \ 50\ 10^\(-6\), \ 
          C\  -> \ 4.4\ 10^\(-6\), R\  -> \ 10, \ r\  -> \ 0.05, \ 
          d\  -> \ 0.3, \ Ts\  -> \ 10^\(-5\)};\)\)], "Input",
  CellTags->"Large-Signal Simulation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Small-Signal Linearization", "Subsection",
  CellTags->"Small-Signal Analysis"],

Cell[TextData[{
  "The small-signal linearized model is written as\n\t",
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox[
            FractionBox[
              StyleBox["d",
                FontSlant->"Plain"], 
              StyleBox[
                RowBox[{"d", 
                  StyleBox["t",
                    FontSlant->"Italic"]}]]],
            FontSize->15], 
          RowBox[{"(", "\[NegativeThinSpace]", GridBox[{
                {
                  SubscriptBox[
                    StyleBox[
                      OverscriptBox[
                        StyleBox["i",
                          FontSlant->"Italic"], "~"],
                      FontSlant->"Italic"], "L"]},
                {
                  SubscriptBox[
                    StyleBox[
                      OverscriptBox[
                        StyleBox["v",
                          FontSlant->"Italic"], "~"],
                      FontSlant->"Italic"], "C"]}
                }], "\[NegativeThinSpace]", ")"}]}], TraditionalForm]]],
  " = ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["A",
            FontWeight->"Bold"], "p"], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{"(", "\[NegativeThinSpace]", GridBox[{
              {
                SubscriptBox[
                  StyleBox[
                    OverscriptBox[
                      StyleBox["i",
                        FontSlant->"Italic"], "~"],
                    FontSlant->"Italic"], "L"]},
              {
                SubscriptBox[
                  StyleBox[
                    OverscriptBox[
                      StyleBox["v",
                        FontSlant->"Italic"], "~"],
                    FontSlant->"Italic"], "C"]}
              }], "\[NegativeThinSpace]", ")"}], TraditionalForm]]],
  " + ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["B",
            FontWeight->"Bold"], "p"], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{"(", "\[NegativeThinSpace]", GridBox[{
              {\(V\&~\_g\)},
              {
                StyleBox[
                  OverscriptBox[
                    StyleBox["d",
                      FontSlant->"Italic"], "~"],
                  FontSlant->"Italic"]}
              }], "\[NegativeThinSpace]", ")"}], TraditionalForm]]],
  "\nwhere ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox[
            OverscriptBox[
              StyleBox["i",
                FontSlant->"Italic"], "~"],
            FontSlant->"Italic"], "L"], TraditionalForm]]],
  ", ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox[
            OverscriptBox[
              StyleBox["v",
                FontSlant->"Italic"], "~"],
            FontSlant->"Italic"], "C"], TraditionalForm]]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`V\&~\_g\)]],
  ", and ",
  Cell[BoxData[
      FormBox[
        StyleBox[
          OverscriptBox[
            StyleBox["d",
              FontSlant->"Italic"], "~"],
          FontSlant->"Italic"], TraditionalForm]]],
  " denote the small-signal pertubations of the corresponding variables. \
Matrices ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["A",
            FontWeight->"Bold"], \(p\ \)], TraditionalForm]]],
  "and ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["B",
            FontWeight->"Bold"], "p"], TraditionalForm]]],
  " can be obtained by calculating the Jacobian matrix of the averaged model \
with respect to the state variables and the input and control variables ",
  Cell[BoxData[
      \(TraditionalForm\`{V\_g\)]],
  ", ",
  StyleBox["d",
    FontSlant->"Italic"],
  "}, respectively."
}], "Text",
  CellTags->"Small-Signal Analysis"],

Cell[BoxData[{
    \(\(jacob[funs_List, \ vars_List]\  := \ 
        Outer[\ D, \ funs, \ vars];\)\), "\n", 
    \(\(Ap\  = \ 
        jacob[{fi[iL, \ vC, \ Vg, \ d], fv[iL, \ vC, \ Vg, \ d]}, {iL, \ 
            vC}];\)\), "\n", 
    \(\(Bp\  = \ 
        jacob[{fi[iL, \ vC, \ Vg, \ d], \ fv[iL, \ vC, \ Vg, \ d]}, {Vg, 
            d}];\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Operation Point", "Subsection",
  CellTags->"Small-Signal Analysis"],

Cell[TextData[{
  "The small-signal model is dependent of the operation point of the \
converter, which corresponds to the steady-state solution of the averaged \
model and can be solved for given input voltage ",
  Cell[BoxData[
      \(TraditionalForm\`V\_g\)]],
  " and control variable ",
  StyleBox["d",
    FontSlant->"Italic"],
  " as follows:"
}], "Text",
  CellTags->"Small-Signal Analysis"],

Cell[CellGroupData[{

Cell[BoxData[
    \(OP\  = \ 
      Solve[{fi[iL, \ vC, \ Vg, \ d], fv[iL, \ vC, \ Vg, \ d]}\  == \ 
          0, {iL, \ vC}]\)], "Input",
  CellTags->"Small-Signal Analysis"],

Cell[BoxData[
    \({{iL \[Rule] \(-\(\(\((r + R)\)\ Vg\)\/\(\((r + R - 
                      d\ R)\)\ \((\(-R\) + d\ R)\)\)\)\), 
        vC \[Rule] \(\((r + R)\)\ Vg\)\/\(r + R - d\ R\)}}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Output Voltage", "Subsection",
  CellTags->"Small-Signal Analysis"],

Cell[TextData[{
  "Because the ESR of the capacitor is considered in the small-signal \
analysis, the average output voltage, ",
  Cell[BoxData[
      \(TraditionalForm\`v\_0\)]],
  ", is not equal to ",
  Cell[BoxData[
      \(TraditionalForm\`v\_C\)]],
  ", but is\n\t",
  Cell[BoxData[
      \(TraditionalForm\`v\_0\)]],
  " = ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(R\/\(R\  + \ r\)\),
            "TraditionalForm"], "[", \(v\_C + r\ \((1 - d)\) i\_L\)}], 
        TraditionalForm]]],
  "].\nHence ",
  Cell[BoxData[
      \(TraditionalForm\`v\&~\_0\)]],
  " = ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          SubscriptBox[
            StyleBox["C",
              FontWeight->"Bold"], "p"], "T"], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{"(", "\[NegativeThinSpace]", GridBox[{
              {\(i\&~\_L\)},
              {\(v\&~\_C\)}
              }], "\[NegativeThinSpace]", ")"}], TraditionalForm]]],
  ", with ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["C",
            FontWeight->"Bold"], "p"], TraditionalForm]]],
  " = ",
  Cell[BoxData[
      FormBox[
        RowBox[{"(", "\[NegativeThinSpace]", GridBox[{
              {
                FractionBox[
                  RowBox[{
                    StyleBox["R",
                      FontSlant->"Italic"], 
                    StyleBox[" ",
                      FontSlant->"Italic"], 
                    StyleBox["r",
                      FontSlant->"Italic"], 
                    StyleBox[" ",
                      FontSlant->"Italic"], \((1 - d)\)}], \(R\  + \ r\)]},
              {\(R\/\(R\  + \ r\)\)}
              }], "\[NegativeThinSpace]", ")"}], TraditionalForm]]]
}], "Text",
  CellTags->"Small-Signal Analysis"],

Cell[BoxData[
    \(\(Cp\  = \ {R\ r\ \((1\  - \ d)\)/\((R\  + \ r)\), \ 
          R/\((R\  + \ r)\)};\)\)], "Input",
  CellTags->"Small-Signal Analysis"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Transfer Functions", "Subsection",
  CellTags->"Small-Signal Analysis"],

Cell[TextData[{
  "Various transfer functions can now be determined using ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["A",
            FontWeight->"Bold"], "p"], TraditionalForm]]],
  ", ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["B",
            FontWeight->"Bold"], "p"], TraditionalForm]]],
  ", and ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["C",
            FontWeight->"Bold"], "p"], TraditionalForm]]],
  ". For example, the input-to-output transfer function, ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(g\_io\)(s), \)\)]],
  " and control-to-output transfer function, ",
  Cell[BoxData[
      \(TraditionalForm\`\(g\_co\)(s)\)]],
  ", are given by\n\t",
  Cell[BoxData[
      FormBox[
        RowBox[{"(", "\[NegativeThinSpace]", GridBox[{
              {\(\(g\_io\)(s)\)},
              {\(\(g\_co\)(s)\)}
              }], "\[NegativeThinSpace]", ")"}], TraditionalForm]]],
  " = ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[
            SubscriptBox[
              StyleBox["C",
                FontWeight->"Bold"], "p"],
            "TraditionalForm"], 
          FormBox[
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  StyleBox[
                    RowBox[{
                      StyleBox["s",
                        FontSlant->"Italic"], 
                      StyleBox["I",
                        FontWeight->"Bold",
                        FontSlant->"Italic"]}]], " ", "-", " ", 
                  FormBox[
                    SubscriptBox[
                      StyleBox["A",
                        FontWeight->"Bold"], "p"],
                    "TraditionalForm"]}], ")"}], \(-1\)],
            "TraditionalForm"], 
          StyleBox[\(B\_p\),
            FontWeight->"Bold"]}], TraditionalForm]]]
}], "Text",
  CellTags->"Small-Signal Analysis"],

Cell[BoxData[
    \(\(gio[s_]\  := 
        Part[Cp\  . \ Inverse[s\ IdentityMatrix[2]\  - \ \ Ap]\  . \ Bp, \ 
            1]\  /. \ OP[\([1]\)];\)\)], "Input",
  CellTags->"Small-Signal Analysis"],

Cell[BoxData[
    \(\(gco[s_]\  := 
        Part[Cp\  . \ Inverse[s\ IdentityMatrix[2]\  - \ Ap]\  . \ Bp, \ 
            2]\  /. \ OP[\([1]\)];\)\)], "Input",
  CellTags->"Small-Signal Analysis"],

Cell["\<\
In an analytic form, these two transfer functions can be simplifies \
as follows\
\>", "Text",
  CellTags->"Small-Signal Analysis"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[gio[s]]\)], "Input",
  CellTags->"Small-Signal Analysis"],

Cell[BoxData[
    \(\(-\(\(\((\(-1\) + d)\)\ R\ \((r + R)\)\ \((1 + 
                C\ r\ s)\)\)\/\(C\ r\^2\ s\ \((R - d\ R + L\ s)\) + 
            R\ \((L\ s + R\ \((1 - 2\ d + d\^2 + C\ L\ s\^2)\))\) + 
            r\ \((L\ s - C\ \((\(-1\) + d)\)\ R\^2\ s + 
                  R\ \((1 - d + 2\ C\ L\ s\^2)\))\)\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[gco[s]]\)], "Input",
  CellTags->"Small-Signal Analysis"],

Cell[BoxData[
    \(\(-\(\((R\ \((\(-C\)\ \((\(-1\) + d)\)\ r\^3\ s + 
                R\ \((\((\(-1\) + d)\)\^2\ R - L\ s)\) + \((\(-1\) + 
                      d)\)\ r\^2\ \((\(-1\) + C\ \((\(-2\) + d)\)\ R\ s)\) + 
                r\ \((2\ \((\(-1\) + d)\)\^2\ R - L\ s + 
                      C\ \((\(-1\) + d)\)\^2\ R\^2\ s)\))\)\ Vg)\)/\((\((\(-1\
\) + d)\)\ \((r + R - d\ R)\)\ \((C\ r\^2\ s\ \((R - d\ R + L\ s)\) + 
                R\ \((L\ s + R\ \((1 - 2\ d + d\^2 + C\ L\ s\^2)\))\) + 
                r\ \((L\ s - C\ \((\(-1\) + d)\)\ R\^2\ s + 
                      R\ \((1 - d + 2\ C\ L\ s\^2)\))\))\))\)\)\)\)], "Output"]
}, Open  ]],

Cell["\<\
The parameters of the converter defined before can also be \
substituted to give numerical results:\
\>", "Text",
  CellTags->"Small-Signal Analysis"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[gio[s]\  /. \ Param2]\)], "Input",
  CellTags->"Small-Signal Analysis"],

Cell[BoxData[
    \(\(3.1659882406151056`*^9 + 696.5174129353233`\ \
s\)\/\(2.2209171240135813`*^9 + 23310.71913161465`\ s + s\^2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[gco[s]\  /. \ Param2]\)], "Input",
  CellTags->"Small-Signal Analysis"],

Cell[BoxData[
    \(\(2.256632577558907`*^10 - 224145.46041094142`\ \
s\)\/\(2.2209171240135813`*^9 + 23310.71913161465`\ s + s\^2\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Bode Diagrams", "Subsection",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],

Cell[TextData[{
  "Bode diagrams can be generated using the additional graphics functions ",
  StyleBox["BodeMagnitudePlot",
    FontWeight->"Bold"],
  " and ",
  StyleBox["BodePhasePlot",
    FontWeight->"Bold"],
  " defined in the file ",
  StyleBox["Graphic.m",
    FontWeight->"Bold"],
  " provided by Wolfram Research. A copy of ",
  StyleBox["Graphic.m",
    FontWeight->"Bold"],
  " has been included in this package. Usages of the two functions have been \
explained in that file. The package has been loaded at the beginning of this \
tutorial (see ",
  ButtonBox["Load the Package",
    ButtonData:>"Load Package",
    ButtonStyle->"Hyperlink"],
  ")"
}], "Text",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],

Cell[CellGroupData[{

Cell["Input-To-Output Transfer Function", "Subsubsection",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(BodeMagnitudePlot[
        gio[2\ Pi\ I\ s]\  /. \ Param2, {s, \ 1, \ 100000}, \ 
        Frame\  -> \ True, PlotStyle\  -> \ RGBColor[0, \ 0, \ 1], 
        PlotRange\  -> \ {{0, \ 5}, \ {\(-40\), \ 20}}, 
        GridLines\  -> \ {LogGridMinor, \ Automatic}];\)\)], "Input",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-017 0.2 0.412023 0.0103006 [
[.2 -0.0125 -11 -12.5625 ]
[.2 -0.0125 11 0 ]
[.4 -0.0125 -14 -12.5625 ]
[.4 -0.0125 14 0 ]
[.6 -0.0125 -17 -12.5625 ]
[.6 -0.0125 17 0 ]
[.8 -0.0125 -20 -12.5625 ]
[.8 -0.0125 20 0 ]
[1 -0.0125 -23 -12.5625 ]
[1 -0.0125 23 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .10301 -18 -4.5 ]
[-0.0125 .10301 0 4.5 ]
[-0.0125 .20601 -18 -4.5 ]
[-0.0125 .20601 0 4.5 ]
[-0.0125 .30902 -18 -4.5 ]
[-0.0125 .30902 0 4.5 ]
[-0.0125 .41202 -6 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .51503 -12 -4.5 ]
[-0.0125 .51503 0 4.5 ]
[-0.0125 .61803 -12 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 -0.125 0 ]
[0 .63053 -5 0 ]
[0 .63053 5 12.5625 ]
[.2 .63053 -11 0 ]
[.2 .63053 11 12.5625 ]
[.4 .63053 -14 0 ]
[.4 .63053 14 12.5625 ]
[.6 .63053 -17 0 ]
[.6 .63053 17 12.5625 ]
[.8 .63053 -20 0 ]
[.8 .63053 20 12.5625 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 -0.0005 0 0 ]
[ 1 .61853 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 0 m
0 .61803 L
s
.06021 0 m
.06021 .61803 L
s
.09542 0 m
.09542 .61803 L
s
.12041 0 m
.12041 .61803 L
s
.13979 0 m
.13979 .61803 L
s
.15563 0 m
.15563 .61803 L
s
.16902 0 m
.16902 .61803 L
s
.18062 0 m
.18062 .61803 L
s
.19085 0 m
.19085 .61803 L
s
.2 0 m
.2 .61803 L
s
.26021 0 m
.26021 .61803 L
s
.29542 0 m
.29542 .61803 L
s
.32041 0 m
.32041 .61803 L
s
.33979 0 m
.33979 .61803 L
s
.35563 0 m
.35563 .61803 L
s
.36902 0 m
.36902 .61803 L
s
.38062 0 m
.38062 .61803 L
s
.39085 0 m
.39085 .61803 L
s
.4 0 m
.4 .61803 L
s
.46021 0 m
.46021 .61803 L
s
.49542 0 m
.49542 .61803 L
s
.52041 0 m
.52041 .61803 L
s
.53979 0 m
.53979 .61803 L
s
.55563 0 m
.55563 .61803 L
s
.56902 0 m
.56902 .61803 L
s
.58062 0 m
.58062 .61803 L
s
.59085 0 m
.59085 .61803 L
s
.6 0 m
.6 .61803 L
s
.66021 0 m
.66021 .61803 L
s
.69542 0 m
.69542 .61803 L
s
.72041 0 m
.72041 .61803 L
s
.73979 0 m
.73979 .61803 L
s
.75563 0 m
.75563 .61803 L
s
.76902 0 m
.76902 .61803 L
s
.78062 0 m
.78062 .61803 L
s
.79085 0 m
.79085 .61803 L
s
.8 0 m
.8 .61803 L
s
.86021 0 m
.86021 .61803 L
s
.89542 0 m
.89542 .61803 L
s
.92041 0 m
.92041 .61803 L
s
.93979 0 m
.93979 .61803 L
s
.95563 0 m
.95563 .61803 L
s
.96902 0 m
.96902 .61803 L
s
.98062 0 m
.98062 .61803 L
s
.99085 0 m
.99085 .61803 L
s
1 0 m
1 .61803 L
s
0 .10301 m
1 .10301 L
s
0 .20601 m
1 .20601 L
s
0 .30902 m
1 .30902 L
s
0 .41202 m
1 .41202 L
s
0 .51503 m
1 .51503 L
s
0 g
.2 0 m
.2 .00625 L
s
gsave
.2 -0.0125 -72 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10.) show
81.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.4 0 m
.4 .00625 L
s
gsave
.4 -0.0125 -75 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100.) show
87.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.6 0 m
.6 .00625 L
s
gsave
.6 -0.0125 -78 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1000.) show
93.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.8 0 m
.8 .00625 L
s
gsave
.8 -0.0125 -81 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10000.) show
99.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
1 0 m
1 .00625 L
s
gsave
1 -0.0125 -84 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100000.) show
105.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.001 w
.06021 0 m
.06021 .00375 L
s
.09542 0 m
.09542 .00375 L
s
.12041 0 m
.12041 .00375 L
s
.13979 0 m
.13979 .00375 L
s
.15563 0 m
.15563 .00375 L
s
.16902 0 m
.16902 .00375 L
s
.18062 0 m
.18062 .00375 L
s
.19085 0 m
.19085 .00375 L
s
.26021 0 m
.26021 .00375 L
s
.29542 0 m
.29542 .00375 L
s
.32041 0 m
.32041 .00375 L
s
.33979 0 m
.33979 .00375 L
s
.35563 0 m
.35563 .00375 L
s
.36902 0 m
.36902 .00375 L
s
.38062 0 m
.38062 .00375 L
s
.39085 0 m
.39085 .00375 L
s
.46021 0 m
.46021 .00375 L
s
.49542 0 m
.49542 .00375 L
s
.52041 0 m
.52041 .00375 L
s
.53979 0 m
.53979 .00375 L
s
.55563 0 m
.55563 .00375 L
s
.56902 0 m
.56902 .00375 L
s
.58062 0 m
.58062 .00375 L
s
.59085 0 m
.59085 .00375 L
s
.66021 0 m
.66021 .00375 L
s
.69542 0 m
.69542 .00375 L
s
.72041 0 m
.72041 .00375 L
s
.73979 0 m
.73979 .00375 L
s
.75563 0 m
.75563 .00375 L
s
.76902 0 m
.76902 .00375 L
s
.78062 0 m
.78062 .00375 L
s
.79085 0 m
.79085 .00375 L
s
.86021 0 m
.86021 .00375 L
s
.89542 0 m
.89542 .00375 L
s
.92041 0 m
.92041 .00375 L
s
.93979 0 m
.93979 .00375 L
s
.95563 0 m
.95563 .00375 L
s
.96902 0 m
.96902 .00375 L
s
.98062 0 m
.98062 .00375 L
s
.99085 0 m
.99085 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .10301 m
.00625 .10301 L
s
[(-30)] -0.0125 .10301 1 0 Mshowa
0 .20601 m
.00625 .20601 L
s
[(-20)] -0.0125 .20601 1 0 Mshowa
0 .30902 m
.00625 .30902 L
s
[(-10)] -0.0125 .30902 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(0)] -0.0125 .41202 1 0 Mshowa
0 .51503 m
.00625 .51503 L
s
[(10)] -0.0125 .51503 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(20)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .0824 m
.00375 .0824 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .16481 m
.00375 .16481 L
s
0 .18541 m
.00375 .18541 L
s
0 .22661 m
.00375 .22661 L
s
0 .24721 m
.00375 .24721 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .32962 m
.00375 .32962 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .49443 m
.00375 .49443 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .57683 m
.00375 .57683 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 .61178 m
0 .61803 L
s
gsave
0 .63053 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
69.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.2 .61178 m
.2 .61803 L
s
gsave
.2 .63053 -72 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10.) show
81.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.4 .61178 m
.4 .61803 L
s
gsave
.4 .63053 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100.) show
87.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.6 .61178 m
.6 .61803 L
s
gsave
.6 .63053 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1000.) show
93.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.8 .61178 m
.8 .61803 L
s
gsave
.8 .63053 -81 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10000.) show
99.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.001 w
.06021 .61428 m
.06021 .61803 L
s
.09542 .61428 m
.09542 .61803 L
s
.12041 .61428 m
.12041 .61803 L
s
.13979 .61428 m
.13979 .61803 L
s
.15563 .61428 m
.15563 .61803 L
s
.16902 .61428 m
.16902 .61803 L
s
.18062 .61428 m
.18062 .61803 L
s
.19085 .61428 m
.19085 .61803 L
s
.26021 .61428 m
.26021 .61803 L
s
.29542 .61428 m
.29542 .61803 L
s
.32041 .61428 m
.32041 .61803 L
s
.33979 .61428 m
.33979 .61803 L
s
.35563 .61428 m
.35563 .61803 L
s
.36902 .61428 m
.36902 .61803 L
s
.38062 .61428 m
.38062 .61803 L
s
.39085 .61428 m
.39085 .61803 L
s
.46021 .61428 m
.46021 .61803 L
s
.49542 .61428 m
.49542 .61803 L
s
.52041 .61428 m
.52041 .61803 L
s
.53979 .61428 m
.53979 .61803 L
s
.55563 .61428 m
.55563 .61803 L
s
.56902 .61428 m
.56902 .61803 L
s
.58062 .61428 m
.58062 .61803 L
s
.59085 .61428 m
.59085 .61803 L
s
.66021 .61428 m
.66021 .61803 L
s
.69542 .61428 m
.69542 .61803 L
s
.72041 .61428 m
.72041 .61803 L
s
.73979 .61428 m
.73979 .61803 L
s
.75563 .61428 m
.75563 .61803 L
s
.76902 .61428 m
.76902 .61803 L
s
.78062 .61428 m
.78062 .61803 L
s
.79085 .61428 m
.79085 .61803 L
s
.86021 .61428 m
.86021 .61803 L
s
.89542 .61428 m
.89542 .61803 L
s
.92041 .61428 m
.92041 .61803 L
s
.93979 .61428 m
.93979 .61803 L
s
.95563 .61428 m
.95563 .61803 L
s
.96902 .61428 m
.96902 .61803 L
s
.98062 .61428 m
.98062 .61803 L
s
.99085 .61428 m
.99085 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .10301 m
1 .10301 L
s
.99375 .20601 m
1 .20601 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .41202 m
1 .41202 L
s
.99375 .51503 m
1 .51503 L
s
.125 Mabswid
.99625 .0206 m
1 .0206 L
s
.99625 .0412 m
1 .0412 L
s
.99625 .0618 m
1 .0618 L
s
.99625 .0824 m
1 .0824 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .14421 m
1 .14421 L
s
.99625 .16481 m
1 .16481 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .22661 m
1 .22661 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26781 m
1 .26781 L
s
.99625 .28842 m
1 .28842 L
s
.99625 .32962 m
1 .32962 L
s
.99625 .35022 m
1 .35022 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .39142 m
1 .39142 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .45322 m
1 .45322 L
s
.99625 .47383 m
1 .47383 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .53563 m
1 .53563 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57683 m
1 .57683 L
s
.99625 .59743 m
1 .59743 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
.5 Mabswid
.00036 .44374 m
.01921 .44374 L
.03398 .44374 L
.0454 .44374 L
.05635 .44374 L
.06683 .44374 L
.07677 .44374 L
.08521 .44374 L
.09221 .44374 L
.09879 .44374 L
.10444 .44374 L
.11025 .44374 L
.11598 .44374 L
.12485 .44374 L
.12951 .44374 L
.13365 .44374 L
.13819 .44374 L
.14287 .44374 L
.14707 .44374 L
.15079 .44374 L
.154 .44374 L
.15741 .44374 L
.16083 .44374 L
.16392 .44374 L
.16721 .44374 L
.17054 .44374 L
.17329 .44374 L
.1762 .44374 L
.17924 .44374 L
.18237 .44374 L
.18523 .44374 L
.18774 .44374 L
.19013 .44374 L
.19268 .44374 L
.19526 .44374 L
.19761 .44374 L
.19976 .44374 L
.20167 .44374 L
.20382 .44374 L
.20582 .44374 L
.20792 .44374 L
.21011 .44374 L
.21212 .44374 L
.2139 .44374 L
.21596 .44374 L
.21813 .44374 L
.22013 .44374 L
.22192 .44374 L
.22346 .44374 L
.22507 .44374 L
Mistroke
.22657 .44374 L
.22791 .44374 L
.22945 .44374 L
.2311 .44374 L
.23263 .44374 L
.23402 .44374 L
.23543 .44374 L
.23673 .44374 L
.23813 .44374 L
.23962 .44374 L
.24107 .44374 L
.24257 .44374 L
.24383 .44374 L
.24519 .44374 L
.24646 .44374 L
.24761 .44374 L
.24885 .44374 L
.25015 .44374 L
.25126 .44374 L
.25229 .44374 L
.2534 .44374 L
.25458 .44374 L
.25561 .44374 L
.25672 .44374 L
.25775 .44374 L
.2587 .44374 L
.26046 .44374 L
.26134 .44374 L
.2623 .44374 L
.26334 .44374 L
.26442 .44374 L
.26533 .44374 L
.26631 .44374 L
.26737 .44374 L
.26848 .44374 L
.26952 .44374 L
.27045 .44374 L
.27138 .44374 L
.27223 .44374 L
.27316 .44374 L
.27413 .44374 L
.27496 .44374 L
.27573 .44374 L
.27657 .44374 L
.27746 .44374 L
.27827 .44374 L
.27911 .44374 L
.2799 .44374 L
.28061 .44374 L
.28145 .44374 L
Mistroke
.28234 .44374 L
.28317 .44374 L
.28393 .44374 L
.28529 .44374 L
.28605 .44374 L
.28676 .44374 L
.2875 .44374 L
.28817 .44374 L
.28893 .44374 L
.28965 .44374 L
.2909 .44374 L
.29152 .44374 L
.2922 .44374 L
.29339 .44374 L
.29449 .44374 L
.29512 .44374 L
.29577 .44374 L
.29693 .44374 L
.29756 .44374 L
.29815 .44374 L
.29879 .44375 L
.29947 .44375 L
.3002 .44375 L
.30088 .44375 L
.30162 .44375 L
.30241 .44375 L
.30322 .44375 L
.30399 .44375 L
.30467 .44375 L
.30541 .44375 L
.30611 .44375 L
.30673 .44375 L
.30745 .44375 L
.30811 .44375 L
.30873 .44375 L
.30929 .44375 L
.3099 .44375 L
.31055 .44375 L
.31125 .44375 L
.31188 .44375 L
.3126 .44375 L
.31329 .44375 L
.31391 .44375 L
.31448 .44375 L
.31511 .44375 L
.31576 .44375 L
.31642 .44375 L
.31711 .44375 L
.31777 .44375 L
.31836 .44375 L
Mistroke
.31905 .44375 L
.3198 .44375 L
.3205 .44375 L
.32114 .44375 L
.32173 .44375 L
.32227 .44375 L
.32285 .44375 L
.32347 .44375 L
.324 .44375 L
.3245 .44375 L
.32504 .44375 L
.32561 .44375 L
.32612 .44375 L
.32667 .44375 L
.32718 .44375 L
.32766 .44375 L
.32808 .44375 L
.32855 .44375 L
.32903 .44375 L
.32948 .44375 L
.32998 .44375 L
.33051 .44375 L
.33107 .44375 L
.33159 .44375 L
.33207 .44375 L
.3325 .44375 L
.333 .44375 L
.33347 .44375 L
.33398 .44375 L
.33451 .44375 L
.33497 .44375 L
.33546 .44375 L
.33599 .44375 L
.33656 .44375 L
.33709 .44375 L
.33758 .44375 L
.33804 .44375 L
.33848 .44375 L
.33887 .44375 L
.33929 .44375 L
.33974 .44375 L
.34022 .44375 L
.34063 .44375 L
.34107 .44375 L
.34148 .44375 L
.34186 .44375 L
.34229 .44375 L
.3427 .44375 L
.34307 .44375 L
.34342 .44375 L
Mistroke
.34379 .44375 L
.34419 .44375 L
.34462 .44375 L
.34501 .44375 L
.34543 .44375 L
.34588 .44375 L
.34627 .44375 L
.34662 .44375 L
.34702 .44375 L
.34743 .44375 L
.34789 .44375 L
.34837 .44375 L
.34883 .44375 L
.34925 .44375 L
.34963 .44375 L
.35004 .44375 L
.35047 .44375 L
.35087 .44375 L
.35125 .44375 L
.35159 .44375 L
.35197 .44375 L
.35237 .44375 L
.35272 .44375 L
.35304 .44375 L
.3534 .44375 L
.35378 .44375 L
.35409 .44375 L
.35443 .44375 L
.35479 .44375 L
.35512 .44375 L
.35572 .44375 L
.35604 .44375 L
.35634 .44375 L
.3567 .44375 L
.35707 .44375 L
.35739 .44375 L
.35774 .44375 L
.35811 .44375 L
.35851 .44375 L
.35888 .44375 L
.35922 .44375 L
.35956 .44375 L
.35987 .44375 L
.36022 .44375 L
.36058 .44375 L
.36093 .44375 L
.36125 .44375 L
.36154 .44375 L
.36186 .44375 L
.36217 .44375 L
Mistroke
.36251 .44375 L
.36283 .44375 L
.36312 .44375 L
.36338 .44375 L
.36367 .44375 L
.36398 .44375 L
.36426 .44375 L
.36457 .44375 L
.36489 .44375 L
.36517 .44375 L
.36547 .44375 L
.3658 .44375 L
.36614 .44375 L
.36647 .44375 L
.36676 .44375 L
.36704 .44375 L
.36733 .44375 L
.36786 .44375 L
.36814 .44375 L
.36841 .44375 L
.36871 .44375 L
.36902 .44375 L
.36929 .44375 L
.36959 .44375 L
.36986 .44375 L
.37012 .44375 L
.3706 .44375 L
.37087 .44375 L
.37111 .44375 L
.37139 .44375 L
.37171 .44375 L
.37204 .44375 L
.37234 .44375 L
.37263 .44375 L
.37288 .44375 L
.37318 .44375 L
.37346 .44375 L
.37376 .44375 L
.37408 .44375 L
.37436 .44375 L
.37465 .44375 L
.37497 .44375 L
.37532 .44375 L
.37564 .44375 L
.37593 .44375 L
.3762 .44375 L
.37649 .44375 L
.37676 .44375 L
.37701 .44375 L
.3773 .44375 L
Mistroke
.37761 .44375 L
.3779 .44375 L
.37817 .44375 L
.37842 .44375 L
.37865 .44375 L
.37891 .44375 L
.37918 .44375 L
.37947 .44375 L
.37974 .44375 L
.37998 .44375 L
.38024 .44375 L
.38051 .44375 L
.38076 .44375 L
.38103 .44375 L
.38131 .44375 L
.38156 .44375 L
.38179 .44375 L
.38204 .44375 L
.38231 .44375 L
.3826 .44375 L
.38291 .44375 L
.38321 .44375 L
.38348 .44375 L
.38373 .44375 L
.384 .44375 L
.38428 .44375 L
.38454 .44375 L
.38481 .44375 L
.38511 .44375 L
.38539 .44375 L
.38565 .44375 L
.38591 .44375 L
.38614 .44375 L
.38641 .44375 L
.38666 .44375 L
.38693 .44375 L
.38722 .44375 L
.38747 .44375 L
.38773 .44375 L
.38802 .44375 L
.38833 .44375 L
.38862 .44375 L
.38888 .44375 L
.38911 .44375 L
.38935 .44375 L
.38961 .44375 L
.38985 .44375 L
.39008 .44375 L
.39028 .44375 L
.3905 .44375 L
Mistroke
.39074 .44375 L
.39098 .44375 L
.39123 .44376 L
.39147 .44376 L
.39169 .44376 L
.39194 .44376 L
.39221 .44376 L
.39247 .44376 L
.39271 .44376 L
.39291 .44376 L
.39313 .44376 L
.39334 .44376 L
.39353 .44376 L
.39375 .44376 L
.39398 .44376 L
.39421 .44376 L
.39442 .44376 L
.39465 .44376 L
.39487 .44376 L
.39507 .44376 L
.39528 .44376 L
.39551 .44376 L
.39575 .44376 L
.39596 .44376 L
.39618 .44376 L
.3964 .44376 L
.39662 .44376 L
.39681 .44376 L
.39702 .44376 L
.39725 .44376 L
.39749 .44376 L
.39771 .44376 L
.39792 .44376 L
.39813 .44376 L
.39832 .44376 L
.39853 .44376 L
.39875 .44376 L
.39896 .44376 L
.39916 .44376 L
.39934 .44376 L
.39953 .44376 L
.39989 .44376 L
.4001 .44376 L
.40029 .44376 L
.40049 .44376 L
.40067 .44376 L
.40088 .44376 L
.40108 .44376 L
.40143 .44376 L
.40175 .44376 L
Mistroke
.4021 .44376 L
.40228 .44376 L
.40247 .44376 L
.40267 .44376 L
.40288 .44376 L
.4031 .44376 L
.40331 .44376 L
.4035 .44376 L
.40368 .44376 L
.40388 .44376 L
.40407 .44376 L
.40428 .44376 L
.4045 .44376 L
.40468 .44376 L
.40489 .44376 L
.4051 .44376 L
.40534 .44376 L
.40556 .44376 L
.40576 .44376 L
.40595 .44376 L
.40614 .44376 L
.40633 .44376 L
.4065 .44376 L
.4067 .44376 L
.40691 .44376 L
.40711 .44376 L
.4073 .44376 L
.40747 .44376 L
.40763 .44376 L
.40781 .44376 L
.408 .44376 L
.4082 .44376 L
.40839 .44376 L
.40855 .44376 L
.40873 .44376 L
.40906 .44376 L
.40924 .44376 L
.40941 .44376 L
.40971 .44376 L
.41003 .44376 L
.4102 .44376 L
.41038 .44376 L
.41055 .44376 L
.41071 .44376 L
.41089 .44376 L
.41108 .44376 L
.41127 .44376 L
.41144 .44376 L
.4116 .44376 L
.41178 .44376 L
Mistroke
.41209 .44376 L
.41228 .44376 L
.41246 .44376 L
.41262 .44376 L
.4128 .44376 L
.41297 .44376 L
.41316 .44376 L
.41335 .44376 L
.41354 .44376 L
.41371 .44376 L
.41387 .44376 L
.41404 .44376 L
.41422 .44376 L
.41452 .44376 L
.4148 .44376 L
.41496 .44376 L
.41512 .44376 L
.41542 .44376 L
.4157 .44376 L
.416 .44376 L
.41625 .44376 L
.41653 .44376 L
.4167 .44376 L
.41685 .44376 L
.41715 .44376 L
.41731 .44376 L
.41747 .44376 L
.41763 .44376 L
.41781 .44376 L
.41796 .44376 L
.41813 .44376 L
.41829 .44376 L
.41843 .44376 L
.41871 .44377 L
.41899 .44377 L
.41914 .44377 L
.4193 .44377 L
.41946 .44377 L
.41961 .44377 L
.41989 .44377 L
.42004 .44377 L
.42019 .44377 L
.42034 .44377 L
.42049 .44377 L
.42064 .44377 L
.42081 .44377 L
.42099 .44377 L
.42115 .44377 L
.4213 .44377 L
.42147 .44377 L
Mistroke
.42174 .44377 L
.4219 .44377 L
.42205 .44377 L
.4222 .44377 L
.42234 .44377 L
.4225 .44377 L
.42266 .44377 L
.42293 .44377 L
.42318 .44377 L
.42345 .44377 L
.42361 .44377 L
.42374 .44377 L
.42401 .44377 L
.42414 .44377 L
.42429 .44377 L
.42454 .44377 L
.42478 .44377 L
.42493 .44377 L
.42506 .44377 L
.42532 .44377 L
.42546 .44377 L
.42559 .44377 L
.42574 .44377 L
.42589 .44377 L
.42612 .44377 L
.42638 .44377 L
.42661 .44377 L
.42683 .44377 L
.42708 .44377 L
.42723 .44377 L
.42736 .44377 L
.4276 .44377 L
.42786 .44377 L
.428 .44377 L
.42815 .44377 L
.42832 .44377 L
.42847 .44377 L
.42861 .44377 L
.42873 .44377 L
.42888 .44377 L
.42902 .44377 L
.42917 .44377 L
.42933 .44377 L
.42947 .44377 L
.42962 .44377 L
.42978 .44377 L
.42995 .44377 L
.43011 .44377 L
.43026 .44377 L
.43039 .44377 L
Mistroke
.43053 .44377 L
.43078 .44377 L
.43093 .44377 L
.43108 .44377 L
.4312 .44377 L
.43134 .44377 L
.43148 .44377 L
.43163 .44377 L
.43179 .44377 L
.43193 .44377 L
.43207 .44377 L
.43219 .44377 L
.43233 .44377 L
.43247 .44377 L
.4326 .44377 L
.43273 .44377 L
.43296 .44377 L
.43309 .44377 L
.43321 .44377 L
.43334 .44377 L
.43347 .44377 L
.4337 .44377 L
.43391 .44377 L
.43415 .44377 L
.43428 .44377 L
.4344 .44377 L
.43452 .44377 L
.43466 .44377 L
.43478 .44377 L
.43491 .44377 L
.43504 .44377 L
.43519 .44378 L
.43533 .44378 L
.43546 .44378 L
.43569 .44378 L
.43582 .44378 L
.43594 .44378 L
.43607 .44378 L
.43619 .44378 L
.43632 .44378 L
.43645 .44378 L
.43666 .44378 L
.43689 .44378 L
.4371 .44378 L
.4373 .44378 L
.43753 .44378 L
.43766 .44378 L
.43777 .44378 L
.43799 .44378 L
.43822 .44378 L
Mistroke
.43844 .44378 L
.43856 .44378 L
.43868 .44378 L
.43888 .44378 L
.4391 .44378 L
.43922 .44378 L
.43934 .44378 L
.43956 .44378 L
.4397 .44378 L
.43982 .44378 L
.43994 .44378 L
.44006 .44378 L
.44026 .44378 L
.44045 .44378 L
.44066 .44378 L
.44088 .44378 L
.44099 .44378 L
.44111 .44378 L
.44133 .44378 L
.44154 .44378 L
.44174 .44378 L
.44194 .44378 L
.44213 .44378 L
.4423 .44378 L
.44248 .44378 L
.44268 .44378 L
.44289 .44378 L
.44306 .44378 L
.44326 .44378 L
.44338 .44378 L
.44349 .44378 L
.44361 .44378 L
.44373 .44378 L
.44395 .44378 L
.44407 .44378 L
.44418 .44378 L
.44439 .44378 L
.44458 .44378 L
.44479 .44378 L
.4449 .44378 L
.44501 .44378 L
.44514 .44378 L
.44526 .44378 L
.44536 .44378 L
.44548 .44378 L
.4456 .44378 L
.44574 .44378 L
.44585 .44378 L
.44597 .44378 L
.44609 .44378 L
Mistroke
.44619 .44378 L
.44631 .44378 L
.44643 .44378 L
.44663 .44378 L
.44675 .44378 L
.44685 .44378 L
.44706 .44378 L
.44718 .44379 L
.44728 .44379 L
.44748 .44379 L
.44769 .44379 L
.44787 .44379 L
.44804 .44379 L
.44823 .44379 L
.44843 .44379 L
.44854 .44379 L
.44865 .44379 L
.44876 .44379 L
.44888 .44379 L
.44908 .44379 L
.44919 .44379 L
.4493 .44379 L
.4495 .44379 L
.44968 .44379 L
.44987 .44379 L
.44997 .44379 L
.45008 .44379 L
.45019 .44379 L
.45032 .44379 L
.45044 .44379 L
.45056 .44379 L
.45068 .44379 L
.45078 .44379 L
.4509 .44379 L
.45101 .44379 L
.45113 .44379 L
.45125 .44379 L
.45136 .44379 L
.45148 .44379 L
.45161 .44379 L
.45175 .44379 L
.45188 .44379 L
.452 .44379 L
.4521 .44379 L
.45221 .44379 L
.45232 .44379 L
.45241 .44379 L
.45253 .44379 L
.45265 .44379 L
.45276 .44379 L
Mistroke
.45287 .44379 L
.45297 .44379 L
.45308 .44379 L
.45319 .44379 L
.45331 .44379 L
.45343 .44379 L
.45355 .44379 L
.45366 .44379 L
.45377 .44379 L
.45388 .44379 L
.45399 .44379 L
.4541 .44379 L
.4542 .44379 L
.45439 .44379 L
.45449 .44379 L
.45459 .44379 L
.4547 .44379 L
.45481 .44379 L
.45493 .44379 L
.45504 .44379 L
.45514 .44379 L
.45524 .44379 L
.45534 .44379 L
.45545 .44379 L
.45564 .44379 L
.45574 .44379 L
.45583 .44379 L
.45601 .44379 L
.45619 .44379 L
.45628 .44379 L
.45639 .44379 L
.45657 .4438 L
.45668 .4438 L
.45679 .4438 L
.45689 .4438 L
.45699 .4438 L
.4571 .4438 L
.4572 .4438 L
.45732 .4438 L
.45743 .4438 L
.45753 .4438 L
.45763 .4438 L
.45773 .4438 L
.45784 .4438 L
.45796 .4438 L
.45809 .4438 L
.45821 .4438 L
.45832 .4438 L
.45843 .4438 L
.45854 .4438 L
Mistroke
.45865 .4438 L
.45876 .4438 L
.45888 .4438 L
.459 .4438 L
.45912 .4438 L
.45923 .4438 L
.45933 .4438 L
.45944 .4438 L
.45955 .4438 L
.45966 .4438 L
.45978 .4438 L
.45989 .4438 L
.46 .4438 L
.46011 .4438 L
.46023 .4438 L
.46037 .4438 L
.46049 .4438 L
.4606 .4438 L
.4607 .4438 L
.4608 .4438 L
.46092 .4438 L
.46102 .4438 L
.46112 .4438 L
.4612 .4438 L
.46131 .4438 L
.4614 .4438 L
.46151 .4438 L
.46162 .4438 L
.46171 .4438 L
.46182 .4438 L
.46193 .4438 L
.46205 .4438 L
.46216 .4438 L
.46226 .4438 L
.46235 .4438 L
.46245 .4438 L
.46262 .4438 L
.46273 .4438 L
.46283 .4438 L
.46292 .4438 L
.46302 .4438 L
.46311 .4438 L
.46321 .4438 L
.46332 .4438 L
.46342 .4438 L
.46352 .4438 L
.46361 .4438 L
.46371 .4438 L
.4638 .4438 L
.4639 .4438 L
Mistroke
.46398 .4438 L
.46408 .4438 L
.46418 .4438 L
.46435 .44381 L
.46445 .44381 L
.46454 .44381 L
.46471 .44381 L
.46486 .44381 L
.46495 .44381 L
.46504 .44381 L
.46521 .44381 L
.4653 .44381 L
.46538 .44381 L
.46548 .44381 L
.46558 .44381 L
.46568 .44381 L
.46579 .44381 L
.46588 .44381 L
.46597 .44381 L
.46607 .44381 L
.46615 .44381 L
.46625 .44381 L
.46634 .44381 L
.4665 .44381 L
.46659 .44381 L
.46668 .44381 L
.46683 .44381 L
.46699 .44381 L
.46715 .44381 L
.46728 .44381 L
.46745 .44381 L
.46754 .44381 L
.46762 .44381 L
.46778 .44381 L
.46795 .44381 L
.4681 .44381 L
.46819 .44381 L
.46827 .44381 L
.46842 .44381 L
.46858 .44381 L
.46866 .44381 L
.46875 .44381 L
.46891 .44381 L
.469 .44381 L
.46909 .44381 L
.46918 .44381 L
.46927 .44381 L
.46936 .44381 L
.46944 .44381 L
.46953 .44381 L
Mistroke
.46962 .44381 L
.46978 .44381 L
.46987 .44381 L
.46996 .44381 L
.47006 .44381 L
.47015 .44381 L
.47023 .44381 L
.47032 .44381 L
.47042 .44381 L
.47053 .44381 L
.47061 .44381 L
.47071 .44381 L
.47081 .44381 L
.47091 .44382 L
.471 .44382 L
.47109 .44382 L
.47119 .44382 L
.4713 .44382 L
.47139 .44382 L
.47149 .44382 L
.47159 .44382 L
.47167 .44382 L
.47177 .44382 L
.47187 .44382 L
.47196 .44382 L
.47204 .44382 L
.47212 .44382 L
.47222 .44382 L
.47231 .44382 L
.4724 .44382 L
.47248 .44382 L
.47256 .44382 L
.47266 .44382 L
.47276 .44382 L
.47285 .44382 L
.47294 .44382 L
.47302 .44382 L
.4731 .44382 L
.47319 .44382 L
.47328 .44382 L
.47337 .44382 L
.47346 .44382 L
.47353 .44382 L
.47361 .44382 L
.47376 .44382 L
.47385 .44382 L
.47393 .44382 L
.47401 .44382 L
.47409 .44382 L
.47418 .44382 L
Mistroke
.47426 .44382 L
.47441 .44382 L
.47455 .44382 L
.4747 .44382 L
.47477 .44382 L
.47486 .44382 L
.47495 .44382 L
.47506 .44382 L
.47515 .44382 L
.47524 .44382 L
.47532 .44382 L
.47541 .44382 L
.47551 .44382 L
.47559 .44382 L
.47569 .44382 L
.47579 .44382 L
.47587 .44382 L
.47597 .44382 L
.47607 .44382 L
.47618 .44382 L
.47627 .44382 L
.47637 .44382 L
.47646 .44382 L
.47655 .44383 L
.47663 .44383 L
.47671 .44383 L
.47681 .44383 L
.47691 .44383 L
.477 .44383 L
.47708 .44383 L
.47717 .44383 L
.47725 .44383 L
.47734 .44383 L
.47743 .44383 L
.47751 .44383 L
.47758 .44383 L
.47766 .44383 L
.47775 .44383 L
.47785 .44383 L
.47796 .44383 L
.47806 .44383 L
.47815 .44383 L
.47825 .44383 L
.47837 .44383 L
.47848 .44383 L
.47858 .44383 L
.47867 .44383 L
.47876 .44383 L
.47886 .44383 L
.47896 .44383 L
Mistroke
.47907 .44383 L
.47917 .44383 L
.47926 .44383 L
.47936 .44383 L
.47945 .44383 L
.47953 .44383 L
.47963 .44383 L
.47973 .44383 L
.47982 .44383 L
.47993 .44383 L
.48002 .44383 L
.48011 .44383 L
.4802 .44383 L
.48029 .44383 L
.48038 .44383 L
.48047 .44383 L
.48055 .44383 L
.48063 .44383 L
.48072 .44383 L
.4808 .44383 L
.48089 .44383 L
.48098 .44383 L
.48106 .44383 L
.48114 .44383 L
.48122 .44383 L
.48129 .44383 L
.48137 .44383 L
.48145 .44383 L
.48153 .44383 L
.48162 .44384 L
.4817 .44384 L
.48178 .44384 L
.48187 .44384 L
.48197 .44384 L
.48206 .44384 L
.48214 .44384 L
.48222 .44384 L
.4823 .44384 L
.48237 .44384 L
.48244 .44384 L
.48253 .44384 L
.48261 .44384 L
.4827 .44384 L
.48277 .44384 L
.48285 .44384 L
.48292 .44384 L
.483 .44384 L
.48308 .44384 L
.48316 .44384 L
.48323 .44384 L
Mistroke
.48337 .44384 L
.48345 .44384 L
.48353 .44384 L
.48362 .44384 L
.48371 .44384 L
.4838 .44384 L
.48389 .44384 L
.48397 .44384 L
.48406 .44384 L
.48414 .44384 L
.48422 .44384 L
.48431 .44384 L
.48439 .44384 L
.48447 .44384 L
.48454 .44384 L
.48461 .44384 L
.48469 .44384 L
.48478 .44384 L
.48486 .44384 L
.48496 .44384 L
.48505 .44384 L
.48513 .44384 L
.4852 .44384 L
.48528 .44384 L
.48537 .44384 L
.48546 .44384 L
.48555 .44384 L
.48564 .44384 L
.48572 .44384 L
.48582 .44384 L
.48592 .44384 L
.48602 .44384 L
.48611 .44385 L
.48618 .44385 L
.48626 .44385 L
.48634 .44385 L
.48642 .44385 L
.48649 .44385 L
.48656 .44385 L
.48664 .44385 L
.48671 .44385 L
.48679 .44385 L
.48687 .44385 L
.48694 .44385 L
.48702 .44385 L
.48711 .44385 L
.4872 .44385 L
.48728 .44385 L
.48736 .44385 L
.48743 .44385 L
Mistroke
.4875 .44385 L
.48763 .44385 L
.48771 .44385 L
.48779 .44385 L
.48786 .44385 L
.48793 .44385 L
.488 .44385 L
.48808 .44385 L
.48816 .44385 L
.48824 .44385 L
.48831 .44385 L
.48838 .44385 L
.48845 .44385 L
.48853 .44385 L
.48865 .44385 L
.48872 .44385 L
.48879 .44385 L
.48885 .44385 L
.48892 .44385 L
.48899 .44385 L
.48906 .44385 L
.48918 .44385 L
.4893 .44385 L
.48942 .44385 L
.48949 .44385 L
.48955 .44385 L
.48962 .44385 L
.4897 .44385 L
.48976 .44385 L
.48983 .44385 L
.4899 .44385 L
.48998 .44385 L
.49006 .44385 L
.49013 .44385 L
.49019 .44386 L
.49026 .44386 L
.49033 .44386 L
.4904 .44386 L
.49047 .44386 L
.49054 .44386 L
.49066 .44386 L
.49073 .44386 L
.4908 .44386 L
.49087 .44386 L
.49093 .44386 L
.49101 .44386 L
.49109 .44386 L
.49116 .44386 L
.49123 .44386 L
.49131 .44386 L
Mistroke
.49137 .44386 L
.49145 .44386 L
.49153 .44386 L
.49161 .44386 L
.49169 .44386 L
.49176 .44386 L
.49184 .44386 L
.49191 .44386 L
.49198 .44386 L
.49204 .44386 L
.4921 .44386 L
.49218 .44386 L
.49225 .44386 L
.49232 .44386 L
.49239 .44386 L
.49246 .44386 L
.49252 .44386 L
.49259 .44386 L
.49266 .44386 L
.49277 .44386 L
.49284 .44386 L
.4929 .44386 L
.49298 .44386 L
.49305 .44386 L
.49313 .44386 L
.49322 .44386 L
.49329 .44386 L
.49336 .44386 L
.49343 .44386 L
.49351 .44386 L
.49358 .44386 L
.49366 .44386 L
.49373 .44386 L
.49379 .44386 L
.49385 .44386 L
.49392 .44387 L
.49399 .44387 L
.49405 .44387 L
.49413 .44387 L
.49421 .44387 L
.49429 .44387 L
.49436 .44387 L
.49443 .44387 L
.49449 .44387 L
.49457 .44387 L
.49464 .44387 L
.49471 .44387 L
.49479 .44387 L
.49486 .44387 L
.49494 .44387 L
Mistroke
.49502 .44387 L
.49511 .44387 L
.49519 .44387 L
.49526 .44387 L
.49534 .44387 L
.49541 .44387 L
.49548 .44387 L
.49556 .44387 L
.49563 .44387 L
.49569 .44387 L
.49576 .44387 L
.49584 .44387 L
.49592 .44387 L
.49601 .44387 L
.49609 .44387 L
.49617 .44387 L
.49624 .44387 L
.49631 .44387 L
.49639 .44387 L
.49647 .44387 L
.49654 .44387 L
.49663 .44387 L
.49671 .44387 L
.49678 .44387 L
.49686 .44387 L
.49693 .44387 L
.497 .44387 L
.49708 .44387 L
.49716 .44387 L
.49724 .44387 L
.49731 .44387 L
.49739 .44388 L
.49747 .44388 L
.49756 .44388 L
.49765 .44388 L
.49773 .44388 L
.49779 .44388 L
.49787 .44388 L
.49794 .44388 L
.49801 .44388 L
.49808 .44388 L
.49814 .44388 L
.49821 .44388 L
.49828 .44388 L
.49835 .44388 L
.49843 .44388 L
.4985 .44388 L
.49856 .44388 L
.49864 .44388 L
.49872 .44388 L
Mistroke
.4988 .44388 L
.49887 .44388 L
.49894 .44388 L
.499 .44388 L
.49907 .44388 L
.49912 .44388 L
.49919 .44388 L
.49926 .44388 L
.49933 .44388 L
.4994 .44388 L
.49946 .44388 L
.49952 .44388 L
.49959 .44388 L
.49966 .44388 L
.49974 .44388 L
.49981 .44388 L
.49988 .44388 L
.49995 .44388 L
.50003 .44388 L
.50011 .44388 L
.50019 .44388 L
.50028 .44388 L
.50036 .44388 L
.50043 .44388 L
.50051 .44388 L
.50059 .44389 L
.50067 .44389 L
.50075 .44389 L
.50083 .44389 L
.5009 .44389 L
.50097 .44389 L
.50104 .44389 L
.50111 .44389 L
.50118 .44389 L
.50125 .44389 L
.50134 .44389 L
.50142 .44389 L
.50151 .44389 L
.50158 .44389 L
.50165 .44389 L
.50173 .44389 L
.50181 .44389 L
.50189 .44389 L
.50197 .44389 L
.50205 .44389 L
.50213 .44389 L
.50222 .44389 L
.50231 .44389 L
.5024 .44389 L
.50248 .44389 L
Mistroke
.50255 .44389 L
.50261 .44389 L
.50269 .44389 L
.50276 .44389 L
.50282 .44389 L
.50288 .44389 L
.50295 .44389 L
.50302 .44389 L
.50309 .44389 L
.50316 .44389 L
.50323 .44389 L
.5033 .44389 L
.50337 .44389 L
.50345 .44389 L
.50353 .4439 L
.5036 .4439 L
.50368 .4439 L
.50376 .4439 L
.50383 .4439 L
.5039 .4439 L
.50396 .4439 L
.50403 .4439 L
.5041 .4439 L
.50417 .4439 L
.50425 .4439 L
.50432 .4439 L
.50439 .4439 L
.50446 .4439 L
.50454 .4439 L
.50462 .4439 L
.50469 .4439 L
.50477 .4439 L
.50484 .4439 L
.50491 .4439 L
.50497 .4439 L
.50503 .4439 L
.5051 .4439 L
.50517 .4439 L
.50524 .4439 L
.50531 .4439 L
.50538 .4439 L
.50544 .4439 L
.50551 .4439 L
.50558 .4439 L
.50566 .4439 L
.50573 .4439 L
.5058 .4439 L
.50587 .4439 L
.50593 .4439 L
.50599 .4439 L
Mistroke
.50606 .4439 L
.50612 .4439 L
.50619 .4439 L
.50626 .44391 L
.50632 .44391 L
.50639 .44391 L
.50645 .44391 L
.50651 .44391 L
.50658 .44391 L
.50664 .44391 L
.5067 .44391 L
.50675 .44391 L
.50681 .44391 L
.50687 .44391 L
.50694 .44391 L
.50701 .44391 L
.50707 .44391 L
.50713 .44391 L
.5072 .44391 L
.50728 .44391 L
.50735 .44391 L
.50741 .44391 L
.50748 .44391 L
.50754 .44391 L
.50761 .44391 L
.50768 .44391 L
.50774 .44391 L
.50779 .44391 L
.50786 .44391 L
.50792 .44391 L
.508 .44391 L
.50807 .44391 L
.50815 .44391 L
.50823 .44391 L
.5083 .44391 L
.50837 .44391 L
.50845 .44391 L
.50852 .44391 L
.50859 .44391 L
.50866 .44391 L
.50874 .44391 L
.50881 .44391 L
.50888 .44392 L
.50894 .44392 L
.509 .44392 L
.50907 .44392 L
.50913 .44392 L
.50919 .44392 L
.50924 .44392 L
.5093 .44392 L
Mistroke
.50936 .44392 L
.50942 .44392 L
.50949 .44392 L
.50955 .44392 L
.50961 .44392 L
.50967 .44392 L
.50973 .44392 L
.50979 .44392 L
.50985 .44392 L
.50992 .44392 L
.50998 .44392 L
.51005 .44392 L
.51011 .44392 L
.51016 .44392 L
.51021 .44392 L
.51027 .44392 L
.51033 .44392 L
.51039 .44392 L
.51045 .44392 L
.51051 .44392 L
.51056 .44392 L
.51061 .44392 L
.51067 .44392 L
.51073 .44392 L
.51083 .44392 L
.51088 .44392 L
.51094 .44392 L
.511 .44392 L
.51106 .44392 L
.51113 .44392 L
.5112 .44392 L
.51126 .44392 L
.51131 .44392 L
.51138 .44393 L
.51144 .44393 L
.5115 .44393 L
.51157 .44393 L
.51163 .44393 L
.51168 .44393 L
.51173 .44393 L
.51179 .44393 L
.51184 .44393 L
.5119 .44393 L
.51196 .44393 L
.51202 .44393 L
.51209 .44393 L
.51215 .44393 L
.51221 .44393 L
.51227 .44393 L
.51233 .44393 L
Mistroke
.51239 .44393 L
.51245 .44393 L
.51252 .44393 L
.51258 .44393 L
.51264 .44393 L
.51271 .44393 L
.51278 .44393 L
.51285 .44393 L
.51291 .44393 L
.51297 .44393 L
.51303 .44393 L
.51309 .44393 L
.51315 .44393 L
.51321 .44393 L
.51327 .44393 L
.51334 .44393 L
.51339 .44393 L
.51345 .44393 L
.5135 .44393 L
.51356 .44393 L
.51363 .44393 L
.51368 .44394 L
.51373 .44394 L
.51378 .44394 L
.51384 .44394 L
.5139 .44394 L
.51395 .44394 L
.514 .44394 L
.51406 .44394 L
.51412 .44394 L
.51417 .44394 L
.51422 .44394 L
.51428 .44394 L
.51433 .44394 L
.51438 .44394 L
.51444 .44394 L
.51449 .44394 L
.51458 .44394 L
.51463 .44394 L
.51468 .44394 L
.51473 .44394 L
.51479 .44394 L
.51485 .44394 L
.51491 .44394 L
.51497 .44394 L
.51502 .44394 L
.51507 .44394 L
.51513 .44394 L
.51519 .44394 L
.51525 .44394 L
Mistroke
.51531 .44394 L
.51537 .44394 L
.51543 .44394 L
.5155 .44394 L
.51557 .44394 L
.51563 .44394 L
.51568 .44394 L
.51573 .44394 L
.51578 .44394 L
.51583 .44394 L
.51589 .44395 L
.51595 .44395 L
.51601 .44395 L
.51606 .44395 L
.51612 .44395 L
.51618 .44395 L
.51623 .44395 L
.51628 .44395 L
.51634 .44395 L
.5164 .44395 L
.51646 .44395 L
.51652 .44395 L
.51657 .44395 L
.51662 .44395 L
.51668 .44395 L
.51673 .44395 L
.51679 .44395 L
.51683 .44395 L
.51689 .44395 L
.51694 .44395 L
.51699 .44395 L
.51704 .44395 L
.51709 .44395 L
.51714 .44395 L
.51723 .44395 L
.51728 .44395 L
.51733 .44395 L
.51738 .44395 L
.51744 .44395 L
.51749 .44395 L
.51754 .44395 L
.5176 .44395 L
.51766 .44395 L
.51772 .44395 L
.51777 .44395 L
.51783 .44395 L
.51788 .44395 L
.51793 .44395 L
.51799 .44396 L
.51804 .44396 L
Mistroke
.5181 .44396 L
.51815 .44396 L
.51819 .44396 L
.51828 .44396 L
.51833 .44396 L
.51838 .44396 L
.51843 .44396 L
.51848 .44396 L
.51853 .44396 L
.51858 .44396 L
.51867 .44396 L
.51875 .44396 L
.51884 .44396 L
.51889 .44396 L
.51893 .44396 L
.51898 .44396 L
.51904 .44396 L
.51908 .44396 L
.51913 .44396 L
.51918 .44396 L
.51923 .44396 L
.51932 .44396 L
.51937 .44396 L
.51942 .44396 L
.51947 .44396 L
.51953 .44396 L
.51958 .44396 L
.51962 .44396 L
.51968 .44396 L
.51973 .44396 L
.51978 .44396 L
.51984 .44396 L
.5199 .44396 L
.51996 .44396 L
.52001 .44397 L
.52007 .44397 L
.52012 .44397 L
.52017 .44397 L
.52023 .44397 L
.52029 .44397 L
.52036 .44397 L
.52042 .44397 L
.52048 .44397 L
.52053 .44397 L
.52059 .44397 L
.52065 .44397 L
.5207 .44397 L
.52075 .44397 L
.52081 .44397 L
.52086 .44397 L
Mistroke
.52096 .44397 L
.52101 .44397 L
.52106 .44397 L
.52112 .44397 L
.52118 .44397 L
.52123 .44397 L
.52128 .44397 L
.52134 .44397 L
.5214 .44397 L
.52145 .44397 L
.52151 .44397 L
.52156 .44397 L
.52162 .44397 L
.52167 .44397 L
.52172 .44397 L
.52181 .44397 L
.52186 .44397 L
.52191 .44398 L
.52197 .44398 L
.52202 .44398 L
.52208 .44398 L
.52213 .44398 L
.52219 .44398 L
.52223 .44398 L
.52229 .44398 L
.52234 .44398 L
.52239 .44398 L
.52244 .44398 L
.52249 .44398 L
.52255 .44398 L
.52264 .44398 L
.52269 .44398 L
.52274 .44398 L
.52279 .44398 L
.52284 .44398 L
.52292 .44398 L
.52297 .44398 L
.52301 .44398 L
.52311 .44398 L
.52316 .44398 L
.52321 .44398 L
.52331 .44398 L
.52336 .44398 L
.52342 .44398 L
.52347 .44398 L
.52353 .44398 L
.52358 .44398 L
.52364 .44398 L
.52369 .44398 L
.52374 .44398 L
Mistroke
.52383 .44399 L
.52387 .44399 L
.52392 .44399 L
.52398 .44399 L
.52404 .44399 L
.52409 .44399 L
.52415 .44399 L
.52421 .44399 L
.52427 .44399 L
.52433 .44399 L
.52438 .44399 L
.52444 .44399 L
.52449 .44399 L
.52455 .44399 L
.52461 .44399 L
.52466 .44399 L
.52471 .44399 L
.52476 .44399 L
.52481 .44399 L
.52487 .44399 L
.52493 .44399 L
.52499 .44399 L
.52505 .44399 L
.52511 .44399 L
.52517 .44399 L
.52524 .44399 L
.5253 .44399 L
.52535 .44399 L
.5254 .44399 L
.52546 .44399 L
.52552 .444 L
.52558 .444 L
.52564 .444 L
.52569 .444 L
.52575 .444 L
.5258 .444 L
.52585 .444 L
.52591 .444 L
.52596 .444 L
.52602 .444 L
.52608 .444 L
.52613 .444 L
.52619 .444 L
.52624 .444 L
.52629 .444 L
.52635 .444 L
.5264 .444 L
.52649 .444 L
.52655 .444 L
.52659 .444 L
Mistroke
.52668 .444 L
.52673 .444 L
.52678 .444 L
.52687 .444 L
.52691 .444 L
.52696 .444 L
.52704 .444 L
.52709 .444 L
.52713 .444 L
.52721 .44401 L
.52729 .44401 L
.52733 .44401 L
.52738 .44401 L
.52742 .44401 L
.52747 .44401 L
.52752 .44401 L
.52756 .44401 L
.52764 .44401 L
.52769 .44401 L
.52773 .44401 L
.52778 .44401 L
.52783 .44401 L
.52792 .44401 L
.52798 .44401 L
.52803 .44401 L
.5281 .44401 L
.52814 .44401 L
.52818 .44401 L
.52827 .44401 L
.52831 .44401 L
.52835 .44401 L
.52842 .44401 L
.5285 .44401 L
.52858 .44401 L
.52866 .44401 L
.5287 .44401 L
.52875 .44401 L
.52879 .44401 L
.52883 .44402 L
.52891 .44402 L
.52899 .44402 L
.52903 .44402 L
.52907 .44402 L
.52916 .44402 L
.5292 .44402 L
.52925 .44402 L
.52934 .44402 L
.52939 .44402 L
.52944 .44402 L
.52948 .44402 L
Mistroke
.52953 .44402 L
.52957 .44402 L
.52961 .44402 L
.52969 .44402 L
.52974 .44402 L
.52979 .44402 L
.52984 .44402 L
.52989 .44402 L
.52994 .44402 L
.52998 .44402 L
.53003 .44402 L
.53007 .44402 L
.53015 .44402 L
.53023 .44402 L
.53027 .44402 L
.53032 .44402 L
.5304 .44403 L
.53044 .44403 L
.53048 .44403 L
.53056 .44403 L
.53061 .44403 L
.53066 .44403 L
.53075 .44403 L
.53079 .44403 L
.53083 .44403 L
.53088 .44403 L
.53093 .44403 L
.53102 .44403 L
.53108 .44403 L
.53112 .44403 L
.53117 .44403 L
.53121 .44403 L
.53129 .44403 L
.53134 .44403 L
.53139 .44403 L
.53147 .44403 L
.53152 .44403 L
.53157 .44403 L
.53162 .44403 L
.53167 .44403 L
.53172 .44403 L
.53176 .44403 L
.53185 .44403 L
.5319 .44403 L
.53195 .44404 L
.532 .44404 L
.53206 .44404 L
.53215 .44404 L
.5322 .44404 L
.53225 .44404 L
Mistroke
.53234 .44404 L
.53239 .44404 L
.53244 .44404 L
.53248 .44404 L
.53253 .44404 L
.53261 .44404 L
.53266 .44404 L
.53271 .44404 L
.53276 .44404 L
.53281 .44404 L
.5329 .44404 L
.53295 .44404 L
.533 .44404 L
.53304 .44404 L
.53308 .44404 L
.53317 .44404 L
.53321 .44404 L
.53326 .44404 L
.5333 .44404 L
.53336 .44404 L
.5334 .44405 L
.53344 .44405 L
.53352 .44405 L
.53356 .44405 L
.53361 .44405 L
.53369 .44405 L
.53374 .44405 L
.53378 .44405 L
.53386 .44405 L
.53391 .44405 L
.53395 .44405 L
.534 .44405 L
.53405 .44405 L
.53409 .44405 L
.53413 .44405 L
.53421 .44405 L
.53425 .44405 L
.5343 .44405 L
.53434 .44405 L
.53438 .44405 L
.53446 .44405 L
.5345 .44405 L
.53454 .44405 L
.53462 .44405 L
.53466 .44405 L
.5347 .44405 L
.53478 .44405 L
.53484 .44406 L
.53492 .44406 L
.53499 .44406 L
Mistroke
.53504 .44406 L
.53508 .44406 L
.53516 .44406 L
.53521 .44406 L
.53526 .44406 L
.53534 .44406 L
.53539 .44406 L
.53544 .44406 L
.53552 .44406 L
.53557 .44406 L
.53562 .44406 L
.53567 .44406 L
.53571 .44406 L
.53579 .44406 L
.53584 .44406 L
.53588 .44406 L
.53595 .44406 L
.53599 .44406 L
.53603 .44406 L
.53612 .44406 L
.53616 .44407 L
.53621 .44407 L
.53629 .44407 L
.53633 .44407 L
.53637 .44407 L
.53647 .44407 L
.53651 .44407 L
.53657 .44407 L
.53661 .44407 L
.53666 .44407 L
.53674 .44407 L
.53679 .44407 L
.53685 .44407 L
.5369 .44407 L
.53695 .44407 L
.53704 .44407 L
.53709 .44407 L
.53714 .44407 L
.53719 .44407 L
.53723 .44407 L
.53732 .44407 L
.53736 .44407 L
.53741 .44407 L
.53746 .44407 L
.53751 .44408 L
.53755 .44408 L
.53759 .44408 L
.53767 .44408 L
.53772 .44408 L
.53776 .44408 L
Mistroke
.53781 .44408 L
.53785 .44408 L
.53793 .44408 L
.538 .44408 L
.53804 .44408 L
.53808 .44408 L
.53816 .44408 L
.5382 .44408 L
.53824 .44408 L
.53832 .44408 L
.53836 .44408 L
.53841 .44408 L
.53845 .44408 L
.53849 .44408 L
.53856 .44408 L
.53864 .44408 L
.53871 .44408 L
.53877 .44408 L
.53885 .44409 L
.53892 .44409 L
.53896 .44409 L
.539 .44409 L
.53908 .44409 L
.53912 .44409 L
.53917 .44409 L
.53921 .44409 L
.53925 .44409 L
.53929 .44409 L
.53933 .44409 L
.53941 .44409 L
.53946 .44409 L
.5395 .44409 L
.53954 .44409 L
.53959 .44409 L
.53964 .44409 L
.53968 .44409 L
.53972 .44409 L
.53976 .44409 L
.53984 .44409 L
.53991 .44409 L
.53995 .44409 L
.53999 .44409 L
.54007 .4441 L
.54011 .4441 L
.54015 .4441 L
.54023 .4441 L
.54027 .4441 L
.54031 .4441 L
.54035 .4441 L
.54039 .4441 L
Mistroke
.54047 .4441 L
.54053 .4441 L
.54061 .4441 L
.54068 .4441 L
.54076 .4441 L
.5408 .4441 L
.54084 .4441 L
.54091 .4441 L
.54099 .4441 L
.54105 .4441 L
.54112 .4441 L
.54118 .4441 L
.54124 .4441 L
.54131 .44411 L
.54138 .44411 L
.54146 .44411 L
.54153 .44411 L
.5416 .44411 L
.54165 .44411 L
.54169 .44411 L
.54176 .44411 L
.54184 .44411 L
.5419 .44411 L
.54197 .44411 L
.54204 .44411 L
.54211 .44411 L
.54216 .44411 L
.54219 .44411 L
.54224 .44411 L
.54228 .44411 L
.54236 .44411 L
.5424 .44411 L
.54244 .44412 L
.54251 .44412 L
.54258 .44412 L
.54266 .44412 L
.5427 .44412 L
.54274 .44412 L
.54281 .44412 L
.54288 .44412 L
.54295 .44412 L
.54303 .44412 L
.5431 .44412 L
.54316 .44412 L
.54323 .44412 L
.5433 .44412 L
.54336 .44412 L
.54343 .44412 L
.5435 .44412 L
.54357 .44412 L
Mistroke
.54363 .44413 L
.54369 .44413 L
.54375 .44413 L
.54381 .44413 L
.54388 .44413 L
.54395 .44413 L
.54403 .44413 L
.5441 .44413 L
.54416 .44413 L
.54423 .44413 L
.54429 .44413 L
.54436 .44413 L
.54443 .44413 L
.54451 .44413 L
.54458 .44413 L
.54464 .44413 L
.54472 .44414 L
.54476 .44414 L
.5448 .44414 L
.54487 .44414 L
.54495 .44414 L
.54502 .44414 L
.54508 .44414 L
.54513 .44414 L
.54519 .44414 L
.54526 .44414 L
.54533 .44414 L
.54539 .44414 L
.54545 .44414 L
.54551 .44414 L
.54557 .44414 L
.54563 .44414 L
.54569 .44414 L
.54575 .44414 L
.5458 .44415 L
.54586 .44415 L
.54592 .44415 L
.54599 .44415 L
.54605 .44415 L
.54613 .44415 L
.5462 .44415 L
.54627 .44415 L
.54633 .44415 L
.5464 .44415 L
.54647 .44415 L
.54653 .44415 L
.5466 .44415 L
.54667 .44415 L
.54673 .44415 L
.54678 .44415 L
Mistroke
.54684 .44415 L
.54691 .44416 L
.54697 .44416 L
.54704 .44416 L
.5471 .44416 L
.54718 .44416 L
.54725 .44416 L
.54731 .44416 L
.54737 .44416 L
.54744 .44416 L
.54751 .44416 L
.54758 .44416 L
.54765 .44416 L
.54772 .44416 L
.54779 .44416 L
.54786 .44416 L
.5479 .44416 L
.54794 .44417 L
.54802 .44417 L
.54809 .44417 L
.54813 .44417 L
.54817 .44417 L
.54825 .44417 L
.54829 .44417 L
.54832 .44417 L
.54837 .44417 L
.54841 .44417 L
.54849 .44417 L
.54855 .44417 L
.54863 .44417 L
.54867 .44417 L
.54871 .44417 L
.54878 .44417 L
.54884 .44417 L
.54891 .44417 L
.54898 .44418 L
.54905 .44418 L
.54913 .44418 L
.54919 .44418 L
.54926 .44418 L
.54933 .44418 L
.5494 .44418 L
.54947 .44418 L
.54954 .44418 L
.54959 .44418 L
.54966 .44418 L
.54972 .44418 L
.54978 .44418 L
.54985 .44418 L
.54992 .44419 L
Mistroke
.54998 .44419 L
.55005 .44419 L
.55012 .44419 L
.5502 .44419 L
.55026 .44419 L
.55033 .44419 L
.5504 .44419 L
.55046 .44419 L
.55052 .44419 L
.55058 .44419 L
.55065 .44419 L
.55072 .44419 L
.55078 .44419 L
.55084 .44419 L
.5509 .4442 L
.55096 .4442 L
.55102 .4442 L
.55109 .4442 L
.55114 .4442 L
.5512 .4442 L
.55126 .4442 L
.55132 .4442 L
.5514 .4442 L
.55147 .4442 L
.5515 .4442 L
.55155 .4442 L
.55162 .4442 L
.55169 .4442 L
.55177 .4442 L
.55183 .4442 L
.5519 .44421 L
.55194 .44421 L
.55199 .44421 L
.55206 .44421 L
.5521 .44421 L
.55214 .44421 L
.55219 .44421 L
.55223 .44421 L
.5523 .44421 L
.55234 .44421 L
.55238 .44421 L
.55245 .44421 L
.55249 .44421 L
.55254 .44421 L
.55261 .44421 L
.55267 .44421 L
.55275 .44421 L
.55282 .44422 L
.55288 .44422 L
.55295 .44422 L
Mistroke
.55301 .44422 L
.55308 .44422 L
.55315 .44422 L
.55323 .44422 L
.5533 .44422 L
.55337 .44422 L
.55344 .44422 L
.55349 .44422 L
.55353 .44422 L
.5536 .44422 L
.55364 .44422 L
.55368 .44422 L
.55374 .44423 L
.55381 .44423 L
.55388 .44423 L
.55395 .44423 L
.55401 .44423 L
.55407 .44423 L
.55414 .44423 L
.5542 .44423 L
.55427 .44423 L
.55434 .44423 L
.5544 .44423 L
.55447 .44423 L
.55455 .44423 L
.55459 .44424 L
.55462 .44424 L
.5547 .44424 L
.55477 .44424 L
.55483 .44424 L
.55489 .44424 L
.55495 .44424 L
.55502 .44424 L
.55508 .44424 L
.55513 .44424 L
.55519 .44424 L
.55525 .44424 L
.55531 .44424 L
.55538 .44424 L
.55545 .44425 L
.5555 .44425 L
.55557 .44425 L
.55564 .44425 L
.55571 .44425 L
.55578 .44425 L
.55583 .44425 L
.55588 .44425 L
.55594 .44425 L
.556 .44425 L
.55606 .44425 L
Mistroke
.55612 .44425 L
.55617 .44425 L
.55623 .44425 L
.55628 .44425 L
.55633 .44426 L
.55639 .44426 L
.55644 .44426 L
.55654 .44426 L
.55659 .44426 L
.55664 .44426 L
.5567 .44426 L
.55676 .44426 L
.55681 .44426 L
.55686 .44426 L
.55693 .44426 L
.55699 .44426 L
.55705 .44426 L
.55711 .44426 L
.55716 .44427 L
.55721 .44427 L
.55727 .44427 L
.55733 .44427 L
.55739 .44427 L
.55745 .44427 L
.5575 .44427 L
.55756 .44427 L
.55761 .44427 L
.55766 .44427 L
.5577 .44427 L
.55775 .44427 L
.55781 .44427 L
.55787 .44427 L
.55792 .44427 L
.55797 .44428 L
.55803 .44428 L
.55808 .44428 L
.55813 .44428 L
.55818 .44428 L
.55824 .44428 L
.5583 .44428 L
.55835 .44428 L
.55841 .44428 L
.55846 .44428 L
.55851 .44428 L
.55861 .44428 L
.55866 .44428 L
.55871 .44428 L
.55876 .44428 L
.55882 .44429 L
.55886 .44429 L
Mistroke
.55891 .44429 L
.559 .44429 L
.55905 .44429 L
.5591 .44429 L
.55915 .44429 L
.55919 .44429 L
.55925 .44429 L
.55931 .44429 L
.55937 .44429 L
.55942 .44429 L
.55948 .44429 L
.55952 .44429 L
.55958 .4443 L
.55963 .4443 L
.55969 .4443 L
.55976 .4443 L
.55982 .4443 L
.55988 .4443 L
.55994 .4443 L
.55999 .4443 L
.56005 .4443 L
.5601 .4443 L
.56015 .4443 L
.56021 .4443 L
.56026 .4443 L
.56031 .4443 L
.56041 .44431 L
.56046 .44431 L
.56051 .44431 L
.56056 .44431 L
.56061 .44431 L
.56067 .44431 L
.56073 .44431 L
.56078 .44431 L
.56083 .44431 L
.56088 .44431 L
.56094 .44431 L
.561 .44431 L
.56106 .44431 L
.56112 .44432 L
.56117 .44432 L
.56123 .44432 L
.5613 .44432 L
.56136 .44432 L
.56142 .44432 L
.56148 .44432 L
.56154 .44432 L
.5616 .44432 L
.56167 .44432 L
.56174 .44432 L
Mistroke
.5618 .44432 L
.56186 .44433 L
.56192 .44433 L
.56198 .44433 L
.56204 .44433 L
.56209 .44433 L
.56215 .44433 L
.56221 .44433 L
.56227 .44433 L
.56233 .44433 L
.56239 .44433 L
.56244 .44433 L
.56249 .44433 L
.56255 .44433 L
.56261 .44434 L
.56266 .44434 L
.56271 .44434 L
.56276 .44434 L
.56282 .44434 L
.56287 .44434 L
.56292 .44434 L
.56297 .44434 L
.56303 .44434 L
.56309 .44434 L
.56315 .44434 L
.5632 .44434 L
.56326 .44434 L
.56331 .44434 L
.56336 .44435 L
.56342 .44435 L
.56347 .44435 L
.56356 .44435 L
.56361 .44435 L
.56366 .44435 L
.56372 .44435 L
.56378 .44435 L
.56383 .44435 L
.56388 .44435 L
.56394 .44435 L
.56401 .44435 L
.56407 .44436 L
.56412 .44436 L
.56418 .44436 L
.56423 .44436 L
.56428 .44436 L
.56434 .44436 L
.5644 .44436 L
.56445 .44436 L
.56451 .44436 L
.56456 .44436 L
Mistroke
.56461 .44436 L
.56466 .44436 L
.56471 .44436 L
.56476 .44437 L
.56481 .44437 L
.56487 .44437 L
.56492 .44437 L
.56497 .44437 L
.56503 .44437 L
.56508 .44437 L
.56513 .44437 L
.56518 .44437 L
.56523 .44437 L
.56529 .44437 L
.56534 .44437 L
.5654 .44437 L
.56545 .44438 L
.5655 .44438 L
.56559 .44438 L
.56564 .44438 L
.56569 .44438 L
.56574 .44438 L
.5658 .44438 L
.56584 .44438 L
.56589 .44438 L
.56597 .44438 L
.56602 .44438 L
.56607 .44438 L
.56617 .44439 L
.56622 .44439 L
.56628 .44439 L
.56634 .44439 L
.56639 .44439 L
.56644 .44439 L
.56649 .44439 L
.56655 .44439 L
.5666 .44439 L
.56666 .44439 L
.56672 .44439 L
.56677 .44439 L
.56682 .4444 L
.56688 .4444 L
.56695 .4444 L
.56701 .4444 L
.56706 .4444 L
.56711 .4444 L
.56717 .4444 L
.56721 .4444 L
.56726 .4444 L
.56732 .4444 L
Mistroke
.56738 .4444 L
.56743 .4444 L
.56748 .44441 L
.56753 .44441 L
.56758 .44441 L
.56764 .44441 L
.56769 .44441 L
.56778 .44441 L
.56783 .44441 L
.56788 .44441 L
.56794 .44441 L
.568 .44441 L
.56806 .44441 L
.56812 .44442 L
.56818 .44442 L
.56824 .44442 L
.56829 .44442 L
.56835 .44442 L
.5684 .44442 L
.56845 .44442 L
.56851 .44442 L
.56857 .44442 L
.56862 .44442 L
.56867 .44442 L
.56872 .44442 L
.56877 .44443 L
.56883 .44443 L
.56889 .44443 L
.56895 .44443 L
.56901 .44443 L
.56906 .44443 L
.56911 .44443 L
.56917 .44443 L
.56923 .44443 L
.56928 .44443 L
.56935 .44443 L
.56941 .44444 L
.56946 .44444 L
.56952 .44444 L
.56959 .44444 L
.56966 .44444 L
.56972 .44444 L
.56977 .44444 L
.56983 .44444 L
.56988 .44444 L
.56993 .44444 L
.56999 .44445 L
.57005 .44445 L
.57011 .44445 L
.57017 .44445 L
Mistroke
.57022 .44445 L
.57027 .44445 L
.57032 .44445 L
.57038 .44445 L
.57043 .44445 L
.57049 .44445 L
.57054 .44445 L
.57059 .44445 L
.57069 .44446 L
.57074 .44446 L
.57079 .44446 L
.57085 .44446 L
.5709 .44446 L
.57095 .44446 L
.57101 .44446 L
.57105 .44446 L
.5711 .44446 L
.57119 .44446 L
.57124 .44447 L
.57129 .44447 L
.57139 .44447 L
.57144 .44447 L
.57149 .44447 L
.57154 .44447 L
.5716 .44447 L
.57166 .44447 L
.57172 .44447 L
.57177 .44447 L
.57183 .44448 L
.57188 .44448 L
.57193 .44448 L
.57199 .44448 L
.57204 .44448 L
.57214 .44448 L
.57219 .44448 L
.57224 .44448 L
.57229 .44448 L
.57235 .44448 L
.5724 .44449 L
.57245 .44449 L
.57251 .44449 L
.57257 .44449 L
.57263 .44449 L
.57268 .44449 L
.57274 .44449 L
.57279 .44449 L
.57285 .44449 L
.57291 .44449 L
.57297 .4445 L
.57303 .4445 L
Mistroke
.57309 .4445 L
.57315 .4445 L
.5732 .4445 L
.57325 .4445 L
.5733 .4445 L
.57336 .4445 L
.57341 .4445 L
.57346 .4445 L
.57351 .4445 L
.57356 .44451 L
.57361 .44451 L
.57366 .44451 L
.57371 .44451 L
.57376 .44451 L
.5738 .44451 L
.57385 .44451 L
.57393 .44451 L
.57399 .44451 L
.57404 .44451 L
.57409 .44451 L
.57414 .44452 L
.5742 .44452 L
.57426 .44452 L
.57431 .44452 L
.57436 .44452 L
.57441 .44452 L
.57446 .44452 L
.57452 .44452 L
.57457 .44452 L
.57466 .44452 L
.57471 .44453 L
.57476 .44453 L
.57486 .44453 L
.57491 .44453 L
.57496 .44453 L
.57504 .44453 L
.57509 .44453 L
.57514 .44453 L
.57522 .44454 L
.57527 .44454 L
.57531 .44454 L
.57539 .44454 L
.57543 .44454 L
.57548 .44454 L
.57553 .44454 L
.57558 .44454 L
.57567 .44454 L
.57572 .44454 L
.57577 .44455 L
.57582 .44455 L
Mistroke
.57587 .44455 L
.57591 .44455 L
.57596 .44455 L
.57605 .44455 L
.5761 .44455 L
.57615 .44455 L
.57624 .44455 L
.57629 .44455 L
.57634 .44456 L
.5764 .44456 L
.57646 .44456 L
.57651 .44456 L
.57656 .44456 L
.57661 .44456 L
.57667 .44456 L
.57672 .44456 L
.57678 .44456 L
.57683 .44456 L
.57688 .44457 L
.57697 .44457 L
.57701 .44457 L
.57706 .44457 L
.57712 .44457 L
.57717 .44457 L
.57722 .44457 L
.57728 .44457 L
.57734 .44457 L
.5774 .44458 L
.57746 .44458 L
.57751 .44458 L
.57757 .44458 L
.57762 .44458 L
.57767 .44458 L
.57773 .44458 L
.57778 .44458 L
.57783 .44458 L
.57788 .44459 L
.57793 .44459 L
.57798 .44459 L
.57804 .44459 L
.57808 .44459 L
.57813 .44459 L
.57819 .44459 L
.57825 .44459 L
.5783 .44459 L
.57835 .44459 L
.57844 .4446 L
.57849 .4446 L
.57854 .4446 L
.57859 .4446 L
Mistroke
.57864 .4446 L
.57869 .4446 L
.57874 .4446 L
.57883 .4446 L
.57888 .4446 L
.57893 .44461 L
.57897 .44461 L
.57901 .44461 L
.5791 .44461 L
.57914 .44461 L
.57919 .44461 L
.57925 .44461 L
.5793 .44461 L
.57938 .44461 L
.57943 .44462 L
.57948 .44462 L
.57953 .44462 L
.57958 .44462 L
.57964 .44462 L
.5797 .44462 L
.57976 .44462 L
.57982 .44462 L
.57987 .44462 L
.57992 .44463 L
.57998 .44463 L
.58003 .44463 L
.58008 .44463 L
.58014 .44463 L
.58023 .44463 L
.58028 .44463 L
.58033 .44463 L
.58039 .44464 L
.58044 .44464 L
.5805 .44464 L
.58055 .44464 L
.5806 .44464 L
.58067 .44464 L
.58073 .44464 L
.58078 .44464 L
.58084 .44464 L
.58089 .44465 L
.58095 .44465 L
.58101 .44465 L
.58107 .44465 L
.58113 .44465 L
.58119 .44465 L
.58125 .44465 L
.5813 .44465 L
.58136 .44466 L
.58142 .44466 L
Mistroke
.58147 .44466 L
.58156 .44466 L
.58161 .44466 L
.58166 .44466 L
.58172 .44466 L
.58177 .44466 L
.58183 .44467 L
.58189 .44467 L
.58194 .44467 L
.58199 .44467 L
.58204 .44467 L
.5821 .44467 L
.58219 .44467 L
.58225 .44467 L
.58229 .44468 L
.58238 .44468 L
.58243 .44468 L
.58248 .44468 L
.58253 .44468 L
.58258 .44468 L
.58263 .44468 L
.58269 .44468 L
.58278 .44469 L
.58284 .44469 L
.58289 .44469 L
.58298 .44469 L
.58304 .44469 L
.58309 .44469 L
.58318 .44469 L
.58323 .4447 L
.58327 .4447 L
.58336 .4447 L
.58341 .4447 L
.58346 .4447 L
.58354 .4447 L
.58358 .4447 L
.58363 .4447 L
.58368 .44471 L
.58373 .44471 L
.58383 .44471 L
.58388 .44471 L
.58393 .44471 L
.58398 .44471 L
.58404 .44471 L
.58408 .44471 L
.58413 .44472 L
.58422 .44472 L
.58427 .44472 L
.58432 .44472 L
.58437 .44472 L
Mistroke
.58442 .44472 L
.58446 .44472 L
.58451 .44472 L
.58459 .44473 L
.58463 .44473 L
.58468 .44473 L
.58472 .44473 L
.58477 .44473 L
.58481 .44473 L
.58485 .44473 L
.58492 .44473 L
.58497 .44473 L
.58501 .44474 L
.58509 .44474 L
.58514 .44474 L
.58518 .44474 L
.58523 .44474 L
.58528 .44474 L
.58537 .44474 L
.58542 .44475 L
.58548 .44475 L
.58552 .44475 L
.58556 .44475 L
.58564 .44475 L
.58569 .44475 L
.58574 .44475 L
.58583 .44475 L
.58587 .44476 L
.58592 .44476 L
.58601 .44476 L
.58607 .44476 L
.58612 .44476 L
.58621 .44476 L
.58626 .44477 L
.58631 .44477 L
.58636 .44477 L
.58641 .44477 L
.58646 .44477 L
.58652 .44477 L
.58657 .44477 L
.58662 .44477 L
.58666 .44477 L
.58671 .44478 L
.58679 .44478 L
.58684 .44478 L
.58688 .44478 L
.58697 .44478 L
.58702 .44478 L
.58706 .44478 L
.58715 .44479 L
Mistroke
.5872 .44479 L
.58724 .44479 L
.58729 .44479 L
.58734 .44479 L
.58744 .44479 L
.58749 .44479 L
.58754 .4448 L
.5876 .4448 L
.58765 .4448 L
.5877 .4448 L
.58775 .4448 L
.58784 .4448 L
.58789 .4448 L
.58794 .44481 L
.58799 .44481 L
.58803 .44481 L
.58812 .44481 L
.58818 .44481 L
.58823 .44481 L
.58828 .44481 L
.58835 .44482 L
.5884 .44482 L
.58846 .44482 L
.5885 .44482 L
.58856 .44482 L
.58861 .44482 L
.58866 .44482 L
.58871 .44482 L
.58876 .44483 L
.58886 .44483 L
.58891 .44483 L
.58895 .44483 L
.589 .44483 L
.58905 .44483 L
.58914 .44484 L
.58919 .44484 L
.58924 .44484 L
.58934 .44484 L
.58939 .44484 L
.58944 .44484 L
.58948 .44484 L
.58953 .44485 L
.58959 .44485 L
.58964 .44485 L
.5897 .44485 L
.58974 .44485 L
.58979 .44485 L
.58983 .44485 L
.58991 .44486 L
.58996 .44486 L
Mistroke
.59001 .44486 L
.5901 .44486 L
.59014 .44486 L
.59019 .44486 L
.59028 .44486 L
.59032 .44487 L
.59037 .44487 L
.59042 .44487 L
.59047 .44487 L
.59056 .44487 L
.59061 .44487 L
.59067 .44487 L
.59072 .44488 L
.59078 .44488 L
.59083 .44488 L
.59088 .44488 L
.59097 .44488 L
.59102 .44488 L
.59107 .44489 L
.59112 .44489 L
.59116 .44489 L
.59125 .44489 L
.59131 .44489 L
.59136 .44489 L
.59141 .44489 L
.59148 .4449 L
.59153 .4449 L
.59159 .4449 L
.59164 .4449 L
.59169 .4449 L
.59174 .4449 L
.59179 .4449 L
.59184 .44491 L
.5919 .44491 L
.59199 .44491 L
.59204 .44491 L
.59209 .44491 L
.59217 .44491 L
.59222 .44492 L
.59227 .44492 L
.59236 .44492 L
.59241 .44492 L
.59246 .44492 L
.59254 .44492 L
.59258 .44493 L
.59262 .44493 L
.59271 .44493 L
.59276 .44493 L
.5928 .44493 L
.59285 .44493 L
Mistroke
.5929 .44493 L
.59299 .44494 L
.59304 .44494 L
.59309 .44494 L
.59314 .44494 L
.5932 .44494 L
.59324 .44494 L
.5933 .44495 L
.59334 .44495 L
.59339 .44495 L
.59343 .44495 L
.59348 .44495 L
.59357 .44495 L
.59361 .44495 L
.59365 .44496 L
.5937 .44496 L
.59375 .44496 L
.59379 .44496 L
.59383 .44496 L
.59391 .44496 L
.59396 .44496 L
.594 .44497 L
.59405 .44497 L
.5941 .44497 L
.59419 .44497 L
.59424 .44497 L
.59428 .44497 L
.59432 .44497 L
.59437 .44498 L
.59441 .44498 L
.59445 .44498 L
.59449 .44498 L
.59453 .44498 L
.5946 .44498 L
.59465 .44498 L
.59469 .44498 L
.59473 .44499 L
.59478 .44499 L
.59486 .44499 L
.59491 .44499 L
.59495 .44499 L
.59499 .44499 L
.59504 .44499 L
.59512 .445 L
.59517 .445 L
.59521 .445 L
.59526 .445 L
.59531 .445 L
.59535 .445 L
.5954 .44501 L
Mistroke
.59548 .44501 L
.59552 .44501 L
.59557 .44501 L
.59565 .44501 L
.5957 .44501 L
.59574 .44502 L
.59579 .44502 L
.59584 .44502 L
.59589 .44502 L
.59593 .44502 L
.59602 .44502 L
.59607 .44503 L
.59612 .44503 L
.59621 .44503 L
.59627 .44503 L
.59631 .44503 L
.59637 .44503 L
.59642 .44504 L
.59647 .44504 L
.59652 .44504 L
.59661 .44504 L
.59666 .44504 L
.5967 .44504 L
.59675 .44505 L
.5968 .44505 L
.59689 .44505 L
.59693 .44505 L
.59698 .44505 L
.59703 .44505 L
.59707 .44506 L
.59716 .44506 L
.59721 .44506 L
.59726 .44506 L
.5973 .44506 L
.59736 .44506 L
.59741 .44507 L
.59745 .44507 L
.59754 .44507 L
.59759 .44507 L
.59764 .44507 L
.59769 .44507 L
.59775 .44508 L
.5978 .44508 L
.59786 .44508 L
.59791 .44508 L
.59796 .44508 L
.59804 .44509 L
.59809 .44509 L
.59813 .44509 L
.59821 .44509 L
Mistroke
.59825 .44509 L
.59829 .44509 L
.59838 .4451 L
.59843 .4451 L
.59847 .4451 L
.59852 .4451 L
.59856 .4451 L
.59865 .4451 L
.59869 .44511 L
.59873 .44511 L
.59879 .44511 L
.59883 .44511 L
.59892 .44511 L
.59898 .44511 L
.59902 .44512 L
.59907 .44512 L
.59911 .44512 L
.59919 .44512 L
.59924 .44512 L
.59929 .44512 L
.59933 .44513 L
.59937 .44513 L
.59945 .44513 L
.59949 .44513 L
.59953 .44513 L
.59957 .44513 L
.59962 .44513 L
.59965 .44514 L
.5997 .44514 L
.59977 .44514 L
.59984 .44514 L
.59991 .44514 L
.59995 .44515 L
.59999 .44515 L
.60007 .44515 L
.60012 .44515 L
.60016 .44515 L
.6002 .44515 L
.60024 .44516 L
.60031 .44516 L
.60036 .44516 L
.60041 .44516 L
.60045 .44516 L
.60049 .44516 L
.60053 .44516 L
.60058 .44517 L
.60066 .44517 L
.60071 .44517 L
.60076 .44517 L
.60085 .44518 L
Mistroke
.60091 .44518 L
.60096 .44518 L
.60101 .44518 L
.60106 .44518 L
.60116 .44519 L
.6012 .44519 L
.60126 .44519 L
.6013 .44519 L
.60135 .44519 L
.60144 .44519 L
.60149 .4452 L
.60153 .4452 L
.60158 .4452 L
.60163 .4452 L
.60172 .4452 L
.60176 .44521 L
.60181 .44521 L
.60185 .44521 L
.60189 .44521 L
.60197 .44521 L
.60201 .44521 L
.60205 .44522 L
.60214 .44522 L
.6022 .44522 L
.60224 .44522 L
.6023 .44522 L
.60235 .44523 L
.60244 .44523 L
.6025 .44523 L
.60254 .44523 L
.60259 .44523 L
.60264 .44524 L
.60272 .44524 L
.60277 .44524 L
.60281 .44524 L
.60286 .44524 L
.60291 .44524 L
.60296 .44525 L
.603 .44525 L
.60309 .44525 L
.60314 .44525 L
.60319 .44525 L
.60325 .44526 L
.6033 .44526 L
.60335 .44526 L
.6034 .44526 L
.6035 .44527 L
.60355 .44527 L
.6036 .44527 L
.60365 .44527 L
Mistroke
.60371 .44527 L
.60375 .44527 L
.6038 .44528 L
.60384 .44528 L
.60388 .44528 L
.60393 .44528 L
.60397 .44528 L
.60404 .44528 L
.60409 .44529 L
.60413 .44529 L
.60418 .44529 L
.60423 .44529 L
.60431 .44529 L
.60436 .4453 L
.6044 .4453 L
.60444 .4453 L
.60448 .4453 L
.60452 .4453 L
.60456 .4453 L
.60464 .44531 L
.60471 .44531 L
.60475 .44531 L
.60479 .44531 L
.60486 .44531 L
.60491 .44532 L
.60495 .44532 L
.60504 .44532 L
.60508 .44532 L
.60513 .44532 L
.60521 .44533 L
.60526 .44533 L
.6053 .44533 L
.60535 .44533 L
.60539 .44533 L
.60547 .44534 L
.60551 .44534 L
.60555 .44534 L
.6056 .44534 L
.60564 .44534 L
.60572 .44535 L
.60577 .44535 L
.60581 .44535 L
.60586 .44535 L
.60591 .44535 L
.60596 .44535 L
.606 .44536 L
.60608 .44536 L
.60613 .44536 L
.60617 .44536 L
.60622 .44536 L
Mistroke
.60627 .44537 L
.60637 .44537 L
.60642 .44537 L
.60647 .44537 L
.60655 .44538 L
.60662 .44538 L
.60666 .44538 L
.6067 .44538 L
.60677 .44539 L
.60685 .44539 L
.60692 .44539 L
.60699 .44539 L
.60706 .4454 L
.6071 .4454 L
.60714 .4454 L
.60722 .4454 L
.60726 .4454 L
.6073 .44541 L
.60735 .44541 L
.60739 .44541 L
.60743 .44541 L
.60747 .44541 L
.60754 .44542 L
.60758 .44542 L
.60762 .44542 L
.6077 .44542 L
.60774 .44542 L
.60778 .44542 L
.60785 .44543 L
.60789 .44543 L
.60793 .44543 L
.60798 .44543 L
.60802 .44543 L
.60809 .44544 L
.60816 .44544 L
.60823 .44544 L
.60827 .44544 L
.60831 .44545 L
.60835 .44545 L
.60839 .44545 L
.60847 .44545 L
.60851 .44545 L
.60856 .44545 L
.60864 .44546 L
.60868 .44546 L
.60873 .44546 L
.6088 .44546 L
.60885 .44547 L
.6089 .44547 L
.60894 .44547 L
Mistroke
.60898 .44547 L
.60905 .44547 L
.60912 .44548 L
.60919 .44548 L
.60926 .44548 L
.60933 .44549 L
.60938 .44549 L
.60941 .44549 L
.60948 .44549 L
.60955 .44549 L
.60962 .4455 L
.60968 .4455 L
.60975 .4455 L
.6098 .44551 L
.60987 .44551 L
.60994 .44551 L
.61001 .44551 L
.61008 .44552 L
.61015 .44552 L
.61019 .44552 L
.61023 .44552 L
.6103 .44553 L
.61037 .44553 L
.61044 .44553 L
.6105 .44553 L
.61057 .44554 L
.61064 .44554 L
.61068 .44554 L
.61072 .44554 L
.61076 .44554 L
.6108 .44555 L
.61088 .44555 L
.61092 .44555 L
.61095 .44555 L
.61103 .44556 L
.61109 .44556 L
.61116 .44556 L
.61124 .44556 L
.61128 .44557 L
.61132 .44557 L
.61136 .44557 L
.6114 .44557 L
.61148 .44557 L
.61152 .44558 L
.61155 .44558 L
.61163 .44558 L
.61169 .44558 L
.61177 .44559 L
.61184 .44559 L
.61192 .44559 L
Mistroke
.61196 .4456 L
.612 .4456 L
.61207 .4456 L
.61215 .4456 L
.61219 .44561 L
.61223 .44561 L
.6123 .44561 L
.61235 .44561 L
.61239 .44561 L
.61243 .44562 L
.61248 .44562 L
.61255 .44562 L
.61262 .44562 L
.61269 .44563 L
.61276 .44563 L
.61283 .44563 L
.61289 .44564 L
.61296 .44564 L
.61303 .44564 L
.6131 .44564 L
.61318 .44565 L
.61322 .44565 L
.61326 .44565 L
.61333 .44566 L
.61337 .44566 L
.61341 .44566 L
.61348 .44566 L
.61355 .44566 L
.61361 .44567 L
.61369 .44567 L
.61376 .44567 L
.6138 .44568 L
.61384 .44568 L
.61392 .44568 L
.61399 .44568 L
.61407 .44569 L
.61413 .44569 L
.6142 .44569 L
.61424 .4457 L
.61429 .4457 L
.61436 .4457 L
.61441 .4457 L
.61445 .44571 L
.61449 .44571 L
.61454 .44571 L
.61457 .44571 L
.61461 .44571 L
.61469 .44572 L
.61473 .44572 L
.61477 .44572 L
Mistroke
.61485 .44572 L
.61489 .44573 L
.61493 .44573 L
.615 .44573 L
.61504 .44573 L
.61508 .44573 L
.61512 .44574 L
.61517 .44574 L
.61524 .44574 L
.61531 .44575 L
.61538 .44575 L
.61542 .44575 L
.61546 .44575 L
.6155 .44575 L
.61554 .44576 L
.61562 .44576 L
.61566 .44576 L
.61571 .44576 L
.61579 .44577 L
.61583 .44577 L
.61588 .44577 L
.61596 .44578 L
.616 .44578 L
.61604 .44578 L
.61609 .44578 L
.61613 .44578 L
.6162 .44579 L
.61627 .44579 L
.61633 .44579 L
.61639 .4458 L
.61645 .4458 L
.61652 .4458 L
.61659 .44581 L
.61665 .44581 L
.61671 .44581 L
.61677 .44581 L
.61683 .44582 L
.61688 .44582 L
.61695 .44582 L
.61701 .44583 L
.61707 .44583 L
.61713 .44583 L
.6172 .44583 L
.61727 .44584 L
.61733 .44584 L
.6174 .44584 L
.61747 .44585 L
.61751 .44585 L
.61755 .44585 L
.61762 .44586 L
Mistroke
.61769 .44586 L
.61775 .44586 L
.61782 .44586 L
.61787 .44587 L
.61794 .44587 L
.618 .44587 L
.61808 .44588 L
.61814 .44588 L
.61821 .44588 L
.61827 .44589 L
.61833 .44589 L
.6184 .44589 L
.61847 .4459 L
.61853 .4459 L
.61858 .4459 L
.61864 .44591 L
.61871 .44591 L
.61877 .44591 L
.61884 .44592 L
.61891 .44592 L
.61897 .44592 L
.61904 .44593 L
.61911 .44593 L
.61918 .44593 L
.61925 .44594 L
.61931 .44594 L
.61938 .44594 L
.61944 .44595 L
.6195 .44595 L
.61957 .44595 L
.61964 .44596 L
.61971 .44596 L
.61978 .44596 L
.61985 .44597 L
.61992 .44597 L
.61998 .44597 L
.62005 .44598 L
.62012 .44598 L
.62016 .44598 L
.6202 .44599 L
.62027 .44599 L
.62033 .44599 L
.62039 .446 L
.62046 .446 L
.62051 .446 L
.62057 .446 L
.62063 .44601 L
.6207 .44601 L
.62076 .44602 L
.62082 .44602 L
Mistroke
.62087 .44602 L
.62093 .44602 L
.62098 .44603 L
.62104 .44603 L
.6211 .44603 L
.62116 .44604 L
.62122 .44604 L
.62128 .44604 L
.62133 .44605 L
.62138 .44605 L
.62143 .44605 L
.62149 .44605 L
.62155 .44606 L
.62161 .44606 L
.62166 .44606 L
.62172 .44607 L
.62177 .44607 L
.62182 .44607 L
.62188 .44607 L
.62193 .44608 L
.62203 .44608 L
.62208 .44609 L
.62213 .44609 L
.62219 .44609 L
.62224 .44609 L
.6223 .4461 L
.62236 .4461 L
.62241 .4461 L
.62246 .44611 L
.62251 .44611 L
.62257 .44611 L
.62263 .44612 L
.6227 .44612 L
.62276 .44612 L
.62282 .44613 L
.62287 .44613 L
.62293 .44613 L
.62299 .44614 L
.62305 .44614 L
.62311 .44614 L
.62318 .44615 L
.62324 .44615 L
.62329 .44615 L
.62335 .44616 L
.6234 .44616 L
.62346 .44616 L
.62352 .44617 L
.62358 .44617 L
.62364 .44617 L
.6237 .44618 L
Mistroke
.62376 .44618 L
.62382 .44618 L
.62389 .44619 L
.62396 .44619 L
.62401 .44619 L
.62408 .4462 L
.62414 .4462 L
.6242 .4462 L
.62426 .44621 L
.62432 .44621 L
.62439 .44622 L
.62446 .44622 L
.62452 .44622 L
.62459 .44623 L
.62464 .44623 L
.62471 .44623 L
.62478 .44624 L
.62485 .44624 L
.62492 .44625 L
.62499 .44625 L
.62505 .44625 L
.62512 .44626 L
.62518 .44626 L
.62523 .44626 L
.62529 .44627 L
.62536 .44627 L
.62542 .44628 L
.62548 .44628 L
.62555 .44628 L
.6256 .44629 L
.62566 .44629 L
.62572 .44629 L
.62578 .4463 L
.62584 .4463 L
.62589 .4463 L
.62594 .44631 L
.626 .44631 L
.62607 .44631 L
.62614 .44632 L
.62621 .44632 L
.62628 .44633 L
.62635 .44633 L
.62639 .44633 L
.62643 .44633 L
.6265 .44634 L
.62657 .44634 L
.62664 .44635 L
.62671 .44635 L
.62676 .44636 L
.62683 .44636 L
Mistroke
.6269 .44636 L
.62697 .44637 L
.62704 .44637 L
.62711 .44638 L
.62717 .44638 L
.62723 .44638 L
.62729 .44639 L
.62734 .44639 L
.6274 .44639 L
.62747 .4464 L
.62753 .4464 L
.62759 .44641 L
.62765 .44641 L
.62771 .44641 L
.62777 .44642 L
.62784 .44642 L
.62791 .44643 L
.62797 .44643 L
.62803 .44643 L
.6281 .44644 L
.62816 .44644 L
.62821 .44644 L
.62827 .44645 L
.62833 .44645 L
.62838 .44646 L
.62843 .44646 L
.62849 .44646 L
.62855 .44647 L
.62861 .44647 L
.62868 .44647 L
.62874 .44648 L
.6288 .44648 L
.62886 .44649 L
.62892 .44649 L
.62898 .44649 L
.62904 .4465 L
.6291 .4465 L
.62917 .44651 L
.62923 .44651 L
.62929 .44651 L
.62935 .44652 L
.6294 .44652 L
.62946 .44652 L
.62952 .44653 L
.62958 .44653 L
.62965 .44654 L
.6297 .44654 L
.62976 .44654 L
.62983 .44655 L
.6299 .44655 L
Mistroke
.62997 .44656 L
.63003 .44656 L
.63008 .44656 L
.63014 .44657 L
.6302 .44657 L
.63025 .44658 L
.63031 .44658 L
.63035 .44658 L
.63041 .44659 L
.63046 .44659 L
.63052 .44659 L
.63058 .4466 L
.63063 .4466 L
.63069 .4466 L
.63075 .44661 L
.63081 .44661 L
.63087 .44662 L
.63093 .44662 L
.63098 .44662 L
.63103 .44663 L
.63108 .44663 L
.63113 .44663 L
.63119 .44664 L
.63124 .44664 L
.63129 .44665 L
.63134 .44665 L
.6314 .44665 L
.63145 .44666 L
.6315 .44666 L
.63155 .44666 L
.63161 .44667 L
.63167 .44667 L
.63172 .44667 L
.63178 .44668 L
.63184 .44668 L
.6319 .44669 L
.63197 .44669 L
.63204 .4467 L
.63211 .4467 L
.63218 .44671 L
.63225 .44671 L
.63232 .44672 L
.63238 .44672 L
.63244 .44672 L
.63251 .44673 L
.63257 .44673 L
.63263 .44674 L
.63269 .44674 L
.63275 .44675 L
.63281 .44675 L
Mistroke
.63288 .44675 L
.63294 .44676 L
.63301 .44676 L
.63308 .44677 L
.63315 .44677 L
.63321 .44678 L
.63327 .44678 L
.63334 .44679 L
.63341 .44679 L
.63348 .4468 L
.63355 .4468 L
.63361 .44681 L
.63368 .44681 L
.63376 .44682 L
.6338 .44682 L
.63384 .44682 L
.63392 .44683 L
.63398 .44683 L
.63405 .44684 L
.63411 .44684 L
.63417 .44685 L
.63424 .44685 L
.63432 .44686 L
.63439 .44686 L
.63445 .44687 L
.63452 .44687 L
.63458 .44688 L
.63464 .44688 L
.63471 .44689 L
.63478 .44689 L
.63484 .4469 L
.63489 .4469 L
.63495 .4469 L
.63501 .44691 L
.63507 .44691 L
.63513 .44692 L
.63519 .44692 L
.63526 .44693 L
.63532 .44693 L
.63538 .44694 L
.63544 .44694 L
.6355 .44694 L
.63556 .44695 L
.63562 .44695 L
.63568 .44696 L
.63573 .44696 L
.63578 .44697 L
.63583 .44697 L
.63589 .44697 L
.63595 .44698 L
Mistroke
.63601 .44698 L
.63607 .44699 L
.63612 .44699 L
.63617 .44699 L
.63622 .447 L
.63627 .447 L
.63633 .44701 L
.63639 .44701 L
.63644 .44702 L
.63649 .44702 L
.63655 .44702 L
.63661 .44703 L
.63667 .44703 L
.63674 .44704 L
.63679 .44704 L
.63684 .44705 L
.6369 .44705 L
.63694 .44705 L
.637 .44706 L
.63705 .44706 L
.63711 .44707 L
.63717 .44707 L
.63722 .44707 L
.63727 .44708 L
.63732 .44708 L
.63737 .44709 L
.63742 .44709 L
.63752 .4471 L
.63757 .4471 L
.63762 .44711 L
.63768 .44711 L
.63773 .44712 L
.63779 .44712 L
.63786 .44713 L
.63792 .44713 L
.63798 .44713 L
.63803 .44714 L
.63809 .44714 L
.63814 .44715 L
.6382 .44715 L
.63826 .44716 L
.63831 .44716 L
.63836 .44717 L
.63841 .44717 L
.63846 .44717 L
.63852 .44718 L
.63857 .44718 L
.63863 .44719 L
.63869 .44719 L
.63875 .4472 L
Mistroke
.63881 .4472 L
.63886 .44721 L
.63892 .44721 L
.63897 .44721 L
.63903 .44722 L
.6391 .44722 L
.63916 .44723 L
.63921 .44723 L
.63928 .44724 L
.63934 .44724 L
.63941 .44725 L
.63947 .44725 L
.63953 .44726 L
.63959 .44726 L
.63965 .44727 L
.63971 .44727 L
.63976 .44728 L
.63981 .44728 L
.63987 .44729 L
.63992 .44729 L
.63998 .4473 L
.64004 .4473 L
.6401 .44731 L
.64016 .44731 L
.64022 .44732 L
.64029 .44732 L
.64035 .44733 L
.64041 .44733 L
.64046 .44734 L
.64052 .44734 L
.64057 .44735 L
.64062 .44735 L
.64068 .44736 L
.64074 .44736 L
.6408 .44737 L
.64085 .44737 L
.6409 .44737 L
.64095 .44738 L
.641 .44738 L
.64106 .44739 L
.64112 .44739 L
.64117 .4474 L
.64122 .4474 L
.64127 .44741 L
.64133 .44741 L
.6414 .44742 L
.64146 .44742 L
.64151 .44743 L
.64157 .44743 L
.64164 .44744 L
Mistroke
.64171 .44744 L
.64177 .44745 L
.64183 .44745 L
.64189 .44746 L
.64195 .44746 L
.64202 .44747 L
.64209 .44748 L
.64216 .44748 L
.64222 .44749 L
.64229 .44749 L
.64235 .4475 L
.64241 .4475 L
.64246 .44751 L
.64252 .44751 L
.64259 .44752 L
.64265 .44753 L
.64271 .44753 L
.64277 .44754 L
.64283 .44754 L
.64288 .44755 L
.64294 .44755 L
.643 .44756 L
.64305 .44756 L
.6431 .44757 L
.64316 .44757 L
.64322 .44758 L
.64328 .44758 L
.64334 .44759 L
.64341 .44759 L
.64348 .4476 L
.64354 .4476 L
.64359 .44761 L
.64366 .44762 L
.64372 .44762 L
.64378 .44763 L
.64385 .44763 L
.64391 .44764 L
.64398 .44764 L
.64405 .44765 L
.64412 .44766 L
.64419 .44766 L
.64426 .44767 L
.64431 .44768 L
.64438 .44768 L
.64443 .44769 L
.64448 .44769 L
.64455 .4477 L
.64461 .4477 L
.64467 .44771 L
.64473 .44771 L
Mistroke
.64479 .44772 L
.64485 .44772 L
.64491 .44773 L
.64497 .44774 L
.64504 .44774 L
.64511 .44775 L
.64516 .44775 L
.64523 .44776 L
.64529 .44777 L
.64535 .44777 L
.64541 .44778 L
.64547 .44778 L
.64552 .44779 L
.64557 .44779 L
.64563 .4478 L
.64568 .4478 L
.64574 .44781 L
.6458 .44781 L
.64585 .44782 L
.64591 .44782 L
.64597 .44783 L
.64604 .44784 L
.6461 .44784 L
.64616 .44785 L
.64622 .44785 L
.64627 .44786 L
.64632 .44786 L
.64637 .44787 L
.64643 .44787 L
.64649 .44788 L
.64655 .44789 L
.6466 .44789 L
.64665 .4479 L
.6467 .4479 L
.64676 .44791 L
.64681 .44791 L
.64686 .44792 L
.64691 .44792 L
.64696 .44793 L
.64701 .44793 L
.64706 .44794 L
.64712 .44794 L
.64717 .44795 L
.64721 .44795 L
.64726 .44796 L
.64732 .44796 L
.64738 .44797 L
.64743 .44797 L
.64749 .44798 L
.64755 .44798 L
Mistroke
.64759 .44799 L
.64765 .44799 L
.64771 .448 L
.64777 .44801 L
.64782 .44801 L
.64788 .44802 L
.64793 .44802 L
.64798 .44803 L
.64803 .44803 L
.64808 .44804 L
.64813 .44804 L
.64818 .44805 L
.64824 .44805 L
.64829 .44806 L
.64834 .44806 L
.64838 .44807 L
.64843 .44807 L
.64849 .44808 L
.64857 .44809 L
.64862 .44809 L
.64867 .4481 L
.64873 .4481 L
.64878 .44811 L
.64884 .44811 L
.6489 .44812 L
.64896 .44813 L
.64901 .44813 L
.64906 .44814 L
.64912 .44814 L
.64917 .44815 L
.64923 .44815 L
.64928 .44816 L
.64934 .44817 L
.64938 .44817 L
.64943 .44818 L
.64948 .44818 L
.64953 .44819 L
.64959 .44819 L
.64965 .4482 L
.6497 .4482 L
.64976 .44821 L
.64982 .44822 L
.64987 .44822 L
.64994 .44823 L
.65 .44823 L
.65005 .44824 L
.65011 .44825 L
.65016 .44825 L
.65021 .44826 L
.65027 .44826 L
Mistroke
.65033 .44827 L
.65039 .44828 L
.65044 .44828 L
.65049 .44829 L
.65054 .44829 L
.65063 .4483 L
.65068 .44831 L
.65072 .44831 L
.65077 .44832 L
.65082 .44832 L
.65087 .44833 L
.65091 .44833 L
.65099 .44834 L
.65104 .44835 L
.65109 .44835 L
.65118 .44836 L
.65123 .44837 L
.65127 .44837 L
.65133 .44838 L
.65138 .44838 L
.65147 .44839 L
.65152 .4484 L
.65156 .4484 L
.65161 .44841 L
.65166 .44841 L
.65171 .44842 L
.65176 .44842 L
.65182 .44843 L
.65188 .44844 L
.65193 .44844 L
.65198 .44845 L
.65203 .44845 L
.65208 .44846 L
.65212 .44846 L
.65217 .44847 L
.65222 .44847 L
.65228 .44848 L
.65233 .44849 L
.65238 .44849 L
.65244 .4485 L
.65249 .4485 L
.65254 .44851 L
.65259 .44852 L
.65264 .44852 L
.6527 .44853 L
.65275 .44853 L
.65281 .44854 L
.65286 .44855 L
.65291 .44855 L
.653 .44856 L
Mistroke
.65305 .44857 L
.6531 .44857 L
.65315 .44858 L
.6532 .44858 L
.65325 .44859 L
.6533 .4486 L
.65335 .4486 L
.6534 .44861 L
.65344 .44861 L
.65349 .44862 L
.65357 .44863 L
.65362 .44863 L
.65367 .44864 L
.65376 .44865 L
.65381 .44865 L
.65385 .44866 L
.6539 .44866 L
.65396 .44867 L
.654 .44868 L
.65405 .44868 L
.6541 .44869 L
.65415 .44869 L
.65419 .4487 L
.65424 .4487 L
.65432 .44871 L
.65436 .44872 L
.65441 .44872 L
.65449 .44873 L
.65453 .44874 L
.65458 .44874 L
.65462 .44875 L
.65467 .44875 L
.65471 .44876 L
.65475 .44876 L
.65483 .44877 L
.65487 .44878 L
.65492 .44878 L
.655 .44879 L
.65504 .4488 L
.65509 .4488 L
.65518 .44881 L
.65522 .44882 L
.65527 .44883 L
.65532 .44883 L
.65537 .44884 L
.65541 .44884 L
.65546 .44885 L
.65554 .44886 L
.65558 .44886 L
.65563 .44887 L
Mistroke
.65572 .44888 L
.65576 .44888 L
.65581 .44889 L
.65586 .4489 L
.65591 .4489 L
.65596 .44891 L
.656 .44891 L
.65609 .44892 L
.65614 .44893 L
.6562 .44894 L
.65624 .44894 L
.65629 .44895 L
.65634 .44895 L
.65639 .44896 L
.65644 .44897 L
.6565 .44897 L
.65659 .44898 L
.65664 .44899 L
.65668 .449 L
.65676 .44901 L
.65681 .44901 L
.65685 .44902 L
.65693 .44903 L
.65698 .44903 L
.65702 .44904 L
.65709 .44905 L
.65713 .44905 L
.65717 .44906 L
.65722 .44906 L
.65726 .44907 L
.65735 .44908 L
.6574 .44909 L
.65744 .44909 L
.65749 .4491 L
.65754 .4491 L
.65762 .44911 L
.65766 .44912 L
.65771 .44912 L
.65775 .44913 L
.6578 .44914 L
.65784 .44914 L
.65788 .44915 L
.65792 .44915 L
.65796 .44916 L
.65803 .44917 L
.65808 .44917 L
.65812 .44918 L
.65816 .44918 L
.6582 .44919 L
.65827 .4492 L
Mistroke
.65834 .4492 L
.65838 .44921 L
.65842 .44922 L
.65849 .44922 L
.65853 .44923 L
.65857 .44923 L
.65861 .44924 L
.65865 .44925 L
.6587 .44925 L
.65873 .44926 L
.65881 .44927 L
.65885 .44927 L
.65889 .44928 L
.65894 .44928 L
.65898 .44929 L
.65906 .4493 L
.65911 .4493 L
.65915 .44931 L
.65919 .44932 L
.65924 .44932 L
.65932 .44933 L
.65936 .44934 L
.65941 .44934 L
.65945 .44935 L
.6595 .44936 L
.65954 .44936 L
.65959 .44937 L
.65966 .44938 L
.65971 .44938 L
.65975 .44939 L
.6598 .4494 L
.65984 .4494 L
.65992 .44941 L
.65996 .44942 L
.66001 .44942 L
.66008 .44943 L
.66013 .44944 L
.66017 .44944 L
.66024 .44945 L
.66031 .44946 L
.66035 .44947 L
.66039 .44947 L
.66046 .44948 L
.66051 .44949 L
.66056 .4495 L
.6606 .4495 L
.66065 .44951 L
.66069 .44952 L
.66073 .44952 L
.66081 .44953 L
Mistroke
.66086 .44954 L
.66091 .44954 L
.66095 .44955 L
.66101 .44956 L
.66105 .44957 L
.6611 .44957 L
.66114 .44958 L
.66119 .44958 L
.66123 .44959 L
.66126 .44959 L
.66134 .4496 L
.66138 .44961 L
.66142 .44962 L
.66147 .44962 L
.66151 .44963 L
.66159 .44964 L
.66164 .44965 L
.66169 .44965 L
.66176 .44966 L
.66181 .44967 L
.66185 .44968 L
.66192 .44969 L
.66199 .4497 L
.66203 .4497 L
.66207 .44971 L
.66215 .44972 L
.66222 .44973 L
.66229 .44974 L
.66235 .44975 L
.66242 .44976 L
.66246 .44976 L
.6625 .44977 L
.66258 .44978 L
.66262 .44978 L
.66266 .44979 L
.6627 .4498 L
.66275 .4498 L
.66278 .44981 L
.66282 .44981 L
.66289 .44982 L
.66293 .44983 L
.66297 .44983 L
.66305 .44984 L
.66309 .44985 L
.66313 .44986 L
.6632 .44987 L
.66324 .44987 L
.66327 .44988 L
.66332 .44988 L
.66336 .44989 L
Mistroke
.66343 .4499 L
.6635 .44991 L
.66357 .44992 L
.66361 .44993 L
.66365 .44993 L
.66368 .44994 L
.66372 .44994 L
.6638 .44995 L
.66384 .44996 L
.66388 .44997 L
.66396 .44998 L
.664 .44998 L
.66405 .44999 L
.66412 .45 L
.66417 .45001 L
.66421 .45001 L
.66426 .45002 L
.6643 .45003 L
.66438 .45004 L
.66442 .45005 L
.66446 .45005 L
.66454 .45006 L
.66458 .45007 L
.66462 .45007 L
.6647 .45009 L
.66475 .45009 L
.66479 .4501 L
.66488 .45011 L
.66492 .45012 L
.66497 .45013 L
.66501 .45013 L
.66506 .45014 L
.66514 .45015 L
.66519 .45016 L
.66524 .45017 L
.66528 .45017 L
.66533 .45018 L
.66537 .45019 L
.66541 .45019 L
.66549 .45021 L
.66553 .45021 L
.66558 .45022 L
.66562 .45023 L
.66567 .45023 L
.66571 .45024 L
.66575 .45025 L
.66583 .45026 L
.66587 .45026 L
.66592 .45027 L
.666 .45028 L
Mistroke
.66604 .45029 L
.66608 .4503 L
.66612 .4503 L
.66617 .45031 L
.66621 .45032 L
.66624 .45032 L
.66632 .45033 L
.66635 .45034 L
.66639 .45035 L
.66646 .45036 L
.66654 .45037 L
.66658 .45037 L
.66662 .45038 L
.66669 .45039 L
.66675 .4504 L
.66679 .45041 L
.66683 .45041 L
.6669 .45043 L
.66694 .45043 L
.66698 .45044 L
.66703 .45044 L
.66707 .45045 L
.66715 .45046 L
.66719 .45047 L
.66723 .45048 L
.66728 .45048 L
.66732 .45049 L
.66741 .45051 L
.66745 .45051 L
.6675 .45052 L
.66754 .45053 L
.66759 .45054 L
.66764 .45054 L
.66768 .45055 L
.66776 .45056 L
.66781 .45057 L
.66786 .45058 L
.66795 .45059 L
.668 .4506 L
.66804 .45061 L
.6681 .45062 L
.66814 .45062 L
.66823 .45064 L
.66828 .45065 L
.66833 .45065 L
.6684 .45067 L
.66844 .45067 L
.66848 .45068 L
.66852 .45069 L
.66856 .45069 L
Mistroke
.66863 .4507 L
.66867 .45071 L
.66871 .45072 L
.66875 .45072 L
.6688 .45073 L
.66884 .45074 L
.66888 .45075 L
.66896 .45076 L
.669 .45076 L
.66904 .45077 L
.66911 .45078 L
.66916 .45079 L
.6692 .4508 L
.66928 .45081 L
.66932 .45082 L
.66937 .45083 L
.66945 .45084 L
.66949 .45085 L
.66953 .45085 L
.66958 .45086 L
.66962 .45087 L
.6697 .45088 L
.66974 .45089 L
.66978 .4509 L
.66983 .4509 L
.66987 .45091 L
.66991 .45092 L
.66996 .45093 L
.67 .45093 L
.67003 .45094 L
.6701 .45095 L
.67014 .45096 L
.67019 .45096 L
.67027 .45098 L
.67031 .45099 L
.67035 .45099 L
.6704 .451 L
.67044 .45101 L
.67053 .45102 L
.67058 .45103 L
.67063 .45104 L
.67067 .45105 L
.67071 .45105 L
.67079 .45107 L
.67083 .45108 L
.67088 .45108 L
.67092 .45109 L
.67096 .4511 L
.67103 .45111 L
.67107 .45112 L
Mistroke
.67111 .45112 L
.67116 .45113 L
.6712 .45114 L
.67124 .45115 L
.67128 .45115 L
.67135 .45117 L
.67141 .45118 L
.67149 .45119 L
.67153 .4512 L
.67156 .4512 L
.67164 .45122 L
.67168 .45122 L
.67173 .45123 L
.67176 .45124 L
.6718 .45125 L
.67188 .45126 L
.67192 .45127 L
.67197 .45127 L
.67201 .45128 L
.67205 .45129 L
.67209 .4513 L
.67214 .45131 L
.67217 .45131 L
.67221 .45132 L
.67229 .45133 L
.67233 .45134 L
.67237 .45135 L
.67241 .45135 L
.67244 .45136 L
.67251 .45137 L
.67258 .45139 L
.67265 .4514 L
.67272 .45141 L
.67279 .45142 L
.67283 .45143 L
.67286 .45144 L
.6729 .45144 L
.67294 .45145 L
.67302 .45146 L
.67308 .45148 L
.67312 .45148 L
.67317 .45149 L
.67321 .4515 L
.67325 .45151 L
.67329 .45152 L
.67333 .45152 L
.67341 .45154 L
.67347 .45155 L
.67354 .45156 L
.67361 .45157 L
Mistroke
.67367 .45158 L
.67373 .4516 L
.67377 .4516 L
.67381 .45161 L
.67388 .45162 L
.67395 .45164 L
.67401 .45165 L
.67409 .45166 L
.67415 .45168 L
.67422 .45169 L
.67429 .4517 L
.67433 .45171 L
.67437 .45172 L
.67444 .45173 L
.67447 .45174 L
.67451 .45174 L
.67458 .45176 L
.67462 .45176 L
.67466 .45177 L
.67471 .45178 L
.67475 .45179 L
.67478 .45179 L
.67482 .4518 L
.67489 .45182 L
.67496 .45183 L
.67503 .45184 L
.67507 .45185 L
.67511 .45186 L
.67515 .45186 L
.67519 .45187 L
.67523 .45188 L
.67527 .45189 L
.67531 .4519 L
.67535 .4519 L
.67542 .45192 L
.67546 .45193 L
.6755 .45193 L
.67554 .45194 L
.67559 .45195 L
.67567 .45197 L
.67571 .45197 L
.67575 .45198 L
.67582 .452 L
.67586 .452 L
.6759 .45201 L
.67597 .45202 L
.67604 .45204 L
.67611 .45205 L
.67617 .45206 L
.67624 .45208 L
Mistroke
.67631 .45209 L
.67635 .4521 L
.67638 .45211 L
.67642 .45211 L
.67646 .45212 L
.67654 .45214 L
.67658 .45214 L
.67662 .45215 L
.67669 .45217 L
.67675 .45218 L
.67683 .45219 L
.67686 .4522 L
.6769 .45221 L
.67698 .45223 L
.67703 .45224 L
.67708 .45224 L
.67716 .45226 L
.6772 .45227 L
.67724 .45228 L
.67732 .45229 L
.67736 .4523 L
.6774 .45231 L
.67747 .45233 L
.67751 .45233 L
.67755 .45234 L
.67759 .45235 L
.67764 .45236 L
.67772 .45238 L
.67777 .45239 L
.67781 .45239 L
.67786 .4524 L
.67791 .45241 L
.67798 .45243 L
.67802 .45244 L
.67807 .45245 L
.67811 .45246 L
.67816 .45247 L
.6782 .45247 L
.67824 .45248 L
.67827 .45249 L
.67832 .4525 L
.67839 .45251 L
.67843 .45252 L
.67847 .45253 L
.6785 .45254 L
.67854 .45255 L
.67861 .45256 L
.67868 .45257 L
.67872 .45258 L
.67876 .45259 L
Mistroke
.67883 .45261 L
.67886 .45261 L
.67891 .45262 L
.67894 .45263 L
.67899 .45264 L
.67903 .45265 L
.67906 .45266 L
.67914 .45267 L
.67918 .45268 L
.67921 .45269 L
.67926 .4527 L
.6793 .45271 L
.67938 .45272 L
.67943 .45273 L
.67947 .45274 L
.67953 .45276 L
.67959 .45277 L
.67965 .45278 L
.67971 .45279 L
.67977 .45281 L
.67984 .45282 L
.67991 .45284 L
.67996 .45285 L
.68002 .45286 L
.68008 .45287 L
.68014 .45289 L
.6802 .4529 L
.68026 .45291 L
.68031 .45292 L
.68036 .45294 L
.68043 .45295 L
.6805 .45296 L
.68056 .45298 L
.68063 .45299 L
.68067 .453 L
.68071 .45301 L
.68077 .45303 L
.68084 .45304 L
.68091 .45306 L
.68097 .45307 L
.68103 .45308 L
.68109 .4531 L
.68115 .45311 L
.68121 .45312 L
.68128 .45314 L
.68135 .45315 L
.6814 .45317 L
.68147 .45318 L
.68153 .45319 L
.6816 .45321 L
Mistroke
.68166 .45322 L
.68172 .45324 L
.68179 .45325 L
.68182 .45326 L
.68186 .45327 L
.68193 .45329 L
.68199 .4533 L
.68206 .45331 L
.68213 .45333 L
.6822 .45335 L
.68226 .45336 L
.68232 .45338 L
.6824 .45339 L
.68244 .4534 L
.68248 .45341 L
.68255 .45343 L
.68261 .45344 L
.68268 .45346 L
.68274 .45347 L
.68281 .45349 L
.68288 .4535 L
.68293 .45352 L
.683 .45353 L
.68307 .45355 L
.68313 .45356 L
.68319 .45358 L
.68325 .45359 L
.68331 .4536 L
.68337 .45362 L
.68342 .45363 L
.68348 .45364 L
.68353 .45366 L
.68359 .45367 L
.68364 .45368 L
.6837 .4537 L
.68374 .45371 L
.6838 .45372 L
.68386 .45374 L
.68392 .45375 L
.68398 .45376 L
.68403 .45378 L
.68409 .45379 L
.68414 .4538 L
.6842 .45382 L
.68427 .45383 L
.68433 .45385 L
.68438 .45386 L
.68444 .45388 L
.6845 .45389 L
.68455 .4539 L
Mistroke
.6846 .45391 L
.68465 .45393 L
.68471 .45394 L
.68477 .45395 L
.68482 .45397 L
.68488 .45398 L
.68493 .45399 L
.68498 .454 L
.68503 .45402 L
.68508 .45403 L
.68513 .45404 L
.68517 .45405 L
.68522 .45407 L
.68527 .45408 L
.68534 .45409 L
.6854 .45411 L
.68546 .45412 L
.68552 .45414 L
.68558 .45415 L
.68565 .45417 L
.68572 .45419 L
.68578 .4542 L
.68584 .45422 L
.68589 .45423 L
.68595 .45425 L
.68602 .45426 L
.68608 .45428 L
.68613 .45429 L
.68619 .45431 L
.68625 .45432 L
.68631 .45434 L
.68636 .45435 L
.68642 .45436 L
.68648 .45438 L
.68654 .4544 L
.6866 .45441 L
.68665 .45442 L
.6867 .45444 L
.68675 .45445 L
.68681 .45446 L
.68687 .45448 L
.68692 .45449 L
.68697 .45451 L
.68702 .45452 L
.68707 .45453 L
.68712 .45454 L
.68718 .45456 L
.68723 .45457 L
.68729 .45459 L
.68735 .4546 L
Mistroke
.68741 .45462 L
.68746 .45463 L
.68751 .45465 L
.68757 .45466 L
.68764 .45468 L
.68771 .4547 L
.68777 .45471 L
.68783 .45473 L
.68789 .45474 L
.68795 .45476 L
.68801 .45478 L
.68807 .45479 L
.68813 .45481 L
.6882 .45483 L
.68827 .45484 L
.68832 .45486 L
.68838 .45488 L
.68844 .45489 L
.6885 .45491 L
.68856 .45492 L
.68862 .45494 L
.68869 .45496 L
.68874 .45497 L
.6888 .45499 L
.68887 .45501 L
.68894 .45503 L
.68901 .45504 L
.68907 .45506 L
.68913 .45508 L
.68918 .45509 L
.68924 .45511 L
.6893 .45512 L
.68935 .45514 L
.6894 .45515 L
.68946 .45517 L
.68951 .45518 L
.68957 .4552 L
.68963 .45521 L
.68969 .45523 L
.68974 .45524 L
.6898 .45526 L
.68987 .45528 L
.68993 .4553 L
.68999 .45531 L
.69004 .45533 L
.69009 .45534 L
.69014 .45535 L
.69018 .45537 L
.69024 .45538 L
.6903 .4554 L
Mistroke
.69035 .45541 L
.6904 .45543 L
.69046 .45544 L
.69051 .45546 L
.69056 .45547 L
.69062 .45549 L
.69067 .4555 L
.69073 .45552 L
.69079 .45554 L
.69084 .45555 L
.69091 .45557 L
.69097 .45559 L
.69104 .45561 L
.69111 .45563 L
.69117 .45564 L
.69123 .45566 L
.69129 .45568 L
.69136 .4557 L
.69142 .45572 L
.69149 .45574 L
.69155 .45575 L
.6916 .45577 L
.69166 .45579 L
.69171 .4558 L
.69177 .45582 L
.69183 .45583 L
.69189 .45585 L
.69196 .45587 L
.69203 .45589 L
.69209 .45591 L
.69215 .45593 L
.69221 .45594 L
.69227 .45596 L
.69233 .45598 L
.6924 .456 L
.69247 .45602 L
.69252 .45604 L
.69259 .45606 L
.69266 .45608 L
.6927 .45609 L
.69273 .4561 L
.6928 .45612 L
.69287 .45614 L
.69293 .45616 L
.69299 .45618 L
.69304 .45619 L
.69309 .45621 L
.69315 .45623 L
.69322 .45624 L
.69327 .45626 L
Mistroke
.69333 .45628 L
.69339 .4563 L
.69344 .45631 L
.6935 .45633 L
.69355 .45635 L
.6936 .45636 L
.69365 .45638 L
.6937 .45639 L
.69376 .45641 L
.69382 .45643 L
.69388 .45645 L
.69395 .45647 L
.69402 .45649 L
.69408 .45651 L
.69414 .45652 L
.6942 .45654 L
.69426 .45656 L
.69432 .45658 L
.69439 .4566 L
.69445 .45662 L
.6945 .45664 L
.69455 .45665 L
.69461 .45667 L
.69467 .45669 L
.69472 .4567 L
.69478 .45672 L
.69484 .45674 L
.69491 .45676 L
.69496 .45678 L
.69501 .4568 L
.69507 .45681 L
.69513 .45683 L
.69518 .45685 L
.69525 .45687 L
.69531 .45689 L
.69536 .45691 L
.69542 .45693 L
.69548 .45695 L
.69555 .45697 L
.69561 .45699 L
.69567 .45701 L
.69573 .45702 L
.69579 .45704 L
.69584 .45706 L
.69589 .45708 L
.69595 .45709 L
.69601 .45711 L
.69607 .45713 L
.69613 .45715 L
.69618 .45717 L
Mistroke
.69623 .45719 L
.69629 .4572 L
.69634 .45722 L
.69639 .45724 L
.69644 .45725 L
.69649 .45727 L
.69655 .45729 L
.69661 .45731 L
.69668 .45733 L
.69674 .45735 L
.6968 .45737 L
.69687 .45739 L
.69691 .45741 L
.69694 .45742 L
.69701 .45744 L
.69708 .45746 L
.69714 .45748 L
.6972 .4575 L
.69726 .45752 L
.69732 .45754 L
.69738 .45756 L
.69745 .45759 L
.69752 .45761 L
.69758 .45763 L
.69763 .45765 L
.69769 .45767 L
.69774 .45768 L
.69779 .4577 L
.69785 .45772 L
.69791 .45774 L
.69797 .45776 L
.69803 .45778 L
.69809 .4578 L
.69815 .45782 L
.6982 .45784 L
.69826 .45786 L
.69832 .45788 L
.69839 .4579 L
.69844 .45792 L
.69851 .45794 L
.69857 .45797 L
.69863 .45799 L
.6987 .45801 L
.69876 .45803 L
.69883 .45806 L
.69889 .45808 L
.69895 .4581 L
.69901 .45812 L
.69907 .45814 L
.69913 .45816 L
Mistroke
.69919 .45818 L
.69925 .4582 L
.69931 .45822 L
.69936 .45824 L
.69942 .45826 L
.69947 .45828 L
.69953 .4583 L
.69959 .45832 L
.69966 .45835 L
.69973 .45837 L
.69979 .45839 L
.69984 .45841 L
.6999 .45843 L
.69996 .45845 L
.70003 .45848 L
.7001 .4585 L
.70016 .45852 L
.70022 .45854 L
.70029 .45857 L
.70032 .45858 L
.70036 .4586 L
.70043 .45862 L
.70049 .45864 L
.70055 .45867 L
.70061 .45869 L
.70066 .4587 L
.70071 .45872 L
.70077 .45874 L
.70083 .45877 L
.70089 .45879 L
.70095 .45881 L
.701 .45883 L
.70105 .45885 L
.70111 .45887 L
.70116 .45889 L
.70121 .45891 L
.70126 .45892 L
.70131 .45894 L
.70137 .45896 L
.70143 .45899 L
.70149 .45901 L
.70156 .45903 L
.70162 .45906 L
.70168 .45908 L
.70174 .4591 L
.7018 .45912 L
.70187 .45915 L
.70192 .45917 L
.70199 .45919 L
.70205 .45921 L
Mistroke
.7021 .45923 L
.70216 .45926 L
.7022 .45927 L
.70226 .4593 L
.70232 .45932 L
.70237 .45933 L
.70241 .45935 L
.70246 .45937 L
.70252 .45939 L
.70256 .45941 L
.7026 .45943 L
.70265 .45944 L
.7027 .45946 L
.70275 .45948 L
.7028 .4595 L
.70286 .45952 L
.7029 .45954 L
.70295 .45956 L
.70301 .45958 L
.70307 .4596 L
.70312 .45962 L
.70318 .45964 L
.70323 .45967 L
.70329 .45969 L
.70334 .45971 L
.70338 .45972 L
.70343 .45974 L
.70348 .45976 L
.70353 .45978 L
.70359 .4598 L
.70364 .45983 L
.70369 .45984 L
.70374 .45986 L
.7038 .45989 L
.70386 .45991 L
.70391 .45993 L
.70397 .45995 L
.70401 .45997 L
.70406 .45999 L
.70411 .46001 L
.70416 .46003 L
.70425 .46006 L
.7043 .46008 L
.70434 .4601 L
.70439 .46012 L
.70444 .46014 L
.70449 .46016 L
.70455 .46018 L
.70459 .4602 L
.70464 .46022 L
Mistroke
.70469 .46024 L
.70474 .46026 L
.70479 .46028 L
.70483 .4603 L
.70488 .46032 L
.70493 .46034 L
.70498 .46036 L
.70503 .46038 L
.70507 .46039 L
.70512 .46041 L
.70516 .46043 L
.7052 .46045 L
.70528 .46048 L
.70532 .4605 L
.70537 .46052 L
.70546 .46055 L
.70551 .46057 L
.70557 .4606 L
.70563 .46062 L
.70568 .46064 L
.70573 .46066 L
.70578 .46068 L
.70583 .4607 L
.70589 .46073 L
.70594 .46075 L
.706 .46077 L
.70606 .4608 L
.70611 .46082 L
.70617 .46084 L
.70623 .46087 L
.70629 .4609 L
.70635 .46092 L
.7064 .46094 L
.70645 .46096 L
.7065 .46098 L
.70654 .461 L
.7066 .46102 L
.70665 .46105 L
.70671 .46107 L
.70676 .46109 L
.70681 .46111 L
.70687 .46114 L
.70691 .46116 L
.70697 .46118 L
.70702 .4612 L
.70708 .46123 L
.70713 .46125 L
.70719 .46127 L
.70724 .46129 L
.70729 .46132 L
Mistroke
.70734 .46134 L
.70739 .46136 L
.70748 .4614 L
.70753 .46142 L
.70758 .46144 L
.70763 .46146 L
.70768 .46148 L
.70773 .46151 L
.70779 .46153 L
.70784 .46155 L
.70789 .46157 L
.70794 .4616 L
.70799 .46162 L
.70804 .46164 L
.70809 .46166 L
.70814 .46168 L
.70819 .46171 L
.70824 .46173 L
.70828 .46175 L
.70833 .46177 L
.70838 .46179 L
.70843 .46181 L
.70847 .46183 L
.70852 .46185 L
.70856 .46187 L
.70865 .46191 L
.70869 .46193 L
.70873 .46195 L
.70879 .46197 L
.70884 .46199 L
.70889 .46202 L
.70895 .46204 L
.709 .46207 L
.70905 .46209 L
.7091 .46211 L
.70915 .46213 L
.7092 .46216 L
.70926 .46218 L
.70932 .46221 L
.70937 .46223 L
.70943 .46226 L
.70949 .46229 L
.70955 .46231 L
.70961 .46234 L
.70966 .46236 L
.70973 .46239 L
.70978 .46242 L
.70984 .46244 L
.70989 .46247 L
.70994 .46249 L
Mistroke
.71 .46252 L
.71005 .46254 L
.71011 .46257 L
.71017 .46259 L
.71022 .46262 L
.71027 .46264 L
.71034 .46267 L
.7104 .4627 L
.71046 .46273 L
.71052 .46276 L
.71057 .46278 L
.71062 .4628 L
.71067 .46283 L
.71073 .46285 L
.71077 .46288 L
.71082 .4629 L
.71087 .46292 L
.71092 .46294 L
.71097 .46297 L
.71103 .46299 L
.71107 .46302 L
.71112 .46304 L
.71118 .46307 L
.71124 .4631 L
.7113 .46312 L
.71135 .46315 L
.7114 .46317 L
.71145 .46319 L
.7115 .46322 L
.71154 .46324 L
.71159 .46326 L
.71165 .46329 L
.7117 .46331 L
.71175 .46334 L
.71183 .46338 L
.71188 .4634 L
.71193 .46343 L
.71198 .46345 L
.71202 .46347 L
.71207 .46349 L
.71212 .46352 L
.71217 .46354 L
.71223 .46357 L
.71228 .4636 L
.71233 .46362 L
.71238 .46365 L
.71244 .46367 L
.7125 .4637 L
.71255 .46373 L
.71261 .46376 L
Mistroke
.71267 .46379 L
.71272 .46381 L
.71278 .46384 L
.71283 .46387 L
.7129 .4639 L
.71295 .46392 L
.71301 .46395 L
.71307 .46398 L
.71312 .46401 L
.71316 .46403 L
.71321 .46405 L
.71327 .46408 L
.71333 .46411 L
.71338 .46414 L
.71343 .46416 L
.71348 .46419 L
.71353 .46421 L
.71358 .46424 L
.71363 .46426 L
.71368 .46429 L
.71372 .46431 L
.71377 .46433 L
.71382 .46436 L
.71388 .46439 L
.71395 .46442 L
.714 .46445 L
.71406 .46448 L
.71412 .46451 L
.71419 .46455 L
.71426 .46458 L
.71432 .46461 L
.71437 .46464 L
.71443 .46467 L
.71449 .4647 L
.71455 .46473 L
.71461 .46476 L
.71467 .46479 L
.71472 .46482 L
.71478 .46485 L
.71483 .46488 L
.71488 .4649 L
.71494 .46493 L
.715 .46496 L
.71506 .46499 L
.71512 .46502 L
.71517 .46505 L
.71523 .46508 L
.71527 .46511 L
.71533 .46514 L
.71539 .46517 L
Mistroke
.71544 .46519 L
.71549 .46522 L
.71553 .46524 L
.71558 .46527 L
.71563 .4653 L
.71569 .46532 L
.71573 .46535 L
.71578 .46538 L
.71584 .4654 L
.71588 .46543 L
.71593 .46545 L
.71597 .46548 L
.71603 .46551 L
.71607 .46553 L
.71612 .46556 L
.71617 .46558 L
.71621 .4656 L
.71629 .46565 L
.71633 .46567 L
.71638 .46569 L
.71643 .46572 L
.71648 .46575 L
.71653 .46577 L
.71657 .4658 L
.71663 .46583 L
.71668 .46586 L
.71674 .46589 L
.71678 .46592 L
.71683 .46594 L
.71688 .46597 L
.71693 .46599 L
.71698 .46602 L
.71703 .46605 L
.71707 .46607 L
.71712 .4661 L
.71717 .46612 L
.71721 .46615 L
.71726 .46617 L
.7173 .4662 L
.71735 .46622 L
.7174 .46625 L
.71745 .46628 L
.7175 .46631 L
.71754 .46633 L
.71762 .46638 L
.71767 .4664 L
.71772 .46643 L
.71776 .46645 L
.7178 .46648 L
.71785 .46651 L
Mistroke
.7179 .46653 L
.71798 .46658 L
.71802 .4666 L
.71806 .46662 L
.7181 .46665 L
.71814 .46667 L
.71822 .46671 L
.71826 .46673 L
.71831 .46676 L
.71835 .46679 L
.7184 .46681 L
.71848 .46686 L
.71852 .46688 L
.71856 .46691 L
.71861 .46693 L
.71866 .46696 L
.71871 .46699 L
.71876 .46702 L
.71882 .46705 L
.71887 .46708 L
.71892 .46711 L
.71897 .46714 L
.71902 .46717 L
.71906 .46719 L
.71911 .46722 L
.71916 .46725 L
.71924 .4673 L
.71929 .46733 L
.71934 .46735 L
.71939 .46738 L
.71944 .46741 L
.71949 .46744 L
.71953 .46747 L
.71958 .4675 L
.71964 .46753 L
.71969 .46756 L
.71974 .46759 L
.71979 .46762 L
.71984 .46765 L
.71989 .46768 L
.71995 .46771 L
.72001 .46775 L
.72006 .46778 L
.72011 .46781 L
.72017 .46784 L
.72022 .46787 L
.72027 .4679 L
.72032 .46793 L
.72036 .46796 L
.72045 .46801 L
Mistroke
.7205 .46804 L
.72054 .46806 L
.7206 .4681 L
.72064 .46812 L
.7207 .46816 L
.72075 .46819 L
.7208 .46822 L
.72084 .46824 L
.72089 .46827 L
.72094 .4683 L
.72103 .46836 L
.72107 .46838 L
.72112 .46841 L
.7212 .46846 L
.72124 .46848 L
.72128 .46851 L
.72133 .46854 L
.72138 .46857 L
.72143 .4686 L
.72148 .46863 L
.72156 .46868 L
.72161 .46871 L
.72166 .46874 L
.7217 .46877 L
.72175 .4688 L
.72183 .46885 L
.72188 .46888 L
.72192 .4689 L
.72197 .46893 L
.72202 .46896 L
.72206 .46899 L
.7221 .46902 L
.72218 .46906 L
.72222 .46909 L
.72227 .46912 L
.72235 .46917 L
.72239 .4692 L
.72244 .46923 L
.72249 .46926 L
.72254 .46929 L
.72262 .46934 L
.72267 .46937 L
.72271 .4694 L
.72276 .46943 L
.7228 .46946 L
.72285 .46949 L
.7229 .46952 L
.72296 .46956 L
.72301 .46959 L
.72305 .46962 L
Mistroke
.7231 .46965 L
.72314 .46968 L
.72319 .4697 L
.72327 .46976 L
.72332 .46979 L
.72336 .46982 L
.72345 .46987 L
.72349 .4699 L
.72354 .46993 L
.72362 .46998 L
.72367 .47002 L
.72372 .47005 L
.72376 .47008 L
.72381 .47011 L
.72389 .47016 L
.72393 .47019 L
.72397 .47021 L
.72404 .47026 L
.72408 .47029 L
.72412 .47031 L
.7242 .47036 L
.72425 .4704 L
.72429 .47042 L
.72437 .47048 L
.72441 .4705 L
.72446 .47053 L
.72454 .47058 L
.72458 .47061 L
.72462 .47064 L
.7247 .47069 L
.72474 .47072 L
.72478 .47075 L
.72486 .4708 L
.7249 .47083 L
.72494 .47085 L
.72501 .4709 L
.72504 .47092 L
.72508 .47095 L
.72513 .47098 L
.72517 .47101 L
.72525 .47106 L
.72529 .47109 L
.72533 .47112 L
.72538 .47115 L
.72543 .47118 L
.7255 .47123 L
.72554 .47126 L
.72558 .47129 L
.72562 .47132 L
.72567 .47135 L
Mistroke
.72571 .47137 L
.72575 .4714 L
.72578 .47142 L
.72582 .47145 L
.72589 .4715 L
.72593 .47152 L
.72597 .47155 L
.72601 .47158 L
.72604 .4716 L
.72611 .47165 L
.72618 .47169 L
.72621 .47172 L
.72625 .47175 L
.72632 .47179 L
.72636 .47182 L
.72639 .47185 L
.72643 .47187 L
.72647 .4719 L
.72651 .47193 L
.72655 .47195 L
.72662 .472 L
.72666 .47203 L
.72669 .47205 L
.72674 .47208 L
.72678 .47211 L
.72686 .47217 L
.7269 .4722 L
.72694 .47223 L
.72702 .47228 L
.72706 .47231 L
.7271 .47234 L
.72718 .4724 L
.72722 .47242 L
.72726 .47245 L
.72734 .47251 L
.72739 .47255 L
.72744 .47258 L
.72748 .47261 L
.72752 .47264 L
.7276 .4727 L
.72765 .47273 L
.72769 .47276 L
.72774 .47279 L
.72778 .47283 L
.72787 .47289 L
.72792 .47293 L
.72797 .47296 L
.72801 .47299 L
.72805 .47302 L
.72813 .47308 L
Mistroke
.72817 .47311 L
.72821 .47314 L
.72826 .47317 L
.7283 .4732 L
.72838 .47326 L
.72841 .47328 L
.72846 .47331 L
.7285 .47335 L
.72854 .47338 L
.72858 .4734 L
.72862 .47343 L
.72869 .47348 L
.72875 .47353 L
.72882 .47359 L
.72886 .47362 L
.7289 .47364 L
.72894 .47367 L
.72899 .4737 L
.72902 .47373 L
.72906 .47376 L
.7291 .47379 L
.72914 .47382 L
.72921 .47387 L
.72925 .4739 L
.72929 .47393 L
.72934 .47397 L
.72938 .474 L
.72946 .47406 L
.7295 .47409 L
.72954 .47412 L
.72959 .47415 L
.72963 .47419 L
.72972 .47425 L
.72976 .47429 L
.72981 .47432 L
.72985 .47435 L
.7299 .47439 L
.72994 .47442 L
.72999 .47446 L
.73007 .47452 L
.73011 .47455 L
.73016 .47459 L
.7302 .47462 L
.73024 .47465 L
.73033 .47471 L
.73037 .47475 L
.73042 .47478 L
.73049 .47484 L
.73054 .47488 L
.73058 .47491 L
Mistroke
.73062 .47494 L
.73065 .47497 L
.73072 .47502 L
.7308 .47508 L
.73084 .47511 L
.73088 .47514 L
.73097 .47521 L
.73102 .47525 L
.73106 .47528 L
.73115 .47535 L
.73119 .47538 L
.73124 .47542 L
.73132 .47548 L
.73136 .47552 L
.73141 .47555 L
.73144 .47558 L
.73149 .47562 L
.73156 .47567 L
.73161 .47571 L
.73165 .47574 L
.7317 .47578 L
.73175 .47582 L
.73184 .47589 L
.73189 .47593 L
.73194 .47597 L
.73198 .47601 L
.73202 .47604 L
.7321 .47611 L
.73215 .47614 L
.7322 .47618 L
.73224 .47622 L
.73229 .47625 L
.73236 .47631 L
.73242 .47636 L
.73249 .47641 L
.73257 .47648 L
.73263 .47653 L
.7327 .47659 L
.73276 .47663 L
.73283 .47669 L
.7329 .47675 L
.73297 .47681 L
.73305 .47687 L
.73309 .4769 L
.73313 .47694 L
.7332 .47699 L
.73324 .47703 L
.73328 .47706 L
.73335 .47712 L
.73342 .47717 L
Mistroke
.73349 .47723 L
.73356 .47729 L
.7336 .47732 L
.73364 .47735 L
.73371 .47741 L
.73379 .47748 L
.73385 .47753 L
.73392 .47759 L
.73398 .47764 L
.73403 .47768 L
.73407 .47771 L
.73414 .47777 L
.73418 .47781 L
.73422 .47784 L
.73426 .47787 L
.7343 .47791 L
.73437 .47797 L
.73445 .47803 L
.73452 .47809 L
.73456 .47812 L
.7346 .47816 L
.73466 .47821 L
.73473 .47827 L
.7348 .47833 L
.73486 .47838 L
.73493 .47844 L
.73498 .47849 L
.73505 .47854 L
.73511 .4786 L
.73518 .47865 L
.73525 .47871 L
.73532 .47877 L
.73538 .47882 L
.73545 .47889 L
.73553 .47895 L
.7356 .47902 L
.73567 .47908 L
.73573 .47913 L
.73579 .47918 L
.73585 .47923 L
.7359 .47928 L
.73596 .47933 L
.73603 .47939 L
.7361 .47945 L
.73616 .4795 L
.73622 .47956 L
.73628 .47961 L
.73634 .47966 L
.73641 .47972 L
.73647 .47978 L
Mistroke
.73654 .47984 L
.7366 .4799 L
.73667 .47996 L
.73675 .48003 L
.73682 .48009 L
.7369 .48016 L
.73694 .48019 L
.73698 .48023 L
.73705 .4803 L
.73713 .48037 L
.73721 .48044 L
.73728 .4805 L
.73734 .48056 L
.73741 .48062 L
.73748 .48068 L
.73756 .48075 L
.73763 .48082 L
.73769 .48088 L
.73776 .48093 L
.73782 .48099 L
.7379 .48106 L
.73797 .48113 L
.73805 .4812 L
.73809 .48124 L
.73813 .48127 L
.73821 .48135 L
.73826 .48139 L
.7383 .48143 L
.73837 .4815 L
.73846 .48157 L
.73852 .48164 L
.7386 .48171 L
.73867 .48177 L
.73873 .48183 L
.7388 .48189 L
.73888 .48197 L
.73896 .48204 L
.73903 .48211 L
.73911 .48218 L
.73917 .48224 L
.73924 .48231 L
.73932 .48238 L
.73938 .48244 L
.73944 .4825 L
.73951 .48256 L
.73958 .48263 L
.73965 .48269 L
.73972 .48275 L
.73978 .48282 L
.73984 .48287 L
Mistroke
.7399 .48293 L
.73996 .48298 L
.74002 .48304 L
.74008 .4831 L
.74014 .48316 L
.74021 .48323 L
.74029 .4833 L
.74036 .48336 L
.74042 .48343 L
.74048 .48348 L
.74055 .48355 L
.74061 .48361 L
.74068 .48368 L
.74075 .48375 L
.74081 .48381 L
.74088 .48387 L
.74096 .48395 L
.74104 .48402 L
.74111 .4841 L
.74118 .48416 L
.74126 .48424 L
.74133 .48431 L
.74141 .48439 L
.7415 .48448 L
.74157 .48455 L
.74164 .48462 L
.74172 .48469 L
.7418 .48477 L
.74187 .48484 L
.74195 .48492 L
.74202 .48499 L
.74209 .48506 L
.74215 .48512 L
.74222 .48519 L
.74229 .48526 L
.74235 .48532 L
.74243 .4854 L
.74251 .48548 L
.74259 .48556 L
.74267 .48564 L
.74274 .48571 L
.7428 .48578 L
.74288 .48585 L
.74295 .48592 L
.74303 .486 L
.74311 .48609 L
.74318 .48616 L
.74326 .48624 L
.74334 .48632 L
.74343 .48641 L
Mistroke
.74351 .4865 L
.74359 .48657 L
.74366 .48665 L
.74373 .48671 L
.7438 .48679 L
.74387 .48686 L
.74399 .48699 L
.74407 .48706 L
.74413 .48713 L
.7442 .4872 L
.74427 .48727 L
.74434 .48735 L
.74441 .48742 L
.7445 .48751 L
.74457 .48759 L
.74464 .48766 L
.74471 .48773 L
.74484 .48787 L
.74491 .48794 L
.74498 .48801 L
.74512 .48816 L
.74519 .48824 L
.74526 .48831 L
.74538 .48843 L
.74545 .4885 L
.74551 .48857 L
.74564 .4887 L
.74571 .48878 L
.74577 .48884 L
.74592 .489 L
.746 .48908 L
.74607 .48917 L
.74623 .48933 L
.74632 .48942 L
.7464 .48951 L
.74654 .48966 L
.74662 .48974 L
.74669 .48982 L
.74682 .48997 L
.7469 .49005 L
.74697 .49012 L
.74705 .49021 L
.74713 .4903 L
.74729 .49047 L
.74737 .49056 L
.74745 .49064 L
.74754 .49074 L
.74763 .49084 L
.74779 .49101 L
.74793 .49116 L
Mistroke
.74808 .49133 L
.74817 .49142 L
.74825 .49151 L
.74838 .49165 L
.74852 .49181 L
.74866 .49196 L
.74878 .49209 L
.74892 .49225 L
.74908 .49242 L
.74922 .49258 L
.74936 .49273 L
.74951 .4929 L
.74966 .49306 L
.74992 .49336 L
.75007 .49352 L
.75021 .49368 L
.75052 .49402 L
.75078 .4943 L
.75101 .49456 L
.75153 .49514 L
.75245 .49618 L
.75297 .49675 L
.75324 .49706 L
.75352 .49737 L
.75377 .49765 L
.75404 .49796 L
.75416 .49809 L
.75429 .49823 L
.75452 .49849 L
.75465 .49863 L
.75478 .49878 L
.75491 .49892 L
.75502 .49904 L
.75516 .49919 L
.75523 .49927 L
.7553 .49935 L
.75537 .49942 L
.75544 .4995 L
.75557 .49964 L
.75564 .49972 L
.75571 .4998 L
.75584 .49994 L
.75591 .50002 L
.75598 .50009 L
.75605 .50017 L
.75613 .50026 L
.7562 .50033 L
.75627 .50041 L
.75634 .50048 L
.75641 .50055 L
Mistroke
.75646 .50062 L
.75653 .50068 L
.75659 .50076 L
.75665 .50082 L
.75672 .50089 L
.75677 .50095 L
.75684 .50102 L
.7569 .50109 L
.75696 .50115 L
.75702 .50121 L
.75708 .50127 L
.75714 .50134 L
.75719 .5014 L
.75725 .50146 L
.75731 .50153 L
.75737 .50159 L
.75742 .50164 L
.75747 .50169 L
.75753 .50175 L
.75758 .50181 L
.75764 .50187 L
.7577 .50193 L
.75773 .50197 L
.75777 .502 L
.7578 .50203 L
.75783 .50207 L
.75786 .5021 L
.75789 .50213 L
.75795 .50219 L
.758 .50224 L
.75805 .5023 L
.75809 .50233 L
.75811 .50236 L
.75815 .5024 L
.75818 .50243 L
.75824 .50249 L
.75827 .50252 L
.7583 .50255 L
.75833 .50259 L
.75836 .50262 L
.75843 .50268 L
.75846 .50272 L
.75849 .50275 L
.75853 .50279 L
.75856 .50283 L
.7586 .50286 L
.75863 .50289 L
.75868 .50295 L
.75872 .50298 L
.75875 .50301 L
Mistroke
.75878 .50304 L
.75881 .50308 L
.75884 .50311 L
.75887 .50314 L
.75892 .50319 L
.75896 .50322 L
.75899 .50325 L
.75901 .50328 L
.75904 .50331 L
.75908 .50334 L
.75911 .50338 L
.75914 .50341 L
.75918 .50344 L
.75921 .50348 L
.75925 .50352 L
.75928 .50355 L
.75932 .50358 L
.75935 .50362 L
.75938 .50365 L
.75941 .50368 L
.75945 .50371 L
.75948 .50374 L
.7595 .50377 L
.75954 .5038 L
.75957 .50383 L
.7596 .50386 L
.75963 .50389 L
.75965 .50391 L
.75968 .50394 L
.75971 .50397 L
.75974 .504 L
.75977 .50403 L
.7598 .50406 L
.75983 .50408 L
.75985 .50411 L
.75988 .50414 L
.75991 .50417 L
.75994 .50419 L
.75996 .50421 L
.75999 .50424 L
.76002 .50427 L
.76005 .5043 L
.76008 .50433 L
.76012 .50436 L
.76013 .50438 L
.76015 .50439 L
.76018 .50442 L
.7602 .50444 L
.76022 .50445 L
.76025 .50448 L
Mistroke
.76028 .50451 L
.7603 .50454 L
.76033 .50457 L
.76036 .50459 L
.76039 .50462 L
.76042 .50465 L
.76044 .50466 L
.76045 .50468 L
.76048 .5047 L
.76051 .50473 L
.76054 .50476 L
.76057 .50478 L
.76059 .50481 L
.76062 .50483 L
.76064 .50486 L
.76067 .50488 L
.7607 .5049 L
.76072 .50493 L
.76075 .50495 L
.76078 .50498 L
.7608 .505 L
.76083 .50502 L
.76084 .50504 L
.76086 .50505 L
.76087 .50507 L
.76089 .50508 L
.7609 .50509 L
.76092 .50511 L
.76094 .50513 L
.76097 .50515 L
.76099 .50517 L
.76102 .5052 L
.76104 .50522 L
.76106 .50524 L
.76108 .50525 L
.76109 .50526 L
.76112 .50529 L
.76114 .50531 L
.76117 .50533 L
.76118 .50534 L
.7612 .50536 L
.76122 .50538 L
.76123 .50539 L
.76125 .5054 L
.76127 .50543 L
.76129 .50544 L
.7613 .50545 L
.76132 .50547 L
.76135 .50549 L
.76136 .5055 L
Mistroke
.76137 .50552 L
.7614 .50554 L
.76141 .50555 L
.76143 .50556 L
.76146 .50559 L
.76147 .5056 L
.76149 .50561 L
.76151 .50564 L
.76153 .50565 L
.76154 .50566 L
.76156 .50568 L
.76157 .50569 L
.7616 .50571 L
.76161 .50572 L
.76163 .50574 L
.76165 .50576 L
.76166 .50577 L
.76167 .50578 L
.7617 .5058 L
.76172 .50581 L
.76173 .50583 L
.76176 .50585 L
.76177 .50586 L
.76178 .50587 L
.7618 .50588 L
.76181 .5059 L
.76183 .50591 L
.76184 .50592 L
.76187 .50595 L
.76189 .50596 L
.7619 .50597 L
.76192 .50598 L
.76193 .506 L
.76195 .50601 L
.76197 .50603 L
.76198 .50604 L
.76199 .50605 L
.76201 .50606 L
.76203 .50608 L
.76204 .50609 L
.76206 .5061 L
.76207 .50611 L
.76209 .50613 L
.7621 .50614 L
.76212 .50615 L
.76213 .50616 L
.76215 .50618 L
.76216 .50619 L
.76217 .5062 L
.76219 .50621 L
Mistroke
.7622 .50622 L
.76223 .50624 L
.76224 .50626 L
.76226 .50627 L
.76227 .50628 L
.76228 .50629 L
.7623 .5063 L
.76231 .50631 L
.76233 .50633 L
.76234 .50634 L
.76235 .50635 L
.76237 .50636 L
.76239 .50637 L
.7624 .50639 L
.76242 .5064 L
.76244 .50641 L
.76245 .50642 L
.76247 .50644 L
.76248 .50645 L
.7625 .50646 L
.76251 .50647 L
.76253 .50649 L
.76255 .5065 L
.76256 .50651 L
.76258 .50653 L
.76259 .50654 L
.7626 .50655 L
.76262 .50656 L
.76263 .50657 L
.76265 .50659 L
.76267 .5066 L
.76268 .50661 L
.7627 .50662 L
.76271 .50663 L
.76272 .50664 L
.76273 .50665 L
.76275 .50666 L
.76276 .50668 L
.76278 .50668 L
.76279 .5067 L
.76281 .50671 L
.76282 .50672 L
.76284 .50673 L
.76285 .50674 L
.76286 .50675 L
.76288 .50676 L
.76289 .50677 L
.76291 .50678 L
.76292 .5068 L
.76293 .5068 L
Mistroke
.76294 .50681 L
.76296 .50682 L
.76297 .50683 L
.76298 .50685 L
.763 .50686 L
.76301 .50687 L
.76303 .50688 L
.76304 .50689 L
.76305 .5069 L
.76307 .50691 L
.76308 .50692 L
.7631 .50693 L
.76311 .50694 L
.76313 .50695 L
.76314 .50696 L
.76316 .50698 L
.76317 .50698 L
.76317 .50699 L
.76319 .507 L
.7632 .50701 L
.76321 .50701 L
.76322 .50703 L
.76324 .50704 L
.76325 .50705 L
.76326 .50706 L
.76327 .50706 L
.76329 .50707 L
.7633 .50708 L
.76331 .50709 L
.76332 .5071 L
.76333 .50711 L
.76334 .50711 L
.76335 .50712 L
.76335 .50712 L
.76337 .50713 L
.76338 .50714 L
.76339 .50715 L
.7634 .50716 L
.76342 .50717 L
.76343 .50718 L
.76344 .50718 L
.76344 .50719 L
.76346 .5072 L
.76347 .50721 L
.76348 .50721 L
.76349 .50722 L
.7635 .50723 L
.76351 .50724 L
.76353 .50725 L
.76354 .50726 L
Mistroke
.76355 .50726 L
.76356 .50728 L
.76357 .50728 L
.76358 .50729 L
.76359 .5073 L
.7636 .5073 L
.76362 .50731 L
.76363 .50732 L
.76364 .50733 L
.76365 .50734 L
.76366 .50734 L
.76367 .50735 L
.76368 .50736 L
.76369 .50737 L
.7637 .50737 L
.76372 .50738 L
.76373 .50739 L
.76373 .5074 L
.76375 .50741 L
.76376 .50741 L
.76377 .50742 L
.76378 .50743 L
.76379 .50744 L
.7638 .50744 L
.76381 .50745 L
.76382 .50746 L
.76383 .50746 L
.76384 .50747 L
.76385 .50748 L
.76386 .50748 L
.76387 .50749 L
.76387 .50749 L
.76389 .5075 L
.7639 .50751 L
.76391 .50752 L
.76392 .50752 L
.76393 .50753 L
.76393 .50753 L
.76394 .50754 L
.76395 .50755 L
.76396 .50755 L
.76397 .50756 L
.76397 .50756 L
.76399 .50757 L
.764 .50758 L
.76401 .50758 L
.76401 .50759 L
.76402 .50759 L
.76403 .5076 L
.76404 .5076 L
Mistroke
.76405 .50761 L
.76406 .50762 L
.76407 .50762 L
.76407 .50763 L
.76408 .50764 L
.7641 .50765 L
.76411 .50765 L
.76411 .50766 L
.76412 .50766 L
.76413 .50767 L
.76414 .50767 L
.76414 .50768 L
.76415 .50768 L
.76416 .50768 L
.76417 .50769 L
.76418 .5077 L
.76418 .5077 L
.76419 .50771 L
.7642 .50771 L
.76421 .50772 L
.76422 .50772 L
.76423 .50773 L
.76424 .50774 L
.76424 .50774 L
.76425 .50775 L
.76426 .50775 L
.76427 .50776 L
.76428 .50777 L
.76429 .50777 L
.7643 .50778 L
.7643 .50778 L
.76432 .50779 L
.76432 .50779 L
.76433 .5078 L
.76435 .50781 L
.76435 .50781 L
.76436 .50782 L
.76437 .50782 L
.76438 .50783 L
.76439 .50784 L
.7644 .50784 L
.76441 .50785 L
.76441 .50785 L
.76442 .50786 L
.76443 .50786 L
.76444 .50787 L
.76445 .50787 L
.76445 .50788 L
.76446 .50788 L
.76447 .50789 L
Mistroke
.76448 .50789 L
.76449 .5079 L
.7645 .50791 L
.76451 .50791 L
.76452 .50792 L
.76452 .50792 L
.76453 .50793 L
.76454 .50793 L
.76455 .50794 L
.76455 .50794 L
.76456 .50795 L
.76458 .50795 L
.76458 .50796 L
.76459 .50796 L
.7646 .50797 L
.7646 .50797 L
.76462 .50798 L
.76462 .50798 L
.76463 .50799 L
.76464 .50799 L
.76465 .508 L
.76465 .508 L
.76466 .50801 L
.76467 .50801 L
.76467 .50801 L
.76468 .50802 L
.76469 .50802 L
.7647 .50803 L
.7647 .50803 L
.76471 .50804 L
.76472 .50804 L
.76472 .50804 L
.76473 .50805 L
.76474 .50805 L
.76475 .50806 L
.76475 .50806 L
.76476 .50807 L
.76477 .50807 L
.76478 .50808 L
.76478 .50808 L
.76479 .50809 L
.7648 .50809 L
.76481 .5081 L
.76482 .5081 L
.76482 .5081 L
.76483 .50811 L
.76484 .50811 L
.76485 .50812 L
.76485 .50812 L
.76486 .50813 L
Mistroke
.76487 .50813 L
.76487 .50814 L
.76488 .50814 L
.76489 .50814 L
.7649 .50815 L
.76491 .50816 L
.76492 .50816 L
.76492 .50816 L
.76493 .50817 L
.76493 .50817 L
.76494 .50817 L
.76495 .50818 L
.76496 .50818 L
.76497 .50819 L
.76497 .50819 L
.76498 .5082 L
.76499 .5082 L
.76499 .5082 L
.765 .50821 L
.76501 .50821 L
.76501 .50822 L
.76502 .50822 L
.76503 .50823 L
.76504 .50823 L
.76505 .50823 L
.76505 .50824 L
.76506 .50824 L
.76507 .50825 L
.76507 .50825 L
.76508 .50825 L
.76508 .50826 L
.76509 .50826 L
.7651 .50827 L
.76511 .50827 L
.76511 .50827 L
.76512 .50828 L
.76513 .50828 L
.76513 .50828 L
.76514 .50829 L
.76515 .50829 L
.76515 .5083 L
.76516 .5083 L
.76517 .5083 L
.76517 .50831 L
.76518 .50831 L
.76519 .50832 L
.7652 .50832 L
.76521 .50832 L
.76521 .50833 L
.76522 .50833 L
Mistroke
.76523 .50834 L
.76523 .50834 L
.76524 .50835 L
.76525 .50835 L
.76526 .50835 L
.76527 .50836 L
.76527 .50836 L
.76528 .50837 L
.76529 .50837 L
.7653 .50838 L
.76531 .50838 L
.76531 .50838 L
.76532 .50839 L
.76533 .50839 L
.76533 .5084 L
.76534 .5084 L
.76535 .5084 L
.76536 .50841 L
.76536 .50841 L
.76537 .50841 L
.76537 .50842 L
.76538 .50842 L
.76539 .50843 L
.7654 .50843 L
.7654 .50843 L
.76541 .50844 L
.76542 .50844 L
.76542 .50844 L
.76543 .50845 L
.76544 .50845 L
.76544 .50845 L
.76545 .50846 L
.76545 .50846 L
.76546 .50846 L
.76547 .50847 L
.76547 .50847 L
.76548 .50847 L
.76549 .50848 L
.76549 .50848 L
.7655 .50848 L
.76551 .50849 L
.76551 .50849 L
.76552 .50849 L
.76553 .5085 L
.76553 .5085 L
.76554 .50851 L
.76555 .50851 L
.76556 .50851 L
.76556 .50852 L
.76557 .50852 L
Mistroke
.76558 .50853 L
.76558 .50853 L
.76559 .50853 L
.7656 .50853 L
.7656 .50854 L
.76561 .50854 L
.76561 .50854 L
.76562 .50855 L
.76562 .50855 L
.76563 .50855 L
.76563 .50855 L
.76564 .50855 L
.76564 .50856 L
.76565 .50856 L
.76566 .50857 L
.76566 .50857 L
.76567 .50857 L
.76568 .50857 L
.76568 .50858 L
.76569 .50858 L
.76569 .50858 L
.7657 .50858 L
.7657 .50859 L
.76571 .50859 L
.76572 .5086 L
.76572 .5086 L
.76573 .5086 L
.76573 .5086 L
.76574 .50861 L
.76575 .50861 L
.76575 .50861 L
.76576 .50862 L
.76577 .50862 L
.76577 .50862 L
.76578 .50863 L
.76579 .50863 L
.76579 .50863 L
.7658 .50864 L
.76581 .50864 L
.76582 .50864 L
.76582 .50865 L
.76583 .50865 L
.76583 .50865 L
.76583 .50865 L
.76584 .50865 L
.76585 .50866 L
.76585 .50866 L
.76585 .50866 L
.76586 .50866 L
.76587 .50867 L
Mistroke
.76587 .50867 L
.76587 .50867 L
.76588 .50867 L
.76588 .50868 L
.76589 .50868 L
.7659 .50868 L
.7659 .50868 L
.7659 .50869 L
.76591 .50869 L
.76591 .50869 L
.76592 .50869 L
.76592 .50869 L
.76593 .5087 L
.76593 .5087 L
.76594 .5087 L
.76594 .5087 L
.76595 .50871 L
.76595 .50871 L
.76596 .50871 L
.76596 .50871 L
.76597 .50872 L
.76597 .50872 L
.76598 .50872 L
.76598 .50872 L
.76598 .50872 L
.76599 .50873 L
.766 .50873 L
.766 .50873 L
.766 .50873 L
.76601 .50874 L
.76601 .50874 L
.76602 .50874 L
.76602 .50874 L
.76603 .50874 L
.76603 .50875 L
.76604 .50875 L
.76604 .50875 L
.76605 .50875 L
.76605 .50875 L
.76605 .50876 L
.76606 .50876 L
.76606 .50876 L
.76607 .50876 L
.76608 .50877 L
.76608 .50877 L
.76609 .50877 L
.76609 .50878 L
.7661 .50878 L
.7661 .50878 L
.76611 .50878 L
Mistroke
.76612 .50879 L
.76612 .50879 L
.76612 .50879 L
.76613 .50879 L
.76614 .50879 L
.76614 .5088 L
.76615 .5088 L
.76616 .5088 L
.76616 .50881 L
.76617 .50881 L
.76617 .50881 L
.76618 .50881 L
.76618 .50881 L
.76618 .50882 L
.76619 .50882 L
.76619 .50882 L
.7662 .50882 L
.7662 .50882 L
.7662 .50882 L
.76621 .50883 L
.76621 .50883 L
.76622 .50883 L
.76623 .50883 L
.76623 .50884 L
.76624 .50884 L
.76625 .50884 L
.76625 .50884 L
.76625 .50885 L
.76626 .50885 L
.76626 .50885 L
.76627 .50885 L
.76628 .50886 L
.76628 .50886 L
.76629 .50886 L
.76629 .50886 L
.7663 .50886 L
.7663 .50887 L
.7663 .50887 L
.76631 .50887 L
.76631 .50887 L
.76632 .50887 L
.76632 .50888 L
.76633 .50888 L
.76633 .50888 L
.76634 .50888 L
.76634 .50888 L
.76635 .50889 L
.76635 .50889 L
.76636 .50889 L
.76637 .50889 L
Mistroke
.76637 .5089 L
.76638 .5089 L
.76638 .5089 L
.76639 .5089 L
.76639 .5089 L
.76639 .50891 L
.7664 .50891 L
.7664 .50891 L
.76641 .50891 L
.76641 .50891 L
.76642 .50892 L
.76643 .50892 L
.76643 .50892 L
.76644 .50892 L
.76644 .50893 L
.76644 .50893 L
.76645 .50893 L
.76645 .50893 L
.76646 .50893 L
.76646 .50893 L
.76647 .50894 L
.76647 .50894 L
.76647 .50894 L
.76648 .50894 L
.76648 .50894 L
.76649 .50894 L
.76649 .50895 L
.76649 .50895 L
.7665 .50895 L
.7665 .50895 L
.76651 .50895 L
.76651 .50895 L
.76652 .50896 L
.76652 .50896 L
.76652 .50896 L
.76653 .50896 L
.76653 .50896 L
.76653 .50896 L
.76654 .50896 L
.76654 .50897 L
.76655 .50897 L
.76655 .50897 L
.76655 .50897 L
.76656 .50897 L
.76656 .50897 L
.76656 .50898 L
.76657 .50898 L
.76657 .50898 L
.76658 .50898 L
.76658 .50898 L
Mistroke
.76659 .50898 L
.76659 .50899 L
.7666 .50899 L
.7666 .50899 L
.7666 .50899 L
.76661 .50899 L
.76661 .50899 L
.76661 .509 L
.76662 .509 L
.76662 .509 L
.76663 .509 L
.76663 .509 L
.76663 .509 L
.76664 .509 L
.76664 .50901 L
.76665 .50901 L
.76665 .50901 L
.76665 .50901 L
.76666 .50901 L
.76666 .50901 L
.76667 .50902 L
.76667 .50902 L
.76667 .50902 L
.76668 .50902 L
.76668 .50902 L
.76668 .50902 L
.76669 .50902 L
.76669 .50903 L
.76669 .50903 L
.7667 .50903 L
.7667 .50903 L
.7667 .50903 L
.76671 .50903 L
.76671 .50903 L
.76672 .50903 L
.76672 .50904 L
.76672 .50904 L
.76673 .50904 L
.76673 .50904 L
.76673 .50904 L
.76674 .50904 L
.76674 .50904 L
.76674 .50904 L
.76675 .50905 L
.76675 .50905 L
.76676 .50905 L
.76676 .50905 L
.76676 .50905 L
.76677 .50905 L
.76677 .50906 L
Mistroke
.76677 .50906 L
.76678 .50906 L
.76678 .50906 L
.76679 .50906 L
.76679 .50906 L
.76679 .50906 L
.7668 .50906 L
.7668 .50907 L
.76681 .50907 L
.76681 .50907 L
.76681 .50907 L
.76682 .50907 L
.76682 .50907 L
.76683 .50908 L
.76683 .50908 L
.76683 .50908 L
.76684 .50908 L
.76684 .50908 L
.76684 .50908 L
.76685 .50908 L
.76685 .50908 L
.76685 .50909 L
.76686 .50909 L
.76686 .50909 L
.76686 .50909 L
.76687 .50909 L
.76687 .50909 L
.76687 .50909 L
.76688 .50909 L
.76688 .50909 L
.76688 .5091 L
.76689 .5091 L
.76689 .5091 L
.76689 .5091 L
.7669 .5091 L
.7669 .5091 L
.76691 .5091 L
.76691 .5091 L
.76691 .50911 L
.76692 .50911 L
.76692 .50911 L
.76692 .50911 L
.76693 .50911 L
.76693 .50911 L
.76693 .50911 L
.76694 .50911 L
.76694 .50912 L
.76694 .50912 L
.76695 .50912 L
.76695 .50912 L
Mistroke
.76696 .50912 L
.76696 .50912 L
.76696 .50912 L
.76697 .50912 L
.76697 .50913 L
.76697 .50913 L
.76698 .50913 L
.76698 .50913 L
.76698 .50913 L
.76698 .50913 L
.76699 .50913 L
.76699 .50913 L
.767 .50914 L
.767 .50914 L
.767 .50914 L
.76701 .50914 L
.76701 .50914 L
.76701 .50914 L
.76702 .50914 L
.76702 .50914 L
.76702 .50914 L
.76703 .50915 L
.76703 .50915 L
.76703 .50915 L
.76704 .50915 L
.76704 .50915 L
.76705 .50915 L
.76705 .50915 L
.76705 .50915 L
.76706 .50916 L
.76706 .50916 L
.76707 .50916 L
.76707 .50916 L
.76708 .50916 L
.76708 .50916 L
.76708 .50916 L
.76709 .50917 L
.76709 .50917 L
.76709 .50917 L
.7671 .50917 L
.7671 .50917 L
.7671 .50917 L
.7671 .50917 L
.76711 .50917 L
.76711 .50917 L
.76712 .50918 L
.76712 .50918 L
.76712 .50918 L
.76713 .50918 L
.76713 .50918 L
Mistroke
.76713 .50918 L
.76714 .50918 L
.76714 .50918 L
.76714 .50918 L
.76715 .50919 L
.76715 .50919 L
.76716 .50919 L
.76716 .50919 L
.76716 .50919 L
.76716 .50919 L
.76717 .50919 L
.76717 .50919 L
.76718 .50919 L
.76718 .5092 L
.76718 .5092 L
.76718 .5092 L
.76719 .5092 L
.76719 .5092 L
.76719 .5092 L
.7672 .5092 L
.7672 .5092 L
.7672 .5092 L
.7672 .5092 L
.76721 .5092 L
.76721 .50921 L
.76721 .50921 L
.76722 .50921 L
.76722 .50921 L
.76722 .50921 L
.76723 .50921 L
.76723 .50921 L
.76723 .50921 L
.76723 .50921 L
.76724 .50921 L
.76724 .50922 L
.76724 .50922 L
.76725 .50922 L
.76725 .50922 L
.76725 .50922 L
.76726 .50922 L
.76726 .50922 L
.76726 .50922 L
.76727 .50922 L
.76727 .50922 L
.76727 .50923 L
.76728 .50923 L
.76728 .50923 L
.76728 .50923 L
.76729 .50923 L
.76729 .50923 L
Mistroke
.76729 .50923 L
.76729 .50923 L
.7673 .50923 L
.7673 .50923 L
.7673 .50923 L
.76731 .50924 L
.76731 .50924 L
.76731 .50924 L
.76731 .50924 L
.76732 .50924 L
.76732 .50924 L
.76732 .50924 L
.76733 .50924 L
.76733 .50924 L
.76734 .50924 L
.76734 .50924 L
.76734 .50925 L
.76734 .50925 L
.76735 .50925 L
.76735 .50925 L
.76736 .50925 L
.76736 .50925 L
.76736 .50925 L
.76737 .50925 L
.76737 .50925 L
.76737 .50925 L
.76738 .50926 L
.76738 .50926 L
.76738 .50926 L
.76739 .50926 L
.76739 .50926 L
.76739 .50926 L
.7674 .50926 L
.7674 .50926 L
.7674 .50926 L
.76741 .50926 L
.76741 .50927 L
.76741 .50927 L
.76742 .50927 L
.76742 .50927 L
.76742 .50927 L
.76743 .50927 L
.76743 .50927 L
.76743 .50927 L
.76744 .50927 L
.76744 .50927 L
.76744 .50928 L
.76745 .50928 L
.76745 .50928 L
.76745 .50928 L
Mistroke
.76746 .50928 L
.76746 .50928 L
.76746 .50928 L
.76747 .50928 L
.76747 .50928 L
.76747 .50928 L
.76747 .50928 L
.76748 .50928 L
.76748 .50929 L
.76748 .50929 L
.76749 .50929 L
.76749 .50929 L
.76749 .50929 L
.7675 .50929 L
.7675 .50929 L
.7675 .50929 L
.76751 .50929 L
.76751 .50929 L
.76751 .50929 L
.76752 .5093 L
.76752 .5093 L
.76752 .5093 L
.76753 .5093 L
.76753 .5093 L
.76753 .5093 L
.76753 .5093 L
.76754 .5093 L
.76754 .5093 L
.76754 .5093 L
.76755 .5093 L
.76755 .5093 L
.76755 .50931 L
.76756 .50931 L
.76756 .50931 L
.76756 .50931 L
.76757 .50931 L
.76757 .50931 L
.76757 .50931 L
.76757 .50931 L
.76758 .50931 L
.76758 .50931 L
.76758 .50931 L
.76759 .50931 L
.76759 .50932 L
.76759 .50932 L
.7676 .50932 L
.7676 .50932 L
.7676 .50932 L
.76761 .50932 L
.76761 .50932 L
Mistroke
.76761 .50932 L
.76762 .50932 L
.76762 .50932 L
.76762 .50932 L
.76763 .50932 L
.76763 .50933 L
.76763 .50933 L
.76764 .50933 L
.76764 .50933 L
.76764 .50933 L
.76765 .50933 L
.76765 .50933 L
.76765 .50933 L
.76766 .50933 L
.76766 .50933 L
.76766 .50933 L
.76767 .50933 L
.76767 .50934 L
.76767 .50934 L
.76768 .50934 L
.76768 .50934 L
.76768 .50934 L
.76769 .50934 L
.76769 .50934 L
.76769 .50934 L
.76769 .50934 L
.7677 .50934 L
.7677 .50934 L
.76771 .50934 L
.76771 .50935 L
.76771 .50935 L
.76772 .50935 L
.76772 .50935 L
.76772 .50935 L
.76773 .50935 L
.76773 .50935 L
.76774 .50935 L
.76774 .50935 L
.76774 .50935 L
.76775 .50935 L
.76775 .50936 L
.76775 .50936 L
.76776 .50936 L
.76776 .50936 L
.76776 .50936 L
.76777 .50936 L
.76777 .50936 L
.76777 .50936 L
.76778 .50936 L
.76778 .50936 L
Mistroke
.76779 .50936 L
.76779 .50936 L
.76779 .50937 L
.7678 .50937 L
.7678 .50937 L
.7678 .50937 L
.7678 .50937 L
.76781 .50937 L
.76781 .50937 L
.76781 .50937 L
.76782 .50937 L
.76782 .50937 L
.76782 .50937 L
.76783 .50937 L
.76783 .50937 L
.76783 .50937 L
.76784 .50938 L
.76784 .50938 L
.76784 .50938 L
.76785 .50938 L
.76785 .50938 L
.76785 .50938 L
.76786 .50938 L
.76786 .50938 L
.76786 .50938 L
.76786 .50938 L
.76787 .50938 L
.76787 .50938 L
.76787 .50938 L
.76788 .50938 L
.76788 .50939 L
.76788 .50939 L
.76789 .50939 L
.76789 .50939 L
.76789 .50939 L
.76789 .50939 L
.7679 .50939 L
.7679 .50939 L
.7679 .50939 L
.76791 .50939 L
.76791 .50939 L
.76791 .50939 L
.76792 .50939 L
.76792 .50939 L
.76792 .50939 L
.76793 .5094 L
.76793 .5094 L
.76793 .5094 L
.76794 .5094 L
.76794 .5094 L
Mistroke
.76794 .5094 L
.76795 .5094 L
.76795 .5094 L
.76795 .5094 L
.76796 .5094 L
.76796 .5094 L
.76797 .5094 L
.76797 .5094 L
.76797 .50941 L
.76798 .50941 L
.76798 .50941 L
.76798 .50941 L
.76799 .50941 L
.76799 .50941 L
.76799 .50941 L
.768 .50941 L
.768 .50941 L
.768 .50941 L
.768 .50941 L
.76801 .50941 L
.76801 .50941 L
.76801 .50941 L
.76802 .50941 L
.76802 .50941 L
.76802 .50942 L
.76803 .50942 L
.76803 .50942 L
.76803 .50942 L
.76803 .50942 L
.76804 .50942 L
.76804 .50942 L
.76804 .50942 L
.76805 .50942 L
.76805 .50942 L
.76805 .50942 L
.76806 .50942 L
.76806 .50942 L
.76806 .50942 L
.76807 .50942 L
.76807 .50942 L
.76807 .50942 L
.76807 .50943 L
.76808 .50943 L
.76808 .50943 L
.76808 .50943 L
.76809 .50943 L
.76809 .50943 L
.76809 .50943 L
.7681 .50943 L
.7681 .50943 L
Mistroke
.7681 .50943 L
.76811 .50943 L
.76811 .50943 L
.76811 .50943 L
.76811 .50943 L
.76812 .50943 L
.76812 .50943 L
.76812 .50944 L
.76813 .50944 L
.76813 .50944 L
.76813 .50944 L
.76814 .50944 L
.76814 .50944 L
.76814 .50944 L
.76815 .50944 L
.76815 .50944 L
.76815 .50944 L
.76815 .50944 L
.76816 .50944 L
.76816 .50944 L
.76817 .50944 L
.76817 .50944 L
.76817 .50944 L
.76818 .50944 L
.76818 .50945 L
.76818 .50945 L
.76819 .50945 L
.76819 .50945 L
.76819 .50945 L
.7682 .50945 L
.7682 .50945 L
.7682 .50945 L
.76821 .50945 L
.76821 .50945 L
.76821 .50945 L
.76822 .50945 L
.76822 .50945 L
.76822 .50945 L
.76822 .50945 L
.76823 .50945 L
.76823 .50945 L
.76823 .50945 L
.76824 .50946 L
.76824 .50946 L
.76824 .50946 L
.76825 .50946 L
.76825 .50946 L
.76825 .50946 L
.76826 .50946 L
.76826 .50946 L
Mistroke
.76826 .50946 L
.76827 .50946 L
.76827 .50946 L
.76827 .50946 L
.76828 .50946 L
.76828 .50946 L
.76828 .50946 L
.76829 .50946 L
.76829 .50946 L
.76829 .50946 L
.7683 .50947 L
.7683 .50947 L
.7683 .50947 L
.76831 .50947 L
.76831 .50947 L
.76831 .50947 L
.76832 .50947 L
.76832 .50947 L
.76832 .50947 L
.76833 .50947 L
.76833 .50947 L
.76833 .50947 L
.76834 .50947 L
.76834 .50947 L
.76834 .50947 L
.76834 .50947 L
.76835 .50947 L
.76835 .50947 L
.76835 .50947 L
.76836 .50947 L
.76836 .50948 L
.76836 .50948 L
.76837 .50948 L
.76837 .50948 L
.76837 .50948 L
.76838 .50948 L
.76838 .50948 L
.76838 .50948 L
.76838 .50948 L
.76839 .50948 L
.76839 .50948 L
.76839 .50948 L
.7684 .50948 L
.7684 .50948 L
.7684 .50948 L
.7684 .50948 L
.76841 .50948 L
.76841 .50948 L
.76841 .50948 L
.76842 .50948 L
Mistroke
.76842 .50948 L
.76842 .50948 L
.76843 .50948 L
.76843 .50949 L
.76843 .50949 L
.76844 .50949 L
.76844 .50949 L
.76844 .50949 L
.76845 .50949 L
.76845 .50949 L
.76845 .50949 L
.76846 .50949 L
.76846 .50949 L
.76846 .50949 L
.76847 .50949 L
.76847 .50949 L
.76847 .50949 L
.76848 .50949 L
.76848 .50949 L
.76848 .50949 L
.76849 .50949 L
.76849 .50949 L
.76849 .50949 L
.7685 .50949 L
.7685 .50949 L
.7685 .5095 L
.7685 .5095 L
.76851 .5095 L
.76851 .5095 L
.76852 .5095 L
.76852 .5095 L
.76852 .5095 L
.76853 .5095 L
.76853 .5095 L
.76853 .5095 L
.76854 .5095 L
.76854 .5095 L
.76854 .5095 L
.76855 .5095 L
.76855 .5095 L
.76856 .5095 L
.76856 .5095 L
.76856 .5095 L
.76857 .5095 L
.76857 .5095 L
.76857 .5095 L
.76858 .5095 L
.76858 .50951 L
.76859 .50951 L
.76859 .50951 L
Mistroke
.76859 .50951 L
.7686 .50951 L
.7686 .50951 L
.7686 .50951 L
.76861 .50951 L
.76861 .50951 L
.76861 .50951 L
.76861 .50951 L
.76862 .50951 L
.76862 .50951 L
.76863 .50951 L
.76863 .50951 L
.76863 .50951 L
.76864 .50951 L
.76864 .50951 L
.76864 .50951 L
.76865 .50951 L
.76865 .50951 L
.76866 .50951 L
.76866 .50951 L
.76866 .50951 L
.76867 .50951 L
.76867 .50952 L
.76868 .50952 L
.76868 .50952 L
.76868 .50952 L
.76869 .50952 L
.76869 .50952 L
.76869 .50952 L
.7687 .50952 L
.7687 .50952 L
.76871 .50952 L
.76871 .50952 L
.76871 .50952 L
.76872 .50952 L
.76872 .50952 L
.76872 .50952 L
.76873 .50952 L
.76873 .50952 L
.76874 .50952 L
.76874 .50952 L
.76874 .50952 L
.76875 .50952 L
.76875 .50952 L
.76875 .50952 L
.76876 .50952 L
.76876 .50952 L
.76876 .50952 L
.76877 .50952 L
.76877 .50952 L
Mistroke
.76877 .50953 L
.76878 .50953 L
.76878 .50953 L
.76878 .50953 L
.76879 .50953 L
.76879 .50953 L
.7688 .50953 L
.7688 .50953 L
.7688 .50953 L
.76881 .50953 L
.76881 .50953 L
.76881 .50953 L
.76882 .50953 L
.76882 .50953 L
.76882 .50953 L
.76883 .50953 L
.76883 .50953 L
.76883 .50953 L
.76884 .50953 L
.76884 .50953 L
.76884 .50953 L
.76884 .50953 L
.76885 .50953 L
.76885 .50953 L
.76885 .50953 L
.76886 .50953 L
.76886 .50953 L
.76886 .50953 L
.76887 .50953 L
.76887 .50953 L
.76888 .50953 L
.76888 .50953 L
.76888 .50953 L
.76889 .50953 L
.76889 .50953 L
.76889 .50953 L
.76889 .50953 L
.7689 .50954 L
.7689 .50954 L
.7689 .50954 L
.76891 .50954 L
.76891 .50954 L
.76891 .50954 L
.76892 .50954 L
.76892 .50954 L
.76892 .50954 L
.76893 .50954 L
.76893 .50954 L
.76894 .50954 L
.76894 .50954 L
Mistroke
.76894 .50954 L
.76895 .50954 L
.76895 .50954 L
.76895 .50954 L
.76896 .50954 L
.76896 .50954 L
.76896 .50954 L
.76897 .50954 L
.76897 .50954 L
.76898 .50954 L
.76898 .50954 L
.76898 .50954 L
.76898 .50954 L
.76899 .50954 L
.76899 .50954 L
.76899 .50954 L
.769 .50954 L
.769 .50954 L
.769 .50954 L
.76901 .50954 L
.76901 .50954 L
.76901 .50954 L
.76902 .50954 L
.76902 .50954 L
.76902 .50954 L
.76903 .50954 L
.76903 .50954 L
.76903 .50954 L
.76903 .50954 L
.76904 .50954 L
.76904 .50954 L
.76904 .50954 L
.76905 .50954 L
.76905 .50954 L
.76906 .50954 L
.76906 .50954 L
.76906 .50954 L
.76907 .50954 L
.76907 .50954 L
.76907 .50955 L
.76907 .50955 L
.76908 .50955 L
.76908 .50955 L
.76908 .50955 L
.76909 .50955 L
.76909 .50955 L
.76909 .50955 L
.76909 .50955 L
.7691 .50955 L
.7691 .50955 L
Mistroke
.76911 .50955 L
.76911 .50955 L
.76911 .50955 L
.76911 .50955 L
.76912 .50955 L
.76912 .50955 L
.76912 .50955 L
.76913 .50955 L
.76913 .50955 L
.76913 .50955 L
.76914 .50955 L
.76914 .50955 L
.76914 .50955 L
.76914 .50955 L
.76915 .50955 L
.76915 .50955 L
.76915 .50955 L
.76916 .50955 L
.76916 .50955 L
.76916 .50955 L
.76917 .50955 L
.76917 .50955 L
.76917 .50955 L
.76918 .50955 L
.76918 .50955 L
.76918 .50955 L
.76919 .50955 L
.76919 .50955 L
.76919 .50955 L
.7692 .50955 L
.7692 .50955 L
.7692 .50955 L
.76921 .50955 L
.76921 .50955 L
.76921 .50955 L
.76922 .50955 L
.76922 .50955 L
.76922 .50955 L
.76922 .50955 L
.76923 .50955 L
.76923 .50955 L
.76923 .50955 L
.76924 .50955 L
.76924 .50955 L
.76924 .50955 L
.76925 .50955 L
.76925 .50955 L
.76925 .50955 L
.76926 .50955 L
.76926 .50955 L
Mistroke
.76926 .50955 L
.76926 .50955 L
.76927 .50955 L
.76927 .50955 L
.76927 .50955 L
.76928 .50955 L
.76928 .50955 L
.76928 .50955 L
.76929 .50955 L
.76929 .50955 L
.76929 .50955 L
.7693 .50955 L
.7693 .50955 L
.7693 .50955 L
.76931 .50955 L
.76931 .50955 L
.76931 .50955 L
.76932 .50955 L
.76932 .50955 L
.76932 .50955 L
.76933 .50955 L
.76933 .50955 L
.76933 .50955 L
.76933 .50955 L
.76934 .50955 L
.76934 .50955 L
.76934 .50955 L
.76935 .50955 L
.76935 .50955 L
.76935 .50955 L
.76935 .50955 L
.76936 .50955 L
.76936 .50955 L
.76936 .50955 L
.76937 .50955 L
.76937 .50955 L
.76937 .50955 L
.76938 .50955 L
.76938 .50955 L
.76938 .50955 L
.76939 .50955 L
.76939 .50955 L
.76939 .50955 L
.7694 .50955 L
.7694 .50955 L
.7694 .50955 L
.76941 .50955 L
.76941 .50955 L
.76941 .50955 L
.76942 .50955 L
Mistroke
.76942 .50955 L
.76942 .50955 L
.76943 .50955 L
.76943 .50955 L
.76943 .50955 L
.76944 .50955 L
.76944 .50955 L
.76944 .50955 L
.76945 .50955 L
.76945 .50955 L
.76946 .50955 L
.76946 .50955 L
.76946 .50955 L
.76946 .50955 L
.76947 .50955 L
.76947 .50955 L
.76947 .50955 L
.76948 .50955 L
.76948 .50955 L
.76948 .50955 L
.76949 .50955 L
.76949 .50955 L
.76949 .50955 L
.76949 .50955 L
.7695 .50955 L
.7695 .50955 L
.7695 .50955 L
.76951 .50955 L
.76951 .50955 L
.76951 .50955 L
.76952 .50955 L
.76952 .50955 L
.76952 .50955 L
.76952 .50955 L
.76953 .50955 L
.76953 .50955 L
.76953 .50955 L
.76954 .50955 L
.76954 .50955 L
.76954 .50955 L
.76954 .50955 L
.76955 .50955 L
.76955 .50955 L
.76956 .50955 L
.76956 .50955 L
.76956 .50955 L
.76956 .50955 L
.76957 .50955 L
.76957 .50955 L
.76957 .50955 L
Mistroke
.76958 .50955 L
.76958 .50955 L
.76958 .50955 L
.76959 .50955 L
.76959 .50955 L
.76959 .50955 L
.7696 .50955 L
.7696 .50955 L
.7696 .50955 L
.7696 .50955 L
.76961 .50955 L
.76961 .50955 L
.76961 .50955 L
.76962 .50955 L
.76962 .50955 L
.76962 .50955 L
.76963 .50955 L
.76963 .50955 L
.76963 .50954 L
.76963 .50954 L
.76964 .50954 L
.76964 .50954 L
.76964 .50954 L
.76965 .50954 L
.76965 .50954 L
.76965 .50954 L
.76966 .50954 L
.76966 .50954 L
.76967 .50954 L
.76967 .50954 L
.76967 .50954 L
.76968 .50954 L
.76968 .50954 L
.76968 .50954 L
.76969 .50954 L
.76969 .50954 L
.76969 .50954 L
.7697 .50954 L
.7697 .50954 L
.7697 .50954 L
.76971 .50954 L
.76971 .50954 L
.76971 .50954 L
.76972 .50954 L
.76972 .50954 L
.76972 .50954 L
.76973 .50954 L
.76973 .50954 L
.76973 .50954 L
.76974 .50954 L
Mistroke
.76974 .50954 L
.76974 .50954 L
.76975 .50954 L
.76975 .50954 L
.76975 .50954 L
.76976 .50954 L
.76976 .50954 L
.76976 .50954 L
.76977 .50954 L
.76977 .50954 L
.76977 .50954 L
.76978 .50954 L
.76978 .50954 L
.76979 .50954 L
.76979 .50954 L
.76979 .50954 L
.7698 .50954 L
.7698 .50953 L
.7698 .50953 L
.76981 .50953 L
.76981 .50953 L
.76982 .50953 L
.76982 .50953 L
.76982 .50953 L
.76983 .50953 L
.76983 .50953 L
.76983 .50953 L
.76984 .50953 L
.76984 .50953 L
.76985 .50953 L
.76985 .50953 L
.76985 .50953 L
.76986 .50953 L
.76986 .50953 L
.76986 .50953 L
.76987 .50953 L
.76987 .50953 L
.76987 .50953 L
.76988 .50953 L
.76988 .50953 L
.76988 .50953 L
.76989 .50953 L
.76989 .50953 L
.76989 .50953 L
.7699 .50953 L
.7699 .50953 L
.7699 .50953 L
.76991 .50953 L
.76991 .50953 L
.76991 .50953 L
Mistroke
.76992 .50953 L
.76992 .50953 L
.76992 .50952 L
.76993 .50952 L
.76993 .50952 L
.76993 .50952 L
.76994 .50952 L
.76994 .50952 L
.76994 .50952 L
.76995 .50952 L
.76995 .50952 L
.76995 .50952 L
.76996 .50952 L
.76996 .50952 L
.76996 .50952 L
.76996 .50952 L
.76997 .50952 L
.76997 .50952 L
.76997 .50952 L
.76998 .50952 L
.76998 .50952 L
.76998 .50952 L
.76999 .50952 L
.76999 .50952 L
.76999 .50952 L
.77 .50952 L
.77 .50952 L
.77 .50952 L
.77001 .50952 L
.77001 .50952 L
.77001 .50952 L
.77002 .50952 L
.77002 .50952 L
.77002 .50951 L
.77003 .50951 L
.77003 .50951 L
.77004 .50951 L
.77004 .50951 L
.77004 .50951 L
.77005 .50951 L
.77005 .50951 L
.77005 .50951 L
.77006 .50951 L
.77006 .50951 L
.77006 .50951 L
.77007 .50951 L
.77007 .50951 L
.77007 .50951 L
.77008 .50951 L
.77008 .50951 L
Mistroke
.77008 .50951 L
.77009 .50951 L
.77009 .50951 L
.77009 .50951 L
.7701 .50951 L
.7701 .50951 L
.7701 .50951 L
.77011 .50951 L
.77011 .5095 L
.77011 .5095 L
.77012 .5095 L
.77012 .5095 L
.77012 .5095 L
.77013 .5095 L
.77013 .5095 L
.77013 .5095 L
.77014 .5095 L
.77014 .5095 L
.77014 .5095 L
.77015 .5095 L
.77015 .5095 L
.77015 .5095 L
.77016 .5095 L
.77016 .5095 L
.77016 .5095 L
.77017 .5095 L
.77017 .5095 L
.77017 .5095 L
.77018 .5095 L
.77018 .5095 L
.77018 .5095 L
.77019 .50949 L
.77019 .50949 L
.77019 .50949 L
.7702 .50949 L
.7702 .50949 L
.7702 .50949 L
.7702 .50949 L
.77021 .50949 L
.77021 .50949 L
.77021 .50949 L
.77022 .50949 L
.77022 .50949 L
.77023 .50949 L
.77023 .50949 L
.77023 .50949 L
.77024 .50949 L
.77024 .50949 L
.77024 .50949 L
.77025 .50949 L
Mistroke
.77025 .50949 L
.77025 .50949 L
.77025 .50948 L
.77026 .50948 L
.77026 .50948 L
.77027 .50948 L
.77027 .50948 L
.77027 .50948 L
.77028 .50948 L
.77028 .50948 L
.77029 .50948 L
.77029 .50948 L
.77029 .50948 L
.7703 .50948 L
.7703 .50948 L
.7703 .50948 L
.77031 .50948 L
.77031 .50948 L
.77031 .50948 L
.77032 .50948 L
.77032 .50947 L
.77032 .50947 L
.77033 .50947 L
.77033 .50947 L
.77033 .50947 L
.77034 .50947 L
.77034 .50947 L
.77034 .50947 L
.77034 .50947 L
.77035 .50947 L
.77035 .50947 L
.77035 .50947 L
.77036 .50947 L
.77036 .50947 L
.77036 .50947 L
.77037 .50947 L
.77037 .50947 L
.77037 .50947 L
.77038 .50947 L
.77038 .50947 L
.77038 .50946 L
.77039 .50946 L
.77039 .50946 L
.77039 .50946 L
.77039 .50946 L
.7704 .50946 L
.7704 .50946 L
.7704 .50946 L
.77041 .50946 L
.77041 .50946 L
Mistroke
.77041 .50946 L
.77041 .50946 L
.77042 .50946 L
.77042 .50946 L
.77042 .50946 L
.77043 .50946 L
.77043 .50946 L
.77044 .50946 L
.77044 .50945 L
.77044 .50945 L
.77045 .50945 L
.77045 .50945 L
.77045 .50945 L
.77046 .50945 L
.77046 .50945 L
.77046 .50945 L
.77047 .50945 L
.77047 .50945 L
.77047 .50945 L
.77048 .50945 L
.77048 .50945 L
.77048 .50945 L
.77049 .50945 L
.77049 .50945 L
.77049 .50944 L
.7705 .50944 L
.7705 .50944 L
.7705 .50944 L
.77051 .50944 L
.77051 .50944 L
.77051 .50944 L
.77051 .50944 L
.77052 .50944 L
.77052 .50944 L
.77052 .50944 L
.77053 .50944 L
.77053 .50944 L
.77054 .50944 L
.77054 .50944 L
.77054 .50944 L
.77055 .50943 L
.77055 .50943 L
.77055 .50943 L
.77056 .50943 L
.77056 .50943 L
.77056 .50943 L
.77057 .50943 L
.77057 .50943 L
.77057 .50943 L
.77058 .50943 L
Mistroke
.77058 .50943 L
.77058 .50943 L
.77059 .50943 L
.77059 .50943 L
.77059 .50943 L
.7706 .50942 L
.7706 .50942 L
.7706 .50942 L
.77061 .50942 L
.77061 .50942 L
.77062 .50942 L
.77062 .50942 L
.77062 .50942 L
.77063 .50942 L
.77063 .50942 L
.77063 .50942 L
.77064 .50942 L
.77064 .50941 L
.77064 .50941 L
.77065 .50941 L
.77065 .50941 L
.77065 .50941 L
.77066 .50941 L
.77066 .50941 L
.77066 .50941 L
.77067 .50941 L
.77067 .50941 L
.77067 .50941 L
.77068 .50941 L
.77068 .50941 L
.77069 .50941 L
.77069 .5094 L
.77069 .5094 L
.77069 .5094 L
.7707 .5094 L
.7707 .5094 L
.7707 .5094 L
.77071 .5094 L
.77071 .5094 L
.77071 .5094 L
.77072 .5094 L
.77072 .5094 L
.77072 .5094 L
.77073 .5094 L
.77073 .5094 L
.77073 .50939 L
.77074 .50939 L
.77074 .50939 L
.77075 .50939 L
.77075 .50939 L
Mistroke
.77075 .50939 L
.77076 .50939 L
.77076 .50939 L
.77076 .50939 L
.77077 .50939 L
.77077 .50939 L
.77078 .50938 L
.77078 .50938 L
.77078 .50938 L
.77079 .50938 L
.77079 .50938 L
.77079 .50938 L
.7708 .50938 L
.7708 .50938 L
.77081 .50938 L
.77081 .50938 L
.77081 .50938 L
.77082 .50937 L
.77082 .50937 L
.77082 .50937 L
.77083 .50937 L
.77083 .50937 L
.77084 .50937 L
.77084 .50937 L
.77084 .50937 L
.77085 .50937 L
.77085 .50937 L
.77086 .50936 L
.77086 .50936 L
.77087 .50936 L
.77087 .50936 L
.77087 .50936 L
.77088 .50936 L
.77088 .50936 L
.77088 .50936 L
.77089 .50936 L
.77089 .50936 L
.77089 .50936 L
.7709 .50935 L
.7709 .50935 L
.7709 .50935 L
.77091 .50935 L
.77091 .50935 L
.77091 .50935 L
.77092 .50935 L
.77092 .50935 L
.77093 .50935 L
.77093 .50935 L
.77093 .50935 L
.77094 .50934 L
Mistroke
.77094 .50934 L
.77094 .50934 L
.77095 .50934 L
.77095 .50934 L
.77095 .50934 L
.77096 .50934 L
.77096 .50934 L
.77097 .50934 L
.77097 .50934 L
.77097 .50933 L
.77098 .50933 L
.77098 .50933 L
.77098 .50933 L
.77099 .50933 L
.77099 .50933 L
.77099 .50933 L
.771 .50933 L
.771 .50933 L
.771 .50933 L
.771 .50933 L
.77101 .50932 L
.77101 .50932 L
.77101 .50932 L
.77102 .50932 L
.77102 .50932 L
.77102 .50932 L
.77102 .50932 L
.77103 .50932 L
.77103 .50932 L
.77103 .50932 L
.77104 .50932 L
.77104 .50932 L
.77104 .50931 L
.77105 .50931 L
.77105 .50931 L
.77105 .50931 L
.77106 .50931 L
.77106 .50931 L
.77106 .50931 L
.77107 .50931 L
.77107 .50931 L
.77107 .50931 L
.77108 .50931 L
.77108 .5093 L
.77108 .5093 L
.77109 .5093 L
.77109 .5093 L
.77109 .5093 L
.7711 .5093 L
.7711 .5093 L
Mistroke
.7711 .5093 L
.77111 .5093 L
.77111 .5093 L
.77111 .5093 L
.77111 .50929 L
.77112 .50929 L
.77112 .50929 L
.77112 .50929 L
.77113 .50929 L
.77113 .50929 L
.77113 .50929 L
.77114 .50929 L
.77114 .50929 L
.77114 .50929 L
.77115 .50928 L
.77115 .50928 L
.77115 .50928 L
.77116 .50928 L
.77116 .50928 L
.77116 .50928 L
.77117 .50928 L
.77117 .50928 L
.77117 .50928 L
.77118 .50928 L
.77118 .50927 L
.77118 .50927 L
.77119 .50927 L
.77119 .50927 L
.77119 .50927 L
.77119 .50927 L
.7712 .50927 L
.7712 .50927 L
.7712 .50927 L
.77121 .50927 L
.77121 .50927 L
.77121 .50927 L
.77121 .50926 L
.77122 .50926 L
.77122 .50926 L
.77122 .50926 L
.77123 .50926 L
.77123 .50926 L
.77123 .50926 L
.77124 .50926 L
.77124 .50926 L
.77124 .50926 L
.77124 .50925 L
.77125 .50925 L
.77125 .50925 L
.77125 .50925 L
Mistroke
.77125 .50925 L
.77126 .50925 L
.77126 .50925 L
.77126 .50925 L
.77127 .50925 L
.77127 .50925 L
.77127 .50924 L
.77128 .50924 L
.77128 .50924 L
.77128 .50924 L
.77129 .50924 L
.77129 .50924 L
.77129 .50924 L
.77129 .50924 L
.7713 .50924 L
.7713 .50924 L
.7713 .50924 L
.77131 .50923 L
.77131 .50923 L
.77131 .50923 L
.77132 .50923 L
.77132 .50923 L
.77132 .50923 L
.77133 .50923 L
.77133 .50923 L
.77133 .50923 L
.77133 .50923 L
.77134 .50922 L
.77134 .50922 L
.77134 .50922 L
.77135 .50922 L
.77135 .50922 L
.77135 .50922 L
.77135 .50922 L
.77136 .50922 L
.77136 .50922 L
.77136 .50922 L
.77137 .50921 L
.77137 .50921 L
.77137 .50921 L
.77138 .50921 L
.77138 .50921 L
.77138 .50921 L
.77139 .50921 L
.77139 .50921 L
.77139 .5092 L
.7714 .5092 L
.7714 .5092 L
.7714 .5092 L
.77141 .5092 L
Mistroke
.77141 .5092 L
.77142 .5092 L
.77142 .5092 L
.77142 .5092 L
.77142 .50919 L
.77143 .50919 L
.77143 .50919 L
.77143 .50919 L
.77144 .50919 L
.77144 .50919 L
.77144 .50919 L
.77145 .50919 L
.77145 .50918 L
.77145 .50918 L
.77146 .50918 L
.77146 .50918 L
.77146 .50918 L
.77147 .50918 L
.77147 .50918 L
.77147 .50918 L
.77147 .50918 L
.77148 .50917 L
.77148 .50917 L
.77148 .50917 L
.77149 .50917 L
.77149 .50917 L
.7715 .50917 L
.7715 .50917 L
.7715 .50917 L
.77151 .50916 L
.77151 .50916 L
.77151 .50916 L
.77152 .50916 L
.77152 .50916 L
.77152 .50916 L
.77152 .50916 L
.77153 .50916 L
.77153 .50916 L
.77154 .50915 L
.77154 .50915 L
.77154 .50915 L
.77155 .50915 L
.77155 .50915 L
.77155 .50915 L
.77156 .50915 L
.77156 .50914 L
.77156 .50914 L
.77157 .50914 L
.77157 .50914 L
.77158 .50914 L
Mistroke
.77158 .50914 L
.77158 .50914 L
.77159 .50913 L
.77159 .50913 L
.77159 .50913 L
.7716 .50913 L
.7716 .50913 L
.7716 .50913 L
.77161 .50913 L
.77161 .50913 L
.77161 .50912 L
.77161 .50912 L
.77162 .50912 L
.77162 .50912 L
.77162 .50912 L
.77163 .50912 L
.77163 .50912 L
.77163 .50912 L
.77164 .50912 L
.77164 .50911 L
.77164 .50911 L
.77165 .50911 L
.77165 .50911 L
.77165 .50911 L
.77165 .50911 L
.77166 .50911 L
.77166 .50911 L
.77166 .5091 L
.77167 .5091 L
.77167 .5091 L
.77167 .5091 L
.77167 .5091 L
.77168 .5091 L
.77168 .5091 L
.77168 .5091 L
.77169 .5091 L
.77169 .50909 L
.77169 .50909 L
.7717 .50909 L
.7717 .50909 L
.7717 .50909 L
.77171 .50909 L
.77171 .50909 L
.77171 .50909 L
.77171 .50908 L
.77172 .50908 L
.77172 .50908 L
.77172 .50908 L
.77173 .50908 L
.77173 .50908 L
Mistroke
.77173 .50908 L
.77174 .50908 L
.77174 .50907 L
.77174 .50907 L
.77175 .50907 L
.77175 .50907 L
.77175 .50907 L
.77176 .50907 L
.77176 .50907 L
.77177 .50906 L
.77177 .50906 L
.77177 .50906 L
.77177 .50906 L
.77178 .50906 L
.77178 .50906 L
.77178 .50906 L
.77179 .50905 L
.77179 .50905 L
.77179 .50905 L
.77179 .50905 L
.7718 .50905 L
.7718 .50905 L
.7718 .50905 L
.77181 .50905 L
.77181 .50905 L
.77181 .50904 L
.77181 .50904 L
.77182 .50904 L
.77182 .50904 L
.77183 .50904 L
.77183 .50904 L
.77183 .50904 L
.77183 .50903 L
.77184 .50903 L
.77184 .50903 L
.77184 .50903 L
.77185 .50903 L
.77185 .50903 L
.77185 .50903 L
.77185 .50903 L
.77186 .50902 L
.77186 .50902 L
.77186 .50902 L
.77187 .50902 L
.77187 .50902 L
.77187 .50902 L
.77188 .50902 L
.77188 .50902 L
.77188 .50901 L
.77189 .50901 L
Mistroke
.77189 .50901 L
.77189 .50901 L
.77189 .50901 L
.7719 .50901 L
.7719 .50901 L
.7719 .50901 L
.77191 .509 L
.77191 .509 L
.77191 .509 L
.77192 .509 L
.77192 .509 L
.77192 .509 L
.77193 .509 L
.77193 .50899 L
.77193 .50899 L
.77193 .50899 L
.77194 .50899 L
.77194 .50899 L
.77194 .50899 L
.77195 .50899 L
.77195 .50898 L
.77195 .50898 L
.77195 .50898 L
.77196 .50898 L
.77196 .50898 L
.77196 .50898 L
.77197 .50898 L
.77197 .50898 L
.77197 .50897 L
.77198 .50897 L
.77198 .50897 L
.77198 .50897 L
.77199 .50897 L
.77199 .50897 L
.77199 .50896 L
.772 .50896 L
.772 .50896 L
.772 .50896 L
.77201 .50896 L
.77201 .50896 L
.77201 .50896 L
.77202 .50895 L
.77202 .50895 L
.77202 .50895 L
.77203 .50895 L
.77203 .50895 L
.77203 .50895 L
.77204 .50894 L
.77204 .50894 L
.77204 .50894 L
Mistroke
.77205 .50894 L
.77205 .50894 L
.77205 .50894 L
.77206 .50894 L
.77206 .50893 L
.77206 .50893 L
.77207 .50893 L
.77207 .50893 L
.77207 .50893 L
.77208 .50893 L
.77208 .50892 L
.77208 .50892 L
.77209 .50892 L
.77209 .50892 L
.77209 .50892 L
.7721 .50892 L
.7721 .50892 L
.7721 .50891 L
.77211 .50891 L
.77211 .50891 L
.77211 .50891 L
.77212 .50891 L
.77212 .50891 L
.77212 .5089 L
.77213 .5089 L
.77213 .5089 L
.77214 .5089 L
.77214 .5089 L
.77214 .50889 L
.77215 .50889 L
.77215 .50889 L
.77215 .50889 L
.77216 .50889 L
.77216 .50889 L
.77216 .50889 L
.77217 .50888 L
.77217 .50888 L
.77217 .50888 L
.77218 .50888 L
.77218 .50888 L
.77218 .50887 L
.77219 .50887 L
.77219 .50887 L
.77219 .50887 L
.7722 .50887 L
.7722 .50887 L
.77221 .50886 L
.77221 .50886 L
.77221 .50886 L
.77221 .50886 L
Mistroke
.77222 .50886 L
.77222 .50886 L
.77223 .50885 L
.77223 .50885 L
.77223 .50885 L
.77224 .50885 L
.77224 .50885 L
.77224 .50885 L
.77225 .50884 L
.77225 .50884 L
.77225 .50884 L
.77226 .50884 L
.77226 .50884 L
.77226 .50884 L
.77227 .50883 L
.77227 .50883 L
.77227 .50883 L
.77228 .50883 L
.77228 .50883 L
.77228 .50883 L
.77229 .50882 L
.77229 .50882 L
.77229 .50882 L
.7723 .50882 L
.7723 .50882 L
.7723 .50882 L
.77231 .50881 L
.77231 .50881 L
.77232 .50881 L
.77232 .50881 L
.77232 .5088 L
.77233 .5088 L
.77233 .5088 L
.77234 .5088 L
.77234 .5088 L
.77234 .50879 L
.77235 .50879 L
.77235 .50879 L
.77236 .50879 L
.77236 .50879 L
.77236 .50878 L
.77237 .50878 L
.77237 .50878 L
.77237 .50878 L
.77238 .50878 L
.77238 .50877 L
.77239 .50877 L
.77239 .50877 L
.77239 .50877 L
.7724 .50877 L
Mistroke
.7724 .50876 L
.7724 .50876 L
.77241 .50876 L
.77241 .50876 L
.77242 .50876 L
.77242 .50875 L
.77242 .50875 L
.77243 .50875 L
.77243 .50875 L
.77243 .50875 L
.77244 .50874 L
.77244 .50874 L
.77245 .50874 L
.77245 .50874 L
.77245 .50874 L
.77246 .50873 L
.77246 .50873 L
.77246 .50873 L
.77247 .50873 L
.77247 .50873 L
.77247 .50873 L
.77248 .50872 L
.77248 .50872 L
.77248 .50872 L
.77249 .50872 L
.77249 .50872 L
.77249 .50871 L
.7725 .50871 L
.7725 .50871 L
.7725 .50871 L
.77251 .50871 L
.77251 .5087 L
.77252 .5087 L
.77252 .5087 L
.77253 .5087 L
.77253 .5087 L
.77253 .50869 L
.77254 .50869 L
.77254 .50869 L
.77254 .50869 L
.77255 .50868 L
.77255 .50868 L
.77256 .50868 L
.77256 .50868 L
.77256 .50868 L
.77257 .50867 L
.77257 .50867 L
.77258 .50867 L
.77258 .50867 L
.77258 .50866 L
Mistroke
.77259 .50866 L
.77259 .50866 L
.7726 .50866 L
.7726 .50865 L
.7726 .50865 L
.77261 .50865 L
.77262 .50865 L
.77262 .50864 L
.77262 .50864 L
.77263 .50864 L
.77263 .50864 L
.77263 .50864 L
.77264 .50863 L
.77264 .50863 L
.77265 .50863 L
.77265 .50863 L
.77265 .50862 L
.77266 .50862 L
.77266 .50862 L
.77267 .50862 L
.77267 .50861 L
.77268 .50861 L
.77268 .50861 L
.77268 .50861 L
.77269 .5086 L
.77269 .5086 L
.7727 .5086 L
.7727 .5086 L
.77271 .50859 L
.77271 .50859 L
.77272 .50859 L
.77272 .50859 L
.77273 .50858 L
.77273 .50858 L
.77273 .50858 L
.77274 .50857 L
.77274 .50857 L
.77275 .50857 L
.77275 .50857 L
.77275 .50857 L
.77276 .50856 L
.77276 .50856 L
.77277 .50856 L
.77277 .50856 L
.77278 .50855 L
.77278 .50855 L
.77278 .50855 L
.77279 .50854 L
.77279 .50854 L
.7728 .50854 L
Mistroke
.7728 .50854 L
.7728 .50854 L
.77281 .50853 L
.77281 .50853 L
.77281 .50853 L
.77282 .50853 L
.77282 .50852 L
.77283 .50852 L
.77283 .50852 L
.77284 .50852 L
.77284 .50851 L
.77284 .50851 L
.77285 .50851 L
.77285 .50851 L
.77285 .5085 L
.77286 .5085 L
.77286 .5085 L
.77286 .5085 L
.77287 .5085 L
.77287 .50849 L
.77288 .50849 L
.77288 .50849 L
.77288 .50849 L
.77289 .50849 L
.77289 .50848 L
.7729 .50848 L
.7729 .50848 L
.7729 .50847 L
.77291 .50847 L
.77291 .50847 L
.77291 .50847 L
.77292 .50847 L
.77292 .50846 L
.77292 .50846 L
.77293 .50846 L
.77293 .50846 L
.77294 .50845 L
.77294 .50845 L
.77294 .50845 L
.77295 .50845 L
.77295 .50844 L
.77296 .50844 L
.77296 .50844 L
.77297 .50844 L
.77297 .50843 L
.77297 .50843 L
.77298 .50843 L
.77298 .50842 L
.77299 .50842 L
.77299 .50842 L
Mistroke
.773 .50842 L
.773 .50841 L
.773 .50841 L
.77301 .50841 L
.77301 .5084 L
.77302 .5084 L
.77302 .5084 L
.77302 .5084 L
.77303 .50839 L
.77304 .50839 L
.77304 .50839 L
.77304 .50838 L
.77305 .50838 L
.77305 .50838 L
.77306 .50838 L
.77306 .50837 L
.77307 .50837 L
.77307 .50837 L
.77307 .50837 L
.77308 .50836 L
.77308 .50836 L
.77308 .50836 L
.77309 .50836 L
.77309 .50835 L
.77309 .50835 L
.7731 .50835 L
.7731 .50835 L
.77311 .50834 L
.77312 .50834 L
.77312 .50834 L
.77312 .50833 L
.77313 .50833 L
.77313 .50833 L
.77313 .50833 L
.77314 .50832 L
.77314 .50832 L
.77314 .50832 L
.77315 .50832 L
.77315 .50831 L
.77316 .50831 L
.77316 .50831 L
.77316 .50831 L
.77317 .5083 L
.77317 .5083 L
.77318 .5083 L
.77318 .50829 L
.77318 .50829 L
.77319 .50829 L
.7732 .50828 L
.7732 .50828 L
Mistroke
.7732 .50828 L
.77321 .50828 L
.77321 .50827 L
.77321 .50827 L
.77322 .50827 L
.77322 .50826 L
.77323 .50826 L
.77323 .50826 L
.77324 .50826 L
.77324 .50825 L
.77325 .50825 L
.77325 .50825 L
.77326 .50824 L
.77326 .50824 L
.77326 .50824 L
.77327 .50823 L
.77328 .50823 L
.77328 .50823 L
.77328 .50822 L
.77329 .50822 L
.77329 .50822 L
.7733 .50821 L
.7733 .50821 L
.77331 .50821 L
.77331 .5082 L
.77331 .5082 L
.77332 .5082 L
.77333 .50819 L
.77333 .50819 L
.77333 .50819 L
.77334 .50819 L
.77334 .50818 L
.77335 .50818 L
.77335 .50818 L
.77335 .50817 L
.77336 .50817 L
.77336 .50817 L
.77337 .50816 L
.77337 .50816 L
.77337 .50816 L
.77338 .50816 L
.77338 .50815 L
.77339 .50815 L
.77339 .50815 L
.7734 .50814 L
.7734 .50814 L
.77341 .50813 L
.77341 .50813 L
.77342 .50813 L
.77342 .50812 L
Mistroke
.77343 .50812 L
.77344 .50811 L
.77344 .50811 L
.77345 .50811 L
.77345 .5081 L
.77346 .5081 L
.77347 .50809 L
.77347 .50809 L
.77348 .50808 L
.77348 .50808 L
.77349 .50808 L
.77349 .50807 L
.7735 .50807 L
.7735 .50807 L
.77351 .50806 L
.77352 .50806 L
.77352 .50805 L
.77353 .50805 L
.77353 .50804 L
.77354 .50804 L
.77354 .50804 L
.77355 .50803 L
.77355 .50803 L
.77356 .50802 L
.77357 .50802 L
.77357 .50802 L
.77358 .50801 L
.77358 .50801 L
.77359 .508 L
.7736 .508 L
.7736 .50799 L
.77361 .50799 L
.77361 .50798 L
.77362 .50798 L
.77363 .50797 L
.77363 .50797 L
.77364 .50797 L
.77364 .50796 L
.77365 .50796 L
.77366 .50795 L
.77366 .50795 L
.77367 .50794 L
.77367 .50794 L
.77368 .50793 L
.77369 .50793 L
.77369 .50792 L
.7737 .50792 L
.7737 .50792 L
.77371 .50791 L
.77372 .5079 L
Mistroke
.77372 .5079 L
.77373 .5079 L
.77373 .50789 L
.77374 .50789 L
.77374 .50788 L
.77375 .50788 L
.77376 .50787 L
.77376 .50787 L
.77377 .50786 L
.77378 .50786 L
.77378 .50785 L
.77379 .50785 L
.7738 .50784 L
.7738 .50784 L
.77381 .50783 L
.77382 .50783 L
.77382 .50782 L
.77383 .50782 L
.77383 .50781 L
.77384 .50781 L
.77385 .5078 L
.77385 .5078 L
.77386 .50779 L
.77387 .50779 L
.77387 .50778 L
.77388 .50777 L
.77389 .50777 L
.77389 .50776 L
.7739 .50776 L
.7739 .50776 L
.77391 .50775 L
.77391 .50775 L
.77392 .50774 L
.77393 .50773 L
.77394 .50773 L
.77394 .50773 L
.77394 .50772 L
.77395 .50772 L
.77396 .50771 L
.77397 .5077 L
.77397 .5077 L
.77398 .50769 L
.77399 .50769 L
.77399 .50768 L
.774 .50768 L
.77401 .50767 L
.77401 .50767 L
.77402 .50766 L
.77403 .50765 L
.77403 .50765 L
Mistroke
.77404 .50764 L
.77404 .50764 L
.77405 .50763 L
.77406 .50763 L
.77406 .50762 L
.77407 .50762 L
.77408 .50761 L
.77408 .50761 L
.77409 .5076 L
.7741 .5076 L
.7741 .50759 L
.77411 .50758 L
.77412 .50758 L
.77412 .50757 L
.77413 .50757 L
.77414 .50756 L
.77414 .50755 L
.77415 .50755 L
.77415 .50755 L
.77416 .50754 L
.77417 .50753 L
.77417 .50753 L
.77418 .50752 L
.77419 .50752 L
.77419 .50751 L
.7742 .50751 L
.7742 .5075 L
.77421 .5075 L
.77421 .50749 L
.77422 .50749 L
.77423 .50748 L
.77423 .50748 L
.77424 .50747 L
.77425 .50747 L
.77425 .50746 L
.77426 .50745 L
.77427 .50745 L
.77427 .50744 L
.77428 .50743 L
.77429 .50743 L
.77429 .50742 L
.7743 .50742 L
.77431 .50741 L
.77432 .5074 L
.77432 .50739 L
.77433 .50739 L
.77434 .50738 L
.77435 .50738 L
.77435 .50737 L
.77436 .50736 L
Mistroke
.77437 .50736 L
.77437 .50735 L
.77438 .50735 L
.77439 .50734 L
.77439 .50733 L
.7744 .50733 L
.77441 .50732 L
.77441 .50732 L
.77442 .50731 L
.77442 .5073 L
.77443 .5073 L
.77444 .50729 L
.77444 .50729 L
.77445 .50728 L
.77446 .50727 L
.77446 .50727 L
.77447 .50726 L
.77448 .50726 L
.77449 .50725 L
.77449 .50724 L
.7745 .50723 L
.77451 .50723 L
.77451 .50722 L
.77452 .50722 L
.77453 .50721 L
.77453 .5072 L
.77454 .5072 L
.77455 .50719 L
.77455 .50718 L
.77456 .50718 L
.77457 .50717 L
.77458 .50716 L
.77458 .50716 L
.77459 .50715 L
.77459 .50715 L
.7746 .50714 L
.77461 .50713 L
.77462 .50712 L
.77463 .50712 L
.77463 .50711 L
.77464 .5071 L
.77465 .5071 L
.77465 .50709 L
.77466 .50708 L
.77467 .50707 L
.77468 .50707 L
.77468 .50706 L
.77469 .50706 L
.7747 .50705 L
.7747 .50704 L
Mistroke
.77471 .50704 L
.77471 .50703 L
.77472 .50703 L
.77473 .50702 L
.77473 .50701 L
.77474 .507 L
.77475 .507 L
.77476 .50699 L
.77476 .50698 L
.77477 .50698 L
.77478 .50697 L
.77478 .50696 L
.77479 .50696 L
.7748 .50695 L
.77481 .50694 L
.77481 .50694 L
.77482 .50693 L
.77482 .50692 L
.77483 .50692 L
.77484 .50691 L
.77484 .5069 L
.77485 .5069 L
.77486 .50689 L
.77487 .50688 L
.77487 .50687 L
.77488 .50687 L
.77489 .50686 L
.7749 .50685 L
.7749 .50685 L
.77491 .50684 L
.77492 .50683 L
.77492 .50683 L
.77493 .50681 L
.77494 .50681 L
.77495 .5068 L
.77495 .50679 L
.77496 .50679 L
.77497 .50678 L
.77497 .50677 L
.77498 .50677 L
.77499 .50676 L
.77499 .50675 L
.775 .50675 L
.77501 .50674 L
.77501 .50673 L
.77502 .50673 L
.77502 .50672 L
.77503 .50671 L
.77504 .50671 L
.77504 .5067 L
Mistroke
.77505 .50669 L
.77506 .50668 L
.77507 .50668 L
.77508 .50667 L
.77508 .50666 L
.77509 .50666 L
.77509 .50665 L
.7751 .50664 L
.77511 .50664 L
.77511 .50663 L
.77512 .50662 L
.77513 .50661 L
.77514 .5066 L
.77514 .5066 L
.77515 .50659 L
.77516 .50658 L
.77517 .50657 L
.77518 .50656 L
.77518 .50656 L
.77519 .50655 L
.7752 .50654 L
.77521 .50653 L
.77521 .50652 L
.77522 .50652 L
.77523 .50651 L
.77524 .5065 L
.77525 .50649 L
.77525 .50648 L
.77526 .50647 L
.77527 .50646 L
.77528 .50645 L
.77529 .50644 L
.7753 .50643 L
.77531 .50642 L
.77531 .50641 L
.77532 .5064 L
.77533 .5064 L
.77534 .50639 L
.77535 .50638 L
.77535 .50637 L
.77536 .50636 L
.77537 .50635 L
.77538 .50634 L
.77538 .50634 L
.77539 .50633 L
.7754 .50632 L
.77541 .50631 L
.77542 .5063 L
.77543 .50629 L
.77543 .50628 L
Mistroke
.77544 .50627 L
.77545 .50627 L
.77546 .50626 L
.77547 .50625 L
.77547 .50624 L
.77548 .50623 L
.77549 .50622 L
.77549 .50621 L
.7755 .50621 L
.77551 .5062 L
.77552 .50619 L
.77553 .50618 L
.77553 .50617 L
.77554 .50616 L
.77555 .50615 L
.77555 .50615 L
.77556 .50614 L
.77557 .50613 L
.77558 .50612 L
.77559 .50611 L
.7756 .5061 L
.7756 .50609 L
.77561 .50608 L
.77562 .50607 L
.77563 .50606 L
.77564 .50605 L
.77564 .50605 L
.77565 .50604 L
.77566 .50603 L
.77566 .50602 L
.77567 .50601 L
.77568 .506 L
.77569 .50599 L
.7757 .50598 L
.77571 .50597 L
.77572 .50596 L
.77572 .50595 L
.77573 .50594 L
.77574 .50593 L
.77575 .50592 L
.77576 .50591 L
.77576 .50591 L
.77577 .5059 L
.77578 .50589 L
.77579 .50588 L
.77579 .50587 L
.7758 .50586 L
.77581 .50585 L
.77582 .50584 L
.77582 .50583 L
Mistroke
.77583 .50582 L
.77584 .50581 L
.77585 .50581 L
.77586 .50579 L
.77587 .50578 L
.77588 .50577 L
.77588 .50576 L
.77589 .50575 L
.7759 .50574 L
.77591 .50573 L
.77592 .50573 L
.77592 .50572 L
.77593 .50571 L
.77594 .5057 L
.77595 .50568 L
.77596 .50567 L
.77597 .50566 L
.77598 .50565 L
.77598 .50564 L
.77599 .50563 L
.776 .50562 L
.77601 .50562 L
.77601 .50561 L
.77602 .5056 L
.77604 .50558 L
.77604 .50557 L
.77605 .50556 L
.77606 .50555 L
.77606 .50554 L
.77608 .50553 L
.77608 .50552 L
.77609 .50551 L
.7761 .5055 L
.77611 .50549 L
.77611 .50548 L
.77612 .50547 L
.77613 .50546 L
.77614 .50545 L
.77615 .50544 L
.77615 .50543 L
.77616 .50542 L
.77617 .50541 L
.77618 .5054 L
.77619 .50539 L
.7762 .50538 L
.77621 .50537 L
.77621 .50536 L
.77622 .50535 L
.77622 .50534 L
.77624 .50533 L
Mistroke
.77625 .50531 L
.77625 .50531 L
.77626 .5053 L
.77627 .50528 L
.77628 .50527 L
.77629 .50526 L
.7763 .50524 L
.77631 .50523 L
.77632 .50523 L
.77633 .50521 L
.77634 .5052 L
.77634 .50519 L
.77635 .50518 L
.77636 .50517 L
.77637 .50516 L
.77638 .50515 L
.77638 .50514 L
.77639 .50513 L
.7764 .50512 L
.77641 .5051 L
.77642 .50509 L
.77643 .50508 L
.77644 .50507 L
.77644 .50506 L
.77646 .50505 L
.77646 .50504 L
.77647 .50503 L
.77648 .50502 L
.77649 .50501 L
.7765 .50499 L
.77651 .50498 L
.77652 .50497 L
.77652 .50496 L
.77653 .50494 L
.77654 .50493 L
.77655 .50493 L
.77656 .50491 L
.77657 .5049 L
.77657 .50489 L
.77659 .50487 L
.7766 .50486 L
.77661 .50484 L
.77662 .50483 L
.77663 .50481 L
.77664 .5048 L
.77665 .50479 L
.77666 .50478 L
.77667 .50476 L
.77668 .50475 L
.77669 .50473 L
Mistroke
.7767 .50472 L
.77671 .50471 L
.77672 .5047 L
.77673 .50469 L
.77673 .50468 L
.77674 .50466 L
.77676 .50464 L
.77676 .50464 L
.77677 .50463 L
.77678 .50461 L
.77679 .5046 L
.7768 .50459 L
.7768 .50458 L
.77681 .50457 L
.77682 .50455 L
.77683 .50454 L
.77684 .50453 L
.77685 .50451 L
.77686 .5045 L
.77687 .50449 L
.77688 .50448 L
.77689 .50446 L
.7769 .50445 L
.77691 .50444 L
.77692 .50442 L
.77693 .50441 L
.77693 .5044 L
.77694 .50439 L
.77695 .50438 L
.77696 .50436 L
.77697 .50434 L
.77699 .50433 L
.77699 .50431 L
.777 .50431 L
.77701 .5043 L
.77702 .50428 L
.77703 .50427 L
.77704 .50425 L
.77705 .50424 L
.77706 .50422 L
.77707 .50421 L
.77708 .5042 L
.77709 .50418 L
.7771 .50417 L
.77711 .50415 L
.77711 .50414 L
.77712 .50413 L
.77714 .50411 L
.77714 .5041 L
.77715 .50409 L
Mistroke
.77716 .50408 L
.77717 .50407 L
.77718 .50406 L
.77718 .50404 L
.77719 .50403 L
.7772 .50402 L
.77721 .504 L
.77722 .50399 L
.77723 .50398 L
.77724 .50396 L
.77725 .50395 L
.77726 .50394 L
.77726 .50393 L
.77727 .50391 L
.77729 .50389 L
.7773 .50388 L
.77731 .50386 L
.77731 .50385 L
.77732 .50384 L
.77734 .50382 L
.77734 .50381 L
.77735 .5038 L
.77736 .50379 L
.77737 .50378 L
.77738 .50375 L
.77739 .50373 L
.7774 .50372 L
.77741 .50371 L
.77742 .50369 L
.77744 .50367 L
.77745 .50365 L
.77746 .50363 L
.77748 .50361 L
.77749 .50359 L
.7775 .50357 L
.77752 .50355 L
.77753 .50354 L
.77754 .50352 L
.77755 .5035 L
.77757 .50348 L
.77758 .50346 L
.77759 .50344 L
.77761 .50342 L
.77762 .5034 L
.77763 .50338 L
.77765 .50336 L
.77765 .50334 L
.77766 .50333 L
.77768 .50331 L
.77769 .50329 L
Mistroke
.7777 .50328 L
.7777 .50327 L
.77771 .50325 L
.77772 .50324 L
.77774 .50322 L
.77774 .50321 L
.77775 .50319 L
.77776 .50317 L
.77778 .50315 L
.77779 .50313 L
.77781 .50311 L
.77782 .50309 L
.77783 .50307 L
.77785 .50304 L
.77786 .50302 L
.77787 .503 L
.77789 .50298 L
.7779 .50296 L
.77791 .50294 L
.77793 .50292 L
.77794 .5029 L
.77794 .50289 L
.77796 .50287 L
.77797 .50285 L
.77799 .50282 L
.778 .5028 L
.77801 .50278 L
.77803 .50276 L
.77804 .50274 L
.77804 .50273 L
.77806 .50271 L
.77807 .50269 L
.77808 .50268 L
.77809 .50266 L
.77811 .50263 L
.77812 .50261 L
.77813 .50258 L
.77814 .50257 L
.77815 .50256 L
.77817 .50253 L
.77818 .50251 L
.77819 .50249 L
.77821 .50246 L
.77822 .50244 L
.77823 .50242 L
.77825 .5024 L
.77826 .50238 L
.77828 .50235 L
.77829 .50233 L
.7783 .50231 L
Mistroke
.77831 .50229 L
.77833 .50227 L
.77834 .50225 L
.77835 .50222 L
.77837 .5022 L
.77838 .50218 L
.77839 .50216 L
.7784 .50214 L
.77842 .50212 L
.77843 .5021 L
.77844 .50208 L
.77845 .50206 L
.77846 .50204 L
.77848 .50202 L
.77849 .50199 L
.77851 .50197 L
.77852 .50194 L
.77854 .50192 L
.77855 .5019 L
.77856 .50187 L
.77858 .50185 L
.77859 .50183 L
.7786 .5018 L
.77862 .50178 L
.77863 .50175 L
.77865 .50173 L
.77866 .50171 L
.77867 .50169 L
.77868 .50167 L
.7787 .50164 L
.77871 .50162 L
.77873 .5016 L
.77874 .50157 L
.77875 .50155 L
.77877 .50152 L
.77878 .50149 L
.7788 .50147 L
.77881 .50144 L
.77883 .50142 L
.77884 .50139 L
.77886 .50137 L
.77887 .50134 L
.77889 .50131 L
.7789 .50128 L
.77892 .50126 L
.77893 .50123 L
.77895 .50121 L
.77896 .50118 L
.77897 .50116 L
.77899 .50113 L
Mistroke
.779 .50111 L
.77902 .50108 L
.77903 .50106 L
.77905 .50103 L
.77906 .50101 L
.77907 .50099 L
.77909 .50096 L
.7791 .50094 L
.77911 .50092 L
.77912 .50089 L
.77914 .50087 L
.77915 .50084 L
.77917 .50081 L
.77918 .50078 L
.7792 .50075 L
.77922 .50072 L
.77924 .50069 L
.77925 .50066 L
.77927 .50063 L
.77929 .5006 L
.7793 .50057 L
.77932 .50054 L
.77933 .50051 L
.77935 .50048 L
.77937 .50045 L
.77938 .50043 L
.77939 .5004 L
.77941 .50037 L
.77942 .50035 L
.77944 .50032 L
.77945 .50029 L
.77947 .50026 L
.77949 .50023 L
.7795 .5002 L
.77952 .50017 L
.77953 .50015 L
.77954 .50012 L
.77956 .50009 L
.77957 .50007 L
.77959 .50004 L
.7796 .50002 L
.77961 .49999 L
.77963 .49996 L
.77964 .49994 L
.77965 .49992 L
.77967 .49989 L
.77968 .49987 L
.7797 .49983 L
.77971 .4998 L
.77972 .49978 L
Mistroke
.77974 .49976 L
.77975 .49973 L
.77977 .4997 L
.77978 .49968 L
.7798 .49963 L
.77982 .4996 L
.77983 .49958 L
.77985 .49953 L
.77987 .49951 L
.77988 .49949 L
.77989 .49946 L
.7799 .49944 L
.77992 .4994 L
.77993 .49938 L
.77995 .49935 L
.77996 .49933 L
.77997 .4993 L
.77999 .49928 L
.78 .49925 L
.78002 .49921 L
.78003 .49919 L
.78005 .49916 L
.78006 .49914 L
.78007 .49912 L
.78008 .49909 L
.78009 .49907 L
.78011 .49903 L
.78012 .49901 L
.78013 .49899 L
.78015 .49896 L
.78016 .49894 L
.78018 .49889 L
.78019 .49887 L
.78021 .49885 L
.78022 .49882 L
.78023 .4988 L
.78025 .49876 L
.78027 .49872 L
.78029 .49868 L
.78031 .49864 L
.78032 .49862 L
.78033 .49859 L
.78035 .49855 L
.78037 .49853 L
.78038 .4985 L
.78039 .49848 L
.7804 .49845 L
.78042 .49841 L
.78044 .49838 L
.78045 .49836 L
Mistroke
.78046 .49833 L
.78047 .49831 L
.78049 .49827 L
.78051 .49824 L
.78052 .49822 L
.78054 .49818 L
.78055 .49815 L
.78056 .49813 L
.78059 .49808 L
.7806 .49805 L
.78061 .49802 L
.78062 .498 L
.78064 .49797 L
.78066 .49793 L
.78067 .4979 L
.78069 .49787 L
.7807 .49785 L
.78071 .49782 L
.78073 .49779 L
.78074 .49776 L
.78077 .49771 L
.78078 .49769 L
.78079 .49766 L
.7808 .49763 L
.78082 .4976 L
.78083 .49758 L
.78084 .49755 L
.78087 .4975 L
.78088 .49747 L
.78089 .49745 L
.78091 .49741 L
.78092 .49739 L
.78095 .49733 L
.78096 .4973 L
.78097 .49727 L
.781 .49722 L
.78101 .4972 L
.78102 .49717 L
.78105 .49712 L
.78107 .49707 L
.78108 .49705 L
.78109 .49702 L
.78112 .49697 L
.78113 .49694 L
.78114 .49692 L
.78117 .49686 L
.78119 .49681 L
.78121 .49677 L
.78123 .49673 L
.78125 .49668 L
Mistroke
.78127 .49664 L
.7813 .49658 L
.78132 .49654 L
.78134 .49649 L
.78136 .49645 L
.78138 .49641 L
.7814 .49636 L
.78142 .49632 L
.78144 .49628 L
.78146 .49624 L
.78148 .4962 L
.7815 .49615 L
.78152 .4961 L
.78154 .49605 L
.78157 .496 L
.78159 .49594 L
.78162 .49588 L
.78164 .49583 L
.78166 .49579 L
.78169 .49574 L
.78171 .49569 L
.78173 .49564 L
.78175 .49559 L
.78178 .49554 L
.7818 .49549 L
.78182 .49545 L
.78184 .4954 L
.78186 .49535 L
.78188 .49531 L
.7819 .49526 L
.78192 .49521 L
.78195 .49516 L
.78197 .4951 L
.78199 .49505 L
.78202 .495 L
.78204 .49495 L
.78206 .4949 L
.78208 .49485 L
.7821 .4948 L
.78212 .49475 L
.78214 .49471 L
.78216 .49466 L
.78218 .49462 L
.78221 .49457 L
.78223 .49451 L
.78225 .49446 L
.78227 .49441 L
.7823 .49435 L
.78232 .4943 L
.78234 .49426 L
Mistroke
.78236 .49421 L
.78238 .49415 L
.78241 .4941 L
.78243 .49404 L
.78246 .49399 L
.78248 .49394 L
.7825 .49388 L
.78253 .49382 L
.78256 .49376 L
.78258 .4937 L
.78261 .49364 L
.78263 .49358 L
.78266 .49352 L
.78268 .49346 L
.78271 .4934 L
.78273 .49335 L
.78276 .49329 L
.78278 .49323 L
.7828 .49317 L
.78283 .49311 L
.78286 .49304 L
.78288 .49299 L
.78291 .49292 L
.78294 .49285 L
.78297 .49278 L
.78299 .49272 L
.78302 .49267 L
.78304 .49261 L
.78306 .49256 L
.78308 .49251 L
.78311 .49245 L
.78313 .49238 L
.78316 .49232 L
.78318 .49227 L
.78321 .49221 L
.78323 .49216 L
.78325 .4921 L
.78328 .49203 L
.7833 .49197 L
.78333 .4919 L
.78335 .49185 L
.78338 .49178 L
.7834 .49173 L
.78343 .49166 L
.78345 .4916 L
.78348 .49155 L
.78352 .49145 L
.78354 .4914 L
.78356 .49134 L
.78359 .49128 L
Mistroke
.78361 .49122 L
.78364 .49115 L
.78366 .49109 L
.78371 .49098 L
.78373 .49092 L
.78375 .49086 L
.78378 .49081 L
.7838 .49075 L
.78382 .49069 L
.78384 .49064 L
.78386 .49059 L
.78388 .49053 L
.78392 .49044 L
.78395 .49038 L
.78397 .49033 L
.78399 .49027 L
.78402 .49021 L
.78404 .49015 L
.78406 .49009 L
.7841 .48999 L
.78413 .48993 L
.78415 .48987 L
.78419 .48975 L
.78422 .48969 L
.78424 .48963 L
.78427 .48956 L
.78429 .4895 L
.78431 .48944 L
.78434 .48938 L
.78438 .48928 L
.7844 .48922 L
.78443 .48915 L
.78447 .48904 L
.7845 .48897 L
.78452 .48891 L
.78455 .48885 L
.78457 .48878 L
.7846 .48872 L
.78462 .48866 L
.78467 .48854 L
.78469 .48847 L
.78472 .48841 L
.78474 .48834 L
.78477 .48827 L
.78482 .48813 L
.78485 .48806 L
.78488 .48798 L
.78491 .48791 L
.78493 .48785 L
.78498 .48772 L
Mistroke
.785 .48765 L
.78503 .48759 L
.78506 .48752 L
.78508 .48745 L
.78513 .48733 L
.78515 .48727 L
.78517 .4872 L
.7852 .48714 L
.78522 .48707 L
.78527 .48695 L
.7853 .48688 L
.78532 .48681 L
.78535 .48674 L
.78538 .48666 L
.7854 .48659 L
.78543 .48653 L
.78548 .4864 L
.7855 .48633 L
.78553 .48626 L
.78556 .48618 L
.78558 .48611 L
.78564 .48596 L
.78567 .48588 L
.7857 .4858 L
.78573 .4857 L
.78577 .48559 L
.78581 .48549 L
.78585 .4854 L
.78589 .48529 L
.78593 .48517 L
.78597 .48506 L
.78601 .48495 L
.78605 .48485 L
.78609 .48475 L
.78613 .48464 L
.78617 .48452 L
.78621 .48439 L
.78626 .48428 L
.78629 .48417 L
.78634 .48406 L
.78638 .48395 L
.78641 .48384 L
.78648 .48365 L
.78652 .48354 L
.78656 .48344 L
.7866 .48333 L
.78664 .48322 L
.78667 .48312 L
.78671 .48302 L
.78677 .48284 L
Mistroke
.78681 .48273 L
.78685 .48263 L
.78692 .48243 L
.78696 .48233 L
.78699 .48222 L
.78704 .4821 L
.78707 .48199 L
.78711 .48188 L
.78716 .48176 L
.7872 .48164 L
.78724 .48152 L
.78732 .4813 L
.78736 .48118 L
.7874 .48108 L
.78743 .48097 L
.78747 .48086 L
.78754 .48066 L
.78758 .48054 L
.78762 .48043 L
.78767 .4803 L
.78771 .48018 L
.7878 .47993 L
.78785 .47979 L
.78789 .47966 L
.78793 .47954 L
.78797 .47943 L
.78804 .47921 L
.78809 .47909 L
.78812 .47898 L
.78817 .47886 L
.78821 .47873 L
.78828 .47852 L
.78835 .47833 L
.78843 .4781 L
.7885 .4779 L
.78857 .47768 L
.78865 .47745 L
.78872 .47722 L
.78879 .47702 L
.78886 .47683 L
.78892 .47666 L
.78898 .47646 L
.78905 .47626 L
.78912 .47604 L
.78919 .47584 L
.78926 .47565 L
.78932 .47545 L
.7894 .47521 L
.78948 .47498 L
.78955 .47478 L
Mistroke
.78962 .47456 L
.7897 .47431 L
.78979 .47406 L
.78986 .47384 L
.78994 .4736 L
.79001 .47338 L
.79008 .47317 L
.79016 .47294 L
.79023 .47272 L
.79036 .47232 L
.79044 .47209 L
.79051 .47189 L
.79058 .47166 L
.79066 .47142 L
.7908 .47099 L
.79088 .47075 L
.79095 .47052 L
.79104 .47026 L
.79112 .47002 L
.79126 .46957 L
.79139 .46917 L
.79154 .46873 L
.79169 .46826 L
.79184 .46778 L
.79193 .46751 L
.79202 .46725 L
.79218 .46673 L
.79236 .46618 L
.79253 .46567 L
.79268 .46518 L
.79282 .46474 L
.79297 .46427 L
.79313 .46377 L
.79328 .4633 L
.79345 .46279 L
.79362 .46224 L
.79379 .46169 L
.79398 .46112 L
.7943 .46009 L
.79461 .45914 L
.79489 .45825 L
.79519 .45729 L
.79552 .45625 L
.79618 .45416 L
.79678 .45224 L
.79813 .44795 L
.79928 .44429 L
.79993 .44224 L
.80053 .44033 L
.80107 .43862 L
Mistroke
.80156 .43707 L
.80183 .43622 L
.80213 .4353 L
.80266 .43363 L
.80295 .43271 L
.80321 .43188 L
.80352 .43093 L
.8038 .43004 L
.80407 .42921 L
.80431 .42845 L
.80458 .42763 L
.80485 .42676 L
.8051 .426 L
.80536 .42519 L
.80561 .42442 L
.80584 .42371 L
.80604 .42307 L
.80627 .42237 L
.80651 .42165 L
.80673 .42097 L
.80697 .42022 L
.80722 .41944 L
.80744 .41878 L
.80768 .41806 L
.80793 .41728 L
.8082 .41645 L
.80846 .41567 L
.80869 .41497 L
.80891 .4143 L
.80914 .41361 L
.80933 .41301 L
.80955 .41236 L
.80978 .41166 L
.81003 .41092 L
.81026 .41022 L
.81047 .40958 L
.81068 .40894 L
.81088 .40836 L
.81109 .40771 L
.81132 .40703 L
.81154 .40638 L
.81174 .40578 L
.81192 .40523 L
.81212 .40465 L
.81235 .40396 L
.8126 .40323 L
.81283 .40254 L
.81304 .40192 L
.81329 .40119 L
.81342 .40081 L
Mistroke
.81356 .4004 L
.81381 .39966 L
.81404 .39898 L
.81428 .39829 L
.81453 .39755 L
.81477 .39685 L
.81499 .39621 L
.81524 .39547 L
.81538 .39505 L
.81552 .39467 L
.81576 .39396 L
.81602 .39321 L
.81625 .39253 L
.81647 .39191 L
.81671 .39122 L
.81696 .3905 L
.81717 .38988 L
.81738 .3893 L
.8176 .38866 L
.81783 .38799 L
.81804 .38739 L
.81827 .38674 L
.81848 .38613 L
.81868 .38557 L
.81886 .38506 L
.81905 .38451 L
.81926 .38393 L
.81945 .38339 L
.81968 .38274 L
.8199 .38213 L
.82009 .38158 L
.8203 .38099 L
.82053 .38034 L
.82077 .37966 L
.82098 .37909 L
.8212 .37846 L
.82141 .37787 L
.8216 .37733 L
.82182 .37672 L
.82203 .37614 L
.82223 .3756 L
.8224 .37511 L
.8226 .37457 L
.8228 .374 L
.82305 .37333 L
.82327 .37271 L
.82351 .37204 L
.82377 .37133 L
.82399 .37071 L
.8242 .37014 L
Mistroke
.82443 .36952 L
.82467 .36885 L
.82489 .36827 L
.82512 .36763 L
.82534 .36703 L
.82554 .36647 L
.82573 .36598 L
.82593 .36543 L
.82614 .36487 L
.82633 .36434 L
.82654 .36377 L
.82674 .36324 L
.82695 .36266 L
.82718 .36204 L
.82742 .3614 L
.82765 .36079 L
.82785 .36025 L
.82807 .35966 L
.82825 .35916 L
.82846 .3586 L
.82868 .35802 L
.82888 .35748 L
.82924 .35653 L
.82944 .35599 L
.82962 .3555 L
.82984 .35493 L
.83007 .35433 L
.83026 .35382 L
.83047 .35325 L
.8307 .35265 L
.83095 .352 L
.83118 .35139 L
.83139 .35083 L
.83178 .34982 L
.83199 .34925 L
.8322 .34872 L
.8326 .34766 L
.83283 .34706 L
.83304 .34653 L
.83338 .34564 L
.83357 .34513 L
.83375 .34466 L
.83411 .34372 L
.83432 .3432 L
.8345 .34272 L
.83483 .34187 L
.83502 .34139 L
.83519 .34095 L
.8355 .34014 L
.83583 .33928 L
Mistroke
.83604 .33876 L
.83622 .33829 L
.83642 .33778 L
.83664 .33722 L
.837 .33629 L
.8372 .33578 L
.83739 .3353 L
.83774 .33441 L
.83806 .3336 L
.83843 .33266 L
.83877 .33178 L
.83907 .33102 L
.8394 .33019 L
.83975 .32932 L
.84007 .32851 L
.84037 .32776 L
.84064 .32708 L
.84094 .32633 L
.84125 .32554 L
.84157 .32475 L
.8419 .32391 L
.84222 .32311 L
.84251 .3224 L
.84285 .32156 L
.84321 .32066 L
.84355 .31981 L
.84387 .31903 L
.8442 .31821 L
.84455 .31733 L
.84489 .3165 L
.8452 .31573 L
.84551 .31498 L
.84579 .3143 L
.8461 .31352 L
.8464 .31278 L
.84673 .31198 L
.84707 .31114 L
.84736 .31043 L
.84768 .30965 L
.84803 .30882 L
.84839 .30793 L
.84873 .3071 L
.84905 .30634 L
.84934 .30563 L
.84965 .30489 L
.8502 .30356 L
.8505 .30282 L
.85079 .30214 L
.85112 .30134 L
.85143 .30059 L
Mistroke
.8517 .29994 L
.85199 .29925 L
.8525 .29803 L
.85278 .29735 L
.85308 .29662 L
.85363 .29533 L
.85396 .29453 L
.85431 .2937 L
.85493 .29222 L
.85528 .2914 L
.8556 .29064 L
.85597 .28976 L
.85632 .28892 L
.85663 .28821 L
.85695 .28744 L
.85752 .28609 L
.85785 .28534 L
.85819 .28454 L
.85879 .28311 L
.85933 .28186 L
.8599 .28051 L
.86044 .27926 L
.86093 .27811 L
.86147 .27686 L
.86206 .27549 L
.86267 .27408 L
.86323 .27277 L
.86379 .27148 L
.86438 .27012 L
.86488 .26897 L
.86542 .26771 L
.866 .26637 L
.86663 .26494 L
.86721 .26361 L
.86774 .26239 L
.86827 .26119 L
.86875 .26008 L
.86928 .25887 L
.86984 .25759 L
.87037 .25638 L
.87087 .25525 L
.87131 .25425 L
.87179 .25316 L
.87269 .25112 L
.87318 .25002 L
.87363 .249 L
.87443 .24719 L
.87488 .24618 L
.87529 .24527 L
.87619 .24324 L
Mistroke
.8767 .2421 L
.87717 .24105 L
.87802 .23915 L
.87891 .23715 L
.87978 .2352 L
.88069 .23318 L
.88146 .23147 L
.88229 .22962 L
.88318 .22764 L
.88413 .22556 L
.885 .22361 L
.8858 .22186 L
.88659 .22011 L
.88732 .21851 L
.88811 .21678 L
.88893 .21496 L
.89031 .21194 L
.89108 .21025 L
.89179 .20869 L
.89336 .20527 L
.89423 .20336 L
.89504 .20162 L
.89649 .19846 L
.89729 .19673 L
.898 .19517 L
.89935 .19225 L
.90078 .18916 L
.90211 .18629 L
.90331 .1837 L
.90461 .18089 L
.90598 .17795 L
.90726 .17521 L
.9084 .17276 L
.90972 .16992 L
.91114 .16689 L
.91246 .16408 L
.91365 .16152 L
.91479 .15909 L
.916 .15651 L
.91815 .15195 L
.91943 .14922 L
.92063 .14667 L
.92285 .14198 L
.92495 .13754 L
.92713 .13292 L
.92914 .12868 L
.93092 .12492 L
.93473 .11691 L
.9368 .11256 L
.93866 .10865 L
Mistroke
.94207 .1015 L
.94567 .09398 L
.94893 .08716 L
.9523 .08014 L
.95559 .07328 L
.9586 .06703 L
.96405 .0557 L
.96674 .05013 L
.9695 .0444 L
.97426 .03455 L
.97906 .02464 L
.98343 .01561 L
.98786 .00649 L
Mfstroke
.98786 .00649 m
.99101 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  Background->GrayLevel[1],
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005X0000gR000`400?l00000o`00003oo`3ooomY
0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY
0?ooo`00D03oool5000000<0oooo100000050?ooo`800000903oool5000000<0oooo100000040?oo
o`@000001@3oool2000001d0oooo1@0000030?ooo`@00000103oool4000000@0oooo100000050?oo
o`800000503oool5000000<0oooo100000040?ooo`@00000103oool4000000@0oooo100000050?oo
o`800000303oool5000000<0oooo100000040?ooo`@00000103oool4000000@0oooo100000040?oo
o`@000001@3oool2000000P0oooo001B0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`020?ooo`8000009P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`020?oo
o`8000007`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?oo
o`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0080oooo0P00
000F0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?oo
o`0000040?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`00
0000oooo0?ooo`020?ooo`8000003P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`040000003o
ool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`020?ooo`80
0000203oool00580oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000
003oool0oooo000000@0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`000004
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool0
10000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0
303oool00580oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003o
ool0oooo000000@0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?oo
o`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?oo
o`0000040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool01000
0000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0303o
ool00580oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003oool0
oooo000000@0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`04
0000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`00
00040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000
oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0303oool0
0580oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo
000000@0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000
003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`000004
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo
0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0303oool00500
oooo0`0000040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0:03oool3000000@0
oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo
0240oooo0`0000040?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool0
10000000oooo0?ooo`0000040?ooo`030000003oool0oooo01P0oooo0`0000040?ooo`030000003o
ool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`04
0000003oool0oooo000000@0oooo00<000000?ooo`3oool0403oool3000000@0oooo00<000000?oo
o`3oool00P3oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00@0
00000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00`0
oooo001B0?ooo`030000003oool0oooo00<0oooo1000000]0?ooo`030000003oool0oooo00<0oooo
100000040?ooo`@000009P3oool00`000000oooo0?ooo`030?ooo`@00000103oool4000000@0oooo
1000000M0?ooo`030000003oool0oooo00<0oooo100000040?ooo`@00000103oool4000000@0oooo
1000000E0?ooo`030000003oool0oooo00<0oooo100000040?ooo`@00000103oool4000000@0oooo
100000040?ooo`@000003`3oool00?l0ooooJ@3oool00?l0ooooJ@3oool00?l0ooooJ@3oool00?l0
ooooJ@3oool00?l0ooooJ@3oool00?l0ooooJ@3oool001h0ooooo`00000V000000030000o`000000
00000280oooo000N0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool01@00
0000oooo0?ooo`3oool000000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0080oooo00D000000?ooo`3oool0oooo000000020?ooo`040000
003oool0oooo00000080oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`05
0000003oool0oooo0?ooo`0000000`3oool01P000000oooo0?ooo`000000oooo00000140oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00<0oooo00D0
00000?ooo`3oool0oooo000000030?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000
oooo0000000A0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`030?ooo`050000003oool0oooo0?ooo`0000000`3oool02P000000oooo0?ooo`3oool0
00000?ooo`000?l000000?ooo`00000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0
001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00X0001o0?ooo`3oool0oooo0000O`3oool0003o0000O`3oool000008P3oool001h0oooo00<00000
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`0:0000O`3oool0oooo0?ooo`0007l0003o0?ooo`0007l0oooo0000
0280oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`02
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o
0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo
0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool02P0007l0oooo0?ooo`3oool0001o
0000o`3oool0001o0?ooo`00000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o
0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000o`020?ooo`030000O`3oool000000280oooo000N0?ooo`800000403o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`02
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3o
ool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0oooo0?ooo`000?l00P3oool00`0007l000000000000R0?ooo`007P3o
ool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool0
0`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0003o0000O`020?ooo`030000
O`3oool000000280oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`000?l0001o0080oooo00<0001o0?ooo`0000008P3oool001h0oooo00<000000?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`02
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3o
ool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0003o0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`007P3o
ool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool0
0`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`000?l0oooo0000O`020?ooo`030000
O`3oool000000280oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0003o0?oo
o`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool001h0oooo0P00000@0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o
0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0000o`3oool0oooo0000O`020?ooo`030000O`00000000000280oooo000N0?ooo`030000003o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3o
ool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@000?l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`000000
8P3oool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0
001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000o`3oool0oooo0?ooo`0007l0
0P3oool00`0007l0oooo0000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o
0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0
003o0000O`3oool00P3oool01P0007l0oooo0?ooo`0007l0oooo00000280oooo000N0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool00`000?l0001o0?ooo`020?ooo`060000O`3oool0oooo0000O`3o
ool000008P3oool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000
O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00T0001o0?ooo`3oool0oooo0000O`3oool0003o0?ooo`0007l00`3oool0
1P0007l0oooo0?ooo`0007l0oooo00000280oooo000N0?ooo`800000403oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0
001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00T0001o0?ooo`3oool0oooo0000O`3oool0003o0?ooo`00
07l00`3oool0100007l0oooo0?ooo`0007l200000280oooo000N0?ooo`030000003oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0
oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`060000O`3oool0oooo0?ooo`00
07l0003o0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool0
01h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140
oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000o`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`00
07lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0003o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`030000003oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P00
07l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0000
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool0
01h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140
oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0003o0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000000R0?ooo`007P3oool200000100oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0
oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0003o0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00<0001o0000000000008P3oool001h0oooo00<000000?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo
0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`000?l0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`007P3oool00`00
0000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0
oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
003o0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0
00000280oooo000<0?ooo`<000001@3oool4000000H0oooo00<000000?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo
0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0000o`3oool0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`002`3oool01@00
0000oooo0?ooo`3oool0000000<0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?oo
o`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`050000O`000?l0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0
001o0?ooo`0000008P3oool000l0oooo00D000000?ooo`3oool0oooo000000040?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0
001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00D0003o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool00`0007l0oooo0000000R0?ooo`000`3oool6000000H0oooo00D000000?ooo`3oool0oooo
000000040?ooo`030000003oool0oooo00<0oooo0`00003o0000Oa@0001o00<0003o0000O`0007l0
3@0007l200000280oooo000=0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0
001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo0080oooo00<0003o0000O`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0
001o0?ooo`3oool0001o0?ooo`00000R0?ooo`003`3oool01@000000oooo0?ooo`3oool0000000@0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00P3oool00`000?l0001o0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01P0007l0oooo0?ooo`0007l0oooo00000280oooo000;0?ooo`050000003oool0oooo
0?ooo`0000000`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo0080oooo00<0003o0000
O`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?oo
o`00000R0?ooo`00303oool3000000D0oooo100000060?ooo`030000003oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?oo
o`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool01`0007l0oooo0?ooo`3oool0003o0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`00000R0?ooo`007P3oool00`00
0000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0
oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00@0001o0?ooo`3oool0003o0P3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3o
ool000000280oooo000N0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o
0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`040000O`3oool0oooo
0000o`80oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool00`0007l000000000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o
0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0
oooo00@0oooo00@0001o0?ooo`3oool0003o0P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool0100007l0oooo0?ooo`000?l20?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?oo
o`0000008P3oool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000
O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0003o00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool00`0007l0oooo0000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3o
ool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000
O`3oool0oooo00@0oooo00<0001o0000o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`03
0000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000
O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0003o0?ooo`030?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?oo
o`0000008P3oool001h0oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0003o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`030000O`00000000000280oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo
0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0
103oool00`000?l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool001h0oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`06
0000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0
203oool00`0007l0oooo0?ooo`030?ooo`80003o1@3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`03
0000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000
O`3oool0oooo00P0oooo00<0001o0?ooo`3oool00`3oool00`000?l0001o0?ooo`040?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?oo
o`0000008P3oool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000
O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`030?ooo`030000
o`0007l0oooo00@0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool00`0007l0oooo0000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3o
ool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000
O`3oool0oooo0080oooo00<0003o0?ooo`0007l01@3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`80
0000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`06
0000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0
203oool00`0007l0oooo0?ooo`020?ooo`030000o`3oool0001o00D0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l000000000000R0?oo
o`007P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`080?ooo`050000O`3oool0oooo0?ooo`000?l00P3oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00<0001o0?ooo`0000008P3oool001h0oooo00<000000?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo
0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool01@0007l0
oooo0?ooo`3oool0003o0080oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N
0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?oo
o`030000O`3oool0oooo00P0oooo00@0001o0?ooo`3oool0003o0`3oool00`0007l0oooo0?ooo`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00<0001o0?ooo`0000008P3oool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0
001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool0100007l0oooo0?ooo`00
0?l30?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`007P3oool00`000000oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0003o00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280
oooo000N0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0
001o0?ooo`3oool0203oool00`0007l0oooo0000o`040?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0
00000000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0
001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`000?l0003o00@0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`030000003oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo
0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0
001o0000o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool000/0oooo
1@0000040?ooo`@000001P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0
001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000o`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`030000O`3oool000000280oooo000;0?ooo`050000003oool0oooo0?ooo`00
00000`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`06
0000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0
203oool00`000?l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?oo
o`00303oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`070?ooo`030000o`0007l0oooo00D0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`030000O`3oool000000280oooo00030?ooo`H00000103oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool300000?l0001o1P0007l00`000?l0
001o0000O`0K0000O`8000008P3oool000h0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3o
ool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool01P3oool00`000?l0oooo0000
O`060?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`003`3oool01@000000oooo
0?ooo`3oool0000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3o
ool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`060?ooo`030000
o`3oool0001o00H0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000;0?ooo`05
0000003oool0oooo0?ooo`0000000`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140
oooo00<0001o0?ooo`3oool01P3oool00`000?l0oooo0000O`060?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000000R0?ooo`00303oool3000000D0oooo100000060?ooo`030000003oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P00
07l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00D0
oooo00@0003o0?ooo`3oool0001o1P3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3o
ool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o
0140oooo00<0001o0?ooo`3oool01@3oool010000?l0oooo0?ooo`0007l60?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool00`0007l0oooo0000000R0?ooo`007P3oool200000100oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00@0oooo00D0003o0?ooo`3oool0oooo0000
O`060?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool00`0007l000000000000R0?ooo`007P3oool00`000000oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?oo
o`040?ooo`050000o`3oool0oooo0?ooo`0007l01P3oool00`0007l0oooo0?ooo`030?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?oo
o`0000008P3oool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000
O`3oool0001o0140oooo00<0001o0?ooo`3oool00`3oool00`000?l0oooo0?ooo`020?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`03
0000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000
O`3oool0oooo00<0oooo00<0003o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool00`0007l0oooo0000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3o
ool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`020?ooo`80003o
103oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?oo
o`007P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`020?ooo`030000o`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool001h0oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`06
0000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00@0001o0?ooo`3oool0
oooo0P000?l50?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0
00000280oooo000N0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140
oooo00D0001o0?ooo`3oool0oooo0000o`060?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`030000O`00000000000280oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`00
07l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`050000O`3oool0oooo0?ooo`000?l01P3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`007P3o
ool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool0
100007l0oooo0?ooo`000?l70?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`03
0000O`3oool000000280oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000
Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000O`0A0?ooo`040000O`3oool0oooo0000o`L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool001h0oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`06
0000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00@0001o0?ooo`000?l0
003o1`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R
0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`00
07l04@3oool00`0007l0oooo0000o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`030000O`3oool000000280oooo000N0?ooo`800000403oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`03
0000O`3oool0001o0140oooo00<0001o0000o`000?l0203oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool00`0007l000000000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?oo
o`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0003o0?ooo`080?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o
00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?oo
o`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`03
0000O`000?l0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o
0?ooo`0000008P3oool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`03
0000O`3oool0001o0140oooo00<0003o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?oo
o`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`000?l0oooo0?ooo`080?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o
00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?oo
o`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0@0?ooo`80
003o2P3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R
0?ooo`007P3oool200000100oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0@0?ooo`03
0000o`0007l0oooo00T0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o
0000000000008P3oool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`03
0000O`3oool0001o00l0oooo0P000?l00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`030000003oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P00
07l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0?0?ooo`030000o`3oool0001o00X0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool0
00`0oooo1@0000030?ooo`@000001P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`00
07lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00<0001o0?ooo`0007l03`3oool00`000?l0oooo0000O`0:0?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000>0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P00
07l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0>0?ooo`040000o`3oool0oooo0000
O`X0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3o
ool000h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o
00h0oooo00@0003o0?ooo`3oool0001o2P3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool00`0007l0oooo0000000R0?ooo`000`3oool6000000D0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo0`00003g0000O`80003o:P00
07l200000280oooo000>0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`00
07l0oooo0000O`0=0?ooo`050000o`3oool0oooo0?ooo`0007l02P3oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`003P3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0
001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l03@3oool01@000?l0oooo0?ooo`3o
ool0001o00X0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`00
00008P3oool000`0oooo0`0000040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0303o
ool00`000?l0oooo0?ooo`020?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool000h0oooo00<000000?ooo`3oool0
0`3oool4000000H0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3o
ool0001o00`0oooo00<0003o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0;0?ooo`80003o103o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`03
0000O`3oool000000280oooo000N0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3o
ool0001o00/0oooo00<0003o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`00000000000280oooo000N0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0;0?ooo`030000o`3o
ool0oooo00<0oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool00`0007l0oooo0000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0
001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0?ooo`0007l02P3oool00`000?l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`00
00008P3oool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3o
ool0001o00X0oooo00<0003o0?ooo`3oool0103oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`090?ooo`80003o1P3o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`03
0000O`3oool000000280oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000
Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000O`090?ooo`030000o`3oool0oooo00D0oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`00
7P3oool200000100oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`090?ooo`030000o`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool00`0007l000000000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0
001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0?ooo`0007l0203oool00`000?l0oooo0?ooo`060?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`00
00008P3oool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3o
ool0001o00P0oooo00<0003o0?ooo`3oool01P3oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`070?ooo`80003o203o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`03
0000O`3oool000000280oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000
Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000O`070?ooo`030000o`3oool0oooo00L0oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`00
7P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l01`3o
ool00`000?l0oooo0?ooo`070?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool001h0oooo0P00000@0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0
001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0?ooo`0007l01P3oool20000o`T0oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l000000000000R0?ooo`00
7P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l01P3o
ool00`000?l0oooo0?ooo`080?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool001h0oooo00<000000?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000
O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o00H0oooo00<0003o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`03
0000O`3oool000000280oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000
Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000O`050?ooo`80003o2P3oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`050?ooo`030000o`3o
ool0oooo00T0oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool00`0007l0oooo0000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0
001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0?ooo`0007l01@3oool00`000?l0oooo0?ooo`090?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`00
00008P3oool001h0oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0103o
ool00`000?l0oooo0?ooo`0:0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00<0001o0000000000008P3oool001h0oooo00<000000?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000
O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o00@0oooo00<0003o0?ooo`3oool02P3o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`03
0000O`3oool000000280oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000
Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000O`030?ooo`80003o303oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000D0?ooo`@00000
1P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l00`3o
ool00`000?l0oooo0?ooo`0;0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool001<0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000
Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000O`030?ooo`030000o`3oool0oooo00/0oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`00
4`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000
O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o00<0oooo00<0003o0?ooo`3oool02`3o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`03
0000O`3oool000000280oooo000C0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
0`3oool300000>`0001o00<0003o0000O`0007l0=00007l200000280oooo000C0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o
0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00<0001o0?ooo`0007l00P3oool00`000?l0oooo0?ooo`0<0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`000000
8P3oool001<0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3o
ool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0000O`3oool20000o`h0oooo00<0
001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0
oooo0000000R0?ooo`004`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0001o0?ooo`000?l0
3`3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`030000O`3oool000000280oooo000D0?ooo`@000001P3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3o
ool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00D0001o0?ooo`0007l0oooo0000o`0?0?ooo`030000O`3oool0oooo00P0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool0
01h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0001o0?oo
o`000?l03`3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`030000O`3oool000000280oooo000N0?ooo`800000403oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o
4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0001o0000oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool00`0007l000000000000R0?ooo`007P3oool00`000000oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0
001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`0007l0003o403oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280
oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0000
O`000?l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00<0001o0?ooo`0000008P3oool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0003o0P3ooolV0000o`<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`000?l0003o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?oookl0003o103oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3o
ool0003o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool00`0007l0oooo0000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0
001o0?ooo`0007l90?ooo``0003o1P3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000o`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool001h0oooo0P00000@0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0
001o0?ooo`0007lA0?ooo`030000O`000?l0003o00H0003o00<0oooo0000O`3oool01P3oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0000o`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`00000000000280oooo000N0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool0103oool50000
o`H0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`000?l0001o0140oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`00
7P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo
00L0oooo1@000?l30?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0003o0000O`0A0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`00
00008P3oool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0003o00<0oooo0000O`3oool0103oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`80003o00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`030000003oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo
0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`030?oo
o`<0003o1@3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`030000o`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`007P3oool00`000000oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool0103oool30000o`@0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool00`000?l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool001h0oooo
00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00@0001o0000o`000?l0003o0P3oool01@0007l0oooo0?ooo`3o
ool0001o00<0oooo00H0001o0?ooo`3oool0oooo0000O`3oool20000o`030?ooo`0007l0oooo0100
oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000000R0?ooo`007P3oool200000100oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0003o0080003o00<0
oooo0000O`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`000?l0
001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0000000000008P3oool001h0oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0
oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00@0001o0?ooo`3oool0oooo0P000?l01@0007l0oooo0?ooo`3oool0001o00<0oooo00X0001o0?oo
o`3oool0oooo0000O`3oool0003o0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`080?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o
00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?oo
o`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo0P000?l30?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01P0007l0003o0000o`0007l0oooo0000Oa40oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000
000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`80003o
0P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0000o`3oool0001o0?ooo`0007lA
0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00<0001o0?ooo`0000008P3oool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0
001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0003o0000o`3oool0001o00<0oooo00X0001o0?ooo`3oool0oooo0000O`00
0?l0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`030000003oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P00
07l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`000?l0003o0000O`030?ooo`040000
O`3oool0oooo0?ooo`80003o00@0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`080?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N
0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020000o`<0oooo
00D0001o0?ooo`3oool0oooo0000o`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3o
ool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l000000000000R
0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`040000O`3o
ool0oooo0?ooo`80003o0P3oool00`0007l0oooo0?ooo`020000o`80oooo00<0001o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`030000O`3oool000000280oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo
0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`020000o`060?ooo`0007l0oooo0?ooo`000?l0
001o0P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3oool000`0oooo1@0000030?oo
o`@000001P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`060000O`3o
ool0oooo0?ooo`0007l0oooo0P000?l01@0007l0oooo0000o`000?l0001o0080oooo00<0001o0?oo
o`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`030000O`3oool000000280oooo000>0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`00
07l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`@0003o00L0oooo0000O`3o
ool0oooo0000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`003P3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0
001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`08
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3o
ool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo
000>0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`030?ooo`<00000o`0007lT0000O`8000008P3oool000h0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0
oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`003P3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3o
ool000000280oooo000<0?ooo`<00000103oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o
0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000000R0?ooo`003P3oool00`000000oooo0?ooo`030?ooo`@000001P3oool0
0`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3o
ool000000280oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`00
07l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0?ooo`0000008P3oool001h0oooo0P00000@0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?oo
o`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`00000000000280oooo000N0?ooo`030000003oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3o
ool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o
0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?oo
o`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`030000003oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3o
ool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o
0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000000R0?ooo`007P3oool200000100oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`00
07l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0000000000008P3oool001h0oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0
oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`007P3oool0
0`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3o
ool000000280oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`00
07l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0?ooo`0000008P3oool001h0oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0
oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`007P3oool0
0`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3o
ool000000280oooo000N0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o
0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l000000000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?oo
o`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`030000003oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3o
ool001h0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`060000O`3oool0oooo0000O`3oool0001o4@3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o
0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000000R0?ooo`007P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0001o0?ooo`3oool0001o0?oo
o`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo000N0?ooo`030000003oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000008P3o
ool001h0oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00H0001o0?ooo`3oool0001o0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`00
000000000280oooo000N0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`00
07l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0?ooo`0000008P3oool001h0oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`060000O`3oool0
oooo0000O`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000000R0?ooo`002`3oool5
000000@0oooo100000060?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01P0007l0oooo0?ooo`0007l0oooo0000Oa40oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`00
07l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0?ooo`0000008P3oool000/0oooo00D000000?ooo`3oool0oooo0000
00030?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00H0
001o0?ooo`3oool0001o0?ooo`00000A0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000004@3oool00`0007l0oooo0?ooo`08
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3o
ool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool000000280oooo
000<0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool01@000000oooo0?oo
o`3oool000000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0?0?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00D000000?ooo`3oool0oooo000000020?ooo`040000003oool0oooo
00000080oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`050000003oool0
oooo0?ooo`0000000`3oool01P000000oooo0?ooo`000000oooo00000140oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00<0oooo00D000000?ooo`3o
ool0oooo000000030?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0000000A
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`03
0?ooo`050000003oool0oooo0?ooo`0000000`3oool01@000000oooo0?ooo`3oool000000080oooo
00<000000?ooo`0000008P3oool000d0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00<0ooooo`00000Y00000280oooo000>0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3o0?ooodh0oooo000?0?oo
o`050000003oool0oooo0?ooo`000000103oool00`000000oooo0?ooo`3o0?ooodh0oooo000;0?oo
o`050000003oool0oooo0?ooo`0000000`3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0?l0ooooCP3oool000`0oooo0`0000050?ooo`@00000o`3ooomA0?ooo`00o`3ooomY0?ooo`00
o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00703oool5000002l0oooo1@0000030?oo
o`@000001@3oool2000002@0oooo1@0000030?ooo`@00000103oool4000000D0oooo0P00000M0?oo
o`D000000`3oool4000000@0oooo100000040?ooo`@000001@3oool2000001@0oooo1@0000030?oo
o`@00000103oool4000000@0oooo100000040?ooo`@000001@3oool2000004L0oooo000N0?ooo`03
0000003oool0oooo0340oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo0P00000V0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool00P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0080oooo0P00000O0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`000004
0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool00P3oool2000001H0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000000@0oooo
00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo
0080oooo0P0000170?ooo`007P3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo
02<0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo
000000@0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`040000
003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`1;0?oo
o`007P3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool00P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo02<0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000000@0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0
oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`1;0?ooo`007P3oool00`000000
oooo0?ooo`0a0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0Z0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000
oooo0?ooo`0000040?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3o
ool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3o
ool00000103oool00`000000oooo0?ooo`1;0?ooo`007P3oool00`000000oooo0?ooo`0a0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0Z0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?oo
o`030000003oool0oooo02<0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?oo
o`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?oo
o`0000040?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`00
0000oooo0?ooo`1;0?ooo`00703oool300000340oooo0`0000040?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool0:03oool3000000@0oooo00<000000?ooo`3oool00P3oool010000000
oooo0?ooo`0000040?ooo`030000003oool0oooo0240oooo0`0000040?ooo`030000003oool0oooo
0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000003o
ool0oooo01P0oooo0`0000040?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000
103oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?oo
o`3oool0B`3oool001h0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`030?ooo`@0
0000;@3oool00`000000oooo0?ooo`030?ooo`@00000103oool4000002H0oooo00<000000?ooo`3o
ool00`3oool4000000@0oooo100000040?ooo`@000007@3oool00`000000oooo0?ooo`030?ooo`@0
0000103oool4000000@0oooo100000040?ooo`@00000CP3oool00?l0ooooJ@3oool00?l0ooooJ@3o
ool00?l0ooooJ@3oool00001\
\>"],
  ImageRangeCache->{{{0, 359}, {221.375, 0}} -> {-0.516361, -46.3741, \
0.021156, 0.410772}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(BodePhasePlot[gio[2\ Pi\ I\ s]\  /. \ Param2, {s, \ 1, \ 100000}, \ 
        Frame\  -> \ True, PlotRange\  -> \ {{0, \ 5}, \ {0, \ \(-180\)}}, 
        DisplayFunction\  -> \ Identity, 
        PlotStyle\  -> \ RGBColor[0, \ 0, \ 1], 
        GridLines\  -> \ {LogGridMinor, \ Automatic}];\)\)], "Input",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-017 0.2 0.618034 0.00343352 [
[.2 -0.0125 -11 -12.5625 ]
[.2 -0.0125 11 0 ]
[.4 -0.0125 -14 -12.5625 ]
[.4 -0.0125 14 0 ]
[.6 -0.0125 -17 -12.5625 ]
[.6 -0.0125 17 0 ]
[.8 -0.0125 -20 -12.5625 ]
[.8 -0.0125 20 0 ]
[1 -0.0125 -23 -12.5625 ]
[1 -0.0125 23 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01717 -24 -4.5 ]
[-0.0125 .01717 0 4.5 ]
[-0.0125 .10301 -24 -4.5 ]
[-0.0125 .10301 0 4.5 ]
[-0.0125 .18884 -24 -4.5 ]
[-0.0125 .18884 0 4.5 ]
[-0.0125 .27468 -24 -4.5 ]
[-0.0125 .27468 0 4.5 ]
[-0.0125 .36052 -18 -4.5 ]
[-0.0125 .36052 0 4.5 ]
[-0.0125 .44636 -18 -4.5 ]
[-0.0125 .44636 0 4.5 ]
[-0.0125 .5322 -18 -4.5 ]
[-0.0125 .5322 0 4.5 ]
[-0.0125 .61803 -6 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 -0.125 0 ]
[0 .63053 -5 0 ]
[0 .63053 5 12.5625 ]
[.2 .63053 -11 0 ]
[.2 .63053 11 12.5625 ]
[.4 .63053 -14 0 ]
[.4 .63053 14 12.5625 ]
[.6 .63053 -17 0 ]
[.6 .63053 17 12.5625 ]
[.8 .63053 -20 0 ]
[.8 .63053 20 12.5625 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 -0.0005 0 0 ]
[ 1 .61853 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 0 m
0 .61803 L
s
.06021 0 m
.06021 .61803 L
s
.09542 0 m
.09542 .61803 L
s
.12041 0 m
.12041 .61803 L
s
.13979 0 m
.13979 .61803 L
s
.15563 0 m
.15563 .61803 L
s
.16902 0 m
.16902 .61803 L
s
.18062 0 m
.18062 .61803 L
s
.19085 0 m
.19085 .61803 L
s
.2 0 m
.2 .61803 L
s
.26021 0 m
.26021 .61803 L
s
.29542 0 m
.29542 .61803 L
s
.32041 0 m
.32041 .61803 L
s
.33979 0 m
.33979 .61803 L
s
.35563 0 m
.35563 .61803 L
s
.36902 0 m
.36902 .61803 L
s
.38062 0 m
.38062 .61803 L
s
.39085 0 m
.39085 .61803 L
s
.4 0 m
.4 .61803 L
s
.46021 0 m
.46021 .61803 L
s
.49542 0 m
.49542 .61803 L
s
.52041 0 m
.52041 .61803 L
s
.53979 0 m
.53979 .61803 L
s
.55563 0 m
.55563 .61803 L
s
.56902 0 m
.56902 .61803 L
s
.58062 0 m
.58062 .61803 L
s
.59085 0 m
.59085 .61803 L
s
.6 0 m
.6 .61803 L
s
.66021 0 m
.66021 .61803 L
s
.69542 0 m
.69542 .61803 L
s
.72041 0 m
.72041 .61803 L
s
.73979 0 m
.73979 .61803 L
s
.75563 0 m
.75563 .61803 L
s
.76902 0 m
.76902 .61803 L
s
.78062 0 m
.78062 .61803 L
s
.79085 0 m
.79085 .61803 L
s
.8 0 m
.8 .61803 L
s
.86021 0 m
.86021 .61803 L
s
.89542 0 m
.89542 .61803 L
s
.92041 0 m
.92041 .61803 L
s
.93979 0 m
.93979 .61803 L
s
.95563 0 m
.95563 .61803 L
s
.96902 0 m
.96902 .61803 L
s
.98062 0 m
.98062 .61803 L
s
.99085 0 m
.99085 .61803 L
s
1 0 m
1 .61803 L
s
0 .10301 m
1 .10301 L
s
0 .18884 m
1 .18884 L
s
0 .27468 m
1 .27468 L
s
0 .36052 m
1 .36052 L
s
0 .44636 m
1 .44636 L
s
0 .5322 m
1 .5322 L
s
0 g
.2 0 m
.2 .00625 L
s
gsave
.2 -0.0125 -72 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10.) show
81.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.4 0 m
.4 .00625 L
s
gsave
.4 -0.0125 -75 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100.) show
87.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.6 0 m
.6 .00625 L
s
gsave
.6 -0.0125 -78 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1000.) show
93.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.8 0 m
.8 .00625 L
s
gsave
.8 -0.0125 -81 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10000.) show
99.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
1 0 m
1 .00625 L
s
gsave
1 -0.0125 -84 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100000.) show
105.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.001 w
.06021 0 m
.06021 .00375 L
s
.09542 0 m
.09542 .00375 L
s
.12041 0 m
.12041 .00375 L
s
.13979 0 m
.13979 .00375 L
s
.15563 0 m
.15563 .00375 L
s
.16902 0 m
.16902 .00375 L
s
.18062 0 m
.18062 .00375 L
s
.19085 0 m
.19085 .00375 L
s
.26021 0 m
.26021 .00375 L
s
.29542 0 m
.29542 .00375 L
s
.32041 0 m
.32041 .00375 L
s
.33979 0 m
.33979 .00375 L
s
.35563 0 m
.35563 .00375 L
s
.36902 0 m
.36902 .00375 L
s
.38062 0 m
.38062 .00375 L
s
.39085 0 m
.39085 .00375 L
s
.46021 0 m
.46021 .00375 L
s
.49542 0 m
.49542 .00375 L
s
.52041 0 m
.52041 .00375 L
s
.53979 0 m
.53979 .00375 L
s
.55563 0 m
.55563 .00375 L
s
.56902 0 m
.56902 .00375 L
s
.58062 0 m
.58062 .00375 L
s
.59085 0 m
.59085 .00375 L
s
.66021 0 m
.66021 .00375 L
s
.69542 0 m
.69542 .00375 L
s
.72041 0 m
.72041 .00375 L
s
.73979 0 m
.73979 .00375 L
s
.75563 0 m
.75563 .00375 L
s
.76902 0 m
.76902 .00375 L
s
.78062 0 m
.78062 .00375 L
s
.79085 0 m
.79085 .00375 L
s
.86021 0 m
.86021 .00375 L
s
.89542 0 m
.89542 .00375 L
s
.92041 0 m
.92041 .00375 L
s
.93979 0 m
.93979 .00375 L
s
.95563 0 m
.95563 .00375 L
s
.96902 0 m
.96902 .00375 L
s
.98062 0 m
.98062 .00375 L
s
.99085 0 m
.99085 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .01717 m
.00625 .01717 L
s
[(-175)] -0.0125 .01717 1 0 Mshowa
0 .10301 m
.00625 .10301 L
s
[(-150)] -0.0125 .10301 1 0 Mshowa
0 .18884 m
.00625 .18884 L
s
[(-125)] -0.0125 .18884 1 0 Mshowa
0 .27468 m
.00625 .27468 L
s
[(-100)] -0.0125 .27468 1 0 Mshowa
0 .36052 m
.00625 .36052 L
s
[(-75)] -0.0125 .36052 1 0 Mshowa
0 .44636 m
.00625 .44636 L
s
[(-50)] -0.0125 .44636 1 0 Mshowa
0 .5322 m
.00625 .5322 L
s
[(-25)] -0.0125 .5322 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(0)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .03434 m
.00375 .03434 L
s
0 .0515 m
.00375 .0515 L
s
0 .06867 m
.00375 .06867 L
s
0 .08584 m
.00375 .08584 L
s
0 .12017 m
.00375 .12017 L
s
0 .13734 m
.00375 .13734 L
s
0 .15451 m
.00375 .15451 L
s
0 .17168 m
.00375 .17168 L
s
0 .20601 m
.00375 .20601 L
s
0 .22318 m
.00375 .22318 L
s
0 .24035 m
.00375 .24035 L
s
0 .25751 m
.00375 .25751 L
s
0 .29185 m
.00375 .29185 L
s
0 .30902 m
.00375 .30902 L
s
0 .32618 m
.00375 .32618 L
s
0 .34335 m
.00375 .34335 L
s
0 .37769 m
.00375 .37769 L
s
0 .39486 m
.00375 .39486 L
s
0 .41202 m
.00375 .41202 L
s
0 .42919 m
.00375 .42919 L
s
0 .46353 m
.00375 .46353 L
s
0 .48069 m
.00375 .48069 L
s
0 .49786 m
.00375 .49786 L
s
0 .51503 m
.00375 .51503 L
s
0 .54936 m
.00375 .54936 L
s
0 .56653 m
.00375 .56653 L
s
0 .5837 m
.00375 .5837 L
s
0 .60087 m
.00375 .60087 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 .61178 m
0 .61803 L
s
gsave
0 .63053 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
69.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.2 .61178 m
.2 .61803 L
s
gsave
.2 .63053 -72 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10.) show
81.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.4 .61178 m
.4 .61803 L
s
gsave
.4 .63053 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100.) show
87.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.6 .61178 m
.6 .61803 L
s
gsave
.6 .63053 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1000.) show
93.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.8 .61178 m
.8 .61803 L
s
gsave
.8 .63053 -81 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10000.) show
99.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.001 w
.06021 .61428 m
.06021 .61803 L
s
.09542 .61428 m
.09542 .61803 L
s
.12041 .61428 m
.12041 .61803 L
s
.13979 .61428 m
.13979 .61803 L
s
.15563 .61428 m
.15563 .61803 L
s
.16902 .61428 m
.16902 .61803 L
s
.18062 .61428 m
.18062 .61803 L
s
.19085 .61428 m
.19085 .61803 L
s
.26021 .61428 m
.26021 .61803 L
s
.29542 .61428 m
.29542 .61803 L
s
.32041 .61428 m
.32041 .61803 L
s
.33979 .61428 m
.33979 .61803 L
s
.35563 .61428 m
.35563 .61803 L
s
.36902 .61428 m
.36902 .61803 L
s
.38062 .61428 m
.38062 .61803 L
s
.39085 .61428 m
.39085 .61803 L
s
.46021 .61428 m
.46021 .61803 L
s
.49542 .61428 m
.49542 .61803 L
s
.52041 .61428 m
.52041 .61803 L
s
.53979 .61428 m
.53979 .61803 L
s
.55563 .61428 m
.55563 .61803 L
s
.56902 .61428 m
.56902 .61803 L
s
.58062 .61428 m
.58062 .61803 L
s
.59085 .61428 m
.59085 .61803 L
s
.66021 .61428 m
.66021 .61803 L
s
.69542 .61428 m
.69542 .61803 L
s
.72041 .61428 m
.72041 .61803 L
s
.73979 .61428 m
.73979 .61803 L
s
.75563 .61428 m
.75563 .61803 L
s
.76902 .61428 m
.76902 .61803 L
s
.78062 .61428 m
.78062 .61803 L
s
.79085 .61428 m
.79085 .61803 L
s
.86021 .61428 m
.86021 .61803 L
s
.89542 .61428 m
.89542 .61803 L
s
.92041 .61428 m
.92041 .61803 L
s
.93979 .61428 m
.93979 .61803 L
s
.95563 .61428 m
.95563 .61803 L
s
.96902 .61428 m
.96902 .61803 L
s
.98062 .61428 m
.98062 .61803 L
s
.99085 .61428 m
.99085 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .01717 m
1 .01717 L
s
.99375 .10301 m
1 .10301 L
s
.99375 .18884 m
1 .18884 L
s
.99375 .27468 m
1 .27468 L
s
.99375 .36052 m
1 .36052 L
s
.99375 .44636 m
1 .44636 L
s
.99375 .5322 m
1 .5322 L
s
.125 Mabswid
.99625 .03434 m
1 .03434 L
s
.99625 .0515 m
1 .0515 L
s
.99625 .06867 m
1 .06867 L
s
.99625 .08584 m
1 .08584 L
s
.99625 .12017 m
1 .12017 L
s
.99625 .13734 m
1 .13734 L
s
.99625 .15451 m
1 .15451 L
s
.99625 .17168 m
1 .17168 L
s
.99625 .20601 m
1 .20601 L
s
.99625 .22318 m
1 .22318 L
s
.99625 .24035 m
1 .24035 L
s
.99625 .25751 m
1 .25751 L
s
.99625 .29185 m
1 .29185 L
s
.99625 .30902 m
1 .30902 L
s
.99625 .32618 m
1 .32618 L
s
.99625 .34335 m
1 .34335 L
s
.99625 .37769 m
1 .37769 L
s
.99625 .39486 m
1 .39486 L
s
.99625 .41202 m
1 .41202 L
s
.99625 .42919 m
1 .42919 L
s
.99625 .46353 m
1 .46353 L
s
.99625 .48069 m
1 .48069 L
s
.99625 .49786 m
1 .49786 L
s
.99625 .51503 m
1 .51503 L
s
.99625 .54936 m
1 .54936 L
s
.99625 .56653 m
1 .56653 L
s
.99625 .5837 m
1 .5837 L
s
.99625 .60087 m
1 .60087 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
.5 Mabswid
.00036 .61802 m
.01921 .61802 L
.03398 .61802 L
.0454 .61801 L
.05635 .61801 L
.06683 .61801 L
.07677 .618 L
.08521 .618 L
.09221 .618 L
.09879 .61799 L
.10444 .61799 L
.11025 .61799 L
.11598 .61799 L
.12485 .61798 L
.12951 .61798 L
.13365 .61797 L
.13819 .61797 L
.14287 .61797 L
.14707 .61796 L
.15079 .61796 L
.154 .61796 L
.15741 .61796 L
.16083 .61795 L
.16392 .61795 L
.16721 .61795 L
.17054 .61794 L
.17329 .61794 L
.1762 .61794 L
.17924 .61793 L
.18237 .61793 L
.18523 .61793 L
.18774 .61792 L
.19013 .61792 L
.19268 .61792 L
.19526 .61791 L
.19761 .61791 L
.19976 .61791 L
.20167 .6179 L
.20382 .6179 L
.20582 .6179 L
.20792 .61789 L
.21011 .61789 L
.21212 .61789 L
.2139 .61788 L
.21596 .61788 L
.21813 .61788 L
.22013 .61787 L
.22192 .61787 L
.22346 .61787 L
.22507 .61786 L
Mistroke
.22657 .61786 L
.22791 .61786 L
.22945 .61786 L
.2311 .61785 L
.23263 .61785 L
.23402 .61785 L
.23543 .61784 L
.23673 .61784 L
.23813 .61784 L
.23962 .61783 L
.24107 .61783 L
.24257 .61783 L
.24383 .61782 L
.24519 .61782 L
.24646 .61782 L
.24761 .61781 L
.24885 .61781 L
.25015 .61781 L
.25126 .6178 L
.25229 .6178 L
.2534 .6178 L
.25458 .6178 L
.25561 .61779 L
.25672 .61779 L
.25775 .61779 L
.2587 .61778 L
.26046 .61778 L
.26134 .61778 L
.2623 .61777 L
.26334 .61777 L
.26442 .61777 L
.26533 .61776 L
.26631 .61776 L
.26737 .61776 L
.26848 .61775 L
.26952 .61775 L
.27045 .61775 L
.27138 .61775 L
.27223 .61774 L
.27316 .61774 L
.27413 .61774 L
.27496 .61773 L
.27573 .61773 L
.27657 .61773 L
.27746 .61772 L
.27827 .61772 L
.27911 .61772 L
.2799 .61772 L
.28061 .61771 L
.28145 .61771 L
Mistroke
.28234 .61771 L
.28317 .6177 L
.28393 .6177 L
.28529 .61769 L
.28605 .61769 L
.28676 .61769 L
.2875 .61769 L
.28817 .61768 L
.28893 .61768 L
.28965 .61768 L
.2909 .61767 L
.29152 .61767 L
.2922 .61767 L
.29339 .61766 L
.29449 .61766 L
.29512 .61765 L
.29577 .61765 L
.29693 .61765 L
.29756 .61764 L
.29815 .61764 L
.29879 .61764 L
.29947 .61763 L
.3002 .61763 L
.30088 .61763 L
.30162 .61762 L
.30241 .61762 L
.30322 .61762 L
.30399 .61761 L
.30467 .61761 L
.30541 .61761 L
.30611 .6176 L
.30673 .6176 L
.30745 .6176 L
.30811 .61759 L
.30873 .61759 L
.30929 .61759 L
.3099 .61758 L
.31055 .61758 L
.31125 .61758 L
.31188 .61757 L
.3126 .61757 L
.31329 .61757 L
.31391 .61756 L
.31448 .61756 L
.31511 .61756 L
.31576 .61755 L
.31642 .61755 L
.31711 .61754 L
.31777 .61754 L
.31836 .61754 L
Mistroke
.31905 .61753 L
.3198 .61753 L
.3205 .61753 L
.32114 .61752 L
.32173 .61752 L
.32227 .61751 L
.32285 .61751 L
.32347 .61751 L
.324 .6175 L
.3245 .6175 L
.32504 .6175 L
.32561 .61749 L
.32612 .61749 L
.32667 .61749 L
.32718 .61748 L
.32766 .61748 L
.32808 .61748 L
.32855 .61748 L
.32903 .61747 L
.32948 .61747 L
.32998 .61747 L
.33051 .61746 L
.33107 .61746 L
.33159 .61746 L
.33207 .61745 L
.3325 .61745 L
.333 .61745 L
.33347 .61744 L
.33398 .61744 L
.33451 .61744 L
.33497 .61743 L
.33546 .61743 L
.33599 .61743 L
.33656 .61742 L
.33709 .61742 L
.33758 .61741 L
.33804 .61741 L
.33848 .61741 L
.33887 .61741 L
.33929 .6174 L
.33974 .6174 L
.34022 .6174 L
.34063 .61739 L
.34107 .61739 L
.34148 .61739 L
.34186 .61738 L
.34229 .61738 L
.3427 .61738 L
.34307 .61737 L
.34342 .61737 L
Mistroke
.34379 .61737 L
.34419 .61737 L
.34462 .61736 L
.34501 .61736 L
.34543 .61736 L
.34588 .61735 L
.34627 .61735 L
.34662 .61735 L
.34702 .61734 L
.34743 .61734 L
.34789 .61734 L
.34837 .61733 L
.34883 .61733 L
.34925 .61733 L
.34963 .61732 L
.35004 .61732 L
.35047 .61732 L
.35087 .61731 L
.35125 .61731 L
.35159 .61731 L
.35197 .6173 L
.35237 .6173 L
.35272 .6173 L
.35304 .61729 L
.3534 .61729 L
.35378 .61729 L
.35409 .61729 L
.35443 .61728 L
.35479 .61728 L
.35512 .61728 L
.35572 .61727 L
.35604 .61727 L
.35634 .61727 L
.3567 .61726 L
.35707 .61726 L
.35739 .61726 L
.35774 .61725 L
.35811 .61725 L
.35851 .61725 L
.35888 .61724 L
.35922 .61724 L
.35956 .61724 L
.35987 .61723 L
.36022 .61723 L
.36058 .61723 L
.36093 .61722 L
.36125 .61722 L
.36154 .61722 L
.36186 .61722 L
.36217 .61721 L
Mistroke
.36251 .61721 L
.36283 .61721 L
.36312 .6172 L
.36338 .6172 L
.36367 .6172 L
.36398 .61719 L
.36426 .61719 L
.36457 .61719 L
.36489 .61719 L
.36517 .61718 L
.36547 .61718 L
.3658 .61718 L
.36614 .61717 L
.36647 .61717 L
.36676 .61717 L
.36704 .61716 L
.36733 .61716 L
.36786 .61716 L
.36814 .61715 L
.36841 .61715 L
.36871 .61715 L
.36902 .61714 L
.36929 .61714 L
.36959 .61714 L
.36986 .61714 L
.37012 .61713 L
.3706 .61713 L
.37087 .61713 L
.37111 .61712 L
.37139 .61712 L
.37171 .61712 L
.37204 .61711 L
.37234 .61711 L
.37263 .61711 L
.37288 .6171 L
.37318 .6171 L
.37346 .6171 L
.37376 .61709 L
.37408 .61709 L
.37436 .61709 L
.37465 .61709 L
.37497 .61708 L
.37532 .61708 L
.37564 .61707 L
.37593 .61707 L
.3762 .61707 L
.37649 .61706 L
.37676 .61706 L
.37701 .61706 L
.3773 .61706 L
Mistroke
.37761 .61705 L
.3779 .61705 L
.37817 .61705 L
.37842 .61704 L
.37865 .61704 L
.37891 .61704 L
.37918 .61703 L
.37947 .61703 L
.37974 .61703 L
.37998 .61703 L
.38024 .61702 L
.38051 .61702 L
.38076 .61702 L
.38103 .61701 L
.38131 .61701 L
.38156 .61701 L
.38179 .617 L
.38204 .617 L
.38231 .617 L
.3826 .61699 L
.38291 .61699 L
.38321 .61699 L
.38348 .61698 L
.38373 .61698 L
.384 .61698 L
.38428 .61697 L
.38454 .61697 L
.38481 .61697 L
.38511 .61696 L
.38539 .61696 L
.38565 .61696 L
.38591 .61695 L
.38614 .61695 L
.38641 .61695 L
.38666 .61694 L
.38693 .61694 L
.38722 .61694 L
.38747 .61693 L
.38773 .61693 L
.38802 .61693 L
.38833 .61692 L
.38862 .61692 L
.38888 .61692 L
.38911 .61691 L
.38935 .61691 L
.38961 .61691 L
.38985 .6169 L
.39008 .6169 L
.39028 .6169 L
.3905 .6169 L
Mistroke
.39074 .61689 L
.39098 .61689 L
.39123 .61689 L
.39147 .61688 L
.39169 .61688 L
.39194 .61688 L
.39221 .61687 L
.39247 .61687 L
.39271 .61687 L
.39291 .61686 L
.39313 .61686 L
.39334 .61686 L
.39353 .61685 L
.39375 .61685 L
.39398 .61685 L
.39421 .61685 L
.39442 .61684 L
.39465 .61684 L
.39487 .61684 L
.39507 .61683 L
.39528 .61683 L
.39551 .61683 L
.39575 .61682 L
.39596 .61682 L
.39618 .61682 L
.3964 .61682 L
.39662 .61681 L
.39681 .61681 L
.39702 .61681 L
.39725 .6168 L
.39749 .6168 L
.39771 .6168 L
.39792 .61679 L
.39811 .61679 L
.39832 .61679 L
.39854 .61678 L
.39875 .61678 L
.39896 .61678 L
.39916 .61678 L
.39934 .61677 L
.39953 .61677 L
.3999 .61677 L
.40009 .61676 L
.40028 .61676 L
.40061 .61675 L
.40096 .61675 L
.40116 .61675 L
.40137 .61674 L
.40156 .61674 L
.40175 .61674 L
Mistroke
.40193 .61673 L
.40213 .61673 L
.40247 .61673 L
.40267 .61672 L
.40288 .61672 L
.4031 .61672 L
.40331 .61671 L
.40368 .61671 L
.40388 .61671 L
.40407 .6167 L
.40429 .6167 L
.40449 .6167 L
.40471 .61669 L
.40494 .61669 L
.40514 .61669 L
.40533 .61668 L
.40554 .61668 L
.40576 .61668 L
.40596 .61667 L
.40615 .61667 L
.40635 .61667 L
.40657 .61666 L
.40677 .61666 L
.40697 .61666 L
.40714 .61665 L
.40733 .61665 L
.4075 .61665 L
.40768 .61665 L
.40801 .61664 L
.40819 .61664 L
.40838 .61663 L
.40873 .61663 L
.4089 .61663 L
.40909 .61662 L
.40927 .61662 L
.40946 .61662 L
.40964 .61661 L
.40981 .61661 L
.41013 .61661 L
.41031 .6166 L
.41048 .6166 L
.41068 .6166 L
.41086 .61659 L
.41122 .61659 L
.41143 .61658 L
.41161 .61658 L
.41194 .61658 L
.4121 .61657 L
.41228 .61657 L
.41243 .61657 L
Mistroke
.4126 .61657 L
.41289 .61656 L
.41307 .61656 L
.41326 .61655 L
.41343 .61655 L
.41361 .61655 L
.41377 .61655 L
.41394 .61654 L
.41409 .61654 L
.41425 .61654 L
.41456 .61653 L
.41473 .61653 L
.41488 .61653 L
.41516 .61652 L
.41545 .61652 L
.41562 .61651 L
.4158 .61651 L
.41596 .61651 L
.41612 .6165 L
.41644 .6165 L
.41673 .61649 L
.417 .61649 L
.41729 .61648 L
.41756 .61648 L
.41781 .61647 L
.4181 .61647 L
.41827 .61647 L
.41842 .61646 L
.4187 .61646 L
.41899 .61645 L
.4193 .61645 L
.41947 .61644 L
.41963 .61644 L
.41993 .61644 L
.42021 .61643 L
.42037 .61643 L
.42054 .61642 L
.42085 .61642 L
.42102 .61642 L
.42117 .61641 L
.42135 .61641 L
.42153 .61641 L
.42186 .6164 L
.42203 .6164 L
.42219 .61639 L
.42249 .61639 L
.4228 .61638 L
.42297 .61638 L
.42316 .61638 L
.42333 .61637 L
Mistroke
.4235 .61637 L
.42366 .61637 L
.42383 .61636 L
.42414 .61636 L
.42428 .61635 L
.42444 .61635 L
.42473 .61634 L
.42502 .61634 L
.42528 .61633 L
.42558 .61633 L
.42586 .61632 L
.42601 .61632 L
.42618 .61632 L
.42648 .61631 L
.42674 .61631 L
.42703 .6163 L
.42732 .61629 L
.4276 .61629 L
.42786 .61628 L
.42809 .61628 L
.42836 .61627 L
.42861 .61627 L
.42891 .61626 L
.42918 .61626 L
.42948 .61625 L
.42965 .61625 L
.4298 .61624 L
.4301 .61624 L
.43027 .61623 L
.43043 .61623 L
.43073 .61622 L
.43104 .61622 L
.43132 .61621 L
.43163 .61621 L
.43192 .6162 L
.43218 .61619 L
.43242 .61619 L
.43269 .61618 L
.43296 .61618 L
.43321 .61617 L
.43351 .61617 L
.43368 .61616 L
.43384 .61616 L
.43413 .61615 L
.43443 .61615 L
.4347 .61614 L
.43495 .61613 L
.43523 .61613 L
.43553 .61612 L
.43582 .61611 L
Mistroke
.43599 .61611 L
.43615 .61611 L
.43646 .6161 L
.43664 .6161 L
.4368 .61609 L
.43708 .61609 L
.43739 .61608 L
.43768 .61607 L
.43794 .61607 L
.43819 .61606 L
.43842 .61606 L
.43863 .61605 L
.43886 .61605 L
.43911 .61604 L
.43937 .61603 L
.43962 .61603 L
.43984 .61602 L
.44006 .61602 L
.44027 .61601 L
.4405 .61601 L
.44074 .616 L
.44094 .616 L
.44114 .61599 L
.44135 .61599 L
.44157 .61598 L
.44184 .61598 L
.44208 .61597 L
.44234 .61596 L
.44262 .61596 L
.44286 .61595 L
.44309 .61595 L
.44333 .61594 L
.4436 .61593 L
.44382 .61593 L
.44406 .61592 L
.44431 .61592 L
.44454 .61591 L
.44476 .61591 L
.44496 .6159 L
.44519 .6159 L
.4454 .61589 L
.44563 .61589 L
.44584 .61588 L
.44607 .61587 L
.44632 .61587 L
.44657 .61586 L
.44682 .61586 L
.44703 .61585 L
.44727 .61584 L
.44748 .61584 L
Mistroke
.4477 .61583 L
.44793 .61583 L
.44815 .61582 L
.44836 .61582 L
.44854 .61581 L
.44875 .61581 L
.44896 .6158 L
.4492 .61579 L
.44941 .61579 L
.44966 .61578 L
.44989 .61578 L
.4501 .61577 L
.4503 .61577 L
.45052 .61576 L
.45074 .61575 L
.45097 .61575 L
.45122 .61574 L
.45145 .61574 L
.45166 .61573 L
.45191 .61572 L
.45217 .61572 L
.45242 .61571 L
.45266 .6157 L
.45285 .6157 L
.45306 .61569 L
.45328 .61569 L
.45348 .61568 L
.45384 .61567 L
.45405 .61567 L
.45423 .61566 L
.45443 .61566 L
.45465 .61565 L
.45488 .61564 L
.45509 .61564 L
.45548 .61563 L
.4557 .61562 L
.4559 .61561 L
.45627 .6156 L
.45645 .6156 L
.45665 .61559 L
.45682 .61559 L
.45701 .61558 L
.45734 .61557 L
.45753 .61557 L
.45775 .61556 L
.45794 .61556 L
.45813 .61555 L
.45831 .61555 L
.45851 .61554 L
.45885 .61553 L
Mistroke
.45903 .61553 L
.45921 .61552 L
.45938 .61552 L
.45956 .61551 L
.45973 .61551 L
.45989 .6155 L
.46019 .61549 L
.46038 .61549 L
.46055 .61548 L
.46073 .61548 L
.46093 .61547 L
.46126 .61546 L
.46144 .61546 L
.46161 .61545 L
.46177 .61545 L
.46194 .61544 L
.46209 .61544 L
.46225 .61543 L
.46255 .61542 L
.46283 .61541 L
.46298 .61541 L
.46314 .6154 L
.46344 .6154 L
.46361 .61539 L
.46379 .61538 L
.46411 .61537 L
.46429 .61537 L
.46448 .61536 L
.46465 .61536 L
.46483 .61535 L
.46499 .61535 L
.46516 .61534 L
.46547 .61533 L
.46566 .61533 L
.46583 .61532 L
.46601 .61532 L
.46621 .61531 L
.4664 .6153 L
.46657 .6153 L
.46673 .61529 L
.4669 .61529 L
.4672 .61528 L
.46749 .61527 L
.46765 .61526 L
.46782 .61526 L
.46813 .61525 L
.46828 .61524 L
.46845 .61524 L
.46874 .61523 L
.46892 .61522 L
Mistroke
.4691 .61522 L
.46941 .61521 L
.46958 .6152 L
.46974 .6152 L
.46992 .61519 L
.47009 .61519 L
.47044 .61517 L
.47062 .61517 L
.4708 .61516 L
.47096 .61516 L
.47112 .61515 L
.47141 .61514 L
.47159 .61514 L
.47176 .61513 L
.47207 .61512 L
.47223 .61511 L
.47241 .61511 L
.47259 .6151 L
.47277 .6151 L
.47295 .61509 L
.47311 .61508 L
.47342 .61507 L
.4736 .61507 L
.47378 .61506 L
.47396 .61506 L
.47416 .61505 L
.4745 .61504 L
.47468 .61503 L
.47486 .61502 L
.47503 .61502 L
.47522 .61501 L
.47539 .61501 L
.47557 .615 L
.47572 .61499 L
.4759 .61499 L
.4762 .61498 L
.47639 .61497 L
.47656 .61496 L
.47674 .61496 L
.47694 .61495 L
.47712 .61494 L
.47729 .61494 L
.47762 .61493 L
.47778 .61492 L
.47795 .61491 L
.4781 .61491 L
.47826 .6149 L
.47856 .61489 L
.47883 .61488 L
.47898 .61488 L
Mistroke
.47915 .61487 L
.47944 .61486 L
.47962 .61485 L
.47979 .61485 L
.48011 .61484 L
.48027 .61483 L
.48043 .61482 L
.48061 .61482 L
.48078 .61481 L
.48112 .6148 L
.4813 .61479 L
.48148 .61478 L
.48179 .61477 L
.48208 .61476 L
.48225 .61476 L
.4824 .61475 L
.48273 .61474 L
.48291 .61473 L
.4831 .61472 L
.48343 .61471 L
.48361 .6147 L
.48378 .6147 L
.48398 .61469 L
.48417 .61468 L
.48433 .61468 L
.48451 .61467 L
.48482 .61466 L
.48499 .61465 L
.48517 .61464 L
.48533 .61464 L
.4855 .61463 L
.48582 .61462 L
.48599 .61461 L
.48615 .61461 L
.48643 .61459 L
.48674 .61458 L
.48691 .61458 L
.48709 .61457 L
.48725 .61456 L
.48743 .61455 L
.48758 .61455 L
.48775 .61454 L
.48805 .61453 L
.48821 .61452 L
.48835 .61452 L
.48863 .61451 L
.48893 .61449 L
.4891 .61449 L
.48926 .61448 L
.48954 .61447 L
Mistroke
.4898 .61446 L
.4901 .61445 L
.49037 .61443 L
.49068 .61442 L
.49085 .61441 L
.49101 .61441 L
.49133 .61439 L
.49151 .61439 L
.49167 .61438 L
.49196 .61437 L
.49213 .61436 L
.49228 .61435 L
.49256 .61434 L
.49285 .61433 L
.49301 .61432 L
.49316 .61432 L
.49332 .61431 L
.4935 .6143 L
.49379 .61429 L
.4941 .61428 L
.49428 .61427 L
.49444 .61426 L
.49461 .61425 L
.4948 .61425 L
.49498 .61424 L
.49514 .61423 L
.49545 .61422 L
.49571 .61421 L
.49596 .61419 L
.49618 .61418 L
.49642 .61417 L
.49668 .61416 L
.49695 .61415 L
.49718 .61414 L
.49744 .61413 L
.49768 .61412 L
.49789 .61411 L
.49813 .6141 L
.49838 .61409 L
.4986 .61408 L
.4988 .61407 L
.49902 .61406 L
.49926 .61404 L
.49953 .61403 L
.49978 .61402 L
.50006 .61401 L
.50035 .61399 L
.5006 .61398 L
.50084 .61397 L
.5011 .61396 L
Mistroke
.50137 .61395 L
.50162 .61393 L
.50188 .61392 L
.50213 .61391 L
.50236 .6139 L
.50259 .61389 L
.5028 .61388 L
.50303 .61387 L
.50326 .61386 L
.50349 .61384 L
.50371 .61383 L
.50395 .61382 L
.50421 .61381 L
.50448 .6138 L
.50473 .61378 L
.50496 .61377 L
.50521 .61376 L
.50542 .61375 L
.50566 .61374 L
.5059 .61373 L
.50613 .61372 L
.50634 .6137 L
.50653 .6137 L
.50674 .61368 L
.50697 .61367 L
.50721 .61366 L
.50744 .61365 L
.50769 .61364 L
.50793 .61362 L
.50816 .61361 L
.50836 .6136 L
.50859 .61359 L
.50882 .61358 L
.50906 .61357 L
.50932 .61355 L
.50956 .61354 L
.50978 .61353 L
.51003 .61352 L
.51031 .6135 L
.51057 .61349 L
.51081 .61348 L
.51105 .61346 L
.51132 .61345 L
.51159 .61343 L
.51185 .61342 L
.51208 .61341 L
.5123 .6134 L
.51255 .61338 L
.51279 .61337 L
.51304 .61336 L
Mistroke
.51331 .61334 L
.51354 .61333 L
.51379 .61332 L
.51406 .6133 L
.51435 .61328 L
.51462 .61327 L
.51487 .61326 L
.5151 .61324 L
.51534 .61323 L
.51557 .61322 L
.51578 .61321 L
.51602 .61319 L
.51629 .61318 L
.51653 .61316 L
.51676 .61315 L
.51697 .61314 L
.51717 .61313 L
.51739 .61311 L
.51762 .6131 L
.51787 .61309 L
.51809 .61307 L
.5183 .61306 L
.51852 .61305 L
.51872 .61304 L
.51893 .61303 L
.51913 .61301 L
.51932 .613 L
.51952 .61299 L
.51974 .61298 L
.51997 .61297 L
.52019 .61295 L
.52042 .61294 L
.52063 .61293 L
.52082 .61292 L
.52103 .6129 L
.52126 .61289 L
.5215 .61288 L
.5217 .61286 L
.52192 .61285 L
.52213 .61284 L
.52233 .61283 L
.52269 .6128 L
.52289 .61279 L
.52308 .61278 L
.52329 .61277 L
.52351 .61275 L
.5237 .61274 L
.5239 .61273 L
.52408 .61272 L
.52427 .61271 L
Mistroke
.52444 .6127 L
.52463 .61268 L
.52496 .61266 L
.52519 .61265 L
.5254 .61264 L
.52564 .61262 L
.52589 .61261 L
.5261 .61259 L
.5263 .61258 L
.52652 .61257 L
.52676 .61255 L
.52695 .61254 L
.52717 .61253 L
.52739 .61251 L
.5276 .6125 L
.52797 .61247 L
.52818 .61246 L
.52836 .61245 L
.52859 .61243 L
.52882 .61242 L
.52902 .61241 L
.52924 .61239 L
.52948 .61238 L
.52973 .61236 L
.52997 .61234 L
.53018 .61233 L
.5304 .61231 L
.5306 .6123 L
.53082 .61229 L
.53105 .61227 L
.53127 .61226 L
.53148 .61224 L
.53168 .61223 L
.53187 .61222 L
.53223 .61219 L
.53242 .61218 L
.53262 .61217 L
.53283 .61215 L
.53303 .61214 L
.53321 .61213 L
.53341 .61211 L
.53357 .6121 L
.53375 .61209 L
.53407 .61207 L
.53425 .61205 L
.53445 .61204 L
.53479 .61202 L
.535 .612 L
.53519 .61199 L
.5354 .61197 L
Mistroke
.53562 .61196 L
.53583 .61194 L
.53603 .61193 L
.53621 .61192 L
.5364 .6119 L
.53658 .61189 L
.53677 .61188 L
.5371 .61185 L
.53729 .61184 L
.53749 .61182 L
.53768 .61181 L
.53785 .6118 L
.53816 .61178 L
.53832 .61176 L
.53849 .61175 L
.53877 .61173 L
.53893 .61172 L
.53908 .61171 L
.53926 .6117 L
.53943 .61168 L
.53974 .61166 L
.53991 .61165 L
.54007 .61164 L
.54024 .61162 L
.54043 .61161 L
.5406 .6116 L
.54075 .61159 L
.54105 .61156 L
.54122 .61155 L
.5414 .61154 L
.54173 .61151 L
.54191 .6115 L
.54207 .61149 L
.54225 .61147 L
.54243 .61146 L
.54261 .61144 L
.54277 .61143 L
.54308 .61141 L
.54336 .61139 L
.54365 .61136 L
.54392 .61134 L
.54417 .61132 L
.54446 .6113 L
.54462 .61129 L
.54477 .61128 L
.54505 .61126 L
.54534 .61123 L
.54562 .61121 L
.54592 .61119 L
.5462 .61116 L
Mistroke
.54645 .61114 L
.54674 .61112 L
.5469 .61111 L
.54704 .6111 L
.54734 .61107 L
.54751 .61106 L
.54766 .61105 L
.54796 .61102 L
.54813 .61101 L
.54829 .61099 L
.54861 .61097 L
.54879 .61095 L
.54895 .61094 L
.54924 .61092 L
.5494 .6109 L
.54955 .61089 L
.54983 .61087 L
.55012 .61084 L
.55028 .61083 L
.55043 .61082 L
.55059 .6108 L
.55076 .61079 L
.55105 .61076 L
.55121 .61075 L
.55136 .61074 L
.55154 .61072 L
.5517 .61071 L
.55201 .61068 L
.55219 .61067 L
.55235 .61065 L
.55254 .61064 L
.55271 .61062 L
.55297 .6106 L
.55326 .61058 L
.55352 .61055 L
.55377 .61053 L
.55406 .61051 L
.55432 .61048 L
.55461 .61046 L
.55478 .61044 L
.55492 .61043 L
.55522 .6104 L
.5555 .61038 L
.55579 .61035 L
.55595 .61034 L
.55611 .61032 L
.55639 .6103 L
.55668 .61027 L
.55696 .61025 L
.55721 .61022 L
Mistroke
.55746 .6102 L
.55769 .61018 L
.55795 .61016 L
.55823 .61013 L
.5585 .61011 L
.55878 .61008 L
.55902 .61006 L
.55929 .61003 L
.55954 .61001 L
.55976 .60999 L
.56002 .60996 L
.56027 .60994 L
.56049 .60992 L
.5607 .6099 L
.56093 .60988 L
.56117 .60986 L
.56143 .60983 L
.56166 .60981 L
.56194 .60978 L
.56219 .60976 L
.56243 .60973 L
.56265 .60971 L
.56289 .60969 L
.56315 .60966 L
.56343 .60964 L
.56357 .60962 L
.56373 .60961 L
.56401 .60958 L
.56427 .60955 L
.56452 .60953 L
.56475 .60951 L
.565 .60948 L
.56527 .60945 L
.56551 .60943 L
.56576 .6094 L
.56597 .60938 L
.56621 .60936 L
.56646 .60933 L
.56673 .60931 L
.56698 .60928 L
.56721 .60926 L
.56744 .60923 L
.56765 .60921 L
.56789 .60919 L
.56814 .60916 L
.56837 .60914 L
.56859 .60911 L
.56879 .60909 L
.56901 .60907 L
.56921 .60905 L
Mistroke
.56941 .60903 L
.56962 .60901 L
.56984 .60898 L
.57008 .60896 L
.5703 .60893 L
.5705 .60891 L
.57072 .60889 L
.57091 .60887 L
.57111 .60885 L
.57133 .60882 L
.57153 .6088 L
.57189 .60876 L
.57209 .60874 L
.57227 .60872 L
.57246 .6087 L
.57268 .60868 L
.57287 .60866 L
.57306 .60864 L
.5734 .6086 L
.57358 .60858 L
.57377 .60856 L
.57397 .60853 L
.57416 .60851 L
.57438 .60849 L
.57458 .60847 L
.57494 .60843 L
.57515 .6084 L
.57534 .60838 L
.57551 .60836 L
.5757 .60834 L
.57587 .60832 L
.57605 .6083 L
.5762 .60828 L
.57638 .60826 L
.57668 .60823 L
.57687 .60821 L
.57704 .60819 L
.57722 .60817 L
.57742 .60814 L
.5776 .60812 L
.57777 .6081 L
.5781 .60806 L
.57828 .60804 L
.57845 .60802 L
.57863 .608 L
.57883 .60798 L
.57916 .60794 L
.57934 .60792 L
.57951 .6079 L
.57971 .60787 L
Mistroke
.5799 .60785 L
.58025 .60781 L
.58045 .60778 L
.58063 .60776 L
.58083 .60774 L
.58104 .60771 L
.58123 .60769 L
.5814 .60767 L
.58159 .60765 L
.58179 .60762 L
.58198 .6076 L
.58215 .60758 L
.58249 .60753 L
.58267 .60751 L
.58285 .60749 L
.58304 .60747 L
.58325 .60744 L
.58345 .60741 L
.58367 .60739 L
.58385 .60736 L
.58405 .60734 L
.58425 .60731 L
.58443 .60729 L
.58462 .60727 L
.58483 .60724 L
.58518 .6072 L
.58538 .60717 L
.58556 .60715 L
.58574 .60712 L
.5859 .6071 L
.58621 .60706 L
.58639 .60704 L
.58655 .60702 L
.58691 .60697 L
.5871 .60695 L
.5873 .60692 L
.58749 .6069 L
.58767 .60687 L
.588 .60683 L
.58817 .60681 L
.58835 .60678 L
.58855 .60676 L
.58873 .60673 L
.58894 .6067 L
.58914 .60668 L
.58949 .60663 L
.58968 .6066 L
.58986 .60658 L
.59016 .60654 L
.59033 .60652 L
Mistroke
.59048 .6065 L
.59077 .60646 L
.59091 .60644 L
.59106 .60642 L
.59139 .60637 L
.59157 .60635 L
.59174 .60632 L
.59189 .6063 L
.59206 .60628 L
.59237 .60624 L
.59253 .60621 L
.59268 .60619 L
.59286 .60617 L
.59303 .60614 L
.59336 .6061 L
.59354 .60607 L
.59372 .60605 L
.59387 .60602 L
.59402 .606 L
.59431 .60596 L
.59448 .60594 L
.59464 .60591 L
.59495 .60587 L
.59512 .60584 L
.59532 .60582 L
.59549 .60579 L
.59567 .60577 L
.59583 .60574 L
.59601 .60572 L
.59632 .60567 L
.59651 .60564 L
.59668 .60562 L
.59687 .60559 L
.59707 .60556 L
.5974 .60551 L
.59759 .60548 L
.59776 .60546 L
.59792 .60543 L
.5981 .60541 L
.59825 .60538 L
.59841 .60536 L
.59856 .60534 L
.59872 .60531 L
.599 .60527 L
.59917 .60524 L
.59933 .60522 L
.5995 .60519 L
.59968 .60517 L
.59985 .60514 L
.6 .60512 L
Mistroke
.60031 .60507 L
.60048 .60504 L
.60063 .60502 L
.6008 .60499 L
.60098 .60497 L
.60129 .60492 L
.60145 .60489 L
.60161 .60487 L
.60179 .60484 L
.60197 .60481 L
.6023 .60476 L
.60248 .60473 L
.60265 .6047 L
.60283 .60467 L
.60302 .60464 L
.6032 .60461 L
.60336 .60459 L
.60368 .60454 L
.60386 .60451 L
.60405 .60448 L
.6044 .60442 L
.60458 .60439 L
.60476 .60436 L
.60495 .60433 L
.60514 .6043 L
.60547 .60425 L
.60565 .60422 L
.60582 .60419 L
.60599 .60416 L
.60615 .60413 L
.60645 .60408 L
.60661 .60406 L
.60676 .60403 L
.60694 .604 L
.60711 .60397 L
.60738 .60393 L
.60753 .6039 L
.60768 .60388 L
.60784 .60385 L
.608 .60382 L
.60829 .60377 L
.60846 .60374 L
.60861 .60372 L
.60877 .60369 L
.60895 .60366 L
.60911 .60363 L
.60926 .60361 L
.60956 .60355 L
.60972 .60353 L
.60987 .6035 L
Mistroke
.61005 .60347 L
.61022 .60344 L
.61056 .60338 L
.61074 .60335 L
.61092 .60332 L
.61109 .60329 L
.61125 .60326 L
.61154 .6032 L
.61171 .60318 L
.61186 .60315 L
.61204 .60312 L
.61221 .60309 L
.61248 .60304 L
.61263 .60301 L
.61278 .60298 L
.61294 .60295 L
.6131 .60293 L
.61338 .60287 L
.61353 .60285 L
.6137 .60281 L
.61387 .60278 L
.61404 .60275 L
.6142 .60272 L
.61435 .6027 L
.61464 .60264 L
.61482 .60261 L
.61497 .60258 L
.61516 .60254 L
.61534 .60251 L
.61565 .60245 L
.61583 .60242 L
.61599 .60239 L
.61624 .60234 L
.6165 .60229 L
.61675 .60224 L
.61698 .6022 L
.61725 .60215 L
.6174 .60212 L
.61753 .60209 L
.61779 .60205 L
.61805 .60199 L
.61831 .60194 L
.61845 .60192 L
.61858 .60189 L
.61882 .60184 L
.61907 .60179 L
.61921 .60177 L
.61935 .60174 L
.6196 .60169 L
.61976 .60166 L
Mistroke
.61991 .60163 L
.62018 .60157 L
.6204 .60153 L
.62064 .60148 L
.62089 .60143 L
.62112 .60139 L
.62133 .60134 L
.62153 .6013 L
.62176 .60126 L
.62197 .60121 L
.6222 .60117 L
.62245 .60112 L
.62268 .60107 L
.62288 .60102 L
.62313 .60097 L
.62339 .60092 L
.62364 .60087 L
.62387 .60082 L
.62412 .60077 L
.62434 .60072 L
.6246 .60067 L
.62484 .60061 L
.62507 .60057 L
.62527 .60052 L
.6255 .60047 L
.62574 .60042 L
.62598 .60037 L
.62624 .60031 L
.62648 .60026 L
.6267 .60021 L
.62694 .60016 L
.62708 .60013 L
.62721 .6001 L
.62747 .60004 L
.62761 .60001 L
.62775 .59998 L
.628 .59993 L
.62812 .5999 L
.62826 .59987 L
.6285 .59982 L
.62873 .59976 L
.62887 .59973 L
.629 .5997 L
.62924 .59965 L
.6295 .59959 L
.62965 .59956 L
.62978 .59953 L
.63001 .59948 L
.63022 .59943 L
.63045 .59937 L
Mistroke
.6307 .59932 L
.63094 .59926 L
.6312 .5992 L
.63142 .59915 L
.63166 .59909 L
.63178 .59907 L
.63192 .59903 L
.63206 .599 L
.6322 .59897 L
.63246 .59891 L
.6327 .59885 L
.63284 .59882 L
.633 .59878 L
.63313 .59875 L
.63328 .59871 L
.63342 .59868 L
.63355 .59865 L
.63381 .59858 L
.63394 .59855 L
.63407 .59852 L
.6343 .59846 L
.63456 .5984 L
.6347 .59837 L
.63484 .59833 L
.63508 .59828 L
.6353 .59822 L
.63556 .59816 L
.6358 .5981 L
.63601 .59805 L
.63624 .59799 L
.63648 .59793 L
.6367 .59787 L
.63691 .59782 L
.63711 .59777 L
.63732 .59772 L
.63754 .59766 L
.63776 .59761 L
.638 .59755 L
.63823 .59749 L
.63843 .59744 L
.63867 .59737 L
.6388 .59734 L
.63893 .59731 L
.63915 .59725 L
.63939 .59719 L
.63951 .59715 L
.63965 .59712 L
.63988 .59706 L
.64011 .597 L
.64033 .59694 L
Mistroke
.64058 .59687 L
.64081 .59681 L
.64094 .59678 L
.64107 .59674 L
.64132 .59668 L
.64154 .59662 L
.64177 .59656 L
.64189 .59652 L
.64202 .59649 L
.64224 .59643 L
.64247 .59637 L
.64269 .59631 L
.64288 .59626 L
.64308 .5962 L
.64333 .59613 L
.64356 .59607 L
.64369 .59603 L
.64381 .596 L
.64395 .59596 L
.64408 .59592 L
.64434 .59585 L
.64447 .59581 L
.64462 .59577 L
.64476 .59573 L
.64488 .5957 L
.64513 .59563 L
.64535 .59556 L
.64559 .5955 L
.64579 .59544 L
.64602 .59537 L
.64626 .59531 L
.64639 .59527 L
.64651 .59523 L
.64675 .59516 L
.64697 .5951 L
.64716 .59505 L
.64736 .59499 L
.64754 .59493 L
.64772 .59488 L
.64791 .59483 L
.64811 .59476 L
.64833 .5947 L
.64853 .59464 L
.64875 .59458 L
.64895 .59452 L
.64913 .59446 L
.64932 .5944 L
.64954 .59434 L
.64976 .59427 L
.64995 .59421 L
Mistroke
.65016 .59415 L
.65035 .59409 L
.65054 .59403 L
.6507 .59398 L
.65088 .59393 L
.65107 .59387 L
.65128 .5938 L
.65147 .59374 L
.65165 .59369 L
.65182 .59363 L
.65199 .59358 L
.65217 .59353 L
.65235 .59347 L
.65252 .59342 L
.65267 .59337 L
.65284 .59331 L
.65301 .59326 L
.65322 .59319 L
.65344 .59312 L
.65363 .59306 L
.65383 .593 L
.65405 .59293 L
.65417 .59289 L
.65428 .59285 L
.65449 .59278 L
.65471 .59271 L
.6549 .59265 L
.6551 .59258 L
.65529 .59252 L
.65546 .59246 L
.65566 .59239 L
.65588 .59232 L
.65609 .59225 L
.65628 .59219 L
.65645 .59213 L
.65663 .59207 L
.6568 .59201 L
.65695 .59196 L
.65713 .5919 L
.65733 .59183 L
.65751 .59177 L
.65769 .59171 L
.65786 .59165 L
.65802 .5916 L
.6582 .59153 L
.6584 .59147 L
.65859 .5914 L
.65879 .59133 L
.65896 .59127 L
.65915 .5912 L
Mistroke
.65933 .59114 L
.6595 .59108 L
.65969 .59101 L
.65989 .59094 L
.6601 .59087 L
.6603 .5908 L
.66048 .59073 L
.66067 .59066 L
.66085 .5906 L
.66102 .59054 L
.66121 .59047 L
.66139 .5904 L
.66156 .59034 L
.66171 .59029 L
.66188 .59022 L
.66205 .59016 L
.66225 .59009 L
.66242 .59002 L
.66262 .58995 L
.66282 .58988 L
.66299 .58981 L
.66315 .58975 L
.66333 .58968 L
.66352 .58961 L
.66371 .58954 L
.66391 .58947 L
.6641 .58939 L
.66428 .58933 L
.66447 .58925 L
.66468 .58917 L
.66479 .58913 L
.6649 .58909 L
.6651 .58901 L
.66528 .58894 L
.66547 .58887 L
.66563 .5888 L
.6658 .58874 L
.66599 .58866 L
.66619 .58858 L
.66638 .58851 L
.66656 .58844 L
.66673 .58837 L
.66689 .58831 L
.66706 .58824 L
.66725 .58816 L
.66743 .58809 L
.6676 .58802 L
.66774 .58796 L
.66791 .5879 L
.66806 .58783 L
Mistroke
.66821 .58777 L
.66837 .58771 L
.66854 .58764 L
.66872 .58757 L
.66889 .5875 L
.66904 .58743 L
.6692 .58737 L
.66934 .58731 L
.6695 .58724 L
.66966 .58717 L
.66982 .58711 L
.66996 .58705 L
.67009 .587 L
.67023 .58694 L
.67038 .58687 L
.67053 .58681 L
.67069 .58674 L
.67084 .58668 L
.67098 .58662 L
.6711 .58656 L
.67124 .5865 L
.67138 .58644 L
.67152 .58639 L
.67168 .58632 L
.67182 .58625 L
.67196 .5862 L
.6721 .58613 L
.67226 .58606 L
.67242 .58599 L
.67256 .58593 L
.67272 .58586 L
.67285 .5858 L
.673 .58574 L
.67312 .58568 L
.67325 .58563 L
.67339 .58556 L
.67355 .58549 L
.67369 .58543 L
.67382 .58537 L
.67394 .58532 L
.67407 .58526 L
.67422 .58519 L
.67435 .58513 L
.67458 .58503 L
.67472 .58496 L
.67484 .58491 L
.67499 .58484 L
.67513 .58477 L
.67529 .5847 L
.67545 .58463 L
Mistroke
.67561 .58455 L
.67576 .58448 L
.67589 .58442 L
.67603 .58436 L
.67616 .5843 L
.6763 .58423 L
.67645 .58416 L
.67659 .58409 L
.67672 .58403 L
.67684 .58398 L
.67697 .58391 L
.6771 .58385 L
.67724 .58378 L
.67739 .58371 L
.67755 .58364 L
.6777 .58356 L
.67783 .5835 L
.67796 .58344 L
.6781 .58337 L
.67824 .5833 L
.67839 .58323 L
.67855 .58315 L
.6787 .58308 L
.67883 .58301 L
.67899 .58293 L
.67916 .58285 L
.67932 .58277 L
.67947 .5827 L
.67961 .58262 L
.67977 .58255 L
.67991 .58247 L
.68005 .5824 L
.68019 .58234 L
.68031 .58227 L
.68045 .5822 L
.68058 .58214 L
.68072 .58206 L
.68088 .58198 L
.681 .58192 L
.68115 .58185 L
.6813 .58177 L
.68146 .58168 L
.68162 .5816 L
.68176 .58153 L
.68189 .58146 L
.68203 .58139 L
.68214 .58133 L
.68227 .58126 L
.68241 .58118 L
.68256 .58111 L
Mistroke
.6827 .58103 L
.68283 .58096 L
.68296 .58089 L
.68308 .58083 L
.68321 .58076 L
.68335 .58069 L
.68348 .58061 L
.68361 .58055 L
.68372 .58049 L
.68384 .58042 L
.68398 .58035 L
.68411 .58027 L
.68427 .58019 L
.68441 .58011 L
.68456 .58003 L
.68472 .57994 L
.68487 .57986 L
.68501 .57978 L
.68513 .57971 L
.68525 .57965 L
.68538 .57957 L
.68552 .5795 L
.68566 .57942 L
.68581 .57934 L
.68593 .57927 L
.68606 .57919 L
.68623 .5791 L
.68638 .57901 L
.68651 .57894 L
.68666 .57885 L
.68682 .57876 L
.68699 .57866 L
.68713 .57858 L
.68729 .57849 L
.68743 .5784 L
.68757 .57833 L
.68772 .57824 L
.68787 .57815 L
.688 .57807 L
.68813 .578 L
.68826 .57792 L
.68841 .57783 L
.68855 .57775 L
.6887 .57765 L
.68886 .57756 L
.68901 .57747 L
.68916 .57738 L
.68928 .57731 L
.68943 .57722 L
.68957 .57713 L
Mistroke
.68972 .57704 L
.68988 .57694 L
.69001 .57686 L
.69016 .57677 L
.69032 .57667 L
.6905 .57657 L
.69066 .57646 L
.69081 .57637 L
.69093 .57629 L
.69107 .5762 L
.6912 .57612 L
.69132 .57605 L
.69146 .57596 L
.69161 .57587 L
.69175 .57578 L
.69188 .57569 L
.69202 .57561 L
.69214 .57553 L
.69228 .57544 L
.69243 .57534 L
.69258 .57524 L
.69273 .57515 L
.69286 .57506 L
.693 .57497 L
.69312 .57489 L
.69325 .57481 L
.69337 .57473 L
.69348 .57465 L
.69361 .57457 L
.69374 .57448 L
.69388 .57439 L
.69402 .5743 L
.69415 .57421 L
.69429 .57412 L
.6944 .57404 L
.69453 .57395 L
.69467 .57386 L
.69482 .57376 L
.69494 .57368 L
.69508 .57358 L
.69521 .57349 L
.69533 .57341 L
.69544 .57334 L
.69555 .57326 L
.69568 .57317 L
.69581 .57308 L
.69594 .57299 L
.69606 .57291 L
.69617 .57282 L
.69628 .57275 L
Mistroke
.6964 .57267 L
.69652 .57258 L
.69663 .5725 L
.69673 .57243 L
.69684 .57235 L
.69696 .57227 L
.69707 .57219 L
.69719 .57211 L
.6973 .57203 L
.6974 .57195 L
.69751 .57187 L
.69764 .57178 L
.69776 .57169 L
.69788 .57161 L
.69801 .57151 L
.69813 .57143 L
.69824 .57135 L
.69835 .57126 L
.69848 .57117 L
.69861 .57107 L
.69873 .57099 L
.69885 .57089 L
.69897 .57081 L
.69908 .57073 L
.69917 .57065 L
.69928 .57057 L
.6994 .57049 L
.69952 .57039 L
.69964 .57031 L
.69974 .57023 L
.69985 .57015 L
.69994 .57007 L
.70005 .56999 L
.70016 .5699 L
.70026 .56983 L
.70035 .56976 L
.70046 .56968 L
.70056 .5696 L
.70069 .5695 L
.7008 .56941 L
.70092 .56932 L
.70106 .56921 L
.70118 .56912 L
.7013 .56902 L
.70142 .56893 L
.70153 .56884 L
.70164 .56875 L
.70174 .56867 L
.70185 .56858 L
.70197 .56849 L
Mistroke
.70209 .5684 L
.70219 .56831 L
.70229 .56823 L
.7024 .56815 L
.70252 .56805 L
.70263 .56796 L
.70275 .56786 L
.70289 .56775 L
.70302 .56764 L
.70316 .56753 L
.70327 .56743 L
.7034 .56733 L
.70351 .56724 L
.70363 .56714 L
.70376 .56703 L
.70388 .56693 L
.70399 .56684 L
.70409 .56675 L
.7042 .56666 L
.70432 .56656 L
.70442 .56647 L
.70453 .56638 L
.70464 .56629 L
.70473 .56621 L
.70492 .56605 L
.70501 .56597 L
.70511 .56588 L
.70522 .56579 L
.70532 .5657 L
.70544 .5656 L
.70555 .56551 L
.70565 .56542 L
.70574 .56534 L
.70584 .56525 L
.70595 .56515 L
.70605 .56506 L
.70615 .56498 L
.70625 .56489 L
.70636 .56479 L
.70645 .56471 L
.70654 .56463 L
.70664 .56454 L
.70674 .56445 L
.70683 .56437 L
.70693 .56428 L
.70702 .56419 L
.70711 .56412 L
.70727 .56397 L
.70736 .56389 L
.70745 .56381 L
Mistroke
.70755 .56372 L
.70766 .56362 L
.70777 .56351 L
.70788 .56341 L
.70798 .56332 L
.70807 .56323 L
.70818 .56313 L
.70828 .56304 L
.70839 .56294 L
.7085 .56283 L
.7086 .56274 L
.70871 .56264 L
.70882 .56253 L
.70894 .56242 L
.70906 .56231 L
.70917 .5622 L
.70926 .56211 L
.70937 .56201 L
.70947 .56191 L
.70956 .56183 L
.70966 .56173 L
.70978 .56162 L
.70988 .56151 L
.70998 .56142 L
.71007 .56133 L
.71016 .56124 L
.71026 .56115 L
.71036 .56105 L
.71046 .56095 L
.71056 .56085 L
.71065 .56076 L
.71075 .56066 L
.71084 .56057 L
.71093 .56048 L
.7111 .56032 L
.71119 .56022 L
.71127 .56014 L
.71137 .56004 L
.71147 .55994 L
.71155 .55985 L
.71165 .55976 L
.71174 .55967 L
.71182 .55958 L
.71197 .55943 L
.71205 .55935 L
.71213 .55926 L
.71223 .55915 L
.71233 .55905 L
.71242 .55896 L
.71251 .55887 L
Mistroke
.71261 .55876 L
.71272 .55865 L
.71281 .55855 L
.71291 .55845 L
.71301 .55835 L
.71309 .55826 L
.71319 .55816 L
.71328 .55805 L
.71337 .55796 L
.71345 .55787 L
.71354 .55778 L
.71363 .55768 L
.71373 .55757 L
.71384 .55746 L
.71392 .55736 L
.71402 .55726 L
.71413 .55714 L
.71424 .55702 L
.71434 .55691 L
.71444 .5568 L
.71454 .55669 L
.71463 .5566 L
.71472 .55649 L
.71483 .55637 L
.71493 .55626 L
.71502 .55616 L
.7151 .55606 L
.71519 .55596 L
.71528 .55586 L
.71536 .55577 L
.71545 .55567 L
.71555 .55556 L
.71565 .55545 L
.71575 .55534 L
.71583 .55524 L
.71592 .55513 L
.716 .55504 L
.71609 .55494 L
.71618 .55483 L
.71627 .55473 L
.71642 .55456 L
.71651 .55446 L
.71659 .55436 L
.71667 .55427 L
.71675 .55418 L
.71689 .55401 L
.71697 .55391 L
.71705 .55382 L
.71714 .55372 L
.71722 .55362 L
Mistroke
.71735 .55346 L
.71742 .55338 L
.7175 .55329 L
.71758 .55319 L
.71765 .5531 L
.7178 .55293 L
.71787 .55284 L
.71795 .55274 L
.71804 .55263 L
.71812 .55254 L
.71827 .55236 L
.71835 .55226 L
.71842 .55217 L
.7185 .55207 L
.71858 .55197 L
.71867 .55186 L
.71875 .55176 L
.71884 .55164 L
.71893 .55153 L
.71901 .55144 L
.7191 .55133 L
.71917 .55123 L
.71926 .55113 L
.71933 .55103 L
.71941 .55093 L
.7195 .55082 L
.71959 .55071 L
.71967 .5506 L
.71975 .5505 L
.71984 .55038 L
.71992 .55027 L
.72 .55018 L
.72008 .55007 L
.72017 .54995 L
.72027 .54983 L
.72035 .54973 L
.72044 .54961 L
.72052 .5495 L
.7206 .5494 L
.72074 .54921 L
.72082 .5491 L
.7209 .549 L
.72098 .54889 L
.72107 .54877 L
.72114 .54868 L
.72121 .54858 L
.72135 .5484 L
.72143 .5483 L
.72151 .54818 L
.72158 .54809 L
Mistroke
.72166 .54799 L
.72175 .54786 L
.72185 .54773 L
.72194 .5476 L
.72202 .54748 L
.72212 .54735 L
.72223 .5472 L
.72233 .54706 L
.72243 .54692 L
.72253 .54678 L
.72263 .54664 L
.72272 .54651 L
.72282 .54637 L
.72293 .54622 L
.72304 .54607 L
.72313 .54593 L
.72324 .54578 L
.72334 .54565 L
.72343 .54551 L
.72351 .5454 L
.7236 .54527 L
.7237 .54513 L
.7238 .54498 L
.72389 .54485 L
.72398 .54471 L
.72407 .54458 L
.72416 .54446 L
.72425 .54432 L
.72434 .54419 L
.72442 .54406 L
.7245 .54395 L
.72458 .54383 L
.72467 .54369 L
.72478 .54353 L
.72489 .54337 L
.72498 .54323 L
.72509 .54307 L
.7252 .5429 L
.72526 .5428 L
.72532 .54272 L
.72537 .54264 L
.72543 .54255 L
.72554 .54239 L
.72563 .54224 L
.72574 .54207 L
.72579 .54199 L
.72585 .5419 L
.72596 .54174 L
.72605 .54159 L
.72616 .54142 L
Mistroke
.72625 .54129 L
.72634 .54113 L
.72644 .54097 L
.72654 .54082 L
.72662 .54069 L
.72672 .54054 L
.72682 .54038 L
.72692 .54021 L
.72701 .54006 L
.72711 .5399 L
.72721 .53975 L
.72729 .53961 L
.72739 .53945 L
.72748 .5393 L
.72757 .53916 L
.72765 .53903 L
.72774 .53889 L
.72783 .53873 L
.72792 .53859 L
.72801 .53843 L
.72811 .53826 L
.72821 .5381 L
.7283 .53795 L
.72838 .53782 L
.72847 .53766 L
.72856 .53752 L
.72865 .53735 L
.72875 .53718 L
.72884 .53704 L
.72893 .53688 L
.72903 .5367 L
.72909 .53661 L
.72914 .53652 L
.72923 .53635 L
.72933 .53618 L
.72942 .53602 L
.7295 .53588 L
.7296 .53572 L
.72968 .53557 L
.72976 .53543 L
.72984 .53529 L
.72992 .53515 L
.73 .535 L
.73009 .53484 L
.73018 .53468 L
.73027 .53452 L
.73035 .53438 L
.73044 .53421 L
.73055 .53402 L
.73064 .53385 L
Mistroke
.73073 .53369 L
.73081 .53355 L
.73088 .53342 L
.73095 .53328 L
.73104 .53312 L
.73111 .53299 L
.73117 .53287 L
.73125 .53274 L
.73132 .53259 L
.73141 .53243 L
.7315 .53226 L
.73158 .5321 L
.73166 .53195 L
.73174 .53182 L
.73181 .53167 L
.7319 .53151 L
.73197 .53136 L
.73205 .53121 L
.73214 .53104 L
.73223 .53088 L
.7323 .53073 L
.73238 .53058 L
.73245 .53045 L
.73253 .53029 L
.7326 .53015 L
.73269 .52998 L
.73277 .52981 L
.73285 .52967 L
.73293 .52951 L
.73302 .52934 L
.73311 .52915 L
.7332 .52897 L
.73328 .52881 L
.73335 .52867 L
.73343 .52851 L
.7335 .52836 L
.73357 .52823 L
.73364 .52808 L
.73372 .52791 L
.73381 .52774 L
.73389 .52758 L
.73397 .52741 L
.73405 .52724 L
.73412 .5271 L
.7342 .52694 L
.73428 .52676 L
.73437 .52658 L
.73445 .52642 L
.73453 .52625 L
.73461 .52608 L
Mistroke
.73468 .52593 L
.73475 .52579 L
.73482 .52564 L
.73489 .52548 L
.73498 .5253 L
.73505 .52514 L
.73512 .52499 L
.73519 .52484 L
.73526 .5247 L
.73533 .52454 L
.73541 .52438 L
.73547 .52423 L
.73553 .5241 L
.7356 .52395 L
.73567 .5238 L
.73576 .52361 L
.73585 .52342 L
.73592 .52325 L
.736 .52307 L
.73609 .52287 L
.73614 .52276 L
.73619 .52266 L
.73627 .52247 L
.73636 .52227 L
.73644 .52209 L
.73652 .52191 L
.7366 .52173 L
.73667 .52157 L
.73676 .52138 L
.73685 .52117 L
.73693 .52098 L
.73701 .5208 L
.73708 .52064 L
.73715 .52046 L
.73722 .5203 L
.73729 .52015 L
.73736 .51998 L
.73745 .51978 L
.73752 .5196 L
.73759 .51944 L
.73767 .51926 L
.73774 .5191 L
.73781 .51892 L
.73789 .51873 L
.73797 .51854 L
.73806 .51834 L
.73813 .51816 L
.73821 .51797 L
.73828 .51779 L
.73835 .51762 L
Mistroke
.73843 .51743 L
.73851 .51723 L
.73858 .51706 L
.73865 .5169 L
.73872 .51672 L
.7388 .51653 L
.73887 .51635 L
.73895 .51617 L
.73902 .51599 L
.73908 .51583 L
.73915 .51567 L
.73921 .51552 L
.73927 .51535 L
.73934 .51519 L
.73941 .51501 L
.73948 .51482 L
.73956 .51462 L
.73964 .51443 L
.73971 .51425 L
.73977 .51409 L
.73984 .5139 L
.73991 .51372 L
.73999 .51353 L
.74007 .51332 L
.74013 .51315 L
.74021 .51295 L
.74029 .51275 L
.74037 .51252 L
.74045 .51231 L
.74053 .51211 L
.74059 .51194 L
.74065 .51178 L
.74072 .5116 L
.74078 .51143 L
.74084 .51127 L
.7409 .51112 L
.74096 .51096 L
.74102 .51079 L
.74109 .51062 L
.74116 .51043 L
.74122 .51025 L
.74128 .51009 L
.74135 .5099 L
.74142 .5097 L
.7415 .5095 L
.74156 .50932 L
.74163 .50913 L
.7417 .50893 L
.74177 .50874 L
.74184 .50856 L
Mistroke
.74189 .5084 L
.74196 .50822 L
.74202 .50804 L
.74208 .50787 L
.74215 .50766 L
.74222 .50747 L
.74228 .5073 L
.74235 .50711 L
.74242 .5069 L
.7425 .50669 L
.74256 .5065 L
.74263 .50629 L
.74272 .50605 L
.74279 .50583 L
.74283 .50571 L
.74288 .50558 L
.74296 .50534 L
.74304 .50512 L
.74312 .50488 L
.74319 .50467 L
.74327 .50445 L
.74331 .50431 L
.74335 .50418 L
.74344 .50394 L
.74348 .5038 L
.74352 .50368 L
.74357 .50353 L
.74362 .50339 L
.7437 .50316 L
.74378 .50291 L
.74386 .50266 L
.74391 .50252 L
.74395 .5024 L
.74402 .50217 L
.7441 .50194 L
.74417 .50172 L
.74424 .50151 L
.74431 .50127 L
.7444 .50101 L
.74448 .50076 L
.74455 .50054 L
.74463 .5003 L
.7447 .50008 L
.74477 .49984 L
.74486 .49957 L
.7449 .49944 L
.74494 .4993 L
.74503 .49904 L
.74511 .49878 L
.74518 .49854 L
Mistroke
.74526 .4983 L
.74533 .49808 L
.7454 .49783 L
.74548 .49757 L
.74555 .49734 L
.74562 .49713 L
.74569 .4969 L
.74576 .49665 L
.74583 .49643 L
.7459 .49618 L
.74597 .49595 L
.74604 .49574 L
.74609 .49555 L
.74616 .49534 L
.74622 .49512 L
.74629 .49491 L
.74635 .49468 L
.74643 .49442 L
.74651 .49416 L
.74658 .49391 L
.74665 .49368 L
.74671 .49347 L
.74678 .49322 L
.74685 .49299 L
.74692 .49274 L
.747 .49247 L
.74706 .49224 L
.74714 .49199 L
.74721 .49172 L
.74726 .49156 L
.7473 .49142 L
.74737 .49115 L
.74745 .49089 L
.74752 .49064 L
.7476 .49037 L
.74767 .49011 L
.74774 .48987 L
.74781 .4896 L
.74789 .48931 L
.74793 .48916 L
.74797 .48901 L
.74805 .48872 L
.7481 .48857 L
.74814 .48842 L
.74818 .48827 L
.74822 .48809 L
.7483 .4878 L
.74835 .48764 L
.74839 .48749 L
.74846 .48721 L
Mistroke
.74853 .48695 L
.74861 .48665 L
.74869 .48637 L
.74876 .4861 L
.74883 .48585 L
.7489 .48557 L
.74897 .4853 L
.74904 .48505 L
.7491 .48482 L
.74917 .48457 L
.74924 .4843 L
.74931 .48403 L
.74938 .48374 L
.74945 .48346 L
.74952 .48321 L
.74959 .48292 L
.74964 .48275 L
.74968 .48259 L
.74975 .48231 L
.74983 .48201 L
.7499 .4817 L
.74994 .48155 L
.74998 .48139 L
.75006 .48109 L
.75013 .48081 L
.75017 .48064 L
.75021 .48049 L
.75028 .48019 L
.75036 .47986 L
.75041 .47968 L
.75045 .47951 L
.75052 .47923 L
.75059 .47897 L
.75066 .47867 L
.75073 .47836 L
.75077 .4782 L
.75081 .47803 L
.75089 .47772 L
.75096 .47741 L
.75103 .47713 L
.75112 .47678 L
.75116 .47658 L
.75121 .4764 L
.75129 .47607 L
.75133 .47591 L
.75137 .47572 L
.75144 .4754 L
.75149 .47522 L
.75153 .47506 L
.7516 .47474 L
Mistroke
.75167 .47444 L
.75175 .47409 L
.75179 .47391 L
.75184 .47371 L
.75188 .47355 L
.75192 .47337 L
.752 .47304 L
.75204 .47285 L
.75209 .47263 L
.75213 .47244 L
.75218 .47224 L
.75222 .47204 L
.75226 .47186 L
.75235 .47151 L
.75241 .47123 L
.75248 .47092 L
.75254 .47064 L
.7526 .47037 L
.75267 .47006 L
.75275 .46973 L
.75282 .46941 L
.75288 .46911 L
.75295 .46881 L
.75301 .46852 L
.75308 .46821 L
.75316 .46787 L
.75323 .46753 L
.75331 .46717 L
.75337 .46687 L
.75345 .46653 L
.75351 .46622 L
.75358 .46592 L
.75364 .46565 L
.7537 .46536 L
.75377 .46504 L
.75384 .4647 L
.7539 .46441 L
.75397 .46409 L
.75403 .46379 L
.75409 .46352 L
.75415 .4632 L
.75422 .4629 L
.75427 .46263 L
.75433 .46237 L
.75439 .46208 L
.75445 .46178 L
.75452 .46142 L
.7546 .46104 L
.75467 .46072 L
.75474 .46036 L
Mistroke
.75482 .45997 L
.75486 .45975 L
.7549 .45956 L
.75498 .45917 L
.75505 .4588 L
.75512 .45846 L
.7552 .4581 L
.75526 .45779 L
.75533 .45744 L
.7554 .45707 L
.75548 .45667 L
.75555 .45629 L
.75562 .45595 L
.75568 .45563 L
.75575 .45529 L
.75581 .45498 L
.75587 .45469 L
.75593 .45434 L
.756 .45397 L
.75607 .45362 L
.75613 .4533 L
.7562 .45296 L
.75626 .45264 L
.75632 .4523 L
.75639 .45192 L
.75647 .45153 L
.75654 .45116 L
.7566 .45082 L
.75667 .45046 L
.75674 .45009 L
.7568 .44976 L
.75687 .44939 L
.75694 .449 L
.757 .44866 L
.75706 .44834 L
.75712 .44799 L
.75719 .44762 L
.75725 .44729 L
.75732 .44692 L
.75738 .44658 L
.75744 .44626 L
.75749 .44597 L
.75755 .44565 L
.75761 .44532 L
.75766 .44501 L
.75773 .44465 L
.7578 .44425 L
.75786 .44387 L
.75793 .44352 L
.75799 .44317 L
Mistroke
.75804 .44286 L
.75811 .44249 L
.75817 .44215 L
.75823 .44177 L
.7583 .44137 L
.75836 .44103 L
.75843 .44065 L
.7585 .44024 L
.75857 .43981 L
.75864 .43939 L
.75871 .43902 L
.75876 .43869 L
.75882 .43834 L
.75889 .43796 L
.75895 .43761 L
.759 .43728 L
.75905 .43698 L
.75911 .43665 L
.75917 .4363 L
.75922 .43595 L
.75929 .43557 L
.75935 .43521 L
.7594 .43488 L
.75947 .43449 L
.75954 .43407 L
.7596 .43367 L
.75966 .43331 L
.75971 .43299 L
.75977 .43265 L
.75987 .43203 L
.75993 .43165 L
.75999 .43129 L
.76004 .43097 L
.7601 .43062 L
.76016 .43024 L
.76021 .4299 L
.76027 .4295 L
.76033 .42913 L
.76039 .42878 L
.76044 .42846 L
.7605 .42811 L
.76055 .42774 L
.76061 .42739 L
.76067 .42701 L
.76077 .42635 L
.76083 .42597 L
.76089 .42556 L
.76095 .42518 L
.76101 .42483 L
.76111 .42418 L
Mistroke
.76116 .42381 L
.76122 .42347 L
.76132 .42278 L
.76138 .42239 L
.76144 .42204 L
.76149 .42171 L
.76154 .42136 L
.76163 .42073 L
.76168 .42039 L
.76173 .42006 L
.76179 .41968 L
.76185 .41931 L
.76189 .419 L
.76194 .41866 L
.76203 .41807 L
.76208 .41773 L
.76213 .41737 L
.76223 .41672 L
.76228 .41636 L
.76234 .41597 L
.76244 .41529 L
.76249 .41491 L
.76255 .41455 L
.76261 .41413 L
.76266 .41374 L
.76271 .4134 L
.76277 .41303 L
.76286 .41238 L
.76291 .41201 L
.76297 .41161 L
.76307 .41091 L
.76313 .41051 L
.76318 .41014 L
.76324 .40974 L
.7633 .40931 L
.76335 .40891 L
.76341 .40854 L
.76351 .4078 L
.76357 .40735 L
.76363 .40693 L
.76374 .40613 L
.7638 .40568 L
.76386 .40527 L
.76393 .40479 L
.76399 .40434 L
.76408 .40364 L
.76417 .40301 L
.76426 .40233 L
.76432 .40194 L
.76436 .40158 L
Mistroke
.76444 .40099 L
.76453 .40033 L
.76463 .39964 L
.76471 .39899 L
.76481 .39827 L
.76491 .3975 L
.765 .39685 L
.76509 .39613 L
.7652 .39535 L
.76525 .39492 L
.76531 .39452 L
.76541 .39375 L
.7655 .39302 L
.76559 .39231 L
.76569 .39154 L
.7658 .39073 L
.7659 .38997 L
.76599 .38925 L
.76607 .3886 L
.76617 .38785 L
.76626 .38713 L
.76636 .38636 L
.76646 .38553 L
.76656 .38475 L
.76665 .38405 L
.76675 .38321 L
.76687 .3823 L
.76697 .38144 L
.76707 .38065 L
.76718 .37979 L
.76728 .37896 L
.76737 .37822 L
.76747 .37742 L
.76758 .37654 L
.76769 .37561 L
.76779 .37481 L
.7679 .37393 L
.768 .37311 L
.76809 .37236 L
.76819 .37149 L
.76829 .37067 L
.76838 .36991 L
.76847 .3692 L
.76857 .36836 L
.76866 .36759 L
.76876 .36668 L
.76888 .3657 L
.76898 .36483 L
.76909 .36391 L
.76928 .36231 L
Mistroke
.76939 .3614 L
.76949 .36055 L
.7696 .35957 L
.76971 .35864 L
.76991 .3569 L
.77002 .35592 L
.77012 .35502 L
.77025 .35395 L
.77036 .35297 L
.77055 .35127 L
.77066 .35038 L
.77076 .34943 L
.77094 .34784 L
.77114 .3461 L
.77135 .3442 L
.77158 .34217 L
.7718 .34025 L
.77199 .3385 L
.77221 .33652 L
.77242 .33469 L
.77264 .33273 L
.77287 .33057 L
.77327 .32694 L
.7737 .3231 L
.77533 .30824 L
.77612 .30103 L
.7765 .29755 L
.77684 .29441 L
.77724 .29078 L
.77746 .2888 L
.77767 .28695 L
.77789 .28493 L
.7781 .28309 L
.77832 .28108 L
.77856 .27896 L
.77875 .27732 L
.77894 .27557 L
.77913 .27392 L
.7793 .27243 L
.7795 .27069 L
.77961 .26971 L
.77971 .26881 L
.7799 .26716 L
.78 .26634 L
.7801 .26543 L
.78028 .26392 L
.78038 .26307 L
.78047 .26228 L
.78065 .26069 L
.78076 .25981 L
Mistroke
.78085 .25901 L
.78102 .25758 L
.78111 .25678 L
.7812 .25605 L
.78136 .2547 L
.78144 .25402 L
.78153 .25328 L
.78163 .25243 L
.78172 .25163 L
.78181 .25091 L
.78191 .25012 L
.78201 .24928 L
.78212 .24838 L
.78221 .24762 L
.78231 .24679 L
.7824 .24601 L
.78249 .24531 L
.78258 .24454 L
.78268 .24373 L
.78277 .24302 L
.78285 .24237 L
.78294 .24165 L
.78303 .24089 L
.78312 .24018 L
.78322 .23944 L
.7833 .2388 L
.78338 .2381 L
.78348 .23736 L
.78358 .23656 L
.78367 .23581 L
.78376 .23512 L
.78385 .23444 L
.78393 .23381 L
.78401 .23312 L
.78411 .23239 L
.7842 .2317 L
.78428 .23105 L
.78436 .23046 L
.78444 .22983 L
.78452 .22923 L
.78459 .22866 L
.78467 .22804 L
.78476 .22738 L
.78485 .22669 L
.78494 .22604 L
.78501 .22546 L
.7851 .22483 L
.78517 .22429 L
.78525 .22369 L
.78533 .22307 L
Mistroke
.78541 .22249 L
.78548 .22195 L
.78555 .22146 L
.78562 .22092 L
.7857 .22036 L
.78578 .21975 L
.78586 .21919 L
.78594 .21857 L
.78603 .21791 L
.78613 .21722 L
.78622 .21657 L
.7863 .21599 L
.78639 .21535 L
.78647 .21476 L
.78654 .21422 L
.78663 .2136 L
.78671 .21302 L
.78679 .21248 L
.78686 .21199 L
.78694 .21145 L
.78702 .21089 L
.7871 .21027 L
.78718 .20971 L
.78723 .2094 L
.78728 .20906 L
.78736 .20845 L
.78745 .20789 L
.78753 .20729 L
.78761 .20678 L
.78769 .20623 L
.78774 .20589 L
.78778 .20558 L
.78787 .20498 L
.78792 .20465 L
.78797 .20435 L
.78802 .20399 L
.78807 .20366 L
.78816 .20306 L
.78824 .20251 L
.78833 .20193 L
.78838 .2016 L
.78842 .2013 L
.7885 .20079 L
.78857 .20033 L
.78865 .1998 L
.78872 .1993 L
.78879 .19885 L
.78886 .19842 L
.78893 .19796 L
.78901 .19747 L
Mistroke
.78908 .19698 L
.78916 .19646 L
.78924 .19597 L
.78931 .19552 L
.78939 .19502 L
.78947 .19448 L
.78952 .19421 L
.78956 .19392 L
.78964 .1934 L
.78969 .19314 L
.78973 .19286 L
.78981 .19236 L
.78985 .19209 L
.78989 .19183 L
.78994 .19153 L
.78999 .19124 L
.79007 .19074 L
.79015 .19028 L
.79023 .18978 L
.79027 .1895 L
.79031 .18924 L
.79039 .18876 L
.79047 .18831 L
.79053 .1879 L
.79061 .18746 L
.79068 .18699 L
.79072 .18675 L
.79077 .18648 L
.79085 .18602 L
.79092 .18559 L
.79099 .18516 L
.79107 .18471 L
.79113 .18432 L
.7912 .1839 L
.79128 .18345 L
.79132 .18319 L
.79136 .18296 L
.79144 .18254 L
.79151 .18209 L
.79158 .1817 L
.79165 .18129 L
.79172 .1809 L
.79178 .18054 L
.79185 .18013 L
.79193 .17968 L
.792 .17926 L
.79207 .17888 L
.79213 .17854 L
.7922 .17817 L
.79226 .17783 L
Mistroke
.79231 .17752 L
.79238 .17715 L
.79245 .17675 L
.79252 .17638 L
.79258 .17604 L
.79265 .17565 L
.79271 .17529 L
.79277 .17496 L
.79284 .17461 L
.79291 .17423 L
.79298 .17383 L
.79304 .17349 L
.79311 .17312 L
.79319 .17272 L
.79323 .17249 L
.79327 .17228 L
.79334 .1719 L
.79341 .17149 L
.79348 .17113 L
.79355 .17079 L
.79361 .17043 L
.79369 .17003 L
.79377 .16962 L
.79381 .16938 L
.79385 .16916 L
.79393 .16877 L
.79397 .16855 L
.79401 .16835 L
.79408 .16797 L
.79416 .16757 L
.79424 .16718 L
.7943 .16683 L
.79437 .16651 L
.79444 .16616 L
.7945 .16583 L
.79456 .16553 L
.79463 .16518 L
.7947 .1648 L
.79477 .16445 L
.79484 .16412 L
.79491 .16379 L
.79497 .16347 L
.79504 .16313 L
.79511 .16276 L
.79519 .1624 L
.79526 .16202 L
.79533 .1617 L
.7954 .16134 L
.79548 .16095 L
.79552 .16076 L
Mistroke
.79557 .16054 L
.79564 .16016 L
.79572 .15981 L
.79576 .1596 L
.7958 .15939 L
.79588 .15902 L
.79593 .15881 L
.79597 .15861 L
.79601 .1584 L
.79606 .15818 L
.7961 .15798 L
.79614 .15779 L
.79622 .15743 L
.79629 .15711 L
.79636 .15677 L
.79644 .1564 L
.79651 .15606 L
.79655 .15586 L
.79659 .15569 L
.79663 .1555 L
.79668 .15529 L
.79675 .15496 L
.79683 .15461 L
.7969 .15429 L
.79696 .15399 L
.79704 .15364 L
.79711 .15332 L
.79719 .15298 L
.79723 .1528 L
.79727 .15261 L
.79734 .15228 L
.79741 .15199 L
.79748 .15165 L
.79756 .15133 L
.79762 .15103 L
.79768 .15076 L
.79775 .15047 L
.79782 .15016 L
.79786 .14999 L
.7979 .1498 L
.79798 .14947 L
.79806 .14914 L
.79813 .14884 L
.7982 .14852 L
.79824 .14834 L
.79828 .14817 L
.79836 .14783 L
.79841 .14763 L
.79845 .14746 L
.79852 .14716 L
Mistroke
.79858 .14688 L
.79866 .14658 L
.79873 .14626 L
.79877 .1461 L
.79881 .14593 L
.79889 .14561 L
.79896 .1453 L
.79903 .14502 L
.79911 .14471 L
.79915 .14453 L
.79919 .14437 L
.79926 .14406 L
.79933 .14377 L
.79937 .14361 L
.79942 .14344 L
.79949 .14312 L
.79954 .14293 L
.79959 .14275 L
.79962 .1426 L
.79967 .14242 L
.79975 .14209 L
.79979 .14191 L
.79984 .14175 L
.79991 .14145 L
.79999 .14113 L
.80004 .14096 L
.80008 .14076 L
.80013 .1406 L
.80017 .14042 L
.80021 .14026 L
.80026 .14008 L
.80034 .13977 L
.80041 .1395 L
.80047 .13925 L
.80054 .13897 L
.80062 .13868 L
.8007 .13837 L
.80077 .13808 L
.80084 .13783 L
.80092 .13755 L
.80098 .13731 L
.80105 .13704 L
.80112 .13677 L
.80119 .13651 L
.80125 .13628 L
.80131 .13606 L
.80138 .13581 L
.80144 .13558 L
.80151 .1353 L
.80158 .13505 L
Mistroke
.80166 .13476 L
.80173 .1345 L
.8018 .13425 L
.80186 .13402 L
.80193 .13377 L
.802 .13351 L
.80208 .13325 L
.80215 .13297 L
.80223 .1327 L
.80229 .13247 L
.80237 .13219 L
.80242 .13203 L
.80246 .13188 L
.80253 .13162 L
.80261 .13134 L
.80268 .1311 L
.80274 .13088 L
.8028 .13068 L
.80286 .13047 L
.80293 .13023 L
.803 .12998 L
.80307 .12975 L
.80313 .12954 L
.80319 .12933 L
.80325 .12913 L
.80331 .12892 L
.80338 .12869 L
.80343 .1285 L
.80349 .12831 L
.80355 .12812 L
.80361 .1279 L
.80368 .12766 L
.80375 .12743 L
.80382 .12718 L
.8039 .12692 L
.80397 .1267 L
.80403 .12649 L
.8041 .12625 L
.80418 .12601 L
.80424 .1258 L
.80431 .12558 L
.80438 .12534 L
.80445 .12513 L
.80451 .12492 L
.80457 .12474 L
.80463 .12453 L
.80469 .12433 L
.80475 .12413 L
.80481 .12395 L
.80486 .12378 L
Mistroke
.80492 .1236 L
.80498 .12341 L
.80505 .1232 L
.80511 .123 L
.80516 .12283 L
.80522 .12265 L
.80527 .12248 L
.80533 .12231 L
.80539 .12212 L
.80549 .1218 L
.80555 .12162 L
.8056 .12146 L
.80567 .12126 L
.80573 .12106 L
.8058 .12086 L
.80586 .12068 L
.80591 .12052 L
.80597 .12034 L
.80603 .12016 L
.80608 .11999 L
.80615 .1198 L
.80621 .1196 L
.80627 .11943 L
.80633 .11924 L
.8064 .11904 L
.80647 .11883 L
.80653 .11863 L
.8066 .11845 L
.80665 .11828 L
.80672 .11808 L
.80679 .11788 L
.80685 .1177 L
.80691 .11752 L
.80696 .11737 L
.80703 .11718 L
.80709 .11701 L
.80715 .11683 L
.80722 .11663 L
.80728 .11646 L
.80734 .11628 L
.80741 .11608 L
.80748 .11587 L
.80755 .11568 L
.80762 .11549 L
.80767 .11534 L
.80773 .11517 L
.80778 .11502 L
.80784 .11487 L
.8079 .11468 L
.80796 .11451 L
Mistroke
.80802 .11434 L
.80808 .11419 L
.80814 .11403 L
.80819 .11387 L
.80825 .11371 L
.80832 .11353 L
.80838 .11335 L
.80845 .11317 L
.8085 .11302 L
.80857 .11285 L
.80863 .11268 L
.80868 .11254 L
.80874 .11237 L
.8088 .1122 L
.80891 .11192 L
.80897 .11176 L
.80902 .11161 L
.80908 .11146 L
.80913 .11133 L
.80922 .11107 L
.80928 .11093 L
.80933 .1108 L
.80938 .11066 L
.80944 .11051 L
.8095 .11035 L
.80955 .11022 L
.80965 .10995 L
.80971 .1098 L
.80977 .10965 L
.80983 .10949 L
.80989 .10934 L
.80995 .10918 L
.81001 .10903 L
.81012 .10875 L
.81018 .10859 L
.81023 .10845 L
.81029 .10829 L
.81036 .10813 L
.81041 .10799 L
.81046 .10787 L
.81055 .10763 L
.81061 .1075 L
.81065 .10738 L
.81071 .10724 L
.81076 .10711 L
.81081 .10699 L
.81086 .10687 L
.81095 .10665 L
.811 .10653 L
.81105 .1064 L
Mistroke
.81114 .10618 L
.81119 .10604 L
.81124 .10593 L
.81129 .1058 L
.81135 .10566 L
.8114 .10553 L
.81145 .10541 L
.81155 .10518 L
.8116 .10505 L
.81165 .10494 L
.81171 .10479 L
.81176 .10466 L
.81187 .1044 L
.81193 .10426 L
.81199 .10412 L
.81205 .10397 L
.81211 .10384 L
.81217 .10369 L
.81224 .10354 L
.81235 .10328 L
.81241 .10313 L
.81247 .103 L
.81252 .10287 L
.81258 .10275 L
.81267 .10252 L
.81273 .10239 L
.81278 .10227 L
.81284 .10214 L
.8129 .10201 L
.81296 .10187 L
.81301 .10174 L
.81312 .1015 L
.81317 .10137 L
.81324 .10123 L
.8133 .10109 L
.81336 .10095 L
.81343 .10081 L
.81349 .10067 L
.8136 .10042 L
.81366 .10028 L
.81372 .10015 L
.81378 .1 L
.81385 .09985 L
.81391 .09973 L
.81396 .09962 L
.81405 .09941 L
.81411 .09929 L
.81416 .09918 L
.81422 .09905 L
.81427 .09894 L
Mistroke
.81432 .09883 L
.81437 .09872 L
.81446 .09853 L
.81451 .09841 L
.81457 .0983 L
.81466 .09809 L
.81472 .09797 L
.81477 .09787 L
.81482 .09775 L
.81488 .09763 L
.81494 .09751 L
.81499 .0974 L
.81509 .09719 L
.81514 .09708 L
.81519 .09697 L
.81525 .09684 L
.81531 .09672 L
.81542 .09649 L
.81548 .09636 L
.81555 .09623 L
.81561 .09611 L
.81567 .09598 L
.81573 .09586 L
.81578 .09574 L
.81584 .09563 L
.8159 .09551 L
.816 .0953 L
.81606 .09517 L
.81612 .09505 L
.81619 .09492 L
.81625 .09479 L
.81636 .09457 L
.81642 .09444 L
.81648 .09433 L
.81654 .09422 L
.8166 .0941 L
.81665 .094 L
.8167 .09389 L
.81675 .09379 L
.81681 .09368 L
.8169 .09349 L
.81696 .09338 L
.81701 .09328 L
.81707 .09317 L
.81713 .09304 L
.81719 .09293 L
.81724 .09283 L
.81734 .09263 L
.8174 .09252 L
.81746 .09241 L
Mistroke
.81751 .0923 L
.81758 .09218 L
.81763 .09207 L
.81769 .09197 L
.81779 .09177 L
.81786 .09165 L
.81792 .09153 L
.81803 .09132 L
.81809 .0912 L
.81815 .09109 L
.81822 .09097 L
.81828 .09084 L
.81835 .09073 L
.8184 .09062 L
.81851 .09041 L
.81858 .0903 L
.81864 .09018 L
.81876 .08995 L
.81883 .08984 L
.81889 .08972 L
.81896 .08959 L
.81903 .08947 L
.81914 .08926 L
.81921 .08914 L
.81927 .08903 L
.81938 .08883 L
.81944 .08872 L
.8195 .08861 L
.81956 .08852 L
.81961 .08842 L
.81972 .08823 L
.81978 .08812 L
.81985 .088 L
.8199 .0879 L
.81997 .08779 L
.82003 .08768 L
.82009 .08758 L
.8202 .08738 L
.82025 .08729 L
.82031 .08719 L
.82041 .08701 L
.82047 .08691 L
.82052 .08682 L
.82059 .08671 L
.82064 .08661 L
.8207 .08652 L
.82075 .08643 L
.82085 .08626 L
.8209 .08617 L
.82096 .08607 L
Mistroke
.82101 .08598 L
.82106 .08589 L
.82116 .08572 L
.82125 .08557 L
.82135 .08541 L
.82141 .08531 L
.82146 .08523 L
.82155 .08507 L
.82164 .08493 L
.82174 .08477 L
.82183 .08462 L
.82193 .08445 L
.82198 .08436 L
.82204 .08427 L
.82214 .08411 L
.82223 .08396 L
.82229 .08387 L
.82234 .08378 L
.82244 .08362 L
.82249 .08353 L
.82255 .08345 L
.8226 .08335 L
.82266 .08326 L
.82277 .08309 L
.82287 .08293 L
.82297 .08277 L
.82303 .08268 L
.82308 .08259 L
.82318 .08243 L
.82327 .08229 L
.82338 .08212 L
.82348 .08197 L
.82353 .08189 L
.82359 .0818 L
.82364 .08172 L
.8237 .08162 L
.82381 .08146 L
.82391 .08131 L
.82397 .08122 L
.82403 .08112 L
.82409 .08103 L
.82415 .08094 L
.82421 .08085 L
.82426 .08076 L
.82437 .0806 L
.82442 .08052 L
.82448 .08043 L
.82454 .08035 L
.8246 .08025 L
.82466 .08017 L
Mistroke
.82471 .08009 L
.82481 .07994 L
.82487 .07984 L
.82493 .07976 L
.82499 .07967 L
.82506 .07957 L
.82517 .07941 L
.82523 .07932 L
.82529 .07923 L
.8254 .07908 L
.82545 .079 L
.82551 .07891 L
.82561 .07877 L
.8257 .07863 L
.82576 .07854 L
.82582 .07846 L
.82592 .07832 L
.82598 .07824 L
.82603 .07816 L
.82609 .07808 L
.82615 .07799 L
.82624 .07785 L
.82633 .07773 L
.82642 .0776 L
.8265 .07749 L
.8266 .07735 L
.82669 .07723 L
.82679 .07709 L
.82688 .07696 L
.82696 .07684 L
.82705 .07672 L
.82713 .07662 L
.82721 .0765 L
.8273 .07638 L
.82739 .07625 L
.82748 .07613 L
.82756 .07602 L
.82766 .07589 L
.82776 .07575 L
.82784 .07564 L
.82794 .07551 L
.82804 .07537 L
.82815 .07523 L
.82825 .07509 L
.82835 .07496 L
.82844 .07484 L
.82853 .07472 L
.82863 .0746 L
.82873 .07446 L
.82883 .07433 L
Mistroke
.82892 .07421 L
.829 .07411 L
.82909 .07399 L
.82918 .07387 L
.82928 .07375 L
.82937 .07363 L
.82946 .07352 L
.82956 .07339 L
.82966 .07325 L
.82976 .07312 L
.82986 .073 L
.82995 .07288 L
.83005 .07277 L
.83014 .07264 L
.83025 .07251 L
.83036 .07238 L
.83047 .07224 L
.83056 .07212 L
.83067 .07199 L
.83076 .07188 L
.83085 .07176 L
.83094 .07166 L
.83103 .07155 L
.83112 .07144 L
.83123 .07131 L
.83131 .0712 L
.83141 .07109 L
.8315 .07098 L
.83158 .07088 L
.83167 .07077 L
.83176 .07066 L
.83185 .07056 L
.83192 .07047 L
.83201 .07038 L
.83209 .07027 L
.8322 .07015 L
.8323 .07003 L
.83241 .0699 L
.83252 .06977 L
.83258 .0697 L
.83264 .06963 L
.83276 .06949 L
.83287 .06936 L
.83297 .06925 L
.83307 .06914 L
.83316 .06903 L
.83326 .06892 L
.83336 .0688 L
.83347 .06868 L
.83358 .06856 L
Mistroke
.83367 .06845 L
.83377 .06834 L
.83383 .06827 L
.83389 .0682 L
.83401 .06807 L
.83412 .06795 L
.83422 .06784 L
.83433 .06771 L
.83445 .06758 L
.83451 .06751 L
.83458 .06744 L
.8347 .06731 L
.83481 .06719 L
.83493 .06706 L
.83504 .06694 L
.83514 .06683 L
.83525 .06672 L
.83534 .06662 L
.83545 .0665 L
.83555 .06639 L
.83565 .06629 L
.83574 .0662 L
.83583 .06609 L
.83594 .06598 L
.83605 .06587 L
.83615 .06576 L
.83625 .06566 L
.83635 .06555 L
.83643 .06546 L
.83653 .06536 L
.83663 .06526 L
.83672 .06516 L
.83681 .06507 L
.83689 .06499 L
.83698 .0649 L
.83707 .06481 L
.83717 .0647 L
.83727 .06461 L
.83737 .0645 L
.83747 .0644 L
.83757 .06431 L
.83765 .06422 L
.83775 .06413 L
.83784 .06403 L
.83794 .06393 L
.83805 .06382 L
.83815 .06372 L
.83824 .06363 L
.83835 .06352 L
.83847 .06341 L
Mistroke
.83858 .0633 L
.83868 .0632 L
.83877 .06312 L
.83885 .06304 L
.83901 .06288 L
.8391 .0628 L
.83918 .06272 L
.83927 .06264 L
.83936 .06255 L
.83944 .06247 L
.83953 .06238 L
.83962 .0623 L
.8397 .06223 L
.83984 .06209 L
.83992 .06202 L
.84 .06195 L
.84009 .06186 L
.84018 .06177 L
.84027 .06169 L
.84035 .06161 L
.84045 .06152 L
.84055 .06143 L
.84065 .06134 L
.84074 .06126 L
.84083 .06118 L
.84091 .0611 L
.841 .06102 L
.84109 .06093 L
.84117 .06086 L
.84125 .06079 L
.84134 .06071 L
.84143 .06063 L
.84152 .06055 L
.8416 .06047 L
.8417 .06039 L
.8418 .0603 L
.84188 .06022 L
.84198 .06014 L
.84207 .06006 L
.84215 .05999 L
.8423 .05985 L
.84239 .05978 L
.84247 .05971 L
.84263 .05957 L
.84272 .05949 L
.8428 .05942 L
.84289 .05934 L
.84299 .05925 L
.84307 .05918 L
.84316 .0591 L
Mistroke
.84326 .05902 L
.84336 .05893 L
.84346 .05885 L
.84355 .05877 L
.84364 .0587 L
.84372 .05863 L
.84381 .05855 L
.84391 .05847 L
.84407 .05833 L
.84416 .05825 L
.84425 .05818 L
.84433 .05811 L
.84442 .05804 L
.8445 .05797 L
.84458 .0579 L
.84466 .05784 L
.84474 .05777 L
.84489 .05765 L
.84498 .05758 L
.84506 .05751 L
.84515 .05744 L
.84524 .05737 L
.8454 .05724 L
.84548 .05717 L
.84557 .0571 L
.84564 .05704 L
.84573 .05697 L
.8458 .05691 L
.84588 .05685 L
.84595 .0568 L
.84602 .05674 L
.84616 .05663 L
.84624 .05657 L
.84631 .05651 L
.84639 .05644 L
.84648 .05637 L
.84656 .05631 L
.84663 .05625 L
.84678 .05614 L
.84686 .05608 L
.84693 .05602 L
.84701 .05596 L
.8471 .05589 L
.84725 .05577 L
.84733 .05571 L
.84741 .05565 L
.8475 .05559 L
.84758 .05552 L
.84774 .0554 L
.84783 .05533 L
Mistroke
.84791 .05527 L
.848 .0552 L
.84809 .05513 L
.84818 .05507 L
.84826 .05501 L
.84841 .05489 L
.8485 .05483 L
.84859 .05476 L
.84876 .05463 L
.84886 .05456 L
.84894 .0545 L
.84904 .05443 L
.84913 .05436 L
.84929 .05424 L
.84938 .05417 L
.84947 .05411 L
.84955 .05405 L
.84963 .054 L
.84978 .05389 L
.84986 .05383 L
.84993 .05377 L
.85002 .05371 L
.85011 .05365 L
.85024 .05355 L
.85031 .0535 L
.85039 .05345 L
.85047 .05339 L
.85055 .05333 L
.8507 .05323 L
.85078 .05317 L
.85086 .05312 L
.85094 .05306 L
.85103 .053 L
.85111 .05294 L
.85119 .05289 L
.85133 .05279 L
.85142 .05273 L
.85149 .05267 L
.85159 .05261 L
.85167 .05255 L
.85184 .05243 L
.85194 .05237 L
.85203 .05231 L
.85218 .05221 L
.85226 .05215 L
.85235 .05209 L
.85251 .05198 L
.8526 .05192 L
.85268 .05186 L
.85283 .05177 L
Mistroke
.85291 .05172 L
.85298 .05167 L
.85313 .05157 L
.85321 .05151 L
.85329 .05146 L
.85337 .0514 L
.85346 .05135 L
.85363 .05124 L
.85372 .05118 L
.8538 .05113 L
.8539 .05106 L
.85399 .051 L
.85407 .05095 L
.85415 .0509 L
.85431 .0508 L
.85439 .05074 L
.85448 .05068 L
.85457 .05063 L
.85464 .05058 L
.85479 .05049 L
.85486 .05044 L
.85494 .05039 L
.85502 .05034 L
.8551 .05029 L
.85518 .05024 L
.85525 .0502 L
.85539 .05011 L
.85552 .05003 L
.85565 .04994 L
.85573 .0499 L
.85581 .04985 L
.85596 .04976 L
.85603 .04971 L
.85612 .04966 L
.85619 .04961 L
.85627 .04957 L
.8564 .04948 L
.85649 .04943 L
.85658 .04938 L
.85666 .04933 L
.85674 .04928 L
.85681 .04923 L
.8569 .04918 L
.85704 .04909 L
.85711 .04905 L
.85719 .049 L
.85733 .04892 L
.85747 .04884 L
.8576 .04876 L
.85773 .04868 L
Mistroke
.85781 .04864 L
.85788 .04859 L
.85803 .04851 L
.85811 .04846 L
.85818 .04842 L
.85831 .04834 L
.85843 .04827 L
.85856 .0482 L
.8587 .04812 L
.85885 .04803 L
.85898 .04795 L
.85911 .04788 L
.85924 .04781 L
.85936 .04774 L
.85949 .04766 L
.85961 .04759 L
.85973 .04753 L
.85986 .04746 L
.86 .04738 L
.86013 .0473 L
.86026 .04723 L
.8604 .04715 L
.86053 .04708 L
.86065 .04702 L
.86077 .04695 L
.86091 .04687 L
.86106 .04679 L
.86119 .04672 L
.86132 .04665 L
.86145 .04658 L
.86157 .04651 L
.86168 .04645 L
.8618 .04639 L
.86192 .04632 L
.86206 .04625 L
.86217 .04619 L
.8623 .04612 L
.86242 .04606 L
.86252 .046 L
.86264 .04594 L
.86277 .04588 L
.86287 .04582 L
.86297 .04577 L
.86308 .04571 L
.8632 .04565 L
.86334 .04558 L
.86348 .0455 L
.86362 .04543 L
.86374 .04537 L
.86389 .0453 L
Mistroke
.86397 .04525 L
.86405 .04522 L
.86419 .04514 L
.86433 .04507 L
.86442 .04503 L
.86449 .04499 L
.86457 .04495 L
.86466 .04491 L
.8648 .04483 L
.86488 .04479 L
.86496 .04476 L
.8651 .04469 L
.86523 .04462 L
.86538 .04455 L
.86552 .04448 L
.86564 .04442 L
.86577 .04436 L
.86591 .04429 L
.86604 .04422 L
.86616 .04417 L
.86627 .04411 L
.8664 .04405 L
.86652 .04399 L
.86665 .04393 L
.86679 .04386 L
.86692 .0438 L
.86704 .04375 L
.86718 .04368 L
.86733 .04361 L
.86747 .04354 L
.8676 .04348 L
.86774 .04342 L
.86787 .04336 L
.86802 .04329 L
.86816 .04322 L
.86829 .04316 L
.8684 .04311 L
.86853 .04305 L
.86867 .04299 L
.86881 .04292 L
.86896 .04286 L
.8691 .04279 L
.86923 .04274 L
.86937 .04267 L
.86952 .0426 L
.8696 .04257 L
.86968 .04253 L
.86983 .04247 L
.86991 .04243 L
.86999 .0424 L
Mistroke
.87013 .04233 L
.87021 .0423 L
.87029 .04226 L
.87038 .04223 L
.87046 .04219 L
.87061 .04213 L
.87074 .04207 L
.87089 .042 L
.87097 .04197 L
.87104 .04194 L
.87119 .04187 L
.87132 .04182 L
.87144 .04176 L
.87157 .04171 L
.87171 .04165 L
.87186 .04159 L
.87199 .04153 L
.87213 .04147 L
.87227 .04142 L
.87239 .04137 L
.87252 .04131 L
.87267 .04125 L
.87279 .0412 L
.87291 .04115 L
.87304 .0411 L
.87317 .04104 L
.87329 .04099 L
.87341 .04094 L
.87354 .04089 L
.87366 .04084 L
.87378 .04079 L
.87388 .04075 L
.874 .04071 L
.87411 .04066 L
.87423 .04061 L
.87436 .04056 L
.8745 .0405 L
.87463 .04045 L
.87475 .0404 L
.87486 .04036 L
.87499 .04031 L
.87511 .04026 L
.87524 .04021 L
.87538 .04016 L
.87551 .04011 L
.87562 .04006 L
.87576 .04001 L
.87591 .03995 L
.87605 .0399 L
.87618 .03985 L
Mistroke
.87629 .03981 L
.87639 .03977 L
.87651 .03972 L
.87662 .03968 L
.87673 .03964 L
.87682 .03961 L
.87692 .03957 L
.87703 .03953 L
.87714 .03949 L
.87726 .03944 L
.87737 .0394 L
.87747 .03936 L
.87759 .03932 L
.87772 .03927 L
.87785 .03923 L
.87796 .03918 L
.87807 .03914 L
.8782 .0391 L
.87832 .03905 L
.87843 .03901 L
.87853 .03898 L
.87863 .03894 L
.87875 .0389 L
.87885 .03886 L
.87897 .03882 L
.87909 .03877 L
.8792 .03874 L
.87931 .0387 L
.87943 .03865 L
.87956 .03861 L
.87967 .03857 L
.87979 .03853 L
.87991 .03849 L
.88001 .03845 L
.88013 .03841 L
.88024 .03837 L
.88034 .03834 L
.88043 .0383 L
.88054 .03827 L
.88064 .03823 L
.88075 .03819 L
.88087 .03815 L
.88098 .03811 L
.88108 .03808 L
.8812 .03804 L
.88133 .038 L
.88145 .03795 L
.88157 .03792 L
.88168 .03788 L
.88181 .03784 L
Mistroke
.88192 .0378 L
.88203 .03776 L
.88213 .03773 L
.88224 .03769 L
.88235 .03765 L
.88245 .03762 L
.88258 .03758 L
.88269 .03754 L
.8828 .03751 L
.88291 .03747 L
.88303 .03743 L
.88316 .03739 L
.88327 .03735 L
.88339 .03731 L
.88349 .03728 L
.8836 .03725 L
.88371 .03721 L
.88381 .03718 L
.88399 .03712 L
.8841 .03709 L
.88419 .03706 L
.8843 .03702 L
.8844 .03699 L
.88451 .03696 L
.88463 .03692 L
.88473 .03689 L
.88483 .03686 L
.88493 .03683 L
.88504 .03679 L
.88514 .03676 L
.88524 .03673 L
.88534 .0367 L
.88545 .03667 L
.88555 .03664 L
.88564 .03661 L
.88574 .03658 L
.88584 .03655 L
.88593 .03652 L
.88602 .03649 L
.88612 .03646 L
.88622 .03643 L
.88639 .03638 L
.88648 .03636 L
.88656 .03633 L
.88667 .0363 L
.88677 .03627 L
.88688 .03624 L
.887 .0362 L
.88712 .03616 L
.88724 .03613 L
Mistroke
.88734 .0361 L
.88746 .03607 L
.88757 .03603 L
.88766 .03601 L
.88778 .03597 L
.88788 .03594 L
.88798 .03591 L
.88807 .03589 L
.88817 .03586 L
.88828 .03583 L
.88839 .0358 L
.8885 .03577 L
.88862 .03573 L
.88873 .0357 L
.88884 .03567 L
.88893 .03564 L
.88904 .03561 L
.88915 .03558 L
.88926 .03555 L
.88938 .03552 L
.8895 .03548 L
.8896 .03546 L
.88972 .03542 L
.88985 .03539 L
.88998 .03535 L
.8901 .03532 L
.89021 .03529 L
.8903 .03526 L
.89041 .03523 L
.89053 .0352 L
.89062 .03518 L
.89072 .03515 L
.89082 .03512 L
.89093 .0351 L
.89102 .03507 L
.89113 .03504 L
.89122 .03502 L
.89131 .03499 L
.89149 .03495 L
.89158 .03492 L
.89167 .0349 L
.89177 .03487 L
.89188 .03484 L
.89198 .03482 L
.89208 .03479 L
.89219 .03476 L
.89231 .03473 L
.89242 .0347 L
.89252 .03468 L
.89262 .03465 L
Mistroke
.89272 .03463 L
.89282 .0346 L
.89293 .03457 L
.89302 .03455 L
.89311 .03452 L
.89321 .0345 L
.89331 .03447 L
.89342 .03445 L
.89353 .03442 L
.89363 .03439 L
.89373 .03437 L
.89381 .03435 L
.89391 .03432 L
.89401 .0343 L
.8941 .03428 L
.89421 .03425 L
.89431 .03423 L
.8944 .0342 L
.8945 .03418 L
.89461 .03415 L
.89472 .03412 L
.89482 .0341 L
.89493 .03407 L
.89502 .03405 L
.89512 .03403 L
.89521 .03401 L
.8953 .03399 L
.8954 .03396 L
.8955 .03394 L
.8956 .03391 L
.89569 .03389 L
.89578 .03387 L
.89587 .03385 L
.89597 .03383 L
.89606 .0338 L
.89622 .03377 L
.89632 .03374 L
.8964 .03372 L
.89651 .0337 L
.89661 .03368 L
.89672 .03365 L
.89683 .03363 L
.89694 .0336 L
.89704 .03358 L
.89713 .03356 L
.89723 .03353 L
.89732 .03351 L
.89742 .03349 L
.89753 .03347 L
.89763 .03344 L
Mistroke
.89772 .03342 L
.8978 .0334 L
.8979 .03338 L
.89799 .03336 L
.89808 .03334 L
.89819 .03332 L
.8983 .03329 L
.8984 .03327 L
.8985 .03325 L
.89859 .03323 L
.89869 .03321 L
.89879 .03319 L
.89889 .03316 L
.89901 .03314 L
.89911 .03312 L
.8992 .0331 L
.89932 .03307 L
.89944 .03305 L
.89955 .03302 L
.89966 .033 L
.89975 .03298 L
.89983 .03296 L
.89999 .03293 L
.90008 .03291 L
.90016 .03289 L
.90025 .03287 L
.90035 .03285 L
.90042 .03284 L
.90051 .03282 L
.90065 .03279 L
.90074 .03277 L
.90083 .03275 L
.90091 .03274 L
.90099 .03272 L
.90109 .0327 L
.90118 .03268 L
.90127 .03266 L
.90136 .03264 L
.90145 .03262 L
.90156 .0326 L
.90165 .03258 L
.90174 .03257 L
.90183 .03255 L
.90192 .03253 L
.90201 .03251 L
.9021 .03249 L
.90218 .03248 L
.90226 .03246 L
.90235 .03244 L
.90244 .03243 L
Mistroke
.90253 .03241 L
.90263 .03239 L
.90272 .03237 L
.90281 .03235 L
.90291 .03233 L
.90302 .03231 L
.90312 .03229 L
.90322 .03227 L
.90331 .03226 L
.90339 .03224 L
.90349 .03222 L
.90359 .0322 L
.90368 .03219 L
.90376 .03217 L
.90385 .03215 L
.90394 .03214 L
.90403 .03212 L
.90411 .0321 L
.90419 .03209 L
.90429 .03207 L
.90438 .03205 L
.90447 .03204 L
.90456 .03202 L
.90465 .032 L
.90472 .03199 L
.90481 .03197 L
.9049 .03196 L
.90498 .03194 L
.90513 .03192 L
.90521 .0319 L
.90528 .03189 L
.90537 .03187 L
.90545 .03186 L
.90554 .03184 L
.90563 .03182 L
.90572 .03181 L
.90579 .03179 L
.90587 .03178 L
.90596 .03176 L
.90604 .03175 L
.90613 .03174 L
.90621 .03172 L
.90628 .03171 L
.90642 .03168 L
.90649 .03167 L
.90657 .03166 L
.90666 .03164 L
.90675 .03163 L
.90683 .03161 L
.90691 .0316 L
Mistroke
.907 .03158 L
.9071 .03157 L
.90719 .03155 L
.90727 .03154 L
.90736 .03152 L
.90743 .03151 L
.90752 .03149 L
.90761 .03148 L
.90769 .03147 L
.90776 .03145 L
.90784 .03144 L
.90792 .03143 L
.908 .03141 L
.90808 .0314 L
.90815 .03139 L
.90823 .03137 L
.90832 .03136 L
.90841 .03134 L
.90849 .03133 L
.90857 .03132 L
.90864 .03131 L
.90871 .03129 L
.90879 .03128 L
.90886 .03127 L
.90895 .03126 L
.90903 .03124 L
.9091 .03123 L
.90917 .03122 L
.90931 .0312 L
.90939 .03119 L
.90946 .03117 L
.90954 .03116 L
.90961 .03115 L
.90969 .03114 L
.90976 .03113 L
.90983 .03112 L
.9099 .0311 L
.91002 .03108 L
.9101 .03107 L
.91017 .03106 L
.9103 .03104 L
.91038 .03103 L
.91046 .03102 L
.91054 .031 L
.91061 .03099 L
.91069 .03098 L
.91078 .03097 L
.91087 .03095 L
.91094 .03094 L
.91102 .03093 L
Mistroke
.9111 .03092 L
.91118 .03091 L
.91127 .03089 L
.91136 .03088 L
.91145 .03086 L
.91153 .03085 L
.91162 .03084 L
.9117 .03083 L
.91178 .03082 L
.91185 .03081 L
.91192 .0308 L
.912 .03078 L
.91209 .03077 L
.91216 .03076 L
.91224 .03075 L
.91232 .03074 L
.91238 .03073 L
.91246 .03072 L
.91254 .03071 L
.91261 .0307 L
.91267 .03069 L
.91274 .03068 L
.91282 .03067 L
.9129 .03065 L
.913 .03064 L
.91308 .03063 L
.91316 .03062 L
.91326 .0306 L
.91336 .03059 L
.91345 .03058 L
.91354 .03056 L
.91362 .03055 L
.9137 .03054 L
.91379 .03053 L
.91388 .03052 L
.91396 .0305 L
.91404 .03049 L
.91412 .03048 L
.91421 .03047 L
.91429 .03046 L
.91437 .03045 L
.91446 .03044 L
.91455 .03043 L
.91463 .03041 L
.91471 .0304 L
.91479 .03039 L
.91486 .03038 L
.91494 .03037 L
.91501 .03036 L
.9151 .03035 L
Mistroke
.91517 .03034 L
.91524 .03033 L
.91531 .03032 L
.91538 .03031 L
.91546 .03031 L
.91553 .0303 L
.9156 .03029 L
.91566 .03028 L
.91573 .03027 L
.9158 .03026 L
.91588 .03025 L
.91596 .03024 L
.91602 .03023 L
.91609 .03022 L
.91616 .03022 L
.91622 .03021 L
.9163 .0302 L
.91636 .03019 L
.91642 .03018 L
.91649 .03017 L
.91656 .03017 L
.91664 .03016 L
.91671 .03015 L
.91679 .03014 L
.91688 .03013 L
.91696 .03012 L
.91704 .03011 L
.91711 .0301 L
.91718 .03009 L
.91725 .03008 L
.91733 .03007 L
.91741 .03006 L
.91748 .03005 L
.91755 .03004 L
.91761 .03004 L
.91769 .03003 L
.91775 .03002 L
.91783 .03001 L
.91789 .03 L
.91797 .02999 L
.91804 .02999 L
.91812 .02998 L
.91821 .02997 L
.91828 .02996 L
.91835 .02995 L
.91842 .02994 L
.91849 .02993 L
.91857 .02992 L
.91864 .02992 L
.9187 .02991 L
Mistroke
.91876 .0299 L
.91883 .02989 L
.9189 .02989 L
.91898 .02988 L
.91905 .02987 L
.91913 .02986 L
.91921 .02985 L
.91928 .02984 L
.91934 .02984 L
.91941 .02983 L
.91949 .02982 L
.91957 .02981 L
.91965 .0298 L
.91972 .02979 L
.9198 .02979 L
.91987 .02978 L
.91996 .02977 L
.92 .02976 L
.92005 .02976 L
.92013 .02975 L
.9202 .02974 L
.92027 .02974 L
.92034 .02973 L
.92041 .02972 L
.92048 .02971 L
.92054 .02971 L
.92061 .0297 L
.92067 .02969 L
.92074 .02968 L
.92082 .02968 L
.92089 .02967 L
.92095 .02966 L
.92103 .02965 L
.92107 .02965 L
.92111 .02965 L
.92119 .02964 L
.92126 .02963 L
.92132 .02962 L
.92138 .02962 L
.92145 .02961 L
.92152 .0296 L
.92158 .0296 L
.92163 .02959 L
.9217 .02959 L
.92176 .02958 L
.92183 .02957 L
.9219 .02957 L
.92196 .02956 L
.92202 .02955 L
.92209 .02955 L
Mistroke
.92216 .02954 L
.92224 .02953 L
.92231 .02953 L
.92237 .02952 L
.92243 .02951 L
.9225 .02951 L
.92256 .0295 L
.92262 .0295 L
.92268 .02949 L
.92274 .02948 L
.9228 .02948 L
.92286 .02947 L
.92292 .02947 L
.92298 .02946 L
.92304 .02946 L
.92309 .02945 L
.92314 .02945 L
.9232 .02944 L
.92325 .02944 L
.92331 .02943 L
.92337 .02942 L
.92344 .02942 L
.9235 .02941 L
.92356 .02941 L
.92361 .0294 L
.92367 .0294 L
.92373 .02939 L
.92379 .02939 L
.92386 .02938 L
.92392 .02937 L
.92398 .02937 L
.92404 .02936 L
.92412 .02936 L
.92418 .02935 L
.92425 .02934 L
.92432 .02934 L
.92438 .02933 L
.92445 .02933 L
.92449 .02932 L
.92452 .02932 L
.92459 .02931 L
.92466 .02931 L
.92473 .0293 L
.92479 .0293 L
.92485 .02929 L
.92491 .02929 L
.92497 .02928 L
.92504 .02928 L
.9251 .02927 L
.92515 .02927 L
Mistroke
.92521 .02926 L
.92528 .02925 L
.92531 .02925 L
.92535 .02925 L
.92541 .02924 L
.92545 .02924 L
.92549 .02924 L
.92555 .02923 L
.92559 .02923 L
.92563 .02923 L
.92569 .02922 L
.92573 .02922 L
.92577 .02921 L
.9258 .02921 L
.92585 .02921 L
.92588 .0292 L
.92591 .0292 L
.92598 .0292 L
.92603 .02919 L
.9261 .02919 L
.92613 .02918 L
.92617 .02918 L
.9262 .02918 L
.92624 .02918 L
.92627 .02917 L
.9263 .02917 L
.92636 .02917 L
.92643 .02916 L
.92646 .02916 L
.92649 .02916 L
.92655 .02915 L
.92662 .02915 L
.92667 .02914 L
.92673 .02914 L
.92679 .02913 L
.92685 .02913 L
.92688 .02912 L
.92691 .02912 L
.92695 .02912 L
.92698 .02912 L
.92705 .02911 L
.92708 .02911 L
.92712 .02911 L
.92718 .0291 L
.92724 .0291 L
.9273 .02909 L
.92733 .02909 L
.92736 .02909 L
.92739 .02909 L
.92743 .02908 L
Mistroke
.92749 .02908 L
.92754 .02907 L
.9276 .02907 L
.92763 .02907 L
.92766 .02907 L
.92772 .02906 L
.92778 .02906 L
.92781 .02905 L
.92784 .02905 L
.9279 .02905 L
.92795 .02904 L
.928 .02904 L
.92806 .02904 L
.92809 .02903 L
.92812 .02903 L
.92818 .02903 L
.92824 .02902 L
.92829 .02902 L
.92832 .02902 L
.92835 .02902 L
.92838 .02901 L
.92842 .02901 L
.92845 .02901 L
.92848 .02901 L
.92854 .029 L
.92857 .029 L
.9286 .029 L
.92863 .029 L
.92867 .02899 L
.92873 .02899 L
.92877 .02899 L
.9288 .02898 L
.92883 .02898 L
.92886 .02898 L
.92889 .02898 L
.92892 .02898 L
.92898 .02897 L
.92903 .02897 L
.92906 .02897 L
.92909 .02896 L
.92915 .02896 L
.92918 .02896 L
.92922 .02896 L
.92928 .02895 L
.92931 .02895 L
.92935 .02895 L
.92941 .02894 L
.92944 .02894 L
.92947 .02894 L
.92951 .02894 L
Mistroke
.92955 .02893 L
.9296 .02893 L
.92963 .02893 L
.92967 .02893 L
.9297 .02892 L
.92973 .02892 L
.92979 .02892 L
.92983 .02892 L
.92986 .02891 L
.9299 .02891 L
.92993 .02891 L
.92997 .02891 L
.93 .0289 L
.93006 .0289 L
.9301 .0289 L
.93013 .0289 L
.93017 .02889 L
.93021 .02889 L
.93025 .02889 L
.93029 .02889 L
.93032 .02888 L
.93036 .02888 L
.93039 .02888 L
.93042 .02888 L
.93045 .02888 L
.93049 .02887 L
.93055 .02887 L
.93059 .02887 L
.93062 .02887 L
.93065 .02886 L
.93069 .02886 L
.93072 .02886 L
.93075 .02886 L
.93079 .02885 L
.93083 .02885 L
.93086 .02885 L
.9309 .02885 L
.93094 .02885 L
.93098 .02884 L
.93101 .02884 L
.93105 .02884 L
.93108 .02884 L
.93113 .02883 L
.93116 .02883 L
.9312 .02883 L
.93123 .02883 L
.93127 .02883 L
.9313 .02882 L
.93134 .02882 L
.93138 .02882 L
Mistroke
.93141 .02882 L
.93144 .02882 L
.93148 .02881 L
.93151 .02881 L
.93155 .02881 L
.93158 .02881 L
.93161 .02881 L
.93164 .0288 L
.93168 .0288 L
.93172 .0288 L
.93175 .0288 L
.93179 .0288 L
.93182 .0288 L
.93185 .02879 L
.93189 .02879 L
.93192 .02879 L
.93195 .02879 L
.93198 .02879 L
.93201 .02878 L
.93204 .02878 L
.93207 .02878 L
.9321 .02878 L
.93214 .02878 L
.93217 .02878 L
.9322 .02877 L
.93223 .02877 L
.93227 .02877 L
.93229 .02877 L
.93232 .02877 L
.93236 .02877 L
.93239 .02876 L
.93244 .02876 L
.93247 .02876 L
.9325 .02876 L
.93253 .02876 L
.93256 .02875 L
.93259 .02875 L
.93262 .02875 L
.93265 .02875 L
.93269 .02875 L
.93272 .02875 L
.93275 .02874 L
.93278 .02874 L
.93281 .02874 L
.93284 .02874 L
.93287 .02874 L
.93291 .02874 L
.93293 .02873 L
.93296 .02873 L
.93299 .02873 L
Mistroke
.93304 .02873 L
.93307 .02873 L
.9331 .02873 L
.93315 .02872 L
.93318 .02872 L
.9332 .02872 L
.93323 .02872 L
.93326 .02872 L
.93329 .02872 L
.93332 .02872 L
.93335 .02871 L
.93338 .02871 L
.9334 .02871 L
.93343 .02871 L
.93347 .02871 L
.9335 .02871 L
.93353 .0287 L
.93356 .0287 L
.9336 .0287 L
.93363 .0287 L
.93367 .0287 L
.9337 .0287 L
.93373 .0287 L
.93376 .02869 L
.93379 .02869 L
.93382 .02869 L
.93386 .02869 L
.93389 .02869 L
.93392 .02869 L
.93395 .02868 L
.93398 .02868 L
.93401 .02868 L
.93404 .02868 L
.93407 .02868 L
.9341 .02868 L
.93414 .02868 L
.93417 .02867 L
.9342 .02867 L
.93423 .02867 L
.93425 .02867 L
.93428 .02867 L
.93431 .02867 L
.93434 .02867 L
.93437 .02867 L
.93439 .02866 L
.93442 .02866 L
.93445 .02866 L
.93448 .02866 L
.93451 .02866 L
.93453 .02866 L
Mistroke
.93456 .02866 L
.93458 .02866 L
.93461 .02865 L
.93463 .02865 L
.93466 .02865 L
.93469 .02865 L
.93471 .02865 L
.93474 .02865 L
.93477 .02865 L
.9348 .02865 L
.93483 .02864 L
.93486 .02864 L
.93488 .02864 L
.93491 .02864 L
.93493 .02864 L
.93496 .02864 L
.93498 .02864 L
.93501 .02864 L
.93504 .02864 L
.93507 .02863 L
.93509 .02863 L
.93511 .02863 L
.93514 .02863 L
.93516 .02863 L
.93519 .02863 L
.93521 .02863 L
.93524 .02863 L
.93526 .02863 L
.93529 .02863 L
.93532 .02862 L
.93535 .02862 L
.93537 .02862 L
.9354 .02862 L
.93543 .02862 L
.93547 .02862 L
.93549 .02862 L
.93552 .02862 L
.93555 .02861 L
.93558 .02861 L
.93561 .02861 L
.93564 .02861 L
.93568 .02861 L
.93571 .02861 L
.93574 .02861 L
.93577 .02861 L
.9358 .0286 L
.93582 .0286 L
.93585 .0286 L
.93588 .0286 L
.93591 .0286 L
Mistroke
.93594 .0286 L
.93596 .0286 L
.93599 .0286 L
.93602 .0286 L
.93605 .02859 L
.93607 .02859 L
.9361 .02859 L
.93612 .02859 L
.93615 .02859 L
.93618 .02859 L
.93621 .02859 L
.93624 .02859 L
.93627 .02859 L
.93629 .02859 L
.9363 .02859 L
.93634 .02858 L
.93636 .02858 L
.93637 .02858 L
.93641 .02858 L
.93642 .02858 L
.93644 .02858 L
.93647 .02858 L
.9365 .02858 L
.93653 .02858 L
.93655 .02858 L
.93657 .02858 L
.9366 .02857 L
.93661 .02857 L
.93663 .02857 L
.93666 .02857 L
.93668 .02857 L
.93672 .02857 L
.93673 .02857 L
.93675 .02857 L
.93679 .02857 L
.9368 .02857 L
.93682 .02857 L
.93685 .02857 L
.93687 .02856 L
.93689 .02856 L
.93692 .02856 L
.93693 .02856 L
.93695 .02856 L
.93697 .02856 L
.93698 .02856 L
.93702 .02856 L
.93704 .02856 L
.93705 .02856 L
.93708 .02856 L
.93709 .02856 L
Mistroke
.93711 .02856 L
.93713 .02856 L
.93716 .02855 L
.93718 .02855 L
.9372 .02855 L
.93721 .02855 L
.93723 .02855 L
.93726 .02855 L
.93728 .02855 L
.93729 .02855 L
.93732 .02855 L
.93735 .02855 L
.93737 .02855 L
.93738 .02855 L
.9374 .02855 L
.93742 .02855 L
.93743 .02855 L
.93745 .02855 L
.93747 .02854 L
.93749 .02854 L
.9375 .02854 L
.93753 .02854 L
.93755 .02854 L
.93756 .02854 L
.93758 .02854 L
.93759 .02854 L
.93762 .02854 L
.93763 .02854 L
.93765 .02854 L
.93767 .02854 L
.93769 .02854 L
.9377 .02854 L
.93772 .02854 L
.93773 .02854 L
.93776 .02853 L
.93778 .02853 L
.93779 .02853 L
.93781 .02853 L
.93783 .02853 L
.93786 .02853 L
.93787 .02853 L
.93789 .02853 L
.9379 .02853 L
.93792 .02853 L
.93795 .02853 L
.93796 .02853 L
.93798 .02853 L
.938 .02853 L
.93801 .02853 L
.93803 .02853 L
Mistroke
.93805 .02853 L
.93808 .02853 L
.93809 .02852 L
.93811 .02852 L
.93813 .02852 L
.93814 .02852 L
.93816 .02852 L
.93818 .02852 L
.9382 .02852 L
.93822 .02852 L
.93823 .02852 L
.93824 .02852 L
.93827 .02852 L
.93829 .02852 L
.9383 .02852 L
.93831 .02852 L
.93833 .02852 L
.93834 .02852 L
.93836 .02852 L
.93838 .02852 L
.93839 .02852 L
.93841 .02852 L
.93844 .02851 L
.93845 .02851 L
.93846 .02851 L
.93848 .02851 L
.93849 .02851 L
.93851 .02851 L
.93852 .02851 L
.93855 .02851 L
.93857 .02851 L
.93858 .02851 L
.93861 .02851 L
.93863 .02851 L
.93864 .02851 L
.93866 .02851 L
.93868 .02851 L
.93869 .02851 L
.93871 .02851 L
.93873 .02851 L
.93875 .02851 L
.93876 .02851 L
.93878 .02851 L
.9388 .0285 L
.93881 .0285 L
.93882 .0285 L
.93885 .0285 L
.93886 .0285 L
.93888 .0285 L
.93891 .0285 L
Mistroke
.93892 .0285 L
.93894 .0285 L
.93895 .0285 L
.93897 .0285 L
.939 .0285 L
.93901 .0285 L
.93903 .0285 L
.93904 .0285 L
.93906 .0285 L
.93908 .0285 L
.93909 .0285 L
.93911 .0285 L
.93913 .0285 L
.93914 .0285 L
.93916 .0285 L
.93919 .02849 L
.9392 .02849 L
.93921 .02849 L
.93924 .02849 L
.93925 .02849 L
.93927 .02849 L
.93928 .02849 L
.93929 .02849 L
.93931 .02849 L
.93932 .02849 L
.93935 .02849 L
.93937 .02849 L
.93938 .02849 L
.93939 .02849 L
.93941 .02849 L
.93943 .02849 L
.93945 .02849 L
.93946 .02849 L
.93948 .02849 L
.93949 .02849 L
.93951 .02849 L
.93952 .02849 L
.93954 .02849 L
.93955 .02849 L
.93957 .02849 L
.93958 .02848 L
.9396 .02848 L
.93961 .02848 L
.93962 .02848 L
.93964 .02848 L
.93966 .02848 L
.93967 .02848 L
.93969 .02848 L
.9397 .02848 L
.93971 .02848 L
Mistroke
.93974 .02848 L
.93975 .02848 L
.93976 .02848 L
.93977 .02848 L
.93979 .02848 L
.93981 .02848 L
.93982 .02848 L
.93984 .02848 L
.93985 .02848 L
.93986 .02848 L
.93989 .02848 L
.9399 .02848 L
.93991 .02848 L
.93993 .02848 L
.93994 .02848 L
.93995 .02848 L
.93997 .02848 L
.93998 .02848 L
.94 .02848 L
.94001 .02847 L
.94003 .02847 L
.94004 .02847 L
.94005 .02847 L
.94007 .02847 L
.94008 .02847 L
.94009 .02847 L
.94011 .02847 L
.94012 .02847 L
.94013 .02847 L
.94015 .02847 L
.94016 .02847 L
.94017 .02847 L
.94019 .02847 L
.9402 .02847 L
.94022 .02847 L
.94023 .02847 L
.94025 .02847 L
.94026 .02847 L
.94027 .02847 L
.94028 .02847 L
.9403 .02847 L
.94031 .02847 L
.94033 .02847 L
.94034 .02847 L
.94035 .02847 L
.94036 .02847 L
.94038 .02847 L
.94039 .02847 L
.9404 .02847 L
.94041 .02847 L
Mistroke
.94042 .02847 L
.94044 .02847 L
.94045 .02847 L
.94046 .02847 L
.94048 .02847 L
.94049 .02846 L
.9405 .02846 L
.94052 .02846 L
.94053 .02846 L
.94055 .02846 L
.94056 .02846 L
.94057 .02846 L
.94059 .02846 L
.9406 .02846 L
.94061 .02846 L
.94063 .02846 L
.94064 .02846 L
.94065 .02846 L
.94066 .02846 L
.94068 .02846 L
.94069 .02846 L
.94071 .02846 L
.94072 .02846 L
.94073 .02846 L
.94074 .02846 L
.94076 .02846 L
.94077 .02846 L
.94079 .02846 L
.9408 .02846 L
.94081 .02846 L
.94083 .02846 L
.94084 .02846 L
.94086 .02846 L
.94087 .02846 L
.94089 .02846 L
.9409 .02846 L
.94091 .02846 L
.94093 .02846 L
.94094 .02846 L
.94095 .02846 L
.94096 .02846 L
.94097 .02846 L
.94099 .02846 L
.941 .02845 L
.94101 .02845 L
.94103 .02845 L
.94104 .02845 L
.94106 .02845 L
.94107 .02845 L
.94108 .02845 L
Mistroke
.94109 .02845 L
.94111 .02845 L
.94112 .02845 L
.94114 .02845 L
.94115 .02845 L
.94116 .02845 L
.94117 .02845 L
.94119 .02845 L
.9412 .02845 L
.94121 .02845 L
.94123 .02845 L
.94124 .02845 L
.94125 .02845 L
.94127 .02845 L
.94128 .02845 L
.94129 .02845 L
.9413 .02845 L
.94131 .02845 L
.94133 .02845 L
.94134 .02845 L
.94135 .02845 L
.94136 .02845 L
.94137 .02845 L
.94139 .02845 L
.9414 .02845 L
.94141 .02845 L
.94142 .02845 L
.94143 .02845 L
.94145 .02845 L
.94146 .02845 L
.94147 .02845 L
.94148 .02845 L
.94149 .02845 L
.9415 .02845 L
.94151 .02845 L
.94153 .02845 L
.94154 .02845 L
.94155 .02845 L
.94156 .02845 L
.94157 .02845 L
.94159 .02845 L
.9416 .02844 L
.94161 .02844 L
.94162 .02844 L
.94164 .02844 L
.94165 .02844 L
.94166 .02844 L
.94167 .02844 L
.94168 .02844 L
.94169 .02844 L
Mistroke
.9417 .02844 L
.94171 .02844 L
.94172 .02844 L
.94173 .02844 L
.94174 .02844 L
.94174 .02844 L
.94175 .02844 L
.94176 .02844 L
.94177 .02844 L
.94178 .02844 L
.94179 .02844 L
.94179 .02844 L
.9418 .02844 L
.94181 .02844 L
.94182 .02844 L
.94183 .02844 L
.94184 .02844 L
.94185 .02844 L
.94186 .02844 L
.94187 .02844 L
.94188 .02844 L
.94189 .02844 L
.9419 .02844 L
.94191 .02844 L
.94192 .02844 L
.94193 .02844 L
.94194 .02844 L
.94194 .02844 L
.94195 .02844 L
.94197 .02844 L
.94197 .02844 L
.94198 .02844 L
.94199 .02844 L
.942 .02844 L
.942 .02844 L
.94201 .02844 L
.94201 .02844 L
.94202 .02844 L
.94203 .02844 L
.94204 .02844 L
.94205 .02844 L
.94206 .02844 L
.94207 .02844 L
.94207 .02844 L
.94208 .02844 L
.94209 .02844 L
.9421 .02844 L
.94211 .02844 L
.94212 .02844 L
.94212 .02844 L
Mistroke
.94213 .02844 L
.94214 .02844 L
.94215 .02844 L
.94216 .02844 L
.94217 .02844 L
.94218 .02844 L
.94219 .02844 L
.94219 .02844 L
.9422 .02844 L
.94221 .02844 L
.94222 .02844 L
.94223 .02844 L
.94224 .02844 L
.94224 .02844 L
.94225 .02844 L
.94226 .02844 L
.94227 .02844 L
.94227 .02844 L
.94228 .02844 L
.94229 .02844 L
.9423 .02843 L
.94231 .02843 L
.94232 .02843 L
.94232 .02843 L
.94234 .02843 L
.94234 .02843 L
.94235 .02843 L
.94236 .02843 L
.94236 .02843 L
.94237 .02843 L
.94238 .02843 L
.94239 .02843 L
.9424 .02843 L
.94241 .02843 L
.94242 .02843 L
.94242 .02843 L
.94243 .02843 L
.94244 .02843 L
.94245 .02843 L
.94246 .02843 L
.94246 .02843 L
.94247 .02843 L
.94248 .02843 L
.94249 .02843 L
.9425 .02843 L
.9425 .02843 L
.94251 .02843 L
.94252 .02843 L
.94252 .02843 L
.94253 .02843 L
Mistroke
.94254 .02843 L
.94255 .02843 L
.94256 .02843 L
.94256 .02843 L
.94257 .02843 L
.94258 .02843 L
.94259 .02843 L
.94259 .02843 L
.9426 .02843 L
.94261 .02843 L
.94262 .02843 L
.94263 .02843 L
.94264 .02843 L
.94264 .02843 L
.94265 .02843 L
.94266 .02843 L
.94267 .02843 L
.94268 .02843 L
.94268 .02843 L
.94269 .02843 L
.9427 .02843 L
.94271 .02843 L
.94271 .02843 L
.94272 .02843 L
.94273 .02843 L
.94273 .02843 L
.94274 .02843 L
.94275 .02843 L
.94275 .02843 L
.94276 .02843 L
.94277 .02843 L
.94277 .02843 L
.94278 .02843 L
.94279 .02843 L
.94279 .02843 L
.9428 .02843 L
.94281 .02843 L
.94281 .02843 L
.94282 .02843 L
.94283 .02843 L
.94283 .02843 L
.94284 .02843 L
.94284 .02843 L
.94286 .02843 L
.94286 .02843 L
.94287 .02843 L
.94287 .02843 L
.94288 .02843 L
.94289 .02843 L
.9429 .02843 L
Mistroke
.9429 .02843 L
.94291 .02843 L
.94292 .02843 L
.94292 .02843 L
.94293 .02843 L
.94294 .02843 L
.94294 .02843 L
.94295 .02843 L
.94296 .02843 L
.94297 .02843 L
.94298 .02843 L
.94298 .02843 L
.94299 .02843 L
.943 .02843 L
.943 .02843 L
.94301 .02843 L
.94302 .02843 L
.94302 .02843 L
.94303 .02843 L
.94304 .02843 L
.94304 .02843 L
.94305 .02843 L
.94306 .02843 L
.94306 .02843 L
.94307 .02843 L
.94308 .02843 L
.94308 .02843 L
.94309 .02843 L
.9431 .02843 L
.94311 .02843 L
.94311 .02843 L
.94312 .02843 L
.94313 .02843 L
.94314 .02843 L
.94314 .02843 L
.94315 .02843 L
.94316 .02843 L
.94317 .02843 L
.94317 .02843 L
.94318 .02843 L
.94319 .02843 L
.94319 .02843 L
.9432 .02843 L
.94321 .02842 L
.94321 .02842 L
.94322 .02842 L
.94322 .02842 L
.94323 .02842 L
.94324 .02842 L
.94324 .02842 L
Mistroke
.94325 .02842 L
.94326 .02842 L
.94326 .02842 L
.94327 .02842 L
.94328 .02842 L
.94328 .02842 L
.94329 .02842 L
.94329 .02842 L
.9433 .02842 L
.9433 .02842 L
.94331 .02842 L
.94331 .02842 L
.94332 .02842 L
.94333 .02842 L
.94333 .02842 L
.94334 .02842 L
.94335 .02842 L
.94336 .02842 L
.94336 .02842 L
.94337 .02842 L
.94337 .02842 L
.94338 .02842 L
.94338 .02842 L
.94339 .02842 L
.94339 .02842 L
.9434 .02842 L
.9434 .02842 L
.94341 .02842 L
.94342 .02842 L
.94342 .02842 L
.94343 .02842 L
.94344 .02842 L
.94344 .02842 L
.94345 .02842 L
.94346 .02842 L
.94346 .02842 L
.94347 .02842 L
.94347 .02842 L
.94348 .02842 L
.94348 .02842 L
.94349 .02842 L
.9435 .02842 L
.9435 .02842 L
.94351 .02842 L
.94351 .02842 L
.94352 .02842 L
.94353 .02842 L
.94353 .02842 L
.94354 .02842 L
.94355 .02842 L
Mistroke
.94355 .02842 L
.94355 .02842 L
.94356 .02842 L
.94356 .02842 L
.94357 .02842 L
.94357 .02842 L
.94358 .02842 L
.94358 .02842 L
.94359 .02842 L
.94359 .02842 L
.9436 .02842 L
.9436 .02842 L
.9436 .02842 L
.94361 .02842 L
.94361 .02842 L
.94362 .02842 L
.94362 .02842 L
.94363 .02842 L
.94363 .02842 L
.94363 .02842 L
.94364 .02842 L
.94364 .02842 L
.94365 .02842 L
.94365 .02842 L
.94366 .02842 L
.94366 .02842 L
.94366 .02842 L
.94367 .02842 L
.94367 .02842 L
.94367 .02842 L
.94368 .02842 L
.94369 .02842 L
.94369 .02842 L
.94369 .02842 L
.9437 .02842 L
.9437 .02842 L
.9437 .02842 L
.94371 .02842 L
.94371 .02842 L
.94372 .02842 L
.94372 .02842 L
.94373 .02842 L
.94373 .02842 L
.94374 .02842 L
.94374 .02842 L
.94375 .02842 L
.94375 .02842 L
.94376 .02842 L
.94376 .02842 L
.94377 .02842 L
Mistroke
.94377 .02842 L
.94378 .02842 L
.94378 .02842 L
.94379 .02842 L
.94379 .02842 L
.9438 .02842 L
.9438 .02842 L
.9438 .02842 L
.94381 .02842 L
.94381 .02842 L
.94381 .02842 L
.94382 .02842 L
.94382 .02842 L
.94383 .02842 L
.94383 .02842 L
.94384 .02842 L
.94384 .02842 L
.94384 .02842 L
.94385 .02842 L
.94385 .02842 L
.94386 .02842 L
.94386 .02842 L
.94387 .02842 L
.94387 .02842 L
.94388 .02842 L
.94388 .02842 L
.94388 .02842 L
.94389 .02842 L
.94389 .02842 L
.9439 .02842 L
.9439 .02842 L
.9439 .02842 L
.94391 .02842 L
.94391 .02842 L
.94392 .02842 L
.94392 .02842 L
.94393 .02842 L
.94393 .02842 L
.94393 .02842 L
.94394 .02842 L
.94394 .02842 L
.94395 .02842 L
.94395 .02842 L
.94395 .02842 L
.94396 .02842 L
.94396 .02842 L
.94396 .02842 L
.94397 .02842 L
.94397 .02842 L
.94398 .02842 L
Mistroke
.94398 .02842 L
.94398 .02842 L
.94399 .02842 L
.94399 .02842 L
.94399 .02842 L
.944 .02842 L
.944 .02842 L
.94401 .02842 L
.94401 .02842 L
.94401 .02842 L
.94402 .02842 L
.94402 .02842 L
.94402 .02842 L
.94403 .02842 L
.94403 .02842 L
.94403 .02842 L
.94404 .02842 L
.94404 .02842 L
.94405 .02842 L
.94405 .02842 L
.94405 .02842 L
.94406 .02842 L
.94406 .02842 L
.94406 .02842 L
.94407 .02842 L
.94407 .02842 L
.94407 .02842 L
.94408 .02842 L
.94408 .02842 L
.94408 .02842 L
.94409 .02842 L
.94409 .02842 L
.94409 .02842 L
.9441 .02842 L
.9441 .02842 L
.94411 .02842 L
.94411 .02842 L
.94411 .02842 L
.94412 .02842 L
.94412 .02842 L
.94412 .02842 L
.94413 .02842 L
.94413 .02842 L
.94413 .02842 L
.94414 .02842 L
.94414 .02842 L
.94414 .02842 L
.94415 .02842 L
.94415 .02842 L
.94416 .02842 L
Mistroke
.94416 .02842 L
.94416 .02842 L
.94417 .02842 L
.94417 .02842 L
.94417 .02842 L
.94418 .02842 L
.94418 .02842 L
.94418 .02842 L
.94419 .02842 L
.94419 .02842 L
.94419 .02842 L
.9442 .02842 L
.9442 .02842 L
.9442 .02842 L
.94421 .02842 L
.94421 .02842 L
.94421 .02842 L
.94422 .02842 L
.94422 .02842 L
.94422 .02842 L
.94423 .02842 L
.94423 .02842 L
.94423 .02842 L
.94424 .02842 L
.94424 .02842 L
.94424 .02842 L
.94424 .02842 L
.94425 .02842 L
.94425 .02842 L
.94426 .02842 L
.94426 .02842 L
.94426 .02842 L
.94427 .02842 L
.94427 .02842 L
.94427 .02842 L
.94428 .02842 L
.94428 .02842 L
.94428 .02842 L
.94429 .02842 L
.94429 .02842 L
.94429 .02842 L
.9443 .02842 L
.9443 .02842 L
.9443 .02842 L
.94431 .02842 L
.94431 .02842 L
.94431 .02842 L
.94432 .02842 L
.94432 .02842 L
.94432 .02842 L
Mistroke
.94433 .02842 L
.94433 .02842 L
.94433 .02842 L
.94433 .02842 L
.94434 .02842 L
.94434 .02842 L
.94435 .02842 L
.94435 .02842 L
.94435 .02842 L
.94436 .02842 L
.94436 .02842 L
.94436 .02842 L
.94436 .02842 L
.94437 .02842 L
.94437 .02842 L
.94437 .02842 L
.94438 .02842 L
.94438 .02842 L
.94438 .02842 L
.94438 .02842 L
.94439 .02842 L
.94439 .02842 L
.9444 .02842 L
.9444 .02842 L
.9444 .02842 L
.94441 .02842 L
.94441 .02842 L
.94441 .02842 L
.94441 .02842 L
.94442 .02842 L
.94442 .02842 L
.94442 .02842 L
.94443 .02842 L
.94443 .02842 L
.94443 .02842 L
.94444 .02842 L
.94444 .02842 L
.94444 .02842 L
.94445 .02842 L
.94445 .02842 L
.94445 .02842 L
.94445 .02842 L
.94446 .02842 L
.94446 .02842 L
.94446 .02842 L
.94446 .02842 L
.94447 .02842 L
.94447 .02842 L
.94447 .02842 L
.94448 .02842 L
Mistroke
.94448 .02842 L
.94448 .02842 L
.94448 .02842 L
.94448 .02842 L
.94449 .02842 L
.94449 .02842 L
.94449 .02842 L
.9445 .02842 L
.9445 .02842 L
.9445 .02842 L
.9445 .02842 L
.9445 .02842 L
.94451 .02842 L
.94451 .02842 L
.94451 .02842 L
.94451 .02842 L
.94452 .02842 L
.94452 .02842 L
.94452 .02842 L
.94452 .02842 L
.94452 .02842 L
.94453 .02842 L
.94453 .02842 L
.94453 .02842 L
.94453 .02842 L
.94454 .02842 L
.94454 .02842 L
.94454 .02842 L
.94454 .02842 L
.94455 .02842 L
.94455 .02842 L
.94455 .02842 L
.94455 .02842 L
.94455 .02842 L
.94456 .02842 L
.94456 .02842 L
.94456 .02842 L
.94456 .02842 L
.94457 .02842 L
.94457 .02842 L
.94457 .02842 L
.94457 .02842 L
.94457 .02842 L
.94458 .02842 L
.94458 .02842 L
.94458 .02842 L
.94459 .02842 L
.94459 .02842 L
.94459 .02842 L
.94459 .02842 L
Mistroke
.94459 .02842 L
.9446 .02842 L
.9446 .02842 L
.9446 .02842 L
.9446 .02842 L
.9446 .02842 L
.94461 .02842 L
.94461 .02842 L
.94461 .02842 L
.94461 .02842 L
.94462 .02842 L
.94462 .02842 L
.94462 .02842 L
.94462 .02842 L
.94463 .02842 L
.94463 .02842 L
.94463 .02842 L
.94463 .02842 L
.94463 .02842 L
.94464 .02842 L
.94464 .02842 L
.94464 .02842 L
.94464 .02842 L
.94465 .02842 L
.94465 .02842 L
.94465 .02842 L
.94465 .02842 L
.94465 .02842 L
.94466 .02842 L
.94466 .02842 L
.94466 .02842 L
.94466 .02842 L
.94466 .02842 L
.94467 .02842 L
.94467 .02842 L
.94467 .02842 L
.94467 .02842 L
.94467 .02842 L
.94468 .02842 L
.94468 .02842 L
.94468 .02842 L
.94468 .02842 L
.94468 .02842 L
.94469 .02842 L
.94469 .02842 L
.94469 .02842 L
.94469 .02842 L
.9447 .02842 L
.9447 .02842 L
.9447 .02842 L
Mistroke
.9447 .02842 L
.9447 .02842 L
.94471 .02842 L
.94471 .02842 L
.94471 .02842 L
.94471 .02842 L
.94471 .02842 L
.94471 .02842 L
.94472 .02842 L
.94472 .02842 L
.94472 .02842 L
.94472 .02842 L
.94472 .02842 L
.94473 .02842 L
.94473 .02842 L
.94473 .02842 L
.94473 .02842 L
.94473 .02842 L
.94474 .02842 L
.94474 .02842 L
.94474 .02842 L
.94474 .02842 L
.94474 .02842 L
.94475 .02842 L
.94475 .02842 L
.94475 .02842 L
.94475 .02842 L
.94475 .02842 L
.94476 .02842 L
.94476 .02842 L
.94476 .02842 L
.94476 .02842 L
.94476 .02842 L
.94477 .02842 L
.94477 .02842 L
.94477 .02842 L
.94477 .02842 L
.94477 .02842 L
.94477 .02842 L
.94478 .02842 L
.94478 .02842 L
.94478 .02842 L
.94478 .02842 L
.94478 .02842 L
.94479 .02842 L
.94479 .02842 L
.94479 .02842 L
.94479 .02842 L
.94479 .02842 L
.9448 .02842 L
Mistroke
.9448 .02842 L
.9448 .02842 L
.9448 .02842 L
.9448 .02842 L
.9448 .02842 L
.94481 .02842 L
.94481 .02842 L
.94481 .02842 L
.94481 .02842 L
.94481 .02842 L
.94482 .02842 L
.94482 .02842 L
.94482 .02842 L
.94482 .02842 L
.94482 .02842 L
.94483 .02842 L
.94483 .02842 L
.94483 .02842 L
.94483 .02842 L
.94483 .02842 L
.94484 .02842 L
.94484 .02842 L
.94484 .02842 L
.94484 .02842 L
.94484 .02842 L
.94485 .02842 L
.94485 .02842 L
.94485 .02842 L
.94485 .02842 L
.94485 .02842 L
.94485 .02842 L
.94486 .02842 L
.94486 .02842 L
.94486 .02842 L
.94486 .02842 L
.94486 .02842 L
.94486 .02842 L
.94486 .02842 L
.94487 .02842 L
.94487 .02842 L
.94487 .02842 L
.94487 .02842 L
.94487 .02842 L
.94487 .02842 L
.94487 .02842 L
.94488 .02842 L
.94488 .02842 L
.94488 .02842 L
.94488 .02842 L
.94488 .02842 L
Mistroke
.94488 .02842 L
.94488 .02842 L
.94489 .02842 L
.94489 .02842 L
.94489 .02842 L
.94489 .02842 L
.94489 .02842 L
.94489 .02842 L
.9449 .02842 L
.9449 .02842 L
.9449 .02842 L
.9449 .02842 L
.9449 .02842 L
.9449 .02842 L
.9449 .02842 L
.94491 .02842 L
.94491 .02842 L
.94491 .02842 L
.94491 .02842 L
.94491 .02842 L
.94491 .02842 L
.94491 .02842 L
.94492 .02842 L
.94492 .02842 L
.94492 .02842 L
.94492 .02842 L
.94492 .02842 L
.94492 .02842 L
.94492 .02842 L
.94492 .02842 L
.94493 .02842 L
.94493 .02842 L
.94493 .02842 L
.94493 .02842 L
.94493 .02842 L
.94493 .02842 L
.94493 .02842 L
.94494 .02842 L
.94494 .02842 L
.94494 .02842 L
.94494 .02842 L
.94494 .02842 L
.94494 .02842 L
.94494 .02842 L
.94494 .02842 L
.94495 .02842 L
.94495 .02842 L
.94495 .02842 L
.94495 .02842 L
.94495 .02842 L
Mistroke
.94495 .02842 L
.94495 .02842 L
.94496 .02842 L
.94496 .02842 L
.94496 .02842 L
.94496 .02842 L
.94496 .02842 L
.94496 .02842 L
.94497 .02842 L
.94497 .02842 L
.94497 .02842 L
.94497 .02842 L
.94497 .02842 L
.94497 .02842 L
.94497 .02842 L
.94498 .02842 L
.94498 .02842 L
.94498 .02842 L
.94498 .02842 L
.94498 .02842 L
.94498 .02842 L
.94498 .02842 L
.94498 .02842 L
.94499 .02842 L
.94499 .02842 L
.94499 .02842 L
.94499 .02842 L
.94499 .02842 L
.94499 .02842 L
.94499 .02842 L
.94499 .02842 L
.945 .02842 L
.945 .02842 L
.945 .02842 L
.945 .02842 L
.945 .02842 L
.945 .02842 L
.945 .02842 L
.945 .02842 L
.945 .02842 L
.94501 .02842 L
.94501 .02842 L
.94501 .02842 L
.94501 .02842 L
.94501 .02842 L
.94501 .02842 L
.94501 .02842 L
.94501 .02842 L
.94502 .02842 L
.94502 .02842 L
Mistroke
.94502 .02842 L
.94502 .02842 L
.94502 .02842 L
.94502 .02841 L
.94502 .02841 L
.94502 .02841 L
.94502 .02841 L
.94502 .02841 L
.94502 .02841 L
.94503 .02841 L
.94503 .02841 L
.94503 .02841 L
.94503 .02841 L
.94503 .02841 L
.94503 .02841 L
.94503 .02841 L
.94503 .02841 L
.94504 .02841 L
.94504 .02841 L
.94504 .02841 L
.94504 .02841 L
.94504 .02841 L
.94504 .02841 L
.94504 .02841 L
.94504 .02841 L
.94504 .02841 L
.94505 .02841 L
.94505 .02841 L
.94505 .02841 L
.94505 .02841 L
.94505 .02841 L
.94505 .02841 L
.94505 .02841 L
.94505 .02841 L
.94505 .02841 L
.94505 .02841 L
.94506 .02841 L
.94506 .02841 L
.94506 .02841 L
.94506 .02841 L
.94506 .02841 L
.94506 .02841 L
.94506 .02841 L
.94506 .02841 L
.94506 .02841 L
.94507 .02841 L
.94507 .02841 L
.94507 .02841 L
.94507 .02841 L
.94507 .02841 L
Mistroke
.94507 .02841 L
.94507 .02841 L
.94507 .02841 L
.94507 .02841 L
.94507 .02841 L
.94508 .02841 L
.94508 .02841 L
.94508 .02841 L
.94508 .02841 L
.94508 .02841 L
.94508 .02841 L
.94508 .02841 L
.94508 .02841 L
.94508 .02841 L
.94509 .02841 L
.94509 .02841 L
.94509 .02841 L
.94509 .02841 L
.94509 .02841 L
.94509 .02841 L
.94509 .02841 L
.94509 .02841 L
.94509 .02841 L
.9451 .02841 L
.9451 .02841 L
.9451 .02841 L
.9451 .02841 L
.9451 .02841 L
.9451 .02841 L
.9451 .02841 L
.9451 .02841 L
.9451 .02841 L
.9451 .02841 L
.9451 .02841 L
.94511 .02841 L
.94511 .02841 L
.94511 .02841 L
.94511 .02841 L
.94511 .02841 L
.94511 .02841 L
.94511 .02841 L
.94511 .02841 L
.94511 .02841 L
.94511 .02841 L
.94511 .02841 L
.94512 .02841 L
.94512 .02841 L
.94512 .02841 L
.94512 .02841 L
.94512 .02841 L
Mistroke
.94512 .02841 L
.94512 .02841 L
.94512 .02841 L
.94512 .02841 L
.94512 .02841 L
.94512 .02841 L
.94513 .02841 L
.94513 .02841 L
.94513 .02841 L
.94513 .02841 L
.94513 .02841 L
.94513 .02841 L
.94513 .02841 L
.94513 .02841 L
.94513 .02841 L
.94513 .02841 L
.94514 .02841 L
.94514 .02841 L
.94514 .02841 L
.94514 .02841 L
.94514 .02841 L
.94514 .02841 L
.94514 .02841 L
.94514 .02841 L
.94514 .02841 L
.94514 .02841 L
.94514 .02841 L
.94515 .02841 L
.94515 .02841 L
.94515 .02841 L
.94515 .02841 L
.94515 .02841 L
.94515 .02841 L
.94515 .02841 L
.94515 .02841 L
.94515 .02841 L
.94515 .02841 L
.94516 .02841 L
.94516 .02841 L
.94516 .02841 L
.94516 .02841 L
.94516 .02841 L
.94516 .02841 L
.94516 .02841 L
.94516 .02841 L
.94516 .02841 L
.94516 .02841 L
.94516 .02841 L
.94517 .02841 L
.94517 .02841 L
Mistroke
.94517 .02841 L
.94517 .02841 L
.94517 .02841 L
.94517 .02841 L
.94517 .02841 L
.94517 .02841 L
.94517 .02841 L
.94517 .02841 L
.94517 .02841 L
.94518 .02841 L
.94518 .02841 L
.94518 .02841 L
.94518 .02841 L
.94518 .02841 L
.94518 .02841 L
.94518 .02841 L
.94518 .02841 L
.94518 .02841 L
.94518 .02841 L
.94518 .02841 L
.94518 .02841 L
.94519 .02841 L
.94519 .02841 L
.94519 .02841 L
.94519 .02841 L
.94519 .02841 L
.94519 .02841 L
.94519 .02841 L
.94519 .02841 L
.94519 .02841 L
.94519 .02841 L
.94519 .02841 L
.9452 .02841 L
.9452 .02841 L
.9452 .02841 L
.9452 .02841 L
.9452 .02841 L
.9452 .02841 L
.9452 .02841 L
.9452 .02841 L
.9452 .02841 L
.9452 .02841 L
.9452 .02841 L
.9452 .02841 L
.94521 .02841 L
.94521 .02841 L
.94521 .02841 L
.94521 .02841 L
.94521 .02841 L
.94521 .02841 L
Mistroke
.94521 .02841 L
.94521 .02841 L
.94521 .02841 L
.94521 .02841 L
.94521 .02841 L
.94521 .02841 L
.94521 .02841 L
.94521 .02841 L
.94521 .02841 L
.94521 .02841 L
.94522 .02841 L
.94522 .02841 L
.94522 .02841 L
.94522 .02841 L
.94522 .02841 L
.94522 .02841 L
.94522 .02841 L
.94522 .02841 L
.94522 .02841 L
.94522 .02841 L
.94522 .02841 L
.94522 .02841 L
.94522 .02841 L
.94522 .02841 L
.94522 .02841 L
.94522 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94523 .02841 L
.94524 .02841 L
.94524 .02841 L
.94524 .02841 L
.94524 .02841 L
.94524 .02841 L
.94524 .02841 L
.94524 .02841 L
Mistroke
.94524 .02841 L
.94524 .02841 L
.94524 .02841 L
.94524 .02841 L
.94524 .02841 L
.94524 .02841 L
.94524 .02841 L
.94524 .02841 L
.94524 .02841 L
.94524 .02841 L
.94524 .02841 L
.94525 .02841 L
.94525 .02841 L
.94525 .02841 L
.94525 .02841 L
.94525 .02841 L
.94525 .02841 L
.94525 .02841 L
.94525 .02841 L
.94525 .02841 L
.94525 .02841 L
.94525 .02841 L
.94525 .02841 L
.94525 .02841 L
.94525 .02841 L
.94525 .02841 L
.94525 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94526 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
Mistroke
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94527 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94528 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
Mistroke
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.94529 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.9453 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94531 .02841 L
.94532 .02841 L
Mistroke
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94532 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94533 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
Mistroke
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94534 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94535 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
Mistroke
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94536 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94537 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94538 .02841 L
.94539 .02841 L
.94539 .02841 L
.94539 .02841 L
.94539 .02841 L
.94539 .02841 L
Mistroke
.94539 .02841 L
.94539 .02841 L
.94539 .02841 L
.94539 .02841 L
.94539 .02841 L
.94539 .02841 L
.94539 .02841 L
.94539 .02841 L
.94539 .02841 L
.94539 .02841 L
.94539 .02841 L
.94539 .02841 L
.94539 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.9454 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
.94541 .02841 L
Mistroke
.94541 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94542 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94543 .02841 L
.94544 .02841 L
.94544 .02841 L
Mistroke
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94544 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94545 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
Mistroke
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94546 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94547 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
Mistroke
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94548 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.94549 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
Mistroke
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.9455 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94551 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
Mistroke
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94552 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94553 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
Mistroke
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94554 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94555 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
Mistroke
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94556 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94557 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
Mistroke
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94558 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.94559 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
Mistroke
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.9456 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94561 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
Mistroke
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94562 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94563 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94564 .02841 L
.94565 .02841 L
.94565 .02841 L
Mistroke
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94565 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94566 .02841 L
.94567 .02841 L
.94567 .02841 L
.94567 .02841 L
.94567 .02841 L
.94567 .02841 L
.94567 .02841 L
.94567 .02841 L
.94567 .02841 L
.94567 .02841 L
.94567 .02841 L
Mistroke
.94567 .02841 L
.94567 .02841 L
.94567 .02841 L
.94567 .02841 L
.94567 .02841 L
.94567 .02841 L
.94567 .02841 L
.94567 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94568 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.94569 .02841 L
.9457 .02841 L
.9457 .02841 L
.9457 .02841 L
.9457 .02841 L
Mistroke
.9457 .02841 L
.9457 .02841 L
.9457 .02841 L
.9457 .02841 L
.9457 .02841 L
.9457 .02841 L
.9457 .02841 L
.9457 .02841 L
.9457 .02841 L
.9457 .02841 L
.9457 .02841 L
.9457 .02841 L
.9457 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94571 .02841 L
.94572 .02841 L
.94572 .02841 L
.94572 .02841 L
.94572 .02841 L
.94572 .02841 L
.94572 .02841 L
.94572 .02841 L
.94572 .02841 L
.94572 .02841 L
.94572 .02841 L
.94572 .02841 L
.94572 .02841 L
.94572 .02841 L
.94572 .02841 L
.94572 .02841 L
.94572 .02841 L
.94573 .02841 L
.94573 .02841 L
.94573 .02841 L
Mistroke
.94573 .02841 L
.94573 .02841 L
.94573 .02841 L
.94573 .02841 L
.94573 .02841 L
.94573 .02841 L
.94573 .02841 L
.94573 .02841 L
.94573 .02841 L
.94573 .02841 L
.94573 .02841 L
.94574 .02841 L
.94574 .02841 L
.94574 .02841 L
.94574 .02841 L
.94574 .02841 L
.94574 .02841 L
.94574 .02841 L
.94574 .02841 L
.94574 .02841 L
.94574 .02841 L
.94574 .02841 L
.94574 .02841 L
.94574 .02841 L
.94575 .02841 L
.94575 .02841 L
.94575 .02841 L
.94575 .02841 L
.94575 .02841 L
.94575 .02841 L
.94575 .02841 L
.94575 .02841 L
.94575 .02841 L
.94575 .02841 L
.94575 .02841 L
.94575 .02841 L
.94576 .02841 L
.94576 .02841 L
.94576 .02841 L
.94576 .02841 L
.94576 .02841 L
.94576 .02841 L
.94576 .02841 L
.94576 .02841 L
.94576 .02841 L
.94576 .02841 L
.94576 .02841 L
.94576 .02841 L
.94576 .02841 L
.94577 .02841 L
Mistroke
.94577 .02841 L
.94577 .02841 L
.94577 .02841 L
.94577 .02841 L
.94577 .02841 L
.94577 .02841 L
.94577 .02841 L
.94577 .02841 L
.94577 .02841 L
.94577 .02841 L
.94578 .02841 L
.94578 .02841 L
.94578 .02841 L
.94578 .02841 L
.94578 .02841 L
.94578 .02841 L
.94578 .02841 L
.94578 .02841 L
.94578 .02841 L
.94578 .02841 L
.94578 .02841 L
.94578 .02841 L
.94578 .02841 L
.94579 .02841 L
.94579 .02841 L
.94579 .02841 L
.94579 .02841 L
.94579 .02841 L
.94579 .02841 L
.94579 .02841 L
.94579 .02841 L
.94579 .02841 L
.94579 .02841 L
.94579 .02841 L
.94579 .02841 L
.9458 .02841 L
.9458 .02841 L
.9458 .02841 L
.9458 .02841 L
.9458 .02841 L
.9458 .02841 L
.9458 .02841 L
.9458 .02841 L
.9458 .02841 L
.9458 .02841 L
.9458 .02841 L
.9458 .02841 L
.94581 .02841 L
.94581 .02841 L
.94581 .02841 L
Mistroke
.94581 .02841 L
.94581 .02841 L
.94581 .02841 L
.94581 .02841 L
.94581 .02841 L
.94581 .02841 L
.94581 .02841 L
.94581 .02841 L
.94582 .02841 L
.94582 .02841 L
.94582 .02841 L
.94582 .02841 L
.94582 .02841 L
.94582 .02841 L
.94582 .02841 L
.94582 .02841 L
.94582 .02841 L
.94582 .02841 L
.94583 .02841 L
.94583 .02841 L
.94583 .02841 L
.94583 .02841 L
.94583 .02841 L
.94583 .02841 L
.94583 .02841 L
.94583 .02841 L
.94583 .02841 L
.94583 .02841 L
.94583 .02841 L
.94584 .02841 L
.94584 .02841 L
.94584 .02841 L
.94584 .02841 L
.94584 .02841 L
.94584 .02841 L
.94584 .02841 L
.94584 .02841 L
.94584 .02841 L
.94584 .02841 L
.94584 .02841 L
.94585 .02841 L
.94585 .02841 L
.94585 .02841 L
.94585 .02841 L
.94585 .02841 L
.94585 .02841 L
.94585 .02841 L
.94585 .02841 L
.94585 .02841 L
.94585 .02841 L
Mistroke
.94586 .02841 L
.94586 .02841 L
.94586 .02841 L
.94586 .02841 L
.94586 .02841 L
.94586 .02841 L
.94586 .02841 L
.94586 .02841 L
.94586 .02841 L
.94586 .02841 L
.94587 .02841 L
.94587 .02841 L
.94587 .02841 L
.94587 .02841 L
.94587 .02841 L
.94587 .02841 L
.94587 .02841 L
.94587 .02841 L
.94587 .02841 L
.94587 .02841 L
.94588 .02841 L
.94588 .02841 L
.94588 .02841 L
.94588 .02841 L
.94588 .02841 L
.94588 .02841 L
.94588 .02841 L
.94588 .02841 L
.94588 .02841 L
.94588 .02841 L
.94589 .02841 L
.94589 .02841 L
.94589 .02841 L
.94589 .02841 L
.94589 .02841 L
.94589 .02841 L
.94589 .02841 L
.94589 .02841 L
.94589 .02841 L
.9459 .02841 L
.9459 .02841 L
.9459 .02841 L
.9459 .02841 L
.9459 .02841 L
.9459 .02841 L
.9459 .02841 L
.9459 .02841 L
.9459 .02841 L
.9459 .02841 L
.94591 .02841 L
Mistroke
.94591 .02841 L
.94591 .02841 L
.94591 .02841 L
.94591 .02841 L
.94591 .02841 L
.94591 .02841 L
.94591 .02841 L
.94591 .02841 L
.94592 .02841 L
.94592 .02841 L
.94592 .02841 L
.94592 .02841 L
.94592 .02841 L
.94592 .02841 L
.94592 .02841 L
.94592 .02841 L
.94592 .02841 L
.94593 .02841 L
.94593 .02841 L
.94593 .02841 L
.94593 .02841 L
.94593 .02841 L
.94593 .02841 L
.94593 .02841 L
.94593 .02841 L
.94593 .02842 L
.94594 .02842 L
.94594 .02842 L
.94594 .02842 L
.94594 .02842 L
.94594 .02842 L
.94594 .02842 L
.94594 .02842 L
.94594 .02842 L
.94595 .02842 L
.94595 .02842 L
.94595 .02842 L
.94595 .02842 L
.94595 .02842 L
.94595 .02842 L
.94595 .02842 L
.94595 .02842 L
.94596 .02842 L
.94596 .02842 L
.94596 .02842 L
.94596 .02842 L
.94596 .02842 L
.94596 .02842 L
.94596 .02842 L
.94596 .02842 L
Mistroke
.94597 .02842 L
.94597 .02842 L
.94597 .02842 L
.94597 .02842 L
.94597 .02842 L
.94597 .02842 L
.94597 .02842 L
.94597 .02842 L
.94597 .02842 L
.94598 .02842 L
.94598 .02842 L
.94598 .02842 L
.94598 .02842 L
.94598 .02842 L
.94598 .02842 L
.94598 .02842 L
.94598 .02842 L
.94599 .02842 L
.94599 .02842 L
.94599 .02842 L
.94599 .02842 L
.94599 .02842 L
.94599 .02842 L
.94599 .02842 L
.94599 .02842 L
.946 .02842 L
.946 .02842 L
.946 .02842 L
.946 .02842 L
.946 .02842 L
.946 .02842 L
.946 .02842 L
.94601 .02842 L
.94601 .02842 L
.94601 .02842 L
.94601 .02842 L
.94601 .02842 L
.94601 .02842 L
.94601 .02842 L
.94602 .02842 L
.94602 .02842 L
.94602 .02842 L
.94602 .02842 L
.94602 .02842 L
.94602 .02842 L
.94602 .02842 L
.94602 .02842 L
.94603 .02842 L
.94603 .02842 L
.94603 .02842 L
Mistroke
.94603 .02842 L
.94603 .02842 L
.94603 .02842 L
.94604 .02842 L
.94604 .02842 L
.94604 .02842 L
.94604 .02842 L
.94604 .02842 L
.94604 .02842 L
.94605 .02842 L
.94605 .02842 L
.94605 .02842 L
.94605 .02842 L
.94605 .02842 L
.94605 .02842 L
.94606 .02842 L
.94606 .02842 L
.94606 .02842 L
.94606 .02842 L
.94606 .02842 L
.94606 .02842 L
.94607 .02842 L
.94607 .02842 L
.94607 .02842 L
.94607 .02842 L
.94607 .02842 L
.94608 .02842 L
.94608 .02842 L
.94608 .02842 L
.94608 .02842 L
.94608 .02842 L
.94608 .02842 L
.94608 .02842 L
.94609 .02842 L
.94609 .02842 L
.94609 .02842 L
.94609 .02842 L
.94609 .02842 L
.9461 .02842 L
.9461 .02842 L
.9461 .02842 L
.9461 .02842 L
.9461 .02842 L
.9461 .02842 L
.94611 .02842 L
.94611 .02842 L
.94611 .02842 L
.94611 .02842 L
.94611 .02842 L
.94612 .02842 L
Mistroke
.94612 .02842 L
.94612 .02842 L
.94612 .02842 L
.94612 .02842 L
.94612 .02842 L
.94613 .02842 L
.94613 .02842 L
.94613 .02842 L
.94613 .02842 L
.94613 .02842 L
.94613 .02842 L
.94614 .02842 L
.94614 .02842 L
.94614 .02842 L
.94614 .02842 L
.94614 .02842 L
.94614 .02842 L
.94615 .02842 L
.94615 .02842 L
.94615 .02842 L
.94615 .02842 L
.94615 .02842 L
.94616 .02842 L
.94616 .02842 L
.94616 .02842 L
.94616 .02842 L
.94616 .02842 L
.94616 .02842 L
.94617 .02842 L
.94617 .02842 L
.94617 .02842 L
.94617 .02842 L
.94617 .02842 L
.94618 .02842 L
.94618 .02842 L
.94618 .02842 L
.94618 .02842 L
.94618 .02842 L
.94619 .02842 L
.94619 .02842 L
.94619 .02842 L
.94619 .02842 L
.94619 .02842 L
.94619 .02842 L
.9462 .02842 L
.9462 .02842 L
.9462 .02842 L
.9462 .02842 L
.9462 .02842 L
.94621 .02842 L
Mistroke
.94621 .02842 L
.94621 .02842 L
.94621 .02842 L
.94621 .02842 L
.94621 .02842 L
.94622 .02842 L
.94622 .02842 L
.94622 .02842 L
.94622 .02842 L
.94622 .02842 L
.94622 .02842 L
.94623 .02842 L
.94623 .02842 L
.94623 .02842 L
.94623 .02842 L
.94623 .02842 L
.94624 .02842 L
.94624 .02842 L
.94624 .02842 L
.94624 .02842 L
.94625 .02842 L
.94625 .02842 L
.94625 .02842 L
.94625 .02842 L
.94625 .02842 L
.94625 .02842 L
.94626 .02842 L
.94626 .02842 L
.94626 .02842 L
.94626 .02842 L
.94626 .02842 L
.94627 .02842 L
.94627 .02842 L
.94627 .02842 L
.94627 .02842 L
.94627 .02842 L
.94627 .02842 L
.94628 .02842 L
.94628 .02842 L
.94628 .02842 L
.94628 .02842 L
.94628 .02842 L
.94629 .02842 L
.94629 .02842 L
.94629 .02842 L
.94629 .02842 L
.9463 .02842 L
.9463 .02842 L
.9463 .02842 L
.9463 .02842 L
Mistroke
.9463 .02842 L
.94631 .02842 L
.94631 .02842 L
.94631 .02842 L
.94631 .02842 L
.94631 .02842 L
.94632 .02842 L
.94632 .02842 L
.94632 .02842 L
.94632 .02842 L
.94632 .02842 L
.94632 .02842 L
.94633 .02842 L
.94633 .02842 L
.94633 .02842 L
.94633 .02842 L
.94633 .02842 L
.94634 .02842 L
.94634 .02842 L
.94634 .02842 L
.94634 .02842 L
.94635 .02842 L
.94635 .02842 L
.94635 .02842 L
.94635 .02842 L
.94636 .02842 L
.94636 .02842 L
.94636 .02842 L
.94636 .02842 L
.94637 .02842 L
.94637 .02842 L
.94637 .02842 L
.94637 .02842 L
.94638 .02842 L
.94638 .02842 L
.94638 .02842 L
.94638 .02842 L
.94638 .02842 L
.94639 .02842 L
.94639 .02842 L
.94639 .02842 L
.94639 .02842 L
.9464 .02842 L
.9464 .02842 L
.9464 .02842 L
.94641 .02842 L
.94641 .02842 L
.94641 .02842 L
.94641 .02842 L
.94642 .02842 L
Mistroke
.94642 .02842 L
.94642 .02842 L
.94642 .02842 L
.94643 .02842 L
.94643 .02842 L
.94643 .02842 L
.94644 .02842 L
.94644 .02842 L
.94644 .02842 L
.94644 .02842 L
.94645 .02842 L
.94645 .02842 L
.94645 .02842 L
.94645 .02842 L
.94646 .02842 L
.94646 .02842 L
.94646 .02842 L
.94647 .02842 L
.94647 .02842 L
.94647 .02842 L
.94647 .02842 L
.94648 .02842 L
.94648 .02842 L
.94648 .02842 L
.94649 .02842 L
.94649 .02842 L
.94649 .02842 L
.94649 .02842 L
.9465 .02842 L
.9465 .02842 L
.9465 .02842 L
.94651 .02842 L
.94651 .02842 L
.94651 .02842 L
.94652 .02842 L
.94652 .02842 L
.94652 .02842 L
.94653 .02842 L
.94653 .02842 L
.94653 .02842 L
.94654 .02842 L
.94654 .02842 L
.94654 .02842 L
.94654 .02842 L
.94655 .02842 L
.94655 .02842 L
.94655 .02842 L
.94656 .02842 L
.94656 .02842 L
.94656 .02842 L
Mistroke
.94657 .02842 L
.94657 .02842 L
.94657 .02842 L
.94658 .02842 L
.94658 .02842 L
.94658 .02842 L
.94659 .02842 L
.94659 .02842 L
.94659 .02842 L
.9466 .02842 L
.9466 .02842 L
.9466 .02842 L
.94661 .02842 L
.94661 .02842 L
.94661 .02842 L
.94662 .02842 L
.94662 .02842 L
.94662 .02842 L
.94663 .02842 L
.94663 .02842 L
.94663 .02842 L
.94664 .02842 L
.94664 .02842 L
.94664 .02842 L
.94665 .02842 L
.94665 .02842 L
.94665 .02842 L
.94665 .02842 L
.94666 .02842 L
.94666 .02842 L
.94666 .02842 L
.94667 .02842 L
.94667 .02842 L
.94667 .02842 L
.94668 .02842 L
.94668 .02842 L
.94668 .02842 L
.94669 .02842 L
.94669 .02842 L
.94669 .02842 L
.9467 .02842 L
.9467 .02842 L
.9467 .02842 L
.94671 .02842 L
.94671 .02842 L
.94672 .02842 L
.94672 .02842 L
.94672 .02842 L
.94673 .02842 L
.94673 .02842 L
Mistroke
.94673 .02842 L
.94674 .02842 L
.94674 .02842 L
.94674 .02842 L
.94675 .02842 L
.94675 .02842 L
.94675 .02842 L
.94676 .02842 L
.94676 .02842 L
.94676 .02842 L
.94677 .02842 L
.94677 .02842 L
.94677 .02842 L
.94678 .02842 L
.94678 .02842 L
.94678 .02842 L
.94679 .02842 L
.94679 .02842 L
.94679 .02842 L
.9468 .02842 L
.9468 .02842 L
.9468 .02842 L
.94681 .02842 L
.94681 .02842 L
.94681 .02842 L
.94682 .02842 L
.94682 .02842 L
.94682 .02842 L
.94683 .02842 L
.94683 .02842 L
.94683 .02842 L
.94684 .02842 L
.94684 .02842 L
.94684 .02842 L
.94685 .02842 L
.94685 .02842 L
.94685 .02842 L
.94686 .02842 L
.94686 .02842 L
.94687 .02842 L
.94687 .02842 L
.94687 .02842 L
.94688 .02842 L
.94688 .02842 L
.94688 .02842 L
.94689 .02842 L
.94689 .02842 L
.94689 .02842 L
.9469 .02842 L
.9469 .02842 L
Mistroke
.9469 .02842 L
.94691 .02842 L
.94691 .02842 L
.94691 .02842 L
.94691 .02842 L
.94692 .02842 L
.94692 .02842 L
.94693 .02842 L
.94693 .02842 L
.94694 .02842 L
.94694 .02842 L
.94694 .02842 L
.94695 .02842 L
.94695 .02842 L
.94695 .02842 L
.94696 .02842 L
.94696 .02842 L
.94696 .02842 L
.94697 .02842 L
.94697 .02842 L
.94698 .02842 L
.94698 .02842 L
.94698 .02842 L
.94699 .02842 L
.94699 .02842 L
.947 .02842 L
.947 .02842 L
.947 .02842 L
.94701 .02842 L
.94701 .02842 L
.94701 .02842 L
.94702 .02842 L
.94702 .02842 L
.94703 .02842 L
.94703 .02842 L
.94703 .02842 L
.94704 .02842 L
.94704 .02842 L
.94705 .02842 L
.94705 .02842 L
.94705 .02842 L
.94706 .02842 L
.94706 .02842 L
.94707 .02842 L
.94707 .02842 L
.94708 .02842 L
.94708 .02842 L
.94708 .02842 L
.94709 .02842 L
.94709 .02842 L
Mistroke
.94709 .02842 L
.9471 .02842 L
.9471 .02842 L
.9471 .02842 L
.94711 .02842 L
.94711 .02842 L
.94712 .02842 L
.94712 .02842 L
.94712 .02842 L
.94713 .02842 L
.94713 .02842 L
.94713 .02842 L
.94714 .02842 L
.94714 .02842 L
.94714 .02842 L
.94715 .02842 L
.94716 .02842 L
.94716 .02842 L
.94716 .02842 L
.94717 .02842 L
.94717 .02842 L
.94718 .02842 L
.94718 .02842 L
.94719 .02842 L
.94719 .02842 L
.9472 .02842 L
.9472 .02842 L
.94721 .02842 L
.94721 .02842 L
.94722 .02842 L
.94722 .02842 L
.94723 .02842 L
.94723 .02842 L
.94724 .02842 L
.94724 .02842 L
.94724 .02842 L
.94725 .02842 L
.94725 .02842 L
.94726 .02842 L
.94726 .02842 L
.94727 .02842 L
.94728 .02842 L
.94728 .02842 L
.94729 .02842 L
.94729 .02842 L
.94729 .02842 L
.9473 .02842 L
.9473 .02842 L
.94731 .02842 L
.94731 .02842 L
Mistroke
.94732 .02842 L
.94732 .02842 L
.94732 .02842 L
.94733 .02842 L
.94733 .02842 L
.94734 .02842 L
.94734 .02842 L
.94734 .02842 L
.94735 .02842 L
.94735 .02842 L
.94736 .02842 L
.94736 .02842 L
.94736 .02842 L
.94737 .02842 L
.94737 .02842 L
.94738 .02842 L
.94739 .02842 L
.94739 .02842 L
.9474 .02842 L
.9474 .02842 L
.94741 .02842 L
.94741 .02842 L
.94742 .02842 L
.94742 .02842 L
.94743 .02842 L
.94743 .02842 L
.94744 .02842 L
.94744 .02842 L
.94744 .02842 L
.94745 .02842 L
.94746 .02842 L
.94746 .02842 L
.94747 .02842 L
.94747 .02842 L
.94748 .02842 L
.94749 .02842 L
.94749 .02842 L
.9475 .02842 L
.9475 .02842 L
.94751 .02842 L
.94752 .02842 L
.94752 .02842 L
.94753 .02842 L
.94754 .02842 L
.94754 .02842 L
.94755 .02842 L
.94755 .02842 L
.94756 .02842 L
.94756 .02842 L
.94757 .02842 L
Mistroke
.94758 .02842 L
.94758 .02842 L
.94759 .02842 L
.94759 .02842 L
.9476 .02842 L
.94761 .02842 L
.94761 .02842 L
.94762 .02842 L
.94762 .02842 L
.94763 .02842 L
.94764 .02842 L
.94764 .02842 L
.94765 .02842 L
.94765 .02842 L
.94766 .02842 L
.94766 .02842 L
.94767 .02842 L
.94767 .02842 L
.94768 .02842 L
.94768 .02842 L
.94769 .02842 L
.9477 .02842 L
.9477 .02842 L
.94771 .02842 L
.94772 .02842 L
.94772 .02842 L
.94773 .02842 L
.94773 .02842 L
.94774 .02842 L
.94775 .02842 L
.94775 .02842 L
.94776 .02843 L
.94776 .02843 L
.94777 .02843 L
.94778 .02843 L
.94778 .02843 L
.94779 .02843 L
.94779 .02843 L
.9478 .02843 L
.94781 .02843 L
.94781 .02843 L
.94782 .02843 L
.94783 .02843 L
.94783 .02843 L
.94784 .02843 L
.94784 .02843 L
.94785 .02843 L
.94786 .02843 L
.94786 .02843 L
.94787 .02843 L
Mistroke
.94788 .02843 L
.94788 .02843 L
.94789 .02843 L
.9479 .02843 L
.9479 .02843 L
.94791 .02843 L
.94791 .02843 L
.94792 .02843 L
.94793 .02843 L
.94793 .02843 L
.94794 .02843 L
.94795 .02843 L
.94795 .02843 L
.94796 .02843 L
.94797 .02843 L
.94797 .02843 L
.94798 .02843 L
.94798 .02843 L
.94799 .02843 L
.948 .02843 L
.94801 .02843 L
.94801 .02843 L
.94802 .02843 L
.94803 .02843 L
.94804 .02843 L
.94804 .02843 L
.94805 .02843 L
.94805 .02843 L
.94806 .02843 L
.94807 .02843 L
.94807 .02843 L
.94808 .02843 L
.94809 .02843 L
.94809 .02843 L
.9481 .02843 L
.94811 .02843 L
.94812 .02843 L
.94812 .02843 L
.94813 .02843 L
.94814 .02843 L
.94815 .02843 L
.94815 .02843 L
.94816 .02843 L
.94817 .02843 L
.94817 .02843 L
.94818 .02843 L
.94819 .02843 L
.94819 .02843 L
.9482 .02843 L
.94821 .02843 L
Mistroke
.94822 .02843 L
.94822 .02843 L
.94823 .02843 L
.94824 .02843 L
.94825 .02843 L
.94825 .02843 L
.94826 .02843 L
.94826 .02843 L
.94827 .02843 L
.94828 .02843 L
.94829 .02843 L
.94829 .02843 L
.94831 .02843 L
.94831 .02843 L
.94832 .02843 L
.94833 .02843 L
.94833 .02843 L
.94834 .02843 L
.94835 .02843 L
.94835 .02843 L
.94836 .02843 L
.94837 .02843 L
.94838 .02843 L
.94838 .02843 L
.94839 .02843 L
.9484 .02843 L
.94841 .02843 L
.94842 .02843 L
.94842 .02843 L
.94843 .02843 L
.94844 .02843 L
.94844 .02843 L
.94846 .02843 L
.94846 .02843 L
.94847 .02843 L
.94848 .02843 L
.94849 .02843 L
.94849 .02843 L
.9485 .02843 L
.94851 .02843 L
.94851 .02843 L
.94852 .02843 L
.94852 .02843 L
.94853 .02843 L
.94854 .02843 L
.94855 .02843 L
.94855 .02843 L
.94856 .02843 L
.94857 .02843 L
.94857 .02843 L
Mistroke
.94859 .02843 L
.94859 .02843 L
.9486 .02843 L
.94861 .02843 L
.94861 .02843 L
.94863 .02843 L
.94863 .02843 L
.94864 .02843 L
.94865 .02843 L
.94865 .02843 L
.94866 .02843 L
.94867 .02843 L
.94867 .02844 L
.94868 .02844 L
.94869 .02844 L
.9487 .02844 L
.9487 .02844 L
.94871 .02844 L
.94872 .02844 L
.94873 .02844 L
.94873 .02844 L
.94874 .02844 L
.94875 .02844 L
.94876 .02844 L
.94876 .02844 L
.94878 .02844 L
.94878 .02844 L
.94879 .02844 L
.9488 .02844 L
.94881 .02844 L
.94882 .02844 L
.94883 .02844 L
.94884 .02844 L
.94885 .02844 L
.94885 .02844 L
.94886 .02844 L
.94887 .02844 L
.94888 .02844 L
.94889 .02844 L
.94889 .02844 L
.94891 .02844 L
.94891 .02844 L
.94892 .02844 L
.94893 .02844 L
.94894 .02844 L
.94896 .02844 L
.94897 .02844 L
.94898 .02844 L
.94899 .02844 L
.949 .02844 L
Mistroke
.94901 .02844 L
.94902 .02844 L
.94903 .02844 L
.94904 .02844 L
.94906 .02844 L
.94907 .02844 L
.94908 .02844 L
.94909 .02844 L
.94909 .02844 L
.9491 .02844 L
.94912 .02844 L
.94913 .02844 L
.94914 .02844 L
.94915 .02844 L
.94916 .02844 L
.94917 .02844 L
.94918 .02844 L
.94919 .02844 L
.9492 .02844 L
.94921 .02844 L
.94922 .02844 L
.94923 .02844 L
.94924 .02844 L
.94925 .02844 L
.94926 .02844 L
.94927 .02844 L
.94928 .02844 L
.94929 .02844 L
.9493 .02844 L
.94931 .02844 L
.94933 .02844 L
.94934 .02844 L
.94935 .02844 L
.94936 .02844 L
.94937 .02844 L
.94938 .02845 L
.94939 .02845 L
.94941 .02845 L
.94942 .02845 L
.94943 .02845 L
.94944 .02845 L
.94945 .02845 L
.94946 .02845 L
.94947 .02845 L
.94948 .02845 L
.94949 .02845 L
.9495 .02845 L
.94951 .02845 L
.94953 .02845 L
.94954 .02845 L
Mistroke
.94955 .02845 L
.94956 .02845 L
.94957 .02845 L
.94958 .02845 L
.9496 .02845 L
.94961 .02845 L
.94962 .02845 L
.94963 .02845 L
.94965 .02845 L
.94966 .02845 L
.94967 .02845 L
.94968 .02845 L
.94969 .02845 L
.9497 .02845 L
.94972 .02845 L
.94973 .02845 L
.94974 .02845 L
.94975 .02845 L
.94976 .02845 L
.94977 .02845 L
.94978 .02845 L
.94979 .02845 L
.9498 .02845 L
.94981 .02845 L
.94982 .02845 L
.94983 .02845 L
.94985 .02845 L
.94986 .02845 L
.94987 .02845 L
.94989 .02845 L
.9499 .02845 L
.94992 .02845 L
.94993 .02845 L
.94994 .02845 L
.94995 .02845 L
.94997 .02845 L
.94998 .02846 L
.94999 .02846 L
.95 .02846 L
.95002 .02846 L
.95003 .02846 L
.95004 .02846 L
.95005 .02846 L
.95006 .02846 L
.95008 .02846 L
.95009 .02846 L
.9501 .02846 L
.95011 .02846 L
.95013 .02846 L
.95014 .02846 L
Mistroke
.95015 .02846 L
.95016 .02846 L
.95017 .02846 L
.95019 .02846 L
.95021 .02846 L
.95022 .02846 L
.95023 .02846 L
.95024 .02846 L
.95025 .02846 L
.95027 .02846 L
.95028 .02846 L
.95029 .02846 L
.9503 .02846 L
.95031 .02846 L
.95033 .02846 L
.95034 .02846 L
.95036 .02846 L
.95037 .02846 L
.95038 .02846 L
.9504 .02846 L
.95041 .02846 L
.95042 .02846 L
.95044 .02846 L
.95045 .02846 L
.95046 .02846 L
.95048 .02846 L
.95049 .02846 L
.9505 .02847 L
.95051 .02847 L
.95053 .02847 L
.95054 .02847 L
.95056 .02847 L
.95057 .02847 L
.95058 .02847 L
.9506 .02847 L
.95061 .02847 L
.95063 .02847 L
.95064 .02847 L
.95065 .02847 L
.95066 .02847 L
.95068 .02847 L
.95069 .02847 L
.95071 .02847 L
.95072 .02847 L
.95074 .02847 L
.95075 .02847 L
.95076 .02847 L
.95077 .02847 L
.95079 .02847 L
.9508 .02847 L
Mistroke
.95082 .02847 L
.95083 .02847 L
.95084 .02847 L
.95086 .02847 L
.95088 .02847 L
.95089 .02847 L
.9509 .02847 L
.95091 .02847 L
.95093 .02847 L
.95094 .02847 L
.95095 .02847 L
.95096 .02847 L
.95098 .02848 L
.951 .02848 L
.95101 .02848 L
.95103 .02848 L
.95104 .02848 L
.95106 .02848 L
.95107 .02848 L
.95108 .02848 L
.9511 .02848 L
.95111 .02848 L
.95113 .02848 L
.95114 .02848 L
.95115 .02848 L
.95117 .02848 L
.95118 .02848 L
.95119 .02848 L
.95121 .02848 L
.95122 .02848 L
.95124 .02848 L
.95125 .02848 L
.95127 .02848 L
.95128 .02848 L
.9513 .02848 L
.95132 .02848 L
.95133 .02848 L
.95134 .02848 L
.95136 .02848 L
.95138 .02848 L
.95139 .02848 L
.9514 .02848 L
.95142 .02849 L
.95143 .02849 L
.95146 .02849 L
.95147 .02849 L
.95148 .02849 L
.9515 .02849 L
.95151 .02849 L
.95152 .02849 L
Mistroke
.95154 .02849 L
.95155 .02849 L
.95157 .02849 L
.95159 .02849 L
.95161 .02849 L
.95162 .02849 L
.95164 .02849 L
.95165 .02849 L
.95167 .02849 L
.95169 .02849 L
.9517 .02849 L
.95172 .02849 L
.95173 .02849 L
.95175 .02849 L
.95177 .02849 L
.95178 .02849 L
.9518 .02849 L
.95183 .0285 L
.95184 .0285 L
.95186 .0285 L
.95187 .0285 L
.95188 .0285 L
.9519 .0285 L
.95191 .0285 L
.95192 .0285 L
.95194 .0285 L
.95196 .0285 L
.95197 .0285 L
.95199 .0285 L
.952 .0285 L
.95201 .0285 L
.95203 .0285 L
.95205 .0285 L
.95207 .0285 L
.95208 .0285 L
.9521 .0285 L
.95212 .0285 L
.95213 .0285 L
.95214 .0285 L
.95215 .0285 L
.95217 .0285 L
.95218 .0285 L
.9522 .0285 L
.95222 .02851 L
.95223 .02851 L
.95224 .02851 L
.95226 .02851 L
.95228 .02851 L
.9523 .02851 L
.95231 .02851 L
Mistroke
.95233 .02851 L
.95234 .02851 L
.95236 .02851 L
.95238 .02851 L
.95239 .02851 L
.9524 .02851 L
.95242 .02851 L
.95244 .02851 L
.95247 .02851 L
.95249 .02851 L
.9525 .02851 L
.95252 .02851 L
.95253 .02851 L
.95254 .02851 L
.95257 .02852 L
.95258 .02852 L
.9526 .02852 L
.95261 .02852 L
.95263 .02852 L
.95264 .02852 L
.95266 .02852 L
.95268 .02852 L
.95269 .02852 L
.95271 .02852 L
.95272 .02852 L
.95274 .02852 L
.95276 .02852 L
.95279 .02852 L
.95281 .02852 L
.95283 .02852 L
.95286 .02852 L
.95288 .02852 L
.9529 .02852 L
.95291 .02852 L
.95294 .02853 L
.95296 .02853 L
.95299 .02853 L
.95301 .02853 L
.95304 .02853 L
.95306 .02853 L
.95308 .02853 L
.95311 .02853 L
.95313 .02853 L
.95315 .02853 L
.95317 .02853 L
.9532 .02853 L
.95322 .02853 L
.95324 .02853 L
.95326 .02854 L
.95328 .02854 L
Mistroke
.9533 .02854 L
.95332 .02854 L
.95335 .02854 L
.95337 .02854 L
.95339 .02854 L
.95341 .02854 L
.95344 .02854 L
.95347 .02854 L
.95349 .02854 L
.95351 .02854 L
.95354 .02854 L
.95356 .02855 L
.95358 .02855 L
.95361 .02855 L
.95364 .02855 L
.95366 .02855 L
.95369 .02855 L
.95371 .02855 L
.95374 .02855 L
.95376 .02855 L
.95379 .02855 L
.95382 .02855 L
.95385 .02855 L
.95387 .02856 L
.9539 .02856 L
.95393 .02856 L
.95395 .02856 L
.95397 .02856 L
.95399 .02856 L
.95402 .02856 L
.95405 .02856 L
.95407 .02856 L
.9541 .02856 L
.95412 .02856 L
.95414 .02856 L
.95417 .02857 L
.95419 .02857 L
.95421 .02857 L
.95423 .02857 L
.95425 .02857 L
.95428 .02857 L
.9543 .02857 L
.95433 .02857 L
.95435 .02857 L
.95438 .02857 L
.9544 .02857 L
.95443 .02857 L
.95446 .02858 L
.95449 .02858 L
.95451 .02858 L
Mistroke
.95453 .02858 L
.95456 .02858 L
.95459 .02858 L
.95461 .02858 L
.95463 .02858 L
.95465 .02858 L
.95468 .02858 L
.95471 .02858 L
.95474 .02859 L
.95477 .02859 L
.9548 .02859 L
.95483 .02859 L
.95486 .02859 L
.95488 .02859 L
.95491 .02859 L
.95494 .02859 L
.95496 .02859 L
.95499 .0286 L
.95502 .0286 L
.95505 .0286 L
.95507 .0286 L
.9551 .0286 L
.95512 .0286 L
.95514 .0286 L
.95517 .0286 L
.95521 .0286 L
.95523 .0286 L
.95526 .02861 L
.95528 .02861 L
.95531 .02861 L
.95533 .02861 L
.95535 .02861 L
.95538 .02861 L
.95541 .02861 L
.95543 .02861 L
.95545 .02861 L
.95548 .02861 L
.9555 .02861 L
.95554 .02862 L
.95557 .02862 L
.95559 .02862 L
.95561 .02862 L
.95563 .02862 L
.95567 .02862 L
.95569 .02862 L
.95571 .02862 L
.95574 .02862 L
.95576 .02863 L
.95579 .02863 L
.95581 .02863 L
Mistroke
.95585 .02863 L
.95587 .02863 L
.95589 .02863 L
.95592 .02863 L
.95594 .02863 L
.95597 .02863 L
.95599 .02863 L
.95603 .02864 L
.95606 .02864 L
.95608 .02864 L
.9561 .02864 L
.95613 .02864 L
.95615 .02864 L
.95618 .02864 L
.95621 .02864 L
.95623 .02864 L
.95626 .02865 L
.95628 .02865 L
.95633 .02865 L
.95636 .02865 L
.95638 .02865 L
.95641 .02865 L
.95644 .02865 L
.95646 .02865 L
.95649 .02866 L
.95653 .02866 L
.95656 .02866 L
.95658 .02866 L
.95661 .02866 L
.95663 .02866 L
.95667 .02866 L
.95669 .02867 L
.95672 .02867 L
.95675 .02867 L
.95679 .02867 L
.95682 .02867 L
.95684 .02867 L
.95686 .02867 L
.95688 .02867 L
.95691 .02867 L
.95693 .02868 L
.95697 .02868 L
.95699 .02868 L
.95701 .02868 L
.95705 .02868 L
.95708 .02868 L
.9571 .02868 L
.95714 .02869 L
.95716 .02869 L
.95719 .02869 L
Mistroke
.95723 .02869 L
.95726 .02869 L
.95728 .02869 L
.95733 .02869 L
.95736 .0287 L
.9574 .0287 L
.95745 .0287 L
.95749 .0287 L
.95752 .0287 L
.95756 .02871 L
.9576 .02871 L
.95763 .02871 L
.95767 .02871 L
.95771 .02871 L
.95775 .02871 L
.95779 .02872 L
.95783 .02872 L
.95786 .02872 L
.95788 .02872 L
.95792 .02872 L
.95796 .02872 L
.95798 .02873 L
.95801 .02873 L
.95805 .02873 L
.95809 .02873 L
.95813 .02873 L
.95818 .02874 L
.95822 .02874 L
.95825 .02874 L
.95827 .02874 L
.95832 .02874 L
.95836 .02874 L
.9584 .02875 L
.95845 .02875 L
.95849 .02875 L
.95853 .02875 L
.95856 .02876 L
.95861 .02876 L
.95865 .02876 L
.95869 .02876 L
.95874 .02876 L
.95876 .02877 L
.95879 .02877 L
.95883 .02877 L
.95888 .02877 L
.95892 .02877 L
.95896 .02878 L
.959 .02878 L
.95905 .02878 L
.95907 .02878 L
Mistroke
.9591 .02878 L
.95915 .02879 L
.95918 .02879 L
.9592 .02879 L
.95925 .02879 L
.9593 .02879 L
.95934 .0288 L
.95939 .0288 L
.95943 .0288 L
.95948 .0288 L
.95952 .02881 L
.95957 .02881 L
.9596 .02881 L
.95963 .02881 L
.95968 .02882 L
.95972 .02882 L
.95977 .02882 L
.95981 .02882 L
.95985 .02883 L
.95989 .02883 L
.95994 .02883 L
.95999 .02883 L
.96003 .02884 L
.96008 .02884 L
.96012 .02884 L
.96015 .02884 L
.96017 .02884 L
.96023 .02885 L
.96027 .02885 L
.96032 .02885 L
.96037 .02886 L
.96042 .02886 L
.96045 .02886 L
.96048 .02886 L
.96053 .02887 L
.96058 .02887 L
.96063 .02887 L
.96068 .02887 L
.96073 .02888 L
.96078 .02888 L
.96082 .02888 L
.96087 .02889 L
.96091 .02889 L
.96095 .02889 L
.96099 .02889 L
.96104 .0289 L
.96108 .0289 L
.96113 .0289 L
.96118 .0289 L
.96122 .02891 L
Mistroke
.96127 .02891 L
.96131 .02891 L
.96135 .02892 L
.96139 .02892 L
.96144 .02892 L
.96148 .02892 L
.96151 .02893 L
.96155 .02893 L
.96159 .02893 L
.96164 .02893 L
.96168 .02894 L
.96173 .02894 L
.96177 .02894 L
.96182 .02895 L
.96185 .02895 L
.9619 .02895 L
.96194 .02895 L
.96199 .02896 L
.96204 .02896 L
.96208 .02896 L
.96212 .02897 L
.96217 .02897 L
.96223 .02897 L
.96228 .02898 L
.96232 .02898 L
.96236 .02898 L
.9624 .02898 L
.96247 .02899 L
.96251 .02899 L
.96255 .02899 L
.96259 .029 L
.96263 .029 L
.96267 .029 L
.96271 .029 L
.96275 .02901 L
.96279 .02901 L
.96282 .02901 L
.96286 .02901 L
.96292 .02902 L
.96297 .02902 L
.96301 .02903 L
.96309 .02903 L
.96313 .02903 L
.96317 .02904 L
.96322 .02904 L
.96326 .02904 L
.96331 .02905 L
.96335 .02905 L
.96339 .02905 L
.96343 .02905 L
Mistroke
.9635 .02906 L
.96354 .02906 L
.96358 .02907 L
.96363 .02907 L
.96367 .02907 L
.96372 .02908 L
.96376 .02908 L
.96383 .02908 L
.96387 .02909 L
.96391 .02909 L
.96396 .02909 L
.964 .0291 L
.96405 .0291 L
.9641 .0291 L
.96418 .02911 L
.96422 .02911 L
.96427 .02912 L
.96431 .02912 L
.96435 .02912 L
.96439 .02912 L
.96443 .02913 L
.96447 .02913 L
.96451 .02913 L
.96458 .02914 L
.96462 .02914 L
.96466 .02914 L
.9647 .02915 L
.96474 .02915 L
.96479 .02915 L
.96483 .02916 L
.9649 .02916 L
.96495 .02917 L
.96499 .02917 L
.96504 .02917 L
.96508 .02918 L
.96512 .02918 L
.96517 .02918 L
.96522 .02919 L
.96526 .02919 L
.96535 .0292 L
.96539 .0292 L
.96544 .02921 L
.96548 .02921 L
.96552 .02921 L
.9656 .02922 L
.96565 .02922 L
.96569 .02923 L
.96574 .02923 L
.9658 .02923 L
.96585 .02924 L
Mistroke
.96589 .02924 L
.96594 .02925 L
.96598 .02925 L
.96603 .02925 L
.96607 .02926 L
.96616 .02926 L
.9662 .02927 L
.96625 .02927 L
.96634 .02928 L
.96638 .02928 L
.96643 .02929 L
.96651 .02929 L
.96656 .0293 L
.9666 .0293 L
.96665 .0293 L
.9667 .02931 L
.96674 .02931 L
.96678 .02932 L
.96686 .02932 L
.96691 .02933 L
.96695 .02933 L
.967 .02933 L
.96704 .02934 L
.96712 .02934 L
.96716 .02935 L
.96721 .02935 L
.96728 .02936 L
.96733 .02936 L
.96737 .02937 L
.96744 .02937 L
.96751 .02938 L
.96755 .02938 L
.96759 .02938 L
.96767 .02939 L
.96771 .0294 L
.96775 .0294 L
.96779 .0294 L
.96784 .02941 L
.96791 .02941 L
.96795 .02942 L
.96799 .02942 L
.96807 .02943 L
.96813 .02943 L
.96821 .02944 L
.96829 .02945 L
.96836 .02945 L
.9684 .02946 L
.96845 .02946 L
.96853 .02947 L
.9686 .02948 L
Mistroke
.96864 .02948 L
.96868 .02948 L
.96876 .02949 L
.96885 .0295 L
.96889 .0295 L
.96894 .02951 L
.96901 .02951 L
.96909 .02952 L
.96916 .02953 L
.96923 .02953 L
.9693 .02954 L
.96936 .02955 L
.96943 .02955 L
.9695 .02956 L
.96958 .02957 L
.96965 .02957 L
.96972 .02958 L
.96979 .02959 L
.96987 .02959 L
.96995 .0296 L
.97002 .02961 L
.9701 .02962 L
.97017 .02962 L
.97025 .02963 L
.97033 .02964 L
.9704 .02965 L
.97048 .02965 L
.97057 .02966 L
.97065 .02967 L
.97073 .02968 L
.97081 .02969 L
.97088 .02969 L
.97096 .0297 L
.97105 .02971 L
.97114 .02972 L
.97122 .02973 L
.9713 .02974 L
.97138 .02975 L
.97146 .02975 L
.97154 .02976 L
.97161 .02977 L
.97168 .02978 L
.97176 .02978 L
.97184 .02979 L
.97191 .0298 L
.97199 .02981 L
.97206 .02982 L
.97213 .02982 L
.97221 .02983 L
.97228 .02984 L
Mistroke
.97235 .02985 L
.97241 .02985 L
.97248 .02986 L
.97255 .02987 L
.97263 .02988 L
.97271 .02988 L
.97278 .02989 L
.97286 .0299 L
.97294 .02991 L
.97303 .02992 L
.97311 .02993 L
.97319 .02994 L
.97326 .02995 L
.97333 .02995 L
.97341 .02996 L
.97349 .02997 L
.97356 .02998 L
.97363 .02999 L
.9737 .02999 L
.97378 .03 L
.97387 .03001 L
.97395 .03002 L
.97404 .03003 L
.97414 .03004 L
.97423 .03005 L
.97432 .03006 L
.9744 .03007 L
.97448 .03008 L
.97456 .03009 L
.97464 .0301 L
.97473 .03011 L
.97481 .03012 L
.97489 .03013 L
.97496 .03014 L
.97504 .03015 L
.97512 .03016 L
.97519 .03017 L
.97526 .03017 L
.97533 .03018 L
.97541 .03019 L
.97553 .03021 L
.97561 .03021 L
.97568 .03022 L
.97576 .03023 L
.97583 .03024 L
.9759 .03025 L
.97597 .03026 L
.97605 .03027 L
.97614 .03028 L
.97621 .03029 L
Mistroke
.97629 .0303 L
.97637 .03031 L
.97645 .03032 L
.97652 .03032 L
.97659 .03033 L
.97667 .03034 L
.97676 .03036 L
.97684 .03036 L
.97692 .03037 L
.977 .03038 L
.97707 .03039 L
.97714 .0304 L
.97722 .03041 L
.97736 .03043 L
.97744 .03044 L
.97751 .03045 L
.97764 .03047 L
.97772 .03048 L
.97779 .03049 L
.97792 .0305 L
.97799 .03051 L
.97806 .03052 L
.97818 .03054 L
.97825 .03054 L
.97831 .03055 L
.97837 .03056 L
.97844 .03057 L
.97855 .03058 L
.97863 .03059 L
.9787 .0306 L
.97883 .03062 L
.97891 .03063 L
.97898 .03064 L
.97906 .03065 L
.97913 .03066 L
.97925 .03068 L
.97932 .03069 L
.97939 .0307 L
.97946 .03071 L
.97953 .03071 L
.97966 .03073 L
.97973 .03074 L
.97979 .03075 L
.97986 .03076 L
.97993 .03077 L
.98006 .03079 L
.98012 .0308 L
.98019 .0308 L
.98026 .03081 L
.98033 .03082 L
Mistroke
.98047 .03084 L
.98054 .03085 L
.98061 .03086 L
.98069 .03087 L
.98077 .03088 L
.98084 .0309 L
.98091 .0309 L
.98104 .03092 L
.98111 .03093 L
.98119 .03094 L
.98133 .03097 L
.98141 .03098 L
.98148 .03099 L
.98156 .031 L
.98164 .03101 L
.98172 .03102 L
.98179 .03103 L
.98186 .03104 L
.98193 .03105 L
.98205 .03107 L
.98211 .03108 L
.98217 .03109 L
.98229 .0311 L
.98241 .03112 L
.98247 .03113 L
.98254 .03114 L
.98261 .03115 L
.98268 .03116 L
.98281 .03118 L
.98293 .0312 L
.983 .03121 L
.98306 .03122 L
.98314 .03123 L
.98321 .03124 L
.98334 .03126 L
.98341 .03127 L
.98347 .03128 L
.9836 .0313 L
.98371 .03132 L
.98384 .03134 L
.98397 .03136 L
.98404 .03137 L
.9841 .03138 L
.98417 .03139 L
.98425 .0314 L
.98438 .03142 L
.98445 .03143 L
.98451 .03144 L
.98464 .03146 L
.98475 .03148 L
Mistroke
.98487 .0315 L
.985 .03152 L
.98514 .03154 L
.98521 .03155 L
.98528 .03156 L
.98541 .03158 L
.98553 .0316 L
.9856 .03161 L
.98567 .03162 L
.9858 .03164 L
.98588 .03166 L
.98594 .03167 L
.98602 .03168 L
.9861 .03169 L
.98622 .03171 L
.98635 .03173 L
.98647 .03175 L
.98659 .03177 L
.98671 .03179 L
.98683 .03181 L
.98693 .03183 L
.98704 .03185 L
.98716 .03187 L
.98729 .03189 L
.98743 .03191 L
.98756 .03193 L
.98768 .03195 L
.98779 .03197 L
.98792 .03199 L
.98804 .03201 L
.98816 .03204 L
.98829 .03206 L
.98841 .03208 L
.98852 .0321 L
.98864 .03212 L
.98878 .03214 L
.98892 .03217 L
.98905 .03219 L
.98918 .03221 L
.98931 .03224 L
.98943 .03226 L
.98956 .03228 L
.9897 .0323 L
.98984 .03233 L
.98996 .03235 L
.9901 .03238 L
.99023 .0324 L
.99035 .03242 L
.99045 .03244 L
.99057 .03246 L
Mistroke
.9907 .03248 L
.99083 .03251 L
.99094 .03253 L
.99107 .03255 L
.99119 .03257 L
.99129 .03259 L
.99141 .03261 L
.99153 .03264 L
.99164 .03266 L
.99174 .03267 L
.99185 .03269 L
.99196 .03272 L
.99209 .03274 L
.99222 .03277 L
.99236 .03279 L
.99249 .03282 L
.99262 .03284 L
.99273 .03286 L
.99286 .03289 L
.99298 .03291 L
.99311 .03293 L
.99325 .03296 L
.99338 .03299 L
.9935 .03301 L
.99364 .03304 L
.9938 .03307 L
.99394 .03309 L
.99407 .03312 L
.99419 .03314 L
.99431 .03317 L
.99442 .03319 L
.99453 .03321 L
.99465 .03323 L
.99478 .03326 L
.99491 .03329 L
.99502 .03331 L
.99514 .03333 L
.99525 .03335 L
.99537 .03338 L
.99551 .03341 L
.99564 .03343 L
.99577 .03346 L
.99589 .03348 L
.99602 .03351 L
.99614 .03354 L
.99626 .03356 L
.99639 .03359 L
.99652 .03361 L
.99664 .03364 L
.99675 .03366 L
Mistroke
.99686 .03368 L
.99699 .03371 L
.99711 .03373 L
.99723 .03376 L
.99734 .03379 L
.99745 .03381 L
.99765 .03385 L
.99777 .03387 L
.99787 .0339 L
.99799 .03392 L
.99812 .03395 L
.99823 .03397 L
.99835 .034 L
.99848 .03403 L
.99862 .03406 L
.99875 .03409 L
.99887 .03411 L
.99899 .03414 L
.9991 .03416 L
.99923 .03419 L
.99935 .03422 L
.99957 .03426 L
.99969 .03429 L
.99981 .03432 L
1 .03436 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  Background->GrayLevel[1],
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005X0000gR000`400?l00000o`00003oo`3ooomY
0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY
0?ooo`00E03oool5000000<0oooo100000050?ooo`800000903oool5000000<0oooo100000040?oo
o`@000001@3oool2000001`0oooo1@0000030?ooo`@00000103oool4000000@0oooo100000050?oo
o`8000005@3oool5000000<0oooo100000040?ooo`@00000103oool4000000@0oooo100000050?oo
o`800000303oool5000000<0oooo100000040?ooo`@00000103oool4000000@0oooo100000040?oo
o`@000001@3oool2000000@0oooo001F0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`020?ooo`8000009P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`020?oo
o`8000007P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?oo
o`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0080oooo0P00
000G0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?oo
o`0000040?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`00
0000oooo0?ooo`020?ooo`8000003P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`040000003o
ool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`020?ooo`80
0000103oool005H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000
003oool0oooo000000@0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`000004
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool0
10000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0
203oool005H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003o
ool0oooo000000@0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?oo
o`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?oo
o`0000040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool01000
0000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0203o
ool005H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003oool0
oooo000000@0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`03
0000003oool0oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`04
0000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`00
00040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000
oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0203oool0
05H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo
000000@0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000
003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`000004
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo
0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0203oool005@0
oooo0`0000040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0:03oool3000000@0
oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo
0200oooo0`0000040?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool0
10000000oooo0?ooo`0000040?ooo`030000003oool0oooo01T0oooo0`0000040?ooo`030000003o
ool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`04
0000003oool0oooo000000@0oooo00<000000?ooo`3oool0403oool3000000@0oooo00<000000?oo
o`3oool00P3oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00@0
00000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00P0
oooo001F0?ooo`030000003oool0oooo00<0oooo1000000]0?ooo`030000003oool0oooo00<0oooo
100000040?ooo`@000009@3oool00`000000oooo0?ooo`030?ooo`@00000103oool4000000@0oooo
1000000N0?ooo`030000003oool0oooo00<0oooo100000040?ooo`@00000103oool4000000@0oooo
1000000E0?ooo`030000003oool0oooo00<0oooo100000040?ooo`@00000103oool4000000@0oooo
100000040?ooo`@000002`3oool00?l0ooooJ@3oool00?l0ooooJ@3oool00?l0ooooJ@3oool00?l0
ooooJ@3oool00?l0ooooJ@3oool00?l0ooooJ@3oool00280ooooo`00000Y000001h0oooo000R0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool01@000000oooo0?ooo`3oool0
00000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo0080oooo00D000000?ooo`3oool0oooo000000020?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`050000003oool0oooo0?oo
o`0000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool00P3oool01@000000oooo0?ooo`3oool000000080oooo00@000000?ooo`3oool00000
0P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00D000000?ooo`3oool0
oooo000000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0703oool000P0
oooo1@0000050?ooo`030000003oool0oooo00<0oooo100000060?ooo`030000003oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool000X0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo
000:0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703o
ool000D000001@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool00`3oool3000000l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`<000007P3oool000X0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`020?ooo`@000001P3oool00`000000oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000:0?ooo`030000003oool0oooo00L0
oooo00D000000?ooo`3oool0oooo000000090?ooo`030000003oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<000000?ooo`3oool0703oool000P0oooo0`0000040?ooo`030000003oool0oooo0080
oooo00D000000?ooo`3oool0oooo000000090?ooo`030000003oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool00P3ooolF0000o`050?ooo`0007l0oooo0?ooo`00
00007P3oool000X0oooo00<000000?ooo`3oool00P3oool6000000<0oooo1@0000050?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo1`000?l00`3oool0001o0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00H0001o0?ooo`3oool0oooo0000O`3oool80000oah0oooo
000R0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0103oool40000o`L0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
1@0007l0oooo0?ooo`0007l0oooo0080003o7P3oool00280oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00@0001o0?ooo`3oool0oooo10000?l20?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0003o0080003o1@3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`00
0000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool30000o`L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00d0oooo0`000?l90?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`800000403oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00`0oooo0P000?l0
0`3oool0001o0?ooo`080?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01@0007l0oooo0?oo
o`0007l0oooo008000007P3oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00X0oooo0`000?l20?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00T0
oooo0P000?l40?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00P0oooo0P000?l50?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280
oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00L0oooo0P000?l60?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3o
ool0703oool00280oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool01P3oool20000o`L0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000001h0oooo
000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`060?ooo`030000o`3oool0oooo00H0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool01@3oool20000o`P0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
103oool20000o`T0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool0103oool00`000?l0oooo0?ooo`080?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3o
ool0703oool00280oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool00`3oool20000o`X0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000001h0oooo
000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`020?ooo`80003o2`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003o
ool0oooo01`0oooo00080?ooo`D000000`3oool4000000@0oooo100000060?ooo`030000003oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`020?ooo`030000o`3oool0oooo00X0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L
0?ooo`002P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?oo
o`3oool00000103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0100007l0oooo0?ooo`3oool20000
o``0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`002P3oool00`000000oooo0?ooo`07
0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00@0001o0?oo
o`3oool0oooo0P000?l<0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool000D000001@3o
ool00`000000oooo0?ooo`070?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0
0`3oool300000>d0001o00<0003o0000O`0007l0<P0007l3000001h0oooo000:0?ooo`030000003o
ool0oooo00<0oooo100000030?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool00P000?l=0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703o
ool000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00@0001o0?ooo`3oool0003o3P3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000003oool0oooo01`0oooo00080?ooo`<000001@3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00@0001o0?ooo`00
0?l0003o3P3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000:0?ooo`030000003oool0
oooo00<0oooo1@0000030?ooo`@000001P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`000?l03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000003oool0oooo01`0oooo000R0?ooo`800000403oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0003o00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`020000
01h0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0003o0000o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3o
ool0703oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`000?l0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo
0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0000o`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003o
ool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0003o0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<00000
0?ooo`3oool0703oool00280oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`02
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo0P000?l@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000N0?ooo`008P3oool00`000000
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
003o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`000?l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<0
00000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000o`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool0
0`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0oooo0000O`3o
ool00P000?lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo0P00000@0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`070000O`3oool0oooo0000O`3oool0003o0000O`0A0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0oooo
0000O`3oool00P00000N0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`070000O`3oool0oooo0000O`3oool0003o0000O`0A0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<00000
0?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool01`0007l0oooo0?ooo`0007l0oooo0000o`0007l04@3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0
oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P000?l00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?oo
o`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
1`0007l0oooo0?ooo`0007l0003o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo
01`0oooo000R0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01`0007l0oooo0?ooo`00
07l0003o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool01@0007l0oooo0?ooo`0007l0oooo008000007P3oool00280oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool01`0007l0oooo0?ooo`0007l0003o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo00080?ooo`D000000P3oool5000000@0
oooo100000060?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00L0001o
0?ooo`3oool0001o0000o`3oool0001o0140oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?oo
o`002P3oool00`000000oooo0?ooo`020?ooo`050000003oool0oooo0?ooo`0000000`3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool01`0007l0oooo0?ooo`0007l0003o0?ooo`0007l04@3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000003oool0oooo01`0oooo000:0?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0?ooo`3oool00P000?l00`3oool0001o0?ooo`0@0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<000000?ooo`3oool0703oool000D000001@3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool00`3oool300000>H0001o00<0003o0000O`0007l0
>@0007l3000001h0oooo000:0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00`3o
ool4000000H0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`000?l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?oo
o`0L0?ooo`002P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000o`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0
oooo01`0oooo00080?ooo`<00000103oool01@000000oooo0?ooo`3oool0000000@0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000o`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0
oooo01`0oooo000:0?ooo`030000003oool0oooo00<0oooo0`0000050?ooo`D000001@3oool00`00
0000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000o`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`80
0000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0000o`000?l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00D0001o0?ooo`3oool0001o0?ooo`02000001h0oooo000R0?ooo`030000003oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`000?l0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`02
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`02
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0000o`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0003o0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`000?l0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo0P00000@0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0003o0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01@0007l0oooo
0?ooo`0007l0oooo008000007P3oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0000o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0003o0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0000o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0003o0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`000?l0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`800000403oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0003o0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0001o
0?ooo`02000001h0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0000o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
00000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0003o0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`000?l0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0000o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<000000?ooo`3oool0703oool00280oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000o`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo00800000
7P3oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool010000?l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?oo
o`0L0?ooo`00203oool5000000<0oooo100000040?ooo`@000001P3oool00`000000oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000o`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000:0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool010000?l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L
0?ooo`002P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?oo
o`3oool00000103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0003o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<000000?ooo`3oool0703oool000D000001@3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`03
0?ooo`<00000h`0007l00`000?l0001o0000O`0l0000O`<000007P3oool000X0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000000@0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
o`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0
oooo01`0oooo000:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool01000
0000oooo0?ooo`0000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool010000?l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`00203oool3000000@0oooo00<000000?oo
o`3oool00P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool010000?l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`002P3oool00`00
0000oooo0?ooo`030?ooo`@00000103oool4000000H0oooo00<000000?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`080000O`3oool0
oooo0?ooo`0007l0oooo0000o`0007l20?ooo`040000O`3oool0oooo0000Oa40oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`000000oooo0?ooo`0L0?ooo`008P3oool200000100oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0200007l0oooo0?ooo`3oool0001o0?ooo`00
0?l0001o0P3oool0100007l0oooo0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000N0?ooo`008P3o
ool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00P0001o0?ooo`3oool0oooo0000O`3oool0003o0000O`80oooo00@0001o
0?ooo`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool0200007l0oooo0?ooo`3oool0001o0?ooo`000?l0001o0P3oool0100007l0oooo0?oo
o`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`080000O`3oool0oooo0?ooo`0007l0oooo0000o`0007l20?ooo`040000O`3oool0oooo0000Oa40
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00P0001o
0?ooo`3oool0oooo0000O`3oool0003o0000O`80oooo00@0001o0?ooo`3oool0001o4@3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`800000403oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`080000O`3oool0oooo0?ooo`0007l0
oooo0000o`0007l20?ooo`040000O`3oool0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000001h0oooo
000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool0200007l0oooo0?ooo`3oool0001o0?ooo`000?l0001o0P3oool0
100007l0oooo0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo
00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`080000O`3oool0oooo0?ooo`0007l0003o0?ooo`0007l20?ooo`040000O`3o
ool0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00P0001o0?ooo`3oool0oooo0000O`000?l0oooo0000O`80oooo00@0001o0?ooo`3oool0
001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
200007l0oooo0?ooo`3oool0001o0000o`3oool0001o0P3oool0100007l0oooo0?ooo`0007lA0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo0P00000@0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00P0001o0?ooo`3oool0oooo
0000O`000?l0oooo0000O`80oooo00@0001o0?ooo`3oool0001o4@3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo00800000
7P3oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`080000O`3oool0oooo0?ooo`0007l0003o0?ooo`0007l2
0?ooo`040000O`3oool0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`00
8P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00P0001o0?ooo`3oool0oooo0000O`000?l0oooo0000O`80oooo00@0
001o0?ooo`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`03
0000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool0200007l0oooo0?ooo`3oool0001o0000o`3oool0001o0P3oool0100007l0oooo
0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`080000O`3oool0oooo0?ooo`0007l0003o0?ooo`0007l20?ooo`040000O`3oool0oooo0000
Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool200000100oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0200007l0oooo0?oo
o`3oool0001o0000o`3oool0001o0P3oool0100007l0oooo0?ooo`0007lA0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool0
0P00000N0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00P0001o0?ooo`3oool0oooo0000O`000?l0oooo
0000O`80oooo00@0001o0?ooo`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0
oooo000B0?ooo`030000003oool0oooo00<0oooo100000060?ooo`030000003oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0003o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`02
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00180oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`02
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`000?l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`004P3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`000?l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`000000oooo0?ooo`0L0?ooo`001`3oool6000000H0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`030?ooo`<00000h00007l00`000?l0001o0000O`0o0000O`<000007P3oool001<0
oooo00<000000?ooo`3oool00P3oool4000000H0oooo00<000000?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`000?l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`000000oooo0?ooo`0L0?ooo`00503oool01@000000oooo0?ooo`3oool0000000T0
oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`000?l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`00
3`3oool00`000000oooo0?ooo`020?ooo`050000003oool0oooo0?ooo`0000002@3oool00`000000
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000o`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000?0?ooo`H00000
0`3oool5000000D0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`000?l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo
0?ooo`0L0?ooo`008P3oool200000100oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020000o`80oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000N0?ooo`008P3oool00`00
0000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0003o0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`03
0000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`000?l0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo
00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0000o`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3o
ool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0003o0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R
0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0000o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000001h0oooo000R0?ooo`030000003oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`000?l0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0000o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`000?l0003o0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0000o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo0P00000@
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`000?l0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01@00
07l0oooo0?ooo`0007l0oooo008000007P3oool00280oooo00<000000?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
003o0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`000?l0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0000o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`02
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`02
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0003o0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool200000100oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0003o0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0oooo
0000O`3oool00P00000N0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0000o`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000003oool0oooo01`0oooo000@0?ooo`@00000103oool4000000H0oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`000?l0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`003`3oool00`000000oooo
0?ooo`020?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0000o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000D0?ooo`040000
003oool0oooo000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0000
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo00070?ooo`H000001`3oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo00<0oooo0`00003M0000O`030000o`0007l0001o0480
001o0`00000N0?ooo`00403oool4000000<0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@000?l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0
703oool00100oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000o`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?oo
o`0L0?ooo`00403oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0003o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0
oooo01`0oooo000@0?ooo`D000000`3oool4000000H0oooo00<000000?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000o`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool200000100oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@000?l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0oooo0000O`3o
ool00P00000N0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`040000O`3oool0oooo0?ooo`80003o0`3oool0100007l0oooo0?ooo`0007l2
0?ooo`040000O`3oool0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`00
8P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`06
0000O`3oool0oooo0?ooo`000?l0001o0`3oool0100007l0oooo0?ooo`0007l20?ooo`040000O`3o
ool0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`060000O`3oool0oooo
0?ooo`000?l0001o0`3oool0100007l0oooo0?ooo`0007l20?ooo`040000O`3oool0oooo0000Oa40
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`060000O`3oool0oooo0?ooo`000?l0001o
0`3oool0100007l0oooo0?ooo`0007l20?ooo`040000O`3oool0oooo0000Oa40oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080003o00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000003oool0oooo01`0oooo000R0?ooo`800000403oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool01P0007l0oooo0?ooo`000?l0oooo0000O`<0oooo00@0001o0?ooo`3oool0
001o0P3oool0100007l0oooo0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000N0?ooo`008P3oool0
0`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`060000O`3o
ool0oooo0000o`3oool0001o0`3oool0100007l0oooo0?ooo`0007l20?ooo`040000O`3oool0oooo
0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`060000O`3oool0oooo0000o`3o
ool0001o0`3oool0100007l0oooo0?ooo`0007l20?ooo`040000O`3oool0oooo0000Oa40oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`060000O`3oool0003o0000o`3oool0001o0`3oool0
100007l0oooo0?ooo`0007l20?ooo`040000O`3oool0oooo0000Oa40oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
0000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0003o0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000003oool0oooo01`0oooo000R0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0000o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000001h0oooo
000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0000o`000?l00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703o
ool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0003o0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L
0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`02
0?ooo`030000O`000?l0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo
01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo0P000?l30?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`00
8P3oool200000100oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0003o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000N0?ooo`008P3oool00`000000oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool0100007l0oooo0?ooo`3oool20000o`@0oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000003oool0oooo01`0oooo000?0?ooo`D00000103oool4000000H0oooo00<0
00000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00H0001o0?ooo`3oool0oooo0000o`0007l40?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`003`3oool01@000000oooo
0?ooo`3oool0000000<0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`060000O`3oool0oooo0?ooo`000?l0001o103o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00100oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080003o00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<000000?ooo`3oool0703oool000L0oooo1P0000040?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool00`3oool300000=@0001o0P000?m<0000O`<000007P3oool00180oooo
00<000000?ooo`3oool00`3oool4000000H0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00H0001o0?ooo`000?l0003o0?ooo`0007l40?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
0000oooo0?ooo`0L0?ooo`004`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0000o`000?l00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`00
3`3oool01@000000oooo0?ooo`3oool0000000@0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0003o0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000@0?ooo`<00000
1@3oool5000000D0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo0P000?l30?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo
000R0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00@0oooo0P000?l40?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
1@0007l0oooo0?ooo`0007l0oooo008000007P3oool00280oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00<0oooo0P000?l00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`020?ooo`80003o00<0oooo0000O`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00@0001o0?ooo`3oool0oooo0P000?l20?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280
oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo0080003o0`3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0
oooo000R0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`040000O`3oool0003o0000o`@0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000001h0oooo000R0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0003o0000o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<0
00000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`060?ooo`<0003o1P3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01@3o
ool20000o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`00
0000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00<0oooo0`000?l00`3oool0001o0?ooo`060?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280
oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`040000O`3oool0oooo
0?ooo`<0003o0`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000N0?ooo`008P3oool00`000000
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`000?l0003o
0080003o1@3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo
00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03@3oool40000o`P0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00X0oooo1@000?l00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool0
0`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`060?ooo`D0003o103o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L
0?ooo`008P3oool200000100oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`040000O`3oool0oooo0?ooo`H0003o203o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000001h0oooo000R0?oo
o`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l80000o``0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool001P0oooo100000060?ooo`03
0000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0003o00X0003o
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`000000oooo0?ooo`0L0?ooo`005`3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool0103oool?0000o`040000O`3oool0oooo0000O`80oooo00@0001o0?ooo`3oool000004@3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L
0?ooo`005`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`020?ooo`050000003oool0oooo0?ooo`0000000P3oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3o
ool01@000000oooo0?ooo`3oool000000080oooo00@000000?ooo`3oool000000P3oool01P000000
oooo0?ooo`3oool0003o0?ooob00003o0`3oool00`000000oooo0?ooo`020?ooo`050000003oool0
oooo0?ooo`0000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool00P3oool01@000000oooo0?ooo`3oool000000080oooo00@000000?ooo`3o
ool000000P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00D000000?oo
o`3oool0oooo000000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0703o
ool001L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?oooh<0003oY@00000N
0?ooo`005`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?l0ooooBP3oool001L0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3o0?ooodX0oooo000G0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool0o`3ooom:0?ooo`00603oool400000?l0ooooC@3o
ool00?l0ooooJ@3oool00?l0ooooJ@3oool00?l0ooooJ@3oool00?l0ooooJ@3oool00200oooo1@00
000_0?ooo`D000000`3oool4000000D0oooo0P00000T0?ooo`D000000`3oool4000000@0oooo1000
00050?ooo`800000703oool5000000<0oooo100000040?ooo`@00000103oool4000000D0oooo0P00
000E0?ooo`D000000`3oool4000000@0oooo100000040?ooo`@00000103oool4000000D0oooo0P00
00130?ooo`008P3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`8000009P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?oo
o`020?ooo`8000007P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@0
00000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0080
oooo0P00000G0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000
oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103o
ool00`000000oooo0?ooo`020?ooo`800000@`3oool00280oooo00<000000?ooo`3oool0<@3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0:P3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103o
ool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3o
ool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3o
ool00000103oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool0A`3oool00280oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?oo
o`0R0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?oo
o`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool01000
0000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0A`3o
ool00280oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0R0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`040000
003oool0oooo000000@0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`000004
0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0A`3oool00280oooo00<00000
0?ooo`3oool0<@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0:P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0
oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080
oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`040000003oool0
oooo000000@0oooo00<000000?ooo`3oool0A`3oool00200oooo0`00000a0?ooo`<00000103oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo02P0oooo0`0000040?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0P0?ooo`<00000103o
ool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool0
0000103oool00`000000oooo0?ooo`0I0?ooo`<00000103oool00`000000oooo0?ooo`020?ooo`04
0000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`00
00040?ooo`030000003oool0oooo04L0oooo000R0?ooo`030000003oool0oooo0340oooo00<00000
0?ooo`3oool00`3oool4000002d0oooo00<000000?ooo`3oool00`3oool4000000@0oooo1000000U
0?ooo`030000003oool0oooo00<0oooo100000040?ooo`@00000103oool4000001h0oooo00<00000
0?ooo`3oool00`3oool4000000@0oooo100000040?ooo`@00000103oool4000004X0oooo003o0?oo
ofT0oooo003o0?ooofT0oooo003o0?ooofT0oooo0000\
\>"],
  ImageRangeCache->{{{0, 359}, {221.375, 0}} -> {-0.579829, -199.123, \
0.021156, 1.23232}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Control-To-Output Transfer Function", "Subsubsection",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(BodeMagnitudePlot[
        gco[2\ Pi\ I\ s]\  /. \ Param2, {s, \ 1, \ 100000}, \ 
        Frame\  -> \ True, PlotStyle\  -> \ RGBColor[0, \ 0, \ 1], 
        PlotRange\  -> \ {{0, \ 5}, \ {\(-5\), \ 30}}, 
        GridLines\  -> \ {LogGridMinor, \ Automatic}];\)\)], "Input",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-017 0.2 0.0882906 0.0176581 [
[.2 -0.0125 -11 -12.5625 ]
[.2 -0.0125 11 0 ]
[.4 -0.0125 -14 -12.5625 ]
[.4 -0.0125 14 0 ]
[.6 -0.0125 -17 -12.5625 ]
[.6 -0.0125 17 0 ]
[.8 -0.0125 -20 -12.5625 ]
[.8 -0.0125 20 0 ]
[1 -0.0125 -23 -12.5625 ]
[1 -0.0125 23 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .08829 -6 -4.5 ]
[-0.0125 .08829 0 4.5 ]
[-0.0125 .17658 -6 -4.5 ]
[-0.0125 .17658 0 4.5 ]
[-0.0125 .26487 -12 -4.5 ]
[-0.0125 .26487 0 4.5 ]
[-0.0125 .35316 -12 -4.5 ]
[-0.0125 .35316 0 4.5 ]
[-0.0125 .44145 -12 -4.5 ]
[-0.0125 .44145 0 4.5 ]
[-0.0125 .52974 -12 -4.5 ]
[-0.0125 .52974 0 4.5 ]
[-0.0125 .61803 -12 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 -0.125 0 ]
[0 .63053 -5 0 ]
[0 .63053 5 12.5625 ]
[.2 .63053 -11 0 ]
[.2 .63053 11 12.5625 ]
[.4 .63053 -14 0 ]
[.4 .63053 14 12.5625 ]
[.6 .63053 -17 0 ]
[.6 .63053 17 12.5625 ]
[.8 .63053 -20 0 ]
[.8 .63053 20 12.5625 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 -0.0005 0 0 ]
[ 1 .61853 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 0 m
0 .61803 L
s
.06021 0 m
.06021 .61803 L
s
.09542 0 m
.09542 .61803 L
s
.12041 0 m
.12041 .61803 L
s
.13979 0 m
.13979 .61803 L
s
.15563 0 m
.15563 .61803 L
s
.16902 0 m
.16902 .61803 L
s
.18062 0 m
.18062 .61803 L
s
.19085 0 m
.19085 .61803 L
s
.2 0 m
.2 .61803 L
s
.26021 0 m
.26021 .61803 L
s
.29542 0 m
.29542 .61803 L
s
.32041 0 m
.32041 .61803 L
s
.33979 0 m
.33979 .61803 L
s
.35563 0 m
.35563 .61803 L
s
.36902 0 m
.36902 .61803 L
s
.38062 0 m
.38062 .61803 L
s
.39085 0 m
.39085 .61803 L
s
.4 0 m
.4 .61803 L
s
.46021 0 m
.46021 .61803 L
s
.49542 0 m
.49542 .61803 L
s
.52041 0 m
.52041 .61803 L
s
.53979 0 m
.53979 .61803 L
s
.55563 0 m
.55563 .61803 L
s
.56902 0 m
.56902 .61803 L
s
.58062 0 m
.58062 .61803 L
s
.59085 0 m
.59085 .61803 L
s
.6 0 m
.6 .61803 L
s
.66021 0 m
.66021 .61803 L
s
.69542 0 m
.69542 .61803 L
s
.72041 0 m
.72041 .61803 L
s
.73979 0 m
.73979 .61803 L
s
.75563 0 m
.75563 .61803 L
s
.76902 0 m
.76902 .61803 L
s
.78062 0 m
.78062 .61803 L
s
.79085 0 m
.79085 .61803 L
s
.8 0 m
.8 .61803 L
s
.86021 0 m
.86021 .61803 L
s
.89542 0 m
.89542 .61803 L
s
.92041 0 m
.92041 .61803 L
s
.93979 0 m
.93979 .61803 L
s
.95563 0 m
.95563 .61803 L
s
.96902 0 m
.96902 .61803 L
s
.98062 0 m
.98062 .61803 L
s
.99085 0 m
.99085 .61803 L
s
1 0 m
1 .61803 L
s
0 .08829 m
1 .08829 L
s
0 .17658 m
1 .17658 L
s
0 .26487 m
1 .26487 L
s
0 .35316 m
1 .35316 L
s
0 .44145 m
1 .44145 L
s
0 .52974 m
1 .52974 L
s
0 g
.2 0 m
.2 .00625 L
s
gsave
.2 -0.0125 -72 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10.) show
81.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.4 0 m
.4 .00625 L
s
gsave
.4 -0.0125 -75 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100.) show
87.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.6 0 m
.6 .00625 L
s
gsave
.6 -0.0125 -78 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1000.) show
93.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.8 0 m
.8 .00625 L
s
gsave
.8 -0.0125 -81 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10000.) show
99.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
1 0 m
1 .00625 L
s
gsave
1 -0.0125 -84 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100000.) show
105.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.001 w
.06021 0 m
.06021 .00375 L
s
.09542 0 m
.09542 .00375 L
s
.12041 0 m
.12041 .00375 L
s
.13979 0 m
.13979 .00375 L
s
.15563 0 m
.15563 .00375 L
s
.16902 0 m
.16902 .00375 L
s
.18062 0 m
.18062 .00375 L
s
.19085 0 m
.19085 .00375 L
s
.26021 0 m
.26021 .00375 L
s
.29542 0 m
.29542 .00375 L
s
.32041 0 m
.32041 .00375 L
s
.33979 0 m
.33979 .00375 L
s
.35563 0 m
.35563 .00375 L
s
.36902 0 m
.36902 .00375 L
s
.38062 0 m
.38062 .00375 L
s
.39085 0 m
.39085 .00375 L
s
.46021 0 m
.46021 .00375 L
s
.49542 0 m
.49542 .00375 L
s
.52041 0 m
.52041 .00375 L
s
.53979 0 m
.53979 .00375 L
s
.55563 0 m
.55563 .00375 L
s
.56902 0 m
.56902 .00375 L
s
.58062 0 m
.58062 .00375 L
s
.59085 0 m
.59085 .00375 L
s
.66021 0 m
.66021 .00375 L
s
.69542 0 m
.69542 .00375 L
s
.72041 0 m
.72041 .00375 L
s
.73979 0 m
.73979 .00375 L
s
.75563 0 m
.75563 .00375 L
s
.76902 0 m
.76902 .00375 L
s
.78062 0 m
.78062 .00375 L
s
.79085 0 m
.79085 .00375 L
s
.86021 0 m
.86021 .00375 L
s
.89542 0 m
.89542 .00375 L
s
.92041 0 m
.92041 .00375 L
s
.93979 0 m
.93979 .00375 L
s
.95563 0 m
.95563 .00375 L
s
.96902 0 m
.96902 .00375 L
s
.98062 0 m
.98062 .00375 L
s
.99085 0 m
.99085 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .08829 m
.00625 .08829 L
s
[(0)] -0.0125 .08829 1 0 Mshowa
0 .17658 m
.00625 .17658 L
s
[(5)] -0.0125 .17658 1 0 Mshowa
0 .26487 m
.00625 .26487 L
s
[(10)] -0.0125 .26487 1 0 Mshowa
0 .35316 m
.00625 .35316 L
s
[(15)] -0.0125 .35316 1 0 Mshowa
0 .44145 m
.00625 .44145 L
s
[(20)] -0.0125 .44145 1 0 Mshowa
0 .52974 m
.00625 .52974 L
s
[(25)] -0.0125 .52974 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(30)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .01766 m
.00375 .01766 L
s
0 .03532 m
.00375 .03532 L
s
0 .05297 m
.00375 .05297 L
s
0 .07063 m
.00375 .07063 L
s
0 .10595 m
.00375 .10595 L
s
0 .12361 m
.00375 .12361 L
s
0 .14126 m
.00375 .14126 L
s
0 .15892 m
.00375 .15892 L
s
0 .19424 m
.00375 .19424 L
s
0 .2119 m
.00375 .2119 L
s
0 .22956 m
.00375 .22956 L
s
0 .24721 m
.00375 .24721 L
s
0 .28253 m
.00375 .28253 L
s
0 .30019 m
.00375 .30019 L
s
0 .31785 m
.00375 .31785 L
s
0 .3355 m
.00375 .3355 L
s
0 .37082 m
.00375 .37082 L
s
0 .38848 m
.00375 .38848 L
s
0 .40614 m
.00375 .40614 L
s
0 .42379 m
.00375 .42379 L
s
0 .45911 m
.00375 .45911 L
s
0 .47677 m
.00375 .47677 L
s
0 .49443 m
.00375 .49443 L
s
0 .51209 m
.00375 .51209 L
s
0 .5474 m
.00375 .5474 L
s
0 .56506 m
.00375 .56506 L
s
0 .58272 m
.00375 .58272 L
s
0 .60038 m
.00375 .60038 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 .61178 m
0 .61803 L
s
gsave
0 .63053 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
69.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.2 .61178 m
.2 .61803 L
s
gsave
.2 .63053 -72 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10.) show
81.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.4 .61178 m
.4 .61803 L
s
gsave
.4 .63053 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100.) show
87.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.6 .61178 m
.6 .61803 L
s
gsave
.6 .63053 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1000.) show
93.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.8 .61178 m
.8 .61803 L
s
gsave
.8 .63053 -81 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10000.) show
99.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.001 w
.06021 .61428 m
.06021 .61803 L
s
.09542 .61428 m
.09542 .61803 L
s
.12041 .61428 m
.12041 .61803 L
s
.13979 .61428 m
.13979 .61803 L
s
.15563 .61428 m
.15563 .61803 L
s
.16902 .61428 m
.16902 .61803 L
s
.18062 .61428 m
.18062 .61803 L
s
.19085 .61428 m
.19085 .61803 L
s
.26021 .61428 m
.26021 .61803 L
s
.29542 .61428 m
.29542 .61803 L
s
.32041 .61428 m
.32041 .61803 L
s
.33979 .61428 m
.33979 .61803 L
s
.35563 .61428 m
.35563 .61803 L
s
.36902 .61428 m
.36902 .61803 L
s
.38062 .61428 m
.38062 .61803 L
s
.39085 .61428 m
.39085 .61803 L
s
.46021 .61428 m
.46021 .61803 L
s
.49542 .61428 m
.49542 .61803 L
s
.52041 .61428 m
.52041 .61803 L
s
.53979 .61428 m
.53979 .61803 L
s
.55563 .61428 m
.55563 .61803 L
s
.56902 .61428 m
.56902 .61803 L
s
.58062 .61428 m
.58062 .61803 L
s
.59085 .61428 m
.59085 .61803 L
s
.66021 .61428 m
.66021 .61803 L
s
.69542 .61428 m
.69542 .61803 L
s
.72041 .61428 m
.72041 .61803 L
s
.73979 .61428 m
.73979 .61803 L
s
.75563 .61428 m
.75563 .61803 L
s
.76902 .61428 m
.76902 .61803 L
s
.78062 .61428 m
.78062 .61803 L
s
.79085 .61428 m
.79085 .61803 L
s
.86021 .61428 m
.86021 .61803 L
s
.89542 .61428 m
.89542 .61803 L
s
.92041 .61428 m
.92041 .61803 L
s
.93979 .61428 m
.93979 .61803 L
s
.95563 .61428 m
.95563 .61803 L
s
.96902 .61428 m
.96902 .61803 L
s
.98062 .61428 m
.98062 .61803 L
s
.99085 .61428 m
.99085 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .08829 m
1 .08829 L
s
.99375 .17658 m
1 .17658 L
s
.99375 .26487 m
1 .26487 L
s
.99375 .35316 m
1 .35316 L
s
.99375 .44145 m
1 .44145 L
s
.99375 .52974 m
1 .52974 L
s
.125 Mabswid
.99625 .01766 m
1 .01766 L
s
.99625 .03532 m
1 .03532 L
s
.99625 .05297 m
1 .05297 L
s
.99625 .07063 m
1 .07063 L
s
.99625 .10595 m
1 .10595 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .14126 m
1 .14126 L
s
.99625 .15892 m
1 .15892 L
s
.99625 .19424 m
1 .19424 L
s
.99625 .2119 m
1 .2119 L
s
.99625 .22956 m
1 .22956 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .28253 m
1 .28253 L
s
.99625 .30019 m
1 .30019 L
s
.99625 .31785 m
1 .31785 L
s
.99625 .3355 m
1 .3355 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .38848 m
1 .38848 L
s
.99625 .40614 m
1 .40614 L
s
.99625 .42379 m
1 .42379 L
s
.99625 .45911 m
1 .45911 L
s
.99625 .47677 m
1 .47677 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .51209 m
1 .51209 L
s
.99625 .5474 m
1 .5474 L
s
.99625 .56506 m
1 .56506 L
s
.99625 .58272 m
1 .58272 L
s
.99625 .60038 m
1 .60038 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
.5 Mabswid
.00036 .4439 m
.01921 .4439 L
.03398 .4439 L
.0454 .4439 L
.05635 .4439 L
.06683 .4439 L
.07677 .4439 L
.08521 .4439 L
.09221 .4439 L
.09879 .4439 L
.10444 .4439 L
.11025 .4439 L
.11598 .4439 L
.12485 .4439 L
.12951 .4439 L
.13365 .4439 L
.13819 .4439 L
.14287 .4439 L
.14707 .4439 L
.15079 .4439 L
.154 .4439 L
.15741 .4439 L
.16083 .4439 L
.16392 .4439 L
.16721 .4439 L
.17054 .4439 L
.17329 .4439 L
.1762 .4439 L
.17924 .4439 L
.18237 .4439 L
.18523 .4439 L
.18774 .4439 L
.19013 .4439 L
.19268 .4439 L
.19526 .4439 L
.19761 .4439 L
.19976 .4439 L
.20167 .4439 L
.20382 .4439 L
.20582 .4439 L
.20792 .4439 L
.21011 .4439 L
.21212 .4439 L
.2139 .4439 L
.21596 .4439 L
.21813 .4439 L
.22013 .4439 L
.22192 .4439 L
.22346 .4439 L
.22507 .4439 L
Mistroke
.22657 .4439 L
.22791 .4439 L
.22945 .4439 L
.2311 .4439 L
.23263 .4439 L
.23402 .4439 L
.23543 .4439 L
.23673 .4439 L
.23813 .4439 L
.23962 .4439 L
.24107 .4439 L
.24257 .4439 L
.24383 .4439 L
.24519 .4439 L
.24646 .4439 L
.24761 .4439 L
.24885 .4439 L
.25015 .4439 L
.25126 .4439 L
.25229 .4439 L
.2534 .4439 L
.25458 .4439 L
.25561 .4439 L
.25672 .4439 L
.25775 .4439 L
.2587 .4439 L
.26046 .4439 L
.26134 .4439 L
.2623 .4439 L
.26334 .4439 L
.26442 .4439 L
.26533 .4439 L
.26631 .4439 L
.26737 .4439 L
.26848 .4439 L
.26952 .4439 L
.27045 .4439 L
.27138 .4439 L
.27223 .4439 L
.27316 .4439 L
.27413 .4439 L
.27496 .4439 L
.27573 .4439 L
.27657 .4439 L
.27746 .4439 L
.27827 .4439 L
.27911 .4439 L
.2799 .4439 L
.28061 .4439 L
.28145 .4439 L
Mistroke
.28234 .4439 L
.28317 .4439 L
.28393 .4439 L
.28529 .4439 L
.28605 .4439 L
.28676 .4439 L
.2875 .4439 L
.28817 .4439 L
.28893 .4439 L
.28965 .4439 L
.2909 .4439 L
.29152 .4439 L
.2922 .4439 L
.29339 .4439 L
.29449 .4439 L
.29512 .4439 L
.29577 .4439 L
.29693 .4439 L
.29756 .4439 L
.29815 .4439 L
.29879 .4439 L
.29947 .4439 L
.3002 .4439 L
.30088 .4439 L
.30162 .4439 L
.30241 .4439 L
.30322 .4439 L
.30399 .4439 L
.30467 .4439 L
.30541 .4439 L
.30611 .4439 L
.30673 .4439 L
.30745 .4439 L
.30811 .4439 L
.30873 .4439 L
.30929 .4439 L
.3099 .4439 L
.31055 .4439 L
.31125 .4439 L
.31188 .4439 L
.3126 .4439 L
.31329 .4439 L
.31391 .4439 L
.31448 .4439 L
.31511 .4439 L
.31576 .4439 L
.31642 .4439 L
.31711 .4439 L
.31777 .4439 L
.31836 .4439 L
Mistroke
.31905 .4439 L
.3198 .4439 L
.3205 .4439 L
.32114 .4439 L
.32173 .4439 L
.32227 .4439 L
.32285 .4439 L
.32347 .4439 L
.324 .4439 L
.3245 .4439 L
.32504 .4439 L
.32561 .4439 L
.32612 .4439 L
.32667 .4439 L
.32718 .4439 L
.32766 .4439 L
.32808 .4439 L
.32855 .4439 L
.32903 .4439 L
.32948 .44391 L
.32998 .44391 L
.33051 .44391 L
.33107 .44391 L
.33159 .44391 L
.33207 .44391 L
.3325 .44391 L
.333 .44391 L
.33347 .44391 L
.33398 .44391 L
.33451 .44391 L
.33497 .44391 L
.33546 .44391 L
.33599 .44391 L
.33656 .44391 L
.33709 .44391 L
.33758 .44391 L
.33804 .44391 L
.33848 .44391 L
.33887 .44391 L
.33929 .44391 L
.33974 .44391 L
.34022 .44391 L
.34063 .44391 L
.34107 .44391 L
.34148 .44391 L
.34186 .44391 L
.34229 .44391 L
.3427 .44391 L
.34307 .44391 L
.34342 .44391 L
Mistroke
.34379 .44391 L
.34419 .44391 L
.34462 .44391 L
.34501 .44391 L
.34543 .44391 L
.34588 .44391 L
.34627 .44391 L
.34662 .44391 L
.34702 .44391 L
.34743 .44391 L
.34789 .44391 L
.34837 .44391 L
.34883 .44391 L
.34925 .44391 L
.34963 .44391 L
.35004 .44391 L
.35047 .44391 L
.35087 .44391 L
.35125 .44391 L
.35159 .44391 L
.35197 .44391 L
.35237 .44391 L
.35272 .44391 L
.35304 .44391 L
.3534 .44391 L
.35378 .44391 L
.35409 .44391 L
.35443 .44391 L
.35479 .44391 L
.35512 .44391 L
.35572 .44391 L
.35604 .44391 L
.35634 .44391 L
.3567 .44391 L
.35707 .44391 L
.35739 .44391 L
.35774 .44391 L
.35811 .44391 L
.35851 .44391 L
.35888 .44391 L
.35922 .44391 L
.35956 .44391 L
.35987 .44391 L
.36022 .44391 L
.36058 .44391 L
.36093 .44391 L
.36125 .44391 L
.36154 .44391 L
.36186 .44391 L
.36217 .44391 L
Mistroke
.36251 .44391 L
.36283 .44391 L
.36312 .44391 L
.36338 .44391 L
.36367 .44391 L
.36398 .44391 L
.36426 .44391 L
.36457 .44391 L
.36489 .44391 L
.36517 .44391 L
.36547 .44391 L
.3658 .44391 L
.36614 .44391 L
.36647 .44391 L
.36676 .44391 L
.36704 .44391 L
.36733 .44391 L
.36786 .44391 L
.36814 .44391 L
.36841 .44391 L
.36871 .44391 L
.36902 .44391 L
.36929 .44391 L
.36959 .44391 L
.36986 .44391 L
.37012 .44391 L
.3706 .44391 L
.37087 .44391 L
.37111 .44391 L
.37139 .44391 L
.37171 .44391 L
.37204 .44391 L
.37234 .44391 L
.37263 .44391 L
.37288 .44391 L
.37318 .44391 L
.37346 .44391 L
.37376 .44391 L
.37408 .44391 L
.37436 .44391 L
.37465 .44391 L
.37497 .44391 L
.37532 .44391 L
.37564 .44392 L
.37593 .44392 L
.3762 .44392 L
.37649 .44392 L
.37676 .44392 L
.37701 .44392 L
.3773 .44392 L
Mistroke
.37761 .44392 L
.3779 .44392 L
.37817 .44392 L
.37842 .44392 L
.37865 .44392 L
.37891 .44392 L
.37918 .44392 L
.37947 .44392 L
.37974 .44392 L
.37998 .44392 L
.38024 .44392 L
.38051 .44392 L
.38076 .44392 L
.38103 .44392 L
.38131 .44392 L
.38156 .44392 L
.38179 .44392 L
.38204 .44392 L
.38231 .44392 L
.3826 .44392 L
.38291 .44392 L
.38321 .44392 L
.38348 .44392 L
.38373 .44392 L
.384 .44392 L
.38428 .44392 L
.38454 .44392 L
.38481 .44392 L
.38511 .44392 L
.38539 .44392 L
.38565 .44392 L
.38591 .44392 L
.38614 .44392 L
.38641 .44392 L
.38666 .44392 L
.38693 .44392 L
.38722 .44392 L
.38747 .44392 L
.38773 .44392 L
.38802 .44392 L
.38833 .44392 L
.38862 .44392 L
.38888 .44392 L
.38911 .44392 L
.38935 .44392 L
.38961 .44392 L
.38985 .44392 L
.39008 .44392 L
.39028 .44392 L
.3905 .44392 L
Mistroke
.39074 .44392 L
.39098 .44392 L
.39123 .44392 L
.39147 .44392 L
.39169 .44392 L
.39194 .44392 L
.39221 .44392 L
.39247 .44392 L
.39271 .44392 L
.39291 .44392 L
.39313 .44392 L
.39334 .44392 L
.39353 .44392 L
.39375 .44392 L
.39398 .44392 L
.39421 .44392 L
.39442 .44392 L
.39465 .44392 L
.39487 .44392 L
.39507 .44392 L
.39528 .44392 L
.39551 .44392 L
.39575 .44392 L
.39596 .44392 L
.39618 .44392 L
.3964 .44392 L
.39662 .44392 L
.39681 .44392 L
.39702 .44392 L
.39725 .44392 L
.39749 .44393 L
.39771 .44393 L
.39792 .44393 L
.39813 .44393 L
.39832 .44393 L
.39853 .44393 L
.39875 .44393 L
.39896 .44393 L
.39916 .44393 L
.39934 .44393 L
.39953 .44393 L
.39989 .44393 L
.4001 .44393 L
.40029 .44393 L
.40049 .44393 L
.40067 .44393 L
.40088 .44393 L
.40108 .44393 L
.40143 .44393 L
.40175 .44393 L
Mistroke
.4021 .44393 L
.40228 .44393 L
.40247 .44393 L
.40267 .44393 L
.40288 .44393 L
.4031 .44393 L
.40331 .44393 L
.4035 .44393 L
.40368 .44393 L
.40388 .44393 L
.40407 .44393 L
.40428 .44393 L
.4045 .44393 L
.40468 .44393 L
.40489 .44393 L
.4051 .44393 L
.40534 .44393 L
.40556 .44393 L
.40576 .44393 L
.40595 .44393 L
.40614 .44393 L
.40633 .44393 L
.4065 .44393 L
.4067 .44393 L
.40691 .44393 L
.40711 .44393 L
.4073 .44393 L
.40747 .44393 L
.40763 .44393 L
.40781 .44393 L
.408 .44393 L
.4082 .44393 L
.40839 .44393 L
.40855 .44393 L
.40873 .44393 L
.40906 .44393 L
.40924 .44393 L
.40941 .44393 L
.40971 .44393 L
.41003 .44393 L
.4102 .44393 L
.41038 .44393 L
.41055 .44393 L
.41071 .44393 L
.41089 .44393 L
.41108 .44393 L
.41127 .44393 L
.41144 .44393 L
.4116 .44393 L
.41178 .44394 L
Mistroke
.41209 .44394 L
.41228 .44394 L
.41246 .44394 L
.41262 .44394 L
.4128 .44394 L
.41297 .44394 L
.41316 .44394 L
.41335 .44394 L
.41354 .44394 L
.41371 .44394 L
.41387 .44394 L
.41404 .44394 L
.41422 .44394 L
.41452 .44394 L
.4148 .44394 L
.41496 .44394 L
.41512 .44394 L
.41542 .44394 L
.4157 .44394 L
.416 .44394 L
.41625 .44394 L
.41653 .44394 L
.4167 .44394 L
.41685 .44394 L
.41715 .44394 L
.41731 .44394 L
.41747 .44394 L
.41763 .44394 L
.41781 .44394 L
.41796 .44394 L
.41813 .44394 L
.41829 .44394 L
.41843 .44394 L
.41871 .44394 L
.41899 .44394 L
.41914 .44394 L
.4193 .44394 L
.41946 .44394 L
.41961 .44394 L
.41989 .44394 L
.42004 .44394 L
.42019 .44394 L
.42034 .44394 L
.42049 .44394 L
.42064 .44394 L
.42081 .44394 L
.42099 .44394 L
.42115 .44394 L
.4213 .44394 L
.42147 .44394 L
Mistroke
.42174 .44394 L
.4219 .44394 L
.42205 .44394 L
.4222 .44394 L
.42234 .44394 L
.4225 .44394 L
.42266 .44395 L
.42293 .44395 L
.42318 .44395 L
.42345 .44395 L
.42361 .44395 L
.42374 .44395 L
.42401 .44395 L
.42414 .44395 L
.42429 .44395 L
.42454 .44395 L
.42478 .44395 L
.42493 .44395 L
.42506 .44395 L
.42532 .44395 L
.42546 .44395 L
.42559 .44395 L
.42574 .44395 L
.42589 .44395 L
.42612 .44395 L
.42638 .44395 L
.42661 .44395 L
.42683 .44395 L
.42708 .44395 L
.42723 .44395 L
.42736 .44395 L
.4276 .44395 L
.42786 .44395 L
.428 .44395 L
.42815 .44395 L
.42832 .44395 L
.42847 .44395 L
.42861 .44395 L
.42873 .44395 L
.42888 .44395 L
.42902 .44395 L
.42917 .44395 L
.42933 .44395 L
.42947 .44395 L
.42962 .44395 L
.42978 .44395 L
.42995 .44395 L
.43011 .44395 L
.43026 .44395 L
.43039 .44395 L
Mistroke
.43053 .44395 L
.43078 .44395 L
.43093 .44395 L
.43108 .44395 L
.4312 .44395 L
.43134 .44396 L
.43148 .44396 L
.43163 .44396 L
.43179 .44396 L
.43193 .44396 L
.43207 .44396 L
.43219 .44396 L
.43233 .44396 L
.43247 .44396 L
.4326 .44396 L
.43273 .44396 L
.43296 .44396 L
.43309 .44396 L
.43321 .44396 L
.43334 .44396 L
.43347 .44396 L
.4337 .44396 L
.43391 .44396 L
.43415 .44396 L
.43428 .44396 L
.4344 .44396 L
.43452 .44396 L
.43466 .44396 L
.43478 .44396 L
.43491 .44396 L
.43504 .44396 L
.43519 .44396 L
.43533 .44396 L
.43546 .44396 L
.43569 .44396 L
.43582 .44396 L
.43594 .44396 L
.43607 .44396 L
.43619 .44396 L
.43632 .44396 L
.43645 .44396 L
.43666 .44396 L
.43689 .44396 L
.4371 .44396 L
.4373 .44396 L
.43753 .44396 L
.43766 .44396 L
.43777 .44396 L
.43799 .44396 L
.43822 .44396 L
Mistroke
.43844 .44396 L
.43856 .44397 L
.43868 .44397 L
.43888 .44397 L
.4391 .44397 L
.43922 .44397 L
.43934 .44397 L
.43956 .44397 L
.4397 .44397 L
.43982 .44397 L
.43994 .44397 L
.44006 .44397 L
.44026 .44397 L
.44045 .44397 L
.44066 .44397 L
.44088 .44397 L
.44099 .44397 L
.44111 .44397 L
.44133 .44397 L
.44154 .44397 L
.44174 .44397 L
.44194 .44397 L
.44213 .44397 L
.4423 .44397 L
.44248 .44397 L
.44268 .44397 L
.44289 .44397 L
.44306 .44397 L
.44326 .44397 L
.44338 .44397 L
.44349 .44397 L
.44361 .44397 L
.44373 .44397 L
.44395 .44397 L
.44407 .44397 L
.44418 .44397 L
.44439 .44397 L
.44458 .44397 L
.44479 .44398 L
.4449 .44398 L
.44501 .44398 L
.44514 .44398 L
.44526 .44398 L
.44536 .44398 L
.44548 .44398 L
.4456 .44398 L
.44574 .44398 L
.44585 .44398 L
.44597 .44398 L
.44609 .44398 L
Mistroke
.44619 .44398 L
.44631 .44398 L
.44643 .44398 L
.44663 .44398 L
.44675 .44398 L
.44685 .44398 L
.44706 .44398 L
.44718 .44398 L
.44728 .44398 L
.44748 .44398 L
.44769 .44398 L
.44787 .44398 L
.44804 .44398 L
.44823 .44398 L
.44843 .44398 L
.44854 .44398 L
.44865 .44398 L
.44876 .44398 L
.44888 .44398 L
.44908 .44398 L
.44919 .44398 L
.4493 .44398 L
.4495 .44398 L
.44968 .44398 L
.44987 .44398 L
.44997 .44398 L
.45008 .44399 L
.45019 .44399 L
.45032 .44399 L
.45044 .44399 L
.45056 .44399 L
.45068 .44399 L
.45078 .44399 L
.4509 .44399 L
.45101 .44399 L
.45113 .44399 L
.45125 .44399 L
.45136 .44399 L
.45148 .44399 L
.45161 .44399 L
.45175 .44399 L
.45188 .44399 L
.452 .44399 L
.4521 .44399 L
.45221 .44399 L
.45232 .44399 L
.45241 .44399 L
.45253 .44399 L
.45265 .44399 L
.45276 .44399 L
Mistroke
.45287 .44399 L
.45297 .44399 L
.45308 .44399 L
.45319 .44399 L
.45331 .44399 L
.45343 .44399 L
.45355 .44399 L
.45366 .44399 L
.45377 .44399 L
.45388 .44399 L
.45399 .44399 L
.4541 .44399 L
.4542 .44399 L
.45439 .44399 L
.45449 .44399 L
.45459 .44399 L
.4547 .44399 L
.45481 .44399 L
.45493 .444 L
.45504 .444 L
.45514 .444 L
.45524 .444 L
.45534 .444 L
.45545 .444 L
.45564 .444 L
.45574 .444 L
.45583 .444 L
.45601 .444 L
.45619 .444 L
.45628 .444 L
.45639 .444 L
.45657 .444 L
.45668 .444 L
.45679 .444 L
.45689 .444 L
.45699 .444 L
.4571 .444 L
.4572 .444 L
.45732 .444 L
.45743 .444 L
.45753 .444 L
.45763 .444 L
.45773 .444 L
.45784 .444 L
.45796 .444 L
.45809 .444 L
.45821 .444 L
.45832 .444 L
.45843 .444 L
.45854 .444 L
Mistroke
.45865 .444 L
.45876 .444 L
.45888 .444 L
.459 .444 L
.45912 .444 L
.45923 .44401 L
.45933 .44401 L
.45944 .44401 L
.45955 .44401 L
.45966 .44401 L
.45978 .44401 L
.45989 .44401 L
.46 .44401 L
.46011 .44401 L
.46023 .44401 L
.46037 .44401 L
.46049 .44401 L
.4606 .44401 L
.4607 .44401 L
.4608 .44401 L
.46092 .44401 L
.46102 .44401 L
.46112 .44401 L
.4612 .44401 L
.46131 .44401 L
.4614 .44401 L
.46151 .44401 L
.46162 .44401 L
.46171 .44401 L
.46182 .44401 L
.46193 .44401 L
.46205 .44401 L
.46216 .44401 L
.46226 .44401 L
.46235 .44401 L
.46245 .44401 L
.46262 .44401 L
.46273 .44401 L
.46283 .44401 L
.46292 .44401 L
.46302 .44401 L
.46311 .44401 L
.46321 .44402 L
.46332 .44402 L
.46342 .44402 L
.46352 .44402 L
.46361 .44402 L
.46371 .44402 L
.4638 .44402 L
.4639 .44402 L
Mistroke
.46398 .44402 L
.46408 .44402 L
.46418 .44402 L
.46435 .44402 L
.46445 .44402 L
.46454 .44402 L
.46471 .44402 L
.46486 .44402 L
.46495 .44402 L
.46504 .44402 L
.46521 .44402 L
.4653 .44402 L
.46538 .44402 L
.46548 .44402 L
.46558 .44402 L
.46568 .44402 L
.46579 .44402 L
.46588 .44402 L
.46597 .44402 L
.46607 .44402 L
.46615 .44402 L
.46625 .44402 L
.46634 .44402 L
.4665 .44402 L
.46659 .44402 L
.46668 .44402 L
.46683 .44403 L
.46699 .44403 L
.46715 .44403 L
.46728 .44403 L
.46745 .44403 L
.46754 .44403 L
.46762 .44403 L
.46778 .44403 L
.46795 .44403 L
.4681 .44403 L
.46819 .44403 L
.46827 .44403 L
.46842 .44403 L
.46858 .44403 L
.46866 .44403 L
.46875 .44403 L
.46891 .44403 L
.469 .44403 L
.46909 .44403 L
.46918 .44403 L
.46927 .44403 L
.46936 .44403 L
.46944 .44403 L
.46953 .44403 L
Mistroke
.46962 .44403 L
.46978 .44403 L
.46987 .44403 L
.46996 .44403 L
.47006 .44403 L
.47015 .44404 L
.47023 .44404 L
.47032 .44404 L
.47042 .44404 L
.47053 .44404 L
.47061 .44404 L
.47071 .44404 L
.47081 .44404 L
.47091 .44404 L
.471 .44404 L
.47109 .44404 L
.47119 .44404 L
.4713 .44404 L
.47139 .44404 L
.47149 .44404 L
.47159 .44404 L
.47167 .44404 L
.47177 .44404 L
.47187 .44404 L
.47196 .44404 L
.47204 .44404 L
.47212 .44404 L
.47222 .44404 L
.47231 .44404 L
.4724 .44404 L
.47248 .44404 L
.47256 .44404 L
.47266 .44404 L
.47276 .44404 L
.47285 .44404 L
.47294 .44404 L
.47302 .44404 L
.4731 .44404 L
.47319 .44404 L
.47328 .44405 L
.47337 .44405 L
.47346 .44405 L
.47353 .44405 L
.47361 .44405 L
.47376 .44405 L
.47385 .44405 L
.47393 .44405 L
.47401 .44405 L
.47409 .44405 L
.47418 .44405 L
Mistroke
.47426 .44405 L
.47441 .44405 L
.47455 .44405 L
.4747 .44405 L
.47477 .44405 L
.47486 .44405 L
.47495 .44405 L
.47506 .44405 L
.47515 .44405 L
.47524 .44405 L
.47532 .44405 L
.47541 .44405 L
.47551 .44405 L
.47559 .44405 L
.47569 .44405 L
.47579 .44405 L
.47587 .44405 L
.47597 .44405 L
.47607 .44405 L
.47618 .44406 L
.47627 .44406 L
.47637 .44406 L
.47646 .44406 L
.47655 .44406 L
.47663 .44406 L
.47671 .44406 L
.47681 .44406 L
.47691 .44406 L
.477 .44406 L
.47708 .44406 L
.47717 .44406 L
.47725 .44406 L
.47734 .44406 L
.47743 .44406 L
.47751 .44406 L
.47758 .44406 L
.47766 .44406 L
.47775 .44406 L
.47785 .44406 L
.47796 .44406 L
.47806 .44406 L
.47815 .44406 L
.47825 .44406 L
.47837 .44406 L
.47848 .44406 L
.47858 .44406 L
.47867 .44406 L
.47876 .44406 L
.47886 .44407 L
.47896 .44407 L
Mistroke
.47907 .44407 L
.47917 .44407 L
.47926 .44407 L
.47936 .44407 L
.47945 .44407 L
.47953 .44407 L
.47963 .44407 L
.47973 .44407 L
.47982 .44407 L
.47993 .44407 L
.48002 .44407 L
.48011 .44407 L
.4802 .44407 L
.48029 .44407 L
.48038 .44407 L
.48047 .44407 L
.48055 .44407 L
.48063 .44407 L
.48072 .44407 L
.4808 .44407 L
.48089 .44407 L
.48098 .44407 L
.48106 .44407 L
.48114 .44407 L
.48122 .44407 L
.48129 .44407 L
.48137 .44408 L
.48145 .44408 L
.48153 .44408 L
.48162 .44408 L
.4817 .44408 L
.48178 .44408 L
.48187 .44408 L
.48197 .44408 L
.48206 .44408 L
.48214 .44408 L
.48222 .44408 L
.4823 .44408 L
.48237 .44408 L
.48244 .44408 L
.48253 .44408 L
.48261 .44408 L
.4827 .44408 L
.48277 .44408 L
.48285 .44408 L
.48292 .44408 L
.483 .44408 L
.48308 .44408 L
.48316 .44408 L
.48323 .44408 L
Mistroke
.48337 .44408 L
.48345 .44408 L
.48353 .44408 L
.48362 .44408 L
.48371 .44408 L
.4838 .44409 L
.48389 .44409 L
.48397 .44409 L
.48406 .44409 L
.48414 .44409 L
.48422 .44409 L
.48431 .44409 L
.48439 .44409 L
.48447 .44409 L
.48454 .44409 L
.48461 .44409 L
.48469 .44409 L
.48478 .44409 L
.48486 .44409 L
.48496 .44409 L
.48505 .44409 L
.48513 .44409 L
.4852 .44409 L
.48528 .44409 L
.48537 .44409 L
.48546 .44409 L
.48555 .44409 L
.48564 .44409 L
.48572 .44409 L
.48582 .44409 L
.48592 .44409 L
.48602 .44409 L
.48611 .4441 L
.48618 .4441 L
.48626 .4441 L
.48634 .4441 L
.48642 .4441 L
.48649 .4441 L
.48656 .4441 L
.48664 .4441 L
.48671 .4441 L
.48679 .4441 L
.48687 .4441 L
.48694 .4441 L
.48702 .4441 L
.48711 .4441 L
.4872 .4441 L
.48728 .4441 L
.48736 .4441 L
.48743 .4441 L
Mistroke
.4875 .4441 L
.48763 .4441 L
.48771 .4441 L
.48779 .4441 L
.48786 .4441 L
.48793 .4441 L
.488 .4441 L
.48808 .4441 L
.48816 .4441 L
.48824 .44411 L
.48831 .44411 L
.48838 .44411 L
.48845 .44411 L
.48853 .44411 L
.48865 .44411 L
.48872 .44411 L
.48879 .44411 L
.48885 .44411 L
.48892 .44411 L
.48899 .44411 L
.48906 .44411 L
.48918 .44411 L
.4893 .44411 L
.48942 .44411 L
.48949 .44411 L
.48955 .44411 L
.48962 .44411 L
.4897 .44411 L
.48976 .44411 L
.48983 .44411 L
.4899 .44411 L
.48998 .44411 L
.49006 .44411 L
.49013 .44411 L
.49019 .44411 L
.49026 .44411 L
.49033 .44412 L
.4904 .44412 L
.49047 .44412 L
.49054 .44412 L
.49066 .44412 L
.49073 .44412 L
.4908 .44412 L
.49087 .44412 L
.49093 .44412 L
.49101 .44412 L
.49109 .44412 L
.49116 .44412 L
.49123 .44412 L
.49131 .44412 L
Mistroke
.49137 .44412 L
.49145 .44412 L
.49153 .44412 L
.49161 .44412 L
.49169 .44412 L
.49176 .44412 L
.49184 .44412 L
.49191 .44412 L
.49198 .44412 L
.49204 .44412 L
.4921 .44412 L
.49218 .44412 L
.49225 .44413 L
.49232 .44413 L
.49239 .44413 L
.49246 .44413 L
.49252 .44413 L
.49259 .44413 L
.49266 .44413 L
.49277 .44413 L
.49284 .44413 L
.4929 .44413 L
.49298 .44413 L
.49305 .44413 L
.49313 .44413 L
.49322 .44413 L
.49329 .44413 L
.49336 .44413 L
.49343 .44413 L
.49351 .44413 L
.49358 .44413 L
.49366 .44413 L
.49373 .44413 L
.49379 .44413 L
.49385 .44413 L
.49392 .44413 L
.49399 .44413 L
.49405 .44413 L
.49413 .44413 L
.49421 .44414 L
.49429 .44414 L
.49436 .44414 L
.49443 .44414 L
.49449 .44414 L
.49457 .44414 L
.49464 .44414 L
.49471 .44414 L
.49479 .44414 L
.49486 .44414 L
.49494 .44414 L
Mistroke
.49502 .44414 L
.49511 .44414 L
.49519 .44414 L
.49526 .44414 L
.49534 .44414 L
.49541 .44414 L
.49548 .44414 L
.49556 .44414 L
.49563 .44414 L
.49569 .44414 L
.49576 .44414 L
.49584 .44414 L
.49592 .44414 L
.49601 .44415 L
.49609 .44415 L
.49617 .44415 L
.49624 .44415 L
.49631 .44415 L
.49639 .44415 L
.49647 .44415 L
.49654 .44415 L
.49663 .44415 L
.49671 .44415 L
.49678 .44415 L
.49686 .44415 L
.49693 .44415 L
.497 .44415 L
.49708 .44415 L
.49716 .44415 L
.49724 .44415 L
.49731 .44415 L
.49739 .44415 L
.49747 .44415 L
.49756 .44415 L
.49765 .44415 L
.49773 .44416 L
.49779 .44416 L
.49787 .44416 L
.49794 .44416 L
.49801 .44416 L
.49808 .44416 L
.49814 .44416 L
.49821 .44416 L
.49828 .44416 L
.49835 .44416 L
.49843 .44416 L
.4985 .44416 L
.49856 .44416 L
.49864 .44416 L
.49872 .44416 L
Mistroke
.4988 .44416 L
.49887 .44416 L
.49894 .44416 L
.499 .44416 L
.49907 .44416 L
.49912 .44416 L
.49919 .44416 L
.49926 .44416 L
.49933 .44416 L
.4994 .44417 L
.49946 .44417 L
.49952 .44417 L
.49959 .44417 L
.49966 .44417 L
.49974 .44417 L
.49981 .44417 L
.49988 .44417 L
.49995 .44417 L
.50003 .44417 L
.50011 .44417 L
.50019 .44417 L
.50028 .44417 L
.50036 .44417 L
.50043 .44417 L
.50051 .44417 L
.50059 .44417 L
.50067 .44417 L
.50075 .44417 L
.50083 .44417 L
.5009 .44417 L
.50097 .44418 L
.50104 .44418 L
.50111 .44418 L
.50118 .44418 L
.50125 .44418 L
.50134 .44418 L
.50142 .44418 L
.50151 .44418 L
.50158 .44418 L
.50165 .44418 L
.50173 .44418 L
.50181 .44418 L
.50189 .44418 L
.50197 .44418 L
.50205 .44418 L
.50213 .44418 L
.50222 .44418 L
.50231 .44418 L
.5024 .44418 L
.50248 .44418 L
Mistroke
.50255 .44419 L
.50261 .44419 L
.50269 .44419 L
.50276 .44419 L
.50282 .44419 L
.50288 .44419 L
.50295 .44419 L
.50302 .44419 L
.50309 .44419 L
.50316 .44419 L
.50323 .44419 L
.5033 .44419 L
.50337 .44419 L
.50345 .44419 L
.50353 .44419 L
.5036 .44419 L
.50368 .44419 L
.50376 .44419 L
.50383 .44419 L
.5039 .44419 L
.50396 .44419 L
.50403 .4442 L
.5041 .4442 L
.50417 .4442 L
.50425 .4442 L
.50432 .4442 L
.50439 .4442 L
.50446 .4442 L
.50454 .4442 L
.50462 .4442 L
.50469 .4442 L
.50477 .4442 L
.50484 .4442 L
.50491 .4442 L
.50497 .4442 L
.50503 .4442 L
.5051 .4442 L
.50517 .4442 L
.50524 .4442 L
.50531 .4442 L
.50538 .4442 L
.50544 .4442 L
.50551 .44421 L
.50558 .44421 L
.50566 .44421 L
.50573 .44421 L
.5058 .44421 L
.50587 .44421 L
.50593 .44421 L
.50599 .44421 L
Mistroke
.50606 .44421 L
.50612 .44421 L
.50619 .44421 L
.50626 .44421 L
.50632 .44421 L
.50639 .44421 L
.50645 .44421 L
.50651 .44421 L
.50658 .44421 L
.50664 .44421 L
.5067 .44421 L
.50675 .44421 L
.50681 .44421 L
.50687 .44422 L
.50694 .44422 L
.50701 .44422 L
.50707 .44422 L
.50713 .44422 L
.5072 .44422 L
.50728 .44422 L
.50735 .44422 L
.50741 .44422 L
.50748 .44422 L
.50754 .44422 L
.50761 .44422 L
.50768 .44422 L
.50774 .44422 L
.50779 .44422 L
.50786 .44422 L
.50792 .44422 L
.508 .44422 L
.50807 .44422 L
.50815 .44422 L
.50823 .44423 L
.5083 .44423 L
.50837 .44423 L
.50845 .44423 L
.50852 .44423 L
.50859 .44423 L
.50866 .44423 L
.50874 .44423 L
.50881 .44423 L
.50888 .44423 L
.50894 .44423 L
.509 .44423 L
.50907 .44423 L
.50913 .44423 L
.50919 .44423 L
.50924 .44423 L
.5093 .44423 L
Mistroke
.50936 .44423 L
.50942 .44423 L
.50949 .44423 L
.50955 .44424 L
.50961 .44424 L
.50967 .44424 L
.50973 .44424 L
.50979 .44424 L
.50985 .44424 L
.50992 .44424 L
.50998 .44424 L
.51005 .44424 L
.51011 .44424 L
.51016 .44424 L
.51021 .44424 L
.51027 .44424 L
.51033 .44424 L
.51039 .44424 L
.51045 .44424 L
.51051 .44424 L
.51056 .44424 L
.51061 .44424 L
.51067 .44424 L
.51073 .44424 L
.51083 .44425 L
.51088 .44425 L
.51094 .44425 L
.511 .44425 L
.51106 .44425 L
.51113 .44425 L
.5112 .44425 L
.51126 .44425 L
.51131 .44425 L
.51138 .44425 L
.51144 .44425 L
.5115 .44425 L
.51157 .44425 L
.51163 .44425 L
.51168 .44425 L
.51173 .44425 L
.51179 .44425 L
.51184 .44425 L
.5119 .44425 L
.51196 .44425 L
.51202 .44425 L
.51209 .44426 L
.51215 .44426 L
.51221 .44426 L
.51227 .44426 L
.51233 .44426 L
Mistroke
.51239 .44426 L
.51245 .44426 L
.51252 .44426 L
.51258 .44426 L
.51264 .44426 L
.51271 .44426 L
.51278 .44426 L
.51285 .44426 L
.51291 .44426 L
.51297 .44426 L
.51303 .44426 L
.51309 .44426 L
.51315 .44426 L
.51321 .44426 L
.51327 .44427 L
.51334 .44427 L
.51339 .44427 L
.51345 .44427 L
.5135 .44427 L
.51356 .44427 L
.51363 .44427 L
.51368 .44427 L
.51373 .44427 L
.51378 .44427 L
.51384 .44427 L
.5139 .44427 L
.51395 .44427 L
.514 .44427 L
.51406 .44427 L
.51412 .44427 L
.51417 .44427 L
.51422 .44427 L
.51428 .44427 L
.51433 .44427 L
.51438 .44427 L
.51444 .44428 L
.51449 .44428 L
.51458 .44428 L
.51463 .44428 L
.51468 .44428 L
.51473 .44428 L
.51479 .44428 L
.51485 .44428 L
.51491 .44428 L
.51497 .44428 L
.51502 .44428 L
.51507 .44428 L
.51513 .44428 L
.51519 .44428 L
.51525 .44428 L
Mistroke
.51531 .44428 L
.51537 .44428 L
.51543 .44428 L
.5155 .44428 L
.51557 .44429 L
.51563 .44429 L
.51568 .44429 L
.51573 .44429 L
.51578 .44429 L
.51583 .44429 L
.51589 .44429 L
.51595 .44429 L
.51601 .44429 L
.51606 .44429 L
.51612 .44429 L
.51618 .44429 L
.51623 .44429 L
.51628 .44429 L
.51634 .44429 L
.5164 .44429 L
.51646 .44429 L
.51652 .44429 L
.51657 .44429 L
.51662 .44429 L
.51668 .4443 L
.51673 .4443 L
.51679 .4443 L
.51683 .4443 L
.51689 .4443 L
.51694 .4443 L
.51699 .4443 L
.51704 .4443 L
.51709 .4443 L
.51714 .4443 L
.51723 .4443 L
.51728 .4443 L
.51733 .4443 L
.51738 .4443 L
.51744 .4443 L
.51749 .4443 L
.51754 .4443 L
.5176 .4443 L
.51766 .4443 L
.51772 .4443 L
.51777 .44431 L
.51783 .44431 L
.51788 .44431 L
.51793 .44431 L
.51799 .44431 L
.51804 .44431 L
Mistroke
.5181 .44431 L
.51815 .44431 L
.51819 .44431 L
.51828 .44431 L
.51833 .44431 L
.51838 .44431 L
.51843 .44431 L
.51848 .44431 L
.51853 .44431 L
.51858 .44431 L
.51867 .44431 L
.51875 .44431 L
.51884 .44432 L
.51889 .44432 L
.51893 .44432 L
.51898 .44432 L
.51904 .44432 L
.51908 .44432 L
.51913 .44432 L
.51918 .44432 L
.51923 .44432 L
.51932 .44432 L
.51937 .44432 L
.51942 .44432 L
.51947 .44432 L
.51953 .44432 L
.51958 .44432 L
.51962 .44432 L
.51968 .44432 L
.51973 .44432 L
.51978 .44432 L
.51984 .44433 L
.5199 .44433 L
.51996 .44433 L
.52001 .44433 L
.52007 .44433 L
.52012 .44433 L
.52017 .44433 L
.52023 .44433 L
.52029 .44433 L
.52036 .44433 L
.52042 .44433 L
.52048 .44433 L
.52053 .44433 L
.52059 .44433 L
.52065 .44433 L
.5207 .44433 L
.52075 .44433 L
.52081 .44433 L
.52086 .44434 L
Mistroke
.52096 .44434 L
.52101 .44434 L
.52106 .44434 L
.52112 .44434 L
.52118 .44434 L
.52123 .44434 L
.52128 .44434 L
.52134 .44434 L
.5214 .44434 L
.52145 .44434 L
.52151 .44434 L
.52156 .44434 L
.52162 .44434 L
.52167 .44434 L
.52172 .44434 L
.52181 .44434 L
.52186 .44435 L
.52191 .44435 L
.52197 .44435 L
.52202 .44435 L
.52208 .44435 L
.52213 .44435 L
.52219 .44435 L
.52223 .44435 L
.52229 .44435 L
.52234 .44435 L
.52239 .44435 L
.52244 .44435 L
.52249 .44435 L
.52255 .44435 L
.52264 .44435 L
.52269 .44435 L
.52274 .44435 L
.52279 .44435 L
.52284 .44436 L
.52292 .44436 L
.52297 .44436 L
.52301 .44436 L
.52311 .44436 L
.52316 .44436 L
.52321 .44436 L
.52331 .44436 L
.52336 .44436 L
.52342 .44436 L
.52347 .44436 L
.52353 .44436 L
.52358 .44436 L
.52364 .44436 L
.52369 .44436 L
.52374 .44437 L
Mistroke
.52383 .44437 L
.52387 .44437 L
.52392 .44437 L
.52398 .44437 L
.52404 .44437 L
.52409 .44437 L
.52415 .44437 L
.52421 .44437 L
.52427 .44437 L
.52433 .44437 L
.52438 .44437 L
.52444 .44437 L
.52449 .44437 L
.52455 .44437 L
.52461 .44437 L
.52466 .44437 L
.52471 .44438 L
.52476 .44438 L
.52481 .44438 L
.52487 .44438 L
.52493 .44438 L
.52499 .44438 L
.52505 .44438 L
.52511 .44438 L
.52517 .44438 L
.52524 .44438 L
.5253 .44438 L
.52535 .44438 L
.5254 .44438 L
.52546 .44438 L
.52552 .44438 L
.52558 .44439 L
.52564 .44439 L
.52569 .44439 L
.52575 .44439 L
.5258 .44439 L
.52585 .44439 L
.52591 .44439 L
.52596 .44439 L
.52602 .44439 L
.52608 .44439 L
.52613 .44439 L
.52619 .44439 L
.52624 .44439 L
.52629 .44439 L
.52635 .44439 L
.5264 .44439 L
.52649 .4444 L
.52655 .4444 L
.52659 .4444 L
Mistroke
.52668 .4444 L
.52673 .4444 L
.52678 .4444 L
.52687 .4444 L
.52691 .4444 L
.52696 .4444 L
.52704 .4444 L
.52709 .4444 L
.52713 .4444 L
.52721 .4444 L
.52729 .4444 L
.52733 .44441 L
.52738 .44441 L
.52742 .44441 L
.52747 .44441 L
.52752 .44441 L
.52756 .44441 L
.52764 .44441 L
.52769 .44441 L
.52773 .44441 L
.52778 .44441 L
.52783 .44441 L
.52792 .44441 L
.52798 .44441 L
.52803 .44441 L
.5281 .44441 L
.52814 .44441 L
.52818 .44442 L
.52827 .44442 L
.52831 .44442 L
.52835 .44442 L
.52842 .44442 L
.5285 .44442 L
.52858 .44442 L
.52866 .44442 L
.5287 .44442 L
.52875 .44442 L
.52879 .44442 L
.52883 .44442 L
.52891 .44442 L
.52899 .44442 L
.52903 .44443 L
.52907 .44443 L
.52916 .44443 L
.5292 .44443 L
.52925 .44443 L
.52934 .44443 L
.52939 .44443 L
.52944 .44443 L
.52948 .44443 L
Mistroke
.52953 .44443 L
.52957 .44443 L
.52961 .44443 L
.52969 .44443 L
.52974 .44443 L
.52979 .44443 L
.52984 .44444 L
.52989 .44444 L
.52994 .44444 L
.52998 .44444 L
.53003 .44444 L
.53007 .44444 L
.53015 .44444 L
.53023 .44444 L
.53027 .44444 L
.53032 .44444 L
.5304 .44444 L
.53044 .44444 L
.53048 .44444 L
.53056 .44444 L
.53061 .44444 L
.53066 .44445 L
.53075 .44445 L
.53079 .44445 L
.53083 .44445 L
.53088 .44445 L
.53093 .44445 L
.53102 .44445 L
.53108 .44445 L
.53112 .44445 L
.53117 .44445 L
.53121 .44445 L
.53129 .44445 L
.53134 .44445 L
.53139 .44445 L
.53147 .44446 L
.53152 .44446 L
.53157 .44446 L
.53162 .44446 L
.53167 .44446 L
.53172 .44446 L
.53176 .44446 L
.53185 .44446 L
.5319 .44446 L
.53195 .44446 L
.532 .44446 L
.53206 .44446 L
.53215 .44446 L
.5322 .44447 L
.53225 .44447 L
Mistroke
.53234 .44447 L
.53239 .44447 L
.53244 .44447 L
.53248 .44447 L
.53253 .44447 L
.53261 .44447 L
.53266 .44447 L
.53271 .44447 L
.53276 .44447 L
.53281 .44447 L
.5329 .44447 L
.53295 .44448 L
.533 .44448 L
.53304 .44448 L
.53308 .44448 L
.53317 .44448 L
.53321 .44448 L
.53326 .44448 L
.5333 .44448 L
.53336 .44448 L
.5334 .44448 L
.53344 .44448 L
.53352 .44448 L
.53356 .44448 L
.53361 .44448 L
.53369 .44449 L
.53374 .44449 L
.53378 .44449 L
.53386 .44449 L
.53391 .44449 L
.53395 .44449 L
.534 .44449 L
.53405 .44449 L
.53409 .44449 L
.53413 .44449 L
.53421 .44449 L
.53425 .44449 L
.5343 .44449 L
.53434 .44449 L
.53438 .44449 L
.53446 .4445 L
.5345 .4445 L
.53454 .4445 L
.53462 .4445 L
.53466 .4445 L
.5347 .4445 L
.53478 .4445 L
.53484 .4445 L
.53492 .4445 L
.53499 .4445 L
Mistroke
.53504 .4445 L
.53508 .4445 L
.53516 .44451 L
.53521 .44451 L
.53526 .44451 L
.53534 .44451 L
.53539 .44451 L
.53544 .44451 L
.53552 .44451 L
.53557 .44451 L
.53562 .44451 L
.53567 .44451 L
.53571 .44451 L
.53579 .44451 L
.53584 .44451 L
.53588 .44452 L
.53595 .44452 L
.53599 .44452 L
.53603 .44452 L
.53612 .44452 L
.53616 .44452 L
.53621 .44452 L
.53629 .44452 L
.53633 .44452 L
.53637 .44452 L
.53647 .44452 L
.53651 .44452 L
.53657 .44453 L
.53661 .44453 L
.53666 .44453 L
.53674 .44453 L
.53679 .44453 L
.53685 .44453 L
.5369 .44453 L
.53695 .44453 L
.53704 .44453 L
.53709 .44453 L
.53714 .44453 L
.53719 .44453 L
.53723 .44453 L
.53732 .44454 L
.53736 .44454 L
.53741 .44454 L
.53746 .44454 L
.53751 .44454 L
.53755 .44454 L
.53759 .44454 L
.53767 .44454 L
.53772 .44454 L
.53776 .44454 L
Mistroke
.53781 .44454 L
.53785 .44454 L
.53793 .44455 L
.538 .44455 L
.53804 .44455 L
.53808 .44455 L
.53816 .44455 L
.5382 .44455 L
.53824 .44455 L
.53832 .44455 L
.53836 .44455 L
.53841 .44455 L
.53845 .44455 L
.53849 .44455 L
.53856 .44455 L
.53864 .44456 L
.53871 .44456 L
.53877 .44456 L
.53885 .44456 L
.53892 .44456 L
.53896 .44456 L
.539 .44456 L
.53908 .44456 L
.53912 .44456 L
.53917 .44456 L
.53921 .44456 L
.53925 .44457 L
.53929 .44457 L
.53933 .44457 L
.53941 .44457 L
.53946 .44457 L
.5395 .44457 L
.53954 .44457 L
.53959 .44457 L
.53964 .44457 L
.53968 .44457 L
.53972 .44457 L
.53976 .44457 L
.53984 .44457 L
.53991 .44458 L
.53995 .44458 L
.53999 .44458 L
.54007 .44458 L
.54011 .44458 L
.54015 .44458 L
.54023 .44458 L
.54027 .44458 L
.54031 .44458 L
.54035 .44458 L
.54039 .44458 L
Mistroke
.54047 .44458 L
.54053 .44458 L
.54061 .44459 L
.54068 .44459 L
.54076 .44459 L
.5408 .44459 L
.54084 .44459 L
.54091 .44459 L
.54099 .44459 L
.54105 .44459 L
.54112 .44459 L
.54118 .4446 L
.54124 .4446 L
.54131 .4446 L
.54138 .4446 L
.54146 .4446 L
.54153 .4446 L
.5416 .4446 L
.54165 .4446 L
.54169 .4446 L
.54176 .4446 L
.54184 .44461 L
.5419 .44461 L
.54197 .44461 L
.54204 .44461 L
.54211 .44461 L
.54216 .44461 L
.54219 .44461 L
.54224 .44461 L
.54228 .44461 L
.54236 .44461 L
.5424 .44462 L
.54244 .44462 L
.54251 .44462 L
.54258 .44462 L
.54266 .44462 L
.5427 .44462 L
.54274 .44462 L
.54281 .44462 L
.54288 .44462 L
.54295 .44462 L
.54303 .44463 L
.5431 .44463 L
.54316 .44463 L
.54323 .44463 L
.5433 .44463 L
.54336 .44463 L
.54343 .44463 L
.5435 .44463 L
.54357 .44463 L
Mistroke
.54363 .44464 L
.54369 .44464 L
.54375 .44464 L
.54381 .44464 L
.54388 .44464 L
.54395 .44464 L
.54403 .44464 L
.5441 .44464 L
.54416 .44464 L
.54423 .44465 L
.54429 .44465 L
.54436 .44465 L
.54443 .44465 L
.54451 .44465 L
.54458 .44465 L
.54464 .44465 L
.54472 .44465 L
.54476 .44466 L
.5448 .44466 L
.54487 .44466 L
.54495 .44466 L
.54502 .44466 L
.54508 .44466 L
.54513 .44466 L
.54519 .44466 L
.54526 .44466 L
.54533 .44467 L
.54539 .44467 L
.54545 .44467 L
.54551 .44467 L
.54557 .44467 L
.54563 .44467 L
.54569 .44467 L
.54575 .44467 L
.5458 .44467 L
.54586 .44467 L
.54592 .44468 L
.54599 .44468 L
.54605 .44468 L
.54613 .44468 L
.5462 .44468 L
.54627 .44468 L
.54633 .44468 L
.5464 .44468 L
.54647 .44469 L
.54653 .44469 L
.5466 .44469 L
.54667 .44469 L
.54673 .44469 L
.54678 .44469 L
Mistroke
.54684 .44469 L
.54691 .44469 L
.54697 .44469 L
.54704 .4447 L
.5471 .4447 L
.54718 .4447 L
.54725 .4447 L
.54731 .4447 L
.54737 .4447 L
.54744 .4447 L
.54751 .4447 L
.54758 .44471 L
.54765 .44471 L
.54772 .44471 L
.54779 .44471 L
.54786 .44471 L
.5479 .44471 L
.54794 .44471 L
.54802 .44471 L
.54809 .44472 L
.54813 .44472 L
.54817 .44472 L
.54825 .44472 L
.54829 .44472 L
.54832 .44472 L
.54837 .44472 L
.54841 .44472 L
.54849 .44472 L
.54855 .44472 L
.54863 .44473 L
.54867 .44473 L
.54871 .44473 L
.54878 .44473 L
.54884 .44473 L
.54891 .44473 L
.54898 .44473 L
.54905 .44473 L
.54913 .44474 L
.54919 .44474 L
.54926 .44474 L
.54933 .44474 L
.5494 .44474 L
.54947 .44474 L
.54954 .44474 L
.54959 .44474 L
.54966 .44475 L
.54972 .44475 L
.54978 .44475 L
.54985 .44475 L
.54992 .44475 L
Mistroke
.54998 .44475 L
.55005 .44475 L
.55012 .44475 L
.5502 .44476 L
.55026 .44476 L
.55033 .44476 L
.5504 .44476 L
.55046 .44476 L
.55052 .44476 L
.55058 .44476 L
.55065 .44476 L
.55072 .44477 L
.55078 .44477 L
.55084 .44477 L
.5509 .44477 L
.55096 .44477 L
.55102 .44477 L
.55109 .44477 L
.55114 .44477 L
.5512 .44478 L
.55126 .44478 L
.55132 .44478 L
.5514 .44478 L
.55147 .44478 L
.5515 .44478 L
.55155 .44478 L
.55162 .44478 L
.55169 .44479 L
.55177 .44479 L
.55183 .44479 L
.5519 .44479 L
.55194 .44479 L
.55199 .44479 L
.55206 .44479 L
.5521 .44479 L
.55214 .4448 L
.55219 .4448 L
.55223 .4448 L
.5523 .4448 L
.55234 .4448 L
.55238 .4448 L
.55245 .4448 L
.55249 .4448 L
.55254 .4448 L
.55261 .4448 L
.55267 .44481 L
.55275 .44481 L
.55282 .44481 L
.55288 .44481 L
.55295 .44481 L
Mistroke
.55301 .44481 L
.55308 .44481 L
.55315 .44482 L
.55323 .44482 L
.5533 .44482 L
.55337 .44482 L
.55344 .44482 L
.55349 .44482 L
.55353 .44482 L
.5536 .44483 L
.55368 .44483 L
.55375 .44483 L
.55382 .44483 L
.55388 .44483 L
.55395 .44483 L
.55403 .44484 L
.55407 .44484 L
.55411 .44484 L
.55418 .44484 L
.55425 .44484 L
.55431 .44484 L
.55438 .44484 L
.55444 .44484 L
.55451 .44485 L
.55458 .44485 L
.55465 .44485 L
.55472 .44485 L
.55479 .44485 L
.55485 .44485 L
.55491 .44485 L
.55497 .44486 L
.55503 .44486 L
.5551 .44486 L
.55517 .44486 L
.55523 .44486 L
.5553 .44486 L
.55535 .44486 L
.55541 .44487 L
.55547 .44487 L
.55553 .44487 L
.5556 .44487 L
.55566 .44487 L
.55572 .44487 L
.55578 .44487 L
.55583 .44487 L
.55589 .44488 L
.55595 .44488 L
.55601 .44488 L
.55607 .44488 L
.55612 .44488 L
Mistroke
.55617 .44488 L
.55623 .44488 L
.55628 .44489 L
.55635 .44489 L
.5564 .44489 L
.55646 .44489 L
.55652 .44489 L
.55659 .44489 L
.55665 .44489 L
.55671 .44489 L
.55676 .4449 L
.55682 .4449 L
.55687 .4449 L
.55692 .4449 L
.55698 .4449 L
.55704 .4449 L
.5571 .4449 L
.55715 .4449 L
.5572 .44491 L
.55725 .44491 L
.5573 .44491 L
.55735 .44491 L
.55741 .44491 L
.55747 .44491 L
.55752 .44491 L
.55757 .44491 L
.55763 .44492 L
.55769 .44492 L
.55775 .44492 L
.5578 .44492 L
.55786 .44492 L
.55793 .44492 L
.558 .44492 L
.55806 .44493 L
.55813 .44493 L
.55819 .44493 L
.55825 .44493 L
.55831 .44493 L
.55838 .44493 L
.55845 .44494 L
.55851 .44494 L
.55858 .44494 L
.55864 .44494 L
.5587 .44494 L
.55875 .44494 L
.55881 .44494 L
.55887 .44495 L
.55894 .44495 L
.559 .44495 L
.55906 .44495 L
Mistroke
.55912 .44495 L
.55917 .44495 L
.55923 .44495 L
.55929 .44496 L
.55934 .44496 L
.55939 .44496 L
.55945 .44496 L
.5595 .44496 L
.55956 .44496 L
.55963 .44496 L
.5597 .44497 L
.55976 .44497 L
.55982 .44497 L
.55988 .44497 L
.55994 .44497 L
.56 .44497 L
.56007 .44497 L
.56014 .44498 L
.5602 .44498 L
.56026 .44498 L
.56033 .44498 L
.56041 .44498 L
.56048 .44499 L
.56054 .44499 L
.5606 .44499 L
.56066 .44499 L
.56071 .44499 L
.56077 .44499 L
.56083 .44499 L
.56089 .445 L
.56096 .445 L
.56101 .445 L
.56107 .445 L
.56113 .445 L
.56119 .445 L
.56125 .445 L
.56132 .44501 L
.56139 .44501 L
.56145 .44501 L
.56151 .44501 L
.56157 .44501 L
.56163 .44501 L
.56169 .44502 L
.56175 .44502 L
.5618 .44502 L
.56185 .44502 L
.56191 .44502 L
.56196 .44502 L
.56202 .44502 L
.56208 .44503 L
Mistroke
.56213 .44503 L
.56219 .44503 L
.56225 .44503 L
.56232 .44503 L
.56238 .44503 L
.56244 .44504 L
.56249 .44504 L
.56255 .44504 L
.5626 .44504 L
.56265 .44504 L
.56271 .44504 L
.56277 .44504 L
.56283 .44505 L
.56288 .44505 L
.56294 .44505 L
.56298 .44505 L
.56304 .44505 L
.56309 .44505 L
.56314 .44505 L
.56319 .44506 L
.56324 .44506 L
.5633 .44506 L
.56335 .44506 L
.5634 .44506 L
.5635 .44506 L
.56355 .44506 L
.5636 .44507 L
.56366 .44507 L
.56372 .44507 L
.56376 .44507 L
.56381 .44507 L
.5639 .44507 L
.56395 .44508 L
.56401 .44508 L
.5641 .44508 L
.56415 .44508 L
.56421 .44508 L
.56427 .44508 L
.56432 .44509 L
.56437 .44509 L
.56442 .44509 L
.56447 .44509 L
.56452 .44509 L
.56458 .44509 L
.56463 .44509 L
.56469 .4451 L
.56474 .4451 L
.5648 .4451 L
.56486 .4451 L
.56492 .4451 L
Mistroke
.56498 .4451 L
.56503 .44511 L
.56508 .44511 L
.56513 .44511 L
.56518 .44511 L
.56528 .44511 L
.56533 .44511 L
.56538 .44512 L
.56544 .44512 L
.56549 .44512 L
.56554 .44512 L
.56559 .44512 L
.56565 .44512 L
.56571 .44512 L
.56576 .44513 L
.56582 .44513 L
.56588 .44513 L
.56594 .44513 L
.56599 .44513 L
.56604 .44513 L
.5661 .44514 L
.56617 .44514 L
.56622 .44514 L
.56628 .44514 L
.56634 .44514 L
.56639 .44514 L
.56644 .44515 L
.5665 .44515 L
.56655 .44515 L
.5666 .44515 L
.56665 .44515 L
.56671 .44515 L
.56676 .44515 L
.56681 .44516 L
.56686 .44516 L
.56691 .44516 L
.56701 .44516 L
.56706 .44516 L
.5671 .44516 L
.56716 .44517 L
.56721 .44517 L
.56726 .44517 L
.56732 .44517 L
.56738 .44517 L
.56743 .44517 L
.56748 .44518 L
.56753 .44518 L
.56762 .44518 L
.56768 .44518 L
.56773 .44518 L
Mistroke
.56778 .44518 L
.56782 .44519 L
.56788 .44519 L
.56793 .44519 L
.56798 .44519 L
.56802 .44519 L
.56811 .44519 L
.56815 .4452 L
.5682 .4452 L
.56825 .4452 L
.5683 .4452 L
.56839 .4452 L
.56843 .4452 L
.56848 .44521 L
.56853 .44521 L
.56857 .44521 L
.56861 .44521 L
.56866 .44521 L
.5687 .44521 L
.56874 .44521 L
.56881 .44522 L
.56886 .44522 L
.5689 .44522 L
.56899 .44522 L
.56903 .44522 L
.56908 .44522 L
.56913 .44522 L
.56918 .44523 L
.56927 .44523 L
.56932 .44523 L
.56937 .44523 L
.56941 .44523 L
.56946 .44524 L
.56954 .44524 L
.56958 .44524 L
.56962 .44524 L
.56972 .44524 L
.56977 .44524 L
.56983 .44525 L
.56989 .44525 L
.56994 .44525 L
.57 .44525 L
.57006 .44525 L
.57012 .44526 L
.57018 .44526 L
.57024 .44526 L
.5703 .44526 L
.57036 .44526 L
.57041 .44526 L
.57048 .44527 L
Mistroke
.57054 .44527 L
.5706 .44527 L
.57066 .44527 L
.57072 .44527 L
.57078 .44528 L
.57083 .44528 L
.57088 .44528 L
.57094 .44528 L
.571 .44528 L
.57105 .44529 L
.57111 .44529 L
.57117 .44529 L
.57122 .44529 L
.57127 .44529 L
.57133 .44529 L
.57138 .4453 L
.57142 .4453 L
.57147 .4453 L
.57153 .4453 L
.57159 .4453 L
.57166 .4453 L
.57172 .44531 L
.57178 .44531 L
.57185 .44531 L
.57192 .44531 L
.57199 .44532 L
.57205 .44532 L
.57211 .44532 L
.57217 .44532 L
.57224 .44532 L
.5723 .44533 L
.57236 .44533 L
.57242 .44533 L
.57248 .44533 L
.57254 .44533 L
.5726 .44534 L
.57265 .44534 L
.57271 .44534 L
.57277 .44534 L
.57284 .44534 L
.5729 .44535 L
.57296 .44535 L
.57302 .44535 L
.57307 .44535 L
.57313 .44535 L
.57319 .44536 L
.57324 .44536 L
.57329 .44536 L
.57334 .44536 L
.5734 .44536 L
Mistroke
.57345 .44536 L
.5735 .44537 L
.57356 .44537 L
.57362 .44537 L
.57367 .44537 L
.57376 .44537 L
.57381 .44538 L
.57386 .44538 L
.57392 .44538 L
.57397 .44538 L
.57403 .44538 L
.57409 .44539 L
.57414 .44539 L
.57419 .44539 L
.57424 .44539 L
.5743 .44539 L
.57439 .4454 L
.57445 .4454 L
.57449 .4454 L
.57458 .4454 L
.57463 .4454 L
.57467 .44541 L
.57473 .44541 L
.57478 .44541 L
.57483 .44541 L
.57489 .44541 L
.57498 .44542 L
.57503 .44542 L
.57508 .44542 L
.57514 .44542 L
.57519 .44542 L
.57524 .44543 L
.57529 .44543 L
.57539 .44543 L
.57544 .44543 L
.57549 .44543 L
.57555 .44544 L
.57561 .44544 L
.57565 .44544 L
.57571 .44544 L
.57577 .44544 L
.57583 .44545 L
.57588 .44545 L
.57594 .44545 L
.576 .44545 L
.57606 .44545 L
.57611 .44546 L
.57616 .44546 L
.57622 .44546 L
.57629 .44546 L
Mistroke
.57634 .44547 L
.5764 .44547 L
.57646 .44547 L
.57651 .44547 L
.57657 .44547 L
.57662 .44548 L
.57668 .44548 L
.57672 .44548 L
.57678 .44548 L
.57683 .44548 L
.57688 .44548 L
.57693 .44549 L
.57698 .44549 L
.57704 .44549 L
.57709 .44549 L
.57715 .44549 L
.5772 .4455 L
.57725 .4455 L
.57735 .4455 L
.5774 .4455 L
.57745 .44551 L
.5775 .44551 L
.57754 .44551 L
.57763 .44551 L
.57769 .44551 L
.57774 .44552 L
.57779 .44552 L
.57785 .44552 L
.57791 .44552 L
.57797 .44553 L
.57803 .44553 L
.57808 .44553 L
.57813 .44553 L
.57819 .44553 L
.57824 .44554 L
.5783 .44554 L
.57839 .44554 L
.57844 .44554 L
.57849 .44554 L
.57859 .44555 L
.57864 .44555 L
.57869 .44555 L
.57873 .44555 L
.57878 .44556 L
.57887 .44556 L
.57892 .44556 L
.57898 .44556 L
.57903 .44557 L
.57909 .44557 L
.57918 .44557 L
Mistroke
.57924 .44557 L
.57928 .44558 L
.57933 .44558 L
.57938 .44558 L
.57947 .44558 L
.57952 .44558 L
.57956 .44559 L
.57961 .44559 L
.57967 .44559 L
.57971 .44559 L
.57976 .44559 L
.57984 .4456 L
.57989 .4456 L
.57994 .4456 L
.58003 .4456 L
.58007 .44561 L
.58012 .44561 L
.58017 .44561 L
.58022 .44561 L
.58027 .44561 L
.58031 .44562 L
.5804 .44562 L
.58045 .44562 L
.58049 .44562 L
.58055 .44562 L
.5806 .44563 L
.58065 .44563 L
.5807 .44563 L
.58074 .44563 L
.58079 .44563 L
.58088 .44564 L
.58093 .44564 L
.58097 .44564 L
.58105 .44564 L
.5811 .44565 L
.58114 .44565 L
.58123 .44565 L
.58128 .44565 L
.58133 .44566 L
.58142 .44566 L
.58146 .44566 L
.58151 .44566 L
.58155 .44567 L
.5816 .44567 L
.58164 .44567 L
.58168 .44567 L
.58172 .44567 L
.58176 .44567 L
.58184 .44568 L
.58188 .44568 L
Mistroke
.58193 .44568 L
.58197 .44568 L
.58202 .44568 L
.5821 .44569 L
.58215 .44569 L
.58219 .44569 L
.58223 .44569 L
.58228 .44569 L
.58231 .4457 L
.58235 .4457 L
.58243 .4457 L
.5825 .4457 L
.58254 .44571 L
.58258 .44571 L
.58266 .44571 L
.5827 .44571 L
.58275 .44571 L
.58283 .44572 L
.58288 .44572 L
.58293 .44572 L
.58303 .44573 L
.58308 .44573 L
.58313 .44573 L
.58319 .44573 L
.58325 .44574 L
.5833 .44574 L
.58335 .44574 L
.5834 .44574 L
.58345 .44574 L
.58354 .44575 L
.58359 .44575 L
.58364 .44575 L
.58369 .44575 L
.58373 .44576 L
.58382 .44576 L
.58387 .44576 L
.58392 .44576 L
.58397 .44577 L
.58402 .44577 L
.58406 .44577 L
.58411 .44577 L
.58419 .44578 L
.58423 .44578 L
.58428 .44578 L
.58437 .44578 L
.58442 .44579 L
.58447 .44579 L
.58456 .44579 L
.58461 .44579 L
.58466 .4458 L
Mistroke
.58471 .4458 L
.58477 .4458 L
.58482 .4458 L
.58487 .44581 L
.58492 .44581 L
.58497 .44581 L
.58501 .44581 L
.58506 .44581 L
.58514 .44582 L
.5852 .44582 L
.58525 .44582 L
.5853 .44582 L
.58536 .44583 L
.58541 .44583 L
.58547 .44583 L
.58552 .44583 L
.58558 .44584 L
.58563 .44584 L
.58568 .44584 L
.58574 .44584 L
.58579 .44585 L
.58589 .44585 L
.58594 .44585 L
.58599 .44586 L
.58603 .44586 L
.58608 .44586 L
.58617 .44586 L
.58622 .44587 L
.58626 .44587 L
.58631 .44587 L
.58636 .44587 L
.58641 .44587 L
.58645 .44588 L
.58653 .44588 L
.58658 .44588 L
.58663 .44588 L
.58667 .44589 L
.58672 .44589 L
.5868 .44589 L
.58685 .44589 L
.58689 .4459 L
.58697 .4459 L
.58702 .4459 L
.58706 .4459 L
.58714 .44591 L
.58721 .44591 L
.58725 .44591 L
.58729 .44592 L
.58737 .44592 L
.58742 .44592 L
Mistroke
.58746 .44592 L
.58755 .44593 L
.5876 .44593 L
.58765 .44593 L
.58774 .44594 L
.58779 .44594 L
.58784 .44594 L
.58793 .44595 L
.58798 .44595 L
.58803 .44595 L
.58808 .44595 L
.58813 .44595 L
.58822 .44596 L
.58826 .44596 L
.58831 .44596 L
.58838 .44597 L
.58843 .44597 L
.58847 .44597 L
.58855 .44598 L
.58861 .44598 L
.58865 .44598 L
.58874 .44598 L
.58878 .44599 L
.58883 .44599 L
.58887 .44599 L
.58893 .44599 L
.58902 .446 L
.58907 .446 L
.58913 .446 L
.58917 .44601 L
.58923 .44601 L
.58928 .44601 L
.58933 .44601 L
.58939 .44602 L
.58944 .44602 L
.58949 .44602 L
.58954 .44602 L
.58959 .44603 L
.58964 .44603 L
.58969 .44603 L
.58974 .44603 L
.58982 .44604 L
.58987 .44604 L
.58993 .44604 L
.58997 .44604 L
.59002 .44605 L
.59012 .44605 L
.59017 .44605 L
.59021 .44606 L
.5903 .44606 L
Mistroke
.59034 .44606 L
.59038 .44606 L
.59044 .44607 L
.59049 .44607 L
.59054 .44607 L
.5906 .44608 L
.59065 .44608 L
.59069 .44608 L
.59074 .44608 L
.59079 .44609 L
.59085 .44609 L
.5909 .44609 L
.59096 .44609 L
.59101 .4461 L
.59107 .4461 L
.59113 .4461 L
.59118 .44611 L
.59124 .44611 L
.59128 .44611 L
.59133 .44611 L
.59142 .44612 L
.59148 .44612 L
.59153 .44612 L
.59162 .44613 L
.59167 .44613 L
.59173 .44613 L
.59177 .44614 L
.59182 .44614 L
.59188 .44614 L
.59193 .44614 L
.59199 .44615 L
.59203 .44615 L
.59208 .44615 L
.59212 .44615 L
.59221 .44616 L
.59225 .44616 L
.59229 .44616 L
.59234 .44617 L
.59239 .44617 L
.59247 .44617 L
.59251 .44617 L
.59255 .44618 L
.59259 .44618 L
.59264 .44618 L
.59272 .44618 L
.59277 .44619 L
.59281 .44619 L
.59285 .44619 L
.5929 .44619 L
.59295 .4462 L
Mistroke
.59299 .4462 L
.59308 .4462 L
.59312 .44621 L
.59317 .44621 L
.59322 .44621 L
.59326 .44621 L
.59336 .44622 L
.59341 .44622 L
.59346 .44622 L
.59355 .44623 L
.5936 .44623 L
.59364 .44623 L
.59374 .44624 L
.59379 .44624 L
.59383 .44624 L
.59391 .44625 L
.59396 .44625 L
.594 .44625 L
.59409 .44626 L
.59413 .44626 L
.59418 .44626 L
.59422 .44627 L
.59428 .44627 L
.59437 .44627 L
.59442 .44628 L
.59447 .44628 L
.59452 .44628 L
.59457 .44628 L
.59462 .44629 L
.59467 .44629 L
.59472 .44629 L
.59477 .4463 L
.59482 .4463 L
.59486 .4463 L
.59495 .44631 L
.59499 .44631 L
.59504 .44631 L
.59509 .44631 L
.59514 .44632 L
.59522 .44632 L
.59527 .44632 L
.59532 .44633 L
.59536 .44633 L
.5954 .44633 L
.59548 .44634 L
.59552 .44634 L
.59556 .44634 L
.59565 .44635 L
.5957 .44635 L
.59575 .44635 L
Mistroke
.5958 .44635 L
.59584 .44636 L
.59589 .44636 L
.59593 .44636 L
.59602 .44637 L
.59607 .44637 L
.59611 .44637 L
.59616 .44637 L
.59622 .44638 L
.59627 .44638 L
.59631 .44638 L
.59636 .44639 L
.5964 .44639 L
.59649 .44639 L
.59653 .4464 L
.59658 .4464 L
.59666 .4464 L
.5967 .44641 L
.59675 .44641 L
.59682 .44641 L
.59689 .44642 L
.59693 .44642 L
.59698 .44642 L
.59705 .44643 L
.5971 .44643 L
.59714 .44643 L
.59718 .44643 L
.59723 .44644 L
.59732 .44644 L
.59737 .44644 L
.59741 .44645 L
.59745 .44645 L
.59749 .44645 L
.59757 .44646 L
.59762 .44646 L
.59766 .44646 L
.59774 .44647 L
.59778 .44647 L
.59783 .44647 L
.59792 .44648 L
.59797 .44648 L
.59802 .44648 L
.59811 .44649 L
.59815 .44649 L
.5982 .44649 L
.59829 .4465 L
.59834 .4465 L
.59839 .4465 L
.59844 .44651 L
.59849 .44651 L
Mistroke
.59858 .44652 L
.59862 .44652 L
.59867 .44652 L
.59876 .44653 L
.59881 .44653 L
.59885 .44653 L
.59893 .44654 L
.59898 .44654 L
.59902 .44654 L
.5991 .44655 L
.59914 .44655 L
.59918 .44655 L
.59923 .44656 L
.59928 .44656 L
.59933 .44656 L
.59938 .44657 L
.59947 .44657 L
.59952 .44657 L
.59956 .44658 L
.59961 .44658 L
.59966 .44658 L
.59972 .44659 L
.59977 .44659 L
.59987 .4466 L
.59992 .4466 L
.59997 .4466 L
.60002 .4466 L
.60007 .44661 L
.60011 .44661 L
.60016 .44661 L
.6002 .44662 L
.60025 .44662 L
.60033 .44662 L
.60038 .44663 L
.60042 .44663 L
.60047 .44663 L
.60052 .44664 L
.60061 .44664 L
.60066 .44665 L
.60071 .44665 L
.6008 .44665 L
.60085 .44666 L
.6009 .44666 L
.60094 .44666 L
.60099 .44667 L
.60107 .44667 L
.60113 .44668 L
.60118 .44668 L
.60123 .44668 L
.60128 .44669 L
Mistroke
.60137 .44669 L
.60143 .44669 L
.60147 .4467 L
.60152 .4467 L
.60156 .4467 L
.60165 .44671 L
.6017 .44671 L
.60174 .44671 L
.60179 .44672 L
.60184 .44672 L
.60188 .44672 L
.60193 .44673 L
.60201 .44673 L
.60205 .44674 L
.6021 .44674 L
.60214 .44674 L
.60219 .44674 L
.60223 .44675 L
.60227 .44675 L
.60234 .44675 L
.60239 .44676 L
.60243 .44676 L
.60251 .44677 L
.60255 .44677 L
.6026 .44677 L
.60263 .44677 L
.60267 .44678 L
.60275 .44678 L
.60282 .44679 L
.60286 .44679 L
.6029 .44679 L
.60294 .44679 L
.60299 .4468 L
.60303 .4468 L
.60307 .4468 L
.60315 .44681 L
.60319 .44681 L
.60323 .44681 L
.60328 .44682 L
.60332 .44682 L
.60341 .44683 L
.60346 .44683 L
.60351 .44683 L
.60355 .44684 L
.6036 .44684 L
.60365 .44684 L
.60369 .44685 L
.60373 .44685 L
.60378 .44685 L
.60386 .44686 L
Mistroke
.60391 .44686 L
.60395 .44686 L
.604 .44687 L
.60405 .44687 L
.6041 .44687 L
.60415 .44688 L
.60423 .44688 L
.60428 .44689 L
.60432 .44689 L
.60436 .44689 L
.6044 .44689 L
.60444 .4469 L
.60448 .4469 L
.60456 .4469 L
.6046 .44691 L
.60464 .44691 L
.60469 .44691 L
.60474 .44692 L
.60478 .44692 L
.60482 .44692 L
.6049 .44693 L
.60495 .44693 L
.60499 .44693 L
.60503 .44694 L
.60508 .44694 L
.60517 .44695 L
.60521 .44695 L
.60525 .44695 L
.6053 .44696 L
.60535 .44696 L
.60544 .44697 L
.60549 .44697 L
.60553 .44697 L
.60558 .44698 L
.60563 .44698 L
.60568 .44698 L
.60573 .44699 L
.60581 .44699 L
.60586 .447 L
.60591 .447 L
.60601 .44701 L
.60606 .44701 L
.60611 .44701 L
.60616 .44702 L
.60621 .44702 L
.6063 .44703 L
.60636 .44703 L
.6064 .44704 L
.60648 .44704 L
.60652 .44704 L
Mistroke
.60656 .44705 L
.60663 .44705 L
.6067 .44706 L
.60677 .44706 L
.60683 .44707 L
.6069 .44707 L
.60698 .44708 L
.60702 .44708 L
.60705 .44708 L
.6071 .44709 L
.60714 .44709 L
.60721 .4471 L
.60725 .4471 L
.60729 .4471 L
.60737 .44711 L
.60743 .44711 L
.60751 .44712 L
.60754 .44712 L
.60758 .44712 L
.60766 .44713 L
.60773 .44713 L
.60779 .44714 L
.60786 .44714 L
.60793 .44715 L
.60797 .44715 L
.60801 .44715 L
.60808 .44716 L
.60815 .44716 L
.60821 .44717 L
.60828 .44717 L
.60834 .44718 L
.6084 .44718 L
.60847 .44719 L
.60854 .44719 L
.60861 .4472 L
.60867 .4472 L
.60874 .44721 L
.60878 .44721 L
.60882 .44722 L
.6089 .44722 L
.60894 .44723 L
.60898 .44723 L
.60905 .44723 L
.60909 .44724 L
.60913 .44724 L
.6092 .44725 L
.60927 .44725 L
.60931 .44725 L
.60936 .44726 L
.6094 .44726 L
Mistroke
.60945 .44726 L
.60953 .44727 L
.60957 .44727 L
.60961 .44728 L
.60969 .44728 L
.60976 .44729 L
.6098 .44729 L
.60984 .4473 L
.60992 .4473 L
.60999 .44731 L
.61006 .44731 L
.61013 .44732 L
.61018 .44732 L
.61021 .44732 L
.61029 .44733 L
.61033 .44733 L
.61037 .44734 L
.61045 .44734 L
.61049 .44735 L
.61053 .44735 L
.6106 .44736 L
.61066 .44736 L
.61074 .44737 L
.61081 .44737 L
.61088 .44738 L
.61094 .44738 L
.61101 .44739 L
.61108 .44739 L
.61116 .4474 L
.6112 .4474 L
.61124 .44741 L
.61132 .44741 L
.61136 .44742 L
.6114 .44742 L
.61147 .44743 L
.61155 .44743 L
.61158 .44744 L
.61162 .44744 L
.6117 .44744 L
.61174 .44745 L
.61178 .44745 L
.61183 .44746 L
.61188 .44746 L
.61196 .44747 L
.612 .44747 L
.61205 .44747 L
.61209 .44748 L
.61213 .44748 L
.6122 .44749 L
.61224 .44749 L
Mistroke
.61228 .44749 L
.61232 .4475 L
.61236 .4475 L
.61244 .44751 L
.61248 .44751 L
.61252 .44751 L
.61259 .44752 L
.61267 .44753 L
.61274 .44753 L
.61281 .44754 L
.61288 .44754 L
.61294 .44755 L
.61298 .44755 L
.61302 .44756 L
.61309 .44756 L
.61313 .44756 L
.61317 .44757 L
.61321 .44757 L
.61326 .44758 L
.61333 .44758 L
.6134 .44759 L
.61348 .44759 L
.61352 .4476 L
.61356 .4476 L
.61363 .44761 L
.6137 .44761 L
.61377 .44762 L
.61381 .44762 L
.61385 .44763 L
.61391 .44763 L
.61399 .44764 L
.61406 .44764 L
.61412 .44765 L
.61418 .44766 L
.61424 .44766 L
.61431 .44767 L
.61438 .44767 L
.61444 .44768 L
.6145 .44768 L
.61456 .44769 L
.61463 .44769 L
.6147 .4477 L
.61476 .44771 L
.61484 .44771 L
.61488 .44772 L
.61491 .44772 L
.61499 .44773 L
.61503 .44773 L
.61507 .44773 L
.61514 .44774 L
Mistroke
.61522 .44775 L
.61528 .44775 L
.61535 .44776 L
.61541 .44776 L
.61547 .44777 L
.61554 .44778 L
.61562 .44778 L
.61569 .44779 L
.61575 .44779 L
.61579 .4478 L
.61583 .4478 L
.61591 .44781 L
.61595 .44781 L
.61599 .44782 L
.61607 .44782 L
.61611 .44783 L
.61616 .44783 L
.61623 .44784 L
.61628 .44784 L
.61632 .44785 L
.61636 .44785 L
.61641 .44785 L
.61648 .44786 L
.61652 .44786 L
.61656 .44787 L
.61662 .44787 L
.61669 .44788 L
.61673 .44788 L
.61678 .44789 L
.61682 .44789 L
.61686 .44789 L
.61693 .4479 L
.617 .44791 L
.61704 .44791 L
.61709 .44792 L
.61716 .44792 L
.6172 .44793 L
.61724 .44793 L
.61729 .44794 L
.61733 .44794 L
.61741 .44795 L
.61748 .44795 L
.61756 .44796 L
.6176 .44796 L
.61764 .44797 L
.61768 .44797 L
.61772 .44798 L
.6178 .44798 L
.61785 .44799 L
.61789 .44799 L
Mistroke
.61798 .448 L
.61802 .448 L
.61807 .44801 L
.61815 .44802 L
.61819 .44802 L
.61824 .44802 L
.61828 .44803 L
.61833 .44803 L
.6184 .44804 L
.61847 .44805 L
.61851 .44805 L
.61855 .44806 L
.61862 .44806 L
.6187 .44807 L
.61877 .44808 L
.61885 .44808 L
.61889 .44809 L
.61893 .44809 L
.619 .4481 L
.61907 .44811 L
.61914 .44811 L
.61921 .44812 L
.61928 .44813 L
.61934 .44813 L
.61941 .44814 L
.61948 .44815 L
.61955 .44815 L
.61962 .44816 L
.6197 .44817 L
.61973 .44817 L
.61978 .44817 L
.61985 .44818 L
.61992 .44819 L
.61999 .4482 L
.62006 .4482 L
.6201 .44821 L
.62014 .44821 L
.62021 .44822 L
.62026 .44822 L
.62029 .44823 L
.62034 .44823 L
.62038 .44824 L
.62046 .44824 L
.62053 .44825 L
.62061 .44826 L
.62065 .44826 L
.62069 .44827 L
.62076 .44827 L
.62083 .44828 L
.6209 .44829 L
Mistroke
.62096 .44829 L
.62104 .4483 L
.62108 .44831 L
.62112 .44831 L
.62119 .44832 L
.62126 .44832 L
.62132 .44833 L
.62139 .44834 L
.62145 .44834 L
.62151 .44835 L
.62158 .44836 L
.62166 .44837 L
.62173 .44837 L
.62179 .44838 L
.62185 .44839 L
.62191 .44839 L
.62197 .4484 L
.62202 .4484 L
.62208 .44841 L
.62215 .44842 L
.62221 .44842 L
.62227 .44843 L
.62233 .44844 L
.62239 .44844 L
.62245 .44845 L
.62252 .44846 L
.62258 .44846 L
.62265 .44847 L
.62271 .44848 L
.62277 .44848 L
.62285 .44849 L
.62288 .44849 L
.62292 .4485 L
.623 .44851 L
.62306 .44851 L
.62313 .44852 L
.62319 .44853 L
.62325 .44853 L
.62332 .44854 L
.6234 .44855 L
.62344 .44855 L
.62348 .44856 L
.62355 .44857 L
.62363 .44857 L
.6237 .44858 L
.62376 .44859 L
.62383 .4486 L
.6239 .4486 L
.62397 .44861 L
.62404 .44862 L
Mistroke
.62411 .44863 L
.62418 .44863 L
.62425 .44864 L
.62432 .44865 L
.62438 .44866 L
.62445 .44866 L
.6245 .44867 L
.62457 .44868 L
.62464 .44869 L
.6247 .44869 L
.62476 .4487 L
.62482 .44871 L
.62488 .44871 L
.62494 .44872 L
.62501 .44873 L
.62507 .44873 L
.62514 .44874 L
.62521 .44875 L
.62528 .44876 L
.62533 .44876 L
.6254 .44877 L
.62547 .44878 L
.62554 .44879 L
.62559 .44879 L
.62563 .4488 L
.6257 .4488 L
.62577 .44881 L
.62584 .44882 L
.62591 .44883 L
.62598 .44884 L
.62605 .44884 L
.62609 .44885 L
.62613 .44885 L
.6262 .44886 L
.62625 .44887 L
.62629 .44887 L
.62633 .44888 L
.62637 .44888 L
.62644 .44889 L
.62651 .4489 L
.62658 .44891 L
.62662 .44891 L
.62666 .44892 L
.62674 .44892 L
.6268 .44893 L
.62686 .44894 L
.62693 .44895 L
.627 .44896 L
.62704 .44896 L
.62708 .44896 L
Mistroke
.62715 .44897 L
.62722 .44898 L
.62728 .44899 L
.62734 .449 L
.6274 .449 L
.62747 .44901 L
.62755 .44902 L
.62761 .44903 L
.62767 .44903 L
.62774 .44904 L
.6278 .44905 L
.62786 .44906 L
.62793 .44907 L
.62799 .44907 L
.62805 .44908 L
.6281 .44909 L
.62816 .44909 L
.62822 .4491 L
.62828 .44911 L
.62834 .44912 L
.62841 .44912 L
.62848 .44913 L
.62854 .44914 L
.6286 .44915 L
.62866 .44915 L
.62873 .44916 L
.62879 .44917 L
.62886 .44918 L
.62893 .44919 L
.62899 .44919 L
.62906 .4492 L
.6291 .44921 L
.62914 .44921 L
.62921 .44922 L
.62928 .44923 L
.62935 .44924 L
.62942 .44925 L
.62949 .44926 L
.62956 .44927 L
.62963 .44927 L
.62969 .44928 L
.62976 .44929 L
.62982 .4493 L
.62989 .44931 L
.62997 .44932 L
.63003 .44932 L
.6301 .44933 L
.63017 .44934 L
.63022 .44935 L
.63026 .44935 L
Mistroke
.63033 .44936 L
.6304 .44937 L
.63046 .44938 L
.63053 .44939 L
.6306 .4494 L
.63066 .4494 L
.63072 .44941 L
.63078 .44942 L
.63084 .44943 L
.6309 .44943 L
.63097 .44944 L
.63103 .44945 L
.6311 .44946 L
.63116 .44947 L
.63123 .44948 L
.6313 .44949 L
.63138 .4495 L
.63144 .4495 L
.63151 .44951 L
.63158 .44952 L
.63165 .44953 L
.63172 .44954 L
.63179 .44955 L
.63185 .44956 L
.63191 .44957 L
.63198 .44958 L
.63206 .44959 L
.6321 .44959 L
.63214 .4496 L
.63222 .44961 L
.63226 .44961 L
.6323 .44962 L
.63237 .44963 L
.63244 .44964 L
.63251 .44965 L
.63258 .44965 L
.63265 .44966 L
.63269 .44967 L
.63273 .44968 L
.6328 .44968 L
.63287 .44969 L
.63291 .4497 L
.63295 .4497 L
.63302 .44971 L
.63306 .44972 L
.6331 .44973 L
.63314 .44973 L
.63319 .44974 L
.63326 .44975 L
.63333 .44976 L
Mistroke
.6334 .44977 L
.63347 .44978 L
.63354 .44978 L
.63361 .44979 L
.63365 .4498 L
.63369 .44981 L
.63376 .44982 L
.63382 .44982 L
.63388 .44983 L
.63393 .44984 L
.634 .44985 L
.63405 .44986 L
.6341 .44986 L
.63416 .44987 L
.63422 .44988 L
.63428 .44989 L
.63435 .4499 L
.63442 .44991 L
.63448 .44991 L
.63453 .44992 L
.63459 .44993 L
.63465 .44994 L
.63471 .44995 L
.63477 .44996 L
.63484 .44997 L
.63491 .44997 L
.63496 .44998 L
.63502 .44999 L
.63508 .45 L
.63514 .45001 L
.63519 .45002 L
.63526 .45002 L
.63532 .45003 L
.63538 .45004 L
.63544 .45005 L
.63551 .45006 L
.63558 .45007 L
.63565 .45008 L
.63571 .45009 L
.63576 .4501 L
.63582 .45011 L
.63588 .45011 L
.63594 .45012 L
.63599 .45013 L
.63604 .45014 L
.63609 .45014 L
.63615 .45015 L
.6362 .45016 L
.63626 .45017 L
.63632 .45018 L
Mistroke
.63637 .45019 L
.63643 .45019 L
.6365 .4502 L
.63656 .45021 L
.63662 .45022 L
.63667 .45023 L
.63672 .45024 L
.63677 .45024 L
.63682 .45025 L
.63687 .45026 L
.63693 .45027 L
.63698 .45028 L
.63704 .45028 L
.63709 .45029 L
.63715 .4503 L
.6372 .45031 L
.63725 .45032 L
.63731 .45032 L
.63737 .45033 L
.63742 .45034 L
.63747 .45035 L
.63754 .45036 L
.6376 .45037 L
.63767 .45038 L
.63774 .45039 L
.6378 .4504 L
.63786 .45041 L
.63792 .45042 L
.63799 .45043 L
.63805 .45044 L
.63811 .45045 L
.63818 .45046 L
.63823 .45046 L
.63829 .45047 L
.63834 .45048 L
.6384 .45049 L
.63846 .4505 L
.63852 .45051 L
.63858 .45052 L
.63865 .45053 L
.63872 .45054 L
.63878 .45055 L
.63883 .45056 L
.6389 .45057 L
.63896 .45058 L
.63902 .45059 L
.63909 .4506 L
.63915 .45061 L
.63922 .45062 L
.63929 .45063 L
Mistroke
.63936 .45064 L
.63943 .45065 L
.6395 .45066 L
.63955 .45067 L
.6396 .45068 L
.63966 .45069 L
.63972 .4507 L
.63977 .4507 L
.63982 .45071 L
.63987 .45072 L
.63993 .45073 L
.63998 .45074 L
.64004 .45075 L
.6401 .45076 L
.64015 .45076 L
.64021 .45077 L
.64028 .45078 L
.64034 .45079 L
.6404 .4508 L
.64046 .45081 L
.64052 .45082 L
.64058 .45083 L
.64064 .45084 L
.64069 .45085 L
.64074 .45086 L
.6408 .45087 L
.64086 .45088 L
.64092 .45089 L
.64098 .4509 L
.64103 .45091 L
.64109 .45092 L
.64115 .45093 L
.64122 .45094 L
.64128 .45095 L
.64134 .45096 L
.64139 .45097 L
.64145 .45097 L
.6415 .45098 L
.64155 .45099 L
.6416 .451 L
.64166 .45101 L
.64172 .45102 L
.64177 .45103 L
.64183 .45104 L
.64188 .45105 L
.64193 .45106 L
.64199 .45107 L
.64206 .45108 L
.64212 .45109 L
.64217 .45109 L
Mistroke
.64223 .4511 L
.64228 .45111 L
.64232 .45112 L
.64237 .45113 L
.64243 .45114 L
.64248 .45115 L
.64254 .45116 L
.64259 .45117 L
.64264 .45117 L
.6427 .45118 L
.64274 .45119 L
.64279 .4512 L
.64285 .45121 L
.64293 .45122 L
.64298 .45123 L
.64303 .45124 L
.64309 .45125 L
.64313 .45126 L
.64319 .45127 L
.64325 .45128 L
.6433 .45129 L
.64334 .45129 L
.64339 .4513 L
.64345 .45131 L
.64351 .45132 L
.64357 .45133 L
.64363 .45134 L
.64369 .45135 L
.64374 .45136 L
.64379 .45137 L
.64385 .45138 L
.6439 .45139 L
.64396 .4514 L
.64402 .45141 L
.64408 .45142 L
.64413 .45143 L
.64418 .45144 L
.64424 .45145 L
.64429 .45146 L
.64435 .45147 L
.6444 .45148 L
.64446 .45149 L
.64452 .4515 L
.64457 .45151 L
.64463 .45152 L
.6447 .45153 L
.64476 .45154 L
.64482 .45155 L
.64487 .45156 L
.64491 .45157 L
Mistroke
.64496 .45158 L
.64502 .45159 L
.64507 .4516 L
.64513 .45161 L
.64517 .45162 L
.64523 .45163 L
.64532 .45164 L
.64537 .45165 L
.64542 .45166 L
.64547 .45167 L
.64552 .45168 L
.6456 .45169 L
.64566 .4517 L
.64571 .45171 L
.64576 .45172 L
.64582 .45173 L
.64588 .45174 L
.64594 .45175 L
.64599 .45176 L
.64604 .45177 L
.64609 .45178 L
.64615 .45179 L
.6462 .4518 L
.64625 .45181 L
.64635 .45183 L
.6464 .45184 L
.64645 .45185 L
.64654 .45187 L
.64659 .45188 L
.64664 .45189 L
.64668 .45189 L
.64673 .4519 L
.64682 .45192 L
.64687 .45193 L
.64692 .45194 L
.64698 .45195 L
.64703 .45196 L
.64712 .45198 L
.64718 .45199 L
.64722 .45199 L
.64727 .452 L
.64732 .45201 L
.6474 .45203 L
.64745 .45204 L
.6475 .45205 L
.64755 .45206 L
.6476 .45207 L
.64769 .45208 L
.64774 .45209 L
.64779 .4521 L
Mistroke
.64783 .45211 L
.64787 .45212 L
.64796 .45213 L
.648 .45214 L
.64805 .45215 L
.64814 .45217 L
.6482 .45218 L
.64825 .45219 L
.64829 .4522 L
.64835 .45221 L
.6484 .45222 L
.64845 .45223 L
.64851 .45224 L
.64856 .45225 L
.64861 .45226 L
.64865 .45227 L
.6487 .45228 L
.64876 .45229 L
.64884 .45231 L
.64889 .45232 L
.64894 .45232 L
.64898 .45233 L
.64902 .45234 L
.64911 .45236 L
.64915 .45237 L
.6492 .45238 L
.64925 .45239 L
.6493 .4524 L
.64939 .45241 L
.64944 .45242 L
.64949 .45243 L
.64954 .45244 L
.64958 .45245 L
.64964 .45246 L
.64968 .45247 L
.64977 .45249 L
.64982 .4525 L
.64987 .45251 L
.64992 .45252 L
.64997 .45253 L
.65002 .45254 L
.65007 .45255 L
.65013 .45256 L
.65018 .45257 L
.65023 .45258 L
.65028 .45259 L
.65034 .4526 L
.65038 .45261 L
.65044 .45263 L
.65049 .45264 L
Mistroke
.65055 .45265 L
.65061 .45266 L
.65066 .45267 L
.65072 .45268 L
.65077 .45269 L
.65082 .4527 L
.65087 .45271 L
.65092 .45273 L
.65102 .45274 L
.65107 .45276 L
.65112 .45277 L
.65116 .45277 L
.6512 .45278 L
.65128 .4528 L
.65133 .45281 L
.65137 .45282 L
.65142 .45283 L
.65147 .45284 L
.6515 .45285 L
.65155 .45285 L
.65162 .45287 L
.65167 .45288 L
.65171 .45289 L
.65179 .45291 L
.65184 .45292 L
.65188 .45292 L
.65192 .45293 L
.65197 .45294 L
.65202 .45295 L
.65206 .45296 L
.65214 .45298 L
.65219 .45299 L
.65223 .453 L
.65228 .45301 L
.65233 .45302 L
.65242 .45304 L
.65247 .45305 L
.65253 .45306 L
.65258 .45307 L
.65262 .45308 L
.6527 .4531 L
.65275 .45311 L
.6528 .45312 L
.65285 .45313 L
.65289 .45314 L
.65297 .45316 L
.65301 .45317 L
.65306 .45318 L
.65311 .45319 L
.65315 .4532 L
Mistroke
.65323 .45321 L
.65328 .45323 L
.65333 .45324 L
.65337 .45325 L
.65342 .45326 L
.65347 .45327 L
.65351 .45328 L
.6536 .4533 L
.65365 .45331 L
.65369 .45332 L
.65375 .45333 L
.65379 .45334 L
.65385 .45335 L
.65389 .45336 L
.65394 .45337 L
.654 .45338 L
.65404 .45339 L
.65408 .4534 L
.65416 .45342 L
.6542 .45343 L
.65424 .45344 L
.65428 .45345 L
.65433 .45346 L
.65436 .45346 L
.6544 .45347 L
.65447 .45349 L
.65451 .4535 L
.65456 .45351 L
.65463 .45352 L
.65468 .45353 L
.65472 .45354 L
.65476 .45355 L
.6548 .45356 L
.65485 .45357 L
.65489 .45358 L
.65496 .4536 L
.65501 .45361 L
.65505 .45362 L
.6551 .45363 L
.65514 .45364 L
.65523 .45366 L
.65528 .45367 L
.65533 .45368 L
.65537 .45369 L
.65541 .4537 L
.65549 .45372 L
.65554 .45373 L
.65558 .45374 L
.65563 .45375 L
.65567 .45376 L
Mistroke
.65575 .45378 L
.65578 .45379 L
.65583 .4538 L
.65587 .45381 L
.65591 .45382 L
.65599 .45384 L
.65604 .45385 L
.65608 .45386 L
.65612 .45387 L
.65617 .45388 L
.65622 .45389 L
.65626 .4539 L
.65634 .45392 L
.65638 .45393 L
.65642 .45394 L
.65647 .45395 L
.65652 .45396 L
.65661 .45398 L
.65667 .45399 L
.65671 .45401 L
.65679 .45402 L
.65686 .45404 L
.65693 .45406 L
.65697 .45407 L
.65701 .45408 L
.65708 .45409 L
.65713 .4541 L
.65716 .45411 L
.65724 .45413 L
.65729 .45414 L
.65733 .45415 L
.6574 .45417 L
.65747 .45418 L
.65754 .4542 L
.65758 .45421 L
.65762 .45422 L
.65768 .45424 L
.65776 .45426 L
.65783 .45427 L
.65789 .45429 L
.65797 .45431 L
.65801 .45432 L
.65805 .45433 L
.65809 .45434 L
.65813 .45435 L
.6582 .45436 L
.65824 .45437 L
.65829 .45438 L
.65833 .4544 L
.65838 .45441 L
Mistroke
.65842 .45442 L
.65846 .45443 L
.65854 .45445 L
.65858 .45446 L
.65862 .45447 L
.65869 .45448 L
.65873 .45449 L
.65877 .4545 L
.65881 .45451 L
.65885 .45452 L
.65893 .45454 L
.65899 .45456 L
.65907 .45458 L
.65911 .45459 L
.65915 .4546 L
.65919 .45461 L
.65924 .45462 L
.65932 .45464 L
.65937 .45465 L
.65941 .45466 L
.65946 .45468 L
.65951 .45469 L
.65958 .45471 L
.65962 .45472 L
.65967 .45473 L
.65971 .45474 L
.65976 .45475 L
.65984 .45477 L
.65989 .45479 L
.65993 .4548 L
.65998 .45481 L
.66003 .45482 L
.66007 .45483 L
.66011 .45484 L
.6602 .45487 L
.66025 .45488 L
.66029 .45489 L
.66034 .4549 L
.66039 .45491 L
.66044 .45493 L
.66049 .45494 L
.66054 .45495 L
.66059 .45497 L
.66064 .45498 L
.66069 .45499 L
.66074 .45501 L
.66079 .45502 L
.66088 .45504 L
.66092 .45505 L
.66097 .45507 L
Mistroke
.66102 .45508 L
.66107 .45509 L
.66113 .45511 L
.66118 .45512 L
.66127 .45514 L
.66132 .45516 L
.66137 .45517 L
.66141 .45518 L
.66146 .45519 L
.66154 .45522 L
.66158 .45523 L
.66163 .45524 L
.66172 .45526 L
.66177 .45528 L
.66181 .45529 L
.6619 .45531 L
.66195 .45532 L
.662 .45534 L
.66204 .45535 L
.66209 .45536 L
.66213 .45537 L
.66217 .45538 L
.66224 .4554 L
.66229 .45542 L
.66233 .45543 L
.66241 .45545 L
.66245 .45546 L
.6625 .45547 L
.66254 .45548 L
.66259 .4555 L
.66263 .45551 L
.66267 .45552 L
.66271 .45553 L
.66275 .45554 L
.66283 .45556 L
.66289 .45558 L
.66294 .45559 L
.66297 .4556 L
.66305 .45562 L
.66309 .45563 L
.66313 .45564 L
.6632 .45566 L
.66324 .45568 L
.66327 .45569 L
.66332 .4557 L
.66336 .45571 L
.66344 .45573 L
.66349 .45574 L
.66353 .45576 L
.66358 .45577 L
Mistroke
.66362 .45578 L
.66366 .45579 L
.6637 .4558 L
.66374 .45581 L
.66378 .45583 L
.66385 .45585 L
.66389 .45586 L
.66393 .45587 L
.66401 .45589 L
.66406 .45591 L
.6641 .45592 L
.66415 .45593 L
.6642 .45595 L
.66429 .45597 L
.66433 .45598 L
.66438 .456 L
.66442 .45601 L
.66446 .45602 L
.66454 .45604 L
.66458 .45605 L
.66462 .45606 L
.66471 .45609 L
.66475 .4561 L
.6648 .45612 L
.66485 .45613 L
.6649 .45614 L
.66495 .45616 L
.66499 .45617 L
.66508 .4562 L
.66513 .45621 L
.66517 .45622 L
.66522 .45624 L
.66528 .45625 L
.66533 .45627 L
.66538 .45628 L
.66542 .4563 L
.66547 .45631 L
.66555 .45633 L
.6656 .45635 L
.66565 .45636 L
.66572 .45638 L
.66577 .4564 L
.66581 .45641 L
.6659 .45644 L
.66595 .45645 L
.666 .45646 L
.66608 .45649 L
.66613 .4565 L
.66618 .45652 L
.66622 .45653 L
Mistroke
.66626 .45654 L
.6663 .45656 L
.66635 .45657 L
.66639 .45658 L
.66643 .45659 L
.6665 .45661 L
.66655 .45663 L
.66659 .45664 L
.66663 .45665 L
.66668 .45667 L
.66677 .45669 L
.66681 .45671 L
.66685 .45672 L
.66689 .45673 L
.66694 .45674 L
.66697 .45676 L
.66701 .45677 L
.66709 .45679 L
.66716 .45681 L
.6672 .45682 L
.66724 .45684 L
.66731 .45686 L
.66736 .45687 L
.6674 .45689 L
.66749 .45691 L
.66753 .45693 L
.66758 .45694 L
.66762 .45695 L
.66767 .45697 L
.66771 .45698 L
.66775 .45699 L
.66783 .45702 L
.66788 .45703 L
.66793 .45705 L
.66797 .45706 L
.66802 .45708 L
.66807 .45709 L
.66812 .45711 L
.6682 .45713 L
.66825 .45715 L
.66829 .45716 L
.66833 .45717 L
.66837 .45719 L
.66841 .4572 L
.66845 .45721 L
.66853 .45723 L
.66857 .45725 L
.66861 .45726 L
.66865 .45727 L
.6687 .45729 L
Mistroke
.66875 .4573 L
.66879 .45732 L
.66887 .45734 L
.66891 .45736 L
.66895 .45737 L
.669 .45738 L
.66904 .4574 L
.66913 .45742 L
.66917 .45744 L
.66921 .45745 L
.66926 .45747 L
.66931 .45748 L
.6694 .45751 L
.66944 .45753 L
.66949 .45754 L
.66954 .45756 L
.66959 .45757 L
.66968 .4576 L
.66973 .45762 L
.66978 .45763 L
.66982 .45765 L
.66987 .45766 L
.66996 .45769 L
.67 .45771 L
.67006 .45772 L
.67011 .45774 L
.67016 .45776 L
.67025 .45779 L
.67031 .45781 L
.67035 .45782 L
.67043 .45785 L
.67047 .45786 L
.67051 .45787 L
.67058 .4579 L
.67061 .45791 L
.67065 .45792 L
.67072 .45794 L
.6708 .45797 L
.67086 .45799 L
.67093 .45801 L
.67096 .45802 L
.671 .45804 L
.67108 .45806 L
.67112 .45807 L
.67116 .45809 L
.6712 .4581 L
.67124 .45811 L
.67131 .45814 L
.67138 .45816 L
.67146 .45819 L
Mistroke
.6715 .4582 L
.67154 .45821 L
.6716 .45824 L
.67168 .45826 L
.67175 .45829 L
.67181 .45831 L
.67187 .45833 L
.67193 .45835 L
.672 .45837 L
.67206 .45839 L
.67213 .45842 L
.6722 .45844 L
.67227 .45846 L
.67233 .45849 L
.67241 .45851 L
.67245 .45853 L
.67248 .45854 L
.67256 .45856 L
.67262 .45859 L
.6727 .45861 L
.67277 .45864 L
.67284 .45866 L
.67288 .45868 L
.67292 .45869 L
.673 .45872 L
.67304 .45873 L
.67308 .45875 L
.67316 .45877 L
.6732 .45879 L
.67324 .4588 L
.67331 .45883 L
.67337 .45885 L
.67345 .45887 L
.67348 .45889 L
.67352 .4589 L
.67359 .45893 L
.67366 .45895 L
.67373 .45898 L
.6738 .459 L
.67387 .45903 L
.67391 .45904 L
.67396 .45906 L
.67399 .45907 L
.67403 .45908 L
.6741 .45911 L
.67415 .45912 L
.67419 .45914 L
.67423 .45915 L
.67428 .45917 L
.67432 .45919 L
Mistroke
.67436 .4592 L
.67444 .45923 L
.67448 .45924 L
.67452 .45926 L
.67459 .45928 L
.67463 .4593 L
.67467 .45931 L
.67471 .45933 L
.67475 .45934 L
.67483 .45937 L
.67489 .45939 L
.67497 .45942 L
.67501 .45944 L
.67505 .45945 L
.67512 .45948 L
.67519 .4595 L
.67525 .45953 L
.67532 .45955 L
.67539 .45958 L
.67544 .45959 L
.67547 .45961 L
.67555 .45964 L
.67562 .45966 L
.67568 .45969 L
.67575 .45971 L
.67581 .45973 L
.67587 .45976 L
.67594 .45978 L
.67601 .45981 L
.67608 .45984 L
.67615 .45986 L
.67622 .45989 L
.67626 .4599 L
.6763 .45992 L
.67638 .45995 L
.67644 .45997 L
.67648 .45999 L
.67653 .46 L
.6766 .46003 L
.67664 .46005 L
.67668 .46006 L
.67672 .46008 L
.67677 .4601 L
.67681 .46011 L
.67684 .46012 L
.67691 .46015 L
.67698 .46018 L
.67705 .4602 L
.67712 .46023 L
.67719 .46026 L
Mistroke
.67723 .46027 L
.67727 .46029 L
.67731 .4603 L
.67735 .46032 L
.67742 .46035 L
.67749 .46037 L
.67756 .4604 L
.6776 .46042 L
.67764 .46043 L
.6777 .46046 L
.67777 .46048 L
.67784 .46051 L
.67791 .46054 L
.67795 .46055 L
.67799 .46057 L
.67803 .46058 L
.67806 .4606 L
.67814 .46063 L
.6782 .46066 L
.67825 .46067 L
.67829 .46069 L
.67833 .4607 L
.67837 .46072 L
.67841 .46074 L
.67845 .46075 L
.67852 .46078 L
.67856 .4608 L
.6786 .46081 L
.67864 .46083 L
.67868 .46084 L
.67872 .46086 L
.67875 .46088 L
.67882 .4609 L
.67887 .46092 L
.67891 .46094 L
.67895 .46095 L
.67899 .46097 L
.67903 .46099 L
.67907 .461 L
.67911 .46102 L
.67915 .46103 L
.67919 .46105 L
.67923 .46107 L
.6793 .46109 L
.67936 .46112 L
.67943 .46115 L
.67951 .46118 L
.67958 .46121 L
.67962 .46122 L
.67965 .46124 L
Mistroke
.67969 .46126 L
.67973 .46127 L
.6798 .4613 L
.67985 .46132 L
.67992 .46135 L
.67999 .46138 L
.68004 .4614 L
.68011 .46143 L
.68017 .46145 L
.68022 .46147 L
.68029 .4615 L
.68035 .46153 L
.68042 .46156 L
.6805 .46159 L
.68056 .46162 L
.68062 .46164 L
.68069 .46167 L
.68076 .4617 L
.68083 .46173 L
.68089 .46176 L
.68097 .46179 L
.68101 .46181 L
.68105 .46182 L
.68111 .46185 L
.68119 .46188 L
.68126 .46191 L
.68132 .46194 L
.68138 .46196 L
.68144 .46199 L
.6815 .46202 L
.68157 .46204 L
.68164 .46208 L
.68171 .4621 L
.68177 .46213 L
.68183 .46216 L
.68189 .46218 L
.68195 .46221 L
.68201 .46223 L
.68206 .46226 L
.68212 .46228 L
.68218 .46231 L
.68225 .46234 L
.68231 .46237 L
.68238 .4624 L
.68244 .46242 L
.6825 .46245 L
.68256 .46248 L
.68263 .46251 L
.6827 .46254 L
.68276 .46256 L
Mistroke
.68282 .46259 L
.68288 .46262 L
.68294 .46264 L
.68299 .46267 L
.68305 .46269 L
.68311 .46272 L
.68317 .46275 L
.68323 .46277 L
.68329 .4628 L
.68335 .46283 L
.6834 .46285 L
.68346 .46288 L
.68352 .4629 L
.68357 .46293 L
.68362 .46295 L
.68367 .46297 L
.68373 .463 L
.68379 .46303 L
.68387 .46306 L
.6839 .46308 L
.68394 .4631 L
.68401 .46313 L
.68408 .46316 L
.68415 .46319 L
.68421 .46322 L
.68427 .46325 L
.68431 .46327 L
.68435 .46328 L
.68442 .46332 L
.68446 .46333 L
.6845 .46335 L
.68454 .46337 L
.68458 .46339 L
.68465 .46342 L
.68472 .46345 L
.68479 .46349 L
.68483 .4635 L
.68486 .46352 L
.68493 .46355 L
.685 .46358 L
.68506 .46361 L
.68512 .46364 L
.6852 .46368 L
.68524 .46369 L
.68527 .46371 L
.68534 .46374 L
.68541 .46378 L
.68547 .46381 L
.68554 .46384 L
.6856 .46387 L
Mistroke
.68565 .46389 L
.68572 .46392 L
.68579 .46396 L
.68586 .46399 L
.68592 .46402 L
.68598 .46405 L
.68604 .46408 L
.6861 .46411 L
.68616 .46413 L
.68621 .46416 L
.68626 .46418 L
.68632 .46421 L
.68638 .46424 L
.68644 .46427 L
.68651 .46431 L
.68658 .46434 L
.68664 .46437 L
.68669 .46439 L
.68675 .46442 L
.68682 .46445 L
.68688 .46448 L
.68694 .46451 L
.68701 .46455 L
.68707 .46458 L
.68713 .46461 L
.68718 .46463 L
.68724 .46466 L
.6873 .46469 L
.68736 .46472 L
.68743 .46475 L
.68749 .46478 L
.68755 .46481 L
.68761 .46484 L
.68767 .46488 L
.68775 .46491 L
.68781 .46495 L
.68787 .46498 L
.68793 .465 L
.68799 .46503 L
.68804 .46506 L
.68809 .46509 L
.68815 .46512 L
.68822 .46515 L
.68828 .46518 L
.68833 .46521 L
.68839 .46524 L
.68844 .46527 L
.6885 .4653 L
.68857 .46533 L
.68863 .46536 L
Mistroke
.6887 .46539 L
.68875 .46542 L
.68882 .46546 L
.68887 .46549 L
.68893 .46551 L
.68898 .46554 L
.68903 .46557 L
.68909 .4656 L
.68915 .46563 L
.6892 .46566 L
.68926 .46569 L
.68932 .46571 L
.68936 .46574 L
.68942 .46577 L
.68948 .4658 L
.68952 .46582 L
.68957 .46585 L
.68962 .46587 L
.68967 .4659 L
.68974 .46593 L
.68981 .46597 L
.68986 .466 L
.68993 .46603 L
.68999 .46607 L
.69006 .46611 L
.69013 .46614 L
.69019 .46618 L
.69026 .46621 L
.69031 .46624 L
.69038 .46627 L
.69045 .46631 L
.6905 .46634 L
.69056 .46637 L
.69062 .4664 L
.69068 .46644 L
.69074 .46647 L
.69079 .46649 L
.69085 .46653 L
.69091 .46656 L
.69098 .46659 L
.69104 .46663 L
.69109 .46666 L
.69115 .46669 L
.69121 .46672 L
.69126 .46675 L
.69132 .46678 L
.69138 .46681 L
.69143 .46684 L
.69148 .46687 L
.69153 .4669 L
Mistroke
.69158 .46693 L
.69164 .46696 L
.69169 .46699 L
.69175 .46702 L
.6918 .46704 L
.69184 .46707 L
.69189 .4671 L
.69195 .46713 L
.69199 .46715 L
.69205 .46718 L
.6921 .46721 L
.69215 .46724 L
.69221 .46727 L
.69226 .4673 L
.69232 .46734 L
.69237 .46736 L
.69243 .4674 L
.69248 .46743 L
.69253 .46745 L
.69258 .46748 L
.69262 .4675 L
.69268 .46753 L
.69273 .46757 L
.69278 .4676 L
.69283 .46762 L
.69288 .46765 L
.69292 .46768 L
.69298 .46771 L
.69302 .46773 L
.69311 .46778 L
.69316 .46781 L
.69321 .46784 L
.69326 .46787 L
.69332 .4679 L
.69337 .46793 L
.69343 .46797 L
.69349 .468 L
.69354 .46802 L
.69359 .46806 L
.69365 .46809 L
.6937 .46812 L
.69375 .46815 L
.69381 .46818 L
.69386 .46821 L
.69391 .46824 L
.69395 .46826 L
.694 .46829 L
.69405 .46832 L
.6941 .46835 L
.69416 .46839 L
Mistroke
.69422 .46842 L
.69427 .46845 L
.69432 .46848 L
.69437 .46851 L
.69443 .46854 L
.69448 .46857 L
.69453 .46861 L
.69459 .46864 L
.69465 .46868 L
.6947 .46871 L
.69476 .46874 L
.69482 .46878 L
.69489 .46882 L
.69494 .46885 L
.695 .46889 L
.69506 .46892 L
.69512 .46895 L
.69517 .46899 L
.69524 .46903 L
.69531 .46907 L
.69537 .46911 L
.69543 .46914 L
.69548 .46918 L
.69553 .46921 L
.69559 .46924 L
.69565 .46928 L
.69571 .46932 L
.69577 .46935 L
.69583 .46938 L
.69589 .46942 L
.69594 .46945 L
.69599 .46948 L
.69604 .46952 L
.6961 .46955 L
.69616 .46959 L
.69622 .46963 L
.69627 .46966 L
.69633 .46969 L
.69638 .46972 L
.69643 .46976 L
.69649 .46979 L
.69654 .46982 L
.69659 .46985 L
.69664 .46988 L
.69668 .46991 L
.69674 .46995 L
.6968 .46998 L
.69686 .47002 L
.69691 .47005 L
.69696 .47009 L
Mistroke
.69702 .47013 L
.69709 .47017 L
.69714 .4702 L
.69721 .47024 L
.69726 .47028 L
.69731 .47031 L
.69737 .47035 L
.69743 .47038 L
.69748 .47042 L
.69753 .47045 L
.69758 .47048 L
.69764 .47052 L
.69771 .47056 L
.69777 .4706 L
.69783 .47064 L
.69791 .47069 L
.69797 .47073 L
.69802 .47076 L
.69809 .4708 L
.69815 .47085 L
.69821 .47089 L
.69828 .47093 L
.69834 .47097 L
.6984 .471 L
.69845 .47104 L
.69851 .47107 L
.69856 .47111 L
.69862 .47115 L
.69867 .47118 L
.69872 .47122 L
.69877 .47125 L
.69881 .47128 L
.69887 .47131 L
.69893 .47135 L
.69898 .47139 L
.69903 .47142 L
.69908 .47145 L
.69913 .47148 L
.69918 .47152 L
.69924 .47156 L
.6993 .4716 L
.69935 .47163 L
.6994 .47167 L
.69946 .4717 L
.69951 .47174 L
.69956 .47177 L
.6996 .4718 L
.69965 .47183 L
.6997 .47187 L
.69975 .4719 L
Mistroke
.6998 .47193 L
.69985 .47197 L
.6999 .472 L
.69995 .47203 L
.7 .47207 L
.70004 .4721 L
.70013 .47215 L
.70018 .47219 L
.70022 .47222 L
.70027 .47225 L
.70032 .47228 L
.70037 .47232 L
.70042 .47236 L
.70047 .47239 L
.70051 .47242 L
.70056 .47245 L
.70061 .47249 L
.70067 .47253 L
.70073 .47257 L
.70079 .47261 L
.70084 .47264 L
.70089 .47268 L
.70094 .47271 L
.701 .47275 L
.70105 .47278 L
.7011 .47282 L
.70116 .47286 L
.70121 .4729 L
.70127 .47294 L
.70131 .47297 L
.70136 .47301 L
.70142 .47304 L
.70147 .47308 L
.70152 .47312 L
.70158 .47316 L
.70163 .47319 L
.70168 .47323 L
.70174 .47327 L
.7018 .47331 L
.70186 .47336 L
.70191 .47339 L
.70197 .47343 L
.70203 .47347 L
.70208 .47351 L
.70213 .47355 L
.70219 .47359 L
.70225 .47364 L
.70231 .47368 L
.70237 .47372 L
.70243 .47377 L
Mistroke
.70249 .47381 L
.70255 .47385 L
.7026 .47389 L
.70267 .47393 L
.70273 .47398 L
.70279 .47402 L
.70285 .47407 L
.70291 .47411 L
.70296 .47415 L
.70301 .47418 L
.70307 .47422 L
.70312 .47427 L
.70318 .47431 L
.70324 .47435 L
.70329 .47439 L
.70335 .47443 L
.7034 .47447 L
.70345 .47451 L
.70351 .47455 L
.70356 .47458 L
.7036 .47462 L
.70365 .47466 L
.70371 .4747 L
.70377 .47474 L
.70384 .47479 L
.70389 .47483 L
.70396 .47488 L
.70402 .47493 L
.70409 .47499 L
.70416 .47504 L
.70422 .47508 L
.70429 .47513 L
.70434 .47517 L
.70441 .47522 L
.70447 .47527 L
.70453 .47531 L
.70459 .47536 L
.70465 .4754 L
.70471 .47545 L
.70477 .47549 L
.70482 .47553 L
.70488 .47558 L
.70494 .47563 L
.705 .47567 L
.70507 .47572 L
.70512 .47576 L
.70518 .47581 L
.70523 .47585 L
.70529 .47589 L
.70535 .47594 L
Mistroke
.7054 .47598 L
.70546 .47602 L
.7055 .47606 L
.70556 .4761 L
.70561 .47614 L
.70566 .47618 L
.70572 .47623 L
.70578 .47627 L
.70583 .47631 L
.70587 .47635 L
.70592 .47639 L
.70597 .47643 L
.70602 .47646 L
.70608 .47651 L
.70613 .47655 L
.70618 .47659 L
.70623 .47663 L
.70629 .47667 L
.70635 .47672 L
.7064 .47676 L
.70645 .47681 L
.7065 .47685 L
.70656 .47689 L
.7066 .47692 L
.70665 .47696 L
.7067 .477 L
.70676 .47705 L
.70681 .47709 L
.70686 .47713 L
.7069 .47717 L
.70695 .4772 L
.707 .47724 L
.70705 .47728 L
.70714 .47735 L
.70718 .47739 L
.70723 .47743 L
.70729 .47748 L
.70734 .47752 L
.7074 .47756 L
.70746 .47761 L
.70751 .47766 L
.70757 .47771 L
.70762 .47775 L
.70767 .47779 L
.70772 .47783 L
.70777 .47787 L
.70783 .47792 L
.70788 .47796 L
.70793 .478 L
.70797 .47804 L
Mistroke
.70802 .47808 L
.70807 .47812 L
.70812 .47816 L
.70818 .47821 L
.70824 .47826 L
.70829 .4783 L
.70835 .47835 L
.70839 .47839 L
.70845 .47843 L
.7085 .47848 L
.70856 .47852 L
.70862 .47857 L
.70867 .47862 L
.70872 .47866 L
.70878 .47871 L
.70885 .47877 L
.70891 .47882 L
.70896 .47887 L
.70902 .47891 L
.70907 .47895 L
.70912 .479 L
.70918 .47905 L
.70923 .47909 L
.70927 .47913 L
.70932 .47917 L
.70938 .47922 L
.70942 .47926 L
.70947 .4793 L
.70952 .47934 L
.70957 .47938 L
.70966 .47946 L
.70971 .4795 L
.70975 .47954 L
.7098 .47958 L
.70986 .47963 L
.70991 .47967 L
.70996 .47972 L
.71001 .47977 L
.71007 .47982 L
.71013 .47987 L
.71018 .47991 L
.71023 .47996 L
.71028 .48 L
.71033 .48004 L
.71039 .48009 L
.71044 .48014 L
.71049 .48018 L
.71054 .48022 L
.71059 .48027 L
.71063 .48031 L
Mistroke
.71068 .48035 L
.71072 .48039 L
.71077 .48043 L
.71082 .48047 L
.71087 .48052 L
.71092 .48056 L
.71097 .4806 L
.71101 .48064 L
.71106 .48069 L
.71111 .48073 L
.71116 .48078 L
.71121 .48082 L
.71127 .48087 L
.71131 .48091 L
.71136 .48096 L
.71141 .481 L
.71146 .48104 L
.71154 .48112 L
.71159 .48116 L
.71163 .4812 L
.71168 .48125 L
.71173 .48129 L
.71177 .48133 L
.71182 .48137 L
.71189 .48144 L
.71194 .48148 L
.71199 .48153 L
.71207 .4816 L
.71213 .48165 L
.71218 .4817 L
.71224 .48175 L
.71229 .4818 L
.71233 .48184 L
.71238 .48188 L
.71243 .48193 L
.71248 .48197 L
.71253 .48203 L
.71258 .48207 L
.71263 .48212 L
.71268 .48217 L
.71274 .48222 L
.7128 .48227 L
.71285 .48232 L
.7129 .48237 L
.71295 .48242 L
.713 .48246 L
.71305 .4825 L
.71309 .48255 L
.71314 .4826 L
.7132 .48265 L
Mistroke
.71325 .4827 L
.7133 .48274 L
.71334 .48278 L
.71339 .48283 L
.71344 .48287 L
.71349 .48292 L
.71357 .483 L
.71362 .48305 L
.71366 .48309 L
.71372 .48314 L
.71377 .48319 L
.71383 .48324 L
.71389 .4833 L
.71394 .48335 L
.71399 .4834 L
.71404 .48345 L
.7141 .4835 L
.71414 .48355 L
.7142 .4836 L
.71425 .48365 L
.7143 .4837 L
.71435 .48375 L
.71439 .48379 L
.71444 .48384 L
.71449 .48388 L
.71454 .48393 L
.7146 .48399 L
.71465 .48404 L
.71471 .4841 L
.71476 .48415 L
.71481 .48419 L
.71486 .48424 L
.71491 .48429 L
.71497 .48435 L
.71502 .48441 L
.71508 .48446 L
.71513 .48451 L
.71519 .48457 L
.71525 .48463 L
.71531 .48469 L
.71536 .48474 L
.71541 .48479 L
.71547 .48484 L
.71552 .48489 L
.71556 .48494 L
.71565 .48503 L
.71569 .48507 L
.71574 .48512 L
.71579 .48517 L
.71584 .48522 L
Mistroke
.7159 .48527 L
.71595 .48533 L
.716 .48538 L
.71604 .48542 L
.71609 .48547 L
.71614 .48553 L
.71619 .48558 L
.71624 .48562 L
.71629 .48567 L
.71634 .48572 L
.71639 .48577 L
.71643 .48581 L
.71648 .48586 L
.71652 .48591 L
.71656 .48595 L
.71661 .486 L
.71669 .48608 L
.71673 .48613 L
.71678 .48618 L
.71687 .48626 L
.71691 .48631 L
.71697 .48637 L
.71701 .48641 L
.71706 .48646 L
.71711 .48652 L
.71716 .48657 L
.71722 .48663 L
.71726 .48668 L
.71731 .48673 L
.71736 .48677 L
.71741 .48682 L
.71746 .48688 L
.71754 .48697 L
.71759 .48702 L
.71764 .48707 L
.71769 .48712 L
.71775 .48718 L
.7178 .48724 L
.71785 .48729 L
.7179 .48735 L
.71795 .48739 L
.718 .48745 L
.71805 .4875 L
.7181 .48756 L
.71816 .48762 L
.7182 .48767 L
.71826 .48772 L
.71831 .48778 L
.71837 .48785 L
.71843 .48791 L
Mistroke
.71848 .48797 L
.71853 .48802 L
.71858 .48807 L
.71863 .48812 L
.71868 .48818 L
.71874 .48824 L
.71879 .4883 L
.71884 .48835 L
.71889 .48841 L
.71894 .48847 L
.71899 .48852 L
.71904 .48857 L
.71909 .48863 L
.71918 .48872 L
.71923 .48878 L
.71927 .48883 L
.71932 .48888 L
.71938 .48894 L
.71943 .48899 L
.71947 .48904 L
.71953 .4891 L
.71958 .48917 L
.71964 .48923 L
.71969 .48928 L
.71975 .48935 L
.7198 .48941 L
.71985 .48946 L
.7199 .48952 L
.71996 .48958 L
.72002 .48965 L
.72007 .48971 L
.72013 .48977 L
.72018 .48983 L
.72023 .48989 L
.72028 .48994 L
.72033 .49 L
.72042 .4901 L
.72046 .49015 L
.72051 .4902 L
.72056 .49026 L
.72061 .49032 L
.72067 .49038 L
.72072 .49044 L
.72077 .4905 L
.72082 .49055 L
.72087 .49061 L
.72092 .49067 L
.72097 .49073 L
.72102 .49078 L
.72107 .49084 L
Mistroke
.72112 .4909 L
.72117 .49095 L
.72121 .491 L
.72126 .49105 L
.72131 .49111 L
.72135 .49116 L
.7214 .49122 L
.72144 .49127 L
.72149 .49132 L
.72153 .49137 L
.72158 .49142 L
.72166 .49152 L
.7217 .49157 L
.72175 .49162 L
.7218 .49168 L
.72185 .49174 L
.72193 .49184 L
.72198 .4919 L
.72203 .49195 L
.72207 .492 L
.72211 .49205 L
.72219 .49214 L
.72223 .49219 L
.72227 .49224 L
.72237 .49235 L
.72241 .49241 L
.72246 .49247 L
.72251 .49252 L
.72256 .49258 L
.72264 .49268 L
.72268 .49273 L
.72273 .49278 L
.72278 .49285 L
.72283 .4929 L
.72288 .49296 L
.72293 .49303 L
.72302 .49313 L
.72307 .49319 L
.72312 .49325 L
.7232 .49334 L
.72324 .4934 L
.72328 .49344 L
.72335 .49353 L
.72339 .49358 L
.72343 .49363 L
.72351 .49373 L
.72356 .49379 L
.72361 .49384 L
.72365 .49389 L
.72369 .49395 L
Mistroke
.72377 .49404 L
.72381 .4941 L
.72385 .49415 L
.7239 .4942 L
.72395 .49426 L
.72403 .49437 L
.72408 .49443 L
.72413 .49449 L
.72417 .49453 L
.72421 .49459 L
.72429 .49468 L
.72433 .49474 L
.72438 .49479 L
.72446 .49489 L
.7245 .49495 L
.72455 .49501 L
.72463 .49511 L
.72469 .49518 L
.72473 .49524 L
.72483 .49536 L
.72487 .49541 L
.72492 .49548 L
.72501 .49559 L
.72506 .49565 L
.72511 .49571 L
.72516 .49577 L
.72521 .49584 L
.72526 .4959 L
.7253 .49596 L
.72535 .49602 L
.7254 .49608 L
.72548 .49619 L
.72553 .49625 L
.72557 .4963 L
.72561 .49635 L
.72565 .49641 L
.72573 .4965 L
.72577 .49656 L
.72581 .49661 L
.7259 .49672 L
.72595 .49679 L
.72599 .49685 L
.72604 .49691 L
.7261 .49698 L
.72614 .49704 L
.72619 .4971 L
.72624 .49716 L
.72629 .49723 L
.72638 .49735 L
.72642 .4974 L
Mistroke
.72647 .49747 L
.72651 .49752 L
.72656 .49758 L
.72664 .49769 L
.72669 .49776 L
.72674 .49782 L
.7268 .4979 L
.72685 .49797 L
.7269 .49804 L
.72695 .4981 L
.727 .49816 L
.72705 .49824 L
.7271 .4983 L
.72715 .49837 L
.7272 .49844 L
.72725 .4985 L
.72733 .49861 L
.72738 .49868 L
.72743 .49874 L
.72747 .4988 L
.72752 .49886 L
.7276 .49897 L
.72765 .49904 L
.7277 .49911 L
.72779 .49923 L
.72784 .4993 L
.72789 .49936 L
.72798 .49949 L
.72803 .49955 L
.72808 .49962 L
.72813 .49969 L
.72818 .49976 L
.72827 .49988 L
.72831 .49994 L
.72836 .5 L
.72845 .50012 L
.7285 .50019 L
.72854 .50026 L
.72862 .50037 L
.72867 .50043 L
.72871 .50049 L
.72879 .5006 L
.72883 .50065 L
.72887 .50071 L
.72892 .50078 L
.72897 .50085 L
.72903 .50093 L
.72907 .50099 L
.72916 .50112 L
.72921 .50118 L
Mistroke
.72926 .50125 L
.72931 .50132 L
.72936 .50139 L
.72941 .50147 L
.72947 .50155 L
.72952 .50162 L
.72956 .50168 L
.72961 .50175 L
.72967 .50183 L
.72972 .5019 L
.72976 .50196 L
.72981 .50204 L
.72987 .50211 L
.72996 .50224 L
.73001 .50231 L
.73005 .50238 L
.7301 .50245 L
.73014 .50251 L
.73022 .50262 L
.73027 .50269 L
.73031 .50275 L
.73036 .50281 L
.73041 .50288 L
.73045 .50295 L
.7305 .50301 L
.73058 .50313 L
.73062 .5032 L
.73067 .50327 L
.73076 .50341 L
.73081 .50347 L
.73086 .50354 L
.7309 .5036 L
.73094 .50366 L
.73098 .50372 L
.73102 .50379 L
.7311 .5039 L
.73114 .50396 L
.73118 .50402 L
.73126 .50413 L
.7313 .50419 L
.73134 .50425 L
.73138 .50431 L
.73143 .50438 L
.73146 .50443 L
.7315 .50449 L
.73157 .50459 L
.73161 .50465 L
.73165 .50471 L
.73173 .50483 L
.73177 .50489 L
Mistroke
.73182 .50495 L
.73189 .50507 L
.73194 .50514 L
.73198 .5052 L
.73203 .50527 L
.73207 .50534 L
.73211 .5054 L
.73215 .50546 L
.73223 .50557 L
.73227 .50564 L
.73231 .5057 L
.73239 .50582 L
.73244 .50589 L
.73248 .50595 L
.73252 .50602 L
.73257 .50609 L
.73262 .50616 L
.73266 .50622 L
.73274 .50635 L
.73278 .50642 L
.73283 .50648 L
.73288 .50656 L
.73292 .50663 L
.73302 .50677 L
.73307 .50685 L
.73312 .50693 L
.73316 .50699 L
.7332 .50705 L
.73327 .50716 L
.73331 .50723 L
.73335 .50729 L
.73339 .50735 L
.73344 .50742 L
.73351 .50754 L
.73358 .50764 L
.73365 .50776 L
.7337 .50782 L
.73373 .50788 L
.7338 .50799 L
.73386 .50809 L
.73393 .50819 L
.734 .50831 L
.73408 .50842 L
.73415 .50854 L
.73422 .50865 L
.73429 .50876 L
.73433 .50882 L
.73437 .50889 L
.73441 .50895 L
.73445 .50902 L
Mistroke
.73453 .50914 L
.7346 .50925 L
.73464 .50931 L
.73468 .50938 L
.73476 .5095 L
.73481 .50958 L
.73485 .50965 L
.73489 .50972 L
.73493 .50978 L
.735 .5099 L
.73508 .51002 L
.73515 .51013 L
.73522 .51024 L
.73529 .51036 L
.73537 .51048 L
.73544 .51061 L
.73548 .51067 L
.73553 .51074 L
.7356 .51086 L
.73567 .51098 L
.73575 .5111 L
.73581 .51121 L
.73587 .51131 L
.73594 .51142 L
.73601 .51153 L
.73607 .51164 L
.73615 .51176 L
.73622 .51188 L
.73629 .51199 L
.73636 .5121 L
.73643 .51223 L
.73647 .5123 L
.73651 .51237 L
.73659 .51249 L
.73666 .51261 L
.73672 .51271 L
.73679 .51282 L
.73686 .51294 L
.73692 .51305 L
.73698 .51315 L
.73704 .51324 L
.7371 .51335 L
.73716 .51345 L
.73723 .51356 L
.7373 .51368 L
.73736 .51378 L
.73742 .51389 L
.73749 .51401 L
.73757 .51414 L
.73764 .51426 L
Mistroke
.7377 .51437 L
.73778 .5145 L
.73785 .51463 L
.73792 .51474 L
.73799 .51486 L
.73806 .51499 L
.73814 .51513 L
.73821 .51525 L
.73829 .51538 L
.73836 .5155 L
.73842 .51561 L
.7385 .51574 L
.73857 .51586 L
.73863 .51598 L
.73869 .51608 L
.73876 .5162 L
.73883 .51632 L
.73891 .51646 L
.73899 .5166 L
.73907 .51674 L
.73911 .51682 L
.73916 .5169 L
.73924 .51703 L
.73932 .51718 L
.7394 .51732 L
.73947 .51745 L
.73954 .51758 L
.73961 .5177 L
.73969 .51783 L
.73977 .51797 L
.73983 .5181 L
.7399 .51821 L
.73997 .51834 L
.74005 .51847 L
.74012 .51861 L
.7402 .51875 L
.74027 .51888 L
.74034 .51901 L
.74042 .51915 L
.74051 .51931 L
.74059 .51945 L
.74066 .51959 L
.74074 .51973 L
.74081 .51986 L
.74089 .52001 L
.74098 .52016 L
.74107 .52032 L
.74115 .52048 L
.74123 .52062 L
.74131 .52077 L
Mistroke
.74139 .52091 L
.74146 .52105 L
.74152 .52117 L
.7416 .5213 L
.74167 .52144 L
.74176 .5216 L
.74183 .52173 L
.7419 .52187 L
.74197 .522 L
.74204 .52212 L
.74211 .52227 L
.74219 .5224 L
.74225 .52252 L
.74231 .52264 L
.74238 .52276 L
.74245 .52289 L
.74252 .52304 L
.7426 .52319 L
.74268 .52333 L
.74275 .52346 L
.74283 .52361 L
.74292 .52378 L
.743 .52394 L
.74308 .52408 L
.74315 .52422 L
.74322 .52435 L
.74329 .52449 L
.74337 .52464 L
.74345 .52479 L
.74353 .52494 L
.7436 .52507 L
.74367 .52522 L
.74374 .52535 L
.7438 .52548 L
.74387 .52561 L
.74395 .52576 L
.74403 .5259 L
.74411 .52606 L
.74417 .52619 L
.74425 .52633 L
.74432 .52647 L
.74438 .52659 L
.74445 .52672 L
.74452 .52687 L
.74459 .527 L
.74465 .52712 L
.74478 .52737 L
.74485 .52751 L
.74492 .52764 L
.745 .5278 L
Mistroke
.74507 .52794 L
.7452 .5282 L
.74528 .52835 L
.74535 .52849 L
.74543 .52865 L
.7455 .5288 L
.74565 .52909 L
.74573 .52925 L
.7458 .52939 L
.74588 .52955 L
.74597 .52973 L
.74605 .52988 L
.74612 .53002 L
.74619 .53018 L
.74628 .53034 L
.74635 .5305 L
.74642 .53064 L
.74656 .53092 L
.74664 .53108 L
.74671 .53122 L
.7468 .5314 L
.74688 .53156 L
.74704 .53188 L
.74712 .53206 L
.74721 .53224 L
.74728 .53237 L
.74735 .53252 L
.74749 .53279 L
.74761 .53305 L
.74773 .53329 L
.74786 .53356 L
.748 .53384 L
.74815 .53414 L
.74828 .53443 L
.74841 .53469 L
.74853 .53494 L
.74864 .53516 L
.74876 .53541 L
.74889 .53567 L
.74902 .53595 L
.74915 .53622 L
.74926 .53646 L
.7494 .53675 L
.74955 .53706 L
.74968 .53732 L
.74981 .5376 L
.74996 .53791 L
.75012 .53824 L
.7504 .53884 L
.75054 .53912 L
Mistroke
.75068 .53944 L
.75095 .54 L
.75121 .54055 L
.75145 .54106 L
.75258 .54346 L
.7532 .54477 L
.75378 .546 L
.75404 .54655 L
.75432 .54714 L
.75448 .54747 L
.75462 .54778 L
.75477 .54809 L
.7549 .54837 L
.75515 .5489 L
.75528 .54917 L
.75542 .54947 L
.75554 .54972 L
.75566 .54995 L
.75578 .55021 L
.75591 .55049 L
.75598 .55064 L
.75606 .55079 L
.75619 .55108 L
.75627 .55123 L
.75634 .55138 L
.75642 .55154 L
.7565 .55171 L
.75657 .55185 L
.75664 .552 L
.75677 .55226 L
.75684 .55241 L
.75691 .55257 L
.75699 .55271 L
.75705 .55285 L
.75712 .55298 L
.75718 .55312 L
.75724 .55324 L
.75731 .55337 L
.75738 .55351 L
.75745 .55366 L
.75752 .5538 L
.75758 .55393 L
.75764 .55406 L
.7577 .55418 L
.75777 .55431 L
.75784 .55444 L
.7579 .55458 L
.75796 .5547 L
.75802 .55481 L
.75808 .55493 L
Mistroke
.75815 .55507 L
.75819 .55515 L
.75823 .55523 L
.75826 .55529 L
.7583 .55537 L
.75836 .5555 L
.7584 .55557 L
.75844 .55566 L
.75848 .55574 L
.75852 .55582 L
.7586 .55596 L
.75864 .55604 L
.75867 .55611 L
.75871 .55619 L
.75875 .55626 L
.75878 .55633 L
.75882 .5564 L
.75886 .55649 L
.7589 .55657 L
.75894 .55664 L
.75898 .55671 L
.75902 .55679 L
.75905 .55686 L
.75909 .55693 L
.75913 .55702 L
.75918 .5571 L
.75922 .55718 L
.75926 .55725 L
.7593 .55733 L
.75933 .5574 L
.75937 .55747 L
.7594 .55753 L
.75943 .55759 L
.75946 .55765 L
.7595 .55773 L
.75954 .5578 L
.75958 .55787 L
.75962 .55794 L
.75965 .55801 L
.75969 .55808 L
.75972 .55814 L
.75976 .55822 L
.7598 .55829 L
.75984 .55836 L
.75987 .55843 L
.75991 .5585 L
.75994 .55856 L
.75997 .55862 L
.76001 .55868 L
.76004 .55875 L
Mistroke
.76008 .55882 L
.76012 .55889 L
.76015 .55895 L
.76018 .55901 L
.76021 .55907 L
.76024 .55913 L
.76028 .55919 L
.76031 .55925 L
.76034 .5593 L
.76037 .55936 L
.7604 .55942 L
.76043 .55948 L
.76046 .55953 L
.76049 .55958 L
.76052 .55963 L
.76055 .55969 L
.76058 .55975 L
.76061 .5598 L
.76064 .55985 L
.76066 .5599 L
.76069 .55995 L
.76072 .56 L
.76075 .56006 L
.76078 .5601 L
.7608 .56015 L
.76083 .5602 L
.76085 .56025 L
.76089 .56031 L
.76092 .56036 L
.76095 .56042 L
.76097 .56046 L
.76099 .56049 L
.76102 .56054 L
.76104 .56058 L
.76105 .5606 L
.76108 .56066 L
.76111 .56071 L
.76114 .56076 L
.76117 .56082 L
.7612 .56087 L
.76123 .56091 L
.76126 .56097 L
.76127 .561 L
.76129 .56103 L
.76132 .56108 L
.76135 .56113 L
.76137 .56116 L
.76138 .56119 L
.7614 .56121 L
.76141 .56124 L
Mistroke
.76144 .56129 L
.76147 .56134 L
.76149 .56137 L
.7615 .5614 L
.76152 .56143 L
.76154 .56145 L
.76155 .56148 L
.76157 .56151 L
.7616 .56156 L
.76161 .56159 L
.76163 .56161 L
.76165 .56166 L
.76167 .56168 L
.76168 .56171 L
.7617 .56174 L
.76171 .56176 L
.76174 .56181 L
.76175 .56183 L
.76177 .56186 L
.7618 .5619 L
.76181 .56193 L
.76183 .56195 L
.76184 .56198 L
.76186 .56201 L
.76189 .56206 L
.7619 .56209 L
.76192 .56211 L
.76194 .56214 L
.76195 .56217 L
.76198 .56222 L
.762 .56224 L
.76201 .56227 L
.76203 .5623 L
.76204 .56232 L
.76207 .56237 L
.76209 .5624 L
.76211 .56243 L
.76212 .56246 L
.76214 .56249 L
.76217 .56254 L
.76219 .56257 L
.7622 .56259 L
.76222 .56262 L
.76224 .56265 L
.76226 .56268 L
.76227 .56271 L
.76229 .56274 L
.76231 .56277 L
.76233 .5628 L
.76235 .56283 L
Mistroke
.76236 .56285 L
.76237 .56287 L
.7624 .56291 L
.76241 .56294 L
.76243 .56296 L
.76244 .56298 L
.76245 .56301 L
.76247 .56303 L
.76248 .56305 L
.7625 .56309 L
.76252 .56311 L
.76253 .56313 L
.76256 .56317 L
.76257 .5632 L
.76259 .56322 L
.7626 .56324 L
.76262 .56327 L
.76263 .56329 L
.76264 .56331 L
.76267 .56335 L
.76268 .56338 L
.7627 .5634 L
.76271 .56342 L
.76273 .56345 L
.76274 .56347 L
.76276 .5635 L
.76278 .56352 L
.76279 .56355 L
.76281 .56357 L
.76282 .5636 L
.76284 .56362 L
.76285 .56365 L
.76288 .56369 L
.76289 .56371 L
.76291 .56373 L
.76293 .56376 L
.76294 .56378 L
.76296 .56381 L
.76298 .56384 L
.76299 .56386 L
.763 .56388 L
.76302 .56391 L
.76304 .56393 L
.76305 .56396 L
.76307 .56398 L
.76308 .564 L
.7631 .56403 L
.76311 .56405 L
.76312 .56407 L
.76314 .56409 L
Mistroke
.76316 .56412 L
.76317 .56414 L
.76318 .56416 L
.7632 .56418 L
.76321 .5642 L
.76324 .56424 L
.76325 .56426 L
.76326 .56428 L
.76328 .5643 L
.76329 .56432 L
.7633 .56434 L
.76332 .56436 L
.76333 .56439 L
.76335 .56441 L
.76336 .56443 L
.76337 .56445 L
.76338 .56447 L
.7634 .56449 L
.76341 .56451 L
.76342 .56453 L
.76345 .56456 L
.76346 .56458 L
.76347 .5646 L
.76349 .56462 L
.7635 .56464 L
.76352 .56466 L
.76353 .56468 L
.76354 .5647 L
.76356 .56473 L
.76357 .56475 L
.76359 .56477 L
.7636 .56479 L
.76361 .56481 L
.76363 .56483 L
.76364 .56485 L
.76366 .56487 L
.76367 .56489 L
.76368 .56491 L
.7637 .56493 L
.76371 .56495 L
.76372 .56497 L
.76373 .56499 L
.76375 .565 L
.76376 .56502 L
.76377 .56504 L
.76379 .56506 L
.7638 .56508 L
.76381 .5651 L
.76383 .56512 L
.76384 .56514 L
Mistroke
.76385 .56516 L
.76387 .56518 L
.76388 .5652 L
.76389 .56522 L
.76391 .56524 L
.76392 .56526 L
.76393 .56527 L
.76395 .56529 L
.76396 .56531 L
.76397 .56533 L
.76398 .56535 L
.764 .56536 L
.76401 .56538 L
.76402 .5654 L
.76403 .56541 L
.76404 .56543 L
.76406 .56545 L
.76407 .56547 L
.76408 .56548 L
.76409 .5655 L
.7641 .56551 L
.76412 .56553 L
.76413 .56555 L
.76413 .56556 L
.76415 .56558 L
.76416 .5656 L
.76417 .56561 L
.76418 .56562 L
.7642 .56564 L
.76421 .56566 L
.76421 .56567 L
.76422 .56568 L
.76423 .56569 L
.76425 .56571 L
.76426 .56573 L
.76427 .56574 L
.76428 .56576 L
.76429 .56577 L
.7643 .56578 L
.76431 .56581 L
.76432 .56582 L
.76433 .56583 L
.76435 .56585 L
.76436 .56587 L
.76437 .56588 L
.76438 .5659 L
.76439 .56591 L
.7644 .56592 L
.76441 .56594 L
.76443 .56596 L
Mistroke
.76444 .56597 L
.76444 .56598 L
.76446 .566 L
.76447 .56601 L
.76448 .56602 L
.76448 .56604 L
.76449 .56605 L
.76451 .56607 L
.76452 .56608 L
.76453 .56609 L
.76454 .5661 L
.76455 .56612 L
.76456 .56613 L
.76456 .56614 L
.76458 .56616 L
.76459 .56617 L
.7646 .56618 L
.76461 .5662 L
.76462 .56621 L
.76463 .56623 L
.76464 .56625 L
.76465 .56626 L
.76466 .56627 L
.76468 .56629 L
.76469 .5663 L
.76469 .56631 L
.76471 .56634 L
.76472 .56635 L
.76473 .56636 L
.76474 .56637 L
.76475 .56638 L
.76476 .5664 L
.76477 .56642 L
.76478 .56643 L
.7648 .56645 L
.76481 .56646 L
.76481 .56647 L
.76483 .56649 L
.76484 .5665 L
.76484 .56651 L
.76486 .56653 L
.76487 .56654 L
.76487 .56655 L
.76488 .56656 L
.76489 .56657 L
.7649 .56658 L
.76491 .56659 L
.76493 .56661 L
.76494 .56662 L
.76494 .56664 L
Mistroke
.76495 .56665 L
.76496 .56666 L
.76497 .56667 L
.76498 .56668 L
.76499 .5667 L
.765 .56671 L
.76501 .56672 L
.76502 .56673 L
.76503 .56674 L
.76504 .56675 L
.76505 .56676 L
.76506 .56678 L
.76507 .56679 L
.76507 .5668 L
.76508 .56681 L
.76509 .56682 L
.7651 .56683 L
.76511 .56684 L
.76512 .56685 L
.76513 .56686 L
.76514 .56688 L
.76515 .56689 L
.76516 .5669 L
.76516 .56691 L
.76517 .56692 L
.76519 .56694 L
.7652 .56695 L
.7652 .56696 L
.76521 .56697 L
.76522 .56698 L
.76523 .56699 L
.76524 .567 L
.76525 .56701 L
.76525 .56702 L
.76526 .56703 L
.76527 .56704 L
.76528 .56704 L
.76529 .56705 L
.7653 .56707 L
.76531 .56708 L
.76531 .56709 L
.76532 .5671 L
.76533 .56711 L
.76534 .56712 L
.76535 .56713 L
.76536 .56714 L
.76536 .56715 L
.76537 .56716 L
.76539 .56718 L
.76539 .56719 L
Mistroke
.7654 .56719 L
.76541 .5672 L
.76542 .56721 L
.76543 .56722 L
.76544 .56723 L
.76544 .56724 L
.76545 .56725 L
.76546 .56726 L
.76547 .56727 L
.76548 .56728 L
.76548 .56729 L
.76549 .5673 L
.7655 .56731 L
.76551 .56732 L
.76552 .56733 L
.76553 .56734 L
.76554 .56735 L
.76555 .56736 L
.76555 .56737 L
.76556 .56738 L
.76557 .56739 L
.76558 .5674 L
.76558 .56741 L
.76559 .56742 L
.7656 .56742 L
.76561 .56743 L
.76562 .56744 L
.76562 .56745 L
.76563 .56746 L
.76564 .56747 L
.76565 .56748 L
.76566 .56749 L
.76567 .5675 L
.76567 .56751 L
.76568 .56752 L
.76569 .56753 L
.7657 .56754 L
.76571 .56754 L
.76571 .56755 L
.76572 .56756 L
.76573 .56757 L
.76574 .56758 L
.76574 .56759 L
.76575 .56759 L
.76576 .5676 L
.76576 .56761 L
.76577 .56762 L
.76578 .56762 L
.76579 .56763 L
.76579 .56764 L
Mistroke
.7658 .56765 L
.76581 .56766 L
.76582 .56767 L
.76582 .56768 L
.76583 .56768 L
.76584 .56769 L
.76585 .5677 L
.76585 .56771 L
.76586 .56772 L
.76587 .56773 L
.76588 .56773 L
.76589 .56774 L
.7659 .56775 L
.7659 .56776 L
.76591 .56777 L
.76592 .56778 L
.76593 .56779 L
.76593 .56779 L
.76594 .5678 L
.76595 .56781 L
.76595 .56782 L
.76596 .56782 L
.76597 .56783 L
.76597 .56784 L
.76598 .56784 L
.76599 .56785 L
.76599 .56786 L
.766 .56787 L
.76601 .56788 L
.76602 .56788 L
.76602 .56789 L
.76603 .5679 L
.76604 .56791 L
.76605 .56792 L
.76605 .56792 L
.76606 .56793 L
.76607 .56794 L
.76607 .56794 L
.76608 .56795 L
.76609 .56796 L
.7661 .56797 L
.7661 .56797 L
.76611 .56798 L
.76612 .56799 L
.76613 .568 L
.76613 .56801 L
.76614 .56802 L
.76615 .56802 L
.76616 .56803 L
.76616 .56804 L
Mistroke
.76617 .56804 L
.76618 .56805 L
.76618 .56806 L
.76619 .56806 L
.7662 .56807 L
.7662 .56808 L
.76621 .56809 L
.76622 .56809 L
.76623 .5681 L
.76623 .56811 L
.76624 .56812 L
.76625 .56813 L
.76626 .56813 L
.76626 .56814 L
.76627 .56815 L
.76628 .56816 L
.76629 .56816 L
.76629 .56817 L
.7663 .56818 L
.76631 .56819 L
.76632 .56819 L
.76632 .5682 L
.76633 .56821 L
.76634 .56822 L
.76635 .56822 L
.76635 .56823 L
.76636 .56824 L
.76637 .56825 L
.76638 .56826 L
.76639 .56826 L
.76639 .56827 L
.7664 .56828 L
.76641 .56828 L
.76641 .56829 L
.76642 .5683 L
.76643 .5683 L
.76643 .56831 L
.76644 .56832 L
.76645 .56832 L
.76645 .56833 L
.76646 .56834 L
.76647 .56834 L
.76648 .56835 L
.76648 .56836 L
.76649 .56836 L
.7665 .56837 L
.7665 .56838 L
.76651 .56838 L
.76652 .56839 L
.76652 .5684 L
Mistroke
.76653 .5684 L
.76654 .56841 L
.76654 .56842 L
.76655 .56842 L
.76656 .56843 L
.76656 .56843 L
.76657 .56844 L
.76658 .56845 L
.76658 .56845 L
.76659 .56846 L
.7666 .56846 L
.7666 .56847 L
.76661 .56848 L
.76661 .56848 L
.76662 .56849 L
.76663 .56849 L
.76663 .5685 L
.76664 .56851 L
.76665 .56851 L
.76665 .56852 L
.76666 .56852 L
.76666 .56853 L
.76667 .56853 L
.76667 .56854 L
.76668 .56854 L
.76668 .56855 L
.76669 .56855 L
.76669 .56856 L
.7667 .56856 L
.76671 .56857 L
.76671 .56858 L
.76672 .56858 L
.76673 .56859 L
.76673 .56859 L
.76673 .56859 L
.76674 .5686 L
.76675 .56861 L
.76675 .56861 L
.76676 .56861 L
.76676 .56862 L
.76677 .56863 L
.76677 .56863 L
.76678 .56863 L
.76678 .56864 L
.76679 .56864 L
.76679 .56865 L
.7668 .56865 L
.76681 .56866 L
.76681 .56866 L
.76681 .56867 L
Mistroke
.76682 .56867 L
.76683 .56868 L
.76683 .56868 L
.76684 .56869 L
.76684 .56869 L
.76685 .5687 L
.76686 .5687 L
.76686 .56871 L
.76686 .56871 L
.76687 .56872 L
.76687 .56872 L
.76688 .56872 L
.76689 .56873 L
.76689 .56873 L
.76689 .56874 L
.7669 .56874 L
.76691 .56875 L
.76691 .56875 L
.76691 .56875 L
.76692 .56876 L
.76692 .56876 L
.76693 .56877 L
.76693 .56877 L
.76694 .56877 L
.76694 .56878 L
.76694 .56878 L
.76695 .56879 L
.76696 .56879 L
.76696 .5688 L
.76696 .5688 L
.76697 .5688 L
.76698 .56881 L
.76698 .56881 L
.76699 .56882 L
.76699 .56883 L
.767 .56883 L
.767 .56883 L
.76701 .56884 L
.76701 .56884 L
.76702 .56884 L
.76702 .56885 L
.76702 .56885 L
.76703 .56886 L
.76703 .56886 L
.76704 .56886 L
.76704 .56887 L
.76705 .56887 L
.76706 .56888 L
.76706 .56888 L
.76706 .56888 L
Mistroke
.76707 .56889 L
.76707 .56889 L
.76707 .56889 L
.76708 .5689 L
.76709 .5689 L
.76709 .5689 L
.76709 .56891 L
.7671 .56891 L
.7671 .56891 L
.7671 .56892 L
.76711 .56892 L
.76711 .56893 L
.76712 .56893 L
.76712 .56893 L
.76712 .56893 L
.76713 .56894 L
.76714 .56894 L
.76714 .56895 L
.76715 .56895 L
.76715 .56896 L
.76715 .56896 L
.76716 .56896 L
.76717 .56897 L
.76717 .56897 L
.76717 .56898 L
.76718 .56898 L
.76718 .56898 L
.76719 .56899 L
.76719 .56899 L
.7672 .56899 L
.7672 .569 L
.76721 .569 L
.76721 .569 L
.76722 .56901 L
.76722 .56901 L
.76722 .56901 L
.76723 .56902 L
.76723 .56902 L
.76724 .56903 L
.76725 .56903 L
.76725 .56904 L
.76725 .56904 L
.76726 .56904 L
.76726 .56904 L
.76727 .56905 L
.76727 .56905 L
.76728 .56906 L
.76728 .56906 L
.76728 .56906 L
.76729 .56907 L
Mistroke
.76729 .56907 L
.76729 .56907 L
.7673 .56908 L
.7673 .56908 L
.76731 .56908 L
.76731 .56909 L
.76731 .56909 L
.76732 .56909 L
.76733 .5691 L
.76733 .5691 L
.76733 .5691 L
.76734 .56911 L
.76734 .56911 L
.76735 .56911 L
.76735 .56912 L
.76736 .56912 L
.76736 .56912 L
.76737 .56913 L
.76737 .56913 L
.76737 .56913 L
.76738 .56914 L
.76738 .56914 L
.76739 .56915 L
.76739 .56915 L
.7674 .56915 L
.7674 .56916 L
.7674 .56916 L
.76741 .56916 L
.76741 .56916 L
.76742 .56917 L
.76742 .56917 L
.76742 .56917 L
.76743 .56918 L
.76743 .56918 L
.76744 .56918 L
.76744 .56919 L
.76745 .56919 L
.76745 .5692 L
.76746 .5692 L
.76746 .5692 L
.76747 .56921 L
.76747 .56921 L
.76748 .56921 L
.76748 .56922 L
.76749 .56922 L
.76749 .56922 L
.76749 .56923 L
.7675 .56923 L
.7675 .56923 L
.76751 .56924 L
Mistroke
.76751 .56924 L
.76752 .56924 L
.76752 .56925 L
.76753 .56925 L
.76753 .56925 L
.76753 .56926 L
.76754 .56926 L
.76754 .56926 L
.76755 .56927 L
.76755 .56927 L
.76755 .56927 L
.76756 .56927 L
.76756 .56928 L
.76757 .56928 L
.76757 .56928 L
.76758 .56929 L
.76758 .56929 L
.76758 .56929 L
.76759 .5693 L
.76759 .5693 L
.7676 .5693 L
.7676 .5693 L
.7676 .56931 L
.76761 .56931 L
.76761 .56931 L
.76762 .56932 L
.76762 .56932 L
.76762 .56932 L
.76763 .56932 L
.76763 .56933 L
.76763 .56933 L
.76764 .56933 L
.76764 .56933 L
.76765 .56934 L
.76765 .56934 L
.76765 .56934 L
.76766 .56935 L
.76766 .56935 L
.76767 .56935 L
.76767 .56935 L
.76767 .56936 L
.76768 .56936 L
.76768 .56936 L
.76769 .56937 L
.76769 .56937 L
.76769 .56937 L
.7677 .56937 L
.7677 .56938 L
.7677 .56938 L
.76771 .56938 L
Mistroke
.76771 .56939 L
.76772 .56939 L
.76772 .56939 L
.76773 .56939 L
.76773 .5694 L
.76773 .5694 L
.76774 .5694 L
.76774 .5694 L
.76774 .56941 L
.76775 .56941 L
.76775 .56941 L
.76776 .56941 L
.76776 .56942 L
.76776 .56942 L
.76777 .56942 L
.76777 .56942 L
.76778 .56943 L
.76778 .56943 L
.76778 .56943 L
.76778 .56943 L
.76779 .56944 L
.76779 .56944 L
.7678 .56944 L
.7678 .56945 L
.76781 .56945 L
.76781 .56945 L
.76781 .56945 L
.76782 .56946 L
.76782 .56946 L
.76783 .56946 L
.76783 .56946 L
.76783 .56947 L
.76784 .56947 L
.76784 .56947 L
.76784 .56947 L
.76785 .56948 L
.76785 .56948 L
.76785 .56948 L
.76786 .56948 L
.76786 .56948 L
.76786 .56949 L
.76787 .56949 L
.76787 .56949 L
.76788 .56949 L
.76788 .5695 L
.76788 .5695 L
.76789 .5695 L
.76789 .5695 L
.76789 .56951 L
.7679 .56951 L
Mistroke
.7679 .56951 L
.76791 .56951 L
.76791 .56952 L
.76791 .56952 L
.76791 .56952 L
.76792 .56952 L
.76792 .56953 L
.76793 .56953 L
.76793 .56953 L
.76793 .56953 L
.76794 .56953 L
.76794 .56954 L
.76794 .56954 L
.76795 .56954 L
.76795 .56954 L
.76796 .56955 L
.76796 .56955 L
.76796 .56955 L
.76796 .56955 L
.76797 .56955 L
.76797 .56956 L
.76797 .56956 L
.76798 .56956 L
.76798 .56956 L
.76799 .56956 L
.76799 .56957 L
.76799 .56957 L
.76799 .56957 L
.768 .56957 L
.768 .56958 L
.76801 .56958 L
.76801 .56958 L
.76801 .56958 L
.76802 .56958 L
.76802 .56959 L
.76802 .56959 L
.76803 .56959 L
.76803 .56959 L
.76803 .56959 L
.76804 .5696 L
.76804 .5696 L
.76804 .5696 L
.76804 .5696 L
.76805 .5696 L
.76805 .56961 L
.76805 .56961 L
.76806 .56961 L
.76806 .56961 L
.76806 .56961 L
.76807 .56962 L
Mistroke
.76807 .56962 L
.76807 .56962 L
.76808 .56962 L
.76808 .56962 L
.76809 .56963 L
.76809 .56963 L
.76809 .56963 L
.76809 .56963 L
.7681 .56963 L
.7681 .56964 L
.7681 .56964 L
.76811 .56964 L
.76811 .56964 L
.76811 .56964 L
.76812 .56965 L
.76812 .56965 L
.76813 .56965 L
.76813 .56965 L
.76813 .56966 L
.76814 .56966 L
.76814 .56966 L
.76815 .56966 L
.76815 .56967 L
.76815 .56967 L
.76816 .56967 L
.76816 .56967 L
.76816 .56967 L
.76817 .56968 L
.76817 .56968 L
.76817 .56968 L
.76818 .56968 L
.76818 .56968 L
.76818 .56969 L
.76819 .56969 L
.76819 .56969 L
.76819 .56969 L
.7682 .56969 L
.7682 .56969 L
.7682 .5697 L
.7682 .5697 L
.76821 .5697 L
.76821 .5697 L
.76821 .5697 L
.76822 .56971 L
.76822 .56971 L
.76822 .56971 L
.76823 .56971 L
.76823 .56971 L
.76823 .56971 L
.76824 .56972 L
Mistroke
.76824 .56972 L
.76824 .56972 L
.76825 .56972 L
.76825 .56972 L
.76825 .56972 L
.76825 .56973 L
.76826 .56973 L
.76826 .56973 L
.76826 .56973 L
.76827 .56973 L
.76827 .56973 L
.76827 .56974 L
.76828 .56974 L
.76828 .56974 L
.76828 .56974 L
.76829 .56974 L
.76829 .56975 L
.76829 .56975 L
.7683 .56975 L
.7683 .56975 L
.7683 .56975 L
.76831 .56975 L
.76831 .56976 L
.76831 .56976 L
.76832 .56976 L
.76832 .56976 L
.76832 .56977 L
.76833 .56977 L
.76833 .56977 L
.76833 .56977 L
.76834 .56977 L
.76834 .56977 L
.76834 .56978 L
.76835 .56978 L
.76835 .56978 L
.76835 .56978 L
.76836 .56978 L
.76836 .56979 L
.76836 .56979 L
.76837 .56979 L
.76837 .56979 L
.76837 .56979 L
.76838 .56979 L
.76838 .5698 L
.76838 .5698 L
.76839 .5698 L
.76839 .5698 L
.76839 .5698 L
.76839 .5698 L
.7684 .5698 L
Mistroke
.7684 .56981 L
.7684 .56981 L
.76841 .56981 L
.76841 .56981 L
.76841 .56981 L
.76841 .56981 L
.76842 .56982 L
.76842 .56982 L
.76843 .56982 L
.76843 .56982 L
.76843 .56982 L
.76843 .56982 L
.76844 .56983 L
.76844 .56983 L
.76844 .56983 L
.76845 .56983 L
.76845 .56983 L
.76845 .56983 L
.76846 .56984 L
.76846 .56984 L
.76846 .56984 L
.76847 .56984 L
.76847 .56984 L
.76847 .56984 L
.76847 .56984 L
.76848 .56985 L
.76848 .56985 L
.76849 .56985 L
.76849 .56985 L
.76849 .56985 L
.76849 .56985 L
.7685 .56986 L
.7685 .56986 L
.7685 .56986 L
.76851 .56986 L
.76851 .56986 L
.76851 .56986 L
.76851 .56987 L
.76852 .56987 L
.76852 .56987 L
.76853 .56987 L
.76853 .56987 L
.76853 .56987 L
.76854 .56988 L
.76854 .56988 L
.76854 .56988 L
.76855 .56988 L
.76855 .56988 L
.76855 .56988 L
.76856 .56989 L
Mistroke
.76856 .56989 L
.76856 .56989 L
.76857 .56989 L
.76857 .56989 L
.76857 .56989 L
.76858 .5699 L
.76858 .5699 L
.76858 .5699 L
.76858 .5699 L
.76859 .5699 L
.76859 .5699 L
.76859 .5699 L
.7686 .56991 L
.7686 .56991 L
.7686 .56991 L
.76861 .56991 L
.76861 .56991 L
.76861 .56991 L
.76862 .56992 L
.76862 .56992 L
.76863 .56992 L
.76863 .56992 L
.76863 .56992 L
.76863 .56992 L
.76864 .56992 L
.76864 .56993 L
.76864 .56993 L
.76865 .56993 L
.76865 .56993 L
.76865 .56993 L
.76866 .56993 L
.76866 .56994 L
.76867 .56994 L
.76867 .56994 L
.76867 .56994 L
.76868 .56994 L
.76868 .56994 L
.76868 .56995 L
.76868 .56995 L
.76869 .56995 L
.76869 .56995 L
.76869 .56995 L
.7687 .56995 L
.7687 .56995 L
.7687 .56995 L
.76871 .56996 L
.76871 .56996 L
.76871 .56996 L
.76872 .56996 L
.76872 .56996 L
Mistroke
.76872 .56996 L
.76872 .56996 L
.76873 .56997 L
.76873 .56997 L
.76873 .56997 L
.76874 .56997 L
.76874 .56997 L
.76874 .56997 L
.76875 .56997 L
.76875 .56998 L
.76875 .56998 L
.76875 .56998 L
.76876 .56998 L
.76876 .56998 L
.76876 .56998 L
.76877 .56998 L
.76877 .56998 L
.76877 .56999 L
.76878 .56999 L
.76878 .56999 L
.76878 .56999 L
.76878 .56999 L
.76879 .56999 L
.76879 .56999 L
.76879 .57 L
.7688 .57 L
.7688 .57 L
.7688 .57 L
.76881 .57 L
.76881 .57 L
.76882 .57 L
.76882 .57001 L
.76882 .57001 L
.76883 .57001 L
.76883 .57001 L
.76883 .57001 L
.76883 .57001 L
.76884 .57001 L
.76884 .57002 L
.76884 .57002 L
.76885 .57002 L
.76885 .57002 L
.76885 .57002 L
.76886 .57002 L
.76886 .57002 L
.76886 .57002 L
.76887 .57003 L
.76887 .57003 L
.76887 .57003 L
.76888 .57003 L
Mistroke
.76888 .57003 L
.76888 .57003 L
.76889 .57003 L
.76889 .57004 L
.76889 .57004 L
.7689 .57004 L
.7689 .57004 L
.76891 .57004 L
.76891 .57004 L
.76891 .57004 L
.76892 .57005 L
.76892 .57005 L
.76892 .57005 L
.76893 .57005 L
.76893 .57005 L
.76894 .57005 L
.76894 .57006 L
.76894 .57006 L
.76895 .57006 L
.76895 .57006 L
.76895 .57006 L
.76896 .57006 L
.76896 .57006 L
.76896 .57006 L
.76897 .57007 L
.76897 .57007 L
.76897 .57007 L
.76898 .57007 L
.76898 .57007 L
.76898 .57007 L
.76899 .57007 L
.76899 .57008 L
.769 .57008 L
.769 .57008 L
.769 .57008 L
.76901 .57008 L
.76901 .57008 L
.76901 .57008 L
.76902 .57008 L
.76902 .57009 L
.76903 .57009 L
.76903 .57009 L
.76903 .57009 L
.76904 .57009 L
.76904 .57009 L
.76905 .5701 L
.76905 .5701 L
.76905 .5701 L
.76906 .5701 L
.76906 .5701 L
Mistroke
.76906 .5701 L
.76907 .5701 L
.76907 .5701 L
.76907 .57011 L
.76908 .57011 L
.76908 .57011 L
.76909 .57011 L
.76909 .57011 L
.76909 .57011 L
.7691 .57011 L
.7691 .57012 L
.7691 .57012 L
.76911 .57012 L
.76911 .57012 L
.76912 .57012 L
.76912 .57012 L
.76912 .57012 L
.76913 .57012 L
.76913 .57012 L
.76913 .57013 L
.76914 .57013 L
.76914 .57013 L
.76914 .57013 L
.76915 .57013 L
.76915 .57013 L
.76915 .57013 L
.76916 .57013 L
.76916 .57014 L
.76916 .57014 L
.76917 .57014 L
.76917 .57014 L
.76917 .57014 L
.76918 .57014 L
.76918 .57014 L
.76919 .57014 L
.76919 .57014 L
.76919 .57015 L
.7692 .57015 L
.7692 .57015 L
.7692 .57015 L
.76921 .57015 L
.76921 .57015 L
.76921 .57015 L
.76921 .57015 L
.76922 .57015 L
.76922 .57016 L
.76922 .57016 L
.76923 .57016 L
.76923 .57016 L
.76924 .57016 L
Mistroke
.76924 .57016 L
.76924 .57016 L
.76925 .57016 L
.76925 .57016 L
.76925 .57017 L
.76926 .57017 L
.76926 .57017 L
.76926 .57017 L
.76926 .57017 L
.76927 .57017 L
.76927 .57017 L
.76927 .57017 L
.76928 .57017 L
.76928 .57017 L
.76928 .57018 L
.76929 .57018 L
.76929 .57018 L
.76929 .57018 L
.7693 .57018 L
.7693 .57018 L
.76931 .57018 L
.76931 .57018 L
.76931 .57018 L
.76932 .57019 L
.76932 .57019 L
.76933 .57019 L
.76933 .57019 L
.76933 .57019 L
.76934 .57019 L
.76934 .57019 L
.76934 .57019 L
.76934 .57019 L
.76935 .57019 L
.76935 .57019 L
.76935 .5702 L
.76936 .5702 L
.76936 .5702 L
.76936 .5702 L
.76937 .5702 L
.76937 .5702 L
.76937 .5702 L
.76938 .5702 L
.76938 .5702 L
.76938 .5702 L
.76939 .5702 L
.76939 .57021 L
.76939 .57021 L
.7694 .57021 L
.7694 .57021 L
.7694 .57021 L
Mistroke
.76941 .57021 L
.76941 .57021 L
.76941 .57021 L
.76942 .57021 L
.76942 .57021 L
.76942 .57021 L
.76943 .57022 L
.76943 .57022 L
.76943 .57022 L
.76944 .57022 L
.76944 .57022 L
.76944 .57022 L
.76944 .57022 L
.76945 .57022 L
.76945 .57022 L
.76945 .57022 L
.76946 .57022 L
.76946 .57022 L
.76946 .57022 L
.76947 .57023 L
.76947 .57023 L
.76947 .57023 L
.76948 .57023 L
.76948 .57023 L
.76948 .57023 L
.76949 .57023 L
.76949 .57023 L
.76949 .57023 L
.76949 .57023 L
.7695 .57023 L
.7695 .57023 L
.7695 .57023 L
.76951 .57024 L
.76951 .57024 L
.76951 .57024 L
.76952 .57024 L
.76952 .57024 L
.76952 .57024 L
.76953 .57024 L
.76953 .57024 L
.76953 .57024 L
.76954 .57024 L
.76954 .57024 L
.76954 .57024 L
.76954 .57024 L
.76955 .57025 L
.76955 .57025 L
.76955 .57025 L
.76956 .57025 L
.76956 .57025 L
Mistroke
.76956 .57025 L
.76957 .57025 L
.76957 .57025 L
.76957 .57025 L
.76958 .57025 L
.76958 .57025 L
.76958 .57025 L
.76959 .57025 L
.76959 .57025 L
.76959 .57026 L
.7696 .57026 L
.7696 .57026 L
.7696 .57026 L
.76961 .57026 L
.76961 .57026 L
.76961 .57026 L
.76961 .57026 L
.76962 .57026 L
.76962 .57026 L
.76962 .57026 L
.76963 .57026 L
.76963 .57026 L
.76963 .57026 L
.76963 .57026 L
.76964 .57027 L
.76964 .57027 L
.76964 .57027 L
.76965 .57027 L
.76965 .57027 L
.76965 .57027 L
.76966 .57027 L
.76966 .57027 L
.76966 .57027 L
.76967 .57027 L
.76967 .57027 L
.76967 .57027 L
.76968 .57027 L
.76968 .57027 L
.76968 .57027 L
.76968 .57027 L
.76969 .57028 L
.76969 .57028 L
.76969 .57028 L
.7697 .57028 L
.7697 .57028 L
.7697 .57028 L
.76971 .57028 L
.76971 .57028 L
.76971 .57028 L
.76972 .57028 L
Mistroke
.76972 .57028 L
.76972 .57028 L
.76972 .57028 L
.76973 .57028 L
.76973 .57028 L
.76973 .57028 L
.76974 .57028 L
.76974 .57028 L
.76974 .57029 L
.76974 .57029 L
.76975 .57029 L
.76975 .57029 L
.76975 .57029 L
.76976 .57029 L
.76976 .57029 L
.76976 .57029 L
.76977 .57029 L
.76977 .57029 L
.76977 .57029 L
.76977 .57029 L
.76978 .57029 L
.76978 .57029 L
.76978 .57029 L
.76979 .57029 L
.76979 .57029 L
.7698 .57029 L
.7698 .5703 L
.7698 .5703 L
.76981 .5703 L
.76981 .5703 L
.76981 .5703 L
.76981 .5703 L
.76982 .5703 L
.76982 .5703 L
.76982 .5703 L
.76983 .5703 L
.76983 .5703 L
.76983 .5703 L
.76984 .5703 L
.76984 .5703 L
.76984 .5703 L
.76985 .5703 L
.76985 .5703 L
.76985 .5703 L
.76986 .5703 L
.76986 .5703 L
.76986 .5703 L
.76987 .57031 L
.76987 .57031 L
.76987 .57031 L
Mistroke
.76987 .57031 L
.76988 .57031 L
.76988 .57031 L
.76988 .57031 L
.76989 .57031 L
.76989 .57031 L
.76989 .57031 L
.76989 .57031 L
.7699 .57031 L
.7699 .57031 L
.7699 .57031 L
.76991 .57031 L
.76991 .57031 L
.76991 .57031 L
.76992 .57031 L
.76992 .57031 L
.76992 .57031 L
.76993 .57031 L
.76993 .57031 L
.76993 .57031 L
.76994 .57031 L
.76994 .57032 L
.76994 .57032 L
.76995 .57032 L
.76995 .57032 L
.76995 .57032 L
.76996 .57032 L
.76996 .57032 L
.76996 .57032 L
.76997 .57032 L
.76997 .57032 L
.76997 .57032 L
.76998 .57032 L
.76998 .57032 L
.76998 .57032 L
.76999 .57032 L
.76999 .57032 L
.76999 .57032 L
.77 .57032 L
.77 .57032 L
.77 .57032 L
.77001 .57032 L
.77001 .57032 L
.77001 .57032 L
.77002 .57032 L
.77002 .57032 L
.77002 .57032 L
.77002 .57032 L
.77003 .57033 L
.77003 .57033 L
Mistroke
.77004 .57033 L
.77004 .57033 L
.77004 .57033 L
.77004 .57033 L
.77005 .57033 L
.77005 .57033 L
.77005 .57033 L
.77006 .57033 L
.77006 .57033 L
.77006 .57033 L
.77006 .57033 L
.77007 .57033 L
.77007 .57033 L
.77007 .57033 L
.77008 .57033 L
.77008 .57033 L
.77008 .57033 L
.77009 .57033 L
.77009 .57033 L
.77009 .57033 L
.7701 .57033 L
.7701 .57033 L
.7701 .57033 L
.77011 .57033 L
.77011 .57033 L
.77011 .57033 L
.77012 .57033 L
.77012 .57033 L
.77012 .57033 L
.77013 .57033 L
.77013 .57033 L
.77013 .57033 L
.77014 .57033 L
.77014 .57033 L
.77014 .57033 L
.77014 .57033 L
.77015 .57033 L
.77015 .57034 L
.77015 .57034 L
.77016 .57034 L
.77016 .57034 L
.77016 .57034 L
.77017 .57034 L
.77017 .57034 L
.77017 .57034 L
.77017 .57034 L
.77018 .57034 L
.77018 .57034 L
.77018 .57034 L
.77019 .57034 L
Mistroke
.77019 .57034 L
.77019 .57034 L
.7702 .57034 L
.7702 .57034 L
.7702 .57034 L
.77021 .57034 L
.77021 .57034 L
.77021 .57034 L
.77021 .57034 L
.77022 .57034 L
.77022 .57034 L
.77023 .57034 L
.77023 .57034 L
.77023 .57034 L
.77024 .57034 L
.77024 .57034 L
.77024 .57034 L
.77025 .57034 L
.77025 .57034 L
.77025 .57034 L
.77026 .57034 L
.77026 .57034 L
.77027 .57034 L
.77027 .57034 L
.77027 .57034 L
.77028 .57034 L
.77028 .57034 L
.77028 .57034 L
.77029 .57034 L
.77029 .57034 L
.77029 .57034 L
.7703 .57034 L
.7703 .57034 L
.7703 .57034 L
.77031 .57034 L
.77031 .57034 L
.77031 .57034 L
.77032 .57034 L
.77032 .57034 L
.77032 .57034 L
.77033 .57034 L
.77033 .57034 L
.77033 .57034 L
.77034 .57034 L
.77034 .57034 L
.77034 .57034 L
.77035 .57034 L
.77035 .57034 L
.77035 .57034 L
.77036 .57034 L
Mistroke
.77036 .57034 L
.77036 .57034 L
.77037 .57034 L
.77037 .57034 L
.77037 .57034 L
.77038 .57034 L
.77038 .57034 L
.77038 .57034 L
.77039 .57034 L
.77039 .57034 L
.77039 .57034 L
.7704 .57034 L
.7704 .57034 L
.7704 .57034 L
.77041 .57034 L
.77041 .57034 L
.77041 .57034 L
.77041 .57034 L
.77042 .57034 L
.77042 .57034 L
.77042 .57034 L
.77043 .57034 L
.77043 .57034 L
.77043 .57034 L
.77044 .57034 L
.77044 .57034 L
.77044 .57034 L
.77044 .57034 L
.77045 .57034 L
.77045 .57034 L
.77045 .57034 L
.77046 .57034 L
.77046 .57034 L
.77046 .57034 L
.77047 .57034 L
.77047 .57034 L
.77047 .57034 L
.77048 .57034 L
.77048 .57034 L
.77048 .57034 L
.77048 .57034 L
.77049 .57034 L
.77049 .57034 L
.77049 .57034 L
.7705 .57034 L
.7705 .57034 L
.7705 .57034 L
.7705 .57034 L
.77051 .57034 L
.77051 .57034 L
Mistroke
.77052 .57034 L
.77052 .57034 L
.77052 .57034 L
.77052 .57034 L
.77053 .57034 L
.77053 .57034 L
.77053 .57034 L
.77054 .57034 L
.77054 .57034 L
.77054 .57034 L
.77055 .57034 L
.77055 .57034 L
.77055 .57034 L
.77056 .57034 L
.77056 .57034 L
.77056 .57034 L
.77056 .57034 L
.77057 .57034 L
.77057 .57034 L
.77057 .57034 L
.77058 .57034 L
.77058 .57034 L
.77058 .57034 L
.77059 .57034 L
.77059 .57034 L
.77059 .57034 L
.7706 .57034 L
.7706 .57034 L
.7706 .57034 L
.77061 .57034 L
.77061 .57034 L
.77061 .57033 L
.77062 .57033 L
.77062 .57033 L
.77062 .57033 L
.77063 .57033 L
.77063 .57033 L
.77063 .57033 L
.77063 .57033 L
.77064 .57033 L
.77064 .57033 L
.77064 .57033 L
.77065 .57033 L
.77065 .57033 L
.77065 .57033 L
.77065 .57033 L
.77066 .57033 L
.77066 .57033 L
.77066 .57033 L
.77067 .57033 L
Mistroke
.77067 .57033 L
.77067 .57033 L
.77068 .57033 L
.77068 .57033 L
.77068 .57033 L
.77068 .57033 L
.77069 .57033 L
.77069 .57033 L
.77069 .57033 L
.7707 .57033 L
.7707 .57033 L
.7707 .57033 L
.77071 .57033 L
.77071 .57033 L
.77071 .57033 L
.77071 .57033 L
.77072 .57033 L
.77072 .57033 L
.77072 .57033 L
.77073 .57033 L
.77073 .57033 L
.77073 .57032 L
.77074 .57032 L
.77074 .57032 L
.77074 .57032 L
.77074 .57032 L
.77075 .57032 L
.77075 .57032 L
.77075 .57032 L
.77076 .57032 L
.77076 .57032 L
.77076 .57032 L
.77077 .57032 L
.77077 .57032 L
.77077 .57032 L
.77078 .57032 L
.77078 .57032 L
.77078 .57032 L
.77078 .57032 L
.77079 .57032 L
.77079 .57032 L
.77079 .57032 L
.7708 .57032 L
.7708 .57032 L
.7708 .57032 L
.7708 .57032 L
.77081 .57032 L
.77081 .57032 L
.77081 .57032 L
.77082 .57032 L
Mistroke
.77082 .57031 L
.77082 .57031 L
.77083 .57031 L
.77083 .57031 L
.77083 .57031 L
.77083 .57031 L
.77084 .57031 L
.77084 .57031 L
.77084 .57031 L
.77085 .57031 L
.77085 .57031 L
.77086 .57031 L
.77086 .57031 L
.77086 .57031 L
.77086 .57031 L
.77087 .57031 L
.77087 .57031 L
.77087 .57031 L
.77087 .57031 L
.77088 .57031 L
.77088 .57031 L
.77088 .57031 L
.77089 .57031 L
.77089 .57031 L
.77089 .5703 L
.7709 .5703 L
.7709 .5703 L
.7709 .5703 L
.7709 .5703 L
.77091 .5703 L
.77091 .5703 L
.77091 .5703 L
.77092 .5703 L
.77092 .5703 L
.77092 .5703 L
.77092 .5703 L
.77093 .5703 L
.77093 .5703 L
.77093 .5703 L
.77094 .5703 L
.77094 .5703 L
.77094 .5703 L
.77095 .5703 L
.77095 .5703 L
.77095 .5703 L
.77096 .57029 L
.77096 .57029 L
.77096 .57029 L
.77097 .57029 L
.77097 .57029 L
Mistroke
.77097 .57029 L
.77097 .57029 L
.77098 .57029 L
.77098 .57029 L
.77098 .57029 L
.77099 .57029 L
.77099 .57029 L
.77099 .57029 L
.77099 .57029 L
.771 .57029 L
.771 .57029 L
.771 .57029 L
.77101 .57029 L
.77101 .57029 L
.77101 .57029 L
.77101 .57028 L
.77102 .57028 L
.77102 .57028 L
.77102 .57028 L
.77103 .57028 L
.77103 .57028 L
.77103 .57028 L
.77103 .57028 L
.77104 .57028 L
.77104 .57028 L
.77104 .57028 L
.77105 .57028 L
.77105 .57028 L
.77105 .57028 L
.77105 .57028 L
.77106 .57028 L
.77106 .57028 L
.77106 .57028 L
.77107 .57027 L
.77107 .57027 L
.77107 .57027 L
.77108 .57027 L
.77108 .57027 L
.77108 .57027 L
.77109 .57027 L
.77109 .57027 L
.7711 .57027 L
.7711 .57027 L
.77111 .57027 L
.77111 .57027 L
.77111 .57027 L
.77111 .57027 L
.77112 .57026 L
.77112 .57026 L
.77112 .57026 L
Mistroke
.77113 .57026 L
.77113 .57026 L
.77113 .57026 L
.77114 .57026 L
.77114 .57026 L
.77114 .57026 L
.77115 .57026 L
.77115 .57026 L
.77115 .57026 L
.77116 .57026 L
.77116 .57025 L
.77116 .57025 L
.77117 .57025 L
.77117 .57025 L
.77117 .57025 L
.77117 .57025 L
.77118 .57025 L
.77118 .57025 L
.77118 .57025 L
.77119 .57025 L
.77119 .57025 L
.77119 .57025 L
.7712 .57025 L
.7712 .57025 L
.7712 .57024 L
.77121 .57024 L
.77121 .57024 L
.77121 .57024 L
.77122 .57024 L
.77122 .57024 L
.77122 .57024 L
.77123 .57024 L
.77123 .57024 L
.77123 .57024 L
.77124 .57024 L
.77124 .57024 L
.77124 .57023 L
.77125 .57023 L
.77125 .57023 L
.77125 .57023 L
.77126 .57023 L
.77126 .57023 L
.77126 .57023 L
.77127 .57023 L
.77127 .57023 L
.77127 .57023 L
.77128 .57023 L
.77128 .57023 L
.77128 .57022 L
.77129 .57022 L
Mistroke
.77129 .57022 L
.77129 .57022 L
.7713 .57022 L
.7713 .57022 L
.7713 .57022 L
.77131 .57022 L
.77131 .57022 L
.77131 .57022 L
.77132 .57022 L
.77132 .57021 L
.77132 .57021 L
.77133 .57021 L
.77133 .57021 L
.77133 .57021 L
.77134 .57021 L
.77134 .57021 L
.77134 .57021 L
.77135 .57021 L
.77135 .57021 L
.77135 .57021 L
.77136 .5702 L
.77136 .5702 L
.77136 .5702 L
.77137 .5702 L
.77137 .5702 L
.77137 .5702 L
.77138 .5702 L
.77138 .5702 L
.77138 .5702 L
.77139 .5702 L
.77139 .57019 L
.77139 .57019 L
.77139 .57019 L
.7714 .57019 L
.7714 .57019 L
.7714 .57019 L
.77141 .57019 L
.77141 .57019 L
.77141 .57019 L
.77142 .57019 L
.77142 .57019 L
.77142 .57018 L
.77143 .57018 L
.77143 .57018 L
.77143 .57018 L
.77143 .57018 L
.77144 .57018 L
.77144 .57018 L
.77144 .57018 L
.77145 .57018 L
Mistroke
.77145 .57018 L
.77145 .57018 L
.77146 .57017 L
.77146 .57017 L
.77146 .57017 L
.77147 .57017 L
.77147 .57017 L
.77147 .57017 L
.77148 .57017 L
.77148 .57017 L
.77148 .57017 L
.77148 .57017 L
.77149 .57016 L
.77149 .57016 L
.77149 .57016 L
.7715 .57016 L
.7715 .57016 L
.7715 .57016 L
.77151 .57016 L
.77151 .57016 L
.77151 .57016 L
.77152 .57015 L
.77152 .57015 L
.77152 .57015 L
.77153 .57015 L
.77153 .57015 L
.77153 .57015 L
.77153 .57015 L
.77154 .57015 L
.77154 .57015 L
.77154 .57015 L
.77155 .57014 L
.77155 .57014 L
.77155 .57014 L
.77156 .57014 L
.77156 .57014 L
.77156 .57014 L
.77157 .57014 L
.77157 .57014 L
.77157 .57014 L
.77158 .57013 L
.77158 .57013 L
.77158 .57013 L
.77159 .57013 L
.77159 .57013 L
.7716 .57013 L
.7716 .57013 L
.7716 .57012 L
.77161 .57012 L
.77161 .57012 L
Mistroke
.77161 .57012 L
.77162 .57012 L
.77162 .57012 L
.77162 .57012 L
.77163 .57012 L
.77163 .57011 L
.77163 .57011 L
.77164 .57011 L
.77164 .57011 L
.77164 .57011 L
.77165 .57011 L
.77165 .57011 L
.77165 .57011 L
.77166 .5701 L
.77166 .5701 L
.77166 .5701 L
.77167 .5701 L
.77167 .5701 L
.77167 .5701 L
.77168 .5701 L
.77168 .5701 L
.77168 .57009 L
.77169 .57009 L
.77169 .57009 L
.77169 .57009 L
.7717 .57009 L
.7717 .57009 L
.7717 .57009 L
.77171 .57009 L
.77171 .57008 L
.77171 .57008 L
.77172 .57008 L
.77172 .57008 L
.77172 .57008 L
.77173 .57008 L
.77173 .57008 L
.77173 .57007 L
.77174 .57007 L
.77174 .57007 L
.77175 .57007 L
.77175 .57007 L
.77175 .57007 L
.77176 .57007 L
.77176 .57006 L
.77176 .57006 L
.77177 .57006 L
.77177 .57006 L
.77178 .57006 L
.77178 .57006 L
.77178 .57005 L
Mistroke
.77179 .57005 L
.77179 .57005 L
.7718 .57005 L
.7718 .57005 L
.7718 .57005 L
.7718 .57005 L
.77181 .57004 L
.77181 .57004 L
.77182 .57004 L
.77182 .57004 L
.77182 .57004 L
.77183 .57004 L
.77183 .57004 L
.77183 .57003 L
.77184 .57003 L
.77184 .57003 L
.77184 .57003 L
.77184 .57003 L
.77185 .57003 L
.77185 .57003 L
.77185 .57002 L
.77186 .57002 L
.77186 .57002 L
.77186 .57002 L
.77187 .57002 L
.77187 .57002 L
.77187 .57002 L
.77188 .57001 L
.77188 .57001 L
.77188 .57001 L
.77188 .57001 L
.77189 .57001 L
.77189 .57001 L
.77189 .57001 L
.7719 .57001 L
.7719 .57 L
.7719 .57 L
.77191 .57 L
.77191 .57 L
.77191 .57 L
.77192 .57 L
.77192 .57 L
.77192 .56999 L
.77193 .56999 L
.77193 .56999 L
.77193 .56999 L
.77194 .56999 L
.77194 .56999 L
.77194 .56999 L
.77194 .56998 L
Mistroke
.77195 .56998 L
.77195 .56998 L
.77196 .56998 L
.77196 .56998 L
.77196 .56997 L
.77197 .56997 L
.77197 .56997 L
.77197 .56997 L
.77198 .56997 L
.77198 .56997 L
.77199 .56996 L
.77199 .56996 L
.77199 .56996 L
.772 .56996 L
.772 .56996 L
.772 .56996 L
.772 .56996 L
.77201 .56995 L
.77201 .56995 L
.77201 .56995 L
.77202 .56995 L
.77202 .56995 L
.77202 .56995 L
.77203 .56995 L
.77203 .56994 L
.77203 .56994 L
.77203 .56994 L
.77204 .56994 L
.77204 .56994 L
.77204 .56994 L
.77204 .56994 L
.77205 .56993 L
.77205 .56993 L
.77205 .56993 L
.77206 .56993 L
.77206 .56993 L
.77206 .56993 L
.77207 .56992 L
.77207 .56992 L
.77207 .56992 L
.77208 .56992 L
.77208 .56992 L
.77208 .56992 L
.77209 .56991 L
.77209 .56991 L
.77209 .56991 L
.77209 .56991 L
.7721 .56991 L
.7721 .56991 L
.7721 .56991 L
Mistroke
.77211 .5699 L
.77211 .5699 L
.77211 .5699 L
.77212 .5699 L
.77212 .5699 L
.77212 .5699 L
.77213 .56989 L
.77213 .56989 L
.77213 .56989 L
.77214 .56989 L
.77214 .56989 L
.77214 .56989 L
.77214 .56989 L
.77215 .56988 L
.77215 .56988 L
.77215 .56988 L
.77216 .56988 L
.77216 .56988 L
.77216 .56988 L
.77217 .56987 L
.77217 .56987 L
.77217 .56987 L
.77217 .56987 L
.77218 .56987 L
.77218 .56986 L
.77218 .56986 L
.77219 .56986 L
.77219 .56986 L
.7722 .56986 L
.7722 .56986 L
.7722 .56985 L
.7722 .56985 L
.77221 .56985 L
.77221 .56985 L
.77221 .56985 L
.77222 .56985 L
.77222 .56984 L
.77222 .56984 L
.77223 .56984 L
.77223 .56984 L
.77223 .56984 L
.77224 .56984 L
.77224 .56983 L
.77224 .56983 L
.77225 .56983 L
.77225 .56983 L
.77225 .56983 L
.77225 .56982 L
.77226 .56982 L
.77226 .56982 L
Mistroke
.77226 .56982 L
.77227 .56982 L
.77227 .56982 L
.77227 .56981 L
.77227 .56981 L
.77228 .56981 L
.77228 .56981 L
.77228 .56981 L
.77229 .56981 L
.77229 .56981 L
.77229 .5698 L
.7723 .5698 L
.7723 .5698 L
.7723 .5698 L
.7723 .5698 L
.77231 .5698 L
.77231 .56979 L
.77231 .56979 L
.77232 .56979 L
.77232 .56979 L
.77232 .56979 L
.77233 .56978 L
.77233 .56978 L
.77233 .56978 L
.77233 .56978 L
.77234 .56978 L
.77234 .56978 L
.77234 .56977 L
.77235 .56977 L
.77235 .56977 L
.77235 .56977 L
.77236 .56977 L
.77236 .56976 L
.77236 .56976 L
.77236 .56976 L
.77237 .56976 L
.77237 .56976 L
.77237 .56976 L
.77238 .56975 L
.77238 .56975 L
.77238 .56975 L
.77239 .56975 L
.77239 .56975 L
.77239 .56974 L
.7724 .56974 L
.7724 .56974 L
.7724 .56974 L
.7724 .56974 L
.77241 .56973 L
.77241 .56973 L
Mistroke
.77241 .56973 L
.77242 .56973 L
.77242 .56973 L
.77242 .56973 L
.77243 .56972 L
.77243 .56972 L
.77243 .56972 L
.77244 .56972 L
.77244 .56972 L
.77244 .56971 L
.77245 .56971 L
.77245 .56971 L
.77245 .56971 L
.77246 .5697 L
.77246 .5697 L
.77246 .5697 L
.77247 .5697 L
.77247 .5697 L
.77247 .56969 L
.77248 .56969 L
.77248 .56969 L
.77248 .56969 L
.77249 .56969 L
.77249 .56968 L
.77249 .56968 L
.7725 .56968 L
.7725 .56968 L
.7725 .56968 L
.77251 .56967 L
.77251 .56967 L
.77251 .56967 L
.77252 .56967 L
.77252 .56966 L
.77252 .56966 L
.77253 .56966 L
.77253 .56966 L
.77253 .56966 L
.77254 .56966 L
.77254 .56965 L
.77254 .56965 L
.77255 .56965 L
.77255 .56965 L
.77255 .56964 L
.77256 .56964 L
.77256 .56964 L
.77256 .56964 L
.77257 .56964 L
.77257 .56963 L
.77257 .56963 L
.77258 .56963 L
Mistroke
.77258 .56963 L
.77258 .56962 L
.77259 .56962 L
.77259 .56962 L
.77259 .56962 L
.7726 .56961 L
.7726 .56961 L
.7726 .56961 L
.77261 .56961 L
.77261 .56961 L
.77261 .5696 L
.77261 .5696 L
.77262 .5696 L
.77262 .5696 L
.77262 .5696 L
.77263 .56959 L
.77263 .56959 L
.77263 .56959 L
.77263 .56959 L
.77264 .56959 L
.77264 .56959 L
.77264 .56958 L
.77265 .56958 L
.77265 .56958 L
.77265 .56958 L
.77266 .56957 L
.77266 .56957 L
.77266 .56957 L
.77267 .56957 L
.77267 .56957 L
.77267 .56956 L
.77267 .56956 L
.77268 .56956 L
.77268 .56956 L
.77268 .56956 L
.77269 .56955 L
.77269 .56955 L
.77269 .56955 L
.77269 .56955 L
.7727 .56955 L
.7727 .56954 L
.7727 .56954 L
.77271 .56954 L
.77271 .56954 L
.77271 .56953 L
.77272 .56953 L
.77272 .56953 L
.77272 .56953 L
.77273 .56953 L
.77273 .56952 L
Mistroke
.77273 .56952 L
.77273 .56952 L
.77274 .56952 L
.77274 .56952 L
.77274 .56951 L
.77275 .56951 L
.77275 .56951 L
.77275 .56951 L
.77276 .5695 L
.77276 .5695 L
.77276 .5695 L
.77277 .5695 L
.77277 .5695 L
.77277 .56949 L
.77277 .56949 L
.77278 .56949 L
.77278 .56949 L
.77279 .56948 L
.77279 .56948 L
.77279 .56948 L
.7728 .56948 L
.7728 .56947 L
.7728 .56947 L
.77281 .56947 L
.77281 .56946 L
.77281 .56946 L
.77282 .56946 L
.77282 .56946 L
.77282 .56945 L
.77283 .56945 L
.77283 .56945 L
.77284 .56945 L
.77284 .56944 L
.77284 .56944 L
.77285 .56944 L
.77285 .56943 L
.77285 .56943 L
.77286 .56943 L
.77286 .56943 L
.77286 .56942 L
.77287 .56942 L
.77287 .56942 L
.77287 .56942 L
.77288 .56941 L
.77288 .56941 L
.77288 .56941 L
.77289 .56941 L
.77289 .5694 L
.77289 .5694 L
.7729 .5694 L
Mistroke
.7729 .5694 L
.77291 .56939 L
.77291 .56939 L
.77291 .56939 L
.77292 .56938 L
.77292 .56938 L
.77293 .56938 L
.77293 .56937 L
.77293 .56937 L
.77294 .56937 L
.77294 .56937 L
.77295 .56936 L
.77295 .56936 L
.77295 .56936 L
.77296 .56935 L
.77296 .56935 L
.77296 .56935 L
.77297 .56935 L
.77297 .56934 L
.77298 .56934 L
.77298 .56934 L
.77298 .56933 L
.77299 .56933 L
.77299 .56933 L
.77299 .56932 L
.773 .56932 L
.773 .56932 L
.773 .56932 L
.77301 .56931 L
.77301 .56931 L
.77301 .56931 L
.77302 .56931 L
.77302 .5693 L
.77302 .5693 L
.77303 .5693 L
.77303 .56929 L
.77303 .56929 L
.77304 .56929 L
.77304 .56929 L
.77304 .56928 L
.77305 .56928 L
.77305 .56928 L
.77305 .56928 L
.77306 .56927 L
.77306 .56927 L
.77307 .56927 L
.77307 .56926 L
.77307 .56926 L
.77308 .56926 L
.77308 .56926 L
Mistroke
.77308 .56925 L
.77308 .56925 L
.77309 .56925 L
.77309 .56925 L
.77309 .56924 L
.7731 .56924 L
.7731 .56924 L
.7731 .56924 L
.77311 .56923 L
.77311 .56923 L
.77312 .56923 L
.77312 .56922 L
.77312 .56922 L
.77313 .56922 L
.77313 .56921 L
.77313 .56921 L
.77314 .56921 L
.77314 .5692 L
.77314 .5692 L
.77315 .5692 L
.77315 .5692 L
.77315 .56919 L
.77316 .56919 L
.77316 .56919 L
.77316 .56918 L
.77317 .56918 L
.77317 .56918 L
.77317 .56918 L
.77318 .56917 L
.77318 .56917 L
.77318 .56917 L
.77319 .56916 L
.77319 .56916 L
.7732 .56916 L
.7732 .56916 L
.7732 .56915 L
.77321 .56915 L
.77321 .56915 L
.77321 .56914 L
.77322 .56914 L
.77322 .56914 L
.77322 .56913 L
.77323 .56913 L
.77323 .56913 L
.77324 .56912 L
.77324 .56912 L
.77324 .56912 L
.77325 .56911 L
.77325 .56911 L
.77325 .56911 L
Mistroke
.77325 .56911 L
.77326 .5691 L
.77326 .5691 L
.77327 .5691 L
.77327 .56909 L
.77327 .56909 L
.77328 .56909 L
.77328 .56908 L
.77328 .56908 L
.77329 .56908 L
.77329 .56907 L
.77329 .56907 L
.7733 .56907 L
.7733 .56907 L
.7733 .56906 L
.77331 .56906 L
.77331 .56906 L
.77331 .56906 L
.77332 .56905 L
.77332 .56905 L
.77332 .56905 L
.77332 .56904 L
.77333 .56904 L
.77333 .56904 L
.77334 .56903 L
.77334 .56903 L
.77334 .56903 L
.77335 .56902 L
.77335 .56902 L
.77336 .56902 L
.77336 .56901 L
.77336 .56901 L
.77337 .569 L
.77337 .569 L
.77337 .569 L
.77338 .569 L
.77338 .56899 L
.77338 .56899 L
.77339 .56899 L
.77339 .56898 L
.7734 .56898 L
.7734 .56898 L
.7734 .56897 L
.77341 .56897 L
.77341 .56897 L
.77341 .56896 L
.77342 .56896 L
.77342 .56895 L
.77343 .56895 L
.77343 .56895 L
Mistroke
.77343 .56894 L
.77344 .56894 L
.77344 .56894 L
.77344 .56894 L
.77345 .56893 L
.77345 .56893 L
.77345 .56893 L
.77346 .56892 L
.77346 .56892 L
.77347 .56891 L
.77347 .56891 L
.77347 .56891 L
.77348 .5689 L
.77348 .5689 L
.77349 .5689 L
.77349 .56889 L
.77349 .56889 L
.77349 .56889 L
.7735 .56888 L
.7735 .56888 L
.7735 .56888 L
.77351 .56887 L
.77351 .56887 L
.77351 .56887 L
.77352 .56886 L
.77352 .56886 L
.77352 .56886 L
.77353 .56886 L
.77353 .56885 L
.77354 .56885 L
.77354 .56884 L
.77354 .56884 L
.77355 .56884 L
.77355 .56883 L
.77356 .56883 L
.77356 .56882 L
.77356 .56882 L
.77357 .56882 L
.77357 .56881 L
.77358 .56881 L
.77358 .5688 L
.77358 .5688 L
.77359 .5688 L
.77359 .56879 L
.77359 .56879 L
.7736 .56879 L
.7736 .56878 L
.77361 .56878 L
.77361 .56877 L
.77361 .56877 L
Mistroke
.77362 .56877 L
.77362 .56876 L
.77363 .56876 L
.77363 .56875 L
.77363 .56875 L
.77364 .56875 L
.77364 .56874 L
.77365 .56874 L
.77365 .56874 L
.77366 .56873 L
.77366 .56872 L
.77366 .56872 L
.77367 .56872 L
.77367 .56871 L
.77367 .56871 L
.77368 .56871 L
.77368 .5687 L
.77369 .5687 L
.77369 .56869 L
.7737 .56869 L
.7737 .56869 L
.7737 .56868 L
.77371 .56868 L
.77371 .56867 L
.77372 .56867 L
.77372 .56867 L
.77372 .56866 L
.77373 .56866 L
.77373 .56865 L
.77373 .56865 L
.77374 .56865 L
.77374 .56864 L
.77374 .56864 L
.77375 .56863 L
.77375 .56863 L
.77376 .56862 L
.77376 .56862 L
.77377 .56862 L
.77377 .56861 L
.77377 .56861 L
.77378 .5686 L
.77378 .5686 L
.77379 .5686 L
.77379 .56859 L
.77379 .56859 L
.7738 .56858 L
.7738 .56858 L
.77381 .56857 L
.77381 .56857 L
.77382 .56856 L
Mistroke
.77382 .56856 L
.77383 .56855 L
.77383 .56855 L
.77383 .56855 L
.77384 .56854 L
.77384 .56854 L
.77385 .56853 L
.77385 .56853 L
.77385 .56852 L
.77386 .56852 L
.77386 .56852 L
.77387 .56851 L
.77387 .56851 L
.77387 .5685 L
.77388 .5685 L
.77388 .5685 L
.77389 .56849 L
.77389 .56848 L
.77389 .56848 L
.7739 .56848 L
.7739 .56847 L
.77391 .56847 L
.77391 .56846 L
.77391 .56846 L
.77392 .56845 L
.77392 .56845 L
.77393 .56845 L
.77393 .56844 L
.77393 .56844 L
.77394 .56843 L
.77394 .56843 L
.77395 .56842 L
.77395 .56842 L
.77395 .56842 L
.77396 .56841 L
.77396 .56841 L
.77396 .5684 L
.77397 .5684 L
.77397 .5684 L
.77397 .56839 L
.77398 .56839 L
.77398 .56838 L
.77399 .56838 L
.77399 .56837 L
.774 .56837 L
.774 .56836 L
.77401 .56836 L
.77401 .56835 L
.77402 .56834 L
.77402 .56834 L
Mistroke
.77403 .56833 L
.77403 .56833 L
.77403 .56833 L
.77404 .56832 L
.77404 .56832 L
.77405 .56831 L
.77405 .56831 L
.77405 .5683 L
.77406 .5683 L
.77406 .56829 L
.77407 .56829 L
.77407 .56829 L
.77408 .56828 L
.77408 .56827 L
.77408 .56827 L
.77409 .56826 L
.77409 .56826 L
.7741 .56825 L
.7741 .56825 L
.77411 .56824 L
.77411 .56824 L
.77412 .56823 L
.77412 .56822 L
.77413 .56822 L
.77413 .56821 L
.77414 .56821 L
.77414 .5682 L
.77415 .5682 L
.77415 .56819 L
.77416 .56819 L
.77416 .56818 L
.77416 .56818 L
.77417 .56817 L
.77417 .56816 L
.77418 .56816 L
.77418 .56815 L
.77419 .56815 L
.77419 .56814 L
.7742 .56813 L
.7742 .56813 L
.77421 .56812 L
.77421 .56812 L
.77422 .56811 L
.77422 .56811 L
.77423 .5681 L
.77423 .5681 L
.77424 .56809 L
.77424 .56808 L
.77425 .56808 L
.77425 .56807 L
Mistroke
.77426 .56807 L
.77426 .56806 L
.77427 .56806 L
.77427 .56805 L
.77428 .56804 L
.77428 .56804 L
.77428 .56803 L
.77429 .56803 L
.7743 .56802 L
.7743 .56801 L
.7743 .56801 L
.77431 .56801 L
.77431 .568 L
.77432 .56799 L
.77432 .56799 L
.77433 .56798 L
.77433 .56798 L
.77433 .56797 L
.77434 .56797 L
.77435 .56796 L
.77435 .56795 L
.77435 .56795 L
.77436 .56794 L
.77436 .56794 L
.77437 .56793 L
.77437 .56793 L
.77438 .56792 L
.77438 .56791 L
.77439 .56791 L
.77439 .5679 L
.7744 .56789 L
.77441 .56788 L
.77441 .56787 L
.77442 .56787 L
.77442 .56786 L
.77443 .56786 L
.77443 .56785 L
.77444 .56785 L
.77444 .56784 L
.77444 .56784 L
.77445 .56783 L
.77445 .56783 L
.77446 .56782 L
.77446 .56781 L
.77447 .56781 L
.77447 .5678 L
.77447 .5678 L
.77448 .56779 L
.77449 .56778 L
.77449 .56778 L
Mistroke
.77449 .56777 L
.7745 .56777 L
.7745 .56776 L
.7745 .56776 L
.77451 .56775 L
.77452 .56774 L
.77452 .56774 L
.77453 .56773 L
.77453 .56772 L
.77454 .56772 L
.77454 .56771 L
.77455 .5677 L
.77455 .5677 L
.77456 .56769 L
.77457 .56768 L
.77457 .56767 L
.77458 .56767 L
.77458 .56766 L
.77459 .56765 L
.77459 .56764 L
.7746 .56763 L
.77461 .56763 L
.77461 .56762 L
.77462 .56761 L
.77463 .5676 L
.77463 .56759 L
.77464 .56758 L
.77464 .56758 L
.77465 .56757 L
.77466 .56756 L
.77466 .56755 L
.77467 .56755 L
.77467 .56754 L
.77468 .56753 L
.77468 .56752 L
.77469 .56751 L
.7747 .56751 L
.7747 .5675 L
.77471 .56749 L
.77471 .56748 L
.77472 .56748 L
.77472 .56747 L
.77473 .56746 L
.77473 .56746 L
.77474 .56745 L
.77475 .56744 L
.77475 .56743 L
.77476 .56742 L
.77476 .56742 L
.77477 .56741 L
Mistroke
.77478 .5674 L
.77478 .56739 L
.77479 .56738 L
.77479 .56737 L
.7748 .56737 L
.77481 .56736 L
.77481 .56735 L
.77482 .56734 L
.77482 .56733 L
.77483 .56733 L
.77484 .56732 L
.77484 .56731 L
.77485 .5673 L
.77486 .56729 L
.77486 .56728 L
.77487 .56727 L
.77488 .56726 L
.77488 .56725 L
.77489 .56724 L
.7749 .56723 L
.7749 .56722 L
.77491 .56721 L
.77492 .5672 L
.77492 .5672 L
.77493 .56719 L
.77493 .56718 L
.77494 .56717 L
.77494 .56716 L
.77495 .56716 L
.77495 .56715 L
.77496 .56714 L
.77497 .56713 L
.77497 .56712 L
.77498 .56712 L
.77498 .56711 L
.77499 .5671 L
.77499 .56709 L
.775 .56708 L
.77501 .56708 L
.77501 .56707 L
.77502 .56706 L
.77502 .56705 L
.77503 .56704 L
.77503 .56703 L
.77504 .56702 L
.77505 .56701 L
.77506 .567 L
.77506 .567 L
.77507 .56699 L
.77507 .56698 L
Mistroke
.77508 .56697 L
.77508 .56696 L
.77509 .56696 L
.7751 .56694 L
.7751 .56693 L
.77511 .56693 L
.77511 .56692 L
.77512 .56691 L
.77513 .5669 L
.77513 .56689 L
.77514 .56688 L
.77514 .56687 L
.77515 .56687 L
.77515 .56686 L
.77516 .56685 L
.77517 .56684 L
.77517 .56683 L
.77518 .56682 L
.77519 .56681 L
.77519 .5668 L
.7752 .56679 L
.7752 .56678 L
.77521 .56677 L
.77522 .56676 L
.77522 .56676 L
.77523 .56675 L
.77523 .56674 L
.77524 .56673 L
.77525 .56672 L
.77525 .56671 L
.77526 .5667 L
.77526 .56669 L
.77527 .56669 L
.77527 .56668 L
.77528 .56666 L
.77529 .56665 L
.7753 .56664 L
.77531 .56663 L
.77531 .56662 L
.77532 .56661 L
.77532 .5666 L
.77533 .5666 L
.77533 .56658 L
.77534 .56657 L
.77535 .56657 L
.77535 .56656 L
.77536 .56655 L
.77537 .56653 L
.77537 .56652 L
.77538 .56652 L
Mistroke
.77538 .56651 L
.77539 .5665 L
.7754 .56648 L
.77541 .56647 L
.77541 .56646 L
.77542 .56645 L
.77542 .56645 L
.77543 .56643 L
.77544 .56642 L
.77544 .56641 L
.77545 .5664 L
.77545 .56639 L
.77546 .56638 L
.77547 .56637 L
.77547 .56636 L
.77548 .56635 L
.77549 .56634 L
.77549 .56633 L
.7755 .56632 L
.77551 .56631 L
.77551 .5663 L
.77552 .5663 L
.77552 .56628 L
.77553 .56627 L
.77554 .56626 L
.77554 .56626 L
.77555 .56625 L
.77555 .56623 L
.77556 .56622 L
.77557 .56622 L
.77557 .5662 L
.77558 .56619 L
.77558 .56619 L
.77559 .56617 L
.7756 .56616 L
.77561 .56615 L
.77561 .56614 L
.77562 .56613 L
.77562 .56612 L
.77563 .56611 L
.77563 .5661 L
.77564 .56609 L
.77565 .56608 L
.77566 .56607 L
.77566 .56606 L
.77567 .56605 L
.77567 .56604 L
.77568 .56602 L
.77569 .56601 L
.77569 .566 L
Mistroke
.7757 .56599 L
.77571 .56598 L
.77571 .56597 L
.77572 .56596 L
.77573 .56595 L
.77573 .56594 L
.77574 .56593 L
.77575 .56591 L
.77576 .5659 L
.77576 .56589 L
.77577 .56588 L
.77578 .56586 L
.77579 .56585 L
.77579 .56584 L
.7758 .56583 L
.77581 .56581 L
.77581 .5658 L
.77582 .56579 L
.77582 .56578 L
.77583 .56578 L
.77584 .56576 L
.77584 .56575 L
.77585 .56574 L
.77585 .56573 L
.77586 .56572 L
.77587 .56571 L
.77587 .5657 L
.77588 .56569 L
.77589 .56568 L
.77589 .56567 L
.7759 .56565 L
.77591 .56564 L
.77591 .56563 L
.77592 .56561 L
.77593 .5656 L
.77593 .56559 L
.77594 .56558 L
.77595 .56557 L
.77595 .56556 L
.77597 .56554 L
.77597 .56553 L
.77598 .56552 L
.77599 .5655 L
.776 .56548 L
.776 .56547 L
.77601 .56546 L
.77601 .56545 L
.77603 .56543 L
.77603 .56542 L
.77604 .56541 L
Mistroke
.77604 .5654 L
.77605 .56539 L
.77606 .56537 L
.77607 .56536 L
.77607 .56535 L
.77608 .56534 L
.77608 .56533 L
.7761 .56531 L
.7761 .5653 L
.77611 .56529 L
.77611 .56527 L
.77612 .56526 L
.77613 .56525 L
.77613 .56524 L
.77614 .56522 L
.77615 .56521 L
.77615 .5652 L
.77616 .56519 L
.77617 .56517 L
.77618 .56516 L
.77618 .56515 L
.77619 .56514 L
.77619 .56513 L
.7762 .56512 L
.77621 .56511 L
.77621 .5651 L
.77622 .56509 L
.77623 .56507 L
.77623 .56506 L
.77624 .56505 L
.77625 .56503 L
.77625 .56502 L
.77626 .56501 L
.77627 .565 L
.77627 .56498 L
.77628 .56496 L
.77629 .56495 L
.7763 .56494 L
.7763 .56493 L
.77631 .56492 L
.77632 .5649 L
.77633 .56489 L
.77633 .56487 L
.77634 .56486 L
.77634 .56485 L
.77635 .56484 L
.77635 .56483 L
.77636 .56481 L
.77637 .5648 L
.77638 .56479 L
Mistroke
.77638 .56478 L
.77639 .56477 L
.7764 .56475 L
.7764 .56474 L
.77641 .56473 L
.77642 .56471 L
.77643 .56469 L
.77644 .56467 L
.77645 .56466 L
.77645 .56465 L
.77646 .56463 L
.77647 .56461 L
.77648 .56459 L
.77649 .56457 L
.7765 .56456 L
.77651 .56454 L
.77651 .56453 L
.77652 .56451 L
.77653 .56449 L
.77654 .56448 L
.77654 .56447 L
.77655 .56445 L
.77656 .56444 L
.77657 .56443 L
.77658 .5644 L
.77658 .56439 L
.77659 .56438 L
.77659 .56437 L
.7766 .56436 L
.77661 .56433 L
.77662 .56431 L
.77663 .56429 L
.77664 .56428 L
.77665 .56425 L
.77666 .56423 L
.77667 .56422 L
.77668 .56421 L
.77669 .56419 L
.77669 .56418 L
.7767 .56416 L
.77671 .56414 L
.77672 .56413 L
.77672 .56412 L
.77673 .5641 L
.77673 .56409 L
.77674 .56407 L
.77675 .56406 L
.77676 .56405 L
.77677 .56403 L
.77677 .56401 L
Mistroke
.77678 .564 L
.77679 .56398 L
.7768 .56396 L
.7768 .56395 L
.77681 .56394 L
.77682 .56393 L
.77683 .5639 L
.77683 .56389 L
.77684 .56388 L
.77685 .56386 L
.77686 .56385 L
.77686 .56383 L
.77687 .56382 L
.77688 .56379 L
.77689 .56378 L
.77689 .56377 L
.7769 .56375 L
.77691 .56374 L
.77691 .56373 L
.77692 .56371 L
.77693 .56369 L
.77694 .56368 L
.77694 .56366 L
.77695 .56365 L
.77696 .56363 L
.77697 .56361 L
.77698 .56359 L
.77699 .56358 L
.777 .56355 L
.777 .56354 L
.77701 .56352 L
.77702 .5635 L
.77703 .56348 L
.77704 .56346 L
.77705 .56345 L
.77706 .56342 L
.77707 .56341 L
.77707 .5634 L
.77708 .56338 L
.77709 .56337 L
.7771 .56334 L
.77711 .56332 L
.77712 .56329 L
.77713 .56328 L
.77713 .56326 L
.77715 .56324 L
.77716 .56322 L
.77717 .56319 L
.77718 .56317 L
.77719 .56315 L
Mistroke
.7772 .56312 L
.77721 .5631 L
.77722 .56308 L
.77723 .56306 L
.77724 .56303 L
.77725 .56301 L
.77726 .56299 L
.77727 .56296 L
.77729 .56294 L
.7773 .56291 L
.77731 .56289 L
.77732 .56286 L
.77733 .56285 L
.77733 .56284 L
.77734 .56281 L
.77736 .56278 L
.77737 .56276 L
.77738 .56273 L
.77739 .56271 L
.7774 .56268 L
.77741 .56266 L
.77742 .56264 L
.77743 .56263 L
.77744 .5626 L
.77745 .56257 L
.77746 .56255 L
.77747 .56253 L
.77748 .56251 L
.77749 .56249 L
.7775 .56246 L
.77751 .56244 L
.77752 .56241 L
.77753 .56239 L
.77754 .56237 L
.77755 .56235 L
.77756 .56233 L
.77757 .5623 L
.77758 .56227 L
.77759 .56225 L
.7776 .56223 L
.77761 .5622 L
.77762 .56218 L
.77763 .56216 L
.77764 .56214 L
.77765 .56212 L
.77766 .56209 L
.77767 .56207 L
.77768 .56205 L
.77769 .56203 L
.7777 .562 L
Mistroke
.77771 .56198 L
.77772 .56196 L
.77773 .56194 L
.77775 .5619 L
.77776 .56188 L
.77776 .56186 L
.77777 .56184 L
.77778 .56182 L
.77779 .56179 L
.7778 .56177 L
.77781 .56174 L
.77782 .56172 L
.77783 .5617 L
.77784 .56168 L
.77785 .56165 L
.77786 .56162 L
.77788 .5616 L
.77789 .56157 L
.7779 .56155 L
.77791 .56153 L
.77792 .5615 L
.77793 .56148 L
.77794 .56145 L
.77795 .56142 L
.77796 .5614 L
.77797 .56137 L
.77798 .56135 L
.77799 .56133 L
.778 .5613 L
.77801 .56128 L
.77802 .56125 L
.77803 .56122 L
.77804 .5612 L
.77805 .56117 L
.77806 .56115 L
.77808 .56112 L
.77809 .56109 L
.7781 .56106 L
.77811 .56104 L
.77812 .56102 L
.77813 .561 L
.77814 .56097 L
.77815 .56094 L
.77816 .56092 L
.77817 .5609 L
.77818 .56087 L
.77819 .56083 L
.7782 .5608 L
.77821 .56078 L
.77822 .56076 L
Mistroke
.77823 .56074 L
.77825 .5607 L
.77826 .56067 L
.77827 .56065 L
.77828 .56062 L
.77829 .56059 L
.7783 .56057 L
.77831 .56054 L
.77832 .56051 L
.77833 .56049 L
.77834 .56046 L
.77835 .56044 L
.77836 .56041 L
.77837 .56039 L
.77839 .56034 L
.7784 .56032 L
.77841 .56029 L
.77843 .56025 L
.77844 .56022 L
.77844 .5602 L
.77845 .56018 L
.77846 .56016 L
.77848 .56012 L
.77849 .5601 L
.7785 .56007 L
.77851 .56004 L
.77852 .56002 L
.77853 .55998 L
.77854 .55995 L
.77855 .55993 L
.77856 .5599 L
.77857 .55988 L
.77859 .55983 L
.7786 .55981 L
.77861 .55978 L
.77862 .55975 L
.77863 .55973 L
.77864 .55971 L
.77865 .55968 L
.77866 .55964 L
.77867 .55961 L
.77868 .55959 L
.7787 .55954 L
.77871 .55951 L
.77872 .55948 L
.77873 .55945 L
.77875 .55942 L
.77876 .55939 L
.77877 .55936 L
.77878 .55933 L
Mistroke
.77879 .5593 L
.7788 .55927 L
.77881 .55924 L
.77883 .55921 L
.77884 .55918 L
.77886 .55913 L
.77887 .5591 L
.77888 .55907 L
.77889 .55904 L
.7789 .55901 L
.77891 .55898 L
.77892 .55895 L
.77893 .55892 L
.77895 .55889 L
.77896 .55885 L
.77897 .55882 L
.77898 .55879 L
.77899 .55876 L
.77901 .55873 L
.77902 .55869 L
.77903 .55866 L
.77904 .55862 L
.77906 .55859 L
.77907 .55856 L
.77908 .55853 L
.77909 .55849 L
.7791 .55846 L
.77911 .55843 L
.77913 .55838 L
.77914 .55835 L
.77916 .55832 L
.77917 .55829 L
.77918 .55826 L
.77919 .55822 L
.7792 .55819 L
.77922 .55816 L
.77923 .55813 L
.77924 .5581 L
.77925 .55806 L
.77926 .55803 L
.77927 .558 L
.77928 .55797 L
.77929 .55794 L
.77931 .55788 L
.77933 .55785 L
.77934 .55782 L
.77935 .55779 L
.77936 .55776 L
.77937 .55771 L
.77938 .55768 L
Mistroke
.77939 .55766 L
.7794 .55763 L
.77942 .5576 L
.77942 .55757 L
.77944 .55754 L
.77945 .55749 L
.77946 .55746 L
.77947 .55743 L
.77949 .55738 L
.7795 .55735 L
.77951 .55732 L
.77952 .5573 L
.77953 .55727 L
.77955 .55722 L
.77956 .55717 L
.77958 .55714 L
.77958 .55711 L
.77959 .55709 L
.77961 .55705 L
.77962 .55702 L
.77962 .557 L
.77964 .55694 L
.77965 .55691 L
.77967 .55688 L
.77968 .55682 L
.7797 .55679 L
.77971 .55676 L
.77972 .55673 L
.77973 .55669 L
.77975 .55664 L
.77976 .55661 L
.77977 .55658 L
.77979 .55653 L
.7798 .5565 L
.77981 .55647 L
.77983 .55641 L
.77984 .55638 L
.77985 .55635 L
.77987 .55629 L
.77988 .55625 L
.77989 .55622 L
.7799 .55619 L
.77991 .55616 L
.77993 .5561 L
.77994 .55607 L
.77995 .55603 L
.77997 .55597 L
.77998 .55594 L
.78 .55591 L
.78002 .55585 L
Mistroke
.78003 .55581 L
.78004 .55578 L
.78005 .55575 L
.78006 .55571 L
.78007 .55568 L
.78008 .55564 L
.7801 .55558 L
.78012 .55555 L
.78013 .55551 L
.78014 .55547 L
.78015 .55544 L
.78018 .55536 L
.78019 .55532 L
.7802 .55528 L
.78022 .55522 L
.78024 .55518 L
.78025 .55514 L
.78026 .55511 L
.78027 .55508 L
.78029 .55502 L
.7803 .55498 L
.78031 .55495 L
.78033 .55488 L
.78034 .55485 L
.78035 .55482 L
.78038 .55475 L
.78039 .55471 L
.7804 .55468 L
.78042 .55461 L
.78043 .55457 L
.78044 .55454 L
.78046 .5545 L
.78047 .55446 L
.78049 .55438 L
.78051 .55434 L
.78052 .5543 L
.78053 .55427 L
.78054 .55423 L
.78056 .55417 L
.78057 .55413 L
.78058 .5541 L
.78061 .55402 L
.78062 .55398 L
.78063 .55394 L
.78065 .5539 L
.78066 .55386 L
.78067 .55382 L
.78068 .55378 L
.78071 .55371 L
.78072 .55367 L
Mistroke
.78073 .55363 L
.78075 .55359 L
.78076 .55354 L
.78079 .55346 L
.7808 .55342 L
.78081 .55337 L
.78082 .55334 L
.78083 .5533 L
.78086 .55323 L
.78087 .55319 L
.78088 .55315 L
.78089 .55312 L
.7809 .55308 L
.78093 .55301 L
.78095 .55294 L
.78097 .55287 L
.78098 .55283 L
.78099 .5528 L
.78101 .55273 L
.78103 .55266 L
.78105 .5526 L
.78107 .55254 L
.78109 .55247 L
.78111 .5524 L
.78113 .55233 L
.78115 .55226 L
.78116 .55223 L
.78118 .55218 L
.78119 .55215 L
.7812 .55211 L
.78122 .55203 L
.78124 .55196 L
.78126 .55192 L
.78127 .55187 L
.78128 .55183 L
.7813 .55179 L
.78131 .55175 L
.78132 .55171 L
.78134 .55164 L
.78136 .55156 L
.78139 .55149 L
.7814 .55145 L
.78141 .55141 L
.78143 .55133 L
.78145 .55126 L
.78148 .55118 L
.7815 .55112 L
.78152 .55104 L
.78154 .55097 L
.78156 .55091 L
Mistroke
.78158 .55084 L
.7816 .55076 L
.78162 .55068 L
.78165 .5506 L
.78167 .55053 L
.78169 .55046 L
.78171 .5504 L
.78173 .55032 L
.78175 .55025 L
.78177 .55018 L
.78179 .55012 L
.78181 .55006 L
.78183 .54999 L
.78184 .54992 L
.78187 .54984 L
.78189 .54977 L
.78191 .54969 L
.78193 .54962 L
.78195 .54955 L
.78197 .54948 L
.78199 .54941 L
.78201 .54933 L
.78203 .54926 L
.78206 .54918 L
.78208 .5491 L
.7821 .54903 L
.78212 .54894 L
.78215 .54885 L
.78217 .54877 L
.7822 .54869 L
.78221 .54862 L
.78223 .54856 L
.78225 .54849 L
.78227 .54842 L
.78229 .54836 L
.7823 .5483 L
.78232 .54823 L
.78234 .54817 L
.78236 .54809 L
.78238 .54802 L
.7824 .54795 L
.78242 .54789 L
.78244 .54782 L
.78246 .54774 L
.78248 .54766 L
.7825 .54759 L
.78252 .54751 L
.78254 .54743 L
.78256 .54736 L
.78258 .54729 L
Mistroke
.78262 .54716 L
.78264 .54709 L
.78266 .54701 L
.78268 .54693 L
.7827 .54686 L
.78272 .54678 L
.78274 .5467 L
.78276 .54662 L
.78278 .54655 L
.7828 .54648 L
.78283 .5464 L
.78285 .54631 L
.78287 .54623 L
.7829 .54614 L
.78292 .54604 L
.78295 .54594 L
.78297 .54585 L
.783 .54576 L
.78302 .54567 L
.78305 .54558 L
.78307 .5455 L
.78309 .5454 L
.78312 .54532 L
.78314 .54523 L
.78316 .54516 L
.78318 .54508 L
.7832 .54499 L
.78323 .54489 L
.78325 .5448 L
.78328 .5447 L
.7833 .54461 L
.78333 .54452 L
.78335 .54444 L
.78337 .54435 L
.7834 .54425 L
.78342 .54415 L
.78345 .54405 L
.78347 .54395 L
.7835 .54386 L
.78352 .54376 L
.78355 .54365 L
.78358 .54354 L
.78361 .54343 L
.78363 .54334 L
.78365 .54325 L
.78368 .54316 L
.7837 .54306 L
.78373 .54297 L
.78375 .54289 L
.78377 .5428 L
Mistroke
.78379 .54271 L
.78382 .54262 L
.78384 .54253 L
.78386 .54245 L
.78388 .54236 L
.78391 .54228 L
.78393 .5422 L
.78396 .54204 L
.78399 .54195 L
.78401 .54185 L
.78406 .54167 L
.78408 .54158 L
.78411 .54148 L
.78413 .54138 L
.78416 .54127 L
.78418 .54118 L
.78421 .54108 L
.78423 .54099 L
.78425 .54089 L
.78428 .5408 L
.7843 .54071 L
.78434 .54054 L
.78436 .54045 L
.78439 .54037 L
.78441 .54027 L
.78443 .54018 L
.78447 .54001 L
.7845 .53991 L
.78452 .53983 L
.78454 .53975 L
.78456 .53966 L
.7846 .53951 L
.78462 .53943 L
.78464 .53935 L
.78468 .53917 L
.7847 .53908 L
.78473 .53898 L
.78475 .53889 L
.78477 .53879 L
.78479 .53871 L
.78482 .53862 L
.78486 .53845 L
.78488 .53835 L
.7849 .53826 L
.78493 .53816 L
.78495 .53805 L
.785 .53787 L
.78502 .53777 L
.78504 .53768 L
.78509 .53751 L
Mistroke
.78511 .53741 L
.78513 .53732 L
.78517 .53716 L
.78521 .53699 L
.78525 .53683 L
.78528 .53669 L
.78532 .53653 L
.78536 .53636 L
.7854 .53617 L
.78543 .53607 L
.78545 .53598 L
.78549 .5358 L
.78554 .53561 L
.78558 .53543 L
.78562 .53526 L
.78566 .5351 L
.7857 .53493 L
.78574 .53477 L
.78578 .53458 L
.78583 .53439 L
.78587 .5342 L
.78591 .53403 L
.78594 .53388 L
.78598 .53371 L
.78603 .53353 L
.78607 .53334 L
.78611 .53315 L
.78616 .53296 L
.7862 .53279 L
.78624 .53259 L
.78629 .53238 L
.78634 .53217 L
.78638 .53199 L
.78642 .53182 L
.78646 .53165 L
.7865 .53148 L
.78653 .53133 L
.78657 .53116 L
.78662 .53097 L
.78666 .53078 L
.7867 .53061 L
.78674 .53042 L
.78678 .53024 L
.78682 .53008 L
.78686 .52991 L
.7869 .52972 L
.78694 .52952 L
.78698 .52934 L
.78703 .52916 L
.78706 .52898 L
Mistroke
.78711 .5288 L
.78714 .52863 L
.78718 .52846 L
.78722 .5283 L
.78726 .52812 L
.78733 .52782 L
.78737 .52763 L
.7874 .52746 L
.78745 .52728 L
.78749 .52707 L
.78753 .52689 L
.78757 .52672 L
.78761 .52655 L
.78764 .52638 L
.78768 .5262 L
.78772 .52604 L
.78779 .52572 L
.78785 .52543 L
.78792 .52512 L
.78796 .52496 L
.788 .52478 L
.78806 .52447 L
.7881 .52428 L
.78814 .5241 L
.78822 .52377 L
.78826 .52359 L
.7883 .52341 L
.78837 .52307 L
.78841 .52287 L
.78846 .52266 L
.78854 .52229 L
.78858 .5221 L
.78862 .5219 L
.7887 .52154 L
.78875 .52132 L
.78879 .52111 L
.78888 .52072 L
.78895 .52037 L
.78903 .52001 L
.7891 .5197 L
.78917 .51936 L
.78925 .51899 L
.78933 .51859 L
.78941 .51822 L
.78948 .51788 L
.78957 .51748 L
.78966 .51706 L
.78974 .51666 L
.78982 .5163 L
.78991 .51587 L
Mistroke
.79001 .51541 L
.7901 .51497 L
.79018 .51457 L
.79026 .51419 L
.79033 .51384 L
.79041 .51345 L
.7905 .51304 L
.79059 .5126 L
.79068 .51219 L
.79076 .51182 L
.79084 .51142 L
.79098 .51075 L
.79111 .51013 L
.79118 .50977 L
.79126 .50939 L
.7914 .50871 L
.79155 .508 L
.79168 .50733 L
.79183 .50661 L
.79199 .50583 L
.79213 .50517 L
.79228 .50444 L
.79242 .50376 L
.79254 .50313 L
.79279 .50194 L
.79291 .50134 L
.79304 .50068 L
.7932 .4999 L
.79335 .49916 L
.79363 .49778 L
.79393 .4963 L
.79426 .49469 L
.79454 .49328 L
.79484 .49179 L
.79537 .48913 L
.79649 .4836 L
.79764 .47786 L
.79824 .47488 L
.79855 .47331 L
.79889 .47165 L
.7992 .47012 L
.79948 .46874 L
.79974 .46743 L
.80002 .46605 L
.80035 .46442 L
.80065 .46293 L
.80098 .46133 L
.80116 .46042 L
.80133 .45958 L
.80164 .45806 L
Mistroke
.80193 .45668 L
.80208 .45592 L
.80225 .45509 L
.80256 .45359 L
.80272 .45282 L
.80287 .45209 L
.80315 .45073 L
.8033 .44997 L
.80345 .44928 L
.80361 .44847 L
.80377 .44771 L
.80403 .44647 L
.80417 .44581 L
.80431 .4451 L
.80446 .44437 L
.80461 .44367 L
.80475 .443 L
.80488 .44238 L
.80514 .44114 L
.80527 .44049 L
.80542 .43981 L
.80565 .43867 L
.80579 .43803 L
.80592 .43743 L
.80606 .43672 L
.8062 .43606 L
.80646 .43483 L
.80661 .43414 L
.80674 .43351 L
.80689 .43281 L
.80705 .43208 L
.80719 .43142 L
.80731 .43082 L
.80746 .43012 L
.8076 .42946 L
.80785 .4283 L
.80799 .42765 L
.80812 .42704 L
.80827 .42632 L
.80842 .42565 L
.80855 .42505 L
.80869 .4244 L
.80884 .42369 L
.809 .42295 L
.80914 .42231 L
.80929 .42162 L
.80942 .42103 L
.80955 .42039 L
.80969 .41979 L
.80981 .41923 L
Mistroke
.81003 .41819 L
.81015 .41766 L
.81028 .41708 L
.81041 .41646 L
.81054 .41589 L
.81068 .41523 L
.81082 .41461 L
.81094 .41404 L
.81106 .41351 L
.81119 .41293 L
.81132 .41232 L
.81145 .41175 L
.81156 .41122 L
.81169 .41065 L
.81183 .41004 L
.81205 .40902 L
.81218 .40845 L
.8123 .40791 L
.81241 .4074 L
.81252 .40692 L
.81272 .40601 L
.81283 .40551 L
.81294 .40505 L
.81305 .40454 L
.81317 .404 L
.8133 .40344 L
.81343 .40286 L
.81355 .40232 L
.81366 .40182 L
.8138 .40123 L
.81392 .40069 L
.81406 .40008 L
.81419 .3995 L
.81431 .39899 L
.81443 .39846 L
.81464 .3975 L
.81477 .39697 L
.81488 .39648 L
.815 .39594 L
.81513 .39536 L
.81528 .39474 L
.81541 .39416 L
.81555 .39354 L
.81571 .39287 L
.81585 .39223 L
.81599 .39164 L
.81611 .39111 L
.81624 .39053 L
.81637 .39001 L
.8165 .38943 L
Mistroke
.81664 .38883 L
.81677 .38828 L
.81689 .38776 L
.817 .38728 L
.81713 .38674 L
.81725 .38622 L
.81737 .38569 L
.81749 .38519 L
.81762 .38464 L
.81776 .38405 L
.81789 .38347 L
.81804 .38286 L
.81816 .38234 L
.81829 .38177 L
.81842 .38124 L
.81854 .38075 L
.81867 .3802 L
.81879 .37967 L
.81901 .37874 L
.81914 .37822 L
.81925 .37774 L
.81938 .37722 L
.81951 .37667 L
.81963 .37614 L
.81975 .37567 L
.81988 .37511 L
.82003 .3745 L
.82016 .37393 L
.82029 .37341 L
.82042 .37287 L
.82054 .37237 L
.82067 .37182 L
.82082 .37122 L
.82096 .37064 L
.82111 .37002 L
.82124 .3695 L
.82138 .36893 L
.82149 .36847 L
.82159 .36804 L
.82172 .36753 L
.82183 .36707 L
.82194 .36662 L
.82206 .36613 L
.82219 .36561 L
.82231 .36512 L
.82252 .36425 L
.82264 .36376 L
.82275 .36332 L
.82285 .36291 L
.82296 .36247 L
Mistroke
.82316 .36167 L
.82328 .36119 L
.82339 .36074 L
.82351 .36025 L
.82364 .35972 L
.82378 .35917 L
.82391 .35866 L
.82403 .35819 L
.82415 .35769 L
.82438 .35678 L
.8245 .35632 L
.82462 .35581 L
.82473 .35539 L
.82485 .35493 L
.82506 .3541 L
.82516 .35369 L
.82527 .35325 L
.82547 .35247 L
.82559 .35199 L
.8257 .35154 L
.82592 .3507 L
.82603 .35028 L
.82614 .34982 L
.82635 .34901 L
.82646 .34856 L
.82657 .34815 L
.82669 .34769 L
.82681 .3472 L
.82692 .34678 L
.82702 .3464 L
.82712 .346 L
.82721 .34564 L
.82739 .34496 L
.82749 .34458 L
.82758 .34423 L
.82768 .34384 L
.82778 .34344 L
.82797 .34271 L
.82808 .34231 L
.82817 .34194 L
.82828 .34153 L
.8284 .34109 L
.82849 .34073 L
.82859 .34033 L
.82878 .33963 L
.82888 .33924 L
.82899 .33882 L
.82919 .33806 L
.82928 .3377 L
.82939 .33731 L
Mistroke
.82948 .33696 L
.82958 .33659 L
.82976 .33589 L
.82993 .33525 L
.83011 .33457 L
.83021 .33419 L
.83031 .33384 L
.83048 .3332 L
.83066 .33252 L
.83082 .33192 L
.83097 .33135 L
.83113 .33074 L
.83131 .33008 L
.83141 .32971 L
.83151 .32932 L
.8317 .32862 L
.83181 .32823 L
.8319 .32787 L
.83201 .32746 L
.83213 .32704 L
.83232 .32632 L
.8325 .32567 L
.83269 .32497 L
.8328 .32457 L
.8329 .3242 L
.83308 .32352 L
.83326 .32287 L
.83342 .32229 L
.83359 .32167 L
.83377 .32099 L
.83387 .32065 L
.83397 .32028 L
.83415 .31962 L
.83432 .319 L
.83452 .31828 L
.83463 .31788 L
.83473 .31752 L
.83493 .31681 L
.83511 .31615 L
.83522 .31577 L
.83533 .31538 L
.83552 .31467 L
.83563 .31428 L
.83574 .31391 L
.83585 .31352 L
.83597 .31309 L
.83607 .31271 L
.83617 .31237 L
.83636 .31169 L
.83653 .31108 L
Mistroke
.83671 .31044 L
.83691 .30975 L
.83709 .3091 L
.83729 .3084 L
.8374 .308 L
.8375 .30764 L
.83767 .30706 L
.83784 .30643 L
.83801 .30585 L
.83816 .30532 L
.83834 .3047 L
.83853 .30402 L
.83871 .30339 L
.83888 .30281 L
.83905 .30221 L
.83921 .30166 L
.83939 .30105 L
.83958 .30039 L
.83977 .29971 L
.83996 .29907 L
.84013 .2985 L
.84031 .29787 L
.84051 .29718 L
.84063 .29678 L
.84073 .29643 L
.84092 .29577 L
.84113 .29508 L
.84131 .29446 L
.84148 .29388 L
.84166 .29325 L
.84186 .29258 L
.84197 .29221 L
.84207 .29187 L
.84218 .2915 L
.8423 .2911 L
.8425 .29042 L
.84261 .29005 L
.84271 .28971 L
.84291 .28905 L
.84308 .28846 L
.84329 .28778 L
.84348 .28713 L
.84366 .28652 L
.84382 .28598 L
.84401 .28535 L
.84419 .28476 L
.84436 .28421 L
.84451 .28371 L
.84467 .28316 L
.84485 .28258 L
Mistroke
.84502 .282 L
.84521 .28138 L
.84539 .28079 L
.84555 .28026 L
.84574 .27964 L
.84594 .27897 L
.84614 .27834 L
.84632 .27776 L
.84647 .27725 L
.84662 .27678 L
.84678 .27626 L
.84694 .27571 L
.84712 .27514 L
.84729 .2746 L
.84744 .27411 L
.8476 .27359 L
.84774 .27314 L
.84789 .27264 L
.84806 .27212 L
.84821 .27164 L
.84847 .27078 L
.84862 .2703 L
.84876 .26986 L
.84892 .26935 L
.84909 .2688 L
.84925 .26829 L
.84939 .26783 L
.84957 .26729 L
.84975 .2667 L
.84992 .26615 L
.85008 .26565 L
.85023 .26518 L
.85037 .26474 L
.85052 .26425 L
.85069 .26374 L
.85086 .2632 L
.85102 .2627 L
.85116 .26224 L
.85132 .26175 L
.85146 .26133 L
.8516 .26087 L
.85187 .26004 L
.85202 .25957 L
.85218 .25907 L
.85231 .25865 L
.85246 .25818 L
.85262 .25769 L
.85276 .25725 L
.85293 .25674 L
.85309 .25625 L
Mistroke
.85336 .2554 L
.85352 .25492 L
.85366 .25448 L
.85378 .2541 L
.85391 .2537 L
.85415 .25297 L
.8543 .25252 L
.85443 .2521 L
.85469 .25132 L
.85482 .25092 L
.85496 .2505 L
.85521 .24974 L
.85535 .24933 L
.85547 .24893 L
.85577 .24804 L
.85591 .24761 L
.85607 .24714 L
.85634 .24631 L
.85651 .2458 L
.85667 .24532 L
.85697 .24443 L
.85712 .24398 L
.85728 .24349 L
.85757 .24263 L
.85773 .24216 L
.85787 .24172 L
.85804 .24123 L
.85821 .24071 L
.8585 .23987 L
.85866 .2394 L
.8588 .23896 L
.85907 .23816 L
.85922 .23771 L
.85936 .23731 L
.85962 .23655 L
.85976 .23613 L
.8599 .23573 L
.86014 .23502 L
.86026 .23466 L
.8604 .23427 L
.86055 .23381 L
.86072 .23331 L
.86087 .23288 L
.86103 .23242 L
.86117 .232 L
.86133 .23155 L
.8616 .23075 L
.86177 .23027 L
.86192 .22983 L
.86209 .22935 L
Mistroke
.86226 .22884 L
.86256 .22799 L
.86272 .22752 L
.86288 .22707 L
.86302 .22666 L
.86318 .22622 L
.86331 .22583 L
.86345 .22542 L
.86358 .22505 L
.86373 .22464 L
.86397 .22393 L
.86413 .2235 L
.86426 .22311 L
.86441 .22269 L
.86457 .22223 L
.86472 .22181 L
.86486 .22142 L
.865 .22102 L
.86513 .22065 L
.8654 .2199 L
.86554 .2195 L
.86569 .21908 L
.86584 .21864 L
.86599 .21822 L
.86613 .21785 L
.86627 .21745 L
.86653 .21672 L
.86676 .21606 L
.86702 .21534 L
.86716 .21494 L
.8673 .21456 L
.86744 .21416 L
.8676 .21373 L
.86773 .21335 L
.86788 .21295 L
.86802 .21255 L
.86815 .21219 L
.86841 .21149 L
.86856 .21107 L
.8687 .21068 L
.86885 .21027 L
.86901 .20981 L
.86929 .20905 L
.86944 .20864 L
.86958 .20825 L
.86984 .20756 L
.86998 .20717 L
.87011 .2068 L
.87035 .20615 L
.87061 .20545 L
Mistroke
.87083 .20483 L
.87105 .20425 L
.87128 .20362 L
.87153 .20294 L
.87178 .20226 L
.87205 .20153 L
.87219 .20116 L
.87234 .20076 L
.8726 .20005 L
.87286 .19935 L
.87311 .19868 L
.87333 .19809 L
.87357 .19745 L
.87379 .19687 L
.87402 .19624 L
.87424 .19566 L
.87444 .19513 L
.87468 .19451 L
.87493 .19383 L
.87517 .1932 L
.87539 .19261 L
.87564 .19195 L
.87588 .19134 L
.87608 .19079 L
.87631 .19019 L
.87656 .18954 L
.87682 .18886 L
.87704 .18829 L
.87728 .18766 L
.8775 .18707 L
.87772 .18652 L
.87791 .18602 L
.87811 .18549 L
.87833 .18491 L
.87857 .1843 L
.87877 .18377 L
.87899 .1832 L
.8792 .18267 L
.87939 .18219 L
.87959 .18166 L
.87981 .1811 L
.88017 .18017 L
.88037 .17965 L
.88056 .17916 L
.88078 .1786 L
.88102 .17799 L
.88127 .17735 L
.88151 .17675 L
.88173 .1762 L
.88192 .17569 L
Mistroke
.88215 .17511 L
.88237 .17457 L
.8826 .17398 L
.88285 .17335 L
.88306 .17282 L
.88329 .17224 L
.88354 .17162 L
.88381 .17095 L
.88406 .17032 L
.88428 .16974 L
.88448 .16924 L
.8847 .16871 L
.88488 .16825 L
.88508 .16776 L
.88531 .16718 L
.88553 .16663 L
.88575 .1661 L
.88594 .16561 L
.88613 .16514 L
.88634 .16462 L
.88657 .16407 L
.88678 .16354 L
.88701 .16296 L
.88724 .16241 L
.88744 .16192 L
.88765 .16138 L
.88786 .16089 L
.88807 .16036 L
.88826 .15989 L
.88847 .1594 L
.88865 .15895 L
.88884 .15847 L
.88919 .15762 L
.8894 .1571 L
.8896 .15663 L
.8898 .15613 L
.89003 .15559 L
.89024 .15508 L
.89043 .15462 L
.89061 .15418 L
.8908 .15371 L
.89114 .15289 L
.89146 .15212 L
.89165 .15168 L
.89184 .15122 L
.89218 .15039 L
.89236 .14997 L
.89254 .14953 L
.89288 .14873 L
.89308 .14825 L
Mistroke
.89327 .14779 L
.89363 .14694 L
.89381 .1465 L
.89398 .14609 L
.89417 .14566 L
.89437 .14518 L
.89472 .14435 L
.89505 .14356 L
.89526 .14308 L
.89544 .14264 L
.89564 .14217 L
.89586 .14167 L
.89605 .1412 L
.89624 .14077 L
.89659 .13993 L
.89677 .13951 L
.89696 .13907 L
.89731 .13826 L
.89752 .13777 L
.89771 .13731 L
.89808 .13645 L
.89827 .13601 L
.89847 .13555 L
.89883 .13472 L
.89902 .13427 L
.89921 .13384 L
.89963 .13287 L
.89994 .13215 L
.90028 .13136 L
.90063 .13055 L
.90096 .1298 L
.90127 .12909 L
.90154 .12846 L
.90186 .12773 L
.90216 .12704 L
.90249 .1263 L
.90283 .12551 L
.90313 .12485 L
.90345 .12412 L
.90379 .12334 L
.90416 .12251 L
.9045 .12173 L
.90481 .12103 L
.90513 .1203 L
.90543 .11963 L
.90576 .11889 L
.90611 .1181 L
.90646 .11732 L
.90682 .11651 L
.90714 .11581 L
Mistroke
.90747 .11506 L
.90779 .11435 L
.90808 .11371 L
.90841 .11298 L
.90872 .11228 L
.90901 .11165 L
.90927 .11106 L
.90956 .11042 L
.90987 .10975 L
.9102 .10902 L
.9105 .10836 L
.91084 .1076 L
.91117 .10688 L
.91147 .10622 L
.91175 .10561 L
.91205 .10495 L
.91237 .10424 L
.91272 .10347 L
.9131 .10264 L
.91346 .10187 L
.91379 .10116 L
.91408 .10052 L
.9144 .09982 L
.91474 .09909 L
.91505 .09842 L
.91536 .09776 L
.91568 .09706 L
.91626 .09581 L
.91658 .09513 L
.91687 .09448 L
.9172 .09378 L
.91755 .09303 L
.91814 .09177 L
.91844 .09113 L
.91876 .09045 L
.91904 .08986 L
.91933 .08923 L
.91985 .08812 L
.92014 .08751 L
.9204 .08695 L
.92069 .08635 L
.921 .0857 L
.92151 .0846 L
.9218 .084 L
.92206 .08345 L
.92256 .0824 L
.92301 .08145 L
.92352 .08037 L
.92401 .07934 L
.92429 .07877 L
Mistroke
.92458 .07815 L
.92511 .07705 L
.92562 .07597 L
.92609 .075 L
.9266 .07395 L
.92715 .0728 L
.92743 .07223 L
.92772 .07161 L
.92825 .07052 L
.92875 .0695 L
.92928 .0684 L
.92978 .06737 L
.93024 .06643 L
.93065 .06558 L
.9311 .06466 L
.93157 .0637 L
.932 .06281 L
.93252 .06176 L
.93299 .0608 L
.9335 .05975 L
.93405 .05864 L
.93457 .05759 L
.93505 .0566 L
.93548 .05573 L
.93595 .05479 L
.93638 .05393 L
.93684 .05299 L
.93732 .05202 L
.93777 .05112 L
.93857 .04952 L
.93901 .04863 L
.93941 .04782 L
.94029 .04607 L
.94073 .04519 L
.94121 .04423 L
.94162 .04342 L
.94205 .04255 L
.94284 .04099 L
.94328 .04011 L
.94376 .03915 L
.94423 .03823 L
.94473 .03725 L
.94519 .03633 L
.94561 .03551 L
.94642 .03391 L
.94681 .03314 L
.94724 .0323 L
.94765 .03149 L
.94809 .03062 L
.94887 .02908 L
Mistroke
.9496 .02766 L
.95004 .02681 L
.95051 .0259 L
.95135 .02424 L
.95217 .02265 L
.95293 .02118 L
.95372 .01964 L
.95457 .01799 L
.95537 .01645 L
.95609 .01505 L
.95674 .0138 L
.95745 .01243 L
.95818 .01102 L
.95886 .00972 L
.95959 .00831 L
.96038 .0068 L
.96112 .00539 L
.9618 .0041 L
.96306 .00168 L
.96369 .00047 L
Mfstroke
.96369 .00047 m
.96394 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  Background->GrayLevel[1],
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005X0000gR000`400?l00000o`00003oo`3ooomY
0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY
0?ooo`00C03oool5000000<0oooo100000050?ooo`800000903oool5000000<0oooo100000040?oo
o`@000001@3oool2000001d0oooo1@0000030?ooo`@00000103oool4000000@0oooo100000050?oo
o`800000503oool5000000<0oooo100000040?ooo`@00000103oool4000000@0oooo100000050?oo
o`800000303oool5000000<0oooo100000040?ooo`@00000103oool4000000@0oooo100000040?oo
o`@000001@3oool2000000`0oooo001>0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`020?ooo`8000009P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`020?oo
o`8000007`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?oo
o`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0080oooo0P00
000F0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?oo
o`0000040?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`00
0000oooo0?ooo`020?ooo`8000003P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`040000003o
ool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`020?ooo`80
0000303oool004h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000
003oool0oooo000000@0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`000004
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool0
10000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0
403oool004h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003o
ool0oooo000000@0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?oo
o`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?oo
o`0000040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool01000
0000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0403o
ool004h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003oool0
oooo000000@0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`04
0000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`00
00040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000
oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0403oool0
04h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo
000000@0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000
003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`000004
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo
0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0403oool004`0
oooo0`0000040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0:03oool3000000@0
oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo
0240oooo0`0000040?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool0
10000000oooo0?ooo`0000040?ooo`030000003oool0oooo01P0oooo0`0000040?ooo`030000003o
ool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`04
0000003oool0oooo000000@0oooo00<000000?ooo`3oool0403oool3000000@0oooo00<000000?oo
o`3oool00P3oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00@0
00000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0100
oooo001>0?ooo`030000003oool0oooo00<0oooo1000000]0?ooo`030000003oool0oooo00<0oooo
100000040?ooo`@000009P3oool00`000000oooo0?ooo`030?ooo`@00000103oool4000000@0oooo
1000000M0?ooo`030000003oool0oooo00<0oooo100000040?ooo`@00000103oool4000000@0oooo
1000000E0?ooo`030000003oool0oooo00<0oooo100000040?ooo`@00000103oool4000000@0oooo
100000040?ooo`@000004`3oool00?l0ooooJ@3oool00?l0ooooJ@3oool00?l0ooooJ@3oool00?l0
ooooJ@3oool00?l0ooooJ@3oool00?l0ooooJ@3oool001X0ooooo`00000N000000030000o`000000
000000T000009@3oool001X0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool00`3oool01@000000oooo0?ooo`3oool0
000000<0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool000000140oooo00<0
00000?ooo`3oool0203oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo00D0
00000?ooo`3oool0oooo000000030?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000
oooo0000000A0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`030?ooo`050000003oool0oooo0?ooo`0000000`3oool01@000000oooo0?ooo`3oool0
00000080oooo00@000000?ooo`3oool00000403oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool00`3oool01@000000oooo0?ooo`3oool0000000<0oooo
00D000000?ooo`3oool0oooo000000020?ooo`040000003oool0oooo00000100oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool00P3oool01@000000003o0?ooo`3oool000000080oooo00@000000?ooo`3oool000000P3o
ool00`000000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool0
0`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`000?l0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0
oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3o
ool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`030000O`3oool0oooo0080oooo00D0003o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool001X0oooo00<000000?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@000?l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`006P3oool200000100
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o
0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o
0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`060000O`3oool0oooo0?ooo`000?l0001o0`3oool0100007l0oooo0?ooo`00
07l20?ooo`040000O`3oool00000000002D0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0
oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3o
ool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`060000O`3oool0oooo0?ooo`000?l0001o0`3oool0100007l0oooo0?ooo`0007l20?oo
o`040000O`3oool0oooo000002D0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000
O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?oo
o`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`060000O`3oool0oooo0000o`3oool0001o0`3oool0100007l0oooo0?ooo`0007l20?ooo`040000
O`3oool0oooo000002D0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?oo
o`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0
001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000
O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`060000
O`3oool0oooo0000o`3oool0001o0`3oool0100007l0oooo0?ooo`0007l20?ooo`040000O`3oool0
oooo000002D0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000
O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?oo
o`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
003o0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<000000?ooo`3oool08`3oool001X0oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0
oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0
001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0003o0000
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo0080
00009@3oool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000
O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0
203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0000o`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
0000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool0
0`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000
O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0
oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool20000o`<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<00000
0?ooo`3oool08`3oool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0
001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0
oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0003o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`000000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool0
0`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
0`000?l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`800000403oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140
oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o
00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0
001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo0080oooo00<0
003o0000O`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0
001o0?ooo`02000002D0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?oo
o`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0
001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000
O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool0100007l0oooo0?ooo`3oool20000o`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<000000?ooo`3oool08`3oool001X0oooo00<000000?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3o
ool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`070000O`3oool0oooo
0?ooo`000?l0oooo0000O`040?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`030000003oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`00
07l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`020000o`030?ooo`0007l0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool001X0oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool0
0`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`04
0000O`3oool0oooo0000o`80oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`006P3oool2
00000100oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo
00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool0
100007l0oooo0000o`000?l20?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000U0?ooo`006P3oool00`000000oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?oo
o`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`000?l00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`03
0000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo
0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0000o`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool0
0140oooo100000050?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`00
07l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0003o0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
00000?ooo`3oool08`3oool00100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool00`0007l0003o0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0
8`3oool00100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`08
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3o
ool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00
oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`000?l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool00100oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`<00000o`0007l>0000O`030000
o`0007l0001o01<0001o0`00000U0?ooo`00403oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3o
ool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A
0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`030?ooo`80003o1@3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0
oooo000@0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?oo
o`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00<0oooo
00<0003o0000O`3oool0103oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000@0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o
0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo0080oooo0P000?l00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool00140oooo100000050?oo
o`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0
oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool00P3oool00`000?l0oooo0000O`050?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3o
ool001X0oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`040000O`3oool0
oooo0?ooo`80003o00<0oooo0000O`3oool0103oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo008000009@3oool001X0oooo
00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool01@0007l0oooo
0?ooo`3oool0003o0080oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0
oooo02<0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`00
07l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool00P000?l20?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`000000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool0
0`0007l0oooo0?ooo`080?ooo`040000O`3oool0oooo0000o`<0oooo00<0001o0?ooo`3oool00`3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`030000003oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`00
07l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l@0?ooo`030000O`3oool0oooo00P0oooo00@0001o0?ooo`000?l0003o0`3oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool001X0oooo0P00000@0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0
001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0003o00@0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
1@0007l0oooo0?ooo`0007l0oooo008000009@3oool001X0oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000
O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo
0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0000o`040?ooo`030000O`3oool0
oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`03
0000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`000?l0003o00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0000o`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool001X0
oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000
O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool20000o`D0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?oo
o`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`000?l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000002D0oooo000J0?ooo`030000003oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo
00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00L0oooo0P000?l60?ooo`030000O`3oool0
oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`03
0000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`070?ooo`030000o`0007l0oooo00D0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00L0oooo00<0003o0000O`3oool0
1@3oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool001X0
oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000
O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool01P3oool20000o`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0
oooo02<0oooo000J0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`03
0000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool01P3oool00`00
0?l0oooo0000O`060?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000002D0oooo000J
0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00D0oooo0P000?l0
0`3oool0001o0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo
0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0
oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0
001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?oo
o`050?ooo`040000o`3oool0oooo0000O`H0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000003oool0oooo02<0oooo000A0?ooo`@000001@3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o
0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o403oool00`0007l0oooo0?ooo`050?ooo`040000o`3oool0oooo0000O`H0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000@0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool0
0`0007l0oooo0?ooo`040?ooo`80003o0P3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`005@3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool0
0`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00@0oooo00D0
003o0?ooo`3oool0oooo0000O`060?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
0000oooo0?ooo`0S0?ooo`005@3oool00`000000oooo0?ooo`020?ooo`<00000o`0007l20000O`80
003o800007l3000002D0oooo000A0?ooo`@000001@3oool00`000000oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?oo
o`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
403oool00`0007l0oooo0?ooo`030?ooo`030000o`3oool0oooo0080oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool00140
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool0
0`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000
O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0
oooo0?ooo`030?ooo`030000o`3oool0oooo0080oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool00140oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140
oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`020?oo
o`80003o103oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`000000oooo0?ooo`0S0?ooo`004@3oool5000000@0oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000
O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo
0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000Oa00oooo00<0001o0?ooo`3oool00P3oool00`000?l0oooo0?ooo`030?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo
000J0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000Oa00oooo00@0001o0?ooo`3oool0oooo0P000?l50?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo008000009@3oool001X0
oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000
O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00D0001o0?ooo`3oool0oooo0000o`060?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003o
ool0oooo02<0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000
O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?oo
o`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`050000O`3oool0
oooo0?ooo`000?l01P3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`000000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?oo
o`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
403oool00`0007l0oooo0?ooo`020000o`H0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool001X0oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool0
0`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000Oa00oooo00@0001o0?ooo`3oool0003o1`3oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`006P3oool2
00000100oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo
00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l@0?ooo`040000O`3oool0oooo0000o`L0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000U0?ooo`006P3oool00`000000oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?oo
o`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o403oool0100007l0oooo0000o`000?l70?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?oo
o`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0
oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0003o00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3o
ool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0
203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?oo
o`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`000?l0203oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo
0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0
oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0
001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0003o0000
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000003oool0oooo02<0oooo000J0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000
O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0000o`3oool0
203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`020000
02D0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`000?l0oooo00P0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<00000
0?ooo`3oool08`3oool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0
001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0
oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0003o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`000000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool0
0`000?l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0
oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3o
ool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l@0?ooo`030000o`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool001X0oooo00<000000?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`l0oooo00<0003o0000O`3oool02@3oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`006P3oool200000100
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o
0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o
0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l?0?ooo`030000o`0007l0oooo00T0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`050000O`3oool0oooo0000O`3oool00P00000U0?ooo`006P3oool00`000000oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0
001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o3`3oool00`000?l0001o0?ooo`090?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo00090?ooo`D000000`3oool4
000000D0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0
203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?oo
o`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`h0oooo00<0003o0?ooo`0007l02P3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo
0?ooo`0S0?ooo`002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0
oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0
001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o3P3oool00`000?l0oooo0000
O`0:0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000003oool0oooo02<0oooo000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?oo
o`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0
001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l>0?ooo`030000
o`3oool0001o00X0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<000000?ooo`3oool08`3oool000/0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo0080oooo0`00003h0000O`80003o:@0007l3000002D0
oooo000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l=0?ooo`040000o`3oool0oooo0000O`X0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<00000
0?ooo`3oool08`3oool000/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0
001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0
oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`d0oooo00@0003o0?oo
o`3oool0001o2P3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`000000oooo0?ooo`0S0?ooo`002@3oool3000000@0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`03
0000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o
0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l<0?ooo`80003o0P3o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000;0?ooo`030000003oool0oooo00<0oooo
100000050?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l<0?ooo`050000o`3oool0oooo0?ooo`00
07l02P3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`000000oooo0?ooo`0S0?ooo`006P3oool200000100oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`00
07l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l<0?ooo`050000o`3oool0oooo
0?ooo`0007l02P3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0001o
0?ooo`02000002D0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`03
0000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o
0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l;0?ooo`80003o0`3o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`030000003oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`00
07l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l;0?ooo`030000o`3oool0oooo0080oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?oo
o`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`08
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3o
ool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo
00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o2`3oool00`000?l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<000000?ooo`3oool08`3oool001X0oooo00<000000?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3o
ool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`/0oooo00<0003o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J
0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`X0oooo0P000?l40?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000U0?ooo`006P3o
ool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o
00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o
0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o
0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o2P3oool00`000?l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<000000?ooo`3oool08`3oool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o
0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`X0oooo
00<0003o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`03
0000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo
0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l90?ooo`80003o1@3oool00`0007l0oooo0?ooo`080?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0
oooo02<0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`00
07l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l90?ooo`030000o`3oool0oooo
00@0oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`006P3oool200000100oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o
0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo
0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0
203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l9
0?ooo`030000o`3oool0oooo00@0oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000002D0oooo000J0?ooo`030000003o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`08
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3o
ool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00
oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l90?ooo`030000o`3oool0oooo00@0oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000
oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3o
ool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o203oool20000o`H0oooo
00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o
0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o203oool00`000?l0oooo0?ooo`050?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool0
01X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`03
0000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`P0oooo00<0003o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o
0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo
0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0
203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l7
0?ooo`80003o1`3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`800000403o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`03
0000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0
oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`L0oooo00<0003o0?ooo`3oool01P3oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo008000009@3oool001X0oooo
00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`L0oooo00<0003o0?ooo`3oool01P3oool00`0007l0oooo
0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000003oool0oooo02<0oooo00090?ooo`D000000`3oool4000000D0oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool0
0`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`L0oooo00<0003o0?ooo`3oool01P3oool00`0007l0oooo0?ooo`080?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0
oooo02<0oooo000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`00
07l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l60?ooo`80003o203oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000003oool0oooo02<0oooo000;0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo
0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o
0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o1P3oool00`000?l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool000/0oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`020?ooo`<00000l@0007l00`000?l0001o0000O`0_0000O`<000009@3o
ool000/0oooo00<000000?ooo`3oool00`3oool4000000D0oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000
O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo
0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`H0oooo00<0003o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo
000;0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`03
0000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o1@3oool20000o`T0oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`00
2@3oool3000000D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?oo
o`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
1@3oool00`000?l0oooo0?ooo`080?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool000/0
oooo00<000000?ooo`3oool00`3oool5000000@0oooo00<000000?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0
001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`08
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3o
ool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`D0
oooo00<0003o0?ooo`3oool0203oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?oo
o`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`D0oooo00<0003o0?ooo`3oool0203oool00`0007l0oooo0?ooo`080?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo008000009@3o
ool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0
203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?oo
o`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`@0oooo0P000?l:0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<000000?ooo`3oool08`3oool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140
oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o
00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`@0oooo00<0
003o0?ooo`3oool02@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l40?ooo`030000o`3oool0oooo00T0oooo00<0001o0?ooo`3oool0
203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
0000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool0
0`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000
O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o103oool00`000?l0
oooo0?ooo`090?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool001X0oooo0P00000@0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0
001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o103oool00`000?l0oooo0?ooo`090?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000U0?ooo`006P3oool0
0`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0`3oool00`000?l0oooo0?ooo`0:0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<000000?ooo`3oool08`3oool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140
oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o
00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`<0oooo00<0
003o0?ooo`3oool02P3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l30?ooo`030000o`3oool0oooo00X0oooo00<0001o0?ooo`3oool0
203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
0000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool0
0`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000
O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0`3oool00`000?l0
oooo0?ooo`0:0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool001X0oooo00<000000?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo0P000?l<0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3o
ool001X0oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000?l0oooo0?ooo`0;0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool0
0P00000U0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0
oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0
001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000?l0oooo0?oo
o`0;0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool001X0oooo00<000000?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0003o0?ooo`3oool02`3oool00`0007l0oooo0?ooo`080?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo
02<0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo0080003o3@3oool00`0007l0
oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000003oool0oooo02<0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000
O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01P0007l0oooo0?ooo`0007l0oooo
0000o`h0oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`006P3oool200000100oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0
oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3o
ool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01P0007l0oooo0?ooo`00
07l0oooo0000o`h0oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00D0001o0?ooo`3oool0001o0?ooo`02000002D0oooo000J0?ooo`030000003oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o
0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o
0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01P0007l0oooo
0?ooo`0007l0oooo0000o`h0oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`00203oool5
000000@0oooo100000050?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000
O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000
O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?oo
o`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool01P0007l0oooo0?ooo`0007l0oooo0000o`h0oooo00<0
001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`000000oooo0?ooo`0S0?ooo`00203oool01@000000oooo0?ooo`3oool0000000<0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o
0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o
0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20000o`h0oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`002@3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3o
ool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00D0001o0?ooo`3oool0001o0000o`0?0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3o
ool000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo0`00001a0000O`030000o`0007l0003o0140003oIP0007l00`000?l0001o0000O`0e
0000O`<000009@3oool000/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0080oooo]P000?l=0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0001o0000o`0?0?ooo`030000
O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<000000?ooo`3oool08`3oool000`0oooo00D000000?ooo`3oool0oooo000000040?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0
001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0
oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00<0001o0?ooo`3oool03@000?l40?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0001o0000o`0?0?ooo`03
0000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<000000?ooo`3oool08`3oool000P0oooo00D000000?ooo`3oool0oooo00000003
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l90?ooo`P0003o
2P3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0?ooo`020000
o`l0oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`002@3oool3000000D0oooo10000005
0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`l0oooo1@000?l70?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`006P3o
ool200000100oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`08
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3o
ool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00
oooo00<0001o0?ooo`000?l010000?l40?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000002D0oooo000J0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
Oa00oooo00<0001o0?ooo`3oool00P3oool40000o`80oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0003o403oool00`0007l0oooo0?ooo`080?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo
02<0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool01@3oool30000o`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000oa00oooo00<0001o0?ooo`3o
ool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`000000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`03
0000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00L0oooo0`00
0?l50?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0000o`000?l@0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
00000?ooo`3oool08`3oool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3o
ool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo
00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`00
07l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`<0003o
103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`000?l0001o403oool00`0007l0oooo0?ooo`08
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
003oool0oooo02<0oooo000J0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3o
ool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A
0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0003o0080
003o0P3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`000?l0001o403oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo0080
00009@3oool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000
O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00<0003o00<0oooo0000O`3oool0103oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0003o0000Oa00oooo00<0001o0?ooo`3oool0
203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
0000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?oo
o`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool0
0`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000
O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00@0001o
0?ooo`3oool0oooo0`000?l00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`000?l0001o403oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o
0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo
0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0
203oool00`0007l0oooo0?ooo`020?ooo`<0003o1@3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`000?l0oooo0000Oa00oooo00<0001o
0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`000000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`00
07l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0
oooo00<0001o0?ooo`3oool00`3oool30000o`@0oooo00D0001o0?ooo`3oool0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0003o0?ooo`0007l@0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<000000?ooo`3oool08`3oool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o
0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?oo
o`030000O`3oool0oooo00@0oooo0P000?l40?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0000o`3oool0001o403oool00`0007l0oooo
0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000003oool0oooo02<0oooo000J0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o
00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o
0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o
0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo
0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0
oooo00@0oooo00<0001o0000o`000?l00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`000?l0oooo0000Oa00oooo00<0001o0?ooo`3o
ool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`02
000002D0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`00
07l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`040000O`3oool0003o0000o`80oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0003o0?ooo`0007l@0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<000000?ooo`3oool08`3oool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140
oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
0`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00P000?l00`3oool0001o0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool00`0007l0oooo0?ooo`020000o`030?ooo`0007l0oooo
00l0oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000
O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`020000o`030?oo
o`0007l0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`040000O`3oool0oooo0000
o`80oooo00<0001o0?ooo`3oool03P3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J
0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`040000
O`3oool0oooo0?ooo`80003o00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool010000?l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3o
ool08`3oool001X0oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0
001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`020?ooo`030000o`0007l0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`040000O`3oool0oooo0000o`80oooo00<0001o0?ooo`3oool03P3oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo0080
00009@3oool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000
O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00P3oool20000o`<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`040000O`3oool0oooo0000o`80oooo00<0001o0?ooo`3oool03P3oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000003oool0oooo02<0oooo00080?ooo`D00000103oool4000000D0oooo00<000000?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
0?l0oooo0?ooo`3oool0001o00<0oooo00P0001o0?ooo`3oool0oooo0000O`3oool0003o0000O`80
oooo00<0001o0?ooo`3oool03P3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo00080?oo
o`050000003oool0oooo0?ooo`0000000`3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0
203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0
oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?oo
o`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool20000o`80oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`04
0000O`3oool0003o0000O`80oooo00<0001o0?ooo`3oool03P3oool00`0007l0oooo0?ooo`080?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003o
ool0oooo02<0oooo00090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?oo
o`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?oo
o`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`050000O`000?l0003o0?ooo`0007l00`3oool0200007l0oooo0?ooo`3oool0001o
0?ooo`000?l0001o0P3oool00`0007l0oooo0?ooo`0>0?ooo`030000O`3oool0oooo00P0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3o
ool08`3oool000X0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`020?ooo`<00000
f@0007l20000o`/0001o00<0003o0000O`0007l0>P0007l3000002D0oooo000;0?ooo`030000003o
ool0oooo00<0oooo100000050?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`03
0000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o
0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`000?l0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P000?l0100007l0oooo0?ooo`0007l@0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<000000?ooo`3oool08`3oool000`0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3o
ool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00D0001o0?ooo`000?l0003o0000O`030?ooo`050000O`3oool0
oooo0?ooo`0007l00P000?l0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?oo
o`3oool08`3oool000P0oooo00D000000?ooo`3oool0oooo000000040?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`03
0000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`000?l0001o00<0oooo00P0001o0?ooo`3oool0
oooo0000O`000?l0oooo0000O`80oooo00<0001o0?ooo`3oool03P3oool00`0007l0oooo0?ooo`08
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
003oool0oooo02<0oooo00090?ooo`<000001@3oool5000000@0oooo00<000000?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`03
0000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0
oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0003o00<0oooo00P0001o0?ooo`3oool0oooo0000O`000?l0oooo0000O`80oooo
00<0001o0?ooo`3oool03P3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`80
0000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool0
0`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
1@0007l0oooo0?ooo`3oool0003o00<0oooo00@0001o0?ooo`3oool0oooo0P000?l01@3oool0001o
0?ooo`3oool0001o0100oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000002D0oooo000J0?ooo`030000003oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o
00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0
001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00@0
001o0?ooo`3oool0oooo0P000?l20?ooo`040000O`3oool0oooo0?ooo`80003o00D0oooo0000O`3o
ool0oooo0000O`0@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool001X0oooo00<00000
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo
00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo
0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool01P0007l0oooo0?ooo`3oool0001o0000o`80oooo00D0001o0?ooo`3oool0oooo0000o`02
0?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`00
6P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0
001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00P000?l0103oool0001o0?ooo`3o
ool20000o`80oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo
02<0oooo000J0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l0
4@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00H0001o0?ooo`3oool0oooo0000O`3oool20000o`030000
O`3oool0oooo0080003o0P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<00000
0?ooo`3oool08`3oool001X0oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?oo
o`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140
oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`060000O`3oool0oooo0?ooo`0007l0oooo0P000?l01@0007l0
oooo0?ooo`000?l0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`08
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo00800000
9@3oool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3o
ool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A
0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo0P000?l0103o
ool0003o0000o`0007l20?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000
oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3o
ool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool3
0000o`050?ooo`0007l0oooo0?ooo`0007l00P3oool00`0007l0oooo0?ooo`0>0?ooo`030000O`3o
ool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<000000?ooo`3oool08`3oool001X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o
0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo
00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o
0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0
oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0
oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`030000003oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o
00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0
001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool001X0oooo0P00000@
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`03
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000
O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000
O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool01@0007l0oooo0?ooo`0007l0oooo008000009@3oool001X0oooo00<000000?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3o
ool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`03
0000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0
oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0
oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`006P3oool00`000000oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l@0?oo
o`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0oooo000J0?ooo`03
0000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo
0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool0
01X0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o0?ooo`3oool0203o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo
0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`03
0000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`05
0000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?oo
o`0S0?ooo`006P3oool200000100oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`00
07l00`3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`00
07l0oooo0?ooo`080?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@00
07l0oooo0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000Oa00oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000U0?ooo`006P3o
ool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00<0001o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`080?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o
00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o0140oooo00<0001o
0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0001o
0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo02<0
oooo00090?ooo`<000001@3oool4000000D0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo
0?ooo`3oool0001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`030000O`3oool0001o
0140oooo00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000O`0A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o403oool00`0007l0oooo0?ooo`080?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0
001o00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000Oa00oooo
00<0001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`000000oooo0?ooo`0S0?ooo`00203oool01@000000oooo0?ooo`3oool00000
00<0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
00<0oooo00D0001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool00`0007l0oooo0000000A0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`050000O`3oool0oooo0?ooo`0007l00`3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00<0001o0?ooo`0000004@3oool00`0007l0oooo0?ooo`080?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool01@0007l0oooo0?ooo`3oool0001o
00<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo00000100oooo00<0
001o0?ooo`3oool0203oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00D0
001o0?ooo`3oool0oooo0000O`030?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`00000@0?ooo`030000O`3oool0oooo00P0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool08`3oool000`0oooo00D00000
0?ooo`3oool0oooo000000040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00`3o
ool01@000000oooo0?ooo`3oool0000000<0oooo00D000000?ooo`3oool0oooo000000020?ooo`03
0000003oool000000140oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00<0oooo00D000000?ooo`3oool0oooo000000030?ooo`050000003oool0oooo
0?ooo`0000000P3oool00`000000oooo0000000A0?ooo`030000003oool0oooo00P0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`030?ooo`050000003oool0oooo0?ooo`0000000`3o
ool01@000000oooo0?ooo`3oool000000080oooo00@000000?ooo`3oool00000403oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool00`3oool01@000000
oooo0?ooo`3oool0000000<0oooo00D000000?ooo`3oool0oooo000000020?ooo`040000003oool0
oooo00000100oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool00P3oool01@000000oooo0?ooo`3oool000000080oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0S0?ooo`00303oool01@000000oooo
0?ooo`3oool0000000@0oooo00<000000?ooo`3oool00P3ooooo000002X000009@3oool000X0oooo
0P0000040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o`3ooomA0?ooo`00303o
ool01@000000oooo0?ooo`3oool0000000@0oooo00<000000?ooo`3oool0o`3ooomA0?ooo`00203o
ool01@000000oooo0?ooo`3oool0000000<0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`3o0?oooe40oooo00090?ooo`<000001@3oool400000?l0ooooE03oool00?l0ooooJ@3oool0
0?l0ooooJ@3oool00?l0ooooJ@3oool00?l0ooooJ@3oool001T0oooo1@00000^0?ooo`D000000`3o
ool4000000D0oooo0P00000T0?ooo`D000000`3oool4000000@0oooo100000050?ooo`8000007@3o
ool5000000<0oooo100000040?ooo`@00000103oool4000000D0oooo0P00000D0?ooo`D000000`3o
ool4000000@0oooo100000040?ooo`@00000103oool4000000D0oooo0P00001;0?ooo`006`3oool0
0`000000oooo0?ooo`0`0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`020?ooo`8000009P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`020?ooo`8000007`3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000
103oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0080oooo0P00000F0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?oo
o`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?oo
o`020?ooo`800000B`3oool001/0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?oo
o`0S0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?oo
o`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool01000
0000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0C`3o
ool001/0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0S0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`040000
003oool0oooo000000@0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`000004
0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0C`3oool001/0oooo00<00000
0?ooo`3oool0<03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0:P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080
oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`040000003oool0
oooo000000@0oooo00<000000?ooo`3oool0C`3oool001/0oooo00<000000?ooo`3oool0<03oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0:P3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3o
ool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3o
ool00000103oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool0C`3oool001T0oooo0`00000`0?ooo`<00000103oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo02P0oooo0`0000040?ooo`030000003oool0oooo0080oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`0Q0?ooo`<00000103oool00`000000oooo0?oo
o`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`000000
oooo0?ooo`0H0?ooo`<00000103oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo0000
00@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000003o
ool0oooo04l0oooo000K0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool00`3oool4
000002d0oooo00<000000?ooo`3oool00`3oool4000000@0oooo1000000V0?ooo`030000003oool0
oooo00<0oooo100000040?ooo`@00000103oool4000001d0oooo00<000000?ooo`3oool00`3oool4
000000@0oooo100000040?ooo`@00000103oool400000580oooo003o0?ooofT0oooo003o0?ooofT0
oooo003o0?ooofT0oooo0000\
\>"],
  ImageRangeCache->{{{0, 359}, {221.375, 0}} -> {-0.452893, -8.71828, \
0.021156, 0.239618}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(BodePhasePlot[gco[2\ Pi\ I\ s]\  /. \ Param2, {s, \ 1, \ 100000}, \ 
        Frame\  -> \ True, PlotRange\  -> \ {{0, \ 5}, \ {0, \ \(-270\)}}, 
        DisplayFunction\  -> \ Identity, 
        PlotStyle\  -> \ RGBColor[0, \ 0, \ 1], 
        GridLines\  -> \ {LogGridMinor, \ Automatic}];\)\)], "Input",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-017 0.2 0.618034 0.00228901 [
[.2 -0.0125 -11 -12.5625 ]
[.2 -0.0125 11 0 ]
[.4 -0.0125 -14 -12.5625 ]
[.4 -0.0125 14 0 ]
[.6 -0.0125 -17 -12.5625 ]
[.6 -0.0125 17 0 ]
[.8 -0.0125 -20 -12.5625 ]
[.8 -0.0125 20 0 ]
[1 -0.0125 -23 -12.5625 ]
[1 -0.0125 23 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .04578 -24 -4.5 ]
[-0.0125 .04578 0 4.5 ]
[-0.0125 .16023 -24 -4.5 ]
[-0.0125 .16023 0 4.5 ]
[-0.0125 .27468 -24 -4.5 ]
[-0.0125 .27468 0 4.5 ]
[-0.0125 .38913 -24 -4.5 ]
[-0.0125 .38913 0 4.5 ]
[-0.0125 .50358 -18 -4.5 ]
[-0.0125 .50358 0 4.5 ]
[-0.0125 .61803 -6 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 -0.125 0 ]
[0 .63053 -5 0 ]
[0 .63053 5 12.5625 ]
[.2 .63053 -11 0 ]
[.2 .63053 11 12.5625 ]
[.4 .63053 -14 0 ]
[.4 .63053 14 12.5625 ]
[.6 .63053 -17 0 ]
[.6 .63053 17 12.5625 ]
[.8 .63053 -20 0 ]
[.8 .63053 20 12.5625 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 -0.0005 0 0 ]
[ 1 .61853 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 0 m
0 .61803 L
s
.06021 0 m
.06021 .61803 L
s
.09542 0 m
.09542 .61803 L
s
.12041 0 m
.12041 .61803 L
s
.13979 0 m
.13979 .61803 L
s
.15563 0 m
.15563 .61803 L
s
.16902 0 m
.16902 .61803 L
s
.18062 0 m
.18062 .61803 L
s
.19085 0 m
.19085 .61803 L
s
.2 0 m
.2 .61803 L
s
.26021 0 m
.26021 .61803 L
s
.29542 0 m
.29542 .61803 L
s
.32041 0 m
.32041 .61803 L
s
.33979 0 m
.33979 .61803 L
s
.35563 0 m
.35563 .61803 L
s
.36902 0 m
.36902 .61803 L
s
.38062 0 m
.38062 .61803 L
s
.39085 0 m
.39085 .61803 L
s
.4 0 m
.4 .61803 L
s
.46021 0 m
.46021 .61803 L
s
.49542 0 m
.49542 .61803 L
s
.52041 0 m
.52041 .61803 L
s
.53979 0 m
.53979 .61803 L
s
.55563 0 m
.55563 .61803 L
s
.56902 0 m
.56902 .61803 L
s
.58062 0 m
.58062 .61803 L
s
.59085 0 m
.59085 .61803 L
s
.6 0 m
.6 .61803 L
s
.66021 0 m
.66021 .61803 L
s
.69542 0 m
.69542 .61803 L
s
.72041 0 m
.72041 .61803 L
s
.73979 0 m
.73979 .61803 L
s
.75563 0 m
.75563 .61803 L
s
.76902 0 m
.76902 .61803 L
s
.78062 0 m
.78062 .61803 L
s
.79085 0 m
.79085 .61803 L
s
.8 0 m
.8 .61803 L
s
.86021 0 m
.86021 .61803 L
s
.89542 0 m
.89542 .61803 L
s
.92041 0 m
.92041 .61803 L
s
.93979 0 m
.93979 .61803 L
s
.95563 0 m
.95563 .61803 L
s
.96902 0 m
.96902 .61803 L
s
.98062 0 m
.98062 .61803 L
s
.99085 0 m
.99085 .61803 L
s
1 0 m
1 .61803 L
s
0 .04578 m
1 .04578 L
s
0 .16023 m
1 .16023 L
s
0 .27468 m
1 .27468 L
s
0 .38913 m
1 .38913 L
s
0 .50358 m
1 .50358 L
s
0 g
.2 0 m
.2 .00625 L
s
gsave
.2 -0.0125 -72 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10.) show
81.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.4 0 m
.4 .00625 L
s
gsave
.4 -0.0125 -75 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100.) show
87.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.6 0 m
.6 .00625 L
s
gsave
.6 -0.0125 -78 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1000.) show
93.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.8 0 m
.8 .00625 L
s
gsave
.8 -0.0125 -81 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10000.) show
99.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
1 0 m
1 .00625 L
s
gsave
1 -0.0125 -84 -16.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100000.) show
105.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.001 w
.06021 0 m
.06021 .00375 L
s
.09542 0 m
.09542 .00375 L
s
.12041 0 m
.12041 .00375 L
s
.13979 0 m
.13979 .00375 L
s
.15563 0 m
.15563 .00375 L
s
.16902 0 m
.16902 .00375 L
s
.18062 0 m
.18062 .00375 L
s
.19085 0 m
.19085 .00375 L
s
.26021 0 m
.26021 .00375 L
s
.29542 0 m
.29542 .00375 L
s
.32041 0 m
.32041 .00375 L
s
.33979 0 m
.33979 .00375 L
s
.35563 0 m
.35563 .00375 L
s
.36902 0 m
.36902 .00375 L
s
.38062 0 m
.38062 .00375 L
s
.39085 0 m
.39085 .00375 L
s
.46021 0 m
.46021 .00375 L
s
.49542 0 m
.49542 .00375 L
s
.52041 0 m
.52041 .00375 L
s
.53979 0 m
.53979 .00375 L
s
.55563 0 m
.55563 .00375 L
s
.56902 0 m
.56902 .00375 L
s
.58062 0 m
.58062 .00375 L
s
.59085 0 m
.59085 .00375 L
s
.66021 0 m
.66021 .00375 L
s
.69542 0 m
.69542 .00375 L
s
.72041 0 m
.72041 .00375 L
s
.73979 0 m
.73979 .00375 L
s
.75563 0 m
.75563 .00375 L
s
.76902 0 m
.76902 .00375 L
s
.78062 0 m
.78062 .00375 L
s
.79085 0 m
.79085 .00375 L
s
.86021 0 m
.86021 .00375 L
s
.89542 0 m
.89542 .00375 L
s
.92041 0 m
.92041 .00375 L
s
.93979 0 m
.93979 .00375 L
s
.95563 0 m
.95563 .00375 L
s
.96902 0 m
.96902 .00375 L
s
.98062 0 m
.98062 .00375 L
s
.99085 0 m
.99085 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .04578 m
.00625 .04578 L
s
[(-250)] -0.0125 .04578 1 0 Mshowa
0 .16023 m
.00625 .16023 L
s
[(-200)] -0.0125 .16023 1 0 Mshowa
0 .27468 m
.00625 .27468 L
s
[(-150)] -0.0125 .27468 1 0 Mshowa
0 .38913 m
.00625 .38913 L
s
[(-100)] -0.0125 .38913 1 0 Mshowa
0 .50358 m
.00625 .50358 L
s
[(-50)] -0.0125 .50358 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(0)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .06867 m
.00375 .06867 L
s
0 .09156 m
.00375 .09156 L
s
0 .11445 m
.00375 .11445 L
s
0 .13734 m
.00375 .13734 L
s
0 .18312 m
.00375 .18312 L
s
0 .20601 m
.00375 .20601 L
s
0 .2289 m
.00375 .2289 L
s
0 .25179 m
.00375 .25179 L
s
0 .29757 m
.00375 .29757 L
s
0 .32046 m
.00375 .32046 L
s
0 .34335 m
.00375 .34335 L
s
0 .36624 m
.00375 .36624 L
s
0 .41202 m
.00375 .41202 L
s
0 .43491 m
.00375 .43491 L
s
0 .4578 m
.00375 .4578 L
s
0 .48069 m
.00375 .48069 L
s
0 .52647 m
.00375 .52647 L
s
0 .54936 m
.00375 .54936 L
s
0 .57225 m
.00375 .57225 L
s
0 .59514 m
.00375 .59514 L
s
0 .02289 m
.00375 .02289 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 .61178 m
0 .61803 L
s
gsave
0 .63053 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
69.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.2 .61178 m
.2 .61803 L
s
gsave
.2 .63053 -72 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10.) show
81.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.4 .61178 m
.4 .61803 L
s
gsave
.4 .63053 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100.) show
87.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.6 .61178 m
.6 .61803 L
s
gsave
.6 .63053 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1000.) show
93.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.8 .61178 m
.8 .61803 L
s
gsave
.8 .63053 -81 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 20.5625 translate 1 -1 scale
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10000.) show
99.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.001 w
.06021 .61428 m
.06021 .61803 L
s
.09542 .61428 m
.09542 .61803 L
s
.12041 .61428 m
.12041 .61803 L
s
.13979 .61428 m
.13979 .61803 L
s
.15563 .61428 m
.15563 .61803 L
s
.16902 .61428 m
.16902 .61803 L
s
.18062 .61428 m
.18062 .61803 L
s
.19085 .61428 m
.19085 .61803 L
s
.26021 .61428 m
.26021 .61803 L
s
.29542 .61428 m
.29542 .61803 L
s
.32041 .61428 m
.32041 .61803 L
s
.33979 .61428 m
.33979 .61803 L
s
.35563 .61428 m
.35563 .61803 L
s
.36902 .61428 m
.36902 .61803 L
s
.38062 .61428 m
.38062 .61803 L
s
.39085 .61428 m
.39085 .61803 L
s
.46021 .61428 m
.46021 .61803 L
s
.49542 .61428 m
.49542 .61803 L
s
.52041 .61428 m
.52041 .61803 L
s
.53979 .61428 m
.53979 .61803 L
s
.55563 .61428 m
.55563 .61803 L
s
.56902 .61428 m
.56902 .61803 L
s
.58062 .61428 m
.58062 .61803 L
s
.59085 .61428 m
.59085 .61803 L
s
.66021 .61428 m
.66021 .61803 L
s
.69542 .61428 m
.69542 .61803 L
s
.72041 .61428 m
.72041 .61803 L
s
.73979 .61428 m
.73979 .61803 L
s
.75563 .61428 m
.75563 .61803 L
s
.76902 .61428 m
.76902 .61803 L
s
.78062 .61428 m
.78062 .61803 L
s
.79085 .61428 m
.79085 .61803 L
s
.86021 .61428 m
.86021 .61803 L
s
.89542 .61428 m
.89542 .61803 L
s
.92041 .61428 m
.92041 .61803 L
s
.93979 .61428 m
.93979 .61803 L
s
.95563 .61428 m
.95563 .61803 L
s
.96902 .61428 m
.96902 .61803 L
s
.98062 .61428 m
.98062 .61803 L
s
.99085 .61428 m
.99085 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .04578 m
1 .04578 L
s
.99375 .16023 m
1 .16023 L
s
.99375 .27468 m
1 .27468 L
s
.99375 .38913 m
1 .38913 L
s
.99375 .50358 m
1 .50358 L
s
.125 Mabswid
.99625 .06867 m
1 .06867 L
s
.99625 .09156 m
1 .09156 L
s
.99625 .11445 m
1 .11445 L
s
.99625 .13734 m
1 .13734 L
s
.99625 .18312 m
1 .18312 L
s
.99625 .20601 m
1 .20601 L
s
.99625 .2289 m
1 .2289 L
s
.99625 .25179 m
1 .25179 L
s
.99625 .29757 m
1 .29757 L
s
.99625 .32046 m
1 .32046 L
s
.99625 .34335 m
1 .34335 L
s
.99625 .36624 m
1 .36624 L
s
.99625 .41202 m
1 .41202 L
s
.99625 .43491 m
1 .43491 L
s
.99625 .4578 m
1 .4578 L
s
.99625 .48069 m
1 .48069 L
s
.99625 .52647 m
1 .52647 L
s
.99625 .54936 m
1 .54936 L
s
.99625 .57225 m
1 .57225 L
s
.99625 .59514 m
1 .59514 L
s
.99625 .02289 m
1 .02289 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
.5 Mabswid
.00036 .61802 m
.01921 .61801 L
.03398 .61801 L
.0454 .61801 L
.05635 .618 L
.06683 .618 L
.07677 .61799 L
.08521 .61799 L
.09221 .61799 L
.09879 .61798 L
.10444 .61798 L
.11025 .61797 L
.11598 .61797 L
.12485 .61796 L
.12951 .61796 L
.13365 .61796 L
.13819 .61795 L
.14287 .61795 L
.14707 .61794 L
.15079 .61794 L
.154 .61793 L
.15741 .61793 L
.16083 .61793 L
.16392 .61792 L
.16721 .61792 L
.17054 .61791 L
.17329 .61791 L
.1762 .61791 L
.17924 .6179 L
.18237 .6179 L
.18523 .61789 L
.18774 .61789 L
.19013 .61788 L
.19268 .61788 L
.19526 .61787 L
.19761 .61787 L
.19976 .61787 L
.20167 .61786 L
.20382 .61786 L
.20582 .61785 L
.20792 .61785 L
.21011 .61784 L
.21212 .61784 L
.2139 .61784 L
.21596 .61783 L
.21813 .61783 L
.22013 .61782 L
.22192 .61782 L
.22346 .61781 L
.22507 .61781 L
Mistroke
.22657 .61781 L
.22791 .6178 L
.22945 .6178 L
.2311 .61779 L
.23263 .61779 L
.23402 .61778 L
.23543 .61778 L
.23673 .61778 L
.23813 .61777 L
.23962 .61777 L
.24107 .61776 L
.24257 .61776 L
.24383 .61776 L
.24519 .61775 L
.24646 .61775 L
.24761 .61774 L
.24885 .61774 L
.25015 .61773 L
.25126 .61773 L
.25229 .61773 L
.2534 .61772 L
.25458 .61772 L
.25561 .61771 L
.25672 .61771 L
.25775 .61771 L
.2587 .6177 L
.26046 .6177 L
.26134 .61769 L
.2623 .61769 L
.26334 .61768 L
.26442 .61768 L
.26533 .61768 L
.26631 .61767 L
.26737 .61767 L
.26848 .61766 L
.26952 .61766 L
.27045 .61766 L
.27138 .61765 L
.27223 .61765 L
.27316 .61764 L
.27413 .61764 L
.27496 .61763 L
.27573 .61763 L
.27657 .61763 L
.27746 .61762 L
.27827 .61762 L
.27911 .61762 L
.2799 .61761 L
.28061 .61761 L
.28145 .6176 L
Mistroke
.28234 .6176 L
.28317 .6176 L
.28393 .61759 L
.28529 .61758 L
.28605 .61758 L
.28676 .61758 L
.2875 .61757 L
.28817 .61757 L
.28893 .61757 L
.28965 .61756 L
.2909 .61755 L
.29152 .61755 L
.2922 .61755 L
.29339 .61754 L
.29449 .61753 L
.29512 .61753 L
.29577 .61753 L
.29693 .61752 L
.29756 .61752 L
.29815 .61751 L
.29879 .61751 L
.29947 .6175 L
.3002 .6175 L
.30088 .6175 L
.30162 .61749 L
.30241 .61749 L
.30322 .61748 L
.30399 .61748 L
.30467 .61747 L
.30541 .61747 L
.30611 .61746 L
.30673 .61746 L
.30745 .61745 L
.30811 .61745 L
.30873 .61745 L
.30929 .61744 L
.3099 .61744 L
.31055 .61743 L
.31125 .61743 L
.31188 .61742 L
.3126 .61742 L
.31329 .61741 L
.31391 .61741 L
.31448 .61741 L
.31511 .6174 L
.31576 .6174 L
.31642 .61739 L
.31711 .61739 L
.31777 .61738 L
.31836 .61738 L
Mistroke
.31905 .61737 L
.3198 .61737 L
.3205 .61736 L
.32114 .61735 L
.32173 .61735 L
.32227 .61735 L
.32285 .61734 L
.32347 .61734 L
.324 .61733 L
.3245 .61733 L
.32504 .61732 L
.32561 .61732 L
.32612 .61731 L
.32667 .61731 L
.32718 .61731 L
.32766 .6173 L
.32808 .6173 L
.32855 .61729 L
.32903 .61729 L
.32948 .61729 L
.32998 .61728 L
.33051 .61728 L
.33107 .61727 L
.33159 .61727 L
.33207 .61726 L
.3325 .61726 L
.333 .61726 L
.33347 .61725 L
.33398 .61725 L
.33451 .61724 L
.33497 .61724 L
.33546 .61723 L
.33599 .61723 L
.33656 .61722 L
.33709 .61722 L
.33758 .61721 L
.33804 .61721 L
.33848 .6172 L
.33887 .6172 L
.33929 .6172 L
.33974 .61719 L
.34022 .61719 L
.34063 .61718 L
.34107 .61718 L
.34148 .61718 L
.34186 .61717 L
.34229 .61717 L
.3427 .61716 L
.34307 .61716 L
.34342 .61716 L
Mistroke
.34379 .61715 L
.34419 .61715 L
.34462 .61714 L
.34501 .61714 L
.34543 .61714 L
.34588 .61713 L
.34627 .61713 L
.34662 .61712 L
.34702 .61712 L
.34743 .61711 L
.34789 .61711 L
.34837 .6171 L
.34883 .6171 L
.34925 .6171 L
.34963 .61709 L
.35004 .61709 L
.35047 .61708 L
.35087 .61708 L
.35125 .61707 L
.35159 .61707 L
.35197 .61707 L
.35237 .61706 L
.35272 .61706 L
.35304 .61705 L
.3534 .61705 L
.35378 .61705 L
.35409 .61704 L
.35443 .61704 L
.35479 .61703 L
.35512 .61703 L
.35572 .61702 L
.35604 .61702 L
.35634 .61702 L
.3567 .61701 L
.35707 .61701 L
.35739 .617 L
.35774 .617 L
.35811 .61699 L
.35851 .61699 L
.35888 .61699 L
.35922 .61698 L
.35956 .61698 L
.35987 .61697 L
.36022 .61697 L
.36058 .61696 L
.36093 .61696 L
.36125 .61696 L
.36154 .61695 L
.36186 .61695 L
.36217 .61694 L
Mistroke
.36251 .61694 L
.36283 .61694 L
.36312 .61693 L
.36338 .61693 L
.36367 .61693 L
.36398 .61692 L
.36426 .61692 L
.36457 .61691 L
.36489 .61691 L
.36517 .61691 L
.36547 .6169 L
.3658 .6169 L
.36614 .61689 L
.36647 .61689 L
.36676 .61689 L
.36704 .61688 L
.36733 .61688 L
.36786 .61687 L
.36814 .61687 L
.36841 .61686 L
.36871 .61686 L
.36902 .61686 L
.36929 .61685 L
.36959 .61685 L
.36986 .61684 L
.37012 .61684 L
.3706 .61683 L
.37087 .61683 L
.37111 .61683 L
.37139 .61682 L
.37171 .61682 L
.37204 .61681 L
.37234 .61681 L
.37263 .61681 L
.37288 .6168 L
.37318 .6168 L
.37346 .61679 L
.37376 .61679 L
.37408 .61678 L
.37436 .61678 L
.37465 .61678 L
.37497 .61677 L
.37532 .61677 L
.37564 .61676 L
.37593 .61676 L
.3762 .61675 L
.37649 .61675 L
.37676 .61675 L
.37701 .61674 L
.3773 .61674 L
Mistroke
.37761 .61673 L
.3779 .61673 L
.37817 .61672 L
.37842 .61672 L
.37865 .61672 L
.37891 .61671 L
.37918 .61671 L
.37947 .6167 L
.37974 .6167 L
.37998 .6167 L
.38024 .61669 L
.38051 .61669 L
.38076 .61669 L
.38103 .61668 L
.38131 .61668 L
.38156 .61667 L
.38179 .61667 L
.38204 .61666 L
.38231 .61666 L
.3826 .61666 L
.38291 .61665 L
.38321 .61665 L
.38348 .61664 L
.38373 .61664 L
.384 .61663 L
.38428 .61663 L
.38454 .61663 L
.38481 .61662 L
.38511 .61662 L
.38539 .61661 L
.38565 .61661 L
.38591 .6166 L
.38614 .6166 L
.38641 .61659 L
.38666 .61659 L
.38693 .61659 L
.38722 .61658 L
.38747 .61658 L
.38773 .61657 L
.38802 .61657 L
.38833 .61656 L
.38862 .61656 L
.38888 .61655 L
.38911 .61655 L
.38935 .61654 L
.38961 .61654 L
.38985 .61654 L
.39008 .61653 L
.39028 .61653 L
.3905 .61652 L
Mistroke
.39074 .61652 L
.39098 .61652 L
.39123 .61651 L
.39147 .61651 L
.39169 .6165 L
.39194 .6165 L
.39221 .61649 L
.39247 .61649 L
.39271 .61649 L
.39291 .61648 L
.39313 .61648 L
.39334 .61647 L
.39353 .61647 L
.39375 .61647 L
.39398 .61646 L
.39421 .61646 L
.39442 .61646 L
.39465 .61645 L
.39487 .61645 L
.39507 .61644 L
.39528 .61644 L
.39551 .61644 L
.39575 .61643 L
.39596 .61643 L
.39618 .61642 L
.3964 .61642 L
.39662 .61641 L
.39681 .61641 L
.39702 .61641 L
.39725 .6164 L
.39749 .6164 L
.39771 .61639 L
.39792 .61639 L
.39811 .61639 L
.39832 .61638 L
.39854 .61638 L
.39875 .61637 L
.39896 .61637 L
.39916 .61637 L
.39934 .61636 L
.39953 .61636 L
.3999 .61635 L
.40009 .61635 L
.40028 .61635 L
.40061 .61634 L
.40096 .61633 L
.40116 .61633 L
.40137 .61632 L
.40156 .61632 L
.40175 .61632 L
Mistroke
.40193 .61631 L
.40213 .61631 L
.40247 .6163 L
.40267 .6163 L
.40288 .61629 L
.4031 .61629 L
.40331 .61629 L
.40368 .61628 L
.40388 .61627 L
.40407 .61627 L
.40429 .61627 L
.40449 .61626 L
.40471 .61626 L
.40494 .61625 L
.40514 .61625 L
.40533 .61624 L
.40554 .61624 L
.40576 .61624 L
.40596 .61623 L
.40615 .61623 L
.40635 .61622 L
.40657 .61622 L
.40677 .61621 L
.40697 .61621 L
.40714 .61621 L
.40733 .6162 L
.4075 .6162 L
.40768 .61619 L
.40801 .61619 L
.40819 .61618 L
.40838 .61618 L
.40873 .61617 L
.4089 .61617 L
.40909 .61616 L
.40927 .61616 L
.40946 .61616 L
.40964 .61615 L
.40981 .61615 L
.41013 .61614 L
.41031 .61614 L
.41048 .61613 L
.41068 .61613 L
.41086 .61613 L
.41122 .61612 L
.41143 .61611 L
.41161 .61611 L
.41194 .6161 L
.4121 .6161 L
.41228 .61609 L
.41243 .61609 L
Mistroke
.4126 .61609 L
.41289 .61608 L
.41307 .61608 L
.41326 .61607 L
.41343 .61607 L
.41361 .61606 L
.41377 .61606 L
.41394 .61606 L
.41409 .61605 L
.41425 .61605 L
.41456 .61604 L
.41473 .61604 L
.41488 .61604 L
.41516 .61603 L
.41545 .61602 L
.41562 .61602 L
.4158 .61601 L
.41596 .61601 L
.41612 .61601 L
.41644 .616 L
.41673 .61599 L
.417 .61599 L
.41729 .61598 L
.41756 .61597 L
.41781 .61597 L
.4181 .61596 L
.41827 .61596 L
.41842 .61595 L
.4187 .61595 L
.41899 .61594 L
.4193 .61593 L
.41947 .61593 L
.41963 .61592 L
.41993 .61592 L
.42021 .61591 L
.42037 .61591 L
.42054 .6159 L
.42085 .61589 L
.42102 .61589 L
.42117 .61589 L
.42135 .61588 L
.42153 .61588 L
.42186 .61587 L
.42203 .61586 L
.42219 .61586 L
.42249 .61585 L
.4228 .61585 L
.42297 .61584 L
.42316 .61584 L
.42333 .61583 L
Mistroke
.4235 .61583 L
.42366 .61582 L
.42383 .61582 L
.42414 .61581 L
.42428 .61581 L
.42444 .6158 L
.42473 .6158 L
.42502 .61579 L
.42528 .61578 L
.42558 .61577 L
.42586 .61577 L
.42601 .61576 L
.42618 .61576 L
.42648 .61575 L
.42674 .61574 L
.42703 .61574 L
.42732 .61573 L
.4276 .61572 L
.42786 .61571 L
.42809 .61571 L
.42836 .6157 L
.42861 .61569 L
.42891 .61569 L
.42918 .61568 L
.42948 .61567 L
.42965 .61567 L
.4298 .61566 L
.4301 .61565 L
.43027 .61565 L
.43043 .61564 L
.43073 .61564 L
.43104 .61563 L
.43132 .61562 L
.43163 .61561 L
.43192 .6156 L
.43218 .6156 L
.43242 .61559 L
.43269 .61558 L
.43296 .61557 L
.43321 .61557 L
.43351 .61556 L
.43368 .61555 L
.43384 .61555 L
.43413 .61554 L
.43443 .61553 L
.4347 .61552 L
.43495 .61552 L
.43523 .61551 L
.43553 .6155 L
.43582 .61549 L
Mistroke
.43599 .61549 L
.43615 .61548 L
.43646 .61547 L
.43664 .61547 L
.4368 .61546 L
.43708 .61545 L
.43739 .61544 L
.43768 .61544 L
.43794 .61543 L
.43819 .61542 L
.43842 .61541 L
.43863 .61541 L
.43886 .6154 L
.43911 .61539 L
.43937 .61538 L
.43962 .61538 L
.43984 .61537 L
.44006 .61536 L
.44027 .61536 L
.4405 .61535 L
.44074 .61534 L
.44094 .61534 L
.44114 .61533 L
.44135 .61532 L
.44157 .61532 L
.44184 .61531 L
.44208 .6153 L
.44234 .61529 L
.44262 .61528 L
.44286 .61528 L
.44309 .61527 L
.44333 .61526 L
.4436 .61525 L
.44382 .61525 L
.44406 .61524 L
.44431 .61523 L
.44454 .61522 L
.44476 .61521 L
.44496 .61521 L
.44519 .6152 L
.4454 .61519 L
.44563 .61519 L
.44584 .61518 L
.44607 .61517 L
.44632 .61516 L
.44657 .61516 L
.44682 .61515 L
.44703 .61514 L
.44727 .61513 L
.44748 .61513 L
Mistroke
.4477 .61512 L
.44793 .61511 L
.44815 .6151 L
.44836 .6151 L
.44854 .61509 L
.44876 .61508 L
.44896 .61508 L
.44918 .61507 L
.44941 .61506 L
.44963 .61505 L
.44983 .61505 L
.45007 .61504 L
.45029 .61503 L
.45053 .61502 L
.45078 .61501 L
.451 .615 L
.45125 .615 L
.45147 .61499 L
.45168 .61498 L
.45192 .61497 L
.45218 .61496 L
.45243 .61495 L
.45266 .61495 L
.45285 .61494 L
.45306 .61493 L
.45328 .61492 L
.45348 .61492 L
.45384 .6149 L
.45405 .6149 L
.45423 .61489 L
.45443 .61488 L
.45465 .61488 L
.45488 .61487 L
.45509 .61486 L
.45548 .61484 L
.4557 .61484 L
.4559 .61483 L
.45627 .61482 L
.45645 .61481 L
.45665 .6148 L
.45682 .61479 L
.45701 .61479 L
.45734 .61478 L
.45753 .61477 L
.45775 .61476 L
.45794 .61475 L
.45813 .61475 L
.45831 .61474 L
.45851 .61473 L
.45885 .61472 L
Mistroke
.45903 .61471 L
.45921 .6147 L
.45938 .6147 L
.45956 .61469 L
.45973 .61468 L
.45989 .61468 L
.46019 .61467 L
.46038 .61466 L
.46055 .61465 L
.46073 .61465 L
.46093 .61464 L
.46126 .61462 L
.46144 .61462 L
.46161 .61461 L
.46177 .6146 L
.46194 .6146 L
.46209 .61459 L
.46225 .61459 L
.46255 .61457 L
.46283 .61456 L
.46298 .61456 L
.46314 .61455 L
.46344 .61454 L
.46361 .61453 L
.46379 .61452 L
.46411 .61451 L
.46429 .6145 L
.46448 .6145 L
.46465 .61449 L
.46483 .61448 L
.46499 .61448 L
.46516 .61447 L
.46547 .61446 L
.46566 .61445 L
.46583 .61444 L
.46601 .61443 L
.46621 .61442 L
.4664 .61442 L
.46657 .61441 L
.46673 .6144 L
.4669 .6144 L
.4672 .61438 L
.46749 .61437 L
.46765 .61436 L
.46782 .61436 L
.46813 .61434 L
.46828 .61434 L
.46845 .61433 L
.46874 .61432 L
.46892 .61431 L
Mistroke
.4691 .6143 L
.46941 .61429 L
.46958 .61428 L
.46974 .61428 L
.46992 .61427 L
.47009 .61426 L
.47044 .61424 L
.47062 .61424 L
.4708 .61423 L
.47096 .61422 L
.47112 .61421 L
.47141 .6142 L
.47159 .61419 L
.47176 .61419 L
.47207 .61417 L
.47223 .61417 L
.47241 .61416 L
.47259 .61415 L
.47277 .61414 L
.47295 .61413 L
.47311 .61413 L
.47342 .61411 L
.4736 .6141 L
.47378 .6141 L
.47396 .61409 L
.47416 .61408 L
.4745 .61406 L
.47468 .61405 L
.47486 .61405 L
.47503 .61404 L
.47522 .61403 L
.47539 .61402 L
.47557 .61401 L
.47572 .61401 L
.4759 .614 L
.4762 .61398 L
.47639 .61398 L
.47656 .61397 L
.47674 .61396 L
.47694 .61395 L
.47712 .61394 L
.47729 .61393 L
.47762 .61392 L
.47778 .61391 L
.47795 .6139 L
.4781 .6139 L
.47826 .61389 L
.47856 .61387 L
.47883 .61386 L
.47898 .61385 L
Mistroke
.47915 .61384 L
.47944 .61383 L
.47962 .61382 L
.47979 .61381 L
.48011 .6138 L
.48027 .61379 L
.48043 .61378 L
.48061 .61377 L
.48078 .61377 L
.48112 .61375 L
.4813 .61374 L
.48148 .61373 L
.48179 .61371 L
.48208 .6137 L
.48225 .61369 L
.4824 .61368 L
.48273 .61367 L
.48291 .61366 L
.4831 .61365 L
.48343 .61363 L
.48361 .61362 L
.48378 .61361 L
.48398 .6136 L
.48417 .6136 L
.48433 .61359 L
.48451 .61358 L
.48482 .61356 L
.48499 .61355 L
.48517 .61354 L
.48533 .61354 L
.4855 .61353 L
.48582 .61351 L
.48599 .6135 L
.48615 .61349 L
.48643 .61348 L
.48674 .61346 L
.48691 .61345 L
.48709 .61344 L
.48725 .61343 L
.48743 .61343 L
.48758 .61342 L
.48775 .61341 L
.48805 .61339 L
.48835 .61338 L
.48863 .61336 L
.48879 .61335 L
.48895 .61334 L
.48926 .61333 L
.48954 .61331 L
.48984 .6133 L
Mistroke
.49012 .61328 L
.49037 .61327 L
.4907 .61325 L
.49087 .61324 L
.49105 .61323 L
.4912 .61322 L
.49136 .61321 L
.49166 .61319 L
.49184 .61318 L
.49203 .61317 L
.4922 .61316 L
.49238 .61315 L
.49254 .61315 L
.49272 .61314 L
.49303 .61312 L
.49318 .61311 L
.49335 .6131 L
.49364 .61308 L
.49395 .61307 L
.49412 .61306 L
.49429 .61305 L
.49458 .61303 L
.49485 .61301 L
.49516 .613 L
.49545 .61298 L
.49569 .61296 L
.49595 .61295 L
.4962 .61293 L
.49642 .61292 L
.49668 .61291 L
.49696 .61289 L
.49722 .61287 L
.49747 .61286 L
.49772 .61284 L
.49795 .61283 L
.49821 .61282 L
.49848 .6128 L
.49876 .61278 L
.49904 .61276 L
.49929 .61275 L
.49955 .61273 L
.4998 .61272 L
.50003 .6127 L
.50024 .61269 L
.50047 .61268 L
.50072 .61266 L
.50098 .61265 L
.5012 .61263 L
.50144 .61262 L
.50167 .6126 L
Mistroke
.50187 .61259 L
.50211 .61258 L
.50234 .61256 L
.50254 .61255 L
.50273 .61254 L
.50294 .61252 L
.50317 .61251 L
.50343 .61249 L
.5037 .61247 L
.50394 .61246 L
.5042 .61244 L
.50447 .61242 L
.50477 .61241 L
.50505 .61239 L
.5053 .61237 L
.50555 .61235 L
.50578 .61234 L
.50604 .61232 L
.50631 .6123 L
.50654 .61229 L
.50676 .61227 L
.507 .61226 L
.50726 .61224 L
.50749 .61223 L
.5077 .61221 L
.50793 .6122 L
.50818 .61218 L
.50843 .61216 L
.50868 .61215 L
.5089 .61213 L
.50913 .61211 L
.50933 .6121 L
.50956 .61209 L
.50979 .61207 L
.51001 .61205 L
.51021 .61204 L
.5104 .61203 L
.51061 .61201 L
.51081 .612 L
.51107 .61198 L
.51134 .61196 L
.51158 .61195 L
.51183 .61193 L
.51206 .61191 L
.51227 .6119 L
.5125 .61188 L
.51275 .61186 L
.51301 .61184 L
.51329 .61182 L
.51356 .6118 L
Mistroke
.51381 .61179 L
.51403 .61177 L
.51427 .61175 L
.51452 .61173 L
.51476 .61172 L
.51498 .6117 L
.51522 .61168 L
.51545 .61167 L
.51566 .61165 L
.51586 .61164 L
.51605 .61162 L
.51627 .61161 L
.51647 .61159 L
.51669 .61158 L
.51692 .61156 L
.51712 .61154 L
.51734 .61153 L
.51757 .61151 L
.51782 .61149 L
.51806 .61147 L
.51827 .61146 L
.5185 .61144 L
.51875 .61142 L
.51901 .6114 L
.51926 .61138 L
.51949 .61136 L
.51969 .61135 L
.51992 .61133 L
.52016 .61131 L
.52041 .61129 L
.52067 .61127 L
.52091 .61125 L
.52113 .61124 L
.52139 .61122 L
.52167 .61119 L
.52194 .61117 L
.52218 .61115 L
.52239 .61114 L
.52261 .61112 L
.52282 .6111 L
.52301 .61109 L
.52324 .61107 L
.52348 .61105 L
.52371 .61103 L
.52392 .61101 L
.52416 .61099 L
.52438 .61098 L
.52458 .61096 L
.5248 .61094 L
.52504 .61092 L
Mistroke
.52528 .6109 L
.5255 .61088 L
.52573 .61087 L
.52594 .61085 L
.52615 .61083 L
.52636 .61081 L
.5266 .61079 L
.5268 .61078 L
.52699 .61076 L
.52719 .61074 L
.52741 .61072 L
.5276 .61071 L
.5278 .61069 L
.52801 .61067 L
.5282 .61066 L
.52855 .61063 L
.52874 .61061 L
.52892 .6106 L
.52913 .61058 L
.52935 .61056 L
.52954 .61054 L
.52974 .61053 L
.52996 .61051 L
.5302 .61049 L
.53042 .61047 L
.53063 .61045 L
.53082 .61043 L
.53102 .61041 L
.53124 .61039 L
.53144 .61038 L
.53165 .61036 L
.53185 .61034 L
.53203 .61032 L
.53221 .61031 L
.53255 .61028 L
.53273 .61026 L
.53292 .61024 L
.53312 .61023 L
.53331 .61021 L
.53348 .61019 L
.53366 .61018 L
.53381 .61016 L
.53399 .61015 L
.53429 .61012 L
.53446 .6101 L
.53464 .61009 L
.53497 .61006 L
.53516 .61004 L
.53534 .61002 L
.53556 .61 L
Mistroke
.53575 .60999 L
.53611 .60995 L
.5363 .60993 L
.53648 .60992 L
.53665 .6099 L
.53683 .60988 L
.53714 .60985 L
.53732 .60984 L
.53748 .60982 L
.53766 .60981 L
.53785 .60979 L
.53816 .60976 L
.53845 .60973 L
.53861 .60971 L
.53879 .6097 L
.5391 .60967 L
.53926 .60965 L
.53943 .60964 L
.53973 .60961 L
.53992 .60959 L
.54009 .60957 L
.54025 .60956 L
.54042 .60954 L
.5407 .60951 L
.541 .60948 L
.54127 .60946 L
.54153 .60943 L
.54183 .6094 L
.54211 .60937 L
.54226 .60936 L
.54242 .60934 L
.54272 .60931 L
.54289 .60929 L
.54307 .60927 L
.54323 .60926 L
.5434 .60924 L
.54355 .60923 L
.54372 .60921 L
.54401 .60918 L
.54419 .60916 L
.54435 .60914 L
.54452 .60913 L
.54471 .60911 L
.54488 .60909 L
.54504 .60907 L
.5452 .60906 L
.54536 .60904 L
.54565 .60901 L
.54591 .60898 L
.54607 .60897 L
Mistroke
.54623 .60895 L
.54652 .60892 L
.54682 .60889 L
.5471 .60886 L
.54741 .60882 L
.54758 .60881 L
.54773 .60879 L
.548 .60876 L
.54828 .60873 L
.54858 .6087 L
.54886 .60867 L
.54912 .60864 L
.54936 .60861 L
.54964 .60858 L
.54989 .60856 L
.55017 .60853 L
.55047 .60849 L
.55072 .60847 L
.55099 .60843 L
.55129 .6084 L
.55145 .60838 L
.5516 .60837 L
.55188 .60834 L
.55217 .6083 L
.55243 .60827 L
.55266 .60825 L
.55293 .60822 L
.55318 .60819 L
.55341 .60816 L
.55362 .60814 L
.55385 .60811 L
.5541 .60808 L
.55435 .60805 L
.55461 .60802 L
.55486 .608 L
.55509 .60797 L
.55535 .60794 L
.55564 .6079 L
.55591 .60787 L
.55616 .60784 L
.5564 .60781 L
.55665 .60778 L
.55687 .60776 L
.5571 .60773 L
.55735 .6077 L
.55762 .60767 L
.55788 .60764 L
.55811 .60761 L
.55834 .60758 L
.55855 .60756 L
Mistroke
.55879 .60753 L
.55903 .6075 L
.55925 .60747 L
.55945 .60745 L
.55967 .60742 L
.5599 .60739 L
.56011 .60737 L
.5603 .60734 L
.56052 .60732 L
.56074 .60729 L
.56098 .60726 L
.5612 .60723 L
.5614 .6072 L
.56162 .60718 L
.5618 .60715 L
.56201 .60713 L
.56222 .6071 L
.56242 .60708 L
.56278 .60703 L
.56298 .60701 L
.56316 .60698 L
.56339 .60695 L
.5636 .60693 L
.56384 .60689 L
.56407 .60687 L
.56428 .60684 L
.56447 .60681 L
.56468 .60679 L
.5649 .60676 L
.56512 .60673 L
.56536 .6067 L
.56559 .60667 L
.56579 .60664 L
.56603 .60661 L
.56629 .60657 L
.56654 .60654 L
.56676 .60651 L
.56695 .60649 L
.56716 .60646 L
.56734 .60643 L
.56753 .60641 L
.56775 .60638 L
.56797 .60635 L
.56818 .60632 L
.56837 .60629 L
.56859 .60626 L
.5688 .60624 L
.56898 .60621 L
.56918 .60618 L
.5694 .60615 L
Mistroke
.56963 .60612 L
.56983 .60609 L
.57004 .60606 L
.57028 .60603 L
.57052 .606 L
.57073 .60597 L
.57096 .60593 L
.57121 .6059 L
.57148 .60586 L
.57173 .60583 L
.57195 .6058 L
.57217 .60576 L
.57237 .60574 L
.57259 .6057 L
.57282 .60567 L
.57307 .60564 L
.5733 .6056 L
.5735 .60557 L
.57373 .60554 L
.57393 .60551 L
.57412 .60548 L
.57433 .60545 L
.57456 .60542 L
.57479 .60539 L
.57501 .60535 L
.57521 .60532 L
.57542 .60529 L
.57563 .60526 L
.57581 .60523 L
.57602 .6052 L
.57622 .60517 L
.57658 .60512 L
.57677 .60509 L
.57696 .60506 L
.57717 .60503 L
.57737 .605 L
.5776 .60497 L
.57781 .60493 L
.57801 .6049 L
.57819 .60488 L
.57838 .60485 L
.5786 .60481 L
.57881 .60478 L
.57903 .60475 L
.57924 .60471 L
.57944 .60468 L
.57965 .60465 L
.57989 .60461 L
.58013 .60458 L
.58036 .60454 L
Mistroke
.58058 .60451 L
.58081 .60447 L
.58104 .60443 L
.58124 .6044 L
.58143 .60437 L
.58163 .60434 L
.58184 .6043 L
.58204 .60427 L
.58227 .60424 L
.58249 .6042 L
.58269 .60417 L
.5829 .60414 L
.58313 .6041 L
.58337 .60406 L
.58358 .60402 L
.58381 .60399 L
.58416 .60393 L
.58436 .6039 L
.58455 .60387 L
.58475 .60383 L
.58493 .6038 L
.58514 .60377 L
.58533 .60374 L
.5855 .60371 L
.58567 .60368 L
.58599 .60363 L
.58617 .6036 L
.58633 .60357 L
.5867 .60351 L
.58689 .60347 L
.5871 .60344 L
.58728 .60341 L
.58747 .60337 L
.58767 .60334 L
.58784 .60331 L
.58817 .60325 L
.58838 .60322 L
.58856 .60319 L
.58876 .60315 L
.58898 .60312 L
.58934 .60305 L
.58954 .60302 L
.58973 .60298 L
.59006 .60293 L
.59025 .60289 L
.59042 .60286 L
.59074 .60281 L
.5909 .60278 L
.59107 .60275 L
.59138 .60269 L
Mistroke
.59155 .60266 L
.5917 .60263 L
.59199 .60258 L
.59213 .60256 L
.59229 .60253 L
.59247 .6025 L
.59264 .60246 L
.59296 .60241 L
.59314 .60237 L
.5933 .60234 L
.5935 .60231 L
.59367 .60228 L
.59397 .60222 L
.59413 .60219 L
.59429 .60216 L
.59447 .60213 L
.59464 .6021 L
.59495 .60204 L
.59513 .60201 L
.59532 .60197 L
.59553 .60193 L
.59572 .6019 L
.59606 .60183 L
.59625 .6018 L
.59642 .60176 L
.59661 .60173 L
.59681 .60169 L
.59703 .60165 L
.59723 .60161 L
.59759 .60154 L
.59779 .6015 L
.59798 .60146 L
.59833 .6014 L
.5985 .60136 L
.59869 .60133 L
.59885 .60129 L
.59902 .60126 L
.59933 .6012 L
.59951 .60116 L
.59971 .60112 L
.59989 .60109 L
.60007 .60105 L
.60024 .60102 L
.60043 .60098 L
.60075 .60092 L
.60091 .60089 L
.60109 .60085 L
.60124 .60082 L
.60141 .60079 L
.60156 .60076 L
Mistroke
.60172 .60072 L
.60201 .60066 L
.60218 .60063 L
.60234 .6006 L
.60251 .60056 L
.60269 .60052 L
.60301 .60046 L
.60318 .60042 L
.60334 .60039 L
.60348 .60036 L
.60364 .60033 L
.60394 .60027 L
.60422 .60021 L
.60448 .60015 L
.60476 .60009 L
.60492 .60006 L
.60507 .60003 L
.60537 .59997 L
.60553 .59993 L
.60569 .5999 L
.60586 .59986 L
.60603 .59983 L
.60634 .59976 L
.60652 .59972 L
.60668 .59969 L
.60687 .59965 L
.60704 .59961 L
.60733 .59955 L
.60748 .59951 L
.60765 .59948 L
.60782 .59944 L
.60798 .5994 L
.60828 .59934 L
.60844 .5993 L
.60862 .59926 L
.60879 .59923 L
.60898 .59919 L
.60915 .59915 L
.6093 .59911 L
.60961 .59904 L
.60978 .59901 L
.60994 .59897 L
.61013 .59893 L
.61031 .59889 L
.61066 .59881 L
.61085 .59877 L
.61103 .59872 L
.61132 .59866 L
.61148 .59862 L
.61163 .59859 L
Mistroke
.61193 .59852 L
.6121 .59848 L
.61225 .59845 L
.61253 .59838 L
.61268 .59835 L
.61282 .59831 L
.61309 .59825 L
.61336 .59819 L
.61353 .59815 L
.61369 .59811 L
.61385 .59807 L
.61404 .59803 L
.61434 .59796 L
.61451 .59792 L
.61466 .59788 L
.61496 .59781 L
.61522 .59775 L
.61537 .59771 L
.61553 .59767 L
.61582 .59761 L
.61609 .59754 L
.61633 .59748 L
.61662 .59741 L
.61688 .59735 L
.61713 .59729 L
.61735 .59723 L
.6176 .59717 L
.61786 .59711 L
.61815 .59704 L
.61831 .597 L
.61846 .59696 L
.61873 .59689 L
.61902 .59682 L
.61928 .59676 L
.61952 .5967 L
.61978 .59663 L
.62006 .59656 L
.6202 .59652 L
.62035 .59648 L
.62062 .59642 L
.62091 .59634 L
.62107 .5963 L
.62123 .59626 L
.6215 .59619 L
.62176 .59612 L
.62203 .59605 L
.62217 .59602 L
.62232 .59598 L
.62259 .59591 L
.62283 .59584 L
Mistroke
.62311 .59577 L
.62337 .5957 L
.62361 .59564 L
.62383 .59558 L
.62408 .59551 L
.62433 .59545 L
.62457 .59538 L
.6248 .59532 L
.625 .59527 L
.62522 .59521 L
.62546 .59514 L
.62571 .59507 L
.62594 .59501 L
.62616 .59495 L
.62637 .59489 L
.62657 .59484 L
.62679 .59478 L
.62702 .59471 L
.62722 .59466 L
.62741 .59461 L
.62761 .59455 L
.62783 .59449 L
.62808 .59442 L
.62831 .59435 L
.62857 .59428 L
.62884 .59421 L
.62907 .59414 L
.62929 .59408 L
.62952 .59401 L
.62978 .59394 L
.62999 .59388 L
.63022 .59381 L
.63047 .59374 L
.63069 .59368 L
.6309 .59361 L
.63109 .59356 L
.63131 .5935 L
.63152 .59343 L
.63176 .59336 L
.63199 .5933 L
.63224 .59322 L
.63249 .59315 L
.63271 .59308 L
.63291 .59302 L
.63314 .59295 L
.63338 .59288 L
.63362 .59281 L
.63387 .59273 L
.63412 .59266 L
.63434 .59259 L
Mistroke
.63458 .59252 L
.63485 .59244 L
.63512 .59235 L
.63538 .59228 L
.63563 .5922 L
.6359 .59212 L
.63615 .59204 L
.63639 .59196 L
.6366 .5919 L
.63683 .59183 L
.63707 .59175 L
.63729 .59168 L
.63755 .5916 L
.6378 .59152 L
.63802 .59145 L
.63826 .59137 L
.63852 .59129 L
.6388 .5912 L
.63906 .59112 L
.63929 .59104 L
.63949 .59098 L
.63972 .5909 L
.63995 .59083 L
.64017 .59076 L
.64037 .59069 L
.64055 .59063 L
.64077 .59056 L
.64097 .59049 L
.64118 .59042 L
.64141 .59035 L
.64161 .59028 L
.64182 .59021 L
.64205 .59013 L
.6423 .59005 L
.64253 .58997 L
.64275 .5899 L
.64293 .58984 L
.64313 .58977 L
.64332 .58971 L
.64349 .58965 L
.64368 .58958 L
.6439 .58951 L
.6441 .58944 L
.64429 .58937 L
.64448 .58931 L
.64466 .58924 L
.64486 .58918 L
.64507 .5891 L
.64528 .58903 L
.64549 .58895 L
Mistroke
.64568 .58889 L
.64588 .58882 L
.64612 .58873 L
.64635 .58865 L
.64659 .58856 L
.64685 .58847 L
.64707 .58839 L
.64728 .58832 L
.64751 .58824 L
.64775 .58815 L
.64797 .58807 L
.6482 .58799 L
.64842 .58791 L
.64862 .58783 L
.64881 .58777 L
.64901 .58769 L
.64922 .58761 L
.64941 .58754 L
.64964 .58746 L
.64989 .58737 L
.65012 .58728 L
.65033 .5872 L
.65055 .58712 L
.65074 .58705 L
.65096 .58696 L
.65117 .58689 L
.65139 .5868 L
.65163 .58671 L
.65184 .58663 L
.65206 .58655 L
.6523 .58645 L
.65256 .58636 L
.6528 .58626 L
.65302 .58618 L
.65321 .58611 L
.65341 .58603 L
.65363 .58594 L
.65383 .58586 L
.65401 .58579 L
.65418 .58572 L
.65437 .58565 L
.65457 .58557 L
.65477 .58549 L
.65498 .58541 L
.65518 .58533 L
.65536 .58526 L
.65557 .58517 L
.6558 .58508 L
.65602 .58499 L
.65622 .58491 L
Mistroke
.65639 .58484 L
.65657 .58476 L
.65675 .58469 L
.65691 .58463 L
.65709 .58455 L
.65729 .58447 L
.65748 .58439 L
.65765 .58432 L
.65785 .58424 L
.65804 .58416 L
.6582 .5841 L
.65838 .58402 L
.65858 .58394 L
.65878 .58385 L
.65896 .58378 L
.65915 .5837 L
.65933 .58362 L
.6595 .58355 L
.65969 .58347 L
.65989 .58339 L
.66009 .5833 L
.66029 .58321 L
.66047 .58314 L
.66067 .58305 L
.66085 .58297 L
.66101 .5829 L
.6612 .58282 L
.66138 .58274 L
.6617 .5826 L
.66188 .58252 L
.66204 .58245 L
.66222 .58237 L
.66241 .58229 L
.66259 .58221 L
.66276 .58214 L
.66295 .58205 L
.66316 .58196 L
.66336 .58187 L
.66354 .58179 L
.66372 .58171 L
.6639 .58163 L
.66409 .58154 L
.66429 .58145 L
.66449 .58136 L
.66471 .58126 L
.66489 .58118 L
.66509 .58109 L
.66526 .58101 L
.66545 .58092 L
.66563 .58084 L
Mistroke
.6658 .58076 L
.66611 .58062 L
.66626 .58054 L
.66644 .58046 L
.66662 .58038 L
.66679 .5803 L
.66699 .58021 L
.66717 .58012 L
.66734 .58004 L
.6675 .57996 L
.66767 .57988 L
.66786 .57979 L
.66803 .57971 L
.66818 .57964 L
.66836 .57955 L
.66854 .57947 L
.6687 .57939 L
.66884 .57932 L
.669 .57924 L
.66918 .57916 L
.66933 .57908 L
.6695 .579 L
.66964 .57893 L
.6698 .57885 L
.66993 .57879 L
.67009 .57871 L
.67036 .57858 L
.67051 .5785 L
.67065 .57844 L
.67081 .57835 L
.67096 .57828 L
.67126 .57813 L
.67143 .57804 L
.67159 .57796 L
.67172 .57789 L
.67187 .57782 L
.67213 .57769 L
.67227 .57762 L
.6724 .57755 L
.6727 .5774 L
.67286 .57732 L
.67302 .57723 L
.67318 .57715 L
.67333 .57708 L
.6736 .57693 L
.67376 .57685 L
.6739 .57678 L
.67405 .5767 L
.67421 .57662 L
.67439 .57652 L
Mistroke
.67455 .57644 L
.67484 .57628 L
.67501 .5762 L
.67516 .57611 L
.67544 .57597 L
.67559 .57589 L
.67573 .57581 L
.67599 .57567 L
.67612 .5756 L
.67627 .57552 L
.67651 .57539 L
.67665 .57531 L
.67678 .57525 L
.67701 .57512 L
.67713 .57505 L
.67726 .57498 L
.67741 .5749 L
.67755 .57482 L
.67781 .57468 L
.67796 .5746 L
.67809 .57452 L
.67825 .57444 L
.6784 .57435 L
.67864 .57422 L
.67877 .57414 L
.67891 .57407 L
.67905 .57399 L
.67919 .57391 L
.67944 .57376 L
.67959 .57368 L
.67972 .57361 L
.67986 .57353 L
.68001 .57344 L
.68027 .57329 L
.68042 .57321 L
.68055 .57313 L
.6807 .57304 L
.68085 .57296 L
.68113 .5728 L
.68129 .57271 L
.68143 .57262 L
.68158 .57253 L
.68175 .57243 L
.6819 .57235 L
.68204 .57226 L
.68219 .57217 L
.68234 .57208 L
.68261 .57193 L
.68275 .57184 L
.68289 .57176 L
Mistroke
.68306 .57166 L
.68321 .57156 L
.68335 .57148 L
.6835 .57139 L
.68366 .57129 L
.68383 .57119 L
.68398 .5711 L
.68413 .571 L
.68427 .57092 L
.68442 .57083 L
.68455 .57075 L
.68469 .57067 L
.68481 .57059 L
.68494 .57051 L
.68518 .57036 L
.68533 .57027 L
.68546 .57019 L
.6856 .5701 L
.68576 .57 L
.68602 .56984 L
.68616 .56975 L
.68629 .56966 L
.68642 .56958 L
.68655 .5695 L
.68667 .56942 L
.6868 .56934 L
.68691 .56927 L
.68703 .56919 L
.68725 .56905 L
.68738 .56897 L
.6875 .56889 L
.68763 .56881 L
.68778 .56871 L
.68791 .56863 L
.68803 .56855 L
.68826 .5684 L
.68841 .5683 L
.68856 .5682 L
.68869 .56811 L
.68884 .56802 L
.68899 .56792 L
.68915 .56781 L
.68931 .56771 L
.68945 .56761 L
.68958 .56752 L
.68972 .56743 L
.68988 .56733 L
.69002 .56723 L
.69027 .56706 L
.69042 .56696 L
Mistroke
.69055 .56687 L
.69068 .56679 L
.69082 .56669 L
.69094 .56661 L
.69107 .56652 L
.69119 .56643 L
.69131 .56636 L
.69153 .5662 L
.69167 .56611 L
.69179 .56602 L
.69193 .56593 L
.69207 .56583 L
.69232 .56566 L
.69245 .56556 L
.69258 .56547 L
.69271 .56538 L
.69285 .56528 L
.69301 .56517 L
.69315 .56507 L
.69329 .56497 L
.69341 .56488 L
.69355 .56478 L
.6937 .56468 L
.69382 .56459 L
.69396 .56449 L
.6941 .56439 L
.69423 .5643 L
.69446 .56413 L
.69459 .56403 L
.69471 .56395 L
.69485 .56384 L
.695 .56374 L
.69513 .56364 L
.69527 .56354 L
.69542 .56343 L
.69558 .56331 L
.69573 .5632 L
.69587 .5631 L
.696 .56299 L
.69613 .5629 L
.69627 .56279 L
.69642 .56268 L
.69656 .56258 L
.69669 .56247 L
.69682 .56238 L
.69694 .56229 L
.69705 .56221 L
.69716 .56212 L
.69736 .56197 L
.69747 .56189 L
Mistroke
.69759 .5618 L
.69769 .56172 L
.6978 .56163 L
.69799 .56148 L
.69818 .56134 L
.69828 .56126 L
.69839 .56117 L
.69859 .56102 L
.6987 .56093 L
.6988 .56085 L
.69891 .56077 L
.69903 .56067 L
.69914 .56059 L
.69924 .56051 L
.69944 .56035 L
.69955 .56026 L
.69965 .56018 L
.69978 .56008 L
.69989 .55999 L
.70011 .55981 L
.70024 .55971 L
.70036 .55961 L
.70056 .55945 L
.70067 .55936 L
.70078 .55927 L
.701 .55909 L
.70112 .55899 L
.70123 .5589 L
.70142 .55875 L
.70152 .55866 L
.70162 .55858 L
.70182 .55842 L
.70193 .55832 L
.70203 .55824 L
.70215 .55815 L
.70227 .55805 L
.70248 .55787 L
.7026 .55777 L
.70271 .55767 L
.70284 .55757 L
.70296 .55746 L
.70306 .55737 L
.70318 .55728 L
.70338 .55711 L
.70349 .55701 L
.70361 .55691 L
.70371 .55682 L
.70382 .55672 L
.70402 .55655 L
.70413 .55646 L
Mistroke
.70423 .55637 L
.70441 .55621 L
.70452 .55612 L
.70461 .55604 L
.70482 .55586 L
.70493 .55575 L
.70504 .55566 L
.70524 .55548 L
.70534 .5554 L
.70545 .5553 L
.70554 .55521 L
.70565 .55512 L
.70583 .55496 L
.70593 .55487 L
.70602 .55479 L
.70613 .55469 L
.70623 .5546 L
.70632 .55452 L
.70641 .55443 L
.70658 .55428 L
.70667 .55419 L
.70677 .5541 L
.70687 .55402 L
.70695 .55394 L
.70713 .55378 L
.70728 .55363 L
.70745 .55348 L
.70755 .55339 L
.70764 .55331 L
.7078 .55316 L
.70795 .55302 L
.70811 .55286 L
.70827 .55271 L
.70845 .55255 L
.70853 .55246 L
.70863 .55237 L
.7088 .55221 L
.70896 .55206 L
.70906 .55197 L
.70915 .55188 L
.70932 .55172 L
.70941 .55162 L
.7095 .55154 L
.70959 .55145 L
.7097 .55135 L
.7098 .55125 L
.70989 .55116 L
.70999 .55106 L
.71008 .55097 L
.71025 .55081 L
Mistroke
.71034 .55072 L
.71043 .55063 L
.71053 .55054 L
.71063 .55044 L
.71081 .55026 L
.71091 .55016 L
.711 .55007 L
.7111 .54997 L
.71121 .54985 L
.71131 .54976 L
.71139 .54967 L
.71148 .54959 L
.71157 .5495 L
.71172 .54934 L
.71189 .54917 L
.71198 .54908 L
.71207 .54899 L
.71216 .5489 L
.71226 .54879 L
.71243 .54862 L
.71252 .54852 L
.71261 .54844 L
.7128 .54823 L
.71292 .54812 L
.71302 .54801 L
.7132 .54782 L
.7133 .54772 L
.7134 .54761 L
.71358 .54742 L
.71368 .54731 L
.71377 .54722 L
.71386 .54713 L
.71395 .54703 L
.71412 .54685 L
.71423 .54673 L
.71433 .54662 L
.71452 .54642 L
.71463 .5463 L
.71473 .5462 L
.71483 .54608 L
.71495 .54596 L
.71504 .54586 L
.71513 .54575 L
.71531 .54556 L
.71542 .54545 L
.71552 .54533 L
.71561 .54523 L
.71571 .54513 L
.7158 .54502 L
.71589 .54492 L
Mistroke
.71606 .54473 L
.71615 .54463 L
.71625 .54452 L
.71635 .54441 L
.71645 .5443 L
.71664 .54408 L
.71675 .54396 L
.71685 .54384 L
.71703 .54364 L
.71713 .54352 L
.71723 .54342 L
.71731 .54332 L
.71739 .54323 L
.71755 .54305 L
.71764 .54294 L
.71773 .54283 L
.7179 .54264 L
.71799 .54254 L
.71808 .54243 L
.71824 .54224 L
.71834 .54213 L
.71843 .54202 L
.71861 .54181 L
.7187 .5417 L
.71879 .5416 L
.71889 .54147 L
.71899 .54136 L
.71908 .54125 L
.71918 .54113 L
.71928 .54101 L
.71938 .54089 L
.71956 .54067 L
.71965 .54056 L
.71975 .54044 L
.71983 .54034 L
.71992 .54023 L
.72009 .54003 L
.72019 .5399 L
.72029 .53978 L
.7204 .53965 L
.72051 .53951 L
.72062 .53938 L
.72072 .53925 L
.72081 .53914 L
.72091 .53901 L
.721 .5389 L
.7211 .53877 L
.7212 .53865 L
.7213 .53852 L
.72147 .53831 L
Mistroke
.72157 .53818 L
.72166 .53807 L
.72183 .53785 L
.72192 .53773 L
.72202 .53761 L
.7221 .53751 L
.72219 .53739 L
.72234 .53719 L
.72244 .53706 L
.72254 .53693 L
.72264 .5368 L
.72273 .53668 L
.7229 .53646 L
.72299 .53635 L
.72308 .53622 L
.72317 .53611 L
.72325 .536 L
.72341 .53579 L
.7235 .53567 L
.72358 .53556 L
.72368 .53543 L
.72376 .53531 L
.72384 .53521 L
.72393 .5351 L
.72407 .5349 L
.72416 .53479 L
.72424 .53467 L
.7244 .53446 L
.72449 .53433 L
.72459 .5342 L
.72475 .53397 L
.72485 .53384 L
.72494 .53372 L
.72503 .53359 L
.72513 .53345 L
.72521 .53333 L
.7253 .53321 L
.72546 .533 L
.72555 .53287 L
.72564 .53273 L
.72572 .53262 L
.72581 .5325 L
.72591 .53235 L
.72601 .53221 L
.7261 .53208 L
.72619 .53195 L
.7263 .5318 L
.72641 .53164 L
.7265 .53151 L
.7266 .53136 L
Mistroke
.7267 .53123 L
.72678 .5311 L
.72688 .53095 L
.72698 .53081 L
.72707 .53069 L
.72715 .53057 L
.72724 .53044 L
.72733 .5303 L
.72741 .53018 L
.72751 .53004 L
.7276 .5299 L
.72769 .52977 L
.72785 .52953 L
.72794 .5294 L
.72802 .52928 L
.72811 .52914 L
.7282 .52901 L
.72829 .52886 L
.72839 .52871 L
.72848 .52858 L
.72856 .52845 L
.72865 .52831 L
.72875 .52817 L
.72884 .52803 L
.72892 .52791 L
.72901 .52777 L
.7291 .52762 L
.72919 .52748 L
.72928 .52735 L
.72936 .52723 L
.72944 .5271 L
.72951 .52698 L
.72959 .52686 L
.72974 .52663 L
.72982 .5265 L
.72991 .52637 L
.72998 .52625 L
.73006 .52612 L
.73016 .52596 L
.73026 .5258 L
.73035 .52565 L
.73044 .52551 L
.73055 .52534 L
.73064 .52519 L
.73075 .52502 L
.73086 .52484 L
.73095 .52469 L
.73104 .52454 L
.73113 .52439 L
.73121 .52426 L
Mistroke
.73131 .5241 L
.73141 .52393 L
.73151 .52376 L
.7316 .52361 L
.73168 .52348 L
.73177 .52334 L
.73185 .5232 L
.73192 .52308 L
.73201 .52293 L
.7321 .52277 L
.73219 .52262 L
.73227 .52248 L
.73237 .52232 L
.73245 .52218 L
.73253 .52204 L
.73262 .5219 L
.73271 .52174 L
.73281 .52157 L
.73289 .52143 L
.73298 .52127 L
.73306 .52113 L
.73315 .52098 L
.73322 .52086 L
.7333 .52072 L
.73338 .52057 L
.73347 .52041 L
.73356 .52026 L
.73364 .52012 L
.73372 .51998 L
.73379 .51985 L
.73387 .51971 L
.73396 .51955 L
.73404 .51941 L
.73412 .51927 L
.73418 .51915 L
.73426 .51902 L
.7344 .51876 L
.73447 .51862 L
.73455 .51849 L
.73467 .51826 L
.73475 .51813 L
.73481 .51801 L
.73488 .51788 L
.73496 .51774 L
.73504 .51759 L
.73512 .51745 L
.73525 .5172 L
.73533 .51705 L
.7354 .51692 L
.73548 .51678 L
Mistroke
.73555 .51664 L
.73564 .51648 L
.73572 .51633 L
.7358 .51616 L
.73589 .51601 L
.73596 .51586 L
.73604 .51571 L
.73611 .51558 L
.73619 .51543 L
.73627 .51527 L
.73635 .51513 L
.73648 .51487 L
.73656 .51473 L
.73663 .51459 L
.73671 .51444 L
.73679 .51427 L
.73687 .51411 L
.73695 .51397 L
.73703 .5138 L
.73713 .51361 L
.73721 .51344 L
.73729 .51328 L
.73737 .51313 L
.73744 .51299 L
.73752 .51284 L
.7376 .51267 L
.73769 .5125 L
.73777 .51233 L
.73784 .51218 L
.73792 .51202 L
.73806 .51175 L
.73813 .5116 L
.7382 .51146 L
.73833 .5112 L
.73839 .51107 L
.73846 .51093 L
.73854 .51077 L
.73861 .51062 L
.73869 .51046 L
.73876 .5103 L
.73883 .51015 L
.7389 .51002 L
.73897 .50987 L
.73905 .50971 L
.73912 .50956 L
.73919 .50941 L
.73926 .50925 L
.73934 .5091 L
.73947 .50882 L
.73954 .50867 L
Mistroke
.73961 .50852 L
.73968 .50836 L
.73975 .50822 L
.73982 .50806 L
.7399 .50788 L
.73998 .50771 L
.74007 .50752 L
.74014 .50737 L
.74022 .5072 L
.74029 .50705 L
.74036 .50689 L
.74043 .50674 L
.74049 .50659 L
.74061 .50633 L
.74068 .50619 L
.74074 .50604 L
.74081 .50588 L
.74088 .50574 L
.74096 .50557 L
.74103 .5054 L
.74116 .50511 L
.74123 .50495 L
.7413 .5048 L
.74136 .50466 L
.74143 .5045 L
.74149 .50437 L
.74155 .50422 L
.74161 .50409 L
.74167 .50395 L
.74178 .5037 L
.74185 .50354 L
.74191 .5034 L
.74197 .50325 L
.74204 .50309 L
.74211 .50293 L
.74217 .50279 L
.74229 .50251 L
.74236 .50234 L
.74243 .50219 L
.7425 .50201 L
.74258 .50183 L
.74266 .50163 L
.74274 .50145 L
.74281 .50128 L
.74288 .5011 L
.74295 .50095 L
.74302 .50077 L
.7431 .50059 L
.74317 .50042 L
.74323 .50027 L
Mistroke
.74329 .50012 L
.74336 .49995 L
.74343 .49979 L
.7435 .4996 L
.74357 .49943 L
.74365 .49924 L
.74373 .49905 L
.7438 .49888 L
.74386 .49872 L
.74393 .49855 L
.74401 .49836 L
.74408 .49817 L
.74416 .49797 L
.74424 .49778 L
.74431 .49761 L
.74439 .4974 L
.74447 .49718 L
.74456 .49697 L
.74463 .49678 L
.7447 .49661 L
.74476 .49646 L
.74482 .49629 L
.74489 .49612 L
.74501 .49582 L
.74507 .49565 L
.74513 .49549 L
.7452 .4953 L
.74527 .49513 L
.74533 .49497 L
.7454 .4948 L
.74547 .49461 L
.74555 .49441 L
.74561 .49424 L
.74568 .49405 L
.74575 .49386 L
.74582 .49367 L
.74589 .4935 L
.74596 .49332 L
.74603 .49312 L
.74611 .49291 L
.74618 .49273 L
.74625 .49253 L
.74632 .49234 L
.74638 .49217 L
.74645 .49198 L
.74652 .49179 L
.74659 .49161 L
.74664 .49145 L
.74671 .49127 L
.74678 .49108 L
Mistroke
.74685 .49088 L
.74693 .49065 L
.74702 .49041 L
.7471 .49018 L
.74718 .48997 L
.74724 .48978 L
.74732 .48956 L
.7474 .48935 L
.74748 .48912 L
.74757 .48887 L
.74765 .48864 L
.74772 .48843 L
.74781 .48819 L
.7479 .48792 L
.74799 .48766 L
.74807 .48743 L
.74814 .48723 L
.74821 .48701 L
.74828 .4868 L
.74835 .48661 L
.74842 .48639 L
.74851 .48615 L
.74858 .48592 L
.74865 .48571 L
.74873 .48549 L
.7488 .48528 L
.74887 .48506 L
.74895 .48482 L
.74903 .48457 L
.74912 .48432 L
.74919 .4841 L
.74927 .48386 L
.74934 .48364 L
.74942 .4834 L
.74949 .48317 L
.74956 .48296 L
.74962 .48277 L
.74969 .48257 L
.74976 .48235 L
.74983 .48214 L
.7499 .48191 L
.74998 .48168 L
.75004 .48147 L
.75011 .48125 L
.75019 .48101 L
.75027 .48075 L
.75035 .48051 L
.75042 .48028 L
.75049 .48007 L
.75056 .47985 L
Mistroke
.75062 .47965 L
.75068 .47944 L
.75075 .47922 L
.75083 .47897 L
.7509 .47874 L
.75097 .47853 L
.75103 .47832 L
.75109 .47812 L
.75116 .4779 L
.75123 .47767 L
.75129 .47747 L
.75135 .47728 L
.75141 .47708 L
.75148 .47685 L
.75156 .47659 L
.75163 .47634 L
.7517 .47611 L
.75177 .47586 L
.75185 .4756 L
.75194 .47531 L
.75201 .47507 L
.75209 .4748 L
.75216 .47455 L
.75223 .47432 L
.75231 .47405 L
.75238 .4738 L
.75245 .47357 L
.75251 .47335 L
.75258 .47311 L
.75265 .47286 L
.75273 .47259 L
.7528 .47234 L
.75288 .47206 L
.75296 .47177 L
.75304 .47151 L
.7531 .47127 L
.75318 .47101 L
.75326 .47073 L
.75334 .47043 L
.75339 .47025 L
.75344 .47009 L
.75352 .4698 L
.7536 .46949 L
.75368 .46921 L
.75375 .46894 L
.75383 .46865 L
.75392 .46834 L
.75399 .46807 L
.75407 .46778 L
.75415 .46751 L
Mistroke
.75421 .46726 L
.75429 .46697 L
.75438 .46665 L
.75446 .46635 L
.75453 .46608 L
.7546 .46582 L
.75467 .46557 L
.75474 .4653 L
.75481 .46502 L
.75489 .46471 L
.75497 .46443 L
.75503 .46417 L
.75511 .46389 L
.75517 .46363 L
.75524 .46339 L
.75531 .46313 L
.75538 .46284 L
.75545 .46256 L
.75553 .46226 L
.7556 .462 L
.75567 .46172 L
.75573 .46145 L
.7558 .46121 L
.75587 .46093 L
.75593 .46067 L
.756 .46042 L
.75605 .4602 L
.75611 .45995 L
.75618 .45968 L
.75624 .45943 L
.75631 .45915 L
.75638 .45888 L
.75644 .45864 L
.75649 .45841 L
.75655 .45817 L
.75662 .45791 L
.75668 .45767 L
.75674 .45741 L
.75681 .45713 L
.75687 .45689 L
.75693 .45662 L
.757 .45634 L
.75707 .45604 L
.75714 .45575 L
.75721 .45548 L
.75727 .45524 L
.75733 .45497 L
.75738 .45475 L
.75744 .4545 L
.7575 .45423 L
Mistroke
.75757 .45394 L
.75764 .45367 L
.7577 .45342 L
.75775 .45319 L
.75781 .45294 L
.75787 .45267 L
.75793 .45241 L
.75799 .45215 L
.75805 .45191 L
.75815 .45145 L
.75822 .45117 L
.75828 .45091 L
.75834 .45062 L
.75842 .45031 L
.75849 .44999 L
.75856 .44968 L
.75862 .4494 L
.75869 .4491 L
.75875 .44884 L
.75881 .44855 L
.75888 .44825 L
.75894 .44797 L
.75906 .44746 L
.75912 .44718 L
.75918 .44691 L
.75924 .44662 L
.75931 .44631 L
.75938 .44602 L
.75943 .44575 L
.7595 .44544 L
.75958 .4451 L
.75965 .44477 L
.75971 .44447 L
.75978 .44416 L
.75984 .44387 L
.75991 .44355 L
.75999 .44321 L
.76006 .44285 L
.76014 .44251 L
.7602 .4422 L
.76027 .44186 L
.76033 .44158 L
.76039 .44131 L
.76045 .44102 L
.76052 .44071 L
.76059 .44038 L
.76065 .44006 L
.76071 .43978 L
.76078 .43948 L
.76089 .43892 L
Mistroke
.76095 .43864 L
.76101 .43832 L
.76107 .43806 L
.76113 .43777 L
.76123 .43725 L
.7613 .43692 L
.76137 .43661 L
.76144 .43627 L
.76151 .4359 L
.76158 .43555 L
.76165 .43522 L
.76171 .43492 L
.76177 .4346 L
.76189 .43398 L
.76196 .43367 L
.76203 .43333 L
.76208 .43303 L
.76215 .43272 L
.76226 .43214 L
.76232 .43184 L
.76238 .43152 L
.7625 .43094 L
.76256 .43061 L
.76262 .43031 L
.76268 .42997 L
.76275 .42961 L
.76287 .429 L
.76293 .42869 L
.763 .42835 L
.76305 .42806 L
.76311 .42775 L
.76322 .42718 L
.76328 .42688 L
.76333 .42659 L
.76339 .42628 L
.76345 .42595 L
.76356 .42538 L
.76362 .42507 L
.76367 .42477 L
.76378 .42422 L
.76387 .42371 L
.76393 .42343 L
.76398 .42312 L
.76409 .42256 L
.76414 .42225 L
.7642 .42196 L
.76425 .42165 L
.76431 .42131 L
.76442 .42074 L
.76448 .42043 L
Mistroke
.76453 .42014 L
.76463 .41958 L
.76472 .41907 L
.76483 .41848 L
.76493 .41792 L
.76504 .41732 L
.76509 .41702 L
.76515 .41668 L
.76526 .41609 L
.76536 .41553 L
.76547 .41487 L
.76558 .41426 L
.7657 .4136 L
.76576 .41326 L
.76582 .4129 L
.76593 .41228 L
.76603 .4117 L
.76612 .41117 L
.76622 .4106 L
.76633 .40998 L
.76644 .40932 L
.76654 .40876 L
.76664 .40814 L
.76674 .40755 L
.76683 .40702 L
.76693 .40641 L
.76703 .40582 L
.76712 .40528 L
.76721 .40478 L
.7673 .40423 L
.7674 .40365 L
.76751 .40296 L
.76764 .40221 L
.76777 .40142 L
.76789 .40068 L
.768 .39999 L
.76811 .39936 L
.76823 .39863 L
.76834 .39794 L
.76846 .39718 L
.76859 .39639 L
.7687 .3957 L
.76882 .39495 L
.76895 .39414 L
.76909 .39327 L
.76923 .39245 L
.76935 .39169 L
.76946 .391 L
.76958 .39025 L
.76979 .3889 L
Mistroke
.76992 .38808 L
.77004 .3873 L
.77015 .38659 L
.77027 .38585 L
.77037 .38518 L
.77049 .38445 L
.77069 .38316 L
.7708 .38244 L
.77092 .38166 L
.77114 .38025 L
.77136 .37882 L
.77156 .37749 L
.77179 .37603 L
.77202 .37448 L
.77223 .37314 L
.77242 .37188 L
.77263 .37048 L
.77285 .36901 L
.77324 .36644 L
.7736 .36408 L
.77439 .3588 L
.77582 .34922 L
.77623 .3465 L
.77661 .34394 L
.77696 .34163 L
.77733 .3392 L
.77765 .33709 L
.77781 .33598 L
.778 .33479 L
.77816 .3337 L
.77831 .33271 L
.77861 .33077 L
.77878 .32963 L
.77894 .32857 L
.77912 .32742 L
.77931 .32619 L
.77949 .32503 L
.77966 .32394 L
.77981 .32297 L
.77997 .32192 L
.78015 .32078 L
.78034 .31956 L
.78052 .31842 L
.78069 .31738 L
.78088 .31618 L
.78099 .3155 L
.78109 .31487 L
.78127 .31373 L
.78136 .31316 L
.78146 .31252 L
Mistroke
.78156 .31191 L
.78166 .31127 L
.78185 .31013 L
.78195 .3095 L
.78205 .30892 L
.78215 .30827 L
.78226 .30759 L
.78236 .307 L
.78246 .30637 L
.78256 .30577 L
.78265 .30522 L
.78282 .30418 L
.78291 .30367 L
.783 .3031 L
.78309 .30257 L
.78319 .302 L
.78328 .30147 L
.78336 .30097 L
.78352 .30004 L
.7836 .29957 L
.78368 .29906 L
.78378 .2985 L
.78386 .298 L
.78396 .29745 L
.78406 .29687 L
.78414 .29636 L
.78422 .29589 L
.78431 .29538 L
.78441 .29484 L
.78449 .29433 L
.78457 .29387 L
.78466 .29337 L
.78476 .29283 L
.78484 .29237 L
.78491 .29194 L
.785 .29147 L
.78509 .29096 L
.78516 .29052 L
.78525 .29003 L
.78533 .28958 L
.78541 .28916 L
.78555 .28837 L
.78563 .28793 L
.7857 .28753 L
.78578 .28708 L
.78587 .2866 L
.78595 .28615 L
.78602 .28573 L
.78609 .28535 L
.78617 .28494 L
Mistroke
.78625 .28451 L
.78632 .28411 L
.7864 .28366 L
.78649 .2832 L
.78656 .28281 L
.78664 .28238 L
.78673 .28191 L
.78682 .28142 L
.7869 .28096 L
.78698 .28054 L
.78706 .28015 L
.78714 .27973 L
.7872 .27937 L
.78728 .27899 L
.78735 .27857 L
.78744 .27812 L
.78752 .2777 L
.78759 .27732 L
.78767 .27694 L
.78773 .27659 L
.78781 .27621 L
.78789 .2758 L
.78796 .27541 L
.78803 .27505 L
.7881 .27472 L
.78816 .27437 L
.78824 .27396 L
.78832 .27358 L
.78841 .27312 L
.78849 .27271 L
.78857 .27229 L
.78867 .27183 L
.78875 .2714 L
.78883 .271 L
.7889 .27064 L
.78897 .2703 L
.78905 .26993 L
.78913 .26953 L
.78921 .26912 L
.78929 .26872 L
.78936 .26837 L
.78944 .26799 L
.78953 .26754 L
.78962 .26712 L
.7897 .26674 L
.78978 .26633 L
.78987 .26589 L
.78997 .26542 L
.79005 .26502 L
.79014 .26459 L
Mistroke
.79023 .26418 L
.79031 .26381 L
.7904 .26339 L
.79048 .26299 L
.79056 .26262 L
.79063 .26228 L
.79071 .26191 L
.79079 .26152 L
.79087 .26117 L
.79095 .2608 L
.79103 .2604 L
.79111 .26004 L
.79119 .25969 L
.79126 .25938 L
.79133 .25903 L
.79141 .25869 L
.79149 .25833 L
.79157 .25795 L
.79164 .25762 L
.79172 .25726 L
.79181 .25688 L
.7919 .25647 L
.79199 .25608 L
.79206 .25573 L
.79214 .25541 L
.79221 .25506 L
.79229 .25474 L
.79235 .25445 L
.79243 .2541 L
.79252 .25373 L
.7926 .25339 L
.79267 .25307 L
.79274 .25277 L
.7928 .25249 L
.79287 .25219 L
.79295 .25186 L
.79303 .25152 L
.7931 .2512 L
.79317 .25092 L
.79324 .25061 L
.79332 .25027 L
.79341 .24991 L
.79349 .24957 L
.79356 .24926 L
.79364 .24891 L
.79373 .24854 L
.79382 .24815 L
.79391 .24779 L
.794 .24741 L
.79409 .24705 L
Mistroke
.79417 .24673 L
.79425 .24638 L
.79434 .246 L
.79444 .2456 L
.79452 .24527 L
.79462 .2449 L
.7947 .24456 L
.79478 .24423 L
.79485 .24395 L
.79493 .24363 L
.79501 .2433 L
.7951 .24294 L
.79518 .24263 L
.79526 .2423 L
.79534 .24199 L
.79541 .24171 L
.79549 .2414 L
.79558 .24108 L
.79565 .2408 L
.79572 .24053 L
.79579 .24025 L
.79587 .23994 L
.79594 .23967 L
.796 .23942 L
.79608 .23914 L
.79615 .23884 L
.79624 .23853 L
.79631 .23823 L
.79638 .23797 L
.79646 .23768 L
.79653 .23741 L
.79659 .23717 L
.79667 .23689 L
.79674 .23663 L
.7968 .23638 L
.79686 .23616 L
.79693 .23591 L
.797 .23565 L
.79708 .23537 L
.79714 .23512 L
.79722 .23482 L
.7973 .23455 L
.79737 .23429 L
.79743 .23405 L
.7975 .2338 L
.79758 .23352 L
.79765 .23325 L
.79773 .23296 L
.79781 .23269 L
.79788 .23244 L
Mistroke
.79796 .23216 L
.79804 .23186 L
.79813 .23154 L
.79821 .23125 L
.79828 .231 L
.79835 .23077 L
.79842 .23051 L
.7985 .23025 L
.79856 .23001 L
.79862 .2298 L
.79869 .22956 L
.79876 .22931 L
.79883 .22909 L
.7989 .22884 L
.79896 .22863 L
.79902 .2284 L
.79909 .22818 L
.79915 .22798 L
.79926 .22759 L
.79933 .22735 L
.79941 .22709 L
.79947 .22688 L
.79954 .22665 L
.79961 .22639 L
.79969 .22613 L
.79977 .22587 L
.79983 .22564 L
.7999 .22543 L
.79997 .2252 L
.80004 .22495 L
.80011 .22472 L
.80023 .22431 L
.8003 .22408 L
.80037 .22386 L
.80043 .22366 L
.8005 .22344 L
.80062 .22306 L
.80069 .22282 L
.80076 .2226 L
.80082 .2224 L
.80088 .22219 L
.80094 .222 L
.801 .22179 L
.80112 .22143 L
.80118 .22123 L
.80125 .22102 L
.80131 .22081 L
.80137 .22063 L
.80148 .22026 L
.80155 .22006 L
Mistroke
.80161 .21986 L
.80172 .21951 L
.80178 .21931 L
.80184 .21913 L
.80195 .21879 L
.80205 .21848 L
.80216 .21815 L
.80221 .21798 L
.80227 .21779 L
.80234 .21759 L
.80241 .21737 L
.80247 .21718 L
.80254 .21698 L
.8026 .21679 L
.80266 .21659 L
.80278 .21624 L
.80285 .21603 L
.80291 .21584 L
.80298 .21563 L
.80305 .2154 L
.80312 .21519 L
.80319 .215 L
.80331 .21462 L
.80337 .21444 L
.80344 .21425 L
.8035 .21408 L
.80356 .2139 L
.80361 .21373 L
.80367 .21355 L
.80378 .21324 L
.80384 .21305 L
.8039 .21288 L
.80397 .21269 L
.80403 .21249 L
.8041 .21231 L
.80416 .21214 L
.80427 .2118 L
.80434 .21161 L
.8044 .21144 L
.80447 .21123 L
.80454 .21104 L
.80467 .21066 L
.80474 .21045 L
.80481 .21025 L
.80487 .21008 L
.80493 .2099 L
.80505 .20958 L
.80511 .2094 L
.80517 .20924 L
.8053 .20886 L
Mistroke
.80537 .20867 L
.80544 .20847 L
.80551 .20828 L
.80558 .20809 L
.80565 .2079 L
.80571 .20772 L
.80583 .20739 L
.80591 .20719 L
.80597 .207 L
.80604 .2068 L
.80612 .20659 L
.80626 .20622 L
.80633 .20602 L
.8064 .20584 L
.80652 .20551 L
.80659 .20532 L
.80665 .20515 L
.80677 .20483 L
.80683 .20467 L
.80689 .2045 L
.80701 .2042 L
.80713 .20388 L
.80724 .20358 L
.80734 .20331 L
.80741 .20314 L
.80747 .20297 L
.80759 .20266 L
.80766 .20247 L
.80773 .20229 L
.80779 .20213 L
.80786 .20196 L
.80798 .20164 L
.80809 .20135 L
.80821 .20104 L
.80828 .20086 L
.80835 .2007 L
.80841 .20053 L
.80849 .20034 L
.80862 .20001 L
.80869 .19982 L
.80876 .19965 L
.80883 .19946 L
.80891 .19926 L
.80898 .1991 L
.80905 .19892 L
.80917 .19861 L
.80924 .19844 L
.80931 .19825 L
.80938 .19808 L
.80945 .19792 L
Mistroke
.80958 .1976 L
.80964 .19743 L
.80971 .19726 L
.80984 .19696 L
.80997 .19663 L
.81008 .19635 L
.81021 .19604 L
.81033 .19575 L
.81044 .19549 L
.81057 .19516 L
.81065 .19497 L
.81072 .19481 L
.81085 .19449 L
.81098 .19419 L
.81105 .19401 L
.81113 .19383 L
.81126 .19351 L
.81134 .19333 L
.81141 .19317 L
.81149 .19299 L
.81157 .19279 L
.81164 .19263 L
.81171 .19247 L
.81184 .19217 L
.81196 .19189 L
.81208 .1916 L
.81222 .19129 L
.81234 .191 L
.81248 .19068 L
.81256 .1905 L
.81263 .19034 L
.81276 .19004 L
.81289 .18975 L
.813 .1895 L
.81312 .18922 L
.81325 .18892 L
.81339 .18861 L
.81351 .18834 L
.81364 .18805 L
.81376 .18778 L
.81388 .18753 L
.814 .18725 L
.81413 .18698 L
.81424 .18673 L
.81434 .18651 L
.81445 .18626 L
.81458 .18599 L
.8147 .18571 L
.81482 .18546 L
.81495 .18518 L
Mistroke
.81509 .18488 L
.81521 .18463 L
.81532 .18439 L
.81544 .18413 L
.81557 .18385 L
.81571 .18355 L
.81586 .18323 L
.81601 .18293 L
.81614 .18265 L
.81626 .1824 L
.81639 .18212 L
.81653 .18184 L
.81665 .18157 L
.81679 .1813 L
.81691 .18105 L
.81704 .18077 L
.81719 .18047 L
.81732 .1802 L
.81745 .17993 L
.81757 .1797 L
.81769 .17945 L
.8178 .17922 L
.81793 .17897 L
.81806 .1787 L
.81818 .17846 L
.81829 .17823 L
.8184 .17803 L
.81851 .17779 L
.81863 .17757 L
.81875 .17733 L
.81888 .17707 L
.81902 .17679 L
.81915 .17654 L
.81927 .1763 L
.81938 .17609 L
.8195 .17585 L
.81962 .1756 L
.81975 .17535 L
.81989 .17509 L
.82002 .17483 L
.82014 .17461 L
.82028 .17434 L
.82043 .17405 L
.82057 .17378 L
.8207 .17354 L
.82081 .17333 L
.82092 .17311 L
.82113 .17272 L
.82125 .17248 L
.82137 .17226 L
Mistroke
.82148 .17206 L
.82159 .17184 L
.8217 .17165 L
.82181 .17145 L
.822 .17109 L
.82211 .17088 L
.82223 .17067 L
.82234 .17047 L
.82244 .17028 L
.82264 .16991 L
.82275 .16971 L
.82286 .1695 L
.82305 .16916 L
.82316 .16896 L
.82326 .16878 L
.82344 .16844 L
.82361 .16814 L
.82381 .16778 L
.824 .16745 L
.82411 .16725 L
.82421 .16706 L
.82441 .16671 L
.82452 .16651 L
.82462 .16633 L
.82474 .16612 L
.82485 .16593 L
.82504 .16561 L
.82524 .16525 L
.82543 .16491 L
.82554 .16472 L
.82564 .16455 L
.82575 .16436 L
.82587 .16415 L
.82608 .16379 L
.82619 .16359 L
.8263 .1634 L
.82642 .1632 L
.82655 .16298 L
.82677 .16261 L
.82696 .16228 L
.82717 .16192 L
.82729 .16172 L
.8274 .16153 L
.8276 .1612 L
.82778 .1609 L
.82798 .16057 L
.82819 .16022 L
.82837 .15992 L
.82854 .15964 L
.82872 .15933 L
Mistroke
.82892 .15901 L
.82909 .15872 L
.82928 .15841 L
.82946 .15811 L
.82963 .15784 L
.82979 .15758 L
.82994 .15734 L
.83011 .15706 L
.83027 .1568 L
.83045 .15652 L
.83064 .15621 L
.83084 .15589 L
.83103 .15559 L
.8312 .15532 L
.83136 .15507 L
.83154 .15478 L
.83171 .15451 L
.8319 .15421 L
.83209 .1539 L
.83226 .15364 L
.83245 .15335 L
.83264 .15305 L
.83286 .15272 L
.83306 .15241 L
.83324 .15212 L
.83344 .15181 L
.83363 .15152 L
.8338 .15126 L
.83398 .15098 L
.83419 .15067 L
.8344 .15035 L
.83458 .15008 L
.83477 .14978 L
.83496 .1495 L
.83513 .14925 L
.83532 .14896 L
.8355 .14868 L
.83583 .1482 L
.83601 .14793 L
.83618 .14768 L
.83636 .14741 L
.83655 .14712 L
.83674 .14685 L
.8369 .14661 L
.83709 .14634 L
.83729 .14604 L
.8375 .14573 L
.8377 .14545 L
.8379 .14515 L
.8381 .14486 L
Mistroke
.83828 .14461 L
.83847 .14433 L
.83867 .14404 L
.83889 .14372 L
.83908 .14346 L
.83928 .14317 L
.83948 .14289 L
.83969 .14259 L
.83989 .1423 L
.84008 .14204 L
.84042 .14155 L
.84062 .14127 L
.8408 .14103 L
.84099 .14075 L
.84119 .14047 L
.84142 .14016 L
.84162 .13987 L
.84199 .13936 L
.8422 .13907 L
.8424 .13881 L
.84257 .13856 L
.84277 .1383 L
.84294 .13807 L
.84312 .13782 L
.84328 .1376 L
.84345 .13736 L
.84377 .13694 L
.84396 .13668 L
.84413 .13645 L
.84431 .1362 L
.84452 .13593 L
.84486 .13547 L
.84504 .13522 L
.84521 .13499 L
.84554 .13456 L
.84573 .13431 L
.8459 .13408 L
.84625 .13362 L
.84645 .13336 L
.84663 .13313 L
.84694 .13271 L
.84711 .1325 L
.84729 .13227 L
.84761 .13184 L
.84791 .13146 L
.84828 .13098 L
.84849 .13071 L
.84868 .13046 L
.84904 .13001 L
.84921 .12978 L
Mistroke
.84941 .12954 L
.84974 .12911 L
.84993 .12887 L
.8501 .12866 L
.85043 .12824 L
.85074 .12785 L
.85103 .12748 L
.85135 .12709 L
.85168 .12667 L
.85199 .12629 L
.85228 .12593 L
.85254 .12561 L
.85283 .12525 L
.85313 .12488 L
.85343 .12451 L
.85376 .12411 L
.85406 .12373 L
.85434 .1234 L
.85467 .123 L
.85502 .12258 L
.85535 .12218 L
.85565 .12182 L
.85592 .1215 L
.85616 .1212 L
.85645 .12087 L
.85671 .12055 L
.85718 .11999 L
.85745 .11968 L
.85769 .11939 L
.85794 .1191 L
.8582 .11879 L
.85849 .11845 L
.85876 .11814 L
.85905 .11781 L
.85935 .11746 L
.85961 .11716 L
.85989 .11684 L
.86018 .1165 L
.86046 .11618 L
.86071 .1159 L
.86097 .11559 L
.86126 .11526 L
.86157 .11491 L
.86183 .11461 L
.86212 .11428 L
.86239 .11398 L
.86263 .11371 L
.86289 .11342 L
.86318 .1131 L
.86344 .11281 L
Mistroke
.86368 .11254 L
.86415 .11201 L
.86442 .11171 L
.86471 .11139 L
.86502 .11105 L
.8653 .11074 L
.8658 .11019 L
.86608 .10988 L
.86634 .1096 L
.86664 .10928 L
.86691 .10898 L
.86721 .10865 L
.86753 .10831 L
.8678 .10802 L
.86806 .10774 L
.86834 .10744 L
.86864 .10712 L
.86914 .10658 L
.86942 .10629 L
.86969 .10601 L
.86996 .10572 L
.87021 .10545 L
.8705 .10515 L
.87077 .10487 L
.871 .10462 L
.87125 .10437 L
.87169 .10391 L
.87193 .10366 L
.87216 .10342 L
.87242 .10314 L
.87267 .10289 L
.87312 .10242 L
.87336 .10218 L
.87362 .10192 L
.87386 .10167 L
.87408 .10144 L
.87429 .10123 L
.8745 .10101 L
.87492 .10059 L
.8753 .10021 L
.8757 .0998 L
.87593 .09957 L
.87615 .09935 L
.87637 .09913 L
.87662 .09888 L
.87684 .09866 L
.87707 .09844 L
.87727 .09823 L
.8775 .09801 L
.87789 .09761 L
Mistroke
.87813 .09738 L
.87835 .09716 L
.87859 .09693 L
.87885 .09668 L
.87908 .09645 L
.8793 .09624 L
.87973 .09582 L
.87995 .0956 L
.88019 .09536 L
.88043 .09514 L
.88069 .09489 L
.88092 .09466 L
.88114 .09446 L
.88156 .09405 L
.88181 .09382 L
.88204 .0936 L
.88228 .09336 L
.88256 .0931 L
.88301 .09267 L
.88326 .09244 L
.88349 .09222 L
.88392 .09182 L
.88413 .09162 L
.88436 .0914 L
.88477 .09102 L
.88515 .09067 L
.88538 .09046 L
.88559 .09026 L
.886 .08988 L
.88622 .08968 L
.88643 .08949 L
.88667 .08926 L
.8869 .08906 L
.88725 .08873 L
.88764 .08838 L
.88799 .08806 L
.88833 .08776 L
.88871 .08741 L
.88907 .08709 L
.88946 .08674 L
.88968 .08654 L
.88987 .08637 L
.89028 .08601 L
.89049 .08582 L
.89071 .08563 L
.89108 .0853 L
.89149 .08494 L
.89185 .08462 L
.89223 .08429 L
.89259 .08397 L
Mistroke
.89292 .08369 L
.89329 .08337 L
.89364 .08307 L
.89401 .08275 L
.89421 .08258 L
.89442 .0824 L
.89482 .08206 L
.89503 .08187 L
.89523 .0817 L
.89559 .0814 L
.89598 .08107 L
.89633 .08077 L
.89672 .08044 L
.89708 .08014 L
.89742 .07986 L
.89772 .07961 L
.89805 .07934 L
.89839 .07905 L
.89871 .07879 L
.89908 .07848 L
.89928 .07832 L
.89949 .07815 L
.89987 .07783 L
.90023 .07755 L
.90056 .07727 L
.90088 .07702 L
.90122 .07674 L
.90159 .07644 L
.90196 .07615 L
.90236 .07583 L
.90256 .07566 L
.90278 .07549 L
.90316 .07519 L
.90355 .07488 L
.90391 .0746 L
.90423 .07434 L
.90458 .07407 L
.90492 .07381 L
.90522 .07357 L
.90557 .07329 L
.90591 .07304 L
.90621 .0728 L
.90649 .07258 L
.9068 .07235 L
.90713 .0721 L
.90749 .07182 L
.90787 .07153 L
.90823 .07126 L
.90857 .071 L
.90887 .07078 L
Mistroke
.90919 .07053 L
.90953 .07028 L
.90985 .07004 L
.91018 .06979 L
.91055 .06952 L
.91089 .06927 L
.91121 .06904 L
.91149 .06883 L
.9118 .0686 L
.91212 .06836 L
.91242 .06814 L
.91276 .0679 L
.9131 .06765 L
.9134 .06744 L
.91372 .0672 L
.91407 .06695 L
.91444 .06669 L
.91479 .06644 L
.9151 .06622 L
.91539 .06601 L
.91566 .06582 L
.91596 .06561 L
.91627 .06539 L
.9166 .06516 L
.91691 .06494 L
.91719 .06475 L
.91749 .06454 L
.91778 .06434 L
.91803 .06416 L
.91833 .06396 L
.91861 .06377 L
.91887 .06359 L
.9191 .06343 L
.91936 .06325 L
.91963 .06307 L
.91993 .06286 L
.9202 .06268 L
.92051 .06247 L
.92081 .06227 L
.92108 .06209 L
.92133 .06192 L
.9216 .06174 L
.92189 .06155 L
.92218 .06136 L
.92248 .06115 L
.92277 .06096 L
.92304 .06079 L
.92335 .06058 L
.92368 .06036 L
.924 .06016 L
Mistroke
.92429 .05997 L
.92457 .05978 L
.92487 .05959 L
.92512 .05943 L
.92539 .05926 L
.92568 .05907 L
.926 .05886 L
.92629 .05868 L
.92656 .0585 L
.92683 .05833 L
.92708 .05818 L
.92735 .058 L
.92764 .05782 L
.92789 .05766 L
.92812 .05752 L
.92838 .05736 L
.92865 .05719 L
.92889 .05704 L
.92912 .0569 L
.92936 .05675 L
.92963 .05658 L
.9299 .05641 L
.93016 .05626 L
.93039 .05611 L
.93064 .05596 L
.93109 .05568 L
.93132 .05554 L
.93157 .05539 L
.93179 .05526 L
.93201 .05513 L
.93243 .05488 L
.93269 .05472 L
.93293 .05458 L
.9332 .05441 L
.93346 .05426 L
.9337 .05412 L
.93392 .05399 L
.93416 .05384 L
.93441 .0537 L
.93467 .05355 L
.93494 .05339 L
.93519 .05324 L
.93543 .0531 L
.9357 .05294 L
.93599 .05277 L
.93627 .05261 L
.93653 .05246 L
.93675 .05234 L
.93698 .0522 L
.9374 .05196 L
Mistroke
.93764 .05182 L
.9379 .05168 L
.93812 .05155 L
.93835 .05142 L
.93861 .05128 L
.93883 .05115 L
.9391 .051 L
.93934 .05086 L
.93978 .05062 L
.94003 .05048 L
.94025 .05035 L
.94048 .05023 L
.9407 .05011 L
.9411 .04989 L
.94132 .04976 L
.94153 .04965 L
.94177 .04952 L
.942 .04939 L
.94219 .04929 L
.94241 .04917 L
.94278 .04897 L
.94298 .04886 L
.9432 .04874 L
.9436 .04853 L
.94382 .04841 L
.94403 .0483 L
.94424 .04818 L
.94448 .04805 L
.9447 .04794 L
.9449 .04783 L
.9453 .04762 L
.94552 .04751 L
.94573 .0474 L
.94597 .04727 L
.94619 .04715 L
.94641 .04704 L
.94664 .04692 L
.94689 .04679 L
.94713 .04667 L
.94756 .04645 L
.94779 .04632 L
.94801 .04621 L
.94842 .046 L
.94862 .0459 L
.94884 .04579 L
.94922 .04559 L
.94944 .04548 L
.94963 .04539 L
.94981 .0453 L
.95001 .0452 L
Mistroke
.95036 .04502 L
.95057 .04491 L
.95077 .04482 L
.951 .0447 L
.95121 .0446 L
.95164 .04438 L
.95187 .04427 L
.95211 .04415 L
.9523 .04406 L
.9525 .04396 L
.95287 .04378 L
.95309 .04367 L
.9533 .04357 L
.95349 .04348 L
.95369 .04338 L
.95388 .04329 L
.95406 .0432 L
.9544 .04304 L
.9546 .04295 L
.95481 .04285 L
.95519 .04266 L
.95539 .04257 L
.95558 .04248 L
.95578 .04238 L
.956 .04228 L
.9562 .04218 L
.95638 .0421 L
.95656 .04202 L
.95674 .04193 L
.95708 .04177 L
.95738 .04163 L
.95757 .04154 L
.95775 .04146 L
.95808 .04131 L
.95825 .04123 L
.95843 .04115 L
.95875 .041 L
.95893 .04092 L
.95909 .04084 L
.95928 .04076 L
.95947 .04067 L
.95966 .04059 L
.95982 .04051 L
.96015 .04036 L
.96034 .04028 L
.96053 .04019 L
.96089 .04003 L
.96107 .03995 L
.96127 .03986 L
.96146 .03978 L
Mistroke
.96166 .03968 L
.96185 .0396 L
.96202 .03952 L
.96219 .03945 L
.96236 .03937 L
.96269 .03923 L
.963 .03909 L
.96333 .03895 L
.96351 .03887 L
.96369 .03879 L
.964 .03866 L
.96429 .03853 L
.9646 .0384 L
.96493 .03826 L
.9651 .03818 L
.96528 .03811 L
.96561 .03797 L
.96593 .03783 L
.96622 .03771 L
.96654 .03757 L
.96671 .0375 L
.96688 .03743 L
.96722 .03728 L
.96741 .03721 L
.96758 .03713 L
.96789 .03701 L
.96817 .03689 L
.96849 .03676 L
.96882 .03662 L
.96913 .03649 L
.96943 .03637 L
.96969 .03626 L
.96998 .03615 L
.97028 .03602 L
.97061 .03589 L
.97092 .03577 L
.9712 .03565 L
.97152 .03552 L
.9717 .03545 L
.97187 .03538 L
.97218 .03526 L
.97251 .03513 L
.97282 .03501 L
.97298 .03494 L
.97316 .03487 L
.97347 .03475 L
.97376 .03464 L
.97392 .03458 L
.97409 .03451 L
.97441 .03439 L
Mistroke
.9746 .03431 L
.97477 .03424 L
.97511 .03411 L
.9754 .034 L
.97567 .0339 L
.97597 .03379 L
.97628 .03367 L
.97644 .03361 L
.97661 .03354 L
.97693 .03342 L
.97723 .03331 L
.97751 .0332 L
.97779 .0331 L
.97806 .033 L
.9783 .03291 L
.97856 .03281 L
.97884 .03271 L
.97914 .0326 L
.97939 .03251 L
.97966 .03241 L
.97995 .0323 L
.98027 .03218 L
.98056 .03208 L
.98083 .03198 L
.98108 .03189 L
.98135 .03179 L
.98163 .03169 L
.98189 .0316 L
.98219 .03149 L
.98248 .03139 L
.98273 .0313 L
.98301 .0312 L
.98331 .03109 L
.98364 .03098 L
.98394 .03088 L
.98421 .03078 L
.98446 .03069 L
.98474 .0306 L
.98503 .0305 L
.9853 .0304 L
.98555 .03032 L
.98578 .03024 L
.98604 .03015 L
.98629 .03006 L
.98656 .02997 L
.98684 .02988 L
.98708 .0298 L
.98735 .02971 L
.98763 .02961 L
.98793 .02951 L
Mistroke
.98822 .02941 L
.98848 .02933 L
.98874 .02924 L
.98897 .02916 L
.98924 .02907 L
.9895 .02899 L
.98973 .02891 L
.98995 .02884 L
.99018 .02877 L
.99043 .02868 L
.99068 .0286 L
.99095 .02852 L
.9912 .02844 L
.99143 .02836 L
.9917 .02828 L
.99199 .02818 L
.99226 .02809 L
.99251 .02801 L
.99278 .02793 L
.99302 .02785 L
.99328 .02777 L
.99356 .02768 L
.9938 .02761 L
.99402 .02754 L
.99427 .02746 L
.99453 .02738 L
.99476 .02731 L
.99502 .02723 L
.99525 .02716 L
.99547 .02709 L
.99567 .02703 L
.99589 .02696 L
.99611 .02689 L
.99632 .02683 L
.99655 .02676 L
.99677 .02669 L
.997 .02662 L
.99725 .02655 L
.99752 .02647 L
.99776 .02639 L
.99798 .02633 L
.99822 .02625 L
.99843 .02619 L
.99866 .02612 L
.99889 .02605 L
.99912 .02599 L
.99951 .02587 L
.99973 .02581 L
.99993 .02575 L
1 .02573 L
Mistroke
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  Background->GrayLevel[1],
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005X0000gR000`400?l00000o`00003oo`3ooomY
0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY0?ooo`00o`3ooomY
0?ooo`00E03oool5000000<0oooo100000050?ooo`800000903oool5000000<0oooo100000040?oo
o`@000001@3oool2000001`0oooo1@0000030?ooo`@00000103oool4000000@0oooo100000050?oo
o`8000005@3oool5000000<0oooo100000040?ooo`@00000103oool4000000@0oooo100000050?oo
o`800000303oool5000000<0oooo100000040?ooo`@00000103oool4000000@0oooo100000040?oo
o`@000001@3oool2000000@0oooo001F0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`020?ooo`8000009P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`020?oo
o`8000007P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?oo
o`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0080oooo0P00
000G0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?oo
o`0000040?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`00
0000oooo0?ooo`020?ooo`8000003P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`040000003o
ool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`020?ooo`80
0000103oool005H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000
003oool0oooo000000@0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`000004
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool0
10000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0
203oool005H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003o
ool0oooo000000@0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?oo
o`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?oo
o`0000040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool01000
0000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0203o
ool005H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003oool0
oooo000000@0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`03
0000003oool0oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`04
0000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`00
00040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000
oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0203oool0
05H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
02X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo
000000@0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000
003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`000004
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool010000000oooo
0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0203oool005@0
oooo0`0000040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0:03oool3000000@0
oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo
0200oooo0`0000040?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool0
10000000oooo0?ooo`0000040?ooo`030000003oool0oooo01T0oooo0`0000040?ooo`030000003o
ool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`04
0000003oool0oooo000000@0oooo00<000000?ooo`3oool0403oool3000000@0oooo00<000000?oo
o`3oool00P3oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00@0
00000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00P0
oooo001F0?ooo`030000003oool0oooo00<0oooo1000000]0?ooo`030000003oool0oooo00<0oooo
100000040?ooo`@000009@3oool00`000000oooo0?ooo`030?ooo`@00000103oool4000000@0oooo
1000000N0?ooo`030000003oool0oooo00<0oooo100000040?ooo`@00000103oool4000000@0oooo
1000000E0?ooo`030000003oool0oooo00<0oooo100000040?ooo`@00000103oool4000000@0oooo
100000040?ooo`@000002`3oool00?l0ooooJ@3oool00?l0ooooJ@3oool00?l0ooooJ@3oool00?l0
ooooJ@3oool00?l0ooooJ@3oool00?l0ooooJ@3oool00280ooooo`00000Y000001h0oooo000R0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool01@000000oooo0?ooo`3oool0
00000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo0080oooo00D000000?ooo`3oool0oooo000000020?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`050000003oool0oooo0?oo
o`0000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool00P3oool01@000000oooo0?ooo`3oool000000080oooo00@000000?ooo`3oool00000
0P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00D000000?ooo`3oool0
oooo000000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0703oool00280
oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`00
8P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo
000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703o
ool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L
0?ooo`008P3oool200000100oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000N0?ooo`008P3oool00`000000
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool01@0007l0oooo0?ooo`0007l0oooo0080003o7P3oool00280oooo00<000000?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0
001o0?ooo`000?l00`000?l00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`060000O`3oool0oooo0?ooo`0007l0oooo10000?l0
100007l0oooo0?ooo`00000N0?ooo`001`3oool5000000@0oooo100000040?ooo`@000001P3oool0
0`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`040000O`3oool0oooo0?oo
o`<0003o00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`001`3oool01@00
0000oooo0?ooo`3oool0000000<0oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`00
00040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool00`0007l0003o0000o`020000o`80oooo00@0001o0?ooo`3oool0001o0P3oool0
0`000000oooo0?ooo`0L0?ooo`00203oool00`000000oooo0?ooo`090?ooo`040000003oool0oooo
000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0100007l0
oooo0?ooo`3oool30000o`80oooo00@0001o0?ooo`3oool0001o0P3oool0100007l0oooo0?ooo`00
000N0?ooo`001@0000040?ooo`030000003oool0oooo00P0oooo00@000000?ooo`3oool00000103o
ool00`000000oooo0?ooo`030?ooo`<00000o`0007lE0000O`<0003o2`0007l3000001h0oooo000:
0?ooo`030000003oool0oooo00<0oooo100000030?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool30000o`80oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool0
00/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00@0001o0?ooo`3oool0oooo0P000?l40?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0
oooo01`0oooo00070?ooo`050000003oool0oooo0?ooo`000000103oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`00
0?l00P000?l00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo00080?ooo`<000001@3oool50000
00<0oooo100000060?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool30000o`80oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?oo
o`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00@0oooo0P000?l40?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo
00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`03
0?ooo`80003o00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo
000R0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0100007l0oooo0?ooo`3oool3
0000o`030?ooo`0007l0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000001h0oooo000R0?ooo`030000003o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool00P000?l30?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool0
0`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`040000O`3oool0003o0000
o`@0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R
0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo0`000?l50?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`00
0000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`060?ooo`80003o1`3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01@3oool20000o`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00@0oooo0P000?l00`3oool0001o0?ooo`06
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3o
ool200000100oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00<0oooo0P000?l20?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo008000007P3oool00280oooo00<00000
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool00P3oool20000o`<0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool0
0280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00@0001o0?ooo`3oool0oooo0P000?l40?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0
oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo0080003o
1@3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
0000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0100007l0oooo0000
o`000?l60?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`00
0?l0003o00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<000000?ooo`3oool0703oool00280oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool20000o`P0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000N0?ooo`008P3o
ool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03P3oool20000o`T0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<00000
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00h0oooo00<0003o0000O`3oool0203oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`00
0000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03@3oool20000o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280
oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00`0oooo0P000?l00`3oool0001o0?ooo`080?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0
oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0;0?ooo`80003o0P3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0:0?ooo`80003o0`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`800000403oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00X0oooo00<0003o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01@00
07l0oooo0?ooo`0007l0oooo008000007P3oool00280oooo00<000000?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00T0oooo
0P000?l40?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00P0oooo0P000?l50?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo
00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00P0oooo00<0003o0?ooo`3oool0103oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
003oool0oooo01`0oooo00070?ooo`D00000103oool4000000@0oooo100000060?ooo`030000003o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`070?ooo`80003o1P3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo0007
0?ooo`050000003oool0oooo0?ooo`0000000`3oool00`000000oooo0?ooo`020?ooo`040000003o
ool0oooo000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool01P3oool2
0000o`L0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`00203oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`060?ooo`030000o`3oool0oooo00H0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`000000oooo0?ooo`0L0?ooo`001@0000040?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00<0oooo
0`00003a0000O`80003o;`0007l3000001h0oooo000:0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00@0oooo0P000?l90?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0
703oool000/0oooo00D000000?ooo`3oool0oooo000000040?ooo`040000003oool0oooo000000@0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0103oool00`000?l0oooo0?oo
o`080?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool000L0oooo00D000000?ooo`3oool0
oooo000000030?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`030?ooo`80003o2P3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000003oool0oooo01`0oooo00080?ooo`<000001@3oool4000000@0oooo100000060?ooo`03
0000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`030?ooo`030000o`3oool0oooo00T0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000
oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool00P3oool20000o`/0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool00P3oool00`00
0?l0oooo0?ooo`0:0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo0P00000@
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00@0001o0?ooo`3o
ool0oooo0P000?l<0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`050000O`3oool0oooo0000O`3oool00P00000N0?ooo`008P3oool00`000000oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o
0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00D0001o0?ooo`3o
ool0oooo0000o`0=0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<00000
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`05
0000O`3oool0oooo0?ooo`000?l03@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R
0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool0100007l0oooo0?ooo`000?l>0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703o
ool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`040000O`3oool0oooo0000o`h0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?oo
o`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00@0001o0?ooo`000?l0003o3P3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003o
ool0oooo01`0oooo000R0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0003o00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000001h0oooo000R0?oo
o`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0000o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280
oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`000?l0003o00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`00
8P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0000o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo
000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0003o0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703o
ool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`80003o403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?oo
o`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`000?l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280
oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
003o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool01@0007l0oooo0?ooo`0007l0oooo008000007P3oool00280oooo00<000000?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo0080003o4@3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00L0001o0?ooo`3oool0001o0?ooo`000?l0001o0140oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`070000O`3oool0oooo0000O`3oool0003o0000O`0A0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool01`0007l0oooo0?ooo`0007l0oooo0000o`0007l04@3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P000?l00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
00000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool01`0007l0oooo0?ooo`0007l0003o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003o
ool0oooo01`0oooo000R0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01`0007l0oooo
0?ooo`0007l0003o0?ooo`0007l04@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo008000007P3oool00280oooo00<00000
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01`0007l0oooo0?ooo`0007l0003o0?oo
o`0007l04@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00<0001o0?ooo`3oool00P000?l00`3oool0001o
0?ooo`0@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`000?l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`00203oool5000000<0oooo
100000040?ooo`@000001P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000o`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
003oool0oooo01`0oooo000:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3o
ool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`000?l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`002P3oool00`000000oooo0?oo
o`070?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000o`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo0005000000D0oooo00<0
00000?ooo`3oool01`3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00<0oooo
0`00003U0000O`80003o>`0007l3000001h0oooo000:0?ooo`030000003oool0oooo00<0oooo1000
00030?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0003o0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000:0?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0000o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L
0?ooo`00203oool3000000D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0000o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
0000oooo0?ooo`0L0?ooo`002P3oool00`000000oooo0?ooo`030?ooo`D000000`3oool4000000H0
oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0000o`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`00
8P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0003o
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo
000R0?ooo`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0003o0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00D0001o0?ooo`3oool0001o0?ooo`02000001h0oooo000R0?ooo`030000003oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0000o`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0003o0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`000?l0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000
003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000
O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0000o`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<0
00000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0003o0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool0
0`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool0
0`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?oo
o`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`80003o00D0oooo0000O`3oool0
oooo0000O`0A0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo0P00000@0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000o`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01@0007l0
oooo0?ooo`0007l0oooo008000007P3oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool010000?l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000o`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0003o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool010000?l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00H0001o
0?ooo`3oool0oooo0000O`3oool20000o`80oooo00@0001o0?ooo`3oool0001o4@3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0200007l0oooo0?ooo`3o
ool0001o0?ooo`000?l0001o0P3oool0100007l0oooo0?ooo`0007lA0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
00000?ooo`3oool0703oool00280oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00P0001o0?ooo`3oool0oooo0000O`3oool0003o0000
O`80oooo00@0001o0?ooo`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo008000007P3oool00280oooo00<0
00000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo
0?ooo`020?ooo`080000O`3oool0oooo0?ooo`0007l0oooo0000o`0007l20?ooo`040000O`3oool0
oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00P0001o0?ooo`3oool0oooo0000O`3oool0003o0000O`80oooo00@0001o0?ooo`3oool0001o
4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool02000
07l0oooo0?ooo`3oool0001o0?ooo`000?l0001o0P3oool0100007l0oooo0?ooo`0007lA0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`080000O`3oool0
oooo0?ooo`0007l0oooo0000o`0007l20?ooo`040000O`3oool0oooo0000Oa40oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00P0001o0?ooo`3oool0oooo
0000O`3oool0003o0000O`80oooo00@0001o0?ooo`3oool0001o4@3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0200007l0oooo0?ooo`3oool0001o0000
o`3oool0001o0P3oool0100007l0oooo0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3o
ool0703oool00280oooo0P00000@0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00P0001o0?ooo`3oool0oooo0000O`000?l0oooo0000O`80oooo00@0
001o0?ooo`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool01@0007l0oooo0?ooo`0007l0oooo008000007P3oool00280oooo00<000000?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`080000O`3oool0oooo0?ooo`0007l0003o0?ooo`0007l20?ooo`040000O`3oool0oooo0000Oa40
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00P0001o
0?ooo`3oool0oooo0000O`000?l0oooo0000O`80oooo00@0001o0?ooo`3oool0001o4@3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000003oool0oooo01`0oooo00080?ooo`D000000`3oool4000000@0oooo10000006
0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool0200007l0oooo0?ooo`3oool0001o0000o`3oool0001o0P3oool01000
07l0oooo0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool0
0`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000
O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool000X0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000000@0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00P0001o0?ooo`3oool0oooo0000O`000?l0
oooo0000O`80oooo00@0001o0?ooo`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo
01`0oooo000:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000
oooo0?ooo`0000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`000?l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`001@0000050?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo
00<0oooo0`00003P0000O`030000o`0007l0001o03l0001o0`00000N0?ooo`002P3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0003o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?oo
o`3oool0703oool000X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`04
0000003oool0oooo000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o
0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o
0?ooo`3oool0oooo0000o`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool0
0P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo00080?ooo`<00000103oool00`000000
oooo0?ooo`020?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000o`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000:0?ooo`03
0000003oool0oooo00<0oooo100000040?ooo`@000001P3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000o`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020000o`80oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3o
ool0oooo0000o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0
oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080
oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?oo
o`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool200000100oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`00
0?l0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0oooo0000
O`3oool00P00000N0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0003o0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`000?l0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0003o0000o`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`000?l0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0000o`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0003o
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool200000100oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0000o`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool0
0P00000N0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0
001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3o
ool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0000o`000?l0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003o
ool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool0
0`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0003o0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<00000
0?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`000?l0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
0000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o
0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0000o`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0003o0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`80003o0P3oool0100007l0oooo
0?ooo`0007l20?ooo`040000O`3oool0oooo0000Oa40oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?oo
o`0L0?ooo`008P3oool200000100oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0
001o0?ooo`3oool00P3oool01@000?l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000N0?ooo`008P3oool00`00
0000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`00
07l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?oo
o`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?oo
o`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`03
0000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`03
0000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0003o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`03
0000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`03
0000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o
0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@000?l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo
00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0100007l0
oooo0?ooo`3oool20000o`<0oooo00@0001o0?ooo`3oool0001o0P3oool0100007l0oooo0?ooo`00
07lA0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool01P0007l0oooo0?ooo`3oool0003o
0000O`<0oooo00@0001o0?ooo`3oool0001o0P3oool0100007l0oooo0?ooo`0007lA0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00D0oooo00<0001o0?ooo`3oool00P3oool01P0007l0oooo0?ooo`3oool0003o0000O`<0oooo00@0
001o0?ooo`3oool0001o0P3oool0100007l0oooo0?ooo`0007lA0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<00000
0?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020000o`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
0000oooo0?ooo`0L0?ooo`008P3oool200000100oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00H0001o0?ooo`3oool0003o0?ooo`0007l30?ooo`040000O`3oool0oooo0000O`80oooo
00@0001o0?ooo`3oool0001o4@3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool01@0007l0oooo0?ooo`0007l0oooo008000007P3oool00280oooo00<000000?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool01P0007l0oooo0?ooo`00
0?l0oooo0000O`<0oooo00@0001o0?ooo`3oool0001o0P3oool0100007l0oooo0?ooo`0007lA0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3o
ool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool01P0007l0oooo0000o`000?l0oooo0000O`<0
oooo00@0001o0?ooo`3oool0001o0P3oool0100007l0oooo0?ooo`0007lA0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<000000?ooo`3oool0703oool00100oooo100000040?ooo`@000001P3oool00`000000oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0003o0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000?0?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3o
ool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o
0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o
0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3o
ool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool0
1`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`000?l0
0P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`00
07l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`00
07l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`02
0?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool001@0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0000o`000?l00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<000000?ooo`3oool0703oool000L0oooo1P0000070?ooo`040000003oool0
oooo000000@0oooo00<000000?ooo`3oool00`3oool300000=P0001o00<0003o0000O`0007l0A`00
07l3000001h0oooo000@0?ooo`@000000`3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool20000o`<0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000@0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0003o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00100
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00@0001o0?ooo`3oool0oooo0P000?l4
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`00403oool5000000<0
oooo100000060?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`03
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000
O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`060000O`3oool0oooo
0?ooo`000?l0001o103oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool0
0280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0
001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0
001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0
001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P000?l0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?oo
o`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3o
ool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3o
ool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool01P0007l0oooo0?ooo`000?l0
oooo0000O`@0oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?oo
o`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00H0001o0?ooo`000?l0003o0?ooo`0007l40?ooo`050000
O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo
00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo
0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00D0001o
0?ooo`3oool0001o0?ooo`02000001h0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`050?ooo`030000O`000?l0003o0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00D0oooo00<0001o0000o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool01@3oool20000o`<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0oooo
0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo
0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?oo
o`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?oo
o`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0
001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo0P000?l40?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`030?ooo`80003o
00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0
oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0
oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo
00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool00P3o
ool20000o`030?ooo`0007l0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<000000?ooo`3oool0703oool00280oooo0P00000@0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00@0001o0?ooo`3oool0oooo0P00
0?l20?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`050000O`3oool0oooo
0000O`3oool00P00000N0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0
001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3o
ool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3o
ool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool0
0`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool00P000?l3
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`04
0?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`040000O`3oool0003o
0000o`@0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0000
o`000?l01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?oo
o`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?oo
o`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01P3oool30000o`H0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0
oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00D0oooo0P000?l00`0007l0oooo
0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3oool0
0`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool0
0`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?oo
o`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo
0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo
0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?oo
o`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?oo
o`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0
oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`030?ooo`<0003o00<0
oooo0000O`3oool01P3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0
001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3o
ool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3o
ool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool01000
07l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`800000403oool00`00
07l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`00
07l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`00
07l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?oo
o`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?oo
o`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`020?ooo`80003o0`3oool0
0`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool0
1@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo
0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`05
0000O`3oool0oooo0000O`3oool00P00000N0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`03
0000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`03
0000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo
0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0
oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0
oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo
00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo
00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3o
ool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00<0003o103oool00`00
07l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool0
3`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool0
0`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0
oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`00
07l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@00
07l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?oo
o`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool0100007l0003o0000o`000?l60?oo
o`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo
0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo
0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`04
0000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?oo
o`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?oo
o`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3o
ool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0=0?ooo`@0003o203oool00`0007l0
oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`02
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo00<000000?ooo`3oool03`3o
ool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3o
ool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo
0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0
oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0
oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?
0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`02
0?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3o
ool0oooo0000O`80oooo00<0001o0?ooo`3oool02`3oool30000o`030?ooo`0007l0oooo00P0oooo
00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?ooo`030000003oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0
oooo00<0oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo
00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo
00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?oo
o`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?oo
o`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00L0oooo10000?l40?ooo`030000O`3o
ool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3o
ool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80
oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0
oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?oo
o`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000R0?oo
o`800000403oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo00<0001o0?oo
o`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0
100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3o
ool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?ooo`3oool00P3o
ool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0
oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool0103oool40000o`L0oooo00<0001o
0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o
0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o
0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0
1@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0
oooo0000O`020?ooo`050000O`3oool0oooo0000O`3oool00P00000N0?ooo`008P3oool00`000000
oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0
oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?oo
o`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`07
0?ooo`030000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`02
0?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000
O`3oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000
O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080
oooo00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`050000o`X0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0oooo0080oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool0703oool00280oooo
00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00@0oooo
00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`00
07l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool01@3o
ool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`H0003o3P3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0
001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`008P3oool00`000000oooo0?oo
o`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?oo
o`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000
O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`03
0000O`3oool0oooo00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`05
0000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000O`3oool0
oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3oool0
oooo0080oooo00<0001o0?ooo`3oool00P3oool0100007l0oooo0?ooo`3oool80000o`030?ooo`00
07l0oooo0100oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`050?ooo`030000O`3o
ool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo
00@0001o0?ooo`3oool0001o0P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0
oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo0080
oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<00000
0?ooo`3oool0703oool001P0oooo100000060?ooo`030000003oool0oooo00l0oooo00<0001o0?oo
o`3oool01`3oool00`0007l0oooo0?ooo`040?ooo`030000O`3oool0oooo00<0oooo00<0001o0?oo
o`3oool00P3oool01@0007l0oooo0?ooo`3oool0001o0080oooo00@0001o0?ooo`3oool0001o0P3o
ool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103o
ool00`0007l0oooo0?ooo`030?ooo`030000O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0001o0?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool;0000o`05
0?ooo`0007l0oooo0?ooo`0007l00P3oool00`0007l0oooo0?ooo`0?0?ooo`030000O`3oool0oooo
00L0oooo00<0001o0?ooo`3oool01@3oool00`0007l0oooo0?ooo`020?ooo`030000O`3oool0oooo
0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<0
001o0?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0
001o0?ooo`3oool00P3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l0
0P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo01`0oooo000G0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`030000
O`3oool0oooo00L0oooo00<0001o0?ooo`3oool0103oool00`0007l0oooo0?ooo`030?ooo`030000
O`3oool0oooo0080oooo00D0001o0?ooo`3oool0oooo0000O`020?ooo`040000O`3oool0oooo0000
O`80oooo00<000000?ooo`3oool03`3oool00`0007l0oooo0?ooo`070?ooo`030000O`3oool0oooo
00@0oooo00<0001o0?ooo`3oool00`3oool00`0007l0oooo0?ooo`020?ooo`050000O`3oool0oooo
0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l20?ooo`030000003oool0oooo00l0oooo00<0
001o0?ooo`3oool01P3oool00`000?l0001o0?ooo`0=0000o`<0oooo00D0001o0?ooo`3oool0oooo
0000O`020?ooo`040000O`3oool0oooo0000O`80oooo00<000000?ooo`3oool03`3oool00`0007l0
oooo0?ooo`070?ooo`030000O`3oool0oooo00D0oooo00<0001o0?ooo`3oool00P3oool00`0007l0
oooo0?ooo`020?ooo`050000O`3oool0oooo0?ooo`0007l00P3oool0100007l0oooo0?ooo`0007l2
0?ooo`030000003oool0oooo00l0oooo00<0001o0?ooo`3oool01`3oool00`0007l0oooo0?ooo`05
0?ooo`030000O`3oool0oooo0080oooo00<0001o0?ooo`3oool00P3oool01@0007l0oooo0?ooo`3o
ool0001o0080oooo00@0001o0?ooo`3oool0001o0P3oool00`000000oooo0?ooo`0L0?ooo`005`3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`020?ooo`050000003oool0oooo0?ooo`0000000P3oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool01@000000
oooo0?ooo`3oool000000080oooo00@000000?ooo`3oool0003o0P3ooolO0000o`D0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`050000003oool0oooo0?ooo`0000000P3o
ool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
0P3oool01@000000oooo0?ooo`3oool000000080oooo00@000000?ooo`3oool000000P3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00D000000?ooo`3oool0oooo00000002
0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0703oool001L0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooog`0003o[000000N0?ooo`005`3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo0?l0ooooBP3oool001L0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`3o0?ooodX0oooo000G0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool0o`3ooom:0?ooo`00603oool400000?l0ooooC@3oool00?l0ooooJ@3oool0
0?l0ooooJ@3oool00?l0ooooJ@3oool00?l0ooooJ@3oool00200oooo1@00000_0?ooo`D000000`3o
ool4000000D0oooo0P00000T0?ooo`D000000`3oool4000000@0oooo100000050?ooo`800000703o
ool5000000<0oooo100000040?ooo`@00000103oool4000000D0oooo0P00000E0?ooo`D000000`3o
ool4000000@0oooo100000040?ooo`@00000103oool4000000D0oooo0P0000130?ooo`008P3oool0
0`000000oooo0?ooo`0a0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`020?ooo`8000009P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`020?ooo`8000007P3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000
103oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0080oooo0P00000G0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?oo
o`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?oo
o`020?ooo`800000@`3oool00280oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?oo
o`0R0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?oo
o`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool01000
0000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0A`3o
ool00280oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0R0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`040000
003oool0oooo000000@0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`000004
0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0A`3oool00280oooo00<00000
0?ooo`3oool0<@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0:P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool00P3oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0
oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080
oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`040000003oool0
oooo000000@0oooo00<000000?ooo`3oool0A`3oool00280oooo00<000000?ooo`3oool0<@3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0:P3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000103o
ool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3o
ool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3o
ool00000103oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool0A`3oool00200oooo0`00000a0?ooo`<00000103oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo02P0oooo0`0000040?ooo`030000003oool0oooo0080oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`0P0?ooo`<00000103oool00`000000oooo0?oo
o`020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000103oool00`000000
oooo0?ooo`0I0?ooo`<00000103oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo0000
00@0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000040?ooo`030000003o
ool0oooo04L0oooo000R0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool00`3oool4
000002d0oooo00<000000?ooo`3oool00`3oool4000000@0oooo1000000U0?ooo`030000003oool0
oooo00<0oooo100000040?ooo`@00000103oool4000001h0oooo00<000000?ooo`3oool00`3oool4
000000@0oooo100000040?ooo`@00000103oool4000004X0oooo003o0?ooofT0oooo003o0?ooofT0
oooo003o0?ooofT0oooo0000\
\>"],
  ImageRangeCache->{{{0, 359}, {221.375, 0}} -> {-0.579829, -298.684, \
0.021156, 1.84848}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "References\t\t\t\t\t",
  Cell[BoxData[
      FormBox[
        ButtonBox[\(back\ to\ TOC\),
          ButtonData:>"TOC",
          ButtonStyle->"Hyperlink"], TraditionalForm]],
    FontSize->12,
    FontWeight->"Plain"]
}], "Section"],

Cell[TextData[{
  "J. Sun and H. Grotstollen, \"Symbolic analysis methods for averaged \
modeling of switching power converters,\" ",
  StyleBox["IEEE Trans. on Power Electronics",
    FontSlant->"Italic"],
  ", vol. 12, no. 3, pp. 537-546, May 1997."
}], "Reference",
  CellTags->"Reference [1]"],

Cell[TextData[{
  "N. N. Bogoliubov and Y. A. Mitropolsky, ",
  StyleBox["Asymptotic Methods in the Theory of Nonlinear Oscillations,",
    FontSlant->"Italic"],
  " NY: Gordon and Breach Science Pub., 1961."
}], "Reference",
  CellTags->"Reference [2]"],

Cell[TextData[{
  "B. Lehman and R. M. Bass, \"Switching frequency dependent averaged models \
for PWM DC-DC Converters,\" ",
  StyleBox["IEEE Trans. Power Electronics",
    FontSlant->"Italic"],
  ", vol.11, no. 1, pp.89-98, Jan. 1996."
}], "Reference",
  CellTags->"Reference [3]"],

Cell["\<\
J. Sun and R. M. Bass, \"Automated ripple analysis using KBM \
methods,\" in Proceedings of IEEE IECON'97.\
\>", "Reference",
  CellTags->"Reference [4]"],

Cell["\<\
J. Sun and R. M. Bass, \"A new approach to averaged modeling of PWM \
converters with current-mode control,\" in Proceedings of IECON'97.\
\>", \
"Reference",
  CellTags->"Reference [5]"]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 941}},
WindowToolbars->{"RulerBar", "EditBar"},
WindowSize->{975, 946},
WindowMargins->{{64, Automatic}, {Automatic, 2}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"Magnification"->1},
Magnification->1.25,
StyleDefinitions -> "Demo.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "TOC"->{
    Cell[2133, 65, 1520, 49, 481, "Text",
      CellTags->"TOC"]},
  "Introduction"->{
    Cell[3690, 119, 282, 10, 102, "Section",
      CellTags->"Introduction"],
    Cell[3975, 131, 1594, 42, 178, "Text",
      CellTags->"Introduction"],
    Cell[5572, 175, 304, 7, 72, "Text",
      CellTags->"Introduction"],
    Cell[5879, 184, 557, 13, 69, "Text",
      CellTags->"Introduction"]},
  "Load Package"->{
    Cell[6473, 202, 284, 10, 102, "Section",
      CellTags->"Load Package"],
    Cell[6760, 214, 1275, 29, 279, "Text",
      CellTags->"Load Package"],
    Cell[8038, 245, 156, 5, 103, "Input",
      CellTags->"Load Package"]},
  "Circuit Model"->{
    Cell[8231, 255, 284, 10, 102, "Section",
      CellTags->"Circuit Model"],
    Cell[8518, 267, 2109, 45, 411, "Text",
      CellTags->"Circuit Model"],
    Cell[10630, 314, 428, 8, 69, "Text",
      CellTags->"Circuit Model"],
    Cell[11083, 326, 88, 1, 56, "Subsection",
      CellTags->"Circuit Model"],
    Cell[11174, 329, 735, 15, 166, "Input",
      CellTags->{"NetList", "Circuit Model"}]},
  "NetList"->{
    Cell[11174, 329, 735, 15, 166, "Input",
      CellTags->{"NetList", "Circuit Model"}]},
  "State-Space Modeling"->{
    Cell[11946, 349, 73, 1, 56, "Subsection",
      CellTags->"State-Space Modeling"],
    Cell[12022, 352, 1526, 58, 110, "Text",
      CellTags->"State-Space Modeling"],
    Cell[13551, 412, 166, 4, 40, "Input",
      CellTags->"State-Space Modeling"],
    Cell[13766, 422, 294, 10, 102, "Section",
      CellTags->"State-Space Modeling"],
    Cell[14085, 436, 87, 1, 56, "Subsection",
      CellTags->"State-Space Modeling"],
    Cell[14175, 439, 626, 14, 86, "Text",
      CellTags->"State-Space Modeling"],
    Cell[14804, 455, 197, 4, 40, "Input",
      CellTags->"State-Space Modeling"],
    Cell[15038, 464, 76, 1, 56, "Subsection",
      CellTags->"State-Space Modeling"],
    Cell[15117, 467, 4130, 148, 338, "Text",
      CellTags->"State-Space Modeling"],
    Cell[19272, 619, 157, 3, 40, "Input",
      CellTags->"State-Space Modeling"],
    Cell[19939, 638, 101, 1, 35, "Text",
      CellTags->"State-Space Modeling"],
    Cell[20065, 643, 70, 2, 40, "Input",
      CellTags->"State-Space Modeling"],
    Cell[20470, 658, 70, 2, 40, "Input",
      CellTags->"State-Space Modeling"]},
  "Averaging"->{
    Cell[20668, 670, 305, 10, 102, "Section",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[20976, 682, 2061, 44, 386, "Text",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[23062, 730, 87, 1, 56, "Subsection",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[23152, 733, 1533, 53, 145, "Text",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[24688, 788, 162, 3, 40, "Input",
      CellTags->{"SlowFast", "Averaging", "Averaged Modeling"}],
    Cell[24853, 793, 2962, 89, 519, "Text",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[27818, 884, 172, 3, 40, "Input",
      CellTags->{"SlowFast", "BCs", "Averaging", "Averaged Modeling"}],
    Cell[27993, 889, 637, 16, 69, "Text",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[28633, 907, 136, 3, 40, "Input",
      CellTags->{"IV", "Averaging", "Averaged Modeling"}],
    Cell[28806, 915, 87, 1, 56, "Subsection",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[28896, 918, 856, 23, 86, "Text",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[29755, 943, 525, 10, 124, "Input",
      CellTags->{"SlowVars", "Averaging", "Averaged Modeling"}],
    Cell[30305, 957, 97, 1, 54, "Subsubsection",
      CellTags->{"Relaxation", "Averaging", "Averaged Modeling"}],
    Cell[30427, 962, 216, 4, 40, "Input",
      CellTags->{"Relaxation", "Averaging", "Averaged Modeling"}],
    Cell[31212, 986, 329, 6, 52, "Text",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[31578, 997, 97, 1, 54, "Subsubsection",
      CellTags->{"Decoupling", "Averaging", "Averaged Modeling"}],
    Cell[31678, 1000, 277, 5, 61, "Input",
      CellTags->{"Decoupling", "Averaging", "Averaged Modeling"}],
    Cell[31992, 1010, 82, 1, 54, "Subsubsection",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[32277, 1021, 153, 3, 40, "Input",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[32433, 1026, 1685, 65, 129, "Text",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[34143, 1095, 200, 4, 40, "Input",
      CellTags->{"Averaged Model", "Averaging", "Averaged Modeling"}]},
  "Averaged Modeling"->{
    Cell[20668, 670, 305, 10, 102, "Section",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[20976, 682, 2061, 44, 386, "Text",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[23062, 730, 87, 1, 56, "Subsection",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[23152, 733, 1533, 53, 145, "Text",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[24688, 788, 162, 3, 40, "Input",
      CellTags->{"SlowFast", "Averaging", "Averaged Modeling"}],
    Cell[24853, 793, 2962, 89, 519, "Text",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[27818, 884, 172, 3, 40, "Input",
      CellTags->{"SlowFast", "BCs", "Averaging", "Averaged Modeling"}],
    Cell[27993, 889, 637, 16, 69, "Text",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[28633, 907, 136, 3, 40, "Input",
      CellTags->{"IV", "Averaging", "Averaged Modeling"}],
    Cell[28806, 915, 87, 1, 56, "Subsection",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[28896, 918, 856, 23, 86, "Text",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[29755, 943, 525, 10, 124, "Input",
      CellTags->{"SlowVars", "Averaging", "Averaged Modeling"}],
    Cell[30305, 957, 97, 1, 54, "Subsubsection",
      CellTags->{"Relaxation", "Averaging", "Averaged Modeling"}],
    Cell[30427, 962, 216, 4, 40, "Input",
      CellTags->{"Relaxation", "Averaging", "Averaged Modeling"}],
    Cell[31212, 986, 329, 6, 52, "Text",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[31578, 997, 97, 1, 54, "Subsubsection",
      CellTags->{"Decoupling", "Averaging", "Averaged Modeling"}],
    Cell[31678, 1000, 277, 5, 61, "Input",
      CellTags->{"Decoupling", "Averaging", "Averaged Modeling"}],
    Cell[31992, 1010, 82, 1, 54, "Subsubsection",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[32277, 1021, 153, 3, 40, "Input",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[32433, 1026, 1685, 65, 129, "Text",
      CellTags->{"Averaging", "Averaged Modeling"}],
    Cell[34143, 1095, 200, 4, 40, "Input",
      CellTags->{"Averaged Model", "Averaging", "Averaged Modeling"}]},
  "SlowFast"->{
    Cell[24688, 788, 162, 3, 40, "Input",
      CellTags->{"SlowFast", "Averaging", "Averaged Modeling"}],
    Cell[27818, 884, 172, 3, 40, "Input",
      CellTags->{"SlowFast", "BCs", "Averaging", "Averaged Modeling"}]},
  "BCs"->{
    Cell[27818, 884, 172, 3, 40, "Input",
      CellTags->{"SlowFast", "BCs", "Averaging", "Averaged Modeling"}]},
  "IV"->{
    Cell[28633, 907, 136, 3, 40, "Input",
      CellTags->{"IV", "Averaging", "Averaged Modeling"}]},
  "SlowVars"->{
    Cell[29755, 943, 525, 10, 124, "Input",
      CellTags->{"SlowVars", "Averaging", "Averaged Modeling"}]},
  "Relaxation"->{
    Cell[30305, 957, 97, 1, 54, "Subsubsection",
      CellTags->{"Relaxation", "Averaging", "Averaged Modeling"}],
    Cell[30427, 962, 216, 4, 40, "Input",
      CellTags->{"Relaxation", "Averaging", "Averaged Modeling"}]},
  "Decoupling"->{
    Cell[31578, 997, 97, 1, 54, "Subsubsection",
      CellTags->{"Decoupling", "Averaging", "Averaged Modeling"}],
    Cell[31678, 1000, 277, 5, 61, "Input",
      CellTags->{"Decoupling", "Averaging", "Averaged Modeling"}]},
  "Averaged Model"->{
    Cell[34143, 1095, 200, 4, 40, "Input",
      CellTags->{"Averaged Model", "Averaging", "Averaged Modeling"}],
    Cell[34539, 1107, 3088, 97, 174, "Text",
      CellTags->"Averaged Model"]},
  "Ripple Functions"->{
    Cell[37688, 1211, 312, 10, 102, "Section",
      CellTags->{"Ripple Functions", "Ripple Estimation"}],
    Cell[38003, 1223, 3382, 117, 236, "Text",
      CellTags->{"Ripple Functions", "Ripple Estimation"}],
    Cell[41388, 1342, 1422, 46, 116, "Text",
      CellTags->{"Ripple Functions", "Ripple Estimation"}],
    Cell[42813, 1390, 160, 3, 40, "Input",
      CellTags->{"Ripple Functions", "Ripple Estimation"}],
    Cell[42976, 1395, 1815, 42, 418, "Text",
      CellTags->{"Ripple Functions", "Ripple Estimation"}],
    Cell[44816, 1441, 196, 4, 40, "Input",
      CellTags->{"Ripple Functions", "Ripple Estimation"}],
    Cell[46437, 1472, 1540, 29, 229, "Text",
      CellTags->{"Ripple Functions", "Ripple Estimation"}]},
  "Ripple Estimation"->{
    Cell[37688, 1211, 312, 10, 102, "Section",
      CellTags->{"Ripple Functions", "Ripple Estimation"}],
    Cell[38003, 1223, 3382, 117, 236, "Text",
      CellTags->{"Ripple Functions", "Ripple Estimation"}],
    Cell[41388, 1342, 1422, 46, 116, "Text",
      CellTags->{"Ripple Functions", "Ripple Estimation"}],
    Cell[42813, 1390, 160, 3, 40, "Input",
      CellTags->{"Ripple Functions", "Ripple Estimation"}],
    Cell[42976, 1395, 1815, 42, 418, "Text",
      CellTags->{"Ripple Functions", "Ripple Estimation"}],
    Cell[44816, 1441, 196, 4, 40, "Input",
      CellTags->{"Ripple Functions", "Ripple Estimation"}],
    Cell[46437, 1472, 1540, 29, 229, "Text",
      CellTags->{"Ripple Functions", "Ripple Estimation"}]},
  "Large-Signal Simulation"->{
    Cell[48014, 1506, 300, 10, 102, "Section",
      CellTags->"Large-Signal Simulation"],
    Cell[48317, 1518, 493, 11, 69, "Text",
      CellTags->"Large-Signal Simulation"],
    Cell[51392, 1613, 71, 1, 56, "Subsection",
      CellTags->"Large-Signal Simulation"],
    Cell[51466, 1616, 249, 5, 52, "Text",
      CellTags->"Large-Signal Simulation"],
    Cell[51718, 1623, 248, 4, 61, "Input",
      CellTags->"Large-Signal Simulation"],
    Cell[52003, 1632, 86, 1, 56, "Subsection",
      CellTags->"Large-Signal Simulation"],
    Cell[52092, 1635, 616, 20, 69, "Text",
      CellTags->"Large-Signal Simulation"],
    Cell[52711, 1657, 421, 8, 103, "Input",
      CellTags->"Large-Signal Simulation"],
    Cell[53308, 1675, 305, 6, 61, "Input",
      CellTags->"Large-Signal Simulation"],
    Cell[112592, 3090, 88, 1, 56, "Subsection",
      CellTags->"Large-Signal Simulation"],
    Cell[112683, 3093, 658, 18, 69, "Text",
      CellTags->"Large-Signal Simulation"],
    Cell[113344, 3113, 552, 11, 145, "Input",
      CellTags->"Large-Signal Simulation"],
    Cell[113899, 3126, 228, 5, 52, "Text",
      CellTags->"Large-Signal Simulation"],
    Cell[114152, 3135, 364, 7, 82, "Input",
      CellTags->"Large-Signal Simulation"],
    Cell[271476, 7036, 87, 1, 56, "Subsection",
      CellTags->"Large-Signal Simulation"],
    Cell[271566, 7039, 544, 13, 69, "Text",
      CellTags->"Large-Signal Simulation"],
    Cell[272113, 7054, 263, 5, 61, "Input",
      CellTags->"Large-Signal Simulation"],
    Cell[272379, 7061, 492, 9, 103, "Input",
      CellTags->"Large-Signal Simulation"],
    Cell[272896, 7074, 302, 6, 61, "Input",
      CellTags->"Large-Signal Simulation"],
    Cell[443738, 11714, 84, 1, 56, "Subsection",
      CellTags->"Large-Signal Simulation"],
    Cell[443825, 11717, 490, 8, 86, "Text",
      CellTags->"Large-Signal Simulation"],
    Cell[444340, 11729, 130, 2, 40, "Input",
      CellTags->"Large-Signal Simulation"],
    Cell[557695, 15369, 130, 2, 40, "Input",
      CellTags->"Large-Signal Simulation"],
    Cell[651659, 18850, 71, 1, 56, "Subsection",
      CellTags->"Large-Signal Simulation"],
    Cell[651733, 18853, 233, 5, 52, "Text",
      CellTags->"Large-Signal Simulation"],
    Cell[651969, 18860, 247, 4, 61, "Input",
      CellTags->"Large-Signal Simulation"]},
  "Small-Signal Analysis"->{
    Cell[48835, 1533, 96, 1, 56, "Subsection",
      CellTags->"Small-Signal Analysis"],
    Cell[48934, 1536, 2059, 63, 70, "Text",
      CellTags->"Small-Signal Analysis"],
    Cell[50996, 1601, 359, 7, 61, "Input",
      CellTags->"Small-Signal Analysis"],
    Cell[651091, 18829, 307, 10, 102, "Section",
      CellTags->"Small-Signal Analysis"],
    Cell[651401, 18841, 233, 5, 52, "Text",
      CellTags->"Small-Signal Analysis"],
    Cell[652253, 18869, 85, 1, 56, "Subsection",
      CellTags->"Small-Signal Analysis"],
    Cell[652341, 18872, 3812, 122, 128, "Text",
      CellTags->"Small-Signal Analysis"],
    Cell[656548, 19009, 74, 1, 56, "Subsection",
      CellTags->"Small-Signal Analysis"],
    Cell[656625, 19012, 400, 11, 53, "Text",
      CellTags->"Small-Signal Analysis"],
    Cell[657050, 19027, 175, 4, 40, "Input",
      CellTags->"Small-Signal Analysis"],
    Cell[657481, 19042, 73, 1, 56, "Subsection",
      CellTags->"Small-Signal Analysis"],
    Cell[657557, 19045, 1800, 58, 127, "Text",
      CellTags->"Small-Signal Analysis"],
    Cell[659360, 19105, 155, 3, 40, "Input",
      CellTags->"Small-Signal Analysis"],
    Cell[659552, 19113, 77, 1, 56, "Subsection",
      CellTags->"Small-Signal Analysis"],
    Cell[659632, 19116, 1944, 61, 92, "Text",
      CellTags->"Small-Signal Analysis"],
    Cell[661579, 19179, 198, 4, 40, "Input",
      CellTags->"Small-Signal Analysis"],
    Cell[661780, 19185, 196, 4, 40, "Input",
      CellTags->"Small-Signal Analysis"],
    Cell[661979, 19191, 141, 4, 35, "Text",
      CellTags->"Small-Signal Analysis"],
    Cell[662145, 19199, 86, 2, 40, "Input",
      CellTags->"Small-Signal Analysis"],
    Cell[662606, 19213, 86, 2, 40, "Input",
      CellTags->"Small-Signal Analysis"],
    Cell[663350, 19229, 160, 4, 35, "Text",
      CellTags->"Small-Signal Analysis"],
    Cell[663535, 19237, 100, 2, 40, "Input",
      CellTags->"Small-Signal Analysis"],
    Cell[663820, 19248, 100, 2, 40, "Input",
      CellTags->"Small-Signal Analysis"],
    Cell[664118, 19260, 90, 1, 56, "Subsection",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
    Cell[664211, 19263, 728, 21, 71, "Text",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
    Cell[664964, 19288, 113, 1, 54, "Subsubsection",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
    Cell[665102, 19293, 353, 6, 82, "Input",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
    Cell[1027701, 32740, 382, 6, 82, "Input",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
    Cell[1392275, 45839, 115, 1, 54, "Subsubsection",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
    Cell[1392415, 45844, 352, 6, 82, "Input",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
    Cell[1757857, 59695, 382, 6, 82, "Input",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}]},
  "Bode Diagram"->{
    Cell[664118, 19260, 90, 1, 56, "Subsection",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
    Cell[664211, 19263, 728, 21, 71, "Text",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
    Cell[664964, 19288, 113, 1, 54, "Subsubsection",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
    Cell[665102, 19293, 353, 6, 82, "Input",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
    Cell[1027701, 32740, 382, 6, 82, "Input",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
    Cell[1392275, 45839, 115, 1, 54, "Subsubsection",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
    Cell[1392415, 45844, 352, 6, 82, "Input",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
    Cell[1757857, 59695, 382, 6, 82, "Input",
      CellTags->{"Small-Signal Analysis", "Bode Diagram"}]},
  "Reference [1]"->{
    Cell[2035112, 67171, 297, 7, 48, "Reference",
      CellTags->"Reference [1]"]},
  "Reference [2]"->{
    Cell[2035412, 67180, 254, 6, 29, "Reference",
      CellTags->"Reference [2]"]},
  "Reference [3]"->{
    Cell[2035669, 67188, 283, 7, 48, "Reference",
      CellTags->"Reference [3]"]},
  "Reference [4]"->{
    Cell[2035955, 67197, 164, 4, 29, "Reference",
      CellTags->"Reference [4]"]},
  "Reference [5]"->{
    Cell[2036122, 67203, 197, 5, 29, "Reference",
      CellTags->"Reference [5]"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"TOC", 2037165, 67234},
  {"Introduction", 2037252, 67237},
  {"Load Package", 2037574, 67246},
  {"Circuit Model", 2037825, 67253},
  {"NetList", 2038243, 67264},
  {"State-Space Modeling", 2038363, 67267},
  {"Averaging", 2039478, 67294},
  {"Averaged Modeling", 2041669, 67337},
  {"SlowFast", 2043851, 67380},
  {"BCs", 2044084, 67385},
  {"IV", 2044209, 67388},
  {"SlowVars", 2044327, 67391},
  {"Relaxation", 2044455, 67394},
  {"Decoupling", 2044699, 67399},
  {"Averaged Model", 2044948, 67404},
  {"Ripple Functions", 2045167, 67409},
  {"Ripple Estimation", 2045927, 67424},
  {"Large-Signal Simulation", 2046693, 67439},
  {"Small-Signal Analysis", 2049037, 67492},
  {"Bode Diagram", 2051943, 67555},
  {"Reference [1]", 2052829, 67572},
  {"Reference [2]", 2052935, 67575},
  {"Reference [3]", 2053041, 67578},
  {"Reference [4]", 2053147, 67581},
  {"Reference [5]", 2053253, 67584}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1717, 49, 167, 4, 132, "Title"],
Cell[1887, 55, 182, 4, 85, "Section 1"],

Cell[CellGroupData[{
Cell[2094, 63, 36, 0, 102, "Section"],
Cell[2133, 65, 1520, 49, 481, "Text",
  CellTags->"TOC"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3690, 119, 282, 10, 102, "Section",
  CellTags->"Introduction"],
Cell[3975, 131, 1594, 42, 178, "Text",
  CellTags->"Introduction"],
Cell[5572, 175, 304, 7, 72, "Text",
  CellTags->"Introduction"],
Cell[5879, 184, 557, 13, 69, "Text",
  CellTags->"Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6473, 202, 284, 10, 102, "Section",
  CellTags->"Load Package"],
Cell[6760, 214, 1275, 29, 279, "Text",
  CellTags->"Load Package"],
Cell[8038, 245, 156, 5, 103, "Input",
  CellTags->"Load Package"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8231, 255, 284, 10, 102, "Section",
  CellTags->"Circuit Model"],
Cell[8518, 267, 2109, 45, 411, "Text",
  CellTags->"Circuit Model"],
Cell[10630, 314, 428, 8, 69, "Text",
  CellTags->"Circuit Model"],

Cell[CellGroupData[{
Cell[11083, 326, 88, 1, 56, "Subsection",
  CellTags->"Circuit Model"],
Cell[11174, 329, 735, 15, 166, "Input",
  CellTags->{"NetList", "Circuit Model"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[11946, 349, 73, 1, 56, "Subsection",
  CellTags->"State-Space Modeling"],
Cell[12022, 352, 1526, 58, 110, "Text",
  CellTags->"State-Space Modeling"],
Cell[13551, 412, 166, 4, 40, "Input",
  CellTags->"State-Space Modeling"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[13766, 422, 294, 10, 102, "Section",
  CellTags->"State-Space Modeling"],

Cell[CellGroupData[{
Cell[14085, 436, 87, 1, 56, "Subsection",
  CellTags->"State-Space Modeling"],
Cell[14175, 439, 626, 14, 86, "Text",
  CellTags->"State-Space Modeling"],
Cell[14804, 455, 197, 4, 40, "Input",
  CellTags->"State-Space Modeling"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15038, 464, 76, 1, 56, "Subsection",
  CellTags->"State-Space Modeling"],
Cell[15117, 467, 4130, 148, 338, "Text",
  CellTags->"State-Space Modeling"],

Cell[CellGroupData[{
Cell[19272, 619, 157, 3, 40, "Input",
  CellTags->"State-Space Modeling"],
Cell[19432, 624, 113, 2, 58, "Output"],
Cell[19548, 628, 376, 7, 113, "Output"]
}, Open  ]],
Cell[19939, 638, 101, 1, 35, "Text",
  CellTags->"State-Space Modeling"],

Cell[CellGroupData[{
Cell[20065, 643, 70, 2, 40, "Input",
  CellTags->"State-Space Modeling"],
Cell[20138, 647, 295, 6, 75, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20470, 658, 70, 2, 40, "Input",
  CellTags->"State-Space Modeling"],
Cell[20543, 662, 64, 1, 72, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[20668, 670, 305, 10, 102, "Section",
  CellTags->{"Averaging", "Averaged Modeling"}],
Cell[20976, 682, 2061, 44, 386, "Text",
  CellTags->{"Averaging", "Averaged Modeling"}],

Cell[CellGroupData[{
Cell[23062, 730, 87, 1, 56, "Subsection",
  CellTags->{"Averaging", "Averaged Modeling"}],
Cell[23152, 733, 1533, 53, 145, "Text",
  CellTags->{"Averaging", "Averaged Modeling"}],
Cell[24688, 788, 162, 3, 40, "Input",
  CellTags->{"SlowFast", "Averaging", "Averaged Modeling"}],
Cell[24853, 793, 2962, 89, 519, "Text",
  CellTags->{"Averaging", "Averaged Modeling"}],
Cell[27818, 884, 172, 3, 40, "Input",
  CellTags->{"SlowFast", "BCs", "Averaging", "Averaged Modeling"}],
Cell[27993, 889, 637, 16, 69, "Text",
  CellTags->{"Averaging", "Averaged Modeling"}],
Cell[28633, 907, 136, 3, 40, "Input",
  CellTags->{"IV", "Averaging", "Averaged Modeling"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[28806, 915, 87, 1, 56, "Subsection",
  CellTags->{"Averaging", "Averaged Modeling"}],
Cell[28896, 918, 856, 23, 86, "Text",
  CellTags->{"Averaging", "Averaged Modeling"}],
Cell[29755, 943, 525, 10, 124, "Input",
  CellTags->{"SlowVars", "Averaging", "Averaged Modeling"}],

Cell[CellGroupData[{
Cell[30305, 957, 97, 1, 54, "Subsubsection",
  CellTags->{"Relaxation", "Averaging", "Averaged Modeling"}],

Cell[CellGroupData[{
Cell[30427, 962, 216, 4, 40, "Input",
  CellTags->{"Relaxation", "Averaging", "Averaged Modeling"}],
Cell[30646, 968, 126, 2, 30, "Message"],
Cell[30775, 972, 126, 2, 30, "Message"],
Cell[30904, 976, 126, 2, 30, "Message"],
Cell[31033, 980, 164, 3, 51, "Message"]
}, Open  ]],
Cell[31212, 986, 329, 6, 52, "Text",
  CellTags->{"Averaging", "Averaged Modeling"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[31578, 997, 97, 1, 54, "Subsubsection",
  CellTags->{"Decoupling", "Averaging", "Averaged Modeling"}],
Cell[31678, 1000, 277, 5, 61, "Input",
  CellTags->{"Decoupling", "Averaging", "Averaged Modeling"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[31992, 1010, 82, 1, 54, "Subsubsection",
  CellTags->{"Averaging", "Averaged Modeling"}],
Cell[32077, 1013, 197, 6, 35, "Text"],
Cell[32277, 1021, 153, 3, 40, "Input",
  CellTags->{"Averaging", "Averaged Modeling"}],
Cell[32433, 1026, 1685, 65, 129, "Text",
  CellTags->{"Averaging", "Averaged Modeling"}],

Cell[CellGroupData[{
Cell[34143, 1095, 200, 4, 40, "Input",
  CellTags->{"Averaged Model", "Averaging", "Averaged Modeling"}],
Cell[34346, 1101, 178, 3, 76, "Output"]
}, Open  ]],
Cell[34539, 1107, 3088, 97, 174, "Text",
  CellTags->"Averaged Model"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[37688, 1211, 312, 10, 102, "Section",
  CellTags->{"Ripple Functions", "Ripple Estimation"}],
Cell[38003, 1223, 3382, 117, 236, "Text",
  CellTags->{"Ripple Functions", "Ripple Estimation"}],
Cell[41388, 1342, 1422, 46, 116, "Text",
  CellTags->{"Ripple Functions", "Ripple Estimation"}],
Cell[42813, 1390, 160, 3, 40, "Input",
  CellTags->{"Ripple Functions", "Ripple Estimation"}],
Cell[42976, 1395, 1815, 42, 418, "Text",
  CellTags->{"Ripple Functions", "Ripple Estimation"}],

Cell[CellGroupData[{
Cell[44816, 1441, 196, 4, 40, "Input",
  CellTags->{"Ripple Functions", "Ripple Estimation"}],
Cell[45015, 1447, 1407, 22, 315, "Output"]
}, Open  ]],
Cell[46437, 1472, 1540, 29, 229, "Text",
  CellTags->{"Ripple Functions", "Ripple Estimation"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[48014, 1506, 300, 10, 102, "Section",
  CellTags->"Large-Signal Simulation"],
Cell[48317, 1518, 493, 11, 69, "Text",
  CellTags->"Large-Signal Simulation"],

Cell[CellGroupData[{
Cell[48835, 1533, 96, 1, 56, "Subsection",
  CellTags->"Small-Signal Analysis"],
Cell[48934, 1536, 2059, 63, 70, "Text",
  CellTags->"Small-Signal Analysis"],
Cell[50996, 1601, 359, 7, 61, "Input",
  CellTags->"Small-Signal Analysis"]
}, Open  ]],

Cell[CellGroupData[{
Cell[51392, 1613, 71, 1, 56, "Subsection",
  CellTags->"Large-Signal Simulation"],
Cell[51466, 1616, 249, 5, 52, "Text",
  CellTags->"Large-Signal Simulation"],
Cell[51718, 1623, 248, 4, 61, "Input",
  CellTags->"Large-Signal Simulation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[52003, 1632, 86, 1, 56, "Subsection",
  CellTags->"Large-Signal Simulation"],
Cell[52092, 1635, 616, 20, 69, "Text",
  CellTags->"Large-Signal Simulation"],
Cell[52711, 1657, 421, 8, 103, "Input",
  CellTags->"Large-Signal Simulation"],
Cell[53135, 1667, 148, 4, 35, "Text"],

Cell[CellGroupData[{
Cell[53308, 1675, 305, 6, 61, "Input",
  CellTags->"Large-Signal Simulation"],
Cell[53616, 1683, 29714, 677, 243, 5464, 372, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[83345, 2363, 110, 4, 35, "Text"],

Cell[CellGroupData[{
Cell[83480, 2371, 260, 5, 61, "Input"],
Cell[83743, 2378, 28800, 706, 243, 6132, 421, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[112592, 3090, 88, 1, 56, "Subsection",
  CellTags->"Large-Signal Simulation"],
Cell[112683, 3093, 658, 18, 69, "Text",
  CellTags->"Large-Signal Simulation"],
Cell[113344, 3113, 552, 11, 145, "Input",
  CellTags->"Large-Signal Simulation"],
Cell[113899, 3126, 228, 5, 52, "Text",
  CellTags->"Large-Signal Simulation"],

Cell[CellGroupData[{
Cell[114152, 3135, 364, 7, 82, "Input",
  CellTags->"Large-Signal Simulation"],
Cell[114519, 3144, 85066, 2048, 243, 18838, 1225, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[199622, 5197, 327, 6, 82, "Input"],
Cell[199952, 5205, 71475, 1825, 243, 17671, 1156, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[271476, 7036, 87, 1, 56, "Subsection",
  CellTags->"Large-Signal Simulation"],
Cell[271566, 7039, 544, 13, 69, "Text",
  CellTags->"Large-Signal Simulation"],
Cell[272113, 7054, 263, 5, 61, "Input",
  CellTags->"Large-Signal Simulation"],
Cell[272379, 7061, 492, 9, 103, "Input",
  CellTags->"Large-Signal Simulation"],

Cell[CellGroupData[{
Cell[272896, 7074, 302, 6, 61, "Input",
  CellTags->"Large-Signal Simulation"],
Cell[273201, 7082, 91998, 2406, 243, 24150, 1563, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[365236, 9493, 265, 5, 61, "Input"],
Cell[365504, 9500, 78185, 2208, 243, 23421, 1527, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[443738, 11714, 84, 1, 56, "Subsection",
  CellTags->"Large-Signal Simulation"],
Cell[443825, 11717, 490, 8, 86, "Text",
  CellTags->"Large-Signal Simulation"],

Cell[CellGroupData[{
Cell[444340, 11729, 130, 2, 40, "Input",
  CellTags->"Large-Signal Simulation"],
Cell[444473, 11733, 113185, 3631, 243, 42786, 2757, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[557695, 15369, 130, 2, 40, "Input",
  CellTags->"Large-Signal Simulation"],
Cell[557828, 15373, 93202, 3449, 243, 43832, 2834, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[651091, 18829, 307, 10, 102, "Section",
  CellTags->"Small-Signal Analysis"],
Cell[651401, 18841, 233, 5, 52, "Text",
  CellTags->"Small-Signal Analysis"],

Cell[CellGroupData[{
Cell[651659, 18850, 71, 1, 56, "Subsection",
  CellTags->"Large-Signal Simulation"],
Cell[651733, 18853, 233, 5, 52, "Text",
  CellTags->"Large-Signal Simulation"],
Cell[651969, 18860, 247, 4, 61, "Input",
  CellTags->"Large-Signal Simulation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[652253, 18869, 85, 1, 56, "Subsection",
  CellTags->"Small-Signal Analysis"],
Cell[652341, 18872, 3812, 122, 128, "Text",
  CellTags->"Small-Signal Analysis"],
Cell[656156, 18996, 355, 8, 82, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[656548, 19009, 74, 1, 56, "Subsection",
  CellTags->"Small-Signal Analysis"],
Cell[656625, 19012, 400, 11, 53, "Text",
  CellTags->"Small-Signal Analysis"],

Cell[CellGroupData[{
Cell[657050, 19027, 175, 4, 40, "Input",
  CellTags->"Small-Signal Analysis"],
Cell[657228, 19033, 204, 3, 75, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[657481, 19042, 73, 1, 56, "Subsection",
  CellTags->"Small-Signal Analysis"],
Cell[657557, 19045, 1800, 58, 127, "Text",
  CellTags->"Small-Signal Analysis"],
Cell[659360, 19105, 155, 3, 40, "Input",
  CellTags->"Small-Signal Analysis"]
}, Open  ]],

Cell[CellGroupData[{
Cell[659552, 19113, 77, 1, 56, "Subsection",
  CellTags->"Small-Signal Analysis"],
Cell[659632, 19116, 1944, 61, 92, "Text",
  CellTags->"Small-Signal Analysis"],
Cell[661579, 19179, 198, 4, 40, "Input",
  CellTags->"Small-Signal Analysis"],
Cell[661780, 19185, 196, 4, 40, "Input",
  CellTags->"Small-Signal Analysis"],
Cell[661979, 19191, 141, 4, 35, "Text",
  CellTags->"Small-Signal Analysis"],

Cell[CellGroupData[{
Cell[662145, 19199, 86, 2, 40, "Input",
  CellTags->"Small-Signal Analysis"],
Cell[662234, 19203, 335, 5, 75, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[662606, 19213, 86, 2, 40, "Input",
  CellTags->"Small-Signal Analysis"],
Cell[662695, 19217, 640, 9, 121, "Output"]
}, Open  ]],
Cell[663350, 19229, 160, 4, 35, "Text",
  CellTags->"Small-Signal Analysis"],

Cell[CellGroupData[{
Cell[663535, 19237, 100, 2, 40, "Input",
  CellTags->"Small-Signal Analysis"],
Cell[663638, 19241, 145, 2, 75, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[663820, 19248, 100, 2, 40, "Input",
  CellTags->"Small-Signal Analysis"],
Cell[663923, 19252, 146, 2, 75, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[664118, 19260, 90, 1, 56, "Subsection",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
Cell[664211, 19263, 728, 21, 71, "Text",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],

Cell[CellGroupData[{
Cell[664964, 19288, 113, 1, 54, "Subsubsection",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],

Cell[CellGroupData[{
Cell[665102, 19293, 353, 6, 82, "Input",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
Cell[665458, 19301, 362206, 13434, 243, 173507, 11099, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1027701, 32740, 382, 6, 82, "Input",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
Cell[1028086, 32748, 364140, 13085, 243, 166512, 10640, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1392275, 45839, 115, 1, 54, "Subsubsection",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],

Cell[CellGroupData[{
Cell[1392415, 45844, 352, 6, 82, "Input",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
Cell[1392770, 45852, 365050, 13838, 243, 180158, 11550, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1757857, 59695, 382, 6, 82, "Input",
  CellTags->{"Small-Signal Analysis", "Bode Diagram"}],
Cell[1758242, 59703, 276542, 7449, 243, 78043, 4993, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2034857, 67160, 252, 9, 102, "Section"],
Cell[2035112, 67171, 297, 7, 48, "Reference",
  CellTags->"Reference [1]"],
Cell[2035412, 67180, 254, 6, 29, "Reference",
  CellTags->"Reference [2]"],
Cell[2035669, 67188, 283, 7, 48, "Reference",
  CellTags->"Reference [3]"],
Cell[2035955, 67197, 164, 4, 29, "Reference",
  CellTags->"Reference [4]"],
Cell[2036122, 67203, 197, 5, 29, "Reference",
  CellTags->"Reference [5]"]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

